/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.dijit.Catalog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "propertyeditor");

dojo.require("hub.managers.Urls");
hub.urls = new hub.managers.Urls(self.parent.hub.config.base_url);

dojo.require("hub.data.catalog");

dojo.declare(
  "flow.dijit.Catalog",
  [dijit._Widget, dijit._Templated, dijit._Container],
  {
    // attach points
    searchNode: null,
    tableBodyNode: null,
    summaryNode: null,
    previousButton: null,
    refreshButton: null,
    nextButton: null,

    templatePath: dojo.moduleUrl("flow.dijit", "templates/Catalog.html"),
    widgetsInTemplate: true,

    // localization
    i18n: null,

    model: null,
    rowCount: 0,
    selectedRow: null,

    constructor: function(args, node) {
      this.stale = false;
      this.conns = new Array();
    },

    postMixInProperties: function() {
      this.model = new hub.data.catalog.AtomModel({collection: "feeds", term: "*", count: 10, sortBy: "title", sortOrder: "asc"});

      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    postCreate: function() {
      // register as an observer
      this.connect(this.model, 'onFetch', '_onBeforeFetch');
      this.connect(this.model, 'onBegin', '_onBeforeLoad');
      this.connect(this.model, 'onItem', '_onItemLoad');
      this.connect(this.model, 'onComplete', '_onAfterLoad');
      this.connect(this.model, 'onError', '_onLoadError');
      this.connect(this.model, 'onStale', '_onStale');

      // initialize navigation
      this._updateNavBar();

      // interpolate values
      this.inherited(arguments);
      this.startup();
    },

    startup: function() {
      if (this.model) {
        this.model.refresh();
      }
    },

    destroy: function() {
      // cleanup all subscriptions
      dojo.forEach(this.conns, dojo.unsubscribe);
      // call base class method
      this.inherited(arguments);
    },

    clearRows: function() {
      var rows = this.tableBodyNode.rows;

      for (var i = 0; i < rows.length; i++) {
        var row = this.tableBodyNode.rows[i];
        row.cells[0].innerHTML = "&nbsp;";
        row.cells[1].innerHTML = "&nbsp;";
        row.removeAttribute("objectid");
        row.removeAttribute("selflink");
        row.removeAttribute("objectname");
      }
    },

    initialize : function() {
      this.clearRows();
    },

    _onSearch: function() {
      var value = dojo.trim(this.searchNode.value);
      if (!value) {
        value = "*";
      }

      this.model.clear();
      this.model.term = value;
      this.model.refresh();
    },

    _onStale: function() {
      // refresh immediately
      this.stale = false;
      this.model.refresh();
    },

    _onBeforeFetch: function() {
      this.clearRows();
      this.summaryNode.innerHTML = this.i18n.propertyeditor.CATALOG_LOADING;
    },

    _onBeforeLoad: function(total) {
      // reset row count
      this.rowCount = 0;

      if (this.selectedRow) {
        dojo.removeClass(this.selectedRow, "selectedrow");
        this.selectedRow = null;
      }
    },

    _onLoadError: function(error) {
      alert(error);
    },

    _onItemLoad : function(dict) {
      // We reuse rows, lets check if we need to build it or not

      if (this.tableBodyNode.rows.length <= this.rowCount) {
        // build
        var tr = document.createElement("tr");

        var td = document.createElement("td");
        dojo.addClass(td, "flowgridcell");
        tr.appendChild(td)

        td = document.createElement("td");
        dojo.addClass(td, "flowgridcell");
        tr.appendChild(td);

        if (this.rowCount % 2) {
          dojo.addClass(tr, "alt");
        }

        this.tableBodyNode.appendChild(tr);
      }

      var id = dict.objectid;

      var row = this.tableBodyNode.rows[this.rowCount];
      row.cells[0].innerHTML = dijit.byId("flowcanvas").model.stripSctipt(dict.name);
      row.cells[1].innerHTML = dijit.byId("flowcanvas").model.stripSctipt(dict.author);
      row.setAttribute("objectid", id);
      row.setAttribute("selflink", dict.downloadURL);
      row.setAttribute("objectname", dict.name);

      this.rowCount++;
    },

    _onAfterLoad: function() {
      if (!this.rowCount) {
        this.summaryNode.innerHTML = this.i18n.propertyeditor.CATALOG_SUMMARY_EMPTY;
      } else {
        var label = dojo.string.substitute(this.i18n.propertyeditor.CATALOG_SUMMARY, [parseInt(this.model.summary[0], 10) + 1, parseInt(this.model.summary[0], 10) + this.rowCount, this.model.summary[2]]);
        this.summaryNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(label);
      }

      this._updateNavBar();
    },

    _updateNavBar : function() {
      if (this.model.hasPrevious()) {
        this.previousButton.setDisabled(false);
      } else {
        this.previousButton.setDisabled(true);
      }

      if (this.model.hasNext()) {
        this.nextButton.setDisabled(false);
      } else {
        this.nextButton.setDisabled(true);
      }
    },

    _onPrevious: function(event) {
      try {
        this.model.fetchPrevious();
      } catch(e) {}
    },

    _onNext: function(event) {
      try {
        this.model.fetchNext();
      } catch(e) {}
    },

    _onRefresh: function(event) {
      this.model.refresh();
    },

    _onRowClick: function(event) {
      var target = event.target;

      while (target.tagName != "TR" && target.parentNode) {
        target = target.parentNode;
      }

      if (target.tagName != "TR") {
        // should really never happen
        return;
      }

      dojo.stopEvent(event);

      if (this.selectedRow == target) {
        // already selected
        return;
      }

      if (this.selectedRow) {
        dojo.removeClass(this.selectedRow, "selectedrow");
      }

      this.selectedRow = target;
      dojo.addClass(this.selectedRow, "selectedrow");
    },

    getSelectedId: function() {
      var id = null;

      if (this.selectedRow) {
        id = this.selectedRow.getAttribute("objectid");
      }

      return id;
    },

    getSelectedName: function() {
      var name = null;

      if (this.selectedRow) {
        name = this.selectedRow.getAttribute("objectname");
      }

      return name;
    },

    getSelectedUrl: function() {
      var id = null;

      if (this.selectedRow) {
        id = this.selectedRow.getAttribute("selflink");
      }

      return id;
    }
});
