/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("flow.operator.BaseOperator");

/*
  Every Operator has two representations - the visual part (CanvasOperator) and
  a functional part, which derives from BaseOperator.  The functional code will
  be called to render the property editor (setupPropertyEditor) and can intercept
  calls for adding/changing/removing values from the property editor if needed.
*/

dojo.require("dojo.i18n");
dojo.requireLocalization("flow", "propertyeditor");
dojo.requireLocalization("flow", "labels");

dojo.require("flow.operator.Data");

dojo.declare("flow.operator.BaseOperator", null,
{
  type: null,
  model: null,
  id: null,
  node: null,

  flowNS: "http://ibm.com/mashuphub",
  mhubNS: "http://w3.ibm.com/mhub",

  MASHUPPATH: "/flow:mashup/",
  FEEDPATH: "/flow:mashup/flow:data/flow:feed/",
  SOURCEXMLPATH: "/flow:mashup/flow:data/flow:sourcexml",
  OPERATORPATH: "/flow:mashup/flow:flow/flow:operator/",
  SOURCEPATH: "/flow:mashup/flow:flow/flow:source/",
  TEMPLATEPATH: "/flow:mashup/flow:flow/flow:operator/flow:template",

  hasOutput: true,
  canRename: true,
  minimumInputs: 1,

  // -1 means unlimited
  maximumInputs: 1,

  conditions: [],

  init: function(model, id){
    this.model = model;

    if (id) {
      this.id = id;

      if(this.type=="source") {
        var url= this.model.getValue(this.model.flowNode, "flow:source[@id='"+this.id+"']/flow:url/text()");
        if (url!=null && url!="") {
          this.reloadPrimitiveData();
        }
      }

      // TODO: load the data from model?
    }

    
    this.i18n = dojo.i18n.getLocalization("flow", "propertyeditor");
  },

  // generated a random ID for our type
  generateId: function() {
    return this.model.generateId(this.type);
  },

  getLabel: function() {
    var labels = dojo.i18n.getLocalization("flow", "labels");
    return labels["OPERATOR_" + this.type.toUpperCase()];
  },

  getName: function() {
    var name = this.model.getValue(this.node, "@caption");

    if (!name) {
      name = this.id;
    }

    return name;
  },

  getDisplayName: function() {
    var name = this.getName();
    if (name == this.id) {
      name = this.getLabel();
    }

    return name;
  },

  getCanvas: function() {
    return this.model.canvas;
  },

  // returns the XML Document for the Model
  getModel: function() {
    return this.model.xmlModel;
  },

  // adds a top-level Model child
  addToModel: function(elm) {
    return this.model.flowNode.appendChild(elm);
  },

  // adds to data
  addToData: function(elm) {
    return this.model.dataNode.appendChild(elm);
  },

  // XPath helper method
  runXPath: function(xpath, getFirst) {
    var type = this.model.xpathTypes.anyType;

    if (typeof(getFirst) != "undefined" && getFirst) {
      type = this.model.xpathTypes.firstType;
    }

    return this.getModel().evaluate(xpath, this.node, null, type, null);
  },

  // verifies that all requirements are met
  meetsRequirements: function() {
    var result = {pass: true, error: null};

    // check if enough inputs are available
    var count = this.model.getInputCount(this.id);

    if (count < this.minimumInputs) {
      if (this.minimumInputs == 1) {
        result.error = this.i18n.ERROR_REQUIRES_ONE_INPUT;
      } else if (this.maximumInputs == -1) {
        result.error = dojo.string.substitute(this.i18n.ERROR_REQUIRES_MINIMUM_INPUTS, [this.minimumInputs]);
      } else {
        result.error = dojo.string.substitute(this.i18n.ERROR_REQUIRES_MULTIPLE_INPUTS, [this.minimumInputs]);
      }

      result.pass = false;
    }

    // check if inputs generate data
    if (result.pass) {
      var inputsSatisfied = this.model.areAllInputsSatisfied(this.id);
      if (!inputsSatisfied.satisfied) {
        // result.cause is the operator object
        result.error = dojo.string.substitute(this.i18n.LABEL_INPUT_NOT_VAlID, [inputsSatisfied.cause.getName()]);
        result.pass = false;
      }
    }

    // check if enough params/conditions exist
    if (result.pass) {
      var pass = this.hasEnoughParameters()
      // either true or a string with the error msg
      if (pass !== true) {
        result.pass = false;
        result.error = pass;
      }
    }

    return result;
  },

  // returns true or an error message
  hasEnoughParameters: function() {
    return true;
  },

  // returns all child parameter nodes
  getParams: function(options, parent) {
    var xpath = "flow:param";

    if (options) {
      for (var option in options) {
        xpath += "[@"+option+"='"+options[option]+"']"
      }
    }

    return this.model.runXPath(xpath, parent ? parent : this.node);
  },

  // returns a js representation of all child parameters
  getParameters: function(options, parent) {
    var params = this.getParams(options, parent);
    var result = [];

    for (var i = 0; i < params.length; i++) {
      var item = this.generateDataFromParameter(params[i]);

      // use the position attribute to place the item in the right position in
      // the data array
      var pos = parseInt(this.model.getValue(params[i], "@position"), 10);
      if (!pos) {
        pos = 0;
      }

      result[pos] = item;
    }

    return result;
  },

  // generates a data object for a parameter
  generateDataFromParameter: function(parameter) {
    //console.log("generateDataFromParameter called with id: "+parameter.getAttribute("id"))

    /*var data = {id: this.model.getValue(parameter, "@id"),
                type: this.model.getValue(parameter, "@type"),
                position: this.model.getValue(parameter, "@position"),
                comments: this.model.getValue(parameter, "@comments"),
                caption: this.model.getValue(parameter, "@caption")
    };*/

    var id = this.model.getValue(parameter, "@id");
    var type = this.model.getValue(parameter, "@type");

    var data = flow.operator.Data.create(type, id);
    data.position = this.model.getValue(parameter, "@position");
    data.inputid = this.model.getValue(parameter, "@inputid");
    data.comments = this.model.getValue(parameter, "@comments");
    data.caption = this.model.getValue(parameter, "@caption");

    if (type == "function") {
      var funcnode = this.model.getNode("flow:function", parameter);

      data.name = this.model.getValue(funcnode, "@name");

      // build the function params
      data.parameters = this.getParameters(null, funcnode);
    } else if (data.type == "dynamic") {
	    data.value = this.model.getValue(parameter, "text()");
      data.name = this.model.getValue(parameter, "@caption");
    } else {
      data.value = this.model.getValue(parameter, "text()");
    }

    return data;
  },

  getNode: function(name, parent) {
    // adds flow: to name
    return this.model.getNode("flow:"+name, parent ? parent : this.node);
  },

  // creates an operator in the model
  create: function() {
    this.id = this.generateId();

    var template = '<operator xmlns="'+this.model.flowNS+'" dirty="1" caption="'+this.id+'" type="'+this.type+'" id="'+this.id+'" x="" y="" />';
    this.node = this.addToModel(this.model.parseXML(template));
  },

  generateData: function() {
    return {layout: []};
  },

  generateGridData: function() {
    var data = [];

    var conditions = this.model.runXPath("flow:condition", this.node);

    for (var i = 0; i < conditions.length; i++) {
      var item = {};
      var condition = conditions[i];

      item.id = this.model.getValue(condition, "@id");
      item.operation = this.model.getValue(condition, "@operation");
      item.conjunction= this.model.getValue(condition, "@conjunction");

      var pos = this.model.getValue(condition, "@position");
      pos = (!pos) ? 0 : pos;

      item.input = this.generateDataFromParameter(this.model.getNode("flow:param[@position='0']", condition));
      item.value = this.generateDataFromParameter(this.model.getNode("flow:param[@position='1']", condition));

      data[pos] = item;

      var conditem = {id: item.id};
      conditem.inputParamId = this.model.getValue(condition, "flow:param[@position='0']/@id");
      conditem.valueParamId = this.model.getValue(condition, "flow:param[@position='1']/@id");
      this.conditions[item.id] = conditem;
    }

    return data;
  },

  getVariableItems: function(varparams, ignoreexistingparams) {
    var itemlist = [];
    for (var i = 0; i < varparams.length; i++) {
      var param = varparams[i];

      var parentpath = flow.xml.getLocalName(this.model.getNode("..", param)) == "condition"? "../..":".." ;
      var parentid = this.model.getValue(param, parentpath+"/@caption");
      var id = this.model.getValue(param, "@id");
      var caption = this.model.getValue(param, "@caption");

      if (ignoreexistingparams) {
        if (this.model.getValue(this.node,".//flow:condition/flow:param[@position='0' and text()='"+id+"']/@id") == null) {
          itemlist.push({label: this.createVariableLabel(parentid, caption), id: id});
        }
      } else {
        itemlist.push({label: this.createVariableLabel(parentid, caption), id: id});
      }
    }

    return itemlist;
  },

  getDirtyInputs: function(nodeID, includeSelf) {
    var startnode = this.model.getNode("./flow:*[@id='"+nodeID+"']",this.model.flowNode);

    var params = [];

    var inputs = this.model.runXPath("flow:input",startnode);

    for (var i = 0; i < inputs.length; i++) {
      var srcid = this.model.getValue(inputs[i], "@id");

      var retparams = this.getDirtyInputs(srcid, true);
      for (var j=0; j < retparams.length; j++) {
        params.push(retparams[j]);
      }
    }

    // when calling dirty inputs, skip the caller operator
    if (includeSelf) {
      if (this.model.getValue(startnode, "@dirty") == 1 || params.length > 0) {
        params.push(startnode);
      }
    }

    return params;
  },

  isDirty: function() {
    return (this.model.getValue(this.node, "@dirty") == 1);
  },

  getInscopeVaraibles: function(nodeID, all) {
    var startnode = this.model.getNode("./flow:*[@id='"+nodeID+"']",this.model.flowNode);
    var searchstr=".//flow:param";
    searchstr += (all) ? "[@type='dynamic' or (name(..)='source') or (../../@type='filter' and @type='variable')  ]" : "[@type='dynamic']";

    var tparams = this.model.runXPath(searchstr,startnode);
    var params =[];

    for (var j=0; j < tparams.length; j++) {
      if (tparams[j].getAttribute("type") == "variable") {
        params[j] = this.model.getNode("./flow:variables/flow:param[@id='"+tparams[j].firstChild.nodeValue+"']",this.model.flowNode)	      
      } else {
        params[j] = tparams[j];
      }
    }

    var startctr = params.length;
    var inputs = this.model.runXPath("flow:input",startnode);
    for(var i = 0; i < inputs.length; i++) {
      var srcid = this.model.getValue(inputs[i], "@id");
      var retparams = this.getInscopeVaraibles(srcid, all);

      for (var j = 0; j < retparams.length; j++) {
      	var notfound=true;
        for (var k = 0; k < params.length; k++) {
        	if (params[k] == retparams[j]) {
            notfound=false;
            break;
        	}
        }

        if (notfound) {
          params[startctr++]=retparams[j];
        }
      }
    }

    return params;
  },

  getVariableCount: function() {
    var items = this.model.runXPath("//flow:param[@type='dynamic']", this.model.flowNode);
    return items ? items.length : 0;
  },

  createVariableLabel: function(parentid, caption) {
    return dojo.string.substitute(this.i18n.VARIABLE_DISPLAY_LABEL, [parentid, caption]);
  },

  generateVariableListBox: function(selectObj, searchstr, defaultValue) {
    var items = this.model.runXPath(searchstr, this.model.flowNode);

    for(var i = 0; i < items.length; i++) {
      var item = items[i];
      var parentid = this.model.getValue(item, "../@caption");
      if(parentid==null) parentid=this.i18n.VARIABLE_GLOBAL_CAPTION;
      var id = this.model.getValue(item, "@id");
      var caption = this.model.getValue(item, "@caption");

      var option = document.createElement("option");
      option.setAttribute("value", id);
      option.appendChild(document.createTextNode(this.createVariableLabel(parentid, caption)));
      if (defaultValue && id == defaultValue) {
        option.setAttribute("selected", true);
      }

      selectObj.appendChild(option);
    }
  },

  generateConfig: function() {
    return {canAddRows: true, canMoveRows: true, allowGridEditing: true};
  },

  isBuiltInType: function(type) {
    return (type == "text" || type == "xpath" || type == "variable");
  },

  createParameter: function(data, parentnode) {
    if (!parentnode) {
      parentnode = this.node;
    }

    this.model.setValue("@dirty", this.node, "1");

    var template='<param xmlns="'+this.model.flowNS +
                      '" caption="'+((typeof(data.caption) != "undefined" && data.caption) ? data.caption : "") +
                      '" operatorspecific="'+(typeof(data.operatorspecific) !="undefined" ? data.operatorspecific:"") +
                      '" inputid="" id="'+data.id +
                      '" position="'+this.model.runXPath("flow:param", parentnode).length +
                      '" type="'+(typeof(data.type) != "undefined" ? data.type:"") +
                      '" comments=""></param>';

    parentnode.appendChild(this.model.parseXML(template));

    if (this.isBuiltInType(data.type)) {
      var inputid = data.inputid ? data.inputid : "";
      var value = (typeof(data.value) != "undefined" && data.value!=null ?data.value: data);
      if (data.type == "text") {
        if (isNaN(value)) {
          value = dojo.trim(value);
          if (value.charAt(0)=='"') {
            value=value.substring(1);
          }

          if (value.charAt(value.length-1) == '"'){
            value = value.substring(0, value.length-1);
          }
          value = '"' + value + '"';
        }
      }

      this.model.setValue("flow:param[@id='"+data.id+"']/text()", parentnode, value);
      this.model.setValue("flow:param[@id='"+data.id+"']/@inputid", parentnode, inputid);

      if (typeof(data.comments) != "undefined" && data.comments!=null) {
        this.model.setValue("flow:param[@id='"+data.id+"']/@comments", parentnode, data.comments);
      }
    } else if (data.type == "function") {
      if (typeof(data.comments) != "undefined"  && data.comments!=null) {
        this.model.setValue("flow:param[@id='"+data.id+"']/@comments", parentnode, data.comments);
      }

      var pnode = this.model.getNode("flow:param[@id='"+data.id+"']", parentnode);
      this.createFunction(data, pnode);
    }
  },

  createCondition: function(data) {
    var template = '<condition xmlns="'+this.model.flowNS+'" id="'+data.id+'" conjunction="'+data.conjunction+
                    '" operation="" position="'+this.model.runXPath("flow:condition", this.node).length+'"></condition >';

    this.node.appendChild(this.model.parseXML(template));

    var cond = this.model.getNode("flow:condition[@id='"+data.id+"']", this.node);
    this.model.setValue("@operation", cond, data.operation);

    for (var i = 0; i < data.parameters.length; i++) {
      this.createParameter(data.parameters[i], cond);
    }
  },

  createFunction: function(data, parentnode) {
    var fdata=(data.parameters==null && data.value!=null && data.value.type=="function")
    				 ?data.value:data; // the function information is defined in the value
    var template = '<function xmlns="'+this.model.flowNS+'" name="'+fdata.name+'"></function>';

    var funcnode = null;
    if (parentnode) {
      funcnode = parentnode.appendChild(this.model.parseXML(template));
    } else {
      funcnode = this.node.appendChild(this.model.parseXML(template));
    }

    // add any parameters
    for (var i = 0; i < fdata.parameters.length; i++) {
      this.createParameter(fdata.parameters[i], funcnode);
    }
  },

  addChild: function(gridid, data, position) {},

  modifyChild: function(id, key, data) {
    this.model.setValue("@dirty", this.node, "1");

    if (key == "value") {
      var type = data.type ? data.type : "text";

      this.model.setValue(".//flow:param[@id='"+id+"']/@type", this.node, type);
      this.model.removeAllChildren(".//flow:param[@id='"+id+"']", this.node)

      if (type == "function") {
        this.model.setValue(".//flow:param[@id='"+id+"']/@type", this.node, type);

        if (typeof(data.comments) != "undefined") {
          this.model.setValue(".//flow:param[@id='"+id+"']/@comments", this.node, data.comments ? data.comments : "");
        }

        var param = this.model.getNode(".//flow:param[@id='"+id+"']", this.node);
        this.createFunction(data, param);
      } else if (type == "dynamic") {
        var value = typeof(data.value) != "undefined" ? data.value : data;
        /*
        if(isNaN(value)) {
	          value = dojo.trim(value);
              if(value[0]=='"') value=value.substring(1);
              if(value[value.length-1]=='"') value=value.substring(0,value.length-1);
              value = '"' + value + '"';
        }
        */
        this.model.setValue(".//flow:param[@id='"+id+"']/text()", this.node, value);
        this.model.setValue(".//flow:param[@id='"+id+"']/@caption", this.node, data.name);
      } else if (type == "variable") {
        var val= typeof(data.value)=="object" && data.value.value!=null ? data.value.value: data.value;
        this.model.setValue(".//flow:param[@id='"+id+"']/text()", this.node, val);
      } else {
        var value = typeof(data.value) != "undefined" ? data.value : data;
        if (type == "text") {
          if (isNaN(value)) {
            value = dojo.trim(value);

            if (value.charAt(0) == '"') {
              value = value.substring(1);
            }

            if (value.charAt(value.length-1) == '"') {
              value = value.substring(0,value.length-1);
            }

            value = '"' + value + '"';
          }
        }

        this.model.setValue(".//flow:param[@id='"+id+"']/text()", this.node, value);
        if (data.inputid) {
          this.model.setValue(".//flow:param[@id='"+id+"']/@inputid", this.node, data.inputid);
        }
      }
    } else if (key == "position") {
      this.model.setValue(".//flow:*[@id='"+id+"']/@position", this.node, data);
    } else {
      alert("modifyChild"+id +"  "+key+"  "+data.type + " " +data);
    }
  },

  // resets a node to a default value
  // TODO: remove
  resetToValue: function(node) {
    var valuenode = this.model.createElementNS(this.model.flowNS, "Value");
    node.parentNode.replaceChild(valuenode, node);
  },

  handleChainedValue: function(obj) {
    var model = this.getModel();
    var newNode = model.createElementNS(this.model.flowNS, obj.type);
    newNode.setAttribute("name", obj.name)

    if (obj.type) {
      newNode.setAttribute("type", obj.type)
    }

    for (var key in obj.parameters) {
      var param = obj.parameters[key];
      this.createParameter(param, newNode);
    }

    return newNode;
  },

  modifyCondition: function(id, key, data) {
    //console.log("modifyCondition: "+id+"  "+key+"  "+value);

    if (key == "position") {
      this.model.setValue("flow:condition[@id='"+id+"']/@position", this.node, data);
    } else {
      this.model.setValue("flow:condition[@id='"+id+"']/@" +key, this.node, data.value);
    }
  },

  removeChild: function(id) {
    var param = this.model.getById(id);

    if (!param) {
      return;
    }

    param.parentNode.removeChild(param);
  },

  inputRemoved: function(srcOperator) {
    // by default, all params/conditions will be removed.  If an operator
    // wants to take care of that, overwrite and return true
    return false;
  },

  getTree: function(pos, rawxml) {
    // default to 0
    if (!pos) {
      pos = 0;
    }

    var inputid = this.model.getValue(this.model.flowNode, "flow:operator[@id='"+this.id+"']/flow:input[@position='"+pos+"']/@id");

    if (!inputid) {
      // perhaps source - the id is the inputid
      if (this.type == "source") {
        inputid = this.id;
      } else {
        // TODO: error!
        return null;
      }
    }

    var type = this.model.getValue(this.model.dataNode, "flow:feed[@id='"+inputid+"']/@type");
    var timestamp = this.model.getValue(this.model.dataNode, "flow:feed[@id='"+inputid+"']/@timestamp");

    var startpath = this.FEEDPATH +"result/";

    if (rawxml) {
      var rootnode = this.model.getNode(this.SOURCEXMLPATH+ "[@id='"+inputid+"']/*");

      if (!rootnode) {
        return null;
      }

      var preview = this.setNamespace(rootnode.namespaceURI);
      preview = (preview != "") ? (preview + ":") : "";
      startpath = this.SOURCEXMLPATH +"/"+preview + flow.xml.getLocalName(rootnode) + "/";
    }
    else if (type == "RSS") {
      var rootnode = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+inputid+"']/*/rss/channel/item");
      startpath += "rss/channel/item/";
    } else if (type == "XML") {
      var rootnode = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+inputid+"']/*/*");
    } else if (type == "ATOM") {
      var rootnode = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+inputid+"']/*/*");
      startpath += "mhub:entry/";
    } else {
      var rootnode = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+inputid+"']/*/*");

      if (!rootnode) {
        return null;
      }

      var preview = this.setNamespace(rootnode.namespaceURI);
	    preview = (preview != "") ? (preview + ":") : "";

      startpath += preview + flow.xml.getLocalName(rootnode) + "/";
    }

    //console.log("getTree: "+inputid+","+type+","+startpath+","+rootnode);
    return {id: inputid, root: rootnode, path: startpath, timestamp: timestamp};
  },

  setChildValue: function(id, key, value) {
    if (key == "operation" || key == "position") {
      this.modifyCondition(id, key, value);
    } else {
      var condition = this.conditions[id];
      if (!condition) {
        throw "Flow Error: Could not find condition!";
      }

      var paramid = (key == "input") ? condition.inputParamId : condition.valueParamId;
      this.modifyChild(paramid, "value", value);
    }
  },

  loadXMLUrl: function(url, callback) {
    dojo.xhrGet({url: url,
                handleAs: "xml",
                timeout: 5000,
                load: function(response, ioArgs) {
                  callback(response);
                  return response;
                },

                error: function(response, ioArgs) {
                  callback(null);
                  return response;
                }
               });
  },

  callServer: function(actionUrl, postContent, callback) {
     dojo.xhrPost( {
         url: actionUrl,
         handleAs: dojo.isIE ? "text" : "xml",
         content: postContent,
         contentType: "application/x-www-form-urlencoded; charset=utf-8", 
         load: function(response, ioArgs) {
           if (dojo.isIE) {
             var doc = flow.xml.loadDocument(response);
             callback(doc);
             return doc;
           } else {
             callback(response);
             return response;
           }
         },
         error: function(response, ioArgs) {
            alert(response.message +" "+ioArgs.xhr.responseText);
            callback(null);
            return response;
         }
     });
  },

  // Called when operator is connected as input to another operator
  connectedToTarget: function(action) {
    if (action) {
      //If operator does not have any data then do a preview on it.
      if (!this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+this.id+"']/*/*")) {
        this.doPreview();
      }
    }
  },

  reloadPrimitiveData: function() {
  	this.model.flowNode.setAttribute("operatorid",this.id);
  	this.model.flowNode.setAttribute("primitive","true");
    var xmlString = '<?xml version="1.0" encoding="utf-8"?>'+flow.xml.innerXML(this.model.flowNode);
    var url = dijit.byId("flowid_flowwidget").getEditorActionUrl("preview"); 
    var content = {xml: xmlString, op: '$' + this.id, seq: '$' + this.id, topk: 50, format: "xml"};
    this.callServer(url, content, dojo.hitch(this, this.reloadPrimitiveDataCallback));
  },

  reloadPrimitiveDataCallback:function(result) {
    if (result) {
      var startnode = flow.xml.findNodes(result.documentElement, "/*")[0]
      if (flow.xml.getLocalName(startnode) == "ERRORS") return;
      var dt = new Date();
      this.model.removeNodes("flow:sourcexml[@id='"+this.id+"']", this.model.dataNode);
      this.addToData(this.model.parseXML("<sourcexml xmlns='"+this.model.flowNS+"' timestamp='"+dt.getTime()+"' id='"+this.id+"'/>"));
      var node=flow.xml.findNodes(result.documentElement, "/*/*")[0];
      this.model.getNode("flow:sourcexml[@id='"+this.id+"']", this.model.dataNode).appendChild(flow.xml._importNode(this.model.xmlModel, node, true));
      this.removeDuplicateNodes(node);
    }
  },

  removeDuplicateNodes: function(node)
  {

    var children = this.model.runXPath("*", node);
    for (var i = 50; i < children.length; i++) {
            children[i].parentNode.removeChild(children[i]);
    
    }
    this.model.removeDuplicateNodes(node);
  
  },
  doPreview: function(showError, callback) {
  
	if(this.type=="source") {
	  var turl=this.model.getValue(this.node,"./flow:url/text()")
//	  if(turl==null || dojo.trim(turl)=="") return;
	}
	if(this.type=="publish" && this.model.getNode("./flow:input",this.node)==null) return

  	
  	this.model.flowNode.setAttribute("operatorid",this.id);
  	this.model.flowNode.setAttribute("primitive","");

    var xmlString = '<?xml version="1.0" encoding="utf-8"?>'+flow.xml.innerXML(this.model.flowNode);
    var url = dijit.byId("flowid_flowwidget").getEditorActionUrl("preview"); 
    var content = {xml: xmlString, op: '$' + this.id, seq: '$' + this.id, topk: 50, format: "xml"};

    this.loadingPreviewResult = true;

    if (showError) {
      this.previewshowError = showError;
    }

    this.callServer(url, content, dojo.hitch(this, callback ? callback : this.previewCallback));
  },

  previewLoadComplete: function() {
    this.loadingPreviewResult = false;
  },

  previewCallback: function(result) {
    var showError = this.previewshowError;

    if (result) {
      //alert(flow.xml.innerXML (result.documentElement));
      var startnode = flow.xml.findNodes(result.documentElement, "/*")[0];

      if (flow.xml.getLocalName(startnode) == "ERRORS") {
      	var errmsg="<b>"+this.i18n.SERVER_PREVIEW_ERROR+"</b>";
      	var errornodes = flow.xml.findNodes(result.documentElement, "/*/*");

      	for (var er = 0; er < errornodes.length; er++) {
      		if (errornodes[er].lastChild.nodeName=="MSGTEXT") {
            errmsg+="<br>"+ errornodes[er].lastChild.firstChild.nodeValue;
          }
        }

        if (showError) {
          showError.style.display = "block";
          showError.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(errmsg);
        }

        dojo.publish("flow:"+this.id+":loadURL", [{error: true, message: errmsg}]);

        this.previewLoadComplete();
        return;
      }

      this.model.removeNodes("flow:feed[@id='"+this.id+"']", this.model.dataNode);
      var dt = new Date();

      this.addToData(this.model.parseXML("<flow:feed xmlns='' xmlns:flow='"+this.model.flowNS+"' timestamp='"+dt.getTime()+"' id='"+this.id+"'/>"));

      this.model.getNode("flow:feed[@id='"+this.id+"']", this.model.dataNode).appendChild(flow.xml._importNode(this.model.xmlModel, startnode, true));
      var node = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+this.id+"']/*/*");

      // TODO: better error handling
      if (!node) {
        var errmsg=dojo.string.substitute(this.i18n.SERVER_PREVIEW_MISSING_ITEM, [this.model.getValue(this.node,"@caption")]);
        if (showError) {
          showError.style.display = "block";
         	showError.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(errmsg);
        }

        dojo.publish("flow:"+this.id+":loadURL", [{error: true, message: errmsg}]);

        this.previewLoadComplete();
        return;
      }

      var localName = flow.xml.getLocalName(node).toLowerCase();
      this.model.setValue("/flow:mashup/flow:data/flow:feed[@id='"+this.id+"']/@type", null, localName);
      var dtype = "XML";
      
	  var nsuri=node.namespaceURI;
      if (localName == "rss") {
      	var rssnode = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+this.id+"']/*/rss/channel/item");
        if(rssnode !=null) dtype = "RSS";
      } else if (localName == "feed" && nsuri=="http://www.w3.org/2005/Atom") {
        dtype = "ATOM";
      }
      localName = flow.xml.getLocalName(node);

      var stype = "";
      if (flow.xml.getLocalName(this.node) == "source") {
        stype=this.model.getValue(this.model.flowNode,"//flow:source[@id='"+this.id+"']/@type");
        if (stype == null || stype == "") {
          this.model.removeNode(".//flow:sourcexml[@id='"+this.id+"']", this.model.dataNode,true);
          this.addToData(this.model.parseXML("<sourcexml xmlns='"+this.model.flowNS+"' timestamp='"+dt.getTime()+"' id='"+this.id+"'/>"));
          this.model.getNode("flow:sourcexml[@id='"+this.id+"']", this.model.dataNode).appendChild(node);
          this.removeDuplicateNodes(node);

          this.model.setValue("/flow:mashup/flow:flow/flow:source[@id='"+this.id+"']/@type", null, dtype);
          this.model.setValue("/flow:mashup/flow:flow/flow:source[@id='"+this.id+"']/@originaltype", null, dtype);
          var advancednode=this.model.getNode(".//flow:advanced", this.node);
          this.model.removeNodes(".//flow:param", advancednode);
          if (dtype == "RSS") {
           this.model.setValue("@repeatelement", advancednode, "/rss/channel/item");
            this.model.setValue("@originalrepeatelement", advancednode, "/rss/channel/item");
            this.doPreview();
            return;
          } else if (dtype == "ATOM") {
            var preview = this.setNamespace(node.namespaceURI);
            preview = preview!=""?preview+":":"";
            this.model.setValue("@repeatelement", advancednode, "/"+preview+"feed/"+preview+"entry");
            this.model.setValue("@originalrepeatelement", advancednode, "/"+preview+"feed/"+preview+"entry");

            var fid = this.model.getValue(node,"./*[name()='id']/text()");
            this.addAdvancedParamToSource(advancednode,fid,"ID");
            var ftitle = this.model.getValue(node,"./*[name()='title']/text()");
            this.addAdvancedParamToSource(advancednode,ftitle,"Title");
            var flink = this.model.getValue(node,"./*[name()='link']/text()");
            this.addAdvancedParamToSource(advancednode,flink,"Link");
            var authornode = this.model.getNode("./*[name()='author']/text()",node);
            var fname = this.model.getValue(authornode,"./*[name()='link']/text()");

            this.addAdvancedParamToSource(advancednode,fname,"Name");
            this.doPreview();
            return;
          } else {
            var preview = this.setNamespace(node.namespaceURI);
            preview = preview!=""?preview+":":"";
            this.model.setValue("@repeatelement", advancednode, "/"+preview+localName);
            this.model.setValue("@originalrepeatelement", advancednode, "/"+preview+localName);
            this.doPreview();
            return;
          }
          this.refreshPreviewTree();
        } else {
          this.refreshPreviewTree();
	      }
      } else {
        this.refreshPreviewTree();
      }

      dojo.publish("flow:"+this.id+":loadURL", [{success: true}]);
    }
    return;
  },

  addAdvancedParamToSource: function(advancednode, value, caption) {
  	var template='<param xmlns="'+this.model.flowNS+'" position="0"  caption="'+caption+'" operatorspecific="" '+ 
  				'inputid="" id="'+this.id+"_advanced"+caption+'" type="text" comments=""><![CDATA['+value+']]></param>';
    var ps = this.model.parseXML(template);
    advancednode.appendChild(ps);
  },

  refreshPreviewTree: function() {
  	this.model.setValue("@dirty", this.node, "0");

    this.previewLoadComplete();

    var propeditor = this.getCanvas().getPropertyEditor();
    if (propeditor) {
      propeditor.refreshPreviewTree();
    }
  },

  setNamespace: function(namespace) {
    if (namespace == null || namespace == "") {
      return "";
    }

    var prefix = this.model.getValue(this.flowNode, ".//flow:namespaces/flow:namespace[text()='"+namespace+"']/@prefix");
    if (prefix == null || prefix == "") {
      var ns= this.model.runXPath(".//flow:namespaces/flow:namespace",this.flowNode);
      var prefix=(namespace==this.model.flowNS)?"flow":(
        (namespace==this.model.mhubNS)?"mhub":"ns"+ns.length);

      var template='<namespace xmlns="'+this.model.flowNS +
                    '" prefix="'+prefix + '"><![CDATA['+namespace+']]></namespace>';
      var nsnode=this.model.getNode(".//flow:namespaces", this.flowNode);
      nsnode.appendChild(this.model.parseXML(template));
    }
    return prefix;
  },

  saveComment: function(text) {
    // attribute might not be set
    this.node.setAttribute("comment", text);
  },

  getComment: function(text) {
    return this.node.getAttribute("comment");
  },

  createErrorMarkup: function(error) {
    return "<div class='errorMsg'><table width='100%'><tr><td width='27px' class='errorMsgIcon'></td><td valign='center'>"+error+"</td></tr></table></div>"
  },

  edit: function() {},
  setupPropertyEditor: function() {},

  hasAdvancedOptions: false,
  renderAdvancedOptions: function(containerNode){},

  destroy: function() {
    // clear dom reference
    this.node = null;
  }
});

dojo.declare("flow.operator.SourceOperator", flow.operator.BaseOperator,
{
  type: "source",

  maximumInputs: 0,
  minimumInputs: 0,

  hasAdvancedOptions: true,
  isAdvancedOptionsVisible: false,

  create: function() {
    this.id = this.generateId();

    var template = '<source xmlns="'+this.model.flowNS+'" originaltype="" type="" dirty="1" caption="'+this.id+'" id="'+this.id+'" x="" y=""><advanced originalrepeatelement="" repeatelement="/*" interval="3600"></advanced><url xmlns="'+this.model.flowNS+'" catalogid="" caption="" type="url"></url></source>';
    this.node = this.addToModel(this.model.parseXML(template));
  },

  setupPropertyEditor: function() {
    var url = this.model.getValue(this.node, "flow:url/text()");
    if (url == null){ 
      url = "";
    }

    var type = this.model.getValue(this.node, "flow:url/@type");
    var catalogid = this.model.getValue(this.node, "flow:url/@catalogid");
    var caption = this.model.getValue(this.node, "flow:url/@caption");

    var data = this.generateData();
    data.layout.push({type: "url", data: {value: url, type: type, catalogid: catalogid, name: caption}, parent: this, callback: dojo.hitch(this, this.getUrl)});
    var grid = {id: "sourceparams", type: "grid", layout: null, data: [], config: this.generateConfig(), hidden: true, collapsable: true, collapseLabel: this.i18n.SOURCE_PARAM_COLLAPSE, expandLabel: this.i18n.SOURCE_PARAM_EXPAND};
    grid.layout = [
                   {name: "caption", label: this.i18n.SOURCE_HEADER_URL_PARAMETERS, editable: false, width: "5%"},
                   {name: "value", label: this.i18n.SOURCE_HEADER_VALUE_PARAMETERS, options: ["text",  "variable"], type: "param", width: "95%"}
                  ];

    grid.config.canAddRows = false;
    grid.config.canMoveRows = false;
    grid.config.allowGridEditing = false;

    this.showURLParams(grid);

    data.layout.push(grid);

    return data;
  },

  renderAdvancedOptions: function(containerNode) {
    containerNode.innerHTML = "";

    // source has advanced options - repeating item
    var div = document.createElement("div");
    containerNode.appendChild(div);

    // check if we are satisfied
    var url = this.getUrlValue();
    var tree = this.getTree(0,true);

    if (url && tree && tree.root && !this.loadingPreviewResult) {
      dojo.require("flow.dijit.SourceAdvanced");
      var x = new flow.dijit.SourceAdvanced({operator: this}, div);
    } else {
      if (!url) {
        div.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(this.createErrorMarkup(this.i18n.SOURCE_ADVANCED_ERROR_NO_URL));
      } else if (this.loadingPreviewResult) {
        div.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(this.createErrorMarkup(this.i18n.SOURCE_ADVANCED_ERROR_WAITING));
      } else {
        div.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(this.createErrorMarkup(this.i18n.SOURCE_ADVANCED_ERROR));
      }

      this.advancedContainerNode = div;
    }
  },

  showURLParams: function(grid) {
    // reset the grid
    if (this.grid) {
      this.grid.clearGrid(false);

      // grid with no params is hidden
      this.grid.domNode.style.display = "block";
    }

    var parameters = this.model.runXPath("flow:param", this.node);

    for (var i = 0; i < parameters.length; i++) {
      var parameter = parameters[i];
      var id = this.model.getValue(parameter, "@id");
      var caption = this.model.getValue(parameter, "@caption");
      var type = this.model.getValue(parameter, "@type");

      var item = {id: id, operator: this, caption: caption,  type: type};
      item.value = this.generateDataFromParameter(parameter);

      if (grid) {
        grid.data.push(item);
        // unhide grid if items exist
        grid.hidden = false;
      } else if (this.grid) {
        this.grid.addRow(id, item);
      }
    }
  },

  setChildValue: function(id, key, value) {
    this.modifyChild(id, key, value);
  },

  getParametersFromURL: function(url, grid) {
    url=url.replace(/\s*((\S+\s*)*)/, "$1"); 
    var spliturl=url.split("?");
    var addr=spliturl[0].split("/");
    var paramid =[];
    var paramvalue =[];
    var addrvalue =[];
    var j=0;

    var parentid=this.model.getValue(this.node, "@id");
    this.model.removeNodes("flow:param", this.node);
    for (var i = 0; i < addr.length;i ++) {
      if(addr[i] != "") {
        addrvalue[j++] = addr[i];
      }
    }

    var pos=0;
    for (var i = 0; i< addrvalue.length; i++) {
      var value = addrvalue[i];
      var id = parentid+"_path"+pos;
      var caption = "path"+pos;
      var type = "text";
      var def = "/";
      if (i == 0) {
        caption="domain";
        def="http";
        if(value=="http:" || value=="https:")
        {
          def=(value=="http:")?"http":"https";
          value=addrvalue[++i];
        }
      }
      var template='<param xmlns="'+this.model.flowNS+'" caption="'+caption+'" operatorspecific="'+def+'" id="'+id+
            '" position="'+(pos++)+'" type="'+type+'">'+value+'</param>';

      this.node.appendChild(this.model.parseXML(template));
    }

    if (spliturl.length>1) {
      var params = spliturl[1].split("&");
      for (var i=0; i<params.length; i++) {
        var oneparam=params[i].split("=");

        var value = oneparam[0];
        var id = parentid+"_param"+pos;
        var caption="param"+pos;
        var def="";
        if(i==0)
        {
        	def=url.indexOf("/?")>0?"/?":"?";
        }
        if (oneparam.length>1) {
          caption = oneparam[0];
          value = params[i].substring(oneparam[0].length+1);
          def += "=";
        }

	    //value=unescape(value);
        var template='<param xmlns="'+this.model.flowNS+'" caption="'+caption+'" operatorspecific="'+def+'" id="'+id+
            '" position="'+(pos++)+'" type="text"><![CDATA['+value+']]></param>';
        this.node.appendChild(this.model.parseXML(template));
        //this.model.setValue("flow:param[@id='"+id+"']/@operatorspecific",this.node,value);
      }
    }
    this.showURLParams();
  },

  getUrl: function(url, catalogid, name, errorNode) {
    if (url == null) {
      // checkbox changed, no value set
      this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@type", this.model.flowNode, catalogid ? "catalog" : "external");
      this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@catalogid", this.model.flowNode, "");
      this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@caption", this.model.flowNode, "");
      this.model.setValue("flow:source[@id='"+this.id+"']/@type", this.model.flowNode, "");
      this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/text()", this.model.flowNode, "");

      if (this.grid) {
        this.grid.clearGrid(false);
      }

      if (errorNode) {
        errorNode.innerHTML = "";
      }

      return;
    }

    this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@type", this.model.flowNode, catalogid ? "catalog" : "external");
    this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@catalogid", this.model.flowNode, catalogid ? catalogid : "");
    this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@caption", this.model.flowNode, name ? name : "");

    this.model.setValue("flow:source[@id='"+this.id+"']/@type", this.model.flowNode, "");
    this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/text()", this.model.flowNode, url);

    this.getParametersFromURL(url, null);

/*    if (catalogid) {
      // from catalog
      if (this.grid) {
        this.grid.clearGrid(false);
      }
    } else {
      this.getParametersFromURL(url, null);
    }*/

    this.model.setValue("@type", this.node, "");

    this.doPreview(errorNode);
  },

  getUrlValue: function() {
    return this.model.getValue(this.model.flowNode, "flow:source[@id='"+this.id+"']/flow:url/text()");
  },

  getUrlType: function() {
    return this.model.getValue(this.model.flowNode, "flow:source[@id='"+this.id+"']/flow:url/@type");
  },

  getType: function() {
    return this.model.getValue(this.model.flowNode, "flow:source[@id='"+this.id+"']/@type");
  },

  loadUrl: function() {
    // called during startup
    var url = this.getUrlValue();
    if (url) {
      this.doPreview();
    }
  },

  previewLoadComplete: function() {
    this.inherited("previewLoadComplete", arguments);

    // check if advanced options are visible
    if (this.isAdvancedOptionsVisible && this.advancedContainerNode) {
      this.renderAdvancedOptions(this.advancedContainerNode);
    }
  }
});

dojo.declare("flow.operator.CombineOperator", flow.operator.BaseOperator,
{
  type: "combine",

  minimumInputs: 2,
  maximumInputs: -1
});

dojo.declare("flow.operator.SortOperator", flow.operator.BaseOperator,
{
  type: "sort",

  setupPropertyEditor: function() {
    var data = this.generateData();

    var grid = {id: "sortparams", type: "grid", layout: null, data: this.generateGridData(), config: this.generateConfig()};
    grid.layout = [
                   {name: "input", label: this.i18n.SORT_HEADER_INPUT, type: "tree", pos: 0, width: "95%"},
                   {name: "value", label: this.i18n.SORT_HEADER_SORT_ORDER, width: "5%",
                    values: [{label: this.i18n.SORT_ASCENDING, value: 0},
                             {label: this.i18n.SORT_DESCENDING, value: 1}]
                   }
                  ];

    grid.defaultchild = {value: {data: "", type: "xpath"}, type: "asc"};

    data.layout.push(grid);

    return data;
  },

  addChild: function(gridid) {
    var id = this.model.generateId("cond");

    var conddata = {id: id, conjunction: "", operation: ""};
    var conditem = {id: id};

    var paramdata = [];
    var paramid = this.model.generateId("param");

    var dataitem = flow.operator.Data.create("xpath", paramid);
    dataitem.value = "";
    paramdata.push(dataitem);

    conditem.inputParamId = paramid;

    paramid = this.model.generateId("param");
    dataitem = flow.operator.Data.create("text", paramid);
    dataitem.value = 0;
    paramdata.push(dataitem);

    conditem.valueParamId = paramid;

    conddata.parameters = paramdata;
    this.createCondition(conddata);

    this.conditions[id] = conditem;

    return {id: id, input: "",  value: ""};
  }
});

dojo.declare("flow.operator.GroupbyOperator", flow.operator.BaseOperator,
{
  type: "groupby",

  setupPropertyEditor: function() {
    var data = this.generateData();

    var grid = {id: "groupbyparams", type: "grid", label: this.i18n.GROUPBY_GROUPED, labelPrefix: "GROUPBY_GROUPED", layout: null, data: [], config: this.generateConfig()}
    grid.layout = [{name: "value", pos: 0, label: this.i18n.GROUPBY_HEADER_INPUT, type: "tree", passAllData: true}];
    grid.data = this.getParameters({operatorspecific: "grouped"}, this.node);
    data.layout.push(grid);

    grid = {id: "nestedbyparams", type: "grid", label: this.i18n.GROUPBY_NESTED, labelPrefix: "GROUPBY_NESTED", layout: null, data: [], config: this.generateConfig()}
    grid.layout = [{name: "value", pos: 0, label: this.i18n.GROUPBY_HEADER_INPUT_NEST, type: "tree", passAllData: true}];
    grid.data = this.getParameters({operatorspecific: "nested"}, this.node);
    data.layout.push(grid);

    return data;
  },

  addChild: function(gridid) {
    var id = this.model.generateId("param");
    var type = (gridid == "groupbyparams") ? "grouped" : "nested";

    var paramdata = flow.operator.Data.create("xpath", id);
    paramdata.operatorspecific = type
    paramdata.value = "";

    this.createParameter(paramdata);

    return {id: id, value: ""};
  },

  setChildValue: function(id, key, value) {
    this.modifyChild(id, key, value);
  }
});

dojo.declare("flow.operator.FilterOperator", flow.operator.BaseOperator,
{
  type: "filter",

  conjunction: "and",

  setupPropertyEditor: function() {
    var data = this.generateData();

    data.layout.push({id: "satisfy", type: "custom"});

    var grid = {id: "filterparams", type: "grid", layout: null, data: this.generateGridData(), config: this.generateConfig()}
    // can't move rows
    grid.config.canMoveRows = false;

    grid.layout = [
                   {name: "input", pos: 0, label: this.i18n.FILTER_HEADER_INPUT,
                    type: "tree", width: "46%"},
                   {name: "operation", label: this.i18n.FILTER_HEADER_OPERATOR,
                    values: [{value: "<", label: this.i18n.OPERATION_LESSER_THAN},
                             {value: ">", label: this.i18n.OPERATION_GREATER_THAN},
                             {value: "=", label: this.i18n.OPERATION_EQUALS},
                             {value: "!=", label: this.i18n.OPERATION_NOT_EQUALS},
                             {value: "<=", label: this.i18n.OPERATION_LESSER_OR_EQUALS},
                             {value: ">=", label: this.i18n.OPERATION_GREATER_OR_EQUALS},
                             {value: "contains", label: this.i18n.OPERATION_CONTAINS},
                             {value: "notcontains", label: this.i18n.OPERATION_NOTCONTAINS},
                             {value: "starts-with", label: this.i18n.OPERATION_STARTS_WITH},
                             {value: "ends-with", label: this.i18n.OPERATION_ENDS_WITH}
                            ],
                    width: "8%"},
                   {name: "value", label: this.i18n.FILTER_HEADER_VALUE, options: ["text",  "function", "variable", "xpath"], type: "param", width: "46%"}
                  ];
    data.layout.push(grid);

    return data;
  },

  buildCustomValue: function(parentnode, dataitem) {
    // can only be satisfy
    var div = document.createElement("div");

    this.conjunction = this.model.getValue(this.node, "flow:condition/@conjunction");
    this.conjunction = this.conjunction ? this.conjunction : "and";

    var select = document.createElement("select");
    var option = document.createElement("option");
    option.setAttribute("value", "and");
    option.appendChild(document.createTextNode(this.i18n.FILTER_ALL));
    if (this.conjunction == "and") {
      option.setAttribute("selected", "true");
    }
    select.appendChild(option);

    option = document.createElement("option");
    option.setAttribute("value", "or");
    option.appendChild(document.createTextNode(this.i18n.FILTER_ANY));
    if (this.conjunction == "or") {
      option.setAttribute("selected", "true");
    }

    select.appendChild(option);

    div.appendChild(select);
    dojo.connect(select, "onchange", this, "onSelectChange");

    div.appendChild(document.createTextNode(" " + this.i18n.FILTER_CONDITIONS));
    div.appendChild(document.createElement("br"));

    parentnode.appendChild(div);
  },

  onSelectChange: function(event) {
    var select = event.target;
    this.conjunction = select.value;

    var conditions = this.model.runXPath("flow:condition", this.node);

    for (var i = 0; i < conditions.length; i++) {
      this.model.setValue("@conjunction", conditions[i], this.conjunction)
    }
  },

  addChild: function(gridid) {
    var id = this.model.generateId("cond");

    var conddata = {id: id, conjunction: this.conjunction, operation: "<"};

    // we store per condition the two param ids we generate for later use
    var conditem = {id: id};

    var paramid = this.model.generateId("param");

    var paramdata = [];

    var data = flow.operator.Data.create("xpath", paramid);
    data.value = "";
    paramdata.push(data);

    conditem.inputParamId = paramid;

    paramid = this.model.generateId("param");
    var data2 = flow.operator.Data.create("text", paramid);
    data2.value = "";
    paramdata.push(data2);

    conditem.valueParamId = paramid;

    conddata.parameters = paramdata;
    this.createCondition(conddata);

    this.conditions[id] = conditem;

    return {id: id, input: data, value: data2};
  }
});

dojo.declare("flow.operator.MergeOperator", flow.operator.BaseOperator,
{
  type: "merge",

  minimumInputs: 2,
  maximumInputs: 2,
  conjunction: "and",
  
  setupPropertyEditor: function() {
    var data = this.generateData();

    var src1id = this.model.getValue(this.node, "flow:input[@position='0']/@id");
    var src2id = this.model.getValue(this.node, "flow:input[@position='1']/@id");

    var src1cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src1id+"']/@caption");
    var src2cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src2id+"']/@caption");

    // first row
    data.layout.push({type: "inputselect", captions: [src1cap, src2cap], 
                      data: [src1id, src2id],
                      labels: [this.i18n.MERGE_FIRST_INPUT+" ", this.i18n.MERGE_SECOND_INPUT+" "]});

    data.layout.push({id: "satisfy", type: "custom"});

    // the grid
    var grid = {id: "mergeparams", type: "grid", layout: null, data: this.generateGridData(), config: this.generateConfig()}
    grid.layout = [
                   {name: "input", pos: 0, label: this.i18n.MERGE_HEADER_INPUT, type: "tree", width: "46%"},
                   {name: "operation", label: this.i18n.MERGE_HEADER_OPERATOR,
                    values: [{value: "<", label: this.i18n.OPERATION_LESSER_THAN},
                             {value: ">", label: this.i18n.OPERATION_GREATER_THAN},
                             {value: "=", label: this.i18n.OPERATION_EQUALS},
                             {value: "!=", label: this.i18n.OPERATION_NOT_EQUALS},
                             {value: "<=", label: this.i18n.OPERATION_LESSER_OR_EQUALS},
                             {value: ">=", label: this.i18n.OPERATION_GREATER_OR_EQUALS},
                             {value: "contains", label: this.i18n.OPERATION_CONTAINS},
                             {value: "notcontains", label: this.i18n.OPERATION_NOTCONTAINS},
                             {value: "starts-with", label: this.i18n.OPERATION_STARTS_WITH},
                             {value: "ends-with", label: this.i18n.OPERATION_ENDS_WITH}
                            ],
                    width: "8%"},
                   {name: "value", pos: 1, label: this.i18n.MERGE_HEADER_VALUE, type: "tree", width: "46%"}
                  ];
    data.layout.push(grid);

    return data;
  },

  buildCustomValue: function(parentnode, dataitem) {
    // can only be satisfy
    var div = document.createElement("div");

    this.conjunction = this.model.getValue(this.node, "flow:condition/@conjunction");
    this.conjunction = this.conjunction ? this.conjunction : "and";

    var select = document.createElement("select");
    var option = document.createElement("option");
    option.setAttribute("value", "and");
    option.appendChild(document.createTextNode(this.i18n.FILTER_ALL));
    if (this.conjunction == "and") {
      option.setAttribute("selected", "true");
    }
    select.appendChild(option);

    option = document.createElement("option");
    option.setAttribute("value", "or");
    option.appendChild(document.createTextNode(this.i18n.FILTER_ANY));
    if (this.conjunction == "or") {
      option.setAttribute("selected", "true");
    }

    select.appendChild(option);

    div.appendChild(select);
    dojo.connect(select, "onchange", this, "onSelectChange");

    div.appendChild(document.createTextNode(" " + this.i18n.FILTER_CONDITIONS));
    div.appendChild(document.createElement("br"));

    parentnode.appendChild(div);
  },

  onSelectChange: function(event) {
    var select = event.target;
    this.conjunction = select.value;

    var conditions = this.model.runXPath("flow:condition", this.node);

    for (var i = 0; i < conditions.length; i++) {
      this.model.setValue("@conjunction", conditions[i], this.conjunction)
    }
  },

  setChildValue: function(id, key, value) {
    var condition = this.conditions[id];
    if (key == "operation") {
      this.modifyCondition(id, key, value);
    } else if(key == "input") {
      value.inputid = this.model.getValue(this.node, "flow:input[@position='0']/@id");
      this.modifyChild(condition.inputParamId, "value", value);
    } else if (key == "value") {
      value.inputid = this.model.getValue(this.node, "flow:input[@position='1']/@id");
      this.modifyChild(condition.valueParamId, "value", value);
    }
  },

  addChild: function(gridid) {
    var id = this.model.generateId("cond");

    var conddata = {id: id, conjunction: this.conjunction , operation: "="};

    // we store per condition the two param ids we generate for later use
    var conditem = {id: id};

    var paramdata = [];
    var paramid = this.model.generateId("param");
    paramdata.push({id: paramid, value: {type: "xpath", data: ""}});
    conditem.inputParamId = paramid;

    paramid = this.model.generateId("param");
    paramdata.push({id: paramid, value: {type: "xpath", data: ""}});
    conditem.valueParamId = paramid;

    conddata.parameters = paramdata;
    this.createCondition(conddata);

    this.conditions[id] = conditem;

    return {id: id, input: "", value: ""};
  }
});

dojo.declare("flow.operator.ForeachOperator", flow.operator.BaseOperator,
{
  type: "foreach",

  minimumInputs: 2,
  maximumInputs: 2,

  selectNodes: null,
  variableList: null,
  conjunction: "and",

  setupPropertyEditor: function() {
    // variable init
    var src2id = this.model.getValue(this.node, "flow:input[@position='1']/@id");
    var params = this.getInscopeVaraibles(src2id, true);
    this.variableList = this.getVariableItems(params, false);

    this.selectNodes = [];

    // now build the data
    var data = this.generateData();

    var src1id = this.model.getValue(this.node, "flow:input[@position='0']/@id");
    var src2id = this.model.getValue(this.node, "flow:input[@position='1']/@id");

    var src1cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src1id+"']/@caption");
    var src2cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src2id+"']/@caption");

    // first row - input selection
    data.layout.push({type: "inputselect", captions: [src1cap, src2cap],
                      data: [src1id, src2id],
                      labels: [this.i18n.FOREACH_ITEM+" ", this.i18n.FOREACH_EXECUTE+" "]});

    // grid - pick up the tree from input 0
    var inputlabel, valuelabel;
    if (src2cap) {
      inputlabel = dojo.string.substitute(this.i18n.FOREACH_HEADER_INPUT, [src2cap]);
    }

    if (src1cap) {
      valuelabel = dojo.string.substitute(this.i18n.FOREACH_HEADER_VALUE, [src1cap]);
    }

    var grid = {id: "foreachparams", type: "grid", layout: null,
                data: this.generateGridData(), config: this.generateConfig()}
    grid.layout = [{name: "input", label: inputlabel, type: "custom", callback: dojo.hitch(this, this.buildSelect)},
                   {name: "value", pos: 0, label: valuelabel, type: "tree"}
                  ];

    grid.config.canAddRows = true;
    grid.config.canMoveRows = false;
    grid.config.maxRows = this.variableList.length;
    data.layout.push(grid);

    return data;
  },

  setChildValue: function(id, key, value) {
    var condition = this.conditions[id];

    if (key == "input") {
      this.modifyChild(condition.inputParamId , "value", {type:"variable",value:value});
    } else {
      value.inputid = this.model.getValue(this.node, "flow:input[@position='0']/@id");
      this.modifyChild(condition.valueParamId, "value", value);
    }
  },

  buildSelect: function(parentnode, dataitem) {
    var div = document.createElement("div");

    var select = document.createElement("select");
    select.setAttribute("rowid", dataitem.id);
    this.updateSelect(select, null, dataitem.input.value);

    div.appendChild(select);
    dojo.connect(select, "onchange", this, "onSelectChange");

    this.selectNodes.push(select);

    parentnode.appendChild(div);
  },

  onSelectChange: function(event) {
    var select = event.target;
    var value = select.value;

    // notify model
    this.setChildValue(select.getAttribute("rowid"), "input", value);

    // need to rebuild the other selects
    var selectedValues = this.getSelectedValues();

    for (var i = 0; i < this.selectNodes.length; i++) {
      if (this.selectNodes[i] != select) {
        this.updateSelect(this.selectNodes[i], selectedValues);
      }
    }
  },

  getSelectedValues: function() {
    var selectedValues = [];

    var params = this.model.runXPath(".//flow:condition/flow:param[@position='0']", this.node);
    for (var i = 0; i < params.length; i++) {
      var value = this.model.getValue(params[i], "text()");
      if (value) {
        selectedValues[value] = true;
      }
    }

    return selectedValues;
  },

  updateSelect: function(select, selectedValues, defaultValue) {
    if (!selectedValues) {
      selectedValues = this.getSelectedValues();
    }

    var selectedValue = select.value;
    if (defaultValue) {
      selectedValue = defaultValue;
    }

    // clear select
    for (var i = select.options.length-1; i >= 0; i--) {
      select.removeChild(select.options[i]);
    }

    var option = document.createElement("option");
    select.appendChild(option);

    for (var i = 0; i < this.variableList.length; i++) {
      var item = this.variableList[i];

      // skip if already selected, but not if it is our selected value
      if (selectedValues[item.id] && selectedValue != item.id) {
        continue;
      }

      var option = document.createElement("option");
      option.setAttribute("value", item.id);
      option.appendChild(document.createTextNode(item.label));

      if (item.id == selectedValue) {
        option.setAttribute("selected", true);
      }

      select.appendChild(option);
    }
  },

  createVariableDropDown: function() {
    var src2id = this.model.getValue(this.node, "flow:input[@position='1']/@id");
    var params=this.getInscopeVaraibles(src2id,true);
    var itemlist=this.getVariableItems(params, true);
    this.variableList = itemlist;
    var values = new Array();
    var j = 0;
    for (var i = 0; i < itemlist.length; i++) {
      var item = itemlist[i];
      values[j++]={label: item.label, value: item.id};
    }
    return values;
  },

  firstInputSelected: function() {
    try {
      if (this.grid) {
        this.grid.grid.layout[0].values = this.createVariableDropDown();
        this.grid.clearGrid();

        // clear all data
        this.model.clearOperatorData(this.id);

        // add dummy first row
        this.grid.addNewRow();

        var src1id = this.model.getValue(this.node, "flow:input[@position='0']/@id");
        var src2id = this.model.getValue(this.node, "flow:input[@position='1']/@id");
        var src1cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src1id+"']/@caption");
        var src2cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src2id+"']/@caption");

        var inputlabel, valuelabel;
        if (src2cap) {
          this.grid.grid.layout[0].label = dojo.string.substitute(this.i18n.FOREACH_HEADER_INPUT, [src2cap]);
        }

        if (src1cap) {
          this.grid.grid.layout[1].label = dojo.string.substitute(this.i18n.FOREACH_HEADER_VALUE, [src1cap]);
        }

        this.grid.buildHeaders();
      }
    } catch(ex){}

    this.selectNodes = new Array();
  },

  addChild: function(gridid) {
    var id = this.model.generateId("cond");

    var conddata = {id: id, conjunction: this.conjunction, operation: ""};

    // we store per condition the two param ids we generate for later use
    var conditem = {id: id};

    var paramdata = [];
    var paramid = this.model.generateId("param");
    paramdata.push({id: paramid, value: {type: "text", data: ""}});
    conditem.inputParamId = paramid;

    paramid = this.model.generateId("param");
    paramdata.push({id: paramid, value: {type: "xpath", data: ""}});
    conditem.valueParamId = paramid;

    conddata.parameters = paramdata;
    this.createCondition(conddata);

    this.conditions[id] = conditem;

    return {id: id, input: "", value: ""};
  }
});

dojo.declare("flow.operator.TransformOperator", flow.operator.BaseOperator,
{
  type: "transform",

  minimumInputs: 1,
  maximumInputs: 1,

  sourceTree: null,
  targetTree: null,
  menuNode: null,

  setupPropertyEditor: function() {
    if (this.model.getNode("flow:template", this.node) == null) {
      var template = '<template xmlns="'+this.model.flowNS+'"><mhub:entry xmlns=""  xmlns:mhub="'+this.model.mhubNS+'"/></template>';
      this.node.appendChild(this.model.parseXML(template));
    }

    var data = this.generateData();
    data.layout.push({id:"transform", type: "custom"});
    return data;
  },

  buildCustomValue: function(parentnode, dataitem) {
    var div = document.createElement("div");
    parentnode.appendChild(div);

    dojo.require("flow.dijit.Transform");
    this.transformWidget = new flow.dijit.Transform({operator: this}, div);
  },

  dragDrop: function(srctreenode, targettreenode) {
    this.transformWidget.dragDrop(srctreenode, targettreenode);
  }
});


dojo.declare("flow.operator.PublishOperator", flow.operator.BaseOperator,
{
  type: "publish",
  hasOutput: false,
  canRename: false,

  propertiesWidget: null,

  setupPropertyEditor: function() {
    var data = this.generateData();
    data.layout.push({id:"publish", type: "custom"});
    return data;
  },

  buildCustomValue: function(parentnode, dataitem) {
    var div = document.createElement("div");
    parentnode.appendChild(div);

    dojo.require("flow.dijit.Properties");
    this.propertiesWidget = new flow.dijit.Properties({model: this.model}, div);
  },

  inputRemoved: function(srcOperator) {
    // publish is special - only remove the input, not all params
    this.model.removeInput(this.node, srcOperator.id);

    return true;
  }
});

