/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.command.EditCommand");

// ----------------------------------------------------------------------------
// Command classes which encapsulate a request as an object
// ----------------------------------------------------------------------------

btp.widget.command.EditCommand = {

	/*
	 * appendChild Command
	 */
	AppendChildCommand: function(parent, child){
		this._parent = parent;
		this._child = child;
	
		this.execute = function(){
			this._parent.hlp_addChild(this._child);
		}
	
		this.undo = function(){
			this._parent.hlp_removeChild(this._child);
		}
	},

	/*
	 * insertBefore Command
	 */
	InsertBeforeCommand: function(parent, newChild, refChild){
		this._parent = parent;
		this._newChild = newChild;
		this._refChild = refChild;

		this.execute = function(){
			this._parent.hlp_addChild(this._newChild, this._refChild);
		}
	
		this.undo = function(){
			this._parent.hlp_removeChild(this._newChild);
		}
	},

	/*
	 * removeChild Command
	 */
	RemoveChildCommand: function(oldChild){
		this._oldChild = oldChild;
	
		this.execute = function(){
			this._parent = this._oldChild.getParent();
			this._refChild = this._oldChild.getNextSibling();
			this._parent.hlp_removeChild(this._oldChild);
		}
	
		this.undo = function(){
			if(this._refChild){
				this._parent.hlp_addChild(this._oldChild, this._refChild);
			}else{
				this._parent.hlp_addChild(this._oldChild);
			}
		}
	},

	/*
	 * replaceChild Command
	 */
	ReplaceChildCommand: function(parent, newChild, refChild){
		this._parent = parent;
		this._newChild = newChild;
		this._refChild = refChild;
	
		this.execute = function(){
			this._parent.hlp_addChild(this._newChild, this._refChild);
			this._parent.hlp_removeChild(this._refChild);
		}
	
		this.undo = function(){
			this._parent.hlp_addChild(this._refChild, this._newChild);
			this._parent.hlp_removeChild(this._newChild);
		}
	},

	/*
	 * changeName Command (for change operator)
	 */
	ChangeNameCommand: function(target, newName){
		this._target = target;
		this._newName = newName;
	
		this.execute = function(){
			this._oldName = this._target.nodeName;
			this._target.nodeName = this._newName;
		}
	
		this.undo = function(){
			this._target.nodeName = this._oldName;
		}
	},

	/*
	 * move Command
	 */
	MoveCommand: function(widget, /*Object*/toDim, /*Object?*/fromDim){
		// Type of toDim/fromDim: {top:"#px", left:"#px"}
		this._widget = widget;
		this._toDim = toDim;
		this._fromDim = fromDim;
	
		this.execute = function(){
			if(!this._fromDim){
				this._fromDim = {
					top: btp.getStyleValue(this._widget, "top") + "px",
					left: btp.getStyleValue(this._widget, "left") + "px"
				};
			}
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._toDim.top;
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._toDim.left;
			}
			this._widget.hlp_setStyles(props);
		}
	
		this.undo = function(){
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._fromDim.top;
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._fromDim.left;
			}
			this._widget.hlp_setStyles(props);
		}
	},

	/*
	 * resize Command
	 */
	ResizeCommand: function(widget, toDim){
		this._widget = widget;
		this._toDim = toDim;
	
		this.execute = function(){
			this._fromDim = {
				top: btp.getStyleValue(this._widget, "top") + "px",
				left: btp.getStyleValue(this._widget, "left") + "px",
				width: btp.getStyleValue(this._widget, "width") + "px",
				height: btp.getStyleValue(this._widget, "height") + "px"
			};
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._toDim.top;
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._toDim.left;
			}
			if(this._fromDim.width != this._toDim.width){
				props["width"] = this._toDim.width;
			}
			if(this._fromDim.height != this._toDim.height){
				props["height"] = this._toDim.height;
			}
			this._widget.hlp_setStyles(props);
		}
	
		this.undo = function(){
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._fromDim.top;
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._fromDim.left;
			}
			if(this._fromDim.width != this._toDim.width){
				props["width"] = this._fromDim.width;
			}
			if(this._fromDim.height != this._toDim.height){
				props["height"] = this._fromDim.height;
			}
			this._widget.hlp_setStyles(props);
		}
	},

	/*
	 * setPropertyValue Command
	 */
	SetPropertyValueCommand: function(widget, /*String|Object*/attribute, value){
		// Note: ATTRIBUTE should be String, but accepts Object for backward compatibility.
		this._widget = widget;
		this._attribute = attribute;
		this._value = value;
	
		this.execute = function(){
			var attrName = typeof(this._attribute) == "string" ? this._attribute : this._attribute.name;
			this._oldValue = this._widget.ps_getPropertyValue(attrName);
			this._widget.ps_setPropertyValue(attrName, this._value);
		}
	
		this.undo = function(){
			var attrName = typeof(this._attribute) == "string" ? this._attribute : this._attribute.name;
			this._widget.ps_setPropertyValue(attrName, this._oldValue);
		}
	},

	/*
	 * setStyle Command
	 */
	SetStyleCommand: function(widget, styleName, value){
		this._widget = widget;
		this._styleName = styleName;
		this._value = value;
	
		this.execute = function(){
			if(this._styleName == "top" || this._styleName == "left"
				|| this._styleName == "width" || this._styleName == "height"){
				this._oldValue = btp.getStyleValue(this._widget, this._styleName) + "px";
			}else{
				this._oldValue = dojo.style(this._widget.domNode, this._styleName);
			}
			this._widget.hlp_setStyle(this._styleName, this._value);
		}
	
		this.undo = function(){
			this._widget.hlp_setStyle(this._styleName, this._oldValue);
		}
	},

	/*
	 * update Command
	 */
	UpdateCommand: function(widget){
		this._widget = widget;
	
		this.execute = function(){
			this._widget.update();
		}
	
		this.undo = this.execute;
	}
};
