/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.ext.selectionHandles");

// summary:
//		Extension for creating selection handles for a component on the EditArea.
//		Note that this is not a widget.
//		this.owner points to the EditArea.

dojo.require("btp.dnd.DragMoveSource");
dojo.require("btp.dnd.DragManager");

dojo.declare(
	"btp.widget.ext.selectionHandles",
	null,
{
	owner: null,
	selectionKnobType: "blackSquare",

	// constants
	_NORTH: 1,
	_SOUTH: 2,
	_WEST: 4,
	_EAST: 8,

	// initializer
	constructor: function(owner){
		this.owner = owner;
		this._connectors = [];
		this._connectors.push(dojo.connect(owner, "setFocusFrame", this, "addSelectionHandles"));
		this._connectors.push(dojo.connect(owner, "removeFocusFrame", this, "removeSelectionHandles"));
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
	},

	/*              NORTH
	 *       0 ------ 1 ------ 2
	 *       |                 |
	 *  WEST 7                 3 EAST
	 *       |                 |
	 *       6 ------ 5 ------ 4
	 *              SOUTH
	 */
	addSelectionHandles: function(widget){
		if(!widget){
			widget = this.owner.focusWidgets[0];
		}
		if(!widget
			|| widget.domNode.offsetWidth == 0 || widget.domNode.offsetHeight == 0){ // Maybe it is a gfx line widget
			return;
		}
		var info = widget.hlp_getComponentInfo();
		if(!info.resizable){ return; }
		if(widget.__showingSelectionHandles){
			; // already showing
		}else if(!widget.__selectionHandles){
			this.createSelectionHandles(widget);
		}else{
			for(var i = 0, len = widget.__selectionHandles.length; i < len; i++){
				this.owner.domNode.appendChild(widget.__selectionHandles[i]);
			}
		}
		widget.__showingSelectionHandles = true;

		var dim = btp.getStyleDimensions(widget, this.owner);
		var north = dim.top - 4 + "px";
		var south = dim.top + dim.height - 2 + "px";
		var west = dim.left - 4 + "px";
		var east = dim.left + dim.width - 2 + "px";
		var center = dim.left + Math.round(dim.width/2) - 3 + "px";
		var middle = dim.top + Math.round(dim.height/2) - 3 + "px";

		var knob;
		var i = 0;

		// NORTH-WEST
		knob = widget.__selectionHandles[i++];
		knob.style.top = north;
		knob.style.left = west;

		// NORTH
		knob = widget.__selectionHandles[i++];
		knob.style.top = north;
		knob.style.left = center;

		// NORTH-EAST
		knob = widget.__selectionHandles[i++];
		knob.style.top = north;
		knob.style.left = east;

		// EAST
		knob = widget.__selectionHandles[i++];
		knob.style.top = middle;
		knob.style.left = east;

		// SOUTH-EAST
		knob = widget.__selectionHandles[i++];
		knob.style.top = south;
		knob.style.left = east;

		// SOUTH
		knob = widget.__selectionHandles[i++];
		knob.style.top = south;
		knob.style.left = center;

		// SOUTH-WEST
		knob = widget.__selectionHandles[i++];
		knob.style.top = south;
		knob.style.left = west;

		// WEST
		knob = widget.__selectionHandles[i++];
		knob.style.top = middle;
		knob.style.left = west;
	},

	removeSelectionHandles: function(widget){
		if(!widget){
			widget = this.owner.focusWidgets[0];
		}
		if(!widget){
			return; // removeFocus() was called when nothing is focused.
		}
		if(!widget.__showingSelectionHandles){
			return;
		}
		for(var i = 0, len = widget.__selectionHandles.length; i < len; i++){
			this.owner.domNode.removeChild(widget.__selectionHandles[i]);
		}
		widget.__showingSelectionHandles = false;
	},

	createSelectionHandles: function(widget){
		var moveTarget = this.owner.getMoveTarget();
		var knobElem = this.owner.createKnob(this.selectionKnobType);

		var info = widget.hlp_getComponentInfo();
		var dirs = 0;
		if(info.resizable.directions){
			var arr = info.resizable.directions.split("|");
			for(var i = 0; i < arr.length; i++){
				if(arr[i] == "NORTH"){
					dirs |= this._NORTH;
				}else if(arr[i] == "SOUTH"){
					dirs |= this._SOUTH;
				}else if(arr[i] == "WEST"){
					dirs |= this._WEST;
				}else if(arr[i] == "EAST"){
					dirs |= this._EAST;
				}else{
					alert("unrecognized direction: "+arr[i]);
				}
			}
		}
		widget.__resizeDirections = dirs; // bitwise

		widget.__selectionHandles = [];
		var knob;
		var _this = this;
		for(var i = 0; i < 8; i++){
			knob = knobElem.cloneNode(true);
			knob.knobPosition = i; // [custom property]
			knob.getDragTracker = function(){ return _this.owner.resizeTrackerClass; };
			widget.__selectionHandles.push(knob);
			this.owner.domNode.appendChild(knob);

			var ds = new btp.dnd.DragMoveSource(knob);
			ds.setDragTarget(moveTarget);
			
			switch (i){
				case 0:
					knob.style.cursor = "nw-resize"; break;
				case 1:
					knob.style.cursor = "n-resize";
					if(!(dirs & this._NORTH)){
						knob.style.display = "none";
					}
					break;
				case 2:
					knob.style.cursor = "ne-resize"; break;
				case 3:
					knob.style.cursor = "e-resize";
					if(!(dirs & this._EAST)){
						knob.style.display = "none";
					}
					break;
				case 4:
					knob.style.cursor = "se-resize"; break;
				case 5:
					knob.style.cursor = "s-resize";
					if(!(dirs & this._SOUTH)){
						knob.style.display = "none";
					}
					break;
				case 6:
					knob.style.cursor = "sw-resize"; break;
				case 7:
					knob.style.cursor = "w-resize";
					if(!(dirs & this._WEST)){
						knob.style.display = "none";
					}
					break;
			}
		}
	}
});
