/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("btp.widget.outline.TreeContentOutlinePage");
dojo.require("btp.widget.SimpleTree");

// summary:
//		The default implementation of OutlinePage, which displays a structure
//		of some model provided with a ContentProvider in the OutlineView.
//		Your application may want to replace this with yours.
//
//	interface:
//		setContentProvider(/*Widget*/provider)
//		setOutlineView(/*Widget*/view)
//		createOutlinePage()
//		attachOutline()
//		detachOutline()
//		refreshOutline()
//		updateOutline(/*Object*/message)
//		selectOutline(/*Widget*/selectedWidget)
//		selectOutlineTarget(/*Widget*/outlineNode)
//		deleteOutlineTarget()

dojo.declare("btp.widget.outline.TreeContentOutlinePage", null,
{
	editArea: null,
	outlineView: null,
	contentProvider: null,
	outlineViewMenu: null,
	tree: null,
	_connectors: null,
	_topics: null,

	imgDir: dojo.moduleUrl("btp", "widget/templates/OutlineView/"),

	constructor: function(editArea){
		this.editArea = editArea;
		this._connectors = [];
		this._topics = [
			dojo.subscribe("btp:TreeNodeSelected", this, "_treeNodeSelected")
		];
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	setContentProvider: function(/*Widget*/provider){
		this.contentProvider = provider;
	},

	setOutlineView: function(/*Widget*/view){
		this.outlineView = view;
		if(!this._oncontextmenu_connected){
			this._connectors.push(dojo.connect(view.domNode, "oncontextmenu", this, "_onOutlineViewContextMenu"));
			this._oncontextmenu_connected = true;
		}
	},

	createOutlinePage: function(){
		if(!this.tree){
			this.tree = new btp.widget.SimpleTree();
			this.outlineView.addChild(this.tree);
		}else{
			this.detachOutline();
		}

		// if you want to have your own content provider, you should call setContentProvider() first.
		if(!this.contentProvider){
			var providerClassName = this.editArea.getContentProviderClass();
			if(providerClassName){
				dojo["require"](providerClassName);
				impl = dojo.getObject(providerClassName);
				this.setContentProvider(new impl());
			}
		}
	},

	attachOutline: function(){
		if(this.outlineView && this.tree){
			this.outlineView.addChild(this.tree);
			return true;
		}
		return false;
	},

	detachOutline: function(){
		if(this.outlineView && this.tree){
			this.outlineView.removeChild(this.tree);
		}
	},

	refreshOutline: function(){
		this.tree.destroyDescendants();
		var root = this.contentProvider.getElements(this.editArea);
		for(var i = 0, len = root.length; i < len; i++){
			this._refreshTreeView(null, root[i]);
		}
	},

	_createIconNode: function(opt){
		var props = {
			label: opt.label,
			tooltip: opt.tooltip,
			iconImage: this.imgDir + "element.gif"
		};
		var node = new btp.widget.SimpleTreeNode(props);
		return node;
	},

	_refreshTreeView: function(parentObj, obj){
		var node = obj.outlineNode = this._createIconNode(obj);
		node.outlineObj = obj;
		var parentNode = (parentObj && parentObj.outlineNode) ? parentObj.outlineNode : this.tree;
		parentNode.addChild(node);

		var children = this.contentProvider.getChildren(obj);
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			this._refreshTreeView(obj, child);
		}
	},

	updateOutline: function(message){
		var root = this.contentProvider.getElements(this.editArea);
		this._updateTreeView(root, this.tree);
	},

	_updateTreeView: function(objs, /*SimpleTreeNode*/parentNode){
		var nObjs = objs.length;
		var nNodes = parentNode.getChildren().length;
		if(nObjs == nNodes){
			// no changes
			for(var i = 0, len = objs.length; i < len; i++){
				var chlidObjs = this.contentProvider.getChildren(objs[i]);
				this._updateTreeView(chlidObjs, parentNode.getChildren()[i]);
			}
		}else if(nObjs > nNodes){
			// a new node has been added
			for(var i = 0, len = nObjs; i < len; i++){
				var obj = objs[i];
				var node = parentNode.getChildren()[i];
				if(!node || node.outlineObj != obj){ // newly added node found
					var parentObj = this.contentProvider.getParent(obj);
					this._refreshTreeView(parentObj, obj);
					break;
				}
			}
		}else{
			// a node has been removed
			for(var i = 0, len = nNodes; i < len; i++){
				var obj = objs[i];
				var node = parentNode.getChildren()[i];
				if(!obj || obj.outlineNode != node){ // removed node found
					node.destroy();
					break;
				}
			}
		}
	},

	// select a node in the OutlineView
	selectOutline: function(/*Widget*/selectedWidget){
		var node;
		if(selectedWidget){
			node = selectedWidget.outlineObj.outlineNode;
			this.tree.select(node);
		}else{ // BODY was selected on the canvas
			this.tree.deselectAll();
		}
	},


	// select a widget in the EditArea
	selectOutlineTarget: function(/*Widget*/outlineNode){
		var targetNode = outlineNode.outlineObj.targetNode;
		if(targetNode == this.editArea){
			this.editArea.setFocus(null); // BODY was clicked
		}else if(targetNode instanceof dijit._Widget){
			this.editArea.setFocus(targetNode);
		}
	},

	deleteOutlineTarget: function(){
		var treeNodeWidget = this.tree.getSelectedNode();
		var targetNode = treeNodeWidget.outlineObj.targetNode;
		if(targetNode == this.editArea){
			this.editArea.setFocus(null); // BODY was clicked
		}else if(targetNode instanceof dijit._Widget){
			this.editArea.setFocus(targetNode);
			this.editArea.deleteFocusWidget();
		}
	},


	_onOutlineViewContextMenu: function(e){
		if(!this.tree.getParent()){ return; }
		var elem = e.target; // clicked element
		if(elem.tagName != "SPAN"){
			dojo.stopEvent(e);
			return;
		}
		if(!this.outlineViewMenu){
			dojo["require"]("dijit.Menu");
			var outlineViewMenu = this.outlineViewMenu = new dijit.Menu();
			var mItem = new dijit.MenuItem({label:"Delete"});
			mItem.iconNode.style.backgroundImage = "url("+dojo.moduleUrl("btp", "widget/templates/Toolbar/delete.gif")+")";
			this._connectors.push(dojo.connect(mItem, "onClick", this, "deleteOutlineTarget"));
			outlineViewMenu.addChild(mItem);
			outlineViewMenu.startup();
		}
		this.outlineViewMenu._openMyself(e);
	},

	_treeNodeSelected: function(message){
		if(message.src != this.tree){ return; }
		var treeNodeWidget = message.selection;
		this.selectOutlineTarget(treeNodeWidget);
	}
});
