/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc.widget.GfxWidget");
dojo.require("dijit._Widget");
dojo.require("dijit._Container");
dojo.require("dojox.gfx");

// for widgets that is added to common gfx surface

// Structure of container-type nodes:
// +-----------------+
// | groupBox        |
// |    +-------+    |
// |    | shape |    |
// |    +-------+    |
// |    (child1)     |
// |    (child2)     |
// |    ...          |
// +-----------------+

// Structure of non-container-type nodes:
// +-------+
// | shape |
// +-------+

dojo.declare(
	"gdc.widget.GfxWidget",
	[dijit._Widget, dijit._Contained],
{
	shape: null, // gfx Shape object
	groupBox: null, // gfx Group object that contains 'shape' and children
	parentGroupBox: null, // dojox.gfx.Surface or dojox.gfx.Group to add this shape

	createShapeCallbacks: null, // array of functions to be called at the end of createShape()
	restoreShapeCallbacks: null, // array of functions to be called at the end of restoreShape()
	updateCallbacks: null, // array of functions to be called at the end of update()
	removeShapeCallbacks: null, // array of functions to be called at the end of removeShape()

	postCreate: function(){
		for(var n = this.domNode; n; n = n.parentNode){
			if(n.tagName == "BODY"){
				// BODY found means this widget is online, i.e. this widget
				// was instantiated with markup, not by the new operator.
				this.createShape(this.getParent().getShape());
				this.update();
				break;
			}
		}
	},

	startup: function(){
		var children = this.getChildren ? this.getChildren() : null;
		if(children){
			for(var i = 0, len = children.length; i < len; i++){
				children[i].startup();
			}
		}
	},

	// subclass may override
	createShape: function(parentGroupBox){
		this.parentGroupBox = parentGroupBox;
		if(!this.shape){ // create new
			if(this.isContainer){
				this.groupBox = parentGroupBox.createGroup();
				this.shape = this.createTargetShape(this.groupBox);
			}else{
				this.shape = this.createTargetShape(parentGroupBox);
			}
			if(this.hlp_setupDropTarget){
				this.hlp_setupDropTarget();
			}

			// assign this widget to the top level dom node
			// so that the widget can be found from dom node. (See btp._base.utils.getWidgetByElement)
			this.shape.rawNode.widget = this; // [custom property]

			// There may be pending shapes, which are created offline.
			var children = this.getChildren ? this.getChildren() : null;
			if(!children){
				this._callCallbacks(this.createShapeCallbacks);
				return;
			}
			for(var i = 0, len = children.length; i < len; i++){
				var child = children[i];
				if(child instanceof gdc.widget.GfxWidget){
					child.createShape(this.getShape());
				}
			}
		}else if(!this.getShape().getParent()){
			this.restoreShape();
		}/*else{
			// shape exists and it is online
		}*/
		this._callCallbacks(this.createShapeCallbacks);
	},

	// restore the removed shape
	restoreShape: function(){
		this.parentGroupBox.add(this.getShape());
		this.recoverLostInfo(this.shape);
		this._callCallbacks(this.restoreShapeCallbacks);
	},

	// subclass should implement
	update: function(props){
		if(props){
			dojo.mixin(this, props);
			var s = this.domNode.style;
			if(props.top){ s.top = props.top + "px"; }
			if(props.left){ s.left = props.left + "px"; }
			if(props.width){ s.width = props.width + "px"; }
			if(props.height){ s.height = props.height + "px"; }
		}
		this._callCallbacks(this.updateCallbacks);
	},

	updateRecursively: function(){
		this.update();
		var children = this.getChildren ? this.getChildren() : null;
		if(!children){ return; }
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			if(child instanceof gdc.widget.GfxWidget){
				child.update();
				child.updateRecursively();
			}
		}
	},

	removeShape: function(){
		var shape = this.getShape();
		var ret = shape ? shape.removeShape() : null;
		this._callCallbacks(this.removeShapeCallbacks);
		return ret;
	},

	// this retrieves the containing gfx shape that can be used for remove/move etc.
	getShape: function(){
		if(this.isContainer){
			return this.groupBox;
		}else{
			return this.shape;
		}
	},

	// this retrieves the target gfx shape that can be used for hit test
	getTargetShape: function(){
		return this.shape;
	},

	// subclass must implement
	createTargetShape: function(parent){
		console.debug("createTargetShape not implemented!!");
	},

	recoverLostInfo: function(shape){
		if(dojo.isIE){
			// to recover the lost information. VML's bug.
			shape.setTransform(shape.matrix).setFill(shape.fillStyle).setStroke(shape.strokeStyle);
		}
	},

	// event must be either "createShape", "restoreShape", "update", or "removeShape"
	registerCallback: function(/*String*/event, /*Function*/method){
		if(!this[event+"Callbacks"]){
			this[event+"Callbacks"] = [];
		}
		if(dojo.indexOf(this[event+"Callbacks"], method) != -1){ return; } // already registered
		this[event+"Callbacks"].push(method);
	},

	_callCallbacks: function(callbacks){
		if(!callbacks){ return; }
		for(var i = 0, len = callbacks.length; i < len; i++){
			callbacks[i].call(this);
		}
	}
});

