/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("gdc.widget._GfxTemplated");

dojo.require("dojox.data.dom");
dojo.require("dojo.string");

/*
 * Creates a gfx shape from a template SVG file
 * Connection anchors can be added by adding attribute named "gdcAnchor" in an element in SVG
 * if gdcAnchor="source", the corresponding shape becomes source anchor for connection
 * if gdcAnchor="target", the corresponding shape becomes target anchor for connection
 * if gdcAnchor="both", the corresponding shape becomes source and target anchor for connection
 */
dojo.declare(
	"gdc.widget._GfxTemplated",
	null,
{
	// templatePath: String
	//	Path to template (SVG file) for this widget
	templatePath: null,
	anchor: "gdc.widget.anchor.FixedAnchor",
	
	_tagName2Type: {
		"rect": "rect",
		"ellipse": "ellipse",
		"path": "path",
		"polyline": "polyline",
		"text": "text"
	},
	
	_sourceAnchors: [],
	_targetAnchors: [],
	
	createTargetShape: function gdc_widget__GfxTemplated_createTargetShape(parent){
		var txt = dojo._getText(this.templatePath);
		var doc = dojox.data.dom.createDocument(txt, "text/xml");
		var shape = this._createChildShapes(doc, parent);

		// if anchor is defined, create connection anchors
		if(this._sourceAnchors.length > 0 || this._targetAnchors.length > 0){
			this.createConnectionAnchors();
		}
		return shape;
	},
	
	_createChildShapes: function(node, parent){
		var shape = this.createShapeFromNode(node, parent);
		if(node.nodeType === 9 /* Document */){
			shape = parent;
		}
		var self = this;
		dojo.forEach(node.childNodes, function(child){
			self._createChildShapes(child, shape);
		});
		return shape;
	},
	
	createShapeFromNode: function(node, parent){
		var shape = null;
		if(parent && node.nodeType === 1 /*ELEMENT_NODE*/){
			// create properties
			var props = { type: this._tagName2Type[node.nodeName] };
			var attrs = node.attributes;
			var fill = {};
			var stroke = {};
			var matrix = null;
			var attachPointName = null;
			var isSourceAnchor = false;
			var isTargetAnchor = false;
			dojo.forEach(attrs, dojo.hitch(this, function(attr){
				if(attr.nodeType == 2 /*ATTRIBUTE_NODE*/ && (attr.nodeName.indexOf("dojo") != 0)){
					switch(attr.nodeName){
						case "fill":
							fill = attr.nodeValue;
							break;
						case "stroke":
							stroke = attr.nodeValue;
							break;
						case "transform":
							matrix = this._generateMatrix(attr.nodeValue);
							break;
						case "gdcAttachPoint": // custom attribute
							attachPointName = attr.nodeValue;
							break;
						case "gdcAnchor": // custom attribute
							isSourceAnchor = (attr.nodeValue == "source" || attr.nodeValue == "both");
							isTargetAnchor = (attr.nodeValue == "target" || attr.nodeValue == "both");
							break;
						case "x":
						case "y":
						case "width":
						case "height":
						case "cx":
						case "cy":
						case "rx":
						case "ry":
							props[attr.nodeName] = parseInt(attr.nodeValue);
							break;
						default:
							props[attr.nodeName] = attr.nodeValue;
					}
				}
			}));
			if(node.nodeName == "g"){
				shape = parent.createGroup().setTransform(matrix);
			}else{
				shape = parent.createShape(props).setFill(fill).setStroke(stroke).setTransform(matrix);
			}
			if(attachPointName){
				this[attachPointName] = shape;
			}
			if(isSourceAnchor){
				shape.rawNode.widget = this; // [custom property]
				this._sourceAnchors.push(shape);
			}
			if(isTargetAnchor){
				shape.rawNode.widget = this; // [custom property]
				this._targetAnchors.push(shape);
			}
		}
		return shape;
	},
	
	_generateMatrix: function(str){
		if(str){
			var trim = dojo.string.trim;
			str = trim(str);
			var prefix = "matrix(";
			var index1 = str.indexOf(prefix);
			var index2 = str.indexOf(")");
			var len = str.length;
			if(index1 == 0 && index2 == len-1){
				var len2 = prefix.length;
				var str2 = str.substring(len2, len-len2+6);
				var arr = str2.split(",");
				return {
					xx: parseInt(trim(arr[0])),
					xy: parseInt(trim(arr[1])),
					yx: parseInt(trim(arr[2])),
					yy: parseInt(trim(arr[3])),
					dx: parseInt(trim(arr[4])),
					dy: parseInt(trim(arr[5]))
				}
			}
		}
		return {xx: 1, xy: 0, yx: 0, yy: 1, dx: 0, dy: 0};
	},
	
	createConnectionAnchors: function(){
		dojo["require"](this.anchor);
		var anchorImpl = dojo.getObject(this.anchor);

		// input anchors
		dojo.forEach(this._targetAnchors, dojo.hitch(this, function(anchorShape){
			var anchor = new anchorImpl(this);
			anchor.endPointShape = anchorShape;
			this.registerTargetConnectionAnchor(this.getShape().rawNode, "input", anchor);
		}));

		// input anchors
		dojo.forEach(this._sourceAnchors, dojo.hitch(this, function(anchorShape){
			var anchor = new anchorImpl(this);
			anchor.endPointShape = anchorShape;
			this.registerSourceConnectionAnchor(this.getShape().rawNode, "output", anchor);
		}));
	}
});