/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.behaviors');

/**
 * Renderable view behavior. Used by widgets that can render or filter 
 * information for another view.
 */
dojo.declare("hub.behaviors.Renderable", null,  {
    /**
     * Accepts arbitrary data and returns a result. The result may serve as
     * the parameter to another renderable's render method so it should 
     * typically be in the same format as data.
     */
    render: function(data) {
        // default is noop
        return data;
    },

    /**
     * Invoked if an error occurs.
     */
    renderError: function(err) {
        return err;
    }
});

/**
 * Stacked view behavior. Shows only one element at a time while all others are
 * hidden from view.
 */
dojo.declare('hub.behaviors.Stack', null, {
    /**
     * @param elementsIn: Array of id or elements that are stacked
     * @param visibleIdx: First visible index number or id
     * @param display: Display style to use when shown (inline, block, ...)
     */
     constructor: function(elementsIn, visibleIdx, display) {
         this.elements = new Array();
         this.ids = new Object();
         this.currentVisible = null;
         
         if (!display) {
             // style to set when displaying (e.g. inline, block)
             this.display = 'block';
         } else {
             this.display = display;
         }
         
         for(var i=0; i<elementsIn.length; i++) {
             if(dojo.isString(elementsIn[i])) {
                 // key: id, value: index
                 this.ids[elementsIn[i]] = i;
        	 elementsIn[i] = dojo.byId(elementsIn[i]);
	     } else {
                 // key: id, value: index
                 this.ids[elementsIn[i].id] = i;
             }
             
             // always add, even if null, so indices don't collapse
             this.elements.push(elementsIn[i]);		
             
             if(elementsIn[i]) {
        	 // always start hidden
        	 elementsIn[i].style.display = 'none';
             }
         }
         
         // show the initial panel
         this.show(visibleIdx || 0);
     },

    /**
     * Show the element at the given index or with the given id.
     *
     * @param idxIn: Integer index or string id
     * @return: Index of last visible element or null if no action taken
     */
    show : function(idxIn) {
        if(dojo.isString(idxIn)) {
            // if we have a string, look up the corresponding index
            try {
                idxIn = this.ids[idxIn];
            } catch(e) {
                // no match for name
                return null;
            }
        }

        if(this.currentVisible == idxIn || 
           idxIn < 0 || idxIn > this.elements.length-1) {
            // no match for index, or already shown
            return null;
        }

        // hide the old object
        var idxOld = this.currentVisible;
        try {
            var e = this.elements[this.currentVisible];
            e.style.display = 'none';
        } catch (e) {
            // nothing to hide
        }

        // show the new object
        this.currentVisible = idxIn;
        try {
            e = this.elements[this.currentVisible];
            e.style.display = this.display;
        } catch (e) {
            // nothing to show
        }
        return idxOld;
    },
    
    getCurrentVisible: function() {
        return this.currentVisible;
    }
});
