/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.data.catalog");
dojo.require("hub.main");
dojo.require("dojox.data.XmlStore");
dojo.require('dijit._Widget');

/**
 * Abstract class for accessing the Mashup Hub catalog. Subclasses implement
 * fetch logic for specific catalog formats. Derives from dijit._Widget to 
 * support proper cleanup.
 */
dojo.declare('hub.data.catalog._Model', dijit._Widget, {
    // default query is entire catalog
    query: 'all',
    // default term is everything
    term: '*',
    collection: "all",
    searchType: "",
    sortBy: null,
    sortOrder: null,
    count: null,
    constructor: function(args, node) {
        // source for the server data
        this.source = null;
        // url to use to fetch the data
        this.url = null;
        // url to get the next batch of data
        this.next_url = null;
        // url to get the previous batch of data
        this.prev_url = null;
        // 3-tuple summary of the fetched data (first, last, total)
        this.summary = null;
        // outstanding request
        this.request = null;
        // subscriptions to dojo.publish'ed events so we know when to refresh
        this.conns = new Array();
    },

    /**
     * Starts watching for bus events.
     */
    postCreate: function() {
        // watch for bus events about entries
        this.conns.push(dojo.subscribe('hub.tag-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.save-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.delete-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.comment-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.reset', this, 'onStale'));
        if(this.query == 'favorites') {
            // watch for changing of favorites too
            this.conns.push(dojo.subscribe('update-favorites', this, 
                                           'onStale'));
        }
    },
       
    /**
     * Stops watching for bus events.
     */
    destroy: function() {
        // cleanup all subscriptions
        dojo.forEach(this.conns, dojo.unsubscribe);
        // call base class method
        this.inherited(arguments);
    },
    
    /**
     * Clears the model
     */
    clear: function() {
      this.next_url = null;
      this.prev_url = null;
      this.source = null;
      this.url = null;
      this.summary = null;
      this.request = null;
      this.stale = null;
    },
    
    /**
     * Gets the URL last loaded by the catalog.
     */
    getUrl: function() {
        return this.url;
    },
    
    /**
     * Gets a tuple of start index, max per page, and total items. If any
     * of these items is unknown 
     */
    getSummary: function() {
        return this.summary;
    },
    
    /**
     * Get if the catalog has a next set of results.
     */
    hasNext: function() {
        return (this.next_url != null);
    },
    
    /**
     * Get if the catalog has a previous set of results.
     */
    hasPrevious: function() {
        return (this.prev_url != null);
    },
    
    /**
     * Fetch items.
     *
     * @param start: Starting index
     * @param max: Maximum number of items to fetch
     * @param sortby: Name of the field to sort by
     * @param asc: Order of the sort, true = ascending, false = descending
     */
    fetch: function(start, max, sortby, asc) {
        var url = this._createQueryUrl();
        console.debug("URL = " + url);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
        	console.debug("Parsed query = " + query + ", URL = " + url);
        }
        var map = dojo.queryToObject(query);
        map['nologin'] = 'true';
        if (sortby != null) {
        	map["sortBy"] = sortby;
        }
        else if (this.sortBy != null) {
        	map["sortBy"] = this.sortBy;
        }
        if (asc != null) {
        	map["sortOrder"] = asc ? "asc" : "desc";
        }
        else if (this.sortOrder != null) {
        	map["sortOrder"] = this.sortOrder;
        }
        if (start != null) {
        	map["startIndex"] = "" + start;
        }
        if (max != null) {
        	map["count"] = "" + max;
        } else if (this.count) {
            map["count"] = "" + this.count;
        }

        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
        console.debug("Result URL = " + url);
        this.url = url;
        
        // do the fetch
        this._doFetch();
    },
    
    /**
     * Fetch items again.
     */
    refresh: function() {
        if(this.url != null) {
            this._doFetch();
        } else {
            this.fetch();
        }
    },
    
    /**
     * Fetch the next batch of items, if it exists.
     */
    fetchNext: function() {
        if(this.next_url) {
            this.url = this.next_url;
            this._doFetch();
        } else {
            throw new Error("catalog._Model: No next URL");
        }
    },
    
    /**
     * Fetch the previous batch of items, if it exists.
     */
    fetchPrevious: function() {
        if(this.prev_url) {
            this.url = this.prev_url;
            this._doFetch();
        } else {
            throw new Error("catalog._Model: No previous URL");
        }
    },
    
    
    /**
     * Signals 'complete' after the last item signal.
     */
    _onComplete: function(empty, request) {
        this.onComplete();
        this.request = null;
        this.stale = false;
    },
    
    /**
     * Abstract method. Should return URL to query.
     */
    _createQueryUrl : function() {
        throw new Error("Abstract method _createQueryUrl()");
    },
    
    /**
     * Does some basic housekeeping before fetching. Must be overriden to
     * do an actual fetch.
     */
    _doFetch: function() {
        this.next_url = null;
        this.prev_url = null;
        this.summary = new Array(3);
        
        if(this.request) {
            // @todo: really we'd like to cancel the earlier request instead
            // of stopping the new request, but dojo isn't cooperating
            // supposedly there should be an abort() method on request, but
            // not for XML; so how do we reliably stop it?
            throw new Error('catalog._Model: request in progress');
        }
        
        // signal that we're about to fetch
        this.onFetch();          
    },

    /**
     * Signals when catalog data is likely stale because of local changes.
     */
    onStale: function() {
        // extension point
    },

    /**
     * Signals at start of fetch, before network request.
     */
    onFetch: function() {
        // extension point
    },

    /**
     * Signals after data received, before first item.
     */
    onBegin: function(total) {
        // extension point
    },

    /**
     * Signals after data received, before first item.
     */
    onItem: function(item) {
        // extension point
    },
    
    /**
     * Signals after an error.
     */
    onError: function(error) {
        // extension point
    },

    /**
     * Signals after completion.
     */
    onComplete: function() {
        // extension point
    }
});

/**
 * Fetches an Atom feed listing a portion of the Mashup Hub catalog.
 * Pulls out all available data into JS objects for use in rendering.
 **/
dojo.declare('hub.data.catalog.AtomModel', hub.data.catalog._Model, {
    _createQueryUrl: function() {
        return hub.urls.getCatalog(this.collection, this.term, this.searchType, this.sortBy, this.sortOrder);
    },

    /**
     * Performs the remote query. Signals 'fetch' first.
     */
    _doFetch: function() {
        this.inherited(arguments);
        // send the query to the remote data source
        this.source = new dojox.data.XmlStore({'url' : this.url});
        this.request = this.source.fetch(
            {'onBegin' : dojo.hitch(this, this.onBegin),
             'onItem' : dojo.hitch(this, this._onItem),
             'onComplete' :dojo.hitch(this, this._onComplete),
             'onError' : dojo.hitch(this, this.onError)});
    },
    
    /**
     * Parses a single, top level XML element for data. Looks for relative
     * links and entries. Signals onItem with entry data in JS format.
     */
    _onItem: function(item, request) {
        var dict = {};
        var value;
        var tag = this.source.getValue(item, 'tagName')
        if(tag == 'os:startIndex') {
            this.summary[0] = parseInt(this.source.getValue(item, 'text()'));
        } else if(tag == 'os:itemsPerPage') {
            this.summary[1] = parseInt(this.source.getValue(item, 'text()'));
        } else if(tag == 'os:totalResults') {
            this.summary[2] = parseInt(this.source.getValue(item, 'text()'));
        } else if(tag == 'link') {
            var rel = this.source.getValue(item, '@rel');
            var href = this.source.getValue(item, '@href');
            if(rel == 'next') {
                this.next_url = href;
            } else if (rel == 'previous') {
                this.prev_url = href;
            }
        } else if(tag == 'entry') {
            var entryCategoryElement = this.source.getValue(item, "category");
            var category = this.source.getValue(entryCategoryElement, "@term");
            dict["entryCategory"] = category;
            var content = this.source.getValue(item, "content");
            
            var cat = this.source.getValue(content, "childNodes");
            
            var atts = this.source.getValues(cat, "childNodes");
            
            for(var i=0; i < atts.length; i++) {
                var node = atts[i];
                if(this.source.isItem(node)) {
                    if(this.source.hasAttribute(node, "catalog:tag")) {
                        value = this.source.getValues(node, "catalog:tag");
                        value = this._parseArray(value);
                    } else if (this.source.hasAttribute(node, "catalog:category")) {
                        value = this.source.getValues(node, "catalog:category");
                        value = this._parseArray(value);
                    }
                    else {
                        value = this.source.getValue(node, "text()");
                    }
                }
                
                var key = this.source.getValue(atts[i], 'tagName');
                if (key.lastIndexOf(":") > 0)
                	key = key.substring(key.lastIndexOf(":") + 1);
                	
				dict[key] = value;                
            }
            
            var idfield = this.source.getValue(this.source.getValue(item, "id"), "text()");
            var unprefixedId = idfield;
            if (idfield.indexOf("urn:uuid:") == 0) {
            	unprefixedId = idfield.substring("urn:uuid:".length);
            }
            if (idfield.indexOf("urn:id:") == 0) {
            	unprefixedId = idfield.substring("urn:id:".length);
			}
            var pureId = unprefixedId;
            if (unprefixedId.lastIndexOf('-') >=0) {
            	pureId = unprefixedId.substring(unprefixedId.lastIndexOf('-') + 1);
            }
            dict['objectid'] = pureId;

            // signal with parsed data
            this.onItem(dict);
        }
    },
    
    /**
     * Pulls an array of strings out of dojo data XML values.
     *
     * @param values Dojo data items having the values we want
     */
    _parseArray: function(values) {
        var arr = new Array();
        for(i in values) {
            arr.push(this.source.getValue(values[i], "text()"));
        }
        return arr;
    }
});

/**
 * Fetches a JSON list of a portion of the Mashup Hub catalog.
 * Pulls out all available data into JS objects for use in rendering.
 */
dojo.declare('hub.data.catalog.JsonModel', hub.data.catalog._Model, {
    _createQueryUrl: function() {
        return hub.urls.getCatalog(this.query, this.term, 'json');
    },

    /**
     * Performs the remote query. Signals 'fetch' first.
     */
    _doFetch: function() {
        this.inherited(arguments);       
        // signal that we're about to fetch
        this.onFetch();
        // send the query to the remote data source
        this.source = {url : this.url,
                       handleAs: 'json',
                       load : dojo.hitch(this, this._onLoad),
                       error: dojo.hitch(this, this.onError)
                      }
        this.request = dojo.xhrGet(this.source);
    },
    
    _onLoad: function(response, args) {
        // grab relative urls
        this.next_url = response.next;
        this.prev_url = response.previous;
        
        try {
            // signal begin
            this.onBegin(response.content.length);
            
            // iterate over content
            for(i in response.content) {
                this.onItem(response.content[i]);
            }
            
            // signal complete
            this._onComplete(response, args);
        } catch (e) {
            // all errors go to the registered error handler
            this.onError(e, args);
        }
    }
});
