/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.data.social');
dojo.require('dijit._Widget');

/**
 * Abstract base class for fetching social data from the server. Subclasses
 * implement logic for choosing what data to fetch and how to store it. 
 * Derives from dijit._Widget to support proper cleanup.
 */
dojo.declare("hub.data.social._Model", dijit._Widget, {
    // refresh when instantiated
    forceRefresh: false,
    constructor: function(args, node) {
        // initialize arrays for connections and data
        this.conns = new Array();
        // initialize slot for social data
        this.data = null;

        // build request object
        this.request = {'handleAs': 'json',
                        'timeout': 30000,
                        'load' : dojo.hitch(this, this._onLoad),
                        'error' : dojo.hitch(this, this._onError)
                       };
    },

    /**
     * Refresh the model if forced by an option.
     */
    postCreate: function() {
        if(this.forceRefresh) {
            this.refresh();
        }
    },
    
    destroy: function() {
        // cleanup all subscriptions
        dojo.forEach(this.conns, dojo.unsubscribe);
        // call base class method
        this.inherited(arguments);
    },
   
    _onLoad: function(response, args) {
        if(response.code > 0) {
            this._onError(response, args);
            return;
        }
        this.data = response.result;
        this.onComplete(this);
    },
    
    _onError: function(response, args) {
        // ignore the error, keep the current categories
        this.onError(this);
    },
    
    refresh: function() {
        this.onFetch(this);
        dojo.xhrGet(this.request);
    },
    
    getSocialData: function() {
        return this.data;
    },

    onComplete: function(model) {
        // extension point
    },

    onError: function(model) {
        // extension point
    },

    onFetch: function(model) {
        // extension point
    }
});

/**
 * Manages a list of global categories.
 */
dojo.declare("hub.data.social.CategoriesModel", hub.data.social._Model, {
    postCreate: function() {
        // connect to global category update signal
        this.conns.push(dojo.subscribe('hub.update-categories', this, 'refresh'));

        // RESTful URL
        this.request.url = hub.urls.getCategories();

        this.inherited(arguments);
    }
});

/**
 * Manages a list of comments for a single catalog entry.
 */
dojo.declare("hub.data.social.CommentsModel", hub.data.social._Model, {
    objectid: null,
    postCreate: function() {
        if(this.objectid == null) {
            throw new Error("CommentsModel: objectid required");
        }

        // connect to global category update signal
        this.conns.push(dojo.subscribe('hub.comment-entry', this, '_filterEvent'));

        // RESTful URL
        this.request.url = hub.urls.getComments(this.objectid);

        this.inherited(arguments);
    },

    _filterEvent: function(id) {
        if(id == this.objectid) {
            this.refresh();
        }
    }
});

/**
 * Manages the average rating for a single catalog entry.
 */
dojo.declare("hub.data.social.AverageRatingModel", hub.data.social._Model, {
    objectid: null,
    postCreate: function() {
        if(this.objectid == null) {
            throw new Error("AverageRatingModel: objectid required");
        }

        // connect to global rate entry signal
        this.conns.push(dojo.subscribe('hub.rate-entry', this, '_filterEvent'));

        // RESTful URL
        this.request.url = hub.urls.getRatings(this.objectid);

        this.inherited(arguments);
    },

    _filterEvent: function(id) {
        if(id == this.objectid) {
            this.refresh();
        }
    }
});

/**
 * Manages a list of tags, global or for a particular object.
 */
dojo.declare("hub.data.social.TagsModel", hub.data.social._Model, {
    objectid: null,
    postCreate: function() {
        if(this.objectid != null) {
            this.request.url = this._initOne();
        } else {
            this.request.url = this._initAll();
        }        
        this.inherited(arguments);
    },
    
    _initAll: function() {
        // user tagged an entry
        this.conns.push(dojo.subscribe('hub.tag-entry', this, 'refresh'));
        // user deleted an entry
        this.conns.push(dojo.subscribe('hub.delete-entry', this, 'refresh'));
        // user save an entry (and possibly modified tags)
        this.conns.push(dojo.subscribe('hub.save-entry', this, 'refresh'));
        // RESTful URL
        return hub.urls.getTags();
    },

    _initOne: function() {
        if(this.objectid == null) {
            throw new Error("TagsModel: objectid required");
        }
        // user tagged an entry
        this.conns.push(dojo.subscribe('hub.tag-entry', this, '_filterEvent'));
        // RESTful URL
        return hub.urls.getTags(this.objectid);
    },

    _filterEvent: function(id) {
        if(id == this.objectid) {
            this.refresh();
        }
    }
});
