/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide("hub.pages");
dojo.require("dojox.dtl.widget");
dojo.require("dojo.i18n");
dojo.require("hub.main");
dojo.requireLocalization("hub", "popups");
dojo.requireLocalization("hub", "links");
dojo.requireLocalization("hub", "panels");
dojo.requireLocalization("hub", "labels");
dojo.requireLocalization("hub", "messages");

/**
 * Base class for all page templates. Supports fetching of a template given
 * its template path on the server, rendering of the template in a
 * dojox.dtl.AttachPoint, and optional fetching of any server data from a
 * RESTful URL.
 *
 * Subclasses must override getContext() to return an object with properties
 * corresponding to fields required by the template. Subclasses can override
 * getUrl() to return a URL that will be queried for additional server data and
 * passed as the only parameter to getContext().
 *
 * @var rendered: Deferred response to whether a page is rendered or not
 * @var load_hitched: Hitched method to call on server data load
 * @var err_hitched: Hitched method to call on server error
 */
dojo.declare("hub.pages._Page", dojox.dtl._Widget, {
    // path to the template on the server; use dojo.modulePath
    templatePath: null,
    tab: null,
    
    constructor: function(args, node) {
        this.rendered = new dojo.Deferred();
        this.load_hitched = dojo.hitch(this, this._onPageDataLoad);
        this.err_hitched = dojo.hitch(this, this._onPageDataError);
    },

    /**
     * Shows a loading message. Publishes events saying the page is busy.
     * Fetches the template.
     */
    buildRendering: function() {
        this.inherited('buildRendering', arguments);
        // only render busy message to start
        hub.busyMessage(this.domNode);
        // build a reusable template, template will be interned string if
        // running a dojo custom build, otherwise fetch from server.
        if (this.templatePath == null && this.templateString) {
            this.template = new dojox.dtl.HtmlTemplate(this.templateString);
        } else {
            this.template = new dojox.dtl.HtmlTemplate(this.templatePath);
        }
    },

    /**
     * Decides whether to query the server or to go ahead with rendering
     * immediately.
     */
    postCreate: function() {
        // check if the page needs server data
        var url = this.getUrl();
        if(url != null) {
            this._getServerData(url);
        } else {
            this._renderPage({});
        }
    },

    /**
     * Renders the page template. Calls getContext() for the data to interpolate
     * into the template.
     *
     * @param server_data: All data returned from the server if getUrl()
     * returned a valid URL to query.
     */
    _renderPage: function(server_data) {
        //console.log('_renderPage');
        
        // check this.domNode state in case this widget is destroyed 
        // before this callback is invoked.
        var srcNode = this.domNode;
        
        if (srcNode) {
        	// get the context object from the subclass impl.
        	var obj = this.getContext(server_data);
        	// build the dojox.dtl.Context
        	var context = new dojox.dtl.Context(obj);
        	// do the rendering
        	this.render(this.template, context);
        	// call extension point method
        	this.onRendered();
        }
        else {
           console.log('_renderPage - dom node is null');
        }
        
    },

    /**
     * Gets an object with properties corresponding to fields in the template.
     * Must be provided by a subclass.
     *
     * @return: Object
     */
    getContext: function() {
        throw new Error("_Page: must override getContext");
    },

    /**
     * Gets a RESTful URL to query for additional data required to render the
     * page.
     *
     * @return: String URL or null to indicate no REST call is needed
     */
    getUrl: function() {
        return null;
    },

    /**
     * Gets an object describing the properties of this page. Used by a
     * container to decide how to render a page. Will be invoked on the *class*
     * not on an instance (i.e., a static) so avoid use of "this."
     *
     * @return: Object
     */
    getProperties: function() {
        return {};
    },

    /**
     * Performs the fetch for additional server data.
     *
     * @param url: URL to query
     */
    _getServerData: function(url) {
	var kw = {
	    url: url, // url for page data
            handleAs: 'json', // always json encode page data
            timeout: 120000, // 2 minutes in milliseconds
            error: this.err_hitched,
            load: this.load_hitched
	};
        // get page data from server
        dojo.xhrGet(kw);
    },

    /**
     * Handles a successful load of server data.
     *
     * @param response: Response object from the server, de-JSONified
     * @param args: Original query arguments
     */
    _onPageDataLoad: function(response, args) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
	try {
            if(!response) {
                hub.notify(msgs.ERROR_REQUEST_SERVER, this.domNode);
            }
		if(hub.submit.processResponseCode(response)){
		    this._renderPage(response);
	    }
	} catch(e) {
            throw e;
	}
        return response;
    },

    /**
     * Handles an error during a server call.
     *
     * @param response: The error object
     * @param args: Original query arguments
     */
    _onPageDataError: function(response, args) {
        // get error messages
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        if(response.dojoType == "cancel") {
            hub.notify(msgs.ERROR_REQUEST_CANCELLED, this.domNode);
        } else if (response.dojoType == "timeout"){
            hub.notify(msgs.ERROR_REQUEST_TIMEOUT, this.domNode);
        } else {
            hub.notify(response, this.domNode);
        }
        return response;
    },

    /**
     * Uses the rendered deferred to indicate page rendering is complete.
     * Can be used as an extension point for dojo.connect too, but only if it is
     * known ahead of time that rendering is not already complete at time of
     * connection.
     */
    onRendered: function() {
        // extension point
        this.rendered.callback(true);
    },

    /**
     * Gets a deferred which will indicate when page rendering is complete.
     *
     * @return: dojo.Deferred which will notify when the answer becomes true
     */
    isRendered: function() {
        return this.rendered;
    }
});

