/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.widgets');
dojo.require('hub.main');
dojo.require('dijit._Widget');
dojo.require('dijit._Templated');
dojo.require('dijit.form.Button'); // for search form
dojo.require('dijit.form.CheckBox'); // radio button is in here


/**
 * Displays an array of values keyed by column names in a table.
 */
dojo.declare("hub.widgets.Table", [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/Table.html"),

    constructor : function(args, node) {
        // column metadata
        this.columns = null;
        // number of rows added to table so far for this query
        this.row_count = 0;
    },

    initialize: function() {
        // clear out all table headers and body
        this.clearHeaders();
        this.clearRows();

        for(i in this.columns) {
            var th = document.createElement('TH');
            // title and base class
            th.innerHTML = this.columns[i]['name'];
            if(this.columns[i]['class']) {
	            th.className = this.columns[i]['class'];
	        }
            this.table_header.appendChild(th);
        }
    },

    /**
     * Clears all table headers.
     */
    clearHeaders: function() {
        // NOTE: we really want to just do innerHTML = '', but we can't
        // because of IE6/IE7; have to use DOM else get runtime errors
        while (this.table_header.firstChild)  {
            this.table_header.removeChild(this.table_header.firstChild);
        };
    },

    /**
     * Clears the body of the table.
     */
    clearRows: function() {
        // NOTE: we really want to just do innerHTML = '', but we can't
        // because of IE6/IE7; have to use DOM else get runtime errors
        while (this.table_body.firstChild)  {
            this.table_body.removeChild(this.table_body.firstChild);
        };
        this.row_count = 0;
    },

    /**
     * Adds a single row to the table.
     * @param dict single row data keyed by column name:
     *  - json string in format {'col1': data1, 'col2': data2, ...}
     *  - json object
     */
    addRow: function(dict) {
    	var row=dict;

    	// convert to json object data
		if(!(dict instanceof Object)) {
    		row=dojo.fromJson(dict);
    	}

        if(!this.columns) {
            // if no columns are set, use the columns specified in this row
            this.columns = new Array();
            for(key in row) {
                this.columns.push({'name' : key});
            }
            this.initialize();
        }

        // create table row and columns
        var tr = document.createElement('TR');

        for(i in this.columns) {
            var name = this.columns[i]['name'];
            var td = document.createElement('TD');
            var ml = row[name];
            td.innerHTML = ml;
            tr.appendChild(td);
        }

        // alternating row style
        if(this.row_count % 2) {
            dojo.addClass(tr, 'alt');
        }

        this.table_body.appendChild(tr);
        this.row_count += 1;
    },

    /**
     * Adds a set of rows to the table.
     *
     * @param arr Array of row data keyed by column
     *   [{'col1': data1, ...}, {'col1': data2, ...} ...]
     * or
     */
    addRows: function(arr) {
        dojo.forEach(arr, dojo.hitch(this,this.addRow));
    },

    /**
     * Sets an array of column metadata objects. The only required key in
     * each metadata object is 'name' representing the human readable name
     * to be displayed on the column header. The key 'class' is optional
     * and represents the CSS class to be applied to a column. Other keys
     * may be included, but will be ignored
     *
     *
     * @param arr Array of column metadata objects
     *   [{name: 'Foobar', ...}, ...]
     */
    setColumns: function(arr) {
        this.columns = arr;
        // have to reinitialize everything
        this.initialize();
    }
});

/**
 * Renders a list of checkable items managed entirely by JS. Existing items in
 * the box are ignored. All items are initially unchecked when added. External
 * observer can connect to the 'onChange' signal with params of 'label' and
 * 'value'.
 */
dojo.declare('hub.widgets.CheckBoxList', [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/CheckBoxList.html"),
	groups: {},
	checked: {},
	all: {},
	init_labels: [],
	init_names: [],

    /**
     * @param args: ignored
     * @param node: Node to which to attach this widget
     */
    constructor: function(args, node) {
        // key: group id, value: div holding the group
        this.groups = {};
        // key: label, value: checked (true) or unchecked (false)
        this.checked = {};
        // key: group id, value: array of group elements
        this.all = {};

        // keep labels of all elements initially in the list
        this.init_labels = [];
        this.init_names = [];
        dojo.query('li', node).forEach(dojo.hitch(this, function(li) {
            this.init_labels.push(li.innerHTML);
            this.init_names.push(li.innerHTML);
        }));
    },

    /**
     * Makes a group named 'default' for all items that were in the original
     * DOM node.
     */
    postCreate: function() {
        this.appendGroup('default', this.init_labels, this.init_names);
        delete this.init_names;
        delete this.init_labels;
    },

    /**
     * Handles check/uncheck events by signaling observers and updating
     * internal state of what's checked.
     *
     * @return: style class to put on the outer container
     */
    _onCheck: function(name, event) {
        var cb = event.target;
        // keep track of checked values
        if(cb.checked) {
            this.checked[name] = true;
        } else {
            delete this.checked[name];
        }
        this.onChange(name, cb.checked);
    },

    /**
     * Adds a new group of checkboxes to the list. The id is used to later
     * remove all the added checkboxes. All items are initially unchecked
     *
     * @param id: Unique ID for the group within the scope of this widget
     *   only, not global
     * @param labels: Array of labels for the checkboxes
     */
    appendGroup: function(id, labels, names) {
        if(!this.groups[id]) {
        	if(names == null){
        		names = labels;
        	}
            // build a new div for each group within the list
            var div = document.createElement('DIV');
            // loop over all labels and add elements
            this.all[id] = [];
            for(var i in labels) {
                var li = document.createElement('LI');
                // add the checkbox
                var input = document.createElement('INPUT');
                input.type = 'checkbox';
                input.name = names[i];
                li.appendChild(input);
                if(this.checked[names[i]]){
                	input.setAttribute("checked", true);
                }
                // watch for events on the checkbox
                // hitch so we can pass the label too
                this.connect(input, 'onclick',
                   dojo.hitch(this, this._onCheck, names[i]));
                // add the label
                var label = document.createElement('LABEL');
                label.htmlFor = names[i];
                label.innerHTML = labels[i];
                li.appendChild(label);
                // add list item to the div
                div.appendChild(li);
                // i is index (number)
                this.all[id][i] = names[i];
            }
            // add the div to the box
            this.box.appendChild(div);
            // keep track of the div by the group id
            this.groups[id] = div;
        } else {
            throw new Error('CheckBoxList: group already exists');
        }
    },

    /**
     * Removes an existing group of checkboxes from the list.
     *
     * @param id: Unique ID for the group
     */
    removeGroup: function(id) {
        var div = this.groups[id];
        if(!div) {
            throw new Error('CheckBoxList: group does not exist');
        }
        // clear out all checked items in the div that's going away
        dojo.query('input', div).forEach(dojo.hitch(this, function(input) {
            delete this.checked[input.name];
        }));

        // @todo: should really clean up connections for group here, but we
        // do at least get them later

        // throw away the div and its group reference
        this.box.removeChild(div);
        delete this.groups[id];
        delete this.all[id];
    },

    /**
     * Remove all check boxes.
     */
    clear: function() {
        this.all = {};
        this.groups = {};
        this.checked = {};
        this.box.innerHTML = '';

        // clean up event connections
		dojo.forEach(this._connects, function(array){
	    dojo.forEach(array, dojo.disconnect);
	});

    },

    /**
     * Get an array of all labels
     */
    getAll: function() {
        var arr = [];
        for(var i in this.all) {
            for(var j=0; j<this.all[i].length; j++) {
                arr[arr.length] = this.all[i][j];
            }
        }
        return arr;
    },

    /**
     * Gets an array of labels of all checked items in the list.
     *
     * @return: Array of labels
     */
    getAllChecked: function() {
        var checked = new Array();
        for(var name in this.checked) {
            checked.push(name);
        }
        return checked;
    },

    /**
     * Checks or unchecks a specific item in the list
     *
     * @param id: Unique ID for the group
     * @param label: Name of the item in the group
     * @param value: 'true' to check the item, 'false' to uncheck it
     */
    setChecked: function(id, name, value) {
        var div = this.groups[id];
        // NOTE: these are not arrays, be careful
        var cbs = dojo.query('input', div);
        for(var i=0; i < cbs.length; i++) {
            if(cbs[i].name == name) {
                cbs[i].checked = value;
                // have to do manually because IE requires onclick which does
                // fire when we change checked programmatically
                if(value) {
                    this.checked[name] = true;
                }
                else {
                    delete this.checked[name];
                }
            }
        }
    },

    /**
     * Tests if item is checked.
     * @return true if checked, otherwize false
     */
    isChecked: function(name) {
		if(name in this.checked) {
			return true;
		}
		return false;
    },

    /**
     * Checks or unchecks all items in a group.
     *
     * @param id: Unique ID for the group
     * @param value: true to check, false to uncheck
     */
    setGroupChecked: function(id, value) {
        var div = this.groups[id];
        // NOTE: these are not arrays, be careful
        var cbs = dojo.query('input', div);

        for(var i=0; i < cbs.length; i++) {
            cbs[i].checked = value;
            // have to do manually because IE requires onclick which does
            // fire when we change checked programmatically
            var name = cbs[i].name;
            if(value)
                this.checked[name] = true;
            else
                delete this.checked[name];
        }
    },

    /**
     * Checks or unchecks all items in the list.
     *
     * @param value: true to check, false to uncheck
     */
    setAllChecked: function(value) {
        for(var i in this.groups) {
            this.setGroupChecked(i, value);
        }
    },

    /**
     * Signals when any checkbox changes value.
     *
     * @param label: Label text of the checkbox that changed value
     * @param value: Checked or unchecked
     */
    onChange: function(label, value) {
        // extension point
    }
});

dojo.declare('hub.widgets.Button', dijit.form.Button,{
	templateString: null,
	baseClass: 'BlueButton',
	templatePath: dojo.moduleUrl("hub.widgets", "templates/Button.html")
})

