/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.widgets.social.edit');
dojo.require('hub.main');
dojo.require('hub.behaviors');
dojo.require('hub.submit.social');
dojo.require('hub.widgets.social.view');
dojo.require('dijit._Widget');
dojo.require('dijit._Templated');
dojo.require('dijit.form.Button'); // for tag submit
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("hub", "labels");
dojo.requireLocalization("hub", "popups");

/**
 * Rating chooser widget. Users clicks stars or uses the keyboard to set a 
 * rating value.
 */
dojo.declare('hub.widgets.social.edit.RatingChooser', dijit._Widget, {
    // icon to show when mouse/keyboard selection hovers over a star
    hoverIcon: "yellowstar.gif",
    // icon for full star
    fullIcon: "bluestar.gif",
    // icon for half start
    halfIcon: "bluehalfstar.gif",
    // icon for no star
    emptyIcon: "icon-rated-empty.gif",
    // number of stars possible
    maxRating: 5,
    // initial rating value
    value: 0,
    // object we're rating
    objectid: null,
    // form tag id (it is required by 'setTabBusy' function)
    formid: null,

    constructor: function(args, node) {
        // do we have to submit when losing focus?
        this.submit_on_blur = false;
        // store references to stars DOM nodes
        this.stars = null;
    },

    postMixInProperties: function() {
        if(this.objectid == null) {
            throw new Error("RatingChooser: objectid required");
        }
        // configure and reuse the rating star view renderable
        var args = {fullIcon : this.fullIcon,
            emptyIcon: this.emptyIcon,
            halfIcon : this.halfIcon,
            maxRating: this.maxRating};
        this.star_view = new hub.widgets.social.view.RatingStarView(args);
        this.inherited(arguments);
    },

    /**
     * Lay down the widget elements once. Thereafter, only refresh the image
     * URLs, not the entire DOM nodes.
     */
    buildRendering: function() {
        this.inherited(arguments);
        // use value from this object
        var urls = this.star_view.getStarUrls(this.value);
        var est = this.star_view.getRatingEstimate(this.value);
        this.stars = this.star_view.renderStarImages(urls);

        var box = document.createElement('SPAN');
        var _self = this;
        var index = 1;
        dojo.forEach(this.stars, function(star) {
            box.appendChild(star);
            // register event handlers on each star
            _self._registerNodeHandlers(star, index);
            index += 1;
        });
        box.title = est;
        // make sure we can get keyboard focus
        box.tabIndex = 0;
        // register handlers on the parent box
        this._registerHandlers(box);
        // replace attach point with the rendering
        this.domNode.appendChild(box);
    },

    refresh: function() {
        // update URLs without destroying DOM nodes
        var urls = this.star_view.getStarUrls(this.value);
        for(var i=0; i < urls.length; i++) {
            this.stars[i].src = urls[i];
        }

        // don't want the caller to replace its content
        return null;
    },

    _submit: function() {
    	var formnode = dojo.byId(this.formid);
    	formnode.value.value = this.value;
        var form = new hub.submit.social.Rating(formnode, null, this.objectid);
        hub.submit.ajaxSubmit(form, false, true);
    },

    _registerHandlers : function(node) {
        this.connect(node, 'onblur', '_onBlur');
        this.connect(node, 'onmouseout', '_onOut');
        this.connect(node, 'onkeydown', '_onKey');
    },
    
    _registerNodeHandlers : function(node, index) {
        this.connect(node, 'onclick', dojo.hitch(this, '_onClick', index));
        this.connect(node, 'onmouseover', dojo.hitch(this, '_onOver', index));
    },    
   
    _onOver: function(index, event) {
        var node = event.target;
        for(var i=0; i < index; i++) {
            this.stars[i].src = hub.urls.getImage(this.hoverIcon);
        }
        for(var i=index; i < this.maxRating; i++) {
            this.stars[i].src = hub.urls.getImage(this.emptyIcon);
        }
        node.style.cursor = "pointer";
    },
    
    _onOut: function(event) {
        var node = event.target;
        node.style.cursor = "default";
        // refresh this view
        this.refresh();
    },
    
    _onClick: function(index, event) {
        this.value = index;
        // submit value
        this._submit();
    },
    
    _onBlur: function(event) {
        if(this.submit_on_blur) {
            this.submit_on_blur = false;
            this._submit();
        }
    },
    
    _onKey: function(event) {
        var value = this.value;
        switch(event.keyCode) {
        case dojo.keys.RIGHT_ARROW:
        case dojo.keys.DOWN_ARROW:
            if(value >= this.maxRating) {
                dojo.stopEvent(event);
                return;
            }
            // higher
            this.value += 1;
            this.refresh();
            this.submit_on_blur = true;
            dojo.stopEvent(event);
            return;
        case dojo.keys.LEFT_ARROW:
        case dojo.keys.UP_ARROW:
            if(this.value <= 1) {
                dojo.stopEvent(event);
                return;
            }
            // lower
            this.value -= 1;
            this.refresh();
            this.submit_on_blur = true;
            dojo.stopEvent(event);
            return;
        case dojo.keys.ENTER:
            this.submit_on_blur = false;
            this._submit();
            dojo.stopEvent(event);
            return;
        }
    }
});

/**
 * Tag suggestion widget. Displays suggestions in an area below an input box.
 */
dojo.declare("hub.widgets.social.edit.TagSuggest", [dijit._Widget, dijit._Templated, hub.behaviors.Renderable], {
    // tag separator
    separator: ',',
    // initial tag string for input box
    value: '',
    // object which we're tagging
    objectid: null,
    // template to replace node with
    templatePath: dojo.moduleUrl("hub.widgets", "templates/TagSuggest.html"),
    // dojo.form.button in the template
    widgetsInTemplate: true,

    constructor: function(args, node) {
        this.names = [];
        this.counts = [];
    },

    postMixInProperties: function() {
        if(this.objectid == null) {
            throw new Error("TagSuggest: objectid required");
        }

        // get popup messages
        this.pops = dojo.i18n.getLocalization('hub', 'popups');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');

        // get character value for separator
        this.sep = this.separator.charCodeAt(0);
        // init other values
        this.caret = null;
        this.char_code = null;
        this.suggest = [];

        this.inherited(arguments);
    },

    postCreate: function() {
        this.input_box.value = this.value;
        this.inherited(arguments);
    },

    render: function(tags) {
        // split into names and frequencies
        this.names = new Array(tags.length);
        this.counts = new Array(tags.length);
        var i = 0;
        for(var key in tags) {
            this.names[i] = key;
            this.counts[i] = tags[key];
                i += 1;
        }
        // nothing to render visibly
        return null;
    },

    renderError: function() {
        // don't let model error message ovewrite us
        return null;
    },

    _reset: function() {
        this.caret = null;
        this.char_code = null;
        this.suggest = [];
        this._render(null);
    },

    _onSubmit: function() {
        this.value = this.input_box.value;
        var form = new hub.submit.social.Tags(this.tags_form, this.suggest_box, this.objectid);
        hub.submit.ajaxSubmit(form);
    },
    
    _onKeyPress: function(event) {
        // track last character code
        this.char_code = event.charCode;
    },
    
    _onKeyUp: function(event) {
        // ignore enter key since it's used to submit the form
        if(event.keyCode == dojo.keys.ENTER) {
            return;
        }

        // get the current caret position
        var caret = this._getCaretPosition(this.input_box);
        this.caret = caret;
        if(caret == null) {
            // can't find the caret, just abort
            this._reset();
            return;
        }
        
        // did we enter a separator?
        var is_sep = (this.char_code == this.sep);
        
        // find the chunk at the current caret location
        // offset by one if comma so we don't start on it
        var chunk = this._getCurrentChunk(caret, is_sep);
        if(!chunk.text.length) {
            // no chunk, abort
            this._reset();
            return;
        }
        
        if(is_sep) {
            if(this.suggest.length) {
                // insert the first suggestion if the user types the sep
                //this._replaceChunk(this.input_box, chunk, this.suggest[0], true); // disabled because of BZ #3189
            }
            // reset all suggestions
            this._reset();
            return;
        }
        
        // find the first possible match index in the tags list
        var index = this._findFirstTag(chunk.text);
        // narrow to the first five suggestions
        this.suggest = this._narrowSuggestions(this.input_box, chunk.text, 
                                               index, 5);
        // show the suggestions
        this._render(this.suggest);
    },
    
    _getCaretPosition: function(input) {
        if (input.setSelectionRange) {
            // Mozilla caret position code
            var caret_start = input.selectionStart;
            var caret_end = input.selectionEnd;
        } else {
            // IE caret position code hack
            var caret_start = Math.abs(document.selection.createRange().moveStart("character", -100000));
            var caret_end = Math.abs(document.selection.createRange().moveEnd("character", -100000));
        }
        
        if(caret_start != caret_end) {
            // if there is a selection, don't suggest anything
            return null;
        } else {
            // for convenience
            return caret_start;
        }
    },
    
    _getCurrentChunk: function(caret, avoid_sep) {
        var text = this.input_box.value;
        var chunk = {start: 0, end: 0, text: ''};
        
        // treat caret in terms of character positions, not between them
        //caret = Math.min(caret, text.length-1);
        
        // seek backwards for the previous comma or the start of the text
        var sep_char = String.fromCharCode(this.sep);
        // if avoiding the separator, back up one
        var left = text.lastIndexOf(sep_char, caret - (avoid_sep ? 2 : 1));
        if(left == -1) {
            // no comma, slice from string start
            left = 0;
        } else {
            left += 1;
        }
        // slice and left trim
        text = text.substring(left);
        while(text.charAt(0) == ' ') {
            text = text.substring(1)
            left += 1;
        }
        
        // look for the next comma
        // if avoiding the separator, jump ahead one
        caret = caret - left;
        var right = text.indexOf(sep_char, caret);
        if(right == -1) {
            // no comma, consider the rest of the text
            right = text.length;
        } else {
            // slice and right trim
            text = text.substring(0, right);
            while(text.charAt(text.length-1) == ' ') {
                text = text.substring(0, text.length-1);
            }
        }
        
        // package up all info nicely
        chunk.start = left;
        chunk.end = left + right;
        chunk.text = text;
        return chunk;
    },
    
    _replaceChunk: function(input, chunk, tag) {
        var text = this.input_box.value;
        // slice the string
        var first = text.slice(0, chunk.start);
        var second = text.slice(chunk.end+1);
        // rebuild
        this.input_box.value = first + tag + String.fromCharCode(this.sep) 
            + second;
    },
    
    /**
     * Binary search to find the first location where a tag would appear
     * in the list of tags.
     */
    _findFirstTag: function(prefix) {
        var low = 0;
        var high = this.names.length;
        var mid, last;
        while (low <= high) {
            mid = parseInt((low + high) / 2);
            if (this.names[mid] > prefix) {
                // target is before the mid point
                high = mid - 1;
                last = 0;
            } else if (this.names[mid] < prefix) {
                // target is after the mid point
                low = mid + 1;
                last = 1;
            } else {
                // target is the midpoint
                return mid;
            }
        }
        // if we make it here, the mid point is as close to the target
        // as we can get
        return mid + last;
    },
    
    /**
     * Seeks ahead linearly from the starting index and returns up to 
     * max strings starting with the given text.
     */
    _narrowSuggestions: function(input, prefix, start, max) {
        var text = input.value;
        var length = prefix.length;
        var suggest = new Array();
        for(var i=start; i<this.names.length; i++) {
            var tag = this.names[i];
            // only consider as suggestion if the prefix matches, 
            // if it's not already in the text, or if it's an exact match
            // on the current chunk
            if(tag.substr(0, length) == prefix && 
               text.indexOf(tag) == -1) {
                suggest.push(tag);
            }
            if(suggest.length >= 5) {
                break;
            }
        }
        return suggest;
    },
    
    /**
     * Render the list of suggestions as clickable links. Different from 
     * render() which is how the hub.behaviors.Renderables receive data to 
     * render from other widgets.
     */
    _render: function(suggest) {
        // make sure the space doesn't collapse
        this.suggest_box.innerHTML = '&nbsp;';
        // error messages might set weird classes, remove them
        this.suggest_box.className ='';
        if(!suggest || !suggest.length) 
            return;
        // put the title on the suggestion list
        var b = document.createElement('SPAN');
        var text = document.createTextNode(this.pops.TAG_SUGGEST_TITLE);
        b.appendChild(text);
        this.suggest_box.appendChild(b);
        for(i in suggest) {
            var a = document.createElement('A');
            a.innerHTML = hub.htmlspecialchars( suggest[i] );
            a.href = "#";
            // @todo: need to clean these up else we leak memory!
            a.onclick = dojo.hitch(this, this.insert, i);
            this.suggest_box.appendChild(a);
            if(i < suggest.length - 1) {
                var sep = document.createTextNode(", ");
                this.suggest_box.appendChild(sep);
            }
        }
    },
    
    /**
     * Insert the suggestion at index into the text area.
     */
    insert: function(index) {
        // get the current caret position
        //var caret = this._getCaretPosition(this.input);
        var caret = this.caret;
        if(caret == null) {
            // can't find the caret, just abort
            this._reset();
            return;
        }
        
        // find the chunk at the current caret location
        var chunk = this._getCurrentChunk(caret);
        if(!chunk.text.length) {
            // no chunk, abort
            this._reset();
            return;
        }
        
        // replace the current chunk
        var tag = this.suggest[index];
        this._replaceChunk(this.input_box, chunk, tag);
        this._reset();
        this.input_box.focus();
    }
});

dojo.declare("hub.widgets.social.edit.TagSuggestLite", [hub.widgets.social.edit.TagSuggest], {
	inputName: 'tags',
    templatePath: dojo.moduleUrl("hub.widgets", "templates/TagSuggestLite.html")
});
