/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.widgets.social.view');
dojo.require('hub.main');
dojo.require('hub.behaviors');
dojo.require('dijit._Widget');
dojo.require('dijit._Container');
dojo.requireLocalization("hub", "panels");
dojo.requireLocalization("hub", "labels");
dojo.requireLocalization("hub", "popups");

/**
 * Renders site social data through a visualization pipeline.
 */
dojo.declare("hub.widgets.social.view.SocialBox", [dijit._Widget, dijit._Container, hub.behaviors.Renderable], {
    // hub.data.social._Model widgetid
    model: '',
    // force a refresh on the model immediately
    forceRefresh: false,
    // if true, replaces all content with a refresh message while model is 
    // updating; disable if child views do not completely re-render
    showRefresh: true,

    startup: function() {
        // store the pipeline because it's going to get overwritten
        this.pipeline = this.getChildren();

        if(this.model) {
            this.setModel(this.model);
        }
        
        if(this.forceRefresh && this.model) {
            this.model.refresh();
        } else {
            this._onLoad(this.model);
        }
        
        // call parent
        this.inherited(arguments);
    },

    /**
     * Destroys this widget and its descendants. Not sure why destroyRecursive
     * isn't called recursively instead.
     */
    destroy: function() {
        // the pipeline widgets are no longer in our template
	dojo.forEach(this.pipeline, function(widget) { 
            widget.destroy(); 
        });
        // call base implementation to kill this widget
        this.inherited(arguments);
    },

    setModel: function(model) {
        // stop listening to previous model
        this.disconnect();
        // store new model
        this.model = dijit.byId(model);
        // watch for changes on the comment model
        this.connect(this.model, 'onComplete', '_onLoad');
        this.connect(this.model, 'onFetch', '_onFetch');
        this.connect(this.model, 'onError', '_onError');
    },

    redraw: function() {
        // force a redraw with existing data from the model
        this._onLoad(this.model);
    },

    render: function(data) {
        // run through all descendant widgets and see if we can get them to 
        // render (i.e., are they hub.behavior.Renderables or look-alikes)
        for(var i=0; i < this.pipeline.length; i++) {
            try {
                data = this.pipeline[i].render(data);
            } catch(e) {
                // ignore, probably not renderable
                console.warn(e);
            }
        }
        // assume last child widget returned is the rendering
        return data;
    },

    renderError: function(err) {
        // @todo this fetches descendants, not children; probably want to be
        // more careful
        // run through all descendant widgets and see if we can get them to 
        // render (i.e., are they hub.behavior.Renderables or look-alikes)
        for(var i=0; i < this.pipeline.length; i++) {
            try {
                var err = this.pipeline[i].renderError(err);
            } catch(e) {
                // ignore, probably not renderable
            }
        }
        // assume last child widget returned the final message
        return err;
    },

    _onLoad: function(model) {
        try {
            var data = model.getSocialData();
        } catch(e) {
            // still OK; sometimes no model needed
            // use dummy object since null has a meaning as an error
            var data = {};
        }

        if(data == null || data.length == 0) {
            // treat no data as an error
            this._onError(model);
            return;
        }

        // ask child widgets to render
        var result = this.render(data);

        if(result != null && dojo.isObject(result)) {
            // result is a DOM node
            this.domNode.innerHTML = '';
            this.domNode.appendChild(result);
        } else if(result != null) {
            // result is a string
            this.domNode.innerHTML = result;
        }
        // ignore all other result types
    },

    _onFetch : function(model) {
        if(this.showRefresh) {
            hub.busyMessage(this.domNode, false, 'note');
        }
    },

    _onError: function(model) {
        var text = this.renderError('');
        if(text != null) {
            hub.notify(text, this.domNode, 'note');
        }
    }
});


/**
 * Filters tags so only the top X are rendered.
 */
dojo.declare("hub.widgets.social.view.TopTagsFilter", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    // default number of tags to render
    value: 15,

    _getMax: function(total) {
        // value is the maximum number to render
        return this.value;
    },
    
    setValue: function(value) {
        this.value = value;
    },
    
    render: function(arr) {
        // store original array in case we re-render
        this.data = arr;

        var total = 0;
        var bins = [];
        // sort by frequency
        for(var key in arr) {
            var val = arr[key];
            if(!bins[val]) {
                // build an array to store tags with this frequency
                bins[val] = new Array();
            }
            // push into the proper bin
            bins[val].push(key);
            total += 1;
        }
        
        // figure out the maximum number of items
        var max = this._getMax(total);
        
        var top = [];
        for(var i = bins.length - 1; i >= 0; i--) {
            var tags = bins[i];
            // avoid empty bins
            if(!tags) continue;
            if(tags.length + top.length > max) {
                // slice just the tags we need
                top = top.concat(tags.slice(0, max - top.length));
                break;
            } else {
                top = top.concat(tags);
            }
        }
        // sort again by name
        top.sort();
        var top_pairs = {};
        // recombine with frequencies
        for(var i in top) {
            var name = top[i];
            top_pairs[name] = arr[name];
        }

        return top_pairs;
    }    
});

/**
 * Filters tags so some percentage of the total tags is shown.
 */
dojo.declare('hub.widgets.social.view.DensityTagsFilter', hub.widgets.social.view.TopTagsFilter, {
    destroy: function() {
        console.log('destroying density tags filter');
        this.inherited(arguments);
    },
    
    _getMax: function(total) {
        // value is percentage of total tags
        return parseInt(total * (this.value / 100.0));
    }
});

/**
 * Cloud view of tags.
 */
dojo.declare("hub.widgets.social.view.TagsCloudView", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    render: function(arr) {
        var pops = dojo.i18n.getLocalization('hub', 'popups');
        var html = "<div class='TOM_widget_TagsCloudView'>" ;
        for(var name in arr) {
            var size = parseInt(arr[name]);
            var title = dojo.string.substitute(pops.TAG_ITEM_COUNT, [arr[name]]);
            if(size > 10)
                size = 10;
            html += "<span class='tag"+size+"'>";
            //name = hub.htmlspecialchars(name);
            html += hub.buildPageLink(hub.htmlspecialchars(name), 'Catalog', 
                                      {collection: "all", searchType: "tag", term: hub.escapeQuotes3(name)}, false, null, 
                                      title);
            html += "</span>\n";
        }
        html += "</div>";
        return html;
    },

    renderError: function(err) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return msgs.STATUS_TAGS_NONE;
    }
});

/**
 * List view of categories.
 */
dojo.declare("hub.widgets.social.view.CategoriesListView", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    render: function(arr) {
        var html = "<ul class='TOM_action_list' style='margin-left:0.5em;'>\n";
        for (var i in arr) {
            html += "<li>";
            html += hub.buildPageLink(hub.htmlspecialchars(arr[i]), 'Catalog', 
                                      {collection: 'all', searchType: "category", term: hub.escapeQuotes3(arr[i])}, true);
            html += "</li>\n";
        }
        html += "</ul>";
        return html;
    },

    renderError: function(err) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return msgs.STATUS_CATEGORIES_NONE;
    }
});

/**
 * Inline comma separated view of tags with counts displayed.
 */
dojo.declare("hub.widgets.social.view.TagsCountView", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    render: function(arr) {
        var pops = dojo.i18n.getLocalization('hub', 'popups');
        var html = "<div class='TOM_widget_TagsCountView'>";
        for (var name in arr) {
            var title = dojo.string.substitute(pops.TAG_USER_COUNT, [arr[name]]);
//            name = hub.htmlspecialchars(name);
            html += hub.buildPageLink(hub.htmlspecialchars(name), 'Catalog', 
                                      {collection: 'all', searchType: "tag", term: hub.escapeQuotes3(name)}, false,
                                      null, title);
            html += ', ';
        }
        // cut final comma
        html = html.substring(0,html.length-2);
        html += '</div>';
        return html;
    },

    renderError: function(err) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return msgs.STATUS_TAGS_NONE;
    }
});

/**
 * Renders user comments by observing a CommentsModel instance.
 */
dojo.declare("hub.widgets.social.view.CommentsListView", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    render: function(arr) {
        // only require if we actually reach this code
        dojo.require("dojo.date.locale");
        var panels = dojo.i18n.getLocalization('hub', 'panels');

        var html = "<div class='TOM_widget_CommentsListView'";
        for(var i=0; i < arr.length; i++) {
            var comment = arr[i];
            var date = new Date(parseInt(comment.timestamp)*1000);
            var locDate = dojo.date.locale.format(date);
            html += "<div class='comment'><h3 class='title'>"+hub.htmlspecialchars(comment.commenttitle)+"</h3>";
            html += "<pre class='body'>"+hub.htmlspecialchars(comment.commenttext)+"</pre>";
            var byline = dojo.string.substitute(panels.COMMENT_BYLINE, 
                [hub.htmlspecialchars(comment.creatorid), locDate]);
            html += "<p class='byline'>"+byline+"</p></div>";
        }
        html += "</div>";
        return html;
    },

    renderError: function(err) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return msgs.STATUS_COMMENTS_NONE;
    }
});

/**
 * Renders user comments by observing a RatingsModel instance.
 */
dojo.declare("hub.widgets.social.view.RatingStarView", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    // icon for full star
    fullIcon: "redstar.gif",
    // icon for half start
    halfIcon: "redhalfstar.gif",
    // icon for no star
    emptyIcon: "icon-rated-empty.gif",
    // number of stars possible
    maxRating: 5,

    getStarUrls: function(value) {
        var urls = new Array();
        // render stars, half-stars, and empty stars
        for(var i = 1; i <= this.maxRating; i++) {
            if(value >= i) {
                urls.push(hub.urls.getImage(this.fullIcon));
            } else if(value >= (i - .5)) {
                urls.push(hub.urls.getImage(this.halfIcon));
            } else {
                urls.push(hub.urls.getImage(this.emptyIcon));
            }
        }
        return urls;
    },

    getRatingEstimate: function(value) {
        var val = 0;
        for(var i = 1; i <= this.maxRating; i++) {
            if(value >= i) {
                val = i;
            } else if(value >= (i - .5)) {
                val = i-0.5;
            }
        }
        return val;
    },

    renderStarImages: function(urls) {
        var stars = dojo.map(urls, function(url) {
            var img = document.createElement('IMG');
            img.src = url;
            return img;
        });
        return stars;
    },

    renderUserCount: function(count) {
        var panels = dojo.i18n.getLocalization('hub', 'panels');
        // render total votes text
        var span = document.createElement('SPAN');
        span.innerHTML = ' ' + dojo.string.substitute(panels.RATING_USER_COUNT,
            [count]);
        return span;
    },

    render: function(arr) {
        var est = this.getRatingEstimate(arr.value);
        var urls = this.getStarUrls(arr.value);
        var stars = this.renderStarImages(urls);
        var count = this.renderUserCount(arr.count);

        var div = document.createElement('DIV');
//      dojo.forEach(stars, dojo.hitch(div, div.appendChild)); // causes problems in IE7
        for(var i in stars){
        	div.appendChild(stars[i]);
        }
        div.appendChild(count);
        div.title = est;
        return div;
    },

    renderError: function(err) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return msgs.STATUS_RATING_NONE;
    }
});
