/*
	Copyright (c) 2007-2008, IBM Corp.
*/
dojo.provide('hub.widgets.tabs');
dojo.require('hub.main');
dojo.require('hub.pages');
dojo.require('hub.pages.main');
dojo.require('hub.pages.edit');
dojo.require('dojox.dtl.widget');
dojo.require('dojox.layout.ContentPane');
dojo.require('dijit.layout.TabContainer');
dojo.require('dijit.layout.StackContainer');
dojo.require('dojox.collections.Dictionary');
dojo.require('dojo.i18n');
dojo.require('hub.widgets.tabs.Workflow');
dojo.requireLocalization('hub', 'messages');

/**
 * Tracks the history of visited tabs so we can return to the last active,
 * living tab when the selected tab closes.
 */
dojo.declare('hub.widgets.tabs._TabHistory', null, {
	default_id: null,
	max_size: null,
	tab_id_stack: null,
	open_tab_ids: null,
	
    constructor: function(max_size, default_id) {
        this.default_id = default_id;
        this.max_size = max_size;
        this.tab_id_stack = new Array();
        this.open_tab_ids = new Object();
    },

    /**
     * Pops the given tab id out of the hash of all open tabs. Runs through
     * the stack of last visited tabs, but only if the closing tab is the
     * selected tab.
     * 
     * @param tab: Tab that is closing
     * @param need_pop: True to find the last active living tab
     */
    pop: function(tab, need_pop) {
//    console.log('tab_history.pop('+tab.id+')');
        // remove the id of the closing tab from our list of open tabs
        delete this.open_tab_ids[tab.id];
        if(!need_pop) {
            // don't continue if we don't need a pop (i.e., the closing
            // tab is not the selected tab)
            return null;
        }
        
        // pop the last id from the stack
        var id = this.tab_id_stack.pop();
        while(!this.open_tab_ids[id]) {
            if(!this.tab_id_stack.length) {
                return this.default_id;
            }
            id = this.tab_id_stack.pop();
        }
        return id;
    },
        

    /**
     * Pushes a tab id into the history of visited tabs and the stack of
     * all open tabs. Ignores duplicate pushes in the stack.
     *
     * @param tab: Tab that is being selected
     */
    push: function(tab) {
//    console.log('tab_history.push('+tab.id+')');
        if(this.tab_id_stack[this.tab_id_stack.length-1] == tab.id ||
           !tab.id) {
            // don't allow repeats in sequence, blank ids
            return;
        }
        this.tab_id_stack.push(tab.id);
        if(this.tab_id_stack.length > this.max_size) {
            this.tab_id_stack = this.tab_id_stack.slice(1);
        }
        // track that this tab is open
        this.open_tab_ids[tab.id] = true;
    }
});

/**
 * Manages opening, closing, refreshing, status, etc. of tabs.
 */
dojo.declare('hub.widgets.tabs.TabContainer', dijit.layout.TabContainer, {
    // widget to avoid when computing full screen height
    sizeAround: '',
    // maximum size of history ring buffer for tracking visited tabs
    historySize: 10,
    
    tab_history: null,
    
    tab_id_to_obj: null,
    
    next_key: 1,
    
    plugins: null,
    
    _started: false,

    constructor : function(args, node) {
        // tracks tabs opened by the user
        // key: bag of title, template, args, etc. identifying a page
        // value: Tab widget
        this.tab_id_to_obj = new dojox.collections.Dictionary();
        // unique tab id generator
        this.next_key = 1;
        
        this.plugins = Array();
    },

    /**
     * Overridden parent class method. Looks up the element to sizeAround.
     */
    postMixInProperties: function() {
        this.inherited(arguments);
        this.sizeAround = dojo.byId(this.sizeAround);
        // initialize the tab history tracker
        this.tab_history = new hub.widgets.tabs._TabHistory(this.historySize,
                                                            'hub_tab_1');
    },


    /**
     * Overridden parent class method. Start listening for events requesting a
     * page render and events indicating a refresh of one or more pages is 
     * needed.
     */
    postCreate: function() {
        this.inherited(arguments);
        dojo.subscribe('hub.login', this, 'refreshAllTabs');
        dojo.subscribe('hub.logout', this, 'refreshAllTabs');
        dojo.subscribe('hub.save-entry', this, 'refreshAllTabs');
        dojo.subscribe('hub.delete-entry', this, 'onEntryDelete');
        dojo.subscribe('hub.reset', this, 'refreshAllTabs');
        dojo.subscribe('hub.show-template-page', this, '_showTemplatePage');
        dojo.subscribe('hub.show-url-page', this, '_showUrlPage');
        dojo.subscribe('hub.show-plugin-page', this, '_showPluginPage');
        dojo.subscribe('hub.close-page', this, '_closePage');
        dojo.subscribe('hub.close-current-page', this, '_closeCurrentPage');
        dojo.subscribe('hub.is-page-active', this, '_isPageActive');
        dojo.subscribe('hub.busy-page', this, '_busyPage');
        dojo.subscribe('hub.rename-page', this, '_renamePage');

        // @todo: close tab having an object id
        //dojo.subscribe('hub.delete-entry', this, 'closeAllTabs');
        // @todo: close all tabs
    },

    /**
     * Overridden parent class method. Forces a resize.
     */
    startup: function() {
        this.inherited(arguments);
        // give the container its initial size
        this.resize();
    },

    /**
     * Overridden parent class method. Expands the tab container to full height
     * minus that of the sizeAround element.
     */
    resizeOld: function() { // TODO: check if this required
        if(this.sizeAround) {
            var height = (document.body.parentNode.offsetHeight -
                          this.sizeAround.offsetHeight - 100);
            this.domNode.style.height = height + 'px';
            this.layout();
        }
        this.inherited(arguments);
    },

    /**
     * Overridden parent class method. Pushes all selected tabs onto the 
     * history.
     *
     * @param tab: Tab to select
     */
    selectChild: function(tab) {
//    console.log('tabs.selectChild');
        if(this._started) {
            this.inherited(arguments);
            if (dojo.isString(tab)) {
            	tab = dijit.byId(tab);
            }
            // store tab id in history
            this.tab_history.push(tab);
            // publish an event so listeners know about the newly selected tab
            dojo.publish('hub.activate-page', [tab]);
	        hub.resizeLayoutContainers(tab); // MH-87
        }
    },

    /**
     * Overridden parent class method. Closes a tab and returns to the last
     * active tab
     *
     * @param tab: Tab to close
     * @return: false if operation cancelled, true if not
     */
    closeChild: function(tab) {
    	console.log('tabs.closeChild');
        // make sure child not closable first
        if(!tab.closable) {
            // return true, because it wasn't a cancellation by the user
            return true;
        }
        // reimplement here because parent class doesn't return the value of
        // remove for us (ugh)
		var remove = tab.onClose(this, tab);
		if(remove) {
	            // check if we're removing the active widget
	            var need_pop = (tab == this.selectedChildWidget);
	            // pop tab(s) until a live tab is reached
	            var next_tab_id = this.tab_history.pop(tab, need_pop);
	            if(next_tab_id != null) {
	                // select before removing else removeChild() forces us back to
	                // first tab
	                this.selectChild(next_tab_id);
	            }
	            this.tab_id_to_obj.remove(tab.plain_title.toLowerCase());
	            // remove tab from container
		    this.removeChild(tab);
	            // cleanup all widgets inside the tab
		    tab.destroyRecursive();
		}
        // return the value so we can reuse this method
        return remove;
    },

    /**
     * Forces a refresh of content in all tabs the next time each tab shows. 
     * Tabs can still choose to cancel the refresh, but only at the time they
     * are shown, not when this method is called.
     */
    refreshAllTabs: function() {
        dojo.forEach(this.getChildren(), function(tab) {
            if(tab == this.selectedChildWidget) {
                // force refresh now
                tab.refresh();
            } else {
                // do it later when the tab shows
                console.log('refreshAllTabs() - postpone');
                tab.refreshOnShow = !tab.isStatic && tab.entryId == null;
            }
        }, this);
        // force a resize
        this.resize();
    },

    /**
     * Attempts to close all tabs. If a tab aborts, all further closes are 
     * aborted also.
     *
     * @return: true if all closed, false if any one aborted
     */
    closeAllTabs: function() {
        var rv = dojo.every(this.getChildren(), this.closeChild, this);
        return rv;
    },

	closeEntryTabs: function(entryId){
//	console.log('tabs.closeEntryTabs('+entryId+')');
		if(dojo.isString(entryId)){
			entryId = parseInt(entryId);
		}
		if(entryId > 0){
			var tabs = this.getChildren(); 
			for(var i = 0; i < tabs.length; i++){
				if(entryId == tabs[i].getEntryId()){
//					console.log('entryId = ' + entryId + ' tabs[i].getEntryId() = ' + tabs[i].getEntryId());
					tabs[i].forceClose = true;
					this.closeChild(tabs[i]);
				}
			};
		}
	},
	
	onEntryDelete: function(args){
//		console.log('tabs.onEntryDelete');
//		console.dir({args:args});
		if(args){
			if(dojo.isArray(args)){
				for(var i = 0; i < args.length; i++){
					this.closeEntryTabs(args[i]);
				}
			}else{
				this.closeEntryTabs(args);
			}
			this.refreshAllTabs();
		}
	},

	isEverythingSaved: function(){
	console.log('tabs.isEverythingSaved()');
		var tabs = this.getChildren(); 
		for(var i = 0; i < tabs.length; i++){
			if(tabs[i].workflow!=null && tabs[i].workflow.isModified()){
				console.log('not Saved');
				return false;
			}
		};
		
		console.log('Saved');
		return true;
	},
	
    /**
     * Attempts to close one or more tabs. Return true if the operation was
     * not cancelled. This could mean the tab closed or did not exist to begin
     * with.
     *
     * @param node: Any DOM node in the page to close, or null for all pages
     * @param result: Optional deferred which will be notified with the result
     */
    _closePage: function(node, result) {
        // default to true meaning operation was accepted
        var rv = true;
        if(!node) {
            // close all tabs
            var rv = this.closeAllTabs();
        } else {
            // close a particular tab
            var tab = this._getTabForNode(node);
            if(tab) {
                var rv = this.closeChild(tab);
            }
        }
        if(result) {
            // if result provided, do callback
            result.callback(rv);
        }
        return true;
    },

    /**
     * Attempts to close one or more tabs. Return true if the operation was
     * not cancelled. This could mean the tab closed or did not exist to begin
     * with.
     *
     * @param node: Any DOM node in the page to close, or null for all pages
     * @param result: Optional deferred which will be notified with the result
     */
    _closeCurrentPage: function() {
        var tab = this.selectedChildWidget;
        if(tab) {
            var rv = this.closeChild(tab);
        }
        return true;
    },

    /**
     * Sets a tab as busy or not, or locked or not. Busy means the tab shows an
     * indicator near its title. Locked means the tab contents (and only this 
     * tab contents) are obscured by a dialog with a cancel button which 
     * prevents further user interaction with this tab.
     * 
     * @param node: Any node in the page
     * @param busy: True to mark busy, false to unmark
     * @param locked: True to lock, false to unlock
     */
    _busyPage: function(node, busy, locked) {
//    console.dir(node);
        // @todo: implement; do _getTabForNode and then show the busy spinner
        //   on the tab; need a design to support locking
        
        tab = this._getTabForNode(node);
        if(!tab) return;
        if (tab.workflow != null) {
           tab.workflow.setBusy(busy);
        }
        
        tab.setBusy(busy);
        
/*        var spinner = dojo.byId(tab.id+"_busy");
        if(!spinner) return;
        spinner.style.display = (busy) ? "inline" : "none";
*/        
    },

    /**
     * Renames an existing tab.
     * 
     * @param node: Any node in the page
     * @param title: New title
     * @param locked: Options for the newly renamed page
     */
    _renamePage: function(node, title, args) {
//    console.dir(node);
        // @todo: implement; refer to old tab manager for starting code
        tab = this._getTabForNode(node);
        if(!tab) return;
        var oldTitle = tab.plain_title;
        tab.setTitle(title);
        this.tab_id_to_obj.remove(oldTitle.toLowerCase());
        this.tab_id_to_obj.add(title.toLowerCase(), tab);
        
    },

    /**
     * Gets if a tab is active or not.
     * 
     * @param node: Any node in the page
     * @param result: Deferred which will be notified with the result
     */
    _isPageActive: function(node, result) {
//    console.dir(node);
        var rv = this.selectedChildWidget == this._getTabForNode(node);
        if(result) {
            result.callback(rv);
        }
    },

    /**
     * Returns markup for a tab label complete with spinner and full title
     * hover.
     *
     * @param title: Title for the tab, will be truncated if necessary
     * @param tab_id: Dijit id for the tab
     */
    _createTabLabel: function(title, tab_id) {
        // shorten visible label
        var short_label = hub.shortenLabel(title);
        // escape special characeters
        short_label = hub.htmlspecialchars(short_label);
        title = hub.htmlspecialchars(title);
        // return full label markup
        return "<img src='"+hub.urls.getImage('spinner.gif')+"' id='"+tab_id+"_busy' style='display: none;' /> <span title='"+title+"' id='"+tab_id+"_title'> " + short_label + "</span>";
    },

    /**
     * Factory method for creating new tabs for the TabContainer.
     *
     * @param title: Title of the new tab
     * @param closeable: true if the tab can be closed, false if permanent
     * @param workflow: true to create a workflow tab, false if just a regular
     *   content pane tab
     */
//    _createTab : function(title, closeable, workflow) {
    _createTab: function(title, props) {
        // build unique ID
        var tab_id = 'hub_tab_' + this.next_key++;
        // populate the parameters for the tab object
        var params = {
        	id: tab_id,
            plain_title: title,
            title: this._createTabLabel(title, tab_id),
            closable: props.closeable, // yes, dojo spells it differently
            cacheContent: false,
            preload: false,
            executeScripts: true
        };
        
        // create a div for the tab object
        var div = dojo.doc.createElement('div');
        // build the widget
        var tab = new hub.widgets.tabs.ContentPane(params, div);
        // add the widget to the tab container
        this.addChild(tab);
        // track tab by its title
        this.tab_id_to_obj.add(title.toLowerCase(), tab);
        return tab;
    },

    /**
     * Decides whether to reuse a tab or create a new one. Currently, the
     * algorithm considers only the page title as the unique ID.
     *
     * @todo: what else can we use as an ID? the problem is that titles are not
     *   unique, but we want to keep the unique ID as something consistent 
     *   across all links in the UI and easy to construct. *NOT easy to solve*
     *   one possibility is to combine the props and title to form a unique
     *   identifier, but this forces widgets that want to reuse the current tab
     *   to *always* specify props.current=true since the unique ID will differ
     *   when trying to reuse a tab to show a different page from the current
     *
     * @param title: Title
     * @param props: Properties to apply to the tab or to use when decide how
     *   to get/create a tab
     */
    _getTab: function(title, props) {
    	var tab = null;
        if(props.current && this.selectedChildWidget) {
            // reuse the current tab, if one exists
            tab = this.selectedChildWidget;
        }
        else{
        	tab = this.tab_id_to_obj.item(title.toLowerCase());
        }
        
        if(tab) {
	        this.selectChild(tab);
	        if(tab.isProtected && !props.safe) {
	            if(!tab._confirmOverwrite()) {
	                // abort
	                return null;
	            }
	        }
	        
	        if (tab.workflow && !tab.workflow.isReedit()) {
	           // do not reuse this tab as a delete event may close it.
	           tab.cleanTab();
	           tab.isProtected = false;
	           tab.forceClose = true;
	           this.closeChild(tab);
	           tab = null;
	        }   
	        else {
			   tab.cleanTab();
			}
		}
		
		if (!tab) {
            // can only decide closeable or not on creation (I think)
            // limitation of dojo
            props.closeable = !props.permanent;
            tab = this._createTab(title, props);
	        this.selectChild(tab);
        }
        // @todo: need to be smarter about re-using a tab when switching from
        // a wizard to a single screen page, or a single screen page to a wizard
        // when this happens, the easiest solution is to destroy the current
        // tab and create another one with the same title; the old code 
        // attempted to reuse the same tab with much pain and suffering as a 
        // result
        return tab;
    },

    /**
     * Finds the tab containing the given DOM node. Returns null if not
     * within a known tab of this tab container.
     *
     * @param node: Any DOM node
     */
    _getTabForNode: function(node) {
//    console.log('tabs._getTabForNode');
//	        console.log(node);
//	        console.dir({tab: node});
    	if(node != null){
	        if(hub.widgets.tabs.isTab(node)){
//	    		console.log('tab title: ' + node.plain_title);
	        	return node;
	        }
	        if(dojo.isString(node)) {
	            node = dojo.byId(node);
	        }
	        var child = null;
	        var gchild = null;
	        while(node != null) {
	            gchild = child;
	            child = node;
	            node = node.parentNode;
	            var w = node?dijit.byNode(node):null;
	            if(w == this) {
	                // tab is stored two levels below the container, inside
	                // the tab pane wrapper
	                return dijit.byNode(gchild);
	            }
	        }
    	}
        return null;
    },

    /**
     * Shows a tab rendering a plug-in editor. The editor may either start on
     * a page where the user selects the plug-in for a new entry, or on the 
     * first page of the editor for an existing entry. The tab created is a
     * wizard so that the user can flip back and forth through pages.
     * 
     * @param title: Title to put on the tab
     * @param args: Options for rendering the page.
     *   Can also include "current":true specifying that the currently 
     *   shown tab should be reused to render this page template and "safe":true
     *   to force a render in the chosen tab, disregarding any protection on
     *   the tab.
     */
    _showPluginPage: function(title, plugin_id, object_id, args) {
        // @todo: implement; add property "workflow: true" to args so that a 
        // hub.widgets.tabs.edit.Workflow gets constructed instead of just a 
        // content pane
        // get/create tab
        dojo.require('hub.widgets.tabs.edit');

        if(!args) {
            // always provide an object
            args = {};
        }
        args.workflow = true;
        
        var tab = this._getTab(title, args);
        if(tab == null){ 
        	return;
        }
        
		tab.setStatic(false);
        dojo.require('hub.managers.Plugin');
        var div = dojo.doc.createElement('div');

		tab.domNode.appendChild(div);
        var workflow = new hub.widgets.tabs.Workflow({pluginId:plugin_id, entryId:object_id, type:args.type, fullscreen:args.fullscreen, editSource: args.editSource, editAsNew: args.editAsNew}, div);
        workflow.setTab(tab);
        tab.workflow = workflow;
        tab.page_cls = null;

		workflow.startup();
		
        this.resize();
       

    },
    
    /**
     * Shows a tab rendering a hub.pages._Page template. Can either reuse an
     * existing tab or create a new tab depending on passed arguments and 
     * whether an appropriate tab already exists.
     * 
     * @param title: Title to put on the tab
     * @param template: Name of the page/template rooted at hub.pages to render
     * @param args: Options for rendering the page.
     *   Can also include "current":true specifying that the currently 
     *   shown tab should be reused to render this page template and "safe":true
     *   to force a render in the chosen tab, disregarding any protection on
     *   the tab.
     */
    _showTemplatePage: function(title, template, args) {
        // parse out page name versus module name
        var arr = template.split('.');
        var page = arr[arr.length-1];
        var module =  arr.slice(0, -1);
        module = module.join('.');
        // make sure module is imported
        if (module) {
            dojo.require(module);
            arr=arr.slice(2, arr.length-1);
        }
        // loop through modules to page class
        var page_cls = hub.pages;
        dojo.forEach(arr, function(part) {
            page_cls = page_cls[part];
            // if bad path to page, quit
            if(!page_cls) {
                throw new Error('TabContainer: no template named '+template);
            }
        });

        // ask page *class* for properties (static method call)
        var props = page_cls.prototype.getProperties();
        // mix given args on top of class-specified args
        dojo.mixin(props, args);

        // get/create tab
        var tab = this._getTab(title, props);
        if(tab == null){ 
        	return;
        }
        
		tab.setStatic(false);

        // instruct the tab to render the page when appropriate
        tab.setPage(page_cls, props);
    },

    /**
     * Shows a tab rendering HTML returned by fetching HTML at a URL.
     * 
     * @param title: Title to put on the tab
     * @param url: URL to fetch
     * @param args: Options for rendering the page.
     *   Can also include "current":true specifying that the currently 
     *   shown tab should be reused to render this page template and "safe":true
     *   to force a render in the chosen tab, disregarding any protection on
     *   the tab.
     */
    _showUrlPage: function(title, url, args) {
        // get/create tab; no page properties
        var tab = this._getTab(title, {});
        if(tab == null){ 
        	return;
        }
        

        if(!args) {
            // always provide an object
            args = {};
        }
		
		tab.setStatic(true);
		
        // instruct the tab to fetch the url, rooted at the hub.config.base_url
        if (url.indexOf("http") == 0) {
        	tab.setHref(url, args);
        } else {
        	tab.setHref(hub.config.base_url + '/' + url, args);
        }
    }
});

/**
 * Custom content pane. Supports rendering of page templates and content 
 * fetched from URLs. Also supports a flag indicating whether the tab should 
 * be protected from overwriting by showing a confirmation dialog or not.
 * Uses dojox.layout.ContentPane instead of the dijit one to support inline JS
 * returned by plug-ins.
 */ 
//dojo.declare('hub.widgets.tabs.PaneContent', dojox.layout.ContentPane, {
dojo.declare('hub.widgets.tabs.ContentPane', dojox.layout.ContentPane, {
    // stores a reference to a class for a page template
    page_cls: null,
    // stores arguments for rendering a page template
    page_args: null,
    // original, plain-text title
    plain_title: '',
    
    loadingMessage: 'Loading...',
    
    forceClose: false,
    
    isProtected: false,
    
    selected: false,

    workflow: null,
    
    entryId: null,
    
    isStatic: false,

    /**
     * Overridden parent class method. Uses the Mashup Hub loading message 
     * instead of the stock dojo one.
     */
    postCreate: function() {
        this.inherited(arguments);
        // override with our own busy message
        this.loadingMessage = hub.busyMessage();
    },
    
    /**
     * Overridden parent class method. Loads the content of a URL. 
     * Clears any page rendering information.
     *
     * @param url: URL to fetch
     * @param args: Object with flags determining if replacement of 
     * rendered content is safe or not (overrides protection) and if the new 
     * rendering should be protected
     */
    setHref: function(url, args) {
        if(args.entryId){
        	this.entryId = args.entryId;
        }
        if ("protected" in args) {
        	this.isProtected = args['protected'];
        }
        else {
        	this.isProtected = args.isProtected;
        }
        this.page_cls = null;
        this.page_args = null;
        this.inherited('setHref', arguments);
    },

    /**
     * Loads a template page. Clears any URL loading information.
     *
     * @param page_cls: hub.pages._Page subclass
     * @param page_args: Object to pass to the page when rendering. Also 
     *   specifies if replacement of rendered content is safe or not (overrides
     *   protection) and if the new rendering should be protected or not
     */
    setPage: function(page_cls, args) {

        this.refreshOnShow = true;
        this.page_cls = page_cls;
        this.page_cls.tab = this;
        this.page_args = args;
        this.page_args.tab = this;
        this.href = '';
        if ("protected" in args) {
        	this.isProtected = args['protected'];
        }
        else {
        	this.isProtected = args.isProtected;
        }

        // do onShow processing immediately if already selected
        if(this.selected) {
            this.onShow();
        }
    },

    /**
     * Shows or hides a spinner on this tab.
     *
     * @param busy: Show (true) or hide (false) busy indicator
     */
    setBusy: function(busy) {
        var spinner = dojo.byId(this.id+"_busy");
        if(!spinner) return;
        spinner.style.display = (busy) ? "inline" : "none";
    },
    
    setTitle: function(title){
    	this.plain_title = title;
        var titleContainer = dojo.byId(this.id+"_title");
        if(!titleContainer) return;
        var short_label = hub.shortenLabel(title);
        // escape special characeters
        short_label = hub.htmlspecialchars(short_label);
        title = hub.htmlspecialchars(title);
        titleContainer.innerHTML = short_label;
        titleContainer.setAttribute('title', title);
    },

    setStatic: function(val) {
       this.isStatic = val;
    },
    
    /**
     * Sets whether the user has to confirm overwrites to pages or URLs shown
     * in this tab or not.
     *
     * @param val: True to protect, false to unprotect
     */
    setProtected: function(val) {
        this.isProtected = val;
    },
    
    getEntryId: function(){
    	if(this.entryId){
	    	if(dojo.isString(this.entryId)){
    			return parseInt(this.entryId);
    		}
    		return this.entryId; 
    	}
    	return 0;
    },
    
    cleanTab: function(){
        console.log('tabs.cleanTab');
		// cancel any outstanding url request (in case we're reusing a tab)
		this.cancel();

		if(this.workflow){
			if(this.workflow.saved){
				this.workflow.dispose();
			}else{
				this.workflow.cancel({});
			}
		}

		// destroy all existing widgets
		this.destroyDescendants();

		// clear all inner content (the safe for IE way)
		var node = this.containerNode || this.domNode;
		while(node.firstChild) {
			dojo._destroyElement(node.firstChild);
		}
    },

    /**
     * Forces a refresh of the current page. Re-renders a template or fetches
     * fresh data from a URL.
     *
     * @todo: tons of optimization work can be done here to re-use templates,
     * template server data tha never goes stale, etc.
     */
    refresh: function() {
        console.log('tabs.refresh');
        // always reset refresh on show
        this.refreshOnShow = false;
		if(!this.isStatic){
	        if(this.page_cls) {
	            // do template rendering
	            // set tab busy
	            this.setBusy(true);
	            
	            this.cleanTab();
	
	/*
	            // cancel any outstanding url request (in case we're reusing a tab)
	            this.cancel();
	            // destroy all existing widgets
			    this.destroyDescendants();
		            // clear all inner content (the safe for IE way)
			    var node = this.containerNode || this.domNode;
			    while(node.firstChild) {
					dojo._destroyElement(node.firstChild);
			    }
	*/
	            // create divs for dojox.dtl.AttachPoint and hub.pages._Page
	            var attach_div = document.createElement('DIV');
	            var page_div = document.createElement('DIV');
	            attach_div.appendChild(page_div);
	            this.domNode.appendChild(attach_div);
	
	            // create attach point widget
	            var ap = new dojox.dtl.AttachPoint(null, attach_div);
	            // create page widget
	            var p = new this.page_cls(this.page_args, page_div);
	            // listen for render complete
	            return p.isRendered().addCallback(dojo.hitch(this, this._onShowComplete));
	        } else {
	        	console.log('has not page_cls');
	            // do normal content pane loading from server
	            return this.inherited('refresh', arguments);
	        }
        }
    },

    /**
     * Starts rendering of a page template when setPage has been called and 
     * refreshOnShow is set.
     */
    onShow: function() {
        if(this.page_cls && this.refreshOnShow) {
            this.refresh();
        }
        // don't do anything for fetching URLs when setHref has been called
        // the parent class handles that through a different set of functions
    },

    /**
     * Confirms closing if tab is protected.
     *
     * @return: true to continue with closing, false to cancel
     */
    onClose: function() {
//      	console.log('tabs.onClose');
        if(this.isProtected) {
            if(this._confirmOverwrite()){
		        this.cleanTab();
        		return true;    	
            }else{
            	return false;
            }
        }
//        console.dir({workflow: this.workflow});
        if(!this.forceClose
        && this.workflow
        && dojo.isObject(this.workflow) 
        && this.workflow.pluginId != null 
        && this.workflow.pluginId > 0
        && (this.workflow.saveScreen && this.workflow.currentScreen <= this.workflow.saveScreen+1 || !this.workflow.saveScreen)) {
//	      	console.log('plugin defined '+this.workflow.pluginId);
        	if(this.workflow.reedit || this.workflow.saved){
//		      	console.log('plugin saved');
	        	if( this._confirmOverwrite()){
			        this.cleanTab();
	        		return true;    	
	            }else{
	            	return false;
	        	}
        	}else{
//		      	console.log('plugin not saved');
	        	if(this._confirmDiscard()){
			        this.cleanTab();
	        		return true;    	
	            }else{
	            	return false;
	        	}
        	}
        }

        // if not protected, always allow close
        return true;
    },

    /**
     * Completes rendering of a page by parsing for Dojo content and setting
     * the tab as no longer busy.
     */
    _onShowComplete: function() {
        // parse tab for dojo content
        dojo.parser.parse(this.domNode);
        // set tab unbusy
        this.setBusy(false);        
    },
    
    /**
     * If the tab is a protected tab (one that we should be careful about
     * refreshing or closing) ask if the user is sure he or she wants to
     * close the tab.
     *
     * @param tab: The tab that is changing or closing
     * @param args: Arguments object, possibly with "safe" property.
     * @return: true to continue with action, false to cancel
     */
    _confirmOverwrite: function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');        
        return confirm(msgs.PROMPT_CLOSE);
    },

    _confirmDiscard: function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');        
        return confirm(msgs.PROMPT_DISCARD);
    }
});

dojo.declare('hub.widgets.tabs.PaneWorkflow', hub.widgets.tabs.PaneContent, {
    // stores a reference to a class for a page template
    workflow: null,

	onClose: function(){
		if(this.workflow){
			if(this.workflow.saved){
				this.workflow.dispose();
			}else{
				this.workflow.cancel({});
			}
		}
		return this.inherited("onClose",arguments);
	}
	
});

hub.widgets.tabs.isTab = function(node){
//    alert(node);
//    console.dir(node);
	return node!=null && dojo.isObject(node) && node.declaredClass && (" "+node.declaredClass).indexOf("hub.widgets.tabs.ContentPane") > 0;
};

hub.widgets.tabs.getTabForNode = function(node) {
//    console.dir(node);
	if(node!=null){
        if(hub.widgets.tabs.isTab(node)){
        	return node;
        }
        if(dojo.isString(node)) {
            node = dojo.byId(node);
        }
        var child = null;
        var gchild = null;
        while(node != null) {
            gchild = child;
            child = node;
            node = node.parentNode;
            var w = dijit.byNode(node);
			if( w && w.page_cls ) {
                // the content pane
                return w;
            }
        }
    }
    return null;
}

