/*

	Copyright (c) 2007-2008, IBM Corp.
*/

/*
	This is a compiled version of Dojo, built for deployment and not for
	development. To get an editable version, please visit:

		http://dojotoolkit.org

	for documentation and information on getting the source.
*/

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");

// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become actived/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example:  Combobutton --> Menu -->
//		MenuItem.   The onBlur event for Combobutton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		Combobutton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit,
{
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary: tests whether the current selection is empty
		var _window = dojo.global;
		var _document = dojo.doc;
		if(_document.selection){ // IE
			return !_document.selection.createRange().text; // Boolean
		}else if(_window.getSelection){
			var selection = _window.getSelection();
			if(dojo.isString(selection)){ // Safari
				return !selection; // Boolean
			}else{ // Mozilla/W3
				return selection.isCollapsed || !selection.toString(); // Boolean
			}
		}
	},

	getBookmark: function(){
		// summary: Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bookmark, selection = dojo.doc.selection;
		if(selection){ // IE
			var range = selection.createRange();
			if(selection.type.toUpperCase()=='CONTROL'){
				bookmark = range.length ? dojo._toArray(range) : null;
			}else{
				bookmark = range.getBookmark();
			}
		}else{
			if(dojo.global.getSelection){
				selection = dojo.global.getSelection();
				if(selection){
					var range = selection.getRangeAt(0);
					bookmark = range.cloneRange();
				}
			}else{
				console.debug("No idea how to store the current selection for this browser!");
			}
		}
		return bookmark; // Array
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary: Moves current selection to a bookmark
		// bookmark: this should be a returned object from dojo.html.selection.getBookmark()
		var _document = dojo.doc;
		if(_document.selection){ // IE
			var range;
			if(dojo.isArray(bookmark)){
				range = _document.body.createControlRange();
				dojo.forEach(bookmark, range.addElement);
			}else{
				range = _document.selection.createRange();
				range.moveToBookmark(bookmark);
			}
			range.select();
		}else{ //Moz/W3C
			var selection = dojo.global.getSelection && dojo.global.getSelection();
			if(selection && selection.removeAllRanges){
				selection.removeAllRanges();
				selection.addRange(bookmark);
			}else{
				console.debug("No idea how to restore selection for this browser!");
			}
		}
	},

	getFocus: function(/*Widget*/menu, /*Window*/ openedForWindow){
		// summary:
		//	Returns the current focus and selection.
		//	Called when a popup appears (either a top level menu or a dialog),
		//	or when a toolbar/menubar receives focus
		//
		// menu:
		//	the menu that's being opened
		//
		// openedForWindow:
		//	iframe in which menu was opened
		//
		// returns:
		//	a handle to restore focus/selection

		return {
			// Node to return focus to
			node: menu && dojo.isDescendant(dijit._curFocus, menu.domNode) ? dijit._prevFocus : dijit._curFocus,

			// Previously selected text
			bookmark:
				!dojo.withGlobal(openedForWindow||dojo.global, dijit.isCollapsed) ?
				dojo.withGlobal(openedForWindow||dojo.global, dijit.getBookmark) :
				null,

			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase()=="iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}			
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item)
		if(bookmark && dojo.withGlobal(openedForWindow||dojo.global, dijit.isCollapsed)){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow||dojo.global, moveToBookmark, null, [bookmark]);
			}catch(e){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerWin: function(/*Window?*/targetWindow){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe) to detect when the user has clicked somewhere.
		//		Anyone that creates an iframe should call this function.

		if(!targetWindow){
			targetWindow = window;
		}

		dojo.connect(targetWindow.document, "onmousedown", null, function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			dijit._onTouchNode(evt.target||evt.srcElement);
		});
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's body
		var body = targetWindow.document.body || targetWindow.document.getElementsByTagName("body")[0];
		if(body){
			if(dojo.isIE){
				body.attachEvent('onactivate', function(evt){
					if(evt.srcElement.tagName.toLowerCase() != "body"){
						dijit._onFocusNode(evt.srcElement);
					}
				});
				body.attachEvent('ondeactivate', function(evt){ dijit._onBlurNode(evt.srcElement); });
			}else{
				body.addEventListener('focus', function(evt){ dijit._onFocusNode(evt.target); }, true);
				body.addEventListener('blur', function(evt){ dijit._onBlurNode(evt.target); }, true);
			}
		}
		body = null;	// prevent memory leak (apparent circular reference via closure)
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		var w = dijit.getEnclosingWidget(node);
		if (w && w._setStateClass){
			w._focused = false;
			w._setStateClass();
		}
		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer; dijit._setStack([]); }, 100);
	},

	_onTouchNode: function(/*DomNode*/ node){
		// summary
		//		Callback when node is focused or mouse-downed

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				if(node.dijitPopupParent){
					node=dijit.byId(node.dijitPopupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase()=="body"){
					// is this the root of the document or just the root of an iframe?
					if(node===dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead) and continue tracing up the document
					node=dojo.query("iframe").filter(function(iframe){ return iframe.contentDocument.body===node; })[0];
				}else{
					var id = node.getAttribute && node.getAttribute("widgetId");
					if(id){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary
		//		Callback when node is focused
		if(node && node.tagName && node.tagName.toLowerCase() == "body"){
			return;
		}
		dijit._onTouchNode(node);
		if(node==dijit._curFocus){ return; }
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);

		// handle focus/blur styling
		var w = dijit.getEnclosingWidget(node);
		if (w && w._setStateClass){
			w._focused = true;
			w._setStateClass();
		}
	},

	_setStack: function(newStack){
		// summary
		//	The stack of active widgets has changed.  Send out appropriate events and record new stack

		var oldStack = dijit._activeStack;		
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			var widget = dijit.byId(oldStack[i]);
			if(widget){
				dojo.publish("widgetBlur", [widget]);
				if(widget._onBlur){
					widget._onBlur();
				}
			}
		}

		// for all element that have come into focus, send focus event
		for(var i=nCommon; i<newStack.length; i++){
			var widget = dijit.byId(newStack[i]);
			if(widget){
				dojo.publish("widgetFocus", [widget]);
				if(widget._onFocus){
					widget._onFocus();
				}
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(dijit.registerWin);

}

if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");

dojo.declare("dijit.WidgetSet", null, {
	constructor: function(){
		// summary:
		//	A set of widgets indexed by id
		this._hash={};
	},

	add: function(/*Widget*/ widget){
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id]=widget;
	},

	remove: function(/*String*/ id){
		delete this._hash[id];
	},

	forEach: function(/*Function*/ func){
		for(var id in this._hash){
			func(this._hash[id]);
		}
	},

	filter: function(/*Function*/ filter){
		var res = new dijit.WidgetSet();
		this.forEach(function(widget){
			if(filter(widget)){ res.add(widget); }
		});
		return res;		// dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		return this._hash[id];
	},

	byClass: function(/*String*/ cls){
		return this.filter(function(widget){ return widget.declaredClass==cls; });	// dijit.WidgetSet
	}
	});

// registry: list of all widgets on page
dijit.registry = new dijit.WidgetSet();

dijit._widgetTypeCtr = {};

dijit.getUniqueId = function(/*String*/widgetType){
	// summary
	//	Generates a unique id for a given widgetType

	var id;
	do{
		id = widgetType + "_" +
			(dijit._widgetTypeCtr[widgetType] !== undefined ?
				++dijit._widgetTypeCtr[widgetType] : dijit._widgetTypeCtr[widgetType] = 0);
	}while(dijit.byId(id));
	return id; // String
};


if(dojo.isIE){
	// Only run this for IE because we think it's only necessary in that case,
	// and because it causes problems on FF.  See bug #3531 for details.
	dojo.addOnUnload(function(){
		dijit.registry.forEach(function(widget){ widget.destroy(); });
	});
}

dijit.byId = function(/*String|Widget*/id){
	// summary:
	//		Returns a widget by its id, or if passed a widget, no-op (like dojo.byId())
	return (dojo.isString(id)) ? dijit.registry.byId(id) : id; // Widget
};

dijit.byNode = function(/* DOMNode */ node){
	// summary:
	//		Returns the widget as referenced by node
	return dijit.registry.byId(node.getAttribute("widgetId")); // Widget
};

dijit.getEnclosingWidget = function(/* DOMNode */ node){
	// summary:
	//		Returns the widget whose dom tree contains node or null if
	//		the node is not contained within the dom tree of any widget
	while(node){
		if(node.getAttribute && node.getAttribute("widgetId")){
			return dijit.registry.byId(node.getAttribute("widgetId"));
		}
		node = node.parentNode;
	}
	return null;
};

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");

// ported from dojo.html.util

dijit.getViewport = function(){
	//	summary
	//	Returns the dimensions and scroll position of the viewable area of a browser window

	var _window = dojo.global;
	var _document = dojo.doc;

	// get viewport size
	var w = 0, h = 0;
	if(dojo.isMozilla){
		// mozilla
		// _window.innerHeight includes the height taken by the scroll bar
		// clientHeight is ideal but has DTD issues:
		// #4539: FF reverses the roles of body.clientHeight/Width and documentElement.clientHeight/Width based on the DTD!
		// check DTD to see whether body or documentElement returns the viewport dimensions using this algorithm:
		var minw, minh, maxw, maxh;
		if(_document.body.clientWidth>_document.documentElement.clientWidth){
			minw = _document.documentElement.clientWidth;
			maxw = _document.body.clientWidth;
		}else{
			maxw = _document.documentElement.clientWidth;
			minw = _document.body.clientWidth;
		}
		if(_document.body.clientHeight>_document.documentElement.clientHeight){
			minh = _document.documentElement.clientHeight;
			maxh = _document.body.clientHeight;
		}else{
			maxh = _document.documentElement.clientHeight;
			minh = _document.body.clientHeight;
		}
		w = (maxw > _window.innerWidth) ? minw : maxw;
		h = (maxh > _window.innerHeight) ? minh : maxh;
	}else if(!dojo.isOpera && _window.innerWidth){
		//in opera9, dojo.body().clientWidth should be used, instead
		//of window.innerWidth/document.documentElement.clientWidth
		//so we have to check whether it is opera
		w = _window.innerWidth;
		h = _window.innerHeight;
	}else if(dojo.isIE && _document.documentElement && _document.documentElement.clientHeight){
		w = _document.documentElement.clientWidth;
		h = _document.documentElement.clientHeight;
	}else if(dojo.body().clientWidth){
		// IE5, Opera
		w = dojo.body().clientWidth;
		h = dojo.body().clientHeight;
	}

	// get scroll position
	var scroll = dojo._docScroll();

	return { w: w, h: h, l: scroll.x, t: scroll.y };	//	object
};

dijit.placeOnScreen = function(
	/* DomNode */	node,
	/* Object */		pos,
	/* Object */		corners,
	/* boolean? */		tryOnly){
	//	summary:
	//		Keeps 'node' in the visible area of the screen while trying to
	//		place closest to pos.x, pos.y. The input coordinates are
	//		expected to be the desired document position.
	//
	//		Set which corner(s) you want to bind to, such as
	//		
	//			placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])
	//		
	//		The desired x/y will be treated as the topleft(TL)/topright(TR) or
	//		BottomLeft(BL)/BottomRight(BR) corner of the node. Each corner is tested
	//		and if a perfect match is found, it will be used. Otherwise, it goes through
	//		all of the specified corners, and choose the most appropriate one.
	//		
	//		NOTE: node is assumed to be absolutely or relatively positioned.

	var choices = dojo.map(corners, function(corner){ return { corner: corner, pos: pos }; });

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, /* Array */ choices, /* Function */ layoutNode){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	//	layoutNode: Function(node, orient)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dijit.getViewport();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to document.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best=null;
	for(var i=0; i<choices.length; i++){
		var corner = choices[i].corner;
		var pos = choices[i].pos;

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltips size changes based on position, due to triangle)
		if(layoutNode){
			layoutNode(corner);
		}

		// get node's size
		var oldDisplay = node.style.display;
		var oldVis = node.style.visibility;
		node.style.visibility = "hidden";
		node.style.display = "";
		var mb = dojo.marginBox(node);
		node.style.display = oldDisplay;
		node.style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = (corner.charAt(1)=='L' ? pos.x : Math.max(view.l, pos.x - mb.w)),
			startY = (corner.charAt(0)=='T' ? pos.y : Math.max(view.t, pos.y -  mb.h)),
			endX = (corner.charAt(1)=='L' ? Math.min(view.l+view.w, startX+mb.w) : pos.x),
			endY = (corner.charAt(0)=='T' ? Math.min(view.t+view.h, startY+mb.h) : pos.y),
			width = endX-startX,
			height = endY-startY,
			overflow = (mb.w-width) + (mb.h-height);

		if(best==null || overflow<best.overflow){
			best = {
				corner: corner,
				aroundCorner: choices[i].aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow
			};
		}
		if(overflow==0){
			break;
		}
	}

	node.style.left = best.x + "px";
	node.style.top = best.y + "px";
	return best;
}

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	//	summary
	//	Like placeOnScreen, except it accepts aroundNode instead of x,y
	//	and attempts to place node around it.  Uses margin box dimensions.
	//
	//	aroundCorners
	//		specify Which corner of aroundNode should be
	//		used to place the node => which corner(s) of node to use (see the
	//		corners parameter in dijit.placeOnScreen)
	//		e.g. {'TL': 'BL', 'BL': 'TL'}
	//
	//	layoutNode: Function(node, orient)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.


	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var oldDisplay = aroundNode.style.display;
	aroundNode.style.display="";
	// #3172: use the slightly tighter border box instead of marginBox
	var aroundNodeW = aroundNode.offsetWidth; //mb.w;
	var aroundNodeH = aroundNode.offsetHeight; //mb.h;
	var aroundNodePos = dojo.coords(aroundNode, true);
	aroundNode.style.display=oldDisplay;

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: aroundNodePos.x + (nodeCorner.charAt(1)=='L' ? 0 : aroundNodeW),
				y: aroundNodePos.y + (nodeCorner.charAt(0)=='T' ? 0 : aroundNodeH)
			}
		});
	}

	return dijit._place(node, choices, layoutNode);
}

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");

dijit.getDocumentWindow = function(doc){
	//	summary
	// 	Get window object associated with document doc

	// With Safari, there is not way to retrieve the window from the document, so we must fix it.
	if(dojo.isSafari && !doc._parentWindow){
		/*
			This is a Safari specific function that fix the reference to the parent
			window from the document object.
		*/
		var fix=function(win){
			win.document._parentWindow=win;
			for(var i=0; i<win.frames.length; i++){
				fix(win.frames[i]);
			}
		}
		fix(window.top);
	}

	//In some IE versions (at least 6.0), document.parentWindow does not return a
	//reference to the real window object (maybe a copy), so we must fix it as well
	//We use IE specific execScript to attach the real window reference to
	//document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow && !doc._parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc._parentWindow || doc.parentWindow || doc.defaultView;	//	Window
}

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");





dijit.popup = new function(){
	// summary:
	//		This class is used to show/hide widgets as popups.
	//

	var stack = [],
		beginZIndex=1000,
		idGen = 1;

	this.open = function(/*Object*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// args: Object
		//		popup: Widget
		//			widget to display,
		//		parent: Widget
		//			the button etc. that is displaying this popup
		//		around: DomNode
		//			DOM node (typically a button); place popup relative to this node
		//		orient: Object
		//			structure specifying possible positions of popup relative to "around" node
		//		onCancel: Function
		//			callback when user has canceled the popup by
		//				1. hitting ESC or
		//				2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
		//				   ie: whenever popupWidget.onCancel() is called, args.onCancel is called
		//		onClose: Function
		//			callback whenever this popup is closed
		//		onExecute: Function
		//			callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
		//
		// examples:
		//		1. opening at the mouse position
		//			dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//		2. opening the widget as a dropdown
		//			dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}  });
		//
		//	Note that whatever widget called dijit.popup.open() should also listen to it's own _onBlur callback
		//	(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var widget = args.popup,
			orient = args.orient || {'BL':'TL', 'TL':'BL'},
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+idGen++);

		// make wrapper div to hold widget and possibly hold iframe behind it.
		// we can't attach the iframe as a child of the widget.domNode because
		// widget.domNode might be a <table>, <ul>, etc.
		var wrapper = dojo.doc.createElement("div");
		wrapper.id = id;
		wrapper.className="dijitPopup";
		wrapper.style.zIndex = beginZIndex + stack.length;
		wrapper.style.visibility = "hidden";
		if(args.parent){
			wrapper.dijitPopupParent=args.parent.id;
		}
		dojo.body().appendChild(wrapper);

		widget.domNode.style.display="";
		wrapper.appendChild(widget.domNode);

		var iframe = new dijit.BackgroundIframe(wrapper);

		// position the wrapper node
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR']);

		wrapper.style.visibility = "visible";
		// TODO: use effects to fade in wrapper

		var handlers = [];

		// Compute the closest ancestor popup that's *not* a child of another popup.
		// Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		function getTopPopup(){
			for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--);
			return stack[pi];
		}

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.keyCode == dojo.keys.ESCAPE && args.onCancel){
				args.onCancel();
			}else if(evt.keyCode == dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", null, args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", null, function(){
			var topPopup = getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			wrapper: wrapper,
			iframe: iframe,
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			widget.onOpen(best);
		}

		return best;
	};

	this.close = function(/*Widget*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented
		while(dojo.some(stack, function(elem){return elem.widget == popup;})){
			var top = stack.pop(),
				wrapper = top.wrapper,
				iframe = top.iframe,
				widget = top.widget,
				onClose = top.onClose;
	
			if(widget.onClose){
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);
	
			// #2685: check if the widget still has a domNode so ContentPane can change its URL without getting an error
			if(!widget||!widget.domNode){ return; }
			dojo.style(widget.domNode, "display", "none");
			dojo.body().appendChild(widget.domNode);
			iframe.destroy();
			dojo._destroyElement(wrapper);
	
			if(onClose){
				onClose();
			}
		}
	};
}();

dijit._frames = new function(){
	// summary: cache of iframes
	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE){
				var html="<iframe src='javascript:\"\"'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	var iframe = dojo.doc.createElement("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didnt work.
			dojo.body().appendChild(iframe);
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="";
		if(dojo.isIE){
			iframe.style.removeExpression("width");
			iframe.style.removeExpression("height");
		}
		queue.push(iframe);
	}
}();

// fill the queue
if(dojo.isIE && dojo.isIE < 7){
	dojo.addOnLoad(function(){
		var f = dijit._frames;
		dojo.forEach([f.pop()], f.push);
	});
}


dijit.BackgroundIframe = function(/* DomNode */node){
	//	summary:
	//		For IE z-index schenanigans. id attribute is required.
	//
	//	description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if((dojo.isIE && dojo.isIE < 7) || (dojo.isFF && dojo.isFF < 3 && dojo.hasClass(dojo.body(), "dijit_a11y"))){
		var iframe = dijit._frames.pop();
		node.appendChild(iframe);
		if(dojo.isIE){
			iframe.style.setExpression("width", "document.getElementById('" + node.id + "').offsetWidth");
			iframe.style.setExpression("height", "document.getElementById('" + node.id + "').offsetHeight");
		}
		this.iframe = iframe;
	}
};

dojo.extend(dijit.BackgroundIframe, {
	destroy: function(){
		//	summary: destroy the iframe
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");

dijit.scrollIntoView = function(/* DomNode */node){
	//	summary
	//	Scroll the passed node into view, if it is not.

	// don't rely on that node.scrollIntoView works just because the function is there
	// it doesnt work in Konqueror or Opera even though the function is there and probably
	// not safari either
	// dont like browser sniffs implementations but sometimes you have to use it
	if(dojo.isIE){
		//only call scrollIntoView if there is a scrollbar for this menu,
		//otherwise, scrollIntoView will scroll the window scrollbar
		if(dojo.marginBox(node.parentNode).h <= node.parentNode.scrollHeight){ //PORT was getBorderBox
			node.scrollIntoView(false);
		}
	}else if(dojo.isMozilla){
		node.scrollIntoView(false);
	}else{
		var parent = node.parentNode;
		var parentBottom = parent.scrollTop + dojo.marginBox(parent).h; //PORT was getBorderBox
		var nodeBottom = node.offsetTop + dojo.marginBox(node).h;
		if(parentBottom < nodeBottom){
			parent.scrollTop += (nodeBottom - parentBottom);
		}else if(parent.scrollTop > node.offsetTop){
			parent.scrollTop -= (parent.scrollTop - node.offsetTop);
		}
	}
};

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
dojo.provide("dijit._base.sniff");

// ported from dojo.html.applyBrowserClass (style.js)

//	summary:
//		Applies pre-set class names based on browser & version to the
//		top-level HTML node.  Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.
(function(){
	var d = dojo;
	var ie = d.isIE;
	var opera = d.isOpera;
	var maj = Math.floor;
	var classes = {
		dj_ie: ie,
//		dj_ie55: ie == 5.5,
		dj_ie6: maj(ie) == 6,
		dj_ie7: maj(ie) == 7,
		dj_iequirks: ie && d.isQuirks,
// NOTE: Opera not supported by dijit
		dj_opera: opera,
		dj_opera8: maj(opera) == 8,
		dj_opera9: maj(opera) == 9,
		dj_khtml: d.isKhtml,
		dj_safari: d.isSafari,
		dj_gecko: d.isMozilla
	}; // no dojo unsupported browsers

	for(var p in classes){
		if(classes[p]){
			var html = dojo.doc.documentElement; //TODO browser-specific DOM magic needed?
			if(html.className){
				html.className += " " + p;
			}else{
				html.className = p;
			}
		}
	}
})();

}

if(!dojo._hasResource["dijit._base.bidi"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.bidi"] = true;
dojo.provide("dijit._base.bidi");

// summary: applies a class to the top of the document for right-to-left stylesheet rules

dojo.addOnLoad(function(){
	if(!dojo._isBodyLtr()){
		dojo.addClass(dojo.body(), "dijitRtl");
	}
});

}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");

dijit.typematic = {
	// summary:
	//	These functions are used to repetitively call a user specified callback
	//	method when a specific key or mouse click over a specific DOM node is
	//	held down for a specific amount of time.
	//	Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		this._currentTimeout = (this._currentTimeout < 0) ? this._initialDelay : ((this._subsequentDelay > 1) ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay));
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /* Object */ _this, /*DOMNode*/ node, /* Function */ callback, /* Object */ obj, /* Number */ subsequentDelay, /* Number */ initialDelay){
		// summary:
		//      Start a timed, repeating callback sequence.
		//      If already started, the function call is ignored.
		//      This method is not normally called by the user but can be
		//      when the normal listener code is insufficient.
		//	Parameters:
		//	evt: key or mouse event object to pass to the user callback
		//	_this: pointer to the user's widget space.
		//	node: the DOM node object to pass the the callback function
		//	callback: function to call until the sequence is stopped called with 3 parameters:
		//		count: integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		//		node: the DOM node object passed in
		//		evt: key or mouse event object
		//	obj: user space object used to uniquely identify each typematic sequence
		//	subsequentDelay: if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		//	initialDelay: the number of milliseconds until the 2nd event occurs, default=500ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
		}
	},

	stop: function(){
		// summary:
		//	  Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a specific typematic key.
		//	keyObject: an object defining the key to listen for.
		//		key: (mandatory) the keyCode (number) or character (string) to listen for.
		//		ctrlKey: desired ctrl key state to initiate the calback sequence:
		//			pressed (true)
		//			released (false)
		//			either (unspecified)
		//		altKey: same as ctrlKey but for the alt key
		//		shiftKey: same as ctrlKey but for the shift key
		//	See the trigger method for other parameters.
		//	Returns an array of dojo.connect handles
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.keyCode == keyObject.keyCode && (!keyObject.charCode || keyObject.charCode == evt.charCode) &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.ctrlKey) &&
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.ctrlKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(keyObject, _this, node, callback, keyObject, subsequentDelay, initialDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a typematic mouse click.
		//	See the trigger method for other parameters.
		//	Returns an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				dojo.stopEvent(evt);
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay);
					setTimeout(dijit.typematic.stop, 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a specific typematic key and mouseclick.
		//	This is a thin wrapper to addKeyListener and addMouseListener.
		//	mouseNode: the DOM node object to listen on for mouse events.
		//	keyNode: the DOM node object to listen on for key events.
		//	See the addMouseListener and addKeyListener methods for other parameters.
		//	Returns an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");

dijit.wai = {
	onload: function(){
		// summary:
		//		Function that detects if we are in high-contrast mode or not,
		//		and sets up a timer to periodically confirm the value.
		//		figure out the background-image style property
		//		and apply that to the image.src property.
		// description:
		//		This must be a named function and not an anonymous
		//		function, so that the widget parsing code can make sure it
		//		registers its onload function after this function.
		//		DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = document.createElement("div");
		div.id = "a11yTestNode";
		div.style.cssText = 'border: 1px solid;'
			+ 'border-color:red green;'
			+ 'position: absolute;'
			+ 'height: 5px;'
			+ 'top: -999px;'
			+ 'background-image: url("' + dojo.moduleUrl("dijit", "form/templates/blank.gif") + '");';
		dojo.body().appendChild(div);

		// test it
		function check(){
			var cs = dojo.getComputedStyle(div);
			if(cs){
				var bkImg = cs.backgroundImage;
				var needsA11y = (cs.borderTopColor==cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
				dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			}
		}
		check();
		if(dojo.isIE){
			setInterval(check, 4000);
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit,
{
	hasWaiRole: function(/*Element*/ elem){
		// Summary: Return true if elem has a role attribute and false if not.
		if(elem.hasAttribute){
			return elem.hasAttribute("role");
		}else{
			return elem.getAttribute("role") ? true : false;
		}
	},

	getWaiRole: function(/*Element*/ elem){
		// Summary: Return the role of elem or an empty string if
		//		elem does not have a role.
		var value = elem.getAttribute("role");
		if(value){
			var prefixEnd = value.indexOf(":");
			return prefixEnd == -1 ? value : value.substring(prefixEnd+1);
		}else{
			return "";
		}
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// Summary: Set the role on elem. On Firefox 2 and below, "wairole:" is
		//		prepended to the provided role value.
		if(dojo.isFF && dojo.isFF < 3){
			elem.setAttribute("role", "wairole:"+role);
		}else{
			elem.setAttribute("role", role);
		}
	},

	removeWaiRole: function(/*Element*/ elem){
		// Summary: Removes the role attribute from elem.
		elem.removeAttribute("role");
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Return true if elem has a value for the given state and
		//		false if it does not.
		//		On Firefox 2 and below, we check for an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we check for an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			return elem.hasAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			if(elem.hasAttribute){
				return elem.hasAttribute("aria-"+state);
			}else{
				return elem.getAttribute("aria-"+state) ? true : false;
			}
		}
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Return the value of the requested state on elem
		//		or an empty string if elem has no value for state.
		//		On Firefox 2 and below, we check for an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we check for an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			return elem.getAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			var value =  elem.getAttribute("aria-"+state);
			return value ? value : "";
		}
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// Summary: Set state on elem to value.
		//		On Firefox 2 and below, we set an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we set an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			elem.setAttributeNS("http://www.w3.org/2005/07/aaa",
				"aaa:"+state, value);
		}else{
			elem.setAttribute("aria-"+state, value);
		}
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Removes the given state from elem.
		//		On Firefox 2 and below, we remove the attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we remove the attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			elem.removeAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			elem.removeAttribute("aria-"+state);
		}
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");












}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");



dojo.declare("dijit._Widget", null, {
	// summary:
	//		The foundation of dijit widgets. 	
	//
	// id: String
	//		a unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: String
	//	Language to display this widget in (like en-us).
	//	Defaults to brower's specified preferred language (typically the language of the OS)
	lang: "",

	// dir: String
	//  Bi-directional support, as defined by the HTML DIR attribute. Either left-to-right "ltr" or right-to-left "rtl".
	dir: "",

	// class: String
	// HTML class attribute
	"class": "",

	// style: String
	// HTML style attribute
	style: "",

	// title: String
	// HTML title attribute
	title: "",

	// srcNodeRef: DomNode
	//		pointer to original dom node
	srcNodeRef: null,

	// domNode: DomNode
	//		this is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPonit syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// attributeMap: Object
	//		A map of attributes and attachpoints -- typically standard HTML attributes -- to set
	//		on the widget's dom, at the "domNode" attach point, by default.
	//		Other node references can be specified as properties of 'this'
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},  // TODO: add on* handlers?

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(params, srcNodeRef){
		this.create(params, srcNodeRef);
	},

	create: function(params, srcNodeRef){
		// summary:
		//		To understand the process by which widgets are instantiated, it
		//		is critical to understand what other methods create calls and
		//		which of them you'll want to override. Of course, adventurous
		//		developers could override create entirely, but this should
		//		only be done as a last resort.
		//
		//		Below is a list of the methods that are called, in the order
		//		they are fired, along with notes about what they do and if/when
		//		you should over-ride them in your widget:
		//			
		//			postMixInProperties:
		//				a stub function that you can over-ride to modify
		//				variables that may have been naively assigned by
		//				mixInProperties
		//			# widget is added to manager object here
		//			buildRendering
		//				Subclasses use this method to handle all UI initialization
		//				Sets this.domNode.  Templated widgets do this automatically
		//				and otherwise it just uses the source dom node.
		//			postCreate
		//				a stub function that you can over-ride to modify take
		//				actions once the widget has been placed in the UI

		// store pointer to original dom tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects=[];

		// _attaches: String[]
		// 		names of all our dojoAttachPoint variables
		this._attaches=[];

		//mixin our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			dojo.mixin(this,params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.
		if(!this.id){
			this.id=dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
		// The placement of these attributes is according to the property mapping in attributeMap.
		// Note special handling for 'style' and 'class' attributes which are lists and can
		// have elements from both old and new structures, and some attributes like "type"
		// cannot be processed this way as they are not mutable.
		if(this.domNode){
			for(var attr in this.attributeMap){
				var mapNode = this[this.attributeMap[attr] || "domNode"];
				var value = this[attr];
				if(typeof value != "object" && (value !== "" || (params && params[attr]))){
					switch(attr){
					case "class":
						dojo.addClass(mapNode, value);
						break;
					case "style":
						if(mapNode.style.cssText){
							mapNode.style.cssText += "; " + value;// FIXME: Opera
						}else{
							mapNode.style.cssText = value;
						}
						break;
					default:
						mapNode.setAttribute(attr, value);
					}
				}
			}
		}

		if(this.domNode){
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}	
	},

	postMixInProperties: function(){
		// summary
		//	Called after the parameters to the widget have been read-in,
		//	but before the widget template is instantiated.
		//	Especially useful to set properties that are referenced in the widget template.
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode.
		//		Most widgets will mixin TemplatedWidget, which overrides this method.
		this.domNode = this.srcNodeRef || dojo.doc.createElement('div');
	},

	postCreate: function(){
		// summary:
		//		Called after a widget's dom has been setup
	},

	startup: function(){
		// summary:
		//		Called after a widget's children, and other widgets on the page, have been created.
		//		Provides an opportunity to manipulate any children before they are displayed
		//		This is useful for composite widgets that need to control or layout sub-widgets
		//		Many layout widgets can use this as a wiring phase
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean*/ finalize){
		// summary:
		// 		Destroy this widget and it's descendants. This is the generic
		// 		"destructor" function that all widget users should call to
		// 		cleanly discard with a widget. Once a widget is destroyed, it's
		// 		removed from the manager object.
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?

		this.destroyDescendants();
		this.destroy();
	},

	destroy: function(/*Boolean*/ finalize){
		// summary:
		// 		Destroy this widget, but not its descendants
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?
		this.uninitialize();
		dojo.forEach(this._connects, function(array){
			dojo.forEach(array, dojo.disconnect);
		});
		this.destroyRendering(finalize);
		dijit.registry.remove(this.id);
	},

	destroyRendering: function(/*Boolean*/ finalize){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?

		if(this.bgIframe){
			this.bgIframe.destroy();
			delete this.bgIframe;
		}

		if(this.domNode){
			dojo._destroyElement(this.domNode);
			delete this.domNode;
		}

		if(this.srcNodeRef){
			dojo._destroyElement(this.srcNodeRef);
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.

		// TODO: should I destroy in the reverse order, to go bottom up?
		dojo.forEach(this.getDescendants(), function(widget){ widget.destroy(); });
	},

	uninitialize: function(){
		// summary:
		//		stub function. Over-ride to implement custom widget tear-down
		//		behavior.
		return false;
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	toString: function(){
		// summary:
		//		returns a string that represents the widget. When a widget is
		//		cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversable
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//	return all the descendant widgets
		var list = dojo.query('[widgetId]', this.domNode);
		return list.map(dijit.byNode);		// Array
	},

	nodesWithKeyClick : ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String*/ event,
			/*String|Function*/ method){

		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		//		Special event: "ondijitclick" triggers on a click or enter-down or space-up
		//		Similar to dojo.connect() but takes three arguments rather than four.
		var handles =[];
		if(event == "ondijitclick"){
			var w = this;
			// add key based click activation for unsupported nodes.
			if(!this.nodesWithKeyClick[obj.nodeName]){
				handles.push(dojo.connect(obj, "onkeydown", this,
					function(e){
						if(e.keyCode == dojo.keys.ENTER){
							return (dojo.isString(method))?
								w[method](e) : method.call(w, e);
						}else if(e.keyCode == dojo.keys.SPACE){
							// stop space down as it causes IE to scroll
							// the browser window
							dojo.stopEvent(e);
						}
			 		}));
				handles.push(dojo.connect(obj, "onkeyup", this,
					function(e){
						if(e.keyCode == dojo.keys.SPACE){
							return dojo.isString(method) ?
								w[method](e) : method.call(w, e);
						}
			 		}));
			}
			event = "onclick";
		}
		handles.push(dojo.connect(obj, event, this, method));

		// return handles for FormElement and ComboBox
		this._connects.push(handles);
		return handles;
	},

	disconnect: function(/*Object*/ handles){
		// summary:
		//		Disconnects handle created by this.connect.
		//		Also removes handle from this widget's list of connects
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i]==handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Checks the DOM to for the text direction for bi-directional support
		// description:
		//		This method cannot be used during widget construction because the widget
		//		must first be connected to the DOM tree.  Parent nodes are searched for the
		//		'dir' attribute until one is found, otherwise left to right mode is assumed.
		//		See HTML spec, DIR attribute for more information.

		if(typeof this._ltr == "undefined"){
			this._ltr = dojo.getComputedStyle(this.domNode).direction != "rtl";
		}
		return this._ltr; //Boolean
	},

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	}
});

}

if(!dojo._hasResource["dojo.string"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.string"] = true;
dojo.provide("dojo.string");

dojo.string.pad = function(/*String*/text, /*int*/size, /*String?*/ch, /*boolean?*/end){
	// summary:
	//		Pad a string to guarantee that it is at least 'size' length by
	//		filling with the character 'c' at either the start or end of the
	//		string. Pads at the start, by default.
	// text: the string to pad
	// size: length to provide padding
	// ch: character to pad, defaults to '0'
	// end: adds padding at the end if true, otherwise pads at start

	var out = String(text);
	if(!ch){
		ch = '0';
	}
	while(out.length < size){
		if(end){
			out += ch;
		}else{
			out = ch + out;
		}
	}
	return out;	// String
};

dojo.string.substitute = function(	/*String*/template, 
									/*Object or Array*/map, 
									/*Function?*/transform, 
									/*Object?*/thisObject){
	// summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	// description:
	//		For example,
	//		|	dojo.string.substitute("File '${0}' is not found in directory '${1}'.",["foo.html","/temp"]);
	//		|	dojo.string.substitute("File '${name}' is not found in directory '${info.dir}'.",{name: "foo.html", info: {dir: "/temp"}});
	//		both return
	//			"File 'foo.html' is not found in directory '/temp'."
	// template: 
	//		a string with expressions in the form ${key} to be replaced or
	//		${key:format} which specifies a format function.  NOTE syntax has
	//		changed from %{key}
	// map: where to look for substitutions
	// transform: 
	//		a function to process all parameters before substitution takes
	//		place, e.g. dojo.string.encodeXML
	// thisObject: 
	//		where to look for optional format function; default to the global
	//		namespace

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g, function(match, key, format){
		var value = dojo.getObject(key,false,map);
		if(format){ value = dojo.getObject(format,false,thisObject)(value);}
		if(transform){ value = transform(value, key); }
		return value.toString();
	}); // string
};

dojo.string.trim = function(/*String*/ str){
	// summary: trims whitespaces from both sides of the string
	// description:
	//	This version of trim() was taken from Steven Levithan's blog: 
	//	http://blog.stevenlevithan.com/archives/faster-trim-javascript.
	//	The short yet good-performing version of this function is 
	//	dojo.trim(), which is part of the base.
	str = str.replace(/^\s+/, '');
	for(var i = str.length - 1; i > 0; i--){
		if(/\S/.test(str.charAt(i))){
			str = str.substring(0, i + 1);
			break;
		}
	}
	return str;	// String
};

}

if(!dojo._hasResource["dijit._Templated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Templated"] = true;
dojo.provide("dijit._Templated");






dojo.declare("dijit._Templated",
	null,
	{
		// summary:
		//		mixin for widgets that are instantiated from a template

		// templateNode: DomNode
		//		a node that represents the widget template. Pre-empts both templateString and templatePath.
		templateNode: null,

		// templateString String:
		//		a string that represents the widget template. Pre-empts the
		//		templatePath. In builds that have their strings "interned", the
		//		templatePath is converted to an inline templateString, thereby
		//		preventing a synchronous network call.
		templateString: null,

		// templatePath: String
		//	Path to template (HTML file) for this widget
		templatePath: null,

		// widgetsInTemplate Boolean:
		//		should we parse the template to find widgets that might be
		//		declared in markup inside it? false by default.
		widgetsInTemplate: false,

		// containerNode DomNode:
		//		holds child elements. "containerNode" is generally set via a
		//		dojoAttachPoint assignment and it designates where children of
		//		the src dom node will be placed
		containerNode: null,

		// skipNodeCache Boolean:
		//		if using a cached widget template node poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// method over-ride
		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = dijit._Templated.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache);

			var node;
			if(dojo.isString(cached)){
				var className = this.declaredClass, _this = this;
				// Cache contains a string because we need to do property replacement
				// do the property replacement
				var tstr = dojo.string.substitute(cached, this, function(value, key){
					if(key.charAt(0) == '!'){ value = _this[key.substr(1)]; }
					if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
					if(!value){ return ""; }

					// Substitution keys beginning with ! will skip the transform step,
					// in case a user wishes to insert unescaped markup, e.g. ${!foo}
					return key.charAt(0) == "!" ? value :
						// Safer substitution, see heading "Attribute values" in
						// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
						value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
				}, this);

				node = dijit._Templated._createNodesFromText(tstr)[0];
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			// recurse through the node, looking for, and attaching to, our
			// attachment points which should be defined on the template node.
			this._attachTemplateNodes(node);

			var source = this.srcNodeRef;
			if(source && source.parentNode){
				source.parentNode.replaceChild(node, source);
			}

			this.domNode = node;
			if(this.widgetsInTemplate){
				var childWidgets = dojo.parser.parse(node);
				this._attachTemplateNodes(childWidgets, function(n,p){
					return n[p];
				});
			}

			this._fillContent(source);
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		relocate source contents to templated container node
			//		this.containerNode must be able to receive children, or exceptions will be thrown
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint
			//			* dojoAttachEvent	
			//			* waiRole
			//			* waiState
			// rootNode: DomNode|Array[Widgets]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: function?
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget

			getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };

			var nodes = dojo.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x=dojo.isArray(rootNode)?0:-1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && getAttrFunc(baseNode,'dojoType')){
					continue;
				}
				// Process dojoAttachPoint
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while(point=points.shift()){
						if(dojo.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
					}
				}

				// Process dojoAttachEvent
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent");
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = dojo.trim;
					while(event=events.shift()){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							this.connect(baseNode, event, thisFunc);
						}
					}
				}

				// waiRole, waiState
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					dijit.setWaiRole(baseNode, role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							dijit.setWaiState(baseNode, pair[0], pair[1]);
						}
					});
				}

			}
		}
	}
);

// key is either templatePath or templateString; object is either string or DOM tree
dijit._Templated._templateCache = {};

dijit._Templated.getCachedTemplate = function(templatePath, templateString, alwaysUseString){
	// summary:
	//		static method to get a template based on the templatePath or
	//		templateString key
	// templatePath: String
	//		the URL to get the template from. dojo.uri.Uri is often passed as well.
	// templateString: String?
	//		a string to use in lieu of fetching the template from a URL
	// Returns:
	//	Either string (if there are ${} variables that need to be replaced) or just
	//	a DOM tree (if the node can be cloned directly)

	// is it already cached?
	var tmplts = dijit._Templated._templateCache;
	var key = templateString || templatePath;
	var cached = tmplts[key];
	if(cached){
		return cached;
	}

	// If necessary, load template string from template path
	if(!templateString){
		templateString = dijit._Templated._sanitizeTemplateString(dojo._getText(templatePath));
	}

	templateString = dojo.string.trim(templateString);

	if(templateString.match(/\$\{([^\}]+)\}/g) || alwaysUseString){
		// there are variables in the template so all we can do is cache the string
		return (tmplts[key] = templateString); //String
	}else{
		// there are no variables in the template so we can cache the DOM tree
		return (tmplts[key] = dijit._Templated._createNodesFromText(templateString)[0]); //Node
	}
};

dijit._Templated._sanitizeTemplateString = function(/*String*/tString){
	// summary: 
	//		Strips <?xml ...?> declarations so that external SVG and XML
	// 		documents can be added to a document without worry. Also, if the string
	//		is an HTML document, only the part inside the body tag is returned.
	if(tString){
		tString = tString.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
		var matches = tString.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
		if(matches){
			tString = matches[1];
		}
	}else{
		tString = "";
	}
	return tString; //String
};


if(dojo.isIE){
	dojo.addOnUnload(function(){
		var cache = dijit._Templated._templateCache;
		for(var key in cache){
			var value = cache[key];
			if(!isNaN(value.nodeType)){ // isNode equivalent
				dojo._destroyElement(value);
			}
			delete cache[key];
		}
	});
}

(function(){
	var tagMap = {
		cell: {re: /^<t[dh][\s\r\n>]/i, pre: "<table><tbody><tr>", post: "</tr></tbody></table>"},
		row: {re: /^<tr[\s\r\n>]/i, pre: "<table><tbody>", post: "</tbody></table>"},
		section: {re: /^<(thead|tbody|tfoot)[\s\r\n>]/i, pre: "<table>", post: "</table>"}
	};

	// dummy container node used temporarily to hold nodes being created
	var tn;

	dijit._Templated._createNodesFromText = function(/*String*/text){
		// summary:
		//	Attempts to create a set of nodes based on the structure of the passed text.

		if(!tn){
			tn = dojo.doc.createElement("div");
			tn.style.display="none";
			dojo.body().appendChild(tn);
		}
		var tableType = "none";
		var rtext = text.replace(/^\s+/, "");
		for(var type in tagMap){
			var map = tagMap[type];
			if(map.re.test(rtext)){
				tableType = type;
				text = map.pre + text + map.post;
				break;
			}
		}

		tn.innerHTML = text;
		if(tn.normalize){
			tn.normalize();
		}

		var tag = { cell: "tr", row: "tbody", section: "table" }[tableType];
		var _parent = (typeof tag != "undefined") ?
						tn.getElementsByTagName(tag)[0] :
						tn;

		var nodes = [];
		while(_parent.firstChild){
			nodes.push(_parent.removeChild(_parent.firstChild));
		}
		tn.innerHTML="";
		return nodes;	//	Array
	}
})();

// These arguments can be specified for widgets which are used in templates.
// Since any widget can be specified as sub widgets in template, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget,{
	dojoAttachEvent: "",
	dojoAttachPoint: "",
	waiRole: "",
	waiState:""
})

}

if(!dojo._hasResource["dijit._Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Container"] = true;
dojo.provide("dijit._Container");

dojo.declare("dijit._Contained",
	null,
	{
		// summary
		//		Mixin for widgets that are children of a container widget

		getParent: function(){
			// summary:
			//		returns the parent widget of this widget, assuming the parent
			//		implements dijit._Container
			for(var p=this.domNode.parentNode; p; p=p.parentNode){
				var id = p.getAttribute && p.getAttribute("widgetId");
				if(id){
					var parent = dijit.byId(id);
					return parent.isContainer ? parent : null;
				}
			}
			return null;
		},

		_getSibling: function(which){
			var node = this.domNode;
			do{
				node = node[which+"Sibling"];
			}while(node && node.nodeType != 1);
			if(!node){ return null; } // null
			var id = node.getAttribute("widgetId");
			return dijit.byId(id);
		},

		getPreviousSibling: function(){
			// summary:
			//		returns null if this is the first child of the parent,
			//		otherwise returns the next element sibling to the "left".

			return this._getSibling("previous");
		},

		getNextSibling: function(){
			// summary:
			//		returns null if this is the last child of the parent,
			//		otherwise returns the next element sibling to the "right".

			return this._getSibling("next");
		}
	}
);

dojo.declare("dijit._Container",
	null,
	{
		// summary
		//		Mixin for widgets that contain a list of children like SplitContainer

		isContainer: true,

		addChild: function(/*Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Process the given child widget, inserting it's dom node as
			//		a child of our dom node

			if(insertIndex === undefined){
				insertIndex = "last";
			}
			var refNode = this.containerNode || this.domNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = dojo.query("> [widgetid]", refNode);
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1]; insertIndex = "after";
				}
			}
			dojo.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget*/ widget){
			// summary:
			//		removes the passed widget instance from this widget but does
			//		not destroy it
			var node = widget.domNode;
			node.parentNode.removeChild(node);	// detach but don't destroy
		},

		_nextElement: function(node){
			do{
				node = node.nextSibling;
			}while(node && node.nodeType != 1);
			return node;
		},

		_firstElement: function(node){
			node = node.firstChild;
			if(node && node.nodeType != 1){
				node = this._nextElement(node);
			}
			return node;
		},

		getChildren: function(){
			// summary:
			//		Returns array of children widgets
			return dojo.query("> [widgetId]", this.containerNode || this.domNode).map(dijit.byNode); // Array
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has children
			var cn = this.containerNode || this.domNode;
			return !!this._firstElement(cn); // Boolean
		},

		_getSiblingOfChild: function(/*Widget*/ child, /*int*/ dir){
			// summary:
			//		get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			var node = child.domNode;
			var which = (dir>0 ? "nextSibling" : "previousSibling");
			do{
				node = node[which];
			}while(node && (node.nodeType != 1 || !dijit.byNode(node)));
			return node ? dijit.byNode(node) : null;
		}
	}
);

dojo.declare("dijit._KeyNavContainer",
	[dijit._Container],
	{

		// summary:
		//		A _Container with keyboard navigation of its children.
		//		To use this mixin, call connectKeyNavHandlers() in
		//		postCreate() and call startupKeyNavChildren() in startup().

/*=====
		// focusedChild: Widget
		//		The currently focused child widget, or null if there isn't one
		focusedChild: null,
=====*/

		_keyNavCodes: {},

		connectKeyNavHandlers: function(/*Array*/ prevKeyCodes, /*Array*/ nextKeyCodes){
			// summary:
			//		Call in postCreate() to attach the keyboard handlers
			//		to the container.
			// preKeyCodes: Array
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: Array
			//		Key codes for navigating to the next child.

			var keyCodes = this._keyNavCodes = {};
			var prev = dojo.hitch(this, this.focusPrev);
			var next = dojo.hitch(this, this.focusNext);
			dojo.forEach(prevKeyCodes, function(code){ keyCodes[code] = prev });
			dojo.forEach(nextKeyCodes, function(code){ keyCodes[code] = next });
			this.connect(this.domNode, "onkeypress", "_onContainerKeypress");
			if(dojo.isIE){
				this.connect(this.domNode, "onactivate", "_onContainerFocus");
				this.connect(this.domNode, "ondeactivate", "_onContainerBlur");
			}else{
				this.connect(this.domNode, "onfocus", "_onContainerFocus");
				this.connect(this.domNode, "onblur", "_onContainerBlur");
			}
		},

		startupKeyNavChildren: function(){
			// summary:
			//		Call in startup() to set child tabindexes to -1
			dojo.forEach(this.getChildren(), dojo.hitch(this, "_setTabIndexMinusOne"));
		},

		addChild: function(/*Widget*/ widget, /*int?*/ insertIndex){
			// summary: Add a child to our _Container
			dijit._KeyNavContainer.superclass.addChild.apply(this, arguments);
			this._setTabIndexMinusOne(widget);
		},

		focus: function(){
			// summary: Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		focusFirstChild: function(){
			// summary: Focus the first focusable child in the container.
			this.focusChild(this._getFirstFocusableChild());
		},

		focusNext: function(){
			// summary: Focus the next widget or focal node (for widgets
			//		with multiple focal nodes) within this container.
			if(this.focusedChild && this.focusedChild.hasNextFocalNode
					&& this.focusedChild.hasNextFocalNode()){
				this.focusedChild.focusNext();
				return;
			}
			var child = this._getNextFocusableChild(this.focusedChild, 1);
			if(child.getFocalNodes){
				this.focusChild(child, child.getFocalNodes()[0]);
			}else{
				this.focusChild(child);
			}
		},

		focusPrev: function(){
			// summary: Focus the previous widget or focal node (for widgets
			//		with multiple focal nodes) within this container.
			if(this.focusedChild && this.focusedChild.hasPrevFocalNode
					&& this.focusedChild.hasPrevFocalNode()){
				this.focusedChild.focusPrev();
				return;
			}
			var child = this._getNextFocusableChild(this.focusedChild, -1);
			if(child.getFocalNodes){
				var nodes = child.getFocalNodes();
				this.focusChild(child, nodes[nodes.length-1]);
			}else{
				this.focusChild(child);
			}
		},

		focusChild: function(/*Widget*/ widget, /*Node?*/ node){
			// summary: Focus widget. Optionally focus 'node' within widget.
			if(widget){
				if(this.focusedChild && widget !== this.focusedChild){
					this._onChildBlur(this.focusedChild);
				}
				this.focusedChild = widget;
				if(node && widget.focusFocalNode){
					widget.focusFocalNode(node);
				}else{
					widget.focus();
				}
			}
		},

		_setTabIndexMinusOne: function(/*Widget*/ widget){
			if(widget.getFocalNodes){
				dojo.forEach(widget.getFocalNodes(), function(node){
					node.setAttribute("tabIndex", -1);
				});
			}else{
				(widget.focusNode || widget.domNode).setAttribute("tabIndex", -1);
			}
		},

		_onContainerFocus: function(evt){
			this.domNode.setAttribute("tabIndex", -1);
			if(evt.target === this.domNode){
				this.focusFirstChild();
			}else{
				var widget = dijit.getEnclosingWidget(evt.target);
				if(widget && widget.isFocusable()){
					this.focusedChild = widget;
				}
			}
		},

		_onContainerBlur: function(evt){
			if(this.tabIndex){
				this.domNode.setAttribute("tabIndex", this.tabIndex);
			}
		},

		_onContainerKeypress: function(evt){
			if(evt.ctrlKey || evt.altKey){ return; }
			var func = this._keyNavCodes[evt.keyCode];
			if(func){
				func();
				dojo.stopEvent(evt);
			}
		},

		_onChildBlur: function(/*Widget*/ widget){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
		},

		_getFirstFocusableChild: function(){
			return this._getNextFocusableChild(null, 1);
		},

		_getNextFocusableChild: function(child, dir){
			if(child){
				child = this._getSiblingOfChild(child, dir);
			}
			var children = this.getChildren();
			for(var i=0; i < children.length; i++){
				if(!child){
					child = children[(dir>0) ? 0 : (children.length-1)];
				}
				if(child.isFocusable()){
					return child;
				}
				child = this._getSiblingOfChild(child, dir);
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.layout._LayoutWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._LayoutWidget"] = true;
dojo.provide("dijit.layout._LayoutWidget");




dojo.declare("dijit.layout._LayoutWidget",
	[dijit._Widget, dijit._Container, dijit._Contained],
	{
		// summary
		//		Mixin for widgets that contain a list of children like SplitContainer.
		//		Widgets which mixin this code must define layout() to lay out the children

		isLayoutContainer: true,

		postCreate: function(){
			dojo.addClass(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under document.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }
			this._started=true;

			if(this.getChildren){
				dojo.forEach(this.getChildren(), function(child){ child.startup(); });
			}

			// If I am a top level widget
			if(!this.getParent || !this.getParent()){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// since my parent isn't a layout container, and my style is width=height=100% (or something similar),
				// then I need to watch when the window resizes, and size myself accordingly
				// (passing in no argument to resize means that it has to glean the size itself)
				this.connect(window, 'onresize', function(){this.resize();});
			}
		},

		resize: function(args){
			// summary:
			//		Explicitly set this widget's size (in pixels),
			//		and then call layout() to resize contents (and maybe adjust child widgets)
			//	
			// args: Object?
			//		{w: int, h: int, l: int, t: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(args){
				dojo.marginBox(node, args);

				// set offset of the node
				if(args.t){ node.style.top = args.t + "px"; }
				if(args.l){ node.style.left = args.l + "px"; }
			}
			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = dojo.mixin(dojo.marginBox(node), args||{});

			// Save the size of my content box.
			this._contentBox = dijit.layout.marginBox2contentBox(node, mb);

			// Callback for widget to adjust size of it's children
			this.layout();
		},

		layout: function(){
			//	summary
			//		Widgets override this method to size & position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
		}
	}
);

dijit.layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
	// summary:
	//		Given the margin-box size of a node, return it's content box size.
	//		Functions like dojo.contentBox() but is more reliable since it doesn't have
	//		to wait for the browser to compute sizes.
	var cs = dojo.getComputedStyle(node);
	var me=dojo._getMarginExtents(node, cs);
	var pb=dojo._getPadBorderExtents(node, cs);
	return {
		l: dojo._toPixelValue(node, cs.paddingLeft),
		t: dojo._toPixelValue(node, cs.paddingTop),
		w: mb.w - (me.w + pb.w),
		h: mb.h - (me.h + pb.h)
	};
};

(function(){
	var capitalize = function(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	};

	var size = function(widget, dim){
		// size the child
		widget.resize ? widget.resize(dim) : dojo.marginBox(widget.domNode, dim);

		// record child's size, but favor our own numbers when we have them.
		// the browser lies sometimes
		dojo.mixin(widget, dojo.marginBox(widget.domNode));
		dojo.mixin(widget, dim);
	};

	dijit.layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Object[]*/ children){
		/**
		 * summary
		 *		Layout a bunch of child dom nodes within a parent dom node
		 * container:
		 *		parent node
		 * dim:
		 *		{l, t, w, h} object specifying dimensions of container into which to place children
		 * children:
		 *		an array like [ {domNode: foo, layoutAlign: "bottom" }, {domNode: bar, layoutAlign: "client"} ]
		 */

		// copy dim because we are going to modify it
		dim = dojo.mixin({}, dim);

		dojo.addClass(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.
		children = dojo.filter(children, function(item){ return item.layoutAlign != "client"; })
			.concat(dojo.filter(children, function(item){ return item.layoutAlign == "client"; }));

		// set positions/sizes
		dojo.forEach(children, function(child){
			var elm = child.domNode,
				pos = child.layoutAlign;

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.bottom = elmStyle.right = "auto";

			dojo.addClass(elm, "dijitAlign" + capitalize(pos));

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect it's height.
			if(pos=="top" || pos=="bottom"){
				size(child, { w: dim.w });
				dim.h -= child.h;
				if(pos=="top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos=="left" || pos=="right"){
				size(child, { h: dim.h });
				dim.w -= child.w;
				if(pos=="left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos=="client"){
				size(child, dim);
			}
		});
	};

})();

}

if(!dojo._hasResource["dijit.form._FormWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormWidget"] = true;
dojo.provide("dijit.form._FormWidget");




dojo.declare("dijit.form._FormWidget", [dijit._Widget, dijit._Templated],
{
	/*
	Summary:
		FormElement widgets correspond to native HTML elements such as <input> or <button> or <select>.
		Each FormElement represents a single input value, and has a (possibly hidden) <input> element,
		to which it serializes its input value, so that form submission (either normal submission or via FormBind?)
		works as expected.

		All these widgets should have these attributes just like native HTML input elements.
		You can set them during widget construction, but after that they are read only.

		They also share some common methods.
	*/

	// baseClass: String
	//		Root CSS class of the widget (ex: dijitTextBox), used to add CSS classes of widget
	//		(ex: "dijitTextBox dijitTextBoxInvalid dijitTextBoxFocused dijitTextBoxInvalidFocused")
	//		See _setStateClass().
	baseClass: "",

	// value: String
	//		Corresponds to the native HTML <input> element's attribute.
	value: "",

	// name: String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// id: String
	//		Corresponds to the native HTML <input> element's attribute.
	//		Also becomes the id for the widget.
	id: "",

	// alt: String
	//		Corresponds to the native HTML <input> element's attribute.
	alt: "",

	// type: String
	//		Corresponds to the native HTML <input> element's attribute.
	type: "text",

	// tabIndex: Integer
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// These mixins assume that the focus node is an INPUT, as many but not all _FormWidgets are.
	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically 
	// require the 'name' attribute at element creation time.
	attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
		{id:"focusNode", tabIndex:"focusNode", alt:"focusNode"}),

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Set disabled state of widget.

		this.domNode.disabled = this.disabled = disabled;
		if(this.focusNode){
			this.focusNode.disabled = disabled;
		}
		if(disabled){
			//reset those, because after the domNode is disabled, we can no longer receive
			//mouse related events, see #4200
			this._hovering = false;
			this._active = false;
		}
		dijit.setWaiState(this.focusNode || this.domNode, "disabled", disabled);
		this._setStateClass();
	},


	_onMouse : function(/*Event*/ event){
		// summary:
		//	Sets _hovering, _active, and stateModifier properties depending on mouse state,
		//	then calls setStateClass() to set appropriate CSS classes for this.domNode.
		//
		//	To get a different CSS class for hover, send onmouseover and onmouseout events to this method.
		//	To get a different CSS class while mouse button is depressed, send onmousedown to this method.

		var mouseNode = event.target;
		if(mouseNode && mouseNode.getAttribute){
			this.stateModifier = mouseNode.getAttribute("stateModifier") || "";
		}

		if(!this.disabled){
			switch(event.type){
				case "mouseenter" :	
				case "mouseover" :
					this._hovering = true;
					break;

				case "mouseout" :	
				case "mouseleave" :	
					this._hovering = false;
					break;

				case "mousedown" :
					this._active = true;
					// set a global event to handle mouseup, so it fires properly
					//	even if the cursor leaves the button
					var self = this;
					// #2685: use this.connect and disconnect so destroy works properly
					var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
						self._active = false;
						self._setStateClass();
						self.disconnect(mouseUpConnector);
					});
					break;
			}
			this._setStateClass();
		}
	},

	isFocusable: function(){
		return !this.disabled && (dojo.style(this.domNode, "display") != "none");
	},

	focus: function(){
		dijit.focus(this.focusNode);
	},

	_setStateClass: function(){
		// summary
		//	Update the visual state of the widget by setting the css classes on this.domNode
		//  (or this.stateNode if defined) by combining this.baseClass with
		//	various suffixes that represent the current widget state(s).
		//
		//	In the case where a widget has multiple
		//	states, it sets the class based on all possible
		//  combinations.  For example, an invalid form widget that is being hovered
		//	will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//	For complex widgets with multiple regions, there can be various hover/active states,
		//	such as "Hover" or "CloseButtonHover" (for tab buttons).
		//	This is controlled by a stateModifier="CloseButton" attribute on the close button node.
		//
		//	The widget may have one or more of the following states, determined
		//	by this.state, this.checked, this.valid, and this.selected:
		//		Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//		Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//		Selected - ex: currently selected tab will have this.selected==true
		//
		//	In addition, it may have at most one of the following states,
		//	based on this.disabled and flags set in _onMouse (this._active, this._hovering, this._focused):
		//		Disabled	- if the widget is disabled
		//		Active		- if the mouse (or space/enter key?) is being pressed down
		//		Focused		- if the widget has focus
		//		Hover		- if the mouse is over the widget
		//
		//	(even if multiple af the above conditions are true we only pick the first matching one)


		// Get original (non state related, non baseClass related) class specified in template
		if(!("staticClass" in this)){
			this.staticClass = (this.stateNode||this.domNode).className;
		}

		// Compute new set of classes
		var classes = [ this.baseClass ];

		function multiply(modifier){
			classes=classes.concat(dojo.map(classes, function(c){ return c+modifier; }));
		}

		if(this.checked){
			multiply("Checked");
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}

		// Only one of these four can be applied.
		// Active trumps Focused, Focused trumps Hover, and Disabled trumps all.
		if(this.disabled){
			multiply("Disabled");
		}else if(this._active){
			multiply(this.stateModifier+"Active");
		}else{
			if(this._focused){
				multiply("Focused");
			}
			if((this.stateModifier || !this._focused) && this._hovering){
				multiply(this.stateModifier+"Hover");
			}
		}
		(this.stateNode || this.domNode).className = this.staticClass + " " + classes.join(" ");
	},

	onChange: function(newValue){
		// summary: callback when value is changed
	},

	postCreate: function(){
		this.setValue(this.value, null); // null reserved for initial value
		this.setDisabled(this.disabled);
		this._setStateClass();
	},

	setValue: function(/*anything*/ newValue, /*Boolean, optional*/ priorityChange){
		// summary: set the value of the widget.
		this._lastValue = newValue;
		dijit.setWaiState(this.focusNode || this.domNode, "valuenow", this.forWaiValuenow());
		if(priorityChange === undefined){ priorityChange = true; } // setValue with value only should fire onChange
		if(this._lastValueReported == undefined && priorityChange === null){ // don't report the initial value
			this._lastValueReported = newValue;
		}
		if((this.intermediateChanges || priorityChange) && 
			((newValue && newValue.toString)?newValue.toString():newValue) !== ((this._lastValueReported && this._lastValueReported.toString)?this._lastValueReported.toString():this._lastValueReported)){
			this._lastValueReported = newValue;
			this.onChange(newValue);
		}
	},

	getValue: function(){
		// summary: get the value of the widget.
		return this._lastValue;
	},

	undo: function(){
		// summary: restore the value to the last value passed to onChange
		this.setValue(this._lastValueReported, false);
	},

	_onKeyPress: function(e){
		if(e.keyCode == dojo.keys.ESCAPE && !e.shiftKey && !e.ctrlKey && !e.altKey){
			var v = this.getValue();
			var lv = this._lastValueReported;
			// Equality comparison of objects such as dates are done by reference so
			// two distinct objects are != even if they have the same data. So use
			// toStrings in case the values are objects.
			if((typeof lv != "undefined") && ((v!==null && v.toString)?v.toString():null) !== lv.toString()){	
				this.undo();
				dojo.stopEvent(e);
				return false;
			}
		}
		return true;
	},

	forWaiValuenow: function(){
		// summary: returns a value, reflecting the current state of the widget,
		//		to be used for the ARIA valuenow.
		// 		This method may be overridden by subclasses that want
		// 		to use something other than this.getValue() for valuenow
		return this.getValue();
	}
});

}

if(!dojo._hasResource["dijit.dijit"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.dijit"] = true;
dojo.provide("dijit.dijit");

// All the stuff in _base (these are the function that are guaranteed available without an explicit dojo.require)


// And some other stuff that we tend to pull in all the time anyway







}

if(!dojo._hasResource["dojo.i18n"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.i18n"] = true;
dojo.provide("dojo.i18n");

dojo.i18n.getLocalization = function(/*String*/packageName, /*String*/bundleName, /*String?*/locale){
	//	summary:
	//		Returns an Object containing the localization for a given resource
	//		bundle in a package, matching the specified locale.
	//	description:
	//		Returns a hash containing name/value pairs in its prototypesuch
	//		that values can be easily overridden.  Throws an exception if the
	//		bundle is not found.  Bundle must have already been loaded by
	//		dojo.requireLocalization() or by a build optimization step.  NOTE:
	//		try not to call this method as part of an object property
	//		definition (var foo = { bar: dojo.i18n.getLocalization() }).  In
	//		some loading situations, the bundle may not be available in time
	//		for the object definition.  Instead, call this method inside a
	//		function that is run after all modules load or the page loads (like
	//		in dojo.adOnLoad()), or in a widget lifecycle method.
	//	packageName:
	//		package which is associated with this resource
	//	bundleName:
	//		the base filename of the resource bundle (without the ".js" suffix)
	//	locale:
	//		the variant to load (optional).  By default, the locale defined by
	//		the host environment: dojo.locale

	locale = dojo.i18n.normalizeLocale(locale);

	// look for nearest locale match
	var elements = locale.split('-');
	var module = [packageName,"nls",bundleName].join('.');
	var bundle = dojo._loadedModules[module];
	if(bundle){
		var localization;
		for(var i = elements.length; i > 0; i--){
			var loc = elements.slice(0, i).join('_');
			if(bundle[loc]){
				localization = bundle[loc];
				break;
			}
		}
		if(!localization){
			localization = bundle.ROOT;
		}

		// make a singleton prototype so that the caller won't accidentally change the values globally
		if(localization){
			var clazz = function(){};
			clazz.prototype = localization;
			return new clazz(); // Object
		}
	}

	throw new Error("Bundle not found: " + bundleName + " in " + packageName+" , locale=" + locale);
};

dojo.i18n.normalizeLocale = function(/*String?*/locale){
	//	summary:
	//		Returns canonical form of locale, as used by Dojo.
	//
	//  description:
	//		All variants are case-insensitive and are separated by '-' as specified in RFC 3066.
	//		If no locale is specified, the dojo.locale is returned.  dojo.locale is defined by
	//		the user agent's locale unless overridden by djConfig.

	var result = locale ? locale.toLowerCase() : dojo.locale;
	if(result == "root"){
		result = "ROOT";
	}
	return result; // String
};

dojo.i18n._requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale, /*String?*/availableFlatLocales){
	//	summary:
	//		See dojo.requireLocalization()
	//	description:
	// 		Called by the bootstrap, but factored out so that it is only
	// 		included in the build when needed.

	var targetLocale = dojo.i18n.normalizeLocale(locale);
 	var bundlePackage = [moduleName, "nls", bundleName].join(".");
	// NOTE: 
	//		When loading these resources, the packaging does not match what is
	//		on disk.  This is an implementation detail, as this is just a
	//		private data structure to hold the loaded resources.  e.g.
	//		tests/hello/nls/en-us/salutations.js is loaded as the object
	//		tests.hello.nls.salutations.en_us={...} The structure on disk is
	//		intended to be most convenient for developers and translators, but
	//		in memory it is more logical and efficient to store in a different
	//		order.  Locales cannot use dashes, since the resulting path will
	//		not evaluate as valid JS, so we translate them to underscores.
	
	//Find the best-match locale to load if we have available flat locales.
	var bestLocale = "";
	if(availableFlatLocales){
		var flatLocales = availableFlatLocales.split(",");
		for(var i = 0; i < flatLocales.length; i++){
			//Locale must match from start of string.
			if(targetLocale.indexOf(flatLocales[i]) == 0){
				if(flatLocales[i].length > bestLocale.length){
					bestLocale = flatLocales[i];
				}
			}
		}
		if(!bestLocale){
			bestLocale = "ROOT";
		}		
	}

	//See if the desired locale is already loaded.
	var tempLocale = availableFlatLocales ? bestLocale : targetLocale;
	var bundle = dojo._loadedModules[bundlePackage];
	var localizedBundle = null;
	if(bundle){
		if(djConfig.localizationComplete && bundle._built){return;}
		var jsLoc = tempLocale.replace(/-/g, '_');
		var translationPackage = bundlePackage+"."+jsLoc;
		localizedBundle = dojo._loadedModules[translationPackage];
	}

	if(!localizedBundle){
		bundle = dojo["provide"](bundlePackage);
		var syms = dojo._getModuleSymbols(moduleName);
		var modpath = syms.concat("nls").join("/");
		var parent;

		dojo.i18n._searchLocalePath(tempLocale, availableFlatLocales, function(loc){
			var jsLoc = loc.replace(/-/g, '_');
			var translationPackage = bundlePackage + "." + jsLoc;
			var loaded = false;
			if(!dojo._loadedModules[translationPackage]){
				// Mark loaded whether it's found or not, so that further load attempts will not be made
				dojo["provide"](translationPackage);
				var module = [modpath];
				if(loc != "ROOT"){module.push(loc);}
				module.push(bundleName);
				var filespec = module.join("/") + '.js';
				loaded = dojo._loadPath(filespec, null, function(hash){
					// Use singleton with prototype to point to parent bundle, then mix-in result from loadPath
					var clazz = function(){};
					clazz.prototype = parent;
					bundle[jsLoc] = new clazz();
					for(var j in hash){ bundle[jsLoc][j] = hash[j]; }
				});
			}else{
				loaded = true;
			}
			if(loaded && bundle[jsLoc]){
				parent = bundle[jsLoc];
			}else{
				bundle[jsLoc] = parent;
			}
			
			if(availableFlatLocales){
				//Stop the locale path searching if we know the availableFlatLocales, since
				//the first call to this function will load the only bundle that is needed.
				return true;
			}
		});
	}

	//Save the best locale bundle as the target locale bundle when we know the
	//the available bundles.
	if(availableFlatLocales && targetLocale != bestLocale){
		bundle[targetLocale.replace(/-/g, '_')] = bundle[bestLocale.replace(/-/g, '_')];
	}
};

(function(){
	// If other locales are used, dojo.requireLocalization should load them as
	// well, by default. 
	// 
	// Override dojo.requireLocalization to do load the default bundle, then
	// iterate through the extraLocale list and load those translations as
	// well, unless a particular locale was requested.

	var extra = djConfig.extraLocale;
	if(extra){
		if(!extra instanceof Array){
			extra = [extra];
		}

		var req = dojo.i18n._requireLocalization;
		dojo.i18n._requireLocalization = function(m, b, locale, availableFlatLocales){
			req(m,b,locale, availableFlatLocales);
			if(locale){return;}
			for(var i=0; i<extra.length; i++){
				req(m,b,extra[i], availableFlatLocales);
			}
		};
	}
})();

dojo.i18n._searchLocalePath = function(/*String*/locale, /*Boolean*/down, /*Function*/searchFunc){
	//	summary:
	//		A helper method to assist in searching for locale-based resources.
	//		Will iterate through the variants of a particular locale, either up
	//		or down, executing a callback function.  For example, "en-us" and
	//		true will try "en-us" followed by "en" and finally "ROOT".

	locale = dojo.i18n.normalizeLocale(locale);

	var elements = locale.split('-');
	var searchlist = [];
	for(var i = elements.length; i > 0; i--){
		searchlist.push(elements.slice(0, i).join('-'));
	}
	searchlist.push(false);
	if(down){searchlist.reverse();}

	for(var j = searchlist.length - 1; j >= 0; j--){
		var loc = searchlist[j] || "ROOT";
		var stop = searchFunc(loc);
		if(stop){ break; }
	}
};

dojo.i18n._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated){
	//	summary:
	//		Load built, flattened resource bundles, if available for all
	//		locales used in the page. Only called by built layer files.

	function preload(locale){
		locale = dojo.i18n.normalizeLocale(locale);
		dojo.i18n._searchLocalePath(locale, true, function(loc){
			for(var i=0; i<localesGenerated.length;i++){
				if(localesGenerated[i] == loc){
					dojo["require"](bundlePrefix+"_"+loc);
					return true; // Boolean
				}
			}
			return false; // Boolean
		});
	}
	preload();
	var extra = djConfig.extraLocale||[];
	for(var i=0; i<extra.length; i++){
		preload(extra[i]);
	}
};

}

if(!dojo._hasResource["dijit.layout.ContentPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.ContentPane"] = true;
dojo.provide("dijit.layout.ContentPane");








dojo.declare(
	"dijit.layout.ContentPane",
	dijit._Widget,
{
	// summary:
	//		A widget that acts as a Container for other widgets, and includes a ajax interface
	// description:
	//		A widget that can be used as a standalone widget
	//		or as a baseclass for other widgets
	//		Handles replacement of document fragment using either external uri or javascript
	//		generated markup or DOM content, instantiating widgets within that content.
	//		Don't confuse it with an iframe, it only needs/wants document fragments.
	//		It's useful as a child of LayoutContainer, SplitContainer, or TabContainer.
	//		But note that those classes can contain any widget as a child.
	// example:
	//		Some quick samples:
	//		To change the innerHTML use .setContent('<b>new content</b>')
	//
	//		Or you can send it a NodeList, .setContent(dojo.query('div [class=selected]', userSelection))
	//		please note that the nodes in NodeList will copied, not moved
	//
	//		To do a ajax update use .setHref('url')
	//
	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; see setHref();
	href: "",

	// extractContent: Boolean
	//	Extract visible content from inside of <body> .... </body>
	extractContent: false,

	// parseOnLoad: Boolean
	//	parse content and create the widgets, if any
	parseOnLoad:	true,

	// preventCache: Boolean
	//		Cache content retreived externally
	preventCache:	false,

	// preload: Boolean
	//	Force load of data even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//	Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'>${loadingState}</span>", 

	// errorMessage: String
	//	Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'>${errorState}</span>", 

	// isLoaded: Boolean
	//	Tells loading status see onLoad|onUnload for event hooks
	isLoaded: false,

	// class: String
	//	Class name to apply to ContentPane dom nodes
	"class": "dijitContentPane",

	postCreate: function(){
		// remove the title attribute so it doesn't show up when i hover
		// over a node
		this.domNode.title = "";

		if(this.preload){
			this._loadCheck();
		}

		var messages = dojo.i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = dojo.string.substitute(this.loadingMessage, messages);
		this.errorMessage = dojo.string.substitute(this.errorMessage, messages);

		// for programatically created ContentPane (with <span> tag), need to muck w/CSS
		// or it's as though overflow:visible is set
		dojo.addClass(this.domNode, this["class"]);
	},

	startup: function(){
		if(this._started){ return; }
		this._checkIfSingleChild();
		if(this._singleChild){
			this._singleChild.startup();
		}
		this._loadCheck();
		this._started = true;
	},

	_checkIfSingleChild: function(){
		// summary:
		// 	Test if we have exactly one widget as a child, and if so assume that we are a container for that widget,
		//	and should propogate startup() and resize() calls to it.
		var childNodes = dojo.query(">", this.containerNode || this.domNode),
			childWidgets = childNodes.filter("[widgetId]");

		if(childNodes.length == 1 && childWidgets.length == 1){
			this.isContainer = true;
			this._singleChild = dijit.byNode(childWidgets[0]);
		}else{
			delete this.isContainer;
			delete this._singleChild;
		}
	},

	refresh: function(){
		// summary:
		//	Force a refresh (re-download) of content, be sure to turn off cache

		// we return result of _prepareLoad here to avoid code dup. in dojox.layout.ContentPane
		return this._prepareLoad(true);
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false
		//	href:
		//		url to the page you want to get, must be within the same domain as your mainpage
		this.href = href;

		// we return result of _prepareLoad here to avoid code dup. in dojox.layout.ContentPane
		return this._prepareLoad();
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Replaces old content with data content, include style classes from old content
		//	data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we cant run refresh and clear content
		// refresh should only work if we downloaded the content
		if(!this._isDownloaded){
			this.href = "";
			this._onUnloadHandler();
		}

		this._setContent(data || "");

		this._isDownloaded = false; // must be set after _setContent(..), pathadjust in dojox.layout.ContentPane

		if(this.parseOnLoad){
			this._createSubWidgets();
		}

		this._checkIfSingleChild();
		if(this._singleChild && this._singleChild.resize){
			this._singleChild.resize(this._contentBox);
		}

		this._onLoadHandler();
	},

	cancel: function(){
		// summary:
		//		Cancels a inflight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect
	},

	destroy: function(){
		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		// make sure we call onUnload
		this._onUnloadHandler();
		this._beingDestroyed = true;
		this.inherited("destroy",arguments);
	},

	resize: function(size){
		dojo.marginBox(this.domNode, size);

		// Compute content box size in case we [later] need to size child
		// If either height or width wasn't specified by the user, then query node for it.
		// But note that setting the margin box and then immediately querying dimensions may return
		// inaccurate results, so try not to depend on it.
		var node = this.containerNode || this.domNode,
			mb = dojo.mixin(dojo.marginBox(node), size||{});

		this._contentBox = dijit.layout.marginBox2contentBox(node, mb);

		// If we have a single widget child then size it to fit snugly within my borders
		if(this._singleChild && this._singleChild.resize){
			this._singleChild.resize(this._contentBox);
		}
	},

	_prepareLoad: function(forceLoad){
		// sets up for a xhrLoad, load is deferred until widget onShow
		// cancels a inflight download
		this.cancel();
		this.isLoaded = false;
		this._loadCheck(forceLoad);
	},

	_loadCheck: function(forceLoad){
		// call this when you change onShow (onSelected) status when selected in parent container
		// it's used as a trigger for href download when this.domNode.display != 'none'

		// sequence:
		// if no href -> bail
		// forceLoad -> always load
		// this.preload -> load when download not in progress, domNode display doesn't matter
		// this.refreshOnShow -> load when download in progress bails, domNode display !='none' AND
		//						this.open !== false (undefined is ok), isLoaded doesn't matter
		// else -> load when download not in progress, if this.open !== false (undefined is ok) AND
		//						domNode display != 'none', isLoaded must be false

		var displayState = ((this.open !== false) && (this.domNode.style.display != 'none'));

		if(this.href &&	
			(forceLoad ||
				(this.preload && !this._xhrDfd) ||
				(this.refreshOnShow && displayState && !this._xhrDfd) ||
				(!this.isLoaded && displayState && !this._xhrDfd)
			)
		){
			this._downloadExternalContent();
		}
	},

	_downloadExternalContent: function(){
		this._onUnloadHandler();

		// display loading message
		this._setContent(
			this.onDownloadStart.call(this)
		);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(dojo.isObject(this.ioArgs)){
			dojo.mixin(getArgs, this.ioArgs);
		}

		var hand = this._xhrDfd = (this.ioMethod || dojo.xhrGet)(getArgs);

		hand.addCallback(function(html){
			try{
				self.onDownloadEnd.call(self);
				self._isDownloaded = true;
				self.setContent.call(self, html); // onload event is called from here
			}catch(err){
				self._onError.call(self, 'Content', err); // onContentError
			}
			delete self._xhrDfd;
			return html;
		});

		hand.addErrback(function(err){
			if(!hand.cancelled){
				// show error message in the pane
				self._onError.call(self, 'Download', err); // onDownloadError
			}
			delete self._xhrDfd;
			return err;
		});
	},

	_onLoadHandler: function(){
		this.isLoaded = true;
		try{
			this.onLoad.call(this);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code');
		}
	},

	_onUnloadHandler: function(){
		this.isLoaded = false;
		this.cancel();
		try{
			this.onUnload.call(this);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code');
		}
	},

	_setContent: function(cont){
		this.destroyDescendants();

		try{
			var node = this.containerNode || this.domNode;
			while(node.firstChild){
				dojo._destroyElement(node.firstChild);
			}
			if(typeof cont == "string"){
				// dijit.ContentPane does only minimal fixes,
				// No pathAdjustments, script retrieval, style clean etc
				// some of these should be available in the dojox.layout.ContentPane
				if(this.extractContent){
					match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
					if(match){ cont = match[1]; }
				}
				node.innerHTML = cont;
			}else{
				// domNode or NodeList
				if(cont.nodeType){ // domNode (htmlNode 1 or textNode 3)
					node.appendChild(cont);
				}else{// nodelist or array such as dojo.Nodelist
					dojo.forEach(cont, function(n){
						node.appendChild(n.cloneNode(true));
					});
				}
			}
		}catch(e){
			// check if a domfault occurs when we are appending this.errorMessage
			// like for instance if domNode is a UL and we try append a DIV
			var errMess = this.onContentError(e);
			try{
				node.innerHTML = errMess;
			}catch(e){
				console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
			}
		}
	},

	_onError: function(type, err, consoleText){
		// shows user the string that is returned by on[type]Error
		// overide on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent.call(this, errText);
		}
	},

	_createSubWidgets: function(){
		// summary: scan my contents and create subwidgets
		var rootNode = this.containerNode || this.domNode;
		try{
			dojo.parser.parse(rootNode, true);
		}catch(e){
			this._onError('Content', e, "Couldn't create widgets in "+this.id
				+(this.href ? " from "+this.href : ""));
		}
	},

	// EVENT's, should be overide-able
	onLoad: function(e){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
	},

	onUnload: function(e){
		// summary:
		//		Event hook, is called before old content is cleared
	},

	onDownloadStart: function(){
		// summary:
		//		called before download starts
		//		the string returned by this function will be the html
		//		that tells the user we are loading something
		//		override with your own function if you want to change text
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ error){
		// summary:
		//		called on DOM faults, require fault etc in content
		//		default is to display errormessage inside pane
	},

	onDownloadError: function(/*Error*/ error){
		// summary:
		//		Called when download error occurs, default is to display
		//		errormessage inside pane. Overide function to change that.
		//		The string returned by this function will be the html
		//		that tells the user a error happend
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		called when download is finished
	}
});

}

if(!dojo._hasResource["dijit.form.Form"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Form"] = true;
dojo.provide("dijit.form.Form");




dojo.declare("dijit.form._FormMixin", null,
	{
		/*
		summary:
			Widget corresponding to <form> tag, for validation and serialization

		usage:
			<form dojoType="dijit.form.Form" id="myForm">
				Name: <input type="text" name="name" />
			</form>
			myObj={name: "John Doe"};
			dijit.byId('myForm').setValues(myObj);

			myObj=dijit.byId('myForm').getValues();
		TODO:
		* Repeater
		* better handling for arrays.  Often form elements have names with [] like
		* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])

		*/

		// HTML <FORM> attributes

		action: "",
		method: "",
		enctype: "",
		name: "",
		"accept-charset": "",
		accept: "",
		target: "",

		attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
			{action: "", method: "", enctype: "", "accept-charset": "", accept: "", target: ""}),

		// execute: Function
		//	User defined function to do stuff when the user hits the submit button
		execute: function(/*Object*/ formContents){},

		// onCancel: Function
		//	Callback when user has canceled dialog, to notify container
		//	(user shouldn't override)
		onCancel: function(){},

		// onExecute: Function
		//	Callback when user is about to execute dialog, to notify container
		//	(user shouldn't override)
		onExecute: function(){},

		templateString: "<form dojoAttachPoint='containerNode' dojoAttachEvent='onsubmit:_onSubmit' name='${name}' enctype='multipart/form-data'></form>",

		_onSubmit: function(/*event*/e) {
			// summary: callback when user hits submit button
			dojo.stopEvent(e);
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.getValues());
		},

		submit: function() {
			// summary: programatically submit form
			this.containerNode.submit();
		},

		setValues: function(/*object*/obj) {
			// summary: fill in form values from a JSON structure

			// generate map from name --> [list of widgets with that name]
			var map = {};
			dojo.forEach(this.getDescendants(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			// call setValue() or setChecked() for each widget, according to obj
			for(var name in map){
				var widgets = map[name],						// array of widgets w/this name
					values = dojo.getObject(name, false, obj);	// list of values for those widgets
				if(!dojo.isArray(values)){
					values = [ values ];
				}
				if(widgets[0].setChecked){
					// for checkbox/radio, values is a list of which widgets should be checked
					dojo.forEach(widgets, function(w, i){
						w.setChecked(dojo.indexOf(values, w.value) != -1);
					});
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					dojo.forEach(widgets, function(w, i){
						w.setValue(values[i]);
					});					
				}
			}

			/***
			 * 	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets

			dojo.forEach(this.containerNode.elements, function(element){
				if (element.name == ''){return};	// like "continue"	
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j) {
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if (nameA.length > 1) {
						if(typeof(myObj[nameA[0]]) == "undefined") {
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined") {
							myObj[nameA[0]][nameIndex]={};
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined") {
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if (typeof(myObj) == "undefined") {
					return;		// like "continue"
				}
				if (typeof(myObj[name]) == "undefined" && this.ignoreNullValues) {
					return;		// like "continue"
				}

				// TODO: widget values (just call setValue() on the widget)

				switch(element.type) {
					case "checkbox":
						element.checked = (name in myObj) &&
							dojo.some(myObj[name], function(val){ return val==element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name]==element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						dojo.forEach(element.options, function(option){
							option.selected = dojo.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						dojo.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
	  		});
	  		*/
		},

		getValues: function() {
			// summary: generate JSON structure from form values

			// get widget values
			var obj = {};
			dojo.forEach(this.getDescendants(), function(widget){
				var value = widget.getValue ? widget.getValue() : widget.value;
				var name = widget.name;
				if(!name){ return; }

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(widget.setChecked){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(widget.checked){
							dojo.setObject(name, value, obj);
						}
					}else{
						// checkbox/toggle button
						var ary=dojo.getObject(name, false, obj);
						if(!ary){
							ary=[];
							dojo.setObject(name, ary, obj);
						}
						if(widget.checked){
							ary.push(value);
						}
					}
				}else{
					// plain input
					dojo.setObject(name, value, obj);
				}
			});

			/***
			 * code for plain input boxes (see also dojo.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			dojo.forEach(this.containerNode.elements, function(elm){
				if (!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j) {
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if (nameA.length > 1) {
						if(typeof(myObj[nameA[0]]) == "undefined") {
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined") {
							myObj[nameA[0]][nameIndex]={};
						}
					} else if(typeof(myObj[nameA[0]]) == "undefined") {
						myObj[nameA[0]]={}
					} // if

					if (nameA.length == 1) {
						myObj=myObj[nameA[0]];
					} else {
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if ((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type=="radio" && elm.checked)) {
					if(name == name.split("[")[0]) {
						myObj[name]=elm.value;
					} else {
						// can not set value when there is no name
					}
				} else if (elm.type == "checkbox" && elm.checked) {
					if(typeof(myObj[name]) == 'undefined') {
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				} else if (elm.type == "select-multiple") {
					if(typeof(myObj[name]) == 'undefined') {
						myObj[name]=[ ];
					}
					for (var jdx=0,len3=elm.options.length; jdx<len3; ++jdx) {
						if (elm.options[jdx].selected) {
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

	 	isValid: function() {
	 		// TODO: ComboBox might need time to process a recently input value.  This should be async?
	 		// make sure that every widget that has a validator function returns true
	 		return dojo.every(this.getDescendants(), function(widget){
	 			return !widget.isValid || widget.isValid();
	 		});
		}
	});

dojo.declare(
	"dijit.form.Form",
	[dijit._Widget, dijit._Templated, dijit.form._FormMixin],
	null
);

}

if(!dojo._hasResource["dijit.Dialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Dialog"] = true;
dojo.provide("dijit.Dialog");









dojo.declare(
	"dijit.DialogUnderlay",
	[dijit._Widget, dijit._Templated],
	{
		// summary: the thing that grays out the screen behind the dialog

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class=dijitDialogUnderlayWrapper id='${id}_underlay'><div class=dijitDialogUnderlay dojoAttachPoint='node'></div></div>",

		postCreate: function(){
			dojo.body().appendChild(this.domNode);
			this.bgIframe = new dijit.BackgroundIframe(this.domNode);
		},

		layout: function(){
			// summary
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.

			var viewport = dijit.getViewport();
			var is = this.node.style,
				os = this.domNode.style;

			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";

			// process twice since the scroll bar may have been removed
			// by the previous resizing
			var viewport2 = dijit.getViewport();
			if(viewport.w != viewport2.w){ is.width = viewport2.w + "px"; }
			if(viewport.h != viewport2.h){ is.height = viewport2.h + "px"; }
		},

		show: function(){
			this.domNode.style.display = "block";
			this.layout();
			if(this.bgIframe.iframe){
				this.bgIframe.iframe.style.display = "block";
			}
			this._resizeHandler = this.connect(window, "onresize", "layout");
		},

		hide: function(){
			this.domNode.style.display = "none";
			if(this.bgIframe.iframe){
				this.bgIframe.iframe.style.display = "none";
			}
			this.disconnect(this._resizeHandler);
		},

		uninitialize: function(){
			if(this.bgIframe){
				this.bgIframe.destroy();
			}
		}
	}
);

dojo.declare(
	"dijit.Dialog",
	[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin],
	{
		// summary:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.)

		templateString: null,
		templateString:"<div class=\"dijitDialog\">\r\n\t<div dojoAttachPoint=\"titleBar\" class=\"dijitDialogTitleBar\" tabindex=\"0\" waiRole=\"dialog\">\r\n\t<span dojoAttachPoint=\"titleNode\" class=\"dijitDialogTitle\">${title}</span>\r\n\t<span dojoAttachPoint=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" dojoAttachEvent=\"onclick: hide\">\r\n\t\t<span dojoAttachPoint=\"closeText\" class=\"closeText\">x</span>\r\n\t</span>\r\n\t</div>\r\n\t\t<div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\r\n\t<span dojoAttachPoint=\"tabEnd\" dojoAttachEvent=\"onfocus:_cycleFocus\" tabindex=\"0\"></span>\r\n</div>\r\n",

		// open: Boolean
		//		is True or False depending on state of dialog
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: 400,

		_lastFocusItem:null,

		attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
			{title: "titleBar"}),

		postCreate: function(){
			dojo.body().appendChild(this.domNode);
			this.inherited("postCreate",arguments);
			this.domNode.style.display="none";
			this.connect(this, "onExecute", "hide");
			this.connect(this, "onCancel", "hide");
		},

		onLoad: function(){
			// summary: 
			//		when href is specified we need to reposition the dialog after the data is loaded
			this._position();
			this.inherited("onLoad",arguments);
		},

		_setup: function(){
			// summary:
			//		stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons)

			this._modalconnects = [];

			if(this.titleBar){
				this._moveable = new dojo.dnd.Moveable(this.domNode, { handle: this.titleBar });
			}

			this._underlay = new dijit.DialogUnderlay();

			var node = this.domNode;
			this._fadeIn = dojo.fx.combine(
				[dojo.fadeIn({
					node: node,
					duration: this.duration
				 }),
				 dojo.fadeIn({
					node: this._underlay.domNode,
					duration: this.duration,
					onBegin: dojo.hitch(this._underlay, "show")
				 })
				]
			);

			this._fadeOut = dojo.fx.combine(
				[dojo.fadeOut({
					node: node,
					duration: this.duration,
					onEnd: function(){
						node.style.display="none";
					}
				 }),
				 dojo.fadeOut({
					node: this._underlay.domNode,
					duration: this.duration,
					onEnd: dojo.hitch(this._underlay, "hide")
				 })
				]
			);
		},

		uninitialize: function(){
			if(this._underlay){
				this._underlay.destroy();
			}
		},

		_position: function(){
			// summary: position modal dialog in center of screen
			
			if(dojo.hasClass(dojo.body(),"dojoMove")){ return; }
			var viewport = dijit.getViewport();
			var mb = dojo.marginBox(this.domNode);

			var style = this.domNode.style;
			style.left = Math.floor((viewport.l + (viewport.w - mb.w)/2)) + "px";
			style.top = Math.floor((viewport.t + (viewport.h - mb.h)/2)) + "px";
		},

		_findLastFocus: function(/*Event*/ evt){
			// summary:  called from onblur of dialog container to determine the last focusable item
			this._lastFocused = evt.target;
		},

		_cycleFocus: function(/*Event*/ evt){
			// summary: when tabEnd receives focus, advance focus around to titleBar

			// on first focus to tabEnd, store the last focused item in dialog
			if(!this._lastFocusItem){
				this._lastFocusItem = this._lastFocused;
			}
			this.titleBar.focus();
		},

		_onKey: function(/*Event*/ evt){
			if(evt.keyCode){
				var node = evt.target;
				// see if we are shift-tabbing from titleBar
				if(node == this.titleBar && evt.shiftKey && evt.keyCode == dojo.keys.TAB){
					if(this._lastFocusItem){
						this._lastFocusItem.focus(); // send focus to last item in dialog if known
					}
					dojo.stopEvent(evt);
				}else{
					// see if the key is for the dialog
					while(node){
						if(node == this.domNode){
							if(evt.keyCode == dojo.keys.ESCAPE){
								this.hide(); 
							}else{
								return; // just let it go
							}
						}
						node = node.parentNode;
					}
					// this key is for the disabled document window
					if(evt.keyCode != dojo.keys.TAB){ // allow tabbing into the dialog for a11y
						dojo.stopEvent(evt);
					// opera won't tab to a div
					}else if (!dojo.isOpera){
						try{
							this.titleBar.focus();
						}catch(e){/*squelch*/}
					}
				}
			}
		},

		show: function(){
			// summary: display the dialog

			// first time we show the dialog, there's some initialization stuff to do			
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized=true;
			}

			if(this._fadeOut.status() == "playing"){
				this._fadeOut.stop();
			}

			this._modalconnects.push(dojo.connect(window, "onscroll", this, "layout"));
			this._modalconnects.push(dojo.connect(document.documentElement, "onkeypress", this, "_onKey"));

			// IE doesn't bubble onblur events - use ondeactivate instead
			var ev = typeof(document.ondeactivate) == "object" ? "ondeactivate" : "onblur";
			this._modalconnects.push(dojo.connect(this.containerNode, ev, this, "_findLastFocus"));

			dojo.style(this.domNode, "opacity", 0);
			this.domNode.style.display="block";
			this.open = true;
			this._loadCheck(); // lazy load trigger

			this._position();

			this._fadeIn.play();

			this._savedFocus = dijit.getFocus(this);

			// set timeout to allow the browser to render dialog
			setTimeout(dojo.hitch(this, function(){
				dijit.focus(this.titleBar);
			}), 50);
		},

		hide: function(){
			// summary
			//		Hide the dialog

			// if we haven't been initialized yet then we aren't showing and we can just return		
			if(!this._alreadyInitialized){
				return;
			}

			if(this._fadeIn.status() == "playing"){
				this._fadeIn.stop();
			}
			this._fadeOut.play();

			if (this._scrollConnected){
				this._scrollConnected = false;
			}
			dojo.forEach(this._modalconnects, dojo.disconnect);
			this._modalconnects = [];

			this.connect(this._fadeOut,"onEnd",dojo.hitch(this,function(){
				dijit.focus(this._savedFocus);
			}));
			this.open = false;
		},

		layout: function() {
			// summary: position the Dialog and the underlay
			if(this.domNode.style.display == "block"){
				this._underlay.layout();
				this._position();
			}
		}
	}
);

dojo.declare(
	"dijit.TooltipDialog",
	[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin],
	{
		// summary:
		//		Pops up a dialog that appears like a Tooltip
		// title: String
		// 		Description of tooltip dialog (required for a11Y)
		title: "",

		_lastFocusItem: null,

		templateString: null,
		templateString:"<div class=\"dijitTooltipDialog\" >\r\n\t<div class=\"dijitTooltipContainer\">\r\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" dojoAttachPoint=\"containerNode\" tabindex=\"0\" waiRole=\"dialog\"></div>\r\n\t</div>\r\n\t<span dojoAttachPoint=\"tabEnd\" tabindex=\"0\" dojoAttachEvent=\"focus:_cycleFocus\"></span>\r\n\t<div class=\"dijitTooltipConnector\" ></div>\r\n</div>\r\n",

		postCreate: function(){
			this.inherited("postCreate",arguments);
			this.connect(this.containerNode, "onkeypress", "_onKey");

			// IE doesn't bubble onblur events - use ondeactivate instead
			var ev = typeof(document.ondeactivate) == "object" ? "ondeactivate" : "onblur";
			this.connect(this.containerNode, ev, "_findLastFocus");
			this.containerNode.title=this.title;
		},

		orient: function(/*Object*/ corner){
			// summary: configure widget to be displayed in given position relative to the button
			this.domNode.className="dijitTooltipDialog " +" dijitTooltipAB"+(corner.charAt(1)=='L'?"Left":"Right")+" dijitTooltip"+(corner.charAt(0)=='T' ? "Below" : "Above");
		},

		onOpen: function(/*Object*/ pos){
			// summary: called when dialog is displayed
			this.orient(pos.corner);
			this._loadCheck(); // lazy load trigger
			this.containerNode.focus();
		},

		_onKey: function(/*Event*/ evt){
			// summary: keep keyboard focus in dialog; close dialog on escape key
			if(evt.keyCode == dojo.keys.ESCAPE){
				this.onCancel();
			}else if(evt.target == this.containerNode && evt.shiftKey && evt.keyCode == dojo.keys.TAB){
				if (this._lastFocusItem){
					this._lastFocusItem.focus();
				}
				dojo.stopEvent(evt);
			}else if(evt.keyCode == dojo.keys.TAB){
				// we want the browser's default tab handling to move focus
				// but we don't want the tab to propagate upwards
				evt.stopPropagation();
			}
		},

		_findLastFocus: function(/*Event*/ evt){
			// summary: called from onblur of dialog container to determine the last focusable item
			this._lastFocused = evt.target;
		},

		_cycleFocus: function(/*Event*/ evt){
			// summary: when tabEnd receives focus, advance focus around to containerNode

			// on first focus to tabEnd, store the last focused item in dialog
			if(!this._lastFocusItem){
				this._lastFocusItem = this._lastFocused;
			}
			this.containerNode.focus();
		}
	}	
);


}

if(!dojo._hasResource["dijit.Menu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Menu"] = true;
dojo.provide("dijit.Menu");





dojo.declare(
	"dijit.Menu",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	constructor: function() {
		this._bindings = [];
	},

	templateString:
			'<table class="dijit dijitMenu dijitReset dijitMenuTable" waiRole="menu" dojoAttachEvent="onkeypress:_onKeyPress">' +
				'<tbody class="dijitReset" dojoAttachPoint="containerNode"></tbody>'+
			'</table>',

	// targetNodeIds: String[]
	//	Array of dom node ids of nodes to attach to.
	//	Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
	targetNodeIds: [],

	// contextMenuForWindow: Boolean
	//	if true, right clicking anywhere on the window will cause this context menu to open;
	//	if false, must specify targetNodeIds
	contextMenuForWindow: false,

	// parentMenu: Widget
	// pointer to menu that displayed me
	parentMenu: null,

	// popupDelay: Integer
	//	number of milliseconds before hovering (without clicking) causes the popup to automatically open
	popupDelay: 500,

	// _contextMenuWithMouse: Boolean
	//	used to record mouse and keyboard events to determine if a context
	//	menu is being opened with the keyboard or the mouse
	_contextMenuWithMouse: false,

	postCreate: function(){
		if(this.contextMenuForWindow){
			this.bindDomNode(dojo.body());
		}else{
			dojo.forEach(this.targetNodeIds, this.bindDomNode, this);
		}
		this.connectKeyNavHandlers([dojo.keys.UP_ARROW], [dojo.keys.DOWN_ARROW]);
	},

	startup: function(){
		dojo.forEach(this.getChildren(), function(child){ child.startup(); });
		this.startupKeyNavChildren();
	},

	onExecute: function(){
		// summary: attach point for notification about when a menu item has been executed
	},

	onCancel: function(/*Boolean*/ closeAll){
		// summary: attach point for notification about when the user cancels the current menu
	},

	_moveToPopup: function(/*Event*/ evt){
		if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
			this.focusedChild._onClick(evt);
		}
	},

	_onKeyPress: function(/*Event*/ evt){
		// summary
		//	Handle keyboard based menu navigation.
		if(evt.ctrlKey || evt.altKey){ return; }

		switch(evt.keyCode){
			case dojo.keys.RIGHT_ARROW:
				this._moveToPopup(evt);
				dojo.stopEvent(evt);
				break;
			case dojo.keys.LEFT_ARROW:
				if(this.parentMenu){
					this.onCancel(false);
				}else{
					dojo.stopEvent(evt);
				}
				break;
		}
	},

	onItemHover: function(/*MenuItem*/ item){
		this.focusChild(item);

		if(this.focusedChild.popup && !this.focusedChild.disabled && !this.hover_timer){
			this.hover_timer = setTimeout(dojo.hitch(this, "_openPopup"), this.popupDelay);
		}
	},

	_onChildBlur: function(item){
		// Close all popups that are open and descendants of this menu
		dijit.popup.close(item.popup);
		item._blur();
		this._stopPopupTimer();
	},

	onItemUnhover: function(/*MenuItem*/ item){
	},

	_stopPopupTimer: function(){
		if(this.hover_timer){
			clearTimeout(this.hover_timer);
			this.hover_timer = null;
		}
	},

	_getTopMenu: function(){
		for(var top=this; top.parentMenu; top=top.parentMenu);
		return top;
	},

	onItemClick: function(/*Widget*/ item){
		// summary: user defined function to handle clicks on an item
		// summary: internal function for clicks
		if(item.disabled){ return false; }

		if(item.popup){
			if(!this.is_open){
				this._openPopup();
			}
		}else{
			// before calling user defined handler, close hierarchy of menus
			// and restore focus to place it was when menu was opened
			this.onExecute();

			// user defined handler for click
			item.onClick();
		}
	},

	// thanks burstlib!
	_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el) {
		//	summary
		//	returns the window reference of the passed iframe
		var win = dijit.getDocumentWindow(dijit.Menu._iframeContentDocument(iframe_el)) ||
			// Moz. TODO: is this available when defaultView isn't?
			dijit.Menu._iframeContentDocument(iframe_el)['__parent__'] ||
			(iframe_el.name && document.frames[iframe_el.name]) || null;
		return win;	//	Window
	},

	_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
		//	summary
		//	returns a reference to the document object inside iframe_el
		var doc = iframe_el.contentDocument // W3
			|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
			|| (iframe_el.name && document.frames[iframe_el.name] && document.frames[iframe_el.name].document)
			|| null;
		return doc;	//	HTMLDocument
	},

	bindDomNode: function(/*String|DomNode*/ node){
		// summary: attach menu to given node
		node = dojo.byId(node);

		//TODO: this is to support context popups in Editor.  Maybe this shouldn't be in dijit.Menu
		var win = dijit.getDocumentWindow(node.ownerDocument);
		if(node.tagName.toLowerCase()=="iframe"){
			win = this._iframeContentWindow(node);
			node = dojo.withGlobal(win, dojo.body);
		}

		// to capture these events at the top level,
		// attach to document, not body
		var cn = (node == dojo.body() ? dojo.doc : node);

		node[this.id] = this._bindings.push([
			dojo.connect(cn, "oncontextmenu", this, "_openMyself"),
			dojo.connect(cn, "onkeydown", this, "_contextKey"),
			dojo.connect(cn, "onmousedown", this, "_contextMouse")
		]);
	},

	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary: detach menu from given node
		var node = dojo.byId(nodeName);
		var bid = node[this.id]-1, b = this._bindings[bid];
		dojo.forEach(b, dojo.disconnect);
		delete this._bindings[bid];
	},

	_contextKey: function(e){
		this._contextMenuWithMouse = false;
		if (e.keyCode == dojo.keys.F10) {
			dojo.stopEvent(e);
			if (e.shiftKey && e.type=="keydown") {
				// FF: copying the wrong property from e will cause the system
				// context menu to appear in spite of stopEvent. Don't know
				// exactly which properties cause this effect.
				var _e = { target: e.target, pageX: e.pageX, pageY: e.pageY };
				_e.preventDefault = _e.stopPropagation = function(){};
				// IE: without the delay, focus work in "open" causes the system
				// context menu to appear in spite of stopEvent.
				window.setTimeout(dojo.hitch(this, function(){ this._openMyself(_e); }), 1);
			}
		}
	},

	_contextMouse: function(e){
		this._contextMenuWithMouse = true;
	},

	_openMyself: function(/*Event*/ e){
		// summary:
		//		Internal function for opening myself when the user
		//		does a right-click or something similar

		dojo.stopEvent(e);

		// Get coordinates.
		// if we are opening the menu with the mouse or on safari open
		// the menu at the mouse cursor
		// (Safari does not have a keyboard command to open the context menu
		// and we don't currently have a reliable way to determine
		// _contextMenuWithMouse on Safari)
		var x,y;
		if(dojo.isSafari || this._contextMenuWithMouse){
			x=e.pageX;
			y=e.pageY;
		}else{
			// otherwise open near e.target
			var coords = dojo.coords(e.target, true);
			x = coords.x + 10;
			y = coords.y + 10;
		}

		var self=this;
		var savedFocus = dijit.getFocus(this);
		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			dijit.focus(savedFocus);
			dijit.popup.close(self);
		}
		dijit.popup.open({
			popup: this,
			x: x,
			y: y,
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			dijit.popup.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		}
	},

	onOpen: function(/*Event*/ e){
		// summary
		//		Open menu relative to the mouse
		this.isShowingNow = true;
	},

	onClose: function(){
		// summary: callback when this menu is closed
		this._stopPopupTimer();
		this.parentMenu = null;
		this.isShowingNow = false;
		this.currentPopup = null;
		if(this.focusedChild){
			this._onChildBlur(this.focusedChild);
			this.focusedChild = null;
		}
	},

	_openPopup: function(){
		// summary: open the popup to the side of the current menu item
		this._stopPopupTimer();
		var from_item = this.focusedChild;
		var popup = from_item.popup;

		if(popup.isShowingNow){ return; }
		popup.parentMenu = this;
		var self = this;
		dijit.popup.open({
			parent: this,
			popup: popup,
			around: from_item.arrowCell,
			orient: this.isLeftToRight() ? {'TR': 'TL', 'TL': 'TR'} : {'TL': 'TR', 'TR': 'TL'},
			onCancel: function(){
				// called when the child menu is canceled
				dijit.popup.close(popup);
				from_item.focus();	// put focus back on my node
				self.currentPopup = null;
			}
		});

		this.currentPopup = popup;

		if(popup.focus){
			popup.focus();
		}
	}
}
);

dojo.declare(
	"dijit.MenuItem",
	[dijit._Widget, dijit._Templated, dijit._Contained],
{
	// summary
	//	A line item in a Menu2

	// Make 3 columns
	//   icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
	templateString:
		 '<tr class="dijitReset dijitMenuItem"'
		+'dojoAttachEvent="onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick">'
		+'<td class="dijitReset"><div class="dijitMenuItemIcon ${iconClass}" dojoAttachPoint="iconNode" ></div></td>'
		+'<td tabIndex="-1" class="dijitReset dijitMenuItemLabel" dojoAttachPoint="containerNode" waiRole="menuitem"></td>'
		+'<td class="dijitReset" dojoAttachPoint="arrowCell">'
			+'<div class="dijitMenuExpand" dojoAttachPoint="expand" style="display:none">'
			+'<span class="dijitInline dijitArrowNode dijitMenuExpandInner">+</span>'
			+'</div>'
		+'</td>'
		+'</tr>',

	// label: String
	//	menu text
	label: '',

	// iconClass: String
	//	class to apply to div in button to make it display an icon
	iconClass: "",

	// disabled: Boolean
	//  if true, the menu item is disabled
	//  if false, the menu item is enabled
	disabled: false,

	postCreate: function(){
		dojo.setSelectable(this.domNode, false);
		this.setDisabled(this.disabled);
		if(this.label){
			this.containerNode.innerHTML=this.label;
		}
	},

	_onHover: function(){
		// summary: callback when mouse is moved onto menu item
		this.getParent().onItemHover(this);
	},

	_onUnhover: function(){
		// summary: callback when mouse is moved off of menu item
		// if we are unhovering the currently selected item
		// then unselect it
		this.getParent().onItemUnhover(this);
	},

	_onClick: function(evt){
		this.getParent().onItemClick(this);
		dojo.stopEvent(evt);
	},

	onClick: function() {
		// summary
		//	User defined function to handle clicks
	},

	focus: function(){
		dojo.addClass(this.domNode, 'dijitMenuItemHover');
		try{
			dijit.focus(this.containerNode);
		}catch(e){
			// this throws on IE (at least) in some scenarios
		}
	},

	_blur: function(){
		dojo.removeClass(this.domNode, 'dijitMenuItemHover');
	},

	setDisabled: function(/*Boolean*/ value){
		// summary: enable or disable this menu item
		this.disabled = value;
		dojo[value ? "addClass" : "removeClass"](this.domNode, 'dijitMenuItemDisabled');
		dijit.setWaiState(this.containerNode, 'disabled', value ? 'true' : 'false');
	}
});

dojo.declare(
	"dijit.PopupMenuItem",
	dijit.MenuItem,
{
	_fillContent: function(){
		// my inner HTML contains both the menu item text and a popup widget, like
		// <div dojoType="dijit.PopupMenuItem">
		//		<span>pick me</span>
		//		<popup> ... </popup>
		// </div>
		// the first part holds the menu item text and the second part is the popup
		if(this.srcNodeRef){
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.PopupMenuItem.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		// we didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
		// land now.  move it to document.body.
		if(!this.popup){
			var node = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.popup = dijit.byNode(node);
		}
		dojo.body().appendChild(this.popup.domNode);

		this.popup.domNode.style.display="none";
		dojo.addClass(this.expand, "dijitMenuExpandEnabled");
		dojo.style(this.expand, "display", "");
		dijit.setWaiState(this.containerNode, "haspopup", "true");
	}
});

dojo.declare(
	"dijit.MenuSeparator",
	[dijit._Widget, dijit._Templated, dijit._Contained],
{
	// summary
	//	A line between two menu items

	templateString: '<tr class="dijitMenuSeparator"><td colspan=3>'
			+'<div class="dijitMenuSeparatorTop"></div>'
			+'<div class="dijitMenuSeparatorBottom"></div>'
			+'</td></tr>',

	postCreate: function(){
		dojo.setSelectable(this.domNode, false);
	},
	
	isFocusable: function(){
		// summary:
		//		over ride to always return false
		return false;
	}
});

}

if(!dojo._hasResource["dijit.form.Button"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Button"] = true;
dojo.provide("dijit.form.Button");




dojo.declare("dijit.form.Button", dijit.form._FormWidget, {
/*
 * usage
 *	<button dojoType="button" onClick="...">Hello world</button>
 *
 *  var button1 = new dijit.form.Button({label: "hello world", onClick: foo});
 *	dojo.body().appendChild(button1.domNode);
 */
	// summary
	//	Basically the same thing as a normal HTML button, but with special styling.

	// label: String
	//	text to display in button
	label: "",

	// showLabel: Boolean
	//	whether or not to display the text label in button
	showLabel: true,

	// iconClass: String
	//	class to apply to div in button to make it display an icon
	iconClass: "",

	type: "button",
	baseClass: "dijitButton",
	templateString:"<div class=\"dijit dijitLeft dijitInline dijitButton\"\r\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\r\n\t><div class='dijitRight'\r\n\t\t><button class=\"dijitStretch dijitButtonNode dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\r\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\r\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \r\n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \r\n\t\t\t></span\r\n\t\t\t><span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span\r\n\t\t></button\r\n\t></div\r\n></div>\r\n",

	// TODO: set button's title to this.containerNode.innerText

	_onClick: function(/*Event*/ e){
		// summary: internal function to handle click actions
		if(this.disabled){ return false; }
		this._clicked(); // widget click actions
		return this.onClick(e); // user click actions
	},

	_onButtonClick: function(/*Event*/ e){
		// summary: callback when the user mouse clicks the button portion
		dojo.stopEvent(e);
		var okToSubmit = this._onClick(e) !== false; // returning nothing is same as true

		// for some reason type=submit buttons don't automatically submit the form; do it manually
		if(this.type=="submit" && okToSubmit){
			for(var node=this.domNode; node; node=node.parentNode){
				var widget=dijit.byNode(node);
				if(widget && widget._onSubmit){
					widget._onSubmit(e);
					break;
				}
				if(node.tagName.toLowerCase() == "form"){
					if(!node.onsubmit || node.onsubmit()){ node.submit(); }
					break;
				}
			}
		}
	},

	postCreate: function(){
		// summary:
		//	get label and set as title on button icon if necessary
		if (this.showLabel == false){
			var labelText = "";
			this.label = this.containerNode.innerHTML;
			labelText = dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
			// set title attrib on iconNode
			this.titleNode.title=labelText;
			dojo.addClass(this.containerNode,"dijitDisplayNone");
		}
		this.inherited(arguments);
	},

	onClick: function(/*Event*/ e){
		// summary: user callback for when button is clicked
		//      if type="submit", return value != false to perform submit
		return true;
	},

	_clicked: function(/*Event*/ e){
		// summary: internal replaceable function for when the button is clicked
	},

	setLabel: function(/*String*/ content){
		// summary: reset the label (text) of the button; takes an HTML string
		this.containerNode.innerHTML = this.label = content;
		if(dojo.isMozilla){ // Firefox has re-render issues with tables
			var oldDisplay = dojo.getComputedStyle(this.domNode).display;
			this.domNode.style.display="none";
			var _this = this;
			setTimeout(function(){_this.domNode.style.display=oldDisplay;},1);
		}
		if (this.showLabel == false){
				this.titleNode.title=dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
		}
	}		
});

/*
 * usage
 *	<button dojoType="DropDownButton" label="Hello world"><div dojotype=dijit.Menu>...</div></button>
 *
 *  var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
 *	dojo.body().appendChild(button1);
 */
dojo.declare("dijit.form.DropDownButton", [dijit.form.Button, dijit._Container], {
	// summary
	//		push the button and a menu shows up

	baseClass : "dijitDropDownButton",

	templateString:"<div class=\"dijit dijitLeft dijitInline\"\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse,onclick:_onDropDownClick,onkeydown:_onDropDownKeydown,onblur:_onDropDownBlur,onkeypress:_onKey\"\r\n\t><div class='dijitRight'>\r\n\t<button class=\"dijitStretch dijitButtonNode dijitButtonContents\" type=\"${type}\"\r\n\t\tdojoAttachPoint=\"focusNode,titleNode\" waiRole=\"button\" waiState=\"haspopup-true,labelledby-${id}_label\"\r\n\t\t><div class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\"></div\r\n\t\t><span class=\"dijitButtonText\" \tdojoAttachPoint=\"containerNode,popupStateNode\"\r\n\t\tid=\"${id}_label\">${label}</span\r\n\t\t><span class='dijitA11yDownArrow'>&#9660;</span>\r\n\t</button>\r\n</div></div>\r\n",

	_fillContent: function(){
		// my inner HTML contains both the button contents and a drop down widget, like
		// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
		// The first node is assumed to be the button content. The widget is the popup.
		if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
			//FIXME: figure out how to filter out the widget and use all remaining nodes as button
			//	content, not just nodes[0]
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.form.DropDownButton.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
		// make it invisible, and store a reference to pass to the popup code.
		if(!this.dropDown){
			var dropDownNode = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.dropDown = dijit.byNode(dropDownNode);
			delete this.dropDownContainer;
		}
		dojo.body().appendChild(this.dropDown.domNode);
		this.dropDown.domNode.style.display="none";
	},

	_onArrowClick: function(/*Event*/ e){
		// summary: callback when the user mouse clicks on menu popup node
		if(this.disabled){ return; }
		this._toggleDropDown();
	},

	_onDropDownClick: function(/*Event*/ e){
		// on Firefox 2 on the Mac it is possible to fire onclick
		// by pressing enter down on a second element and transferring
		// focus to the DropDownButton;
		// we want to prevent opening our menu in this situation
		// and only do so if we have seen a keydown on this button;
		// e.detail != 0 means that we were fired by mouse
		var isMacFFlessThan3 = dojo.isFF && dojo.isFF < 3
			&& navigator.appVersion.indexOf("Macintosh") != -1;
		if(!isMacFFlessThan3 || e.detail != 0 || this._seenKeydown){
			this._onArrowClick(e);
		}
		this._seenKeydown = false;
	},

	_onDropDownKeydown: function(/*Event*/ e){
		this._seenKeydown = true;
	},

	_onDropDownBlur: function(/*Event*/ e){
		this._seenKeydown = false;
	},

	_onKey: function(/*Event*/ e){
		// summary: callback when the user presses a key on menu popup node
		if(this.disabled){ return; }
		if(e.keyCode == dojo.keys.DOWN_ARROW){
			if(!this.dropDown || this.dropDown.domNode.style.display=="none"){
				dojo.stopEvent(e);
				return this._toggleDropDown();
			}
		}
	},

	_onBlur: function(){
		// summary: called magically when focus has shifted away from this widget and it's dropdown
		this._closeDropDown();
		// don't focus on button.  the user has explicitly focused on something else.
	},

	_toggleDropDown: function(){
		// summary: toggle the drop-down widget; if it is up, close it, if not, open it
		if(this.disabled){ return; }
		dijit.focus(this.popupStateNode);
		var dropDown = this.dropDown;
		if(!dropDown){ return false; }
		if(!dropDown.isShowingNow){
			// If there's an href, then load that first, so we don't get a flicker
			if(dropDown.href && !dropDown.isLoaded){
				var self = this;
				var handler = dojo.connect(dropDown, "onLoad", function(){
					dojo.disconnect(handler);
					self._openDropDown();
				});
				dropDown._loadCheck(true);
				return;
			}else{
				this._openDropDown();
			}
		}else{
			this._closeDropDown();
		}
	},

	_openDropDown: function(){
		var dropDown = this.dropDown;
		var oldWidth=dropDown.domNode.style.width;
		var self = this;

		dijit.popup.open({
			parent: this,
			popup: dropDown,
			around: this.domNode,
			orient: this.isLeftToRight() ? {'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'}
				: {'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'},
			onExecute: function(){
				self._closeDropDown(true);
			},
			onCancel: function(){
				self._closeDropDown(true);
			},
			onClose: function(){
				dropDown.domNode.style.width = oldWidth;
				self.popupStateNode.removeAttribute("popupActive");
				this._opened = false;
			}
		});
		if(this.domNode.offsetWidth > dropDown.domNode.offsetWidth){
			var adjustNode = null;
			if(!this.isLeftToRight()){
				adjustNode = dropDown.domNode.parentNode;
				var oldRight = adjustNode.offsetLeft + adjustNode.offsetWidth;
			}
			// make menu at least as wide as the button
			dojo.marginBox(dropDown.domNode, {w: this.domNode.offsetWidth});
			if(adjustNode){
				adjustNode.style.left = oldRight - this.domNode.offsetWidth + "px";
			}
		}
		this.popupStateNode.setAttribute("popupActive", "true");
		this._opened=true;
		if(dropDown.focus){
			dropDown.focus();
		}
		// TODO: set this.checked and call setStateClass(), to affect button look while drop down is shown
	},
	
	_closeDropDown: function(/*Boolean*/ focus){
		if(this._opened){
			dijit.popup.close(this.dropDown);
			if(focus){ this.focus(); }
			this._opened = false;			
		}
	}
});

/*
 * usage
 *	<button dojoType="ComboButton" onClick="..."><span>Hello world</span><div dojoType=dijit.Menu>...</div></button>
 *
 *  var button1 = new dijit.form.ComboButton({label: "hello world", onClick: foo, dropDown: "myMenu"});
 *	dojo.body().appendChild(button1.domNode);
 */
dojo.declare("dijit.form.ComboButton", dijit.form.DropDownButton, {
	// summary
	//		left side is normal button, right side displays menu
	templateString:"<table class='dijit dijitReset dijitInline dijitLeft'\r\n\tcellspacing='0' cellpadding='0'\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\">\r\n\t<tr>\r\n\t\t<td\tclass=\"dijitStretch dijitButtonContents dijitButtonNode\"\r\n\t\t\ttabIndex=\"${tabIndex}\"\r\n\t\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\"  dojoAttachPoint=\"titleNode\"\r\n\t\t\twaiRole=\"button\" waiState=\"labelledby-${id}_label\">\r\n\t\t\t<div class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\"></div>\r\n\t\t\t<span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span>\r\n\t\t</td>\r\n\t\t<td class='dijitReset dijitRight dijitButtonNode dijitDownArrowButton'\r\n\t\t\tdojoAttachPoint=\"popupStateNode,focusNode\"\r\n\t\t\tdojoAttachEvent=\"ondijitclick:_onArrowClick, onkeypress:_onKey\"\r\n\t\t\tstateModifier=\"DownArrow\"\r\n\t\t\ttitle=\"${optionsTitle}\" name=\"${name}\"\r\n\t\t\twaiRole=\"button\" waiState=\"haspopup-true\"\r\n\t\t><div waiRole=\"presentation\">&#9660;</div>\r\n\t</td></tr>\r\n</table>\r\n",

	attributeMap: dojo.mixin(dojo.clone(dijit.form._FormWidget.prototype.attributeMap),
		{id:"", name:""}),

	// optionsTitle: String
	//  text that describes the options menu (accessibility)
	optionsTitle: "",

	baseClass: "dijitComboButton",

	_focusedNode: null,

	postCreate: function(){
		this.inherited(arguments);
		this._focalNodes = [this.titleNode, this.popupStateNode];
		dojo.forEach(this._focalNodes, dojo.hitch(this, function(node){
			if(dojo.isIE){
				this.connect(node, "onactivate", this._onNodeFocus);
			}else{
				this.connect(node, "onfocus", this._onNodeFocus);
			}
		}));
	},

	focusFocalNode: function(node){
		// summary: Focus the focal node node.
		this._focusedNode = node;
		dijit.focus(node);
	},

	hasNextFocalNode: function(){
		// summary: Returns true if this widget has no node currently
		//		focused or if there is a node following the focused one.
		//		False is returned if the last node has focus.
		return this._focusedNode !== this.getFocalNodes()[1];
	},

	focusNext: function(){
		// summary: Focus the focal node following the current node with focus
		//		or the first one if no node currently has focus.
		this._focusedNode = this.getFocalNodes()[this._focusedNode ? 1 : 0];
		dijit.focus(this._focusedNode);
	},

	hasPrevFocalNode: function(){
		// summary: Returns true if this widget has no node currently
		//		focused or if there is a node before the focused one.
		//		False is returned if the first node has focus.
		return this._focusedNode !== this.getFocalNodes()[0];
	},

	focusPrev: function(){
		// summary: Focus the focal node before the current node with focus
		//		or the last one if no node currently has focus.
		this._focusedNode = this.getFocalNodes()[this._focusedNode ? 0 : 1];
		dijit.focus(this._focusedNode);
	},

	getFocalNodes: function(){
		// summary: Returns an array of focal nodes for this widget.
		return this._focalNodes;
	},

	_onNodeFocus: function(evt){
		this._focusedNode = evt.currentTarget;
	},

	_onBlur: function(evt){
		this.inherited(arguments);
		this._focusedNode = null;
	}
});

dojo.declare("dijit.form.ToggleButton", dijit.form.Button, {
	// summary
	//	A button that can be in two states (checked or not).
	//	Can be base class for things like tabs or checkbox or radio buttons

	baseClass: "dijitToggleButton",

	// checked: Boolean
	//		Corresponds to the native HTML <input> element's attribute.
	//		In markup, specified as "checked='checked'" or just "checked".
	//		True if the button is depressed, or the checkbox is checked,
	//		or the radio button is selected, etc.
	checked: false,

	_clicked: function(/*Event*/ evt){
		this.setChecked(!this.checked);
	},

	setChecked: function(/*Boolean*/ checked){
		// summary
		//	Programatically deselect the button
		this.checked = checked;
		dijit.setWaiState(this.focusNode || this.domNode, "pressed", this.checked);
		this._setStateClass();		
		this.onChange(checked);
	}
});

}

if(!dojo._hasResource["dijit.form.CheckBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.CheckBox"] = true;
dojo.provide("dijit.form.CheckBox");



dojo.declare(
	"dijit.form.CheckBox",
	dijit.form.ToggleButton,
	{
		// summary:
		// 		Same as an HTML checkbox, but with fancy styling.
		//
		// description:
		// User interacts with real html inputs.
		// On onclick (which occurs by mouse click, space-bar, or
		// using the arrow keys to switch the selected radio button),
		// we update the state of the checkbox/radio.
		//
		// There are two modes:
		//   1. High contrast mode
		//   2. Normal mode
		// In case 1, the regular html inputs are shown and used by the user.
		// In case 2, the regular html inputs are invisible but still used by
		// the user. They are turned quasi-invisible and overlay the background-image.

		templateString:"<fieldset class=\"dijitReset dijitInline\" waiRole=\"presentation\"\r\n\t><input\r\n\t \ttype=\"${type}\" name=\"${name}\"\r\n\t\tclass=\"dijitReset dijitCheckBoxInput\"\r\n\t\tdojoAttachPoint=\"inputNode,focusNode\"\r\n\t \tdojoAttachEvent=\"onmouseover:_onMouse,onmouseout:_onMouse,onclick:_onClick\"\r\n/></fieldset>\r\n",

		baseClass: "dijitCheckBox",

		//	Value of "type" attribute for <input>
		type: "checkbox",

		// value: Value
		//	equivalent to value field on normal checkbox (if checked, the value is passed as
		//	the value when form is submitted)
		value: "on",

		postCreate: function(){
			dojo.setSelectable(this.inputNode, false);
			this.setChecked(this.checked);
			this.inherited(arguments);
		},

		setChecked: function(/*Boolean*/ checked){
			if(dojo.isIE){
				if(checked){ this.inputNode.setAttribute('checked', 'checked'); }
				else{ this.inputNode.removeAttribute('checked'); }
			}else{ this.inputNode.checked = checked; }
			this.inherited(arguments);
		},

		setValue: function(/*String*/ value){
			if(value == null){ value = ""; }
			this.inputNode.value = value;
			dijit.form.CheckBox.superclass.setValue.call(this,value);
		}
	}
);

dojo.declare(
	"dijit.form.RadioButton",
	dijit.form.CheckBox,
	{
		// summary:
		// 		Same as an HTML radio, but with fancy styling.
		//
		// description:
		// Implementation details
		//
		// Specialization:
		// We keep track of dijit radio groups so that we can update the state
		// of all the siblings (the "context") in a group based on input
		// events. We don't rely on browser radio grouping.

		type: "radio",
		baseClass: "dijitRadio",

		// This shared object keeps track of all widgets, grouped by name
		_groups: {},

		postCreate: function(){
			// add this widget to _groups
			(this._groups[this.name] = this._groups[this.name] || []).push(this);

			this.inherited(arguments);
		},

		uninitialize: function(){
			// remove this widget from _groups
			dojo.forEach(this._groups[this.name], function(widget, i, arr){
				if(widget === this){
					arr.splice(i, 1);
					return;
				}
			}, this);
		},

		setChecked: function(/*Boolean*/ checked){
			// If I am being checked then have to deselect currently checked radio button
			if(checked){
				dojo.forEach(this._groups[this.name], function(widget){
					if(widget != this && widget.checked){
						widget.setChecked(false);
					}
				}, this);
			}
			this.inherited(arguments);			
		},

		_clicked: function(/*Event*/ e){
			if(!this.checked){
				this.setChecked(true);
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.form.TextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.TextBox"] = true;
dojo.provide("dijit.form.TextBox");



dojo.declare(
	"dijit.form.TextBox",
	dijit.form._FormWidget,
	{
		// summary:
		//		A generic textbox field.
		//		Serves as a base class to derive more specialized functionality in subclasses.

		//	trim: Boolean
		//		Removes leading and trailing whitespace if true.  Default is false.
		trim: false,

		//	uppercase: Boolean
		//		Converts all characters to uppercase if true.  Default is false.
		uppercase: false,

		//	lowercase: Boolean
		//		Converts all characters to lowercase if true.  Default is false.
		lowercase: false,

		//	propercase: Boolean
		//		Converts the first character of each word to uppercase if true.
		propercase: false,

		// maxLength: String
		//		HTML INPUT tag maxLength declaration.
		maxLength: "",

		templateString:"<input class=\"dojoTextBox\" dojoAttachPoint='textbox,focusNode' name=\"${name}\"\r\n\tdojoAttachEvent='onmouseenter:_onMouse,onmouseleave:_onMouse,onfocus:_onMouse,onblur:_onMouse,onkeyup,onkeypress:_onKeyPress'\r\n\tautocomplete=\"off\" type=\"${type}\"\r\n\t/>\r\n",
		baseClass: "dijitTextBox",

		attributeMap: dojo.mixin(dojo.clone(dijit.form._FormWidget.prototype.attributeMap),
			{maxLength:"focusNode"}),

		getDisplayedValue: function(){
			return this.filter(this.textbox.value);
		},

		getValue: function(){
			return this.parse(this.getDisplayedValue(), this.constraints);
		},

		setValue: function(value, /*Boolean, optional*/ priorityChange, /*String, optional*/ formattedValue){
			var filteredValue = this.filter(value);
			if((typeof filteredValue == typeof value) && (formattedValue == null || formattedValue == undefined)){
				formattedValue = this.format(filteredValue, this.constraints);
			}
			if(formattedValue != null && formattedValue != undefined){
				this.textbox.value = formattedValue;
			}
			dijit.form.TextBox.superclass.setValue.call(this, filteredValue, priorityChange);
		},

		setDisplayedValue: function(/*String*/value){
			this.textbox.value = value;
			this.setValue(this.getValue(), true);
		},

		forWaiValuenow: function(){
			return this.getDisplayedValue();
		},

		format: function(/* String */ value, /* Object */ constraints){
			// summary: Replacable function to convert a value to a properly formatted string
			return ((value == null || value == undefined) ? "" : (value.toString ? value.toString() : value));
		},

		parse: function(/* String */ value, /* Object */ constraints){
			// summary: Replacable function to convert a formatted string to a value
			return value;
		},

		postCreate: function(){
			// setting the value here is needed since value="" in the template causes "undefined"
			// and setting in the DOM (instead of the JS object) helps with form reset actions
			this.textbox.setAttribute("value", this.getDisplayedValue());
			this.inherited('postCreate', arguments);

			if(this.srcNodeRef){
				dojo.style(this.textbox, "cssText", this.style);
				this.textbox.className += " " + this["class"];
			}
			this._layoutHack();
		},

		_layoutHack: function(){
			// summary: work around table sizing bugs on FF2 by forcing redraw
			if(dojo.isFF == 2 && this.domNode.tagName=="TABLE"){
				var node=this.domNode;
				var old = node.style.opacity;
				node.style.opacity = "0.999";
				setTimeout(function(){
					node.style.opacity = old;
				}, 0);
			}			
		},

		filter: function(val){
			// summary: Apply various filters to textbox value
			if(val == undefined || val == null){ return ""; }
			else if(typeof val != "string"){ return val; }
			if(this.trim){
				val = dojo.trim(val);
			}
			if(this.uppercase){
				val = val.toUpperCase();
			}
			if(this.lowercase){
				val = val.toLowerCase();
			}
			if(this.propercase){
				val = val.replace(/[^\s]+/g, function(word){
					return word.substring(0,1).toUpperCase() + word.substring(1);
				});
			}
			return val;
		},

		// event handlers, you can over-ride these in your own subclasses
		_onBlur: function(){
			this.setValue(this.getValue(), (this.isValid ? this.isValid() : true));
		},

		onkeyup: function(){
			// TODO: it would be nice to massage the value (ie: automatic uppercase, etc) as the user types
			// but this messes up the cursor position if you are typing into the middle of a word, and
			// also trimming doesn't work correctly (it prevents spaces between words too!)
			// this.setValue(this.getValue());
		}
	}
);

}

if(!dojo._hasResource["dijit.Tooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Tooltip"] = true;
dojo.provide("dijit.Tooltip");




dojo.declare(
	"dijit._MasterTooltip",
	[dijit._Widget, dijit._Templated],
	{
		// summary
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: 200,

		templateString:"<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\">\r\n\t<div class=\"dijitTooltipContainer dijitTooltipContents\" dojoAttachPoint=\"containerNode\" waiRole='alert'></div>\r\n\t<div class=\"dijitTooltipConnector\"></div>\r\n</div>\r\n",

		postCreate: function(){
			dojo.body().appendChild(this.domNode);

			this.bgIframe = new dijit.BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = dojo.fadeIn({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onShow") });
			this.fadeOut = dojo.fadeOut({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onHide") });

		},

		show: function(/*String*/ innerHTML, /*DomNode*/ aroundNode){
			// summary:
			//	Display tooltip w/specified contents to right specified node
			//	(To left if there's no space on the right, or if LTR==right)

			if(this.aroundNode && this.aroundNode === aroundNode){
				return;
			}

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			// Firefox bug. when innerHTML changes to be shorter than previous
			// one, the node size will not be updated until it moves.
			this.domNode.style.top = (this.domNode.offsetTop + 1) + "px";

			// position the element and change CSS according to position	
			var align = this.isLeftToRight() ? {'BR': 'BL', 'BL': 'BR'} : {'BL': 'BR', 'BR': 'BL'};
			var pos = dijit.placeOnScreenAroundElement(this.domNode, aroundNode, align);
			this.domNode.className="dijitTooltip dijitTooltip" + (pos.corner=='BL' ? "Right" : "Left");//FIXME: might overwrite class

			// show it
			dojo.style(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		_onShow: function(){
			if(dojo.isIE){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary: hide the tooltip
			if(!this.aroundNode || this.aroundNode !== aroundNode){
				return;
			}
			if(this._onDeck){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
				return;
			}
			this.fadeIn.stop();
			this.isShowingNow = false;
			this.aroundNode = null;
			this.fadeOut.play();
		},

		_onHide: function(){
			this.domNode.style.cssText="";	// to position offscreen again
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		}

	}
);

dijit.showTooltip = function(/*String*/ innerHTML, /*DomNode*/ aroundNode){
	// summary:
	//	Display tooltip w/specified contents to right specified node
	//	(To left if there's no space on the right, or if LTR==right)
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.show(innerHTML, aroundNode);
};

dijit.hideTooltip = function(aroundNode){
	// summary: hide the tooltip
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.hide(aroundNode);
};

dojo.declare(
	"dijit.Tooltip",
	dijit._Widget,
	{
		// summary
		//		Pops up a tooltip (a help message) when you hover over a node.

		// label: String
		//		Text to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: String[]
		//		Id(s) of domNodes to attach the tooltip to.
		//		When user hovers over any of the specified dom nodes, the tooltip will appear.
		connectId: [],

		postCreate: function(){
			if(this.srcNodeRef){
				this.srcNodeRef.style.display = "none";
			}

			this._connectNodes = [];
			
			dojo.forEach(this.connectId, function(id) {
				var node = dojo.byId(id);
				if (node) {
					this._connectNodes.push(node);
					dojo.forEach(["onMouseOver", "onMouseOut", "onFocus", "onBlur", "onHover", "onUnHover"], function(event){
						this.connect(node, event.toLowerCase(), "_"+event);
					}, this);
					if(dojo.isIE){
						// BiDi workaround
						node.style.zoom = 1;
					}
				}
			}, this);
		},

		_onMouseOver: function(/*Event*/ e){
			this._onHover(e);
		},

		_onMouseOut: function(/*Event*/ e){
			if(dojo.isDescendant(e.relatedTarget, e.target)){
				// false event; just moved from target to target child; ignore.
				return;
			}
			this._onUnHover(e);
		},

		_onFocus: function(/*Event*/ e){
			this._focus = true;
			this._onHover(e);
		},
		
		_onBlur: function(/*Event*/ e){
			this._focus = false;
			this._onUnHover(e);
		},

		_onHover: function(/*Event*/ e){
			if(!this._showTimer){
				var target = e.target;
				this._showTimer = setTimeout(dojo.hitch(this, function(){this.open(target)}), this.showDelay);
			}
		},

		_onUnHover: function(/*Event*/ e){
			// keep a tooltip open if the associated element has focus
			if(this._focus){ return; }
			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
 			// summary: display the tooltip; usually not called directly.
			target = target || this._connectNodes[0];
			if(!target){ return; }

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			dijit.showTooltip(this.label || this.domNode.innerHTML, target);
			
			this._connectNode = target;
		},

		close: function(){
			// summary: hide the tooltip; usually not called directly.
			dijit.hideTooltip(this._connectNode);
			delete this._connectNode;
			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
		},

		uninitialize: function(){
			this.close();
		}
	}
);

}

if(!dojo._hasResource["dijit.form.ValidationTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ValidationTextBox"] = true;
dojo.provide("dijit.form.ValidationTextBox");








dojo.declare(
	"dijit.form.ValidationTextBox",
	dijit.form.TextBox,
	{
		// summary:
		//		A subclass of TextBox.
		//		Over-ride isValid in subclasses to perform specific kinds of validation.

		templateString:"<table style=\"display: -moz-inline-stack;\" class=\"dijit dijitReset dijitInlineTable\" cellspacing=\"0\" cellpadding=\"0\"\r\n\tid=\"widget_${id}\" name=\"${name}\"\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse\" waiRole=\"presentation\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset dijitInputField\" width=\"100%\"\r\n\t\t\t><input dojoAttachPoint='textbox,focusNode' dojoAttachEvent='onfocus,onblur:_onMouse,onkeyup,onkeypress:_onKeyPress' autocomplete=\"off\"\r\n\t\t\ttype='${type}' name='${name}'\r\n\t\t/></td\r\n\t\t><td class=\"dijitReset dijitValidationIconField\" width=\"0%\"\r\n\t\t\t><div dojoAttachPoint='iconNode' class='dijitValidationIcon'></div><div class='dijitValidationIconText'>&Chi;</div\r\n\t\t></td\r\n\t></tr\r\n></table>\r\n",
		baseClass: "dijitTextBox",

		// default values for new subclass properties
		// required: Boolean
		//		Can be true or false, default is false.
		required: false,
		// promptMessage: String
		//		Hint string
		promptMessage: "",
		// invalidMessage: String
		// 		The message to display if value is invalid.
		invalidMessage: "$_unset_$", // read from the message file if not overridden
		// constraints: Object
		//		user-defined object needed to pass parameters to the validator functions
		constraints: {},
		// regExp: String
		//		regular expression string used to validate the input
		//		Do not specify both regExp and regExpGen
		regExp: ".*",
		// regExpGen: Function
		//		user replaceable function used to generate regExp when dependent on constraints
		//		Do not specify both regExp and regExpGen
		regExpGen: function(constraints){ return this.regExp; },
		
		// state: String
		//		Shows current state (ie, validation result) of input (Normal, Warning, or Error)
		state: "",

		setValue: function(){
			this.inherited('setValue', arguments);
			this.validate(false);
		},

		validator: function(value,constraints){
			// summary: user replaceable function used to validate the text input against the regular expression.
			return (new RegExp("^(" + this.regExpGen(constraints) + ")"+(this.required?"":"?")+"$")).test(value) &&
				(!this.required || !this._isEmpty(value)) &&
				(this._isEmpty(value) || this.parse(value, constraints) !== null);
		},

		isValid: function(/* Boolean*/ isFocused){
			// summary: Need to over-ride with your own validation code in subclasses
			return this.validator(this.textbox.value, this.constraints);
		},

		_isEmpty: function(value){
			// summary: Checks for whitespace
			return /^\s*$/.test(value); // Boolean
		},

		getErrorMessage: function(/* Boolean*/ isFocused){
			// summary: return an error message to show if appropriate
			return this.invalidMessage;
		},

		getPromptMessage: function(/* Boolean*/ isFocused){
			// summary: return a hint to show if appropriate
			return this.promptMessage;
		},

		validate: function(/* Boolean*/ isFocused){
			// summary:
			//		Called by oninit, onblur, and onkeypress.
			// description:
			//		Show missing or invalid messages if appropriate, and highlight textbox field.
			var message = "";
			var isValid = this.isValid(isFocused);
			var isEmpty = this._isEmpty(this.textbox.value);
			this.state = (isValid || (!this._hasBeenBlurred && isEmpty)) ? "" : "Error";
			this._setStateClass();
			dijit.setWaiState(this.focusNode, "invalid", (isValid? "false" : "true"));
			if(isFocused){
				if(isEmpty){
					message = this.getPromptMessage(true);
				}
				if(!message && !isValid){
					message = this.getErrorMessage(true);
				}
			}
			this._displayMessage(message);
		},

		// currently displayed message
		_message: "",

		_displayMessage: function(/*String*/ message){
			if(this._message == message){ return; }
			this._message = message;
			this.displayMessage(message);
		},

		displayMessage: function(/*String*/ message){
			// summary:
			//		User overridable method to display validation errors/hints.
			//		By default uses a tooltip.
			if(message){
				dijit.showTooltip(message, this.domNode);
			}else{
				dijit.hideTooltip(this.domNode);
			}
		},

		_hasBeenBlurred: false,

		_onBlur: function(evt){
			this._hasBeenBlurred = true;
			this.validate(false);
			this.inherited('_onBlur', arguments);
		},

		onfocus: function(evt){
			// TODO: change to _onFocus?
			this.validate(true);
			this._onMouse(evt);	// update CSS classes
		},

		onkeyup: function(evt){
			this.onfocus(evt);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////
		constructor: function(){
			this.constraints = {};
		},

		postMixInProperties: function(){
			this.inherited('postMixInProperties', arguments);
			this.constraints.locale=this.lang;
			this.messages = dojo.i18n.getLocalization("dijit.form", "validate", this.lang);
			if(this.invalidMessage == "$_unset_$"){ this.invalidMessage = this.messages.invalidMessage; }
			var p = this.regExpGen(this.constraints);
			this.regExp = p;
			// make value a string for all types so that form reset works well
		}
	}
);

dojo.declare(
	"dijit.form.MappedTextBox",
	dijit.form.ValidationTextBox,
	{
		// summary:
		//		A subclass of ValidationTextBox.
		//		Provides a hidden input field and a serialize method to override

		serialize: function(val, /*Object?*/options){
			// summary: user replaceable function used to convert the getValue() result to a String
			return (val.toString ? val.toString() : "");
		},

		toString: function(){
			// summary: display the widget as a printable string using the widget's value
			var val = this.filter(this.getValue());
			return (val!=null) ? ((typeof val == "string") ? val : this.serialize(val, this.constraints)) : "";
		},

		validate: function(){
			this.valueNode.value = this.toString();
			this.inherited('validate', arguments);
		},

		postCreate: function(){
			var textbox = this.textbox;
			var valueNode = (this.valueNode = document.createElement("input"));
			valueNode.setAttribute("type", textbox.type);
			valueNode.setAttribute("value", this.toString());
			dojo.style(valueNode, "display", "none");
			valueNode.name = this.textbox.name;
			this.textbox.name = "_" + this.textbox.name + "_displayed_";
			this.textbox.removeAttribute("name");
			dojo.place(valueNode, textbox, "after");

			this.inherited('postCreate', arguments);
		}
	}
);

dojo.declare(
	"dijit.form.RangeBoundTextBox",
	dijit.form.MappedTextBox,
	{
		// summary:
		//		A subclass of MappedTextBox.
		//		Tests for a value out-of-range
		/*===== contraints object:
		// min: Number
		//		Minimum signed value.  Default is -Infinity
		min: undefined,
		// max: Number
		//		Maximum signed value.  Default is +Infinity
		max: undefined,
		=====*/

		// rangeMessage: String
		//		The message to display if value is out-of-range
		rangeMessage: "",

		compare: function(val1, val2){
			// summary: compare 2 values
			return val1 - val2;
		},

		rangeCheck: function(/* Number */ primitive, /* Object */ constraints){
			// summary: user replaceable function used to validate the range of the numeric input value
			var isMin = (typeof constraints.min != "undefined");
			var isMax = (typeof constraints.max != "undefined");
			if(isMin || isMax){
				return (!isMin || this.compare(primitive,constraints.min) >= 0) &&
					(!isMax || this.compare(primitive,constraints.max) <= 0);
			}else{ return true; }
		},

		isInRange: function(/* Boolean*/ isFocused){
			// summary: Need to over-ride with your own validation code in subclasses
			return this.rangeCheck(this.getValue(), this.constraints);
		},

		isValid: function(/* Boolean*/ isFocused){
			return this.inherited('isValid', arguments) &&
				((this._isEmpty(this.textbox.value) && !this.required) || this.isInRange(isFocused));
		},

		getErrorMessage: function(/* Boolean*/ isFocused){
			if(dijit.form.RangeBoundTextBox.superclass.isValid.call(this, false) && !this.isInRange(isFocused)){ return this.rangeMessage; }
			else{ return this.inherited('getErrorMessage', arguments); }
		},

		postMixInProperties: function(){
			this.inherited('postMixInProperties', arguments);
			if(!this.rangeMessage){
				this.messages = dojo.i18n.getLocalization("dijit.form", "validate", this.lang);
				this.rangeMessage = this.messages.rangeMessage;
			}
		},

		postCreate: function(){
			this.inherited('postCreate', arguments);
			if(typeof this.constraints.min != "undefined"){
				dijit.setWaiState(this.focusNode, "valuemin", this.constraints.min);
			}
			if(typeof this.constraints.max != "undefined"){
				dijit.setWaiState(this.focusNode, "valuemax", this.constraints.max);
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.form.ComboBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ComboBox"] = true;
dojo.provide("dijit.form.ComboBox");





dojo.declare(
	"dijit.form.ComboBoxMixin",
	null,
	{
		// summary:
		//		Auto-completing text box, and base class for FilteringSelect widget.
		//
		//		The drop down box's values are populated from an class called
		//		a data provider, which returns a list of values based on the characters
		//		that the user has typed into the input box.
		//
		//		Some of the options to the ComboBox are actually arguments to the data
		//		provider.
		//
		//		You can assume that all the form widgets (and thus anything that mixes
		//		in ComboBoxMixin) will inherit from _FormWidget and thus the "this"
		//		reference will also "be a" _FormWidget.

		// item: Object
		//		This is the item returned by the dojo.data.store implementation that
		//		provides the data for this cobobox, it's the currently selected item.
		item: null,

		// pageSize: Integer
		//		Argument to data provider.
		//		Specifies number of search results per page (before hitting "next" button)
		pageSize: Infinity,

		// store: Object
		//		Reference to data provider object used by this ComboBox
		store: null,

		// query: Object
		//		A query that can be passed to 'store' to initially filter the items,
		//		before doing further filtering based on searchAttr and the key.
		query: {},

		// autoComplete: Boolean
		//		If you type in a partial string, and then tab out of the <input> box,
		//		automatically copy the first entry displayed in the drop down list to
		//		the <input> field
		autoComplete: true,

		// searchDelay: Integer
		//		Delay in milliseconds between when user types something and we start
		//		searching based on that value
		searchDelay: 100,

		// searchAttr: String
		//		Searches pattern match against this field
		searchAttr: "name",

		// ignoreCase: Boolean
		//		Set true if the ComboBox should ignore case when matching possible items
		ignoreCase: true,

		// hasDownArrow: Boolean
		//		Set this textbox to have a down arrow button.
		//		Defaults to true.
		hasDownArrow:true,

		// _hasFocus: Boolean
		//		Represents focus state of the textbox
		// TODO: get rid of this; it's unnecessary (but currently referenced in FilteringSelect)
		_hasFocus:false,

		templateString:"<table class=\"dijit dijitReset dijitInlineTable dijitLeft\" cellspacing=\"0\" cellpadding=\"0\"\r\n\tid=\"widget_${id}\" name=\"${name}\" dojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse\" waiRole=\"presentation\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class='dijitReset dijitStretch dijitInputField' width=\"100%\"\r\n\t\t\t><input type=\"text\" autocomplete=\"off\" name=\"${name}\"\r\n\t\t\tdojoAttachEvent=\"onkeypress, onkeyup, onfocus, compositionend\"\r\n\t\t\tdojoAttachPoint=\"textbox,focusNode\" waiRole=\"combobox\"\r\n\t\t/></td\r\n\t\t><td class=\"dijitReset dijitValidationIconField\" width=\"0%\"\r\n\t\t\t><div dojoAttachPoint='iconNode' class='dijitValidationIcon'></div\r\n\t\t\t><div class='dijitValidationIconText'>&Chi;</div\r\n\t\t></td\r\n\t\t><td class='dijitReset dijitRight dijitButtonNode dijitDownArrowButton' width=\"0%\"\r\n\t\t\tdojoAttachPoint=\"downArrowNode\"\r\n\t\t\tdojoAttachEvent=\"onmousedown:_onArrowMouseDown,onmouseup:_onMouse,onmouseenter:_onMouse,onmouseleave:_onMouse\"\r\n\t\t\t><div class=\"dijitDownArrowButtonInner\" waiRole=\"presentation\"\r\n\t\t\t\t><div class=\"dijitDownArrowButtonChar\">&#9660;</div\r\n\t\t\t></div\r\n\t\t></td\t\r\n\t></tr\r\n></table>\r\n",

		baseClass:"dijitComboBox",

		_lastDisplayedValue: "",

		getValue:function(){
			// don't get the textbox value but rather the previously set hidden value
			return dijit.form.TextBox.superclass.getValue.apply(this, arguments);
		},

		setDisplayedValue:function(/*String*/ value){
			this._lastDisplayedValue = value;
			this.setValue(value, true);
		},

		_getCaretPos: function(/*DomNode*/ element){
			// khtml 3.5.2 has selection* methods as does webkit nightlies from 2005-06-22
			if(typeof(element.selectionStart)=="number"){
				// FIXME: this is totally borked on Moz < 1.3. Any recourse?
				return element.selectionStart;
			}else if(dojo.isIE){
				// in the case of a mouse click in a popup being handled,
				// then the document.selection is not the textarea, but the popup
				// var r = document.selection.createRange();
				// hack to get IE 6 to play nice. What a POS browser.
				var tr = document.selection.createRange().duplicate();
				var ntr = element.createTextRange();
				tr.move("character",0);
				ntr.move("character",0);
				try{
					// If control doesnt have focus, you get an exception.
					// Seems to happen on reverse-tab, but can also happen on tab (seems to be a race condition - only happens sometimes).
					// There appears to be no workaround for this - googled for quite a while.
					ntr.setEndPoint("EndToEnd", tr);
					return String(ntr.text).replace(/\r/g,"").length;
				}catch(e){
					return 0; // If focus has shifted, 0 is fine for caret pos.
				}
			}
		},

		_setCaretPos: function(/*DomNode*/ element, /*Number*/ location){
			location = parseInt(location);
			this._setSelectedRange(element, location, location);
		},

		_setSelectedRange: function(/*DomNode*/ element, /*Number*/ start, /*Number*/ end){
			if(!end){
				end = element.value.length;
			}  // NOTE: Strange - should be able to put caret at start of text?
			// Mozilla
			// parts borrowed from http://www.faqts.com/knowledge_base/view.phtml/aid/13562/fid/130
			if(element.setSelectionRange){
				dijit.focus(element);
				element.setSelectionRange(start, end);
			}else if(element.createTextRange){ // IE
				var range = element.createTextRange();
				with(range){
					collapse(true);
					moveEnd('character', end);
					moveStart('character', start);
					select();
				}
			}else{ //otherwise try the event-creation hack (our own invention)
				// do we need these?
				element.value = element.value;
				element.blur();
				dijit.focus(element);
				// figure out how far back to go
				var dist = parseInt(element.value.length)-end;
				var tchar = String.fromCharCode(37);
				var tcc = tchar.charCodeAt(0);
				for(var x = 0; x < dist; x++){
					var te = document.createEvent("KeyEvents");
					te.initKeyEvent("keypress", true, true, null, false, false, false, false, tcc, tcc);
					element.dispatchEvent(te);
				}
			}
		},

		onkeypress: function(/*Event*/ evt){
			// summary: handles keyboard events

			//except for pasting case - ctrl + v(118)
			if(evt.altKey || (evt.ctrlKey && evt.charCode != 118)){
				return;
			}
			var doSearch = false;
			this.item = null; // #4872
			if(this._isShowingNow){this._popupWidget.handleKey(evt);}
			switch(evt.keyCode){
				case dojo.keys.PAGE_DOWN:
				case dojo.keys.DOWN_ARROW:
					if(!this._isShowingNow||this._prev_key_esc){
						this._arrowPressed();
						doSearch=true;
					}else{
						this._announceOption(this._popupWidget.getHighlightedOption());
					}
					dojo.stopEvent(evt);
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					break;

				case dojo.keys.PAGE_UP:
				case dojo.keys.UP_ARROW:
					if(this._isShowingNow){
						this._announceOption(this._popupWidget.getHighlightedOption());
					}
					dojo.stopEvent(evt);
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					break;

				case dojo.keys.ENTER:
					// prevent submitting form if user presses enter
					// also prevent accepting the value if either Next or Previous are selected
					var highlighted;
					if(this._isShowingNow&&(highlighted=this._popupWidget.getHighlightedOption())){
						// only stop event on prev/next
						if(highlighted==this._popupWidget.nextButton){
							this._nextSearch(1);
							dojo.stopEvent(evt);
							break;
						}
						else if(highlighted==this._popupWidget.previousButton){
							this._nextSearch(-1);
							dojo.stopEvent(evt);
							break;
						}
					}else{
						this.setDisplayedValue(this.getDisplayedValue());
					}
					// default case:
					// prevent submit, but allow event to bubble
					evt.preventDefault();
					// fall through

				case dojo.keys.TAB:
					var newvalue=this.getDisplayedValue();
					// #4617: if the user had More Choices selected fall into the _onBlur handler
					if(this._popupWidget &&
						(newvalue == this._popupWidget._messages["previousMessage"] ||
							newvalue == this._popupWidget._messages["nextMessage"])){
						break;
					}
					if(this._isShowingNow){
						this._prev_key_backspace = false;
						this._prev_key_esc = false;
						if(this._popupWidget.getHighlightedOption()){
							this._popupWidget.setValue({target:this._popupWidget.getHighlightedOption()}, true);
						}
						this._hideResultList();
					}
					break;

				case dojo.keys.SPACE:
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					if(this._isShowingNow && this._popupWidget.getHighlightedOption()){
						dojo.stopEvent(evt);
						this._selectOption();
						this._hideResultList();
					}else{
						doSearch = true;
					}
					break;

				case dojo.keys.ESCAPE:
					this._prev_key_backspace = false;
					this._prev_key_esc = true;
					this._hideResultList();
					if(this._lastDisplayedValue != this.getDisplayedValue()){
						this.setDisplayedValue(this._lastDisplayedValue);
						dojo.stopEvent(evt);
					}else{
						this.setValue(this.getValue(), false);
					}
					break;

				case dojo.keys.DELETE:
				case dojo.keys.BACKSPACE:
					this._prev_key_esc = false;
					this._prev_key_backspace = true;
					doSearch = true;
					break;

				case dojo.keys.RIGHT_ARROW: // fall through

				case dojo.keys.LEFT_ARROW: // fall through
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					break;

				default:// non char keys (F1-F12 etc..)  shouldn't open list
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					if(dojo.isIE || evt.charCode != 0){
						doSearch=true;
					}
			}
			if(this.searchTimer){
				clearTimeout(this.searchTimer);
			}
			if(doSearch){
				// need to wait a tad before start search so that the event bubbles through DOM and we have value visible
				this.searchTimer = setTimeout(dojo.hitch(this, this._startSearchFromInput), this.searchDelay);
			}
		},

		_autoCompleteText: function(/*String*/ text){
			// summary:
			// Fill in the textbox with the first item from the drop down list, and
			// highlight the characters that were auto-completed.   For example, if user
			// typed "CA" and the drop down list appeared, the textbox would be changed to
			// "California" and "ifornia" would be highlighted.

			// IE7: clear selection so next highlight works all the time
			this._setSelectedRange(this.focusNode, this.focusNode.value.length, this.focusNode.value.length);
			// does text autoComplete the value in the textbox?
			// #3744: escape regexp so the user's input isn't treated as a regular expression.
			// Example: If the user typed "(" then the regexp would throw "unterminated parenthetical."
			// Also see #2558 for the autocompletion bug this regular expression fixes.
			if(new RegExp("^"+escape(this.focusNode.value), this.ignoreCase ? "i" : "").test(escape(text))){
				var cpos = this._getCaretPos(this.focusNode);
				// only try to extend if we added the last character at the end of the input
				if((cpos+1) > this.focusNode.value.length){
					// only add to input node as we would overwrite Capitalisation of chars
					// actually, that is ok
					this.focusNode.value = text;//.substr(cpos);
					// visually highlight the autocompleted characters
					this._setSelectedRange(this.focusNode, cpos, this.focusNode.value.length);
					dijit.setWaiState(this.focusNode, "valuenow", text);
				}
			}else{
				// text does not autoComplete; replace the whole value and highlight
				this.focusNode.value = text;
				this._setSelectedRange(this.focusNode, 0, this.focusNode.value.length);
				dijit.setWaiState(this.focusNode, "valuenow", text);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ dataObject){
			if(this.disabled || dataObject.query[this.searchAttr] != this._lastQuery){
				return;
			}
			this._popupWidget.clearResultList();
			if(!results.length){
				this._hideResultList();
				return;
			}

			// Fill in the textbox with the first item from the drop down list, and
			// highlight the characters that were auto-completed.   For example, if user
			// typed "CA" and the drop down list appeared, the textbox would be changed to
			// "California" and "ifornia" would be highlighted.

			var zerothvalue=new String(this.store.getValue(results[0], this.searchAttr));
			if(zerothvalue && this.autoComplete && !this._prev_key_backspace &&
			// when the user clicks the arrow button to show the full list,
			// startSearch looks for "*".
			// it does not make sense to autocomplete
			// if they are just previewing the options available.
				(dataObject.query[this.searchAttr] != "*")){
				this._autoCompleteText(zerothvalue);
				// announce the autocompleted value
				dijit.setWaiState(this.focusNode || this.domNode, "valuenow", zerothvalue);
			}
			this._popupWidget.createOptions(results, dataObject, dojo.hitch(this, this._getMenuLabelFromItem));

			// show our list (only if we have content, else nothing)
			this._showResultList();

			// #4091: tell the screen reader that the paging callback finished by shouting the next choice
			if(dataObject.direction){
				if(dataObject.direction==1){
					this._popupWidget.highlightFirstOption();
				}else if(dataObject.direction==-1){
					this._popupWidget.highlightLastOption();
				}
				this._announceOption(this._popupWidget.getHighlightedOption());
			}
		},

		_showResultList: function(){
			this._hideResultList();
			var items = this._popupWidget.getItems(),
				visibleCount = Math.min(items.length,this.maxListLength);
			this._arrowPressed();
			// hide the tooltip
			this._displayMessage("");
			
			// Position the list and if it's too big to fit on the screen then
			// size it to the maximum possible height
			// Our dear friend IE doesnt take max-height so we need to calculate that on our own every time
			// TODO: want to redo this, see http://trac.dojotoolkit.org/ticket/3272, http://trac.dojotoolkit.org/ticket/4108
			with(this._popupWidget.domNode.style){
				// natural size of the list has changed, so erase old width/height settings,
				// which were hardcoded in a previous call to this function (via dojo.marginBox() call) 
				width="";
				height="";
			}
			var best=this.open();
			// #3212: only set auto scroll bars if necessary
			// prevents issues with scroll bars appearing when they shouldn't when node is made wider (fractional pixels cause this)
			var popupbox=dojo.marginBox(this._popupWidget.domNode);
			this._popupWidget.domNode.style.overflow=((best.h==popupbox.h)&&(best.w==popupbox.w))?"hidden":"auto";
			// #4134: borrow TextArea scrollbar test so content isn't covered by scrollbar and horizontal scrollbar doesn't appear
			var newwidth=best.w;
			if(best.h<this._popupWidget.domNode.scrollHeight){newwidth+=16;}
			dojo.marginBox(this._popupWidget.domNode, {h:best.h,w:Math.max(newwidth,this.domNode.offsetWidth)});
		},

		_hideResultList: function(){
			if(this._isShowingNow){
				dijit.popup.close(this._popupWidget);
				this._arrowIdle();
				this._isShowingNow=false;
			}
		},

		_onBlur: function(){
			// summary: called magically when focus has shifted away from this widget and it's dropdown
			this._hasFocus=false;
			this._hasBeenBlurred = true;
			this._hideResultList();
			this._arrowIdle();
			// if the user clicks away from the textbox OR tabs away, set the value to the textbox value
			// #4617: if value is now more choices or previous choices, revert the value
			var newvalue=this.getDisplayedValue();
			if(this._popupWidget&&(newvalue==this._popupWidget._messages["previousMessage"]||newvalue==this._popupWidget._messages["nextMessage"])){
				this.setValue(this._lastValueReported, true);
			}else{
				this.setDisplayedValue(newvalue);
			}
		},

		onfocus:function(/*Event*/ evt){
			this._hasFocus=true;
			
			// update styling to reflect that we are focused
			this._onMouse(evt);
		},

		_announceOption: function(/*Node*/ node){
			// summary:
			//	a11y code that puts the highlighted option in the textbox
			//	This way screen readers will know what is happening in the menu

			if(node==null){return;}
			// pull the text value from the item attached to the DOM node
			var newValue;
			if(node==this._popupWidget.nextButton||node==this._popupWidget.previousButton){
				newValue=node.innerHTML;
			}else{
				newValue=this.store.getValue(node.item, this.searchAttr);
			}
			// get the text that the user manually entered (cut off autocompleted text)
			this.focusNode.value=this.focusNode.value.substring(0, this._getCaretPos(this.focusNode));
			// autocomplete the rest of the option to announce change
			this._autoCompleteText(newValue);
		},

		_selectOption: function(/*Event*/ evt){
			var tgt = null;
			if(!evt){
				evt ={ target: this._popupWidget.getHighlightedOption()};
			}
				// what if nothing is highlighted yet?
			if(!evt.target){
				// handle autocompletion where the the user has hit ENTER or TAB
				this.setDisplayedValue(this.getDisplayedValue());
				return;
			// otherwise the user has accepted the autocompleted value
			}else{
				tgt = evt.target;
			}
			if(!evt.noHide){
				this._hideResultList();
				this._setCaretPos(this.focusNode, this.store.getValue(tgt.item, this.searchAttr).length);
			}
			this._doSelect(tgt);
		},

		_doSelect: function(tgt){
			this.item = tgt.item;
			this.setValue(this.store.getValue(tgt.item, this.searchAttr), true);
		},

		_onArrowMouseDown: function(evt){
			// summary: callback when arrow is clicked
			if(this.disabled){
				return;
			}
			dojo.stopEvent(evt);
			this.focus();
			if(this._isShowingNow){
				this._hideResultList();
			}else{
				// forces full population of results, if they click
				// on the arrow it means they want to see more options
				this._startSearch("");
			}
		},

		_startSearchFromInput: function(){
			this._startSearch(this.focusNode.value);
		},

		_startSearch: function(/*String*/ key){
			if(!this._popupWidget){
				this._popupWidget = new dijit.form._ComboBoxMenu({
					onChange: dojo.hitch(this, this._selectOption)
				});
			}
			// create a new query to prevent accidentally querying for a hidden value from FilteringSelect's keyField
			var query=this.query;
			this._lastQuery=query[this.searchAttr]=key+"*";
			var dataObject=this.store.fetch({queryOptions:{ignoreCase:this.ignoreCase, deep:true}, query: query, onComplete:dojo.hitch(this, "_openResultList"), start:0, count:this.pageSize});
			function nextSearch(dataObject, direction){
				dataObject.start+=dataObject.count*direction;
				// #4091: tell callback the direction of the paging so the screen reader knows which menu option to shout
				dataObject.direction=direction;
				dataObject.store.fetch(dataObject);
			}
			this._nextSearch=this._popupWidget.onPage=dojo.hitch(this, nextSearch, dataObject);
		},

		_getValueField:function(){
			return this.searchAttr;
		},

		/////////////// Event handlers /////////////////////

		_arrowPressed: function(){
			if(!this.disabled&&this.hasDownArrow){
				dojo.addClass(this.downArrowNode, "dijitArrowButtonActive");
			}
		},

		_arrowIdle: function(){
			if(!this.disabled&&this.hasDownArrow){
				dojo.removeClass(this.downArrowNode, "dojoArrowButtonPushed");
			}
		},

		compositionend: function(/*Event*/ evt){
			// summary: When inputting characters using an input method, such as Asian
			// languages, it will generate this event instead of onKeyDown event
			// Note: this event is only triggered in FF (not in IE)
			this.onkeypress({charCode:-1});
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////
		constructor: function(){
			this.query={};
		},

		postMixInProperties: function(){
			if(!this.hasDownArrow){
				this.baseClass = "dijitTextBox";
			}
			if(!this.store){
				// if user didn't specify store, then assume there are option tags
				var items = this.srcNodeRef ? dojo.query("> option", this.srcNodeRef).map(function(node){
					node.style.display="none";
					return { value: node.getAttribute("value"), name: String(node.innerHTML) };
				}) : {};
				this.store = new dojo.data.ItemFileReadStore({data: {identifier:this._getValueField(), items:items}});

				// if there is no value set and there is an option list,
				// set the value to the first value to be consistent with native Select
				if(items && items.length && !this.value){
					// For <select>, IE does not let you set the value attribute of the srcNodeRef (and thus dojo.mixin does not copy it).
					// IE does understand selectedIndex though, which is automatically set by the selected attribute of an option tag
					this.value = items[this.srcNodeRef.selectedIndex != -1 ? this.srcNodeRef.selectedIndex : 0]
						[this._getValueField()];
				}
			}
		},

		uninitialize:function(){
			if(this._popupWidget){
				this._hideResultList();
				this._popupWidget.destroy()
			};
		},

		_getMenuLabelFromItem:function(/*Item*/ item){
			return {html:false, label:this.store.getValue(item, this.searchAttr)};
		},

		open:function(){
			this._isShowingNow=true;
			return dijit.popup.open({
				popup: this._popupWidget,
				around: this.domNode,
				parent: this
			});
		}
	}
);

dojo.declare(
	"dijit.form._ComboBoxMenu",
	[dijit._Widget, dijit._Templated],

	{
		// summary:
		//	Focus-less div based menu for internal use in ComboBox

		templateString:"<div class='dijitMenu' dojoAttachEvent='onmousedown,onmouseup,onmouseover,onmouseout' tabIndex='-1' style='overflow:\"auto\";'>"
				+"<div class='dijitMenuItem dijitMenuPreviousButton' dojoAttachPoint='previousButton'></div>"
				+"<div class='dijitMenuItem dijitMenuNextButton' dojoAttachPoint='nextButton'></div>"
			+"</div>",
		_messages:null,

		postMixInProperties:function(){
			this._messages = dojo.i18n.getLocalization("dijit.form", "ComboBox", this.lang);
			this.inherited("postMixInProperties", arguments);
		},

		setValue:function(/*Object*/ value){
			this.value=value;
			this.onChange(value);
		},

		onChange:function(/*Object*/ value){},
		onPage:function(/*Number*/ direction){},

		postCreate:function(){
			// fill in template with i18n messages
			this.previousButton.innerHTML=this._messages["previousMessage"];
			this.nextButton.innerHTML=this._messages["nextMessage"];
			this.inherited("postCreate", arguments);
		},

		onClose:function(){
			this._blurOptionNode();
		},

		_createOption:function(/*Object*/ item, labelFunc){
			// summary: creates an option to appear on the popup menu
			// subclassed by FilteringSelect

			var labelObject=labelFunc(item);
			var menuitem = document.createElement("div");
			if(labelObject.html){menuitem.innerHTML=labelObject.label;}
			else{menuitem.appendChild(document.createTextNode(labelObject.label));}
			// #3250: in blank options, assign a normal height
			if(menuitem.innerHTML==""){
				menuitem.innerHTML="&nbsp;"
			}
			menuitem.item=item;
			return menuitem;
		},

		createOptions:function(results, dataObject, labelFunc){
			//this._dataObject=dataObject;
			//this._dataObject.onComplete=dojo.hitch(comboBox, comboBox._openResultList);
			// display "Previous . . ." button
			this.previousButton.style.display=dataObject.start==0?"none":"";
			// create options using _createOption function defined by parent ComboBox (or FilteringSelect) class
			// #2309: iterate over cache nondestructively
			var _this=this;
			dojo.forEach(results, function(item){
				var menuitem=_this._createOption(item, labelFunc);
				menuitem.className = "dijitMenuItem";
				_this.domNode.insertBefore(menuitem, _this.nextButton);
			});
			// display "Next . . ." button
			this.nextButton.style.display=dataObject.count==results.length?"":"none";
		},

		clearResultList:function(){
			// keep the previous and next buttons of course
			while(this.domNode.childNodes.length>2){
				this.domNode.removeChild(this.domNode.childNodes[this.domNode.childNodes.length-2]);
			}
		},

		// these functions are called in showResultList
		getItems:function(){
			return this.domNode.childNodes;
		},

		getListLength:function(){
			return this.domNode.childNodes.length-2;
		},

		onmousedown:function(/*Event*/ evt){
			dojo.stopEvent(evt);
		},

		onmouseup:function(/*Event*/ evt){
			if(evt.target === this.domNode){
				return;
			}else if(evt.target==this.previousButton){
				this.onPage(-1);
			}else if(evt.target==this.nextButton){
				this.onPage(1);
			}else{
				var tgt=evt.target;
				// while the clicked node is inside the div
				while(!tgt.item){
					// recurse to the top
					tgt=tgt.parentNode;
				}
				this.setValue({target:tgt}, true);
			}
		},

		onmouseover:function(/*Event*/ evt){
			if(evt.target === this.domNode){ return; }
			var tgt=evt.target;
			if(!(tgt==this.previousButton||tgt==this.nextButton)){
				// while the clicked node is inside the div
				while(!tgt.item){
					// recurse to the top
					tgt=tgt.parentNode;
				}
			}
			this._focusOptionNode(tgt);
		},

		onmouseout:function(/*Event*/ evt){
			if(evt.target === this.domNode){ return; }
			this._blurOptionNode();
		},

		_focusOptionNode:function(/*DomNode*/ node){
			// summary:
			//	does the actual highlight
			if(this._highlighted_option != node){
				this._blurOptionNode();
				this._highlighted_option = node;
				dojo.addClass(this._highlighted_option, "dijitMenuItemHover");
			}
		},

		_blurOptionNode:function(){
			// summary:
			//	removes highlight on highlighted option
			if(this._highlighted_option){
				dojo.removeClass(this._highlighted_option, "dijitMenuItemHover");
				this._highlighted_option = null;
			}
		},

		_highlightNextOption:function(){
			// because each press of a button clears the menu,
			// the highlighted option sometimes becomes detached from the menu!
			// test to see if the option has a parent to see if this is the case.
			if(!this.getHighlightedOption()){
				this._focusOptionNode(this.domNode.firstChild.style.display=="none"?this.domNode.firstChild.nextSibling:this.domNode.firstChild);
			}else if(this._highlighted_option.nextSibling&&this._highlighted_option.nextSibling.style.display!="none"){
				this._focusOptionNode(this._highlighted_option.nextSibling);
			}
			// scrollIntoView is called outside of _focusOptionNode because in IE putting it inside causes the menu to scroll up on mouseover
			dijit.scrollIntoView(this._highlighted_option);
		},

		highlightFirstOption:function(){
			// highlight the non-Previous choices option
			this._focusOptionNode(this.domNode.firstChild.nextSibling);
			dijit.scrollIntoView(this._highlighted_option);
		},

		highlightLastOption:function(){
			// highlight the noon-More choices option
			this._focusOptionNode(this.domNode.lastChild.previousSibling);
			dijit.scrollIntoView(this._highlighted_option);
		},

		_highlightPrevOption:function(){
			// if nothing selected, highlight last option
			// makes sense if you select Previous and try to keep scrolling up the list
			if(!this.getHighlightedOption()){
				this._focusOptionNode(this.domNode.lastChild.style.display=="none"?this.domNode.lastChild.previousSibling:this.domNode.lastChild);
			}else if(this._highlighted_option.previousSibling&&this._highlighted_option.previousSibling.style.display!="none"){
				this._focusOptionNode(this._highlighted_option.previousSibling);
			}
			dijit.scrollIntoView(this._highlighted_option);
		},

		_page:function(/*Boolean*/ up){
			var scrollamount=0;
			var oldscroll=this.domNode.scrollTop;
			var height=parseInt(dojo.getComputedStyle(this.domNode).height);
			// if no item is highlighted, highlight the first option
			if(!this.getHighlightedOption()){this._highlightNextOption();}
			while(scrollamount<height){
				if(up){
					// stop at option 1
					if(!this.getHighlightedOption().previousSibling||this._highlighted_option.previousSibling.style.display=="none"){break;}
					this._highlightPrevOption();
				}else{
					// stop at last option
					if(!this.getHighlightedOption().nextSibling||this._highlighted_option.nextSibling.style.display=="none"){break;}
					this._highlightNextOption();
				}
				// going backwards
				var newscroll=this.domNode.scrollTop;
				scrollamount+=(newscroll-oldscroll)*(up ? -1:1);
				oldscroll=newscroll;
			}
		},

		pageUp:function(){
			this._page(true);
		},

		pageDown:function(){
			this._page(false);
		},

		getHighlightedOption:function(){
			// summary:
			//	Returns the highlighted option.
			return this._highlighted_option&&this._highlighted_option.parentNode ? this._highlighted_option : null;
		},

		handleKey:function(evt){
			switch(evt.keyCode){
				case dojo.keys.DOWN_ARROW:
					this._highlightNextOption();
					break;
				case dojo.keys.PAGE_DOWN:
					this.pageDown();
					break;	
				case dojo.keys.UP_ARROW:
					this._highlightPrevOption();
					break;
				case dojo.keys.PAGE_UP:
					this.pageUp();
					break;	
			}
		}
	}
);

dojo.declare(
	"dijit.form.ComboBox",
	[dijit.form.ValidationTextBox, dijit.form.ComboBoxMixin],
	{
		postMixInProperties: function(){
			dijit.form.ComboBoxMixin.prototype.postMixInProperties.apply(this, arguments);
			dijit.form.ValidationTextBox.prototype.postMixInProperties.apply(this, arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit.form.FilteringSelect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.FilteringSelect"] = true;
dojo.provide("dijit.form.FilteringSelect");



dojo.declare(
	"dijit.form.FilteringSelect",
	[dijit.form.MappedTextBox, dijit.form.ComboBoxMixin],
	{
		/*
		 * summary
		 *	Enhanced version of HTML's <select> tag.
		 *
		 *	Similar features:
		 *	  - There is a drop down list of possible values.
		 *	- You can only enter a value from the drop down list.  (You can't enter an arbitrary value.)
		 *	- The value submitted with the form is the hidden value (ex: CA),
		 *	  not the displayed value a.k.a. label (ex: California)
		 *
		 *	Enhancements over plain HTML version:
		 *	- If you type in some text then it will filter down the list of possible values in the drop down list.
		 *	- List can be specified either as a static list or via a javascript function (that can get the list from a server)
		 */

		// searchAttr: String
		//		Searches pattern match against this field

		// labelAttr: String
		//		Optional.  The text that actually appears in the drop down.
		//		If not specified, the searchAttr text is used instead.
		labelAttr: "",

		// labelType: String
		//		"html" or "text"
		labelType: "text",

		_isvalid:true,

		isValid:function(){
			return this._isvalid;
		},

		_callbackSetLabel: function(/*Array*/ result, /*Object*/ dataObject, /*Boolean, optional*/ priorityChange){
			// summary
			//	Callback function that dynamically sets the label of the ComboBox

			// setValue does a synchronous lookup,
			// so it calls _callbackSetLabel directly,
			// and so does not pass dataObject
			// dataObject==null means do not test the lastQuery, just continue
			if(dataObject&&dataObject.query[this.searchAttr]!=this._lastQuery){return;}
			if(!result.length){
				//#3268: do nothing on bad input
				//this._setValue("", "");
				//#3285: change CSS to indicate error
				if(!this._hasFocus){ this.valueNode.value=""; }
				dijit.form.TextBox.superclass.setValue.call(this, undefined, !this._hasFocus);
				this._isvalid=false;
				this.validate(this._hasFocus);
			}else{
				this._setValueFromItem(result[0], priorityChange);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ dataObject){
			// #3285: tap into search callback to see if user's query resembles a match
			if(dataObject.query[this.searchAttr]!=this._lastQuery){return;}
			this._isvalid=results.length!=0;
			this.validate(true);
			dijit.form.ComboBoxMixin.prototype._openResultList.apply(this, arguments);
		},

		getValue:function(){
			// don't get the textbox value but rather the previously set hidden value
			return this.valueNode.value;
		},

		_getValueField:function(){
			// used for option tag selects
			return "value";
		},

		_setValue:function(/*String*/ value, /*String*/ displayedValue, /*Boolean, optional*/ priorityChange){
			this.valueNode.value = value;
			dijit.form.FilteringSelect.superclass.setValue.call(this, value, priorityChange, displayedValue);
			this._lastDisplayedValue = displayedValue;
		},

		setValue: function(/*String*/ value, /*Boolean, optional*/ priorityChange){
			// summary
			//	Sets the value of the select.
			//	Also sets the label to the corresponding value by reverse lookup.

			//#3347: fetchItemByIdentity if no keyAttr specified
			var self=this;
			var handleFetchByIdentity = function(item, priorityChange){
				if(item){
					if(self.store.isItemLoaded(item)){
						self._callbackSetLabel([item], undefined, priorityChange);
					}else{
						self.store.loadItem({item:item, onItem: function(result, dataObject){self._callbackSetLabel(result, dataObject, priorityChange)}});
					}
				}else{
					self._isvalid=false;
					// prevent errors from Tooltip not being created yet
					self.validate(false);
				}
			}
			this.store.fetchItemByIdentity({identity: value, onItem: function(item){handleFetchByIdentity(item, priorityChange)}});
		},

		_setValueFromItem: function(/*item*/ item, /*Boolean, optional*/ priorityChange){
			// summary
			//	Set the displayed valued in the input box, based on a selected item.
			//	Users shouldn't call this function; they should be calling setDisplayedValue() instead
			this._isvalid=true;
			this._setValue(this.store.getIdentity(item), this.labelFunc(item, this.store), priorityChange);
		},

		labelFunc: function(/*item*/ item, /*dojo.data.store*/ store){
			// summary: Event handler called when the label changes
			// returns the label that the ComboBox should display
			return store.getValue(item, this.searchAttr);
		},

		onkeyup: function(/*Event*/ evt){
			// summary: internal function
			// FilteringSelect needs to wait for the complete label before committing to a reverse lookup
			//this.setDisplayedValue(this.textbox.value);
		},

		_doSelect: function(/*Event*/ tgt){
			// summary:
			//	ComboBox's menu callback function
			//	FilteringSelect overrides this to set both the visible and hidden value from the information stored in the menu
			this.item = tgt.item;
			this._setValueFromItem(tgt.item, true);
		},

		setDisplayedValue:function(/*String*/ label){
			// summary:
			//	Set textbox to display label
			//	Also performs reverse lookup to set the hidden value
			//	Used in InlineEditBox

			if(this.store){
				var query={};
				this._lastQuery=query[this.searchAttr]=label;
				// if the label is not valid, the callback will never set it,
				// so the last valid value will get the warning textbox
				// set the textbox value now so that the impending warning will make sense to the user
				this.textbox.value=label;
				this._lastDisplayedValue=label;
				this.store.fetch({query:query, queryOptions:{ignoreCase:this.ignoreCase, deep:true}, onComplete: dojo.hitch(this, this._callbackSetLabel)});
			}
		},

		_getMenuLabelFromItem:function(/*Item*/ item){
			// internal function to help ComboBoxMenu figure out what to display
			if(this.labelAttr){return {html:this.labelType=="html", label:this.store.getValue(item, this.labelAttr)};}
			else{
				// because this function is called by ComboBoxMenu, this.inherited tries to find the superclass of ComboBoxMenu
				return dijit.form.ComboBoxMixin.prototype._getMenuLabelFromItem.apply(this, arguments);
			}
		},

		postMixInProperties: function(){
			dijit.form.ComboBoxMixin.prototype.postMixInProperties.apply(this, arguments);
			dijit.form.MappedTextBox.prototype.postMixInProperties.apply(this, arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit.layout.LayoutContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.LayoutContainer"] = true;
dojo.provide("dijit.layout.LayoutContainer");



dojo.declare(
	"dijit.layout.LayoutContainer",
	dijit.layout._LayoutWidget,
{
	// summary
	//	Provides Delphi-style panel layout semantics.
	//
	// details
	//	A LayoutContainer is a box with a specified size (like style="width: 500px; height: 500px;"),
	//	that contains children widgets marked with "layoutAlign" of "left", "right", "bottom", "top", and "client".
	//	It takes it's children marked as left/top/bottom/right, and lays them out along the edges of the box,
	//	and then it takes the child marked "client" and puts it into the remaining space in the middle.
	//
	//  Left/right positioning is similar to CSS's "float: left" and "float: right",
	//	and top/bottom positioning would be similar to "float: top" and "float: bottom", if there were such
	//	CSS.
	//
	//	Note that there can only be one client element, but there can be multiple left, right, top,
	//	or bottom elements.
	//
	// usage
	//	<style>
	//		html, body{ height: 100%; width: 100%; }
	//	</style>
	//	<div dojoType="dijit.layout.LayoutContainer" style="width: 100%; height: 100%">
	//		<div dojoType="dijit.layout.ContentPane" layoutAlign="top">header text</div>
	//		<div dojoType="dijit.layout.ContentPane" layoutAlign="left" style="width: 200px;">table of contents</div>
	//		<div dojoType="dijit.layout.ContentPane" layoutAlign="client">client area</div>
	//	</div>
	//
	//	Lays out each child in the natural order the children occur in.
	//	Basically each child is laid out into the "remaining space", where "remaining space" is initially
	//	the content area of this widget, but is reduced to a smaller rectangle each time a child is added.
	//	

	layout: function(){
		dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
	},

	addChild: function(/*Widget*/ child, /*Integer?*/ insertIndex){
		dijit._Container.prototype.addChild.apply(this, arguments);
		if(this._started){
			dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
		}
	},

	removeChild: function(/*Widget*/ widget){
		dijit._Container.prototype.removeChild.apply(this, arguments);
		if(this._started){
			dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
		}
	}
});

// This argument can be specified for the children of a LayoutContainer.
// Since any widget can be specified as a LayoutContainer child, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// layoutAlign: String
	//		"none", "left", "right", "bottom", "top", and "client".
	//		See the LayoutContainer description for details on this parameter.
	layoutAlign: 'none'
});

}

if(!dojo._hasResource["dijit.layout.StackContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.StackContainer"] = true;
dojo.provide("dijit.layout.StackContainer");





dojo.declare(
	"dijit.layout.StackContainer",
	dijit.layout._LayoutWidget,

	// summary
	//	A container that has multiple children, but shows only
	//	one child at a time (like looking at the pages in a book one by one).
	//
	//	Publishes topics <widgetId>-addChild, <widgetId>-removeChild, and <widgetId>-selectChild
	//
	//	Can be base class for container, Wizard, Show, etc.
{
	// doLayout: Boolean
	//  if true, change the size of my currently displayed child to match my size
	doLayout: true,

	_started: false,

	// selectedChildWidget: Widget
	//	References the currently selected child widget, if any

	postCreate: function(){
		dijit.setWaiRole((this.containerNode || this.domNode), "tabpanel");
		this.connect(this.domNode, "onkeypress", this._onKeyPress);
	},
	
	startup: function(){
		if(this._started){ return; }

		var children = this.getChildren();

		// Setup each page panel
		dojo.forEach(children, this._setupChild, this);

		// Figure out which child to initially display
		dojo.some(children, function(child){
			if(child.selected){
				this.selectedChildWidget = child;
			}
			return child.selected;
		}, this);

		var selected = this.selectedChildWidget;

		// Default to the first child
		if(!selected && children[0]){
			selected = this.selectedChildWidget = children[0];
			selected.selected = true;
		}
		if(selected){
			this._showChild(selected);
		}

		// Now publish information about myself so any StackControllers can initialize..
		dojo.publish(this.id+"-startup", [{children: children, selected: selected}]);
		this.inherited("startup",arguments);
		this._started = true;
	},

	_setupChild: function(/*Widget*/ page){
		// Summary: prepare the given child

		page.domNode.style.display = "none";

		// since we are setting the width/height of the child elements, they need
		// to be position:relative, or IE has problems (See bug #2033)
		page.domNode.style.position = "relative";

		return page; // dijit._Widget
	},

	addChild: function(/*Widget*/ child, /*Integer?*/ insertIndex){
		// summary: Adds a widget to the stack
		 
		dijit._Container.prototype.addChild.apply(this, arguments);
		child = this._setupChild(child);

		if(this._started){
			// in case the tab titles have overflowed from one line to two lines
			this.layout();

			dojo.publish(this.id+"-addChild", [child, insertIndex]);

			// if this is the first child, then select it
			if(!this.selectedChildWidget){
				this.selectChild(child);
			}
		}
	},

	removeChild: function(/*Widget*/ page){
		// summary: Removes the pane from the stack

		dijit._Container.prototype.removeChild.apply(this, arguments);

		// If we are being destroyed than don't run the code below (to select another page), because we are deleting
		// every page one by one
		if(this._beingDestroyed){ return; }

		if(this._started){
			// this will notify any tablists to remove a button; do this first because it may affect sizing
			dojo.publish(this.id+"-removeChild", [page]);

			// in case the tab titles now take up one line instead of two lines
			this.layout();
		}

		if(this.selectedChildWidget === page){
			this.selectedChildWidget = undefined;
			if(this._started){
				var children = this.getChildren();
				if(children.length){
					this.selectChild(children[0]);
				}
			}
		}
	},

	selectChild: function(/*Widget*/ page){
		// summary:
		//	Show the given widget (which must be one of my children)

		page = dijit.byId(page);

		if(this.selectedChildWidget != page){
			// Deselect old page and select new one
			this._transition(page, this.selectedChildWidget);
			this.selectedChildWidget = page;
			dojo.publish(this.id+"-selectChild", [page]);
		}
	},

	_transition: function(/*Widget*/newWidget, /*Widget*/oldWidget){
		if(oldWidget){
			this._hideChild(oldWidget);
		}
		this._showChild(newWidget);

		// Size the new widget, in case this is the first time it's being shown,
		// or I have been resized since the last time it was shown.
		// page must be visible for resizing to work
		if(this.doLayout && newWidget.resize){
			newWidget.resize(this._containerContentBox || this._contentBox);
		}
	},

	_adjacent: function(/*Boolean*/ forward){
		// summary: Gets the next/previous child widget in this container from the current selection
		var children = this.getChildren();
		var index = dojo.indexOf(children, this.selectedChildWidget);
		index += forward ? 1 : children.length - 1;
		return children[ index % children.length ]; // dijit._Widget
	},

	forward: function(){
		// Summary: advance to next page
		this.selectChild(this._adjacent(true));
	},

	back: function(){
		// Summary: go back to previous page
		this.selectChild(this._adjacent(false));
	},

	_onKeyPress: function(e){
		dojo.publish(this.id+"-containerKeyPress", [{ e: e, page: this}]);
	},

	layout: function(){
		if(this.doLayout && this.selectedChildWidget && this.selectedChildWidget.resize){
			this.selectedChildWidget.resize(this._contentBox);
		}
	},

	_showChild: function(/*Widget*/ page){
		var children = this.getChildren();
		page.isFirstChild = (page == children[0]);
		page.isLastChild = (page == children[children.length-1]);
		page.selected = true;

		page.domNode.style.display="";
		if(page._loadCheck){
			page._loadCheck(); // trigger load in ContentPane
		}
		if(page.onShow){
			page.onShow();
		}
	},

	_hideChild: function(/*Widget*/ page){
		page.selected=false;
		page.domNode.style.display="none";
		if(page.onHide){
			page.onHide();
		}
	},

	closeChild: function(/*Widget*/ page){
		// summary
		//	callback when user clicks the [X] to remove a page
		//	if onClose() returns true then remove and destroy the childd
		var remove = page.onClose(this, page);
		if(remove){
			this.removeChild(page);
			// makes sure we can clean up executeScripts in ContentPane onUnLoad
			page.destroy();
		}
	},

	destroy: function(){
		this._beingDestroyed = true;
		this.inherited("destroy",arguments);
	}
});

dojo.declare(
	"dijit.layout.StackController",
	[dijit._Widget, dijit._Templated, dijit._Container],
	{
		// summary:
		//	Set of buttons to select a page in a page list.
		//	Monitors the specified StackContainer, and whenever a page is
		//	added, deleted, or selected, updates itself accordingly.

		templateString: "<span wairole='tablist' dojoAttachEvent='onkeypress' class='dijitStackController'></span>",

		// containerId: String
		//	the id of the page container that I point to
		containerId: "",

		// buttonWidget: String
		//	the name of the button widget to create to correspond to each page
		buttonWidget: "dijit.layout._StackButton",

		postCreate: function(){
			dijit.setWaiRole(this.domNode, "tablist");

			this.pane2button = {};		// mapping from panes to buttons
			this._subscriptions=[
				dojo.subscribe(this.containerId+"-startup", this, "onStartup"),
				dojo.subscribe(this.containerId+"-addChild", this, "onAddChild"),
				dojo.subscribe(this.containerId+"-removeChild", this, "onRemoveChild"),
				dojo.subscribe(this.containerId+"-selectChild", this, "onSelectChild"),
				dojo.subscribe(this.containerId+"-containerKeyPress", this, "onContainerKeyPress")
			];
		},

		onStartup: function(/*Object*/ info){
			// summary: called after StackContainer has finished initializing
			dojo.forEach(info.children, this.onAddChild, this);
			this.onSelectChild(info.selected);
		},

		destroy: function(){
			dojo.forEach(this._subscriptions, dojo.unsubscribe);
			this.inherited("destroy",arguments);
		},

		onAddChild: function(/*Widget*/ page, /*Integer?*/ insertIndex){
			// summary:
			//   Called whenever a page is added to the container.
			//   Create button corresponding to the page.

			// add a node that will be promoted to the button widget
			var refNode = document.createElement("span");
			this.domNode.appendChild(refNode);
			// create an instance of the button widget
			var cls = dojo.getObject(this.buttonWidget);
			var button = new cls({label: page.title, closeButton: page.closable}, refNode);
			this.addChild(button, insertIndex);
			this.pane2button[page] = button;
			page.controlButton = button;	// this value might be overwritten if two tabs point to same container
			
			dojo.connect(button, "onClick", dojo.hitch(this,"onButtonClick",page));
			dojo.connect(button, "onClickCloseButton", dojo.hitch(this,"onCloseButtonClick",page));
			
			if(!this._currentChild){ // put the first child into the tab order
				button.focusNode.setAttribute("tabIndex","0");
				this._currentChild = page;
			}
		},

		onRemoveChild: function(/*Widget*/ page){
			// summary:
			//   Called whenever a page is removed from the container.
			//   Remove the button corresponding to the page.
			if(this._currentChild === page){ this._currentChild = null; }
			var button = this.pane2button[page];
			if(button){
				// TODO? if current child { reassign }
				button.destroy();
			}
			this.pane2button[page] = null;
		},

		onSelectChild: function(/*Widget*/ page){
			// summary:
			//	Called when a page has been selected in the StackContainer, either by me or by another StackController

			if(!page){ return; }

			if(this._currentChild){
				var oldButton=this.pane2button[this._currentChild];
				oldButton.setChecked(false);
				oldButton.focusNode.setAttribute("tabIndex", "-1");
			}

			var newButton=this.pane2button[page];
			newButton.setChecked(true);
			this._currentChild = page;
			newButton.focusNode.setAttribute("tabIndex", "0");
		},

		onButtonClick: function(/*Widget*/ page){
			// summary:
			//   Called whenever one of my child buttons is pressed in an attempt to select a page
			var container = dijit.byId(this.containerId);	// TODO: do this via topics?
			container.selectChild(page); 
		},

		onCloseButtonClick: function(/*Widget*/ page){
			// summary:
			//   Called whenever one of my child buttons [X] is pressed in an attempt to close a page
			var container = dijit.byId(this.containerId);
			container.closeChild(page);
			var b = this.pane2button[this._currentChild];
			if(b){
				dijit.focus(b.focusNode || b.domNode);
			}
		},
		
		// TODO: this is a bit redundant with forward, back api in StackContainer
		adjacent: function(/*Boolean*/ forward){
			// find currently focused button in children array
			var children = this.getChildren();
			var current = dojo.indexOf(children, this.pane2button[this._currentChild]);
			// pick next button to focus on
			var offset = forward ? 1 : children.length - 1;
			return children[ (current + offset) % children.length ]; // dijit._Widget
		},

		onkeypress: function(/*Event*/ e){
			// summary:
			//   Handle keystrokes on the page list, for advancing to next/previous button
			//   and closing the current page if the page is closable.

			if(this.disabled || e.altKey ){ return; }
			var forward = true;
			if(e.ctrlKey || !e._djpage){
				var k = dojo.keys;
				switch(e.keyCode){
					case k.LEFT_ARROW:
					case k.UP_ARROW:
					case k.PAGE_UP:
						forward = false;
						// fall through
					case k.RIGHT_ARROW:
					case k.DOWN_ARROW:
					case k.PAGE_DOWN:
						this.adjacent(forward).onClick();
						dojo.stopEvent(e);
						break;
					case k.DELETE:
						if(this._currentChild.closable){
							this.onCloseButtonClick(this._currentChild);
						}
						dojo.stopEvent(e);
						break;
					default:
						if(e.ctrlKey){
							if(e.keyCode == k.TAB){
								this.adjacent(!e.shiftKey).onClick();
								dojo.stopEvent(e);
							}else if(e.keyChar == "w"){
								if(this._currentChild.closable){
									this.onCloseButtonClick(this._currentChild);
								}
								dojo.stopEvent(e); // avoid browser tab closing.
							}
						}
				}
			}
		},

		onContainerKeyPress: function(/*Object*/ info){
			info.e._djpage = info.page;
			this.onkeypress(info.e);
		}
});

dojo.declare("dijit.layout._StackButton",
	dijit.form.ToggleButton,
	{
	// summary
	//	Internal widget used by StackContainer.
	//	The button-like or tab-like object you click to select or delete a page
	
	tabIndex: "-1", // StackContainer buttons are not in the tab order by default
	
	postCreate: function(/*Event*/ evt){
		dijit.setWaiRole((this.focusNode || this.domNode), "tab");
		this.inherited("postCreate", arguments);
	},
	
	onClick: function(/*Event*/ evt){
		// summary: This is for TabContainer where the tabs are <span> rather than button,
		// 	so need to set focus explicitly (on some browsers)
		dijit.focus(this.focusNode);

		// ... now let StackController catch the event and tell me what to do
	},

	onClickCloseButton: function(/*Event*/ evt){
		// summary
		//	StackContainer connects to this function; if your widget contains a close button
		//	then clicking it should call this function.
		evt.stopPropagation();
	}
});

// These arguments can be specified for the children of a StackContainer.
// Since any widget can be specified as a StackContainer child, mix them
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// title: String
	//		Title of this widget.  Used by TabContainer to the name the tab, etc.
	title: "",

	// selected: Boolean
	//		Is this child currently selected?
	selected: false,

	// closable: Boolean
	//		True if user can close (destroy) this child, such as (for example) clicking the X on the tab.
	closable: false,	// true if user can close this tab pane

	onClose: function(){
		// summary: Callback if someone tries to close the child, child will be closed if func returns true
		return true;
	}
});

}

if(!dojo._hasResource["dijit.layout.TabContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.TabContainer"] = true;
dojo.provide("dijit.layout.TabContainer");




dojo.declare("dijit.layout.TabContainer",
	[dijit.layout.StackContainer, dijit._Templated],
	{	
	// summary: 
	//	A Container with Title Tabs, each one pointing at a pane in the container.
	// description:
	//	A TabContainer is a container that has multiple panes, but shows only
	//	one pane at a time.  There are a set of tabs corresponding to each pane,
	//	where each tab has the title (aka title) of the pane, and optionally a close button.
	//
	//	Publishes topics <widgetId>-addChild, <widgetId>-removeChild, and <widgetId>-selectChild
	//	(where <widgetId> is the id of the TabContainer itself.
	//
	// tabPosition: String
	//   Defines where tabs go relative to tab content.
	//   "top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	templateString: null,	// override setting in StackContainer
	templateString:"<div class=\"dijitTabContainer\">\r\n\t<div dojoAttachPoint=\"tablistNode\"></div>\r\n\t<div class=\"dijitTabPaneWrapper\" dojoAttachPoint=\"containerNode\"></div>\r\n</div>\r\n",

	postCreate: function(){	
		dijit.layout.TabContainer.superclass.postCreate.apply(this, arguments);
		// create the tab list that will have a tab (a.k.a. tab button) for each tab panel
		this.tablist = new dijit.layout.TabController(
			{
				id: this.id + "_tablist",
				tabPosition: this.tabPosition,
				doLayout: this.doLayout,
				containerId: this.id
			}, this.tablistNode);		
	},

	_setupChild: function(/* Widget */tab){
		dojo.addClass(tab.domNode, "dijitTabPane");
		this.inherited("_setupChild",arguments);
		return tab; // Widget
	},

	startup: function(){
		if(this._started){ return; }

		// wire up the tablist and its tabs
		this.tablist.startup();
		this.inherited("startup",arguments);

		if(dojo.isSafari){
			// sometimes safari 3.0.3 miscalculates the height of the tab labels, see #4058
			setTimeout(dojo.hitch(this, "layout"), 0);
		}
	},

	layout: function(){
		// Summary: Configure the content pane to take up all the space except for where the tabs are
		if(!this.doLayout){ return; }

		// position and size the titles and the container node
		var titleAlign=this.tabPosition.replace(/-h/,"");
		var children = [
			{domNode: this.tablist.domNode, layoutAlign: titleAlign},
			{domNode: this.containerNode, layoutAlign: "client"}
		];
		dijit.layout.layoutChildren(this.domNode, this._contentBox, children);

		// Compute size to make each of my children.
		// children[1] is the margin-box size of this.containerNode, set by layoutChildren() call above
		this._containerContentBox = dijit.layout.marginBox2contentBox(this.containerNode, children[1]);

		if(this.selectedChildWidget){
			this._showChild(this.selectedChildWidget);
			if(this.doLayout && this.selectedChildWidget.resize){
				this.selectedChildWidget.resize(this._containerContentBox);
			}
		}
	},

	destroy: function(){
		this.tablist.destroy();
		this.inherited("destroy",arguments);
	}
});

//TODO: make private?
dojo.declare("dijit.layout.TabController",
	dijit.layout.StackController,
	{
	// summary:
	// 	Set of tabs (the things with titles and a close button, that you click to show a tab panel).
	// description:
	//	Lets the user select the currently shown pane in a TabContainer or StackContainer.
	//	TabController also monitors the TabContainer, and whenever a pane is
	//	added or deleted updates itself accordingly.

	templateString: "<div wairole='tablist' dojoAttachEvent='onkeypress:onkeypress'></div>",

	// tabPosition: String
	//   Defines where tabs go relative to the content.
	//   "top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	// doLayout: Boolean
	// 	TODOC: deprecate doLayout? not sure.
	doLayout: true,

	// buttonWidget: String
	//	the name of the tab widget to create to correspond to each page
	buttonWidget: "dijit.layout._TabButton",

	postMixInProperties: function(){
		this["class"] = "dijitTabLabels-" + this.tabPosition + (this.doLayout ? "" : " dijitTabNoLayout");
		this.inherited("postMixInProperties",arguments);
	}
});

dojo.declare("dijit.layout._TabButton",
	dijit.layout._StackButton,
	{
	// summary:
	//	A tab (the thing you click to select a pane).
	// description:
	//	Contains the title of the pane, and optionally a close-button to destroy the pane.
	//	This is an internal widget and should not be instantiated directly.

	baseClass: "dijitTab",

	templateString:"<div dojoAttachEvent='onclick:onClick,onmouseenter:_onMouse,onmouseleave:_onMouse'>\r\n    <div class='dijitTabInnerDiv' dojoAttachPoint='innerDiv'>\r\n        <span dojoAttachPoint='containerNode,focusNode'>${!label}</span>\r\n        <span dojoAttachPoint='closeButtonNode' class='closeImage' dojoAttachEvent='onmouseenter:_onMouse, onmouseleave:_onMouse, onclick:onClickCloseButton' stateModifier='CloseButton'>\r\n            <span dojoAttachPoint='closeText' class='closeText'>x</span>\r\n        </span>\r\n    </div>\r\n</div>\r\n",

	postCreate: function(){
		if(this.closeButton){
			dojo.addClass(this.innerDiv, "dijitClosable");
		} else {
			this.closeButtonNode.style.display="none";
		}
		this.inherited("postCreate",arguments); 
		dojo.setSelectable(this.containerNode, false);
	}
});

}

if(!dojo._hasResource["dijit.TitlePane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.TitlePane"] = true;
dojo.provide("dijit.TitlePane");






dojo.declare(
	"dijit.TitlePane",
	[dijit.layout.ContentPane, dijit._Templated],
{
	// summary
	//		A pane with a title on top, that can be opened or collapsed.
	//
	// title: String
	//		Title of the pane
	title: "",

	// open: Boolean
	//		Whether pane is opened or closed.
	open: true,

	// duration: Integer
	//		Time in milliseconds to fade in/fade out
	duration: 250,

	// baseClass: String
	//	the root className to use for the various states of this widget
	baseClass: "dijitTitlePane",

	templateString:"<div class=\"dijitTitlePane\">\r\n\t<div dojoAttachEvent=\"onclick:toggle,onkeypress: _onTitleKey,onfocus:_handleFocus,onblur:_handleFocus\" tabindex=\"0\"\r\n\t\t\twaiRole=\"button\" class=\"dijitTitlePaneTitle\" dojoAttachPoint=\"focusNode\">\r\n\t\t<div dojoAttachPoint=\"arrowNode\" class=\"dijitInline dijitArrowNode\"><span dojoAttachPoint=\"arrowNodeInner\" class=\"dijitArrowNodeInner\"></span></div>\r\n\t\t<div dojoAttachPoint=\"titleNode\" class=\"dijitTitlePaneTextNode\"></div>\r\n\t</div>\r\n\t<div class=\"dijitTitlePaneContentOuter\" dojoAttachPoint=\"hideNode\">\r\n\t\t<div class=\"dijitReset\" dojoAttachPoint=\"wipeNode\">\r\n\t\t\t<div class=\"dijitTitlePaneContentInner\" dojoAttachPoint=\"containerNode\" waiRole=\"region\" tabindex=\"-1\">\r\n\t\t\t\t<!-- nested divs because wipeIn()/wipeOut() doesn't work right on node w/padding etc.  Put padding on inner div. -->\r\n\t\t\t</div>\r\n\t\t</div>\r\n\t</div>\r\n</div>\r\n",

	postCreate: function(){
		this.setTitle(this.title);
		if(!this.open){
			this.hideNode.style.display = this.wipeNode.style.display = "none";
		}
		this._setCss();
		dojo.setSelectable(this.titleNode, false);
		this.inherited("postCreate",arguments);
		dijit.setWaiState(this.containerNode, "labelledby", this.titleNode.id);
		dijit.setWaiState(this.focusNode, "haspopup", "true");

		// setup open/close animations
		var hideNode = this.hideNode, wipeNode = this.wipeNode;
		this._wipeIn = dojo.fx.wipeIn({
			node: this.wipeNode,
			duration: this.duration,
			beforeBegin: function(){
				hideNode.style.display="";
			}
		});
		this._wipeOut = dojo.fx.wipeOut({
			node: this.wipeNode,
			duration: this.duration,
			onEnd: function(){
				hideNode.style.display="none";
			}
		});
	},

	setContent: function(content){
		// summary
		// 		Typically called when an href is loaded.  Our job is to make the animation smooth
		if(this._wipeOut.status() == "playing"){
			// we are currently *closing* the pane, so just let that continue
			this.inherited("setContent",arguments);
		}else{
			if(this._wipeIn.status() == "playing"){
				this._wipeIn.stop();
			}

			// freeze container at current height so that adding new content doesn't make it jump
			dojo.marginBox(this.wipeNode, {h: dojo.marginBox(this.wipeNode).h});

			// add the new content (erasing the old content, if any)
			this.inherited("setContent",arguments);

			// call _wipeIn.play() to animate from current height to new height
			this._wipeIn.play();
		}
	},

	toggle: function(){
		// summary: switches between opened and closed state
		dojo.forEach([this._wipeIn, this._wipeOut], function(animation){
			if(animation.status() == "playing"){
				animation.stop();
			}
		});

		this[this.open ? "_wipeOut" : "_wipeIn"].play();
		this.open =! this.open;

		// load content (if this is the first time we are opening the TitlePane
		// and content is specified as an href, or we have setHref when hidden)
		this._loadCheck();

		this._setCss();
	},

	_setCss: function(){
		// summary: set the open/close css state for the TitlePane
		var classes = ["dijitClosed", "dijitOpen"];
		var boolIndex = this.open;
		dojo.removeClass(this.focusNode, classes[!boolIndex+0]);
		this.focusNode.className += " " + classes[boolIndex+0];

		// provide a character based indicator for images-off mode
		this.arrowNodeInner.innerHTML = this.open ? "-" : "+";
	},

	_onTitleKey: function(/*Event*/ e){
		// summary: callback when user hits a key
		if(e.keyCode == dojo.keys.ENTER || e.charCode == dojo.keys.SPACE){
			this.toggle();
		}
		else if(e.keyCode == dojo.keys.DOWN_ARROW){
			if(this.open){
				this.containerNode.focus();
				e.preventDefault();
			}
	 	}
	},
	
	_handleFocus: function(/*Event*/ e){
		// summary: handle blur and focus for this widget
		
		// add/removeClass is safe to call without hasClass in this case
		dojo[(e.type=="focus" ? "addClass" : "removeClass")](this.focusNode,this.baseClass+"Focused");
	},

	setTitle: function(/*String*/ title){
		// summary: sets the text of the title
		this.titleNode.innerHTML=title;
	}
});

}

if(!dojo._hasResource["dojox.collections._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.collections._base"] = true;
dojo.provide("dojox.collections._base");

dojox.collections.DictionaryEntry=function(/* string */k, /* object */v){
	//	summary
	//	return an object of type dojox.collections.DictionaryEntry
	this.key=k;
	this.value=v;
	this.valueOf=function(){ 
		return this.value; 	//	object
	};
	this.toString=function(){ 
		return String(this.value);	//	string 
	};
}

/*	Iterators
 *	The collections.Iterators (Iterator and DictionaryIterator) are built to
 *	work with the Collections included in this module.  However, they *can*
 *	be used with arrays and objects, respectively, should one choose to do so.
 */
dojox.collections.Iterator=function(/* array */arr){
	//	summary
	//	return an object of type dojox.collections.Iterator
	var a=arr;
	var position=0;
	this.element=a[position]||null;
	this.atEnd=function(){
		//	summary
		//	Test to see if the internal cursor has reached the end of the internal collection.
		return (position>=a.length);	//	bool
	};
	this.get=function(){
		//	summary
		//	Get the next member in the collection.
		if(this.atEnd()){
			return null;		//	object
		}
		this.element=a[position++];
		return this.element;	//	object
	};
	this.map=function(/* function */fn, /* object? */scope){
		//	summary
		//	Functional iteration with optional scope.
		return dojo.map(a, fn, scope);
	};
	this.reset=function(){
		//	summary
		//	reset the internal cursor.
		position=0;
		this.element=a[position];
	};
}

/*	Notes:
 *	The DictionaryIterator no longer supports a key and value property;
 *	the reality is that you can use this to iterate over a JS object
 *	being used as a hashtable.
 */
dojox.collections.DictionaryIterator=function(/* object */obj){
	//	summary
	//	return an object of type dojox.collections.DictionaryIterator
	var a=[];	//	Create an indexing array
	var testObject={};
	for(var p in obj){
		if(!testObject[p]){
			a.push(obj[p]);	//	fill it up
		}
	}
	var position=0;
	this.element=a[position]||null;
	this.atEnd=function(){
		//	summary
		//	Test to see if the internal cursor has reached the end of the internal collection.
		return (position>=a.length);	//	bool
	};
	this.get=function(){
		//	summary
		//	Get the next member in the collection.
		if(this.atEnd()){
			return null;		//	object
		}
		this.element=a[position++];
		return this.element;	//	object
	};
	this.map=function(/* function */fn, /* object? */scope){
		//	summary
		//	Functional iteration with optional scope.
		return dojo.map(a, fn, scope);
	};
	this.reset=function() { 
		//	summary
		//	reset the internal cursor.
		position=0; 
		this.element=a[position];
	};
};

}

if(!dojo._hasResource["dojox.collections.Dictionary"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.collections.Dictionary"] = true;
dojo.provide("dojox.collections.Dictionary");


dojox.collections.Dictionary=function(/* dojox.collections.Dictionary? */dictionary){
	//	summary
	//	Returns an object of type dojox.collections.Dictionary
	var items={};
	this.count=0;

	//	comparator for property addition and access.
	var testObject={};

	this.add=function(/* string */k, /* object */v){
		//	summary
		//	Add a new item to the Dictionary.
		var b=(k in items);
		items[k]=new dojox.collections.DictionaryEntry(k,v);
		if(!b){
			this.count++;
		}
	};
	this.clear=function(){
		//	summary
		//	Clears the internal dictionary.
		items={};
		this.count=0;
	};
	this.clone=function(){
		//	summary
		//	Returns a new instance of dojox.collections.Dictionary; note the the dictionary is a clone but items might not be.
		return new dojox.collections.Dictionary(this);	//	dojox.collections.Dictionary
	};
	this.contains=this.containsKey=function(/* string */k){
		//	summary
		//	Check to see if the dictionary has an entry at key "k".
		if(testObject[k]){
			return false;			// bool
		}
		return (items[k]!=null);	//	bool
	};
	this.containsValue=function(/* object */v){
		//	summary
		//	Check to see if the dictionary has an entry with value "v".
		var e=this.getIterator();
		while(e.get()){
			if(e.element.value==v){
				return true;	//	bool
			}
		}
		return false;	//	bool
	};
	this.entry=function(/* string */k){
		//	summary
		//	Accessor method; similar to dojox.collections.Dictionary.item but returns the actual Entry object.
		return items[k];	//	dojox.collections.DictionaryEntry
	};
	this.forEach=function(/* function */ fn, /* object? */ scope){
		//	summary
		//	functional iterator, following the mozilla spec.
		var a=[];	//	Create an indexing array
		for(var p in items) {
			if(!testObject[p]){
				a.push(items[p]);	//	fill it up
			}
		}
		dojo.forEach(a, fn, scope);
	};
	this.getKeyList=function(){
		//	summary
		//	Returns an array of the keys in the dictionary.
		return (this.getIterator()).map(function(entry){ 
			return entry.key; 
		});	//	array
	};
	this.getValueList=function(){
		//	summary
		//	Returns an array of the values in the dictionary.
		return (this.getIterator()).map(function(entry){ 
			return entry.value; 
		});	//	array
	};
	this.item=function(/* string */k){
		//	summary
		//	Accessor method.
		if(k in items){
			return items[k].valueOf();	//	object
		}
		return undefined;	//	object
	};
	this.getIterator=function(){
		//	summary
		//	Gets a dojox.collections.DictionaryIterator for iteration purposes.
		return new dojox.collections.DictionaryIterator(items);	//	dojox.collections.DictionaryIterator
	};
	this.remove=function(/* string */k){
		//	summary
		//	Removes the item at k from the internal collection.
		if(k in items && !testObject[k]){
			delete items[k];
			this.count--;
			return true;	//	bool
		}
		return false;	//	bool
	};

	if (dictionary){
		var e=dictionary.getIterator();
		while(e.get()) {
			 this.add(e.element.key, e.element.value);
		}
	}
};

}

if(!dojo._hasResource["dojox.data.dom"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.dom"] = true;
dojo.provide("dojox.data.dom");

//DOM type to int value for reference.
//Ints make for more compact code than full constant names.
//ELEMENT_NODE                  = 1;
//ATTRIBUTE_NODE                = 2;
//TEXT_NODE                     = 3;
//CDATA_SECTION_NODE            = 4;
//ENTITY_REFERENCE_NODE         = 5;
//ENTITY_NODE                   = 6;
//PROCESSING_INSTRUCTION_NODE   = 7;
//COMMENT_NODE                  = 8;
//DOCUMENT_NODE                 = 9;
//DOCUMENT_TYPE_NODE            = 10;
//DOCUMENT_FRAGMENT_NODE        = 11;
//NOTATION_NODE                 = 12;

//FIXME:  Remove this file when possible.
//This file contains internal/helper APIs as holders until the true DOM apis of Dojo 0.9 are finalized.
//Therefore, these should not be generally used, they are present only for the use by XmlStore and the
//wires project until proper dojo replacements are available.  When such exist, XmlStore and the like
//will be ported off these and this file will be deleted.
dojo.experimental("dojox.data.dom");

dojox.data.dom.createDocument = function(/*string?*/ str, /*string?*/ mimetype){
	//	summary:
	//		cross-browser implementation of creating an XML document object.
	//
	//	str:
	//		Optional text to create the document from.  If not provided, an empty XML document will be created.
	//	mimetype:
	//		Optional mimetype of the text.  Typically, this is text/xml.  Will be defaulted to text/xml if not provided.
	var _document = dojo.doc;

	if(!mimetype){ mimetype = "text/xml"; }
	if(str && (typeof dojo.global["DOMParser"]) !== "undefined"){
		var parser = new DOMParser();
		return parser.parseFromString(str, mimetype);	//	DOMDocument
	}else if((typeof dojo.global["ActiveXObject"]) !== "undefined"){
		var prefixes = [ "MSXML2", "Microsoft", "MSXML", "MSXML3" ];
		for(var i = 0; i<prefixes.length; i++){
			try{
				var doc = new ActiveXObject(prefixes[i]+".XMLDOM");
				if(str){
					if(doc){
						doc.async = false;
						doc.loadXML(str);
						return doc;	//	DOMDocument
					}else{
						console.log("loadXML didn't work?");
					}
				}else{
					if(doc){ 
						return doc; //DOMDocument
					}
				}
			}catch(e){ /* squelch */ };
		}
	}else if((_document.implementation)&&
		(_document.implementation.createDocument)){
		if(str){
			if(_document.createElement){
				// FIXME: this may change all tags to uppercase!
				var tmp = _document.createElement("xml");
				tmp.innerHTML = str;
				var xmlDoc = _document.implementation.createDocument("foo", "", null);
				for(var i = 0; i < tmp.childNodes.length; i++) {
					xmlDoc.importNode(tmp.childNodes.item(i), true);
				}
				return xmlDoc;	//	DOMDocument
			}
		}else{
			return _document.implementation.createDocument("", "", null); // DOMDocument
		}
	}
	return null;	//	DOMDocument
}

dojox.data.dom.textContent = function(/*Node*/node, /*string?*/text){
	//	summary:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//	description:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//		This function can also update the text of a node by replacing all child 
	//		content of the node.
	//	node:
	//		The node to get the text off of or set the text on.
	//	text:
	//		Optional argument of the text to apply to the node.
	if(arguments.length>1){
		var _document = node.ownerDocument || dojo.doc;  //Preference is to get the node owning doc first or it may fail
		dojox.data.dom.replaceChildren(node, _document.createTextNode(text));
		return text;	//	string
	} else {
		if(node.textContent !== undefined){ //FF 1.5
			return node.textContent;	//	string
		}
		var _result = "";
		if(node == null){
			return _result; //empty string.
		}
		for(var i = 0; i < node.childNodes.length; i++){
			switch(node.childNodes[i].nodeType){
				case 1: // ELEMENT_NODE
				case 5: // ENTITY_REFERENCE_NODE
					_result += dojox.data.dom.textContent(node.childNodes[i]);
					break;
				case 3: // TEXT_NODE
				case 2: // ATTRIBUTE_NODE
				case 4: // CDATA_SECTION_NODE
					_result += node.childNodes[i].nodeValue;
					break;
				default:
					break;
			}
		}
		return _result;	//	string
	}
}

dojox.data.dom.replaceChildren = function(/*Element*/node, /*Node || array*/ newChildren){
	//	summary:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	//	description:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	// 	node:
	//		The node to modify the children on
	//	newChildren:
	//		The children to add to the node.  It can either be a single Node or an
	//		array of Nodes.
	var nodes = [];
	
	if(dojo.isIE){
		for(var i=0;i<node.childNodes.length;i++){
			nodes.push(node.childNodes[i]);
		}
	}

	dojox.data.dom.removeChildren(node);
	for(var i=0;i<nodes.length;i++){
		dojo._destroyElement(nodes[i]);
	}

	if(!dojo.isArray(newChildren)){
		node.appendChild(newChildren);
	}else{
		for(var i=0;i<newChildren.length;i++){
			node.appendChild(newChildren[i]);
		}
	}
}

dojox.data.dom.removeChildren = function(/*Element*/node){
	//	summary:
	//		removes all children from node and returns the count of children removed.
	//		The children nodes are not destroyed. Be sure to call dojo._destroyElement on them
	//		after they are not used anymore.
	//	node:
	//		The node to remove all the children from.
	var count = node.childNodes.length;
	while(node.hasChildNodes()){
		node.removeChild(node.firstChild);
	}
	return count; // int
}


dojox.data.dom.innerXML = function(/*Node*/node){
	//	summary:
	//		Implementation of MS's innerXML function.
	//	node:
	//		The node from which to generate the XML text representation.
	if(node.innerXML){
		return node.innerXML;	//	string
	}else if (node.xml){
		return node.xml;		//	string
	}else if(typeof XMLSerializer != "undefined"){
		return (new XMLSerializer()).serializeToString(node);	//	string
	}
}


}

if(!dojo._hasResource["dojox.data.XmlStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.XmlStore"] = true;
dojo.provide("dojox.data.XmlStore");
dojo.provide("dojox.data.XmlItem");





dojo.declare("dojox.data.XmlStore", null, {
	//	summary:
	//		A data store for XML based services or documents
	//	description:
	//		A data store for XML based services or documents
	
	constructor: function(/* object */ args) {
		//	summary:
		//		Constructor for the XML store.  
		//	args:
		//		An anonymous object to initialize properties.  It expects the following values:
		//		url:		The url to a service or an XML document that represents the store
		//		rootItem:	A tag name for root items
		//		keyAttribute:	An attribute name for a key or an indentify
		// 		attributeMap:   An anonymous object contains properties for attribute mapping,
		//						{"tag_name.item_attribute_name": "@xml_attribute_name", ...}
		//		sendQuery:		A boolean indicate to add a query string to the service URL 
		console.log("XmlStore()");
		if(args){
			this.url = args.url;
			this.rootItem = (args.rootItem || args.rootitem || this.rootItem);
			this.keyAttribute = (args.keyAttribute || args.keyattribute || this.keyAttribute);
			this._attributeMap = (args.attributeMap || args.attributemap);
			this.label = args.label || this.label;
			this.sendQuery = (args.sendQuery || args.sendquery || this.sendQuery);
		}
		this._newItems = [];
		this._deletedItems = [];
		this._modifiedItems = [];
	},

	//Values that may be set by the parser.  
	//Ergo, have to be instantiated to something
	//So the parser knows how to set them.
	url: "",

	rootItem: "",

	keyAttribute: "",

	label: "",

	sendQuery: false,

/* dojo.data.api.Read */

	getValue: function(/* item */ item, /* attribute || attribute-name-string */ attribute, /* value? */ defaultValue){
		//	summary:
		//		Return an attribute value
		//	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		If 'attribute' specifies "tagName", the tag name of the element is
		//		returned.
		//		If 'attribute' specifies "childNodes", the first element child is
		//		returned.
		//		If 'attribute' specifies "text()", the value of the first text
		//		child is returned.
		//		For generic attributes, if '_attributeMap' is specified,
		//		an actual attribute name is looked up with the tag name of
		//		the element and 'attribute' (concatenated with '.').
		//		Then, if 'attribute' starts with "@", the value of the XML
		//		attribute is returned.
		//		Otherwise, the first child element of the tag name specified with
		//		'attribute' is returned.
		//	item:
		//		An XML element that holds the attribute
		//	attribute:
		//		A tag name of a child element, An XML attribute name or one of
		// 		special names
		//	defaultValue:
		//		A default value
		//	returns:
		//		An attribute value found, otherwise 'defaultValue'
		var element = item.element;
		if(attribute === "tagName"){
			return element.nodeName;
		}else if (attribute === "childNodes"){
			for (var i = 0; i < element.childNodes.length; i++) {
				var node = element.childNodes[i];
				if (node.nodeType === 1 /*ELEMENT_NODE*/) {
					return this._getItem(node); //object
				}
			}
			return defaultValue;
		}else if(attribute === "text()"){
			for(var i = 0; i < element.childNodes.length; i++){
				var node = element.childNodes[i];
				if(node.nodeType === 3 /*TEXT_NODE*/ ||
					node.nodeType === 4 /*CDATA_SECTION_NODE*/){
					return node.nodeValue; //string
				}
			}
			return defaultValue;
		}else{
			attribute = this._getAttribute(element.nodeName, attribute);
			if(attribute.charAt(0) === '@'){
				var name = attribute.substring(1);
				var value = element.getAttribute(name);
				return (value !== undefined) ? value : defaultValue; //object
			}else{
				for(var i = 0; i < element.childNodes.length; i++){
					var node = element.childNodes[i];
					if(	node.nodeType === 1 /*ELEMENT_NODE*/ &&
						node.nodeName === attribute){
						return this._getItem(node); //object
					}
				}
				return defaultValue; //object
			}
		}
	},

	getValues: function(/* item */ item, /* attribute || attribute-name-string */ attribute){
		//	summary:
		//		Return an array of attribute values
		//	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		If 'attribute' specifies "tagName", the tag name of the element is
		//		returned.
		//		If 'attribute' specifies "childNodes", child elements are returned.
		//		If 'attribute' specifies "text()", the values of child text nodes
		//		are returned.
		//		For generic attributes, if '_attributeMap' is specified,
		//		an actual attribute name is looked up with the tag name of
		//		the element and 'attribute' (concatenated with '.').
		//		Then, if 'attribute' starts with "@", the value of the XML
		//		attribute is returned.
		//		Otherwise, child elements of the tag name specified with
		//		'attribute' are returned.
		//	item:
		//		An XML element that holds the attribute
		//	attribute:
		//		A tag name of child elements, An XML attribute name or one of
		//		special names
		//	returns:
		//		An array of attribute values found, otherwise an empty array
		var element = item.element;
		if(attribute === "tagName"){
			return [element.nodeName];
		}else if(attribute === "childNodes"){
			var values = [];
			for(var i = 0; i < element.childNodes.length; i++){
				var node = element.childNodes[i];
				if(node.nodeType === 1 /*ELEMENT_NODE*/){
					values.push(this._getItem(node));
				}
			}
			return values; //array
		}else if(attribute === "text()"){
			var values = [];
			for(var i = 0; i < element.childNodes.length; i++){
				var node = childNodes[i];
				if(node.nodeType === 3){
					values.push(node.nodeValue);
				}
			}
			return values; //array
		}else{
			attribute = this._getAttribute(element.nodeName, attribute);
			if(attribute.charAt(0) === '@'){
				var name = attribute.substring(1);
				var value = element.getAttribute(name);
				return (value !== undefined) ? [value] : []; //array
			}else{
				var values = [];
				for(var i = 0; i < element.childNodes.length; i++){
					var node = element.childNodes[i];
					if(	node.nodeType === 1 /*ELEMENT_NODE*/ &&
						node.nodeName === attribute){
						values.push(this._getItem(node));
					}
				}
				return values; //array
			}
		}
	},

	getAttributes: function(/* item */ item) {
		//	summary:
		//		Return an array of attribute names
		// 	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		tag names of child elements and XML attribute names of attributes
		//		specified to the element are returned along with special attribute
		//		names applicable to the element including "tagName", "childNodes"
		//		if the element has child elements, "text()" if the element has
		//		child text nodes, and attribute names in '_attributeMap' that match
		//		the tag name of the element.
		//	item:
		//		An XML element
		//	returns:
		//		An array of attributes found
		var element = item.element;
		var attributes = [];
		attributes.push("tagName");
		if(element.childNodes.length > 0){
			var names = {};
			var childNodes = true;
			var text = false;
			for(var i = 0; i < element.childNodes.length; i++){
				var node = element.childNodes[i];
				if (node.nodeType === 1 /*ELEMENT_NODE*/) {
					var name = node.nodeName;
					if(!names[name]){
						attributes.push(name);
						names[name] = name;
					}
					childNodes = true;
				}else if(node.nodeType === 3){
					text = true;
				}
			}
			if(childNodes){
				attributes.push("childNodes");
			}
			if(text){
				attributes.push("text()");
			}
		}
		for(var i = 0; i < element.attributes.length; i++){
			attributes.push("@" + element.attributes[i].nodeName);
		}
		if(this._attributeMap){
			for (var key in this._attributeMap){
				var i = key.indexOf('.');
				if(i > 0){
					var tagName = key.substring(0, i);
					if (tagName === element.nodeName){
						attributes.push(key.substring(i + 1));
					}
				}else{ // global attribute
					attributes.push(key);
				}
			}
		}
		return attributes; //array
	},

	hasAttribute: function(/* item */ item, /* attribute || attribute-name-string */ attribute){
		//	summary:
		//		Check whether an element has the attribute
		//	item:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//	attribute:
		//		A tag name of a child element, An XML attribute name or one of
		//		special names
		//	returns:
		//		True if the element has the attribute, otherwise false
		return (this.getValue(item, attribute) !== undefined); //boolean
	},

	containsValue: function(/* item */ item, /* attribute || attribute-name-string */ attribute, /* anything */ value){
		//	summary:
		//		Check whether the attribute values contain the value
		//	item:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//	attribute:
		//		A tag name of a child element, An XML attribute name or one of
		//		special names
		//	returns:
		//		True if the attribute values contain the value, otherwise false
		var values = this.getValues(item, attribute);
		for(var i = 0; i < values.length; i++){
			if((typeof value === "string")){
				if(values[i].toString && values[i].toString() === value){
					return true;
				}
			}else if (values[i] === value){
				return true; //boolean
			}
		}
		return false;//boolean
	},

	isItem: function(/* anything */ something){
		//	summary:
		//		Check whether the object is an item (XML element)
		//	item:
		//		An object to check
		// 	returns:
		//		True if the object is an XML element, otherwise false
		if(something && something.element && something.store && something.store === this){
			return true; //boolean
		}
		return false; //boolran
	},

	isItemLoaded: function(/* anything */ something){
		//	summary:
		//		Check whether the object is an item (XML element) and loaded
		//	item:
		//		An object to check
		//	returns:
		//		True if the object is an XML element, otherwise false
		return this.isItem(something); //boolean
	},

	loadItem: function(/* object */ keywordArgs){
		//	summary:
		//		Load an item (XML element)
		//	keywordArgs:
		//		object containing the args for loadItem.  See dojo.data.api.Read.loadItem()
	},

	getFeatures: function() {
		//	summary:
		//		Return supported data APIs
		//	returns:
		//		"dojo.data.api.Read" and "dojo.data.api.Write"
		var features = {
			"dojo.data.api.Read": true,
			"dojo.data.api.Write": true
		};
		return features; //array
	},

	getLabel: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabel()
		if((this.label !== "") && this.isItem(item)){
			var label = this.getValue(item,this.label);
			if(label){
				return label.toString();
			}
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabelAttributes()
		if(this.label !== ""){
			return [this.label]; //array
		}
		return null; //null
	},

	_fetchItems: function(request, fetchHandler, errorHandler) {
		//	summary:
		//		Fetch items (XML elements) that match to a query
		//	description:
		//		If 'sendQuery' is true, an XML document is loaded from
		//		'url' with a query string.
		//		Otherwise, an XML document is loaded and list XML elements that
		//		match to a query (set of element names and their text attribute
		//		values that the items to contain).
		//		A wildcard, "*" can be used to query values to match all
		//		occurrences.
		//		If 'rootItem' is specified, it is used to fetch items.
		//	request:
		//		A request object
		//	fetchHandler:
		//		A function to call for fetched items
		//	errorHandler:
		//		A function to call on error
		var url = this._getFetchUrl(request);
		console.log("XmlStore._fetchItems(): url=" + url);
		if(!url){
			errorHandler(new Error("No URL specified."));
			return;
		}
		var localRequest = (!this.sendQuery ? request : null); // use request for _getItems()

		var self = this;
		var getArgs = {
				url: url,
                                // BEGIN FIX FOR: http://trac.dojotoolkit.org/ticket/5388
                                handleAs: (dojo.isIE ? "text" : "xml"),
                                // END FIX FOR: http://trac.dojotoolkit.org/ticket/5388
				preventCache: true
			};
		var getHandler = dojo.xhrGet(getArgs);
		getHandler.addCallback(function(data){
                        // BEGIN FIX FOR: http://trac.dojotoolkit.org/ticket/5388
 	  	        var document = data;
 	  	        if (dojo.isIE) {
 	  	            document = dojox.data.dom.createDocument(document);
 	  	        }
 	  	        var items = self._getItems(document, localRequest);
 	  	        // END FIX FOR: http://trac.dojotoolkit.org/ticket/5388
			console.log("XmlStore._fetchItems(): length=" + (items ? items.length : 0));
			if (items && items.length > 0) {
				fetchHandler(items, request);
			}
			else {
				fetchHandler([], request);
			}
		});
		getHandler.addErrback(function(data){
			errorHandler(data, request);
		});
	},

	_getFetchUrl: function(request){
		//	summary:
		//		Generate a URL for fetch
		//	description:
		//		This default implementation generates a query string in the form of
		//		"?name1=value1&name2=value2..." off properties of 'query' object
		//		specified in 'request' and appends it to 'url', if 'sendQuery'
		//		is set to false.
		//		Otherwise, 'url' is returned as is.
		//		Sub-classes may override this method for the custom URL generation.
		//	request:
		//		A request object
		//	returns:
		//		A fetch URL
		if(!this.sendQuery){
			return this.url;
		}
		var query = request.query;
		if(!query){
			return this.url;
		}
		if(dojo.isString(query)){
			return this.url + query;
		}
		var queryString = "";
		for(var name in query){
			var value = query[name];
			if(value){
				if(queryString){
					queryString += "&";
				}
				queryString += (name + "=" + value);
			}
		}
		if(!queryString){
			return this.url;
		}
		//Check to see if the URL already has query params or not.
		var fullUrl = this.url;
		if(fullUrl.indexOf("?") < 0){
			fullUrl += "?";
		}else{
			fullUrl += "&";
		}
		return fullUrl + queryString;
	},

	_getItems: function(document, request) {
		//	summary:
		//		Fetch items (XML elements) in an XML document based on a request
		//	description:
		//		This default implementation walks through child elements of
		//		the document element to see if all properties of 'query' object
		//		match corresponding attributes of the element (item).
		//		If 'request' is not specified, all child elements are returned.
		//		Sub-classes may override this method for the custom search in
		//		an XML document.
		//	document:
		//		An XML document
		//	request:
		//		A request object
		//	returns:
		//		An array of items
		var query = null;
		if(request){
			query = request.query;
		}
		var items = [];
		var nodes = null;

		console.log("Looking up root item: " + this.rootItem);
		if(this.rootItem !== ""){
			
			nodes = document.getElementsByTagName(this.rootItem);
		}
		else{
			nodes = document.documentElement.childNodes;
		}
		for(var i = 0; i < nodes.length; i++){
			var node = nodes[i];
			if(node.nodeType != 1 /*ELEMENT_NODE*/){
				continue;
			}
            var item = this._getItem(node);
			if(query){
				var found = true;
				var ignoreCase = request.queryOptions ? request.queryOptions.ignoreCase : false; 

				//See if there are any string values that can be regexp parsed first to avoid multiple regexp gens on the
				//same value for each item examined.  Much more efficient.
				var regexpList = {};
				for(var key in query){
					var value = query[key];
					if(typeof value === "string"){
						regexpList[key] = dojo.data.util.filter.patternToRegExp(value, ignoreCase);
					}
				}

				for(var attribute in query){
					var value = this.getValue(item, attribute);
					if(value){
						var queryValue = query[attribute];
						if ((typeof value) === "string" && 
							(regexpList[attribute])){
							if((value.match(regexpList[attribute])) !== null){
								continue;
							}
						}else if((typeof value) === "object"){
							if(	value.toString && 
								(regexpList[attribute])){
								var stringValue = value.toString();
								if((stringValue.match(regexpList[attribute])) !== null){
									continue;
								}
							}else{
								if(queryValue === "*" || queryValue === value){
									continue;
								}
							}
						}
					}
					found = false;
					break;
				}
				if(!found){
					continue;
				}
			}
			items.push(item);
		}
		dojo.forEach(items,function(item){ 
			item.element.parentNode.removeChild(item.element); // make it root
		},this); 
		return items;
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		 //	summary: 
		 //		See dojo.data.api.Read.close()
	},

/* dojo.data.api.Write */

	newItem: function(/* object? */ keywordArgs){
		//	summary:
		//		Return a new dojox.data.XmlItem
		//	description:
		//		At least, 'keywordArgs' must contain "tagName" to be used for
		//		the new	element.
		//		Other attributes in 'keywordArgs' are set to the new element,
		//		including "text()", but excluding "childNodes".
		// 	keywordArgs:
		//		An object containing initial attributes
		//	returns:
		//		An XML element
		console.log("XmlStore.newItem()");
		keywordArgs = (keywordArgs || {});
		var tagName = keywordArgs.tagName;
		if(!tagName){
			tagName = this.rootItem;
			if(tagName === ""){
				return null;
			}
		}

		var document = this._getDocument();
		var element = document.createElement(tagName);
		for(var attribute in keywordArgs){
			if(attribute === "tagName"){
				continue;
			}else if(attribute === "text()"){
				var text = document.createTextNode(keywordArgs[attribute]);
				element.appendChild(text);
			}else{
				attribute = this._getAttribute(tagName, attribute);
				if(attribute.charAt(0) === '@'){
					var name = attribute.substring(1);
					element.setAttribute(name, keywordArgs[attribute]);
				}else{
					var child = document.createElement(attribute);
					var text = document.createTextNode(keywordArgs[attribute]);
					child.appendChild(text);
					element.appendChild(child);
				}
			}
		}

		var item = this._getItem(element);
		this._newItems.push(item);
		return item; //object
	},
	
	deleteItem: function(/* item */ item){
		//	summary:
		//		Delete an dojox.data.XmlItem (wrapper to a XML element).
		//	item:
		//		An XML element to delete
		//	returns:
		//		True
		console.log("XmlStore.deleteItem()");
		var element = item.element;
		if(element.parentNode){
			this._backupItem(item);
			element.parentNode.removeChild(element);
			return true;
		}
		this._forgetItem(item);
		this._deletedItems.push(item);
		return true; //boolean
	},
	
	setValue: function(/* item */ item, /* attribute || string */ attribute, /* almost anything */ value){
		//	summary:
		//		Set an attribute value
		//	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		If 'attribute' specifies "tagName", nothing is set and false is
		//		returned.
		//		If 'attribute' specifies "childNodes", the value (XML element) is
		//		added to the element.
		//		If 'attribute' specifies "text()", a text node is created with
		//		the value and set it to the element as a child.
		//		For generic attributes, if '_attributeMap' is specified,
		//		an actual attribute name is looked up with the tag name of
		//		the element and 'attribute' (concatenated with '.').
		//		Then, if 'attribute' starts with "@", the value is set to the XML
		//		attribute.
		//		Otherwise, a text node is created with the value and set it to
		//		the first child element of the tag name specified with 'attribute'.
		//		If the child element does not exist, it is created.
		//	item:
		//		An XML element that holds the attribute
		//	attribute:
		//		A tag name of a child element, An XML attribute name or one of
		//		special names
		//	value:
		//		A attribute value to set
		//	returns:
		//		False for "tagName", otherwise true
		if(attribute === "tagName"){
			return false; //boolean
		}

		this._backupItem(item);

		var element = item.element;
		if(attribute === "childNodes"){
			var child = value.element;
			element.appendChild(child);
		}else if(attribute === "text()"){
			while (element.firstChild){
				element.removeChild(element.firstChild);
			}
			var text = this._getDocument(element).createTextNode(value);
			element.appendChild(text);
		}else{
			attribute = this._getAttribute(element.nodeName, attribute);
			if(attribute.charAt(0) === '@'){
				var name = attribute.substring(1);
				element.setAttribute(name, value);
			}else{
				var child = null;
				for(var i = 0; i < element.childNodes.length; i++){
					var node = element.childNodes[i];
					if(	node.nodeType === 1 /*ELEMENT_NODE*/&&
						node.nodeName === attribute){
						child = node;
						break;
					}
				}
				var document = this._getDocument(element);
				if(child){
					while(child.firstChild){
						child.removeChild(child.firstChild);
					}
				}else{
					child = document.createElement(attribute);
					element.appendChild(child);
				}
				var text = document.createTextNode(value);
				child.appendChild(text);
			}
		}
		return true; //boolean
	},
	
	setValues: function(/* item */ item, /* attribute || string */ attribute, /* array */ values){
		//	summary:
		//		Set attribute values
		//	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		If 'attribute' specifies "tagName", nothing is set and false is
		//		returned.
		//		If 'attribute' specifies "childNodes", the value (array of XML
		//		elements) is set to the element's childNodes.
		//		If 'attribute' specifies "text()", a text node is created with
		//		the values and set it to the element as a child.
		//		For generic attributes, if '_attributeMap' is specified,
		//		an actual attribute name is looked up with the tag name of
		//		the element and 'attribute' (concatenated with '.').
		//		Then, if 'attribute' starts with "@", the first value is set to
		//		the XML attribute.
		//		Otherwise, child elements of the tag name specified with
		//		'attribute' are replaced with new child elements and their
		//		child text nodes of values.
		//	item:
		//		An XML element that holds the attribute
		//	attribute:
		//		A tag name of child elements, an XML attribute name or one of
		//		special names
		//	value:
		//		A attribute value to set
		//	returns:
		//		False for "tagName", otherwise true
		if(attribute === "tagName"){
			return false; //boolean
		}

		this._backupItem(item);

		var element = item.element;
		if(attribute === "childNodes"){
			while(element.firstChild){
				element.removeChild(element.firstChild);
			}
			for(var i = 0; i < values.length; i++){
				var child = values[i].element;
				element.appendChild(child);
			}
		}else if(attribute === "text()"){
			while (element.firstChild){
				element.removeChild(element.firstChild);
			}
			var value = "";
			for(var i = 0; i < values.length; i++){
				value += values[i];
			}
			var text = this._getDocument(element).createTextNode(value);
			element.appendChild(text);
		}else{
			attribute = this._getAttribute(element.nodeName, attribute);
			if(attribute.charAt(0) === '@'){
				var name = attribute.substring(1);
				element.setAttribute(name, values[0]);
			}else{
				for(var i = element.childNodes.length - 1; i >= 0; i--){
					var node = element.childNodes[i];
					if(	node.nodeType === 1 /*ELEMENT_NODE*/ &&
						node.nodeName === attribute){
						element.removeChild(node);
					}
				}
				var document = this._getDocument(element);
				for(var i = 0; i < values.length; i++){
					var child = document.createElement(attribute);
					var text = document.createTextNode(values[i]);
					child.appendChild(text);
					element.appendChild(child);
				}
			}
		}
		return true; //boolean
	},
	
	unsetAttribute: function(/* item */ item, /* attribute || string */ attribute){
		//	summary:
		//		Remove an attribute
		//	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		'attribute' can be an XML attribute name of the element or one of
		//		special names described below.
		//		If 'attribute' specifies "tagName", nothing is removed and false is
		//		returned.
		//		If 'attribute' specifies "childNodes" or "text()", all child nodes
		//		are removed.
		//		For generic attributes, if '_attributeMap' is specified,
		//		an actual attribute name is looked up with the tag name of
		//		the element and 'attribute' (concatenated with '.').
		//		Then, if 'attribute' starts with "@", the XML attribute is removed.
		//		Otherwise, child elements of the tag name specified with
		//		'attribute' are removed.
		//	item:
		//		An XML element that holds the attribute
		//	attribute:
		//		A tag name of child elements, an XML attribute name or one of
		//		special names
		//	returns:
		//		False for "tagName", otherwise true
		if(attribute === "tagName"){
			return false; //boolean
		}

		this._backupItem(item);

		var element = item.element;
		if(attribute === "childNodes" || attribute === "text()"){
			while(element.firstChild){
				element.removeChild(element.firstChild);
			}
		}else{
			attribute = this._getAttribute(element.nodeName, attribute);
			if(attribute.charAt(0) === '@'){
				var name = attribute.substring(1);
				element.removeAttribute(name);
			}else{
				for(var i = element.childNodes.length - 1; i >= 0; i--){
					var node = element.childNodes[i];
					if(	node.nodeType === 1 /*ELEMENT_NODE*/ &&
						node.nodeName === attribute){
						element.removeChild(node);
					}
				}
			}
		}
		return true; //boolean
	},
	
	save: function(/* object */ keywordArgs){
		//	summary:
		//		Save new and/or modified items (XML elements)
		// 	description:
		//		'url' is used to save XML documents for new, modified and/or
		//		deleted XML elements.
		// 	keywordArgs:
		//		An object for callbacks
		if(!keywordArgs){
			keywordArgs = {};
		}
		for(var i = 0; i < this._modifiedItems.length; i++){
			this._saveItem(this._modifiedItems[i], keywordArgs, "PUT");
		}
		for(var i = 0; i < this._newItems.length; i++){
			var item = this._newItems[i];
			if(item.element.parentNode){ // reparented
				this._newItems.splice(i, 1);
				i--;
				continue;
			}
			this._saveItem(this._newItems[i], keywordArgs, "POST");
		}
		for(var i = 0; i < this._deletedItems.length; i++){
			this._saveItem(this._deletedItems[i], keywordArgs, "DELETE");
		}
	},

	revert: function(){
		// summary:
		//	Invalidate changes (new and/or modified elements)
		// returns:
		//	True
		console.log("XmlStore.revert() _newItems=" + this._newItems.length);
		console.log("XmlStore.revert() _deletedItems=" + this._deletedItems.length);
		console.log("XmlStore.revert() _modifiedItems=" + this._modifiedItems.length);
		this._newItems = [];
		this._restoreItems(this._deletedItems);
		this._deletedItems = [];
		this._restoreItems(this._modifiedItems);
		this._modifiedItems = [];
		return true; //boolean
	},
	
	isDirty: function(/* item? */ item){
		//	summary:
		//		Check whether an item is new, modified or deleted
		//	description:
		//		If 'item' is specified, true is returned if the item is new,
		//		modified or deleted.
		//		Otherwise, true is returned if there are any new, modified
		//		or deleted items.
		//	item:
		//		An item (XML element) to check
		//	returns:
		//		True if an item or items are new, modified or deleted, otherwise
		//		false
		if (item) {
			var element = this._getRootElement(item.element);
			return (this._getItemIndex(this._newItems, element) >= 0 ||
				this._getItemIndex(this._deletedItems, element) >= 0 ||
				this._getItemIndex(this._modifiedItems, element) >= 0); //boolean
		}
		else {
			return (this._newItems.length > 0 ||
				this._deletedItems.length > 0 ||
				this._modifiedItems.length > 0); //boolean
		}
	},

	_saveItem: function(item, keywordArgs, method){
		if(method === "PUT"){
			url = this._getPutUrl(item);
		}else if(method === "DELETE"){
			url = this._getDeleteUrl(item);
		}else{ // POST
			url = this._getPostUrl(item);
		}
		if(!url){
			if(keywordArgs.onError){
				keywordArgs.onError.call(scope, new Error("No URL for saving content: " + postContent));
			}
			return;
		}

		var saveArgs = {
			url: url,
			method: (method || "POST"),
			contentType: "text/xml",
			handleAs: "xml"
		};
		var saveHander;
		if(method === "PUT"){
			saveArgs.putData = this._getPutContent(item);
			saveHandler = dojo.rawXhrPut(saveArgs);
		}else if(method === "DELETE"){
			saveHandler = dojo.xhrDelete(saveArgs);
		}else{ // POST
			saveArgs.postData = this._getPostContent(item);
			saveHandler = dojo.rawXhrPost(saveArgs);
		}
		var scope = (keywordArgs.scope || dojo.global);
		var self = this;
		saveHandler.addCallback(function(data){
			self._forgetItem(item);
			if(keywordArgs.onComplete){
				keywordArgs.onComplete.call(scope);
			}
		});
		saveHandler.addErrback(function(error){
			if(keywordArgs.onError){
				keywordArgs.onError.call(scope, error);
			}
		});
	},

	_getPostUrl: function(item){
		//	summary:
		//		Generate a URL for post
		//	description:
		//		This default implementation just returns 'url'.
		//		Sub-classes may override this method for the custom URL.
		//	item:
		//		An item to save
		//	returns:
		//		A post URL
		return this.url; //string
	},

	_getPutUrl: function(item){
		//	summary:
		//		Generate a URL for put
		//	description:
		//		This default implementation just returns 'url'.
		//		Sub-classes may override this method for the custom URL.
		//	item:
		//		An item to save
		//	returns:
		//		A put URL
		return this.url; //string
	},

	_getDeleteUrl: function(item){
		//	summary:
		//		Generate a URL for delete
		// 	description:
		//		This default implementation returns 'url' with 'keyAttribute'
		//		as a query string.
		//		Sub-classes may override this method for the custom URL based on
		//		changes (new, deleted, or modified).
		// 	item:
		//		An item to delete
		// 	returns:
		//		A delete URL
		if (!this.url !== "") {
			return this.url; //string
		}
		var url = this.url;
		if (item && this.keyAttribute !== "") {
			var value = this.getValue(item, this.keyAttribute);
			if (value) {
				url = url + '?' + this.keyAttribute + '=' + value;
			}
		}
		return url;	//string
	},

	_getPostContent: function(item){
		//	summary:
		//		Generate a content to post
		// 	description:
		//		This default implementation generates an XML document for one
		//		(the first only) new or modified element.
		//		Sub-classes may override this method for the custom post content
		//		generation.
		//	item:
		//		An item to save
		//	returns:
		//		A post content
		var element = item.element;
		var declaration = "<?xml version=\"1.0\"?>"; // FIXME: encoding?
		return declaration + dojox.data.dom.innerXML(element); //XML string
	},

	_getPutContent: function(item){
		//	summary:
		//		Generate a content to put
		// 	description:
		//		This default implementation generates an XML document for one
		//		(the first only) new or modified element.
		//		Sub-classes may override this method for the custom put content
		//		generation.
		//	item:
		//		An item to save
		//	returns:
		//		A post content
		var element = item.element;
		var declaration = "<?xml version=\"1.0\"?>"; // FIXME: encoding?
		return declaration + dojox.data.dom.innerXML(element); //XML string
	},

/* internal API */

	_getAttribute: function(tagName, attribute){
		if(this._attributeMap){
			var key = tagName + "." + attribute;
			var value = this._attributeMap[key];
			if(value){
				attribute = value;
			}else{ // look for global attribute
				value = this._attributeMap[attribute];
				if(value){
					attribute = value;
				}
			}
		}
		return attribute; //object
	},

	_getItem: function(element){
		return new dojox.data.XmlItem(element, this); //object
	},

	_getItemIndex: function(items, element){
		for(var i = 0; i < items.length; i++){
			if(items[i].element === element){
				return i; //int
			}
		}
		return -1; //int
	},

	_backupItem: function(item){
		var element = this._getRootElement(item.element);
		if(	this._getItemIndex(this._newItems, element) >= 0 ||
			this._getItemIndex(this._modifiedItems, element) >= 0){
			return; // new or already modified
		}
		if(element != item.element){
			item = this._getItem(element);
		}
		item._backup = element.cloneNode(true);
		this._modifiedItems.push(item);
	},

	_restoreItems: function(items){

		dojo.forEach(items,function(item){ 
			if(item._backup){
				item.element = item._backup;
				item._backup = null;
			}
		},this); 
	},

	_forgetItem: function(item){
		var element = item.element;
		var index = this._getItemIndex(this._newItems, element);
		if(index >= 0){
			this._newItems.splice(index, 1);
		}
		index = this._getItemIndex(this._deletedItems, element);
		if(index >= 0){
			this._deletedItems.splice(index, 1);
		}
		index = this._getItemIndex(this._modifiedItems, element);
		if(index >= 0){
			this._modifiedItems.splice(index, 1);
		}
	},

	_getDocument: function(element){
		if(element){
			return element.ownerDocument;  //DOMDocument
		}else if(!this._document){
			return dojox.data.dom.createDocument(); // DOMDocument
		}
	},

	_getRootElement: function(element){
		while(element.parentNode){
			element = element.parentNode;
		}
		return element; //DOMElement
	}

});

//FIXME: Is a full class here really needed for containment of the item or would
//an anon object work fine?
dojo.declare("dojox.data.XmlItem", null, {
	constructor: function(element, store) {
		//	summary:
		//		Initialize with an XML element
		//	element:
		//		An XML element
		//	store:
		//		The containing store, if any.
		this.element = element;
		this.store = store;
	}, 
	//	summary:
	//		A data item of 'XmlStore'
	//	description:
	//		This class represents an item of 'XmlStore' holding an XML element.
	//		'element'
	//	element:
	//		An XML element

	toString: function() {
		//	summary:
		//		Return a value of the first text child of the element
		// 	returns:
		//		a value of the first text child of the element
		var str = "";
		if (this.element) {
			for (var i = 0; i < this.element.childNodes.length; i++) {
				var node = this.element.childNodes[i];
				if (node.nodeType === 3) {
					str = node.nodeValue;
					break;
				}
			}
		}
		return str;	//String
	}

});
dojo.extend(dojox.data.XmlStore,dojo.data.util.simpleFetch);

}

if(!dojo._hasResource["dojox.string.Builder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.string.Builder"] = true;
dojo.provide("dojox.string.Builder");

(function(){
	dojox.string.Builder = function(/*String?*/str){
		// summary:
		//		A fast buffer for creating large strings
		// str: The initial string to seed the buffer with
		this.b = dojo.isIE ? [] : "";
		if(str){ this.append(str); }
	};
	
	var m = {
	 	append: function(/*String*/s){ 
			// summary: Append all arguments to the end of the buffer 
			return this.appendArray(dojo._toArray(arguments)); // dojox.string.Builder
		},
		concat: function(/*String*/s){
			return this.append(s);
		},
		appendArray: function(/*Array*/strings) {
			this.b = String.prototype.concat.apply(this.b, strings);
			return this;
		},
		clear: function(){
			// summary: Remove all characters from the buffer
			this._clear();
			this.length = 0;
			return this;
		},
		replace: function(oldStr,newStr){
			// summary: Replace instances of one string with another in the buffer
			var s = this.toString();
			s = s.replace(oldStr,newStr);
			this._reset(s);
			this.length = s.length;
			return this;
		},
		remove: function(start, len){
			// summary: Remove len characters starting at index start
			if(len == 0){ return this; }
			var s = this.toString();
			this.clear();
			if(start > 0){
				this.append(s.substring(0, start));
			}
			if(start+len < s.length){
				this.append(s.substring(start+len));
			}
			return this;
		},
		insert: function(index, str){
			// summary: Insert string str starting at index
			var s = this.toString();
			this.clear();
			if(index == 0){
				this.append(str);
				this.append(s);
				return this;
			}else{
				this.append(s.substring(0, index));
				this.append(str);
				this.append(s.substring(index));
			}
			return this;
		},
		toString: function(){
			return this.b;
		},
		_clear: function(){
			this.b = "";
		},
		_reset: function(s){
			this.b = s;
		}
	}; // will hold methods for Builder
	
	if(dojo.isIE){
		dojo.mixin(m, {
			toString: function(){ 
				// Summary: Get the buffer as a string
				return this.b.join(""); 
			},
			appendArray: function(strings){
				this.b = this.b.concat(strings);
				return this;
			},
			_clear: function(){
				this.b = [];
			},
			_reset: function(s){
				this.b = [ s ];
			}
		});
	}
	
	dojo.extend(dojox.string.Builder, m);
})();

}

if(!dojo._hasResource["dojox.string.tokenize"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.string.tokenize"] = true;
dojo.provide("dojox.string.tokenize");

dojox.string.tokenize = function(/*String*/ str, /*RegExp*/ re, /*Function?*/ parseDelim, /*Object?*/ instance){
	// summary:
	//		Split a string by a regular expression with the ability to capture the delimeters
	// parseDelim:
	//		Each group (excluding the 0 group) is passed as a parameter. If the function returns
	//		a value, it's added to the list of tokens.
	// instance:
	//		Used as the "this" instance when calling parseDelim
	var tokens = [];
	var match, content, lastIndex = 0;
	while(match = re.exec(str)){
		content = str.substring(lastIndex, re.lastIndex - match[0].length);
		if(content.length){
			tokens.push(content);
		}
		if(parseDelim){
			var parsed = parseDelim.apply(instance, match.slice(1));
			if(typeof parsed != "undefined"){
				tokens.push(parsed);
			}
		}
		lastIndex = re.lastIndex;
	}
	content = str.substr(lastIndex);
	if(content.length){
		tokens.push(content);
	}
	return tokens;
}

}

if(!dojo._hasResource["dojox.dtl._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl._base"] = true;
dojo.provide("dojox.dtl._base");




dojox.dtl.Context = function(dict){
	dojo.mixin(this, dict || {});
	this._dicts = [];
	this._this = {};
}
dojo.extend(dojox.dtl.Context, {
	_dicts: [],
	_this: {},
	extend: function(/*dojox.dtl.Context|Object*/ obj){
		// summary: Returns a clone of this context object, with the items from the
		//		passed objecct mixed in.
		var context = new dojox.dtl.Context();
		var keys = this.getKeys();
		for(var i = 0, key; key = keys[i]; i++){
			if(typeof obj[key] != "undefined"){
				context[key] = obj[key];
			}else{
				context[key] = this[key];
			}
		}

		if(obj instanceof dojox.dtl.Context){
			keys = obj.getKeys();
		}else if(typeof obj == "object"){
			keys = [];
			for(var key in obj){
				keys.push(key);
			}
		}

		for(var i = 0, key; key = keys[i]; i++){
			context[key] = obj[key];
		}

		return context;
	},
	filter: function(/*dojox.dtl.Context|Object|String...*/ filter){
		// summary: Returns a clone of this context, only containing the items
		//		defined in the filter.
		var context = new dojox.dtl.Context();
		var keys = [];
		if(filter instanceof dojox.dtl.Context){
			keys = filter.getKeys();
		}else if(typeof filter == "object"){
			for(var key in filter){
				keys.push(key);
			}
		}else{
			for(var i = 0, arg; arg = arguments[i]; i++){
				if(typeof arg == "string"){
					keys.push(arg);
				}
			}
		}

		for(var i = 0, key; key = keys[i]; i++){
			context[key] = this[key];
		}

		return context;
	},
	setThis: function(/*Object*/ _this){
		this._this = _this;
	},
	getThis: function(){
		return this._this;
	},
	push: function(){
		var dict = {};
		var keys = this.getKeys();
		for(var i = 0, key; key = keys[i]; i++){
			dict[key] = this[key];
			delete this[key];
		}
		this._dicts.unshift(dict);
	},
	pop: function(){
		if(!this._dicts.length){
			throw new Error("pop() has been called more times than push() on the Context");
		}
		var dict = this._dicts.shift();
		dojo.mixin(this, dict);
	},
	hasKey: function(key){
		if(typeof this[key] != "undefined"){
			return true;
		}

		for(var i = 0, dict; dict = this._dicts[i]; i++){
			if(typeof dict[key] != "undefined"){
				return true;
			}
		}

		return false;
	},
	getKeys: function(){
		var keys = [];
		for(var key in this){
			if(isNaN(key)){
				var found = false;
				for(var protoKey in dojox.dtl.Context.prototype){
					if(key == protoKey){
						found = true;
						break;
					}
				}
				if(!found){
					keys.push(key);
				}
			}
		}
		return keys;
	},
	get: function(key, otherwise){
		if(typeof this[key] != "undefined"){
			return this[key];
		}

		for(var i = 0, dict; dict = this._dicts[i]; i++){
			if(typeof dict[key] != "undefined"){
				return dict[key];
			}
		}

		return otherwise;
	},
	update: function(dict){
		this.push();
		if(dict){
			dojo.mixin(this, dict);
		}
	},
	toString: function(){ return "dojox.dtl.Context"; }
});

dojox.dtl.text = {
	types: {tag: -1, varr: -2,	text: 3},
	pySplit: function(str){
		// summary: Split a string according to Python's split function
		str = str.replace(/^\s+|\s+$/, "");
		if(!str.length){
			return [];
		}
		return str.split(/\s+/g);
	},
	urlquote: function(/*String*/ url, /*String?*/ safe){
		if(!safe){
			safe = "/";
		}
		return dojox.string.tokenize(url, /([^\w-_.])/g, function(token){
			if(safe.indexOf(token) == -1){
				if(token == " "){
					return "+";
				}else{
					return "%" + token.charCodeAt(0).toString(16).toUpperCase();
				}
			}
			return token;
		}).join("");
	},
	_get: function(module, name, errorless){
		// summary: Used to find both tags and filters
		var params = dojox.dtl.register.get(module, name, errorless);
		if(!params) return;

		var require = params.getRequire();
		var obj = params.getObj();
		var fn = params.getFn();

		if(fn.indexOf(":") != -1){
			var parts = fn.split(":");
			fn = parts.pop();
		}

		dojo["require"](require);

		var parent = window;
		var parts = obj.split(".");
		for(var i = 0, part; part = parts[i]; i++){
			if(!parent[part]) return;
			parent = parent[part];
		}
		return parent[fn || name] || parent[name + "_"];
	},
	getTag: function(name, errorless){
		return dojox.dtl.text._get("tag", name, errorless);
	},
	getFilter: function(name, errorless){
		return dojox.dtl.text._get("filter", name, errorless);
	},
	getTemplate: function(file){
		return new dojox.dtl.Template(dojox.dtl.getTemplateString(file));
	},
	getTemplateString: function(file){
		return dojo._getText(file.toString()) || "";
	},
	_re: /(?:\{\{\s*(.+?)\s*\}\}|\{%\s*(.+?)\s*%\})/g,
	tokenize: function(str){
		return dojox.string.tokenize(str, dojox.dtl.text._re, dojox.dtl.text._parseDelims);
	},
	_parseDelims: function(varr, tag){
		var types = dojox.dtl.text.types;
		if(varr){
			return [types.varr, varr];
		}else{
			return [types.tag, tag];
		}
	}
}

dojox.dtl.Template = function(str){
	var st = dojox.dtl;
	var tokens = st.text.tokenize(str);
	var parser = new st.Parser(tokens);
	this.nodelist = parser.parse();
}
dojo.extend(dojox.dtl.Template, {
	render: function(context, /*concatenatable?*/ buffer){
		context = context || new dojox.dtl.Context({});
		if(!buffer){
			
			buffer = new dojox.string.Builder();
		}
		return this.nodelist.render(context, buffer) + "";
	},
	toString: function(){ return "dojox.dtl.Template"; }
});

dojox.dtl.Filter = function(token){
	// summary: Uses a string to find (and manipulate) a variable
	if(!token) throw new Error("Filter must be called with variable name");
	this.contents = token;
	var key = null;
	var re = this._re;
	var matches, filter, arg, fn;
	var filters = [];
	while(matches = re.exec(token)){
		if(key === null){
			if(this._exists(matches, 3)){
				// variable
				key = matches[3];
			}else if(this._exists(matches, 1)){
				// _("text")
				key = '"' + matches[1] + '"';
			}else if(this._exists(matches, 2)){
				// "text"
				key = '"' + matches[2] + '"';
			}else if(this._exists(matches, 9)){
				// 'text'
				key = '"' + matches[9] + '"';
			}
		}else{
			if(this._exists(matches, 7)){
				// :variable
				arg = [true, matches[7]];
			}else if(this._exists(matches, 5)){
				// :_("text")
				arg = [false, dojox.dtl.replace(matches[5], '\\"', '"')];
			}else if(this._exists(matches, 6)){
				// :"text"
				arg = [false, dojox.dtl.replace(matches[6], '\\"', '"')];
			}else if(this._exists(matches, 8)){
				// :"text"
				arg = [false, dojox.dtl.replace(matches[8], "\\'", "'")];
			}
			// Get a named filter
			fn = dojox.dtl.text.getFilter(matches[4]);
			if(typeof fn != "function") throw new Error(matches[4] + " is not registered as a filter");
			filters.push([fn, arg]);
		}
	}

	this.key = key;
	this.filters = filters;
} 
dojo.extend(dojox.dtl.Filter, {
	_re: /(?:^_\("([^\\"]*(?:\\.[^\\"])*)"\)|^"([^\\"]*(?:\\.[^\\"]*)*)"|^([a-zA-Z0-9_.]+)|\|(\w+)(?::(?:_\("([^\\"]*(?:\\.[^\\"])*)"\)|"([^\\"]*(?:\\.[^\\"]*)*)"|([a-zA-Z0-9_.]+)|'([^\\']*(?:\\.[^\\']*)*)'))?|^'([^\\']*(?:\\.[^\\']*)*)')/g,
	_exists: function(arr, index){
		if(typeof arr[index] != "undefined" && arr[index] !== ""){
			return true;
		}
		return false;
	},
	resolve: function(context){
		var str = this.resolvePath(this.key, context);
		for(var i = 0, filter; filter = this.filters[i]; i++){
			// Each filter has the function in [0], a boolean in [1][0] of whether it's a variable or a string
			// and [1][1] is either the variable name of the string content.
			if(filter[1]){
				if(filter[1][0]){
					str = filter[0](str, this.resolvePath(filter[1][1], context));
				}else{
					str = filter[0](str, filter[1][1]);
				}
			}else{
				str = filter[0](str);
			}
		}
		return str;
	},
	resolvePath: function(path, context){
		var current, parts;
		var first = path.charAt(0);
		var last = path.charAt(path.length - 1);
		if(!isNaN(parseInt(first))){
			current = (path.indexOf(".") == -1) ? parseInt(path) : parseFloat(path);
		}else if(first == '"' && first == last){
			current = path.substring(1, path.length - 1);
		}else{;
			if(path == "true") return true;
			if(path == "false") return false;
			if(path == "null" || path == "None") return null;
			parts = path.split(".");
			current = context.get(parts.shift());
			while(parts.length){
				if(current && typeof current[parts[0]] != "undefined"){
					current = current[parts[0]];
					if(typeof current == "function"){
						if(current.alters_data){
							current = "";
						}else{
							current = current();
						}
					}
				}else{
					return "";
				}
				parts.shift();
			}
		}
		return current;
	},
	toString: function(){ return "dojox.dtl.Filter"; }
});

dojox.dtl.Node = function(/*Object*/ obj){
	// summary: Basic catch-all node
	this.contents = obj;
}
dojo.extend(dojox.dtl.Node, {
	render: function(context, buffer){
		// summary: Adds content onto the buffer
		return buffer.concat(this.contents);
	},
	toString: function(){ return "dojox.dtl.Node"; }
});

dojox.dtl.NodeList = function(/*Node[]*/ nodes){
	// summary: Allows us to render a group of nodes
	this.contents = nodes || [];
}
dojo.extend(dojox.dtl.NodeList, {
	push: function(node){
		// summary: Add a new node to the list
		this.contents.push(node);
	},
	render: function(context, buffer){
		// summary: Adds all content onto the buffer
		for(var i = 0; i < this.contents.length; i++){
			buffer = this.contents[i].render(context, buffer);
			if(!buffer) throw new Error("Template node render functions must return their buffer");
		}
		return buffer;
	},
	unrender: function(context, buffer){ return buffer; },
	clone: function(){ return this; },
	toString: function(){ return "dojox.dtl.NodeList"; }
});

dojox.dtl.TextNode = dojox.dtl.Node;

dojox.dtl.VarNode = function(str){
	// summary: A node to be processed as a variable
	this.contents = new dojox.dtl.Filter(str);
}
dojo.extend(dojox.dtl.VarNode, {
	render: function(context, buffer){
		var str = this.contents.resolve(context);
		return buffer.concat(str);
	},
	toString: function(){ return "dojox.dtl.VarNode"; }
});

dojox.dtl.Parser = function(tokens){
	// summary: Parser used during initialization and for tag groups.
	this.contents = tokens;
}
dojo.extend(dojox.dtl.Parser, {
	parse: function(/*Array?*/ stop_at){
		// summary: Turns tokens into nodes
		// description: Steps into tags are they're found. Blocks use the parse object
		//		to find their closing tag (the stop_at array). stop_at is inclusive, it
		//		returns the node that matched.
		var st = dojox.dtl;
		var types = st.text.types;
		var terminators = {};
		var tokens = this.contents;
		stop_at = stop_at || [];
		for(var i = 0; i < stop_at.length; i++){
			terminators[stop_at[i]] = true;
		}

		var nodelist = new st.NodeList();
		while(tokens.length){
			token = tokens.shift();
			if(typeof token == "string"){
				nodelist.push(new st.TextNode(token));
			}else{
				var type = token[0];
				var text = token[1];
				if(type == types.varr){
					nodelist.push(new st.VarNode(text));
				}else if(type == types.tag){
					if(terminators[text]){
						tokens.unshift(token);
						return nodelist;
					}
					var cmd = text.split(/\s+/g);
					if(cmd.length){
						cmd = cmd[0];
						var fn = dojox.dtl.text.getTag(cmd);
						if(fn){
							nodelist.push(fn(this, text));
						}
					}
				}
			}
		}

		if(stop_at.length){
			throw new Error("Could not find closing tag(s): " + stop_at.toString());
		}

		return nodelist;
	},
	next: function(){
		// summary: Returns the next token in the list.
		var token = this.contents.shift();
		return {type: token[0], text: token[1]};
	},
	skipPast: function(endtag){
		var types = dojox.dtl.text.types;
		while(this.contents.length){
			var token = this.contents.shift();
			if(token[0] == types.tag && token[1] == endtag){
				return;
			}
		}
		throw new Error("Unclosed tag found when looking for " + endtag);
	},
	getVarNode: function(){
		return dojox.dtl.VarNode;
	},
	getTextNode: function(){
		return dojox.dtl.TextNode;
	},
	getTemplate: function(file){
		return new dojox.dtl.Template(file);
	},
	toString: function(){ return "dojox.dtl.Parser"; }
});

dojox.dtl.register = function(module, cols, args, /*Function*/ normalize){
	// summary: Used to create dojox.dtl.register[module] function, and as a namespace
	// expand: Used if the call structure is reformatted for a more compact view.
	//		Should return an array of normalized arguments.
	// description: The function produced will accept a "name"
	//		as the first parameter and all other parameters will
	//		be associated with the parameter names listed in cols.
	var ddr = dojox.dtl.register;
	var registry = ddr._mod[module] = {
		params: [],
		Getter: function(params){
			ddr._params = params || {};
		}
	};

	cols.unshift("name");
	for(var i = 0, col; col = cols[i]; i++){
		registry.Getter.prototype["get" + col.substring(0, 1).toUpperCase() + col.substring(1, col.length)] = ddr._ret(i);
	}

	ddr[module] = function(/*String*/ name, /*mixed...*/ parameters){
		if(normalize){
			var normalized = normalize(arguments);
		}else{
			var normalized = [arguments];
		}

		for(var i = 0, args; args = normalized[i]; i++){
			var params = [];
			for(var j = 0; j < cols.length; j++){
				params.push(args[j] || null);
			}
			if(typeof args[0] == "string"){
				// Strings before regexes for speed
				registry.params.unshift(params);
			}else{
				// break
				// module RegExp
				registry.params.push(params);
			}
		}
	}

	ddr[module].apply(null, args);
}
dojo.mixin(dojox.dtl.register, {
	_mod: {},
	_ret: function(i){
		// summary: Just lets use i and _params within a closure
		return function(){
			return dojox.dtl.register._params[i] || "";
		}
	},
	get: function(/*String*/ module, /*String*/ name, /*Boolean*/ errorless){
		// summary: Returns a "Getter", based on the registry
		// description: The getter functions correspond with the registered cols
		//		used in dojo.register
		var registry = this._mod[module] || {};
		if(registry.params){
			for(var i = 0, param; param = registry.params[i]; i++){
				var search = param[0];
				if(typeof search == "string"){
					if(search == name){
						return new registry.Getter(param);
					}
				}else if(name.match(search)){
					var matches = search.exec(name);
					var mixin = [];
					dojo.mixin(mixin, param);
					mixin[0] = matches[1];
					return new registry.Getter(param);
				}
			}
		}
		if(!errorless) throw new Error("'" + module + "' of name '" + name + "' does not exist");
	},
	_normalize: function(args){
		// summary:
		//		Translates to the signature (/*String*/ name, /*String*/ require, /*String*/ obj, /*String*/ fn)
		var items = args[2];
		var output = [];
		for(var i = 0, item; item = items[i]; i++){
			if(typeof item == "string"){
				output.push([item, args[0], args[1], item]);
			}else{
				output.push([item[0], args[0], args[1], item[1]]);
			}
		}
		return output;
	},
 	tag: function(/*String*/ require, /*String*/ obj, /*String[]|[RegExp, String][]*/ fns){
		// summary:
		//		Specify the location of a given tag function.
		// require:
		//		The file this function is in
		// obj:
		//		The base object to use for lookups
		// fn:
		//		List of functions within obj to use
		// description:
		//		When we are looking up a tag as specified in a template, we either use a
		//		string in the fns array, or the RegExp item of the [RegExp, String] pair.
		//		When that string is found, it requires the file specified in the require
		//		parameter, uses the base object as a starting point and checks for obj.fn
		//		or obj.fn_ in case fn is a reserved word.
		this("tag", ["require", "obj", "fn"], arguments, this._normalize);
	},
	filter: function(/*String*/ require, /*String*/ obj, /*String[]|[RegExp, String][]*/ fns){
		// summary:
		//		Specify the location of a given filter function.
		// require:
		//		The file this function is in
		// obj:
		//		The base object to use for lookups
		// fn:
		//		List of functions within obj to use
		// description:
		//		When we are looking up a tag as specified in a template, we either use a
		//		string in the fns array, or the RegExp item of the [RegExp, String] pair.
		//		When that string is found, it requires the file specified in the require
		//		parameter, uses the base object as a starting point and checks for obj.fn
		//		or obj.fn_ in case fn is a reserved word.
		this("filter", ["require", "obj", "fn"], arguments, this._normalize);
	}
});

(function(){
	var register = dojox.dtl.register;
	var dtt = "dojox.dtl.tag";
	register.tag(dtt + ".logic", dtt + ".logic", ["if", "for"]);
	register.tag(dtt + ".loader", dtt + ".loader", ["extends", "block"]);
	register.tag(dtt + ".misc", dtt + ".misc", ["comment", "debug", "filter"]);
	register.tag(dtt + ".loop", dtt + ".loop", ["cycle"]);

	var dtf = "dojox.dtl.filter";
	register.filter(dtf + ".dates", dtf + ".dates", ["date", "time", "timesince", "timeuntil"]);
	register.filter(dtf + ".htmlstrings", dtf + ".htmlstrings", ["escape", "linebreaks", "linebreaksbr", "removetags", "striptags"]);
	register.filter(dtf + ".integers", dtf + ".integers", ["add", "get_digit"]);
	register.filter(dtf + ".lists", dtf + ".lists", ["dictsort", "dictsortreversed", "first", "join", "length", "length_is", "random", "slice", "unordered_list"]);
	register.filter(dtf + ".logic", dtf + ".logic", ["default", "default_if_none", "divisibleby", "yesno"]);
	register.filter(dtf + ".misc", dtf + ".misc", ["filesizeformat", "pluralize", "phone2numeric", "pprint"]);
	register.filter(dtf + ".strings", dtf + ".strings", ["addslashes", "capfirst", "center", "cut", "fix_ampersands", "floatformat", "iriencode", "linenumbers", "ljust", "lower", "make_list", "rjust", "slugify", "stringformat", "title", "truncatewords", "truncatewords_html", "upper", "urlencode", "urlize", "urlizetrunc", "wordcount", "wordwrap"]);
})();

dojox.dtl.replace = function(str, token, repl){
	repl = repl || "";	
	var pos, len = token.length;
	while(1){
		pos = str.indexOf(token);
		if(pos == -1) break;
		str = str.substring(0, pos) + repl + str.substring(pos + len);
	}
	return str;
}

dojox.dtl.resolveVariable = function(token, context){
	// summary: Quickly resolve a variables
	var filter = new dojox.dtl.Filter(token);
	return filter.resolve(context);
}

}

if(!dojo._hasResource["dojox.dtl.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.html"] = true;
dojo.provide("dojox.dtl.html");



dojox.dtl.ObjectMap = function(){
	this.contents = [];
}
dojo.extend(dojox.dtl.ObjectMap, {
	get: function(key){
		var contents = this.contents;
		for(var i = 0, content; content = contents[i]; i++){
			if(content[0] === key){
				return content[1];
			}
		}
	},
	put: function(key, value){
		var contents = this.contents;
		for(var i = 0, content; content = contents[i]; i++){
			if(content[0] === key){
				if(arguments.length == 1){
					contents.splice(i, 1);
					return;
				}
				content[1] = value;
				return;
			}
		}
		contents.push([key, value]);
	},
	toString: function(){ return "dojox.dtl.ObjectMap"; }
});

dojox.dtl.html = {
	types: dojo.mixin({change: -11, attr: -12, elem: 1, text: 3}, dojox.dtl.text.types),
	_attributes: {},
	_re: /(^\s+|\s+$)/g,
	_re2: /\b([a-zA-Z]+)="/g,
	_re3: /<!--({({|%).*?(%|})})-->/g,
	_re4: /^function anonymous\(\)\s*{\s*(.*)\s*}$/,
	_trim: function(/*String*/ str){
		return str.replace(this._re, "");
	},
	getTemplate: function(text){
		if(typeof this._commentable == "undefined"){
			// Check to see if the browser can handle comments
			this._commentable = false;
			var div = document.createElement("div");
			div.innerHTML = "<!--Test comment handling, and long comments, using comments whenever possible.-->";
			if(div.childNodes.length && div.childNodes[0].nodeType == 8 && div.childNodes[0].data == "comment"){
				this._commentable = true;
			}
		}

		if(!this._commentable){
			// Strip comments
			text = text.replace(this._re3, "$1");
		}

		var match;
		while(match = this._re2.exec(text)){
			this._attributes[match[1]] = true;
		}
		var div = document.createElement("div");
		div.innerHTML = text;
		var output = { pres: [], posts: []}
		while(div.childNodes.length){
			if(!output.node && div.childNodes[0].nodeType == 1){
				output.node = div.removeChild(div.childNodes[0]);
			}else if(!output.node){
				output.pres.push(div.removeChild(div.childNodes[0]));
			}else{
				output.posts.push(div.removeChild(div.childNodes[0]));
			}
		}

		if(!output.node){
			throw new Error("Template did not provide any content");
		}

		return output;
	},
	tokenize: function(/*Node*/ node, /*Array?*/ tokens, /*Array?*/ preNodes, /*Array?*/ postNodes){
		tokens = tokens || [];
		var first = !tokens.length;
		var types = this.types;

		var children = [];
		for(var i = 0, child; child = node.childNodes[i]; i++){
			children.push(child);
		}

		if(preNodes){
			for(var i = 0, child; child = preNodes[i]; i++){
				this._tokenize(node, child, tokens);
			}
		}

		tokens.push([types.elem, node]);
		tokens.push([types.change, node]);

		for(var key in this._attributes){
			var value = "";
			if(key == "class"){
				value = node.className || value;
			}else if(key == "for"){
				value = node.htmlFor || value;
			}else if(node.getAttribute){
				value = node.getAttribute(key, 2) || value;
				if(key == "href" || key == "src"){
					if(dojo.isIE){
						var hash = location.href.lastIndexOf(location.hash);
						var href = location.href.substring(0, hash).split("/");
						href.pop();
						href = href.join("/") + "/";
						if(value.indexOf(href) == 0){
							value = value.replace(href, "");
						}
						value = value.replace(/%20/g, " ").replace(/%7B/g, "{").replace(/%7D/g, "}").replace(/%25/g, "%");
					}
					if(value.indexOf("{%") != -1 || value.indexOf("{{") != -1){
						node.setAttribute(key, "");
					}
				}
			}
			if(typeof value == "function"){
				value = value.toString().replace(this._re4, "$1");
			}
			if(typeof value == "string" && (value.indexOf("{%") != -1 || value.indexOf("{{") != -1 || (value && dojox.dtl.text.getTag("attr:" + key, true)))){
			//	this if clause was added to fix the bug with double rendering of 
			//	<textarea> and <button> innerHTML contents in IE7 inside the Workflow.StackContainer
				if(key != 'value' || key == 'value' && (node.tagName == 'INPUT' || node.tagName == 'OPTION' || node.tagName == 'PARAM' || node.tagName == 'LI' || node.tagName == 'DIV')){
				        tokens.push([types.attr, node, key, value]);
                                }
			}
		}

		if(!children.length){
			tokens.push([types.change, node.parentNode, true]);
			if(postNodes){
				for(var i = 0, child; child = postNodes[i]; i++){
					this._tokenize(node, child, tokens);
				}
			}
			return tokens;
		}

		for(var i = 0, child; child = children[i]; i++){
			this._tokenize(node, child, tokens);
		}

		if(node.parentNode && node.parentNode.tagName){
			tokens.push([types.change, node.parentNode, true]);
			node.parentNode.removeChild(node);
		}
		
		if(postNodes){
			for(var i = 0, child; child = postNodes[i]; i++){
				this._tokenize(node, child, tokens);
			}
		}

		if(first){
			tokens.push([types.change, node, true]);
		}

		return tokens;
	},
	_tokenize: function(parent, child, tokens){
		var types = this.types;
		var data = child.data;
		switch(child.nodeType){
			case 1:
				this.tokenize(child, tokens);
				break;
			case 3:
				if(data.match(/[^\s\n]/)){
					if(data.indexOf("{{") != -1 || data.indexOf("{%") != -1){
						var texts = dojox.dtl.text.tokenize(data);
						for(var j = 0, text; text = texts[j]; j++){
							if(typeof text == "string"){
								tokens.push([types.text, text]);
							}else{
								tokens.push(text);
							}
						}
					}else{
						tokens.push([child.nodeType, child]);
					}
				}
				if(child.parentNode) child.parentNode.removeChild(child);
				break;
			case 8:
				if(data.indexOf("{%") == 0){
					tokens.push([types.tag, this._trim(data.substring(2, data.length - 3))]);
				}
				if(data.indexOf("{{") == 0){
					tokens.push([types.varr, this._trim(data.substring(2, data.length - 3))]);
				}
				if(child.parentNode) child.parentNode.removeChild(child);
				break;
		}
	}
}

dojox.dtl.HtmlTemplate = function(/*String|dojo._Url*/ obj){
	// summary: Use this object for HTML templating
	var dd = dojox.dtl;
	var ddh = dd.html;

	if(!obj.node){
		if(typeof obj == "object"){
			obj = dojox.dtl.text.getTemplateString(obj);
		}
		obj = ddh.getTemplate(obj);
	}

	var tokens = ddh.tokenize(obj.node, [], obj.pres, obj.posts);
	var parser = new dd.HtmlParser(tokens);
	this.nodelist = parser.parse();
}
dojo.extend(dojox.dtl.HtmlTemplate, {
	_count: 0,
	_re: /\bdojo:([a-zA-Z0-9_]+)\b/g,
	setClass: function(str){
		this.getRootNode().className = str;
	},
	getRootNode: function(){
		return this.rootNode;
	},
	getBuffer: function(){
		return new dojox.dtl.HtmlBuffer();
	},
	render: function(context, buffer){
		buffer = buffer || this.getBuffer();
		this.rootNode = null;
		var onSetParent = dojo.connect(buffer, "onSetParent", this, function(node){
			if(!this.rootNode){
				this.rootNode = node || true;
			}
		});
		var output = this.nodelist.render(context || new dojox.dtl.Context({}), buffer);
		dojo.disconnect(onSetParent);
		buffer._flushCache();
		return output;
	},
	unrender: function(context, buffer){
		return this.nodelist.unrender(context, buffer);
	},
	toString: function(){ return "dojox.dtl.HtmlTemplate"; }
});

dojox.dtl.HtmlBuffer = function(/*Node*/ parent){
	// summary: Allows the manipulation of DOM
	// description:
	//		Use this to append a child, change the parent, or
	//		change the attribute of the current node.
	this._parent = parent;
	this._cache = [];
}
dojo.extend(dojox.dtl.HtmlBuffer, {
	concat: function(/*DOMNode*/ node){
		if(!this._parent) return this;
		if(node.nodeType){
			var caches = this._getCache(this._parent);
			if(node.parentNode === this._parent){
				// If we reach a node that already existed, fill in the cache for this same parent
				var i = 0;
				for(var i = 0, cache; cache = caches[i]; i++){
					this.onAddNode(node);
					this._parent.insertBefore(cache, node);
				}
				caches.length = 0;
			}
			if(!node.parentNode || !node.parentNode.tagName){
				if(!this._parent.childNodes.length){
					this.onAddNode(node);
					this._parent.appendChild(node);
				}else{
					caches.push(node);
				}
			}
		}
		return this;
	},
	remove: function(obj){
		if(typeof obj == "string"){
			this._parent.removeAttribute(obj);
		}else{
			if(obj.parentNode === this._parent){
				this.onRemoveNode();
				this._parent.removeChild(obj);
			}
		}
		return this;
	},
	setAttribute: function(key, value){
		if(key == "class"){
			this._parent.className = value;
		}else if(key == "for"){
			this._parent.htmlFor = value;
		}else if(this._parent.setAttribute){
			this._parent.setAttribute(key, value);
		}
		return this;
	},
	setParent: function(node, /*Boolean?*/ up){
		if(!this._parent) this._parent = node;
		var caches = this._getCache(this._parent);
		if(caches && caches.length && up){
			for(var i = 0, cache; cache = caches[i]; i++){
				if(cache !== this._parent && (!cache.parentNode || !cache.parentNode.tagName)){
					this.onAddNode(cache);
					this._parent.appendChild(cache);
				}
			}
			caches.length = 0;
		}

		this.onSetParent(node, up);
		this._parent = node;
		return this;
	},
	getParent: function(){
		return this._parent;
	},
	onSetParent: function(){
		// summary: Stub called when setParent is used.
	},
	onAddNode: function(){
		// summary: Stub called when new nodes are added
	},
	onRemoveNode: function(){
		// summary: Stub called when nodes are removed
	},
	_getCache: function(node){
		for(var i = 0, cache; cache = this._cache[i]; i++){
			if(cache[0] === node){
				return cache[1];
			}
		}
		var arr = [];
		this._cache.push([node, arr]);
		return arr;
	},
	_flushCache: function(node){
		for(var i = 0, cache; cache = this._cache[i]; i++){
			if(!cache[1].length){
				this._cache.splice(i--, 1);
			}
		}
	},
	toString: function(){ return "dojox.dtl.HtmlBuffer"; }
});

dojox.dtl.HtmlNode = function(node){
	// summary: Places a node into DOM
	this.contents = node;
}
dojo.extend(dojox.dtl.HtmlNode, {
	render: function(context, buffer){
		return buffer.concat(this.contents);
	},
	unrender: function(context, buffer){
		return buffer.remove(this.contents);
	},
	clone: function(buffer){
		return new dojox.dtl.HtmlNode(this.contents);
	},
	toString: function(){ return "dojox.dtl.HtmlNode"; }
});

dojox.dtl.HtmlNodeList = function(/*Node[]*/ nodes){
	// summary: A list of any HTML-specific node object
	// description:
	//		Any object that's used in the constructor or added
	//		through the push function much implement the
	//		render, unrender, and clone functions.
	this.contents = nodes || [];
}
dojo.extend(dojox.dtl.HtmlNodeList, {
	parents: new dojox.dtl.ObjectMap(),
	push: function(node){
		this.contents.push(node);
	},
	unshift: function(node){
		this.contents.unshift(node);
	},
	render: function(context, buffer, /*Node*/ instance){
		if(instance){
			var parent = buffer.getParent();
		}
		for(var i = 0; i < this.contents.length; i++){
			buffer = this.contents[i].render(context, buffer);
			if(!buffer) throw new Error("Template node render functions must return their buffer");
		}
		if(parent){
			buffer.setParent(parent, true);
		}
		return buffer;
	},
	unrender: function(context, buffer){
		for(var i = 0; i < this.contents.length; i++){
			buffer = this.contents[i].unrender(context, buffer);
			if(!buffer) throw new Error("Template node render functions must return their buffer");
		}
		return buffer;
	},
	clone: function(buffer){
		// summary:
		//		Used to create an identical copy of a NodeList, useful for things like the for tag.
		var dd = dojox.dtl;
		var ddh = dd.html;
		var parent = buffer.getParent();
		var contents = this.contents;
		var nodelist = new dd.HtmlNodeList();
		var cloned = [];
		for(var i = 0; i < contents.length; i++){
			var clone = contents[i].clone(buffer);
			if(clone instanceof dd.ChangeNode || clone instanceof dd.HtmlNode){
				var item = this.parents.get(clone.contents);
				if(item){
					clone.contents = item;
				}else if(parent !== clone.contents && clone instanceof dd.HtmlNode){
					var node = clone.contents;
					clone.contents = clone.contents.cloneNode(false);
					cloned.push(node);
					this.parents.put(node, clone.contents);
				}
			}
			nodelist.push(clone);
		}

		for(var i = 0, clone; clone = cloned[i]; i++){
			this.parents.put(clone);
		}

		return nodelist;
	},
	toString: function(){ return "dojox.dtl.HtmlNodeList"; }
});

dojox.dtl.HtmlVarNode = function(str){
	// summary: A node to be processed as a variable
	// description:
	//		Will render an object that supports the render function
	// 		and the getRootNode function
	this.contents = new dojox.dtl.Filter(str);
	this._lists = {};
}
dojo.extend(dojox.dtl.HtmlVarNode, {
	render: function(context, buffer){
		this._rendered = true;
		var dd = dojox.dtl;
		var ddh = dd.html;
		var str = this.contents.resolve(context);
		if(str && str.render && str.getRootNode){
			var root = this._curr = str.getRootNode();
			var lists = this._lists;
			var list = lists[root];
			if(!list){
				list = lists[root] = new dd.HtmlNodeList();
				list.push(new dd.ChangeNode(buffer.getParent()));
				list.push(new dd.HtmlNode(root));
				list.push(str);
				list.push(new dd.ChangeNode(buffer.getParent(), true));
			}
			return list.render(context, buffer);
		}else{
			if(!this._txt) this._txt = document.createTextNode(str);
			if(this._txt.data != str) this._txt.data = str;
			return buffer.concat(this._txt);
		}
		return buffer;
	},
	unrender: function(context, buffer){
		if(this._rendered){
			this._rendered = false;
			if(this._curr){
				return this._lists[this._curr].unrender(context, buffer);
			}else if(this._txt){
				return buffer.remove(this._txt);
			}
		}
		return buffer;
	},
	clone: function(){
		return new dojox.dtl.HtmlVarNode(this.contents.contents);
	},
	toString: function(){ return "dojox.dtl.HtmlVarNode"; }
});

dojox.dtl.ChangeNode = function(node, /*Boolean?*/ up){
	// summary: Changes the parent during render/unrender
	this.contents = node;
	this._up = up;
}
dojo.extend(dojox.dtl.ChangeNode, {
	render: function(context, buffer){
		return buffer.setParent(this.contents, this._up);
	},
	unrender: function(context, buffer){
		return buffer.setParent(this.contents);
	},
	clone: function(buffer){
		return new dojox.dtl.ChangeNode(this.contents, this._up);
	},
	toString: function(){ return "dojox.dtl.ChangeNode"; }
});

dojox.dtl.AttributeNode = function(key, value){
	// summary: Works on attributes
	this._key = key;
	this._value = value;
	this._tpl = new dojox.dtl.Template(value);
	this.contents = "";
}
dojo.extend(dojox.dtl.AttributeNode, {
	render: function(context, buffer){
		var key = this._key;
		var value = this._tpl.render(context);
		if(this._rendered){
			if(value != this.contents){
				this.contents = value;
				return buffer.setAttribute(key, value);
			}
		}else{
			this._rendered = true;
			this.contents = value;
			return buffer.setAttribute(key, value);
		}
		return buffer;
	},
	unrender: function(context, buffer){
		if(this._rendered){
			this._rendered = false;
			this.contents = "";
			return buffer.remove(this.contents);
		}
		return buffer;
	},
	clone: function(){
		return new dojox.dtl.AttributeNode(this._key, this._value);
	},
	toString: function(){ return "dojox.dtl.AttributeNode"; }
});

dojox.dtl.HtmlTextNode = function(str){
	// summary: Adds a straight text node without any processing
	this.contents = document.createTextNode(str);
}
dojo.extend(dojox.dtl.HtmlTextNode, {
	render: function(context, buffer){
		return buffer.concat(this.contents);
	},
	unrender: function(context, buffer){
		return buffer.remove(this.contents);
	},
	clone: function(){
		return new dojox.dtl.HtmlTextNode(this.contents.data);
	},
	toString: function(){ return "dojox.dtl.HtmlTextNode"; }
});

dojox.dtl.HtmlParser = function(tokens){
	// summary: Turn a simple array into a set of objects
	// description:
	//	This is also used by all tags to move through
	//	the list of nodes.
	this.contents = tokens;
}
dojo.extend(dojox.dtl.HtmlParser, {
	parse: function(/*Array?*/ stop_at){
		var dd = dojox.dtl;
		var ddh = dd.html;
		var types = ddh.types;
		var terminators = {};
		var tokens = this.contents;
		if(!stop_at){
			stop_at = [];
		}
		for(var i = 0; i < stop_at.length; i++){
			terminators[stop_at[i]] = true;
		}
		var nodelist = new dd.HtmlNodeList();
		while(tokens.length){
			var token = tokens.shift();
			var type = token[0];
			var value = token[1];
			if(type == types.change){
				nodelist.push(new dd.ChangeNode(value, token[2]));
			}else if(type == types.attr){
				var fn = dojox.dtl.text.getTag("attr:" + token[2], true);
				if(fn){
					nodelist.push(fn(null, token[2] + " " + token[3]));
				}else{
					nodelist.push(new dd.AttributeNode(token[2], token[3]));
				}
			}else if(type == types.elem){
				var fn = dojox.dtl.text.getTag("node:" + value.tagName.toLowerCase(), true);
				if(fn){
					// TODO: We need to move this to tokenization so that it's before the
					// 				node and the parser can be passed here instead of null
					nodelist.push(fn(null, value, value.tagName.toLowerCase()));
				}
				nodelist.push(new dd.HtmlNode(value));
			}else if(type == types.varr){
				nodelist.push(new dd.HtmlVarNode(value));
			}else if(type == types.text){
				nodelist.push(new dd.HtmlTextNode(value.data || value));
			}else if(type == types.tag){
				if(terminators[value]){
					tokens.unshift(token);
					return nodelist;
				}
				var cmd = value.split(/\s+/g);
				if(cmd.length){
					cmd = cmd[0];
					var fn = dojox.dtl.text.getTag(cmd);
					if(typeof fn != "function"){
						throw new Error("Function not found for ", cmd);
					}
					var tpl = fn(this, value);
					if(tpl){
						nodelist.push(tpl);
					}
				}
			}
		}

		if(stop_at.length){
			throw new Error("Could not find closing tag(s): " + stop_at.toString());
		}

		return nodelist;
	},
	next: function(){
		// summary: Used by tags to discover what token was found
		var token = this.contents.shift();
		return {type: token[0], text: token[1]};
	},
	skipPast: function(endtag){
		return dojox.dtl.Parser.prototype.skipPast.call(this, endtag);
	},
	getVarNode: function(){
		return dojox.dtl.HtmlVarNode;
	},
	getTextNode: function(){
		return dojox.dtl.HtmlTextNode;
	},
	getTemplate: function(/*String*/ loc){
		return new dojox.dtl.HtmlTemplate(dojox.dtl.html.getTemplate(loc));
	},
	toString: function(){ return "dojox.dtl.HtmlParser"; }
});

dojox.dtl.register.tag("dojox.dtl.tag.event", "dojox.dtl.tag.event", [[/(attr:)?on(click|key(up))/i, "on"]]);
dojox.dtl.register.tag("dojox.dtl.tag.html", "dojox.dtl.tag.html", ["html", "attr:attach", "attr:tstyle"]);

}

if(!dojo._hasResource["dojox.dtl.render.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.render.html"] = true;
dojo.provide("dojox.dtl.render.html");

dojox.dtl.render.html.sensitivity = {
	// summary:
	//		Set conditions under which to buffer changes
	// description:
	//		Necessary if you make a lot of changes to your template.
	//		What happens is that the entire node, from the attached DOM Node
	//		down gets swapped with a clone, and until the entire rendering
	//		is complete, we don't replace the clone again. In this way, renders are
	//		"batched".
	//
	//		But, if we're only changing a small number of nodes, we might no want to buffer at all.
	//		The higher numbers mean that even small changes will result in buffering.
	//		Each higher level includes the lower levels.
	NODE: 1, // If a node changes, implement buffering
	ATTRIBUTE: 2, // If an attribute or node changes, implement buffering
	TEXT: 3 // If any text at all changes, implement buffering
}
dojox.dtl.render.html.Render = function(/*DOMNode?*/ attachPoint, /*dojox.dtl.HtmlTemplate?*/ tpl){
	this._tpl = tpl;
	this._node = attachPoint;
	this._swap = dojo.hitch(this, function(){
		// summary: Swaps the node out the first time the DOM is changed
		// description: Gets swapped back it at end of render
		if(this._node === this._tpl.getRootNode()){
			var frag = this._node;
			this._node = this._node.cloneNode(true);
			frag.parentNode.replaceChild(this._node, frag);
		}
	});
}
dojo.extend(dojox.dtl.render.html.Render, {
	sensitivity: dojox.dtl.render.html.sensitivity,
	setAttachPoint: function(/*Node*/ node){
		this._node = node;
	},
	render: function(/*dojox.dtl.HtmlTemplate*/ tpl, /*Object*/ context, /*dojox.dtl.HtmlBuffer?*/ buffer){
		if(!this._node){
			throw new Error("You cannot use the Render object without specifying where you want to render it");
		}

		buffer = buffer || tpl.getBuffer();

		if(context.getThis() && context.getThis().buffer == this.sensitivity.NODE){
			var onAddNode = dojo.connect(buffer, "onAddNode", this, "_swap");
			var onRemoveNode = dojo.connect(buffer, "onRemoveNode", this, "_swap");
		}

		if(this._tpl && this._tpl !== tpl){
			this._tpl.unrender(context, buffer);
		}
		this._tpl = tpl;

		var frag = tpl.render(context, buffer).getParent();

		dojo.disconnect(onAddNode);
		dojo.disconnect(onRemoveNode);

		if(this._node !== frag){
			this._node.parentNode.replaceChild(frag, this._node);
			dojo._destroyElement(this._node);
			this._node = frag;
		}
	}
});

}

if(!dojo._hasResource["dojox.dtl.widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.widget"] = true;
dojo.provide("dojox.dtl.widget");






dojo.declare("dojox.dtl._Widget", [dijit._Widget, dijit._Contained],
	{
		buffer: 0,
		buildRendering: function(){
			this.domNode = this.srcNodeRef;

			if(this.domNode){
				var parent = this.getParent();
				if(parent){
					this.setAttachPoint(parent);
				}
			}
		},
		setAttachPoint: function(/*dojox.dtl.AttachPoint*/ attach){
			this._attach = attach;
		},
		render: function(/*dojox.dtl.HtmlTemplate*/ tpl, /*dojox.dtl.Context*/ context){
			if(!this._attach){
				throw new Error("You must use an attach point with dojox.dtl.TemplatedWidget");
			}

			context.setThis(this);
			this._attach.render(tpl, context);
		}
	}
);

dojo.declare("dojox.dtl.AttachPoint", [dijit._Widget, dijit._Container],
	{
		constructor: function(props, node){
			this._render = new dojox.dtl.render.html.Render(node);
		},
		render: function(/*dojox.dtl.HtmlTemplate*/ tpl, /*dojox.dtl.Context*/ context){
			this._render.render(tpl, context);
		}
	}
);

}

if(!dojo._hasResource["dojox.date.php"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.php"] = true;
dojo.provide("dojox.date.php");


dojox.date.php.format = function(/*Date*/ date, /*String*/ format, /*Object?*/ overrides){
	// summary: Get a formatted string for a given date object
	var df = new dojox.date.php.DateFormat(date);
	return df.format(format, overrides);	
}

dojox.date.php.DateFormat = function(/*Date*/ date){
	this.date = date;
}
dojo.extend(dojox.date.php.DateFormat, {
	weekdays: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
	weekdays_3: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
	months: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
	months_3: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
	monthdays: [31,28,31,30,31,30,31,31,30,31,30,31],

	format: function(/*String*/ format, /*Object?*/ overrides){
		// summary: Format the internal date object
		var parts = [];
		for(var i = 0; i < format.length; i++){
			var chr = format.charAt(i);
			if(overrides && typeof overrides[chr] == "function"){
				parts.push(overrides[chr].call(this));
			}else if(typeof this[chr] == "function"){
				parts.push(this[chr]());
			}else{
				parts.push(chr);
			}
		}
		return parts.join("");
	},

	// Day

	d: function(){
		// summary: Day of the month, 2 digits with leading zeros
		var j = this.j();
		return (j.length == 1) ? "0" + j : j;
	},

	D: function(){
		// summary: A textual representation of a day, three letters
		return this.weekdays_3[this.date.getDay()];
	},

	j: function(){
		// summary: Day of the month without leading zeros
		return this.date.getDate() + "";
	},

	l: function(){
		// summary: A full textual representation of the day of the week
		return this.weekdays[this.date.getDay()];
	},
	
	N: function(){
		// summary: ISO-8601 numeric representation of the day of the week (added in PHP 5.1.0)
		var w = this.w();
		return (!w) ? 7 : w;
	},

	S: function(){
		// summary: English ordinal suffix for the day of the month, 2 characters
		switch(this.date.getDate()){
			case 11: case 12: case 13: return "th";
			case 1: case 21: case 31: return "st";
			case 2: case 22: return "nd";
			case 3: case 23: return "rd";
			default: return "th";
		}
	},

	w: function(){
		// summary: Numeric representation of the day of the week
		return this.date.getDay() + "";
	},

	z: function(){
		// summary: The day of the year (starting from 0)
		var millis = this.date.getTime() - new Date(this.date.getFullYear(), 0, 1).getTime();
		return Math.floor(millis/86400000) + "";
	},

	// Week

	W: function(){
		// summary: ISO-8601 week number of year, weeks starting on Monday (added in PHP 4.1.0)
		var week;
		var jan1_w = new Date(this.date.getFullYear(), 0, 1).getDay() + 1;
		var w = this.date.getDay() + 1;
		var z = parseInt(this.z());

		if(z <= (8 - jan1_w) && jan1_w > 4){
			var last_year = new Date(this.date.getFullYear() - 1, this.date.getMonth(), this.date.getDate());
			if(jan1_w == 5 || (jan1_w == 6 && dojo.date.isLeapYear(last_year))){
				week = 53;
			}else{
				week = 52;
			}
		}else{
			var i;
			if(Boolean(this.L())){
				i = 366;
			}else{
				i = 365;
			}
			if((i - z) < (4 - w)){
				week = 1;
			}else{
				var j = z + (7 - w) + (jan1_w - 1);
				week = Math.ceil(j / 7);
				if(jan1_w > 4){
					--week;
				}
			}
		}
		
		return week;
	},

	// Month

	F: function(){
		// summary: A full textual representation of a month, such as January or March
		return this.months[this.date.getMonth()];
	},

	m: function(){
		// summary: Numeric representation of a month, with leading zeros
		var n = this.n();
		return (n.length == 1) ? "0" + n : n;
	},

	M: function(){
		// summary: A short textual representation of a month, three letters
		return months_3[this.date.getMonth()];
	},

	n: function(){
		// summary: Numeric representation of a month, without leading zeros
		return this.date.getMonth() + 1 + "";
	},

	t: function(){
		// summary: Number of days in the given month
		return (Boolean(this.L()) && this.date.getMonth() == 1) ? 29 : this.monthdays[this.getMonth()];
	},

	// Year

	L: function(){
		// summary: Whether it's a leap year
		return (dojo.date.isLeapYear(this.date)) ? "1" : "0";
	},

	o: function(){
		// summary:
		//		ISO-8601 year number. This has the same value as Y, except that if
		//		the ISO week number (W) belongs to the previous or next year, that year is used instead. (added in PHP 5.1.0)
		// TODO: Figure out what this means
	},

	Y: function(){
		// summary: A full numeric representation of a year, 4 digits
		return this.date.getFullYear() + "";
	},

	y: function(){
		// summary: A two digit representation of a year
		return this.date.getFullYear.substsring(2, 4);
	},

	// Time

	a: function(){
		// summary: Lowercase Ante meridiem and Post meridiem
		return this.date.getHours() >= 12 ? "pm" : "am";
	},

	b: function(){
		// summary: Uppercase Ante meridiem and Post meridiem
		return this.a().toUpperCase();
	},

	B: function(){
		// summary:
		//	Swatch Internet time
		//	A day is 1,000 beats. All time is measured from GMT + 1
		var off = this.date.getTimezoneOffset() + 60;
		var secs = (this.date.getHours() * 3600) + (this.date.getMinutes() * 60) + this.getSeconds() + (off * 60);
		var beat = Math.abs(Math.floor(secs / 86.4) % 1000) + "";
		while(beat.length <  2) beat = "0" + beat;
		return beat;
	},

	g: function(){
		// summary: 12-hour format of an hour without leading zeros
		return (this.date.getHours() > 12) ? this.date.getHours() - 12 + "" : this.date.getHours() + "";
	},

	G: function(){
		// summary: 24-hour format of an hour without leading zeros
		return this.date.getHours() + "";
	},

	h: function(){
		// summary: 12-hour format of an hour with leading zeros
		var g = this.g();
		return (g.length == 1) ? "0" + g : g;
	},

	H: function(){
		// summary: 24-hour format of an hour with leading zeros
		var G = this.G();
		return (G.length == 1) ? "0" + G : G;
	},

	i: function(){
		// summary: Minutes with leading zeros
		var mins = this.date.getMinutes() + "";
		return (mins.length == 1) ? "0" + mins : mins;
	},

	s: function(){
		// summary: Seconds, with leading zeros
		var secs = this.date.getSeconds() + "";
		return (secs.length == 1) ? "0" + secs : secs;
	},

	// Timezone

	e: function(){
		// summary: Timezone identifier (added in PHP 5.1.0)
		return dojo.date.getTimezoneName(this.date);
	},

	I: function(){
		// summary: Whether or not the date is in daylight saving time
		// TODO: Can dojo.date do this?
	},

	O: function(){
		// summary: Difference to Greenwich time (GMT) in hours
		var off = Math.abs(this.date.getTimezoneOffset());
		var hours = Math.floor(off / 60) + "";
		var mins = (off % 60) + "";
		if(hours.length == 1) hours = "0" + hours;
		if(mins.length == 1) hours = "0" + mins;
		return ((this.date.getTimezoneOffset() < 0) ? "+" : "-") + hours + mins;
	},

	P: function(){
		// summary: Difference to Greenwich time (GMT) with colon between hours and minutes (added in PHP 5.1.3)
		var O = this.O();
		return O.substring(0, 2) + ":" + O.substring(2, 4);
	},

	T: function(){
		// summary: Timezone abbreviation

		// Guess...
		return this.e().substring(0, 3);
	},

	Z: function(){
		// summary:
		//		Timezone offset in seconds. The offset for timezones west of UTC is always negative,
		//		and for those east of UTC is always positive.
		return this.date.getTimezoneOffset() * -60;
	},

	// Full Date/Time

	c: function(){
		// summary: ISO 8601 date (added in PHP 5)
		return this.Y() + "-" + this.m() + "-" + this.d() + "T" + this.h() + ":" + this.i() + ":" + this.s() + this.P();
	},

	r: function(){
		// summary: RFC 2822 formatted date
		return this.D() + ", " + this.d() + " " + this.M() + " " + this.Y() + " " + this.H() + ":" + this.i() + ":" + this.s() + " " + this.O();
	},

	U: function(){
		// summary: Seconds since the Unix Epoch (January 1 1970 00:00:00 GMT)
		return Math.floor(this.date.getTime() / 1000);
	}

});

}

if(!dojo._hasResource["dojox.dtl.utils.date"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.utils.date"] = true;
dojo.provide("dojox.dtl.utils.date");



dojo.mixin(dojox.dtl.utils.date, {
	format: function(/*Date*/ date, /*String*/ format){
		return dojox.date.php.format(date, format, dojox.dtl.utils.date._overrides);
	},
	timesince: function(d, now){
		// summary:
		//		Takes two datetime objects and returns the time between then and now
		//		as a nicely formatted string, e.g "10 minutes"
		// description:
		//		Adapted from http://blog.natbat.co.uk/archive/2003/Jun/14/time_since
		if(!(d instanceof Date)){
			d = new Date(d.year, d.month, d.day);
		}
		if(!now){
			now = new Date();
		}

		var delta = Math.abs(now.getTime() - d.getTime());
		for(var i = 0, chunk; chunk = dojox.dtl.utils.date._chunks[i]; i++){
			var count = Math.floor(delta / chunk[0]);
			if(count) break;
		}
		return count + " " + chunk[1](count);
	},
	_chunks: [
		[60 * 60 * 24 * 365 * 1000, function(n){ return (n == 1) ? 'year' : 'years'; }],
		[60 * 60 * 24 * 30 * 1000, function(n){ return (n == 1) ? 'month' : 'months'; }],
		[60 * 60 * 24 * 7 * 1000, function(n){ return (n == 1) ? 'week' : 'weeks'; }],
		[60 * 60 * 24 * 1000, function(n){ return (n == 1) ? 'day' : 'days'; }],
		[60 * 60 * 1000, function(n){ return (n == 1) ? 'hour' : 'hours'; }],
		[60 * 1000, function(n){ return (n == 1) ? 'minute' : 'minutes'; }]
	],
	_months_ap: ["Jan.", "Feb.", "March", "April", "May", "June", "July", "Aug.", "Sept.", "Oct.", "Nov.", "Dec."],
	_overrides: {
		f: function(){
			// summary:
			//		Time, in 12-hour hours and minutes, with minutes left off if they're zero.
			// description: 
			//		Examples: '1', '1:30', '2:05', '2'
			//		Proprietary extension.
			if(!this.date.getMinutes()) return this.g();
		},
		N: function(){
			// summary: Month abbreviation in Associated Press style. Proprietary extension.
			return dojox.dtl.utils.date._months_ap[this.date.getMonth()];
		},
		P: function(){
			// summary:
			//		Time, in 12-hour hours, minutes and 'a.m.'/'p.m.', with minutes left off
			//		if they're zero and the strings 'midnight' and 'noon' if appropriate.
			// description:
			//		Examples: '1 a.m.', '1:30 p.m.', 'midnight', 'noon', '12:30 p.m.'
			//		Proprietary extension.
			if(!this.date.getMinutes() && !this.date.getHours()) return 'midnight';
			if(!this.date.getMinutes() && this.date.getHours() == 12) return 'noon';
			return self.f() + " " + self.a();
		}
	}
});

}

if(!dojo._hasResource["dojox.dtl.filter.dates"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.filter.dates"] = true;
dojo.provide("dojox.dtl.filter.dates");



dojo.mixin(dojox.dtl.filter.dates, {
	date: function(value, arg){
		// summary: Formats a date according to the given format
		if(!value || !(value instanceof Date)) return "";
		arg = arg || "N j, Y";
		return dojox.dtl.utils.date.format(value, arg);
	},
	time: function(value, arg){
		// summary: Formats a time according to the given format
		if(!value || !(value instanceof Date)) return "";
		arg = arg || "P";
		return dojox.dtl.utils.date.format(value, arg);
	},
	timesince: function(value, arg){
		// summary: Formats a date as the time since that date (i.e. "4 days, 6 hours")
		var timesince = dojox.dtl.utils.date.timesince;
		if(!value) return "";
		if(arg) return timesince(arg, value);
		return timesince(value);
	},
	timeuntil: function(value, arg){
		// summary: Formats a date as the time until that date (i.e. "4 days, 6 hours")
		var timesince = dojox.dtl.utils.date.timesince;
		if(!value) return "";
		if(arg) return timesince(arg, value);
		return timesince(new Date(), value);
	}
});

}

if(!dojo._hasResource["dojox.dtl.filter.htmlstrings"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.filter.htmlstrings"] = true;
dojo.provide("dojox.dtl.filter.htmlstrings");



dojo.mixin(dojox.dtl.filter.htmlstrings, {
	_escapeamp: /&/g,
	_escapelt: /</g,
	_escapegt: />/g,
	_escapeqt: /'/g,
	_escapedblqt: /"/g,
	_linebreaksrn: /(\r\n|\n\r)/g,
	_linebreaksn: /\n{2,}/g,
	_linebreakss: /(^\s+|\s+$)/g,
	_linebreaksbr: /\n/g,
	_removetagsfind: /[a-z0-9]+/g,
	_striptags: /<[^>]*?>/g,
	escape: function(value){
		// summary: Escapes a string's HTML
		var dh = dojox.dtl.filter.htmlstrings;
		return value.replace(dh._escapeamp, '&amp;').replace(dh._escapelt, '&lt;').replace(dh._escapegt, '&gt;').replace(dh._escapedblqt, '&quot;').replace(dh._escapeqt, '&#39;');
	},
	linebreaks: function(value){
		// summary: Converts newlines into <p> and <br />s
		var output = [];
		var dh = dojox.dtl.filter.htmlstrings;
		value = value.replace(dh._linebreaksrn, "\n");
		var parts = value.split(dh._linebreaksn);
		for(var i = 0; i < parts.length; i++){
			var part = parts[i].replace(dh._linebreakss, "").replace(dh._linebreaksbr, "<br />")
			output.push("<p>" + part + "</p>");
		}

		return output.join("\n\n");
	},
	linebreaksbr: function(value){
		// summary: Converts newlines into <br />s
		var dh = dojox.dtl.filter.htmlstrings;
		return value.replace(dh._linebreaksrn, "\n").replace(dh._linebreaksbr, "<br />");
	},
	removetags: function(value, arg){
		// summary: Removes a space separated list of [X]HTML tags from the output"
		var dh = dojox.dtl.filter.htmlstrings;
		var tags = [];
		var group;
		while(group = dh._removetagsfind.exec(arg)){
			tags.push(group[0]);
		}
		tags = "(" + tags.join("|") + ")";
		return value.replace(new RegExp("</?\s*" + tags + "\s*[^>]*>", "gi"), "");
	},
	striptags: function(value){
		// summary: Strips all [X]HTML tags
		return value.replace(dojox.dtl.filter.htmlstrings._striptags, "");
	}
});

}

if(!dojo._hasResource["dojox.dtl.filter.integers"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.filter.integers"] = true;
dojo.provide("dojox.dtl.filter.integers");

dojo.mixin(dojox.dtl.filter.integers, {
	add: function(value, arg){
		value = parseInt(value);
		arg = parseInt(arg);
		return isNaN(arg) ? value : value + arg;
	},
	get_digit: function(value, arg){
		// summary:
		//		Given a whole number, returns the 1-based requested digit of it
		// desciprtion:
		//		1 is the right-most digit, 2 is the second-right-most digit, etc. Returns the
		//		original value for invalid input (if input or argument is not an integer,
		//		or if argument is less than 1). Otherwise, output is always an integer.
		value = parseInt(value);
		arg = parseInt(arg) - 1;
		if(arg >= 0){
			value += "";
			if(arg < value.length){
				value = parseInt(value.charAt(arg));
			}else{
				value = 0;
			}
		}
		return (isNaN(value) ? 0 : value);
	}
});

}

if(!dojo._hasResource["dojox.dtl.filter.lists"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.filter.lists"] = true;
dojo.provide("dojox.dtl.filter.lists")



dojo.mixin(dojox.dtl.filter.lists, {
	_dictsort: function(a, b){
		if(a[0] == b[0]) return 0;
		return (a[0] < b[0]) ? -1 : 1;
	},
	dictsort: function(value, arg){
		// summary: Takes a list of dicts, returns that list sorted by the property given in the argument.
		if(!arg) return value;

		var items = [];
		for(var key in value){
			items.push([dojox.dtl.resolveVariable('var.' + arg, new dojox.dtl.Context({ 'var' : value[key]})), value[key]]);
		}
		items.sort(dojox.dtl.filter.lists._dictsort);
		var output = [];
		for(var i = 0, item; item = items[i]; i++){
			output.push(item[1]);
		}
		return output;
	},
	dictsortreversed: function(value, arg){
		// summary: Takes a list of dicts, returns that list sorted in reverse order by the property given in the argument.
		if(!arg) return value;

		var dictsort = dojox.dtl.filter.lists.dictsort(value, arg);
		return dictsort.reverse();
	},
	first: function(value){
		// summary: Returns the first item in a list
		return (value.length) ? value[0] : "";
	},
	join: function(value, arg){
		// summary: Joins a list with a string, like Python's ``str.join(list)``
		// description:
		//		Django throws a compile error, but JS can't do arg checks
		//		so we're left with run time errors, which aren't wise for something
		//		as trivial here as an empty arg.
		return value.join(arg || ",");
	},
	length: function(value){
		// summary: Returns the length of the value - useful for lists
		return (isNaN(value.length)) ? (value + "").length : value.length;
	},
	length_is: function(value, arg){
		// summary: Returns a boolean of whether the value's length is the argument
		return value.length == parseInt(arg);
	},
	random: function(value){
		// summary: Returns a random item from the list
		return value[Math.floor(Math.random() * value.length)];
	},
	slice: function(value, arg){
		// summary: Returns a slice of the list.
		// description:
		//		Uses the same syntax as Python's list slicing; see
		//		http://diveintopython.org/native_data_types/lists.html#odbchelper.list.slice
		//		for an introduction.
		//		Also uses the optional third value to denote every X item.
		arg = arg || "";
		var parts = arg.split(":");
		var bits = [];
		for(var i = 0; i < parts.length; i++){
			if(!parts[i].length){
				bits.push(null);
			}else{
				bits.push(parseInt(parts[i]));
			}
		}

		if(bits[0] === null){
			bits[0] = 0;
		}
		if(bits[0] < 0){
			bits[0] = value.length + bits[0];
		}
		if(bits.length < 2 || bits[1] === null){
			bits[1] = value.length;
		}
		if(bits[1] < 0){
			bits[1] = value.length + bits[1];
		}
		
		return value.slice(bits[0], bits[1]);
	},
	_unordered_list: function(value, tabs){
		var ddl = dojox.dtl.filter.lists;
		var indent = "";
		for(var i = 0; i < tabs; i++){
			indent += "\t";
		}
		if(value[1] && value[1].length){
			var recurse = [];
			for(var i = 0; i < value[1].length; i++){
				recurse.push(ddl._unordered_list(value[1][i], tabs + 1))
			}
			return indent + "<li>" + value[0] + "\n" + indent + "<ul>\n" + recurse.join("\n") + "\n" + indent + "</ul>\n" + indent + "</li>";
		}else{
			return indent + "<li>" + value[0] + "</li>";
		}
	},
	unordered_list: function(value){
		// summary:
		//		Recursively takes a self-nested list and returns an HTML unordered list --
		//		WITHOUT opening and closing <ul> tags.
		//	description:
		//		The list is assumed to be in the proper format. For example, if ``var`` contains
		//		``['States', [['Kansas', [['Lawrence', []], ['Topeka', []]]], ['Illinois', []]]]``,
		//		then ``{{ var|unordered_list }}`` would return::
		//
		//		<li>States
		//		<ul>
		//			<li>Kansas
		//			<ul>
		//				<li>Lawrence</li>
		//				<li>Topeka</li>
		//			</ul>
		//			</li>
		//			<li>Illinois</li>
		//		</ul>
		//		</li>
		return dojox.dtl.filter.lists._unordered_list(value, 1);
	}
});

}

if(!dojo._hasResource["dojox.dtl.filter.logic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.filter.logic"] = true;
dojo.provide("dojox.dtl.filter.logic");

dojo.mixin(dojox.dtl.filter.logic, {
	default_: function(value, arg){
		// summary: If value is unavailable, use given default
		return value || arg || "";
	},
	default_if_none: function(value, arg){
		// summary: If value is null, use given default
		return (value === null) ? arg || "" : value || "";
	},
	divisibleby: function(value, arg){
		// summary: Returns true if the value is devisible by the argument"
		return (parseInt(value) % parseInt(arg)) == 0;
	},
	_yesno: /\s*,\s*/g,
	yesno: function(value, arg){
		// summary:
		//		arg being a comma-delimited string, value of true/false/none
		//		chooses the appropriate item from the string
		if(!arg) arg = 'yes,no,maybe';
		var parts = arg.split(dojox.dtl.filter.logic._yesno);
		if(parts.length < 2){
			return value;
		}
		if(value) return parts[0];
		if((!value && value !== null) || parts.length < 3) return parts[1];
		return parts[2];
	}
});

}

if(!dojo._hasResource["dojox.dtl.filter.misc"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.filter.misc"] = true;
dojo.provide("dojox.dtl.filter.misc");

dojo.mixin(dojox.dtl.filter.misc, {
	filesizeformat: function(value){
		// summary: Format the value like a 'human-readable' file size (i.e. 13 KB, 4.1 MB, 102bytes, etc).
		value = parseFloat(value);
		if(value < 1024){
			return (value == 1) ? value + " byte" : value + " bytes";
		}else if(value < 1024 * 1024){
			return (value / 1024).toFixed(1) + " KB";
		}else if(value < 1024 * 1024 * 1024){
			return (value / 1024 / 1024).toFixed(1) + " MB";
		}
		return (value / 1024 / 1024 / 1024).toFixed(1) + " GB";
	},
	pluralize: function(value, arg){
		// summary:
		//		Returns a plural suffix if the value is not 1, for '1 vote' vs. '2 votes'
		//	description:
		//		By default, 's' is used as a suffix; if an argument is provided, that string
		//		is used instead. If the provided argument contains a comma, the text before
		//		the comma is used for the singular case.
		arg = arg || 's';
		if(arg.indexOf(",") == -1){
			arg = "," + arg;
		}
		var parts = arg.split(",");
		if(parts.length > 2){
			return "";
		}
		var singular = parts[0];
		var plural = parts[1];

		if(parseInt(value) != 1){
			return plural;
		}
		return singular;
	},
	_phone2numeric: { a: 2, b: 2, c: 2, d: 3, e: 3, f: 3, g: 4, h: 4, i: 4, j: 5, k: 5, l: 5, m: 6, n: 6, o: 6, p: 7, r: 7, s: 7, t: 8, u: 8, v: 8, w: 9, x: 9, y: 9 },
	phone2numeric: function(value){
		// summary: Takes a phone number and converts it in to its numerical equivalent
		var dm = dojox.dtl.filter.misc;
		value = value + "";
		var output = "";
		for(var i = 0; i < value.length; i++){
			var chr = value.charAt(i).toLowerCase();
			(dm._phone2numeric[chr]) ? output += dm._phone2numeric[chr] : output += value.charAt(i);
		}
		return output;
	},
	pprint: function(value){
		// summary: A wrapper around toJson unless something better comes along
		return dojo.toJson(value);
	}
});

}

if(!dojo._hasResource["dojox.string.sprintf"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.string.sprintf"] = true;
dojo.provide("dojox.string.sprintf");



dojox.string.sprintf = function(/*String*/ format, /*mixed...*/ filler){
	for(var args = [], i = 1; i < arguments.length; i++){
		args.push(arguments[i]);
	}
	var formatter = new dojox.string.sprintf.Formatter(format);
	return formatter.format.apply(formatter, args);
}

dojox.string.sprintf.Formatter = function(/*String*/ format){
	var tokens = [];
	this._mapped = false;
	this._format = format;
	this._tokens = dojox.string.tokenize(format, this._re, this._parseDelim, this);
}
dojo.extend(dojox.string.sprintf.Formatter, {
	_re: /\%(?:\(([\w_]+)\)|([1-9]\d*)\$)?([0 +\-\#]*)(\*|\d+)?(\.)?(\*|\d+)?[hlL]?([\%scdeEfFgGiouxX])/g,
	_parseDelim: function(mapping, intmapping, flags, minWidth, period, precision, specifier){
		if(mapping){
			this._mapped = true;
		}
		return {
			mapping: mapping,
			intmapping: intmapping,
			flags: flags,
			_minWidth: minWidth, // May be dependent on parameters
			period: period,
			_precision: precision, // May be dependent on parameters
			specifier: specifier
		};
	},
	_specifiers: {
		b: {
			base: 2,
			isInt: true
		},
		o: {
			base: 8,
			isInt: true
		},
		x: {
			base: 16,
			isInt: true
		},
		X: {
			extend: ["x"],
			toUpper: true
		},
		d: {
			base: 10,
			isInt: true
		},
		i: {
			extend: ["d"]
		},
		u: {
			extend: ["d"],
			isUnsigned: true
		},
		c: {
			setArg: function(token){
				if(!isNaN(token.arg)){
					var num = parseInt(token.arg);
					if(num < 0 || num > 127){
						throw new Error("invalid character code passed to %c in sprintf");
					}
					token.arg = isNaN(num) ? "" + num : String.fromCharCode(num);
				}
			}
		},
		s: {
			setMaxWidth: function(token){
				token.maxWidth = (token.period == ".") ? token.precision : -1;
			}
		},
		e: {
			isDouble: true,
			doubleNotation: "e"
		},
		E: {
			extend: ["e"],
			toUpper: true
		},
		f: {
			isDouble: true,
			doubleNotation: "f"
		},
		F: {
			extend: ["f"]
		},
		g: {
			isDouble: true,
			doubleNotation: "g"
		},
		G: {
			extend: ["g"],
			toUpper: true
		}
	},
	format: function(/*mixed...*/ filler){
		if(this._mapped && typeof filler != "object"){
			throw new Error("format requires a mapping");
		}

		var str = "";
		var position = 0;
		for(var i = 0, token; i < this._tokens.length; i++){
			token = this._tokens[i];
			if(typeof token == "string"){
				str += token;
			}else{
				if(this._mapped){
					if(typeof filler[token.mapping] == "undefined"){
						throw new Error("missing key " + token.mapping);
					}
					token.arg = filler[token.mapping];
				}else{
					if(token.intmapping){
						var position = parseInt(token.intmapping) - 1;
					}
					if(position >= arguments.length){
						throw new Error("got " + arguments.length + " printf arguments, insufficient for '" + this._format + "'");
					}
					token.arg = arguments[position++];
				}

				if(!token.compiled){
					token.compiled = true;
					token.sign = "";
					token.zeroPad = false;
					token.rightJustify = false;
					token.alternative = false;

					var flags = {};
					for(var fi = token.flags.length; fi--;){
						var flag = token.flags.charAt(fi);
						flags[flag] = true;
						switch(flag){
							case " ":
								token.sign = " ";
								break;
							case "+":
								token.sign = "+";
								break;
							case "0":
								token.zeroPad = (flags["-"]) ? false : true;
								break;
							case "-":
								token.rightJustify = true;
								token.zeroPad = false;
								break;
							case "\#":
								token.alternative = true;
								break;
							default:
								throw Error("bad formatting flag '" + token.flags.charAt(fi) + "'");
						}
					}

					token.minWidth = (token._minWidth) ? parseInt(token._minWidth) : 0;
					token.maxWidth = -1;
					token.toUpper = false;
					token.isUnsigned = false;
					token.isInt = false;
					token.isDouble = false;
					token.precision = 1;
					if(token.period == '.'){
						if(token._precision){
							token.precision = parseInt(token._precision);
						}else{
							token.precision = 0;
						}
					}

					var mixins = this._specifiers[token.specifier];
					if(typeof mixins == "undefined"){
						throw new Error("unexpected specifier '" + token.specifier + "'");
					}
					if(mixins.extend){
						dojo.mixin(mixins, this._specifiers[mixins.extend]);
						delete mixins.extend;
					}
					dojo.mixin(token, mixins);
				}

				if(typeof token.setArg == "function"){
					token.setArg(token);
				}

				if(typeof token.setMaxWidth == "function"){
					token.setMaxWidth(token);
				}

				if(token._minWidth == "*"){
					if(this._mapped){
						throw new Error("* width not supported in mapped formats");
					}
					token.minWidth = parseInt(arguments[position++]);
					if(isNaN(token.minWidth)){
						throw new Error("the argument for * width at position " + position + " is not a number in " + this._format);
					}
					// negative width means rightJustify
					if (token.minWidth < 0) {
						token.rightJustify = true;
						token.minWidth = -token.minWidth;
					}
				}

				if(token._precision == "*" && token.period == "."){
					if(this._mapped){
						throw new Error("* precision not supported in mapped formats");
					}
					token.precision = parseInt(arguments[position++]);
					if(isNaN(token.precision)){
						throw Error("the argument for * precision at position " + position + " is not a number in " + this._format);
					}
					// negative precision means unspecified
					if (token.precision < 0) {
						token.precision = 1;
						token.period = '';
					}
				}

				if(token.isInt){
					// a specified precision means no zero padding
					if(token.period == '.'){
						token.zeroPad = false;
					}
					this.formatInt(token);
				}else if(token.isDouble){
					if(token.period != '.'){
						token.precision = 6;
					}
					this.formatDouble(token); 
				}
				this.fitField(token);

				str += "" + token.arg;
			}
		}

		return str;
	},
	_zeros10: '0000000000',
	_spaces10: '          ',
	formatInt: function(token) {
		var i = parseInt(token.arg);
		if(!isFinite(i)){ // isNaN(f) || f == Number.POSITIVE_INFINITY || f == Number.NEGATIVE_INFINITY)
			// allow this only if arg is number
			if(typeof token.arg != "number"){
				throw new Error("format argument '" + token.arg + "' not an integer; parseInt returned " + i);
			}
			//return '' + i;
			i = 0;
		}

		// if not base 10, make negatives be positive
		// otherwise, (-10).toString(16) is '-a' instead of 'fffffff6'
		if(i < 0 && (token.isUnsigned || token.base != 10)){
			i = 0xffffffff + i + 1;
		} 

		if(i < 0){
			token.arg = (- i).toString(token.base);
			this.zeroPad(token);
			token.arg = "-" + token.arg;
		}else{
			token.arg = i.toString(token.base);
			// need to make sure that argument 0 with precision==0 is formatted as ''
			if(!i && !token.precision){
				token.arg = "";
			}else{
				this.zeroPad(token);
			}
			if(token.sign){
				token.arg = token.sign + token.arg;
			}
		}
		if(token.base == 16){
			if(token.alternative){
				token.arg = '0x' + token.arg;
			}
			toke.art = token.toUpper ? token.arg.toUpperCase() : token.arg.toLowerCase();
		}
		if(token.base == 8){
			if(token.alternative && token.arg.charAt(0) != '0'){
				token.arg = '0' + token.arg;
			}
		}
	},
	formatDouble: function(token) {
		var f = parseFloat(token.arg);
		if(!isFinite(f)){ // isNaN(f) || f == Number.POSITIVE_INFINITY || f == Number.NEGATIVE_INFINITY)
			// allow this only if arg is number
			if(typeof token.arg != "number"){
				throw new Error("format argument '" + token.arg + "' not a float; parseFloat returned " + f);
			}
			// C99 says that for 'f':
			//   infinity -> '[-]inf' or '[-]infinity' ('[-]INF' or '[-]INFINITY' for 'F')
			//   NaN -> a string  starting with 'nan' ('NAN' for 'F')
			// this is not commonly implemented though.
			//return '' + f;
			f = 0;
		}

		switch(token.doubleNotation) {
			case 'e': {
				token.arg = f.toExponential(token.precision); 
				break;
			}
			case 'f': {
				token.arg = f.toFixed(token.precision); 
				break;
			}
			case 'g': {
				// C says use 'e' notation if exponent is < -4 or is >= prec
				// ECMAScript for toPrecision says use exponential notation if exponent is >= prec,
				// though step 17 of toPrecision indicates a test for < -6 to force exponential.
				if(Math.abs(f) < 0.0001){
					//print("forcing exponential notation for f=" + f);
					token.arg = f.toExponential(token.precision > 0 ? token.precision - 1 : token.precision);
				}else{
					token.arg = f.toPrecision(token.precision); 
				}

				// In C, unlike 'f', 'gG' removes trailing 0s from fractional part, unless alternative format flag ("#").
				// But ECMAScript formats toPrecision as 0.00100000. So remove trailing 0s.
				if(!token.alternative){ 
					//print("replacing trailing 0 in '" + s + "'");
					token.arg = token.arg.replace(/(\..*[^0])0*/, "$1");
					// if fractional part is entirely 0, remove it and decimal point
					token.arg = token.arg.replace(/\.0*e/, 'e').replace(/\.0$/,'');
				}
				break;
			}
			default: throw new Error("unexpected double notation '" + token.doubleNotation + "'");
		}

		// C says that exponent must have at least two digits.
		// But ECMAScript does not; toExponential results in things like "1.000000e-8" and "1.000000e+8".
		// Note that s.replace(/e([\+\-])(\d)/, "e$10$2") won't work because of the "$10" instead of "$1".
		// And replace(re, func) isn't supported on IE50 or Safari1.
		token.arg = token.arg.replace(/e\+(\d)$/, "e+0$1").replace(/e\-(\d)$/, "e-0$1");

		// Ensure a '0' before the period.
		// Opera implements (0.001).toString() as '0.001', but (0.001).toFixed(1) is '.001'
		if(dojo.isOpera){
			token.arg = token.arg.replace(/^\./, '0.');
		}

		// if alt, ensure a decimal point
		if(token.alternative){
			token.arg = token.arg.replace(/^(\d+)$/,"$1.");
			token.arg = token.arg.replace(/^(\d+)e/,"$1.e");
		}

		if(f >= 0 && token.sign){
			token.arg = token.sign + token.arg;
		}

		token.arg = token.toUpper ? token.arg.toUpperCase() : token.arg.toLowerCase();
	},
	zeroPad: function(token, /*Int*/ length) {
		length = (arguments.length == 2) ? length : token.precision;
		if(typeof token.arg != "string"){
			token.arg = "" + token.arg;
		}

		var tenless = length - 10;
		while(token.arg.length < tenless){
			token.arg = (token.rightJustify) ? token.arg + this._zeros10 : this._zeros10 + token.arg;
		}
		var pad = length - token.arg.length;
		token.arg = (token.rightJustify) ? token.arg + this._zeros10.substring(0, pad) : this._zeros10.substring(0, pad) + token.arg;
	},
	fitField: function(token) {
		if(token.maxWidth >= 0 && token.arg.length > token.maxWidth){
			return token.arg.substring(0, token.maxWidth);
		}
		if(token.zeroPad){
			this.zeroPad(token, token.minWidth);
			return;
		}
		this.spacePad(token);
	},
	spacePad: function(token, /*Int*/ length) {
		length = (arguments.length == 2) ? length : token.minWidth;
		if(typeof token.arg != 'string'){
			token.arg = '' + token.arg;
		}

		var tenless = length - 10;
		while(token.arg.length < tenless){
			token.arg = (token.rightJustify) ? token.arg + this._spaces10 : this._spaces10 + token.arg;
		}
		var pad = length - token.arg.length;
		token.arg = (token.rightJustify) ? token.arg + this._spaces10.substring(0, pad) : this._spaces10.substring(0, pad) + token.arg;
	}
});

}

if(!dojo._hasResource["dojox.dtl.filter.strings"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.filter.strings"] = true;
dojo.provide("dojox.dtl.filter.strings");





dojo.mixin(dojox.dtl.filter.strings, {
	addslashes: function(value){
		// summary: Adds slashes - useful for passing strings to JavaScript, for example.
		return value.replace(/\\/g, "\\\\").replace(/"/g, '\\"').replace(/'/g, "\\'");
	},
	capfirst: function(value){
		// summary: Capitalizes the first character of the value
		value = "" + value;
		return value.charAt(0).toUpperCase() + value.substring(1);
	},
	center: function(value, arg){
		// summary: Centers the value in a field of a given width
		arg = arg || value.length;
		value = value + "";
		var diff = arg - value.length;
		if(diff % 2){
			value = value + " ";
			diff -= 1;
		}
		for(var i = 0; i < diff; i += 2){
			value = " " + value + " ";
		}
		return value;
	},
	cut: function(value, arg){
		// summary: Removes all values of arg from the given string
		arg = arg + "" || "";
		value = value + "";
		return value.replace(new RegExp(arg, "g"), "");
	},
	_fix_ampersands: /&(?!(\w+|#\d+);)/g,
	fix_ampersands: function(value){
		// summary: Replaces ampersands with ``&amp;`` entities
		return value.replace(dojox.dtl.filter.strings._fix_ampersands, "&amp;");
	},
	floatformat: function(value, arg){
		// summary: Format a number according to arg
		// description:
		//		If called without an argument, displays a floating point
		//		number as 34.2 -- but only if there's a point to be displayed.
		//		With a positive numeric argument, it displays that many decimal places
		//		always.
		//		With a negative numeric argument, it will display that many decimal
		//		places -- but only if there's places to be displayed.
		arg = parseInt(arg || -1);
		value = parseFloat(value);
		var m = value - value.toFixed(0);
		if(!m && arg < 0){
			return value.toFixed();
		}
		value = value.toFixed(Math.abs(arg));
		return (arg < 0) ? parseFloat(value) + "" : value;
	},
	iriencode: function(value){
		return dojox.dtl.text.urlquote(value, "/#%[]=:;$&()+,!");
	},
	linenumbers: function(value){
		// summary: Displays text with line numbers
		var df = dojox.dtl.filter;
		var lines = value.split("\n");
		var output = [];
		var width = (lines.length + "").length;
		for(var i = 0, line; i < lines.length; i++){
			line = lines[i];
			output.push(df.strings.ljust(i + 1, width) + ". " + df.htmlstrings.escape(line));
		}
		return output.join("\n");
	},
	ljust: function(value, arg){
		value = value + "";
		arg = parseInt(arg);
		while(value.length < arg){
			value = value + " ";
		}
		return value;
	},
	lower: function(value){
		// summary: Converts a string into all lowercase
		return (value + "").toLowerCase();
	},
	make_list: function(value){
		// summary:
		//		Returns the value turned into a list. For an integer, it's a list of
		//		digits. For a string, it's a list of characters.
		var output = [];
		if(typeof value == "number"){
			value = value + "";
		}
		if(value.charAt){
			for(var i = 0; i < value.length; i++){
				output.push(value.charAt(i));
			}
			return output;
		}
		if(typeof value == "object"){
			for(var key in value){
				output.push(value[key]);
			}
			return output;
		}
		return [];
	},
	rjust: function(value, arg){
		value = value + "";
		arg = parseInt(arg);
		while(value.length < arg){
			value = " " + value;
		}
		return value;
	},
	slugify: function(value){
		// summary: Converts to lowercase, removes
		//		non-alpha chars and converts spaces to hyphens
		value = value.replace(/[^\w\s-]/g, "").toLowerCase();
		return value.replace(/[\-\s]+/g, "-");
	},
	_strings: {},
	stringformat: function(value, arg){
		// summary:
		//		Formats the variable according to the argument, a string formatting specifier.
		//		This specifier uses Python string formating syntax, with the exception that
		//		the leading "%" is dropped.
		arg = "" + arg;
		var strings = dojox.dtl.filter.strings._strings;
		if(!strings[arg]){
			strings[arg] = new dojox.string.sprintf.Formatter("%" + arg);
		}
		return strings[arg].format(value);
	},
	title: function(value){
		// summary: Converts a string into titlecase
		var last, title = "";
		for(var i = 0, current; i < value.length; i++){
			current = value.charAt(i);
			if(last == " " || last == "\n" || last == "\t" || !last){
				title += current.toUpperCase();
			}else{
				title += current.toLowerCase();
			}
			last = current;
		}
		return title;
	},
	_truncatewords: /[ \n\r\t]/,
	truncatewords: function(value, arg){
		// summary: Truncates a string after a certain number of words
		// arg: Integer
		//		Number of words to truncate after
		arg = parseInt(arg);
		if(!arg){
			return value;
		}

		for(var i = 0, j = value.length, count = 0, current, last; i < value.length; i++){
			current = value.charAt(i);
			if(dojox.dtl.filter.strings._truncatewords.test(last)){
				if(!dojox.dtl.filter.strings._truncatewords.test(current)){
					++count;
					if(count == arg){
						return value.substring(0, j + 1);
					}
				}
			}else if(!dojox.dtl.filter.strings._truncatewords.test(current)){
				j = i;
			}
			last = current;
		}
		return value;
	},
	_truncate_words: /(&.*?;|<.*?>|(\w[\w-]*))/g,
	_truncate_tag: /<(\/)?([^ ]+?)(?: (\/)| .*?)?>/,
	_truncate_singlets: { br: true, col: true, link: true, base: true, img: true, param: true, area: true, hr: true, input: true },
	truncatewords_html: function(value, arg){
		arg = parseInt(arg);

		if(arg <= 0){
			return "";
		}

		var strings = dojox.dtl.filter.strings;
		var words = 0;
		var open = [];

		var output = dojox.string.tokenize(value, strings._truncate_words, function(all, word){
			if(word){
				// It's an actual non-HTML word
				++words;
				if(words < arg){
					return word;
				}else if(words == arg){
					return word + " ...";
				}
			}
			// Check for tag
			var tag = all.match(strings._truncate_tag);
			if(!tag || words >= arg){
				// Don't worry about non tags or tags after our truncate point
				return;
			}
			var closing = tag[1];
			var tagname = tag[2].toLowerCase();
			var selfclosing = tag[3];
			if(closing || strings._truncate_singlets[tagname]){
			}else if(closing){
				var i = dojo.indexOf(open, tagname);
				if(i != -1){
					open = open.slice(i + 1);
				}
			}else{
				open.unshift(tagname);
			}
			return all;
		}).join("");

		output = output.replace(/\s+$/g, "");

		for(var i = 0, tag; tag = open[i]; i++){
			output += "</" + tag + ">";
		}

		return output;
	},
	upper: function(value){
		return value.toUpperCase();
	},
	urlencode: function(value){
		return dojox.dtl.text.urlquote(value);
	},
	_urlize: /^((?:[(>]|&lt;)*)(.*?)((?:[.,)>\n]|&gt;)*)$/,
	_urlize2: /^\S+@[a-zA-Z0-9._-]+\.[a-zA-Z0-9._-]+$/,
	urlize: function(value){
		return dojox.dtl.filter.strings.urlizetrunc(value);
	},
	urlizetrunc: function(value, arg){
		arg = parseInt(arg);
		return dojox.string.tokenize(value, /(\S+)/g, function(word){
			var matches = dojox.dtl.filter.strings._urlize.exec(word);
			if(!matches){
				return word;
			}
			var lead = matches[1];
			var middle = matches[2];
			var trail = matches[3];

			var startsWww = middle.indexOf("www.") == 0;
			var hasAt = middle.indexOf("@") != -1;
			var hasColon = middle.indexOf(":") != -1;
			var startsHttp = middle.indexOf("http://") == 0;
			var startsHttps = middle.indexOf("https://") == 0;
			var firstAlpha = /[a-zA-Z0-9]/.test(middle.charAt(0));
			var last4 = middle.substring(middle.length - 4);

			var trimmed = middle;
			if(arg > 3){
				trimmed = trimmed.substring(0, arg - 3) + "...";
			}

			if(startsWww || (!hasAt && !startsHttp && middle.length && firstAlpha && (last4 == ".org" || last4 == ".net" || last4 == ".com"))){
				return '<a href="http://' + middle + '" rel="nofollow">' + trimmed + '</a>';
			}else if(startsHttp || startsHttps){
				return '<a href="' + middle + '" rel="nofollow">' + trimmed + '</a>';
			}else if(hasAt && !startsWww && !hasColon && dojox.dtl.filter.strings._urlize2.test(middle)){
				return '<a href="mailto:' + middle + '">' + middle + '</a>';
			}
			return word;
		}).join("");
	},
	wordcount: function(value){
		return dojox.dtl.text.pySplit(value).length;
	},
	wordwrap: function(value, arg){
		arg = parseInt(arg);
		// summary: Wraps words at specified line length
		var output = [];
		var parts = value.split(/ /g);
		if(parts.length){
			var word = parts.shift();
			output.push(word);
			var pos = word.length - word.lastIndexOf("\n") - 1;
			for(var i = 0; i < parts.length; i++){
				word = parts[i];
				if(word.indexOf("\n") != -1){
					var lines = word.split(/\n/g);
				}else{
					var lines = [word];
				}
				pos += lines[0].length + 1;
				if(arg && pos > arg){
					output.push("\n");
					pos = lines[lines.length - 1].length;
				}else{
					output.push(" ");
					if(lines.length > 1){
						pos = lines[lines.length - 1].length;
					}
				}
				output.push(word);
			}
		}
		return output.join("");
	}
});

}

if(!dojo._hasResource["dojox.dtl.tag.event"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.tag.event"] = true;
dojo.provide("dojox.dtl.tag.event");



dojox.dtl.tag.event.EventNode = function(type, fn){
	this._type = type;
	this.contents = fn;
}
dojo.extend(dojox.dtl.tag.event.EventNode, {
	render: function(context, buffer){
		if(!this._clear){
			buffer.getParent()[this._type] = null;
			this._clear = true;
		}
		if(this.contents && !this._rendered){
			if(!context.getThis()) throw new Error("You must use Context.setObject(instance)");
			this._rendered = dojo.connect(buffer.getParent(), this._type, context.getThis(), this.contents);
		}
		return buffer;
	},
	unrender: function(context, buffer){
		if(this._rendered){
			dojo.disconnect(this._rendered);
			this._rendered = false;
		}
		return buffer;
	},
	clone: function(){
		return new dojox.dtl.tag.event.EventNode(this._type, this.contents);
	},
	toString: function(){ return "dojox.dtl.tag.event." + this._type; }
});

dojox.dtl.tag.event.on = function(parser, text){
	// summary: Associates an event type to a function (on the current widget) by name
	var parts = text.split(" ");
	return new dojox.dtl.tag.event.EventNode(parts[0], parts[1]);
}

}

if(!dojo._hasResource["dojox.dtl.tag.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.tag.html"] = true;
dojo.provide("dojox.dtl.tag.html");



dojox.dtl.tag.html.HtmlNode = function(name){
	this.contents = new dojox.dtl.Filter(name);
	this._div = document.createElement("div");
	this._lasts = [];
}
dojo.extend(dojox.dtl.tag.html.HtmlNode, {
	render: function(context, buffer){
		var text = this.contents.resolve(context);
		text = text.replace(/<(\/?script)/ig, '&lt;$1').replace(/\bon[a-z]+\s*=/ig, '');
		if(this._rendered && this._last != text){
			buffer = this.unrender(context, buffer);
		}
		this._last = text;

		// This can get reset in the above tag
		if(!this._rendered){
			this._rendered = true;
			var div = this._div;
			div.innerHTML = text;
			var children = div.childNodes;
			while(children.length){
				var removed = div.removeChild(children[0]);
				this._lasts.push(removed);
				buffer = buffer.concat(removed);
			}
		}

		return buffer;
	},
	unrender: function(context, buffer){
		if(this._rendered){
			this._rendered = false;
			this._last = "";
			for(var i = 0, node; node = this._lasts[i++];){
				buffer = buffer.remove(node);
				dojo._destroyElement(node);
			}
			this._lasts = [];
		}
		return buffer;
	},
	clone: function(buffer){
		return new dojox.dtl.tag.html.HtmlNode(this.contents.contents);
	},
	toString: function(){ return "dojox.dtl.tag.html.HtmlNode"; }
});

dojox.dtl.tag.html.StyleNode = function(styles){
	this.contents = {};
	this._styles = styles;
	for(var key in styles){
		this.contents[key] = new dojox.dtl.Template(styles[key]);
	}
}
dojo.extend(dojox.dtl.tag.html.StyleNode, {
	render: function(context, buffer){
		for(var key in this.contents){
			dojo.style(buffer.getParent(), key, this.contents[key].render(context));
		}
		return buffer;
	},
	unrender: function(context, buffer){
		return buffer;
	},
	clone: function(buffer){
		return new dojox.dtl.tag.html.HtmlNode(this._styles);
	},
	toString: function(){ return "dojox.dtl.tag.html.StyleNode"; }
});

dojox.dtl.tag.html.AttachNode = function(key){
	this.contents = key;
}
dojo.extend(dojox.dtl.tag.html.AttachNode, {
	render: function(context, buffer){
		if(!this._rendered){
			this._rendered = true;
			context.getThis()[this.contents] = buffer.getParent();
		}
		return buffer;
	},
	unrender: function(context, buffer){
		if(this._rendered){
			this._rendered = false;
			if(context.getThis()[this.contents] === buffer.getParent()){
				delete context.getThis()[this.contents];
			}
		}
		return buffer;
	},
	clone: function(buffer){
		return new dojox.dtl.tag.html.HtmlNode(this._styles);
	},
	toString: function(){ return "dojox.dtl.tag.html.AttachNode"; }
});

dojox.dtl.tag.html.html = function(parser, text){
	var parts = text.split(" ", 2);
	return new dojox.dtl.tag.html.HtmlNode(parts[1]);
}

dojox.dtl.tag.html.tstyle = function(parser, text){
	var styles = {};
	text = text.replace(dojox.dtl.tag.html.tstyle._re, "");
	var rules = text.split(dojox.dtl.tag.html.tstyle._re1);
	for(var i = 0, rule; rule = rules[i]; i++){
		var parts = rule.split(dojox.dtl.tag.html.tstyle._re2);
		var key = parts[0];
		var value = parts[1];
		if(value.indexOf("{{") == 0){
			styles[key] = value;
		}
	}
	return new dojox.dtl.tag.html.StyleNode(styles);
}
dojo.mixin(dojox.dtl.tag.html.tstyle, {
	_re: /^tstyle\s+/,
	_re1: /\s*;\s*/g,
	_re2: /\s*:\s*/g
});

dojox.dtl.tag.html.attach = function(parser, text){
	var parts = text.split(dojox.dtl.tag.html.attach._re);
	return new dojox.dtl.tag.html.AttachNode(parts[1]);
}
dojo.mixin(dojox.dtl.tag.html.attach, {
	_re: /\s+/g
})

}

if(!dojo._hasResource["dojox.dtl.tag.loader"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.tag.loader"] = true;
dojo.provide("dojox.dtl.tag.loader");



dojox.dtl.tag.loader.BlockNode = function(name, nodelist){
	this.name = name;
	this.nodelist = nodelist; // Can be overridden
}
dojo.extend(dojox.dtl.tag.loader.BlockNode, {
	render: function(context, buffer){
		if(this.override){
			buffer = this.override.render(context, buffer, this);
			this.rendered = this.override;
		}else{
			buffer =  this.nodelist.render(context, buffer, this);
			this.rendered = this.nodelist;
		}
		this.override = null;
		return buffer;
	},
	unrender: function(context, buffer){
		return this.rendered.unrender(context, buffer);
	},
	setOverride: function(nodelist){
		// summary: In a shared parent, we override, not overwrite
		if(!this.override){
			this.override = nodelist;
		}
	},
	toString: function(){ return "dojox.dtl.tag.loader.BlockNode"; }
});
dojox.dtl.tag.loader.block = function(parser, text){
	var parts = text.split(" ");
	var name = parts[1];
	
	parser._blocks = parser._blocks || {};
	parser._blocks[name] = parser._blocks[name] || [];
	parser._blocks[name].push(name);
	
	var nodelist = parser.parse(["endblock", "endblock " + name]);
	parser.next();
	return new dojox.dtl.tag.loader.BlockNode(name, nodelist);
}

dojox.dtl.tag.loader.ExtendsNode = function(getTemplate, nodelist, shared, parent, key){
	this.getTemplate = getTemplate;
	this.nodelist = nodelist;
	this.shared = shared;
	this.parent = parent;
	this.key = key;
}
dojo.extend(dojox.dtl.tag.loader.ExtendsNode, {
	parents: {},
	getParent: function(context){
		if(!this.parent){
			this.parent = context.get(this.key, false);
			if(!this.parent){
				throw new Error("extends tag used a variable that did not resolve");
			}
			if(typeof this.parent == "object"){
				if(this.parent.url){
					if(this.parent.shared){
						this.shared = true;
					}
					this.parent = this.parent.url.toString();
				}else{
					this.parent = this.parent.toString();
				}
			}
			if(this.parent && this.parent.indexOf("shared:") == 0){
				this.shared = true;
				this.parent = this.parent.substring(7, parent.length);
			}
		}
		var parent = this.parent;
		if(!parent){
			throw new Error("Invalid template name in 'extends' tag.");
		}
		if(parent.render){
			return parent;
		}
		if(this.parents[parent]){
			return this.parents[parent];
		}
		this.parent = this.getTemplate(dojox.dtl.text.getTemplateString(parent));
		if(this.shared){
			this.parents[parent] = this.parent;
		}
		return this.parent;
	},
	render: function(context, buffer){
		var st = dojox.dtl;
		var stbl = dojox.dtl.tag.loader;
		var parent = this.getParent(context);
		var isChild = parent.nodelist[0] instanceof this.constructor;
		var parentBlocks = {};
		for(var i = 0, node; node = parent.nodelist.contents[i]; i++){
			if(node instanceof stbl.BlockNode){
				parentBlocks[node.name] = node;
			}
		}
		for(var i = 0, node; node = this.nodelist.contents[i]; i++){
			if(node instanceof stbl.BlockNode){
				var block = parentBlocks[node.name];
				if(!block){
					if(isChild){
						parent.nodelist[0].nodelist.append(node);
					}
				}else{
					if(this.shared){
						block.setOverride(node.nodelist);
					}else{
						block.nodelist = node.nodelist;
					}
				}
			}
		}
		this.rendered = parent;
		return parent.render(context, buffer, this);
	},
	unrender: function(context, buffer){
		return this.rendered.unrender(context, buffer, this);
	},
	toString: function(){ return "dojox.dtl.block.ExtendsNode"; }
});
dojox.dtl.tag.loader.extends_ = function(parser, text){
	var parts = text.split(" ");
	var shared = false;
	var parent = null;
	var key = null;
	if(parts[1].charAt(0) == '"' || parts[1].charAt(0) == "'"){
		parent = parts[1].substring(1, parts[1].length - 1);
	}else{
		key = parts[1];
	}
	if(parent && parent.indexOf("shared:") == 0){
		shared = true;
		parent = parent.substring(7, parent.length);
	}
	var nodelist = parser.parse();
	return new dojox.dtl.tag.loader.ExtendsNode(parser.getTemplate, nodelist, shared, parent, key);
}

}

if(!dojo._hasResource["dojox.dtl.tag.logic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.tag.logic"] = true;
dojo.provide("dojox.dtl.tag.logic");



dojox.dtl.tag.logic.IfNode = function(bools, trues, falses, type){
	this.bools = bools;
	this.trues = trues;
	this.falses = falses;
	this.type = type;
}
dojo.extend(dojox.dtl.tag.logic.IfNode, {
	render: function(context, buffer){
		if(this.type == "or"){
			for(var i = 0, bool; bool = this.bools[i]; i++){
				var ifnot = bool[0];
				var filter = bool[1];
				var value = filter.resolve(context);
				if((value && !ifnot) || (ifnot && !value)){
					if(this.falses){
						buffer = this.falses.unrender(context, buffer);
					}
					return this.trues.render(context, buffer, this);
				}
				buffer = this.trues.unrender(context, buffer);
				if(this.falses)	return this.falses.render(context, buffer, this);
			}
		}else{
			for(var i = 0, bool; bool = this.bools[i]; i++){
				var ifnot = bool[0];
				var filter = bool[1];
				var value = filter.resolve(context);
				if(!((value && !ifnot) || (ifnot && !value))){
					if(this.trues){
						buffer = this.trues.unrender(context, buffer);
					}
					return this.falses.render(context, buffer, this);
				}
				buffer = this.falses.unrender(context, buffer);
				if(this.falses)	return this.trues.render(context, buffer, this);
			}
		}
		return buffer;
	},
	unrender: function(context, buffer){
		if(this.trues) buffer = this.trues.unrender(context, buffer);
		if(this.falses) buffer = this.falses.unrender(context, buffer);
		return buffer;
	},
	clone: function(buffer){
		var trues = this.trues;
		var falses = this.falses;
		if(trues){
			trues = trues.clone(buffer);
		}
		if(falses){
			falses = falses.clone(buffer);
		}
		return new this.constructor(this.bools, trues, falses, this.type);
	},
	toString: function(){ return "dojox.dtl.tag.logic.IfNode"; }
});

dojox.dtl.tag.logic.ForNode = function(assign, loop, reversed, nodelist){
	this.assign = assign;
	this.loop = loop;
	this.reversed = reversed;
	this.nodelist = nodelist;
	this.pool = [];
}
dojo.extend(dojox.dtl.tag.logic.ForNode, {
	render: function(context, buffer){
		var parentloop = {};
		if(context.forloop){
			parentloop = context.forloop;
		}
		var items = dojox.dtl.resolveVariable(this.loop, context);
		context.push();
		for(var i = items.length; i < this.pool.length; i++){
			this.pool[i].unrender(context, buffer);
		}
		if(this.reversed){
			items = items.reversed();
		}
		var j = 0;
		for(var i in items){
			var item = items[i];
			context.forloop = {
				key: i,
				counter0: j,
				counter: j + 1,
				revcounter0: items.length - j - 1,
				revcounter: items.length - j,
				first: j == 0,
				parentloop: parentloop
			};
			context[this.assign] = item;
			if(j + 1 > this.pool.length){
				this.pool.push(this.nodelist.clone(buffer));
		 	}
			buffer = this.pool[j].render(context, buffer, this);
			++j;
		}
		context.pop();
		return buffer;
	},
	unrender: function(context, buffer){
		for(var i = 0, pool; pool = this.pool[i]; i++){
			buffer = pool.unrender(context, buffer);
		}
		return buffer;
	},
	clone: function(buffer){
		return new this.constructor(this.assign, this.loop, this.reversed, this.nodelist.clone(buffer));
	},
	toString: function(){ return "dojox.dtl.tag.logic.ForNode"; }
});

dojox.dtl.tag.logic.if_ = function(parser, text){
	var parts = text.split(/\s+/g);
	var type;
	var bools = [];
	parts.shift();
	text = parts.join(" ");
	parts = text.split(" and ");
	if(parts.length == 1){
		type = "or";
		parts = text.split(" or ");
	}else{
		type = "and";
		for(var i = 0; i < parts.length; i++){
			if(parts[i] == "or"){
				throw new Error("'if' tags can't mix 'and' and 'or'");
			}
		}
	}
	for(var i = 0, part; part = parts[i]; i++){
		var not = false;
		if(part.indexOf("not ") == 0){
			part = part.substring(4);
			not = true;
		}
		bools.push([not, new dojox.dtl.Filter(part)]);
	}
	var trues = parser.parse(["else", "endif"]);
	var falses = false;
	var token = parser.next();
	if(token.text == "else"){
		var falses = parser.parse(["endif"]);
		parser.next();
	}
	return new dojox.dtl.tag.logic.IfNode(bools, trues, falses, type);
}

dojox.dtl.tag.logic.for_ = function(parser, text){
	var parts = text.split(/\s+/g);
	var reversed = parts.length == 5;
	var nodelist = parser.parse(["endfor"]);
	parser.next();
	return new dojox.dtl.tag.logic.ForNode(parts[1], parts[3], reversed, nodelist);
}

}

if(!dojo._hasResource["dojox.dtl.tag.loop"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.tag.loop"] = true;
dojo.provide("dojox.dtl.tag.loop");



dojox.dtl.tag.loop.CycleNode = function(cyclevars, name, VarNode){
	this._cyclevars = cyclevars;
	this._counter = -1
	this._name = name;
	this._map = {};
	this._VarNode = VarNode;
}
dojo.extend(dojox.dtl.tag.loop.CycleNode, {
	render: function(context, buffer){
		if(context.forloop && !context.forloop.counter0){
			this._counter = -1;
		}

		++this._counter;
		var value = this._cyclevars[this._counter % this._cyclevars.length];
		if(this._name){
			context[this._name] = value;
		}
		if(!this._map[value]){
			this._map[value] = {};
		}
		var node = this._map[value][this._counter] = new this._VarNode(value);

		return node.render(context, buffer, this);
	},
	unrender: function(context, buffer){
		return buffer;
	},
	clone: function(){
		return new this.constructor(this._cyclevars, this._name);
	},
	_onEnd: function(){
		this._counter = -1;
	},
	toString: function(){ return "dojox.dtl.tag.loop.CycleNode"; }
});

dojox.dtl.tag.loop.cycle = function(parser, text){
	// summary: Cycle among the given strings each time this tag is encountered
	var args = text.split(" ");

	if(args.length < 2){
		throw new Error("'cycle' tag requires at least two arguments");
	}

	if(args[1].indexOf(",") != -1){
		var vars = args[1].split(",");
		args = [args[0]];
		for(var i = 0; i < vars.length; i++){
			args.push('"' + vars[i] + '"');
		}
	}

	if(args.length == 2){
		var name = args[args.length - 1];

		if(!parser._namedCycleNodes){
			throw new Error("No named cycles in template: '" + name + "' is not defined");
		}
		if(!parser._namedCycleNodes[name]){
			throw new Error("Named cycle '" + name + "' does not exist");
		}

        return parser._namedCycleNodes[name];
	}

	if(args.length > 4 && args[args.length - 2] == "as"){
		var name = args[args.length - 1];

		var node = new dojox.dtl.tag.loop.CycleNode(args.slice(1, args.length - 2), name, parser.getVarNode());

		if(!parser._namedCycleNodes){
			parser._namedCycleNodes = {};
		}
		parser._namedCycleNodes[name] = node;
	}else{
		node = new dojox.dtl.tag.loop.CycleNode(args.slice(1), null, parser.getVarNode());
	}

	return node;
}

}

if(!dojo._hasResource["dojox.dtl.tag.misc"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.tag.misc"] = true;
dojo.provide("dojox.dtl.tag.misc");



dojox.dtl.tag.misc.commentNode = new function(){
	this.render = this.unrender = function(context, buffer){ return buffer; };
	this.clone = function(){ return this; };
	this.toString = function(){ return "dojox.dtl.tag.misc.CommentNode"; };
}

dojox.dtl.tag.misc.DebugNode = function(TextNode){
	this._TextNode = TextNode;
}
dojo.extend(dojox.dtl.tag.misc.DebugNode, {
	render: function(context, buffer){
		var keys = context.getKeys();
		var debug = "";
		for(var i = 0, key; key = keys[i]; i++){
			console.debug("DEBUG", key, ":", context[key]);
			debug += key + ": " + dojo.toJson(context[key]) + "\n\n";
		}
		return new this._TextNode(debug).render(context, buffer, this);
	},
	unrender: function(context, buffer){
		return buffer;
	},
	clone: function(buffer){
		return new this.constructor(this._TextNode);
	},
	toString: function(){ return "dojox.dtl.tag.misc.DebugNode"; }
});

dojox.dtl.tag.misc.FilterNode = function(varnode, nodelist){
	this._varnode = varnode;
	this._nodelist = nodelist;
}
dojo.extend(dojox.dtl.tag.misc.FilterNode, {
	render: function(context, buffer){
		// Doing this in HTML requires a different buffer with a fake root node
		var output = this._nodelist.render(context, new dojox.string.Builder());
		context.update({ "var": output.toString() });
		var filtered = this._varnode.render(context, buffer);
		context.pop();
		return buffer;
	},
	unrender: function(context, buffer){
		return buffer;
	},
	clone: function(buffer){
		return new this.constructor(this._expression, this._nodelist.clone(buffer));
	}
});

dojox.dtl.tag.misc.comment = function(parser, text){
	// summary: Ignore everything between {% comment %} and {% endcomment %}
	parser.skipPast("endcomment");
	return dojox.dtl.tag.misc.commentNode;
}

dojox.dtl.tag.misc.debug = function(parser, text){
	// summary: Output the current context, maybe add more stuff later.
	return new dojox.dtl.tag.misc.DebugNode(parser.getTextNode());
}

dojox.dtl.tag.misc.filter = function(parser, text){
	// summary: Filter the contents of the blog through variable filters.
	var parts = text.split(" ", 2);
	var varnode = new (parser.getVarNode())("var|" + parts[1]);
	var nodelist = parser.parse(["endfilter"]);
	parser.next();
	return new dojox.dtl.tag.misc.FilterNode(varnode, nodelist);
}

}

if(!dojo._hasResource["dojox.layout.ContentPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.layout.ContentPane"] = true;
dojo.provide("dojox.layout.ContentPane");



(function(){ // private scope, sort of a namespace

	// TODO: should these methods be moved to dojox.html.cssPathAdjust or something?

	// css at-rules must be set before any css declarations according to CSS spec
	// match:
	// @import 'http://dojotoolkit.org/dojo.css';
	// @import 'you/never/thought/' print;
	// @import url("it/would/work") tv, screen;
	// @import url(/did/you/now.css);
	// but not:
	// @namespace dojo "http://dojotoolkit.org/dojo.css"; /* namespace URL should always be a absolute URI */
	// @charset 'utf-8';
	// @media print{ #menuRoot {display:none;} }

		
	// we adjust all paths that dont start on '/' or contains ':'
	//(?![a-z]+:|\/)

	if(dojo.isIE){
		var alphaImageLoader = /(AlphaImageLoader\([^)]*?src=(['"]))(?![a-z]+:|\/)([^\r\n;}]+?)(\2[^)]*\)\s*[;}]?)/g;
	}

	var cssPaths = /(?:(?:@import\s*(['"])(?![a-z]+:|\/)([^\r\n;{]+?)\1)|url\(\s*(['"]?)(?![a-z]+:|\/)([^\r\n;]+?)\3\s*\))([a-z, \s]*[;}]?)/g;

	function adjustCssPaths(cssUrl, cssText){
		//	summary:
		//		adjusts relative paths in cssText to be relative to cssUrl
		//		a path is considered relative if it doesn't start with '/' and not contains ':'
		//	description:
		//		Say we fetch a HTML page from level1/page.html
		//		It has some inline CSS:
		//			@import "css/page.css" tv, screen;
		//			...
		//			background-image: url(images/aplhaimage.png);
		//
		//		as we fetched this HTML and therefore this CSS
		//		from level1/page.html, these paths needs to be adjusted to:
		//			@import 'level1/css/page.css' tv, screen;
		//			...
		//			background-image: url(level1/images/alphaimage.png);
		//		
		//		In IE it will also adjust relative paths in AlphaImageLoader()
		//			filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='images/alphaimage.png');
		//		will be adjusted to:
		//			filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='level1/images/alphaimage.png');
		//
		//		Please note that any relative paths in AlphaImageLoader in external css files wont work, as
		//		the paths in AlphaImageLoader is MUST be declared relative to the HTML page,
		//		not relative to the CSS file that declares it

		if(!cssText || !cssUrl){ return; }

		// support the ImageAlphaFilter if it exists, most people use it in IE 6 for transparent PNGs
		// We are NOT going to kill it in IE 7 just because the PNGs work there. Somebody might have
		// other uses for it.
		// If user want to disable css filter in IE6  he/she should
		// unset filter in a declaration that just IE 6 doesn't understands
		// like * > .myselector { filter:none; }
		if(alphaImageLoader){
			cssText = cssText.replace(alphaImageLoader, function(ignore, pre, delim, url, post){
				return pre + (new dojo._Url(cssUrl, './'+url).toString()) + post;
			});
		}

		return cssText.replace(cssPaths, function(ignore, delimStr, strUrl, delimUrl, urlUrl, media){
			if(strUrl){
				return '@import "' + (new dojo._Url(cssUrl, './'+strUrl).toString()) + '"' + media;
			}else{
				return 'url(' + (new dojo._Url(cssUrl, './'+urlUrl).toString()) + ')' + media;
			}
		});
	}

	// attributepaths one tag can have multiple paths, example:
	// <input src="..." style="url(..)"/> or <a style="url(..)" href="..">
	// <img style='filter:progid...AlphaImageLoader(src="noticeTheSrcHereRunsThroughHtmlSrc")' src="img">
	var htmlAttrPaths = /(<[a-z][a-z0-9]*\s[^>]*)(?:(href|src)=(['"]?)([^>]*?)\3|style=(['"]?)([^>]*?)\5)([^>]*>)/gi;

	function adjustHtmlPaths(htmlUrl, cont){
		var url = htmlUrl || "./";

		return cont.replace(htmlAttrPaths,
			function(tag, start, name, delim, relUrl, delim2, cssText, end){
				return start + (name ?
							(name + '=' + delim + (new dojo._Url(url, relUrl).toString()) + delim)
						: ('style=' + delim2 + adjustCssPaths(url, cssText) + delim2)
				) + end;
			}
		);
	}

	function secureForInnerHtml(cont){
		/********* remove <!DOCTYPE.. and <title>..</title> tag **********/
		// khtml is picky about dom faults, you can't attach a <style> or <title> node as child of body
		// must go into head, so we need to cut out those tags
		return cont.replace(/(?:\s*<!DOCTYPE\s[^>]+>|<title[^>]*>[\s\S]*?<\/title>)/ig, "");
	}

	function snarfStyles(/*String*/cssUrl, /*String*/cont, /*Array*/styles){
		/****************  cut out all <style> and <link rel="stylesheet" href=".."> **************/
		// also return any attributes from this tag (might be a media attribute)
		// if cssUrl is set it will adjust paths accordingly
		styles.attributes = [];

		return cont.replace(/(?:<style([^>]*)>([\s\S]*?)<\/style>|<link\s+(?=[^>]*rel=['"]?stylesheet)([^>]*?href=(['"])([^>]*?)\4[^>\/]*)\/?>)/gi,
			function(ignore, styleAttr, cssText, linkAttr, delim, href){
				// trim attribute
				var i, attr = (styleAttr||linkAttr||"").replace(/^\s*([\s\S]*?)\s*$/i, "$1"); 
				if(cssText){
					i = styles.push(cssUrl ? adjustCssPaths(cssUrl, cssText) : cssText);
				}else{
					i = styles.push('@import "' + href + '";')
					attr = attr.replace(/\s*(?:rel|href)=(['"])?[^\s]*\1\s*/gi, ""); // remove rel=... and href=...
				}
				if(attr){
					attr = attr.split(/\s+/);// split on both "\n", "\t", " " etc
					var atObj = {}, tmp;
					for(var j = 0, e = attr.length; j < e; j++){
						tmp = attr[j].split('=')// split name='value'
						atObj[tmp[0]] = tmp[1].replace(/^\s*['"]?([\s\S]*?)['"]?\s*$/, "$1"); // trim and remove ''
					}
					styles.attributes[i - 1] = atObj;
				}
				return ""; // squelsh the <style> or <link>
			}
		);
	}

	function snarfScripts(cont, byRef){
		// summary
		//		strips out script tags from cont
		// invoke with 
		//	byRef = {errBack:function(){/*add your download error code here*/, downloadRemote: true(default false)}}
		//	byRef will have {code: 'jscode'} when this scope leaves
		byRef.code = "";

		function download(src){
			if(byRef.downloadRemote){
				// console.debug('downloading',src);
				dojo.xhrGet({
					url: src,
					sync: true,
					load: function(code){
						byRef.code += code+";";
					},
					error: byRef.errBack
				});
			}
		}
		
		// match <script>, <script type="text/..., but not <script type="dojo(/method)...
		return cont.replace(/<script\s*(?![^>]*type=['"]?dojo)(?:[^>]*?(?:src=(['"]?)([^>]*?)\1[^>]*)?)*>([\s\S]*?)<\/script>/gi,
			function(ignore, delim, src, code){
				if(src){
					download(src);
				}else{
					byRef.code += code;
				}
				return "";
			}
		);
	}

	function evalInGlobal(code, appendNode){
		// we do our own eval here as dojo.eval doesn't eval in global crossbrowser
		// This work X browser but but it relies on a DOM
		// plus it doesn't return anything, thats unrelevant here but not for dojo core
		appendNode = appendNode || dojo.doc.body;
		var n = appendNode.ownerDocument.createElement('script');
		n.type = "text/javascript";
		appendNode.appendChild(n);
		n.text = code; // DOM 1 says this should work
	}

	/*=====
	dojox.layout.ContentPane.DeferredHandle = {
		// cancel: Function
		cancel: function(){
			// summary: cancel a in flight download
		},

		addOnLoad: function(func){
			// summary: add a callback to the onLoad chain
			// func: Function
		},

		addOnUnload: function(func){
			// summary: add a callback to the onUnload chain
			// func: Function
		}
	}
	=====*/


dojo.declare("dojox.layout.ContentPane", dijit.layout.ContentPane, {
	// summary:
	//		An extended version of dijit.layout.ContentPane
	//		Supports infile scrips and external ones declared by <script src=''
	//		relative path adjustments (content fetched from a different folder)
	//		<style> and <link rel='stylesheet' href='..'> tags,
	//		css paths inside cssText is adjusted (if you set adjustPaths = true)
	//
	//		NOTE that dojo.require in script in the fetched file isn't recommended
	//		Many widgets need to be required at page load to work properly

	// adjustPaths: Boolean
	//		Adjust relative paths in html string content to point to this page
	//		Only usefull if you grab content from a another folder then the current one
	adjustPaths: false,

	// cleanContent: Boolean
	//	summary:
	//		cleans content to make it less likly to generate DOM/JS errors.
	//	description:
	//		usefull if you send contentpane a complete page, instead of a html fragment
	//		scans for 
	//			style nodes, inserts in Document head
	//			title Node, remove
	//			DOCTYPE tag, remove
	//			<!-- *JS code here* -->
	//			<![CDATA[ *JS code here* ]]>
	cleanContent: false,

	// renderStyles: Boolean
	//		trigger/load styles in the content
	renderStyles: false,

	// executeScripts: Boolean
	//		Execute (eval) scripts that is found in the content
	executeScripts: true,

	// scriptHasHooks: Boolean
	//		replace keyword '_container_' in scripts with 'dijit.byId(this.id)'
	// NOTE this name might change in the near future
	scriptHasHooks: false,

	/*======
	// ioMethod: dojo.xhrGet|dojo.xhrPost
	//		reference to the method that should grab the content
	ioMethod: dojo.xhrGet,
	
	// ioArgs: Object
	//		makes it possible to add custom args to xhrGet, like ioArgs.headers['X-myHeader'] = 'true'
	ioArgs: {},

	// onLoadDeferred: dojo.Deferred
	//		callbackchain will start when onLoad occurs
	onLoadDeferred: new dojo.Deferred(),

	// onUnloadDeferred: dojo.Deferred
	//		callbackchain will start when onUnload occurs
	onUnloadDeferred: new dojo.Deferred(),

	setHref: function(url){
		// summary: replace current content with url's content
		return ;// dojox.layout.ContentPane.DeferredHandle
	},

	refresh: function(){
		summary: force a re-download of content
		return ;// dojox.layout.ContentPane.DeferredHandle 
	},

	======*/

	constructor: function(){
		// init per instance properties, initializer doesn't work here because how things is hooked up in dijit._Widget
		this.ioArgs = {};
		this.ioMethod = dojo.xhrGet;
		this.onLoadDeferred = new dojo.Deferred();
		this.onUnloadDeferred = new dojo.Deferred();
	},

	postCreate: function(){
		// override to support loadDeferred
		this._setUpDeferreds();

		dijit.layout.ContentPane.prototype.postCreate.apply(this, arguments);
	},

	onExecError: function(e){
		// summary
		//		event callback, called on script error or on java handler error
		//		overide and return your own html string if you want a some text 
		//		displayed within the ContentPane
	},

	setContent: function(data){
		// summary: set data as new content, sort of like innerHTML
		// data: String|DomNode|NodeList|dojo.NodeList
		if(!this._isDownloaded){
			var defObj = this._setUpDeferreds();
		}

		dijit.layout.ContentPane.prototype.setContent.apply(this, arguments);
		return defObj; // dojox.layout.ContentPane.DeferredHandle
	},

	cancel: function(){
		// summary: cancels a inflight download
		if(this._xhrDfd && this._xhrDfd.fired == -1){
			// we are still in flight, which means we should reset our DeferredHandle
			// otherwise we will trigger onUnLoad chain of the canceled content,
			// the canceled content have never gotten onLoad so it shouldn't get onUnload
			this.onUnloadDeferred = null;
		}
		dijit.layout.ContentPane.prototype.cancel.apply(this, arguments);
	},

	_setUpDeferreds: function(){
		var _t = this, cancel = function(){ _t.cancel();	}
		var onLoad = (_t.onLoadDeferred = new dojo.Deferred());
		var onUnload = (_t._nextUnloadDeferred = new dojo.Deferred());
		return {
			cancel: cancel,
			addOnLoad: function(func){onLoad.addCallback(func);},
			addOnUnload: function(func){onUnload.addCallback(func);}
		};
	},

	_onLoadHandler: function(){
		dijit.layout.ContentPane.prototype._onLoadHandler.apply(this, arguments);
		if(this.onLoadDeferred){
			this.onLoadDeferred.callback(true);
		}
	},

	_onUnloadHandler: function(){
		this.isLoaded = false;
		this.cancel();// need to cancel so we don't get any inflight suprises
		if(this.onUnloadDeferred){
			this.onUnloadDeferred.callback(true);
		}

		dijit.layout.ContentPane.prototype._onUnloadHandler.apply(this, arguments);

		if(this._nextUnloadDeferred){
			this.onUnloadDeferred = this._nextUnloadDeferred;
		}
	},

	_onError: function(type, err){
		dijit.layout.ContentPane.prototype._onError.apply(this, arguments);
		if(this.onLoadDeferred){
			this.onLoadDeferred.errback(err);
		}
	},

	_prepareLoad: function(forceLoad){
		// sets up for a xhrLoad, load is deferred until widget is showing
		var defObj = this._setUpDeferreds();

		dijit.layout.ContentPane.prototype._prepareLoad.apply(this, arguments);

		return defObj;
	},

	_setContent: function(cont){
		// override dijit.layout.ContentPane._setContent, to enable path adjustments
		var styles = [];// init vars
		if(dojo.isString(cont)){
			if(this.adjustPaths && this.href){
				cont = adjustHtmlPaths(this.href, cont);
			}
			if(this.cleanContent){
				cont = secureForInnerHtml(cont);
			}
			if(this.renderStyles || this.cleanContent){
				cont = snarfStyles(this.href, cont, styles);
			}

			// because of a bug in IE, script tags that is first in html hierarchy doesnt make it into the DOM 
			//	when content is innerHTML'ed, so we can't use dojo.query to retrieve scripts from DOM
			if(this.executeScripts){
				var _t = this, code, byRef = {
					downloadRemote: true,
					errBack:function(e){
						_t._onError.call(_t, 'Exec', 'Error downloading remote script in "'+_t.id+'"', e);
					}
				};
				cont = snarfScripts(cont, byRef);
				code = byRef.code;
			}

			// rationale for this block:
			// if containerNode/domNode is a table derivate tag, some browsers dont allow innerHTML on those
			var node = (this.containerNode || this.domNode), pre = post = '', walk = 0;
			switch(name = node.nodeName.toLowerCase()){
				case 'tr':
					pre = '<tr>'; post = '</tr>';
					walk += 1;//fallthrough
				case 'tbody': case 'thead':// children of THEAD is of same type as TBODY
					pre = '<tbody>' + pre; post += '</tbody>';
					walk += 1;// falltrough
				case 'table':
					pre = '<table>' + pre; post += '</table>';
					walk += 1;
					break;
			}
			if(walk){
				var n = node.ownerDocument.createElement('div');
				n.innerHTML = pre + cont + post;
				do{
					n = n.firstChild;
				}while(--walk);
				cont = n.childNodes;
			}
		}

		// render the content
		dijit.layout.ContentPane.prototype._setContent.call(this, cont);

		// clear old stylenodes from the DOM
		if(this._styleNodes && this._styleNodes.length){
			while(this._styleNodes.length){
				dojo._destroyElement(this._styleNodes.pop());
			}
		}
		// render new style nodes
		if(this.renderStyles && styles && styles.length){
			this._renderStyles(styles);
		}

		if(this.executeScripts && code){
			if(this.cleanContent){
				// clean JS from html comments and other crap that browser
				// parser takes care of in a normal page load
				code = code.replace(/(<!--|(?:\/\/)?-->|<!\[CDATA\[|\]\]>)/g, '');
			}
			if(this.scriptHasHooks){
				// replace _container_ with dijit.byId(this.id)
				code = code.replace(/_container_(?!\s*=[^=])/g, "dijit.byId('"+this.id+"')");
			}
			try{
				evalInGlobal(code, (this.containerNode || this.domNode));
			}catch(e){
				this._onError('Exec', 'Error eval script in '+this.id+', '+e.message, e);
			}
		}
	},

	_renderStyles: function(styles){
		// insert css from content into document head
		this._styleNodes = [];
		var st, att, cssText, doc = this.domNode.ownerDocument;
		var head = doc.getElementsByTagName('head')[0];

		for(var i = 0, e = styles.length; i < e; i++){
			cssText = styles[i]; att = styles.attributes[i];
			st = doc.createElement('style');
			st.setAttribute("type", "text/css"); // this is required in CSS spec!

			for(var x in att){
				st.setAttribute(x, att[x])
			}
			
			this._styleNodes.push(st);
			head.appendChild(st); // must insert into DOM before setting cssText

			if(st.styleSheet){ // IE
				st.styleSheet.cssText = cssText;
			}else{ // w3c
				st.appendChild(doc.createTextNode(cssText));
			}
		}
	}
});

})();

}

if(!dojo._hasResource["dojox.layout.ResizeHandle"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.layout.ResizeHandle"] = true;
dojo.provide("dojox.layout.ResizeHandle");
dojo.experimental("dojox.layout.ResizeHandle"); 


 
 

dojo.declare("dojox.layout.ResizeHandle", [dijit._Widget, dijit._Templated], {
	// summary
	//	The handle on the bottom-right corner of FloatingPane or other widgets that allows
	//	the widget to be resized.
	//	Typically not used directly.

	// targetId: String
	//	id of the Widget OR DomNode that I will size
	targetId: '',

	// targetContainer: DomNode
	//	over-ride targetId and attch this handle directly to a reference of a DomNode
	targetContainer: null, 

	// resizeAxis: String
	//	one of: x|y|xy limit resizing to a single axis, default to xy ... 
	resizeAxis: "xy",

	// activeResize: Boolean
	// 	if true, node will size realtime with mouse movement, 
	//	if false, node will create virtual node, and only resize target on mouseUp
	activeResize: false,
	
	// activeResizeClass: String
	//	css class applied to virtual resize node. 
	activeResizeClass: 'dojoxResizeHandleClone',

	// animateSizing: Boolean
	//	only applicable if activeResize = false. onMouseup, animate the node to the
	//	new size
	animateSizing: true,
	
	// animateMethod: String
	// 	one of "chain" or "combine" ... visual effect only. combine will "scale" 
	// 	node to size, "chain" will alter width, then height
	animateMethod: 'chain',

	// animateDuration: Integer
	//	time in MS to run sizing animation. if animateMethod="chain", total animation 
	//	playtime is 2*animateDuration
	animateDuration: 225,

	// minHeight: Integer
	//	smallest height in px resized node can be
	minHeight: 100,

	// minWidth: Integer
	//	smallest width in px resize node can be
	minWidth: 100,

	// resize handle template, fairly easy to override: 
	templateString: '<div dojoAttachPoint="resizeHandle" class="dojoxResizeHandle"><div></div></div>',

	// private propteries and holders
	_isSizing: false,
	_connects: [],
	_activeResizeNode: null,	
	_activeResizeLastEvent: null,
	// defaults to match default resizeAxis. set resizeAxis variable to modify. 
	_resizeX: true,
	_resizeY: true,


	postCreate: function(){
		// summary: setup our one major listener upon creation
		dojo.connect(this.resizeHandle, "onmousedown", this, "_beginSizing");
		if(!this.activeResize){ 
			this._activeResizeNode = document.createElement('div');
			dojo.addClass(this._activeResizeNode,this.activeResizeClass); 
		}else{ this.animateSizing = false; } 	

		if (!this.minSize) { 
			this.minSize = { w: this.minWidth, h: this.minHeight };
		}
		// should we modify the css for the cursor hover to n-resize nw-resize and w-resize?
		this._resizeX = this._resizeY = false; 
		switch (this.resizeAxis.toLowerCase()) {
		case "xy" : 
			this._resizeX = this._resizeY = true; 
			// FIXME: need logic to determine NW or NE class to see
			// based on which [todo] corner is clicked
			dojo.addClass(this.resizeHandle,"dojoxResizeNW"); 
			break; 
		case "x" : 
			this._resizeX = true; 
			dojo.addClass(this.resizeHandle,"dojoxResizeW");
			break;
		case "y" : 
			this._resizeY = true; 
			dojo.addClass(this.resizeHandle,"dojoxResizeN");
			break;
		}
	},

	_beginSizing: function(/*Event*/ e){
		// summary: setup movement listeners and calculate initial size
		
		if (this._isSizing){ return false; }

		this.targetWidget = dijit.byId(this.targetId);

		// FIXME: resizing widgets does weird things, disable virtual resizing for now:
		if (this.targetWidget) { this.activeResize = true; } 

		this.targetDomNode = this.targetWidget ? this.targetWidget.domNode : dojo.byId(this.targetId);
		if (this.targetContainer) { this.targetDomNode = this.targetContainer; } 
		if (!this.targetDomNode){ return; }

		if (!this.activeResize) {
			this.targetDomNode.appendChild(this._activeResizeNode); 
			dojo.fadeIn({ node: this._activeResizeNode, duration:120, 
				beforeBegin: dojo.hitch(this,function(){
					this._activeResizeNode.style.display=''; 
				})
			}).play(); 
		}

		this._isSizing = true;
		this.startPoint  = {'x':e.clientX, 'y':e.clientY};

		// FIXME: this is funky: marginBox adds height, contentBox ignores padding (expected, but foo!)
		var mb = (this.targetWidget) ? dojo.marginBox(this.targetDomNode) : dojo.contentBox(this.targetDomNode);  
		this.startSize  = { 'w':mb.w, 'h':mb.h };

		this._connects = []; 
		this._connects.push(dojo.connect(document,"onmousemove",this,"_updateSizing")); 
		this._connects.push(dojo.connect(document,"onmouseup", this, "_endSizing"));

		e.preventDefault();
	},

	_updateSizing: function(/*Event*/ e){
		// summary: called when moving the ResizeHandle ... determines 
		//	new size based on settings/position and sets styles.

		if(this.activeResize){
			this._changeSizing(e);
		}else{
			var tmp = this._getNewCoords(e);	
			if(tmp === false){ return; }
			dojo.style(this._activeResizeNode,"width",tmp.width+"px");
			dojo.style(this._activeResizeNode,"height",tmp.height+"px"); 
			this._activeResizeNode.style.display=''; 
		}
	},

	_getNewCoords: function(/* Event */ e){
		
		// On IE, if you move the mouse above/to the left of the object being resized,
		// sometimes clientX/Y aren't set, apparently.  Just ignore the event.
		try{
			if(!e.clientX  || !e.clientY){ return false; }
		}catch(e){
			// sometimes you get an exception accessing above fields...
			return false;
		}
		this._activeResizeLastEvent = e; 

		var dx = this.startPoint.x - e.clientX;
		var dy = this.startPoint.y - e.clientY;
		
		var newW = (this._resizeX) ? this.startSize.w - dx : this.startSize.w;
		var newH = (this._resizeY) ? this.startSize.h - dy : this.startSize.h;

		// minimum size check
		if(this.minSize){
			//var mb = dojo.marginBox(this.targetDomNode);
			if(newW < this.minSize.w){
				newW = this.minSize.w;
			}
			if(newH < this.minSize.h){
				newH = this.minSize.h;
			}
		}
		return {width:newW, height:newH};  // Object
	},
	
	_changeSizing: function(/*Event*/ e){
		// summary: apply sizing information based on information in (e) to attached node
		var tmp = this._getNewCoords(e);
		if(tmp===false){ return; }

		if(this.targetWidget && typeof this.targetWidget.resize == "function"){ 
			this.targetWidget.resize({ w: tmp.width, h: tmp.height });
		}else{
			if(this.animateSizing){
				var anim = dojo.fx[this.animateMethod]([
					dojo.animateProperty({
						node: this.targetDomNode,
						properties: { 
							width: { start: this.startSize.w, end: tmp.width, unit:'px' } 
						},	
						duration: this.animateDuration
					}),
					dojo.animateProperty({
						node: this.targetDomNode,
						properties: { 
							height: { start: this.startSize.h, end: tmp.height, unit:'px' }
						},
						duration: this.animateDuration
					})
				]);
				anim.play();
			}else{
				dojo.style(this.targetDomNode,"width",tmp.width+"px"); 
				dojo.style(this.targetDomNode,"height",tmp.height+"px");
			}
		}	
		e.preventDefault();
	},

	_endSizing: function(/*Event*/ e){
		// summary: disconnect listenrs and cleanup sizing
		dojo.forEach(this._connects,function(c){
			dojo.disconnect(c); 
		});
		if(!this.activeResize){
			dojo.fadeOut({ node:this._activeResizeNode, duration:250,
				onEnd: dojo.hitch(this,function(){
					this._activeResizeNode.style.display="none";
				})
			}).play();
			this._changeSizing(e);
		}
		this._isSizing = false;
	}

});

}

if(!dojo._hasResource['hub.misc.popup']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.misc.popup'] = true;
dojo.provide('hub.misc.popup');
//
// JavaScript popup layer support
// Version 1.11
//
// Author: Scott Morgan (morgans@us.ibm.com)
//
// Change history:
//    13 Oct 2006 - 1.00 Initial release.
//    25 Oct 2006 - 1.01 Add SurfAid support.
//    07 Nov 2006 - 1.02 Change visuals to support latest UI design.
//    11 Nov 2006 - 1.03 Modify zIndex on show and hide.
//    29 Nov 2006 - 1.04 Add support for lining up the layer to the left/right of the target.
//    17 Jan 2007 - 1.05 Add support for layers to be displayed in "bubbles" with new positioning styles.
//    22 Jan 2007 - 1.06 Put popup styles into a flag and add showPopupLayer2 function that uses them.
//    22 Jan 2007 - 1.07 Modal dialog layers now reposition like other layers on window resize, and
//                       moveable layers now reposition on window resize until the user has manually moved them.
//                       Previously neither repostioned on window resize.
//    29 Jan 2007 - 1.08 Allow the bubble width or height dimension to be smaller than the fin base if the fin
//                       is not to be displayed on that side.
//    08 Feb 2007 - 1.09 Allow forms displayed in bubble to retain field values on subsequent appearances.
//    09 Feb 2007 - 1.10 Add a flag that indicates the layer is a form, and stop using the modal dialog flag
//                       to indicate this as well, as not all forms need be modal.
//    18 Feb 2007 - 1.11 Prepare file for sharing with other development organizations.
//

//
// Global constants
//

//
// Path of the common image repository where the images will be found.
// The popup_layer.css file also references an image whose path will need to be changed to correspond
// to the path defined here.
//
pl_IMAGE_PATH = "images/popup/";

set_pl_IMAGE_PATH = function (path)
{
	pl_IMAGE_PATH = path;
	pl_BUBBLE_FIN_FULLPATH_NL = pl_IMAGE_PATH + "fin_nl.gif";
	pl_BUBBLE_FIN_FULLPATH_NR = pl_IMAGE_PATH + "fin_nr.gif";
	pl_BUBBLE_FIN_FULLPATH_SL = pl_IMAGE_PATH + "fin_sl.gif";
	pl_BUBBLE_FIN_FULLPATH_SR = pl_IMAGE_PATH + "fin_sr.gif";
	pl_BUBBLE_FIN_FULLPATH_EU = pl_IMAGE_PATH + "fin_eu.gif";
	pl_BUBBLE_FIN_FULLPATH_ED = pl_IMAGE_PATH + "fin_ed.gif";
	pl_BUBBLE_FIN_FULLPATH_WU = pl_IMAGE_PATH + "fin_wu.gif";
	pl_BUBBLE_FIN_FULLPATH_WD = pl_IMAGE_PATH + "fin_wd.gif";
	pl_BUBBLE_CORNER_FULLPATH_NW = pl_IMAGE_PATH + "corner_nw.gif";
	pl_BUBBLE_CORNER_FULLPATH_NE = pl_IMAGE_PATH + "corner_ne.gif";
	pl_BUBBLE_CORNER_FULLPATH_SW = pl_IMAGE_PATH + "corner_sw.gif";
	pl_BUBBLE_CORNER_FULLPATH_SE = pl_IMAGE_PATH + "corner_se.gif";
	pl_BUBBLE_SIDE_FULLPATH_N = pl_IMAGE_PATH + "side_n.gif";
	pl_BUBBLE_SIDE_FULLPATH_S = pl_IMAGE_PATH + "side_s.gif";
	pl_BUBBLE_SIDE_FULLPATH_E = pl_IMAGE_PATH + "side_e.gif";
	pl_BUBBLE_SIDE_FULLPATH_W = pl_IMAGE_PATH + "side_w.gif";
	// Images for the banner - may also be referenced in popup_layers.css.
	pl_POPUPBANNER_FULLPATH = pl_IMAGE_PATH + "popupbanner.gif";
	pl_POPUPBANNERCLOSE_FULLPATH = pl_IMAGE_PATH + "popupclosex.gif";
	pl_CLEARPIX_FULLPATH = pl_IMAGE_PATH + "c.gif";
	pl_BUBBLE_CONTENT_MINWIDTH_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='1' alt='' />";
	pl_BUBBLE_CONTENT_MINHEIGHT_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='1' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	// HTML that is swapped to replace the fine being used with one pointing in a different direction.
	pl_BUBBLE_FIN_NL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_NR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_SL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_SR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_EU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_EU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	pl_BUBBLE_FIN_ED = "<img src='" + pl_BUBBLE_FIN_FULLPATH_ED + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	pl_BUBBLE_FIN_WU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	pl_BUBBLE_FIN_WD = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WD + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
}

//
// Externalized variables and constants
//
BANNER_ID_SUFFIX = "popupbanner";  // Banner id is the layer id + this suffix
login_iframe_active = false;


//
// Image constants
//
// Images for the bubble
pl_BUBBLE_FIN_FULLPATH_NL = pl_IMAGE_PATH + "fin_nl.gif";
pl_BUBBLE_FIN_FULLPATH_NR = pl_IMAGE_PATH + "fin_nr.gif";
pl_BUBBLE_FIN_FULLPATH_SL = pl_IMAGE_PATH + "fin_sl.gif";
pl_BUBBLE_FIN_FULLPATH_SR = pl_IMAGE_PATH + "fin_sr.gif";
pl_BUBBLE_FIN_FULLPATH_EU = pl_IMAGE_PATH + "fin_eu.gif";
pl_BUBBLE_FIN_FULLPATH_ED = pl_IMAGE_PATH + "fin_ed.gif";
pl_BUBBLE_FIN_FULLPATH_WU = pl_IMAGE_PATH + "fin_wu.gif";
pl_BUBBLE_FIN_FULLPATH_WD = pl_IMAGE_PATH + "fin_wd.gif";
pl_BUBBLE_CORNER_FULLPATH_NW = pl_IMAGE_PATH + "corner_nw.gif";
pl_BUBBLE_CORNER_FULLPATH_NE = pl_IMAGE_PATH + "corner_ne.gif";
pl_BUBBLE_CORNER_FULLPATH_SW = pl_IMAGE_PATH + "corner_sw.gif";
pl_BUBBLE_CORNER_FULLPATH_SE = pl_IMAGE_PATH + "corner_se.gif";
pl_BUBBLE_SIDE_FULLPATH_N = pl_IMAGE_PATH + "side_n.gif";
pl_BUBBLE_SIDE_FULLPATH_S = pl_IMAGE_PATH + "side_s.gif";
pl_BUBBLE_SIDE_FULLPATH_E = pl_IMAGE_PATH + "side_e.gif";
pl_BUBBLE_SIDE_FULLPATH_W = pl_IMAGE_PATH + "side_w.gif";
// Images for the banner - may also be referenced in popup_layers.css.
pl_POPUPBANNER_FULLPATH = pl_IMAGE_PATH + "popupbanner.gif";
pl_POPUPBANNERCLOSE_FULLPATH = pl_IMAGE_PATH + "popupclosex.gif";
pl_POPUPBANNERCLOSE_WIDTH = "6";
pl_POPUPBANNERCLOSE_HEIGHT = "7";
// 1 pixel transparent image
pl_CLEARPIX_FULLPATH = pl_IMAGE_PATH + "c.gif";
// Bubble image dimensions
pl_BUBBLE_FIN_REACH = 42;//33;  // From the tip of the fin to the base.
pl_BUBBLE_FIN_BASE = 61;//56;   // Length of the base of the fin.
pl_BUBBLE_FIN_INSET = 12;//11;  // Amount the fin image overlays the bubble.
pl_BUBBLE_FIN_INSET_Y = 15; //@GINO: necessary for new GIF images
pl_BUBBLE_CORNER_WIDTH = 15;
pl_BUBBLE_CORNER_HEIGHT = 15;
pl_BUBBLE_SIDE_DEPTH = 15;
pl_BUBBLE_SIDE_LENGTH = 5;
// Banner image dimensions
pl_BANNER_BG_WIDTH = 84;
pl_BANNER_BG_HEIGHT = 28;
pl_BANNER_CLOSE_X_WIDTH = 6;
pl_BANNER_CLOSE_X_HEIGHT = 7;

// Bubble-specific table cell IDs
pl_BUBBLE_CONTENT_MINHEIGHT_ID = "popupLayerBubbleHeight";
pl_BUBBLE_CONTENT_MINWIDTH_ID = "popupLayerBubbleWidth";
// HTML that is swapped in and out of the table cell IDs that regulate a minimum dimension to ensure the fin fits on that side.
pl_BUBBLE_CONTENT_MINWIDTH_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='1' alt='' />";
pl_BUBBLE_CONTENT_MINHEIGHT_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='1' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
// HTML that is swapped to replace the fine being used with one pointing in a different direction.
pl_BUBBLE_FIN_NL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_NR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_SL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_SR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_EU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_EU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
pl_BUBBLE_FIN_ED = "<img src='" + pl_BUBBLE_FIN_FULLPATH_ED + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
pl_BUBBLE_FIN_WU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
pl_BUBBLE_FIN_WD = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WD + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";

pl_LOGIN_IFRAME_ID_SUFFIX = "w3signonIFrame";  // IFrame id is the layer id + this suffix
pl_DRAG_START_DIFFERENTIAL = 0;  // Pixels the mouse must be moved more than while help down to start the drag operation
pl_MOUSEOVER_SHOW_DELAY = 250;  // Milliseconds
pl_MOUSEOUT_HIDE_DELAY = 250;  // Milliseconds
pl_DRAG_VISIBILITY_BUFFER = 20;  // Pixels of drag object to always appears in view when moved top and left

pl_BUBBLE_ID = "popupLayerBubble";  // ID of the bubble
pl_BUBBLE_CONTENT_ID = "popupLayerBubbleContent";  // ID of the bubble cell that contains the layer content
pl_BUBBLE_FIN_ID = "popupLayerBubbleFin";  // ID of the object that contains the bubble fin
pl_BUBBLECLONE_ID_ADDON = "_bubbleclone";  // ID of the layer within a bubble id layer id + this suffix
pl_CONTENT_MAIN_ID = "content-main";  // ID of section of web page where cloned layer within bubble is placed

// Sizing and positioning constants
pl_IDEAL_NORTH_SOUTH_POSITION = .4;
pl_IDEAL_EAST_WEST_FIN_POSITION = .25;
pl_BIG_OFFSCREEN = 10000;
pl_FINPOS_INDENT_DEFAULT = 8;

//
// External flag bits for the showPopupLayer call
//
PL_ADD_CLOSE_LINK =       0x0001;
PL_ADD_BANNER =           0x0002;
PL_DIALOG_BEHAVIOR =      0x0004;
PL_LOGIN_REQUIRED =       0x0008;
PL_MOUSE_OVER_SUPPORT =   0x0010;
PL_DISPLAY_IN_BUBBLE =    0x0020;
PL_SUGGESTED_POS_BELOW =  0x0040;
PL_SUGGESTED_POS_ABOVE =  0x0080;
PL_SUGGESTED_POS_LEFT =   0x0100;
PL_SUGGESTED_POS_RIGHT =  0x0200;
PL_FORCE_VISIBLE_TARGET = 0x0400;
PL_FORCE_POSITION =       0x0800;
PL_FORM =                 0x1000;


//
// Global variables
//
pl_active_layer_id = "";
pl_active_layer_obj = null;
pl_active_clonedlayer_obj = null;
pl_active_contentclone_obj = null;
pl_active_trigger_id = "";
pl_active_trigger_obj = null;
pl_active_target_obj = null;
pl_active_drag_obj = null;
pl_active_mybubblefin = null;
pl_active_layer_html = "";
pl_active_IFrame_id = "";
pl_active_surfaid_base_string = "";
pl_active_style_flags = 0x0;
pl_active_has_dragged = false;
pl_active_bubble_minwidth_cell = null;
pl_active_bubble_minheight_cell = null;
pl_clicked_layer = false;
pl_onclick_doc_prevEH =  "";
pl_onkeypress_doc_prevEH =  "";
pl_onresize_win_prevEH =  "";
pl_onunload_win_prevEH = "";
pl_onmousemove_body_prevEH = "";
pl_onmouseup_body_prevEH = "";
pl_onclick_trigger_prevEH =  "";
pl_onmouseover_trigger_prevEH = "";
pl_onfocus_trigger_prevEH = "";
pl_active_drag_obj_prevcursor = "";
pl_queued_show_layer_request = false;
pl_queued_hide_layer_request = false;
pl_queued_show_layer_timer_id = -1;
pl_queued_hide_layer_timer_id = -1;
pl_move_in_process = false;
pl_drag_mouse_down = false;
pl_move_init_x = 0;
pl_move_init_y = 0;
pl_move_min_x = 0;
pl_move_min_y = 0;
pl_move_offset_x = 0;
pl_move_offset_y = 0;
pl_bubble_images_preloaded = false;
pl_bubble_orig_height = 0;
pl_bubble_orig_width = 0;

//
// Preloaded images
//
pl_img_clearpix = null;
pl_img_banner_bg = null;
pl_img_close_x = null;
pl_img_bubble_fin_nl = null;
pl_img_bubble_fin_nr = null;
pl_img_bubble_fin_sl = null;
pl_img_bubble_fin_sr = null;
pl_img_bubble_fin_eu = null;
pl_img_bubble_fin_ed = null;
pl_img_bubble_fin_wu = null;
pl_img_bubble_fin_wd = null;
pl_img_bubble_corner_nw = null;
pl_img_bubble_corner_ne = null;
pl_img_bubble_corner_sw = null;
pl_img_bubble_corner_se = null;
pl_img_bubble_side_w = null;
pl_img_bubble_side_e = null;
pl_img_bubble_side_n = null;
pl_img_bubble_side_s = null;




//
// Call to preload the images when this script file loads, so they will already be loaded the first time requested.
//
//pl_preloadPopupLayerImages();



// -----------------------------------------------------------------------------
//
// ENTRY POINT functions called externally (no pl_ prefix)
//
// showPopupLayer2 - position and show the popup layer.
// hidePopupLayer - hide the popup layer if one is showing.
// getPopupLayerElementById - return the object in the popup layer with the specified ID.
//
// showPopupLayer - DEPRECATED, included for backward compatibility only.
//
// -----------------------------------------------------------------------------

//
// ENTRY POINT from web pages: position and show the popup layer.
// Calling convention:
//     href="javascript:showPopupLayer2(....)" or href="#" onclick="showPopupLayer(....)" for anchor tag click support
//     onclick="showPopupLayer2(....)" for spans, divs, etc. click support
//     onmouseover=showPopupLayer2(....) if mouse over and mouse out support desired
//     onfocus=showPopupLayer2(....) for keyboard support when mouse over and mouse out support desired
// Required parms:
// Parm 1 (id of the popup layer)
// Parm 2 (id of the target object to which it should align itself)
// Parm 3 (id of the trigger object which when clicked on caused this call)
// Optional parms:
// Parm 4 (style flags, bitwise OR'ed or added together, default = no styles set)
// Parm 5 (id of the object that will be used for dragging the layer, default = "")
// Parm 6 (SurfAid base string used for logging that layer was dismissed, default = "")
//
// Style flags passed in parm 4:
// - PL_ADD_CLOSE_LINK = add close link at lower left corner of popup.
// - PL_ADD_BANNER = add the onePlace banner with close icon to top of popup.
// - PL_FORM = the layer contains a form that may be submitted by the user.
// - PL_DIALOG_BEHAVIOR = popup layer acts like a modal dialog.
// - PL_LOGIN_REQUIRED = secure login is required.
// - PL_MOUSE_OVER_SUPPORT = mouse over and out support is desired to show and hide this object.
// - PL_DISPLAY_IN_BUBBLE = display the popup as a bubble enclosing the user-defined popup layer.
// - PL_SUGGESTED_POS_BELOW = Default if no position suggested or multiple positions suggested: position the popup below the target if it fits, else try above, left, then right.
// - PL_SUGGESTED_POS_ABOVE = position the popup above the target if it fits, else try below, left, then right.
// - PL_SUGGESTED_POS_LEFT = position the popup to the left of the target if it fits, else try right, below, then above.
// - PL_SUGGESTED_POS_RIGHT = position the popup to the right of the target if it fits, else try left, below, then above.
// - PL_FORCE_VISIBLE_TARGET = Always kep the target visible, even if this results in the popup layer not being fully visible.
// - PL_FORCE_POSITION = always use the suggested position and keep the target visible, even if this results in the popup layer not being fully visible.
//
showPopupLayer2 = function (idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string) {
   // If acts like a dialog, clicking or mouse over outside the layer does not get processed,
   // and we do not display any other layers while this one is up, simulating the layer being modal.
   if (pl_active_style_flags & PL_DIALOG_BEHAVIOR) {
      return;
   }

   // First we see if the optional parms are passed in - if not, we set any not passed in to their default values.
   // If the three required parms are not passed in, nothing we can do here so just return.
   var numArgs = showPopupLayer2.arguments.length;
   if (numArgs < 6) {
      surfaid_base_string = "";
      if (numArgs < 5) {
         idDragger = "";
         if (numArgs < 4) {
            style_flags = 0x0;
            if (numArgs < 3) {
               return;
            }
         }
      }
   }
   // Now support passing in null for any optional parm to take its default value.
   if (style_flags == null) {
      style_flags = 0x0;
   }
   if (idDragger == null) {
      idDragger = "";
   }
   if (surfaid_base_string == null) {
      surfaid_base_string = "";
   }

   // If no suggested position was passed in, or more than one suggested position was passed in, set to default position of below pos.
   var sug_pos = style_flags & (PL_SUGGESTED_POS_BELOW | PL_SUGGESTED_POS_ABOVE | PL_SUGGESTED_POS_LEFT | PL_SUGGESTED_POS_RIGHT);
   if (sug_pos == 0x0) {
      style_flags |= PL_SUGGESTED_POS_BELOW;
   } else if ((sug_pos != PL_SUGGESTED_POS_BELOW) && (sug_pos != PL_SUGGESTED_POS_ABOVE) && (sug_pos != PL_SUGGESTED_POS_LEFT) && (sug_pos != PL_SUGGESTED_POS_RIGHT)) {
      if (style_flags & PL_SUGGESTED_POS_ABOVE) {
         style_flags = ~((~style_flags) ^ PL_SUGGESTED_POS_ABOVE);
      }
      if (style_flags & PL_SUGGESTED_POS_LEFT) {
         style_flags = ~((~style_flags) ^ PL_SUGGESTED_POS_LEFT);
      }
      if (style_flags & PL_SUGGESTED_POS_RIGHT) {
         style_flags = ~((~style_flags) ^ PL_SUGGESTED_POS_RIGHT);
      }
      if (!(style_flags & PL_SUGGESTED_POS_BELOW)) {
         style_flags |= PL_SUGGESTED_POS_BELOW;
      }
   }

   var trigger = document.getElementById(idTrigger);
   var layer = document.getElementById(idLayer);
   var target = document.getElementById(idTarget);
   // If any of the three objects is undefined, do not attempt to process.
   if ((layer == null) || (target == null) || (trigger == null)) {
      return;
   }

   var mybubble = null;
   var mybubblefin = null;

   // Set the mouse out and blur event handlers since they are cleared when called if mouse support requested.
   if (style_flags & PL_MOUSE_OVER_SUPPORT) {
      trigger.onmouseout = pl_processTriggerMouseOutAndBlur;
      trigger.onblur = pl_processTriggerMouseOutAndBlur;
   }

   // If this is a mouse over action, and we do not have a previous request in the queue to show a layer.
   if ((style_flags & PL_MOUSE_OVER_SUPPORT) && (!pl_queued_show_layer_request)) {
      // If there is a queued hide request, kill it as any layer will be hidden when this queued show request runs.
      if (pl_queued_hide_layer_request) {
         pl_queued_hide_layer_request = false;
         pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 1");
      }
      pl_queued_show_layer_request = true;
      // Set a timer to show the layer.
      pl_queued_show_layer_timer_id = window.setTimeout("pl_showDelayedPopupLayer('" + idLayer + "', '" + idTarget + "', '" + idTrigger + "', " + style_flags + ", '" + idDragger + "', '" + surfaid_base_string + "')", pl_MOUSEOVER_SHOW_DELAY);
      // Exit and wait for the timer.
      return;
   }

   // Hide current layer if one is already showing.
   if ((pl_active_layer_obj != null)) {
      hidePopupLayer();
   }

   // Clear any possible queued requests
   if (pl_queued_show_layer_request) {
      pl_queued_show_layer_request = false;
      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 2");
   }
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 3");
   }

   // If we are displaying the layer in a bubble, create the bubble now.
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {

      mybubblefin = document.getElementById(pl_BUBBLE_FIN_ID);
      if (mybubblefin == null) {
         // Create the new bubble fin
         mybubblefin = document.createElement("div");
         mybubblefin.id = pl_BUBBLE_FIN_ID;
         mybubblefin.style.position = "absolute";
         mybubblefin.style.top = "10px";
         mybubblefin.style.left = "10px";
         mybubblefin.style.zIndex = -1;
         mybubblefin.style.visibility = "hidden";
         mybubblefin.innerHTML = "";
         // Add it to the document
         document.body.appendChild(mybubblefin);
      }
      pl_active_mybubblefin = mybubblefin;

      // Get the bubble object, creating it the first time needed.
      mybubble = document.getElementById(pl_BUBBLE_ID);
      if (mybubble == null) {
         // Create the table layout for the bubble.
         var bubble_table = "";
         bubble_table += "<table class='w3pluto1' cellpadding='0' cellspacing='0' border='0' summary=''>";
         bubble_table += "<tr>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_NW + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "<td id='" + pl_BUBBLE_CONTENT_MINWIDTH_ID + "' style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_N + "); background-repeat:repeat-x; background-position:top left; font-size: 1px;'></td>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_NE + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "</tr>";
         bubble_table += "<tr>";
         bubble_table += "<td id='" + pl_BUBBLE_CONTENT_MINHEIGHT_ID + "' style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_W + "); background-repeat:repeat-y; background-position:top left; font-size: 1px;'></td>";
         bubble_table += "<td class='pl_bubble_content_area' id='" + pl_BUBBLE_CONTENT_ID + "'></td>";
         bubble_table += "<td style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_E + "); background-repeat:repeat-y; background-position:top right; font-size: 1px;'></td>";
         bubble_table += "</tr>";
         bubble_table += "<tr>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_SW + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "<td style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_S + "); background-repeat:repeat-x; background-position:bottom left; font-size: 1px;'></td>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_SE + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "</tr>";
         bubble_table += "</table>";

         // Create the new bubble object and assign the bubble table to it.
         mybubble = document.createElement("div");
         mybubble.id = pl_BUBBLE_ID;
         mybubble.style.position = "absolute";
         mybubble.style.top = "10px";
         mybubble.style.left = "10px";
         mybubble.style.zIndex = -1;
         mybubble.style.visibility = "hidden";
         mybubble.innerHTML = bubble_table;
         // Make it a child of the w3 content area if one exists so it picks up styles like the rest of the content.
         var content_main = document.getElementById(pl_CONTENT_MAIN_ID);
         if (content_main != null) {
            content_main.appendChild(mybubble);
         } else {
            document.body.appendChild(mybubble);
         }
      }
      // Store the bubble cells that dictate minumum width and height depending in which side the fin will appear.
      if (pl_active_bubble_minwidth_cell == null) {
         pl_active_bubble_minwidth_cell = document.getElementById(pl_BUBBLE_CONTENT_MINWIDTH_ID);
      }
      if (pl_active_bubble_minheight_cell == null) {
         pl_active_bubble_minheight_cell = document.getElementById(pl_BUBBLE_CONTENT_MINHEIGHT_ID);
      }
      // Start with no min width or height until the positioning calc is run.
      pl_active_bubble_minwidth_cell.innerHTML = "";
      pl_active_bubble_minheight_cell.innerHTML = "";
      // Reset global vars that indicate the original layer width and height have not yet been saved for the positioning calc.
      pl_bubble_orig_width = 0;
      pl_bubble_orig_height = 0;

      // Put the layout table in the bubble.
      var mybubblecontent = document.getElementById(pl_BUBBLE_CONTENT_ID);
      // Clear out any html from a previous bubble.
      mybubblecontent.innerHTML = "";
      // Clone the display layer so we get the entire DIV and all its styles, etc.
      var mycontentclone = layer.cloneNode(true);
      mycontentclone.style.position = "static";
      mycontentclone.style.visibility = "inherit";
      // Give the clone layer a unique ID so there is no confusion on the layer level caused by duplicate IDs.
      mycontentclone.id = mycontentclone.id + pl_BUBBLECLONE_ID_ADDON;
      // Add the cloned display layer into the bubble.
      mybubblecontent.appendChild(mycontentclone);

      // Now set the layer variable to point to the cloned layer for the following code, as we no longer need
      // to reference the original layer.
      pl_active_clonedlayer_obj = layer;
      pl_active_contentclone_obj = mycontentclone;
      layer = mycontentclone;
   }

   // If we are displaying the layer in a bubble, it is the bubble that will be the active layer.
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      pl_active_layer_id = pl_BUBBLE_ID;
      pl_active_layer_obj = mybubble;
   } else {
      pl_active_layer_id = idLayer;
      pl_active_layer_obj = layer;
   }
   // Set globals for this showing of the layer
   pl_active_trigger_id = idTrigger;
   pl_active_trigger_obj = trigger;
   pl_active_target_obj = target;
   pl_active_style_flags = style_flags;
   pl_active_surfaid_base_string = surfaid_base_string;

   // Store the original html for the popup layer
   pl_active_layer_html = layer.innerHTML;
   // Add the Close link unless the optional parm said to not show it, and as long as it has not already been added.
   if (style_flags & PL_ADD_CLOSE_LINK) {
      if (layer.innerHTML.indexOf("pl_bottom_close_link") < 0) {
         layer.innerHTML = layer.innerHTML + "<table width='100%' cellpadding='0' cellspacing='0' border='0' summary=''><tr><td align='right' class='pl_bottom_close_link'><a class='pl_bottom_close_link_anchor' href='javascript:hidePopupLayer();'>Close</a></td></tr></table>"
      }
   }

   // Add the w3 banner to the beginning if optional parm said to do so.
   var banner = null;
   if (style_flags & PL_ADD_BANNER) {
      var banner_id = idLayer + BANNER_ID_SUFFIX;
     if (layer.innerHTML.indexOf(banner_id) < 0) {
        var banner_html = "<div id='" + banner_id + "' class='pl_banner'><a href='javascript:hidePopupLayer();'><img src='" + pl_POPUPBANNERCLOSE_FULLPATH + "' class='pl_banner_close' width='" + pl_POPUPBANNERCLOSE_WIDTH + "' height='" + pl_POPUPBANNERCLOSE_HEIGHT + "' alt='Close' /></a></div>"
        layer.innerHTML = banner_html + layer.innerHTML;
     }
     banner = document.getElementById(banner_id);
   }

   // Secure login support (supported for the Pluto-driven onePlace site only)
   var IFrame_id = "";
   if (style_flags & PL_LOGIN_REQUIRED) {
      // Set the flag that tells the login failure page code we are playing in an IFrame
      login_iframe_active = true;
      IFrame_id = idLayer + pl_LOGIN_IFRAME_ID_SUFFIX;
      pl_active_IFrame_id = IFrame_id;
      // Add the hidden layer that will contain the IFrame if not already added.
      if (layer.innerHTML.indexOf(IFrame_id) < 0) {
         layer.innerHTML = layer.innerHTML + "<div id='" + IFrame_id + "' name='" + IFrame_id + "' style='width:1px; height:1px; border-style:none; display:none; position:absolute; top:1px; left:1px;'></div>"
      }
      // Load the blank secure page into the IFrame
      document.getElementById(IFrame_id).innerHTML = "<iframe src='../spage/13' style='width:1px; height:1px; border-style:none; display:none; position:absolute; top:1px; left:1px;'></iframe>";
   }

   // Set the zIndex if not already set so the layer will appear above the w3 banner, which is zIndex=1
   // If display in bubble, set the bubble's z-order
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      mybubble.style.zIndex = 2;
      mybubblefin.style.zIndex = 3;
   }
   if (layer.style.zIndex < 2) {
      layer.style.zIndex = 2;
   }

   // Get the object that will be used for dragging the layer, if any.
   if (idDragger != "") {
      // If display in bubble, we need to get the object from the cloned layer, not the display layer.
      if (style_flags & PL_DISPLAY_IN_BUBBLE) {
         // if the request was to make the entire layer draggable, and the layer is in a bubble, make the bubble draggable instead..
         if (idDragger == idLayer) {
            pl_active_drag_obj = document.getElementById(pl_BUBBLE_ID);
         } else {
            pl_active_drag_obj = getPopupLayerElementById(idDragger);
         }
      } else {
         pl_active_drag_obj = document.getElementById(idDragger);
      }
   }

   // Position the layer relative to the target
   var show_fin;
   show_fin = pl_positionPopupLayer();

   // Now make the popup layer visible
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      mybubble.style.visibility = "visible";
      // Is fin showing or not based on position?
      if (show_fin) {
         mybubblefin.style.visibility = "visible";
      } else {
         mybubblefin.style.zIndex = -1;
      }
   } else {
      layer.style.visibility = "visible";
   }

   // And we reassign the layer var to be the bubble itself for event handling assignment below.
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      layer = mybubble;
   }

   // Capture relevant events while the popup layer is up.
   // First we need to save any event handlers already assigned for system objects before we override them.
   // Generic handlers for all types of layers.
   pl_onkeypress_doc_prevEH = document.onkeypress;
   document.onkeypress = pl_processBodyKey;
   pl_onresize_win_prevEH = window.onresize;
   window.onresize = pl_processResize;
   pl_onunload_win_prevEH = window.onunload;
   window.onunload = pl_processWindowUnload;
   // Handlers for layers that support mouse over and out for display
   if (style_flags & PL_MOUSE_OVER_SUPPORT) {
      pl_onmouseover_trigger_prevEH = trigger.onmouseover;
      trigger.onmouseover = pl_processTriggerMouseOverAndFocus;
      pl_onfocus_trigger_prevEH =  trigger.onfocus;
      trigger.onfocus = pl_processTriggerMouseOverAndFocus;
      layer.onmouseover = pl_processLayerMouseOver;
      layer.onmouseout = pl_processLayerMouseOut;
      if ((style_flags & PL_DISPLAY_IN_BUBBLE) && (mybubblefin != null)) {
         mybubblefin.onmouseover = pl_processLayerMouseOver;
         mybubblefin.onmouseout = pl_processLayerMouseOut;
      }

   } else {
      // Handlers for layers that support clicking for display
      pl_onclick_doc_prevEH = document.onclick;
      document.onclick = pl_processBodyClick;
      pl_onclick_trigger_prevEH = trigger.onclick;
      trigger.onclick = pl_processTriggerClick;
      layer.onclick = pl_processLayerClick;
      if ((style_flags & PL_DISPLAY_IN_BUBBLE) && (mybubblefin != null)) {
         mybubblefin.onclick = pl_processLayerClick;
      }
   }
   // Handlers for layers that support drag/drop.
   if (pl_active_drag_obj != null) {
      pl_onmousemove_body_prevEH = document.body.onmousemove;
      document.body.onmousemove = pl_processDragMouseMove;
      pl_onmouseup_body_prevEH = document.body.onmouseup;
      document.body.onmouseup = pl_processDragMouseUp;
      pl_active_drag_obj.onmousedown = pl_processDragMouseDown;
      pl_active_drag_obj.onlosecapture = pl_processDragLoseCapture;
      // Set cursor to the move pointer when the mouse is over the drag area.
      pl_active_drag_obj_prevcursor = pl_active_drag_obj.style.cursor;
      pl_active_drag_obj.style.cursor = "move";
   }
}


//
// DEPRECTATED - OLD ENTRY POINT from web pages, supported for backward compatibility: position and show the popup layer.
// Calling convention:
//     href="javascript:showPopupLayer(....)" or href="#" onclick="showPopupLayer(....)" for anchor tag click support
//     onclick="showPopupLayer(....)" for spans, divs, etc. click support
//     onmouseover=showPopupLayer(....) if mouse over and mouse out support desired
//     onfocus=showPopupLayer(....) for keyboard support when mouse over and mouse out support desired
// Required parms:
// Parm 1 (id of the popup layer)
// Parm 2 (id of the target object to which it should align itself)
// Parm 3 (id of the trigger object which when clicked on caused this call)
// Optional parms:
// Parm 4 (add the close link, default = true)
// Parm 5 (add the onePlace banner with close icon to top, default = false)
// Parm 6 (popup layer acts like a modal dialog, default = false)
// Parm 7 (secure login is required, default = false)
// Parm 8 (id of the object that will be used for dragging the layer, default = "")
// Parm 9 (mouse over and out support is desired to show and hide this object, default = false)
// Parm 10 (SurfAid base string used for logging that layer was dismissed, default = "")
// Parm 11 (align popup layer to the left or right of the target if possible, default = false)
//
showPopupLayer = function (idLayer, idTarget, idTrigger, add_close, add_w3_banner, dialog_behavior, login_required, idDragger, mouse_over_out_support, surfaid_base_string, align_left_right) {

   // First we see if the optional parms are passed in - if not, we set any not passed in to their default values.
   // If the three required parms are not passed in, nothing we can do here so just return.
   var numArgs = showPopupLayer.arguments.length;
   if (numArgs < 11) {
      align_left_right = false;
      if (numArgs < 10) {
         surfaid_base_string = "";
         if (numArgs < 9) {
            mouse_over_out_support = false;
            if (numArgs < 8) {
               idDragger = "";
               if (numArgs < 7) {
                  login_required = false;
                  if (numArgs < 6) {
                     dialog_behavior = false;
                     if (numArgs < 5) {
                        add_w3_banner = false;
                        if (numArgs < 4) {
                           add_close = true;
                           if (numArgs < 3) {
                              return;
                           }
                        }
                     }
                  }
               }
            }
         }
      }
   }

   // Now set the new style flag to pass to the new function.
   var style_flags = 0x0;
   if (add_close) {
      style_flags |= PL_ADD_CLOSE_LINK;
   }
   if (add_w3_banner) {
      style_flags |= PL_ADD_BANNER;
   }
   if (dialog_behavior) {
      style_flags |= PL_DIALOG_BEHAVIOR;
      style_flags |= PL_FORM;
   }
   if (login_required) {
      style_flags |= PL_LOGIN_REQUIRED;
   }
   if (mouse_over_out_support) {
      style_flags |= PL_MOUSE_OVER_SUPPORT;
   }
   if (align_left_right) {
      style_flags |= PL_SUGGESTED_POS_LEFT;
   }

   // Now pass the modified parms to the new function call.
   showPopupLayer2(idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string);
}


//
// ENTRY POINT from web pages: hide the popup layer if one is showing.
//
// Optional parm:
// Parm 1 (do not notify SurfAid of this layer closing, default = false)
//
hidePopupLayer = function (ignore_surfaid) {
   if (pl_active_layer_obj != null) {
      var numArgs = hidePopupLayer.arguments.length;
      // Default SurfAid action is to log that the layer was canceled
      if (numArgs < 1) {
         ignore_surfaid = false;
      }
      // End any dragging that may be going on.
      pl_endLayerDrag();
      // Hid the bubble fin if visible
      if (pl_active_mybubblefin != null) {
         pl_active_mybubblefin.style.visibility = "hidden";
         pl_active_mybubblefin.style.zIndex = -1;
         pl_active_mybubblefin.style.left = 0;
         pl_active_mybubblefin.style.top = 0;
         pl_active_mybubblefin = null;
      }
      // Hide the popup layer
      pl_active_layer_obj.style.visibility = "hidden";
      pl_active_layer_obj.style.left = 0;
      pl_active_layer_obj.style.top = 0;

      // 1.03 Set the zIndex so the layer is below the content.
      pl_active_layer_obj.style.zIndex = -1;

      // Restore the original html to the popup layer, unless this is a layer
      // which will be submitting a form and therefore can not have its html reset,
      // or the form submission stops.
      if (!(pl_active_style_flags & PL_FORM)) {
         if (pl_active_layer_html != "") {
            // No need to restore the HTML if in a bubble since we cloned the original layer and used the clone.
            if (!(pl_active_style_flags & PL_DISPLAY_IN_BUBBLE)) {
               pl_active_layer_obj.innerHTML = pl_active_layer_html;
            }
            pl_active_layer_html = "";
         }
      } else {
         // So this is a form.
         // Clear secure hidden IFrame if set. There was no need to clear them if we restored the inner html above,
         // which would have cleared the hidden IFrame as well.
         if (login_iframe_active) {
            if (pl_active_IFrame_id != "") {
               var secure_iframe_obj = document.getElementById(pl_active_IFrame_id);
               if (secure_iframe_obj != null) {
                  secure_iframe_obj.innerHTML = "";
               }
            }
         }
         // If the form is displayed in a bubble, we want to copy the cloned layer contents to the
         // original layer so that any changes to form controls will be present the next time the form is displayed.
         if (pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) {
            if ((pl_active_clonedlayer_obj != null) && (pl_active_contentclone_obj != null)) {
               pl_active_clonedlayer_obj.innerHTML = pl_active_contentclone_obj.innerHTML;
            }
         }
      }

      login_iframe_active = false;
      pl_active_layer_id = "";
      pl_active_trigger_id = "";
      pl_active_IFrame_id = "";
      pl_clicked_layer = false;
      pl_active_has_dragged = false;

      // Reset or clear event handlers.
      // Generic handlers for all types of layers.
      document.onkeypress = pl_onkeypress_doc_prevEH;
      pl_onkeypress_doc_prevEH = "";
      window.onresize = pl_onresize_win_prevEH;
      pl_onresize_win_prevEH = "";
      window.onunload = pl_onunload_win_prevEH;
      pl_onunload_win_prevEH = "";
      // Handlers for layers that support mouse over and out for display
      if (pl_active_style_flags & PL_MOUSE_OVER_SUPPORT) {
         pl_active_trigger_obj.onmouseout = "";
         pl_active_trigger_obj.onblur = "";
         pl_active_trigger_obj.onmouseover = pl_onmouseover_trigger_prevEH;
         pl_onmouseover_trigger_prevEH = "";
         pl_active_trigger_obj.onfocus = pl_onfocus_trigger_prevEH;
         pl_onfocus_trigger_prevEH = "";
         pl_active_layer_obj.onmouseout = "";
         pl_active_layer_obj.onmouseover = "";
         if ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) && (pl_active_mybubblefin != null)) {
            pl_active_mybubblefin.onmouseover = "";
            pl_active_mybubblefin.onmouseout = "";
         }
      } else {
         // Handlers for layers that support clicking for display
         document.onclick = pl_onclick_doc_prevEH;
         pl_onclick_doc_prevEH = "";
         pl_active_trigger_obj.onclick = pl_onclick_trigger_prevEH;
         pl_onclick_trigger_prevEH = "";
         pl_active_layer_obj.onclick = "";
         if ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) && (pl_active_mybubblefin != null)) {
            pl_active_mybubblefin.onclick = "";
         }
      }
      // Handlers for layers that support drag/drop.
      if (pl_active_drag_obj != null) {
         document.body.onmousemove = pl_onmousemove_body_prevEH;
         pl_onmousemove_body_prevEH = "";
         document.body.onmouseup = pl_onmouseup_body_prevEH;
         pl_onmouseup_body_prevEH = "";
         pl_active_drag_obj.onmousedown = "";
         pl_active_drag_obj.onlosecapture = "";
         pl_active_drag_obj.style.cursor = pl_active_drag_obj_prevcursor;
         pl_active_drag_obj_prevcursor = "default";
         pl_active_drag_obj = null;
      }

// Do not reset queued items here as there may be valid queued requests when the current popup layer is hidden.
//      pl_queued_show_layer_request = false;
//      pl_queued_hide_layer_request = false;
//      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 8");
//      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 9");

      // Clear out the global objects.
      pl_active_target_obj = null;
      pl_active_trigger_obj = null;
      pl_active_layer_obj = null;
      pl_active_clonedlayer_obj = null;
      pl_active_contentclone_obj = null;
      pl_active_style_flags = 0x0;

      // Finally, notify SurfAid if necessary
      if ((pl_active_surfaid_base_string != "") && (!ignore_surfaid)) {
         sa_onclick(pl_active_surfaid_base_string);
      }
      pl_active_surfaid_base_string = "";
   }
}


//
// ENTRY POINT from web pages: return the object in the popup layer with the specified ID.
// Special function needed to return the field of the passed ID when included in a bubble layer.
// Also works when not in a bubble, but the field can be called directly using document.getElementById in that case.
//
getPopupLayerElementById = function (id) {
   var obj;
   var mybubblecontent = document.getElementById(pl_BUBBLE_CONTENT_ID);
   obj = pl_findChildNode(mybubblecontent, id);
   if (obj == null) {
      obj = document.getElementById(id);
   }
   return obj;
}


// -----------------------------------------------------------------------------
//
// Local functions not called externally (pl_ prefix)
//
// -----------------------------------------------------------------------------

//
// Position the layer.
//
pl_positionPopupLayer = function () {

   var left_set = false;
   var top_set = false;
   var index = -1;
   var pos_left = false;
   var pos_right = false;
   var pos_below= false;
   var pos_above = false;
   var bubble_fin_space = 0;
   var left_layer_left;
   var left_offscreen;
   var right_layer_left;
   var right_offscreen;
   var below_layer_top;
   var below_offscreen;
   var above_layer_top;
   var above_offscreen;
   var offscreen = 0;
   var aOffScreen;
   var offset_for_bubble = 0;
   // Amount to adjust the fin from the corner of the target
   var finpos_indent = 0;

   // Return value of whether or not to show the fin
   var show_fin = false;

   // Global positioning attributes - assign to local variables here for convenience.
   var layer = pl_active_layer_obj;
   var target = pl_active_target_obj;
   var suggested_pos = (pl_active_style_flags & (PL_SUGGESTED_POS_BELOW | PL_SUGGESTED_POS_ABOVE | PL_SUGGESTED_POS_LEFT | PL_SUGGESTED_POS_RIGHT));
   var force_pos = ((pl_active_style_flags & PL_FORCE_POSITION) != 0x0);
   var force_visible_target = ((pl_active_style_flags & PL_FORCE_VISIBLE_TARGET) != 0x0);
   var display_in_bubble = ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) != 0x0);

   // Default position is below the target.
   if ((suggested_pos != PL_SUGGESTED_POS_LEFT) && (suggested_pos != PL_SUGGESTED_POS_RIGHT) && (suggested_pos != PL_SUGGESTED_POS_ABOVE) && (suggested_pos != PL_SUGGESTED_POS_BELOW)) {
      suggested_pos = PL_SUGGESTED_POS_BELOW;
   }

   var target_parent_offsetTop = 0;
   var target_parent_offsetLeft = 0;
   var target_parent = target.offsetParent;
   // Work up the object offset hierarchy, adding up the offsets up to the body tag
   while (target_parent != null) {
// 1.11 Keep going past the BODY tag in case the body itself has a margin, even though w3 and onePlace pages do not.
//      if (target_parent.tagName == "BODY") {
//         break;
//      }
      target_parent_offsetTop += target_parent.offsetTop;
      target_parent_offsetLeft += target_parent.offsetLeft;
      target_parent_offsetTop -= target_parent.scrollTop;   // MH-75
      target_parent_offsetLeft -= target_parent.scrollLeft; // MH-75
      target_parent = target_parent.offsetParent;
   }
   var client_width = document.documentElement.clientWidth;
   var client_height = document.documentElement.clientHeight;
   var client_v_scrolled = document.documentElement.scrollTop;
   var client_h_scrolled = document.documentElement.scrollLeft;
   var layer_left = -1;
   var layer_top = -1;
   var amt_hidden_below = 0;
   var amt_hidden_above = 0;
   var layer_height;
   var layer_width;
   // If we are in a bubble, then we need to run the positioning calcs based on the original size of the
   // bubble layer without considering any increases in width or height we may make later to accomodate the fin
   // if the original layer is not larget enough for the fin to fit on the desired side.  This comes into play
   // on resizes of the window which may change which side the fin appears on.
   if ((display_in_bubble) && (pl_bubble_orig_width > 0)) {
      layer_width = pl_bubble_orig_width;
   } else {
      layer_width = layer.offsetWidth;
      pl_bubble_orig_width = layer_width;
   }
   if ((display_in_bubble) && (pl_bubble_orig_height > 0)) {
      layer_height = pl_bubble_orig_height;
   } else {
      layer_height = layer.offsetHeight;
      pl_bubble_orig_height = layer_height;
   }

   // First we calculate all 4 positions (for left and right, if doesn't fit going down try again going up
   // and choose the best). We store how much it is off the screen if any for that axis.

   // Leave extra space for the fin if we are displaying the layer in a bubble.
   if (display_in_bubble) {
      bubble_fin_space = pl_BUBBLE_FIN_REACH - pl_BUBBLE_FIN_INSET;
   }

   // Left
   left_layer_left = target.offsetLeft + target_parent_offsetLeft - layer_width - bubble_fin_space;
   left_offscreen = Math.max(0, client_h_scrolled - left_layer_left);
   // Not allowed to be all the way off the left if we can't move it back over the target
   if (force_visible_target && (left_offscreen > client_h_scrolled)) {
      left_offscreen = pl_BIG_OFFSCREEN;
   }

   // Right
   right_layer_left = target.offsetLeft + target_parent_offsetLeft + target.offsetWidth + bubble_fin_space;
   right_offscreen = Math.max(0, (right_layer_left + layer_width) - (client_width + client_h_scrolled));

   // Below
   below_layer_top = target.offsetTop + target_parent_offsetTop + target.offsetHeight + bubble_fin_space;
   below_offscreen = Math.max(0, (below_layer_top + layer_height) - (client_height + client_v_scrolled));

   // Above
   above_layer_top = target.offsetTop + target_parent_offsetTop - layer_height - bubble_fin_space;
   above_offscreen = Math.max(0, client_v_scrolled - above_layer_top);
   // Not allowed to be all the way off the top if we can't move it back over the target
   if (force_visible_target && (above_offscreen > client_v_scrolled)) {
      above_offscreen = pl_BIG_OFFSCREEN;
   }

   // If we are forcing the suggested position to be used, use it even if if doesn't fit well.
   if (force_pos) {

      switch(suggested_pos) {
      case PL_SUGGESTED_POS_LEFT:
         layer_left = left_layer_left;
         left_set = true;
         pos_left = true;
         break;
      case PL_SUGGESTED_POS_RIGHT:
         layer_left = right_layer_left;
         left_set = true;
         pos_right = true;
         break;
      case PL_SUGGESTED_POS_BELOW:
         layer_top = below_layer_top;
         top_set = true;
         pos_below = true;
         break;
      case PL_SUGGESTED_POS_ABOVE:
         layer_top = above_layer_top;
         top_set = true;
         pos_above = true;
         break;
      }
      // Since forced to be where it wants to be, always show the fin.
      if (display_in_bubble) {
         show_fin = true;
      }

   } else {
      // Else we are sugesting a desired placement, but choose the first that fits.  The order we check for a fit depends on
      // which was the suggested pos.
      switch(suggested_pos) {
      case PL_SUGGESTED_POS_LEFT:
         aOffScreen = [left_offscreen, right_offscreen, below_offscreen, above_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 0:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 1:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 2:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 3:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;

      case PL_SUGGESTED_POS_RIGHT:
         aOffScreen = [right_offscreen, left_offscreen, below_offscreen, above_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 1:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 0:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 2:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 3:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;

      case PL_SUGGESTED_POS_BELOW:
         aOffScreen = [below_offscreen, above_offscreen, left_offscreen, right_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 2:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 3:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 0:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 1:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;

      case PL_SUGGESTED_POS_ABOVE:
         aOffScreen = [above_offscreen, below_offscreen, left_offscreen, right_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 2:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 3:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 1:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 0:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;
      }
      // Now if we wanted to force that the target be visible, we are done now, even if the layer is off the screen a bit.
      // Otherwise we need to move the layer back onto the screen if it is off screen.
      if (force_visible_target) {
         if (display_in_bubble) {
            show_fin = true;
         }
      } else {
         // Are we off the screen? Then bring it back on.
         // If we wanted a bubble but have to adjust the layer because it was off the screen, and we therefore
         // are moving closer to the target than optimal, we don't get to display the bubble fin.
         if (offscreen > 0) {
            if (pos_left) {
               layer_left += offscreen;
            } else if (pos_right) {
               layer_left -= offscreen;
            } else if (pos_below) {
               layer_top -= offscreen;
            } else if (pos_above) {
               layer_top += offscreen;
            }
         } else {
            // If we were not off the screen, and we want a bubble, then we can display the fin.
            if (display_in_bubble) {
               show_fin = true;
            }
         }
      }
      // Now one last check to ensure that the layer is not off the screen in the opposite direction of what was
      // calculated when there is room to move it fully on. But such a move removes the space for the fin, if any.
      var offscreen_opposite = 0;
      if (pos_left) {
         offscreen_opposite = layer_left + layer_width - (client_width + client_h_scrolled);
         if ((layer_left > client_h_scrolled) && (offscreen_opposite > 0)) {
            layer_left -= Math.min(layer_left - client_h_scrolled, offscreen_opposite);
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      } else if (pos_right) {
         offscreen_opposite = client_h_scrolled - layer_left;
         if (offscreen_opposite > 0) {
            layer_left += offscreen_opposite;
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      } else if (pos_below) {
         offscreen_opposite = client_v_scrolled - layer_top;
         if (offscreen_opposite > 0) {
            layer_top += offscreen_opposite;
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      } else if (pos_above) {
         offscreen_opposite = layer_top + layer_height - (client_height + client_v_scrolled);
         if ((layer_top > client_v_scrolled) && (offscreen_opposite > 0)) {
            layer_top -= Math.min(layer_top - client_v_scrolled, offscreen_opposite);
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      }
   }

   // Now we need to ensure the bubble is large enough for the fin to fit on the decided side by forcing the
   // necessary dimension to be able to accomodate the fin.  We then need to manually adjust the layer height
   // and width variables because IE does not reflect the changes immediately if we query them after we
   // make these changes.
   if (display_in_bubble) {
      if (pos_left || pos_right) {
         pl_active_bubble_minheight_cell.innerHTML = pl_BUBBLE_CONTENT_MINHEIGHT_HTML;
         pl_active_bubble_minwidth_cell.innerHTML = "";
         layer_height = Math.max(layer_height, pl_BUBBLE_FIN_BASE + (2 * pl_BUBBLE_CORNER_HEIGHT));
      } else if (pos_below || pos_above) {
         pl_active_bubble_minwidth_cell.innerHTML = pl_BUBBLE_CONTENT_MINWIDTH_HTML;
         pl_active_bubble_minheight_cell.innerHTML = "";
         layer_width = Math.max(layer_width, pl_BUBBLE_FIN_BASE + (2 * pl_BUBBLE_CORNER_WIDTH));
      }
   }

   // So we now have the key positioning, left/right/below/above, and we need to slide/position it along the
   // opposite axis now.

   // If we positioned it to the left or the right, we need to position it vertically to line up with
   // the target as best we can.  It may extend down from the top of the target, or extend up from the bottom
   // of the target.
   if (pos_left || pos_right) {
      if (display_in_bubble) {
         finpos_indent = Math.min(pl_FINPOS_INDENT_DEFAULT, Math.floor((target.offsetHeight / 2) + .25));
         var layer_top2 = 0;
         var bubblefin_top = 0;
         offset_for_bubble = Math.min(layer_height * pl_IDEAL_EAST_WEST_FIN_POSITION, layer_height - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_HEIGHT)
         layer_top = Math.max(target.offsetTop + target_parent_offsetTop - offset_for_bubble + finpos_indent, client_v_scrolled);

         // If layer off the top, move it on
         if (layer_top < client_v_scrolled) {
            layer_top = client_v_scrolled;
         // Else if layer off the bottom, move it on but keep it on the top as well
         } else if ((layer_top + layer_height) > (client_height + client_v_scrolled)) {
            layer_top = Math.max(client_v_scrolled, client_height + client_v_scrolled - layer_height);
         }

         // Now handle the case where the layer is being pushed up by the edge of the window, and there is not enough height under the
         // target to fit the fin. In this case we want to move the layer above the target so the fin can point downward, as long
         // as the layer is not off the glass. If neither fits then we do not want to display the fin.
         if (target.offsetTop + target_parent_offsetTop + finpos_indent + pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_HEIGHT > layer_top + layer_height) {
            layer_top2 = Math.min(layer_top, target.offsetTop + target_parent_offsetTop + finpos_indent - (pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_HEIGHT));
            if (layer_top2 != layer_top) {
               if ((layer_top2 + layer_height) > (client_height + client_v_scrolled)) {
                  layer_top = Math.max(client_v_scrolled, client_height + client_v_scrolled - layer_height);
               } else if (layer_top2 >= client_v_scrolled) {
                  layer_top = layer_top2;
               } else {
                  show_fin = false;
               }
            }
         }

         // Now where do we position the bubble fin, if any?
         if (show_fin) {
            // If the bubble is not aligned with the target, no fin to show.
            if ((layer_top + pl_BUBBLE_CORNER_HEIGHT > target.offsetTop + target_parent_offsetTop + target.offsetHeight - finpos_indent) || (layer_top + layer_height - pl_BUBBLE_CORNER_HEIGHT < target.offsetTop + target_parent_offsetTop + finpos_indent)) {
               show_fin = false;
            } else {
               if (pos_left) {
                  if (layer_top + (layer_height / 2) < target.offsetTop + target_parent_offsetTop) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_ED;
                     bubblefin_top = target.offsetTop + target_parent_offsetTop - pl_BUBBLE_FIN_BASE + finpos_indent;
                     if (bubblefin_top >= layer_top + pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_EU;
                     bubblefin_top = Math.max(target.offsetTop + target_parent_offsetTop + finpos_indent, layer_top + pl_BUBBLE_CORNER_HEIGHT);
                     if (bubblefin_top <= layer_top + layer_height - pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  }
                  pl_active_mybubblefin.style.left = layer_left + layer_width - pl_BUBBLE_FIN_INSET + "px";
               } else {
                  if (layer_top + (layer_height / 2) < target.offsetTop + target_parent_offsetTop) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_WD;
                     bubblefin_top = target.offsetTop + target_parent_offsetTop - pl_BUBBLE_FIN_BASE + finpos_indent;
                     if (bubblefin_top >= layer_top + pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_WU;
                     bubblefin_top = Math.max(target.offsetTop + target_parent_offsetTop + finpos_indent, layer_top + pl_BUBBLE_CORNER_HEIGHT);
                     if (bubblefin_top <= layer_top + layer_height - pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  }
                  pl_active_mybubblefin.style.left = layer_left - (pl_BUBBLE_FIN_REACH - pl_BUBBLE_FIN_INSET) + "px";
               }
            }
         }
      // Else not displayed in bubble.
      } else {
         // Default top position is aligned with the top of the object but can't be off the top of the screen.
         layer_top = Math.max(target.offsetTop + target_parent_offsetTop, client_v_scrolled);
         amt_hidden_below = Math.max(0, (layer_top + layer_height) - (client_height + client_v_scrolled));
         if (amt_hidden_below > 0) {
            layer_top = Math.min(target.offsetTop + target_parent_offsetTop + target.offsetHeight - layer_height, client_height + client_v_scrolled - layer_height);
            amt_hidden_above = Math.max(0, (client_v_scrolled - layer_top));
            if (amt_hidden_above > 0) {
               // If the layer is bigger than the client area, top-justify it.
               if (client_height <= layer_height) {
                  layer_top = client_v_scrolled;
               } else {
                  // Else popup layer will fit fully in client, so move it up until it all shows.
                  // Move it the smallest amount from the two default positions.
                  if (amt_hidden_below > amt_hidden_above) {
                     // If more is hidden below, the top-justify the layer
                     layer_top = client_v_scrolled;
                  } else {
                     // Else more is hidden above, so slide the layer up so its bottom is on the screen.
                     layer_top = target.offsetTop + target_parent_offsetTop - amt_hidden_below;
                  }
               }
            }
         }
      }

   // If we positioned it below or above the target, we need to position it horizontally. We try to line the left side
   // of the layer with the left side of the target as best we can.
   } else if (pos_below || pos_above) {
      var layer_left2 = 0;
      var bubblefin_left = 0;
      if (display_in_bubble) {
         offset_for_bubble = Math.min(layer_width * pl_IDEAL_NORTH_SOUTH_POSITION, layer_width - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_WIDTH)
         finpos_indent = Math.min(pl_FINPOS_INDENT_DEFAULT, Math.floor((target.offsetWidth / 2) + .25));
      }
      // Default left position is left-aligned with the object
      layer_left = target.offsetLeft + target_parent_offsetLeft - offset_for_bubble + finpos_indent;
      // But if it extends off the screen to the right, right-justify it
      if (layer_left + layer_width > client_width + client_h_scrolled) {
         layer_left = client_width + client_h_scrolled - layer_width ;
      }
      // But if it now extends off the screen to the left, left-justify it so you can read the beginning of the text
      layer_left = Math.max(client_h_scrolled, layer_left);

      // Now if it is in a bubble, we slide it to further to the left if we can, then attach the fin unless not needed.
      // If we are displaying in a bubble, reposition the layer if possible so it is offset so the fin (if needed) will
      // not be on the end of the layer unless that moves it off the glass.
      if (display_in_bubble) {
         // Now handle the case where the layer is being pushed to the left by the edge of the window, and there is not enough width under the
         // target to fit the fin. In this case we want to move the layer to the left of the target so the fin can point to the right, as long
         // as the layer is not off the glass. If neither fits then we do not want to display the fin.
         if (target.offsetLeft + target_parent_offsetLeft + finpos_indent + pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_WIDTH > layer_left + layer_width) {
            layer_left2 = Math.min(layer_left, target.offsetLeft + target_parent_offsetLeft + finpos_indent - (pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_HEIGHT));
            if (layer_left2 != layer_left) {
               if ((layer_left2 + layer_width) > (client_width + client_h_scrolled)) {
                  layer_left = Math.max(client_h_scrolled, client_width + client_h_scrolled - layer_width);
               } else if (layer_left2 >= client_h_scrolled) {
                  layer_left = layer_left2;
               } else {
                  show_fin = false;
               }
            }
         }
         if (show_fin) {
            // Can only show fin if layer is under or over the target to a degree.
            if ((target.offsetLeft + target_parent_offsetLeft + target.offsetWidth - finpos_indent < layer_left + pl_BUBBLE_CORNER_WIDTH) || (target.offsetLeft + target_parent_offsetLeft + finpos_indent > layer_left + layer_width - pl_BUBBLE_CORNER_WIDTH)) {
               show_fin = false;
            // Now see where the layer ended out in relation to the target.
            // Is layer to the left of target by at least the width of the fin plus the table corner, but not all
            // the way past the target?  Then we will use the arrow pointing to the right.
            } else if ((target.offsetLeft + target_parent_offsetLeft + finpos_indent >= layer_left + pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_WIDTH) && (layer_left + layer_width - pl_BUBBLE_CORNER_WIDTH >= target.offsetLeft + target_parent_offsetLeft + finpos_indent)) {
               bubblefin_left = target.offsetLeft + target_parent_offsetLeft + finpos_indent - pl_BUBBLE_FIN_BASE;
               // Ensure fin fits at final layer position.
               if ((bubblefin_left >= layer_left + pl_BUBBLE_CORNER_WIDTH) && (bubblefin_left <= layer_left + layer_width - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_WIDTH)) {
                  pl_active_mybubblefin.style.left = bubblefin_left + "px"
                  if (pos_below) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_NR;
                     pl_active_mybubblefin.style.top = layer_top - pl_BUBBLE_FIN_REACH + pl_BUBBLE_FIN_INSET_Y + "px";
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_SR;
                     pl_active_mybubblefin.style.top = layer_top + layer_height - pl_BUBBLE_FIN_INSET_Y + "px";
                  }
               } else {
                  show_fin = false;
               }
            // Else we use the arrow pointing to the left, as long as it fits and will still point to the target
            } else {
               bubblefin_left = Math.max(target.offsetLeft + target_parent_offsetLeft + finpos_indent, layer_left + pl_BUBBLE_CORNER_WIDTH);
               // Ensure fin fits at final layer position.
               if ((bubblefin_left >= layer_left + pl_BUBBLE_CORNER_WIDTH) && (bubblefin_left <= layer_left + layer_width - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_WIDTH)) {
                  pl_active_mybubblefin.style.left = bubblefin_left + "px";
                  if (pos_below) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_NL;
                     pl_active_mybubblefin.style.top = layer_top - pl_BUBBLE_FIN_REACH + pl_BUBBLE_FIN_INSET_Y + "px";
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_SL;
                     pl_active_mybubblefin.style.top = layer_top + layer_height - pl_BUBBLE_FIN_INSET_Y + "px";
                  }
               } else {
                  show_fin = false;
               }
            }
         }
      }
   }

   // Set the position of the popup layer
   layer.style.top = layer_top + "px";
   layer.style.left = layer_left + "px";

   // return whether the fin should be displayed if appropriate.
   return show_fin;
}


//
// Called by the layer positioning routine, it is passed in an array of values indicatng how much the layer is off the screen
// in the default position in the four directions. It returns the index of the first that is not off the screen, and if all
// are off the screen it returns the index of which is off the screen the least.
//
pl_getFirstPosition = function (aOffScreen) {
   var index = -1;
   var minOffScreen = -1;

   if (aOffScreen[0] == 0) {
      index = 0;
   } else if (aOffScreen[1] == 0) {
      index = 1;
   } else if (aOffScreen[2] == 0) {
      index = 2;
   } else if (aOffScreen[3] == 0) {
      index = 3;
   } else {
      // Else all are off the screen.  Find which is off the screen the least.
      if (aOffScreen[0] <= aOffScreen[1]) {
         minOffScreen = aOffScreen[0];
         index = 0;
      } else {
         minOffScreen = aOffScreen[1];
         index = 1;
      }
      if (aOffScreen[2] < minOffScreen) {
         minOffScreen = aOffScreen[2];
         index = 2;
      }
      if (aOffScreen[3] < minOffScreen) {
         minOffScreen = aOffScreen[3];
         index = 3;
      }
   }
   return index;
}


//
// Recurse through children nodes, looking for node with the passed in id.
//
pl_findChildNode = function (root, idDragger) {
   var x;
   if (root != null) {
      if (root.id == idDragger) {
         return root;
      }
      if (root.childNodes.length > 0) {
         var childnode;
         for (x = 0; x < root.childNodes.length; x++) {
            childnode = pl_findChildNode(root.childNodes[x], idDragger);
            if (childnode != null) {
               return childnode;
            }
         }
      }
   }
   return null;
}


//
// Preloads the images used by the layer so they wil display quickly when first needed.
//
pl_preloadPopupLayerImages = function () {
   if (!pl_bubble_images_preloaded) {
      pl_img_clearpix = new Image(1, 1);
      pl_img_clearpix.src = pl_CLEARPIX_FULLPATH;
      pl_img_banner_bg = new Image(pl_BANNER_BG_WIDTH, pl_BANNER_BG_HEIGHT);
      pl_img_banner_bg.src = pl_POPUPBANNER_FULLPATH;
      pl_img_close_x = new Image(pl_BANNER_CLOSE_X_WIDTH, pl_BANNER_CLOSE_X_HEIGHT);
      pl_img_close_x.src = pl_POPUPBANNERCLOSE_FULLPATH;

      pl_img_bubble_corner_nw = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_nw.src = pl_BUBBLE_CORNER_FULLPATH_NW;
      pl_img_bubble_corner_ne = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_ne.src = pl_BUBBLE_CORNER_FULLPATH_NE;
      pl_img_bubble_corner_sw = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_sw.src = pl_BUBBLE_CORNER_FULLPATH_SW;
      pl_img_bubble_corner_se = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_se.src = pl_BUBBLE_CORNER_FULLPATH_SE;

      pl_img_bubble_side_w = new Image(pl_BUBBLE_SIDE_DEPTH, pl_BUBBLE_SIDE_LENGTH);
      pl_img_bubble_side_w.src = pl_BUBBLE_SIDE_FULLPATH_W;
      pl_img_bubble_side_e = new Image(pl_BUBBLE_SIDE_DEPTH, pl_BUBBLE_SIDE_LENGTH);
      pl_img_bubble_side_e.src = pl_BUBBLE_SIDE_FULLPATH_E;
      pl_img_bubble_side_n = new Image(pl_BUBBLE_SIDE_LENGTH, pl_BUBBLE_SIDE_DEPTH);
      pl_img_bubble_side_n.src = pl_BUBBLE_SIDE_FULLPATH_N;
      pl_img_bubble_side_s = new Image(pl_BUBBLE_SIDE_LENGTH, pl_BUBBLE_SIDE_DEPTH);
      pl_img_bubble_side_s.src = pl_BUBBLE_SIDE_FULLPATH_S;

      pl_img_bubble_fin_nl = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_nl.src = pl_BUBBLE_FIN_FULLPATH_NL;
      pl_img_bubble_fin_nr = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_nr.src = pl_BUBBLE_FIN_FULLPATH_NR;
      pl_img_bubble_fin_sl = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_sl.src = pl_BUBBLE_FIN_FULLPATH_SL;
      pl_img_bubble_fin_sr = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_sr.src = pl_BUBBLE_FIN_FULLPATH_SR;
      pl_img_bubble_fin_eu = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_eu.src = pl_BUBBLE_FIN_FULLPATH_EU;
      pl_img_bubble_fin_ed = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_ed.src = pl_BUBBLE_FIN_FULLPATH_ED;
      pl_img_bubble_fin_wu = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_wu.src = pl_BUBBLE_FIN_FULLPATH_WU;
      pl_img_bubble_fin_wd = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_wd.src = pl_BUBBLE_FIN_FULLPATH_WD;

      pl_bubble_images_preloaded = true;
   }
}


//
// If there is a queued request to show the layer, do it now.
//
pl_showDelayedPopupLayer = function (idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string) {
   if (pl_queued_show_layer_request) {
      showPopupLayer2(idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string);
   }
}


//
// If there is a queued request to hide the layer, do it now.
//
pl_hideDelayedPopupLayer = function () {
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      hidePopupLayer()
   }
}


//
// If a timer is set, clear it.
//
pl_cancelTimeout = function (timer_id, str_debug) {
   if (timer_id != -1) {
      clearTimeout(timer_id);
   }
   return -1;
}


//
// Event handler for cancelling a queued hide request, set for the layer object if displayed on mouse over.
//
pl_processLayerMouseOver = function (event) {
   // If there is a queued hide request, cancel it because we are back over again.
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 4");

   }
   return true;
}


//
// Event handler for cancelling a queued hide request, set for the trigger object if displayed on mouse over.
//
pl_processTriggerMouseOverAndFocus = function (event) {
   // Reset the mouse out and blur event handlers for the trigger now that we are over it again.
   if (event == null) {
      event = window.event
   }
   if (event.target) {
      event.target.onblur = pl_processTriggerMouseOutAndBlur;
      event.target.onmouseout = pl_processTriggerMouseOutAndBlur;
   } else if (event.srcElement) {
      event.srcElement.onblur = pl_processTriggerMouseOutAndBlur;
      event.srcElement.onmouseout = pl_processTriggerMouseOutAndBlur;
   }
   // If there is a queued hide request, cancel it because we are back over again.
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 4");
   }
   return true;
}


//
// Event handler for queueing to dismiss the layer, set for the layer object if displayed on mouse over.
//
pl_processLayerMouseOut = function (event) {
   // If there is a queued show event, cancel it because we have moved back out again.
   if (pl_queued_show_layer_request) {
      pl_queued_show_layer_request = false;
      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 6");
   }
   // Modal dialogs do not go away on mouse out. Otherwise queue a hide event as long as the active layer
   // supports mouse outs.
   if ((!(pl_active_style_flags & PL_DIALOG_BEHAVIOR)) && (pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
      pl_queued_hide_layer_request = true;
      pl_queued_hide_layer_timer_id = window.setTimeout("pl_hideDelayedPopupLayer()", pl_MOUSEOUT_HIDE_DELAY);
   }
   return true;
}


//
// Event handler for queueing to dismiss the layer, set for the trigger object if displayed on mouse over or focus.
//
pl_processTriggerMouseOutAndBlur = function (event) {
   // When we get a blur or mouseout, we need to clear the objects event handler that caused it. We can't do
   // this on the close of the layer because blur and mouseover are set when a show is requested, and may
   // need to be cleared before the show takes place.
   if (event == null) {
      event = window.event
   }
   if (event.target) {
      event.target.onblur = "";
      event.target.onmouseout = "";
   } else if (event.srcElement) {
      event.srcElement.onblur = "";
      event.srcElement.onmouseout = "";
   }

   // If there is a queued show event, cancel it because we have moved back out again.
   if (pl_queued_show_layer_request) {
      pl_queued_show_layer_request = false;
      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 7");
   }
   // Modal dialogs do not go away on mouse out. Otherwise queue a hide event as long as the active layer
   // supports mouse outs.
   if ((!(pl_active_style_flags & PL_DIALOG_BEHAVIOR)) && (pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
      pl_queued_hide_layer_request = true;
      pl_queued_hide_layer_timer_id = window.setTimeout("pl_hideDelayedPopupLayer()", pl_MOUSEOUT_HIDE_DELAY);
   }
   return true;
}


//
// Event handler for drag support, set for the drag object.
//
pl_processDragMouseDown = function (event) {
   if (event == null) {
      event = window.event
   }
   if (pl_isButton1(event)) {
      if ((event != null) && (pl_active_layer_obj != null) && (pl_active_drag_obj != null)) {
         // Store the initial pos of the mouse so we can know when it is moved enough to start the drag.
         pl_drag_mouse_down = true;
         pl_move_init_x = event.clientX;
         pl_move_init_y = event.clientY;
         // Return false if user clicks on the frag area so default drag action on an object in Firefox does not occur.
         return pl_setReturnValue(false, event);
      }
   }
   return pl_setReturnValue(true, event);
}


//
// Event handler for drag support, set for the document body.
//
pl_processDragMouseUp = function (event) {
   if (pl_drag_mouse_down) {
      pl_endLayerDrag();
   }
   return pl_setReturnValue(true, event);
}


//
// Event handler for drag support, set for the document body.
//
pl_processDragMouseMove = function (event) {
   if (event == null) {
      event = window.event
   }
   // If we are not dragging yet, and the mouse is down, have we moved enough to start the drag?
   if ((!pl_move_in_process) && (pl_drag_mouse_down) && (event != null) && (pl_active_layer_obj != null) && (pl_active_drag_obj != null)) {
      if ((Math.abs(pl_move_init_x - event.clientX) >= pl_DRAG_START_DIFFERENTIAL) || (Math.abs(pl_move_init_y - event.clientY) >= pl_DRAG_START_DIFFERENTIAL)) {
         // Record we are dragging now.
         pl_move_in_process = true;
         pl_active_has_dragged = true;
         // Store the initial offset of the mouse from the layer top-left, so we can move the layer with the mouse.
         pl_move_offset_x = pl_active_layer_obj.offsetLeft  - document.documentElement.scrollLeft - event.clientX;
         pl_move_offset_y = pl_active_layer_obj.offsetTop - document.documentElement.scrollTop - event.clientY;
         // Store the min position the layer can move up and left. so part of the drag area is always visible.
         var drag_x = pl_active_drag_obj.offsetLeft;
         var drag_y = pl_active_drag_obj.offsetTop;
         var drag_width = pl_active_drag_obj.clientWidth;
         var drag_height = pl_active_drag_obj.clientHeight;
         // Account for the drag area being the layer itself, or nested within the layer. We want the offset from the top left of the layer.
         if (pl_active_drag_obj.id == pl_active_layer_id) {
            drag_x = 0;
            drag_y = 0;
         } else {
            var parent = pl_active_drag_obj.offsetParent;
            while (parent != null) {
               if (parent.id == pl_active_layer_id) {
                  break;
               }
               drag_x += parent.offsetLeft;
               drag_y += parent.offsetTop;
               parent = parent.offsetParent;
            }
         }
         // Now we get the offset of the lower right corner of the drag area from the upper left of the layer, adjusted by how much we want to stay visible.
         var drag_x_right = Math.max(drag_x, drag_x + drag_width - pl_DRAG_VISIBILITY_BUFFER);
         var drag_y_bottom = Math.max(drag_y, drag_y + drag_height - pl_DRAG_VISIBILITY_BUFFER);
         // And now we calculate the min position of the layer so that the visibility buffer of the drag area is always visible upper left.
         pl_move_min_x = -drag_x_right;
         pl_move_min_y = -drag_y_bottom;
         // Capture the mouse to catch mouse ups when the mouse is outside the window.
         if (pl_active_drag_obj.setCapture) {
            pl_active_drag_obj.setCapture(true);
         }
      }
   }

   // If we are dragging, adjust the layer position
   if (pl_move_in_process && pl_isButton1(event) && (event != null) && (pl_active_layer_obj != null)) {
      pl_active_layer_obj.style.left = Math.max(pl_move_min_x, event.clientX + pl_move_offset_x + document.documentElement.scrollLeft) + "px";
      pl_active_layer_obj.style.top = Math.max(pl_move_min_y, event.clientY + pl_move_offset_y + document.documentElement.scrollTop) + "px";
      // Remove the bubble fin on a move if visible
      if (pl_active_mybubblefin != null) {
         pl_active_mybubblefin.style.visibility = "hidden";
         pl_active_mybubblefin.style.left = 0;
         pl_active_mybubblefin.style.top = 0;
         pl_active_mybubblefin = null;
      }
      return pl_setReturnValue(false, event);
   }
   return pl_setReturnValue(true, event);
}


//
// Event handler for drag support, set for the drag object, called when mouse capture is terminated.
//
pl_processDragLoseCapture = function (event) {
   pl_endLayerDrag();
   return pl_setReturnValue(true, event);
}


//
// Called when drag ends.
//
pl_endLayerDrag = function () {
   pl_move_in_process = false;
   pl_drag_mouse_down = false;
   pl_move_init_x = 0;
   pl_move_init_y = 0;
   pl_move_min_x = 0;
   pl_move_min_y = 0;
   pl_move_offset_x = 0;
   pl_move_offset_y = 0;
   if (document.releaseCapture) {
      document.releaseCapture();
   }
}


//
// Called to set return values during drag operations.
//
pl_setReturnValue = function (retval, event) {
   if (event == null) {
      event = window.event
   }
   if (event) {
      event.returnValue = retval;
//      if (event.cancelBubble) {
//         event.cancelBubble = retval;
//      }
//      if (!retval) {
//         if (event.stopPropagation) {
//            event.stopPropagation();
//         }
//         if (event.preventDefault) {
//            event.preventDefault();
//         }
//      }
   }
   return retval;
}


//
// Called to see if button 1 is down during drag operations.
//
pl_isButton1 = function (event) {
   if (event == null) {
      event = window.event
   }
   if (event) {
      if (event.button) {
         if (event.button != 1) {
            return false;
         }
      } else {
         if (event.which) {
            if (event.which != 1) {
               return false;
            }
         }
      }
   }
   return true;
}


//
// Event handler: Set a flag that the popup layer was just clicked and send the click on to the general click handler
//
pl_processLayerClick = function () {
   pl_clicked_layer = true;
   return true;
}


//
// Event handler: Set a flag that the trigger was just clicked and send the click on to the general click handler
//
pl_processTriggerClick = function () {
   // If acts like a dialog, clicking outside the layer does not get processed, like it was modal.
   if (pl_active_style_flags & PL_DIALOG_BEHAVIOR) {
      return false;
   }
   // Clicking on trigger when layer is showing gets rid if it, and does not pass click on to the body.
   if (pl_active_layer_obj != null) {
      // Unless the layer was shown using mouse over and out support, in which case click does not get rid of layer.
      if (!(pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
         hidePopupLayer();
      }
      return false;
   }
   return true;
}


//
// Event handler: Cancel popup layer on a mouse click anywhere except on the popup layer itself
//
pl_processBodyClick = function (event) {
   if (!pl_isButton1(event)) {
      return true;
   }
   if (pl_active_layer_obj != null) {
      if (event == null) {
         event = window.event
      }
      // Do not dismiss the popup layer if it was the popup layer that was clicked
      if (pl_clicked_layer) {
         pl_clicked_layer = false;
         return true;
      }
      // If acts like a dialog, clicking outside the layer does not get processed, like it was modal.
      if (pl_active_style_flags & PL_DIALOG_BEHAVIOR) {
         return false;
      }
      // If the mouse was released outside of the window, ignore this click event generated during a drag. Only IE will
      // handle this since Firefox does not capture mouse events outside the window.
      if ((event.clientX < 0) || (event.clientY < 0)) {
         return false;
      }
      if (document.documentElement) {
         if (document.documentElement.clientWidth && document.documentElement.clientHeight) {
            if ((event.clientX > document.documentElement.clientWidth) || (event.clientY > document.documentElement.clientHeight)) {
               return false;
            }
         }
      }
      // If the click came from the trigger, ignore it.
      if (event.target) {
         if (event.target.id == pl_active_trigger_id) {
            return false;
         }
      } else if (event.srcElement) {
         if (event.srcElement.id == pl_active_trigger_id) {
            return false;
         }
      }
      // Otherwise a click hides the layer unless the layer was shown using mouse over and out support.
      if (!(pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
         hidePopupLayer();
      }
   }
   pl_clicked_layer = false;
   return true;
}


//
// Event handler: Cancel popup layer on ESC key
//
pl_processBodyKey = function (event) {
   if (event == null) {
      event = window.event
   }
   if (event != null) {
      // Modal dialogs do not get dismissed with the ESC key.
      if ((event.keyCode == 27) && (!(pl_active_style_flags & PL_DIALOG_BEHAVIOR))) {
         hidePopupLayer();
      }
   }
   return true;
}


//
// Event handler: Reposition the popup layer on resize to stay aligned with target if it reflowed
//
pl_processResize = function () {
//   // Movable objects and modal objects do not get repositioned on window resize.
//   if ((pl_active_drag_obj != null) || (pl_active_style_flags & PL_DIALOG_BEHAVIOR)) {
   // Movable objects that have been moved by the user do not get repositioned on window resize.
   if ((pl_active_drag_obj != null) && pl_active_has_dragged) {
      return true;
   }
   // Reposition the layer relative to the target
   var show_fin;
   show_fin = pl_positionPopupLayer();
   if ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) && (pl_active_mybubblefin != null)) {
      if (show_fin) {
         pl_active_mybubblefin.style.visibility = "visible";
         pl_active_mybubblefin.style.zIndex = 3;
      } else {
         pl_active_mybubblefin.style.visibility = "hidden";
         pl_active_mybubblefin.style.zIndex = -1;
      }
   }
   return true;
}


//
// Event handler: Cancel popup layer if another page loads in the same window
//
pl_processWindowUnload = function () {
   hidePopupLayer();
   return true;
}

}

if(!dojo._hasResource['hub.managers.Urls']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.managers.Urls'] = true;
dojo.provide('hub.managers.Urls');


/**
 * Manages the cache of all RESTful server URLs required by the core UI.
 * Currently, all paths are hardcoded off of a base URL calcuated when the
 * initial page renders. Later, this manager could grow to receive the entire
 * REST API in some format from the server (e.g., from the APP service
 * document).
 */
dojo.declare('hub.managers.Urls', null, {
    constructor: function(base_url) {
        this.base_url = base_url;
        this.server_url = base_url.substring(0, base_url.lastIndexOf('/client'));
    },

    /////////////////////////////////////
    // URLs for template page data
    ////////////////////////////////////

    /**
     * URL to object containing all data required for rendering the home page.
     */
    getHomePage: function() {
        return this.base_url + '/ui/home';
    },

    /**
     * URL to object containing search type info required for rendering the search box.
     */
    getSearchTypes: function() {
        return this.base_url + '/ui/searchtypes';
    },

    /**
     * URL to object containing all entry details.
     */
    getDetailsPage: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: getDetailsPage requires object_id');
        }
        return this.base_url + '/ui/details/entryid/'+object_id;
    },

    /**
     * URL to object containing information about the logged in user.
     */
    getProfilePage: function() {
        return base_url + '/ui/profile';
    },

    /**
     * URL to object containing all details about feed parameters.
     */
    getParametersPage: function(object_id, plugin_id) {
        if(object_id == null) {
            throw new Error('Urls: getParametersPage requires object_id');
        }
        return this.base_url + '/plugin/params/entryid/'+object_id+'/pluginid/'+plugin_id;
    },

    /**
     * URL to object containing user permissions.
     */
    getUpdateAdminTables: function() {
        return this.base_url + '/ui/update/name/admintables';
    },

    /**
     * URL to object containing user permissions.
     */
    getUpdateAdminProxy: function() {
        return this.base_url + '/ui/update/name/proxy';
    },

    /**
     * URL to object containing user permissions.
     */
    getAdminPermissionsPage: function() {
        return this.base_url + '/ui/editor/name/adminpermissions';
    },

    /**
     * URL to object containing list of items that can be deleted.
     */
    getAdminDeletePage: function() {
        return this.base_url + '/ui/editor/name/adminobjects';
    },

    /**
     * URL to object containing banned users.
     */
    getAdminBanPage: function() {
        return this.base_url + '/ui/editor/name/adminusers';
    },

    /**
     * URL to object containing email configuration.
     */
    getAdminEmailPage: function() {
        return this.base_url + '/ui/editor/name/adminreports';
    },

    /**
     * URL to object containing email configuration.
     */
    getAdminCategoryPage: function() {
        return this.base_url + '/ui/editor/name/admincategory';
    },

    /**
     * URL to object containing tracking configuration.
     */
    getAdminEditTrackingPage: function() {
        return this.base_url + '/ui/editor/name/admintracking';
    },

    /**
     * URL to object containing tracked feeds.
     */
    getAdminTrackingData: function() {
        return this.base_url + '/ui/editor/name/admintrackeddata';
    },

    /**
     * URL to object containing tracked feeds.
     */
    getAdminProxyPage: function() {
        return this.base_url + '/ui/editor/name/adminproxy';
    },

    /**
     * URL to object containing admintdi.
     */
    getAdminTDI: function() {
        return this.base_url + '/ui/editor/name/admintdi';
    },

    /**
     * URL to object containing tracked feeds.
     */
    getSettingsPage: function() {
        // @todo
    },

    /////////////////////////////////////
    // URLs for widget data / actions
    ////////////////////////////////////

    /**
     * URL to object or XML feed listing catalog entries for the given search
     * query and term.
     */
    getCatalog: function(collection, term, searchType, sortBy, sortOrder) {
		var url = "";
		var map = {
		};
		var c = collection;
		if (c.indexOf("ui.") != 0) {
			c = "ui." + c; 
		}
		map["collection"] = c;
		if (searchType != null) {
			map["searchType"] = searchType;
		} else if (term != null) {
			map["searchType"] = ""; // Just ensure it is set
		}
		if (term != null) {
			map["searchTerms"] = term;
		}
		else if (searchType != null){
			map["searchTerms"] = ""; // Just ensure it is set
		}
		if (sortBy != null) {
			map["sortBy"] = sortBy;
		}
		if (sortOrder != null) {
			map["sortOrder"] = sortOrder;
		}
		url = this.base_url + "/.." + "/atom"; // TODO: make it more robust
        url = url + '?' + dojo.objectToQuery(map);
        return url;
    },

    /**
     * URL to array of all category names if object_id is null or all names
     * categories assigned to the object_id if not null.
     */
    getCategories: function(object_id) {
        if(object_id == null) {
            return this.base_url + '/ui/categories';
        } else {
            return this.base_url + '/ui/categories/entryid/'+object_id;
        }
    },

    /**
     * URL to array of all category names if object_id is null or all names
     * categories assigned to the object_id if not null.
     */
    getCategoriesList: function(object_id) {
        if(object_id == null) {
            return this.base_url + '/ui/categorieslist';
        } else {
            return this.base_url + '/ui/categorieslist/entryid/'+object_id;
        }
    },

    /**
     * URL to array of all association names if object_id is null or all names
     * associations assigned to the object_id if not null.
     */
    getAssociationsList: function(object_id) {
        if(object_id == null) {
            return this.base_url + '/ui/associationslist';
        } else {
            return this.base_url + '/ui/associationslist/entryid/'+object_id;
        }
    },

    /**
     * URL to array of ids of all association assigned to the object_id.
     */
    getAssociationsIdsList: function(object_id) {
        return this.base_url + '/ui/associationsidslist/entryid/'+object_id;
    },

    /**
     * URL to array of all comments for the object_id. Array items are objects
     * having 'commenttitle', 'commenttext', 'creatorid', and 'timestamp'.
     */
    getComments: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: getComments requires object_id');
        }
        return this.base_url + '/ui/comments/entryid/'+object_id;
    },

    /**
     * URL to object containing average rating 'value' and voter 'count' for the
     * object_id.
     */
    getRatings: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: getRatings requires object_id');
        }
        return this.base_url + '/ui/ratings/entryid/'+object_id;
    },

    /**
     * URL to object containing average rating 'value' and voter 'count' for the
     * object_id.
     */
    getTags: function(objectid) {
        if(objectid == null) {
            return this.base_url + '/ui/tags';
        } else {
            return this.base_url + '/ui/tags/entryid/'+objectid;
        }
    },

    /**
     * URL to array of objects with username as key and fullname as value.
     */
    getUserSearch: function(user) {
	    user = user.replace(/\%/g,'%25');
        return this.base_url + '/ui/searchusers/user/' + user;
    },

    /**
     * URL to array of objects with groupname as both key and value.
     */
    getGroupSearch: function(group) {
	    group = group.replace(/\%/g,'%25');
        return this.base_url + '/ui/searchgroups/group/' + group;
    },

    /////////////////////////////////////
    // URLs for submitting data back to server
    ////////////////////////////////////

    /**
     * URL to post a report about abuse on an object_id.
     */
    postAbuse: function(object_id) {
        // @todo
    },

    /**
     * URL to post a deletion request for an object_id.
     */
    postDelete: function(object_id, plugin_id) {
        if(object_id == null) {
            throw new Error('Urls: postDelete requires object_id');
        }
        return this.base_url+'/plugin/delete/entryid/'+object_id+'/pluginid/'+plugin_id;
    },

    /**
     * URL to post a cancel request for an object_id.
     */
    postCancel: function(object_id, plugin_id) {
        if(object_id == null) {
            throw new Error('Urls: postCancel requires object_id');
        }
        return this.base_url+'/plugin/cancel/entryid/'+object_id+'/pluginid/'+plugin_id;
    },

    /**
     * URL to post tags for an object_id with 'tags' as key and a comma
     * separated string of tag names as value.
     */
    postTags: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postTags requires object_id');
        }
        return this.base_url+'/ui/update/name/tags/entryid/'+object_id;
    },

    /**
     * URL to post a comment for an object_id.
     */
    postComment: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postComment requires object_id');
        }
        return this.base_url+'/ui/update/name/comment/entryid/'+object_id;
    },

    /**
     * URL to post a rating for an object_id.
     */
    postRating: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postRating requires object_id');
        }
        return this.base_url+'/ui/update/name/rating/entryid/'+object_id;
    },

    /**
     * URL to post a bunch of metadata for an object_id.
     */
    postMetadata: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postMetadata requires object_id');
        }
        // @todo: implement
    },

    /**
     * URL to post profile updates for an object_id.
     */
    postProfile: function() {
        return this.base_url + '/ui/update/name/profile';
    },

    /**
     * URL to post a reset of the entire Mashup Hub database.
     */
    postAdminInit: function() {
        return this.base_url + '/ui/update/name/admintables';
    },

    /**
     * URL to post multiple object deletions.
     */
    postAdminDelete: function() {
        return this.base_url + '/ui/update/name/adminobjects';
    },

    /**
     * URL to post changes to categories.
     */
    postAdminCategories: function() {
        return this.base_url + '/ui/update/name/admincategory';
    },

    /**
     * URL to post changes to user permissions.
     */
    postAdminPermissions: function() {
        return this.base_url + '/ui/update/name/adminpermissions';
    },

    /**
     * URL to post changes to the banned users.
     */
    postAdminBan: function() {
        return this.base_url + '/ui/update/name/adminusers';
    },

    /**
     * URL to post changes to the email notification config
     */
    postAdminEmail: function() {
        return this.base_url + '/ui/update/name/adminreports';
    },

    /**
     * URL to post changes to the email notification config
     */
    postReportEmail: function() {
        return this.base_url + '/ui/update/name/report';
    },

    /**
     * URL to post tracking status configuration.
     */
    postAdminTrackingStatus: function() {
        return this.base_url + '/ui/update/name/admintrackingglobal';
    },

    /**
     * URL to post tracking status configuration.
     */
    postAdminTrackingSettings: function() {
        return this.base_url + '/ui/update/name/trackingedittype';
    },
    
    /**
     * URL to clear tracking data.
     */
    postAdminTrackingClear: function() {
        return this.base_url + '/ui/update/name/trackingcleardata';
    },

    /**
     * URL to post tracking status configuration.
     */
    postAdminTrackingPermissions: function() {
        return this.base_url + '/ui/update/name/admintrackingobject';
    },

    /**
     * URL to post tdi configuration.
     */
    postAdminTDI: function() {
        return this.base_url + '/ui/update/name/admintdi';
    },
    
    /**
     * URL to post connection info for database migration.
     */
    postAdminMigrateDB: function() {
        return this.base_url + '/ui/update/name/adminmigratedb';
    },

    /**
     * URL to post multiple object deletions.
     */
    postAdminProxy: function() {
        return this.base_url + '/ui/update/name/adminproxy';
    },


    /**
     * URL to post a login request.
     */
    postLogin: function() {
        return this.base_url + '/ui/login';
    },

    /**
     * URL to post a logout request.
     */
    postLogout: function() {
        return this.base_url + '/ui/logout';
    },

    /**
     * URL to post an account registration request.

     */
    postRegister: function() {
        return this.base_url + '/ui/register';
    },

    /**
     * URL to post data for verification before continuing in the installer
     * wizard.
     *
     * @param action: Action name corresponding with the page to verify
     */
    postInstall: function(action) {
        return this.base_url + '/installer/' + action;
    },

    /////////////////////////////////////
    // URLs for static resources
    ////////////////////////////////////

    /**
     * URL to the base images directory.
     */
    getImage: function(filename) {
        return this.base_url + '/images/' + filename;
    },

    /**
     * URL to the icons directory under the images folder.
     *
     */
    getIcon: function(filename) {
        return this.base_url + '/images/icons/' + filename;
    },

    //////////////////////////////////////
    // FROM DETAILS PAGE
    //////////////////////////////////////

    postFavorites: function(objectid) {
        if(objectid == null) {
            throw new Error('Urls: postFavorites requires objectid');
        }
        return this.base_url + '/ui/update/name/stuff/entryid/'+objectid;
    },
    
    getPluginGenerate: function(objectid, pluginid) {
        if(objectid == null) {
        	return "#";
//            throw new Error('Urls: getPluginGenerate requires objectid');
        }       
        if(pluginid == null) {
        	return "#";
//            throw new Error('Urls: getPluginGenerate requires pluginid');
        }       
        return this.base_url + '/plugin/generate/entryid/'+objectid+'/pluginid/'+pluginid;
    },
    
    getPluginParams: function(objectid, pluginid) {
        if(objectid == null) {
        	return "#";
//            throw new Error('Urls: getPluginParams requires objectid');
        }       
        if(pluginid == null) {
        	return "#";
//            throw new Error('Urls: getPluginParams requires pluginid');
        }       
        return this.base_url + '/plugin/params/entryid/'+objectid+'/pluginid/'+pluginid;
    },

    getAjaxUrl: function(plugin_id,entry_id,method){
    	return this.base_url + '/plugin/ajax/entryid/'+entry_id+'/pluginid/'+plugin_id+ '/method/'+method+'/';
    },
    
    secureUrl: function(ourl) {
    	if (hub.config.isSecure) {
	        var purl = new dojo._Url(ourl);
	        if ((purl.scheme == 'http') && purl.authority) {
	        	// we need to securize it:
	        	var result = "https://" + purl.host + hub.config.securePort + 
	        	(purl.path ? purl.path : "")  + 
	        	(purl.query ? ("?" + purl.query) : "") +
	        	(purl.fragment ? ("#" + purl.fragment) : "");
	        	return result;   
	        }
	    }
    	return ourl;
    }
});

}

if(!dojo._hasResource["hub.widgets.Dialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.widgets.Dialog"] = true;
dojo.provide("hub.widgets.Dialog");





dojo.declare(
	"hub.widgets.Dialog",
	[dijit.Dialog, dijit._Container],
  {
    templateString: null,
    templateString:"<div class=\"dijitDialog\">\r\n  <div dojoAttachPoint=\"titleBar\" class=\"dijitDialogTitleBar\" tabindex=\"0\" waiRole=\"dialog\">\r\n    <span dojoAttachPoint=\"titleNode\" class=\"dijitDialogTitle\">${title}</span>\r\n    <span dojoAttachPoint=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" dojoAttachEvent=\"onclick: hide\">\r\n      <span dojoAttachPoint=\"closeText\" class=\"closeText\">x</span>\r\n    </span>\r\n  </div>\r\n  <div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\" style=\"overflow:auto;\"></div>\r\n  <div dojoAttachPoint=\"actionNode\" class=\"dijitDialogPaneContent flowDialogActionBar\" style=\"padding:2px;\">\r\n    <div dojoAttachPoint=\"buttonNode\"></div>\r\n    <span dojoAttachPoint=\"resizeHandleNode\" class=\"dojoxFloatingResizeHandle\"></span>\r\n  </div>\r\n  <span dojoAttachPoint=\"tabEnd\" dojoAttachEvent=\"onfocus:_cycleFocus\" tabindex=\"0\"></span>\r\n</div>\r\n",
    actionNode: null,
    buttonNode: null,
    resizeHandleNode: null,
    operatorId: null,

    showUnderlay: false,
    hideTitleBar: false,
    hideActionBar: false,
    buttons: null,

    postCreate: function() {
      if (this.hideTitleBar) {
        this.titleBar.style.display = "none";
      }

      if (this.hideActionBar) {
        this.actionNode.style.display = "none";
      }

      //this.inherited("postCreate",arguments);
	  this.bgIframe = new dijit.BackgroundIframe(this.domNode);

      if (this.buttons) {
        for (var i = 0; i < this.buttons.length; i++) {
          var button = document.createElement("button")
          button.appendChild(document.createTextNode(this.buttons[i].label));

          if (this.buttons[i].onclick) {
            button.onclick = this.buttons[i].onclick;
          }

          this.buttons[i].node = this.buttonNode.appendChild(button);
        }
      } 
      else {
        // since IE doesn't do min-height
        this.actionNode.style.height = "13px";  // resizehandle image height size
      }
    },
    
    _setup: function() {
		if (this.titleBar) {
			//this._moveable = new dojo.dnd.move.parentConstrainedMoveable(this.domNode, {handle: this.titleBar, area: "margin", within: true});
			new dojo.dnd.Moveable(this.domNode, { handle: this.titleBar });
            dojo.connect(this._moveable, "onFirstMove", function(){this.handle.focus()});
        }
        
        if (this.showUnderlay) {
           this._underlay = new dijit.DialogUnderlay();
        }
        
        //

		var tmp = new dojox.layout.ResizeHandle({ 
				targetId: this.id,
				resizeAxis: "xy"}, this.resizeHandleNode);
    },
    
    layout: function() {
			// summary: position the Dialog and the underlay
			if(this.domNode.style.display == "block"){
			    if (this.showUnderlay) {
				   this._underlay.layout();
				}
				this._position();
			}
	},
		
    show: function() {
      var doLayout = false;

      if (!this._alreadyInitialized) {
        this._setup();
        this._alreadyInitialized = true;
        doLayout = true;
      }

      this.domNode.style.display = "block";

      dojo.addClass(this.domNode, "hubDialog");

      if (this.showUnderlay) {
        this._underlay.show();
      } 
      else {
      }

      if (doLayout) {
        this.layout();
      }

      this.open = true;
      this._savedFocus = dijit.getFocus(this);
      dijit.focus(this.titleBar);

      // TODO: doron - hack to make tabs work in dialogs for us
      this.resizeChildren();
    },

    hide: function() {
      if (!this._alreadyInitialized) {
        return;
      }

      this.domNode.style.display = "none";

      if (this.showUnderlay) {
        this._underlay.hide();
      }

      dijit.focus(this._savedFocus);
      this.open = false;
    },
    
    resize: function(data) {
      var w = data.w;
      var h = data.h;

      dojo.style(this.domNode, "width", w+"px"); 
      dojo.style(this.domNode, "height", h+"px");

      // now resize the dialog contents
      var titlebarBox = dojo.marginBox(this.titleBar);

      // TODO: 4 is fixed padding (both sides, so 8), needs to be calculated
      // IE works differently, so no padding is needed (else we get a visual drawing bug)
      h -= titlebarBox.h + dojo.marginBox(this.actionNode).h + (dojo.isIE ? 1 : 9);
      
      dojo.style(this.containerNode, "height", h+"px");
   
      // TODO: doron - hack to make tabs work in dialogs for us
      this.resizeChildren();
    },

    resizeChildren: function(data) {
      var children = this.getChildren();
      for (var i = 0; i < children.length; i++) {
        if (children[i].resize) {
          children[i].resize();
        }
      }
/*
      dojo.query(".hubResizable", this.domNode).forEach(function(node){
         dijit.byNode(node).resize();
      });
*/      
    }
    
    
  }
);

}

if(!dojo._hasResource["hub.main"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.main"] = true;
dojo.provide("hub.main");






/**
 * Initializes the Mashup Hub UI and client-side configuration variables. 
 * It's a class so we can use the hub package elsewhere without all this
 *  initialization code running on page load.
 */
dojo.declare("hub.Main", dijit._Widget, {
    // holds the initial full name of the logged in user, or blank if anonymous
    username: '',
    userfullname: '',
    isAdmin: '',
    isBanned: '',
    isSecure: false,
    securePort: '',
    baseurl: '',
    pingSessionHandle: null,
    
    postMixInProperties: function() {
        // create base URL first
        var split = dojo.baseUrl.split('/');
        // based on dojo-1.0.1 folder location only
        split = split.slice(0, -4);
		this.baseurl = split.join('/');		
        var durl = new dojo._Url(window.location, this.baseurl);
        this.baseurl = durl.uri;
        this.isSecure = (durl.scheme == 'https');
        if (this.isSecure) {
        	console.debug("isSecure == " + this.isSecure);
        	if (durl.port) {
        		this.securePort = ":" + durl.port; 
        	} 
        }
        if(this.baseurl.charAt(this.baseurl.length-1)=="/"){
        	this.baseurl = this.baseurl.substring(0, this.baseurl.length-1);
        }
		 
        // get username  TODO !!!
		_this = this;
		
		dojo.xhrGet({
			url: this.baseurl + "/ui/currentuser",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				_this.username = response.username;
				_this.userfullname = response.userfullname;
				_this.isAdmin = ("true" == response.isAdmin);
				_this.isBanned= ("true" == response.isBanned);
				var configured = ("true" == response.configured);
				if (!configured) {
					// come away from this layout.html to application root
					console.info("Application is not configured!");
					var p = window.location.pathname;
					var cut = p.indexOf("client/layout/layout.");
					if (cut >= 0) {
						p = p.substring(0, cut);
						console.info("Go to application root: " + p);
						if (dojo.isIE) {
							window.setTimeout('window.location.href = "' + p + '";', 50);
						}
						else {
							window.location.href = p;
						}
						return;
					}
				}
			}
		});
		
        // global configuration
        hub.config = {base_url: this.baseurl,
                      username: this.username,
                      userfullname: this.userfullname,
                      isAdmin: this.isAdmin,
                      isBanned: this.isBanned,
                      isSecure: this.isSecure,
                      securePort: this.securePort,
                      max_label_length: 15};
		
		// load paltette url
		dojo.xhrGet({
			url: this.baseurl + "/ui/paletteurl",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				hub.config.palette_url = response.palette_url;
			}
		});
                      
        hub.pingSession = function() {
        /*
			dojo.xhrGet({
				url: hub.config.base_url + "/ui/pingSession?dt=" + new Date().getTime(),
				handleAs: 'json',
				sync: false,
				load: function(response, args) {
					// we just ignore it
					console.debug("session pinged");
				}
			});
		*/	
        };

        // start listening for events that affect configuration
        dojo.subscribe('hub.login', this, '_onLogin');
        dojo.subscribe('hub.logout', this, '_onLogout');

        // initialize popup library
        
        set_pl_IMAGE_PATH(hub.config.base_url+'/images/popup/');
        pl_preloadPopupLayerImages();

        // create important managers
        
        hub.urls = new hub.managers.Urls(hub.config.base_url);
    },

    startup: function() {
        // show the home page
        var labels = dojo.i18n.getLocalization('hub', 'labels');
        hub.showPage(labels.HOME_TAB, 'Home');
        
        this.pingSessionHandle = null;
        /*
        if (this.pingSessionHandle != null) {
            // startup() normally should never called twice,
            // but to have it 100% safe
        	window.clearInterval(pingSessionHandle);
        	this.pingSessionHandle = null;
        }
        this.pingSessionHandle = window.setInterval(hub.pingSession, 15 * 60 * 1000); // 15 minutes are adequate
        */
    },

    _onLogin: function(username) {
        hub.config.username = username;
    },

    _onLogout: function() {
        hub.config.username = '';
    }
});

/**
 * General exception class for hub. Contains a message.
 */
dojo.declare("hub.Exception", null, {
    message: '',
    constructor: function(msg) {
        this.message = msg ? msg : "";
    }
});


/**
 * Notifies the user of a message, either by writing it into the given element.
 * or by popping up a JS alert box.
 *
 * @param text: Text to display
 * @param element: Element in which to insert the text, or null to use a popup
 * @param cls: CSS classname to use
 */
hub.notify = function(text, element, class_name) {
    if(element) {
        if(!class_name) {
            class_name = 'TOM_editor_error';
        }
        element.className = class_name;
        element.innerHTML = text;
    } else {
        alert(text);
    }
}

/**
 * Ellipsizes a label to the given maximum number of characters, including the
 * ellipsis.
 *
 * @param label
 * @param max
 */
hub.shortenLabel = function(label) {
    // truncate name chars first, so comparison by label works
    if(label.length > hub.config.max_label_length) {
    	label = hub.srahclaicepslmth(label);
        return hub.htmlspecialchars(label.substr(0 , hub.config.max_label_length-3)) + '...';
    } else {
        return label;
    }
}

/**
 * Mimick's PHP's htmlspecialchars for &, <, and >. Use to encode text before
 * output from JS to avoid security issues.
 *
 * @param text String to encode
 * @return Encoded text
 */
hub.htmlspecialchars = function(text) {
    text = text.replace(/&/g,"&amp;");
    text = text.replace(/</g,"&lt;");
    text = text.replace(/>/g,"&gt;");
    text = text.replace(/\'/g,"&#39;");
    text = text.replace(/\"/g,'&quot;');
    return text;
}

hub.srahclaicepslmth = function(text) {
    text = text.replace(/&amp;/g,"&");
    text = text.replace(/&lt;/g,"<");
    text = text.replace(/&gt;/g,">");
    text = text.replace(/&#039;/g,"'");
    text = text.replace(/&quot;/g,'"');
    return text;
}

/**
 * Replace ' -> \' and " -> \". Use to encode text before
 * output as JS strings.
 *
 * @param text String to encode
 * @return Encoded text
 */
hub.escapeQuotes = function(text) {
    text = text.replace(/\'/g,"\\\'");
    text = text.replace(/\"/g,'\\\"');
    return text;
}
hub.escapeQuotes3 = function(text) {
    text = text.replace(/\'/g,"&#92;&#39;");
    text = text.replace(/\"/g,'&#92;&#34;');
    return text;
}
hub.escapeQuotes2 = function(text) {
    text = text.replace(/&#39;/g,"&#92;&#39;");
    text = text.replace(/&quot;/g,'&#92;&#34;');
    return text;
}

/**
 * Evaluates JavaScript and Dojo content in the given DOM node.
 * 
 * @param node The DOM node to evaluate
 * @param js_eval True to eval all script tags
 * @param dojo_prase True to parse with Dojo
 */
hub.evaluate = function(node, js_eval, dojo_parse) {
    var html = node.innerHTML;
    
    if(js_eval) {
        // execute any scripts
        var expr = new RegExp('(?:<!--(?:\n|\r|\s|.)*?-->)|(?:<SCRIPT.*?>)((\n|\r|\s|.)*?)(?:<\/SCRIPT>)', 'ig');
        for (var results = expr.exec(html); results != null; results = expr.exec(html)) {
            if (!results[1]) {
                continue;
            }
            // IE doesn't allow script elements to have childnodes,
            // so another way is to use the non-standard .text for
            // IE:  script.text = results[1]
            if (dojo.isIE > 0) {
                window.execScript(results[1]);
            } else {
                eval(results[1]);
            }
        }
    }
    
    if(dojo_parse) {
        // tell dojo to parse
        dojo.parser.parse(node); 
    }
}

/**
 * Renders a standard refreshing message either inline or as a block element.
 * 
 * @param node: Node to render into
 * @param block: True to render as a <p>, false as a <span>
 * @param className: CSS class name to attach to the <p> or <span>
 */
hub.busyMessage = function(node, block, class_name) {
    var display;
    // decide how to render
    if(block) {
        display = "block";
    } else {
        display = "inline";
    }
    if(!class_name) {
        // make sure we have an empty string
        class_name = '';
    }
    var msgs = dojo.i18n.getLocalization('hub', 'messages');
    var ml = "<span class='${0}' style='display: ${1}'><img src='${2}' /> ${3}</span>";
    var img = hub.urls.getImage('spinner.gif');
    var str = dojo.string.substitute(ml, [class_name, display, img, 
                                                     msgs.STATUS_LOADING]);
    if(node) {
        node.innerHTML = str;
    } else {
        return str;
    }
}

/**
 * Builds a link representation of a call to hub.showPage.
 *
 * First three params match those of hub.showPage.
 * 
 * @param shorten: Truncate the displayed text?
 * @param text: Text to display, defaults to title
 * @param popup: Text to popup, defaults to title
 */
hub.buildPageLink = function(title, template, args, shorten, text, popup) {
    if(!text) {
        // use title if no other text specified
        text = title;
    }
    if(shorten) {
        // truncate to configured length with ellipsis
        text = hub.shortenLabel(text);
    }
     if(popup == null) {
        // use the full title as the popup
        popup = title;
    } else if(shorten) {
        // use the full title plus the provided popup
        popup = title + " (" + popup + ")";
    }
    // always escape strings, even to prevent ' and " problems
//    title = hub.escapeQuotes2(title);
//    text = hub.escapeQuotes2(text);
//    popup = hub.escapeQuotes2(popup);
    result = "<a href='javascript:hub.showPage(\"";
    result += hub.escapeQuotes2(title);
    result += "\",\"";
    result += template;
    result += "\",";
    result += dojo.toJson(args);
    result += ")\' title='";
    result += hub.escapeQuotes2(popup);
    result += "'>";
    result += text;
    result += "</a>";
    return result;
//     var slots = "<a href='javascript:hub.showPage(\"${title}\",\"${template}\",${args})\' title='${popup}'>${text}</a>";
//    return dojo.string.substitute(slots, {title: title,
//                                          text: text,
//                                          popup: popup,
//                                          template: template,
//                                          args: dojo.toJson(args)});  
}

/**
 * Posts a request to show a client side templated page. The page will render
 * if an observer is available that can render it, else the request is safely 
 * ignored.
 * 
 * @param title: Title of the page
 * @param template: Name of the template to render, rooted at hub.pages
 * @param args: Object of args to pass to the listener
 */
hub.showPage = function(title, template, args) {
    dojo.publish('hub.show-template-page', [title, template, args]);
}

/**
 * Posts a request to show the result of visiting a URL. The URL will render
 * if an observer is available that can render it, else the request is safely 
 * ignored.
 * 
 * @param title: Title of the page
 * @param url: Location of the page to render
 * @param args: Object of options to pass to the listener
 */
hub.showUrl = function(title, url, args) {
    dojo.publish('hub.show-url-page', [title, url, args]);
}

/**
 * Posts a request to show the first screen of a plug-in editor UI. The plug-in
 * will render if an observer is available that can render it, else the request
 * is safely ignored.
 * 
 * @param title: Title of the page
 * @param plugin_id: Plug-in editor to show
 * @param object_id: Object to edit in the plug-in. If null, creates a new 
 * object.
 * @param args: Object of options to pass to the listener
 */
hub.showPlugin = function(title, plugin_id, object_id, args) {
    dojo.publish('hub.show-plugin-page', [title, plugin_id, object_id, args]);
}

/**
 * Posts a request to rename an existing page. The rename will happen if the 
 * given node exists in a page and an observer is available that can render it,
 * else the request is safely ignored.
 * 
 * @param node: Any node in the page
 * @param title: New title of the page
 * @param args: Object of options to pass to the listener
 */
hub.renamePage = function(node, title,  args) {
    dojo.publish('hub.rename-page', [node, title, args]);
}

/**
 * Posts a request to mark or unmark a page as busy and/or to lock it from all
 * interaction with the user. The page will indicate busy or lock if an 
 * observer is available that can handle the request, else the request is 
 * safely ignored.
 * 
 * @param node: Any node in the page
 * @param busy: True to mark busy, false to unmark
 * @param locked: True to lock, false to unlock
 */
hub.busyPage = function(node, busy, locked) {
    dojo.publish('hub.busy-page', [node, busy, locked]);
}

/**
 * Posts a request to close a page. The page will close if there an 
 * observer is available that can handle the request, else the request is 
 * safely ignored. The observer will indicate whether the close event was
 * successful or if something cancelled it.
 * 
 * @param node: Any node in the page
 * @return dojo.Deferred: Result is true if closed, false if aborted
 */
hub.closePage = function(node) {
    var d = new dojo.Deferred();
    if(node){
    	dojo.publish('hub.close-page', [node, d]);
    }
    // answer will come back later
    return d;
}

hub.closeAllPages = function() {
    var d = new dojo.Deferred();
    dojo.publish('hub.close-page', [null, d]);
    // answer will come back later
    return d;
}

hub.closeCurrentPage = function() {
    dojo.publish('hub.close-current-page', []);
}

hub.getCurrentTab = function(){
	var tab_container = dijit.byId('hub.tabs');
	return tab_container.selectedChildWidget;
}

/**
 * Posts a request to check if a page is active. The request will be answered
 * if an observer is available to handle it, else the request is 
 * safely ignored. The observer will indicate the result.
 * 
 * @param node: Any node in the page
 * @return dojo.Deferred: Result is true if active, false if not
 */
hub.isPageActive = function(node) {
    var d = new dojo.Deferred();
    dojo.publish('hub.is-page-active', [node, d]);
    // answer will come back later
    return d;
}

/**
 * Finds all child layout containers and resizes them (also resizes passed dijit).
 * @param d: dijit to find layout containers in and resize them
 */
hub.resizeLayoutContainers = function(d) {
	d.resize();
	var layoutContainers = dojo.query(".dijitLayoutContainer", d.containerNode || d.domNode).map(dijit.byNode);
	dojo.forEach(layoutContainers, function(child){hub.resizeLayoutContainers(child); });
}

hub.showHelp = function( title, url ) {
    
    
    if (url.indexOf("http") == 0) {
    } 
    else {
        url = hub.config.base_url + '/' + url;
    }
    
    //<%= me %>.workflow.test(); 
    //var node = dojo.doc.createElement('div');
    var node = dojo.body().appendChild(document.createElement("div"));
    var dialog = new hub.widgets.Dialog({title: title, showUnderlay: false, resizable: true, closeable: true}, node);
    
    //dialog.setContent("<img src='https://localhost:8443/mashuphub/client/plugins/com.ibm.mashuphub.plugin.feed/icons/btn32_atom1.gif' />");
    dialog.resize({w: 950, h: 450});
    dialog.show();
    
    // var test_url = 'https://localhost:8443/mashuphub/client/doc/userhelp/feedhelp/help_feeds_register.htm';
    
    dojo.style(dialog.containerNode, "overflow", "hidden");
    var iframe = document.createElement("iframe");
    iframe.style.width = "100%";
    iframe.style.height = "100%";
    iframe.style.borderWidth = "0";
    iframe.setAttribute("src", url);
    dialog.containerNode.appendChild(iframe);
}

}

if(!dojo._hasResource["hub.pages"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.pages"] = true;
dojo.provide("hub.pages");









/**
 * Base class for all page templates. Supports fetching of a template given
 * its template path on the server, rendering of the template in a
 * dojox.dtl.AttachPoint, and optional fetching of any server data from a
 * RESTful URL.
 *
 * Subclasses must override getContext() to return an object with properties
 * corresponding to fields required by the template. Subclasses can override
 * getUrl() to return a URL that will be queried for additional server data and
 * passed as the only parameter to getContext().
 *
 * @var rendered: Deferred response to whether a page is rendered or not
 * @var load_hitched: Hitched method to call on server data load
 * @var err_hitched: Hitched method to call on server error
 */
dojo.declare("hub.pages._Page", dojox.dtl._Widget, {
    // path to the template on the server; use dojo.modulePath
    templatePath: null,
    tab: null,
    
    constructor: function(args, node) {
        this.rendered = new dojo.Deferred();
        this.load_hitched = dojo.hitch(this, this._onPageDataLoad);
        this.err_hitched = dojo.hitch(this, this._onPageDataError);
    },

    /**
     * Shows a loading message. Publishes events saying the page is busy.
     * Fetches the template.
     */
    buildRendering: function() {
        this.inherited('buildRendering', arguments);
        // only render busy message to start
        hub.busyMessage(this.domNode);
        // build a reusable template, template will be interned string if
        // running a dojo custom build, otherwise fetch from server.
        if (this.templatePath == null && this.templateString) {
            this.template = new dojox.dtl.HtmlTemplate(this.templateString);
        } else {
            this.template = new dojox.dtl.HtmlTemplate(this.templatePath);
        }
    },

    /**
     * Decides whether to query the server or to go ahead with rendering
     * immediately.
     */
    postCreate: function() {
        // check if the page needs server data
        var url = this.getUrl();
        if(url != null) {
            this._getServerData(url);
        } else {
            this._renderPage({});
        }
    },

    /**
     * Renders the page template. Calls getContext() for the data to interpolate
     * into the template.
     *
     * @param server_data: All data returned from the server if getUrl()
     * returned a valid URL to query.
     */
    _renderPage: function(server_data) {
        //console.log('_renderPage');
        
        // check this.domNode state in case this widget is destroyed 
        // before this callback is invoked.
        var srcNode = this.domNode;
        
        if (srcNode) {
        	// get the context object from the subclass impl.
        	var obj = this.getContext(server_data);
        	// build the dojox.dtl.Context
        	var context = new dojox.dtl.Context(obj);
        	// do the rendering
        	this.render(this.template, context);
        	// call extension point method
        	this.onRendered();
        }
        else {
           console.log('_renderPage - dom node is null');
        }
        
    },

    /**
     * Gets an object with properties corresponding to fields in the template.
     * Must be provided by a subclass.
     *
     * @return: Object
     */
    getContext: function() {
        throw new Error("_Page: must override getContext");
    },

    /**
     * Gets a RESTful URL to query for additional data required to render the
     * page.
     *
     * @return: String URL or null to indicate no REST call is needed
     */
    getUrl: function() {
        return null;
    },

    /**
     * Gets an object describing the properties of this page. Used by a
     * container to decide how to render a page. Will be invoked on the *class*
     * not on an instance (i.e., a static) so avoid use of "this."
     *
     * @return: Object
     */
    getProperties: function() {
        return {};
    },

    /**
     * Performs the fetch for additional server data.
     *
     * @param url: URL to query
     */
    _getServerData: function(url) {
	var kw = {
	    url: url, // url for page data
            handleAs: 'json', // always json encode page data
            timeout: 120000, // 2 minutes in milliseconds
            error: this.err_hitched,
            load: this.load_hitched
	};
        // get page data from server
        dojo.xhrGet(kw);
    },

    /**
     * Handles a successful load of server data.
     *
     * @param response: Response object from the server, de-JSONified
     * @param args: Original query arguments
     */
    _onPageDataLoad: function(response, args) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
	try {
            if(!response) {
                hub.notify(msgs.ERROR_REQUEST_SERVER, this.domNode);
            }
		if(hub.submit.processResponseCode(response)){
		    this._renderPage(response);
	    }
	} catch(e) {
            throw e;
	}
        return response;
    },

    /**
     * Handles an error during a server call.
     *
     * @param response: The error object
     * @param args: Original query arguments
     */
    _onPageDataError: function(response, args) {
        // get error messages
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        if(response.dojoType == "cancel") {
            hub.notify(msgs.ERROR_REQUEST_CANCELLED, this.domNode);
        } else if (response.dojoType == "timeout"){
            hub.notify(msgs.ERROR_REQUEST_TIMEOUT, this.domNode);
        } else {
            hub.notify(response, this.domNode);
        }
        return response;
    },

    /**
     * Uses the rendered deferred to indicate page rendering is complete.
     * Can be used as an extension point for dojo.connect too, but only if it is
     * known ahead of time that rendering is not already complete at time of
     * connection.
     */
    onRendered: function() {
        // extension point
        this.rendered.callback(true);
    },

    /**
     * Gets a deferred which will indicate when page rendering is complete.
     *
     * @return: dojo.Deferred which will notify when the answer becomes true
     */
    isRendered: function() {
        return this.rendered;
    }
});


}

if(!dojo._hasResource["hub.table.model"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.table.model"] = true;
dojo.provide("hub.table.model");


/**
 * Abstaract interface for table model.
 */
dojo.declare("hub.table.model.Model", null, {

	/**
	 * Planned page size, how many records it was planned for this table,
	 * this attribute value may be used as a hint, how many rows should be
	 * preallocated.
	 */
	getPlannedPageSize: function() {
		throw new Error("hub.table.model.Model.getPlannedPageSize() is abstract");
	},
	
	/**
	 * Size of current page, pricipally equivalent of getCurrentPage().size. But 
	 * may work quicker.
	 */
	getCurrentPageSize: function() {
		throw new Error("hub.table.model.Model.getCurrentPageSize() is abstract");
	},
	
	/**
	 * Returns row of the table, by row number.
	 */
	getRow: function(rowNum) {
		throw new Error("hub.table.model.Model.getRow(n) is abstract");
	},
	
	getSupportedSortings: function() {
		throw new Error("hub.table.model.Model.getSupportedSortings() is abstract");
	},
	
	getGuaranteedRowCount: function() {
		throw new Error("hub.table.model.Model.getGuaranteedRowCount() is abstract");
	},
	
	hasMoreRecords: function() {
		throw new Error("hub.table.model.Model.hasMoreRecords() is abstract");
	},
	
	getCurrentPage: function() {
		throw new Error("hub.table.model.Model.getCurrentPage() is abstract");
	},

	getAvailablePages: function() {
		throw new Error("hub.table.model.Model.getAvailablePages() is abstract");
	},
	
	setPageToLoad: function(page) {
		throw new Error("hub.table.model.Model.setPageToLoad(page) is abstract");
	},
	
	getPageToLoad: function() {
		throw new Error("hub.table.model.Model.getPageToLoad() is abstract");
	},
	
	setSortingToApply: function(sorting) {
		throw new Error("hub.table.model.Model.setSortingToApply(s) is abstract");
	},
	
	getSortingToApply: function() {
		throw new Error("hub.table.model.Model.getSortingToApply() is abstract");
	},
	
	getCurrentSorting: function() {
		throw new Error("hub.table.model.Model.getCurrentSorting() is abstract");
	},
	
	fetch: function() {
		throw new Error("hub.table.model.Model.fetch() is abstract");
	},
	
	isFetched: function() {
		throw new Error("hub.table.model.Model.isFetched() is abstract");
	},

	onBeforeFetched: function() {
		throw new Error("hub.table.model.Model.onBeforeFetched() is abstract");	
	},
	
	onFetched: function() {
		throw new Error("hub.table.model.Model.onFetched() is abstract");	
	},

	onFetchError: function(error) {
		throw new Error("hub.table.model.Model.onFetchError() is abstract");
	}
});

dojo.declare("hub.table.model.Page", null, {
	fromRow: null, // is not necessar
	toRow: null, // is not necessar
	size: 0, // will be toRow - fromRow in most case
	forwardAbsolute: null, // is not necessar
	backwardAbsolute: null, // is not necessar
	relative: null // is not necessar
});

dojo.declare("hub.table.model.Sorting", null, {
	by: null, // name of criterion to order by
	asc: true // sort ascenting
});

dojo.declare("hub.table.model.AbstractModel", [dijit._Widget, hub.table.model.Model], {
	pageToLoad: null,
	sortingToApply: null,
	currentSorting: null,
	constructor: function(args, node) {
		this.changesElements = {};
		this.isSelectedElements = {};
	},

	getPlannedPageSize: function() {
		return 0;
	},
	
	getCurrentPageSize: function() {
		return 0;
	},
	
	getRow: function(rowNum) {
		return null;
	},
	
	getSupportedSortings: function() {
		return new Array();
	},
	
	getGuaranteedRowCount: function() {
		return 0;
	},
	
	hasMoreRecords: function() {
		return false;
	},
	
	getCurrentPage: function() {
		return null;
	},

	getAvailablePages: function() {
		return null;
	},
	
	setPageToLoad: function(page) {
		this.pageToLoad = page;
	},
	
	getPageToLoad: function() {
		return this.pageToLoad;
	},
	
	setSortingToApply: function(sorting) {
		this.sortingToApply = sorting;
	},
	
	getSortingToApply: function() {
		return this.sortingToApply;
	},
	
	getCurrentSorting: function() {
		return this.currentSorting;
	},
	
	onBeforeFetched: function() {
	
	},
	
	onFetched: function() {
	
	},

	onFetchError: function(error) {
		
	}
});

dojo.declare("hub.table.model.ItemizedModel", hub.table.model.AbstractModel, {
	rows: [],
	newRows: [],
	plannedSize: 0,
	lock: 0,
	fetched: false,
	constructor: function(args, node) {
		this.rows = [];
		this.plannedSize = 0;
	},
	
	onItem: function(l, item) {
		if (l == this.lock) {
			this.newRows.push(item);
			return true;
		} 
		else {
			return false;
		}
	},
	
	getRow: function(rowNum) {
		return this.rows[rowNum];
	},
	
	onPageBegin: function() {
		this.newRows = [];
		this.lock++;
		return this.lock;
	},
	
	onPageEnd: function(l, error) {
		if (l == this.lock) {
			if (error) {
				this.newRows = new Array();
				return true;
			}
			else {
				this.fetched = true;
				this.rows = this.newRows;
				this.plannedSize = Math.max(this.rows.length, this.plannedSize);
				return true;
			}
		}
		else {
			return false;
		}
	},
	
	isFetched: function() {
		return this.fetched;
	}
});

dojo.declare("hub.table.model.DataSelection", null, {
	getRowId: function(row) {
		if ("id" in row) {
			return row.id;
		}
		else if (row[0]) {
			return row[0];
		}
		else {
			throw new Error("row id is not defined");
		}
	},
	
	isDataSelected: function(row, area) {
		if ("selected" in row) {
			return (false != row.selected);
		}
		return false;
	}
});

dojo.declare("hub.table.model.NoSelection", hub.table.model.DataSelection, {
	getRowId: function(row) {
		return "";
	},
	
	isDataSelected: function(row, area) {
		return false;
	}
});

dojo.declare("hub.table.model.DebugModel", hub.table.model.ItemizedModel, {
	from: null,
	pageSize: 20,
	totalRecords: 111,
	fetched: false,

	constructor: function(args, node) {
	},

	getPlannedPageSize: function() {
		return this.pageSize;
	},
	
	getCurrentPageSize: function() {
		if (this.from == null) {
			return null;
		}
		return this.getCurrentPage().size;
	},
	
	getSupportedSortings: function() {
		var sortings = new Array();
		var sorting = new hub.table.model.Sorting();
		sorting.by = 'value';
		sorting.asc = true;
		sortings.push(sorting);
		sorting = new hub.table.model.Sorting();
		sorting.by = 'value';
		sorting.asc = false;
		sortings.push(sorting);
		return sortings;
	},
	
	getGuaranteedRowCount: function() {
		return this.totalRecords;
	},
	
	hasMoreRecords: function() {
		return false;
	},
	
	getCurrentPage: function() {
		if (this.from == null) {
			return null;
		}
		var page = new hub.table.model.Page();
		page.fromRow = this.from;
		page.toRow = Math.min(this.from + this.pageSize, this.totalRecords);
		page.size = page.toRow - page.fromRow;
		page.forwardAbsolute = this.from / this.pageSize; 
		// backwardAbsolute: null, // is not necessar
		page.relative = 0;
		return page;
	},

	getAvailablePages: function() {
		var pages = new Array();
		if (this.from != null) {
			var i = 0;
			for(i = 0; i < this.totalRecords; i = i + this.pageSize) {
				var page = new hub.table.model.Page();
				page.fromRow = i;
				page.toRow = Math.min(i + this.pageSize, this.totalRecords);
				page.size = page.toRow - page.fromRow;
				page.forwardAbsolute = i / this.pageSize; 
				// backwardAbsolute: null, // is not necessar
				page.relative = (i - this.from) / this.pageSize;
				pages.push(page);
			}
		}
		return pages;
	},
	
	fetch: function() {
		var from = 0;
		if (this.pageToLoad != null) {
			from = this.pageToLoad.fromRow;
		}
		var i = from;
		this.from = from;
		var sorting = this.sortingToApply;
		var asc = true;
		if (sorting != null) {
			asc = sorting.asc;
		}
		else {
			sorting = new hub.table.model.Sorting();
			sorting.asc = asc;
			sorting.by = 'value'; 
		}
		onBeforeFetched();
		var lock = this.onPageBegin();
		for(i = from; i < Math.max(from + this.pageSize, this.totalRecords); i++) {
			var row = {'value': i, 'id': i * i + 3};
			if (!asc) {
				row.value = this.totalRecords - i - 1;
			}
			row.id = row.value * row.value + 3
			this.onItem(lock, row);
		}
		if (this.onPageEnd(lock)) {
			this.currentSorting = sorting;
			this.pageToLoad = this.getCurrentPage();
			this.onFetched();
		}
	},
	
	onFetched: function() {
	},

	onBeforeFetched: function() {
	},
	
	onFetchError: function(error) {
		// called never
	}
});

}

if(!dojo._hasResource['hub.table.table']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.table.table'] = true;
dojo.provide('hub.table.table');



/**
 * Generic table component with pagination, sorting and selection capabilities.
 */
dojo.declare('hub.table.table.Table', [dijit._Widget, dijit._Templated], {
	/**
	 * Table mode.
	 */ 
	model: null,
	
	/**
	 * Selection hook.
	 */
	selection: null,
	
	/**
	 * List of columns.
	 */
	columns: [],
	
	/**
	 * Rows
	 */
	rows: [],
	
	/**
	 * Should we reset to the first page when sorting is changed.
	 */
	resetOnSort: true,
	
	/**
	 * Selection area which will be used in setValue/getValue or null.
	 */
	valueSelectionArea: null,
	
	/**
	 * Selection area which will be "default". Clickable rows.
	 */
	defaultSelectionArea: "default",
	
	/**
	 * Default selection area mode.
	 */ 
	defaultSelectionAreaMode: 0,
	uniselectionReselectValue: true,
	selection: new hub.table.model.DataSelection(),
	changedElements: {},
	isSelectedElements: {},
	noDefaultSelectionTags: ['A'],
	templateString:"<div class=\"TOM_widget_Table\">\r\n    <table>\r\n      <thead>\r\n        <tr dojoAttachPoint=\"table_header\"></tr>\r\n      </thead>\r\n      <tbody dojoAttachPoint=\"table_body\"></tbody>\r\n      <tfoot>\r\n        <tr dojoAttachPoint=\"table_footer\" class=\"centered note\"></tr>\r\n      </tfoot>\r\n    </table>\r\n</div>\r\n",
	headersRendered: false,

    constructor: function(args, node) {
        // column metadata
        this.columns = new Array();
        // rows added to table so far for this query
        this.rows = new Array();
        this._model_connects = new Array();
        this._header_connects = new Array();
        this._cell_connects = new Array();
		this.changedElements = {};
		this.isSelectedElements = {};
		this.headersRendered = false;
    },

    postMixInProperties: function() {
        // urls for icons and such
        this.checkboxOnImageURL = hub.urls.getImage('checkboxOn.png');
        this.checkboxOffImageURL = hub.urls.getImage('checkboxOff.png');
        this.radioOnImageURL = hub.urls.getImage('radioOn.png');
        this.radioOffImageURL = hub.urls.getImage('radioOff.png');
    },
    
    destroy: function() {
    	this._model_connects = this._disconnect(this._model_connects);
    	this._header_connects = this._disconnect(this._header_connects);
    	this._cell_connects = this._disconnect(this._cell_connects);
    },
    
    /**
     * Sets model for the table.
     */
    setModel: function(newModel) {
    	this._model_connects = this._disconnect(this._model_connects);
    	this.model = null;
    	this.model = newModel;
    	this.initialize();
    },
    
    /**
     * Connects table to model hooks.
     */
    initialize: function() {
		if (this.model != null) {
			this._model_connects.push(dojo.connect(this.model, "onFetched", this, "fetched"));
			this._model_connects.push(dojo.connect(this.model, "onBeforeFetched", this, "beforeFetched"));
			this._model_connects.push(dojo.connect(this.model, "onFetchError", this, "fetchError")); 
		}
		if (this.model != null) {
			this.fetched();
		}
    },

	/**
	 * Called when model is fetched, or when it may be fetched (some initial state).
	 */
    fetched: function() {
        // clear out all table headers and body
        this.clearRows();

		if (!this.headersRendered) {
	        for(i in this.columns) {
	            var th = document.createElement('TH');
	            // title and base class
	            th.innerHTML = this.columns[i]['name'];
	            if (this.columns[i]['class']) {
		            dojo.addClass(th, this.columns[i]['class']);
		        }
		        if (this.columns[i]['sorting']) {
	                // make focusable
	                th.tabIndex = 0;
	                // mouse and key handler for sorting
	                this._header_connects.push(dojo.connect(th, 'onclick', this, '_sort'));
	                this._header_connects.push(dojo.connect(th, 'onkeypress', this, '_sort'));
	                // @todo: initialize first column sorting for now, need to be
	                //   smarter about what order the results are in
		        }
	            this.table_header.appendChild(th);
	        }
	        // footer status area, built here because we need to know how many 
	        // columns
	        if (this.table_footer) {
		        this.status_row = document.createElement('TD');
		        this.status_row.colSpan = this.columns.length;
		        this.status_row.className = 'note';
		        this.table_footer.appendChild(this.status_row);
		    }	        
	        this.headersRendered = true;
	    }

	    
        if ((this.model != null) && this.model.isFetched()) {
	        var currentSorting = this.model.getCurrentSorting();
	        if (currentSorting != null) {
	        	var j = 0;
		        for(var i in this.table_header.childNodes) {
		            var th = this.table_header.childNodes[i];
		            if ( (th.tagName) && ('TH' == th.tagName) ) {
		            	var sorting = this.columns[j]['sorting'];
						if (currentSorting.by == sorting) {
				           	if (currentSorting.asc) {
				               	dojo.addClass(th, 'selectedUp');
				               	dojo.removeClass(th, 'selectedDown');
				            }
				            else {
				               	dojo.addClass(th, 'selectedDown');
				               	dojo.removeClass(th, 'selectedUp');
			                }
			            }
			            else {
			                dojo.removeClass(th, 'selectedDown');
			                dojo.removeClass(th, 'selectedUp');
			            }
			            j = j + 1;
			        }
		        }
		    }
        	var i = 0;
        	var l = this.model.getCurrentPageSize();
        	for(i = 0; i < l; i++) {
        		var row = this.model.getRow(i);
        		this.addRow(row, i);
        	}
        	if (this.model.getCurrentPageSize() > 0) {
        		this._showNoStatus();
        	}
        	else {
        		this._showNoEntriesStatus();
        	}
			var area;
			for(area in this.changedElements) {
				var i;
				var count = this.model.getCurrentPageSize();
				for(i = 0; i < count; i++) {
					this._mergeSelected(i, area);
				}
			}
		}
    },
    
    /**
     * Called before request to model is done.
     */
    beforeFetched: function() {
    	this._showBusyStatus();
    },
    
    /**
     * Handles error situation in model.
     */
    fetchError: function(error) {
    	console.debug("fetchError: " + error);
    	this._showErrorStatus(error);
    },
	
	/**
	 * Adds convinient column.
	 */
	addColumn: function(id, name, sorting, buildFunction, clazz) {
        this.clearHeaders();
        this.clearRows();
        if (!clazz) {
        	clazz = id;
        }
		this.columns.push({
			'id': id,
			'name': name,
			'sorting': sorting,
			'buildFunction': buildFunction,
			'selectionArea': null,
			'selectionMode': null,
			'class': clazz
		});
		this.fetched();
	},
	
	/**
	 * Adds selection column.
	 */
	addSelectionColumn: function(id, name, selectionArea, mode, clazz) {
        this.clearHeaders();
        this.clearRows();
        if (!clazz) {
        	clazz = id;
        }
		this.columns.push({
			'id': id,
			'name': name,
			'sorting': null,
			'buildFunction': null,
			'selectionArea': selectionArea,
			'selectionMode': mode,
			'class': clazz
		});
		this.fetched();
	},
	
	/**
	 * Sets default selection area.
	 */
	setDefaultSelectionArea: function(area) {
		this.defaultSelectionArea = area;
	},
	
    /**
     * Clears all table headers.
     */
    clearHeaders: function() {
    	this._header_connects = this._disconnect(this._header_connects);
        // NOTE: we really want to just do innerHTML = '', but we can't
        // because of IE6/IE7; have to use DOM else get runtime errors
        while (this.table_header.firstChild)  {
            this.table_header.removeChild(this.table_header.firstChild);
        }
        if (this.table_footer) {
	        while (this.table_footer.firstChild)  {
	            this.table_footer.removeChild(this.table_footer.firstChild);
	        }
	        this.status_row = null;
	    }
        this.headersRendered = false;
    },

    /**
     * Clears the body of the table.
     */
    clearRows: function() {
    	this._cell_connects = this._disconnect(this._cell_connects);
        // NOTE: we really want to just do innerHTML = '', but we can't
        // because of IE6/IE7; have to use DOM else get runtime errors
        while (this.table_body.firstChild)  {
            this.table_body.removeChild(this.table_body.firstChild);
        };
        this.rows = [];
    },

    /**
     * Adds a single row to the table.
     * @param dict single row data keyed by column name:
     *  - json string in format {'col1': data1, 'col2': data2, ...}
     *  - json object
     */
    addRow: function(dict, rowNum) {
    	var row = dict;

    	// convert to json object data
		if(!(dict instanceof Object)) {
    		row = dojo.fromJson(dict);
    	}

        // create table row and columns
        var tr = document.createElement('TR');
        tr.setAttribute("rowNum", "" + rowNum);
        this._cell_connects.push(dojo.connect(tr, 'onclick', this, '_select'));
        this._cell_connects.push(dojo.connect(tr, 'onkeypress', this, '_select'));
        if ( (this.defaultSelectionArea != null) && this._isSelected(rowNum, this.defaultSelectionArea) ) {
        	dojo.addClass(tr, 'defaultSelected');
        }

        for(i in this.columns) {
        	var column = this.columns[i];
            var td = document.createElement('TD');
            if (column['class']) {
            	dojo.addClass(td, column['clazz']);
            }
            if (column['selectionArea'] == null) {
            	column['buildFunction'](td, row);
            }
            else {
            	td.tabIndex = 0;
            	td.setAttribute("selectionArea", column['selectionArea']);
            	var s = this._isSelected(rowNum, column['selectionArea']);
            	this._buildSelectionCell(td, column['selectionArea'], column['selectionMode'], s);
            }
            tr.appendChild(td);
        }

        // alternating row style
        if(this.rows.length % 2) {
            dojo.addClass(tr, 'alt');
        }

        this.table_body.appendChild(tr);
        this.rows.push(row);
    },

	/**
	 * Build selection cell.
	 */
	_buildSelectionCell: function(td, area, mode, s) {
		var imgOn = document.createElement('IMG');
		var imgOff = document.createElement('IMG');
		dojo.addClass(imgOn, "imgOn");
		dojo.addClass(imgOff, "imgOff");
		imgOn.src = this._is0__n(mode) ? this.checkboxOnImageURL : this.radioOnImageURL;
		imgOff.src = this._is0__n(mode) ? this.checkboxOffImageURL : this.radioOffImageURL;
		this._togleSelection(td, s);
		td.appendChild(imgOff);
		td.appendChild(imgOn);  
	},
	
	/**
	 * Toggles selection CSS classes for TD.
	 */
	_togleSelection: function(td, s) {
       	if (s) {
       		dojo.addClass(td, 'selected');
       		dojo.removeClass(td, 'unselected');
       	}
       	else {
       		dojo.addClass(td, 'unselected');
       		dojo.removeClass(td, 'selected');
       	}
	},
	
	/**
	 * Toggles default selection CSS classes for TR.
	 */
	_togleDefaultSelection: function(tr, s) {
       	if (s) {
       		dojo.addClass(tr, "defaultSelected");
       	}
       	else {
       		dojo.removeClass(tr, "defaultSelected");
       	}
	},
    
    /**
     * Called when sotring is activated.
     */
    _sort: function(event) {
        var src = event.target;

        // if key but not space, don't count as a sort
        if(event.charCode != 0 && event.charCode != null && event.charCode != dojo.keys.SPACE) {
            return;
        }
        
        var index = null;
        var asc = null;
        var th = null;
        // clear all styles
        for(var i in this.table_header.childNodes) {
            th = this.table_header.childNodes[i];
            if(src == th) {
                index = i;
                if(dojo.hasClass(th, 'selectedUp')) {
                    // switch from up to down
                    asc = false;
                } 
				else if(dojo.hasClass(th, 'selectedDown')) {
                    // switch from down to up
                    asc = true;
                }
                break;
            }
        }
        var selectedSorting = null;
        if (index != null) {
	        var sorting = this.columns[index]['sorting'];
	        var supportedSortings = this.model.getSupportedSortings();
	        var j = 0;
	        for(j = 0; j < supportedSortings.length; j++) {
	        	if (supportedSortings[j].by == sorting) {
	        		if ( (asc == null) || (asc == supportedSortings[j].asc) ) {
	        			selectedSorting = supportedSortings[j];
	        			break;
	        		}
	        	}
	        }
	        if (selectedSorting != null) {
		        for(var i in this.table_header.childNodes) {
		            th = this.table_header.childNodes[i];
		            if(src == th) {
		               	if (selectedSorting.asc) {
		                   	dojo.addClass(th, 'selectedUp');
		                   	dojo.removeClass(th, 'selectedDown');
		                }
		                else {
		                   	dojo.addClass(th, 'selectedDown');
		                   	dojo.removeClass(th, 'selectedUp');
		                }
		            }
		            else {
		            	dojo.removeClass(th, 'selectedDown');
		            	dojo.removeClass(th, 'selectedUp');
		            }
		        }
	        }
	        // fetch the newly sorted results from the server
	        // no start or max because a new sort means we're starting at
	        // the beginning again anyways
		}     

        if (selectedSorting != null) {
        	if (this.resetOnSort) {
        		// We start from the initial paging:
        		this.model.setPageToLoad(null);
        	}
        	this.model.setSortingToApply(selectedSorting);
        	this._showBusyStatus();
        	this.model.fetch();
        }	
    },
    
    /**
     * Shows empty status.
     */
    _showNoStatus: function() {
    	if (this.status_row) {
    		this.status_row.innerHTML = "";
    	}
    },
    
    /**
     * Shows busy status.
     */
    _showBusyStatus: function() {
    	if (this.status_row) {
    		hub.busyMessage(this.status_row, false, "note");
    	}
    },
    
    /**
     * Shows error status.
     */
    _showErrorStatus: function(error) {
    	if (this.status_row) {
    		hub.notify(error, this.status_row, 'TOM_editor_error');
    	}
    },
    
    /**
     * Show no entries status.
     */
    _showNoEntriesStatus: function() {
    	if (this.status_row) {
    		this._loadNLS();
            this.status_row.innerHTML = this.msgs.STATUS_TABLE_NONE;
    	}
    },
    
    /**
     * Loads NLS for messages.
     */
    _loadNLS: function() {
    	if (!this.msgs) {
    		this.msgs = dojo.i18n.getLocalization('hub', 'messages'); 
    	}
    },
    
    /**
     * Sets selection for the area.
     */
	_setSelected: function(rowNum, selected, area) {
		if (selected == "uni") {
			// Uni selection case:
			if ( (rowNum != null) && this._isDataSelected(rowNum, area) ) { // 
				this.changedElements[area] = this._getRowId(rowNum); // selection is repeated
				this.isSelectedElements[area] = false;
			}
			else if (rowNum != null) {
				this.changedElements[area] = this._getRowId(rowNum); // 1 row is selected 
				this.isSelectedElements[area] = true;
			}
			else {
				this.changedElements[area] = null; // selected nothing
				this.isSelectedElements[area] = true;
			}
			return;
		}
		// Usual selection algorithm:
		var id = this._getRowId(rowNum);
		var s = this._isDataSelected(rowNum, area);
		var changedElements = null;
		var isSelectedElements = null;
		if ( area in this.isSelectedElements ) {
			if ( this._isBoolean(this.isSelectedElements[area]) ) {
				this.isSelectedElements[area] = [];
				this.changedElements[area] = [];
			}
			changedElements = this.changedElements[area];
			isSelectedElements = this.isSelectedElements[area];
		}
		else {
			if ( selected == s ) {
				return; // no need to set the same info as in data source
			}
			this.changedElements[area] = [];
			this.isSelectedElements[area] = [];
			changedElements = this.changedElements[area];
			isSelectedElements = this.isSelectedElements[area];
		}
		
		var i;
		for(i = 0; i < changedElements.length; i++) {
			if (changedElements[i] == id) {
				break;
			}
		}
		changedElements[i] = id; // i == length, we create new, if not we just reassign
		isSelectedElements[i] = selected;
		if ( (s && isSelectedElements[i]) || (!s && !isSelectedElements[i]) ) {
			// information in arrays is extra, we a to delete it:
			delete changedElements[i];
			delete isSelectedElements[i];
		}
	},
	
	/**
	 * Merges current selection with selection that commes from this row from the data.
	 */
	_mergeSelected: function(rowNum, area) {
		var id = this._getRowId(rowNum);
		if (area in this.isSelectedElements) {
			var changedElements = this.changedElements[area];
			var isSelectedElements = this.isSelectedElements[area];
			if (this._isBoolean(isSelectedElements)) {
				// Uni selection:
				if (id == changedElements) {
					this.isSelectedElements[area] = !this._isDataSelected(rowNum, area);
				}
				return;
			}
			var s = this._isDataSelected(rowNum, area);
			var i;
			for(i = 0; i < changedElements.length; i++) {
				if (changedElements[i] == id) {
					break;
				}
			}
			if (i == changedElements.length) {
				return;
			}
			if ( (s && isSelectedElements[i]) || (!s && !isSelectedElements[i]) ) {
				// information in arrays is extra, we a to delete it:
				delete changedElements[i];
				delete isSelectedElements[i];
			}
		}
	},

	/**
	 * Checks if row is selected in specified area.
	 */	
	_isSelected: function(rowNum, area) {
		var id = this._getRowId(rowNum);
		var s = this._isDataSelected(rowNum, area);
		if ( area in this.isSelectedElements ) {
			if (this._isBoolean(this.isSelectedElements[area])) {
				var soleId = this.changedElements[area];
				if (soleId == null) {
					return s;
				}
				else {
					return id == soleId;
				}
			}
			var changedElements = this.changedElements[area];
			var isSelectedElements = this.isSelectedElements[area];
			var i;
			for(i = 0; i < changedElements.length; i++) {
				if (changedElements[i] == id) {
					s = isSelectedElements[i];
					break;
				}
			}
		}
		return s;
	},
	
	/**
	 * Gets status of selection for row (delegates call to selection module).
	 */
	_isDataSelected: function(rowNum, area) {
		return this.selection.isDataSelected(this.model.getRow(rowNum), area);
	},
	
	/**
	 * Gets row id (delegates call to selection module).
	 */
	_getRowId: function(rowNum) {
		return this.selection.getRowId(this.model.getRow(rowNum));
	},
	
	/**
	 * Check is value is changed from original data selection.
	 */
	_isChanged: function(rowNum, area) {
		var s1 = this._isDataSelected(rowNum, area);
		var s2 = this._isSelected(rowNum, area);
	},

	/**
	 * Changes selection.
	 * Listens for events on TR and TD. Find appropriate area and toggles selection.
	 */	
	_select: function(event) {
		var src = event.target;
		var k = 0;
		for(k = 0; k < this.noDefaultSelectionTags.length; k++) {
			if (src.tagName == this.noDefaultSelectionTags[k]) {
				return;
			}
		}
		var srcTD = null;
		var selectionDefault = true;
		var selectionArea = null;
		var selectionMode = this.defaultSelectionAreaMode;
		if (this.defaultSelectionArea) {
			selectionArea = "default";
			selectionMode = this.defaultSelectionAreaMode;
		}
		while (src.tagName != 'TR') {
			if (src.tagName == 'TD') {
				var area = src.getAttribute("selectionArea");
				if (area) {
					srcTD = src;
					var i;
					for(i in this.columns) {
						if (this.columns[i]['selectionArea'] == area) {
							selectionArea = area;
							selectionMode = this.columns[i]['selectionMode'];
							selectionDefault = false;
							break;
						}
					}
				}
			}
			src = src.parentNode;
		}
		var rowNum = src.getAttribute("rowNum");
		if ((rowNum != null) && (selectionArea != null)) {
			rowNum = parseInt(rowNum);
			if (this._is1__1(selectionMode)) {
				this._setSelected(rowNum, "uni", selectionArea);
					if (selectionDefault) {
						this._defaultSelectionRedraw();
					}
					else {
						this._selectionRedraw(selectionArea);
					}
			}
			else if (this._is0__1(selectionMode)) {
				if (this._isSelected(rowNum, selectionArea)) {
					this._setSelected(null, "uni", selectionArea);
					if (selectionDefault) {
						this._togleDefaultSelection(src, false);
					}
					else {
						this._togleSelection(srcTD, false);
					}
				}
				else {
					this._setSelected(rowNum, "uni", selectionArea);
					if (selectionDefault) {
						this._defaultSelectionRedraw();
					}
					else {
						this._selectionRedraw(selectionArea);
					}
				}
			}
			else if (this._is0__n(selectionMode)) {
				if (this._isSelected(rowNum, selectionArea)) {
					this._setSelected(rowNum, false, selectionArea);
					if (selectionDefault) {
						this._togleDefaultSelection(src, false);
					}
					else {
						this._togleSelection(srcTD, false);
					}
				}
				else {
					this._setSelected(rowNum, true, selectionArea);
					if (selectionDefault) {
						this._togleDefaultSelection(src, true);
					}
					else {
						this._togleSelection(srcTD, true);
					}
				}
			}
		}
	},
	
	/**
	 * Redraw selection for default area. Adjusts styles accoriding to current selection.
	 */
	_defaultSelectionRedraw: function() {
		var _this = this; 
		dojo.query("tr[rowNum]", this.table_body).forEach(function(tr) {
			var rowNum = tr.getAttribute("rowNum");
			if (rowNum != null) {
				rowNum = parseInt(rowNum);
		        if ( (_this.defaultSelectionArea != null) && _this._isSelected(rowNum, _this.defaultSelectionArea) ) {
		        	_this._togleDefaultSelection(tr, true);
		        }
		        else {
		        	_this._togleDefaultSelection(tr, false);
		        }
			}			
		});
	},
	
	/**
	 * Redraw selection for area. Adjusts styles accoriding to current selection.
	 */
	_selectionRedraw: function(area) {
		var _this = this;
		dojo.query('td[selectionArea=\"' + area + '\"]', this.table_body).forEach(function(td) {
			var rowNum = td.parentNode.getAttribute("rowNum");
			if (rowNum != null) {
				rowNum = parseInt(rowNum);
	        	_this._togleSelection(td, _this._isSelected(rowNum, area));
			}			
		});
	},
	
	/**
	 * Check is selection mode code corresponds to 1..1
	 */
	_is1__1: function(mode) {
		return ( (1 == mode) || ("1..1" == mode) || ("1" == mode) );
	},
	
	/**
	 * Check is selection mode code corresponds to 0..1
	 */
	_is0__1: function(mode) {
		return ( (0 == mode) || ("0..1" == mode) || ("0" == mode) );
	},

	/**
	 * Check is selection mode code corresponds to 0..n
	 */ 
	_is0__n: function(mode) {
		return ( (mode > 1) || ("0..n" == mode) || (parseInt(mode) > 1) );
	},
	
	/**
	 * Check if value is exactly true of false.
	 */
	_isBoolean: function(v) {
		return (true == v) || (false == v);
	},
	
	/**
	 * Retursn value for individual ares.
	 * If selection is uni, selected element id is returned, or null if no selection, or [] if original selection was not changed.
	 * If selection is multi, array of changed elements is returned: [{id: ..., selected: true/false}, {id: ..., selected: true/false}, ...]
	 */
	getValueForArea: function(area) {
		if (area in this.isSelectedElements) {
			var changedElements = this.changedElements[area];
			var isSelectedElements = this.isSelectedElements[area];
			if (this._isBoolean(isSelectedElements)) {
				if (isSelectedElements || uniselectionReselectValue) {
					return this.changedElements[area];
				}
				else {
					return [];
				}
			}
			var result = new Array();
			var i;
			for(i = 0; i < changedElements.length; i++) {
				result.push({'id':changedElements[i], 'selected':isSelectedElements[i]});
			}
			return result;
		}
		else {
			return [];
		}
	},
	
	/**
	 * Sets value for individual area.
	 * If value is singleton selection will be uni, with value as selected element.
	 * If value is null      selection will be uni(0..1), with no selection.
	 * If value is []        selection will be uni or multi, meaning no change from original value.
	 * If value is non empty array, selection will be multi, each record of form {id: ..., selected: true/false}.
	 */
	setValueForArea: function(area, value) {
		if (dojo.isArray(value)) {
			this.changedElements[area] = new Array();
			this.isSelectedElements[area] = new Array();
			var i = 0;
			for(i = 0; i < value.length; i++) {
				this.changedElements[area].push(value[i].id);
				this.isSelectedElements[area].push(value[i].selected);
			}
		}
		else { // uni selection
			this.changedElements[area] = new Array();
			this.isSelectedElements[area] = new Array();
			this.changedElements[area].push(value); // as id
			this.isSelectedElements[area].push(true);
		}
	},
	
	/**
	 * Sets value for this control,
	 * value may be only one area (valueSelectionArea), or all areas (whne valueSelectionArea = null).
	 */
	setValue: function(value) {
		var area;
		for(area in this.isSelectedElements) {
			if (this.valueSelectionArea == area) {
				this.setValueForArea(area, value);
				break;
			}
			if (this.formSelectionArea == null) {
				var v = [];
				if (value[area]) {
					v = value[area];
				}
				this.setValueForArea(area, v);
			}
		}
	},
	
	/**
	 * Returns value of this control,
	 * value may be only one area (valueSelectionArea), or all areas (whne valueSelectionArea = null).
	 */
	getValue: function() {
		var area;
		var result = {};
		for(area in this.isSelectedElements) {
			if (this.valueSelectionArea == area) {
				return this.getValueForArea(area);
				break;
			}
			if (this.formSelectionArea == null) {
				result[area] = this.getValueForArea(area);
			}
		}
		return result;
	},
	
	/**
	 * Disconnects array or individual listner.
	 * @return empty array or null
	 */
	_disconnect: function(a) {
		if (dojo.isArray(a)) {
			for(;;) {
				if (a.length == 0) {
					break;
				}
				var v = a.pop();
				if (v) {
					dojo.disconnect(v);
				}
				else {
					break;
				}
			}
		}
		else {
			if (a) {
				dojo.disconnect(a);
				a = null;
			}
		}
		return a;
	}
});

}

if(!dojo._hasResource["hub.table.navigation"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.table.navigation"] = true;
dojo.provide("hub.table.navigation");

/**
 * Represents navigation bar for table (normaly list of tabs, but may be other type of control.
 */
dojo.declare("hub.table.navigation.Navigation", [dijit._Widget, dijit._Templated], {
	/**
	 * Table model to listen to.
	 */
	model: null,
	
	/**
	 * Connection to model.
	 */
	_model_connects: [],
	
	constructor: function(args, node) {
		this._model_connects = new Array();
	},
	
    postMixInProperties: function() {
        if (dojo.isString(this.model)) {
        	// get model widget from id
        	this.model = dijit.byId(this.model);
        }
    },
    
	postCreate: function() {
        this.inherited(arguments);
        this.initialize();		
	},
	
	startup: function() {
		if (this.model != null) {
			this.fetched();
		}
	},
	
    /**
     * Sets new model.
     */
    setModel: function(newModel) {
    	this.model = newModel;
    	this.initialize();
		if (this.model != null) {
			this.fetched();
		}
    },
 
    /**
     * Initialize listeners.
     */
    initialize: function() {
   		dojo.forEach(this._model_connects, dojo.disconnect);
   		this._model_connects = new Array();
		if (this.model != null) {
			this._model_connects.push(dojo.connect(this.model, "onFetched", this, "fetched")); 
		}
    },
    
    destroy: function() {
    	dojo.forEach(this._model_connects, dojo.disconnect);
    	this._model_connects = new Array();
    },
    	
 	fetched: function() {
 		throw new Error("hub.table.navigation.Navigation is abstract");
 	}	
});

/**
 * Represents the simplest imaginable navigation with button to previous and button to the 
 * next pages.
 */
dojo.declare("hub.table.navigation.PrevNextNavigation", hub.table.navigation.Navigation, {
    templateString: '',
    // template to replace node with
    templateString:"<div class='navigation'>\r\n   <span class='prevButton'><button dojoAttachPoint='prevButton' type='button' dojoType='dijit.form.Button' dojoAttachEvent='onClick:_onPrev'></button></span>\r\n   <span style='font-size:0.8em;' dojoAttachPoint='navigationStatus'></span>\r\n   <span class='nextButton'><button dojoAttachPoint='nextButton' type='button' dojoType='dijit.form.Button' dojoAttachEvent='onClick:_onNext'></button></span>\r\n   <span class='refreshButton'><button dojoAttachPoint='refreshButton' type='button' dojoType='dijit.form.Button' dojoAttachEvent='onClick:_onRefresh'></button></span>\r\n</div>\r\n",
    widgetsInTemplate: true,

	/**
	 * Prev page ref. (if any)
	 */
	prevPage: null,
	
	/**
	 * Next page ref. (if any)
	 */
	nextPage: null,
	
	/**
	 * Current page ref. (if any)
	 */
	currentPage: null,
    
    /**
     * Adjust navigation controls to the state of model.
     */
	fetched: function() {
		if (this.model.isFetched()) {
			var pages = this.model.getAvailablePages();
			var i = 0;
			var currentPage = null;
			var prevPage = null;
			var nextPage = null;
			for(i = 0; i < pages.length; i++) {
				var page = pages[i];
				var relative = page.relative;
				if (relative == 0) {
					currentPage = page;
				}
				else if (relative == -1) {
					prevPage = page;
				}
				else if (relative == +1) {
					nextPage = page;
				}
			}
			this.currentPage = currentPage;
			this.prevPage = prevPage;
			this.nextPage = nextPage;
			this._updateContent();
		}
	},
	
	/**
	 * Updates content (buttons) of the navigation.
	 */
	_updateContent: function() {
		this.prevButton.setDisabled(this.prevPage == null);
		this.nextButton.setDisabled(this.nextPage == null);
		this.navigationStatus.innerHTML = this.getSummaryMarkup();
	},
	
    /**
     * Creates markup for summary text (selected record from XX to YY total ZZZ).
     */
    getSummaryMarkup: function() {
    	if (this.model == null) {
    		return "";
    	}
    	var currentPage = this.model.getCurrentPage();
    	if (currentPage != null) {
    		if (!this.msgs) {
    			this.msgs = dojo.i18n.getLocalization('hub', 'messages');
    		}
	        var start = currentPage.fromRow + 1;
	        var end = currentPage.toRow;
	        var total = this.model.getGuaranteedRowCount();
	        if (this.model.hasMoreRecords()) {
	        	total = "" + total + "+"; 
	        }
	        if (currentPage.size == 0) {
	        	return this.msgs.STATUS_CATALOG_NONE;
	        }
	        return dojo.string.substitute(this.msgs.STATUS_TABLE_SUMMARY, 
	                                          [start, end, total]);
	    }
	    return "";
    },	

	/**
	 * Called when prev button is activated.
	 */	
	_onPrev: function() {
		if ((this.prevPage != null) && (this.model != null)) {
			this.model.setPageToLoad(this.prevPage);
			this.model.fetch();
		} 
	},

    /**
     * Called when next button is activated.
     */
	_onNext: function() {
		if ((this.nextPage != null) && (this.model != null)) {
			this.model.setPageToLoad(this.nextPage);
			this.model.fetch();
		} 
	},
	
	/**
	 * Called when refresh button is activated.
	 */
	_onRefresh: function() {
		if ((this.currentPage != null) && (this.model != null)) {
			this.model.setPageToLoad(this.currentPage);
		}
		if (this.model != null) {
			this.model.fetch();
		}
	}
});

}

if(!dojo._hasResource["hub.data.jsonmodel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.data.jsonmodel"] = true;
dojo.provide("hub.data.jsonmodel");



dojo.declare("hub.table.jsonmodel.DataSelection", hub.table.model.DataSelection, {
	isDataSelected: function(row, area) {
		if ("isChecked" in row) {
			return (false != row.selected);
		}
		return false;
	}
});


dojo.declare('hub.data.jsonmodel.TableModel', hub.table.model.ItemizedModel, {

	pageSize: 20,
	totalRecords: null,
	initialSorting: "title",
	supportedSortings: ["title", "id", "rating"],
	availablePages: [],
	url: '',
	url_used: '',
	url_prev: '',
	url_next: '',
	currentPage: null,
	
	constructor: function(args, node) {
		availablePages = new Array();
		var sorting = new hub.table.model.Sorting();
		sorting.by = "title";
		sorting.asc = true;
		this.setSortingToApply(sorting);

		var page = new hub.table.model.Page();
		this.url = args.url;
    if (args.pageSize) {
      this.pageSize = args.pageSize;
    }
		page.url = this._getInitialUrl();
		page.relative = 0;
		page.fromRow = 0;
		page.size = this.pageSize;
		this.setPageToLoad(page);
		
	},
	
	_getInitialUrl: function() {
///->	
		var url = this.url;//hub.urls.getCatalog(this.collection, this.term, this.searchType, null, null);
//        console.debug("URL = " + url);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
//        	console.debug("Parsed query = " + query + ", URL = " + url);
        }
        var map = dojo.queryToObject(query);
        map["nologin"] = "true";
        
        if (this.pageSize != null) {
        	map["count"] = "" + this.pageSize;
        }
       	map["from"] = 0;

        if (this.term) {
          map["term"] = this.term;
        }

        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
//        console.debug("Result URL = " + url);
        return url;
	},

	getSupportedSortings: function() {
		var sortings = new Array();
		var i;
		for(i = 0; i < this.supportedSortings.lenght; i++) {
			var s = new hub.table.model.Soting();
			s.by = this.supportedSortings[i];
			s.asc = true;
			sortings.push(s);
			s = new hub.table.model.Soting();
			s.by = this.supportedSortings[i];
			s.asc = false;
			sortings.push(s);
		}
		return sortings;
	},
	
	getGuaranteedRowCount: function() {
		return this.totalRecords;
	},
	
	hasMoreRecords: function() {
		return false;
	},
	
	getCurrentPage: function() {
		return this.currentPage;
	},
	
	getCurrentPageSize: function() {
		if (this.getCurrentPage() == null) {
			return null;
		}
		return this.getCurrentPage().size;
	},	

	getAvailablePages: function() {
		return this.availablePages;
	},
	
	_mixinSorting: function(url, sorting) {
//        console.debug("URL = " + url);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
//        	console.debug("Parsed query = " + query + ", URL = " + url);
        }
        var map = dojo.queryToObject(query);
        map['nologin'] = 'true';
        if (sorting != null) {
        	map["sortBy"] = sorting.by;
        }
        if (sorting != null) {
        	map["sortOrder"] = sorting.asc ? "asc" : "desc";
        }
        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
//        console.debug("Result URL = " + url);
        return url;
	},
	
	_mixinOffsetCount: function(url, offset, count){
//        console.debug("URL = " + url);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
//        	console.debug("Parsed query = " + query + ", URL = " + url);
        }
        var map = dojo.queryToObject(query);
        map['nologin'] = 'true';
        if (offset != null) {
        	map["from"] = offset;
        }
        if (count != null) {
        	map["count"] = count;
        }else{
        	map["count"] = this.pageSize;
        }
        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
//        console.debug("Result URL = " + url);
        return url;
	},
	
	_mixinPageNumber: function(url, pageNumber){
		if(pageNumber == null || pageNumber < 1){
			pageNumber = 1;
		}
		return this._mixinOffsetCount(url, (pageNumber-1)*this.pageSize, this.pageSize);
	},
	
	fetch: function() {
		var url = null;
		var sorting = this.getSortingToApply();
		var pageToLoad = this.getPageToLoad();
		if (pageToLoad == null) {
			url = this._getInitialUrl();
		}
		else {
			url = pageToLoad.url;
		}
		this.url = url
		url = this._mixinSorting(url, sorting);
//        console.debug("point A");
        
		this.onBeforeFetched();
		var lock = this.onPageBegin();

        var __this = this;

		var kw = {
		    url: url,
            handleAs: 'text', // let the callback decode
            timeout: 120000, // 2 minutes in milliseconds
            model: this,
            sync: false,
            lock: lock,
            
            error: dojo.hitch(this, this.onPageError),
            
            load: function(response, args) {
				try {
                    if(!response) {
                        throw new Error('Form: Empty server response');
                    }
                    if(hub.submit.processResponseCode(response)){
///->
						var data = dojo.fromJson(response)
						var loaded_page = this.model.getPageToLoad();
            if (!loaded_page) {
              // ugly hack here
              loaded_page = new hub.table.model.Page();
              loaded_page.url = this.url;
            }
						var i;
						//	retrieving rows from result
						for(i = 0; i < data.rows.length; i++) {
//							console.debug("1111 = " + this.lock);
							this.model.onItem(this.lock, data.rows[i]);
						}
						if (this.model.onPageEnd(this.lock)) {
//							console.debug("2222 = " + this.lock);				
							this.sorting = this.model.getSortingToApply();

							// generating navigation pages
							var pages = new Array();
							// generating current (refresh) page
							var page = new hub.table.model.Page();
///->
							page.url = loaded_page.url;
							if (loaded_page.fromRow == 0) {
								page.forwardAbsolute = 0;
							}
							if (loaded_page.fromRow + this.model.pageSize >= data.total) {
								page.backwardAbsolute = 0;
							}
							page.relative = 0;
///->
							page.fromRow = loaded_page.fromRow;
							page.toRow = loaded_page.fromRow + data.rows.length;
							page.size = data.rows.length;
//							console.dir({loaded_page: loaded_page});
							if (loaded_page.fromRow > 0) {
							// generating previous page
								var prev_page = new hub.table.model.Page();
								prev_page.url = this.model._mixinOffsetCount(loaded_page.url, loaded_page.fromRow-this.model.pageSize);
								if (page.backwardAbsolute != null) {
									prev_page.backwardAbsolute = page.backwardAbsolute + 1; 
								}
								if (page.forwardAbsolute != null) {
									prev_page.forwardAbsolute = page.forwardAbsolute - 1; 
								}
								prev_page.relative = -1;
								prev_page.fromRow = loaded_page.fromRow - this.model.pageSize;
								pages.push(prev_page);
							}
							pages.push(page);
							if (loaded_page.fromRow + this.model.pageSize < data.total) {
							// generating next page
								var next_page = new hub.table.model.Page();
								next_page.url = this.model._mixinOffsetCount(loaded_page.url, loaded_page.fromRow+this.model.pageSize);
								if (page.backwardAbsolute != null) {
									next_page.backwardAbsolute = page.backwardAbsolute - 1; 
								}
								if (page.forwardAbsolute != null) {
									next_page.forwardAbsolute = page.forwardAbsolute + 1; 
								}
								next_page.relative = +1;
								next_page.fromRow = loaded_page.fromRow + this.model.pageSize;
								pages.push(next_page);
							}
							this.model.availablePages = pages;
							this.model.currentPage = page;
							this.model.totalRecords = data.total;
							this.model.pageToLoad = page;
							this.model.onFetched();
//							console.debug("onFetched called!!!");
						}
    
				    }
				} catch(e) {
                    var msgs = dojo.i18n.getLocalization('hub', 'messages');
                    var err = msgs.ERROR_REQUEST_CLIENT + " " + e.message;
//                    hub.notify(err, this.getStatus());
				}
                return response;
            }
		};
	        // do the post
		dojo.xhrPost(kw);

//        console.debug("point C");		
	},
	
	onFetched: function() {
//		console.debug("onFetched");
	},

	onBeforeFetched: function() {
//		console.debug("onBeforeFetched");
	},
	
	onPageError: function() {
		// called never
	}
});

}

if(!dojo._hasResource['hub.widgets']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.widgets'] = true;
dojo.provide('hub.widgets');



 // for search form
 // radio button is in here


/**
 * Displays an array of values keyed by column names in a table.
 */
dojo.declare("hub.widgets.Table", [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templateString:"<div class=\"TOM_widget_Table\">\r\n  <table>\r\n    <thead>\r\n      <tr dojoAttachPoint=\"table_header\"></tr>\r\n    </thead>\r\n    <tbody dojoAttachPoint=\"table_body\"></tbody>\r\n  </table>\r\n</div>\r\n",

    constructor : function(args, node) {
        // column metadata
        this.columns = null;
        // number of rows added to table so far for this query
        this.row_count = 0;
    },

    initialize: function() {
        // clear out all table headers and body
        this.clearHeaders();
        this.clearRows();

        for(i in this.columns) {
            var th = document.createElement('TH');
            // title and base class
            th.innerHTML = this.columns[i]['name'];
            if(this.columns[i]['class']) {
	            th.className = this.columns[i]['class'];
	        }
            this.table_header.appendChild(th);
        }
    },

    /**
     * Clears all table headers.
     */
    clearHeaders: function() {
        // NOTE: we really want to just do innerHTML = '', but we can't
        // because of IE6/IE7; have to use DOM else get runtime errors
        while (this.table_header.firstChild)  {
            this.table_header.removeChild(this.table_header.firstChild);
        };
    },

    /**
     * Clears the body of the table.
     */
    clearRows: function() {
        // NOTE: we really want to just do innerHTML = '', but we can't
        // because of IE6/IE7; have to use DOM else get runtime errors
        while (this.table_body.firstChild)  {
            this.table_body.removeChild(this.table_body.firstChild);
        };
        this.row_count = 0;
    },

    /**
     * Adds a single row to the table.
     * @param dict single row data keyed by column name:
     *  - json string in format {'col1': data1, 'col2': data2, ...}
     *  - json object
     */
    addRow: function(dict) {
    	var row=dict;

    	// convert to json object data
		if(!(dict instanceof Object)) {
    		row=dojo.fromJson(dict);
    	}

        if(!this.columns) {
            // if no columns are set, use the columns specified in this row
            this.columns = new Array();
            for(key in row) {
                this.columns.push({'name' : key});
            }
            this.initialize();
        }

        // create table row and columns
        var tr = document.createElement('TR');

        for(i in this.columns) {
            var name = this.columns[i]['name'];
            var td = document.createElement('TD');
            var ml = row[name];
            td.innerHTML = ml;
            tr.appendChild(td);
        }

        // alternating row style
        if(this.row_count % 2) {
            dojo.addClass(tr, 'alt');
        }

        this.table_body.appendChild(tr);
        this.row_count += 1;
    },

    /**
     * Adds a set of rows to the table.
     *
     * @param arr Array of row data keyed by column
     *   [{'col1': data1, ...}, {'col1': data2, ...} ...]
     * or
     */
    addRows: function(arr) {
        dojo.forEach(arr, dojo.hitch(this,this.addRow));
    },

    /**
     * Sets an array of column metadata objects. The only required key in
     * each metadata object is 'name' representing the human readable name
     * to be displayed on the column header. The key 'class' is optional
     * and represents the CSS class to be applied to a column. Other keys
     * may be included, but will be ignored
     *
     *
     * @param arr Array of column metadata objects
     *   [{name: 'Foobar', ...}, ...]
     */
    setColumns: function(arr) {
        this.columns = arr;
        // have to reinitialize everything
        this.initialize();
    }
});

/**
 * Renders a list of checkable items managed entirely by JS. Existing items in
 * the box are ignored. All items are initially unchecked when added. External
 * observer can connect to the 'onChange' signal with params of 'label' and
 * 'value'.
 */
dojo.declare('hub.widgets.CheckBoxList', [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templateString:"<div class=\"TOM_widget_CheckBoxList\"\r\n  ><ul dojoAttachPoint=\"box\"\r\n  ></ul>\r\n</div>\r\n",
	groups: {},
	checked: {},
	all: {},
	init_labels: [],
	init_names: [],

    /**
     * @param args: ignored
     * @param node: Node to which to attach this widget
     */
    constructor: function(args, node) {
        // key: group id, value: div holding the group
        this.groups = {};
        // key: label, value: checked (true) or unchecked (false)
        this.checked = {};
        // key: group id, value: array of group elements
        this.all = {};

        // keep labels of all elements initially in the list
        this.init_labels = [];
        this.init_names = [];
        dojo.query('li', node).forEach(dojo.hitch(this, function(li) {
            this.init_labels.push(li.innerHTML);
            this.init_names.push(li.innerHTML);
        }));
    },

    /**
     * Makes a group named 'default' for all items that were in the original
     * DOM node.
     */
    postCreate: function() {
        this.appendGroup('default', this.init_labels, this.init_names);
        delete this.init_names;
        delete this.init_labels;
    },

    /**
     * Handles check/uncheck events by signaling observers and updating
     * internal state of what's checked.
     *
     * @return: style class to put on the outer container
     */
    _onCheck: function(name, event) {
        var cb = event.target;
        // keep track of checked values
        if(cb.checked) {
            this.checked[name] = true;
        } else {
            delete this.checked[name];
        }
        this.onChange(name, cb.checked);
    },

    /**
     * Adds a new group of checkboxes to the list. The id is used to later
     * remove all the added checkboxes. All items are initially unchecked
     *
     * @param id: Unique ID for the group within the scope of this widget
     *   only, not global
     * @param labels: Array of labels for the checkboxes
     */
    appendGroup: function(id, labels, names) {
        if(!this.groups[id]) {
        	if(names == null){
        		names = labels;
        	}
            // build a new div for each group within the list
            var div = document.createElement('DIV');
            // loop over all labels and add elements
            this.all[id] = [];
            for(var i in labels) {
                var li = document.createElement('LI');
                // add the checkbox
                var input = document.createElement('INPUT');
                input.type = 'checkbox';
                input.name = names[i];
                li.appendChild(input);
                if(this.checked[names[i]]){
                	input.setAttribute("checked", true);
                }
                // watch for events on the checkbox
                // hitch so we can pass the label too
                this.connect(input, 'onclick',
                   dojo.hitch(this, this._onCheck, names[i]));
                // add the label
                var label = document.createElement('LABEL');
                label.htmlFor = names[i];
                label.innerHTML = labels[i];
                li.appendChild(label);
                // add list item to the div
                div.appendChild(li);
                // i is index (number)
                this.all[id][i] = names[i];
            }
            // add the div to the box
            this.box.appendChild(div);
            // keep track of the div by the group id
            this.groups[id] = div;
        } else {
            throw new Error('CheckBoxList: group already exists');
        }
    },

    /**
     * Removes an existing group of checkboxes from the list.
     *
     * @param id: Unique ID for the group
     */
    removeGroup: function(id) {
        var div = this.groups[id];
        if(!div) {
            throw new Error('CheckBoxList: group does not exist');
        }
        // clear out all checked items in the div that's going away
        dojo.query('input', div).forEach(dojo.hitch(this, function(input) {
            delete this.checked[input.name];
        }));

        // @todo: should really clean up connections for group here, but we
        // do at least get them later

        // throw away the div and its group reference
        this.box.removeChild(div);
        delete this.groups[id];
        delete this.all[id];
    },

    /**
     * Remove all check boxes.
     */
    clear: function() {
        this.all = {};
        this.groups = {};
        this.checked = {};
        this.box.innerHTML = '';

        // clean up event connections
		dojo.forEach(this._connects, function(array){
	    dojo.forEach(array, dojo.disconnect);
	});

    },

    /**
     * Get an array of all labels
     */
    getAll: function() {
        var arr = [];
        for(var i in this.all) {
            for(var j=0; j<this.all[i].length; j++) {
                arr[arr.length] = this.all[i][j];
            }
        }
        return arr;
    },

    /**
     * Gets an array of labels of all checked items in the list.
     *
     * @return: Array of labels
     */
    getAllChecked: function() {
        var checked = new Array();
        for(var name in this.checked) {
            checked.push(name);
        }
        return checked;
    },

    /**
     * Checks or unchecks a specific item in the list
     *
     * @param id: Unique ID for the group
     * @param label: Name of the item in the group
     * @param value: 'true' to check the item, 'false' to uncheck it
     */
    setChecked: function(id, name, value) {
        var div = this.groups[id];
        // NOTE: these are not arrays, be careful
        var cbs = dojo.query('input', div);
        for(var i=0; i < cbs.length; i++) {
            if(cbs[i].name == name) {
                cbs[i].checked = value;
                // have to do manually because IE requires onclick which does
                // fire when we change checked programmatically
                if(value) {
                    this.checked[name] = true;
                }
                else {
                    delete this.checked[name];
                }
            }
        }
    },

    /**
     * Tests if item is checked.
     * @return true if checked, otherwize false
     */
    isChecked: function(name) {
		if(name in this.checked) {
			return true;
		}
		return false;
    },

    /**
     * Checks or unchecks all items in a group.
     *
     * @param id: Unique ID for the group
     * @param value: true to check, false to uncheck
     */
    setGroupChecked: function(id, value) {
        var div = this.groups[id];
        // NOTE: these are not arrays, be careful
        var cbs = dojo.query('input', div);

        for(var i=0; i < cbs.length; i++) {
            cbs[i].checked = value;
            // have to do manually because IE requires onclick which does
            // fire when we change checked programmatically
            var name = cbs[i].name;
            if(value)
                this.checked[name] = true;
            else
                delete this.checked[name];
        }
    },

    /**
     * Checks or unchecks all items in the list.
     *
     * @param value: true to check, false to uncheck
     */
    setAllChecked: function(value) {
        for(var i in this.groups) {
            this.setGroupChecked(i, value);
        }
    },

    /**
     * Signals when any checkbox changes value.
     *
     * @param label: Label text of the checkbox that changed value
     * @param value: Checked or unchecked
     */
    onChange: function(label, value) {
        // extension point
    }
});

dojo.declare('hub.widgets.Button', dijit.form.Button,{
	templateString: null,
	baseClass: 'BlueButton',
	templateString:"<button\r\n\t\t\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\" \r\n\t\t\tclass=\"\" dojoAttachPoint=\"focusNode,titleNode\"\r\n\t\t\tttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\r\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\"\r\n\t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \r\n\t\t\t></span\r\n\t\t\t><span class=\"dijitButtonText\" style=\"border: none; background: transparent; color: inherited;\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span\r\n\t\t></button\r\n\t>\r\n"
})


}

if(!dojo._hasResource["dojo.cldr.supplemental"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cldr.supplemental"] = true;
dojo.provide("dojo.cldr.supplemental");



dojo.cldr.supplemental.getFirstDayOfWeek = function(/*String?*/locale){
// summary: Returns a zero-based index for first day of the week
// description:
//		Returns a zero-based index for first day of the week, as used by the local (Gregorian) calendar.
//		e.g. Sunday (returns 0), or Monday (returns 1)

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/firstDay
	var firstDay = {/*default is 1=Monday*/
		mv:5,
		ae:6,af:6,bh:6,dj:6,dz:6,eg:6,er:6,et:6,iq:6,ir:6,jo:6,ke:6,kw:6,lb:6,ly:6,ma:6,om:6,qa:6,sa:6,
		sd:6,so:6,tn:6,ye:6,
		as:0,au:0,az:0,bw:0,ca:0,cn:0,fo:0,ge:0,gl:0,gu:0,hk:0,ie:0,il:0,is:0,jm:0,jp:0,kg:0,kr:0,la:0,
		mh:0,mo:0,mp:0,mt:0,nz:0,ph:0,pk:0,sg:0,th:0,tt:0,tw:0,um:0,us:0,uz:0,vi:0,za:0,zw:0,
		et:0,mw:0,ng:0,tj:0,
		gb:0,
		sy:4
	};

	var country = dojo.cldr.supplemental._region(locale);
	var dow = firstDay[country];
	return (typeof dow == 'undefined') ? 1 : dow; /*Number*/
};

dojo.cldr.supplemental._region = function(/*String?*/locale){
	locale = dojo.i18n.normalizeLocale(locale);
	var tags = locale.split('-');
	var region = tags[1];
	if(!region){
		// IE often gives language only (#2269)
		// Arbitrary mappings of language-only locales to a country:
        region = {de:"de", en:"us", es:"es", fi:"fi", fr:"fr", hu:"hu", it:"it",
        ja:"jp", ko:"kr", nl:"nl", pt:"br", sv:"se", zh:"cn"}[tags[0]];
	}else if(region.length == 4){
		// The ISO 3166 country code is usually in the second position, unless a
		// 4-letter script is given. See http://www.ietf.org/rfc/rfc4646.txt
		region = tags[2];
	}
	return region;
}

dojo.cldr.supplemental.getWeekend = function(/*String?*/locale){
// summary: Returns a hash containing the start and end days of the weekend
// description:
//		Returns a hash containing the start and end days of the weekend according to local custom using locale,
//		or by default in the user's locale.
//		e.g. {start:6, end:0}

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/weekend{Start,End}
	var weekendStart = {/*default is 6=Saturday*/
		eg:5,il:5,sy:5,
		'in':0,
		ae:4,bh:4,dz:4,iq:4,jo:4,kw:4,lb:4,ly:4,ma:4,om:4,qa:4,sa:4,sd:4,tn:4,ye:4		
	};

	var weekendEnd = {/*default is 0=Sunday*/
		ae:5,bh:5,dz:5,iq:5,jo:5,kw:5,lb:5,ly:5,ma:5,om:5,qa:5,sa:5,sd:5,tn:5,ye:5,af:5,ir:5,
		eg:6,il:6,sy:6
	};

	var country = dojo.cldr.supplemental._region(locale);
	var start = weekendStart[country];
	var end = weekendEnd[country];
	if(typeof start == 'undefined'){start=6;}
	if(typeof end == 'undefined'){end=0;}
	return {start:start, end:end}; /*Object {start,end}*/
};

}

if(!dojo._hasResource["dojo.regexp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.regexp"] = true;
dojo.provide("dojo.regexp");

dojo.regexp.escapeString = function(/*String*/str, /*String?*/except){
	//	summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

//	return str.replace(/([\f\b\n\t\r[\^$|?*+(){}])/gm, "\\$1"); // string
	return str.replace(/([\.$?*!=:|{}\(\)\[\]\\\/^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
}

dojo.regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	//	summary:
	//		Builds a regular expression that groups subexpressions
	//	description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression. 
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return dojo.regexp.group(b.join("|"), nonCapture); // String
}

dojo.regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. 
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
}

}

if(!dojo._hasResource["dojo.date.locale"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.locale"] = true;
dojo.provide("dojo.date.locale");

// Localization methods for Date.   Honor local customs using locale-dependent dojo.cldr data.







// Load the bundles containing localization information for
// names and formats


//NOTE: Everything in this module assumes Gregorian calendars.
// Other calendars will be implemented in separate modules.

(function(){
	// Format a pattern without literals
	function formatPattern(dateObject, bundle, pattern){
		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s;
			var c = match.charAt(0);
			var l = match.length;
			var pad;
			var widthList = ["abbr", "wide", "narrow"];
			switch(c){
				case 'G':
					s = bundle[(l < 4) ? "eraAbbr" : "eraNames"][dateObject.getFullYear() < 0 ? 0 : 1];
					break;
				case 'y':
					s = dateObject.getFullYear();
					switch(l){
						case 1:
							break;
						case 2:
							s = String(s); s = s.substr(s.length - 2);
							break;
						default:
							pad = true;
					}
					break;
				case 'Q':
				case 'q':
					s = Math.ceil((dateObject.getMonth()+1)/3);
//					switch(l){
//						case 1: case 2:
							pad = true;
//							break;
//						case 3: case 4: // unimplemented
//					}
					break;
				case 'M':
				case 'L':
					var m = dateObject.getMonth();
					var width;
					switch(l){
						case 1: case 2:
							s = m+1; pad = true;
							break;
						case 3: case 4: case 5:
							width = widthList[l-3];
							break;
					}
					if(width){
						var type = (c == "L") ? "standalone" : "format";
						var prop = ["months",type,width].join("-");
						s = bundle[prop][m];
					}
					break;
				case 'w':
					var firstDay = 0;
					s = dojo.date.locale._getWeekOfYear(dateObject, firstDay); pad = true;
					break;
				case 'd':
					s = dateObject.getDate(); pad = true;
					break;
				case 'D':
					s = dojo.date.locale._getDayOfYear(dateObject); pad = true;
					break;
				case 'E':
				case 'e':
				case 'c': // REVIEW: don't see this in the spec?
					var d = dateObject.getDay();
					var width;
					switch(l){
						case 1: case 2:
							if(c == 'e'){
								var first = dojo.cldr.supplemental.getFirstDayOfWeek(options.locale);
								d = (d-first+7)%7;
							}
							if(c != 'c'){
								s = d+1; pad = true;
								break;
							}
							// else fallthrough...
						case 3: case 4: case 5:
							width = widthList[l-3];
							break;
					}
					if(width){
						var type = (c == "c") ? "standalone" : "format";
						var prop = ["days",type,width].join("-");
						s = bundle[prop][d];
					}
					break;
				case 'a':
					var timePeriod = (dateObject.getHours() < 12) ? 'am' : 'pm';
					s = bundle[timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c) {
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'v': // FIXME: don't know what this is. seems to be same as z?
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = dojo.date.getTimezoneName(dateObject);
					if(s){break;}
					l=4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = dateObject.getTimezoneOffset();
					var tz = [
						(offset<=0 ? "+" : "-"),
						dojo.string.pad(Math.floor(Math.abs(offset)/60), 2),
						dojo.string.pad(Math.abs(offset)% 60, 2)
					];
					if(l==4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
//				case 'Y': case 'u': case 'W': case 'F': case 'g': case 'A':
//					console.debug(match+" modifier unimplemented");
				default:
					throw new Error("dojo.date.locale.format: invalid pattern char: "+pattern);
			}
			if(pad){ s = dojo.string.pad(s, l); }
			return s;
		});
	}

dojo.date.locale.format = function(/*Date*/dateObject, /*Object?*/options){
	// summary:
	//		Format a Date object as a String, using locale-specific settings.
	//
	// description:
	//		Create a string from a Date object using a known localized pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the addCustomFormats method.
	//		Formatting patterns are implemented using the syntax described at
	//		http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.
	//
	// options: object {selector: string, formatLength: string, datePattern: string, timePattern: string, locale: string}
	//		selector- choice of 'time','date' (default: date and time)
	//		formatLength- choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//		datePattern,timePattern- override pattern with this string
	//		am,pm- override strings for am/pm in times
	//		locale- override the locale used to determine formatting rules

	options = options || {};

	var locale = dojo.i18n.normalizeLocale(options.locale);
	var formatLength = options.formatLength || 'short';
	var bundle = dojo.date.locale._getGregorianBundle(locale);
	var str = [];
	var sauce = dojo.hitch(this, formatPattern, dateObject, bundle);
	if(options.selector == "year"){
		// Special case as this is not yet driven by CLDR data
		var year = dateObject.getFullYear();
		if(locale.match(/^zh|^ja/)){
			year += "\u5E74";
		}
		return year;
	}
	if(options.selector != "time"){
		var datePattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(datePattern){str.push(_processPattern(datePattern, sauce));}
	}
	if(options.selector != "date"){
		var timePattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(timePattern){str.push(_processPattern(timePattern, sauce));}
	}
	var result = str.join(" "); //TODO: use locale-specific pattern to assemble date + time
	return result; // String
};

dojo.date.locale.regexp = function(/*Object?*/options){
	// summary:
	//		Builds the regular needed to parse a localized date
	//
	// options: object {selector: string, formatLength: string, datePattern: string, timePattern: string, locale: string, strict: boolean}
	//		selector- choice of 'time', 'date' (default: date and time)
	//		formatLength- choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//		datePattern,timePattern- override pattern with this string
	//		locale- override the locale used to determine formatting rules

	return dojo.date.locale._parseInfo(options).regexp; // String
};

dojo.date.locale._parseInfo = function(/*Object?*/options){
	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojo.date.locale._getGregorianBundle(locale);
	var formatLength = options.formatLength || 'short';
	var datePattern = options.datePattern || bundle["dateFormat-" + formatLength];
	var timePattern = options.timePattern || bundle["timeFormat-" + formatLength];
	var pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = datePattern + ' ' + timePattern; //TODO: use locale-specific pattern to assemble date + time
	}

	var tokens = [];
	var re = _processPattern(pattern, dojo.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};

dojo.date.locale.parse = function(/*String*/value, /*Object?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Date object,
	//		using locale-specific settings.
	//
	// description:
	//		Create a Date object from a string using a known localized pattern.
	//		By default, this method parses looking for both date and time in the string.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the addCustomFormats method.
	//		Formatting patterns are implemented using the syntax described at
	//		http://www.unicode.org/reports/tr35/#Date_Format_Patterns
	//
	// value:
	//		A string representation of a date
	//
	// options: object {selector: string, formatLength: string, datePattern: string, timePattern: string, locale: string, strict: boolean}
	//		selector- choice of 'time', 'date' (default: date and time)
	//		formatLength- choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//		datePattern,timePattern- override pattern with this string
	//		am,pm- override strings for am/pm in times
	//		locale- override the locale used to determine formatting rules
	//		strict- strict parsing, off by default

	var info = dojo.date.locale._parseInfo(options);
	var tokens = info.tokens, bundle = info.bundle;
	var re = new RegExp("^" + info.regexp + "$");
	var match = re.exec(value);
	if(!match){ return null; } // null

	var widthList = ['abbr', 'wide', 'narrow'];
	//1972 is a leap year.  We want to avoid Feb 29 rolling over into Mar 1,
	//in the cases where the year is parsed after the month and day.
	var result = new Date(1972, 0);
	var expected = {};
	var amPm = "";
	dojo.forEach(match, function(v, i){
		if(!i){return;}
		var token=tokens[i-1];
		var l=token.length;
		switch(token.charAt(0)){
			case 'y':
				if(l != 2){
					//interpret year literally, so '5' would be 5 A.D.
					result.setFullYear(v);
					expected.year = v;
				}else{
					if(v<100){
						v = Number(v);
						//choose century to apply, according to a sliding window
						//of 80 years before and 20 years after present year
						var year = '' + new Date().getFullYear();
						var century = year.substring(0, 2) * 100;
						var yearPart = Number(year.substring(2, 4));
						var cutoff = Math.min(yearPart + 20, 99);
						var num = (v < cutoff) ? century + v : century - 100 + v;
						result.setFullYear(num);
						expected.year = num;
					}else{
						//we expected 2 digits and got more...
						if(options.strict){
							return null;
						}
						//interpret literally, so '150' would be 150 A.D.
						//also tolerate '1950', if 'yyyy' input passed to 'yy' format
						result.setFullYear(v);
						expected.year = v;
					}
				}
				break;
			case 'M':
				if(l>2){
					var months = bundle['months-format-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = dojo.map(months, function(s){ return s.replace(".","").toLowerCase(); } );
					}
					v = dojo.indexOf(months, v);
					if(v == -1){
//						console.debug("dojo.date.locale.parse: Could not parse month name: '" + v + "'.");
						return null;
					}
				}else{
					v--;
				}
				result.setMonth(v);
				expected.month = v;
				break;
			case 'E':
			case 'e':
				var days = bundle['days-format-' + widthList[l-3]].concat();
				if(!options.strict){
					//Case-insensitive comparison
					v = v.toLowerCase();
					days = dojo.map(days, "".toLowerCase);
				}
				v = dojo.indexOf(days, v);
				if(v == -1){
//					console.debug("dojo.date.locale.parse: Could not parse weekday name: '" + v + "'.");
					return null;
				}

				//TODO: not sure what to actually do with this input,
				//in terms of setting something on the Date obj...?
				//without more context, can't affect the actual date
				//TODO: just validate?
				break;
			case 'd':
				result.setDate(v);
				expected.date = v;
				break;
			case 'D':
				//FIXME: need to defer this until after the year is set for leap-year?
				result.setMonth(0);
				result.setDate(v);
				break;
			case 'a': //am/pm
				var am = options.am || bundle.am;
				var pm = options.pm || bundle.pm;
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
//					console.debug("dojo.date.locale.parse: Could not parse am/pm part.");
					return null;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v==24){v=0;}
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//TODO: strict bounds checking, padding
				if(v > 23){
//					console.debug("dojo.date.locale.parse: Illegal hours value");
					return null;
				}

				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result.setHours(v);
				break;
			case 'm': //minutes
				result.setMinutes(v);
				break;
			case 's': //seconds
				result.setSeconds(v);
				break;
			case 'S': //milliseconds
				result.setMilliseconds(v);
//				break;
//			case 'w':
//TODO				var firstDay = 0;
//			default:
//TODO: throw?
//				console.debug("dojo.date.locale.parse: unsupported pattern char=" + token.charAt(0));
		}
	});

	var hours = result.getHours();
	if(amPm === 'p' && hours < 12){
		result.setHours(hours + 12); //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result.setHours(0); //12am -> 0
	}

	//validate parse date fields versus input date fields
	if(expected.year && result.getFullYear() != expected.year){
//		console.debug("dojo.date.locale.parse: Parsed year: '" + result.getFullYear() + "' did not match input year: '" + expected.year + "'.");
		return null;
	}
	if(expected.month && result.getMonth() != expected.month){
//		console.debug("dojo.date.locale.parse: Parsed month: '" + result.getMonth() + "' did not match input month: '" + expected.month + "'.");
		return null;
	}
	if(expected.date && result.getDate() != expected.date){
//		console.debug("dojo.date.locale.parse: Parsed day of month: '" + result.getDate() + "' did not match input day of month: '" + expected.date + "'.");
		return null;
	}

	//TODO: implement a getWeekday() method in order to test 
	//validity of input strings containing 'EEE' or 'EEEE'...
	return result; // Date
};

function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings) 
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g); 
	var literal = false;

	dojo.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk);
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE(tokens, bundle, options, pattern){
	pattern = dojo.regexp.escapeString(pattern);
	if(!options.strict){ pattern = pattern.replace(" a", " ?a"); } // kludge to tolerate no space before am/pm
	return pattern.replace(/([a-z])\1*/ig, function(match){
		// Build a simple regexp.  Avoid captures, which would ruin the tokens list
		var s;
		var c = match.charAt(0);
		var l = match.length;
		var p2 = '', p3 = '';
		if(options.strict){
			if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
			if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
		}else{
			p2 = '0?'; p3 = '0{0,2}';
		}
		switch(c){
			case 'y':
				s = '\\d{2,4}';
				break;
			case 'M':
				s = (l>2) ? '\\S+' : p2+'[1-9]|1[0-2]';
				break;
			case 'D':
				s = p2+'[1-9]|'+p3+'[1-9][0-9]|[12][0-9][0-9]|3[0-5][0-9]|36[0-6]';
				break;
			case 'd':
				s = p2+'[1-9]|[12]\\d|3[01]';
				break;
			case 'w':
				s = p2+'[1-9]|[1-4][0-9]|5[0-3]';
				break;
		    case 'E':
				s = '\\S+';
				break;
			case 'h': //hour (1-12)
				s = p2+'[1-9]|1[0-2]';
				break;
			case 'k': //hour (0-11)
				s = p2+'\\d|1[01]';
				break;
			case 'H': //hour (0-23)
				s = p2+'\\d|1\\d|2[0-3]';
				break;
			case 'K': //hour (1-24)
				s = p2+'[1-9]|1\\d|2[0-4]';
				break;
			case 'm':
			case 's':
				s = '[0-5]\\d';
				break;
			case 'S':
				s = '\\d{'+l+'}';
				break;
			case 'a':
				var am = options.am || bundle.am || 'AM';
				var pm = options.pm || bundle.pm || 'PM';
				if(options.strict){
					s = am + '|' + pm;
				}else{
					s = am + '|' + pm;
					if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
					if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
				}
				break;
			default:
			// case 'v':
			// case 'z':
			// case 'Z':
				s = ".*";
//				console.debug("parse of date format, pattern=" + pattern);
		}

		if(tokens){ tokens.push(match); }

		return "(" + s + ")"; // add capture
	}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE.
}
})();

(function(){
var _customFormats = [];
dojo.date.locale.addCustomFormats = function(/*String*/packageName, /*String*/bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo for the CLDR data: dateFormat-xxxx / timeFormat-xxxx
	//		The pattern string should match the format used by the CLDR.
	//		See dojo.date.format for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

dojo.date.locale._getGregorianBundle = function(/*String*/locale){
	var gregorian = {};
	dojo.forEach(_customFormats, function(desc){
		var bundle = dojo.i18n.getLocalization(desc.pkg, desc.name, locale);
		gregorian = dojo.mixin(gregorian, bundle);
	}, this);
	return gregorian; /*Object*/
};
})();

dojo.date.locale.addCustomFormats("dojo.cldr","gregorian");

dojo.date.locale.getNames = function(/*String*/item, /*String*/type, /*String?*/use, /*String?*/locale){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item: 'months' || 'days'
	// type: 'wide' || 'narrow' || 'abbr' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// use: 'standAlone' || 'format' (default)
	// locale: override locale used to find the names

	var label;
	var lookup = dojo.date.locale._getGregorianBundle(locale);
	var props = [item, use, type];
	if(use == 'standAlone'){
		label = lookup[props.join('-')];
	}
	props[1] = 'format';

	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};

dojo.date.locale.isWeekend = function(/*Date?*/dateObject, /*String?*/locale){
	// summary:
	//	Determines if the date falls on a weekend, according to local custom.

	var weekend = dojo.cldr.supplemental.getWeekend(locale);
	var day = (dateObject || new Date()).getDay();
	if(weekend.end < weekend.start){
		weekend.end += 7;
		if(day < weekend.start){ day += 7; }
	}
	return day >= weekend.start && day <= weekend.end; // Boolean
};

// These are used only by format and strftime.  Do they need to be public?  Which module should they go in?

dojo.date.locale._getDayOfYear = function(/*Date*/dateObject){
	// summary: gets the day of the year as represented by dateObject
	return dojo.date.difference(new Date(dateObject.getFullYear(), 0, 1), dateObject) + 1; // Number
};

dojo.date.locale._getWeekOfYear = function(/*Date*/dateObject, /*Number*/firstDayOfWeek){
	if(arguments.length == 1){ firstDayOfWeek = 0; } // Sunday

	var firstDayOfYear = new Date(dateObject.getFullYear(), 0, 1).getDay();
	var adj = (firstDayOfYear - firstDayOfWeek + 7) % 7;
	var week = Math.floor((dojo.date.locale._getDayOfYear(dateObject) + adj - 1) / 7);

	// if year starts on the specified day, start counting weeks at 1
	if(firstDayOfYear == firstDayOfWeek){ week++; }

	return week; // Number
};

}

if(!dojo._hasResource["hub.data.atommodel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.data.atommodel"] = true;
dojo.provide("hub.data.atommodel");


dojo.declare('hub.data.atommodel.TableModel', hub.table.model.ItemizedModel, {

	pageSize: 20,
	totalRecords: null,
	initialSorting: "title",
	supportedSortings: ["title", "updated", "rating", "author", "id"],
	availablePages: [],
	currentPage: null,
	collection: "all",
	term: "*",
	searchType: "",
	
	constructor: function(args, node) {
		availablePages = new Array();
		var sorting = new hub.table.model.Sorting();
		sorting.by = "title";
		sorting.asc = true;
		this.setSortingToApply(sorting);
	},
	
	_getInitialUrl: function() {
		var url = hub.urls.getCatalog(this.collection, this.term, this.searchType, null, null);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
        }
        var map = dojo.queryToObject(query);
        map["nologin"] = "true";
        if (this.pageSize != null) {
        	map["count"] = "" + this.pageSize;
        }
        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
        return url;
	},

	getSupportedSortings: function() {
		var sortings = new Array();
		var i;
		for(i = 0; i < this.supportedSortings.length; i++) {
			var s = new hub.table.model.Sorting();
			s.by = this.supportedSortings[i];
			s.asc = true;
			sortings.push(s);
			s = new hub.table.model.Sorting();
			s.by = this.supportedSortings[i];
			s.asc = false;
			sortings.push(s);
		}
		return sortings;
	},
	
	getGuaranteedRowCount: function() {
		return this.totalRecords;
	},
	
	hasMoreRecords: function() {
		return false;
	},
	
	getCurrentPage: function() {
		return this.currentPage;
	},
	
	getCurrentPageSize: function() {
		if (this.getCurrentPage() == null) {
			return null;
		}
		return this.getCurrentPage().size;
	},	

	getAvailablePages: function() {
		return this.availablePages;
	},
	
	_mixinSorting: function(url, sorting) {
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
        }
        var map = dojo.queryToObject(query);
        if (sorting != null) {
        	map["sortBy"] = sorting.by;
        }
        if (sorting != null) {
        	map["sortOrder"] = sorting.asc ? "asc" : "desc";
        }
        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
        console.debug("Complete URL with sorting = " + url);
        return url;
	},
	
	fetch: function() {
		var url = null;
		var sorting = this.getSortingToApply();
		var pageToLoad = this.getPageToLoad();
		if (pageToLoad == null) {
			url = this._getInitialUrl();
		}
		else {
			url = pageToLoad.url;
		}
		url = this._mixinSorting(url, sorting);
        var source = new dojox.data.XmlStore({'url': url});
        var __this = this;
		this.onBeforeFetched();
		var lock = this.onPageBegin();
        var handle = {
        	'startIndex': 0,
        	'itemsPerPage': 0,
        	'url_used': url,
        	'model': __this,
        	'source': source,
        	'sorting': sorting,
        	'rows': new Array(),
        	'prev_url': null,
        	'next_url': null,
        	'lock': lock,
        	
			__onBegin: function() {
			},
			
			__onComplete: function() {
				var i;
				for(i = 0; i < this.rows.length; i++) {
					this.model.onItem(this.lock, this.rows[i]);
				}
				if (this.model.onPageEnd(this.lock, false)) {
					this.model.currentSorting = this.sorting;
					var pages = new Array();
					var currentPosition = null;
					if (this.prev_link == null) {
						currentPosition = 0;
					}
					else if (this.next_link == null) {
						currentPosition = -1;
					}
					
					var page = new hub.table.model.Page();
					page.url = this.url_used;
					if (this.prev_link == null) {
						page.forwardAbsolute = 0;
					}
					if (this.next_link == null) {
						page.backwardAbsolute = 0;
					}
					page.relative = 0;
					page.fromRow = this.startIndex;
					page.toRow = this.startIndex + this.rows.length;
					page.size = this.rows.length;
					if (this.prev_url != null) {
						var prev_page = new hub.table.model.Page();
						prev_page.url = this.prev_url;
						if (page.backwardAbsolute != null) {
							prev_page.backwardAbsolute = page.backwardAbsolute + 1; 
						}
						if (page.forwardAbsolute != null) {
							prev_page.forwardAbsolute = page.forwardAbsolute - 1; 
						}
						prev_page.relative = -1;
						pages.push(prev_page);
					}
					pages.push(page);
					if (this.next_url != null) {
						var next_page = new hub.table.model.Page();
						next_page.url = this.next_url;
						if (page.backwardAbsolute != null) {
							next_page.backwardAbsolute = page.backwardAbsolute - 1; 
						}
						if (page.forwardAbsolute != null) {
							next_page.forwardAbsolute = page.forwardAbsolute + 1; 
						}
						next_page.relative = +1;
						pages.push(next_page);
					}
					this.model.availablePages = pages;
					this.model.currentPage = page;
					this.model.totalRecords = this.totalResults;
					this.model.pageToLoad = page;
					this.model.onFetched();
				}
			},
		
			__onItem: function(item, request) {
		        var dict = {};
		        var value;
		        var tag = this.source.getValue(item, 'tagName')
		        if(tag == 'os:startIndex') {
		            this.startIndex = parseInt(this.source.getValue(item, 'text()'));
		        } else if(tag == 'os:itemsPerPage') {
		            this.itemsPerPage = parseInt(this.source.getValue(item, 'text()'));
		        } else if(tag == 'os:totalResults') {
		            this.totalResults = parseInt(this.source.getValue(item, 'text()'));
		        } else if(tag == 'link') {
		            var rel = this.source.getValue(item, '@rel');
		            var href = this.source.getValue(item, '@href');
		            if(rel == 'next') {
		                this.next_url = href;
		            } else if (rel == 'previous') {
		                this.prev_url = href;
		            }
		        } else if(tag == 'entry') {
		            var entryCategoryElement = this.source.getValue(item, "category");
		            var category = this.source.getValue(entryCategoryElement, "@term");
		            dict["entryCategory"] = category;
		            var content = this.source.getValue(item, "content");
		            
		            var cat = this.source.getValue(content, "childNodes");
		            
		            var atts = this.source.getValues(cat, "childNodes");
		            
		            for(var i=0; i < atts.length; i++) {
		                var node = atts[i];
		                if(this.source.isItem(node)) {
		                    if(this.source.hasAttribute(node, "catalog:tag")) {
		                        value = this.source.getValues(node, "catalog:tag");
		                        value = this._parseArray(value);
		                    } else if (this.source.hasAttribute(node, "catalog:category")) {
		                        value = this.source.getValues(node, "catalog:category");
		                        value = this._parseArray(value);
		                    }
		                    else {
		                        value = this.source.getValue(node, "text()");
		                    }
		                }
		                
		                var key = this.source.getValue(atts[i], 'tagName');
		                if (key.lastIndexOf(":") > 0)
		                	key = key.substring(key.lastIndexOf(":") + 1);
		                	
						dict[key] = value;                
		            }
		            
		            var idfield = this.source.getValue(this.source.getValue(item, "id"), "text()");
		            var unprefixedId = idfield;
		            if (idfield.indexOf("urn:uuid:") == 0) {
		            	unprefixedId = idfield.substring("urn:uuid:".length);
		            }
		            if (idfield.indexOf("urn:id:") == 0) {
		            	unprefixedId = idfield.substring("urn:id:".length);
					}
		            var pureId = unprefixedId;
		            if (unprefixedId.lastIndexOf('-') >=0) {
		            	pureId = unprefixedId.substring(unprefixedId.lastIndexOf('-') + 1);
		            }
		            dict['objectid'] = pureId;
		            dict['id'] = pureId;
		            this.rows.push(dict);
		        }
			},
			
			__onError: function(error) {
				if (this.lock == null) {
					this.model.onBeforeFetched();
					this.lock = this.model.onPageBegin();
				}
				if (this.model.onPageEnd(this.lock, true)) { // there is a error
					this.model.onFetchError(error);
				}
			},
			
		    /**
		     * Pulls an array of strings out of dojo data XML values.
		     *
		     * @param values Dojo data items having the values we want
		     */
		    _parseArray: function(values) {
		        var arr = new Array();
		        for(i in values) {
		            arr.push(this.source.getValue(values[i], "text()"));
		        }
		        return arr;
		    }			
        };
        source.fetch(
            {'onBegin' : dojo.hitch(handle, handle.__onBegin),
             'onItem' : dojo.hitch(handle, handle.__onItem),
             'onComplete' :dojo.hitch(handle, handle.__onComplete),
             'onError' : dojo.hitch(handle, handle.__onError)});
	},
	
	onFetched: function() {
	},

	onBeforeFetched: function() {
	},
	
	onFetchError: function(error) {
	}
});

}

if(!dojo._hasResource['hub.widgets.adminobjectscatalog']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.widgets.adminobjectscatalog'] = true;
dojo.provide('hub.widgets.adminobjectscatalog');













/**
 * Displays an array of values keyed by column names in a table.
 */
dojo.declare("hub.widgets.adminobjectscatalog.Catalog",  [dijit._Widget, dijit._Templated], {
    // @todo: do we need to do this for build?
    templateString: '',
    // template to replace node with
    templateString:"<div>\r\n   \t<div class='TOM_catalog_SearchBox'><input dojoAttachPoint='searchTerm' dojoType=\"dijit.form.TextBox\" name=\"searchTerm\" style=\" width:218px; padding: 0px; height: 18px;\" type=\"text\" value=\"\"/>\r\n   \t\t<button type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick: _onSearch\"></button>\r\n   \t</div>\r\n   \t<div dojoType='hub.table.table.Table' dojoAttachPoint='table' defaultSelectionArea=\"default\" defaultSelectionAreaMode=\"2\">\r\n   \t</div>\r\n   \t<div dojoType='hub.table.navigation.PrevNextNavigation' dojoAttachPoint='navigation'>\r\n   \t</div>\r\n</div>\r\n    \r\n",
    widgetsInTemplate: true,

    // hub.data.catalog._Model widgetid
    model: '',
    // force a refresh on the model immediately
    forceRefresh: false,

    constructor: function(args, node) {
    },

    postMixInProperties: function() {
    	this.inherited(arguments);
    	this.model = new hub.data.atommodel.TableModel();
    	this.model.pageSize = 10;
    },

    postCreate: function() {
    	this.inherited(arguments);
    },

    startup: function() {
    	this.inherited(arguments);
    	
    	var panels = dojo.i18n.getLocalization('hub', 'panels');
    	
    	this.table.addColumn("id", panels.ADMIN_CATALOG_TABLE_ID_LABEL, 'id', function(element, row) {
          element.innerHTML = row.objectid;
      });
      this.table.addColumn("name", panels.ADMIN_CATALOG_TABLE_NAME_LABEL, 'title', function(element, row) {
          element.innerHTML = hub.buildPageLink(hub.htmlspecialchars(row.name), 'Details', {object_id: row.objectid}, 
                                                false, null, '');
      });
      this.table.addColumn("desc", panels.ADMIN_CATALOG_TABLE_DESC_LABEL, null, function(element, row) {
          element.innerHTML = hub.htmlspecialchars(row.description);
      });
      this.table.setModel(this.model);
      this.navigation.setModel(this.model);
      this.model.fetch();
    },

    destroy: function() {
    	
    },
    
    getFormData: function() {
    	var selection = this.table.getValueForArea("default");
    	var result = {};
    	var i = 0;
    	for(i = 0; i < selection.length; i++) {
    		var label = selection[i].id;
    		if (selection[i].selected) {
    			result["deletion" + label] = "";
    		}
    	}
    	return result;
    },

	onBeforeFetched: function() {
//	console.log('Catalog.onBeforeFetched()');
	},
     
	_onSearch: function() {
		var value = "";
		if (this.searchTerm) {
			value = this.searchTerm.getValue();
		}
		if (value == "") {
			value = "*";
		}
		this.model.term = value;
		this.model.setPageToLoad(null);
		this.model.fetch();
	}
});

dojo.declare("hub.widgets.adminobjectscatalog.RelatedSelection", hub.table.model.DataSelection, {
	container: null,
	isDataSelected: function(row, area) {
//		console.log("RelatedSelection.isDataSelected");
		var result = false;
		var ids = this.container.selected_ids;
		for(var i = 0; i < ids.length; i++){
			if(ids[i] == row.id){
				result = true;
				break;
			}
		} 
//		console.dir({result: result, rowId: row.id, ids:this.container.selected_ids});
		return result; 
	}
});

dojo.declare("hub.widgets.adminobjectscatalog.CategoriesSelection", hub.table.model.DataSelection, {
	container: null,
	isDataSelected: function(row, area) {
		if ("isChecked" in row) {
			return (false != row.isChecked);
		}
		return false;
	}
});

dojo.declare("hub.widgets.adminobjectscatalog.Related",  [hub.widgets.adminobjectscatalog.Catalog], {
	selected_ids: new Array(),
    entryId: 0,
    startup: function() {
    },

	onBeforeFetched: function(){
//	console.log('Related.onBeforeFetched');
    	var url = hub.urls.getAssociationsIdsList(this.entryId);
    	var req = new hub.submit.Request(url);
    	req._onLoad = dojo.hitch(this, function(response){
    			response = dojo.fromJson(response);
//    			console.dir({response: response});
    			this.selected_ids = response.ids;
    		});
    	req.submit();
	},

    postCreate: function() {
    	this.inherited(arguments);
    	var panels = dojo.i18n.getLocalization('hub', 'panels');
		this.table.defaultSelectionAreaMode=2;
		this.table.selection = new hub.widgets.adminobjectscatalog.RelatedSelection();
		this.table.selection.container = this;
		this.table.addColumn("name", panels.ADMIN_RELATED_TABLE_RELATED_LABEL, 'title', function(element, row) {
			element.innerHTML = hub.htmlspecialchars(row.name);			
		});
		this.table.setModel(this.model);
		dojo.connect(this.model, 'onBeforeFetched', this, 'onBeforeFetched');
		this.navigation.setModel(this.model);
		this.model.fetch();
    },
    
    
    getFormData: function() {
    	var selection = this.table.getValueForArea("default");
    	var result = {};
    	var i = 0;
    	for(i = 0; i < selection.length; i++) {
    		var label = selection[i].id;
   			result["rel_" + label] = selection[i].selected?"on":"off";
    	}
    	return result;
    }
});


/**
 * Displays an array of values keyed by column names in a table.
 */
dojo.declare("hub.widgets.adminobjectscatalog.Categories",  [dijit._Widget, dijit._Templated], {
    // @todo: do we need to do this for build?
    templateString: '',
    // template to replace node with
    templateString:"<div>\r\n   \t<div dojoType='hub.table.table.Table' dojoAttachPoint='table' defaultSelectionArea=\"default\" defaultSelectionAreaMode=\"2\">\r\n   \t</div>\r\n   \t<div dojoType='hub.table.navigation.PrevNextNavigation' dojoAttachPoint='navigation'>\r\n   \t</div>\r\n</div>\r\n    \r\n",
    widgetsInTemplate: true,

    // hub.data.catalog._Model widgetid
    model: '',
    // force a refresh on the model immediately
    forceRefresh: false,
    
    entryId: 0,

    constructor: function(args, node) {
    },

    postMixInProperties: function() {
    	this.inherited(arguments);
    	this.model = new hub.data.jsonmodel.TableModel({url:hub.urls.getCategoriesList(this.entryId)});
    	this.model.pageSize = 10;
    },

    postCreate: function() {
    	this.inherited(arguments);
    	var panels = dojo.i18n.getLocalization('hub', 'panels');
		this.table.defaultSelectionAreaMode=2;
		this.table.selection = new hub.widgets.adminobjectscatalog.CategoriesSelection();
/*		this.table.selection.isDataSelected =function(row, area) {
				if ("isChecked" in row) {
					return (false != row.isChecked);
				}
				return false;
			}*/
		this.table.addColumn("name", panels.ADMIN_CATEGORIES_TABLE_CATEGORIES_LABEL, 'title', function(element, row) {
			element.innerHTML = hub.htmlspecialchars(row.name);			
		});
		this.table.setModel(this.model);
		this.navigation.setModel(this.model);
		this.model.fetch();
    },

/*    startup: function() {
    console.log('Categories.startup()');
    	this.inherited(arguments);
    },*/

    destroy: function() {
    	
    },
    
    getFormData: function() {
    	var selection = this.table.getValueForArea("default");
    	var result = {};
    	var i = 0;
    	for(i = 0; i < selection.length; i++) {
    		var label = selection[i].id;
   			result["cat_" + label] = selection[i].selected?"on":"off";
    	}
    	return result;
    }
     
});

/**
 * Subclass for object tracking
 */
dojo.declare("hub.widgets.adminobjectscatalog.ObjectTracking",  [hub.widgets.adminobjectscatalog.Catalog], {
    changedRows: null,

    postMixInProperties: function() {
    	this.inherited(arguments);

    	
    	this.model = new hub.data.jsonmodel.TableModel({url: hub.urls.getAdminEditTrackingPage(), pageSize: 10});
    },

    startup: function() {
      this.changedRows = [];

      // no selection
      this.table.setDefaultSelectionArea(null);

      var panels = dojo.i18n.getLocalization('hub', 'panels');

			this.table.addColumn("id", panels.ADMIN_TRACKING_SETTINGS_TABLE_COL1_LABEL, 'id', function(element, row) {
          element.innerHTML = row.id;
      });
			this.table.addColumn("title", panels.ADMIN_TRACKING_SETTINGS_TABLE_COL2_LABEL, 'title', function(element, row) {
          element.innerHTML = hub.buildPageLink(hub.htmlspecialchars(row.title), 'Details', {object_id: row.id}, 
                                                false, null, '');
      });

      var myThis = this;
      this.table.addColumn("isTrackable", panels.ADMIN_TRACKING_SETTINGS_TABLE_COL3_LABEL, 'isTrackable', function(element, row) {
          var input = document.createElement("input");
          input.setAttribute("type", "checkbox");
          input.setAttribute("objecttype", "isTrackable");

          input.setAttribute("objectid", row.id);

          if (row.isTrackable) {
            input.setAttribute("checked", "true");
          }

          dojo.connect(input, "onchange", myThis, "onCheckboxClick");

          element.appendChild(input);
      });
      this.table.addColumn("isPerm", panels.ADMIN_TRACKING_SETTINGS_TABLE_COL4_LABEL, 'isPerm', function(element, row) {
          var input = document.createElement("input");
          input.setAttribute("type", "checkbox");
          input.setAttribute("objecttype", "isPerm");

          input.setAttribute("objectid", row.id);

          if (row.isPerm) {
            input.setAttribute("checked", "true");
          }

          dojo.connect(input, "onchange", myThis, "onCheckboxClick");

          element.appendChild(input);
      });
      this.table.setModel(this.model);
      this.navigation.setModel(this.model);
      this.model.fetch();
    },

    onCheckboxClick: function(event) {
      var target = event.target;
      var id = target.getAttribute("objectid");
      var checked = target.checked;
      var type = target.getAttribute("objecttype");

      // update model
      var rows = this.model.rows;
      for (var i = 0; i < rows.length; i++) {
        var row = rows[i];
        if (row.id == id) {
          if (type == "isTrackable") {
            row.isTrackable = checked;
          } else {
            row.isPerm = checked;
          }

          this.changedRows[id] = {isTrackable: row.isTrackable, isPerm: row.isPerm}
          break;
        }
      }
    },

    getFormData: function() {
    	var result = {};

    	for (var id in this.changedRows) {
        var row = this.changedRows[id];
        result["track"+id] = row.isTrackable ? "on" : "off";
        result["perm"+id] = row.isPerm ? "on" : "off";
    	}

    	return result;
    }
});

/**
 * Subclass for object tracking data
 */
dojo.declare("hub.widgets.adminobjectscatalog.ObjectTrackingData",  [hub.widgets.adminobjectscatalog.Catalog], {
    postMixInProperties: function() {
    	this.inherited(arguments);

    	
    	this.model = new hub.data.jsonmodel.TableModel({url: hub.urls.getAdminTrackingData(), pageSize: 10});
    },

    startup: function() {
      // no selection
      this.table.setDefaultSelectionArea(null);

      var panels = dojo.i18n.getLocalization('hub', 'panels');

			this.table.addColumn("title", panels.ADMIN_TRACKED_TABLE_TITLE, 'title', function(element, row) {
          element.innerHTML = "<a href='"+hub.config.base_url+"/tracking/generate/entryid/"+row.id+"' target='_blank'><img src='"+hub.urls.getImage(row.img)+"' border='0' alt='"+ panels.ADMIN_TRACKED_DATA_ICON_ALT+"'></a> " + 
                              hub.buildPageLink(hub.htmlspecialchars(row.title), 'Details', {object_id: row.id}, 
                                                false, null, '');
      });

      this.table.addColumn("hits", panels.ADMIN_TRACKED_TABLE_HITS, 'hits', function(element, row) {
          element.innerHTML = row.hits;
      });

      this.table.addColumn("avg_load", panels.ADMIN_TRACKED_TABLE_AVGRESPONSE, 'avg_load', function(element, row) {
          element.innerHTML = row.avg_load;
      });

      this.table.setModel(this.model);
      this.navigation.setModel(this.model);
      this.model.fetch();
    }
});

}

if(!dojo._hasResource["dojo.cookie"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cookie"] = true;
dojo.provide("dojo.cookie");

/*=====
dojo.__cookieProps = function(kwArgs){
	//	expires: Date|Number?
	//		If a number, seen as the number of days from today. If a date, the
	//		date past which the cookie is invalid. If expires is in the past,
	//		the cookie will be deleted If expires is left out or is 0, the
	//		cookie will expire when the browser closes.
	//	path: String?
	//		The path to use for the cookie.
	//	domain: String?
	//		The domain to use for the cookie.
	//	secure: Boolean?
	//		Whether to only send the cookie on secure connections
}
=====*/


dojo.cookie = function(/*String*/name, /*String?*/value, /*dojo.__cookieProps?*/props){
	//	summary: 
	//		Get or set a cookie.
	//	description:
	// 		If you pass in one argument, the the value of the cookie is returned
	//
	// 		If you pass in two arguments, the cookie value is set to the second
	// 		argument.
	//
	// 		If you pass in three arguments, the cookie value is set to the
	// 		second argument, and the options on the third argument are used for
	// 		extended properties on the cookie
	//	name:
	//		The name of the cookie
	//	value:
	//		Optional. The value for the cookie.
	//	props: 
	//		Optional additional properties for the cookie
	//	example:
	//		set a cookie with the JSON-serialized contents of an object which
	//		will expire 5 days from now:
	//	|	dojo.cookie("configObj", dojo.toJson(config), { expires: 5 });
	//	
	//	example:
	//		de-serialize a cookie back into a JavaScript object:
	//	|	var config = dojo.fromJson(dojo.cookie("configObj"));
	//	
	//	example:
	//		delete a cookie:
	//	|	dojo.cookie("configObj", null);
	var c = document.cookie;
	if(arguments.length == 1){
		var idx = c.lastIndexOf(name+'=');
		if(idx == -1){ return null; }
		var start = idx+name.length+1;
		var end = c.indexOf(';', idx+name.length+1);
		if(end == -1){ end = c.length; }
		return decodeURIComponent(c.substring(start, end)); 
	}else{
		props = props || {};
		value = encodeURIComponent(value);
		if(typeof(props.expires) == "number"){ 
			var d = new Date();
			d.setTime(d.getTime()+(props.expires*24*60*60*1000));
			props.expires = d;
		}
		document.cookie = name + "=" + value 
			+ (props.expires ? "; expires=" + props.expires.toUTCString() : "")
			+ (props.path ? "; path=" + props.path : "")
			+ (props.domain ? "; domain=" + props.domain : "")
			+ (props.secure ? "; secure" : "");
		return null;
	}
};

}

if(!dojo._hasResource['hub.submit']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.submit'] = true;
dojo.provide('hub.submit');



		
		
		
		


dojo.declare('hub.submit.Request', null,{
	content: {},
    constants : {
        NO_CONTENT : -1,
        PAGE_CONTENT : -2,
        MESSAGE_CONTENT : -3,
        ERROR_LOGIN_REQUIRED : 1,
        ERROR_ADMIN_REQUIRED : 2,
        ERROR_BANNED: 16,
        ERROR_ACCESS_DENIED: 18,
        ERROR_DATA_COMP: 21
    },
    
    response: {},
    
    /**
     * @param form: Form or JS object to submit
     * @param status: Message area where error / server messages are posted
     * @param url: Post to this URL, or use it to form a full URL after calling
     *   getUrl()
     */
    constructor : function(url, content ,status) {
        // get reference to status element
        if(content){
        	this.content = content; 
        }
        if(status) {
            if(dojo.isString(status)) {
                this.status = dojo.byId(status);
            } else {
                this.status = status;
            }
        } else {
            this.status = null;
        }
        
        // store something as the URL
        if(url != null) {
            // use provided url
            this.url = url;
        }
    },

    getStatus: function() {
        return this.status;
    },

    getURL : function() {
        return this.url;
    },
    
    getContent : function() {
        return this.content;
    },
    
    /*
     * Submits the Form using dojo.xhrPost. 
     * 
     * @param validate: If false, does not perform validation.
     */
    submit: function(validate) {
        // indicate busy
///		hub.busyPage(this.status, true, false);
	      
		// build a request object and handler for dojo
		var kw = {
		    url: this.getURL(),
		    content: this.getContent(),
            handleAs: 'text', // let the callback decode
            timeout: 120000, // 2 minutes in milliseconds
            sync: true,
            
            error: dojo.hitch(this, function(response, args) {
                // get error messages
                var msgs = dojo.i18n.getLocalization('hub', 'messages');
                
                if(response.dojoType == "cancel") {
                    hub.notify(msgs.ERROR_REQUEST_CANCELLED, this.getStatus());
                } else if (response.dojoType == "timeout"){
                    hub.notify(msgs.ERROR_REQUEST_TIMEOUT, this.getStatus());
                } else {
                    var err = msgs.ERROR_REQUEST_SERVER+" "+response.message;
                    hub.notify(err, this.getStatus());
                }
                // indicate no longer busy
///		        hub.busyPage(this.status, false, false);
                return response;
            }),
            
            load: dojo.hitch(this, function(response, args) {
				try {
                    if(!response) {
                        throw new Error('Form: Empty server response');
                    }
                    if(hub.submit.processResponseCode(response)){
				    	this._onLoad(response);
				    }
				} catch(e) {
                    var msgs = dojo.i18n.getLocalization('hub', 'messages');
                    var err = msgs.ERROR_REQUEST_CLIENT + " " + e.message;
                    hub.notify(err, this.getStatus());
				}
	                // indicate no longer busy
///		        hub.busyPage(this.status, false, false);
                return response;
            })
		};
	        // do the post
		dojo.xhrPost(kw);
		return true;
    },
    
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        this.response = data;
        if (!data) {
            throw new Error('Form: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
//            hub.notify(data.result, this.status, 'TOM_editor_msg');
        }/* else {
           	console.log('submit _onload 2');
            hub.notify(data.result, this.status);
        }*/
        return data.code;
    }
});
/**
 * Defines a base class for form validation, submission using XMLHTTPRequest,
 * notification when the request is complete, and basic handling of result
 * messages. This base class posts all form data and uses hub.notify to output
 * any message returned with a server result.
 */
dojo.declare('hub.submit.Form', null, {
    constants : {
        NO_CONTENT : -1,
        PAGE_CONTENT : -2,
        MESSAGE_CONTENT : -3,
        ERROR_DATA_COMP: 21
    },
    
    content: null,
    form: null,
    
    /**
     * @param form: Form or JS object to submit
     * @param status: Message area where error / server messages are posted
     * @param url: Post to this URL, or use it to form a full URL after calling
     *   getUrl()
     */
    constructor : function(form, status, url) {
        // get form or data object
        if(dojo.isString(form)){
            form = dojo.byId(form);
        }
        this.form = form;
        
        // get reference to status element
        if(status) {
            if(dojo.isString(status)) {
                this.status = dojo.byId(status);
            } else {
                this.status = status;
            }
        } else {
            this.status = null;
        }
        
        // store something as the URL
        if(url != null) {
            // use provided url
            this.url = url;
        } else if(this.form) {
            // if url is not defined, set from action attribute
            this.url = form.action;
        }
    },

    getStatus: function() {
        return this.status;
    },

    /**
     * Do a manual step of converting a form into a JS object. This way,
     * the Form base class can submit both form data and JS objects passed
     * manually.
     */
    getContent: function() {
    	if(this.content!=null){
    		return this.content;
    	}else{
	    	var widgets = dojo.query('div[id^="hub_widgets"]', this.form);
	    	var data = new Array();
	    	for(var i=0; i<widgets.length; i++){
	    		var didget = dijit.byId(widgets[i].id);
	    		if(didget!=null && didget.getFormData!=null && dojo.isFunction(didget.getFormData)){
			    	data = dojo.mixin(data, didget.getFormData());
		    	}
	    	}
    		
	        try {
		 	    return dojo.mixin(data, dojo.formToObject(this.form));
	        } catch(e) {
	            return dojo.mixin(data, this.form);
	        }
        }
        return null;
    },
        
    /*
     * Submits the Form using dojo.xhrPost. 
     * 
     * @param validate: If false, does not perform validation.
     */
    submit: function(validate) {
    console.log('Submit.submit()');
	// make sure the result is valid first
	if(validate && !this.validate()) {
	    hub.busyPage(this.form, false, false);
	    return false;
	}
        // indicate busy
	hub.busyPage(this.form, true, true);
      
	// build a request object and handler for dojo
	console.dir({data:this.getContent()});
	var kw = {
	    url: this.getURL(),
	    content: this.getContent(),
        handleAs: 'text', // let the callback decode
        timeout: 120000, // 2 minutes in milliseconds
        
        error: dojo.hitch(this, function(response, args) {
            // get error messages
            var msgs = dojo.i18n.getLocalization('hub', 'messages');
            
            if(response.dojoType == "cancel") {
                hub.notify(msgs.ERROR_REQUEST_CANCELLED, this.getStatus());
            } else if (response.dojoType == "timeout"){
                hub.notify(msgs.ERROR_REQUEST_TIMEOUT, this.getStatus());
            } else {
                var err = msgs.ERROR_REQUEST_SERVER+" "+response.message;
                hub.notify(err, this.getStatus());
            }
            // indicate no longer busy
		    hub.busyPage(this.form, false, false);
            return response;
        }),
        
        load: dojo.hitch(this, function(response, args) {
			try {
	            if(!response) {
	                throw new Error('Form: Empty server response');
	            }
				if(hub.submit.processResponseCode(response)){
					this._onLoad(response);
				}
			} catch(e) {
	            var msgs = dojo.i18n.getLocalization('hub', 'messages');
	            var err = msgs.ERROR_REQUEST_CLIENT + " " + e.message;
	            hub.notify(err, this.getStatus());
			}
 	        // indicate no longer busy
		    hub.busyPage(this.form, false, false);
            return response;
       })
	};
        // do the post
	dojo.xhrPost(kw);
	return true;
    },
          
    /**
     * Validates a required form field to ensure it is not blank, null, or
     * contains only whitespace. Gives the field focus if it is invalid.
     * 
     * @param field Form element to validate
     * @param text Notify the user with this text if invalid
     */
    _validateRequired : function(field, text) {
        with (field) {
	    if ((value == null) || (value == "")) {
	        hub.notify(text, this.status);
                field.focus();
	        return false;
	    }
	    value = value.replace(/^\s+|\s+$/, '');
	    if(value.length < 1) {
	        hub.notify(text, this.status);
	        return false;
	    }
	    return true;
        }
    },
    
    /**
     * Validates a field having a maximum and minimum length.
     * 
     * @param field Form element to validate
     * @param max Maximum required length, null if none
     * @param min Minimum required length, null if none
     * @param text Notify the user with this text if invalid
     */
    _validateLength : function(field, min, max, text) {
        with (field) {
	    value = value.replace(/^\s+|\s+$/, '');
	    if(max != null && value.length > max) {
                field.focus();
                hub.notify(text, this.status);
	        return false;
	    }
	    if(min != null && value.length < min) {
                field.focus();
                hub.notify(text, this.status);
	        return false;
	    }
        }
        return true;
    },
    
    getURL : function() {
        return this.url;
    },
    
    validate : function() {
        return true;
    },
    
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('Form: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});


/**
 * Submits data required to login.
 */
dojo.declare('hub.submit.Login', hub.submit.Form, {
    constructor : function(form, message_id, url, from_page) {
        this.from_page = from_page;
    },
    getURL : function() {
        return hub.urls.postLogin();
    },
    
    validate: function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
	    if (!this._validateRequired(login,
                msgs.ERROR_LOGIN_LOGIN, this.status)) {
	        return false;
	    }
	    if (!this._validateRequired(password,
                msgs.ERROR_LOGIN_PASSWORD, this.status)) {
	        return false;
	    }
        }
        return true;
    },

    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Login: empty response');
        } else if(data.code < 0) {
            // welcome the user
            var username = data.result;

            if(this.from_page) {
                // close the login page, if open
                hub.closePage(this.form);
            }

            // notify other components
            dojo.publish("hub.login", [username]);
            
            // clear any previous error
            hub.notify("", this.status);
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});


/**
 * Submits a new user registration.
 */
dojo.declare('hub.submit.Register', hub.submit.Form, {
    getURL : function() {
        return hub.urls.postRegister();
    },
    
    validate : function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
	    if (!this._validateRequired(registerName,
                msgs.ERROR_REGISTER_NAME, this.status)) {
	        return false;
	    }
	    if (!this._validateRequired(registerEmail,
                msgs.ERROR_REGISTER_EMAIL, this.status)) {
	        return false;
	    }
	    if (!this._validateRequired(registerLogin,
                msgs.ERROR_REGISTER_LOGIN, this.status)) {
	        return false;
	    }
	    if (!this._validateRequired(registerPassword,
                msgs.ERROR_REGISTER_PASSWORD, this.status)) {
	        return false;
	    }
            if (registerPassword.value != registerPassword2.value) {
                hub.notify(msgs.ERROR_REGISTER_PASSWORD_MATCH, this.status);
                registerPassword.value = "";
                registerPassword2.value = "";
                registerPassword.focus();
                return false;
            }
        }
        return true;
    },
    
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Register: empty response');
        } else if(data.code < 0) {
            // welcome the user
            var username = data.result;
            // close the registration page
            hub.closePage(this.form);
            // notify other components
            dojo.publish('hub.login', [username]);
        } else {	
            hub.notify(data.result, this.status);
        }
        return data.code;
    }   
});


/**
 * Submits a request to add or remove an entry as a favorite
 */
dojo.declare('hub.submit.Favorites', hub.submit.Favorites, {
    // URL will be provided by caller, not hardcoded
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Favorites: empty response');
        } else if (data.code < 0) {
            // success, switch the label
            this.status.innerHTML = data.result;
            // notify other components
            dojo.publish('hub.update-favorites');
        } else {
            // IMPORTANT: purposely not giving the message field since
            // it's pointing at the link, not at an error area; always
            // use the popup alert
            hub.notify(data.result, null);
        }
        return data.code;
    }
});

/**
 * Submits a request to logout.
 */
dojo.declare('hub.submit.Logout', hub.submit.Form, {
    validate: function() {
        // try to close all pages first
        var d = hub.closeAllPages();
        var _self = this;
        // listen to deferred result to continue the submit
        d.addCallback(function(success) {
            if(success) {
                // resend this form without validating again
                hub.submit.ajaxSubmit(_self, false, true);
            } else {
                // @todo: seems like we should cleanup here, form or deferred?
            }
        });
        // never succeed right away, reinvoke submit later if all pages closed
        return false;
    },
    
    getURL : function() {
        return hub.urls.postLogout();
    },
    
    _onLoad : function(response, tab) {
        
        // alphaWorks support code
        // redirect to alphaWorks main site after logout
        if (dojo.cookie('IBM_WI_Username') != null) {
	    dojo.cookie('IBM_WI_Username', '', {expires: -1});
	    dojo.cookie('IBM_WI_Password', '', {expires: -1});
//	    window.location='http://aws01.alphaworks.ibm.com/mhrun/WSLogin.php';
	    window.location='/';
	}
        
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Logout: empty response');
        } else if(data.code < 0) {
            // notify components that logout completed
            dojo.publish('hub.logout');
        } else {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        }
        return data.code;
    }
});

hub.submit.processResponseCode = function(response){
//	console.log('hub.submit.processResponseCode: ');
    var constants = {
        ERROR_UNKNOWN: 0,
        NO_CONTENT : -1,
        PAGE_CONTENT : -2,
        MESSAGE_CONTENT : -3,
        ERROR_LOGIN_REQUIRED : 1,
        ERROR_ADMIN_REQUIRED : 2,
        ERROR_BANNED: 16,
        ERROR_ACCESS_DENIED: 18,
        ERROR_DATA_COMP: 21
    }
    var data = dojo.isString(response)?dojo.fromJson(response):response;
//	console.dir({response: data});
    var code = data.code;
    var result = true;
    if(code != null){
	   	if(dojo.isString(code)){
	   		code = parseInt(code); 
	   	}
	   	result = true;
	   	switch(code){
			case constants.ERROR_LOGIN_REQUIRED :
				hub.showPage('Login', 'LoginPlease', {current: true});
				result = false;
				break; 
			case constants.ERROR_ADMIN_REQUIRED :
			//	todo: processing
				console.warn('missing error processing')
				break; 
				result = false;
			case constants.ERROR_BANNED:
			//	todo: processing
				console.warn('missing error processing')
				result = false;
				break;
			case constants.ERROR_DATA_COMP:
				hub.notify('Data error on server side');
				result = false;
				break;
			case constants.ERROR_ACCESS_DENIED:
			    hub.notify('Access Denied');
			    result = false;
			    break;		
			case constants.ERROR_UNKNOWN :
				hub.notify('Unknown error on server side');
				result = true;
				break; 
			case constants.NO_CONTENT : 
			case constants.PAGE_CONTENT : 
			case constants.MESSAGE_CONTENT :
			default:
				break;
	   	}
   	}
   	return result;
};

/**
 * Simple helper function that invokes submit on the given form object and
 * either returns a value of false or undefined to prevent a refresh of the 
 * entire browser when used on an HTML form or HTML link respectively.
 *
 * @param hub.submit.Form: Form object to do validation and handle the response
 * @param no_result: If true, returns undefined instead of false; useful for 
 *   links
 * @param no_validate: If true, skips validation
 */
hub.submit.ajaxSubmit = function(form, no_result, no_validate) {
    try {
        form.submit(!no_validate);
    } catch (e) {
        hub.notify(e, form.getStatus());
    }
    return no_result ? undefined : false;
}

}

if(!dojo._hasResource["dojo.number"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.number"] = true;
dojo.provide("dojo.number");







/*=====
dojo.number.__formatOptions = function(kwArgs){
	//	pattern: String?
	//		override formatting pattern with this string (see
	//		dojo.number._applyPattern)
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific, percent, currency. decimal by default.
	//	places: Number?
	//		fixed number of decimal places to show.  This overrides any
	//		information in the provided pattern.
	//	round: NUmber?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means don't round.
	//	currency: String?
	//		iso4217 currency code
	//	symbol: String?
	//		localized currency symbol
	//	locale: String?
	//		override the locale used to determine formatting rules
}
=====*/

dojo.number.format = function(/*Number*/value, /*dojo.number.__formatOptions?*/options){
	// summary:
	//		Format a Number as a String, using locale-specific settings
	// description:
	//		Create a string from a Number using a known localized pattern.
	//		Formatting patterns appropriate to the locale are chosen from the
	//		CLDR http://unicode.org/cldr as well as the appropriate symbols and
	//		delimiters.  See http://www.unicode.org/reports/tr35/#Number_Elements
	// value:
	//		the number to be formatted.  If not a valid JavaScript number,
	//		return null.

	options = dojo.mixin({}, options || {});
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojo.i18n.getLocalization("dojo.cldr", "number", locale);
	options.customs = bundle;
	var pattern = options.pattern || bundle[(options.type || "decimal") + "Format"];
	if(isNaN(value)){ return null; } // null
	return dojo.number._applyPattern(value, pattern, options); // String
};

//dojo.number._numberPatternRE = /(?:[#0]*,?)*[#0](?:\.0*#*)?/; // not precise, but good enough
dojo.number._numberPatternRE = /[#0,]*[#0](?:\.0*#*)?/; // not precise, but good enough

dojo.number._applyPattern = function(/*Number*/value, /*String*/pattern, /*dojo.number.__formatOptions?*/options){
	// summary:
	//		Apply pattern to format value as a string using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted.
	// pattern:
	//		a pattern string as described in
	//		http://www.unicode.org/reports/tr35/#Number_Format_Patterns
	// options: dojo.number.__formatOptions?
	//		_applyPattern is usually called via dojo.number.format() which
	//		populates an extra property in the options parameter, "customs".
	//		The customs object specifies group and decimal parameters if set.

	//TODO: support escapes
	options = options || {};
	var group = options.customs.group;
	var decimal = options.customs.decimal;

	var patternList = pattern.split(';');
	var positivePattern = patternList[0];
	pattern = patternList[(value < 0) ? 1 : 0] || ("-" + positivePattern);

	//TODO: only test against unescaped
	if(pattern.indexOf('%') != -1){
		value *= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		value *= 1000; // per mille
	}else if(pattern.indexOf('\u00a4') != -1){
		group = options.customs.currencyGroup || group;//mixins instead?
		decimal = options.customs.currencyDecimal || decimal;// Should these be mixins instead?
		pattern = pattern.replace(/\u00a4{1,3}/, function(match){
			var prop = ["symbol", "currency", "displayName"][match.length-1];
			return options[prop] || options.currency || "";
		});
	}else if(pattern.indexOf('E') != -1){
		throw new Error("exponential notation not supported");
	}
	
	//TODO: support @ sig figs?
	var numberPatternRE = dojo.number._numberPatternRE;
	var numberPattern = positivePattern.match(numberPatternRE);
	if(!numberPattern){
		throw new Error("unable to find a number expression in pattern: "+pattern);
	}
	return pattern.replace(numberPatternRE,
		dojo.number._formatAbsolute(value, numberPattern[0], {decimal: decimal, group: group, places: options.places}));
}

dojo.number.round = function(/*Number*/value, /*Number*/places, /*Number?*/multiple){
	//	summary:
	//		Rounds the number at the given number of places
	//	value:
	//		the number to round
	//	places:
	//		the number of decimal places where rounding takes place
	//	multiple:
	//		rounds next place to nearest multiple

	var pieces = String(value).split(".");
	var length = (pieces[1] && pieces[1].length) || 0;
	if(length > places){
		var factor = Math.pow(10, places);
		if(multiple > 0){factor *= 10/multiple;places++;} //FIXME
		value = Math.round(value * factor)/factor;

		// truncate to remove any residual floating point values
		pieces = String(value).split(".");
		length = (pieces[1] && pieces[1].length) || 0;
		if(length > places){
			pieces[1] = pieces[1].substr(0, places);
			value = Number(pieces.join("."));
		}
	}
	return value; //Number
}

/*=====
dojo.number.__formatAbsoluteOptions = function(kwArgs){
	//	decimal: String?
	//		the decimal separator
	//	group: String?
	//		the group separator
	//	places: Integer?
	//		number of decimal places
	//	round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means don't round.
}
=====*/

dojo.number._formatAbsolute = function(/*Number*/value, /*String*/pattern, /*dojo.number.__formatAbsoluteOptions?*/options){
	// summary: 
	//		Apply numeric pattern to absolute value using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted, ignores sign
	// pattern:
	//		the number portion of a pattern (e.g. #,##0.00)
	options = options || {};
	if(options.places === true){options.places=0;}
	if(options.places === Infinity){options.places=6;} // avoid a loop; pick a limit

	var patternParts = pattern.split(".");
	var maxPlaces = (options.places >= 0) ? options.places : (patternParts[1] && patternParts[1].length) || 0;
	if(!(options.round < 0)){
		value = dojo.number.round(value, maxPlaces, options.round);
	}

	var valueParts = String(Math.abs(value)).split(".");
	var fractional = valueParts[1] || "";
	if(options.places){
		valueParts[1] = dojo.string.pad(fractional.substr(0, options.places), options.places, '0', true);
	}else if(patternParts[1] && options.places !== 0){
		// Pad fractional with trailing zeros
		var pad = patternParts[1].lastIndexOf("0") + 1;
		if(pad > fractional.length){
			valueParts[1] = dojo.string.pad(fractional, pad, '0', true);
		}

		// Truncate fractional
		var places = patternParts[1].length;
		if(places < fractional.length){
			valueParts[1] = fractional.substr(0, places);
		}
	}else{
		if(valueParts[1]){ valueParts.pop(); }
	}

	// Pad whole with leading zeros
	var patternDigits = patternParts[0].replace(',', '');
	pad = patternDigits.indexOf("0");
	if(pad != -1){
		pad = patternDigits.length - pad;
		if(pad > valueParts[0].length){
			valueParts[0] = dojo.string.pad(valueParts[0], pad);
		}

		// Truncate whole
		if(patternDigits.indexOf("#") == -1){
			valueParts[0] = valueParts[0].substr(valueParts[0].length - pad);
		}
	}

	// Add group separators
	var index = patternParts[0].lastIndexOf(',');
	var groupSize, groupSize2;
	if(index != -1){
		groupSize = patternParts[0].length - index - 1;
		var remainder = patternParts[0].substr(0, index);
		index = remainder.lastIndexOf(',');
		if(index != -1){
			groupSize2 = remainder.length - index - 1;
		}
	}
	var pieces = [];
	for(var whole = valueParts[0]; whole;){
		var off = whole.length - groupSize;
		pieces.push((off > 0) ? whole.substr(off) : whole);
		whole = (off > 0) ? whole.slice(0, off) : "";
		if(groupSize2){
			groupSize = groupSize2;
			delete groupSize2;
		}
	}
	valueParts[0] = pieces.reverse().join(options.group || ",");

	return valueParts.join(options.decimal || ".");
};

/*=====
dojo.number.__regexpOptions = function(kwArgs){
	//	pattern: String?
	//		override pattern with this string.  Default is provided based on
	//		locale.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific, percent, currency. decimal by default.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	strict: Boolean?
	//		strict parsing, false by default
	//	places: Number|String?
	//		number of decimal places to accept: Infinity, a positive number, or
	//		a range "n,m".  By default, defined by pattern.
}
=====*/
dojo.number.regexp = function(/*dojo.number.__regexpOptions?*/options){
	//	summary:
	//		Builds the regular needed to parse a number
	//	description:
	//		Returns regular expression with positive and negative match, group
	//		and decimal separators
	return dojo.number._parseInfo(options).regexp; // String
}

dojo.number._parseInfo = function(/*Object?*/options){
	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojo.i18n.getLocalization("dojo.cldr", "number", locale);
	var pattern = options.pattern || bundle[(options.type || "decimal") + "Format"];
//TODO: memoize?
	var group = bundle.group;
	var decimal = bundle.decimal;
	var factor = 1;

	if(pattern.indexOf('%') != -1){
		factor /= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		factor /= 1000; // per mille
	}else{
		var isCurrency = pattern.indexOf('\u00a4') != -1;
		if(isCurrency){
			group = bundle.currencyGroup || group;
			decimal = bundle.currencyDecimal || decimal;
		}
	}

	//TODO: handle quoted escapes
	var patternList = pattern.split(';');
	if(patternList.length == 1){
		patternList.push("-" + patternList[0]);
	}

	var re = dojo.regexp.buildGroupRE(patternList, function(pattern){
		pattern = "(?:"+dojo.regexp.escapeString(pattern, '.')+")";
		return pattern.replace(dojo.number._numberPatternRE, function(format){
			var flags = {
				signed: false,
				separator: options.strict ? group : [group,""],
				fractional: options.fractional,
				decimal: decimal,
				exponent: false};
			var parts = format.split('.');
			var places = options.places;
			if(parts.length == 1 || places === 0){flags.fractional = false;}
			else{
				if(typeof places == "undefined"){ places = parts[1].lastIndexOf('0')+1; }
				if(places && options.fractional == undefined){flags.fractional = true;} // required fractional, unless otherwise specified
				if(!options.places && (places < parts[1].length)){ places += "," + parts[1].length; }
				flags.places = places;
			}
			var groups = parts[0].split(',');
			if(groups.length>1){
				flags.groupSize = groups.pop().length;
				if(groups.length>1){
					flags.groupSize2 = groups.pop().length;
				}
			}
			return "("+dojo.number._realNumberRegexp(flags)+")";
		});
	}, true);

	if(isCurrency){
		// substitute the currency symbol for the placeholder in the pattern
		re = re.replace(/(\s*)(\u00a4{1,3})(\s*)/g, function(match, before, target, after){
			var prop = ["symbol", "currency", "displayName"][target.length-1];
			var symbol = dojo.regexp.escapeString(options[prop] || options.currency || "");
			before = before ? "\\s" : "";
			after = after ? "\\s" : "";
			if(!options.strict){
				if(before){before += "*";}
				if(after){after += "*";}
				return "(?:"+before+symbol+after+")?";
			}
			return before+symbol+after;
		});
	}

//TODO: substitute localized sign/percent/permille/etc.?

	// normalize whitespace and return
	return {regexp: re.replace(/[\xa0 ]/g, "[\\s\\xa0]"), group: group, decimal: decimal, factor: factor}; // Object
}

/*=====
dojo.number.__parseOptions = function(kwArgs){
	//	pattern: String
	//		override pattern with this string.  Default is provided based on
	//		locale.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific, percent, currency. decimal by default.
	//	locale: String
	//		override the locale used to determine formatting rules
	//	strict: Boolean?
	//		strict parsing, false by default
	//	currency: Object
	//		object with currency information
}
=====*/
dojo.number.parse = function(/*String*/expression, /*dojo.number.__parseOptions?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Number, using
	//		locale-specific settings.
	// description:
	//		Create a Number from a string using a known localized pattern.
	//		Formatting patterns are chosen appropriate to the locale.
	//		Formatting patterns are implemented using the syntax described at
	//		*URL*
	// expression:
	//		A string representation of a Number
	var info = dojo.number._parseInfo(options);
	var results = (new RegExp("^"+info.regexp+"$")).exec(expression);
	if(!results){
		return NaN; //NaN
	}
	var absoluteMatch = results[1]; // match for the positive expression
	if(!results[1]){
		if(!results[2]){
			return NaN; //NaN
		}
		// matched the negative pattern
		absoluteMatch =results[2];
		info.factor *= -1;
	}

	// Transform it to something Javascript can parse as a number.  Normalize
	// decimal point and strip out group separators or alternate forms of whitespace
	absoluteMatch = absoluteMatch.
		replace(new RegExp("["+info.group + "\\s\\xa0"+"]", "g"), "").
		replace(info.decimal, ".");
	// Adjust for negative sign, percent, etc. as necessary
	return Number(absoluteMatch) * info.factor; //Number
};

/*=====
dojo.number.__realNumberRegexpFlags = function(kwArgs){
	//	places: Number?
	//		The integer number of decimal places or a range given as "n,m".  If
	//		not given, the decimal part is optional and the number of places is
	//		unlimited.
	//	decimal: String?
	//		A string for the character used as the decimal point.  Default
	//		is ".".
	//	fractional: Boolean|Array?
	//		Whether decimal places are allowed.  Can be true, false, or [true,
	//		false].  Default is [true, false]
	//	exponent: Boolean|Array?
	//		Express in exponential notation.  Can be true, false, or [true,
	//		false]. Default is [true, false], (i.e. will match if the
	//		exponential part is present are not).
	//	eSigned: Boolean|Array?
	//		The leading plus-or-minus sign on the exponent.  Can be true,
	//		false, or [true, false].  Default is [true, false], (i.e. will
	//		match if it is signed or unsigned).  flags in regexp.integer can be
	//		applied.
}
=====*/

dojo.number._realNumberRegexp = function(/*dojo.number.__realNumberRegexpFlags?*/flags){
	// summary:
	//		Builds a regular expression to match a real number in exponential
	//		notation
	// flags:
	//		An object

	// assign default values to missing paramters
	flags = flags || {};
	if(typeof flags.places == "undefined"){ flags.places = Infinity; }
	if(typeof flags.decimal != "string"){ flags.decimal = "."; }
	if(typeof flags.fractional == "undefined" || /^0/.test(flags.places)){ flags.fractional = [true, false]; }
	if(typeof flags.exponent == "undefined"){ flags.exponent = [true, false]; }
	if(typeof flags.eSigned == "undefined"){ flags.eSigned = [true, false]; }

	// integer RE
	var integerRE = dojo.number._integerRegexp(flags);

	// decimal RE
	var decimalRE = dojo.regexp.buildGroupRE(flags.fractional,
		function(q){
			var re = "";
			if(q && (flags.places!==0)){
				re = "\\" + flags.decimal;
				if(flags.places == Infinity){ 
					re = "(?:" + re + "\\d+)?"; 
				}else{
					re += "\\d{" + flags.places + "}"; 
				}
			}
			return re;
		},
		true
	);

	// exponent RE
	var exponentRE = dojo.regexp.buildGroupRE(flags.exponent,
		function(q){ 
			if(q){ return "([eE]" + dojo.number._integerRegexp({ signed: flags.eSigned}) + ")"; }
			return ""; 
		}
	);

	// real number RE
	var realRE = integerRE + decimalRE;
	// allow for decimals without integers, e.g. .25
	if(decimalRE){realRE = "(?:(?:"+ realRE + ")|(?:" + decimalRE + "))";}
	return realRE + exponentRE; // String
};

/*=====
dojo.number.__integerRegexpFlags = function(kwArgs){
	//	signed: Boolean?
	//		The leading plus-or-minus sign. Can be true, false, or [true,
	//		false]. Default is [true, false], (i.e. will match if it is signed
	//		or unsigned).
	//	separator: String?
	//		The character used as the thousands separator. Default is no
	//		separator. For more than one symbol use an array, e.g. [",", ""],
	//		makes ',' optional.
	//	groupSize: Number?
	//		group size between separators
	//	flags.groupSize2: Number?
	//		second grouping (for India)
}
=====*/

dojo.number._integerRegexp = function(/*dojo.number.__integerRegexpFlags?*/flags){
	// summary: 
	//		Builds a regular expression that matches an integer
	// flags: 
	//		An object

	// assign default values to missing paramters
	flags = flags || {};
	if(typeof flags.signed == "undefined"){ flags.signed = [true, false]; }
	if(typeof flags.separator == "undefined"){
		flags.separator = "";
	}else if(typeof flags.groupSize == "undefined"){
		flags.groupSize = 3;
	}
	// build sign RE
	var signRE = dojo.regexp.buildGroupRE(flags.signed,
		function(q) { return q ? "[-+]" : ""; },
		true
	);

	// number RE
	var numberRE = dojo.regexp.buildGroupRE(flags.separator,
		function(sep){
			if(!sep){
				return "(?:0|[1-9]\\d*)";
			}

			sep = dojo.regexp.escapeString(sep);
			if(sep == " "){ sep = "\\s"; }
			else if(sep == "\xa0"){ sep = "\\s\\xa0"; }

			var grp = flags.groupSize, grp2 = flags.groupSize2;
			if(grp2){
				var grp2RE = "(?:0|[1-9]\\d{0," + (grp2-1) + "}(?:[" + sep + "]\\d{" + grp2 + "})*[" + sep + "]\\d{" + grp + "})";
				return ((grp-grp2) > 0) ? "(?:" + grp2RE + "|(?:0|[1-9]\\d{0," + (grp-1) + "}))" : grp2RE;
			}
			return "(?:0|[1-9]\\d{0," + (grp-1) + "}(?:[" + sep + "]\\d{" + grp + "})*)";
		},
		true
	);

	// integer RE
	return signRE + numberRE; // String
}

}

if(!dojo._hasResource["dijit.form.Slider"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Slider"] = true;
dojo.provide("dijit.form.Slider");







dojo.declare(
	"dijit.form.HorizontalSlider",
	[dijit.form._FormWidget, dijit._Container],
{
	// summary
	//	A form widget that allows one to select a value with a horizontally draggable image

	templateString:"<table class=\"dijit dijitReset dijitSlider\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" rules=\"none\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\r\n\t\t><td dojoAttachPoint=\"containerNode,topDecoration\" class=\"dijitReset\" style=\"text-align:center;width:100%;\"></td\r\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\r\n\t></tr\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitHorizontalSliderButtonContainer\"\r\n\t\t\t><div class=\"dijitHorizontalSliderDecrementIcon\" tabIndex=\"-1\" style=\"display:none\" dojoAttachPoint=\"decrementButton\" dojoAttachEvent=\"onclick: decrement\"><span class=\"dijitSliderButtonInner\">-</span></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset\"\r\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitHorizontalSliderBumper dijitSliderLeftBumper dijitHorizontalSliderLeftBumper\"></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset\"\r\n\t\t\t><input dojoAttachPoint=\"valueNode\" type=\"hidden\" name=\"${name}\"\r\n\t\t\t/><div style=\"position:relative;\" dojoAttachPoint=\"sliderBarContainer\"\r\n\t\t\t\t><div dojoAttachPoint=\"progressBar\" class=\"dijitSliderBar dijitHorizontalSliderBar dijitSliderProgressBar dijitHorizontalSliderProgressBar\" dojoAttachEvent=\"onclick:_onBarClick\"\r\n\t\t\t\t\t><div dojoAttachPoint=\"sliderHandle,focusNode\" class=\"dijitSliderMoveable dijitHorizontalSliderMoveable\" dojoAttachEvent=\"onkeypress:_onKeyPress,onclick:_onHandleClick\" waiRole=\"slider\" valuemin=\"${minimum}\" valuemax=\"${maximum}\"\r\n\t\t\t\t\t\t><div class=\"dijitSliderImageHandle dijitHorizontalSliderImageHandle\"></div\r\n\t\t\t\t\t></div\r\n\t\t\t\t></div\r\n\t\t\t\t><div dojoAttachPoint=\"remainingBar\" class=\"dijitSliderBar dijitHorizontalSliderBar dijitSliderRemainingBar dijitHorizontalSliderRemainingBar\" dojoAttachEvent=\"onclick:_onBarClick\"></div\r\n\t\t\t></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset\"\r\n\t\t\t><div class=\"dijitSliderBar dijitSliderBumper dijitHorizontalSliderBumper dijitSliderRightBumper dijitHorizontalSliderRightBumper\"></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitHorizontalSliderButtonContainer\" style=\"right:0px;\"\r\n\t\t\t><div class=\"dijitHorizontalSliderIncrementIcon\" tabIndex=\"-1\" style=\"display:none\" dojoAttachPoint=\"incrementButton\" dojoAttachEvent=\"onclick: increment\"><span class=\"dijitSliderButtonInner\">+</span></div\r\n\t\t></td\r\n\t></tr\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\r\n\t\t><td dojoAttachPoint=\"containerNode,bottomDecoration\" class=\"dijitReset\" style=\"text-align:center;\"></td\r\n\t\t><td class=\"dijitReset\" colspan=\"2\"></td\r\n\t></tr\r\n></table>\r\n",
	value: 0,

	// showButtons: boolean
	//	Show increment/decrement buttons at the ends of the slider?
	showButtons: true,

	// minimum:: integer
	//	The minimum value allowed.
	minimum: 0,

	// maximum: integer
	//	The maximum allowed value.
	maximum: 100,

	// discreteValues: integer
	//	The maximum allowed values dispersed evenly between minimum and maximum (inclusive).
	discreteValues: Infinity,

	// pageIncrement: integer
	//	The amount of change with shift+arrow
	pageIncrement: 2,

	// clickSelect: boolean
	//	If clicking the progress bar changes the value or not
	clickSelect: true,

	widgetsInTemplate: true,

	attributeMap: dojo.mixin(dojo.clone(dijit.form._FormWidget.prototype.attributeMap),
		{id:"", name:"valueNode"}),

	baseClass: "dijitSlider",

	_mousePixelCoord: "pageX",
	_pixelCount: "w",
	_startingPixelCoord: "x",
	_startingPixelCount: "l",
	_handleOffsetCoord: "left",
	_progressPixelSize: "width",
	_upsideDown: false,

	_onKeyPress: function(/*Event*/ e){
		if(this.disabled || e.altKey || e.ctrlKey){ return; }
		switch(e.keyCode){
			case dojo.keys.HOME:
				this.setValue(this.minimum, false);
				break;
			case dojo.keys.END:
				this.setValue(this.maximum, false);
				break;
			case dojo.keys.UP_ARROW:
			case (this._isReversed() ? dojo.keys.LEFT_ARROW : dojo.keys.RIGHT_ARROW):
			case dojo.keys.PAGE_UP:
				this.increment(e);
				break;
			case dojo.keys.DOWN_ARROW:
			case (this._isReversed() ? dojo.keys.RIGHT_ARROW : dojo.keys.LEFT_ARROW):
			case dojo.keys.PAGE_DOWN:
				this.decrement(e);
				break;
			default:
				this.inherited("_onKeyPress", arguments);
				return;
		}
		dojo.stopEvent(e);
	},

	_onHandleClick: function(e){
		if(this.disabled){ return; }
		if(!dojo.isIE){
			// make sure you get focus when dragging the handle
			// (but don't do on IE because it causes a flicker on mouse up (due to blur then focus)
			dijit.focus(this.sliderHandle);
		}
		dojo.stopEvent(e);
	},
	
	_isReversed: function() {
		return !(this._upsideDown || this.isLeftToRight());
	},

	_onBarClick: function(e){
		if(this.disabled || !this.clickSelect){ return; }
		dijit.focus(this.sliderHandle);
		dojo.stopEvent(e);
		var abspos = dojo.coords(this.sliderBarContainer, true);
		var pixelValue = e[this._mousePixelCoord] - abspos[this._startingPixelCoord];
		this._setPixelValue(this._isReversed() || this._upsideDown ? (abspos[this._pixelCount] - pixelValue) : pixelValue, abspos[this._pixelCount], true);
	},

	_setPixelValue: function(/*Number*/ pixelValue, /*Number*/ maxPixels, /*Boolean, optional*/ priorityChange){
		if(this.disabled){ return; }
		pixelValue = pixelValue < 0 ? 0 : maxPixels < pixelValue ? maxPixels : pixelValue;
		var count = this.discreteValues;
		if(count <= 1 || count == Infinity){ count = maxPixels; }
		count--;
		var pixelsPerValue = maxPixels / count;
		var wholeIncrements = Math.round(pixelValue / pixelsPerValue);
		this.setValue((this.maximum-this.minimum)*wholeIncrements/count + this.minimum, priorityChange);
	},

	setValue: function(/*Number*/ value, /*Boolean, optional*/ priorityChange){
		this.valueNode.value = this.value = value;
		this.inherited('setValue', arguments);
		var percent = (value - this.minimum) / (this.maximum - this.minimum);
		this.progressBar.style[this._progressPixelSize] = (percent*100) + "%";
		this.remainingBar.style[this._progressPixelSize] = ((1-percent)*100) + "%";
	},

	_bumpValue: function(signedChange){
		if(this.disabled){ return; }
		var s = dojo.getComputedStyle(this.sliderBarContainer);
		var c = dojo._getContentBox(this.sliderBarContainer, s);
		var count = this.discreteValues;
		if(count <= 1 || count == Infinity){ count = c[this._pixelCount]; }
		count--;
		var value = (this.value - this.minimum) * count / (this.maximum - this.minimum) + signedChange;
		if(value < 0){ value = 0; }
		if(value > count){ value = count; }
		value = value * (this.maximum - this.minimum) / count + this.minimum;
		this.setValue(value, true);
	},

	decrement: function(e){
		// summary
		//	decrement slider by 1 unit
		this._bumpValue(e.keyCode == dojo.keys.PAGE_DOWN?-this.pageIncrement:-1);
	},

	increment: function(e){
		// summary
		//	increment slider by 1 unit
		this._bumpValue(e.keyCode == dojo.keys.PAGE_UP?this.pageIncrement:1);
	},

	_mouseWheeled: function(/*Event*/ evt){
		dojo.stopEvent(evt);
		var scrollAmount = 0;
		if(typeof evt.wheelDelta == 'number'){ // IE
			scrollAmount = evt.wheelDelta;
		}else if(typeof evt.detail == 'number'){ // Mozilla+Firefox
			scrollAmount = -evt.detail;
		}
		if(scrollAmount > 0){
			this.increment(evt);
		}else if(scrollAmount < 0){
			this.decrement(evt);
		}
	},

	startup: function(){
		dojo.forEach(this.getChildren(), function(child){
			if(this[child.container] != this.containerNode){
				this[child.container].appendChild(child.domNode);
			}
		}, this);
	},

	_onBlur: function(){
		dijit.form.HorizontalSlider.superclass.setValue.call(this, this.value, true);
	},

	postCreate: function(){
		if(this.showButtons){
			this.incrementButton.style.display="";
			this.decrementButton.style.display="";
		}
		this.connect(this.domNode, dojo.isIE ? "onmousewheel" : 'DOMMouseScroll', "_mouseWheeled");

		// define a custom constructor for a SliderMover that points back to me
		var _self = this;
		var mover = function(){
			dijit.form._SliderMover.apply(this, arguments);
			this.widget = _self;
		};
		dojo.extend(mover, dijit.form._SliderMover.prototype);

		this._movable = new dojo.dnd.Moveable(this.sliderHandle, {mover: mover});
		this.inherited('postCreate', arguments);
	},

	destroy: function(){
		this._movable.destroy();
		this.inherited('destroy', arguments);	
	}
});

dojo.declare(
	"dijit.form.VerticalSlider",
	dijit.form.HorizontalSlider,
{
	// summary
	//	A form widget that allows one to select a value with a vertically draggable image

	templateString:"<table class=\"dijitReset dijitSlider\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" rules=\"none\"\r\n><tbody class=\"dijitReset\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset\"></td\r\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitVerticalSliderButtonContainer\"\r\n\t\t\t><div class=\"dijitVerticalSliderIncrementIcon\" tabIndex=\"-1\" style=\"display:none\" dojoAttachPoint=\"incrementButton\" dojoAttachEvent=\"onclick: increment\"><span class=\"dijitSliderButtonInner\">+</span></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset\"></td\r\n\t></tr\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset\"></td\r\n\t\t><td class=\"dijitReset\"\r\n\t\t\t><center><div class=\"dijitSliderBar dijitSliderBumper dijitVerticalSliderBumper dijitSliderTopBumper dijitVerticalSliderTopBumper\"></div></center\r\n\t\t></td\r\n\t\t><td class=\"dijitReset\"></td\r\n\t></tr\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td dojoAttachPoint=\"leftDecoration\" class=\"dijitReset\" style=\"text-align:center;height:100%;\"></td\r\n\t\t><td class=\"dijitReset\" style=\"height:100%;\"\r\n\t\t\t><input dojoAttachPoint=\"valueNode\" type=\"hidden\" name=\"${name}\"\r\n\t\t\t/><center style=\"position:relative;height:100%;\" dojoAttachPoint=\"sliderBarContainer\"\r\n\t\t\t\t><div dojoAttachPoint=\"remainingBar\" class=\"dijitSliderBar dijitVerticalSliderBar dijitSliderRemainingBar dijitVerticalSliderRemainingBar\" dojoAttachEvent=\"onclick:_onBarClick\"></div\r\n\t\t\t\t><div dojoAttachPoint=\"progressBar\" class=\"dijitSliderBar dijitVerticalSliderBar dijitSliderProgressBar dijitVerticalSliderProgressBar\" dojoAttachEvent=\"onclick:_onBarClick\"\r\n\t\t\t\t\t><div dojoAttachPoint=\"sliderHandle,focusNode\" class=\"dijitSliderMoveable\" dojoAttachEvent=\"onkeypress:_onKeyPress,onclick:_onHandleClick\" style=\"vertical-align:top;\" waiRole=\"slider\" valuemin=\"${minimum}\" valuemax=\"${maximum}\"\r\n\t\t\t\t\t\t><div class=\"dijitSliderImageHandle dijitVerticalSliderImageHandle\"></div\r\n\t\t\t\t\t></div\r\n\t\t\t\t></div\r\n\t\t\t></center\r\n\t\t></td\r\n\t\t><td dojoAttachPoint=\"containerNode,rightDecoration\" class=\"dijitReset\" style=\"text-align:center;height:100%;\"></td\r\n\t></tr\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset\"></td\r\n\t\t><td class=\"dijitReset\"\r\n\t\t\t><center><div class=\"dijitSliderBar dijitSliderBumper dijitVerticalSliderBumper dijitSliderBottomBumper dijitVerticalSliderBottomBumper\"></div></center\r\n\t\t></td\r\n\t\t><td class=\"dijitReset\"></td\r\n\t></tr\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset\"></td\r\n\t\t><td class=\"dijitReset dijitSliderButtonContainer dijitVerticalSliderButtonContainer\"\r\n\t\t\t><div class=\"dijitVerticalSliderDecrementIcon\" tabIndex=\"-1\" style=\"display:none\" dojoAttachPoint=\"decrementButton\" dojoAttachEvent=\"onclick: decrement\"><span class=\"dijitSliderButtonInner\">-</span></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset\"></td\r\n\t></tr\r\n></tbody></table>\r\n",
	_mousePixelCoord: "pageY",
	_pixelCount: "h",
	_startingPixelCoord: "y",
	_startingPixelCount: "t",
	_handleOffsetCoord: "top",
	_progressPixelSize: "height",
	_upsideDown: true
});

dojo.declare("dijit.form._SliderMover",
	dojo.dnd.Mover,
{
	onMouseMove: function(e){
		var widget = this.widget;
		var c = this.constraintBox;
		if(!c){
			var container = widget.sliderBarContainer;
			var s = dojo.getComputedStyle(container);
			var c = dojo._getContentBox(container, s);
			c[widget._startingPixelCount] = 0;
			this.constraintBox = c;
		}
		var m = this.marginBox;
		var pixelValue = widget._isReversed() ?
			e[widget._mousePixelCoord] - dojo._abs(widget.sliderBarContainer).x : 
			m[widget._startingPixelCount] + e[widget._mousePixelCoord];
		dojo.hitch(widget, "_setPixelValue")(widget._isReversed() || widget._upsideDown? (c[widget._pixelCount]-pixelValue) : pixelValue, c[widget._pixelCount]);
	},
	
	destroy: function(e){
		var widget = this.widget;
		widget.setValue(widget.value, true);
		dojo.dnd.Mover.prototype.destroy.call(this);
	}
});


dojo.declare("dijit.form.HorizontalRule", [dijit._Widget, dijit._Templated],
{
	//	Summary:
	//		Create hash marks for the Horizontal slider
	templateString: '<div class="RuleContainer HorizontalRuleContainer"></div>',

	// count: Integer
	//		Number of hash marks to generate
	count: 3,

	// container: Node
	//		If this is a child widget, connect it to this parent node
	container: "containerNode",

	// ruleStyle: String
	//		CSS style to apply to individual hash marks
	ruleStyle: "",

	_positionPrefix: '<div class="RuleMark HorizontalRuleMark" style="left:',
	_positionSuffix: '%;',
	_suffix: '"></div>',

	_genHTML: function(pos, ndx){
		return this._positionPrefix + pos + this._positionSuffix + this.ruleStyle + this._suffix;
	},
	
	_isHorizontal: true,

	postCreate: function(){
		if(this.count==1){
			var innerHTML = this._genHTML(50, 0);
		}else{
			var interval = 100 / (this.count-1);
			if(!this._isHorizontal || this.isLeftToRight()){
				var innerHTML = this._genHTML(0, 0);
				for(var i=1; i < this.count-1; i++){
					innerHTML += this._genHTML(interval*i, i);
				}
				innerHTML += this._genHTML(100, this.count-1);
			}else{
				var innerHTML = this._genHTML(100, 0);
				for(var i=1; i < this.count-1; i++){
					innerHTML += this._genHTML(100-interval*i, i);
				}
				innerHTML += this._genHTML(0, this.count-1);
			}
		}
		this.domNode.innerHTML = innerHTML;
	}
});

dojo.declare("dijit.form.VerticalRule", dijit.form.HorizontalRule,
{
	//	Summary:
	//		Create hash marks for the Vertical slider
	templateString: '<div class="RuleContainer VerticalRuleContainer"></div>',
	_positionPrefix: '<div class="RuleMark VerticalRuleMark" style="top:',
	
	_isHorizontal: false
});

dojo.declare("dijit.form.HorizontalRuleLabels", dijit.form.HorizontalRule,
{
	//	Summary:
	//		Create labels for the Horizontal slider
	templateString: '<div class="RuleContainer HorizontalRuleContainer"></div>',

	// labelStyle: String
	//		CSS style to apply to individual text labels
	labelStyle: "",

	// labels: Array
	//	Array of text labels to render - evenly spaced from left-to-right or bottom-to-top
	labels: [],

	// numericMargin: Integer
	//	Number of generated numeric labels that should be rendered as '' on the ends when labels[] are not specified
	numericMargin: 0,

	// numericMinimum: Integer
	//	Leftmost label value for generated numeric labels when labels[] are not specified
	minimum: 0,

	// numericMaximum: Integer
	//	Rightmost label value for generated numeric labels when labels[] are not specified
	maximum: 1,

	// constraints: object
	//	pattern, places, lang, et al (see dojo.number) for generated numeric labels when labels[] are not specified
	constraints: {pattern:"#%"},

	_positionPrefix: '<div class="RuleLabelContainer HorizontalRuleLabelContainer" style="left:',
	_labelPrefix: '"><span class="RuleLabel HorizontalRuleLabel">',
	_suffix: '</span></div>',

	_calcPosition: function(pos){
		return pos;
	},

	_genHTML: function(pos, ndx){
		return this._positionPrefix + this._calcPosition(pos) + this._positionSuffix + this.labelStyle + this._labelPrefix + this.labels[ndx] + this._suffix;
	},

	getLabels: function(){
		// summary: user replaceable function to return the labels array

		// if the labels array was not specified directly, then see if <li> children were
		var labels = this.labels;
		if(!labels.length){
			// for markup creation, labels are specified as child elements
			labels = dojo.query("> li", this.srcNodeRef).map(function(node){
				return String(node.innerHTML);
			});
		}
		this.srcNodeRef.innerHTML = '';
		// if the labels were not specified directly and not as <li> children, then calculate numeric labels
		if(!labels.length && this.count > 1){
			var start = this.minimum;
			var inc = (this.maximum - start) / (this.count-1);
			for (var i=0; i < this.count; i++){
				labels.push((i<this.numericMargin||i>=(this.count-this.numericMargin))? '' : dojo.number.format(start, this.constraints));
				start += inc;
			}
		}
		return labels;
	},

	postMixInProperties: function(){
		this.inherited('postMixInProperties', arguments);
		this.labels = this.getLabels();
		this.count = this.labels.length;
	}
});

dojo.declare("dijit.form.VerticalRuleLabels", dijit.form.HorizontalRuleLabels,
{
	//	Summary:
	//		Create labels for the Vertical slider
	templateString: '<div class="RuleContainer VerticalRuleContainer"></div>',

	_positionPrefix: '<div class="RuleLabelContainer VerticalRuleLabelContainer" style="top:',
	_labelPrefix: '"><span class="RuleLabel VerticalRuleLabel">',

	_calcPosition: function(pos){
		return 100-pos;
	},
	
	_isHorizontal: false
});

}

if(!dojo._hasResource['hub.behaviors']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.behaviors'] = true;
dojo.provide('hub.behaviors');

/**
 * Renderable view behavior. Used by widgets that can render or filter 
 * information for another view.
 */
dojo.declare("hub.behaviors.Renderable", null,  {
    /**
     * Accepts arbitrary data and returns a result. The result may serve as
     * the parameter to another renderable's render method so it should 
     * typically be in the same format as data.
     */
    render: function(data) {
        // default is noop
        return data;
    },

    /**
     * Invoked if an error occurs.
     */
    renderError: function(err) {
        return err;
    }
});

/**
 * Stacked view behavior. Shows only one element at a time while all others are
 * hidden from view.
 */
dojo.declare('hub.behaviors.Stack', null, {
    /**
     * @param elementsIn: Array of id or elements that are stacked
     * @param visibleIdx: First visible index number or id
     * @param display: Display style to use when shown (inline, block, ...)
     */
     constructor: function(elementsIn, visibleIdx, display) {
         this.elements = new Array();
         this.ids = new Object();
         this.currentVisible = null;
         
         if (!display) {
             // style to set when displaying (e.g. inline, block)
             this.display = 'block';
         } else {
             this.display = display;
         }
         
         for(var i=0; i<elementsIn.length; i++) {
             if(dojo.isString(elementsIn[i])) {
                 // key: id, value: index
                 this.ids[elementsIn[i]] = i;
        	 elementsIn[i] = dojo.byId(elementsIn[i]);
	     } else {
                 // key: id, value: index
                 this.ids[elementsIn[i].id] = i;
             }
             
             // always add, even if null, so indices don't collapse
             this.elements.push(elementsIn[i]);		
             
             if(elementsIn[i]) {
        	 // always start hidden
        	 elementsIn[i].style.display = 'none';
             }
         }
         
         // show the initial panel
         this.show(visibleIdx || 0);
     },

    /**
     * Show the element at the given index or with the given id.
     *
     * @param idxIn: Integer index or string id
     * @return: Index of last visible element or null if no action taken
     */
    show : function(idxIn) {
        if(dojo.isString(idxIn)) {
            // if we have a string, look up the corresponding index
            try {
                idxIn = this.ids[idxIn];
            } catch(e) {
                // no match for name
                return null;
            }
        }

        if(this.currentVisible == idxIn || 
           idxIn < 0 || idxIn > this.elements.length-1) {
            // no match for index, or already shown
            return null;
        }

        // hide the old object
        var idxOld = this.currentVisible;
        try {
            var e = this.elements[this.currentVisible];
            e.style.display = 'none';
        } catch (e) {
            // nothing to hide
        }

        // show the new object
        this.currentVisible = idxIn;
        try {
            e = this.elements[this.currentVisible];
            e.style.display = this.display;
        } catch (e) {
            // nothing to show
        }
        return idxOld;
    },
    
    getCurrentVisible: function() {
        return this.currentVisible;
    }
});

}

if(!dojo._hasResource['hub.widgets.social.view']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.widgets.social.view'] = true;
dojo.provide('hub.widgets.social.view');








/**
 * Renders site social data through a visualization pipeline.
 */
dojo.declare("hub.widgets.social.view.SocialBox", [dijit._Widget, dijit._Container, hub.behaviors.Renderable], {
    // hub.data.social._Model widgetid
    model: '',
    // force a refresh on the model immediately
    forceRefresh: false,
    // if true, replaces all content with a refresh message while model is 
    // updating; disable if child views do not completely re-render
    showRefresh: true,

    startup: function() {
        // store the pipeline because it's going to get overwritten
        this.pipeline = this.getChildren();

        if(this.model) {
            this.setModel(this.model);
        }
        
        if(this.forceRefresh && this.model) {
            this.model.refresh();
        } else {
            this._onLoad(this.model);
        }
        
        // call parent
        this.inherited(arguments);
    },

    /**
     * Destroys this widget and its descendants. Not sure why destroyRecursive
     * isn't called recursively instead.
     */
    destroy: function() {
        // the pipeline widgets are no longer in our template
	dojo.forEach(this.pipeline, function(widget) { 
            widget.destroy(); 
        });
        // call base implementation to kill this widget
        this.inherited(arguments);
    },

    setModel: function(model) {
        // stop listening to previous model
        this.disconnect();
        // store new model
        this.model = dijit.byId(model);
        // watch for changes on the comment model
        this.connect(this.model, 'onComplete', '_onLoad');
        this.connect(this.model, 'onFetch', '_onFetch');
        this.connect(this.model, 'onError', '_onError');
    },

    redraw: function() {
        // force a redraw with existing data from the model
        this._onLoad(this.model);
    },

    render: function(data) {
        // run through all descendant widgets and see if we can get them to 
        // render (i.e., are they hub.behavior.Renderables or look-alikes)
        for(var i=0; i < this.pipeline.length; i++) {
            try {
                data = this.pipeline[i].render(data);
            } catch(e) {
                // ignore, probably not renderable
                console.warn(e);
            }
        }
        // assume last child widget returned is the rendering
        return data;
    },

    renderError: function(err) {
        // @todo this fetches descendants, not children; probably want to be
        // more careful
        // run through all descendant widgets and see if we can get them to 
        // render (i.e., are they hub.behavior.Renderables or look-alikes)
        for(var i=0; i < this.pipeline.length; i++) {
            try {
                var err = this.pipeline[i].renderError(err);
            } catch(e) {
                // ignore, probably not renderable
            }
        }
        // assume last child widget returned the final message
        return err;
    },

    _onLoad: function(model) {
        try {
            var data = model.getSocialData();
        } catch(e) {
            // still OK; sometimes no model needed
            // use dummy object since null has a meaning as an error
            var data = {};
        }

        if(data == null || data.length == 0) {
            // treat no data as an error
            this._onError(model);
            return;
        }

        // ask child widgets to render
        var result = this.render(data);

        if(result != null && dojo.isObject(result)) {
            // result is a DOM node
            this.domNode.innerHTML = '';
            this.domNode.appendChild(result);
        } else if(result != null) {
            // result is a string
            this.domNode.innerHTML = result;
        }
        // ignore all other result types
    },

    _onFetch : function(model) {
        if(this.showRefresh) {
            hub.busyMessage(this.domNode, false, 'note');
        }
    },

    _onError: function(model) {
        var text = this.renderError('');
        if(text != null) {
            hub.notify(text, this.domNode, 'note');
        }
    }
});


/**
 * Filters tags so only the top X are rendered.
 */
dojo.declare("hub.widgets.social.view.TopTagsFilter", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    // default number of tags to render
    value: 15,

    _getMax: function(total) {
        // value is the maximum number to render
        return this.value;
    },
    
    setValue: function(value) {
        this.value = value;
    },
    
    render: function(arr) {
        // store original array in case we re-render
        this.data = arr;

        var total = 0;
        var bins = [];
        // sort by frequency
        for(var key in arr) {
            var val = arr[key];
            if(!bins[val]) {
                // build an array to store tags with this frequency
                bins[val] = new Array();
            }
            // push into the proper bin
            bins[val].push(key);
            total += 1;
        }
        
        // figure out the maximum number of items
        var max = this._getMax(total);
        
        var top = [];
        for(var i = bins.length - 1; i >= 0; i--) {
            var tags = bins[i];
            // avoid empty bins
            if(!tags) continue;
            if(tags.length + top.length > max) {
                // slice just the tags we need
                top = top.concat(tags.slice(0, max - top.length));
                break;
            } else {
                top = top.concat(tags);
            }
        }
        // sort again by name
        top.sort();
        var top_pairs = {};
        // recombine with frequencies
        for(var i in top) {
            var name = top[i];
            top_pairs[name] = arr[name];
        }

        return top_pairs;
    }    
});

/**
 * Filters tags so some percentage of the total tags is shown.
 */
dojo.declare('hub.widgets.social.view.DensityTagsFilter', hub.widgets.social.view.TopTagsFilter, {
    destroy: function() {
        console.log('destroying density tags filter');
        this.inherited(arguments);
    },
    
    _getMax: function(total) {
        // value is percentage of total tags
        return parseInt(total * (this.value / 100.0));
    }
});

/**
 * Cloud view of tags.
 */
dojo.declare("hub.widgets.social.view.TagsCloudView", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    render: function(arr) {
        var pops = dojo.i18n.getLocalization('hub', 'popups');
        var html = "<div class='TOM_widget_TagsCloudView'>" ;
        for(var name in arr) {
            var size = parseInt(arr[name]);
            var title = dojo.string.substitute(pops.TAG_ITEM_COUNT, [arr[name]]);
            if(size > 10)
                size = 10;
            html += "<span class='tag"+size+"'>";
            //name = hub.htmlspecialchars(name);
            html += hub.buildPageLink(hub.htmlspecialchars(name), 'Catalog', 
                                      {collection: "all", searchType: "tag", term: hub.escapeQuotes3(name)}, false, null, 
                                      title);
            html += "</span>\n";
        }
        html += "</div>";
        return html;
    },

    renderError: function(err) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return msgs.STATUS_TAGS_NONE;
    }
});

/**
 * List view of categories.
 */
dojo.declare("hub.widgets.social.view.CategoriesListView", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    render: function(arr) {
        var html = "<ul class='TOM_action_list' style='margin-left:0.5em;'>\n";
        for (var i in arr) {
            html += "<li>";
            html += hub.buildPageLink(hub.htmlspecialchars(arr[i]), 'Catalog', 
                                      {collection: 'all', searchType: "category", term: hub.escapeQuotes3(arr[i])}, true);
            html += "</li>\n";
        }
        html += "</ul>";
        return html;
    },

    renderError: function(err) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return msgs.STATUS_CATEGORIES_NONE;
    }
});

/**
 * Inline comma separated view of tags with counts displayed.
 */
dojo.declare("hub.widgets.social.view.TagsCountView", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    render: function(arr) {
        var pops = dojo.i18n.getLocalization('hub', 'popups');
        var html = "<div class='TOM_widget_TagsCountView'>";
        for (var name in arr) {
            var title = dojo.string.substitute(pops.TAG_USER_COUNT, [arr[name]]);
//            name = hub.htmlspecialchars(name);
            html += hub.buildPageLink(hub.htmlspecialchars(name), 'Catalog', 
                                      {collection: 'all', searchType: "tag", term: hub.escapeQuotes3(name)}, false,
                                      null, title);
            html += ', ';
        }
        // cut final comma
        html = html.substring(0,html.length-2);
        html += '</div>';
        return html;
    },

    renderError: function(err) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return msgs.STATUS_TAGS_NONE;
    }
});

/**
 * Renders user comments by observing a CommentsModel instance.
 */
dojo.declare("hub.widgets.social.view.CommentsListView", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    render: function(arr) {
        // only require if we actually reach this code
        
        var panels = dojo.i18n.getLocalization('hub', 'panels');

        var html = "<div class='TOM_widget_CommentsListView'";
        for(var i=0; i < arr.length; i++) {
            var comment = arr[i];
            var date = new Date(parseInt(comment.timestamp)*1000);
            var locDate = dojo.date.locale.format(date);
            html += "<div class='comment'><h3 class='title'>"+hub.htmlspecialchars(comment.commenttitle)+"</h3>";
            html += "<pre class='body'>"+hub.htmlspecialchars(comment.commenttext)+"</pre>";
            var byline = dojo.string.substitute(panels.COMMENT_BYLINE, 
                [hub.htmlspecialchars(comment.creatorid), locDate]);
            html += "<p class='byline'>"+byline+"</p></div>";
        }
        html += "</div>";
        return html;
    },

    renderError: function(err) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return msgs.STATUS_COMMENTS_NONE;
    }
});

/**
 * Renders user comments by observing a RatingsModel instance.
 */
dojo.declare("hub.widgets.social.view.RatingStarView", [dijit._Widget, dijit._Contained, hub.behaviors.Renderable], {
    // icon for full star
    fullIcon: "redstar.gif",
    // icon for half start
    halfIcon: "redhalfstar.gif",
    // icon for no star
    emptyIcon: "icon-rated-empty.gif",
    // number of stars possible
    maxRating: 5,

    getStarUrls: function(value) {
        var urls = new Array();
        // render stars, half-stars, and empty stars
        for(var i = 1; i <= this.maxRating; i++) {
            if(value >= i) {
                urls.push(hub.urls.getImage(this.fullIcon));
            } else if(value >= (i - .5)) {
                urls.push(hub.urls.getImage(this.halfIcon));
            } else {
                urls.push(hub.urls.getImage(this.emptyIcon));
            }
        }
        return urls;
    },

    getRatingEstimate: function(value) {
        var val = 0;
        for(var i = 1; i <= this.maxRating; i++) {
            if(value >= i) {
                val = i;
            } else if(value >= (i - .5)) {
                val = i-0.5;
            }
        }
        return val;
    },

    renderStarImages: function(urls) {
        var stars = dojo.map(urls, function(url) {
            var img = document.createElement('IMG');
            img.src = url;
            return img;
        });
        return stars;
    },

    renderUserCount: function(count) {
        var panels = dojo.i18n.getLocalization('hub', 'panels');
        // render total votes text
        var span = document.createElement('SPAN');
        span.innerHTML = ' ' + dojo.string.substitute(panels.RATING_USER_COUNT,
            [count]);
        return span;
    },

    render: function(arr) {
        var est = this.getRatingEstimate(arr.value);
        var urls = this.getStarUrls(arr.value);
        var stars = this.renderStarImages(urls);
        var count = this.renderUserCount(arr.count);

        var div = document.createElement('DIV');
//      dojo.forEach(stars, dojo.hitch(div, div.appendChild)); // causes problems in IE7
        for(var i in stars){
        	div.appendChild(stars[i]);
        }
        div.appendChild(count);
        div.title = est;
        return div;
    },

    renderError: function(err) {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return msgs.STATUS_RATING_NONE;
    }
});

}

if(!dojo._hasResource["hub.data.catalog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.data.catalog"] = true;
dojo.provide("hub.data.catalog");




/**
 * Abstract class for accessing the Mashup Hub catalog. Subclasses implement
 * fetch logic for specific catalog formats. Derives from dijit._Widget to 
 * support proper cleanup.
 */
dojo.declare('hub.data.catalog._Model', dijit._Widget, {
    // default query is entire catalog
    query: 'all',
    // default term is everything
    term: '*',
    collection: "all",
    searchType: "",
    sortBy: null,
    sortOrder: null,
    count: null,
    constructor: function(args, node) {
        // source for the server data
        this.source = null;
        // url to use to fetch the data
        this.url = null;
        // url to get the next batch of data
        this.next_url = null;
        // url to get the previous batch of data
        this.prev_url = null;
        // 3-tuple summary of the fetched data (first, last, total)
        this.summary = null;
        // outstanding request
        this.request = null;
        // subscriptions to dojo.publish'ed events so we know when to refresh
        this.conns = new Array();
    },

    /**
     * Starts watching for bus events.
     */
    postCreate: function() {
        // watch for bus events about entries
        this.conns.push(dojo.subscribe('hub.tag-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.save-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.delete-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.comment-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.reset', this, 'onStale'));
        if(this.query == 'favorites') {
            // watch for changing of favorites too
            this.conns.push(dojo.subscribe('update-favorites', this, 
                                           'onStale'));
        }
    },
       
    /**
     * Stops watching for bus events.
     */
    destroy: function() {
        // cleanup all subscriptions
        dojo.forEach(this.conns, dojo.unsubscribe);
        // call base class method
        this.inherited(arguments);
    },
    
    /**
     * Clears the model
     */
    clear: function() {
      this.next_url = null;
      this.prev_url = null;
      this.source = null;
      this.url = null;
      this.summary = null;
      this.request = null;
      this.stale = null;
    },
    
    /**
     * Gets the URL last loaded by the catalog.
     */
    getUrl: function() {
        return this.url;
    },
    
    /**
     * Gets a tuple of start index, max per page, and total items. If any
     * of these items is unknown 
     */
    getSummary: function() {
        return this.summary;
    },
    
    /**
     * Get if the catalog has a next set of results.
     */
    hasNext: function() {
        return (this.next_url != null);
    },
    
    /**
     * Get if the catalog has a previous set of results.
     */
    hasPrevious: function() {
        return (this.prev_url != null);
    },
    
    /**
     * Fetch items.
     *
     * @param start: Starting index
     * @param max: Maximum number of items to fetch
     * @param sortby: Name of the field to sort by
     * @param asc: Order of the sort, true = ascending, false = descending
     */
    fetch: function(start, max, sortby, asc) {
        var url = this._createQueryUrl();
        console.debug("URL = " + url);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
        	console.debug("Parsed query = " + query + ", URL = " + url);
        }
        var map = dojo.queryToObject(query);
        map['nologin'] = 'true';
        if (sortby != null) {
        	map["sortBy"] = sortby;
        }
        else if (this.sortBy != null) {
        	map["sortBy"] = this.sortBy;
        }
        if (asc != null) {
        	map["sortOrder"] = asc ? "asc" : "desc";
        }
        else if (this.sortOrder != null) {
        	map["sortOrder"] = this.sortOrder;
        }
        if (start != null) {
        	map["startIndex"] = "" + start;
        }
        if (max != null) {
        	map["count"] = "" + max;
        } else if (this.count) {
            map["count"] = "" + this.count;
        }

        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
        console.debug("Result URL = " + url);
        this.url = url;
        
        // do the fetch
        this._doFetch();
    },
    
    /**
     * Fetch items again.
     */
    refresh: function() {
        if(this.url != null) {
            this._doFetch();
        } else {
            this.fetch();
        }
    },
    
    /**
     * Fetch the next batch of items, if it exists.
     */
    fetchNext: function() {
        if(this.next_url) {
            this.url = this.next_url;
            this._doFetch();
        } else {
            throw new Error("catalog._Model: No next URL");
        }
    },
    
    /**
     * Fetch the previous batch of items, if it exists.
     */
    fetchPrevious: function() {
        if(this.prev_url) {
            this.url = this.prev_url;
            this._doFetch();
        } else {
            throw new Error("catalog._Model: No previous URL");
        }
    },
    
    
    /**
     * Signals 'complete' after the last item signal.
     */
    _onComplete: function(empty, request) {
        this.onComplete();
        this.request = null;
        this.stale = false;
    },
    
    /**
     * Abstract method. Should return URL to query.
     */
    _createQueryUrl : function() {
        throw new Error("Abstract method _createQueryUrl()");
    },
    
    /**
     * Does some basic housekeeping before fetching. Must be overriden to
     * do an actual fetch.
     */
    _doFetch: function() {
        this.next_url = null;
        this.prev_url = null;
        this.summary = new Array(3);
        
        if(this.request) {
            // @todo: really we'd like to cancel the earlier request instead
            // of stopping the new request, but dojo isn't cooperating
            // supposedly there should be an abort() method on request, but
            // not for XML; so how do we reliably stop it?
            throw new Error('catalog._Model: request in progress');
        }
        
        // signal that we're about to fetch
        this.onFetch();          
    },

    /**
     * Signals when catalog data is likely stale because of local changes.
     */
    onStale: function() {
        // extension point
    },

    /**
     * Signals at start of fetch, before network request.
     */
    onFetch: function() {
        // extension point
    },

    /**
     * Signals after data received, before first item.
     */
    onBegin: function(total) {
        // extension point
    },

    /**
     * Signals after data received, before first item.
     */
    onItem: function(item) {
        // extension point
    },
    
    /**
     * Signals after an error.
     */
    onError: function(error) {
        // extension point
    },

    /**
     * Signals after completion.
     */
    onComplete: function() {
        // extension point
    }
});

/**
 * Fetches an Atom feed listing a portion of the Mashup Hub catalog.
 * Pulls out all available data into JS objects for use in rendering.
 **/
dojo.declare('hub.data.catalog.AtomModel', hub.data.catalog._Model, {
    _createQueryUrl: function() {
        return hub.urls.getCatalog(this.collection, this.term, this.searchType, this.sortBy, this.sortOrder);
    },

    /**
     * Performs the remote query. Signals 'fetch' first.
     */
    _doFetch: function() {
        this.inherited(arguments);
        // send the query to the remote data source
        this.source = new dojox.data.XmlStore({'url' : this.url});
        this.request = this.source.fetch(
            {'onBegin' : dojo.hitch(this, this.onBegin),
             'onItem' : dojo.hitch(this, this._onItem),
             'onComplete' :dojo.hitch(this, this._onComplete),
             'onError' : dojo.hitch(this, this.onError)});
    },
    
    /**
     * Parses a single, top level XML element for data. Looks for relative
     * links and entries. Signals onItem with entry data in JS format.
     */
    _onItem: function(item, request) {
        var dict = {};
        var value;
        var tag = this.source.getValue(item, 'tagName')
        if(tag == 'os:startIndex') {
            this.summary[0] = parseInt(this.source.getValue(item, 'text()'));
        } else if(tag == 'os:itemsPerPage') {
            this.summary[1] = parseInt(this.source.getValue(item, 'text()'));
        } else if(tag == 'os:totalResults') {
            this.summary[2] = parseInt(this.source.getValue(item, 'text()'));
        } else if(tag == 'link') {
            var rel = this.source.getValue(item, '@rel');
            var href = this.source.getValue(item, '@href');
            if(rel == 'next') {
                this.next_url = href;
            } else if (rel == 'previous') {
                this.prev_url = href;
            }
        } else if(tag == 'entry') {
            var entryCategoryElement = this.source.getValue(item, "category");
            var category = this.source.getValue(entryCategoryElement, "@term");
            dict["entryCategory"] = category;
            var content = this.source.getValue(item, "content");
            
            var cat = this.source.getValue(content, "childNodes");
            
            var atts = this.source.getValues(cat, "childNodes");
            
            for(var i=0; i < atts.length; i++) {
                var node = atts[i];
                if(this.source.isItem(node)) {
                    if(this.source.hasAttribute(node, "catalog:tag")) {
                        value = this.source.getValues(node, "catalog:tag");
                        value = this._parseArray(value);
                    } else if (this.source.hasAttribute(node, "catalog:category")) {
                        value = this.source.getValues(node, "catalog:category");
                        value = this._parseArray(value);
                    }
                    else {
                        value = this.source.getValue(node, "text()");
                    }
                }
                
                var key = this.source.getValue(atts[i], 'tagName');
                if (key.lastIndexOf(":") > 0)
                	key = key.substring(key.lastIndexOf(":") + 1);
                	
				dict[key] = value;                
            }
            
            var idfield = this.source.getValue(this.source.getValue(item, "id"), "text()");
            var unprefixedId = idfield;
            if (idfield.indexOf("urn:uuid:") == 0) {
            	unprefixedId = idfield.substring("urn:uuid:".length);
            }
            if (idfield.indexOf("urn:id:") == 0) {
            	unprefixedId = idfield.substring("urn:id:".length);
			}
            var pureId = unprefixedId;
            if (unprefixedId.lastIndexOf('-') >=0) {
            	pureId = unprefixedId.substring(unprefixedId.lastIndexOf('-') + 1);
            }
            dict['objectid'] = pureId;

            // signal with parsed data
            this.onItem(dict);
        }
    },
    
    /**
     * Pulls an array of strings out of dojo data XML values.
     *
     * @param values Dojo data items having the values we want
     */
    _parseArray: function(values) {
        var arr = new Array();
        for(i in values) {
            arr.push(this.source.getValue(values[i], "text()"));
        }
        return arr;
    }
});

/**
 * Fetches a JSON list of a portion of the Mashup Hub catalog.
 * Pulls out all available data into JS objects for use in rendering.
 */
dojo.declare('hub.data.catalog.JsonModel', hub.data.catalog._Model, {
    _createQueryUrl: function() {
        return hub.urls.getCatalog(this.query, this.term, 'json');
    },

    /**
     * Performs the remote query. Signals 'fetch' first.
     */
    _doFetch: function() {
        this.inherited(arguments);       
        // signal that we're about to fetch
        this.onFetch();
        // send the query to the remote data source
        this.source = {url : this.url,
                       handleAs: 'json',
                       load : dojo.hitch(this, this._onLoad),
                       error: dojo.hitch(this, this.onError)
                      }
        this.request = dojo.xhrGet(this.source);
    },
    
    _onLoad: function(response, args) {
        // grab relative urls
        this.next_url = response.next;
        this.prev_url = response.previous;
        
        try {
            // signal begin
            this.onBegin(response.content.length);
            
            // iterate over content
            for(i in response.content) {
                this.onItem(response.content[i]);
            }
            
            // signal complete
            this._onComplete(response, args);
        } catch (e) {
            // all errors go to the registered error handler
            this.onError(e, args);
        }
    }
});

}

if(!dojo._hasResource['hub.data.social']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.data.social'] = true;
dojo.provide('hub.data.social');


/**
 * Abstract base class for fetching social data from the server. Subclasses
 * implement logic for choosing what data to fetch and how to store it. 
 * Derives from dijit._Widget to support proper cleanup.
 */
dojo.declare("hub.data.social._Model", dijit._Widget, {
    // refresh when instantiated
    forceRefresh: false,
    constructor: function(args, node) {
        // initialize arrays for connections and data
        this.conns = new Array();
        // initialize slot for social data
        this.data = null;

        // build request object
        this.request = {'handleAs': 'json',
                        'timeout': 30000,
                        'load' : dojo.hitch(this, this._onLoad),
                        'error' : dojo.hitch(this, this._onError)
                       };
    },

    /**
     * Refresh the model if forced by an option.
     */
    postCreate: function() {
        if(this.forceRefresh) {
            this.refresh();
        }
    },
    
    destroy: function() {
        // cleanup all subscriptions
        dojo.forEach(this.conns, dojo.unsubscribe);
        // call base class method
        this.inherited(arguments);
    },
   
    _onLoad: function(response, args) {
        if(response.code > 0) {
            this._onError(response, args);
            return;
        }
        this.data = response.result;
        this.onComplete(this);
    },
    
    _onError: function(response, args) {
        // ignore the error, keep the current categories
        this.onError(this);
    },
    
    refresh: function() {
        this.onFetch(this);
        dojo.xhrGet(this.request);
    },
    
    getSocialData: function() {
        return this.data;
    },

    onComplete: function(model) {
        // extension point
    },

    onError: function(model) {
        // extension point
    },

    onFetch: function(model) {
        // extension point
    }
});

/**
 * Manages a list of global categories.
 */
dojo.declare("hub.data.social.CategoriesModel", hub.data.social._Model, {
    postCreate: function() {
        // connect to global category update signal
        this.conns.push(dojo.subscribe('hub.update-categories', this, 'refresh'));

        // RESTful URL
        this.request.url = hub.urls.getCategories();

        this.inherited(arguments);
    }
});

/**
 * Manages a list of comments for a single catalog entry.
 */
dojo.declare("hub.data.social.CommentsModel", hub.data.social._Model, {
    objectid: null,
    postCreate: function() {
        if(this.objectid == null) {
            throw new Error("CommentsModel: objectid required");
        }

        // connect to global category update signal
        this.conns.push(dojo.subscribe('hub.comment-entry', this, '_filterEvent'));

        // RESTful URL
        this.request.url = hub.urls.getComments(this.objectid);

        this.inherited(arguments);
    },

    _filterEvent: function(id) {
        if(id == this.objectid) {
            this.refresh();
        }
    }
});

/**
 * Manages the average rating for a single catalog entry.
 */
dojo.declare("hub.data.social.AverageRatingModel", hub.data.social._Model, {
    objectid: null,
    postCreate: function() {
        if(this.objectid == null) {
            throw new Error("AverageRatingModel: objectid required");
        }

        // connect to global rate entry signal
        this.conns.push(dojo.subscribe('hub.rate-entry', this, '_filterEvent'));

        // RESTful URL
        this.request.url = hub.urls.getRatings(this.objectid);

        this.inherited(arguments);
    },

    _filterEvent: function(id) {
        if(id == this.objectid) {
            this.refresh();
        }
    }
});

/**
 * Manages a list of tags, global or for a particular object.
 */
dojo.declare("hub.data.social.TagsModel", hub.data.social._Model, {
    objectid: null,
    postCreate: function() {
        if(this.objectid != null) {
            this.request.url = this._initOne();
        } else {
            this.request.url = this._initAll();
        }        
        this.inherited(arguments);
    },
    
    _initAll: function() {
        // user tagged an entry
        this.conns.push(dojo.subscribe('hub.tag-entry', this, 'refresh'));
        // user deleted an entry
        this.conns.push(dojo.subscribe('hub.delete-entry', this, 'refresh'));
        // user save an entry (and possibly modified tags)
        this.conns.push(dojo.subscribe('hub.save-entry', this, 'refresh'));
        // RESTful URL
        return hub.urls.getTags();
    },

    _initOne: function() {
        if(this.objectid == null) {
            throw new Error("TagsModel: objectid required");
        }
        // user tagged an entry
        this.conns.push(dojo.subscribe('hub.tag-entry', this, '_filterEvent'));
        // RESTful URL
        return hub.urls.getTags(this.objectid);
    },

    _filterEvent: function(id) {
        if(id == this.objectid) {
            this.refresh();
        }
    }
});

}

if(!dojo._hasResource['hub.widgets.catalog']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.widgets.catalog'] = true;
dojo.provide('hub.widgets.catalog');









/**
 * Displays an array of values keyed by column names in a table.
 */
dojo.declare("hub.widgets.catalog.CatalogTable", hub.widgets.Table, {
    // @todo: do we need to do this for build?
    templateString: '',
    // template to replace node with
    templateString:"<div>\r\n    \t<form onsubmit=\"return false;\" >\r\n  <div class=\"TOM_widget_Table\">\r\n    <table>\r\n      <thead>\r\n        <tr dojoAttachPoint=\"table_header\"></tr>\r\n      </thead>\r\n      <tbody dojoAttachPoint=\"table_body\"></tbody>\r\n      <tfoot>\r\n        <tr dojoAttachPoint=\"table_footer\" class=\"centered note\"></tr>\r\n      </tfoot>\r\n    </table>\r\n    <div class=\"navigation\">\r\n    <!--\r\n      <a href=\"#\" dojoAttachEvent=\"onclick: _onPrevious\" dojoAttachPoint=\"previous_button2\">${links.CATALOG_TABLE_PREVIOUS}</a>\r\n      <a href=\"#\" dojoAttachEvent=\"onclick: _onRefresh\" dojoAttachPoint=\"refresh_button2\">${links.CATALOG_TABLE_REFRESH}</a>\r\n      <a href=\"#\" dojoAttachEvent=\"onclick: _onNext\" dojoAttachPoint=\"next_button2\">${links.CATALOG_TABLE_NEXT}</a>\r\n    -->\r\n      <span class='prevButton'><button dojoAttachPoint=\"previous_button\" type=\"submit\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onPrevious\"></button></span>\r\n      <span style=\"font-size:0.8em;\" dojoAttachPoint=\"status_nav\"></span>\r\n      <span class='nextButton'><button dojoAttachPoint=\"next_button\" type=\"submit\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onNext\"></button></span>\r\n      <span class='refreshButton'><button dojoAttachPoint=\"refresh_button\" type=\"submit\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onRefresh\"></button></span>\r\n    </div>\r\n  </div>\r\n  <p style=\"font-size:0.8em;\" dojoAttachPoint=\"atom\">${panels.CATALOG_TABLE_ATOM} <a href=\"#\" class=\"noLink\" target=\"_blank\" dojoAttachPoint=\"feed_link\"><img src=\"${atom_img_url}\" alt=\"${pops.CATALOG_TABLE_ATOM_ALT}\" title=\"${pops.CATALOG_TABLE_ATOM_TITLE}\" /></a></p>\r\n  <div dojoAttachPoint=\"popups\"></div>\r\n  \t\t</form>\r\n</div>\r\n",
    widgetsInTemplate: true,

    // hub.data.catalog._Model widgetid
    model: '',
    // force a refresh on the model immediately
    forceRefresh: false,

    constructor: function(args, node) {
        // has the model indicated it is stale?
        this.stale = false;
        // subscriptions to dojo.publish'ed events so we know when to refresh
        this.conns = new Array();
    },

    postMixInProperties: function() {
        // get model widget from id
        this.model = dijit.byId(this.model);

        // fetch all localized text
        this.links = dojo.i18n.getLocalization('hub', 'links');
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.msgs = dojo.i18n.getLocalization('hub', 'messages');
        this.pops = dojo.i18n.getLocalization('hub', 'popups');

        // urls for icons and such
        this.atom_img_url = hub.urls.getImage('xml.gif');
        this.feed_img_url = hub.urls.getImage('feedSmall.gif');
        this.widget_img_url = hub.urls.getImage('widgetSmall.gif');
        this.mashup_img_url = hub.urls.getImage('mashupSmall.gif');
        this.page_img_url = hub.urls.getImage('pageSmall.gif');
    },

    postCreate: function() {
        var labels = dojo.i18n.getLocalization('hub', 'labels');

        // register as an observer
        this.connect(this.model, 'onFetch', '_onBeforeFetch');
        this.connect(this.model, 'onBegin', '_onBeforeLoad');
        this.connect(this.model, 'onItem', '_onItemLoad');
        this.connect(this.model, 'onComplete', '_onAfterLoad');
        this.connect(this.model, 'onError', '_onLoadError');
        this.connect(this.model, 'onStale', '_onStale');
        // watch for bus events about page activation
        this.conns.push(dojo.subscribe('hub.activate-page', this, '_onShow'));


        // column metadata
        this.setColumns([{'name' : labels.CATALOG_COL_TITLE,
                          'class' : 'title',
                          'sortby' : 'title',
                          'markup' : this._getTitleMarkup},
                         {'name' : labels.CATALOG_COL_TAGS,
                          'class' : 'tags',
                          'sortby' : null,
                          'markup' : this._getTagsMarkup},
                         {'name' : labels.CATALOG_COL_UPDATED,
                          'class' : 'updated',
                          'sortby' : 'updated',
                          'markup' : this._getDateMarkup},
                         {'name' : labels.CATALOG_COL_RATING,
                          'class' : 'rating',
                          'sortby' : 'rating',
                          'markup' : this._getRatingMarkup},
                         {'name' : labels.CATALOG_COL_AUTHOR,
                          'class' : 'author',
                          'sortby' : 'author',
                          'markup' : this._getAuthorMarkup}
                        ]);

        // initialize navigation
        this._updateNavBar();

        // interpolate values
        this.inherited(arguments);
    },

    startup: function() {
        if(this.model && this.forceRefresh) {
            this.model.refresh();
        }
    },

    destroy: function() {
        // cleanup all subscriptions
        dojo.forEach(this.conns, dojo.unsubscribe);
        // call base class method
        this.inherited(arguments);
    },

    /**
     * Clears table rows and associated popups.
     */
    clearRows: function() {
        this.inherited(arguments);
        // clear out popups too
        this.popups.innerHTML = '';
    },
    
    /**
     * Render the table columns, navigation, and atom link.
     */
    initialize : function()  {
        // clear everything out
        this.clearHeaders();
        this.clearRows();
        
        for(var i in this.columns) {
            var th = document.createElement('TH');
            // title and base class
            th.innerHTML = this.columns[i]['name'];
            th.className = this.columns[i]['class'];
            if(this.columns[i]['sortby']) {
                // make focusable
                th.tabIndex = 0;
                // mouse and key handler for sorting
                this.connect(th, 'onclick', '_onSort');
                this.connect(th, 'onkeypress', '_onSort');
                // @todo: initialize first column sorting for now, need to be
                //   smarter about what order the results are in
                dojo.removeClass(th, 'selectedUp');
                dojo.removeClass(th, 'selectedDown');
                if(this.model.sortBy == this.columns[i]['sortby']) {
                	if (this.model.sortOrder == 'asc') {
                    	dojo.addClass(th, 'selectedUp');
                    }
                    if (this.model.sortOrder == 'desc') {
                    	dojo.addClass(th, 'selectedDown');
                    }
                }
            }
            this.table_header.appendChild(th);
        }

        // footer status area, built here because we need to know how many 
        // columns
        this.status_row = document.createElement('TD');
        this.status_row.colSpan = this.columns.length;
        this.status_row.className = 'note';
        this.table_footer.appendChild(this.status_row);
    },

    /**
     * Sorts the catalog ascending or descending by the clicked column 
     * header. Pressing space while the header has focus also sorts.
     */
    _onSort : function(event) {
        var src = event.target;

        // if key but not space, don't count as a sort
        if(event.charCode != 0 && event.charCode != null && event.charCode != dojo.keys.SPACE) {
            return;
        }
        
        var index = null;
        var asc = 1;
        // clear all styles
        for(var i in this.table_header.childNodes) {
            var th = this.table_header.childNodes[i];
            if(src == th) {
                index = i;
                if(dojo.hasClass(th, 'selectedUp')) {
                    // switch from up to down
                    dojo.removeClass(th, 'selectedUp');
                    dojo.addClass(th, 'selectedDown');
                    asc = 0;
                } else {
                    // switch from down to up
                    dojo.removeClass(th, 'selectedDown');
                    dojo.addClass(th, 'selectedUp');
                    asc = 1;
                }
            } else {
                // not soting by this column
                dojo.removeClass(th, 'selectedUp');
                dojo.removeClass(th, 'selectedDown');
            }
        }
        
        // fetch the newly sorted results from the server
        // no start or max because a new sort means we're starting at
        // the beginning again anyways
        this.model.fetch(null, null, this.columns[index]['sortby'], asc);
    },
    

    /**
     * Refresh if active tab. Else mark stale for later refresh.
     */
    _onStale: function() {
        var def = hub.isPageActive(this.domNode);
        def.addCallback(function(active) {
            if(active) {
                // refresh immediately
                this.stale = false;
                this.model.refresh();
            } else {
                // mark stale for later refresh
                this.stale = true;
            }
        });
    },

    /**
     * Refresh if stale and the page containing this widget is active.
     */
    _onShow: function(page) {
        if(this.stale && dojo.isDescendant(this.domNode, page.domNode)) {
            this.stale = false;
            this.model.refresh();
        }
    },
    
    /**
     * Show the refreshing message.
     */
    _onBeforeFetch: function() {
        this.clearRows();
        hub.busyMessage(this.status_row, false, "note");
    },
    
    _onBeforeLoad: function(total) {
        // reset row count
        this.row_count = 0;
    },
    
    /**
     * Show the error message.
     */
    _onLoadError: function(error) {
        hub.notify(error, this.status_row, 'TOM_editor_error');
    },

    /**
     * Render all columns of a single item row.
     */        
    _onItemLoad : function(dict) {
        // create popup and save its id
        var pid = this._createPopup(dict);
        // create table row and columns
        var tr = document.createElement('TR');
        
        for(i in this.columns) {
            var func = this.columns[i]['markup'];
            var td = document.createElement('TD');
            //td.style.overflow='hidden';
            var ml = func.call(this, dict);
            td.innerHTML = ml;
            dojo.addClass(td, this.columns[i]['class']);
            tr.appendChild(td);
        }
        
        // alternating row style
        if(this.row_count % 2) {
            dojo.addClass(tr, 'alt');
        }
        
        this.table_body.appendChild(tr);
        this.row_count += 1;
    },
    
    /**
     * Remove the refreshing message. Update the navigation and Atom feed
     * link.
     */        
    _onAfterLoad: function() {
        if(!this.row_count) {
            // no data, say so
            this.status_row.innerHTML = this.msgs.STATUS_CATALOG_NONE;
            this.status_nav.innerHTML = "";
        } else {
            // show a summary in the status row
            this.status_nav.innerHTML = this._getSummaryMarkup();
            this.status_row.innerHTML = "";
        }
        this._updateNavBar();
        this._updateFeedLink();
    },    
    
    /**
     * Builds a status line with start, end, and total information if
     * available.
     */
    _getSummaryMarkup: function() {
        var sum = this.model.getSummary();
        // check what information we have
        var start = sum[0];
        var max = sum[1];
        var total = sum[2];
        if(start != undefined && max != undefined && total != undefined) {
            var end = Math.min(start + max, total);
            return dojo.string.substitute(this.msgs.STATUS_CATALOG_SUMMARY, 
                                          [start+1, end, total]);
        } else {
            return '';
        }
    },
    
    /**
     * Title column content.
     */        
    _getTitleMarkup : function (dict) {
        // encode the title name
        var title = "";
        if (dict.name) {
        	title = hub.htmlspecialchars(dict.name);
        }
        
        // build an icon based on category
        var iconSrc;
        var iconTitle;
        var entryCategory = "";
        if (dict.entryCategory) {
        	entryCategory = dict.entryCategory;
        }
        switch(entryCategory) {
        case 'widget':
        case 'iwidget':
        case 'qedwidget':
            iconSrc = this.widget_img_url;
            iconTitle = this.pops.CATALOG_WIDGET_ICON;
            break;
        case 'mashuppage':
            iconSrc = this.page_img_url;
            iconTitle = this.pops.CATALOG_PAGE_ICON;
            break;    
        case 'mashup':
            iconSrc = this.mashup_img_url;
            iconTitle = this.pops.CATALOG_MASHUP_ICON;
            break;
        default:
            iconSrc = this.feed_img_url;
            iconTitle = this.pops.CATALOG_FEED_ICON;
            break;
        }
        
        // build ID names
        var spanId = this.id+"_entry_"+dict.objectid;
        var popupId = this.id+"_popup_"+dict.objectid+"_popupbanner";
        var popupId2 = this.id+"_popup_"+dict.objectid;
        
        var ml = "<div style=\"overflow:hidden;text-overflow:ellipsis;\">";
        // check if the icon should link to parameters form or 
        // directly to generate URL
        if(dict.parameters && parseInt(dict.parameters)) {
            // has parameters, link to parameters form
            // @todo: how do we show the parameters form?
            var genUrl = hub.config.base_url + '/plugin/params/entryid/'+dict.objectid+'/pluginid/'+dict.pluginid;

            var js = 'javascript:hub.tabs.showTab("'+genUrl+'","'+title+'","'+dict.objectid+'");';
            ml += "<a class='noLink' href='"+js+"' >";
        } else if (dict.downloadURL) { 
            // no parameters, use generate link
            var genUrl = dict.downloadURL;
            
            ml += "<a class='noLink' href='" + hub.urls.secureUrl(genUrl) + "' target='_blank'>";
        }               
        else {
        	ml += "<a class='noLink' href='javascript:void(0)'>";
        }
        ml += "<img width='19px' height='19px' src='"+iconSrc+"' alt='"+iconTitle+"' title='"+iconTitle+"'/></a>";
        
        ml += "&nbsp;<span id='"+spanId+"' ";
        ml += "onmouseover='showPopupLayer2(\""+popupId2+"\",\""+spanId+"\",\""+spanId+"\", PL_DISPLAY_IN_BUBBLE | PL_SUGGESTED_POS_RIGHT | PL_MOUSE_OVER_SUPPORT | PL_ADD_BANNER, \""+popupId+"\")'>";
        // @note: last arg is important; supresses creation of link title popup
        ml += hub.buildPageLink(title, 'Details', {object_id: dict.objectid}, 
                                false, null, '');
        ml += "</span></div>";

        return ml;
    },
    
    /**
     * Tags column content.
     */        
    _getTagsMarkup : function(dict) {
        // bail right away if tags is empty or non-existant
        if(!dict.tags) {
            // make sure we respect styles that require content
            return '&nbsp;';
        }
        var ml = "";
        for(var i=0; i < dict.tags.length; i++) {
            var tag = hub.htmlspecialchars(dict.tags[i]);
            ml += hub.buildPageLink(tag, 'Catalog', {collection: "all", searchType: 'tag', term: tag});
            if (i < dict.tags.length-1) {
                ml += ", ";
            }
        }
        return ml;
    },
    
    /**
     * Date column content.
     */        
    _getDateMarkup : function(dict) {
    	var locDate = "&nbsp;";
    	if (dict.dateModified) {
        	var date = new Date(parseInt(dict.dateModified)*1000);
        	locDate = dojo.date.locale.format(date);
        }
        // in a span with a title in case of overflow
        return '<span title="'+locDate+'">'+locDate+'</span>';
    },
    
    /**
     * Rating column content.
     */        
    _getRatingMarkup: function(dict) {
        var ml = '';
        var value = parseInt(dict.rating);
        
        var w = new hub.widgets.social.view.RatingStarView();
        var urls = w.getStarUrls(value);
        var title = w.getRatingEstimate(value);
        ml += "<span title='"+title+"'>";
        for(var i=0; i < urls.length; i++) {
            ml += "<img src='"+urls[i]+"' />";
        }
        ml += '</span>';
        return ml;
    },
    
    /**
     * Author column content.
     */                
    _getAuthorMarkup : function(dict) {
    	var author = "";
    	if (dict.author) {
    		author = dict.author;
    	}
        var ml = hub.buildPageLink(author, 'Catalog', 
            {collection: "all", searchType: 'user', term: author});
        return ml;
    },
    
    /**
     * Creates all popup controls for titles.
     */                        
    _createPopup : function(dict) {
        var pid = this.id+"_popup_"+dict.objectid;
        var ml = "";
        
        var title = "";
        if (dict.name) {
        	title = hub.htmlspecialchars(dict.name);
        }
        
        var desc = "";
        if (dict.description) {
        	desc = hub.htmlspecialchars(dict.description);
        }
        
        var version = "";
        if (dict.version) {
        	version = hub.htmlspecialchars(dict.version);
        }
        
        ml += "<div id='"+pid+"' class='newPopup'>";
        ml += "<div style='padding:10px' >";
        ml += "<h3 class='clip'><span title='"+title+"'>"+title+"</span></h3>";
        ml += "<b>"+ this.panels.DETAILS_VERSION_TEXT +"&nbsp;</b>"+version;
        ml += "<p class='clip'><span title='"+desc+"'>"+desc+"</span></p>";
        ml += hub.buildPageLink(title, 'Details', 
                                {object_id: dict.objectid}, false, 
                                this.links.CATALOG_VIEW_DETAILS);
		
		//display addtomashups link if user is login
		if (hub.config.username && (dict.entryCategory == "feed" || dict.entryCategory == "mashup" || dict.entryCategory == "iwidget" || dict.entryCategory == "mashuppage")) {

			var serverUrl = hub.config.palette_url ? hub.config.palette_url : mm.getDefaultMMServerUrl();
			var templateName = "";
			
			ml += "&nbsp;|&nbsp;";
			
			// if is package page
			if (dict.entryCategory == "mashuppage" && dict.definitionURL) {
				// is mashuppage url
				ml += "<a target='_blank' href='" + dict.definitionURL + "'>" + this.links.ACTION_VIEW_PAGE_LINK + "</a>";
			} else {
				//the template to show
				if (!hub.config.palette_url) {
					templateName = "SelectMashupsServer";
				} else if (dict.entryCategory == 'feed' || dict.entryCategory == 'mashup') {
					templateName = "AddFeedToMashups";
				} else if (dict.entryCategory == 'iwidget') {
					templateName = "AddIWidgetToMashups";
				} else if (dict.entryCategory == 'mashuppage'){
					templateName = "AddMashupPageToMashups";
				};
				
				ml += hub.buildPageLink(title, templateName, {
											object_id: dict.objectid,
											object_title: dict.name,
											type: dict.entryCategory,
											server: serverUrl
										}, false, this.links.ACTION_ADD_TO_MASHUPS);
			}
		}
		
        ml += "</div></div>";
        this.popups.innerHTML += ml;
        return pid;
    },

    /**
     * Updates the link to the Atom feed content.
     */                
    _updateFeedLink : function() {
        this.feed_link.href = this.model.getUrl();
    },
    
    /**
     * Updates the navigation bar links, disabling next/prev as needed. Visibly
     * enables/disables as well as functionally.
     */                        
    _updateNavBar : function() {
    
        if(this.model.hasPrevious()) {
            this.previous_button.setDisabled(false);
        } else {
            this.previous_button.setDisabled(true);
        }        
       
        if(this.model.hasNext()) { 
            this.next_button.setDisabled(false);
        } else {
            this.next_button.setDisabled(true);
        }        
    },

    /**
     * Fetches the previous page of catalog items.
     */
    _onPrevious: function(event) {
        try {
            this.model.fetchPrevious();
        } catch(e) {}
    },

    /**
     * Fetches the next page of catalog items.
     */    
    _onNext: function(event) { 
        try {
            this.model.fetchNext();
        } catch(e) {}
    },

    /**
     * Refreshes the current page of catalog items.
     */    
    _onRefresh: function(event) {
        this.model.refresh();
    }
});

}

if(!dojo._hasResource['hub.submit.edit']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.submit.edit'] = true;
dojo.provide('hub.submit.edit');




    
/**
 * Submits metadata required to save an entry.
 */
dojo.declare('hub.submit.edit.Save', hub.submit.Form, {
    // URL will be provided by caller, not hardcoded
    validate : function() {
		console.log('hub.submit.edit.Save.validate');
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
		    if (!this._validateRequired(title,
	                msgs.ERROR_SAVE_TITLE, this.status)) {
		        return false;
		    }
		    if (!this._validateRequired(description,
	                msgs.ERROR_SAVE_DESCRIPTION, this.status)) {
		        return false;
		    }
		    if (!this._validateRequired(version, 
	                msgs.ERROR_SAVE_VERSION, this.status)) {
		        return false;
		    }
        }
        return true;
    },
    
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Save: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            // make sure the title of the tab is up to date
            // @todo: unsafe, user might change after submit
            tab = hub.tabs.moveTab(tab, this.form.title.value);
            // notify other components
            dojo.publish("hub.save-entry", [object_id]);
            
            // set a flag indicating we're at the end of the workflow;
            // don't destroy it though in case the user wants to go back
            // and continue editing
            tab.getAttribute('workflow').setComplete();
            
            // go to the landing page
            var object_id = tab.getAttribute('object_id');
            hub.showPage(this.form.title.value, 'Landing', 
                         {object_id: object_id, current: true, safe: true});
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits data to select a new entry type.
 */
dojo.declare('hub.submit.edit.New', hub.submit.Form, {

	workflow: null,
	plugin_id: 0,

    constructor : function(url, workflow) {
    	this.url = url;
    	this.workflow = workflow;
    	this.plugin_id = workflow.plugin_id;
    	this.content={pluginid: workflow.plugin_id};
    	this.form = workflow.plugin_dedicated?null:workflow.selectPluginForm.domNode;
    },
        
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('New: empty response');
        } else if(data.code < 0) {
            var labels = dojo.i18n.getLocalization('hub', 'labels');

            // generate a reasonable title
            var title = labels.UNTITLED_TAB + " " + data.result.object_id;
           	this.workflow.lastResponse = data;
           	this.workflow.entry_id = data.result.object_id;
           	hub.renamePage(this.workflow.tab, title);
           	this.workflow.plugin = new hub.managers.InvokePlugin(this.plugin_id, this.workflow.entry_id, this.workflow);
	        this.workflow.appendScreenHTML(this.workflow.lastResponse.result['html']);
           	dojo.publish('workflow-next-'+this.workflow.tab.id ,[]);
        } else {
            // error, show the message
            hub.notify(data.result, this.status);
        }
        
        return data.code;
    }
});

/**
 * Submits a request for deletion without confirming it.
 */
dojo.declare('hub.submit.edit.DeleteAlways', hub.submit.Form, {
    _onLoad : function(response, tab) {    
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('DeleteAlways: empty response');
        } else {
            if (data.code < 0) {
	            // success, close the tab
	            try {
	                hub.closePage(dojo.byId( this.status ));
	            } catch(e) {
	                // tab might already be closed
	            }
	        } else {
	            hub.notify(data.result, this.status, 'TOM_editor_msg');
	        }
	    }
        // fire an event so all tabs refresh
        dojo.publish('hub.delete-entry', [data.entryid]);
        return data.code;
	}
});

/**
 * Submits a request for deletion without confirming it.
 */
dojo.declare('hub.submit.edit.Cancel', hub.submit.Request, {
    _onLoad : function(response, tab) {    
        return -5;
	}
});

/**
 * Submits a deletion request after confirm it with the user.
 */
dojo.declare('hub.submit.edit.DeleteConfirm', hub.submit.edit.DeleteAlways, {
	getContent: function() {
		return "";
	},
    // URL will be provided by caller, not hardcoded
    validate: function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        return confirm(msgs.PROMPT_DELETE);
    }
});

/**
 * Submits a request to toggle the favorite status.
 */
dojo.declare('hub.submit.edit.FavoritesForm', hub.submit.Form, {
	getContent: function() {
		return "";
	},
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('FavoritesForm: empty response');
	    }
// TODO just change text content this.status
        if (data.code < 0) {
        	var tab = hub.widgets.tabs.getTabForNode(this.status);
            tab.refresh();
	    } else {
//			hub.notify(data.result, this.status, 'TOM_editor_msg');
		}
        return data.code;
	}
});

}

if(!dojo._hasResource['hub.submit.social']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.submit.social'] = true;
dojo.provide('hub.submit.social');




/**
 * Submits a rating for an entry.
 */
dojo.declare('hub.submit.social.Rating', hub.submit.Form, {
    getURL: function() {
        return hub.urls.postRating(this.url);
    },
    
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Rating: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            // notify other components, url holds the object id
            dojo.publish("hub.rate-entry", [this.url]);
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});


/**
 * Submits tags for an entry.
 */
dojo.declare('hub.submit.social.Tags', hub.submit.Form, {
    getURL: function() {
        return hub.urls.postTags(this.url);
    },
    
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Tags: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
            // notify other components, url holds the object id
            dojo.publish("hub.tag-entry", [this.url]);
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a comment for an entry.
 */
dojo.declare('hub.submit.social.Comment', hub.submit.Form, {
    getURL : function() {
        return hub.urls.postComment(this.url);
    },

    validate: function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
	    if (!this._validateRequired(commentTitle, 
                msgs.ERROR_COMMENT_TITLE, this.status)) {
	        return false;
	    }
	    if (!this._validateLength(commentText, 1, 500,
                msgs.ERROR_COMMENT_BODY, this.status)) {
	        return false;
	    }
        }
        return true;
    },
    
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if(!data) {
            throw new Error('Comment: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
            // notify other components
            dojo.publish("hub.comment-entry", [this.url]);
            this.form.reset();
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Handles validation of the user profile form.
 */
dojo.declare('hub.submit.social.Profile', hub.submit.Form, {
    methods : {
        getURL : function() {
            return hub.urls.postProfile();
        },

        validate : function() {
            var msgs = dojo.i18n.getLocalization('hub', 'messages');
            with (this.form) {
	        if (!this._validateRequired(profileName,
                    msgs.ERROR_REGISTER_NAME, this.status)) {
	            return false;
	        }
	        if (!this._validateRequired(profileEmail,
                    msgs.ERROR_REGISTER_EMAIL, this.status)) {
	            return false;
	        }
                // blank password ok, means no change
                if (profilePassword.value != profilePassword2.value) {
                    hub.notify(msgs.ERROR_REGISTER_PASSWORD_MATCH, this.status);
                    profilePassword.value = "";
                    profilePassword2.value = "";
                    profilePassword.focus();
                    return false;
                }
            }
            return true;
        },

        _onLoad : function(response, tab) {
            var code = this.inherited(arguments);
            if(code < 0) {
                // notify other components
                dojo.publish("hub.update-profile", [this.form.profileName.value]);
            }
            return code;
        }
    }
});


}

if(!dojo._hasResource['hub.widgets.social.edit']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.widgets.social.edit'] = true;
dojo.provide('hub.widgets.social.edit');






 // for tag submit





/**
 * Rating chooser widget. Users clicks stars or uses the keyboard to set a 
 * rating value.
 */
dojo.declare('hub.widgets.social.edit.RatingChooser', dijit._Widget, {
    // icon to show when mouse/keyboard selection hovers over a star
    hoverIcon: "yellowstar.gif",
    // icon for full star
    fullIcon: "bluestar.gif",
    // icon for half start
    halfIcon: "bluehalfstar.gif",
    // icon for no star
    emptyIcon: "icon-rated-empty.gif",
    // number of stars possible
    maxRating: 5,
    // initial rating value
    value: 0,
    // object we're rating
    objectid: null,
    // form tag id (it is required by 'setTabBusy' function)
    formid: null,

    constructor: function(args, node) {
        // do we have to submit when losing focus?
        this.submit_on_blur = false;
        // store references to stars DOM nodes
        this.stars = null;
    },

    postMixInProperties: function() {
        if(this.objectid == null) {
            throw new Error("RatingChooser: objectid required");
        }
        // configure and reuse the rating star view renderable
        var args = {fullIcon : this.fullIcon,
            emptyIcon: this.emptyIcon,
            halfIcon : this.halfIcon,
            maxRating: this.maxRating};
        this.star_view = new hub.widgets.social.view.RatingStarView(args);
        this.inherited(arguments);
    },

    /**
     * Lay down the widget elements once. Thereafter, only refresh the image
     * URLs, not the entire DOM nodes.
     */
    buildRendering: function() {
        this.inherited(arguments);
        // use value from this object
        var urls = this.star_view.getStarUrls(this.value);
        var est = this.star_view.getRatingEstimate(this.value);
        this.stars = this.star_view.renderStarImages(urls);

        var box = document.createElement('SPAN');
        var _self = this;
        var index = 1;
        dojo.forEach(this.stars, function(star) {
            box.appendChild(star);
            // register event handlers on each star
            _self._registerNodeHandlers(star, index);
            index += 1;
        });
        box.title = est;
        // make sure we can get keyboard focus
        box.tabIndex = 0;
        // register handlers on the parent box
        this._registerHandlers(box);
        // replace attach point with the rendering
        this.domNode.appendChild(box);
    },

    refresh: function() {
        // update URLs without destroying DOM nodes
        var urls = this.star_view.getStarUrls(this.value);
        for(var i=0; i < urls.length; i++) {
            this.stars[i].src = urls[i];
        }

        // don't want the caller to replace its content
        return null;
    },

    _submit: function() {
    	var formnode = dojo.byId(this.formid);
    	formnode.value.value = this.value;
        var form = new hub.submit.social.Rating(formnode, null, this.objectid);
        hub.submit.ajaxSubmit(form, false, true);
    },

    _registerHandlers : function(node) {
        this.connect(node, 'onblur', '_onBlur');
        this.connect(node, 'onmouseout', '_onOut');
        this.connect(node, 'onkeydown', '_onKey');
    },
    
    _registerNodeHandlers : function(node, index) {
        this.connect(node, 'onclick', dojo.hitch(this, '_onClick', index));
        this.connect(node, 'onmouseover', dojo.hitch(this, '_onOver', index));
    },    
   
    _onOver: function(index, event) {
        var node = event.target;
        for(var i=0; i < index; i++) {
            this.stars[i].src = hub.urls.getImage(this.hoverIcon);
        }
        for(var i=index; i < this.maxRating; i++) {
            this.stars[i].src = hub.urls.getImage(this.emptyIcon);
        }
        node.style.cursor = "pointer";
    },
    
    _onOut: function(event) {
        var node = event.target;
        node.style.cursor = "default";
        // refresh this view
        this.refresh();
    },
    
    _onClick: function(index, event) {
        this.value = index;
        // submit value
        this._submit();
    },
    
    _onBlur: function(event) {
        if(this.submit_on_blur) {
            this.submit_on_blur = false;
            this._submit();
        }
    },
    
    _onKey: function(event) {
        var value = this.value;
        switch(event.keyCode) {
        case dojo.keys.RIGHT_ARROW:
        case dojo.keys.DOWN_ARROW:
            if(value >= this.maxRating) {
                dojo.stopEvent(event);
                return;
            }
            // higher
            this.value += 1;
            this.refresh();
            this.submit_on_blur = true;
            dojo.stopEvent(event);
            return;
        case dojo.keys.LEFT_ARROW:
        case dojo.keys.UP_ARROW:
            if(this.value <= 1) {
                dojo.stopEvent(event);
                return;
            }
            // lower
            this.value -= 1;
            this.refresh();
            this.submit_on_blur = true;
            dojo.stopEvent(event);
            return;
        case dojo.keys.ENTER:
            this.submit_on_blur = false;
            this._submit();
            dojo.stopEvent(event);
            return;
        }
    }
});

/**
 * Tag suggestion widget. Displays suggestions in an area below an input box.
 */
dojo.declare("hub.widgets.social.edit.TagSuggest", [dijit._Widget, dijit._Templated, hub.behaviors.Renderable], {
    // tag separator
    separator: ',',
    // initial tag string for input box
    value: '',
    // object which we're tagging
    objectid: null,
    // template to replace node with
    templateString:"<div class=\"TOM_widget_TagSuggest\">\r\n  <form onsubmit=\"return false;\" dojoAttachPoint=\"tags_form\">\r\n    <input name=\"tags\" type=\"text\" autocomplete=\"off\" maxlength=\"500\" title=\"${pops.TAG_SUGGEST_EXAMPLE}\" dojoAttachEvent=\"onkeypress: _onKeyPress, onkeyup: _onKeyUp\" dojoAttachPoint=\"input_box\"/>\r\n    <button type=\"submit\" dojoType=\"hub.widgets.Button\" dojoAttachEvent=\"onClick: _onSubmit\">${labels.BUTTON_SUBMIT}</button>\r\n  </form>\r\n  <div dojoAttachPoint=\"suggest_box\"></div>\r\n</div>\r\n",
    // dojo.form.button in the template
    widgetsInTemplate: true,

    constructor: function(args, node) {
        this.names = [];
        this.counts = [];
    },

    postMixInProperties: function() {
        if(this.objectid == null) {
            throw new Error("TagSuggest: objectid required");
        }

        // get popup messages
        this.pops = dojo.i18n.getLocalization('hub', 'popups');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');

        // get character value for separator
        this.sep = this.separator.charCodeAt(0);
        // init other values
        this.caret = null;
        this.char_code = null;
        this.suggest = [];

        this.inherited(arguments);
    },

    postCreate: function() {
        this.input_box.value = this.value;
        this.inherited(arguments);
    },

    render: function(tags) {
        // split into names and frequencies
        this.names = new Array(tags.length);
        this.counts = new Array(tags.length);
        var i = 0;
        for(var key in tags) {
            this.names[i] = key;
            this.counts[i] = tags[key];
                i += 1;
        }
        // nothing to render visibly
        return null;
    },

    renderError: function() {
        // don't let model error message ovewrite us
        return null;
    },

    _reset: function() {
        this.caret = null;
        this.char_code = null;
        this.suggest = [];
        this._render(null);
    },

    _onSubmit: function() {
        this.value = this.input_box.value;
        var form = new hub.submit.social.Tags(this.tags_form, this.suggest_box, this.objectid);
        hub.submit.ajaxSubmit(form);
    },
    
    _onKeyPress: function(event) {
        // track last character code
        this.char_code = event.charCode;
    },
    
    _onKeyUp: function(event) {
        // ignore enter key since it's used to submit the form
        if(event.keyCode == dojo.keys.ENTER) {
            return;
        }

        // get the current caret position
        var caret = this._getCaretPosition(this.input_box);
        this.caret = caret;
        if(caret == null) {
            // can't find the caret, just abort
            this._reset();
            return;
        }
        
        // did we enter a separator?
        var is_sep = (this.char_code == this.sep);
        
        // find the chunk at the current caret location
        // offset by one if comma so we don't start on it
        var chunk = this._getCurrentChunk(caret, is_sep);
        if(!chunk.text.length) {
            // no chunk, abort
            this._reset();
            return;
        }
        
        if(is_sep) {
            if(this.suggest.length) {
                // insert the first suggestion if the user types the sep
                //this._replaceChunk(this.input_box, chunk, this.suggest[0], true); // disabled because of BZ #3189
            }
            // reset all suggestions
            this._reset();
            return;
        }
        
        // find the first possible match index in the tags list
        var index = this._findFirstTag(chunk.text);
        // narrow to the first five suggestions
        this.suggest = this._narrowSuggestions(this.input_box, chunk.text, 
                                               index, 5);
        // show the suggestions
        this._render(this.suggest);
    },
    
    _getCaretPosition: function(input) {
        if (input.setSelectionRange) {
            // Mozilla caret position code
            var caret_start = input.selectionStart;
            var caret_end = input.selectionEnd;
        } else {
            // IE caret position code hack
            var caret_start = Math.abs(document.selection.createRange().moveStart("character", -100000));
            var caret_end = Math.abs(document.selection.createRange().moveEnd("character", -100000));
        }
        
        if(caret_start != caret_end) {
            // if there is a selection, don't suggest anything
            return null;
        } else {
            // for convenience
            return caret_start;
        }
    },
    
    _getCurrentChunk: function(caret, avoid_sep) {
        var text = this.input_box.value;
        var chunk = {start: 0, end: 0, text: ''};
        
        // treat caret in terms of character positions, not between them
        //caret = Math.min(caret, text.length-1);
        
        // seek backwards for the previous comma or the start of the text
        var sep_char = String.fromCharCode(this.sep);
        // if avoiding the separator, back up one
        var left = text.lastIndexOf(sep_char, caret - (avoid_sep ? 2 : 1));
        if(left == -1) {
            // no comma, slice from string start
            left = 0;
        } else {
            left += 1;
        }
        // slice and left trim
        text = text.substring(left);
        while(text.charAt(0) == ' ') {
            text = text.substring(1)
            left += 1;
        }
        
        // look for the next comma
        // if avoiding the separator, jump ahead one
        caret = caret - left;
        var right = text.indexOf(sep_char, caret);
        if(right == -1) {
            // no comma, consider the rest of the text
            right = text.length;
        } else {
            // slice and right trim
            text = text.substring(0, right);
            while(text.charAt(text.length-1) == ' ') {
                text = text.substring(0, text.length-1);
            }
        }
        
        // package up all info nicely
        chunk.start = left;
        chunk.end = left + right;
        chunk.text = text;
        return chunk;
    },
    
    _replaceChunk: function(input, chunk, tag) {
        var text = this.input_box.value;
        // slice the string
        var first = text.slice(0, chunk.start);
        var second = text.slice(chunk.end+1);
        // rebuild
        this.input_box.value = first + tag + String.fromCharCode(this.sep) 
            + second;
    },
    
    /**
     * Binary search to find the first location where a tag would appear
     * in the list of tags.
     */
    _findFirstTag: function(prefix) {
        var low = 0;
        var high = this.names.length;
        var mid, last;
        while (low <= high) {
            mid = parseInt((low + high) / 2);
            if (this.names[mid] > prefix) {
                // target is before the mid point
                high = mid - 1;
                last = 0;
            } else if (this.names[mid] < prefix) {
                // target is after the mid point
                low = mid + 1;
                last = 1;
            } else {
                // target is the midpoint
                return mid;
            }
        }
        // if we make it here, the mid point is as close to the target
        // as we can get
        return mid + last;
    },
    
    /**
     * Seeks ahead linearly from the starting index and returns up to 
     * max strings starting with the given text.
     */
    _narrowSuggestions: function(input, prefix, start, max) {
        var text = input.value;
        var length = prefix.length;
        var suggest = new Array();
        for(var i=start; i<this.names.length; i++) {
            var tag = this.names[i];
            // only consider as suggestion if the prefix matches, 
            // if it's not already in the text, or if it's an exact match
            // on the current chunk
            if(tag.substr(0, length) == prefix && 
               text.indexOf(tag) == -1) {
                suggest.push(tag);
            }
            if(suggest.length >= 5) {
                break;
            }
        }
        return suggest;
    },
    
    /**
     * Render the list of suggestions as clickable links. Different from 
     * render() which is how the hub.behaviors.Renderables receive data to 
     * render from other widgets.
     */
    _render: function(suggest) {
        // make sure the space doesn't collapse
        this.suggest_box.innerHTML = '&nbsp;';
        // error messages might set weird classes, remove them
        this.suggest_box.className ='';
        if(!suggest || !suggest.length) 
            return;
        // put the title on the suggestion list
        var b = document.createElement('SPAN');
        var text = document.createTextNode(this.pops.TAG_SUGGEST_TITLE);
        b.appendChild(text);
        this.suggest_box.appendChild(b);
        for(i in suggest) {
            var a = document.createElement('A');
            a.innerHTML = hub.htmlspecialchars( suggest[i] );
            a.href = "#";
            // @todo: need to clean these up else we leak memory!
            a.onclick = dojo.hitch(this, this.insert, i);
            this.suggest_box.appendChild(a);
            if(i < suggest.length - 1) {
                var sep = document.createTextNode(", ");
                this.suggest_box.appendChild(sep);
            }
        }
    },
    
    /**
     * Insert the suggestion at index into the text area.
     */
    insert: function(index) {
        // get the current caret position
        //var caret = this._getCaretPosition(this.input);
        var caret = this.caret;
        if(caret == null) {
            // can't find the caret, just abort
            this._reset();
            return;
        }
        
        // find the chunk at the current caret location
        var chunk = this._getCurrentChunk(caret);
        if(!chunk.text.length) {
            // no chunk, abort
            this._reset();
            return;
        }
        
        // replace the current chunk
        var tag = this.suggest[index];
        this._replaceChunk(this.input_box, chunk, tag);
        this._reset();
        this.input_box.focus();
    }
});

dojo.declare("hub.widgets.social.edit.TagSuggestLite", [hub.widgets.social.edit.TagSuggest], {
	inputName: 'tags',
    templateString:"<div class=\"TOM_widget_TagSuggest\">\r\n  <input name=\"${inputName}\" style=\"width: 32em;\" type=\"text\" value=\"${value}\" autocomplete=\"off\" maxlength=\"500\" title=\"${pops.SAVE_TAGS}\" dojoAttachEvent=\"onkeypress: _onKeyPress, onkeyup: _onKeyUp\" dojoAttachPoint=\"input_box\"/>\r\n  <div dojoAttachPoint=\"suggest_box\"></div>\r\n</div>\r\n"
});

}

if(!dojo._hasResource["hub.pages.edit"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.pages.edit"] = true;
dojo.provide("hub.pages.edit");










/**
 * Save page shown at the end of the wizard. Fetches metadata about the entry
 * being saved from the server to pre-populate the form fields.
 */
dojo.declare('hub.pages.edit.Save', hub.pages._Page, {
    // template to replace node with
    templateString:"<div class='TOM_panel'>\r\n  <form id='saveEntry_{{ entry_id }}' onsubmit=\"alert(this);\">\r\n      <h1>{{ panels.SAVE_TITLE }}</h1>\r\n\t       \r\n      <label for='title'><span class='required'>*</span> {{ panels.SAVE_TITLE_LABEL }}</label>\r\n      <div class=\"rightCol\">\r\n        <input name='title' style=\"width: 32em;\" maxlength='200' type='text' value=\"{{ title|default:'' }}\" title=\"{{ popups.SAVE_TITLE }}\"/><br />\r\n      </div>\r\n      \r\n      <label for='description'><span class='required'>*</span> {{ panels.SAVE_DESCRIPTION_LABEL }}</label>\r\n      <div class=\"rightCol\">\r\n        <textarea name='description' style=\"width: 32em;\" maxlength='1000' title=\"{{ popups.SAVE_DESCRIPTION }}\">{{ description }}</textarea>\r\n      </div>\r\n      \r\n      <label for='version'><span class='required'>*</span> {{ panels.SAVE_VERSION_LABEL }}</label>\r\n      <div class=\"rightCol\">\r\n        <input name='version' style=\"width: 32em;\" maxlength='20' type='text' value=\"{{ version|default:'1.0' }}\" title=\"{{ popups.SAVE_VERSION }}\" /><br />\r\n      </div>\r\n      \r\n      <label for='tags'>{{ panels.SAVE_TAGS_LABEL }}</label>\r\n      <div class=\"rightCol\">\r\n\t\t<div dojoType=\"hub.widgets.social.view.SocialBox\" model=\"hub.tags\" class=\"TOM_widget_TagsBox\" showRefresh=\"false\">\r\n\t\t\t<div inputName='tags' value=\"{{ tags }}\" dojoType=\"hub.widgets.social.edit.TagSuggestLite\" objectid=\"{{ entry_id }}\" ></div>\r\n\t\t</div>\r\n      </div>\r\n      \r\n      \r\n      <label for='tags'>{{ panels.SAVE_PERMISSIONS_LABEL }}</label>\r\n      <div class=\"rightCol\" dojoType=\"hub.widgets.edit.AclAssign\" entryid=\"{{ entry_id }}\"></div>\r\n\r\n      <label>{{ panels.SAVE_ADVANCED_LABEL }}</label>\r\n      <div class=\"rightCol\">\r\n\t    <table style=\"padding: 0px; border: 0px none; margin: 0px;\"><tr><td>\r\n\t      <div style=\"width: 32em;\" dojoType=\"dijit.TitlePane\" title=\"{{ panels.SAVE_CATEGORIES_LABEL }}\" open=\"false\">\r\n\t\t      <div dojoType=\"hub.widgets.adminobjectscatalog.Categories\" entryid=\"{{ entry_id }}\"></div>\r\n\t      </div>    \r\n\t\r\n\t      <div style=\"width: 32em;\" dojoType=\"dijit.TitlePane\" title=\"{{ panels.SAVE_RELATED_LABEL }}\" open=\"false\">\r\n\t\t      <div dojoType=\"hub.widgets.adminobjectscatalog.Related\" entryid=\"{{ entry_id }}\"></div>\r\n\t      </div>    \r\n\t\r\n\t      <div style=\"width: 32em;\" dojoType=\"dijit.TitlePane\" title=\"{{ panels.SAVE_DOCUMENTATION_LABEL }}\" open=\"false\">\r\n\t\t\t\t<label class=\"unsized\" for=\"docurl\">{{ panels.SAVE_DOCUMENTATION_URL }}</label>\r\n\t\t\t\t<input type=\"text\" title=\"{{ popups.SAVE_DOC_URL }}\" value=\"{{ doc }}\" maxlength=\"100\" name=\"docurl\" style=\"width: 25em;\"/>\r\n\t      </div>    \r\n\r\n\t      <div style=\"width: 32em;\" dojoType=\"dijit.TitlePane\" title=\"{{ panels.SAVE_CACHE_LABEL }}\" open=\"false\">\r\n{% if useCache %}\r\n\t\t\t\t<label class=\"unsized\" for=\"usecache\">{{ panels.SAVE_CACHE_USE_LABEL }}</label>\r\n\t\t\t\t<input type=\"checkbox\" title=\"{{ popups.SAVE_CACHE_USE }}\" name=\"usecache\" onChange=\"if(this.checked){this.form.lifetime.disabled=false;}else{this.form.lifetime.disabled=true;}\" checked=\"checked\" />\r\n\t\t\t\t<br />\r\n\t\t\t\t<br />\r\n\t\t\t\t<label class=\"unsized\" for=\"lifetime\">{{ panels.SAVE_CACHE_LIFETIME_LABEL }}</label>\r\n\t\t\t\t<br />\r\n\t\t\t\t<input type=\"text\" title=\"{{ popups.SAVE_CACHE_LIFETIME }}\" value=\"{{ lifeTime }}\" maxlength=\"20\" name=\"lifetime\" style=\"width: 9em;\"/>\r\n{% else %}\r\n\t\t\t\t<label class=\"unsized\" for=\"usecache\">{{ panels.SAVE_CACHE_USE_LABEL }}</label>\r\n\t\t\t\t<input type=\"checkbox\" title=\"{{ popups.SAVE_CACHE_USE }}\" name=\"usecache\" onChange=\"if(this.checked){this.form.lifetime.disabled=false;}else{this.form.lifetime.disabled=true;}\" />\r\n\t\t\t\t<br />\r\n\t\t\t\t<br />\r\n\t\t\t\t<label class=\"unsized\" for=\"lifetime\">{{ panels.SAVE_CACHE_LIFETIME_LABEL }}</label>\r\n\t\t\t\t<br />\r\n\t\t\t\t<input type=\"text\" title=\"{{ popups.SAVE_CACHE_LIFETIME }}\" value=\"\" maxlength=\"20\" name=\"lifetime\" style=\"width: 9em;\" disabled=\"disabled\" />\r\n{% endif %}\r\n\t      </div> \r\n\t      </td></tr></table>  \r\n\t         \r\n      </div>\r\n      <div class=\"rightCol\">\r\n        <p id=\"saveEntryMessage_{{ id }}\">{{ message|default\"\" }}</p>\r\n      </div>\r\n  </form>\r\n</div>\r\n",
    entry_id: 0,

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
    	if(server_data.object.isSaved){
	    	hub.renamePage(this.tab, server_data.object.title, {});
    	}
        return {entry_id: this.entry_id,
        		legend: 'legend',
        		title: server_data.object.title,
        		description: server_data.object.description,
        		version: server_data.object.version,
        		tags: server_data.mytags, // where are tags?
        		doc: server_data.object.doc?server_data.object.doc:'',
        		useCache: server_data.object.useCache,
        		lifeTime: server_data.object.lifeTime,
		        panels: dojo.i18n.getLocalization('hub', 'panels'),
		        popups: dojo.i18n.getLocalization('hub', 'popups'),
                labels: dojo.i18n.getLocalization('hub', 'labels')};
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getDetailsPage(this.entry_id);
    }
});

/**
 * Landing page shown after the save page with links to where the user can go
 * next. Fetches data from the server about the entry so it can build a
 * proper set of links based on the type (e.g., mashup versus widget).
 */
dojo.declare('hub.pages.edit.Landing', hub.pages._Page, {
    // template to replace node with
    templateString:"<div class=\"TOM_panel\" >\r\n  <h1 dojoAttachPoint=\"heading\">{{ landingTitle }}</h1>\r\n  <p>{{ subtitle }}</p>\r\n  <div class=\"taskbox\">\r\n    <h2>{{ panels.LANDING_NEXT_TASKS }}</h2>\r\n    <ul class=\"TOM_action_list\">\r\n    {% if hasPromptParams %}\r\n      <li><a href=\"{{ url }}\" title='{{ taskGenerateTip }}'>{{ taskGenerate }}</a></li>\r\n    {% else %}\r\n      <li><a href=\"{{ url }}\" target=\"_blank\" title='{{ taskGenerateTip }}' >{{ taskGenerate }}</a></li>\r\n    {% endif %}\r\n      <li><a href='javascript:hub.showPage(\"{{ title }}\",\"Details\",{\"object_id\": \"{{ entry_id }}\"})' title='{{ taskDetailsTip }}'>{{ taskDetails }}</a></li>\r\n      <!--li><a href='javascript:hub.managers.plugins.hub_plugins_{{ plugin_id }}_{{ entry_id }}.workflow.showSave()' >{{ panels.LANDING_TASK_EDIT_METADATA }}</a></li>\r\n      <li><a href='javascript:hub.managers.plugins.hub_plugins_{{ plugin_id }}_{{ entry_id }}.workflow.showPlugin()' >{{ panels.LANDING_TASK_EDIT_SOURCE }}</a></li-->\r\n      <li><a href='{{ editInfoUrl }}' title='{{ taskMetaDataTip }}' >{{ taskMetaData }}</a></li>\r\n      <li><a href='{{ editSourceUrl }}' title='{{ taskSourceTip }}' >{{ taskSource }}</a></li>\r\n      <!-- li><a href=\"\" >{{ panels.LANDING_TASK_USE }}</a></li-->\r\n      \r\n      {% if object.isFeed %}\r\n        <li><a href=\"javascript:hub.showPlugin('{{ labels.HOME_NEW_FEED }}', 0, 0, {type: 'Feeds'});\" title='{{ popups.LANDING_CREATE_ANOTHER_FEED }}' >{{ taskCreateAnother }}</a></li>\r\n      {% endif %}\r\n      \r\n      {% if object.isMashup %}\r\n        <li><a href=\"javascript:hub.showPlugin('{{ labels.HOME_NEW_MASHUP }}', {{ plugin_id }}, 0, {type: 'Mashups', fullscreen: true});\" title='{{ popups.LANDING_CREATE_ANOTHER_MASHUP }}' >{{ taskCreateAnother }}</a></li>\r\n      {% endif %}\r\n      \r\n      {% if object.isWidget %}\r\n        <li><a href=\"javascript:hub.showPlugin('{{ labels.HOME_UPLOAD_WIDGET }}', 0, 0, {type: 'Widgets'});\" title='{{ popups.LANDING_UPLOAD_ANOTHER_WIDGET }}' >{{ taskUploadAnother }}</a></li>\r\n      {% endif %}\r\n      \r\n      {% if object.isPage %}\r\n        <li><a href=\"javascript:hub.showPlugin('{{ labels.HOME_UPLOAD_WORKSPACE }}', {{ plugin_id }}, 0, {type: 'pages'} );\" title='{{ popups.LANDING_UPLOAD_ANOTHER_PAGE }}' >{{ taskUploadAnother }}</a></li>\r\n      {% endif %}\r\n      \r\n      <li><a href='javascript:hub.closeCurrentPage();' title='{{ popups.LANDING_CLOSE_TAB }}' >{{ panels.LANDING_TASK_CLOSE }}</a></li>\r\n    </ul>\r\n  </div>\r\n</div>\r\n",
   	entry_title:'',

    // Note:  objectType is from package.xml and it is not a translatable string
    //        feed, mashup, widget, page, etc.
     getObjectTypeForDisplay: function(objectType) {
       if (objectType == null) return this.labels.NEW_FEED_TAB;
       var result = objectType;

       if (objectType == 'mashup') {
          result = this.labels.NEW_MASHUP_TAB;
       } 
       else if (objectType == 'widget' || objectType == 'iwidget') {
          result = this.labels.NEW_WIDGET_TAB;
       }
       else if (objectType == 'page' || objectType == 'mashuppage') {
          result = this.labels.NEW_PAGE_TAB;
       }
       else {
          result = this.labels.NEW_FEED_TAB;
       }
       
       return result;
    },
    
    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.popups = dojo.i18n.getLocalization('hub', 'popups');
    	this.entry_title = server_data.object.title;
    	hub.renamePage(this.tab, this.entry_title, {});
    	var sourceString = (server_data.object.source!=null)?server_data.object.source.toLowerCase():"unknown";
    	var objectType = (server_data.object.objectType!=null)?server_data.object.objectType.toLowerCase():"object";
    	var otTitle = this.getObjectTypeForDisplay(objectType);
    	var otDisplayName = otTitle.toLowerCase();
		if (server_data.object.externalURL) {
			server_data.object.secureURL = hub.urls.secureUrl(server_data.object.externalURL);
		}
        return {entry_id: this.entry_id,
                plugin_id: this.plugin_id,
        		title: hub.escapeQuotes( server_data.object.title ),
        		landingTitle: dojo.string.substitute(this.panels.LANDING_TITLE, [otTitle]),
                subtitle: dojo.string.substitute(this.panels.LANDING_SUBTITLE, [otDisplayName, server_data.object.title]),
                taskGenerate: dojo.string.substitute(this.panels.LANDING_TASK_GENERATE, [otDisplayName]),
                taskDetails: dojo.string.substitute(this.panels.LANDING_TASK_DETAILS, [otDisplayName]),
                taskMetaData: dojo.string.substitute(this.panels.LANDING_TASK_EDIT_METADATA, [otDisplayName]),
                taskSource: dojo.string.substitute(this.panels.LANDING_TASK_EDIT_SOURCE, [otDisplayName]),
                taskCreateAnother: dojo.string.substitute(this.panels.LANDING_TASK_CREATE_ANOTHER, [otDisplayName]),
                taskUploadAnother: dojo.string.substitute(this.panels.LANDING_TASK_UPLOAD_ANOTHER, [otDisplayName]),
                taskGenerateTip: dojo.string.substitute(this.popups.LANDING_GENERATE, [otDisplayName]),
                taskDetailsTip: dojo.string.substitute(this.popups.LANDING_DETAILS, [otDisplayName]),
                taskMetaDataTip: dojo.string.substitute(this.popups.LANDING_EDIT_METADATA, [otDisplayName]),
                taskSourceTip: dojo.string.substitute(this.popups.LANDING_EDIT_SOURCE, [otDisplayName]),
                object: server_data.object,
                base_url: hub.config.base_url,
                editSourceUrl: hub.getCurrentTab().workflow==null || hub.getCurrentTab().workflow.pluginStartScreen==null?
                	'javascript:hub.showPlugin("'+hub.escapeQuotes( server_data.object.title )+'", '+this.plugin_id+', '+this.entry_id+', {editSource: true, current: true})':
                	'javascript:hub.getCurrentTab().workflow.showPlugin()',
                editInfoUrl:hub.getCurrentTab().workflow==null || hub.getCurrentTab().workflow.saveScreen==null?
                	'javascript:hub.showPlugin("'+hub.escapeQuotes( server_data.object.title )+'", '+this.plugin_id+', '+this.entry_id+', {current: true})':
                	'javascript:hub.getCurrentTab().workflow.showSave()',
                url:(server_data.object.hasPromptParams?
                	'javascript:hub.showUrl("Parameters","'+hub.urls.getParametersPage(this.entry_id,this.plugin_id)+'", {entryId: '+this.entry_id+'});':
                	server_data.object.secureURL),
                hasPromptParams: server_data.object.hasPromptParams,
                labels: this.labels,
                popups: this.popups,
                panels: this.panels};
    },

/*    postCreate: function(){
    	this.inherited('pstCreate', arguments);
//    	hub.renamePage(this.heading, this.entry_title, {});
    },*/

    /**
     * Gets an object indicating this page should no longer be permanent. The
     * save page has been submitted, so the user can now close the page without
     * giving confirmation.
     */
    getProperties: function() {
        return {permanent: false};
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
//        return hub.urls.getLandingPage();
        return hub.urls.getDetailsPage(this.entry_id);
    }
});

/**
 * Feed chooser page shown when the user decides to create or register a new
 * feed. No server data needed.
 */
dojo.declare('hub.pages.edit.FeedChooser', hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function(response) {
        // @todo
        alert(dojo.toJson(response.payload.categories));
        return {panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels'),
                categories: response.payload.categories};

    },

    getUrl: function(){
    	return '../plugin/editor/source/Feeds';
    }
});

/**
 * Page collecting user name and password. Can be shown by plug-ins to get
 * credentials. Fetches any previously entered username and password from the
 * server.
 */
dojo.declare('hub.pages.edit.ProtectedResource', hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getProtectedResourcePage();
    }
});

/**
 * Page collecting user name and password, plus database host, port, name, and
 * type. Can be shown by plug-ins. Fetches any previously defined and/or
 * selected databases from the server.
 */
dojo.declare('hub.pages.edit.DatabaseConnection', hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getDatabaseConnectionPage();
    }
});

}

if(!dojo._hasResource['hub.widgets.edit']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.widgets.edit'] = true;
dojo.provide('hub.widgets.edit');








/**
 * Builds and handles all the UI for the permissions editor
 * portion of the 'edit details' page.
 */
dojo.declare('hub.widgets.edit.Connection', [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templateString:"<div class=\"TOM_widget_Connection\">\r\n\r\n  <label for=\"datasource\"><span class=\"required\">*</span> ${labels.CONN_SELECT}: </label>\r\n  <div class=\"rightCol\">\r\n    <select name=\"datasource\" dojoAttachPoint=\"node_source\" \r\n      dojoAttachEvent=\"onchange: onSelectConnection\">\r\n      <!-- Saved connections added here -->\r\n    </select>\r\n  </div>\r\n  <br/><br/>\r\n\r\n  <label for=\"username\"><span class=\"required\">*</span> ${labels.CONN_USER}: </label>\r\n  <div class=\"rightCol\">\r\n    <input name=\"username\" dojoAttachPoint=\"node_username\" size=\"40\" type=\"text\" value=\"\" /><br/>\r\n  </div>\r\n\r\n  <label for=\"password\"><span class=\"required\">*</span> ${labels.CONN_PWD}: </label>\r\n  <div class=\"rightCol\">\r\n    <input name=\"password\" dojoAttachPoint=\"node_password\" size=\"40\" type=\"password\" value=\"\" /><br/>\r\n  </div>\r\n\r\n</div>\r\n",    
    conns: '',

    /**
     * Called before page loads. Fetch all localized text.
     */ 
    postMixInProperties: function() {
        //this.panels = dojo.i18n.getLocalization('hub', 'panels');
        //this.links = dojo.i18n.getLocalization('hub', 'links');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
    },
    
    /**
     * Called after page loads.
     */
    postCreate: function() {
        this.setConnections(dojo.fromJson(this.conns));
    },
    
    /**
     * Add connections to the connection list
     *
     * @param conns: array of connection objects
     */
    setConnections: function(conns) {
        this.node_source.innerHTML = "<option value='-1'>New...</option>"
        for(var i in conns) {
            var option = document.createElement('option');
            option.value = conns[i].id;
            option.innerHTML = conns[i].name;
            this.node_source.appendChild(option);
        }        
        this.conns = conns;
    },

    /**
     * Called when a connection is selected. Populates the
     * other fields on the page based on the connection selected
     */
    onSelectConnection: function(event) {
        if(!this.conns) return;
        
        var selectedIndex = event.target.selectedIndex;
        var connId = event.target.options[selectedIndex].value;
        
        if(connId == "-1") {
            this.reset();
            return;
        }
    
        for(var i=0; i<this.conns.length; i++) {
            if(this.conns[i].id != connId)
                continue;
            this.node_username.value = this.conns[i].username;
            this.node_password.value = this.conns[i].password;
        }
    },
    
    /**
     * Reset the connection fields in the form
     */
    reset: function() {
        this.node_source.selectedIndex = 0;
        this.node_username.value = "";
        this.node_password.value = "";
    }
    
});

/**
 * Builds and handles all the UI for the permissions editor
 * portion of the 'edit details' page.
 */
dojo.declare('hub.widgets.edit.DatabaseConnection', hub.widgets.edit.Connection, {
    // template to replace node with
    templateString:"<div class=\"TOM_widget_DatabaseConnection\">\r\n\r\n  <label for=\"datasource\"><span class=\"required\">*</span> ${labels.CONN_SELECT}: </label>\r\n  <div class=\"rightCol\">\r\n    <select name=\"datasource\" dojoAttachPoint=\"node_source\" \r\n      dojoAttachEvent=\"onchange: onSelectConnection\">\r\n      <!-- Saved db connections added here -->\r\n    </select>\r\n  </div>\r\n  <br/><br/>\r\n\r\n  <label for=\"dbtype\"><span class=\"required\">*</span> ${labels.DBCONN_TYPE}: </label>\r\n  <div class=\"rightCol\">\r\n    <select name=\"dbtype\" dojoAttachPoint=\"node_dbtype\">\r\n      <option value=\"db2\">DB2</option>\r\n      <option value=\"ids\">Informix</option>\r\n      <option value=\"mssql\">Mssql</option>\r\n      <option value=\"mysql\">MySQL</option>\r\n      <option value=\"oracle\">Oracle</option>\r\n    </select>\r\n  </div>\r\n\r\n  <label for=\"dbhost\"><span class=\"required\">*</span> ${labels.DBCONN_HOST}: </label>\r\n  <div class=\"rightCol\">\r\n    <input name=\"dbhost\" dojoAttachPoint=\"node_dbhost\" size=\"40\" type=\"text\" value=\"\" /><br/>\r\n  </div>\r\n\r\n  <label for=\"dbname\"><span class=\"required\">*</span> ${labels.DBCONN_DB}: </label>\r\n  <div class=\"rightCol\">\r\n    <input name=\"dbname\" dojoAttachPoint=\"node_dbname\" type=\"text\" size=\"40\" value=\"\" /><br/>\r\n  </div>\r\n\r\n  <label for=\"dbusr\"><span class=\"required\">*</span> ${labels.CONN_USER}: </label>\r\n  <div class=\"rightCol\">\r\n    <input name=\"dbusr\" dojoAttachPoint=\"node_dbusr\" size=\"40\" type=\"text\" value=\"\" /><br/>\r\n  </div>\r\n\r\n  <label for=\"dbpwd\"><span class=\"required\">*</span> ${labels.CONN_PWD}: </label>\r\n  <div class=\"rightCol\">\r\n    <input name=\"dbpwd\" dojoAttachPoint=\"node_dbpwd\" size=\"40\" type=\"password\" value=\"\" /><br/>\r\n  </div>\r\n\r\n</div>\r\n",

    /**
     * Called when a connection is selected. Populates
     * the form fields based on the selected connection.
     */
    onSelectConnection: function(event) {
        if(!this.conns) return;
        
        var selectedIndex = event.target.selectedIndex;
        var connId = event.target.options[selectedIndex].value;
        
        if(connId == "-1") {
            this.reset();
            return;
        }
    
        for(var i=0; i<this.conns.length; i++) {
            if(this.conns[i].id != connId)
                continue;

            var connection = this.conns[i];            
            this.node_dbname.value = this.conns[i].dbname;
            this.node_dbhost.value = this.conns[i].dbhost;
            this.node_dbusr.value = this.conns[i].dbusr;
            this.node_dbpwd.value = this.conns[i].dbpwd;  
                      
            var select = this.node_dbtype;            
            for(var j=0; j<select.options.length; j++) {
                if(select.options[j].value == connection.dbtype) {
                    select.selectedIndex = j;
                    break;
                }
            }
        }
    },
    
    /**
     * Resets all connection fields in the form
     */
    reset: function() {
        this.node_dbtype.selectedIndex = 0;
        this.node_dbhost.value = "";
        this.node_dbname.value = "";
        this.node_dbusr.value = "";
        this.node_dbpwd.value = "";
    }
    
});






/**
 * Builds and handles all the UI for the permissions editor
 * portion of the 'edit details' page.
 */
dojo.declare('hub.widgets.edit.AclAssign', [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templateString:"    <div class=\"TOM_widget_AclAssign\">\r\n    <table style=\"padding: 0px; border: 0px none; margin: 0px;\"><tr><td>\r\n      <div style=\"margin-bottom:5px;\">\r\n        <input type=\"radio\" name=\"permissions\" value=\"public\" \r\n          dojoAttachPoint=\"radio_public\" dojoAttachEvent=\"onclick: radioButtonClick\" checked /> ${labels.ACL_PUBLIC} \r\n        <span class=\"note\">(${panels.ACL_PUBLIC_HELP})</span>\r\n      </div>\r\n      <div style=\"margin-bottom:5px;\">\r\n        <input type=\"radio\" name=\"permissions\" value=\"private\" \r\n          dojoAttachPoint=\"radio_private\" dojoAttachEvent=\"onclick: radioButtonClick\" /> ${labels.ACL_PRIVATE} \r\n        <span class=\"note\">(${panels.ACL_PRIVATE_HELP})</span>\r\n      </div>\r\n      <div style=\"margin-bottom:5px;\">\r\n        <input type=\"radio\" name=\"permissions\" value=\"custom\" \r\n          dojoAttachPoint=\"radio_custom\" dojoAttachEvent=\"onclick: radioButtonClick\" /> ${labels.ACL_CUSTOM} \r\n        <span class=\"note\">(${panels.ACL_CUSTOM_HELP})</span>\r\n      </div>\r\n      \r\n      <div dojoAttachPoint=\"node_custom\" style=\"display:none;\">\r\n        <p style=\"line-height:135%; width:30em;\">${panels.ACL_HELP_TEXT}</p><br/>\r\n        <table>\r\n          <thead>\r\n            <tr>\r\n              <td style=\"font-weight:bold; font-size:14px; width:150px;\">${labels.ACL_GROUPS}</td>\r\n              <td style=\"width:100px;\">${labels.ACL_EDIT}</td>\r\n              <td style=\"width:100px;\">${labels.ACL_READ}</td>\r\n              <td></td>\r\n            </tr>\r\n          </thead>\r\n          <tbody dojoAttachPoint=\"node_groups\">\r\n            <!-- Groups added here -->\r\n          </tbody>\r\n        </table><br/>\r\n        <table>\r\n          <thead>\r\n            <tr>\r\n              <td style=\"font-weight:bold; font-size:14px; width:150px;\">${labels.ACL_USERS}</td>\r\n              <td style=\"width:100px;\">${labels.ACL_EDIT}</td>\r\n              <td style=\"width:100px;\">${labels.ACL_READ}</td>\r\n              <td></td>\r\n            </tr>\r\n          </thead>\r\n          <tbody dojoAttachPoint=\"node_users\">\r\n            <!-- Users added here -->\r\n          </tbody>\r\n        </table><br/>\r\n        \r\n        <span style=\"font-size: 14px; font-weight:bold; margin-bottom:8px; display:block;\">${labels.BUTTON_SEARCH}</span>\r\n        <input type=\"text\" dojoAttachPoint=\"node_search\" style=\"width:220px;\" />\r\n        <input type=\"button\" dojoAttachEvent=\"onclick: searchUsers\" value=\"${labels.ACL_SEARCHUSERS}\" />\r\n        <input type=\"button\" dojoAttachEvent=\"onclick: searchGroups\" value=\"${labels.ACL_SEARCHGROUPS}\" />\r\n          <span dojoAttachPoint=\"spinner\" style=\"visibility:hidden;\">\r\n            <img src=\"${spinner_url}\" /> ${messages.STATUS_SEARCHING}\r\n          </span><br/>\r\n        <select dojoAttachPoint=\"node_selected\" style=\"width:220px; margin-top:3px; height:75px;\" multiple>\r\n          <!-- Search results -->\r\n        </select><br/>\r\n        <input type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachPoint=\"node_add\" dojoAttachEvent=\"onclick: addItems\" value=\"${labels.ACL_ADDTOLIST}\" style=\"width:220px;\" />\r\n      </div>\r\n      \r\n      <input dojoAttachPoint=\"udelta\" name=\"_acl_user_delta\" type=\"hidden\" />\r\n      <input dojoAttachPoint=\"gdelta\" name=\"_acl_group_delta\" type=\"hidden\" />\r\n    </td></tr></table>  \r\n    <div>\r\n",    
    registered: "REGISTERED",
    anonymous: "ANONYMOUS",
    creator: "HubAdmin",
    entryid: 0,
    users: '',
    groups: '',
    lastSearch: '',
    
    /**
     * Constructor
     *
     * @param args: array of parameters
     * @param node: parent node to inject widget into
     */
    constructor: function(args, node) {
        //users, groups, registered, anonymous, creator
        this.userDelta = {};
        this.groupDelta = {};
        this.defaultPermission = {'edit':0, 'read':1};
        //this.status = dojo.byId('saveEntryMessage_' + this.id);
        this.spinner_url = hub.urls.getImage('spinner.gif');
    },
        
    /**
     * Called before page is rendered
     */
    postMixInProperties: function() {
        // fetch all localized text
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.messages = dojo.i18n.getLocalization('hub', 'messages');
    },
        
    /**
     * Called after page is rendered. Set the appropriate
     * permission level of the feed (public, private, or custom)
     */
//    init: function() {
    postCreate: function() {
    	var req = new hub.submit.Request(hub.config.base_url + '/ui/acl/entryid/' + this.entryid);
    	req.submit();
        var users = [];
        var groups = [];
        var groupRegistered = [];
        var groupAnonymous = [];
        var creator = '';
        this.creator = req.response.creator;
        for(i = 0; i < req.response.user_acls.length; i++){
        	var acl = req.response.user_acls[i]
        	users[i]=[acl.username, parseInt(acl.r), parseInt(acl.e)];
        	if(acl.username == this.creator){
        		creator = users[i];
        	}
        }
        for(i = 0; i < req.response.group_acls.length; i++){
        	var acl = req.response.group_acls[i]
        	groups[i]=[acl.groupname, parseInt(acl.r), parseInt(acl.e)];
        	if(acl.groupname == this.registered){
        		groupRegistered = groups[i];
        	}else if(acl.groupname == this.anonymous){
        		groupAnonymous = groups[i];
        	}
        }
        
        //console.debug('ACL users:')
        //console.dir(users);
        //console.debug('ACL groups:')
        //console.dir(groups);
        
/*        var users = dojo.fromJson(this.users);
        var groups = dojo.fromJson(this.groups);
        delete this.users;
        delete this.groups;*/
        this.initUsers(users);      
        this.initGroups(groups);
        
        //console.debug('ACL: users & groups intialized.');
        
        // REGISTERED must be the first group returned by UserManager
        // and ANONYMOUS the second group returned
        var isPublic = (groups.length == 2
            && this.sameArray(groupRegistered,[this.registered,1,0])
            && this.sameArray(groupAnonymous,[this.anonymous,1,0])
            && users.length == 1
            && this.sameArray(creator,[this.creator,1,1]));

        var isPrivate = (groups.length == 2
            && this.sameArray(groupRegistered,[this.registered,0,0])
            && this.sameArray(groupAnonymous,[this.anonymous,0,0])
            && users.length == 1
            && this.sameArray(creator,[this.creator,1,1]));


    
        if(isPublic) {
            this.radio_public.checked = true;
        } else if(isPrivate) {
            this.radio_private.checked = true; 
        } else {
            this.radio_custom.checked = true;
            this.node_custom.style.display = 'block';
        }
    },
        
    /**
     * Render the initial users who have acl's assigned to them
     *
     * @param users: array of user/permission arrays
     */
    initUsers: function(users) {
    	//console.debug('ACL: initUsers');
        var user, arr, obj = null;
        for(var i=0; i<users.length; i++) {
            // arr = [username, read, edit, admin]
            // ex: ['jchristi', 1, 0, 0]
            arr = users[i];
            user  = arr[0];
            obj = {'read':arr[1], 'edit':arr[2]};
            // skip creator or existing user
            if(user != this.creator && !this.userDelta[user]) {
                this.addUser(user);
                this.setPermissionCheckboxes(user, obj);
            }
            this.userDelta[user] = obj;
        }
        this.udelta.value = dojo.toJson(this.userDelta);   
        //console.debug('ACL: End of initUsers');     
    },
        
    /**
     * Render the initial groups who have acl's assigned to them
     *
     * @param users: array of group/permission arrays
     */
    initGroups: function(groups) {
    	//console.debug('ACL: initGroups'); 
        var group, arr, obj = null;
        for(var j=0; j<groups.length; j++) {
            // arr = [group, read, edit]
            // ex: ['REGISTERED', 1, 0]
            arr = groups[j];
            group = arr[0];
            obj = {'read':arr[1], 'edit':arr[2]}; 
            // skip duplicate groups
            if(!this.groupDelta[group]) {
            	this.addGroup(group);
                this.setPermissionCheckboxes(group, obj);
            }
            
            this.groupDelta[group] = obj;
        }
        
        this.gdelta.value = dojo.toJson(this.groupDelta);
        //console.debug('ACL: End of initGroups'); 
    },
    
    /**
     * Called when a permission level is checked (public, private, custom)
     */
    radioButtonClick: function(event) {
        this.toggleCustomSettings(event.target);
        this.setPermissions(event.target.value);
    },
        
    /**
     * Show or hide the custom permissions menu
     */
    toggleCustomSettings: function(input) {
        if(input.value == 'custom' && input.checked)
            this.node_custom.style.display = 'block';
        else
            this.node_custom.style.display = 'none';
    },
        
    /**
     * Set initial permissions for given permission level
     *
     * @param permissionLevel: permission level "public", "private", or "custom"
     */
    setPermissions: function(permissionLevel) {                    
        var regObj;
        var anonObj;            
        if(permissionLevel == "public") {
            regObj  = {'edit':1, 'read':1};
            anonObj = {'edit':0, 'read':1};
        } else if(permissionLevel == "private") {
            regObj  = {'edit':0, 'read':0};
            anonObj = {'edit':0, 'read':0};
        } else { // permissionLevel == "custom"
            regObj  = {'edit':0, 'read':1};
            anonObj = {'edit':0, 'read':1};
        }
        
        this.userDelta = {};
        this.userDelta[this.creator] = {'edit':1, 'read':1};
        this.groupDelta = {}; 
        this.groupDelta[this.registered] = regObj;
        this.groupDelta[this.anonymous] = anonObj;
        this.udelta.value = dojo.toJson(this.userDelta);
        this.gdelta.value = dojo.toJson(this.groupDelta);
    },
        
    /**
     * Make a call to the server to search for a user or users
     * Callback function is handleSearch()
     */
    searchUsers: function() {
//        this.connect(this.node_add, 'onclick', 'addUsers'); 
		this.lastSearch = 'users';
        var user = this.node_search.value;
        
        if(user == "") {
          //alert(TOM.i18n.man._('EMPTY_SEARCH'));
          hub.notify(this.panels.WARNING_EMPTY_SEARCH_QUERY);
          return;
        }  
        else if(user.indexOf('*')!=-1 || user.indexOf('?')!=-1) {
          hub.notify(this.panels.WARNING_NOT_SUPPORT_SEARCH_QUERY);
          return;
        }
        
        this.spinner.style.visibility = 'visible';        
        var url = hub.urls.getUserSearch(user);
/*        var form = new hub.submit.Form({'user':user}, 'saveEntry_' + this.id, url);
        console.dir(form);
        hub.submit.ajaxSubmit(form);*/
        var req = new hub.submit.Request(url, {user: user});
        this.connect(req, '_onLoad', 'handleUsersSearch');
        req.submit();
    },
        
    /**
     * Make a call to the server to search for a group or groups
     * Callback function is handleSearch()
     */
    searchGroups: function() {
//        this.connect(this.node_add, 'onclick', 'addGroups');
		this.lastSearch = 'groups';
        var group = this.node_search.value;
        
        if(group == "") {
          // use saveEntryMessage instead of popup
          hub.notify(this.panels.WARNING_EMPTY_SEARCH_QUERY);
          return;
        }        
        else if(group.indexOf('*')!=-1 || group.indexOf('?')!=-1) {
           hub.notify(this.panels.WARNING_NOT_SUPPORT_SEARCH_QUERY);
           return;
        }
        
        this.spinner.style.visibility = 'visible';
        var url = hub.urls.getGroupSearch(group);
/*        var form = new hub.submit.Form({'group':group}, 'saveEntry_' + this.id, url);
        this.connect(form, '_onLoad', 'handleSearch');
        hub.submit.ajaxSubmit(form);*/
        var req = new hub.submit.Request(url, {group: group});
        this.connect(req, '_onLoad', 'handleGroupsSearch');
        req.submit();
        
    },
    
    /**
     * Callback function after a user or group search. List the 
     * results in an multi-select form
     *
     * @param response: array of users
     */
    handleUsersSearch: function (response){
    	var users = dojo.fromJson(response).users;
        var selected = this.node_selected;
        selected.innerHTML = "";
        for(var key=0; key < users.length; key++) {
        	if(users[key].username != this.creator){
				var option = dojo.doc.createElement('option');
				option.setAttribute('value', users[key].username);
				option.innerHTML = users[key].fullname;
				selected.appendChild(option);
            }    
        }
        this.spinner.style.visibility = 'hidden';        
    },
    
    /**
     * Callback function after a user or group search. List the 
     * results in an multi-select form
     *
     * @param response: array of users
     */
    handleGroupsSearch: function (response){
    	var groups = dojo.fromJson(response).groups;
        var selected = this.node_selected;
        selected.innerHTML = "";
        for(var key=0; key < groups.length; key++) {
			var option = dojo.doc.createElement('option');
			option.setAttribute('value', groups[key]);
			option.innerHTML = groups[key];
			selected.appendChild(option);
        }
        this.spinner.style.visibility = 'hidden';        
    },
    
    addItems: function(){
    	switch(this.lastSearch){
    		case 'users':
    			this.addUsers();
    			break;
    		case 'groups':
    			this.addGroups();
    			break;
    	}
    },
        
    /**
     * Add selected users to the acl list
     */
    addUsers: function() {
        var selected = this.node_selected;
        for(var i=0; i<selected.options.length; i++) {
            if(selected.options[i].selected) {
                var username = selected.options[i].value;
                var fullname = selected.options[i].innerHTML;
                this.addUser(username, fullname);
            }
        }
        this.udelta.value = dojo.toJson(this.userDelta);
    },
        
    /**
     * Add selected groups to the acl list
     */
    addGroups: function() {
        var selected = this.node_selected;
        for(var i=0; i<selected.options.length; i++) {
            if(selected.options[i].selected) {
                var group = selected.options[i].value;
                var fullname = selected.options[i].innerHTML;
                this.addGroup(group);
            }
        }
        this.gdelta.value = dojo.toJson(this.groupDelta);
    },
        
    /**
     * Add a user to the acl list
     */
    addUser: function(user, fullname) {
        if(user == this.creator || this.userDelta[user]) 
            return;
        var id = user + '_' + this.entryid;
        var users = this.node_users;
        users.appendChild(this.addHTML(id, user, 'user', fullname));
        this.setPermissionCheckboxes(user, this.defaultPermission);
        this.userDelta[user] = this.defaultPermission;
        // call is made at end of addUsers for better performance
        // this.udelta.value = dojo.toJson(this.userDelta);
    },
        
    /**
     * Add a group to the acl list
     */
    addGroup: function(group) {
    	//console.debug('ACL: addGroup');
        if(this.groupDelta[group])
            return;
        var id = group + '_' + this.entryid;
        var groups = this.node_groups;
        groups.appendChild(this.addHTML(id, group, 'group'));
        this.setPermissionCheckboxes(group, this.defaultPermission);
        this.groupDelta[group] = this.defaultPermission;
        // call is made at end of addGroups for better performance
        // this.gdelta.value = dojo.toJson(this.groupDelta);
        //console.debug('ACL: End of addGroup');
    },
    
    /**
     * Generate the html elements for adding a user or group
     * 
     * @param id: id of the group or user
     * @param name: name of the group or user
     * @param type: "group" or "user"
     * @return: html element of the user or group
     */
    addHTML: function(id, name, type, hint) {
        var tr = document.createElement('tr');
        var td, input = null;
        var display = 'block';
        var setPermissionMethod = 'setUserPermission';
        var removeMethod = 'removeUser'; 

        if(name == this.registered  || name == this.anonymous || name == this.creator) {
            display = 'none';
        }

        if(type == 'group') {
            setPermissionMethod = 'setGroupPermission';
            removeMethod = 'removeGroup';
        }

        for(var i=0; i<4; i++) {
            td = document.createElement('td');
            td.style.paddingRight = "13px";
            tr.appendChild(td);
            input = document.createElement('input');
            
            switch(i) {
                case 0:
                    td.innerHTML = hint ? 
                    	name + ' <span style="white-space: nowrap; font: 10px; color: gray">('+hint+')</span>':
                    	name;
                    
                    break;
                case 1:
                	input.type = "checkbox";
                    input.value = 'e';
                    input.id = id + '_e';
                    td.appendChild(input);
                    // IE:  set it after this node is attached to the doc
                    //input.checked = (this.defaultPermission.edit==1)?true:false;
                    this.connect(input, 'onclick', setPermissionMethod);
                    break;
                case 2:
                	input.type = "checkbox";
                    input.value = 'r';
                    input.id = id + '_r';
                    td.appendChild(input);
                    // IE:  set it after this node is attached to the doc
                    //input.checked = (this.defaultPermission.read==1)?true:false;
                    this.connect(input, 'onclick', setPermissionMethod);
                    break;
                case 3:
                    input.type = "button";
                    input.id = id + '_remove';
                    input.value = this.labels.ACL_REMOVE;
                    input.style.display = display;
                    input.name = name;
                    td.appendChild(input);
                    this.connect(input, 'onclick', removeMethod);
                    break;
            }
        }
        
        return tr;
    },
        
    /**
     * Called when a permission check box is checked. Evaluates and sets
     * appropriate permissions and checks or unchecks other checkboxes if
     * necessary.
     */
    setUserPermission: function(event) {
        var id = event.target.id;
        var permission = event.target.value;
        var user = id.substring(0, id.indexOf('_' + this.entryid));
        var obj = this.evalPermission(permission, event.target.checked);
        this.userDelta[user] = obj;                     
        this.udelta.value = dojo.toJson(this.userDelta);            
        this.setPermissionCheckboxes(user, obj);
    },        
        
    /**
     * Called when a permission check box is checked. Evaluates and sets
     * appropriate permissions and checks or unchecks other checkboxes if
     * necessary.
     */
    setGroupPermission: function(event) {
        var id = event.target.id;
        var permission = event.target.value;
        var group = id.substring(0, id.indexOf('_' + this.entryid));
        var obj = this.evalPermission(permission, event.target.checked);
        this.groupDelta[group] = obj;
        this.gdelta.value = dojo.toJson(this.groupDelta);            
        this.setPermissionCheckboxes(group, obj);
    },
        
    /**
     * Returns a permission object based on the given permission
     *
     * @param permission: 'a', 'e', or 'r', standing for 'admin', 'edit', or 'read'
     * @param value: true or false (if the permission is checked or unchecked)
     * @return: an object representing the permission
     */
    evalPermission: function(permission, value) {
        var obj = {};         
/*        if(permission == 'a') {
            obj.admin = (value ? 1 : 0);
            obj.edit = 1;
            obj.read = 1;
        } else*/ if(permission == 'e') {
//            obj.admin = 0;
            obj.edit = (value ? 1 : 0);
            obj.read = 1;
        } else if(permission == 'r') {
//            obj.admin = 0;
            obj.edit = 0;
            obj.read = (value ? 1 : 0);
        } else { // permission == 'n'
//            obj.admin = 0;
            obj.edit = 0;
            obj.read = 0;
        }            
        return obj;
    },
        
    /**
     * Set appropriate checkboxes for a specific group or user,
     * given a permissions object
     *
     * @param user: name of the group or user
     * @param obj: permissions object for the group or user
     */
    setPermissionCheckboxes: function(user, obj) {          
//        var a = dojo.byId(user + '_' + this.entryid + '_a');
        var e = dojo.byId(user + '_' + this.entryid + '_e');
        var r = dojo.byId(user + '_' + this.entryid + '_r');         
//        a.checked = (obj.admin == 1);
        e.checked = (obj.edit == 1);
        r.checked = (obj.read == 1);
    },
        
    /**
     * Remove a user from the acl list
     */
    removeUser: function(event) {
        delete this.userDelta[event.target.name];
        this.udelta.value = dojo.toJson(this.userDelta);   
        this.remove(event);
    },
        
    /**
     * Remove a user from the acl list
     */
    removeGroup: function(event) {
        delete this.groupDelta[event.target.name];
        this.gdelta.value = dojo.toJson(this.groupDelta);   
        this.remove(event);
    },
        
    /**
     * Remove a group or user from the acl list
     */
    remove: function(event) {
        var tr = event.target.parentNode.parentNode;
        tr.parentNode.removeChild(tr);
    },
    
    /**
     * Compares two arrays to see if they have the same values
     * Not the same as arr1 == arr2, which checks to see if it
     * is the same object.
     *
     * @arr1: first array
     * @arr2: second array to compare with the first
     * @return: true if the two arrays contain the same elements
     */
    sameArray: function(arr1, arr2) {
//    	console.log('acl.sameArray');
//    	console.dir({a1: arr1, a2: arr2});
        if(arr1.length != arr2.length)
            return false;
           
        for(var i=0; i<arr1.length; i++) {
            if(dojo.isArray(arr1[i]) && dojo.isArray(arr2[i])) {
                if(!this.sameArray(arr1[i], arr2[i]))
                    return false;
            } else {
                if(arr1[i] != arr2[i])
                    return false;
            }
        }
        return true;        
    }
});

/**
 * Builds and handles the UI for the AdminPermissions.html editor
 */
dojo.declare('hub.widgets.edit.AdminAssign', [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templateString:"    <div class=\"TOM_widget_AdminAssign\">\r\n    <table style=\"padding: 0px; border: 0px none; margin: 0px;\"><tr><td>\r\n      <div dojoAttachPoint=\"node_custom\" style=\"display:block;\">\r\n        <table>\r\n          <thead>\r\n            <tr>\r\n              <td style=\"font-weight:bold; font-size:14px; width:150px;\">${labels.ACL_ADMIN}</td>\r\n              <td></td>\r\n            </tr>\r\n          </thead>\r\n          <tbody dojoAttachPoint=\"node_admins\">\r\n            <!-- Admin list added here -->\r\n          </tbody>\r\n        </table><br/>\r\n        \r\n        <span style=\"font-size: 14px; font-weight:bold; margin-bottom:8px; display:block;\">${labels.BUTTON_SEARCH}</span>\r\n        <input type=\"text\" dojoAttachPoint=\"node_search\" style=\"width:220px;\" />\r\n        <input type=\"button\" dojoAttachEvent=\"onclick: searchUsers\" value=\"${labels.ACL_SEARCHUSERS}\" />\r\n          <span dojoAttachPoint=\"spinner\" style=\"visibility:hidden;\">\r\n            <img src=\"${spinner_url}\" /> ${messages.STATUS_SEARCHING}\r\n          </span><br/>\r\n        <select dojoAttachPoint=\"node_selected\" style=\"width:220px; margin-top:3px; height:75px;\" multiple>\r\n          <!-- Search results -->\r\n        </select><br/>\r\n        <input type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachPoint=\"node_add\" dojoAttachEvent=\"onclick: addItems\" value=\"${labels.ACL_ADDTOLIST}\" style=\"width:220px;\" />\r\n      </div>\r\n    </td></tr></table>  \r\n    <div>\r\n",
    currentUser: '',
    adminList: [],
    
    /**
     * Constructor
     *
     * @param args: array of parameters
     * @param node: parent node to inject widget into
     */
    constructor: function(args, node) {
        this.spinner_url = hub.urls.getImage('spinner.gif');
    },
        
    /**
     * Called before page is rendered
     */
    postMixInProperties: function() {
        // fetch all localized text
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.messages = dojo.i18n.getLocalization('hub', 'messages');
    },
        
    /**
     * Called after page is rendered. Set the initial list of administrators
     */
    postCreate: function() {
    	this.adminList = [];
	   	var req = new hub.submit.Request(hub.config.base_url + '/ui/adminlist');
    	req.submit();        
        this.currentUser = req.response.currentUser;
        for (i=0; i<req.response.admins.length; i++) {
        	var id = req.response.admins[i].id;
        	this.addAdmin(id);
        	this.adminList[i] = id;
        }
    },
        
    /**
     * Make a call to the server to search for users
     * Callback function is handleSearch()
     */
    searchUsers: function() {    
        var searchSelection = this.node_search.value;
        if (searchSelection == "") {
          hub.notify(this.panels.WARNING_EMPTY_SEARCH_QUERY);
          return;
        }  
        else if(searchSelection.indexOf('*')!=-1 || searchSelection.indexOf('?')!=-1) {
          hub.notify(this.panels.WARNING_NOT_SUPPORT_SEARCH_QUERY);
          return;
        }
        
        this.spinner.style.visibility = 'visible';        
        var url = hub.urls.getUserSearch(searchSelection);
        var req = new hub.submit.Request(url, {user: searchSelection});
        this.connect(req, '_onLoad', 'handleUsersSearch');
        req.submit();
    },
        
    /**
     * Callback function after a user search. List the results in an multi-select form
     *
     * @param response: array of users
     */
    handleUsersSearch: function(response) {
    	var users = dojo.fromJson(response).users;
        var selected = this.node_selected;
        selected.innerHTML = "";
        for (var i=0; i<users.length; i++) {
			var option = dojo.doc.createElement('option');
			option.setAttribute('value', users[i].username);
			option.innerHTML = users[i].username;
			selected.appendChild(option);
        }
        this.spinner.style.visibility = 'hidden';        
    },
    
    /**
     * Add selected users to the admin list
     */
    addItems: function() {
        var selected = this.node_selected;
        for (var i=0; i<selected.options.length; i++) {
            if (selected.options[i].selected) {
                var username = selected.options[i].value;
                this.addAdmin(username);
            }
        }
    },
    
    /**
     * Add a user to the admin list
     */
    addAdmin: function(id) {
    
    	// Check for duplicates
    	for (var i=0; i<this.adminList.length; i++) {
    		if (this.adminList[i] == id) {
    			return;
    		}
    	}
        var admins = this.node_admins;
        admins.appendChild(this.addHTML(id));
    },
    
    /**
     * Generate the html elements for adding a user to the admin list
     * 
     * @param id: id of the user
     * @return: html element
     */
    addHTML: function(id) {
		var tr = document.createElement('tr');
		var td, input = null;
		
		// Add name
		td = document.createElement('td');
		td.style.paddingRight = '13px';
		td.innerHTML = id + "<input type='hidden' name='admin" + id + "' value='" + id + "'/>";
		tr.appendChild(td);
		
		// Add remove button
		td = document.createElement('td');
		td.style.paddingRight = '13px';
		if (id != this.currentUser) {
			input = document.createElement('input');
			input.type = 'button';
			input.id = id + '_remove';
			input.value = this.labels.ACL_REMOVE;
			input.style.display = 'block';
			input.name = id;
			td.appendChild(input);
			this.connect(input, 'onclick', 'removeAdmin');
		} else {
			td.innerHTML = this.labels.ACL_CURRENTUSER;
		}
		tr.appendChild(td);		
		
		return tr;
    },
        
    /**
     * Remove a user from the admin list
     */
    removeAdmin: function(event) {
        var tr = event.target.parentNode.parentNode;
        tr.parentNode.removeChild(tr);
    }
});

/**
 * Builds and handles the UI for the AdminBannedUsers.html editor
 */
dojo.declare('hub.widgets.edit.BannedAssign', [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templateString:"    <div class=\"TOM_widget_BannedAssign\">\r\n    <table style=\"padding: 0px; border: 0px none; margin: 0px;\"><tr><td>\r\n      <div dojoAttachPoint=\"node_custom\" style=\"display:block;\">\r\n        <table>\r\n          <thead>\r\n            <tr>\r\n              <td style=\"font-weight:bold; font-size:14px; width:170px;\">${labels.ACL_BANNED}</td>\r\n              <td></td>\r\n            </tr>\r\n          </thead>\r\n          <tbody dojoAttachPoint=\"node_bans\">\r\n            <!-- Banned users added here -->\r\n          </tbody>\r\n        </table><br/>\r\n        \r\n        <span style=\"font-size: 14px; font-weight:bold; margin-bottom:8px; display:block;\">${labels.BUTTON_SEARCH}</span>\r\n        <input type=\"text\" dojoAttachPoint=\"node_search\" style=\"width:220px;\" />\r\n        <input type=\"button\" dojoAttachEvent=\"onclick: searchUsers\" value=\"${labels.ACL_SEARCHUSERS}\" />\r\n          <span dojoAttachPoint=\"spinner\" style=\"visibility:hidden;\">\r\n            <img src=\"${spinner_url}\" /> ${messages.STATUS_SEARCHING}\r\n          </span><br/>\r\n        <select dojoAttachPoint=\"node_selected\" style=\"width:220px; margin-top:3px; height:75px;\" multiple>\r\n          <!-- Search results -->\r\n        </select><br/>\r\n        <input type=\"button\" dojoType=\"dijit.form.Button\" dojoAttachPoint=\"node_add\" dojoAttachEvent=\"onclick: addItems\" value=\"${labels.ACL_ADDTOLIST}\" style=\"width:220px;\" />\r\n      </div>\r\n    </td></tr></table>  \r\n    <div>\r\n",
    currentUser: '',
    banList: [],
    
    /**
     * Constructor
     *
     * @param args: array of parameters
     * @param node: parent node to inject widget into
     */
    constructor: function(args, node) {
        this.spinner_url = hub.urls.getImage('spinner.gif');
    },
        
    /**
     * Called before page is rendered
     */
    postMixInProperties: function() {
        // fetch all localized text
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.messages = dojo.i18n.getLocalization('hub', 'messages');
    },
        
    /**
     * Called after page is rendered. Set the initial list of banned users
     */
    postCreate: function() {
    	this.banList = [];
	   	var req = new hub.submit.Request(hub.config.base_url + '/ui/bannedlist');
    	req.submit();
        this.currentUser = req.response.currentUser;
        for (i=0; i<req.response.banned.length; i++) {
        	var id = req.response.banned[i].id;
        	this.addBan(id);
        	this.banList[i] = id;
        }
    },
        
    /**
     * Make a call to the server to search for users
     * Callback function is handleSearch()
     */
    searchUsers: function() {    
        var searchSelection = this.node_search.value;
        if (searchSelection == "") {
          hub.notify('Search query is empty');
          return;
        }
        this.spinner.style.visibility = 'visible';        
        var url = hub.urls.getUserSearch(searchSelection);
        var req = new hub.submit.Request(url, {user: searchSelection});
        this.connect(req, '_onLoad', 'handleUsersSearch');
        req.submit();
    },
        
    /**
     * Callback function after a user search. List the results in an multi-select form
     *
     * @param response: array of users
     */
    handleUsersSearch: function(response) {
    	var users = dojo.fromJson(response).users;
        var selected = this.node_selected;
        selected.innerHTML = "";
        for (var i=0; i<users.length; i++) {
			var option = dojo.doc.createElement('option');
			option.setAttribute('value', users[i].username);
			option.innerHTML = users[i].username;
			selected.appendChild(option);
        }
        this.spinner.style.visibility = 'hidden';        
    },
    
    /**
     * Add selected users to the ban list
     */
    addItems: function() {
        var selected = this.node_selected;
        for (var i=0; i<selected.options.length; i++) {
            if (selected.options[i].selected) {
                var username = selected.options[i].value;
                this.addBan(username);
            }
        }
    },
    
    /**
     * Add a user to the ban list
     */
    addBan: function(id) {
    
    	// Check for duplicates
    	for (var i=0; i<this.banList.length; i++) {
    		if (this.banList[i] == id) {
    			return;
    		}
    	}
        var bans = this.node_bans;
        bans.appendChild(this.addHTML(id));
    },
    
    /**
     * Generate the html elements for adding a user to the ban list
     * 
     * @param id: id of the user
     * @return: html element
     */
    addHTML: function(id) {
		var tr = document.createElement('tr');
		var td, input = null;
		
		// Add name
		td = document.createElement('td');
		td.style.paddingRight = '13px';
		td.innerHTML = id + "<input type='hidden' name='ban" + id + "' value='" + id + "'/>";
		tr.appendChild(td);
		
		// Add remove button
		td = document.createElement('td');
		td.style.paddingRight = '13px';
		input = document.createElement('input');
		input.type = 'button';
		input.id = id + '_remove';
		input.value = this.labels.ACL_REMOVE;
		input.style.display = 'block';
		input.name = id;
		td.appendChild(input);
		this.connect(input, 'onclick', 'removeBan');
		tr.appendChild(td);		
		
		return tr;
    },
        
    /**
     * Remove a user from the ban list
     */
    removeBan: function(event) {
        var tr = event.target.parentNode.parentNode;
        tr.parentNode.removeChild(tr);
    }
});

dojo.declare('hub.widgets.edit.CategoriesCBL', [hub.widgets.CheckBoxList],{
	groups: {},
	checked: {},
	all: {},
	init_labels: [],
	init_names: [],
	entryid: 0,
	constructor: function(args, node){
		this.entryid = args.entryid; 
        // key: group id, value: div holding the group
        this.groups = {};
        // key: label, value: checked (true) or unchecked (false)
        this.checked = {};
        // key: group id, value: array of group elements
        this.all = {};

        // keep labels of all elements initially in the list
        this.init_labels = [];
    	var req = new hub.submit.Request(hub.urls.getCategoriesList(this.entryid));
    	if(req.submit()){
//	    	console.dir(req.response);
    	}
//		this.init_labels = req.response.result;
		var items = req.response.rows;
		for(i in items){
			this.init_names[i] = 'cat_'+items[i].id;
			this.init_labels[i] = items[i].name;
            if( items[i].isChecked){
            	this.checked['cat_'+items[i].id]=true;
            }
		}    	
	}
});

dojo.declare('hub.widgets.edit.ObjectsCBL', [hub.widgets.CheckBoxList],{
	groups: {},
	checked: {},
	all: {},
	init_labels: [],
	init_names: [],
	entryid: 0,
	constructor: function(args, node){
		this.entryid = args.entryid; 
        // key: group id, value: div holding the group
        this.groups = {};
        // key: label, value: checked (true) or unchecked (false)
        this.checked = {};
        // key: group id, value: array of group elements
        this.all = {};

        // keep labels of all elements initially in the list
        this.init_labels = [];
    	var req = new hub.submit.Request(hub.urls.getAssociationsList(this.entryid));
    	if(req.submit()){
//	    	console.dir(req.response);
    	}
//		this.init_labels = req.response.result;
		var items = req.response.rows;
		for(i in items){
			this.init_names[i] = 'rel_'+items[i].id;
			this.init_labels[i] = items[i].name;
            if( items[i].isChecked){
            	this.checked['rel_'+items[i].id]=true;
            }
		}    	
	}

});

}

if(!dojo._hasResource["hub.widgets.forms.SelectPlugin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.widgets.forms.SelectPlugin"] = true;
dojo.provide("hub.widgets.forms.SelectPlugin");
 
 
// our declared class
dojo.declare("hub.widgets.forms.SelectPlugin",
        // we inherit from this class, which in turn mixes
        // in _Templated and _Layout
        [dijit._Widget, dijit._Templated ],
        // class properties:
        {
        uniqueId: 0,
        inputSelectName: "pluginid",
        inputSelectSize: 15,
        href: '/plugin/editor/source/',
		widgetsInTemplate: true,
		templateString:"<form dojoAttachPoint=\"selectPluginForm\">\r\n    <div class='TOM_panel'>\r\n      <div dojoAttachPoint=\"feedBanner\">\r\n\t\t\t<h1><img width='32' height='32' src='../images/feedLarge.gif'/>&nbsp;${panels.CHOOSER_FEED_TITLE_TEXT}</h1>\r\n\t  </div>\r\n\t  \r\n\t  <div dojoAttachPoint=\"widgetBanner\" style=\"display:none;\">\r\n\t\t\t\t<h1><img width='32' height='32' src='../images/widgetLarge.gif'/>&nbsp;${panels.CHOOSER_WIDGET_TITLE_TEXT}</h1>\r\n\t  </div>\r\n\t  \r\n      <label for=\"pluginid\"><span class=\"required\">*</span>${panels.DETAILS_SOURCE_TEXT}</label>\r\n      <div class=\"rightCol\">\r\n        <select name=\"pluginid\" dojoAttachPoint=\"inputSelect\" style=\"width: 400px;\" size=\"17\"></select>\r\n      </div>\r\n    </div>\r\n</form>\r\n \r\n",
		init: 0,
		type: 'Feeds',
		labels: null,
		startup: function(){
			this.inputSelect.setAttribute('size', this.inputSelectSize);
			this.inputSelect.setAttribute('name', this.inputSelectName);
	        this.panels = dojo.i18n.getLocalization('hub', 'panels');
			
			this.inherited("startup", arguments);
		},
		
		postMixInProperties: function(){
	        this.panels = dojo.i18n.getLocalization('hub', 'panels');
			this.inherited("postMixInProperties", arguments);
		},
		
		setOptions: function(response/*, ioArgs*/){
//			console.debug('SelectPlugin: SetOptions');
			response = dojo.fromJson(response);
//			console.dir(response);
//			this.domNode.innerHTML = '';
			this.inputSelect.domNode = '';
			var optGroup = null;
			var option = null;
			
			if (this.init == 1) {
				this.inputSelect.removeAll();
			}
			
			if (this.type == 'Widgets') {
				this.feedBanner.style.display='none';
				this.widgetBanner.style.display='';
			} else {
				this.feedBanner.style.display='';
				this.widgetBanner.style.display='none';
			}
			
			var msgs = dojo.i18n.getLocalization('hub', 'messages');    
			for(category in response.payload){
				//console.debug('SelectPlugin: SetOptions: Category: ' + category);
				optGroup = dojo.doc.createElement('optgroup');
				
				optGroup.setAttribute('label', category);
				var items = response.payload[category];
//				console.debug('Items length: ' + items.length);
//				console.debug('Items[0]: ' + items[0]);
				for (i = 0; i < items.length; i++) { 
					var item = items[i];
					option = dojo.doc.createElement('option');
					option.setAttribute('value', item.pluginid);
					option.innerHTML = item.name;
					optGroup.appendChild(option);
				}
				this.inputSelect.appendChild(optGroup);
			}

			this.init = 1;
//			console.debug('SelectPlugin: End of SetOptions');
		},
		
		setType: function(type) {
//			console.debug('SelectPlugin: SetType');
			this.type = type;
			this.init = 0;
	    	var req = new hub.submit.Request(hub.config.base_url + this.href + this.type);
		    req._onLoad = dojo.hitch(this, this.setOptions);
	    	if(req.submit()){
	//	    	console.dir(req.response);
	    	}
/*			
			var kw = {
				url: hub.config.base_url + this.href + this.type,//hub.urls.postInstall(action),
				sync: true,
		        handleAs: 'json',
		        timeout: 120000, // 2 minutes in milliseconds
		        
		        error: dojo.hitch(this, function(response, args) {           
		            return response;
		        }),
		        
		        load: dojo.hitch(this, this.setOptions)//dojo.hitch(this, function(response, args) {
		    };
		   
		    // do the post
		    dojo.xhrPost(kw);
*/		    
//		    console.debug('SelectPlugin: End of SetType');
		}
});

}

if(!dojo._hasResource['hub.managers.Plugin']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.managers.Plugin'] = true;
dojo.provide('hub.managers.Plugin');
dojo.provide('hub.managers.InvokePlugin');
dojo.provide('hub.behaviors.Stack');
//dojo.require('hub.widgets.tabs.TabContainer');



if (!hub.managers) {
    hub.managers = {};
}

// storage so plug-ins can easily emit code server side that references this
// client-side instance
hub.managers.plugins = {};



dojo.declare('hub.managers.Plugin', null, {
    // @todo: implement, port from tom/client/scripts/old/plugin.js
    // probably needs some updates to work with new 
    // hub.widgets.tabs.edit.WorkflowPane widget

    /**
     * Used by the server-side plug-in to build a new instance.
     * 
     * @param ajax_url URL to the server-side plug-in
     * @param editor_id Unique id of this client-side plug-in instance
     * @param unique Unique id of HTML elements in a single screen of
     *   this plug-in instance
     * @param entry_id Unique id of the object under editing
     * @param plugin_id Unique id of the plug-in
     */
    workflow: null,
    ajax_url: '',
    save_url: '',
    editor_id: null,
    entry_id: null,
    plugin_id: null,
    tab: null,
    upload_stack: null,
    filetype: null,
    msg_dialog: null,
    
    constructor: function(plugin_id, entry_id, workflow) {
        // build client ID from editor and unique ids
        this.entry_id = entry_id;
        this.plugin_id = plugin_id;
        this.editor_id = this.getEditorId();
/*        this.tab = TOM.tabs.man.getTabForObjectId(entry_id);
        if(this.tab == null) {
            // we might be in a tab that doesn't have an object id yet
            // so find our tab and give it the proper id
            this.tab = TOM.tabs.man.getMyTab(dojo.byId(entry_id));
            this.tab.setAttribute('entry_id', entry_id);
        }*/
        this.workflow = workflow;
        this.tab=this.workflow.getTab();

        // inform the workflow manager for this tab that we exist
//        var wm = this.tab.getAttribute('workflow');
//        wm.setPlugin(this);
        
        // a URL to use to show the next editor form when saving
        this.save_url = hub.config.base_url+"/ui/editor/name/save/entryid/"+this.entry_id;
		this.ajax_url = hub.config.base_url+'/plugin/ajax/pluginid/'+this.plugin_id+'/entryid/'+this.entry_id;
        // store this instance so the server side screens can access it
        // directly
        hub.managers.plugins[this.getEditorId()] = this;
    },
    
    getEditorId: function(){
    	return 'hub_plugins_'+this.plugin_id+'_'+this.entry_id;
    },

    /**
     * Returns an object that a TOM.editor.Workflow instance can use to 
     * compare whether the state of this screen has changed since the user
     * last visited it. If not implemented (i.e., raises a Error),
     * then the workflow instance uses its own logic to save state, namely
     * getting values from all named elements in the first form on the 
     * screen. Override this method only if form field checks will not work
     * for this plug-in.
     * 
     * @param screen The DOM node for the screen whose state is needed
     * @param index The index of the screen, include screens shown before
     *   the plug-in's first screen
     */
    getScreenState: function(screen, index) {
        // let the workflow use its default algorithm
        throw new Error();
    },

    /**
     * Proxy for TOM.editor.Workflow.previous for convenience of the 
     * plug-in.
     */
    previous: function() {
        //var wm = this.tab.getAttribute('workflow');
        //wm.previous();
        this.workflow.previous();
    },

    /**
     * Proxy for TOM.editor.Workflow.next for convenience of the 
     * plug-in.
     */
    next: function(func) {
        // get the workflow for this editor
//        var wm = this.tab.getAttribute('workflow');
//        wm.next(func);
		this.workflow.next(func);
    },

    /**
     * Gets the id of the object under editing.
     *
     * @return ID
     */
    getObjectId: function() {
        return this.entry_id;
    },

    /**
     * Gets the id of the plugin under editing.
     *
     * @return ID
     */
    getPluginId: function() {
        return this.plugin_id;
    },


    /**
     * Convenience method. Call to close the current tab, go to the 
     * details page, or show a message when the editor is complete. Fires
     * an 'edit-entry' event.
     *
     * @param params 
     *   'action' is 'save' to save 
     *   'text' is message text
     */
    done: function(params) {
//    	console.log('plugin.done');
        var action = params['action'];
        var text = params['text'];
        var saveCallbackObject = params['saveCallbackObject'];
        var saveCallbackFunction = params['saveCallbackFunction'];
		this.saveCallbackObject = saveCallbackObject;
		this.saveCallbackFunction = saveCallbackFunction;
        if (action == 'message' || action == 'save') {
            // go to the metadata / save page
            dojo.publish('workflow-done-' + this.tab.id, []);
            
        } else if (action == 'cancel') {
            // close tab, workflow and tab will make sure to ask for 
            // confirmation when needed
            hub.closePage(this.tab);
        }
    },
    
    /**
     * A workflow manager instance invokes this method when a tab closes
     * or an editor wizard is complete / cancelled.
     */
    dispose: function() {
        // throw away this plug-in object
        console.log('Plugin.dispose');
        for(member in this){
        	if(member != 'tab' && member != 'workflow'
        	&& this[member]!= null
        	&& dojo.isObject(this[member]) 
        	&& this[member].destroy!= null 
        	&& dojo.isFunction(this[member].destroy)){
        		console.log('destroyed');
        		this[member].destroy();
        	}
        }
        delete hub.managers.plugins[this.editor_id];
    },
    
    /**
     * Convenience method. Call to replace the entire contents of the 
     * editor area with new HTML.
     * 
     * @param html HTML to set as innerHTML
     */
    showEditor: function(html) {
//    alert('showEditor');
        // ask workflow manager for a new pane to render into
//        var wm = this.tab.getAttribute('workflow');
//        var screen = wm.appendScreen(html);
		var screen = this.workflow.appendScreenHTML(html);

        // let the plug-in do something now that the editor has loaded
        // the plug-in can change this method on each screen if it wants
        // since the script eval above runs first
/*        if(this.onLoadEditor) {
            this.onLoadEditor(screen);
        }*/
    },

    /**
     * Convenience method. Call to show a text message in a popup
     * 
     * @param msg Message content
     * @param typ Message type - 'I' for information, 'W' for warning, 'E' for error
     */
    showMessage: function(msg, typ) {
       //var e = dojo.byId(this.getEditorId()+'_status');
       //hub.notify(msg, e);
       this._showMsg(msg, typ, 500, 350);
    },
    
    // TODO:  destroy dialog
    _showMsg: function(msg, typ, width, height) {
       
       var labels = dojo.i18n.getLocalization('hub', 'labels');
       var title = "MashupHub ";
       var msgType = "";
       var icon = "";
       
       if (this.msg_dialog) {
          this.msg_dialog.hide();
          this.msg_dialog.destroyRecursive();
          this.msg_dialog = null;
       }
       
       if (!typ) {
          typ = 'E';
       }
       else {
          typ = typ.toUpperCase();
       }   
       
       if (typ == 'I') {
          msgType = labels.DIALOG_TITLE_INFO;
          icon = "<img border='0' src='../images/info_status.gif' />";
       }
       else if (typ == 'W') {
          msgType = labels.DIALOG_TITLE_WARNING;
          icon = "<img border='0' src='../images/warning_status.gif' />";
       }
       else if (typ == 'E') {
          msgType = labels.DIALOG_TITLE_ERROR;
          icon = "<img border='0' src='../images/error_status.gif' />";
       }
    
       title += msgType;
    
       var node = dojo.body().appendChild(document.createElement("div"));
       this.msg_dialog = new hub.widgets.Dialog({title: title, showUnderlay: false, resizable: true, closeable: true }, node);
    
       var msgContent = '<div><table width="100%"><tr>';
       msgContent += '<td valign="top" align="left" width="27">' + icon + '</td>';
       msgContent += '<td>' + msg + '</td>';
       msgContent += '</tr></table></div>';
    
       this.msg_dialog.setContent( msgContent );
       this.msg_dialog.resize({w: width, h: height});
       this.msg_dialog.show();
    },
    
    /**
     * Notifies the plugin that the entry has been renamed by the editor. 
     * This method makes sure that the surrounding UI is updated 
     * accordingly.
     * 1. Update tab title
     * 2. Update menu links
     */
    notifyRename: function( newName ){
        // @todo: remove after changing flow editor
        // no-op
    },
    
    /*
     * Used to make changes to the status string in the menu area
     * 
     * @param published - boolean true for PUBLISHED, false for UNPUBLISHED
     */
    notifyStatus: function( published ){
        // @todo: remove after changing flow editor
        // no-op
    },
    
    /**
     * The current implementation will refresh the content of the tab.
     *
     * @param pid - plugin id
     * @param oid - object id of clone
     * @param isStatic - signal to use a static tab or one that is refreshed 
     *    on changes
     */
    notifyClone: function( pid, oid, isStatic ){
        // @todo: convert or remove after changing flow editor
        //Since the clone is supposed to replace the content and is 
        // initiated by the user... there is no need to confirm
        this.confirmDispose = function(){ return true };
        var editorURL = hub.config.base_url + '/plugin/editor/entryid/'+oid+'/pluginid/'+pid;
        if( isStatic ) {
/*            TOM.tabs.man.showEditorTab(editorURL, this.tab.title, oid, 
                                       false, true);*/
            hub.showPlugin(	this.tab.title, 
            				oid, 
            				oid, 
            				{safe: true});
           
        } else {
/*            TOM.tabs.man.showTab(editorURL, this.tab.title, oid, 
                                 false, true);*/
			hub.showUrl(this.tab.title, editorURL, {safe: true});                             
        }
    }


});

dojo.declare('hub.managers.InvokePlugin', [hub.managers.Plugin], {
    // @todo: implement, port from tom/client/scripts/old/plugin.js
    
	dispose: function() {
console.log("hub.managers.InvokePlugin.dispose() start");
		this.inherited('dispose', arguments);
console.log("hub.managers.InvokePlugin.dispose() end");
	},

    /**
     * Handles a RPC from the server. Properly logs error codes.
     * 
     * @param response Response from the server, JSON encoded
     * @param args Args used to send the request
     */
    _handleResponse: function(response, args) {
        if(!response) {
            // no data from server
            throw new Error('Server response was empty.');
        }

        var data = dojo.fromJson(response);
        this.workflow.lastResponse = data;
//        alert(response);

        if(data['code'] == 0) {
            // unknown server error
            this.showMessage(data['payload']);
            return;
        } else if (data['code'] > 0) {
            // known server error
            if (data['payload']['text']) {
            	this.showMessage(data['payload']['text']);
            } else {
            	this.showMessage(data['payload']);
            }
            
            return;
        }
//        alert('_handleResponse');
        var method = data['method'];
        var params = data['payload'];
        // @todo: do we do anything with encoding? data is json encoded
        // already, so previous decode should unwrap, right?

        try {
            // dispatch
            this[method](params);
        } catch (e) {
            // known client error
            var msgs = dojo.i18n.getLocalization('hub', 'messages');
            this.showMessage(msgs.CLIENT_ERROR + "\n" + 
                             e.message);
        }

//        alert('_handleResponse workflow-next');
		if(method!='done'){
			dojo.publish('workflow-next-'+this.tab.id ,[]);
		}	        
    },

    /**
     * Makes a RPC from client to server.
     * 
     * @method Method to invoke server-side
     * @data Parameters for the server side method
     * @transport Method of transport, either 'xhr' or 'iframe'
     */
    invokeServer: function(method, data, transport) {
        var _self = this;
        _self.workflow.setBusy(true);
        
        if(typeof(data)=='string'){
        	data = dojo.by.id(data);
        }

        // set tab as busy
//		TOM.tabs.man.setTabBusy(true, this.tab);
//		hub.busyPage(this.tab, true, true);

        if(!transport) {
            transport = 'xhr';
        }

        if(transport == 'xhr') {
            var handleAs = "text";
        } else if (transport == 'iframe') {
            var handleAs = "text/html";
        } else {
            throw new Error("Unknown transport");
        }

        // clear any notification
        var e = document.getElementById(this.getEditorId()+'_status');
        if(e)
            e.innerHTML = '';

		var _self = this;
        var msgs = dojo.i18n.getLocalization('hub', 'messages');

		// build a request object and handler for dojo
		
		var kw = {
			url: this.ajax_url+'/method/'+method+'/transport/'+transport,
            handleAs: 'text', // let the callback decode
            timeout: 120000, // 2 minutes in milliseconds
//            sync: true,

            error: function(response, args) {
                if(response.dojoType == "cancel"){
                    _self.showMessage(msgs.CANCELLED);
                } else if (response.dojoType == "timeout"){
                    _self.showMessage(msgs.TIMEOUT);
                } else {
//                    var err = TOM.i18n.man._('SERVER_ERROR') + " " + response.message;
//                    _self.showMessage(TOM.i18n.man._(err));
					_self.showMessage(msgs.SERVER_ERROR+ " " + response.message);
                }
                // always set tab to not busy when done
//                TOM.tabs.man.setTabBusy(false, _self.tab);
                _self.workflow.setBusy(false);
                
                return response;
            },

            load: function(response, args) {
                try {
                    // pull from text area
                    data = response.body.childNodes[0].value;
                } catch(e) {
                    // not loaded via iframe
                }

				try {
                    if(hub.submit.processResponseCode(response)){
					    _self._handleResponse(response, args);
				    }
				} catch(e) {
				                  // error handling response
				                  var err = msgs.CLIENT_ERROR + " " + e.message;
				                  _self.showMessage(err);
				}

                // stop spinner
//                TOM.tabs.man.setTabBusy(false, _self.tab);
//				hub.busyPage(_self.tab, false, false);
                _self.workflow.setBusy(false);
                return response;
            }
		};

        // decide what the data source is
        if(data.elements != undefined) {
            // data from a form
            kw['form'] = data;
        } else {
            // data from object name/value pairs
            kw['content'] = data;
        }
        
        // pick a transport and send
        if(transport == "iframe") {
            
			dojo.io.iframe.send(kw);
        } else {
            dojo.xhrPost(kw);
        }

        // convenience for use in onsubmit
        return false;
    }
});

}

if(!dojo._hasResource['hub.widgets.tabs.Workflow']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.widgets.tabs.Workflow'] = true;
dojo.provide('hub.widgets.tabs.Workflow');










/**
 * Manages a set of child content panes. UI can drive tabs to flip foward and 
 * backward through panes. Supports caching of previous panes so they can be 
 * restored without going to the server. Also supports rendering of both 
 * plug-ins and standard template forms as separate pages in the stack.
 */ 
dojo.declare('hub.widgets.tabs.Workflow', [dijit._Widget, dijit._Templated ], {
    // @todo: implement
    widgetsInTemplate: true,
    templateString:"<div class=\"workflow\" style=\"height: 100%;\">\r\n    <div dojoAttachPoint=\"workflowLayout\" dojoType=\"dijit.layout.LayoutContainer\" style=\"height: 100%; width: 100%; padding: 0px; margin: 0px\">\r\n\t\t<div dojoType='dijit.layout.ContentPane' layoutAlign='client' style='overflow: hidden'>\r\n\t\t\t<div class=\"workflowContainer\" dojoType=\"dijit.layout.StackContainer\" dojoAttachPoint=\"workflowContainer\" doLayout=\"false\" \r\n\t\t\t\t></div>\r\n\t\t</div>\r\n\t\t<div dojoType='dijit.layout.ContentPane' layoutAlign='bottom'>\r\n\t\t\t<div dojoAttachPoint=\"workflowNavigation\" style=\"text-align: left; padding: 10px 0px 10px 180px; display: none;\"\r\n\t\t\t\t><button dojoType=\"hub.widgets.Button\" dojoAttachPoint=\"buttonPrevious\" dojoAttachEvent=\"onClick:previous\" >${labels.BUTTON_PREVIOUS}</button \r\n\t\t\t\t><button dojoType=\"hub.widgets.Button\" dojoAttachPoint=\"buttonCancel\" dojoAttachEvent=\"onClick:onCancel\" >${labels.BUTTON_CANCEL}</button\r\n\t\t\t\t><button dojoType=\"hub.widgets.Button\" dojoAttachPoint=\"buttonNext\" dojoAttachEvent=\"onClick:next\" >${labels.BUTTON_NEXT}</button\r\n\t\t\t\t\r\n\t\t\t></div>\r\n\t\t</div>\r\n\t</div>\r\n</div>\r\n<!-- dojoType=\"hub.pages.edit.FeedChooser\" -->\t \r\n",
    plugin: null,
    entryId: null,
    pluginId: null,
    pluginDedicated: false,
    pluginStartScreen: null,
    saveScreen: null,
    reedit: false,
    selectPluginForm: null,
    complete: false,
    totalScreens: 0,
    currentScreen: 0,
//    processors: null,
    states: null,
    lastResponse: null,
    type: null,
    tab: null,
    saved: false,
    locked: false,
    fullscreen: false,
    editAsNew: false,
    labels: null,
    
	startup: function(){
//        console.log('>>Workflow.startup');
		this.setBusy(true);
//        console.log('<<Workflow.startup');
		this.inherited('startup', arguments);
        this.reedit = (!this.editAsNew && this.entryId != null && this.entryId > 0);
        
//        dojo.connect(this.workflowContainer, 'selectChild', this, 'setNotBusy');
        
//        console.log('this.reedit = ' + this.reedit + ' this.entryId = ' + this.entryId);
        this._hitchedInvalidate = dojo.hitch(this, this._onInvalidate);
        this._observerHandle = null;
/*        if(this.entryId){
        	this.reedit = true;
        }*/
		if(this.entryId && !this.editSource){
        	this._loadSaveForm({entry_id: this.entryId});
        	this.currentScreen++;
		}else if(this.pluginId 
        && (!this.entryId || this.entryId && this.editSource)){
        	this.pluginDedicated = true;
        	this._loadPluginForm();
//        	this.currentScreen++;
        }else{
        	this._loadPluginChooser();
			this.selectPluginForm.setType(this.type);
        }
        this.states = new Array();
        
        if(!this.fullscreen){
        	this.workflowNavigation.style.display = "";
        	//this.workflowContainer.style.height = "100%";
        }
        // if we have an initial object id, make sure we watch for tab
        // invalidation
        if(this.entryId && !this.editAsNew) {
            this._observerHandle = dojo.subscribe('invalidate-tab-'+this.entryId, null, this._hitchedInvalidate);
        }
        this._nextHandle = dojo.subscribe('workflow-next-'+this.tab.id, null, dojo.hitch(this, this._onNextStepComplete));
        this._doneHandle = dojo.subscribe('workflow-done-'+this.tab.id, null, dojo.hitch(this, this._onDone));
        if(this.domNode!=null){
	        this.updateControls();
	        this.workflowLayout.layout();
        }
	},
	
	postMixInProperties: function() {
	    this.labels = dojo.i18n.getLocalization('hub', 'labels');
	},

    /**
     * Watches for tab closing and cleans up any junk we may have created.
     * Watches for tab content replacement, and updates the stack pointer
     * appropriately.
     *
     * @param signal Always 'invalidate-tab-' plus the entryId
     * @param payload Object having an action that is either 
     *   'replace' or 'close'
     */
    _onInvalidate: function(signal, payload) {
        if(payload.action == 'close') {
            // only dispose if closing, not replacing with new content
            this.cancel(payload);
        }
    },


    /**
     * Performs a diff detection for changes based on all values in a form.
     * Checks to see if the plug-in has methods
     *
     */
    _cmpStates: function(s1, s2) {
        if(s1 == null || s2 == null) {
            // not objects, just compare if both are null
            return s1 == s2;
        }
        // check that all values in s1 are equal to those in s2
        for(var key in s1) {
            if(s1[key] != s2[key]) {
                return false;
            }
        }
        // check that all values in s2 are equal to those in s1
        // NOTE: have to do this step incase size(s2) > size(s1) but we have
        //   no length property on objects
        for(var key in s2) {
            if(s1[key] != s2[key]) {
                return false;
            }
        }
        return true;
    },

    /**
     * Gets the state of a single screen in workflow for later diff 
     * detection when we must decide if we should go to the server for the
     * the next page or use a cached copy.
     *
     * @return State object that will be compared later
     */
    _getScreenState: function() {
//    console.log('Workflow._getScreenState()');
        var screen = this.workflowContainer.selectedChildWidget.domNode;

        // default to getting state from all form controls
        var forms = dojo.query('form', screen);
        var state = null;
        if(forms.length) {
            // only consider first form by default
            var form = forms[0];
//            console.dir({form: form});
            state = dojo.formToObject(form);
            var files = dojo.query("input[type=file]", form);
            for(var i = 0 ; i < files.length; i++){
            	state[files[i].name] = files[i].value;
            }
            
            
        }
        return state;
    },

	setTab: function(tab){
		this.tab = tab;
	},
	
	getTab: function(){
		return this.tab;
	},

    _onScreenAdded: function() {
        // parse tab for dojo content
//        console.log('workflow._onScreenAdded');
        var children = this.workflowContainer.getChildren();
//        dojo.parser.parse(children[children.length-1].domNode);
		if(dojo.isIE > 0){
	        hub.evaluate(children[children.length-1].domNode, true, true);
        }else{
	        dojo.parser.parse(children[children.length-1].domNode);
        }
        // set tab unbusy
//        console.log('>>workflow._onScreenAdded');
        this.setBusy(false);        
//        console.log('<<workflow._onScreenAdded');
    },
    /**
     * Invoked by a plug-in to create a panel showing its next screen.
     * Evaluates all JS and Dojo markup. Consequently hides the previously
     * shown screen.
     * 
     * @param node DomNode to initialize on the screen
     * @param options abortRender|showControls|pluginStart
     * @return New screen DOM node
     */
    appendScreenDOM: function(node, options) {
//		console.log('Workflow.appendScreenDOM');
		if(options == null){
			options = {};
		}

        var newTab =  new dojox.layout.ContentPane({
		                        title:'newChild',
		                        refreshOnShow:true
			                }
				        );
       
        // find my tabContainer and add our new ContentPane
        newTab.domNode.appendChild(node);
        newTab.resize = true;
        this.workflowContainer.addChild(newTab);
        newTab.domNode.style.width = "100%";
        newTab.domNode.style.height = "100%";
        newTab.domNode.style.marginTop = "0px";
        newTab.domNode.style.marginRigth = "0px";
        newTab.domNode.style.marginBottom = "0px";
        newTab.domNode.style.marginLeft = "0px";
        newTab.domNode.style.paddingTop = "0px";
        newTab.domNode.style.paddingRigth = "0px";
        newTab.domNode.style.paddingBottom = "0px";
        newTab.domNode.style.paddingLeft = "0px";
        // evaluate Dojo only, JS seems to be evaluated for us by the tab
//        hub.evaluate(newTab, true, true);
        this.totalScreens++;
        if(options.pluginStart==null && options.pluginStart){
        	this.pluginStartScreen = this.currentScreen;
        }
        if(options.showControls==null){
        	newTab.showControls = this.fullscreen?false:true;
        }else{
        	newTab.showControls = options.showControls;
        }
        if(options.abortRender==null){
        	this._onScreenAdded();
        }
        return newTab;
    },

    /**
     * Invoked by a plug-in to create a panel showing its next screen.
     * Evaluates all JS and Dojo markup. Consequently hides the previously
     * shown screen.
     * 
     * @param html Content to initialize on the screen
     * @return New screen DOM node
     */
    appendScreenHTML: function(html, options) {
// 		alert('appendScreenHTML');
        // create, populate, append element
        var div = document.createElement('DIV');
        // to make sure we don't mess up large editors
        if(this.fullscreen) {       
        	div.style.height = "100%";
       	} else {
			div.style.height = "1%"; 
       	}
        div.style.width = "100%";
        
        div.innerHTML = html;
        var result = this.appendScreenDOM(div, options);
        if (dojo.isSafari) {
        	var lowercase = html.toLowerCase();
        	for(;;) {
        		if (lowercase.indexOf("<script") < 0) {
        			break;
        		}
       			var sstart = lowercase.indexOf("<script");
       			var start = lowercase.indexOf(">", sstart);
       			var end = lowercase.indexOf("</script", sstart);
       			if ((end < 0) || (start < 0)) {
       				console.warn("Incomplete Script: " + html.substring(sstart));
       				break;
       			}
       			start = start + 1;
       			var js = html.substring(start, end);
       			html = html.substring(end + 1);
       			lowercase = html.toLowerCase();
       			dojo.eval(js);
        	}
        }
        return result;
    },

    /**
     * Invoked by a plug-in to create a panel showing its next screen.
     * Evaluates all JS and Dojo markup. Consequently hides the previously
     * shown screen.
     * 
     * @param page_cls Templated class to initialize on the screen
     * @return New screen DOM node
     */
 	appendScreenDijit: function(/*page_cls*/ dijit_cls, args){
// 		alert('appendScreenDijit');
        var div = document.createElement('DIV');

        var widget = new dijit_cls(args, div);

        return this.appendScreenDOM(div, args);
	},

    /**
     * Invoked by a plug-in to create a panel showing its next screen.
     * Evaluates all JS and Dojo markup. Consequently hides the previously
     * shown screen.
     * 
     * @param page_cls Templated class to initialize on the screen
     * @return New screen DOM node
     */
 	appendScreenDjango: function(/*page_cls*/ page_cls, args){
// 		alert('appendScreenDjango');
        var attach_div = document.createElement('DIV');
        var page_div = document.createElement('DIV');
        attach_div.appendChild(page_div);
//        console.log('>>workflow.appendScreenDjango');
        this.setBusy(true)
//        console.log('<<workflow.appendScreenDjango');
        args.abortRender = true;
        
        var newTab = this.appendScreenDOM(attach_div, args);
        // create attach point widget
        var ap = new dojox.dtl.AttachPoint(null, attach_div);
        // create page widget
        args.tab = this.tab;
        var p = new page_cls(args, page_div);
        // listen for render complete
        p.isRendered().addCallback(dojo.hitch(this, this._onScreenAdded));
        
        return newTab;
	},

	onCancel: function(){
//		console.log('workflow.onCancel')
		this._callSaveCallback({cancel: true, action: 'cancel'});
		if(this.reedit){
			this.tab.setPage(hub.pages.Details, {object_id: this.entryId});
			this.dispose;
		}else{
			hub.closePage(this.tab);
		}
	},
	
    /**
     * Contacts the server to delete the current entry if it's new.
     *
     * @param payload Contains information about a close signal
     */
    cancel: function(payload) {
//    	console.log('Workflow.cancel');
    	if(this.locked){
    		return false;
    	}
        // only need to delete if defining a new entry and we have a 
        // plug-in reference
//		console.log('this.reedit = ' + this.reedit);

		if (this.reedit) {
			// make cancel call (restore an entry data from temporary table, if the entry has been opened for editing source)
			// server-side code will check if really nessesary to restore data 
	        try {
	            var cancel_url = hub.config.base_url + "/plugin/cancel/entryid/" + this.entryId + "/pluginid/" + this.pluginId;
	            var form = new hub.submit.edit.Cancel(cancel_url, null, null);
	            form.submit();
	        } catch (e) {}
		}
		
        if(this.plugin && !this.reedit) {
            // delete existing entry, we already asked for confirmation
            var delete_url = hub.config.base_url + "/plugin/delete/entryid/" +
                this.entryId + "/pluginid/" + this.pluginId;
            var form = new hub.submit.edit.DeleteAlways(null, null, delete_url);
            try {
                form.submit();
            } catch (e) {
//                this.showMessage(e);
                hub.notify(e);
            }
            // continue with final dispose operations
            this.dispose();
        } else if(payload.external) {
            // if triggered by an external source, go back to details page,
            // user is not trying to explicitly close the tab

            // continue with final dispose operations
            // important that his comes before reuse of tab
            this.dispose();

            // cancel any close operation
            this.domNode.setAttribute('abort_close', true);

            // go to the details page
            var entryId = this.domNode.getAttribute('object_id');
            var title = this.domNode.getAttribute('title');
/*            TOM.tabs.man.showTab(hub.config.base_url+'/ui/details/entryid/'+entryId, // url
                                 title, // title 
                                 entryId, // object
                                 true, // current tab
                                 true); // doesn't matter if safe or not*/
			hub.showUrl(title, hub.config.base_url+'/ui/details/entryid/'+entryId, {safe: true});                             
                    
        } else {
            // user is explicitly trying to close the tab
            // continue with final dispose operations
            this.dispose();
        }
    },

    /**
     * Completes editing by disposing of this object and any plug-in.
     */
    dispose: function() {
        // remove as listener else we're still hanging around
//        console.log('Workflow.dispose')
/*        TOM.bus.man.removeObserver(this._hitchedInvalidate, 
                                   'invalidate-tab-'+this.entryId);*/
                                   
        dojo.unsubscribe(this._observerHandle);
        dojo.unsubscribe(this._nextHandle);
        dojo.unsubscribe(this._doneHandle);

        // inform a plug-in that it is also complete
        if(this.plugin) {
            this.plugin.dispose();
            this.plugin = null;
        }

        // for good measure, dump our history
        this.history = new Array();
        this.screens = new Array();
//        this.complete = true;
        this.setComplete();

        // remove us from the tab, avoid cyclic invocation of this method
//        this.domNode.delAttribute('workflow', true);
    },
    
    setBusy: function(/*boolean*/ busy){
//    	console.log('>>workflow.setBusy('+busy+')');
    	this.locked = busy;
		this.tab.setBusy(busy);
//    	console.log('<<workflow.setBusy('+busy+')');
		
		return busy;
    },
    
    setNotBusy: function(){
//     	console.log('>>workflow.setNotBusy');
    	this.setBusy(false);
//     	console.log('<<workflow.setNotBusy');
    },
    
    /**
     * Go to the next screen in the workflow. This might mean getting it
     * from the server or using a hidden panel if the current screen has
     * not changed in any way.
     *
     * @param func Function to invoke if we have to go to server
     */
    next: function() {
		console.log('>>workflow.next');
    	if(this.locked){
    		return false;
    	}
    	this.setBusy(true);
		console.log('<<workflow.next');
    	console.log('totalScreens='+this.totalScreens+' > currentScreen='+this.currentScreen);
   		var currentState = this._getScreenState();
   		var savedState = this.states[this.currentScreen];
    	if(this.currentScreen < this.totalScreens){
    	console.log('it is not the last screen');
    	//  it is not the last screen
    		console.dir({currentState:currentState,savedState:savedState});
	   		if(this._cmpStates(currentState,savedState)){
	    	console.log('>>the state hasn`t changed');
	   		//  the state hasn`t changed
   				this.workflowContainer.forward();
   				this.currentScreen++;
		    	this.updateControls();
		    	this.setBusy(false);
//	    	console.log('<<the state hasn`t changed');
   				return true;
   			}else{
	    	console.log('the state has changed');
	   		//  the state has changed
	   			this.saved = false;
   				var nextTab=null;
//   				while(this.totalScreens>this.currentScreen+1){
   				while(this.totalScreens>this.currentScreen){
		    	console.log('deleting next screens');
   				//  deleting next screens
   					nextTab = this.workflowContainer.getChildren()[this.totalScreens-1];
   					nextTab.destroyDescendants();
   					this.workflowContainer.removeChild(nextTab);
   					this.totalScreens--;
   					if(this.currentScreen < this.saveScreen+1){
   						this.complete = false;
   					}
//   					console.log('next false-ing complete');
   				}
   			}
    	}
    	var processor = null;//this.processors[this.currentScreen];
//		console.log('complete = ' + this.complete);
    	if(!this.isComplete()){
    	console.log('workflow is not completed');
			if(!this.pluginDedicated && this.currentScreen==1){    	
//    		console.log('it is time to load plugin form');
		        processor = dojo.hitch(this, this._loadPluginForm);
		    }else{
    		console.log('submiting current step');
		        processor = dojo.hitch(this, this._submitCurrentForm);
		    }
    	}else{
	   	console.log('workflow completed saving current entry');
	        processor = dojo.hitch(this, this._saveCurrentEntry);
    	}
		var result = processor();
   		return true;
    },


    /**
     * Go to the previous screen in the workflow. Doesn't contact the server
     * in any way. Just shows a previously hidden panel.
     */
    previous: function() {
    	if(this.locked){
    		return false;
    	}
    	var fromSave = ((this.saveScreen != null) && (this.saveScreen + 1 == this.currentScreen));
    	if (fromSave) {
    		this._callSaveCallback({cancel: false, action: "prev"});
    	}	
		this.currentScreen--;
    	this.workflowContainer.back();
//        console.log('Workflow.previous -> updateControls');
    	this.updateControls();
    },
    
	_onDone: function(){
   		var currentState = this._getScreenState();
   		this.states[this.currentScreen] = currentState;
		//	the entry was inserted, now it should be saved via saveForm
		this._loadSaveForm({entry_id: this.entryId});
		//	show the next screen
		this.workflowContainer.forward();
		this.currentScreen++;
//        console.log('>>Workflow.onDone');
		this.updateControls();
		this.setBusy(false);
//        console.log('<<Workflow.onDone');
		return true;
	},
    
    _onNextStepComplete: function(){
//    console.log('Workflow._onNextStepComplete');
   		var currentState = this._getScreenState();
   		this.states[this.currentScreen] = currentState;
   		if(this.lastResponse.method){
   			if(this.lastResponse.method =='done'){
   			//	the entry was inserted, now it should be saved via saveForm
				this._loadSaveForm({entry_id: this.entryId});
	//			this.processors[this.currentScreen] = dojo.hitch(this, this._saveCurrentEntry);
			}else{
//				this.plugin[this.lastResponse.method](this.lastResponse.payload);
			} 
   		}else{
   			if(this.isComplete()){
	   		//	the entry was saved via saveForm
	   			if(this.reedit){
				//  show details page (it will be rendered, as tabs.refreshAllTabs() will be called as a result of hub.save-entry event)	   			
			        var page_cls = hub.pages.Details;
			        
			        this.tab.page_args = {};
			        this.tab.page_args.object_id = this.entryId;
					this.tab.page_args.tab = this.tab;
					this.tab.page_cls = page_cls;
					this.dispose();				
		   			return true;
	   			}else{
			        var page_cls = hub.pages.edit.Landing;
			        
			        // ask page *class* for properties (static method call)
			        var props = page_cls.prototype.getProperties();
			        // mix given args on top of class-specified args
			        //dojo.mixin(props, args);
			        props.entry_id = this.entryId;
			        props.plugin_id = this.pluginId;
			        props.entry_title = 'New entry'; // @TODO Put title from Save here 
					props.tab = this.tab;
			        // get/create tab
			        // instruct the tab to render the page when appropriate
			        props.showControls = false;
			        this.appendScreenDjango(page_cls, props);
	   			}
   			}
   		}
		//	show the next screen
		this.workflowContainer.forward();
		this.currentScreen++;
//        console.log('>>Workflow._onNextStepComplete()');
		this.updateControls();
		this.setBusy(false);
//        console.log('<<Workflow._onNextStepComplete()');
		return true;
	},

	_loadPluginChooser: function(){
        var div = document.createElement('div');
        var newTab = this.appendScreenDOM(div);

        var pluginChooser = new hub.widgets.forms.SelectPlugin({type:this.type}, div);
        
        this.selectPluginForm = pluginChooser;
        dojo.connect(this.selectPluginForm.inputSelect, 'onchange', this, 'releaseNextButton');
        this.buttonNext.setDisabled(true);
		this.workflowContainer.selectChild(newTab);
//		console.log('>>workflow._loadPluginChooser');
    	this.setBusy(false);
//		console.log('<<workflow._loadPluginChooser');
    	this.currentScreen++;
        return newTab; 
	},

	/**
	 * load first plugin page
	 */
    _loadPluginForm: function(){
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    if(!this.pluginDedicated){
		    this.pluginId = this._getScreenState().pluginid;
	    }
	    var url = hub.config.base_url + '/plugin/editor/json/1';
	    if(this.entryId > 0){
	    	url = url +'/entryid/'+this.entryId;
	    }
//	    var form = new hub.submit.Form(	this.pluginDedicated?null:this.selectPluginForm.domNode,
	    var form = new hub.submit.Form(	null, null,	hub.config.base_url + '/plugin/editor/json/1');
	    if(this.pluginDedicated){
	    	form.content = {pluginid:this.pluginId, entryid: this.entryId, cloneentry: this.editAsNew};
	    }else{
	    	form.content = dojo.formToObject(this.selectPluginForm.domNode);
	    }
	    form._onLoad = dojo.hitch(this, function(response){
            var labels = dojo.i18n.getLocalization('hub', 'labels');
	        var response = dojo.fromJson(response);
	       	this.lastResponse = response;
	       	this.entryId = response.result.object_id;
		    this.tab.entryId = this.entryId;
		    if(this.entryId && !this._observerHandle) {
               this._observerHandle = dojo.subscribe('invalidate-tab-'+this.entryId, null, this._hitchedInvalidate);
            }
	       	if(!this.reedit && !this.editAsNew){
	       		var label = '';
	       		switch(this.type){
	       			case 'Mashups': 
	       				label = labels.NEW_MASHUP_TAB;
	       				break;
	       			case 'Widgets': 
	       				label = labels.NEW_WIDGET_TAB;
	       				break;
	       			case 'pages': 
	       				label = labels.NEW_PAGE_TAB;
	       				break;
	       			case 'Feeds':
	       			default: 
	       				label = labels.NEW_FEED_TAB;
	       				break;
	       		}
		       	hub.renamePage(this.tab, label+' '+this.entryId);
	       	}
	       	this.plugin = new hub.managers.InvokePlugin(this.pluginId, this.entryId, this);
			this.appendScreenHTML(this.lastResponse.result['html'], {pluginStart:true});
			this.pluginStartScreen = this.currentScreen;
//		console.log('>>workflow._loadPluginForm');
//        	this.setBusy(false);
//		console.log('<<workflow._loadPluginForm');
           	dojo.publish('workflow-next-'+this.tab.id ,[]);
	    });

	    hub.submit.ajaxSubmit(form);
	    return true;
    },

    _submitCurrentForm: function() {
//        var screen = this.workflowContainer.getChildren()[this.currentScreen].domNode;
//        var screen = this.workflowContainer.getChildren()[this.currentScreen-1].domNode;
//		console.log('_submitCurrentForm');
        var screen = this.workflowContainer.selectedChildWidget.domNode;
        // first see if the plug-in wants to give us a custom state

        // default to getting state from all form controls
        var forms = dojo.query('form', screen);
//        var state = new Object();
        if(forms.length) {
            // only consider first form by default
            var form = forms[0];
            //form.submit();
            var func = form.onsubmit;
            func();
//            state = dojo.formToObject(form);
        }
        return null;//state;
    },

    _loadSaveForm: function(args) {
        
        args.showControls = true;
        this.appendScreenDjango(hub.pages.edit.Save, args);
        this.saveScreen = this.currentScreen;
        this.setComplete();
//		console.log('load save complete = ' + this.complete);
		
		
    },

	_saveCurrentEntry: function(){
//        var screen = this.workflowContainer.getChildren()[this.currentScreen].domNode;
//        var screen = this.workflowContainer.getChildren()[this.currentScreen-1].domNode;
//		console.log('_saveCurrentEntry');
        var screen = this.workflowContainer.selectedChildWidget.domNode;
        // first see if the plug-in wants to give us a custom state

        // default to getting state from all form controls
        var forms = dojo.query('form', screen);
        if(forms.length) {
            // only consider first form by default
            var form = forms[0];
		    var saveForm = new hub.submit.edit.Save(form,
		    	 							null, 
		    	 							hub.config.base_url + '/plugin/update/entryid/' + this.entryId + '/pluginid/' + this.pluginId);
		    saveForm._onLoad = dojo.hitch(this, function(response){
				this.lastResponse = response;
				this.saved = true;
				this._callSaveCallback({cancel: false, action: "save"});
				if (form && form.title) {
				   // _renamePage calls htmlspecialchars on title...
				   hub.renamePage(this.tab, form.title.value);
				}
				dojo.publish('workflow-next-'+this.tab.id,[]);
//				console.log('workflow._saveCurrentEntry -> publishing events');
		        dojo.publish('hub.save-entry', [this.entryId]);
//		        dojo.publish('hub.edit-entry', [this.entryId]);
		    });
	
		    hub.submit.ajaxSubmit(saveForm);
//		console.log('>>workflow._saveCurrentEntry');
//		    this.setBusy(false);
//		console.log('<<workflow._saveCurrentEntry');
	    }
	    
	    return true; //result;
	},

	updateControls: function(){
//    	this.buttonPrevious.disabled = this.currentScreen==1?"true":"";
    	this.buttonPrevious.setDisabled(this.currentScreen==1? true: false);
    	var finished = this.isComplete() && this.currentScreen == this.saveScreen+1;
//    	console.log('Workflow.updateControls: isComplete()=' + this.isComplete() 
//    										+ ' currentScreen='+ this.currentScreen + '==' 
//    										+ ' totalScreens=' + this.totalScreens);
/*    	if(finished){
    		this.buttonNext.innerHTML = "Finish";
    	}else{
    		this.buttonNext.innerHTML = "Next";
    	}*/
    	this.buttonNext.setLabel(finished?this.labels.BUTTON_FINISH:this.labels.BUTTON_NEXT);
    	
//    	if(this.fullscreen && !finished){
		if(this.workflowContainer.selectedChildWidget!=null){
	    	if(!this.workflowContainer.selectedChildWidget.showControls){
	    	   	this.workflowNavigation.style.display = "none";
	    	}else{
	    	   	this.workflowNavigation.style.display = "";
	    	}
    	}
//    	this.workflowLayout.resize();
        hub.resizeLayoutContainers(this.workflowLayout); // MH-87
	},
	
	releaseNextButton: function(event){
        this.buttonNext.setDisabled(false);
	},

    /**
     * Go to the first screen in the plug-in editor if it has been visited
     * at least once in the workflow. Useful for continuing editing after
     * a save in the workflow. Always sets the reedit flag to true.
     */
    showPlugin: function() {
    	var pluginStartTab = this.workflowContainer.getChildren()[this.pluginStartScreen]; 
    	this.workflowContainer.selectChild(pluginStartTab);
    	this.currentScreen = this.pluginStartScreen+1;
//    	this.complete = false;
    	this.updateControls();
    },

    /**
     * Go to the save form if it has been visited
     * at least once in the workflow. Useful for continuing editing after
     * a save in the workflow. Always sets the reedit flag to true.
     */
    showSave: function() {
    	var saveTab = this.workflowContainer.getChildren()[this.saveScreen]; 
    	this.workflowContainer.selectChild(saveTab);
    	this.currentScreen = this.saveScreen+1;
//    	this.complete = true;
    	this.updateControls();
    },

    /** 
     * Stores a reference to the plug-in that is part of the workflow.
     * 
     * @param plugin The plugin
     */
    setPlugin: function(plugin) {
        // make sure we throw away any old observer for an old object id
        if(this.entryId) {
            // remove as listener else we're still hanging around
            /*TOM.bus.man.removeObserver(this._hitchedInvalidate, 
                                       'invalidate-tab-'+this.entryId);*/
            dojo.unsubscribe(this._observerHandle);
        }
        this.plugin = plugin;
        // for convenience, pull object id and plug-in id into this object
        this.entryId = this.plugin.getObjectId();
        this.pluginId = this.plugin.getPluginId();
        // track at which part in the history the plug-in was shown
        this.plugin_ptr = this.history_ptr+1;

        // listen for close events on the tab because we need to clean-up
/*        TOM.bus.man.addObserver(this._hitchedInvalidate, 
                                'invalidate-tab-'+this.entryId);*/
		this._observerHandle = dojo.subscribe('invalidate-tab-'+this.entryId, null, this._hitchedInvalidate);                
    },

    /** 
     * Gets a reference to the plug-in that is part of the workflow.
     */
    getPlugin: function() {
        return this.plugin;
    },

    /**
     * Gets the number of screens in the history.
     *
     * @return integer count
     */
    getHistorySize: function() {
        return this.history.length;
    },

    /**
     * Sets a flag indicating this workflow has been run to 
     * completion. The flag is unset if the workflow moves to any screen
     * after the flag is set.
     */
    setComplete: function() {
        this.complete = true;
    },

    /**
     * Gets the value of the completion flag.
     */
    isComplete: function() {
        return this.complete;
    }, 
    
    isModified: function(){
    	return (this.plugin!=null || this.saveScreen!=null) && !this.saved;
    },
    
    isReedit: function() {
        return this.reedit;
    },
    
    isSaved: function() {
        return this.saved;
    },
    
    _callSaveCallback: function(m) {
		if (this.plugin) {
			if (this.plugin.saveCallbackObject && this.plugin.saveCallbackFunction) {
				var p = {};
				p.saved = this.isSaved();
				p.reedit = this.isReedit();
				dojo.mixin(p, m);
				this.plugin.saveCallbackObject[this.plugin.saveCallbackFunction](p);
			}
		}    
	}
});

}

if(!dojo._hasResource['hub.widgets.tabs.edit']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.widgets.tabs.edit'] = true;
dojo.provide('hub.widgets.tabs.edit');



/**
 * Manages a set of child content panes. UI can drive tabs to flip foward and 
 * backward through panes. Supports caching of previous panes so they can be 
 * restored without going to the server. Also supports rendering of both 
 * plug-ins and standard template forms as separate pages in the stack.
 */ 
dojo.declare('hub.widgets.tabs.edit.WorkflowPane', [dijit._Widget, dijit._Templated ], {
    // @todo: implement
    widgetsInTemplate: true,
    templateString:"<div class=\"workflow\" style=\"height: 100%;\">\r\n    <div dojoAttachPoint=\"workflowLayout\" dojoType=\"dijit.layout.LayoutContainer\" style=\"height: 100%; width: 100%; padding: 0px; margin: 0px\">\r\n\t\t<div dojoType='dijit.layout.ContentPane' layoutAlign='client' style='overflow: hidden'>\r\n\t\t\t<div class=\"workflowContainer\" dojoType=\"dijit.layout.StackContainer\" dojoAttachPoint=\"workflowContainer\" doLayout=\"false\" \r\n\t\t\t\t></div>\r\n\t\t</div>\r\n\t\t<div dojoType='dijit.layout.ContentPane' layoutAlign='bottom'>\r\n\t\t\t<div dojoAttachPoint=\"workflowNavigation\" style=\"text-align: left; padding: 10px 0px 10px 180px; display: none;\"\r\n\t\t\t\t><button dojoType=\"hub.widgets.Button\" dojoAttachPoint=\"buttonPrevious\" dojoAttachEvent=\"onClick:previous\" >${labels.BUTTON_PREVIOUS}</button \r\n\t\t\t\t><button dojoType=\"hub.widgets.Button\" dojoAttachPoint=\"buttonCancel\" dojoAttachEvent=\"onClick:onCancel\" >${labels.BUTTON_CANCEL}</button\r\n\t\t\t\t><button dojoType=\"hub.widgets.Button\" dojoAttachPoint=\"buttonNext\" dojoAttachEvent=\"onClick:next\" >${labels.BUTTON_NEXT}</button\r\n\t\t\t\t\r\n\t\t\t></div>\r\n\t\t</div>\r\n\t</div>\r\n</div>\r\n<!-- dojoType=\"hub.pages.edit.FeedChooser\" -->\t \r\n",
    tab: null,
    plugin: null,
    object_id: null,
    plugin_id: null,
    moved_next: false,
    history_ptr: -1,
    screen_ptr: -1,
    plugin_ptr: null,
    screens: null,
    history: null,
    reedit: false,
    complete: false,
    _hitchedInvalidate: null,
    _observerHandle: null,
    
    /**
     * @param tab Reference to the tab containing this workflow
     * @param object_id Define if object under editing is known up front,
     *   otherwise it will be set when setPlugin is called later
     */
/*    constructor: function(tab, object_id) {
    	this.inherited({}, tab);
        // keep track of our tab
        this.tab = tab;
        this.plugin = null;
        this.object_id = object_id;
        this.plugin_id = null;
        this.moved_next = false;
        this.history_ptr = -1;
        this.screen_ptr = -1;
        this.plugin_ptr = null;
        this.screens = this.tab.childNodes;
        this.history = new Array();
        this.reedit = (object_id != null);
        this.complete = false;
        this._hitchedInvalidate = dojo.hitch(this, this._onInvalidate);
        this._observerHandle = null;

        // if we have an initial object id, make sure we watch for tab
        // invalidation
        if(this.object_id) {
/*            TOM.bus.man.addObserver(this._hitchedInvalidate, 
                                    'invalidate-tab-'+this.object_id);*
            this._observerHandle = dojo.subscribe('invalidate-tab-'+this.object_id, null, this._hitchedInvalidate);
        }

        // clear out any old content when we're getting started, but don't
        // use setContent since we overrode that to append screens in our
        // tab
        this.tab.innerHTML = '';
    },*/

	startup: function(){
//        this.tab = tab;
//        this.screens = this.workflowContainer.childNodes;
        this.history = new Array();
        this.reedit = (this.object_id != null);
        this._hitchedInvalidate = dojo.hitch(this, this._onInvalidate);
        this._observerHandle = null;

        // if we have an initial object id, make sure we watch for tab
        // invalidation
        if(this.object_id) {
            this._observerHandle = dojo.subscribe('invalidate-tab-'+this.object_id, null, this._hitchedInvalidate);
        }

        // clear out any old content when we're getting started, but don't
        // use setContent since we overrode that to append screens in our
        // tab
//        this.tab.innerHTML = '';
//        this.workflowContainer.innerHTML = '';
		dojo.byId('intro').style.display = "block";
		dojo.byId('intro').style.visibility = "visible";
		
	},

    /**
     * Watches for tab closing and cleans up any junk we may have created.
     * Watches for tab content replacement, and updates the stack pointer
     * appropriately.
     *
     * @param signal Always 'invalidate-tab-' plus the object_id
     * @param payload Object having an action that is either 
     *   'replace' or 'close'
     */
    _onInvalidate: function(signal, payload) {
        if(payload.action == 'close') {
            // only dispose if closing, not replacing with new content
            this.cancel(payload);
        }
    },

    /**
     * Advanced to the next item in history and its corresponding screen.
     */
    _ptrNext: function() {
        this.history_ptr += 1;
        for(var i = this.screen_ptr+1; i < this.screens.length; i++) {
            if(this.screens[i].nodeName == 'DIV') {
                this.screen_ptr = i;
                break;
            }
        }
    },

    /**
     * Retreat to the previous item in history and its corresponding 
     * screen.
     */
    _ptrPrevious: function() {
        this.history_ptr -= 1;
        for(var i = this.screen_ptr-1; i >= 0; i--) {
            if(this.screens[i].nodeName == 'DIV') {
                this.screen_ptr = i;
                break;
            }
        }
    },

    /**
     * Jumps to a specific item in history and its corresponding screen.
     */
    _ptrJump: function(history_ptr) {
        // compute difference in history counts
        var diff = Math.abs(history_ptr - this.history_ptr);
        // determine direction of jump
        if(history_ptr > this.history_ptr) {
            var i = this.screen_ptr + 1;
        } else {
            var i = this.screen_ptr - 1;
        }
        // store new history pointer
        this.history_ptr = history_ptr;

        var count = 0;
        for(; i >= 0; i--) {
            if(this.screens[i].nodeName == 'DIV') {
                // count number of screens we move
                count += 1;
            }
            if(count == diff) {
                // stop when we've moved the proper number of screens
                this.screen_ptr = i;
                break;
            }
        }
    },

    /**
     * Updates the held plug-in with the unique client ID for the shown
     * screen.
     */
    _updatePlugin: function(node) {
        // do nothing if no plug-in set
        if(!this.plugin)
            return;

        var divs = dojo.query('div', node);
        for(var i = 0; i < divs.length; i++) {
            if(divs[i].id) {
                // find first div with an id, and assume that's the unique
                // id for this screen of the editor
                this.plugin.setClientId(divs[i].id);
                break;
            }
        }
    },

    /**
     * Performs a diff detection for changes based on all values in a form.
     * Checks to see if the plug-in has methods
     *
     */
    _diffStates: function(s1, s2) {
        if(s1 == null || s2 == null) {
            // not objects, just compare if both are null
            return s1 == s2;
        }
        // check that all values in s1 are equal to those in s2
        for(var key in s1) {
            if(s1[key] != s2[key]) {
                return false;
            }
        }
        // check that all values in s2 are equal to those in s1
        // NOTE: have to do this step incase size(s2) > size(s1) but we have
        //   no length property on objects
        for(var key in s2) {
            if(s1[key] != s2[key]) {
                return false;
            }
        }
        return true;
    },

    /**
     * Gets the state of a single screen in workflow for later diff 
     * detection when we must decide if we should go to the server for the
     * the next page or use a cached copy.
     *
     * @return State object that will be compared later
     */
    _getScreenState: function() {
        var screen = this.screens[this.screen_ptr];

        // first see if the plug-in wants to give us a custom state
        if(this.plugin) {
            try {
                // if plug-in does not implement, we get a TOM exception
                // give the order index for this screen so the plug-in can
                // implement save logic for multiple screens
                return this.plugin.getScreenState(screen, this.history_ptr);
            } catch(e) {
                try {
                    if(e.getClass().getName() != 'TOM.Exception') {
                        // wasn't the TOM exception we were expecting
                        throw e;                            
                    }
                } catch (tmp) {
                    // wasn't the TOM exception we were expecting
                    throw e;
                }
            }
        }

        // default to getting state from all form controls
        var forms = dojo.query('form', screen);
        var state = new Object();
        if(forms.length) {
            // only consider first form by default
            var form = forms[0];
            for(var i = 0; i < form.elements.length; i++) {
                // get the current element
                var e = form.elements[i];
                var name = e.name;
                if(name) {
                    // if it has a name, get its value
                    var val = e.value || e.checked;
                    // store value keyed by name
                    state[name] = val;
                }
            }
        }
        return state;
    },


    /**
     * Invoked by a plug-in to create a panel showing its next screen.
     * Evaluates all JS and Dojo markup. Consequently hides the previously
     * shown screen.
     * 
     * @param html Content to initialize on the screen
     * @return New screen DOM node
     */
    appendScreen: function(html) {
        // hide current screen if it exists
        try {
            this.screens[this.screen_ptr].style.display = 'none';
        } catch(e) {
        }
        // create, populate, append element
        var div = document.createElement('DIV');
        // to make sure we don't mess up large editors
        div.style.height = "100%";
        div.innerHTML = html;
        this.workflowContainer.appendChild(div);
        // update the plug-in client id based on the content
        this._updatePlugin(div);
        // evaluate Dojo only, JS seems to be evaluated for us by the tab
        hub.evaluate(div, false, true);
        if(this.history_ptr < 0) {
            // try to locate a previous button on the first screen; we have
            // to jump through hoops here because we don't know the unique
            // id for the button since it may be from a plugin or not
            var nodes = dojo.query('button');
            if(nodes.length) {
                // only consider the third match from the end
                // super ugly!
                var button = dijit.byNode(nodes[nodes.length-3].parentNode.parentNode);
                if(button)
                    button.setDisabled(true);
            }
        }
        // update our pointers
        this._ptrNext();
        return div;
    },

    /**
     * Contacts the server to delete the current entry if it's new.
     *
     * @param payload Contains information about a close signal
     */
    cancel: function(payload) {
        // only need to delete if defining a new entry and we have a 
        // plug-in reference
        if(!this.reedit && this.plugin) {
            // delete existing entry, we already asked for confirmation
            var delete_url = hub.config.base_url + "/plugin/delete/entryid/" +
                this.object_id + "/pluginid/" + this.plugin_id
            var form = new TOM.editor.DeleteAlways(delete_url);
            try {
                form.submit();
            } catch (e) {
                this.showMessage(e);
            }
            // continue with final dispose operations
            this.dispose();
        } else if(payload.external) {
            // if triggered by an external source, go back to details page,
            // user is not trying to explicitly close the tab

            // continue with final dispose operations
            // important that his comes before reuse of tab
            this.dispose();

            // cancel any close operation
            this.domNode.setAttribute('abort_close', true);

            // go to the details page
            var object_id = this.domNode.getAttribute('object_id');
            var title = this.domNode.getAttribute('title');
/*            TOM.tabs.man.showTab(hub.config.base_url+'/ui/details/entryid/'+object_id, // url
                                 title, // title 
                                 object_id, // object
                                 true, // current tab
                                 true); // doesn't matter if safe or not*/
			hub.showUrl(title, hub.config.base_url+'/ui/details/entryid/'+object_id, {safe: true})                             
                    
        } else {
            // user is explicitly trying to close the tab
            // continue with final dispose operations
            this.dispose();
        }
    },

    /**
     * Completes editing by disposing of this object and any plug-in.
     */
    dispose: function() {
        // remove as listener else we're still hanging around
/*        TOM.bus.man.removeObserver(this._hitchedInvalidate, 
                                   'invalidate-tab-'+this.object_id);*/
        dojo.unsubscribe(this._observerHandle);

        // inform a plug-in that it is also complete
        if(this.plugin) {
            this.plugin.dispose();
            this.plugin = null;
        }

        // for good measure, dump our history
        this.history = new Array();
        this.screens = new Array();
        this.complete = true;

        // remove us from the tab, avoid cyclic invocation of this method
        this.domNode.delAttribute('workflow', true);
    },
    
    myNext: function(){
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    var kw = {
		url: hub.config.base_url + '/plugin/editor/json/1',
		sync: true,
		form: "formSelectPlugin",
	        handleAs: 'json',
	        timeout: 120000, // 2 minutes in milliseconds
	        
	        error: dojo.hitch(this, function(response, args) {           
	            if(response.dojoType == "cancel") {
	                alert(msgs.ERROR_REQUEST_CANCELLED);
	            } else if (response.dojoType == "timeout"){
	                alert(msgs.ERROR_REQUEST_TIMEOUT);
	            } else {
	                alert(response.message);
	            }
	            return response;
	        }),
	        
	        load: dojo.hitch(this, function(response, args) {
	            if(!response) {
	                alert( msgs.ERROR_REQUEST_SERVER);
	            } else if(response.error) {
	                alert(response.message);
	            } else {
//					this.workflowContainer.forward();
					var childDiv = dojo.doc.createElement('div');
					childDiv.innerHTML=response['result']['html'];
			        var newChild =  new dojox.layout.ContentPane({
							                        title:'newChild',
							                        refreshOnShow:true
							                },
							                childDiv
							        );
			       
			        // find my tabContainer and add our new ContentPane
			        this.workflowContainer.addChild(newChild);
			        this.workflowContainer.selectChild(newChild);
	            }
	            return response;
	        })
	    };
	    // do the post
	    dojo.xhrPost(kw);
	    
    	
    },

    /**
     * Go to the next screen in the workflow. This might mean getting it
     * from the server or using a hidden panel if the current screen has
     * not changed in any way.
     *
     * @param func Function to invoke if we have to go to server
     */
    next: function(func) {
        this.complete = false;
        // check if DOM in history equals DOM in screen
        var old_state = this.history[this.history_ptr];
        var new_state = this._getScreenState(); //this.screens[this.screen_ptr].innerHTML;
        var same = this._diffStates(old_state, new_state);
        // check if we have history from the next screen
        var next_state = this.history[this.history_ptr+1];
        // save the current state as history
        this.history[this.history_ptr] = new_state;

        if(same && next_state != null) { 
            // hide this element
            this.screens[this.screen_ptr].style.display = 'none';
            // move to next available screen
            this._ptrNext();
            // show the next element, it's up to date
            this.screens[this.screen_ptr].style.display = 'block';
        } else {
            // pop all screens past the current since they are now all 
            // stale; keep current
            while(this.screens.length > this.screen_ptr+1) {
                this.workflowContainer.removeChild(this.workflowContainer.lastChild);
            }
            this.moved_next = true;
            // invoke the next function
            func();
        }
    },

    /**
     * Go to the previous screen in the workflow. Doesn't contact the server
     * in any way. Just shows a previously hidden panel.
     */
    previous: function() {
        if(this.history_ptr > 0) {
            this.complete = false;
            // we use this as an indicator that we've visted this screen
            this.history[this.history_ptr] = this._getScreenState(); 
            // hide the current item, no signals
            this.screens[this.screen_ptr].style.display = 'none';
            // update the pointers
            this._ptrPrevious();
            // show the previous item, no signals
            this.screens[this.screen_ptr].style.display = 'block';
            // didn't move next
            this.moved_next = false;
            // update the plug-in client id based on the content
            this._updatePlugin(this.screens[this.screen_ptr]);
        }
        // do nothing if there is no previous step
    },

    /**
     * Go to the first screen in the plug-in editor if it has been visited
     * at least once in the workflow. Useful for continuing editing after
     * a save in the workflow. Always sets the reedit flag to true.
     */
    showPlugin: function() {
        if(this.plugin_ptr != null) {
            this.complete = false;
            // hide the current screen
            this.screens[this.screen_ptr].style.display = 'none';
            // update the pointers
            this._ptrJump(this.plugin_ptr);
            // show the first screen of the plug-in
            this.screens[this.screen_ptr].style.display = 'block';
            // didn't move next
            this.moved_next = false;
            // change the re-editing flag to true
            this.reedit = true;
            // update the plug-in client id based on the content
            this._updatePlugin(this.screens[this.screen_ptr]);
        }
    },

    /** 
     * Stores a reference to the plug-in that is part of the workflow.
     * 
     * @param plugin The plugin
     */
    setPlugin: function(plugin) {
        // make sure we throw away any old observer for an old object id
        if(this.object_id) {
            // remove as listener else we're still hanging around
            /*TOM.bus.man.removeObserver(this._hitchedInvalidate, 
                                       'invalidate-tab-'+this.object_id);*/
            dojo.unsubscribe(this._observerHandle);
        }
        this.plugin = plugin;
        // for convenience, pull object id and plug-in id into this object
        this.object_id = this.plugin.getObjectId();
        this.plugin_id = this.plugin.getPluginId();
        // track at which part in the history the plug-in was shown
        this.plugin_ptr = this.history_ptr+1;

        // listen for close events on the tab because we need to clean-up
/*        TOM.bus.man.addObserver(this._hitchedInvalidate, 
                                'invalidate-tab-'+this.object_id);*/
		this._observerHandle = dojo.subscribe('invalidate-tab-'+this.object_id, null, this._hitchedInvalidate);                
    },

    /** 
     * Gets a reference to the plug-in that is part of the workflow.
     */
    getPlugin: function() {
        return this.plugin;
    },

    /**
     * Gets the number of screens in the history.
     *
     * @return integer count
     */
    getHistorySize: function() {
        return this.history.length;
    },

    /**
     * Sets a flag indicating this workflow has been run to 
     * completion. The flag is unset if the workflow moves to any screen
     * after the flag is set.
     */
    setComplete: function() {
        this.complete = true;
    },

    /**
     * Gets the value of the completion flag.
     */
    isComplete: function() {
        return this.complete;
    }
});

}

if(!dojo._hasResource['hub.widgets.tabs']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.widgets.tabs'] = true;
dojo.provide('hub.widgets.tabs');













/**
 * Tracks the history of visited tabs so we can return to the last active,
 * living tab when the selected tab closes.
 */
dojo.declare('hub.widgets.tabs._TabHistory', null, {
	default_id: null,
	max_size: null,
	tab_id_stack: null,
	open_tab_ids: null,
	
    constructor: function(max_size, default_id) {
        this.default_id = default_id;
        this.max_size = max_size;
        this.tab_id_stack = new Array();
        this.open_tab_ids = new Object();
    },

    /**
     * Pops the given tab id out of the hash of all open tabs. Runs through
     * the stack of last visited tabs, but only if the closing tab is the
     * selected tab.
     * 
     * @param tab: Tab that is closing
     * @param need_pop: True to find the last active living tab
     */
    pop: function(tab, need_pop) {
//    console.log('tab_history.pop('+tab.id+')');
        // remove the id of the closing tab from our list of open tabs
        delete this.open_tab_ids[tab.id];
        if(!need_pop) {
            // don't continue if we don't need a pop (i.e., the closing
            // tab is not the selected tab)
            return null;
        }
        
        // pop the last id from the stack
        var id = this.tab_id_stack.pop();
        while(!this.open_tab_ids[id]) {
            if(!this.tab_id_stack.length) {
                return this.default_id;
            }
            id = this.tab_id_stack.pop();
        }
        return id;
    },
        

    /**
     * Pushes a tab id into the history of visited tabs and the stack of
     * all open tabs. Ignores duplicate pushes in the stack.
     *
     * @param tab: Tab that is being selected
     */
    push: function(tab) {
//    console.log('tab_history.push('+tab.id+')');
        if(this.tab_id_stack[this.tab_id_stack.length-1] == tab.id ||
           !tab.id) {
            // don't allow repeats in sequence, blank ids
            return;
        }
        this.tab_id_stack.push(tab.id);
        if(this.tab_id_stack.length > this.max_size) {
            this.tab_id_stack = this.tab_id_stack.slice(1);
        }
        // track that this tab is open
        this.open_tab_ids[tab.id] = true;
    }
});

/**
 * Manages opening, closing, refreshing, status, etc. of tabs.
 */
dojo.declare('hub.widgets.tabs.TabContainer', dijit.layout.TabContainer, {
    // widget to avoid when computing full screen height
    sizeAround: '',
    // maximum size of history ring buffer for tracking visited tabs
    historySize: 10,
    
    tab_history: null,
    
    tab_id_to_obj: null,
    
    next_key: 1,
    
    plugins: null,
    
    _started: false,

    constructor : function(args, node) {
        // tracks tabs opened by the user
        // key: bag of title, template, args, etc. identifying a page
        // value: Tab widget
        this.tab_id_to_obj = new dojox.collections.Dictionary();
        // unique tab id generator
        this.next_key = 1;
        
        this.plugins = Array();
    },

    /**
     * Overridden parent class method. Looks up the element to sizeAround.
     */
    postMixInProperties: function() {
        this.inherited(arguments);
        this.sizeAround = dojo.byId(this.sizeAround);
        // initialize the tab history tracker
        this.tab_history = new hub.widgets.tabs._TabHistory(this.historySize,
                                                            'hub_tab_1');
    },


    /**
     * Overridden parent class method. Start listening for events requesting a
     * page render and events indicating a refresh of one or more pages is 
     * needed.
     */
    postCreate: function() {
        this.inherited(arguments);
        dojo.subscribe('hub.login', this, 'refreshAllTabs');
        dojo.subscribe('hub.logout', this, 'refreshAllTabs');
        dojo.subscribe('hub.save-entry', this, 'refreshAllTabs');
        dojo.subscribe('hub.delete-entry', this, 'onEntryDelete');
        dojo.subscribe('hub.reset', this, 'refreshAllTabs');
        dojo.subscribe('hub.show-template-page', this, '_showTemplatePage');
        dojo.subscribe('hub.show-url-page', this, '_showUrlPage');
        dojo.subscribe('hub.show-plugin-page', this, '_showPluginPage');
        dojo.subscribe('hub.close-page', this, '_closePage');
        dojo.subscribe('hub.close-current-page', this, '_closeCurrentPage');
        dojo.subscribe('hub.is-page-active', this, '_isPageActive');
        dojo.subscribe('hub.busy-page', this, '_busyPage');
        dojo.subscribe('hub.rename-page', this, '_renamePage');

        // @todo: close tab having an object id
        //dojo.subscribe('hub.delete-entry', this, 'closeAllTabs');
        // @todo: close all tabs
    },

    /**
     * Overridden parent class method. Forces a resize.
     */
    startup: function() {
        this.inherited(arguments);
        // give the container its initial size
        this.resize();
    },

    /**
     * Overridden parent class method. Expands the tab container to full height
     * minus that of the sizeAround element.
     */
    resizeOld: function() { // TODO: check if this required
        if(this.sizeAround) {
            var height = (document.body.parentNode.offsetHeight -
                          this.sizeAround.offsetHeight - 100);
            this.domNode.style.height = height + 'px';
            this.layout();
        }
        this.inherited(arguments);
    },

    /**
     * Overridden parent class method. Pushes all selected tabs onto the 
     * history.
     *
     * @param tab: Tab to select
     */
    selectChild: function(tab) {
//    console.log('tabs.selectChild');
        if(this._started) {
            this.inherited(arguments);
            if (dojo.isString(tab)) {
            	tab = dijit.byId(tab);
            }
            // store tab id in history
            this.tab_history.push(tab);
            // publish an event so listeners know about the newly selected tab
            dojo.publish('hub.activate-page', [tab]);
	        hub.resizeLayoutContainers(tab); // MH-87
        }
    },

    /**
     * Overridden parent class method. Closes a tab and returns to the last
     * active tab
     *
     * @param tab: Tab to close
     * @return: false if operation cancelled, true if not
     */
    closeChild: function(tab) {
    	console.log('tabs.closeChild');
        // make sure child not closable first
        if(!tab.closable) {
            // return true, because it wasn't a cancellation by the user
            return true;
        }
        // reimplement here because parent class doesn't return the value of
        // remove for us (ugh)
		var remove = tab.onClose(this, tab);
		if(remove) {
	            // check if we're removing the active widget
	            var need_pop = (tab == this.selectedChildWidget);
	            // pop tab(s) until a live tab is reached
	            var next_tab_id = this.tab_history.pop(tab, need_pop);
	            if(next_tab_id != null) {
	                // select before removing else removeChild() forces us back to
	                // first tab
	                this.selectChild(next_tab_id);
	            }
	            this.tab_id_to_obj.remove(tab.plain_title.toLowerCase());
	            // remove tab from container
		    this.removeChild(tab);
	            // cleanup all widgets inside the tab
		    tab.destroyRecursive();
		}
        // return the value so we can reuse this method
        return remove;
    },

    /**
     * Forces a refresh of content in all tabs the next time each tab shows. 
     * Tabs can still choose to cancel the refresh, but only at the time they
     * are shown, not when this method is called.
     */
    refreshAllTabs: function() {
        dojo.forEach(this.getChildren(), function(tab) {
            if(tab == this.selectedChildWidget) {
                // force refresh now
                tab.refresh();
            } else {
                // do it later when the tab shows
                console.log('refreshAllTabs() - postpone');
                tab.refreshOnShow = !tab.isStatic && tab.entryId == null;
            }
        }, this);
        // force a resize
        this.resize();
    },

    /**
     * Attempts to close all tabs. If a tab aborts, all further closes are 
     * aborted also.
     *
     * @return: true if all closed, false if any one aborted
     */
    closeAllTabs: function() {
        var rv = dojo.every(this.getChildren(), this.closeChild, this);
        return rv;
    },

	closeEntryTabs: function(entryId){
//	console.log('tabs.closeEntryTabs('+entryId+')');
		if(dojo.isString(entryId)){
			entryId = parseInt(entryId);
		}
		if(entryId > 0){
			var tabs = this.getChildren(); 
			for(var i = 0; i < tabs.length; i++){
				if(entryId == tabs[i].getEntryId()){
//					console.log('entryId = ' + entryId + ' tabs[i].getEntryId() = ' + tabs[i].getEntryId());
					tabs[i].forceClose = true;
					this.closeChild(tabs[i]);
				}
			};
		}
	},
	
	onEntryDelete: function(args){
//		console.log('tabs.onEntryDelete');
//		console.dir({args:args});
		if(args){
			if(dojo.isArray(args)){
				for(var i = 0; i < args.length; i++){
					this.closeEntryTabs(args[i]);
				}
			}else{
				this.closeEntryTabs(args);
			}
			this.refreshAllTabs();
		}
	},

	isEverythingSaved: function(){
	console.log('tabs.isEverythingSaved()');
		var tabs = this.getChildren(); 
		for(var i = 0; i < tabs.length; i++){
			if(tabs[i].workflow!=null && tabs[i].workflow.isModified()){
				console.log('not Saved');
				return false;
			}
		};
		
		console.log('Saved');
		return true;
	},
	
    /**
     * Attempts to close one or more tabs. Return true if the operation was
     * not cancelled. This could mean the tab closed or did not exist to begin
     * with.
     *
     * @param node: Any DOM node in the page to close, or null for all pages
     * @param result: Optional deferred which will be notified with the result
     */
    _closePage: function(node, result) {
        // default to true meaning operation was accepted
        var rv = true;
        if(!node) {
            // close all tabs
            var rv = this.closeAllTabs();
        } else {
            // close a particular tab
            var tab = this._getTabForNode(node);
            if(tab) {
                var rv = this.closeChild(tab);
            }
        }
        if(result) {
            // if result provided, do callback
            result.callback(rv);
        }
        return true;
    },

    /**
     * Attempts to close one or more tabs. Return true if the operation was
     * not cancelled. This could mean the tab closed or did not exist to begin
     * with.
     *
     * @param node: Any DOM node in the page to close, or null for all pages
     * @param result: Optional deferred which will be notified with the result
     */
    _closeCurrentPage: function() {
        var tab = this.selectedChildWidget;
        if(tab) {
            var rv = this.closeChild(tab);
        }
        return true;
    },

    /**
     * Sets a tab as busy or not, or locked or not. Busy means the tab shows an
     * indicator near its title. Locked means the tab contents (and only this 
     * tab contents) are obscured by a dialog with a cancel button which 
     * prevents further user interaction with this tab.
     * 
     * @param node: Any node in the page
     * @param busy: True to mark busy, false to unmark
     * @param locked: True to lock, false to unlock
     */
    _busyPage: function(node, busy, locked) {
//    console.dir(node);
        // @todo: implement; do _getTabForNode and then show the busy spinner
        //   on the tab; need a design to support locking
        
        tab = this._getTabForNode(node);
        if(!tab) return;
        if (tab.workflow != null) {
           tab.workflow.setBusy(busy);
        }
        
        tab.setBusy(busy);
        
/*        var spinner = dojo.byId(tab.id+"_busy");
        if(!spinner) return;
        spinner.style.display = (busy) ? "inline" : "none";
*/        
    },

    /**
     * Renames an existing tab.
     * 
     * @param node: Any node in the page
     * @param title: New title
     * @param locked: Options for the newly renamed page
     */
    _renamePage: function(node, title, args) {
//    console.dir(node);
        // @todo: implement; refer to old tab manager for starting code
        tab = this._getTabForNode(node);
        if(!tab) return;
        var oldTitle = tab.plain_title;
        tab.setTitle(title);
        this.tab_id_to_obj.remove(oldTitle.toLowerCase());
        this.tab_id_to_obj.add(title.toLowerCase(), tab);
        
    },

    /**
     * Gets if a tab is active or not.
     * 
     * @param node: Any node in the page
     * @param result: Deferred which will be notified with the result
     */
    _isPageActive: function(node, result) {
//    console.dir(node);
        var rv = this.selectedChildWidget == this._getTabForNode(node);
        if(result) {
            result.callback(rv);
        }
    },

    /**
     * Returns markup for a tab label complete with spinner and full title
     * hover.
     *
     * @param title: Title for the tab, will be truncated if necessary
     * @param tab_id: Dijit id for the tab
     */
    _createTabLabel: function(title, tab_id) {
        // shorten visible label
        var short_label = hub.shortenLabel(title);
        // escape special characeters
        short_label = hub.htmlspecialchars(short_label);
        title = hub.htmlspecialchars(title);
        // return full label markup
        return "<img src='"+hub.urls.getImage('spinner.gif')+"' id='"+tab_id+"_busy' style='display: none;' /> <span title='"+title+"' id='"+tab_id+"_title'> " + short_label + "</span>";
    },

    /**
     * Factory method for creating new tabs for the TabContainer.
     *
     * @param title: Title of the new tab
     * @param closeable: true if the tab can be closed, false if permanent
     * @param workflow: true to create a workflow tab, false if just a regular
     *   content pane tab
     */
//    _createTab : function(title, closeable, workflow) {
    _createTab: function(title, props) {
        // build unique ID
        var tab_id = 'hub_tab_' + this.next_key++;
        // populate the parameters for the tab object
        var params = {
        	id: tab_id,
            plain_title: title,
            title: this._createTabLabel(title, tab_id),
            closable: props.closeable, // yes, dojo spells it differently
            cacheContent: false,
            preload: false,
            executeScripts: true
        };
        
        // create a div for the tab object
        var div = dojo.doc.createElement('div');
        // build the widget
        var tab = new hub.widgets.tabs.ContentPane(params, div);
        // add the widget to the tab container
        this.addChild(tab);
        // track tab by its title
        this.tab_id_to_obj.add(title.toLowerCase(), tab);
        return tab;
    },

    /**
     * Decides whether to reuse a tab or create a new one. Currently, the
     * algorithm considers only the page title as the unique ID.
     *
     * @todo: what else can we use as an ID? the problem is that titles are not
     *   unique, but we want to keep the unique ID as something consistent 
     *   across all links in the UI and easy to construct. *NOT easy to solve*
     *   one possibility is to combine the props and title to form a unique
     *   identifier, but this forces widgets that want to reuse the current tab
     *   to *always* specify props.current=true since the unique ID will differ
     *   when trying to reuse a tab to show a different page from the current
     *
     * @param title: Title
     * @param props: Properties to apply to the tab or to use when decide how
     *   to get/create a tab
     */
    _getTab: function(title, props) {
    	var tab = null;
        if(props.current && this.selectedChildWidget) {
            // reuse the current tab, if one exists
            tab = this.selectedChildWidget;
        }
        else{
        	tab = this.tab_id_to_obj.item(title.toLowerCase());
        }
        
        if(tab) {
	        this.selectChild(tab);
	        if(tab.isProtected && !props.safe) {
	            if(!tab._confirmOverwrite()) {
	                // abort
	                return null;
	            }
	        }
	        
	        if (tab.workflow && !tab.workflow.isReedit()) {
	           // do not reuse this tab as a delete event may close it.
	           tab.cleanTab();
	           tab.isProtected = false;
	           tab.forceClose = true;
	           this.closeChild(tab);
	           tab = null;
	        }   
	        else {
			   tab.cleanTab();
			}
		}
		
		if (!tab) {
            // can only decide closeable or not on creation (I think)
            // limitation of dojo
            props.closeable = !props.permanent;
            tab = this._createTab(title, props);
	        this.selectChild(tab);
        }
        // @todo: need to be smarter about re-using a tab when switching from
        // a wizard to a single screen page, or a single screen page to a wizard
        // when this happens, the easiest solution is to destroy the current
        // tab and create another one with the same title; the old code 
        // attempted to reuse the same tab with much pain and suffering as a 
        // result
        return tab;
    },

    /**
     * Finds the tab containing the given DOM node. Returns null if not
     * within a known tab of this tab container.
     *
     * @param node: Any DOM node
     */
    _getTabForNode: function(node) {
//    console.log('tabs._getTabForNode');
//	        console.log(node);
//	        console.dir({tab: node});
    	if(node != null){
	        if(hub.widgets.tabs.isTab(node)){
//	    		console.log('tab title: ' + node.plain_title);
	        	return node;
	        }
	        if(dojo.isString(node)) {
	            node = dojo.byId(node);
	        }
	        var child = null;
	        var gchild = null;
	        while(node != null) {
	            gchild = child;
	            child = node;
	            node = node.parentNode;
	            var w = node?dijit.byNode(node):null;
	            if(w == this) {
	                // tab is stored two levels below the container, inside
	                // the tab pane wrapper
	                return dijit.byNode(gchild);
	            }
	        }
    	}
        return null;
    },

    /**
     * Shows a tab rendering a plug-in editor. The editor may either start on
     * a page where the user selects the plug-in for a new entry, or on the 
     * first page of the editor for an existing entry. The tab created is a
     * wizard so that the user can flip back and forth through pages.
     * 
     * @param title: Title to put on the tab
     * @param args: Options for rendering the page.
     *   Can also include "current":true specifying that the currently 
     *   shown tab should be reused to render this page template and "safe":true
     *   to force a render in the chosen tab, disregarding any protection on
     *   the tab.
     */
    _showPluginPage: function(title, plugin_id, object_id, args) {
        // @todo: implement; add property "workflow: true" to args so that a 
        // hub.widgets.tabs.edit.Workflow gets constructed instead of just a 
        // content pane
        // get/create tab
        

        if(!args) {
            // always provide an object
            args = {};
        }
        args.workflow = true;
        
        var tab = this._getTab(title, args);
        if(tab == null){ 
        	return;
        }
        
		tab.setStatic(false);
        
        var div = dojo.doc.createElement('div');

		tab.domNode.appendChild(div);
        var workflow = new hub.widgets.tabs.Workflow({pluginId:plugin_id, entryId:object_id, type:args.type, fullscreen:args.fullscreen, editSource: args.editSource, editAsNew: args.editAsNew}, div);
        workflow.setTab(tab);
        tab.workflow = workflow;
        tab.page_cls = null;

		workflow.startup();
		
        this.resize();
       

    },
    
    /**
     * Shows a tab rendering a hub.pages._Page template. Can either reuse an
     * existing tab or create a new tab depending on passed arguments and 
     * whether an appropriate tab already exists.
     * 
     * @param title: Title to put on the tab
     * @param template: Name of the page/template rooted at hub.pages to render
     * @param args: Options for rendering the page.
     *   Can also include "current":true specifying that the currently 
     *   shown tab should be reused to render this page template and "safe":true
     *   to force a render in the chosen tab, disregarding any protection on
     *   the tab.
     */
    _showTemplatePage: function(title, template, args) {
        // parse out page name versus module name
        var arr = template.split('.');
        var page = arr[arr.length-1];
        var module =  arr.slice(0, -1);
        module = module.join('.');
        // make sure module is imported
        if (module) {
            dojo.require(module);
            arr=arr.slice(2, arr.length-1);
        }
        // loop through modules to page class
        var page_cls = hub.pages;
        dojo.forEach(arr, function(part) {
            page_cls = page_cls[part];
            // if bad path to page, quit
            if(!page_cls) {
                throw new Error('TabContainer: no template named '+template);
            }
        });

        // ask page *class* for properties (static method call)
        var props = page_cls.prototype.getProperties();
        // mix given args on top of class-specified args
        dojo.mixin(props, args);

        // get/create tab
        var tab = this._getTab(title, props);
        if(tab == null){ 
        	return;
        }
        
		tab.setStatic(false);

        // instruct the tab to render the page when appropriate
        tab.setPage(page_cls, props);
    },

    /**
     * Shows a tab rendering HTML returned by fetching HTML at a URL.
     * 
     * @param title: Title to put on the tab
     * @param url: URL to fetch
     * @param args: Options for rendering the page.
     *   Can also include "current":true specifying that the currently 
     *   shown tab should be reused to render this page template and "safe":true
     *   to force a render in the chosen tab, disregarding any protection on
     *   the tab.
     */
    _showUrlPage: function(title, url, args) {
        // get/create tab; no page properties
        var tab = this._getTab(title, {});
        if(tab == null){ 
        	return;
        }
        

        if(!args) {
            // always provide an object
            args = {};
        }
		
		tab.setStatic(true);
		
        // instruct the tab to fetch the url, rooted at the hub.config.base_url
        if (url.indexOf("http") == 0) {
        	tab.setHref(url, args);
        } else {
        	tab.setHref(hub.config.base_url + '/' + url, args);
        }
    }
});

/**
 * Custom content pane. Supports rendering of page templates and content 
 * fetched from URLs. Also supports a flag indicating whether the tab should 
 * be protected from overwriting by showing a confirmation dialog or not.
 * Uses dojox.layout.ContentPane instead of the dijit one to support inline JS
 * returned by plug-ins.
 */ 
//dojo.declare('hub.widgets.tabs.PaneContent', dojox.layout.ContentPane, {
dojo.declare('hub.widgets.tabs.ContentPane', dojox.layout.ContentPane, {
    // stores a reference to a class for a page template
    page_cls: null,
    // stores arguments for rendering a page template
    page_args: null,
    // original, plain-text title
    plain_title: '',
    
    loadingMessage: 'Loading...',
    
    forceClose: false,
    
    isProtected: false,
    
    selected: false,

    workflow: null,
    
    entryId: null,
    
    isStatic: false,

    /**
     * Overridden parent class method. Uses the Mashup Hub loading message 
     * instead of the stock dojo one.
     */
    postCreate: function() {
        this.inherited(arguments);
        // override with our own busy message
        this.loadingMessage = hub.busyMessage();
    },
    
    /**
     * Overridden parent class method. Loads the content of a URL. 
     * Clears any page rendering information.
     *
     * @param url: URL to fetch
     * @param args: Object with flags determining if replacement of 
     * rendered content is safe or not (overrides protection) and if the new 
     * rendering should be protected
     */
    setHref: function(url, args) {
        if(args.entryId){
        	this.entryId = args.entryId;
        }
        if ("protected" in args) {
        	this.isProtected = args['protected'];
        }
        else {
        	this.isProtected = args.isProtected;
        }
        this.page_cls = null;
        this.page_args = null;
        this.inherited('setHref', arguments);
    },

    /**
     * Loads a template page. Clears any URL loading information.
     *
     * @param page_cls: hub.pages._Page subclass
     * @param page_args: Object to pass to the page when rendering. Also 
     *   specifies if replacement of rendered content is safe or not (overrides
     *   protection) and if the new rendering should be protected or not
     */
    setPage: function(page_cls, args) {

        this.refreshOnShow = true;
        this.page_cls = page_cls;
        this.page_cls.tab = this;
        this.page_args = args;
        this.page_args.tab = this;
        this.href = '';
        if ("protected" in args) {
        	this.isProtected = args['protected'];
        }
        else {
        	this.isProtected = args.isProtected;
        }

        // do onShow processing immediately if already selected
        if(this.selected) {
            this.onShow();
        }
    },

    /**
     * Shows or hides a spinner on this tab.
     *
     * @param busy: Show (true) or hide (false) busy indicator
     */
    setBusy: function(busy) {
        var spinner = dojo.byId(this.id+"_busy");
        if(!spinner) return;
        spinner.style.display = (busy) ? "inline" : "none";
    },
    
    setTitle: function(title){
    	this.plain_title = title;
        var titleContainer = dojo.byId(this.id+"_title");
        if(!titleContainer) return;
        var short_label = hub.shortenLabel(title);
        // escape special characeters
        short_label = hub.htmlspecialchars(short_label);
        title = hub.htmlspecialchars(title);
        titleContainer.innerHTML = short_label;
        titleContainer.setAttribute('title', title);
    },

    setStatic: function(val) {
       this.isStatic = val;
    },
    
    /**
     * Sets whether the user has to confirm overwrites to pages or URLs shown
     * in this tab or not.
     *
     * @param val: True to protect, false to unprotect
     */
    setProtected: function(val) {
        this.isProtected = val;
    },
    
    getEntryId: function(){
    	if(this.entryId){
	    	if(dojo.isString(this.entryId)){
    			return parseInt(this.entryId);
    		}
    		return this.entryId; 
    	}
    	return 0;
    },
    
    cleanTab: function(){
        console.log('tabs.cleanTab');
		// cancel any outstanding url request (in case we're reusing a tab)
		this.cancel();

		if(this.workflow){
			if(this.workflow.saved){
				this.workflow.dispose();
			}else{
				this.workflow.cancel({});
			}
		}

		// destroy all existing widgets
		this.destroyDescendants();

		// clear all inner content (the safe for IE way)
		var node = this.containerNode || this.domNode;
		while(node.firstChild) {
			dojo._destroyElement(node.firstChild);
		}
    },

    /**
     * Forces a refresh of the current page. Re-renders a template or fetches
     * fresh data from a URL.
     *
     * @todo: tons of optimization work can be done here to re-use templates,
     * template server data tha never goes stale, etc.
     */
    refresh: function() {
        console.log('tabs.refresh');
        // always reset refresh on show
        this.refreshOnShow = false;
		if(!this.isStatic){
	        if(this.page_cls) {
	            // do template rendering
	            // set tab busy
	            this.setBusy(true);
	            
	            this.cleanTab();
	
	/*
	            // cancel any outstanding url request (in case we're reusing a tab)
	            this.cancel();
	            // destroy all existing widgets
			    this.destroyDescendants();
		            // clear all inner content (the safe for IE way)
			    var node = this.containerNode || this.domNode;
			    while(node.firstChild) {
					dojo._destroyElement(node.firstChild);
			    }
	*/
	            // create divs for dojox.dtl.AttachPoint and hub.pages._Page
	            var attach_div = document.createElement('DIV');
	            var page_div = document.createElement('DIV');
	            attach_div.appendChild(page_div);
	            this.domNode.appendChild(attach_div);
	
	            // create attach point widget
	            var ap = new dojox.dtl.AttachPoint(null, attach_div);
	            // create page widget
	            var p = new this.page_cls(this.page_args, page_div);
	            // listen for render complete
	            return p.isRendered().addCallback(dojo.hitch(this, this._onShowComplete));
	        } else {
	        	console.log('has not page_cls');
	            // do normal content pane loading from server
	            return this.inherited('refresh', arguments);
	        }
        }
    },

    /**
     * Starts rendering of a page template when setPage has been called and 
     * refreshOnShow is set.
     */
    onShow: function() {
        if(this.page_cls && this.refreshOnShow) {
            this.refresh();
        }
        // don't do anything for fetching URLs when setHref has been called
        // the parent class handles that through a different set of functions
    },

    /**
     * Confirms closing if tab is protected.
     *
     * @return: true to continue with closing, false to cancel
     */
    onClose: function() {
//      	console.log('tabs.onClose');
        if(this.isProtected) {
            if(this._confirmOverwrite()){
		        this.cleanTab();
        		return true;    	
            }else{
            	return false;
            }
        }
//        console.dir({workflow: this.workflow});
        if(!this.forceClose
        && this.workflow
        && dojo.isObject(this.workflow) 
        && this.workflow.pluginId != null 
        && this.workflow.pluginId > 0
        && (this.workflow.saveScreen && this.workflow.currentScreen <= this.workflow.saveScreen+1 || !this.workflow.saveScreen)) {
//	      	console.log('plugin defined '+this.workflow.pluginId);
        	if(this.workflow.reedit || this.workflow.saved){
//		      	console.log('plugin saved');
	        	if( this._confirmOverwrite()){
			        this.cleanTab();
	        		return true;    	
	            }else{
	            	return false;
	        	}
        	}else{
//		      	console.log('plugin not saved');
	        	if(this._confirmDiscard()){
			        this.cleanTab();
	        		return true;    	
	            }else{
	            	return false;
	        	}
        	}
        }

        // if not protected, always allow close
        return true;
    },

    /**
     * Completes rendering of a page by parsing for Dojo content and setting
     * the tab as no longer busy.
     */
    _onShowComplete: function() {
        // parse tab for dojo content
        dojo.parser.parse(this.domNode);
        // set tab unbusy
        this.setBusy(false);        
    },
    
    /**
     * If the tab is a protected tab (one that we should be careful about
     * refreshing or closing) ask if the user is sure he or she wants to
     * close the tab.
     *
     * @param tab: The tab that is changing or closing
     * @param args: Arguments object, possibly with "safe" property.
     * @return: true to continue with action, false to cancel
     */
    _confirmOverwrite: function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');        
        return confirm(msgs.PROMPT_CLOSE);
    },

    _confirmDiscard: function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');        
        return confirm(msgs.PROMPT_DISCARD);
    }
});

dojo.declare('hub.widgets.tabs.PaneWorkflow', hub.widgets.tabs.PaneContent, {
    // stores a reference to a class for a page template
    workflow: null,

	onClose: function(){
		if(this.workflow){
			if(this.workflow.saved){
				this.workflow.dispose();
			}else{
				this.workflow.cancel({});
			}
		}
		return this.inherited("onClose",arguments);
	}
	
});

hub.widgets.tabs.isTab = function(node){
//    alert(node);
//    console.dir(node);
	return node!=null && dojo.isObject(node) && node.declaredClass && (" "+node.declaredClass).indexOf("hub.widgets.tabs.ContentPane") > 0;
};

hub.widgets.tabs.getTabForNode = function(node) {
//    console.dir(node);
	if(node!=null){
        if(hub.widgets.tabs.isTab(node)){
        	return node;
        }
        if(dojo.isString(node)) {
            node = dojo.byId(node);
        }
        var child = null;
        var gchild = null;
        while(node != null) {
            gchild = child;
            child = node;
            node = node.parentNode;
            var w = dijit.byNode(node);
			if( w && w.page_cls ) {
                // the content pane
                return w;
            }
        }
    }
    return null;
}


}

if(!dojo._hasResource['hub.submit.admin']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.submit.admin'] = true;
dojo.provide('hub.submit.admin');






/**
 * Submits settings for tracking configuration.
 */
dojo.declare('hub.submit.admin.TrackingSettingsForm', hub.submit.Form, {
/*
    getURL : function() {
        return hub.urls.postAdminTrackingSettings();
    },
*/
    validate : function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
	    if (!this._validateRequired(trackingHost,
	            msgs.ERROR_TRACKING_HOST, this.status)) {
		    return false;
		}
		if (!this._validateRequired(trackingVender,
	            msgs.ERROR_TRACKING_VENDOR, this.status)) {
		    return false;
		}
		if (!this._validateRequired(trackingVpk,
	            msgs.ERROR_TRACKING_VENDOR_KEY, this.status)) {
		    return false;
		}
        }
        return true;
    }
});

/**
 * Submits settings for tracking configuration.
 */
dojo.declare('hub.submit.admin.GlobalStatusForm', hub.submit.Form, {
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('GlobalStatusForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits settings for tracking configuration.
 */
dojo.declare('hub.submit.admin.AdminTrackingPermissionsForm', hub.submit.Form, {
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('AdminTrackingPermissionsForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to delete mutiple objects.
 */
dojo.declare('hub.submit.admin.ObjectsDeleteForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('ObjectsForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	//var tab = hub.widgets.tabs.getTabForNode(this.form);
            dojo.publish('hub.delete-entry', [data.objects]);
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to reinitialize the database.
 */
dojo.declare('hub.submit.admin.InitForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('InitForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) { 
            // now notify about reset of the entire system
            dojo.publish('hub.reset');
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submit form for the category form in the admin settings.
 */
dojo.declare('hub.submit.admin.CategoryForm', hub.submit.Form, {
    validate : function() {
        return true;
    },
    
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new TOM.Exception();
        } else if (data.code == TOM.editor.Form.MESSAGE_CONTENT) {
            tab.refresh();
            // notify other components
            dojo.publish("hub.update-categories");
            TOM.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            TOM.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to reinitialize the database.
 */
dojo.declare('hub.submit.admin.ProxyForm', hub.submit.Form, {
    validate: function() {
        // try to close all pages first
        var d = hub.closeAllPages();
        var _self = this;
        // listen to deferred result to continue the submit
        d.addCallback(function(success) {
            if(success) {
                // resend this form without validating again
                hub.submit.ajaxSubmit(_self, false, true);
            } else {
                // @todo: seems like we should cleanup here, form or deferred?
            }
        });
        // never succeed right away, reinvoke submit later if all pages closed
        return false;
    },
          
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('ProxyForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) { 
            // now notify about reset of the entire system
            // dojo.publish('hub.reset');
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to set admins permissions.
 */
dojo.declare('hub.submit.admin.PermissionsForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('PermissionsForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to delete mutiple objects.
 */
dojo.declare('hub.submit.admin.BannedForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('BannedForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to delete mutiple objects.
 */
dojo.declare('hub.submit.admin.CategoriesForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('CategoriesForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            dojo.publish("hub.update-categories");
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to send a email.
 */
dojo.declare('hub.submit.admin.ReportForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('CategoriesForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits a request to change email list.
 */
dojo.declare('hub.submit.admin.EmailListForm', hub.submit.Form, {
    _onLoad : function(response, tab) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('CategoriesForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
        	var tab = hub.widgets.tabs.getTabForNode(this.form);
            tab.refresh();
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits tdi configuration.
 */
dojo.declare('hub.submit.admin.TDIForm', hub.submit.Form, {
    _onLoad : function(response) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('TDIForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits connection info for database migration. 
 */
dojo.declare('hub.submit.admin.MigrateForm', hub.submit.Form, {
    _onLoad: function(response) {
        var data = dojo.fromJson(response);
        if (!data) {
            throw new Error('MigrateForm: empty response');
        } else if (data.code == this.constants.MESSAGE_CONTENT) {
            hub.notify(data.result, this.status, 'TOM_editor_msg');
        } else {
            hub.notify(data.result, this.status);
        }
        return data.code;
    }
});

/**
 * Submits tracking date to clear from.
 */
dojo.declare('hub.submit.admin.ClearTrackedDataForm', hub.submit.Form, {

    validate : function() {
        var msgs = dojo.i18n.getLocalization('hub', 'messages');
        with (this.form) {
	    if (!this._validateRequired(trackingDate,
	            msgs.ERROR_TRACKING_CLEAR_DATE, this.status)) {
		    return false;
		}
		with(trackingDate) {
			//do some regex validation
			if(!value.match("^([0-9]){2}(-){1}([0-9]){2}(-){1}([0-9]){4}$")) {
			    hub.notify(msgs.ERROR_TRACKING_CLEAR_FORMAT, this.status);
			    return false;
			}
		}
        }
        return true;
    }
});

}

if(!dojo._hasResource["mm.pages.SelectMashupsServer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.pages.SelectMashupsServer"] = true;
dojo.provide("mm.pages.SelectMashupsServer");




dojo.declare("mm.pages.SelectMashupsServer", hub.pages._Page, {
	// template to replace node with
	templateString:"<div>\r\n\t<!-- Select Mashups Server Form  -->\r\n\t<form class=\"TOM_panel TOM_admin_panel\" id=\"select_server_form_{{ object_id }}\" action=\"javascript:void(0);\">\r\n\t\t<div>\r\n\t\t{% if isFeed %}\r\n\t\t\t<label>\t<img src='{{ icon_feed }}' /> </label>\r\n\t\t{% endif %}\r\n\t\t{% if isMashup %}\r\n\t\t\t<label>\t<img src='{{ icon_mashup }}' /> </label>\r\n\t\t{% endif %}\r\n\t\t{% if isWidget %}\r\n\t\t\t<label>\t<img src='{{ icon_widget }}' /> </label>\r\n\t\t{% endif %}\r\n\t\t{% if isPage %}\r\n\t\t\t<label>\t<img src='{{ icon_page }}' /> </label>\r\n\t\t{% endif %}\r\n\t\t\t<div class='rightCol'><h1>{{ labels.MASHUPS_TITLE_TEXT }}</h1></div>\r\n\t\t</div>\r\n\t\t<br/>\r\n\t\t<label for=\"server\"><span class='required'>*</span> {{ labels.MASHUPS_SERVER }}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<input name=\"server\" type=\"text\" value='{{ server }}' size=\"60\"></input>\r\n\t\t</div>\r\n\t\t<br/>\r\n\t\t<label for=\"username\">{{ labels.MASHUPS_USERNAME }}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<input name=\"username\" type=\"text\" value='{{ username }}'></input>\r\n\t\t</div>\r\n\t\t\r\n\t\t<label for=\"password\">{{ labels.MASHUPS_PASSWORD }}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<input name=\"password\" type='password' value='{{ password }}'></input>\r\n\t\t</div>\r\n\t\t\r\n\t\t<input name=\"object_id\" type=\"hidden\" value=\"{{ object_id }}\"/>\r\n\t\t<input name=\"object_title\" type=\"hidden\" value=\"{{ object_title }}\"/>\r\n\t\t<input name=\"status_id\" type=\"hidden\" value=\"select_server_message_{{ object_id }}\"/>\r\n\t\t\r\n\t\t<!-- Status Text Div -->\r\n\t\t<div id=\"select_server_message_{{ object_id }}\" class='rightCol' style=\"color:red;\">{{ error }}</div>\r\n\t</form>\r\n\t\r\n\t<div style=\"padding: 10px 0px 10px 180px; text-align: left;\">\r\n\t\t<button class=\"BlueButton BlueButtonDisabled\" id=\"previous_button_{{ object_id }}\" disabled=\"true\"></button>\r\n\t\t<button class=\"BlueButton\" id=\"cancel_button_{{ object_id }}\"></button>\r\n\t\t<button class=\"BlueButton\" id=\"select_server_button_{{ object_id }}\"></button>\r\n\t</div>\r\n</div>\r\n",
	
	//attach points
	form: null,
	button: null,
	message: null,
	
	object_id: "",
	object_title: "",
	type: "", //one of the value "feed", "iwidget", "mashuppage"
	
	server: "",
	username: "",
	password: "",
	error: "",
	
	
	/**
	 * Builds an object with properties matching fields in the template. Merges
	 * fetched server data with local data.
	 *
	 * @param server_data: JSON decoded object fetched from server URL
	 */
	getContext: function(server_data){
		var obj = {
			object_id: this.object_id,
			object_title: this.object_title,
			// mm parameters
			server: this.server ? this.server : mm.getDefaultMMServerUrl(),
			username: this.username ? this.username : "",
			password: this.password ? this.password : "",
			error: this.error,
			//page
			isFeed: this.type == "feed",
			isWidget: this.type == "iwidget",
			isPage: this.type == "mashuppage",
			isMashup: this.type == "mashup",
			// messages
			labels: dojo.i18n.getLocalization('hub', 'labels'),
			messages: dojo.i18n.getLocalization('hub', 'messages'),
			//icons
			icon_feed: hub.urls.getImage('feedLarge.gif'),
			icon_widget: hub.urls.getImage('widgetLarge.gif'),
			icon_page: hub.urls.getImage('pageLarge.gif'),
			icon_mashup: hub.urls.getImage('mashupLarge.gif')
		};
		return obj;
	},
	
	onRendered: function(){
		hub.pages._Page.prototype.onRendered.call(this);
		window.setTimeout(dojo.hitch(this, function(){
			this.form = dojo.byId("select_server_form_" + this.object_id);
			this.message = dojo.byId("select_server_message_" + this.object_id);
			this.button = dojo.byId("select_server_button_" + this.object_id);
			
			var labels = dojo.i18n.getLocalization('hub', 'labels');
			this.button.innerHTML = labels.MASHUPS_NEXT;
			dojo.byId("previous_button_" + this.object_id).innerHTML = labels.MASHUPS_PREVIOUS;
			var cancelButton = dojo.byId("cancel_button_" + this.object_id);
			cancelButton.innerHTML = labels.MASHUPS_CANCEL;
			
			dojo.connect(this.button, "onclick", this, "onSubmit");
			dojo.connect(cancelButton, "onclick", this, "onCancel");
		}), 0);
	},
	
	/**
	 * click handler: continue to addtomashup page
	 */
	onSubmit: function(){
		var form = dojo.formToObject(this.form);
		var messages = dojo.i18n.getLocalization('hub', 'messages');

		//validate inputs
		if (form.server == "") {
			this.message.innerHTML = messages.ERROR_MASHUPS_SERVER;
			return false;
		}
		//show add to mashups page
		var templateName;
		if(this.type == "feed" || this.type == "mashup"){
			templateName = "AddFeedToMashups";
		}else if(this.type == "iwidget"){
			templateName = "AddIWidgetToMashups";
		}else if(this.type == "mashuppage"){
			templateName = "AddMashupPageToMashups";
		}
		hub.showPage(form.object_title, templateName, form);
	},
	
	/**
	 * click handler: cancel the "add to mashups" action
	 */
	onCancel: function() {
		hub.showPage(this.object_title, "Details", {"object_id": this.object_id});
	}
});

}

if(!dojo._hasResource["mm.pages.AddFeedToMashups"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.pages.AddFeedToMashups"] = true;
dojo.provide("mm.pages.AddFeedToMashups");




/**
 * AddFeedToMashups page.
 */
dojo.declare("mm.pages.AddFeedToMashups", hub.pages._Page, {
	// template to replace node with
	templateString:"<div>\r\n\t{% if object %}\r\n\t<!-- Add Feed To Mashups Form  -->\r\n\t<form class=\"TOM_panel TOM_admin_panel\" id=\"add_feed_to_mashups_form_{{ object.id }}\" action=\"javascript:void(0);\">\r\n\t\t<!--the title bar-->\r\n\t\t<div>\r\n\t\t\t<label>\r\n\t\t\t\t{% if object.isMashup %}\r\n\t\t\t\t<img src='{{ icon_mashup }}' />\r\n\t\t\t\t{% else %}\r\n\t\t\t\t<img src='{{ icon_feed }}' />\r\n\t\t\t\t{% endif %}\r\n\t\t\t</label>\r\n\t\t\t<div class='rightCol'><h1>{{ labels.ADDTOMASHUPS_FEED_TITLE_TEXT }}</h1></div>\r\n\t\t</div>\r\n\t\t<br/>\r\n\t\t<!--feed title-->\r\n\t\t<label for=\"title\"><span class='required'>*</span> {{ labels.ADDTOMASHUPS_TITLE }}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<input name=\"title\" type=\"text\" value=\"{{ object.title }}\" maxlength=\"200\"></input>\r\n\t\t</div>\r\n\t\t\r\n\t\t<!--feed description-->\r\n\t\t<label for=\"description\">{{ labels.ADDTOMASHUPS_DESCRIPTION }}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<textarea rows=\"4\" name=\"description\" maxlength=\"1000\">{{ object.description }}</textarea>\r\n\t\t</div>\r\n\t\t\r\n\t\t<!--feed url-->\r\n\t\t<label for=\"feedurl\">{{ labels.ADDTOMASHUPS_FEED_URL}}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<input type=\"text\" readonly name=\"feedurl\" value=\"{{ object.externalURL }}\" size=\"60\"></input>\r\n\t\t</div>\r\n\t\t\r\n\t\t<!--feed parameters-->\r\n\t\t<span>\r\n\t\t{% if object.hasParams %}\r\n\t\t<label>{{ labels.ADDTOMASHUPS_PARAMETERS }}:</label>\r\n\t\t<div class=\"rightCol\">\r\n\t\t\t{% for param in object.parameters %}\r\n\t\t\t<p>\r\n\t\t\t\t<label for=\"param_{{ param.parametername }}\">{% if param.optional %}{%else%}<span class='required'>*</span>{% endif %}{{ param.parametername }}</label>\r\n\t\t\t\t<input name=\"param_{{ param.parametername }}\" type=\"text\" value=\"{{ param.defaultvalue }}\" style=\"width:5em\"/>\r\n\t\t\t\t<input name=\"option_param_{{ param.parametername }}\" type=\"hidden\" value=\"{{ param.optional }}\"/>\r\n\t\t\t\t<span class='helpertext'>&nbsp;&nbsp;(Type:&nbsp;{{ param.parametertype }})</span>\r\n\t\t\t</p>\r\n\t\t\t{% endfor %}\r\n\t\t</div>\r\n\t\t{% endif %}\r\n\t\t</span>\r\n\t\t\r\n\t\t<input type=\"hidden\" name=\"object_id\" value=\"{{object.id}}\"></input>\r\n\t\t<!--Categories Selector -->\r\n\t\t<label for=\"category\" style=\"height: 7em;\">{{ labels.ADDTOMASHUPS_CATEGORY }}:</label>\r\n\t\t<div class=\"rightCol\">\r\n\t\t\t{% for cat in mm_categories %}\r\n\t\t\t\t{% if forloop.first %}\r\n\t\t\t\t<div style=\"margin-bottom: 5px;\">\r\n\t\t\t\t\t<input type=\"radio\" name='mm_category' value='{{ cat.name }}' title=\"{{cat.title}}\" dojoType='dijit.form.RadioButton' checked='checked'/>\r\n\t\t\t\t\t{{ cat.title }}\r\n\t\t\t\t</div>\r\n\t\t\t\t{% else %}\r\n\t\t\t\t<div style=\"margin-bottom: 5px;\">\r\n\t\t\t\t\t<input type=\"radio\" name='mm_category' value='{{ cat.name }}' title=\"{{cat.title}}\" dojoType='dijit.form.RadioButton'/>\r\n\t\t\t\t\t{{ cat.title }}\r\n\t\t\t\t</div>\r\n\t\t\t\t{% endif %}\r\n\t\t\t{% endfor %}\r\n\t\t</div>\r\n\t\t\r\n\t\t<!-- Feed Viewers Selector -->\r\n\t\t<label for=\"feedViewer\" style=\"height: 3em;\">{{ labels.ADDTOMASHUPS_FEEDVIEWER }}:</label>\r\n\t\t<div class=\"rightCol\">\r\n\t\t\t\t{% for viewer in feedViewers %}\r\n\t\t\t\t\t{% if viewer.checked %}\r\n\t\t\t\t<div style=\"margin-bottom: 5px;\">\r\n\t\t\t\t\t<input type=\"radio\" name=\"feedViewer\" value='{{ viewer.displayName }}' dojoType='dijit.form.RadioButton' checked='checked'/>\r\n\t\t\t\t\t{{ viewer.displayName }}\r\n\t\t\t\t</div>\r\n\t\t\t\t\t{% else %}\r\n\t\t\t\t<div style=\"margin-bottom: 5px;\">\r\n\t\t\t\t\t<input type=\"radio\" name=\"feedViewer\" value='{{ viewer.displayName }}' dojoType='dijit.form.RadioButton'/>\r\n\t\t\t\t\t{{ viewer.displayName }}\r\n\t\t\t\t</div>\r\n\t\t\t\t\t{% endif %}\r\n\t\t\t\t{% endfor %}\r\n\t\t</div>\r\n\t\t\r\n\t\t<!-- Icon Selector -->\r\n\t\t<label for=\"icon\" style=\"height: 11em;\">{{ labels.ADDTOMASHUPS_ICON }}:</label>\r\n\t\t\r\n\t\t<div class=\"rightCol\">\r\n\t\t\t{% for icon in icons %}\r\n\t\t\t\t{% if forloop.first %}\r\n\t\t\t<div style=\"margin-bottom: 5px;\">\r\n\t\t\t\t<input type='radio' name='icon' value=\"{{ icon.url }}\" dojoType='dijit.form.RadioButton' checked=\"checked\"></input>\r\n\t\t\t\t<span><img src=\"{{ icon.url }}\"></img></span>\r\n\t\t\t</div>\r\n\t\t\t\t{% else %}\r\n\t\t\t<div style=\"margin-bottom: 5px;\">\r\n\t\t\t\t<input type='radio' name='icon' value=\"{{ icon.url }}\" dojoType='dijit.form.RadioButton'></input>\r\n\t\t\t\t<span><img src=\"{{ icon.url }}\"></img></span>\r\n\t\t\t</div>\r\n\t\t\t\t{% endif %}\r\n\t\t\t{% endfor %}\r\n\t\t</div>\r\n\t\t\r\n\t\t\r\n\t\t<!--MUM Server URL-->\r\n\t\t<input type=\"hidden\" name=\"server\" value='{{ server }}' size=\"50\" readonly></input>\r\n\t\t\r\n\t\t<!-- Status bar -->\r\n\t\t<div class='rightCol' id=\"add_feed_to_mashups_message_{{ object.id }}\" style=\"color:red;\"></div>\r\n\t</form>\r\n\r\n\t<!-- workflow buttons -->\r\n\t<div style=\"padding: 50px 0px 10px 180px; text-align: left;\">\r\n\t\t{% if palette_url %}\r\n\t\t<button class=\"BlueButton BlueButtonDisabled\" id=\"add_feed_to_mashups_serverbutton_{{ object.id }}\" disabled=\"true\"></button>\r\n\t\t{% else %}\r\n\t\t<button class=\"BlueButton\" id=\"add_feed_to_mashups_serverbutton_{{ object.id }}\"></button>\r\n\t\t{% endif %}\r\n\t\t<button class=\"BlueButton\" id=\"add_to_mashups_cancel_{{ object.id }}\"></button>\r\n\t\t<button class=\"BlueButton\" id=\"add_feed_to_mashups_button_{{ object.id }}\" name=\"submit\"></button>\r\n\t</div>\r\n\t{% endif %}\r\n</div>\r\n",
	
	type: "feed", //constant
	
	//attach points
	form: null,
	button: null,
	serverbutton: null,
	message: null,
	
	// parameters form caller
	object_id: "",
	object_title: "",
	server: "",
	username: "",
	password: "",
	
	//parameters from MashupMaker
	categories: null,
	feedViewers: null,
	icons: null,
	
	constructor: function(){
	},
	
	/**
	 * Gets the RESTful URL to query for information required to complete the
	 * rendering of the page.
	 */
	getUrl: function(){
		return hub.urls.getDetailsPage(this.object_id);
	},
	
	/**
	 * Builds an object with properties matching fields in the template. Merges
	 * fetched server data with local data.
	 *
	 * @param server_data: JSON decoded object fetched from server URL
	 */
	getContext: function(server_data){
		var viewerTitle = " ";
		
		//find default feed viewer for feed type
		for (var i = 0; i < this.feedViewers.length; i++) {
			if (dojo.indexOf(this.feedViewers[i].types, server_data.object.source) != -1) {
				viewerTitle = this.feedViewers[i].displayName;
				this.feedViewers[i].checked = true;
				break;
			}
		}
		
		// If no default viewer, check the first feed viewer
		if (viewerTitle === " " && this.feedViewers.length > 0) {
			viewerTitle = this.feedViewers[0].displayName;
			this.feedViewers[0].checked = true;
		}
		
		if (server_data && server_data.object.parameters) {
			dojo.forEach(server_data.object.parameters, function(param) {
				param.optional = param.optional === "Y" ? true : false;
			});
		}
		
		var obj = {
			server: this.server,
			mm_categories: this.categories,
			feedViewers: this.feedViewers,
			icons: this.icons,
			categoryTitle: this.categories[0] ? this.categories[0].title : "", //select the first category
			viewerTitle: viewerTitle,
			palette_url: hub.config.palette_url,
			//messages
			links: dojo.i18n.getLocalization('hub', 'links'),
			labels: dojo.i18n.getLocalization('hub', 'labels'),
			messages: dojo.i18n.getLocalization('hub', 'messages'),
			//icons
			icon_feed: hub.urls.getImage('feedLarge.gif'),
			icon_mashup: hub.urls.getImage('mashupLarge.gif')
		};
		return dojo.mixin(obj, server_data);
	},
	
	// @Override
	// Initial attach points and connect events
	onRendered: function(){
		hub.pages._Page.prototype.onRendered.call(this);
		window.setTimeout(dojo.hitch(this, function(){
			this.form = dojo.byId("add_feed_to_mashups_form_" + this.object_id);
			this.message = dojo.byId("add_feed_to_mashups_message_" + this.object_id);
			this.button = dojo.byId("add_feed_to_mashups_button_" + this.object_id);
			this.serverbutton = dojo.byId("add_feed_to_mashups_serverbutton_" + this.object_id);
			
			var labels = dojo.i18n.getLocalization('hub', 'labels');
			this.button.innerHTML = labels.MASHUPS_FINISH;
			this.serverbutton.innerHTML = labels.MASHUPS_PREVIOUS;
			var cancelButton = dojo.byId("add_to_mashups_cancel_" + this.object_id);
			cancelButton.innerHTML = labels.MASHUPS_CANCEL;
			
			dojo.connect(this.button, "onclick", this, "onSubmit");
			dojo.connect(this.serverbutton, "onclick", this, "onChangeServer");
			dojo.connect(cancelButton, "onclick", this, "onCancel");
		}), 0);
	},
	
	onSubmit: function(){
		var form = dojo.formToObject(this.form);
		var messages = dojo.i18n.getLocalization('hub', 'messages');
		
		// Validating form
		if (!form.title) {
			this.message.innerHTML = messages.ERROR_MASHUPS_TITLE;
			this.button.disabled = false;
			return;
		}
		if (!form.feedurl) {
			this.message.innerHTML = messages.ERROR_MASHUPS_URL;
			this.button.disabled = false;
			return;
		}
		
		if (!form.mm_category) {
			this.message.innerHTML = messages.ERROR_MASHUPS_CATEGORY;
			this.button.disabled = false;
			return;
		}
		
		// Validate and generate url with parameters
		var feedParams = {};
		for(var key in form){
			if(key.indexOf("param_") == 0){
				var paramName = key.substring(6);
				var paramRequired = key["option_param_" + paramName];
				if(paramRequired && (form["param_" + paramName] == "")){
					this.message.innerHTML = messages.ERROR_MASHUPS_PARAMETER + paramName;
					return;
				}else{
					feedParams[paramName] = form["param_" + paramName];
				}
			}
		}
		
		//add to mashup maker
		var mashupMaker = new mm.MashupMaker(this.server, this.username, this.password);
		var catalogModel = new mm.CatalogModel(mashupMaker);
		
		var feedEntry = new mm.model.CatalogFeedEntry();
		feedEntry.feedUrl = form.feedurl;
		feedEntry.feedParams = feedParams;
		feedEntry.category = form.mm_category;
		feedEntry.title = form.title;
		feedEntry.description = form.description;
		feedEntry.icon = form.icon;
		for(var i=0; i<this.feedViewers.length;i++){
			if(this.feedViewers[i].displayName == form.feedViewer){
				feedEntry.feedViewer = this.feedViewers[i].definition;
				for(var key in this.feedViewers[i].metadata){
					feedEntry.metadata[key] = this.feedViewers[i].metadata[key];
				};
				feedEntry.feedUrlItemName = this.feedViewers[i].feedUrlItemName;
			}
		}
		var _this = this;
		catalogModel.addToCatalog(feedEntry, function(data, error){
			if (!error) {
				hub.showPage(_this.object_title, "FinishedAddToMashups", {
					object_id: _this.object_id,
					object_title: _this.object_title,
					type: _this.type
				});
			} else {
				_this.message.innerHTML = messages.STATUS_MASHUPS_FAIL + " " + error.status + "," + error.statusText;
				_this.button.disabled = false;
			}
		});
		this.button.disabled = true;		
	},
	
	/**
	 * click handler: cancel the "add to mashups" action
	 */
	onCancel: function() {
		hub.showPage(this.object_title, "Details", {"object_id": this.object_id});
	},
	
	onChangeServer: function(){
		hub.showPage(this.object_title, "SelectMashupsServer", {
			object_id: this.object_id,
			object_title: this.object_title,
			server: this.server,
			username: this.username,
			password: this.password,
			type: this.type
		});
	},
	
	/**
	 * @Override
	 * Callback function, get category, feed viewer, icon set from lotus mashups
	 * @param {Object} response
	 * @param {Object} args
	 */
	_onPageDataLoad: function(response, args){
		var mashupMaker = new mm.MashupMaker(this.server, this.username, this.password);
		var catalogMode = new mm.CatalogModel(mashupMaker);
		var feedViewerConfig = new mm.FeedViewerConfig(mashupMaker);
		var iconConfig = new mm.IconConfig(mashupMaker);
		
		var inError = false;
		
		function callback(type, data, error){
			if (!error) {
				this[type] = data;
				if (this.categories && this.feedViewers && this.icons) {
					hub.pages._Page.prototype._onPageDataLoad.apply(this, [response, args]);
				};
			} else if(!inError){
				inError = true;
				hub.showPage(this.object_title, "SelectMashupsServer", {
					object_id: this.object_id,
					object_title: this.object_title,
					server: this.server,
					username: this.username,
					password: this.password,
					type: this.type,
					error: dojo.i18n.getLocalization('hub', 'messages').ERROR_MASHUPS_GET_CATEGORIES
				});
			}
		};
		
		catalogMode.getCategories(dojo.hitch(this, callback, "categories"));
		feedViewerConfig.load(dojo.hitch(this, callback, "feedViewers"));
		iconConfig.load(dojo.hitch(this, callback, "icons"));
	}
});

}

if(!dojo._hasResource["mm.IWidgetPackageModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.IWidgetPackageModel"] = true;
dojo.provide("mm.IWidgetPackageModel");

dojo.declare("mm.IWidgetPackageModel", null, {
	packageUrl: "", // url to the package
	
	getWidgetList: function(/*function*/callback) {
		var url = this.packageUrl + "?action=list_widgets";
        url = url.substring(url.indexOf("://") + 3);
		url = url.substring(url.indexOf("/"));
		dojo.xhrGet({
			url: url,
			error: dojo.hitch(this, function(response, args){
				callback(response, {
					"status": args.xhr.status,
					"statusText": args.xhr.statusText
				})
			}),
			load: dojo.hitch(this, function(response, args){
				response = dojo.fromJson(response);
				if (dojo.isArray(response)) {
					callback(response);
				} else {
					callback(response,  {
						"status": args.xhr.status,
						"statusText": args.xhr.statusText
					});
				}
			})
		});
	}
});

}

if(!dojo._hasResource["mm.utils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.utils"] = true;
dojo.provide("mm.utils");

mm.utils.encodeModelID4Uri = function(uri){
    // encode uri part correctly
    var pos = uri.indexOf(":");
    if (pos!=-1) {
        var start = uri.slice(0,pos+1);
        var end = uri.slice(pos+1);
        uri = start+encodeURIComponent(end);
    }
    uri = encodeURIComponent(uri);
    return uri;
};

mm.utils.getDefaultMMServerUrl = function(){
	var path = (window.location.port == "8443") ? "/mashupmaker" : "/mum"; 
	return window.location.protocol + "//" + window.location.host + path;
};

// converse String - > XMLDOM
mm.utils.createDocument = function (/*String*/response){
	if(window.ActiveXObject){
		var xmlDom=new ActiveXObject("Microsoft.XMLDOM");
		xmlDom.loadXML(response);
		return xmlDom;
	}else return new DOMParser().parseFromString(response, "text/xml");
};
mm.utils.getTimeStamp = function(/*Date*/time){
	if(!time)time = new Date();
	var timezone = time.getTimezoneOffset() / 60;
	return [time.getFullYear(), "-", time.getMonth(), "-", time.getDay(), "T", time.getHours(), ":", time.getMinutes(), ":", time.getSeconds(), timezone, ":00"].join("");
};
// escape  Xml
mm.utils.escapeXml = function(/*String*/str) {
	if(typeof str == "undefined")return "";
	return str.toString().replace(/&/gm, "&amp;")
			.replace(/</gm, "&lt;")
			.replace(/>/gm, "&gt;")
			.replace(/"/gm, "&quot;")
			.replace(/'/gm, "&#39;")
			.replace(/\[/gm, "&#91;")
			.replace(/\]/gm, "&#93;");
};

// get locale message	
mm.utils.getLocaleElement = function(/*Element[]*/elements){
	var locale = dojo.locale.toLowerCase().replace("-", "_");
    var country = (locale.indexOf("_") > 0) ? locale.split("_")[0] : locale;
	var result = null;
    var en_result = null;//by default, return english
    var first_result = null;//first result
	for(var i=0;i<elements.length;i++){
		var lang = elements[i].getAttribute("xml:lang");
		if(lang == locale){
			result = elements[i];
			break;
		}else if(lang == country && !result){
            result = elements[i];
        }else if(lang == "en"){
			en_result = elements[i];
		}else if(i === 0){
			first_result = elements[i];
		}
	}
	return result || (en_result || first_result);
};

mm.utils.summary = function(/*string*/str, /*number*/len){
// summary:
//	Truncates 'str' after 'len' characters and appends periods as necessary so that it ends with "..."

	if(!len || str.length <= len){
		return str; // string
	}

	return str.substring(0, len).replace(/\.+$/, "") + "..."; // string
}

}

if(!dojo._hasResource["mm.pages.AddIWidgetToMashups"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.pages.AddIWidgetToMashups"] = true;
dojo.provide("mm.pages.AddIWidgetToMashups");






/**
 * AddIWidgetToMashups page.
 */
dojo.declare("mm.pages.AddIWidgetToMashups", hub.pages._Page, {
	// template to replace node with
	templateString:"<div>\r\n\t{% if object %}\r\n\t<!-- Add IWidget To Mashups Form  -->\r\n\t<form class=\"TOM_panel TOM_admin_panel\" id=\"add_iwidget_to_mashups_form_{{ object.id }}\" action=\"javascript:void(0);\">\r\n\t\t<!--the title bar-->\r\n\t\t<div>\r\n\t\t\t<label>\t<img src='{{ icon_widget }}' /></label>\r\n\t\t\t<div class='rightCol'><h1>{{ labels.ADDTOMASHUPS_WIDGET_TITLE_TEXT }}</h1></div>\r\n\t\t</div>\r\n\t\t<br/>\r\n\t\t<!--iwidget title-->\r\n\t\t<label for=\"title\"><span class='required'>*</span> {{ labels.ADDTOMASHUPS_TITLE }}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t{% if isPackage %}\r\n\t\t\t<div>{{ object.title }}</div>\r\n\t\t\t<input name=\"title\" type=\"text\" value=\"{{ object.title }}\" readonly=\"readonly\" style=\"display:none\" maxlength=\"200\"></input>\r\n\t\t\t{% else %}\r\n\t\t\t<input name=\"title\" type=\"text\" value=\"{{ object.title }}\" maxlength=\"200\"></input>\r\n\t\t\t{% endif %}\r\n\t\t</div>\r\n\t\t\r\n\t\t<!--iwidget description-->\r\n\t\t<label for=\"description\">{{ labels.ADDTOMASHUPS_DESCRIPTION }}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t{% if isPackage %}\r\n\t\t\t<div>{{ object.description }}</div>\r\n\t\t\t<textarea rows=\"4\" name=\"description\" readonly=\"readonly\" style=\"display:none\" maxlength=\"1000\">{{ object.description }}</textarea>\r\n\t\t\t{% else %}\r\n\t\t\t<textarea rows=\"4\" name=\"description\" maxlength=\"1000\">{{ object.description }}</textarea>\r\n\t\t\t{% endif %}\r\n\t\t</div>\r\n\t\t\r\n\t\t<!--iwidget list for iwidget war package-->\r\n\t\t{% if isPackage %}\r\n\t\t<label>{{ labels.ADDTOMASHUPS_WIDGET_PACKAGE }}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<table id=\"iwidget_package_table_{{ object.id }}\" border=\"1\">\r\n\t\t\t\t<thead>\r\n\t\t\t\t\t<tr>\r\n\t\t\t\t\t\t<th>Title</th>\r\n\t\t\t\t\t\t<th>Description</th>\r\n\t\t\t\t\t</tr>\r\n\t\t\t\t</thead>\r\n\t\t\t\t<tbody>\r\n\t\t\t\t</tbody>\r\n\t\t\t</table>\r\n\t\t</div>\r\n\t\t{% endif %}\r\n\t\t\r\n\t\t<input type=\"hidden\" name=\"object_id\" value=\"{{object.id}}\"></input>\r\n\t\t<!--Categories Selector -->\r\n\t\t<label for=\"category\" style=\"height: 7em;\">{{ labels.ADDTOMASHUPS_CATEGORY }}:</label>\r\n\t\t<div class=\"rightCol\">\r\n\t\t\t{% for cat in mm_categories %}\r\n\t\t\t\t{% if forloop.first %}\r\n\t\t\t\t<div style=\"margin-bottom: 5px;\">\r\n\t\t\t\t\t<input type=\"radio\" name='mm_category' value='{{ cat.name }}' title=\"{{cat.title}}\" dojoType='dijit.form.RadioButton' checked='checked' />\r\n\t\t\t\t\t{{ cat.title }}\r\n\t\t\t\t</div>\r\n\t\t\t\t{% else %}\r\n\t\t\t\t<div style=\"margin-bottom: 5px;\">\r\n\t\t\t\t\t<input type=\"radio\" name='mm_category' value='{{ cat.name }}' title=\"{{cat.title}}\" dojoType='dijit.form.RadioButton'/>\r\n\t\t\t\t\t{{ cat.title }}\r\n\t\t\t\t</div>\r\n\t\t\t\t{% endif %}\r\n\t\t\t{% endfor %}\r\n\t\t</div>\r\n\t\t\r\n\t\t<!-- Show Icon Selector for iWidget definition URL-->\r\n\t\t{% if isPackage %}\r\n\t\t{% else %}\r\n\t\t<label for=\"icon\">{{ labels.ADDTOMASHUPS_ICON }}:</label>\r\n\t\t<div class=\"rightCol\">\r\n\t\t\t{% for icon in icons %}\r\n\t\t\t\t{% if forloop.first %}\r\n\t\t\t<div style=\"margin-bottom: 5px;\">\r\n\t\t\t\t<input type='radio' name='icon' value=\"{{ icon.url }}\" dojoType='dijit.form.RadioButton' checked=\"checked\"></input>\r\n\t\t\t\t<span><img src=\"{{ icon.url }}\"></img></span>\r\n\t\t\t</div>\r\n\t\t\t\t{% else %}\r\n\t\t\t<div style=\"margin-bottom: 5px;\">\r\n\t\t\t\t<input type='radio' name='icon' value=\"{{ icon.url }}\" dojoType='dijit.form.RadioButton'></input>\r\n\t\t\t\t<span><img src=\"{{ icon.url }}\"></img></span>\r\n\t\t\t</div>\r\n\t\t\t\t{% endif %}\r\n\t\t\t{% endfor %}\r\n\t\t</div>\r\n\t\t{% endif %}\r\n\t\t\r\n\t\t<!--MUM Server URL-->\r\n\t\t<input type=\"hidden\" name=\"server\" value='{{ server }}' size=\"50\" readonly></input>\r\n\t\t\r\n\t\t<!-- Status bar -->\r\n\t\t<div class='rightCol' id=\"add_iwidget_to_mashups_message_{{ object.id }}\" style=\"color:red;\"></div>\r\n\t</form>\r\n\t<div style=\"padding: 50px 0px 10px 180px; text-align: left;\">\r\n\t\t{% if palette_url %}\r\n\t\t<button class=\"BlueButton BlueButtonDisabled\" id=\"add_iwidget_to_mashups_serverbutton_{{ object.id }}\" disabled=\"true\"></button>\r\n\t\t{% else %}\r\n\t\t<button class=\"BlueButton\" id=\"add_iwidget_to_mashups_serverbutton_{{ object.id }}\"></button>\r\n\t\t{% endif %}\r\n\t\t<button class=\"BlueButton\" id=\"add_to_mashups_cancel_{{ object.id }}\"></button>\r\n\t\t<button class=\"BlueButton\" id=\"add_iwidget_to_mashups_button_{{ object.id }}\" name=\"submit\"></button>\r\n\t</div>\r\n\t{% endif %}\r\n</div>\r\n",
	
	type: "iwidget", //constant
	
	isPackage: false,
	
	definition: "",
	
	//attach points
	form: null,
	button: null,
	serverbutton: null,
	message: null,
	
	// parameters form caller
	object_id: "",
	object_title: "",
	server: "",
	username: "",
	password: "",
	
	//parameters from MashupMaker
	categories: null,
	icons: null,
	//parameters from Hub
	details: null,
	
	constructor: function(){
	},
	
	/**
	 * Gets the RESTful URL to query for information required to complete the
	 * rendering of the page.
	 */
	getUrl: function(){
		return hub.urls.getDetailsPage(this.object_id);
	},
	
	/**
	 * Builds an object with properties matching fields in the template. Merges
	 * fetched server data with local data.
	 *
	 * @param server_data: JSON decoded object fetched from server URL
	 */
	getContext: function(server_data){
		this.details = server_data;
		if (!server_data.pluginData.definitionUrl) {
			this.isPackage = true;
		}else {
			this.definition = server_data.pluginData.definitionUrl;
		}
		var obj = {
			server: this.server,
			isPackage: this.isPackage,
			mm_categories: this.categories,
			icons: this.icons,
			categoryTitle: this.categories[0] ? this.categories[0].title : "", //select the first category
			palette_url: hub.config.palette_url,
			//messages
			links: dojo.i18n.getLocalization('hub', 'links'),
			labels: dojo.i18n.getLocalization('hub', 'labels'),
			messages: dojo.i18n.getLocalization('hub', 'messages'),
			//icons
			icon_widget: hub.urls.getImage('widgetLarge.gif')
		};
		return dojo.mixin(obj, server_data);
	},
	
	onRendered: function(){
		hub.pages._Page.prototype.onRendered.call(this);
		var messages = dojo.i18n.getLocalization('hub', 'messages');
		var _this = this;
		window.setTimeout(dojo.hitch(this, function(){
			if (this.isPackage) {
				var widgetTable = dojo.byId("iwidget_package_table_" + this.object_id);
				var packageModel = new mm.IWidgetPackageModel();
				packageModel.packageUrl = this.details.object.url;
				packageModel.getWidgetList(dojo.hitch(this, function(data, error){
					if (!error) {
						widgetTable.style.display = "none";
						var tBody = document.createElement("tbody");
						widgetTable.appendChild(tBody);
						for (var i = 0; i < data.length; i++) {
							var tr = tBody.insertRow(i);
							var tdTitle = tr.insertCell(0);
							tdTitle.innerHTML = mm.utils.summary(data[i].title, 20);
							var tdDesc = tr.insertCell(1);
							tdDesc.innerHTML = mm.utils.summary(data[i].description, 50);
						}
						widgetTable.style.display = "";
					} else {
						_this.message.innerHTML = messages.ERROR_MASHUPS_GET_WIDGETPACKAGELIST + " " + error.status + "," + error.statusText;
						_this.button.disabled = false;
					}
				}));
			}
			this.form = dojo.byId("add_iwidget_to_mashups_form_" + this.object_id);
			this.message = dojo.byId("add_iwidget_to_mashups_message_" + this.object_id);
			this.button = dojo.byId("add_iwidget_to_mashups_button_" + this.object_id);
			this.serverbutton = dojo.byId("add_iwidget_to_mashups_serverbutton_" + this.object_id);
			
			var labels = dojo.i18n.getLocalization('hub', 'labels');
			this.button.innerHTML = labels.MASHUPS_FINISH;
			this.serverbutton.innerHTML = labels.MASHUPS_PREVIOUS;
			var cancelButton = dojo.byId("add_to_mashups_cancel_" + this.object_id);
			cancelButton.innerHTML = labels.MASHUPS_CANCEL;
			
			dojo.connect(this.button, "onclick", this, "onSubmit");
			dojo.connect(this.serverbutton, "onclick", this, "onChangeServer");
			dojo.connect(cancelButton, "onclick", this, "onCancel");
		}), 0);
	},
	
	onSubmit: function(){
		var form = dojo.formToObject(this.form);
		var messages = dojo.i18n.getLocalization('hub', 'messages');
		
		// Validating form
		if (!form.title) {
			this.message.innerHTML = messages.ERROR_MASHUPS_TITLE;
			this.button.disabled = false;
			return;
		}
		
		if (!form.mm_category) {
			this.message.innerHTML = messages.ERROR_MASHUPS_CATEGORY;
			this.button.disabled = false;
			return;
		}
		
		//add to mashup maker
		var mashupMaker = new mm.MashupMaker(this.server, this.username, this.password);
		var catalogModel = new mm.CatalogModel(mashupMaker);
		
		var entry = new mm.model.CatalogEntry();
		var _this = this;
		if(this.isPackage){
			var deployer = new mm.IWidgetDeployer(mashupMaker);
			deployer.packageUrl = this.details.object.url;
			deployer.category = form.mm_category;
			deployer.title = form.title;
			deployer.description = form.description; 
			
			this.button.disabled = true;
			
			deployer.deploy(dojo.hitch(this, function(data, error){
				if (!error) {
					hub.showPage(_this.object_title, "FinishedAddToMashups", {
						object_id: _this.object_id,
						object_title: _this.object_title,
						type: _this.type
					});
				} else {
					this.message.innerHTML = messages.STATUS_MASHUPS_FAIL + " " + error.status + "," + error.statusText;
					this.button.disabled = false;
				}
			}));
			
		}else{
			//add to catalog model
			entry.category = form.mm_category;
			entry.title = form.title;
			entry.description = form.description;
			entry.icon = form.icon;
			entry.definition = this.definition;
			
			this.button.disabled = true;

			catalogModel.addToCatalog(entry, dojo.hitch(this, function(data, error){
				if (!error) {
					hub.showPage(_this.object_title, "FinishedAddToMashups", {
						object_id: _this.object_id,
						object_title: _this.object_title,
						type: _this.type
					});
				} else {
					this.message.innerHTML = messages.STATUS_MASHUPS_FAIL + " " + error.status + "," + error.statusText;
					this.button.disabled = false;
				}
			}));
		}	
	},
	
	/**
	 * click handler: cancel the "add to mashups" action
	 */
	onCancel: function() {
		hub.showPage(this.object_title, "Details", {"object_id": this.object_id});
	},
	
	onChangeServer: function(){
		hub.showPage(this.object_title, "SelectMashupsServer", {
			object_id: this.object_id,
			object_title: this.object_title,
			server: this.server,
			username: this.username,
			password: this.password,
			type: this.type
		});
	},
	
	/**
	 * @Override
	 * Callback function, get category, feed viewer, icon set from lotus mashups
	 * @param {Object} response
	 * @param {Object} args
	 */
	_onPageDataLoad: function(response, args){
		var mashupMaker = new mm.MashupMaker(this.server, this.username, this.password);
		var catalogMode = new mm.CatalogModel(mashupMaker);
		var iconConfig = new mm.IconConfig(mashupMaker);
		
		var inError = false;
		
		function callback(type, data, error){
			if (!error) {
				this[type] = data;
				if (this.categories && this.icons) {
					hub.pages._Page.prototype._onPageDataLoad.apply(this, [response, args]);
				};
			} else  if(!inError){
				inError = true;
				hub.showPage(this.object_title, "SelectMashupsServer", {
					object_id: this.object_id,
					object_title: this.object_title,
					server: this.server,
					username: this.username,
					password: this.password,
					type: this.type,
					error: dojo.i18n.getLocalization('hub', 'messages').ERROR_MASHUPS_GET_CATEGORIES
				});
			}
		};
		
		catalogMode.getCategories(dojo.hitch(this, callback, "categories"));		
		iconConfig.load(dojo.hitch(this, callback, "icons"));
	}
});

}

if(!dojo._hasResource["dijit.Tree"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Tree"] = true;
dojo.provide("dijit.Tree");








dojo.declare(
	"dijit._TreeNode",
	[dijit._Widget, dijit._Templated, dijit._Container, dijit._Contained],
{
	// summary
	//		Single node within a tree

	// item: dojo.data.Item
	//		the dojo.data entry this tree represents
	item: null,	

	isTreeNode: true,

	// label: String
	//		Text of this tree node
	label: "",
	
	isExpandable: null, // show expando node
	
	isExpanded: false,

	// state: String
	//		dynamic loading-related stuff.
	//		When an empty folder node appears, it is "UNCHECKED" first,
	//		then after dojo.data query it becomes "LOADING" and, finally "LOADED"	
	state: "UNCHECKED",
	
	templateString:"<div class=\"dijitTreeNode dijitTreeExpandLeaf dijitTreeChildrenNo\" waiRole=\"presentation\"\r\n\t><span dojoAttachPoint=\"expandoNode\" class=\"dijitTreeExpando\" waiRole=\"presentation\"\r\n\t></span\r\n\t><span dojoAttachPoint=\"expandoNodeText\" class=\"dijitExpandoText\" waiRole=\"presentation\"\r\n\t></span\r\n\t>\r\n\t<div dojoAttachPoint=\"contentNode\" class=\"dijitTreeContent\" waiRole=\"presentation\">\r\n\t\t<div dojoAttachPoint=\"iconNode\" class=\"dijitInline dijitTreeIcon\" waiRole=\"presentation\"></div>\r\n\t\t<span dojoAttachPoint=\"labelNode\" class=\"dijitTreeLabel\" wairole=\"treeitem\" tabindex=\"-1\"></span>\r\n\t</div>\r\n</div>\r\n",		

	postCreate: function(){
		// set label, escaping special characters
		this.setLabelNode(this.label);

		// set expand icon for leaf 	
		this._setExpando();

		// set icon and label class based on item
		this._updateItemClasses(this.item);

		if(this.isExpandable){
			dijit.setWaiState(this.labelNode, "expanded", this.isExpanded);
		}
	},

	markProcessing: function(){
		// summary: visually denote that tree is loading data, etc.
		this.state = "LOADING";
		this._setExpando(true);	
	},

	unmarkProcessing: function(){
		// summary: clear markup from markProcessing() call
		this._setExpando(false);	
	},

	_updateItemClasses: function(item){
		// summary: set appropriate CSS classes for item (used to allow for item updates to change respective CSS)
		this.iconNode.className = "dijitInline dijitTreeIcon " + this.tree.getIconClass(item);
		this.labelNode.className = "dijitTreeLabel " + this.tree.getLabelClass(item);
	},
	
	_updateLayout: function(){
		// summary: set appropriate CSS classes for this.domNode
		var parent = this.getParent();
		if(parent && parent.isTree && parent._hideRoot){
			/* if we are hiding the root node then make every first level child look like a root node */
			dojo.addClass(this.domNode, "dijitTreeIsRoot");
		}else{
			dojo.toggleClass(this.domNode, "dijitTreeIsLast", !this.getNextSibling());
		}
	},

	_setExpando: function(/*Boolean*/ processing){
		// summary: set the right image for the expando node

		// apply the appropriate class to the expando node
		var styles = ["dijitTreeExpandoLoading", "dijitTreeExpandoOpened",
			"dijitTreeExpandoClosed", "dijitTreeExpandoLeaf"];
		var idx = processing ? 0 : (this.isExpandable ?	(this.isExpanded ? 1 : 2) : 3);
		dojo.forEach(styles,
			function(s){
				dojo.removeClass(this.expandoNode, s);
			}, this
		);
		dojo.addClass(this.expandoNode, styles[idx]);

		// provide a non-image based indicator for images-off mode
		this.expandoNodeText.innerHTML =
			processing ? "*" :
				(this.isExpandable ?
					(this.isExpanded ? "-" : "+") : "*");
	},	

	expand: function(){
		// summary: show my children
		if(this.isExpanded){ return; }
		// cancel in progress collapse operation
		if(this._wipeOut.status() == "playing"){
			this._wipeOut.stop();
		}

		this.isExpanded = true;
		dijit.setWaiState(this.labelNode, "expanded", "true");
		dijit.setWaiRole(this.containerNode, "group");

		this._setExpando();

		this._wipeIn.play();
	},

	collapse: function(){					
		if(!this.isExpanded){ return; }

		// cancel in progress expand operation
		if(this._wipeIn.status() == "playing"){
			this._wipeIn.stop();
		}

		this.isExpanded = false;
		dijit.setWaiState(this.labelNode, "expanded", "false");
		this._setExpando();

		this._wipeOut.play();
	},

	setLabelNode: function(label){
		this.labelNode.innerHTML="";
		this.labelNode.appendChild(document.createTextNode(label));
	},

	_setChildren: function(/* Object[] */ childrenArray){
		// summary:
		//		Sets the children of this node.
		//		Sets this.isExpandable based on whether or not there are children
		// 		Takes array of objects like: {label: ...} (_TreeNode options basically)
		//		See parameters of _TreeNode for details.

		this.destroyDescendants();

		this.state = "LOADED";
		var nodeMap= {};
		if(childrenArray && childrenArray.length > 0){
			this.isExpandable = true;
			if(!this.containerNode){ // maybe this node was unfolderized and still has container
				this.containerNode = this.tree.containerNodeTemplate.cloneNode(true);
				this.domNode.appendChild(this.containerNode);
			}

			// Create _TreeNode widget for each specified tree node
			dojo.forEach(childrenArray, function(childParams){
				var child = new dijit._TreeNode(dojo.mixin({
					tree: this.tree,
					label: this.tree.getLabel(childParams.item)
				}, childParams));
				this.addChild(child);
				var identity = this.tree.store.getIdentity(childParams.item);
				nodeMap[identity] = child;
				if(this.tree.persist){
					if(this.tree._openedItemIds[identity]){
						this.tree._expandNode(child);
					}
				}
			}, this);

			// note that updateLayout() needs to be called on each child after
			// _all_ the children exist
			dojo.forEach(this.getChildren(), function(child, idx){
				child._updateLayout();
			});
		}else{
			this.isExpandable=false;
		}

		if(this._setExpando){
			// change expando to/form dot or + icon, as appropriate
			this._setExpando(false);
		}

		if(this.isTree && this._hideRoot){
			// put first child in tab index if one exists.
			var fc = this.getChildren()[0];
			var tabnode = fc ? fc.labelNode : this.domNode;
			tabnode.setAttribute("tabIndex", "0");
		}

		// create animations for showing/hiding the children (if children exist)
		if(this.containerNode && !this._wipeIn){
			this._wipeIn = dojo.fx.wipeIn({node: this.containerNode, duration: 150});
			this._wipeOut = dojo.fx.wipeOut({node: this.containerNode, duration: 150});
		}

		return nodeMap;
	},

	_addChildren: function(/* object[] */ childrenArray){
		// summary:
		//		adds the children to this node.
		// 		Takes array of objects like: {label: ...}  (_TreeNode options basically)

		//		See parameters of _TreeNode for details.
		var nodeMap = {};
		if(childrenArray && childrenArray.length > 0){
			dojo.forEach(childrenArray, function(childParams){
				var child = new dijit._TreeNode(
					dojo.mixin({
						tree: this.tree,
						label: this.tree.getLabel(childParams.item)
					}, childParams)
				);
				this.addChild(child);
				nodeMap[this.tree.store.getIdentity(childParams.item)] = child;
			}, this);

			dojo.forEach(this.getChildren(), function(child, idx){
				child._updateLayout();
			});
		}

		return nodeMap;
	},

	deleteNode: function(/* treeNode */ node){
		node.destroy();

		var children = this.getChildren();		
		if(children.length == 0){
			this.isExpandable = false;
			this.collapse();
		}

		dojo.forEach(children, function(child){
				child._updateLayout();
		});
	},

	makeExpandable: function(){
		//summary
		//		if this node wasn't already showing the expando node,
		//		turn it into one and call _setExpando()
		this.isExpandable = true;
		this._setExpando(false);
	}
});

dojo.declare(
	"dijit.Tree",
	dijit._TreeNode,
{
	// summary
	//	This widget displays hierarchical data from a store.  A query is specified
	//	to get the "top level children" from a data store, and then those items are
	//	queried for their children and so on (but lazily, as the user clicks the expand node).
	//
	//	Thus in the default mode of operation this widget is technically a forest, not a tree,
	//	in that there can be multiple "top level children".  However, if you specify label,
	//	then a special top level node (not corresponding to any item in the datastore) is
	//	created, to father all the top level children.

	// store: String||dojo.data.Store
	//	The store to get data to display in the tree
	store: null,

	// query: String
	//	query to get top level node(s) of tree (ex: {type:'continent'})
	query: null,

	// childrenAttr: String
	//		one ore more attributes that holds children of a tree node
	childrenAttr: ["children"],

	templateString:"<div class=\"dijitTreeContainer\" style=\"\" waiRole=\"tree\"\r\n\tdojoAttachEvent=\"onclick:_onClick,onkeypress:_onKeyPress\">\r\n\t<div class=\"dijitTreeNode  dijitTreeIsRoot dijitTreeExpandLeaf dijitTreeChildrenNo\" waiRole=\"presentation\"\r\n\t\tdojoAttachPoint=\"rowNode\"\r\n\t\t><span dojoAttachPoint=\"expandoNode\" class=\"dijitTreeExpando\" waiRole=\"presentation\"\r\n\t\t></span\r\n\t\t><span dojoAttachPoint=\"expandoNodeText\" class=\"dijitExpandoText\" waiRole=\"presentation\"\r\n\t\t></span\r\n\t\t>\r\n\t\t<div dojoAttachPoint=\"contentNode\" class=\"dijitTreeContent\" waiRole=\"presentation\">\r\n\t\t\t<div dojoAttachPoint=\"iconNode\" class=\"dijitInline dijitTreeIcon\" waiRole=\"presentation\"></div>\r\n\t\t\t<span dojoAttachPoint=\"labelNode\" class=\"dijitTreeLabel\" wairole=\"treeitem\" tabindex=\"0\"></span>\r\n\t\t</div>\r\n\t</div>\r\n</div>\r\n",		

	isExpandable: true,

	isTree: true,

	// persist: Boolean
	//	enables/disables use of cookies for state saving.
	persist: true,
	
	// dndController: String
	//	class name to use as as the dnd controller
	dndController: null,

	//parameters to pull off of the tree and pass on to the dndController as its params
	dndParams: ["onDndDrop","itemCreator","onDndCancel","checkAcceptance", "checkItemAcceptance"],

	//declare the above items so they can be pulled from the tree's markup
	onDndDrop:null,
	itemCreator:null,
	onDndCancel:null,
	checkAcceptance:null,	
	checkItemAcceptance:null,

	_publish: function(/*String*/ topicName, /*Object*/ message){
		// summary:
		//		Publish a message for this widget/topic
		dojo.publish(this.id, [dojo.mixin({tree: this, event: topicName}, message||{})]);
	},

	postMixInProperties: function(){
		this.tree = this;
		this.lastFocused = this.labelNode;

		this._itemNodeMap={};

		this._hideRoot = !this.label;

		if(!this.store.getFeatures()['dojo.data.api.Identity']){
			throw new Error("dijit.tree requires access to a store supporting the dojo.data Identity api");			
		}

		if(!this.cookieName){
			this.cookieName = this.id + "SaveStateCookie";
		}

		// if the store supports Notification, subscribe to the notification events
		if(this.store.getFeatures()['dojo.data.api.Notification']){
			this.connect(this.store, "onNew", "_onNewItem");
			this.connect(this.store, "onDelete", "_onDeleteItem");
			this.connect(this.store, "onSet", "_onSetItem");
		}
	},

	postCreate: function(){
		// load in which nodes should be opened automatically
		if(this.persist){
			var cookie = dojo.cookie(this.cookieName);
			this._openedItemIds = {};
			if(cookie){
				dojo.forEach(cookie.split(','), function(item){
					this._openedItemIds[item] = true;
				}, this);
			}
		}
		
		// make template for container node (we will clone this and insert it into
		// any nodes that have children)
		var div = document.createElement('div');
		div.style.display = 'none';
		div.className = "dijitTreeContainer";	
		dijit.setWaiRole(div, "presentation");
		this.containerNodeTemplate = div;

		if(this._hideRoot){
			this.rowNode.style.display="none";
		}

		this.inherited("postCreate", arguments);

		// load top level children
		this._expandNode(this);

		if(this.dndController){
			if(dojo.isString(this.dndController)){
				this.dndController= dojo.getObject(this.dndController);
			}	
			var params={};
			for (var i=0; i<this.dndParams.length;i++){
				if(this[this.dndParams[i]]){
					params[this.dndParams[i]]=this[this.dndParams[i]];
				}
			}
			this.dndController= new this.dndController(this, params);
		}

		if (dojo.isIE) {
			this.connect(this.domNode, "onactivate", "_onTreeFocus");
		} else if ((dojo.isFF > 2) || dojo.isSafari) {
			// On FF3beta3, onfocus appears to be broken currently.
			// When it is fixed, this case should be removed
			// Emulate onfocus event
			this.connect(this.domNode, "onclick", "_onTreeFocus");
			this.connect(this.domNode, "onkeyup", "_onTreeFocus");
		} else {
			this.connect(this.domNode, "onfocus", "_onTreeFocus");
		}

	},

	////////////// Data store related functions //////////////////////

	mayHaveChildren: function(/*dojo.data.Item*/ item){
		// summary
		//		User overridable function to tell if an item has or may have children.
		//		Controls whether or not +/- expando icon is shown.
		//		(For efficiency reasons we may not want to check if an element has
		//		children until user clicks the expando node)

		return dojo.some(this.childrenAttr, function(attr){
			return this.store.hasAttribute(item, attr);
		}, this);
	},

	getItemChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ onComplete){
		// summary
		// 		User overridable function that return array of child items of given parent item,
		//		or if parentItem==null then return top items in tree
		var store = this.store;
		if(parentItem == null){
			// get top level nodes
			store.fetch({ query: this.query, onComplete: onComplete});
		}else{
			// get children of specified node
			var childItems = [];
			for (var i=0; i<this.childrenAttr.length; i++){	
				childItems= childItems.concat(store.getValues(parentItem, this.childrenAttr[i]));
			}
			// count how many items need to be loaded
			var _waitCount = 0;
			dojo.forEach(childItems, function(item){ if(!store.isItemLoaded(item)){ _waitCount++; } });

			if(_waitCount == 0){
				// all items are already loaded.  proceed..
				onComplete(childItems);
			}else{
				// still waiting for some or all of the items to load
				function onItem(item){
					if(--_waitCount == 0){
						// all nodes have been loaded, send them to the tree
						onComplete(childItems);
					}
				}
				dojo.forEach(childItems, function(item){
					if(!store.isItemLoaded(item)){
						store.loadItem({item: item, onItem: onItem});
					}
				});
			}
		}
	},

	getItemParentIdentity: function(/*dojo.data.Item*/ item, /*Object*/ parentInfo){
		// summary
		//		User overridable function, to return id of parent (or null if top level).
		//		It's called with args from dojo.store.onNew
		return this.store.getIdentity(parentInfo.item);		// String
	},

	getLabel: function(/*dojo.data.Item*/ item){
		// summary: user overridable function to get the label for a tree node (given the item)
		return this.store.getLabel(item);	// String
	},

	getIconClass: function(/*dojo.data.Item*/ item){
		// summary: user overridable function to return CSS class name to display icon
	},

	getLabelClass: function(/*dojo.data.Item*/ item){
		// summary: user overridable function to return CSS class name to display label
	},

	_onLoadAllItems: function(/*_TreeNode*/ node, /*dojo.data.Item[]*/ items){
		// sumary: callback when all the children of a given node have been loaded
		var childParams=dojo.map(items, function(item){
			return {
				item: item,
				isExpandable: this.mayHaveChildren(item)
			};
		}, this);

		dojo.mixin(this._itemNodeMap,node._setChildren(childParams));

		this._expandNode(node);
	},

	/////////// Keyboard and Mouse handlers ////////////////////

	_onKeyPress: function(/*Event*/ e){
		// summary: translates keypress events into commands for the controller
		if(e.altKey){ return; }
		var treeNode = dijit.getEnclosingWidget(e.target);
		if(!treeNode){ return; }

		// Note: On IE e.keyCode is not 0 for printables so check e.charCode.
		// In dojo charCode is universally 0 for non-printables.
		if(e.charCode){  // handle printables (letter navigation)
			// Check for key navigation.
			var navKey = e.charCode;
			if(!e.altKey && !e.ctrlKey && !e.shiftKey && !e.metaKey){
				navKey = (String.fromCharCode(navKey)).toLowerCase();
				this._onLetterKeyNav( { node: treeNode, key: navKey } );
				dojo.stopEvent(e);
			}
		}else{  // handle non-printables (arrow keys)
			var map = this._keyHandlerMap;
			if(!map){
				// setup table mapping keys to events
				map = {};
				map[dojo.keys.ENTER]="_onEnterKey";
				map[dojo.keys.LEFT_ARROW]="_onLeftArrow";
				map[dojo.keys.RIGHT_ARROW]="_onRightArrow";
				map[dojo.keys.UP_ARROW]="_onUpArrow";
				map[dojo.keys.DOWN_ARROW]="_onDownArrow";
				map[dojo.keys.HOME]="_onHomeKey";
				map[dojo.keys.END]="_onEndKey";
				this._keyHandlerMap = map;
			}
			if(this._keyHandlerMap[e.keyCode]){
				this[this._keyHandlerMap[e.keyCode]]( { node: treeNode, item: treeNode.item } );	
				dojo.stopEvent(e);
			}
		}
	},

	_onEnterKey: function(/*Object*/ message){
		this._publish("execute", { item: message.item, node: message.node} );
		this.onClick(message.item, message.node);
	},

	_onDownArrow: function(/*Object*/ message){
		// summary: down arrow pressed; get next visible node, set focus there
		var returnNode = this._navToNextNode(message.node);
		if(returnNode && returnNode.isTreeNode){
			returnNode.tree.focusNode(returnNode);
			return returnNode;
		}	
	},

	_onUpArrow: function(/*Object*/ message){
		// summary: up arrow pressed; move to previous visible node

		var nodeWidget = message.node;
		var returnWidget = nodeWidget;

		// if younger siblings		
		var previousSibling = nodeWidget.getPreviousSibling();
		if(previousSibling){
			nodeWidget = previousSibling;
			// if the previous nodeWidget is expanded, dive in deep
			while(nodeWidget.isExpandable && nodeWidget.isExpanded && nodeWidget.hasChildren()){
				returnWidget = nodeWidget;
				// move to the last child
				var children = nodeWidget.getChildren();
				nodeWidget = children[children.length-1];
			}
		}else{
			// if this is the first child, return the parent
			// unless the parent is the root of a tree with a hidden root
			var parent = nodeWidget.getParent();
			if(!(this._hideRoot && parent === this)){
				nodeWidget = parent;
			}
		}

		if(nodeWidget && nodeWidget.isTreeNode){
			returnWidget = nodeWidget;
		}

		if(returnWidget && returnWidget.isTreeNode){
			returnWidget.tree.focusNode(returnWidget);
			return returnWidget;
		}
	},

	_onRightArrow: function(/*Object*/ message){
		// summary: right arrow pressed; go to child node
		var nodeWidget = message.node;
		var returnWidget = nodeWidget;

		// if not expanded, expand, else move to 1st child
		if(nodeWidget.isExpandable && !nodeWidget.isExpanded){
			this._expandNode(nodeWidget);
		}else if(nodeWidget.hasChildren()){
			nodeWidget = nodeWidget.getChildren()[0];
		}

		if(nodeWidget && nodeWidget.isTreeNode){
			returnWidget = nodeWidget;
		}

		if(returnWidget && returnWidget.isTreeNode){
			returnWidget.tree.focusNode(returnWidget);
			return returnWidget;
		}
	},

	_onLeftArrow: function(/*Object*/ message){
		// summary: left arrow pressed; go to parent

		var node = message.node;
		var returnWidget = node;

		// if not collapsed, collapse, else move to parent
		if(node.isExpandable && node.isExpanded){
			this._collapseNode(node);
		}else{
			node = node.getParent();
		}
		if(node && node.isTreeNode){
			returnWidget = node;
		}

		if(returnWidget && returnWidget.isTreeNode){
			returnWidget.tree.focusNode(returnWidget);
			return returnWidget;
		}
	},

	_onHomeKey: function(){
		// summary: home pressed; get first visible node, set focus there
		var returnNode = this._navToRootOrFirstNode();
		if(returnNode){
			returnNode.tree.focusNode(returnNode);
			return returnNode;
		}
	},

	_onEndKey: function(/*Object*/ message){
		// summary: end pressed; go to last visible node

		var returnWidget = message.node.tree;

		var lastChild = returnWidget;
		while(lastChild.isExpanded){
			var c = lastChild.getChildren();
			lastChild = c[c.length - 1];
			if(lastChild.isTreeNode){
				returnWidget = lastChild;
			}
		}

		if(returnWidget && returnWidget.isTreeNode){
			returnWidget.tree.focusNode(returnWidget);
			return returnWidget;
		}
	},

	_onLetterKeyNav: function(message){
		// summary: letter key pressed; search for node starting with first char = key
		var node = startNode = message.node;
		var key = message.key;
		do{
			node = this._navToNextNode(node);
			//check for last node, jump to first node if necessary
			if(!node){
				node = this._navToRootOrFirstNode();
			}
		}while(node !== startNode && (node.label.charAt(0).toLowerCase() != key));
		if(node && node.isTreeNode){
			// no need to set focus if back where we started
			if(node !== startNode){
				node.tree.focusNode(node);
			}
			return node;
		}
	},

	_onClick: function(/*Event*/ e){
		// summary: translates click events into commands for the controller to process
		var domElement = e.target;

		// find node
		var nodeWidget = dijit.getEnclosingWidget(domElement);	
		if(!nodeWidget || !nodeWidget.isTreeNode){
			return;
		}

		if(domElement == nodeWidget.expandoNode ||
			 domElement == nodeWidget.expandoNodeText){
			// expando node was clicked
			if(nodeWidget.isExpandable){
				this._onExpandoClick({node:nodeWidget});
			}
		}else{
			this._publish("execute", { item: nodeWidget.item, node: nodeWidget} );
			this.onClick(nodeWidget.item, nodeWidget);
			this.focusNode(nodeWidget);
		}
		dojo.stopEvent(e);
	},

	_onExpandoClick: function(/*Object*/ message){
		// summary: user clicked the +/- icon; expand or collapse my children.
		var node = message.node;
		if(node.isExpanded){
			this._collapseNode(node);
		}else{
			this._expandNode(node);
		}
	},

	onClick: function(/* dojo.data */ item, /*TreeNode*/ node){
		// summary: user overridable function for executing a tree item
	},

	_navToNextNode: function(node){
		// summary: get next visible node
		var returnNode;
		// if this is an expanded node, get the first child
		if(node.isExpandable && node.isExpanded && node.hasChildren()){
			returnNode = node.getChildren()[0];			
		}else{
			// find a parent node with a sibling
			while(node && node.isTreeNode){
				returnNode = node.getNextSibling();
				if(returnNode){
					break;
				}
				node = node.getParent();
			}	
		}
		return returnNode;
	},

	_navToRootOrFirstNode: function(){
		// summary: get first visible node
		if(!this._hideRoot){
			return this;
		}else{
			var returnNode = this.getChildren()[0];
			if(returnNode && returnNode.isTreeNode){
				return returnNode;
			}
		}
	},

	_collapseNode: function(/*_TreeNode*/ node){
		// summary: called when the user has requested to collapse the node

		if(node.isExpandable){
			if(node.state == "LOADING"){
				// ignore clicks while we are in the process of loading data
				return;
			}
			if(this.lastFocused){
				// are we collapsing a descendant with focus?
				if(dojo.isDescendant(this.lastFocused.domNode, node.domNode)){
					this.focusNode(node);
				}else{
					// clicking the expando node might have erased focus from
					// the current item; restore it
					this.focusNode(this.lastFocused);
				}
			}
			node.collapse();
			if(this.persist && node.item){
				delete this._openedItemIds[this.store.getIdentity(node.item)];
				this._saveState();
			}
		}
	},

	_expandNode: function(/*_TreeNode*/ node){
		// summary: called when the user has requested to expand the node

		// clicking the expando node might have erased focus from the current item; restore it
		var t = node.tree;
		if(t.lastFocused){ t.focusNode(t.lastFocused); }

		if(!node.isExpandable){
			return;
		}

		var store = this.store;
		var getValue = this.store.getValue;

		switch(node.state){
			case "LOADING":
				// ignore clicks while we are in the process of loading data
				return;

			case "UNCHECKED":
				// need to load all the children, and then expand
				node.markProcessing();
				var _this = this;
				var onComplete = function(childItems){
					node.unmarkProcessing();
					_this._onLoadAllItems(node, childItems);
				};
				this.getItemChildren(node.item, onComplete);
				break;

			default:
				// data is already loaded; just proceed
				if(node.expand){	// top level Tree doesn't have expand() method
					node.expand();
					if(this.persist && node.item){
						this._openedItemIds[this.store.getIdentity(node.item)] = true;
						this._saveState();
					}
				}
				break;
		}
	},

	////////////////// Miscellaneous functions ////////////////

	blurNode: function(){
		// summary
		//	Removes focus from the currently focused node (which must be visible).
		//	Usually not called directly (just call focusNode() on another node instead)
		var node = this.lastFocused;
		if(!node){ return; }
		var labelNode = node.labelNode;
		dojo.removeClass(labelNode, "dijitTreeLabelFocused");
		labelNode.setAttribute("tabIndex", "-1");
		this.lastFocused = null;
	},

	focusNode: function(/* _tree.Node */ node){
		// summary
		//	Focus on the specified node (which must be visible)

		// set focus so that the label will be voiced using screen readers
		node.labelNode.focus();
	},

	_onBlur: function(){
		// summary:
		// 		We've moved away from the whole tree.  The currently "focused" node
		//		(see focusNode above) should remain as the lastFocused node so we can
		//		tab back into the tree.  Just change CSS to get rid of the dotted border
		//		until that time
		if(this.lastFocused){
			var labelNode = this.lastFocused.labelNode;
			dojo.removeClass(labelNode, "dijitTreeLabelFocused");	
		}
	},

	_onTreeFocus: function(evt){
		var node = dijit.getEnclosingWidget(evt.target);
		if(node != this.lastFocused){
			this.blurNode();
		}
		var labelNode = node.labelNode;
		// set tabIndex so that the tab key can find this node
		labelNode.setAttribute("tabIndex", "0");
		dojo.addClass(labelNode, "dijitTreeLabelFocused");
		this.lastFocused = node;
	},

	//////////////// Events from data store //////////////////////////


	_onNewItem: function(/*Object*/ item, parentInfo){
		//summary: callback when new item has been added to the store.

		var loadNewItem;	// should new item be displayed in tree?

		if(parentInfo){
			var parent = this._itemNodeMap[this.getItemParentIdentity(item, parentInfo)];
			
			// If new item's parent item not in tree view yet, can safely ignore.
			// Also, if a query of specified parent wouldn't return this item, then ignore.
			if(!parent ||
				dojo.indexOf(this.childrenAttr, parentInfo.attribute) == -1){
				return;
			}
		}

		var childParams = {
			item: item,
			isExpandable: this.mayHaveChildren(item)
		};
		if(parent){
			if(!parent.isExpandable){
				parent.makeExpandable();
			}
			if(parent.state=="LOADED" || parent.isExpanded){
				var childrenMap=parent._addChildren([childParams]);
			}
		}else{
			// top level node
			var childrenMap=this._addChildren([childParams]);		
		}

		if(childrenMap){
			dojo.mixin(this._itemNodeMap, childrenMap);
			//this._itemNodeMap[this.store.getIdentity(item)]=child;
		}
	},
	
	_onDeleteItem: function(/*Object*/ item){
		//summary: delete event from the store
		//since the object has just been deleted, we need to
		//use the name directly
		var identity = this.store.getIdentity(item);
		var node = this._itemNodeMap[identity];

		if(node){
			var parent = node.getParent();
			parent.deleteNode(node);
			this._itemNodeMap[identity]=null;
		}
	},

	_onSetItem: function(/*Object*/ item){
		//summary: set data event  on an item in the store
		var identity = this.store.getIdentity(item);
		node = this._itemNodeMap[identity];

		if(node){
			node.setLabelNode(this.getLabel(item));
			node._updateItemClasses(item);
		}
	},
	
	_saveState: function(){
		//summary: create and save a cookie with the currently expanded nodes identifiers
		if(!this.persist){
			return;
		}
		var ary = [];
		for(var id in this._openedItemIds){
			ary.push(id);
		}
		dojo.cookie(this.cookieName, ary.join(","));
	}
});

}

if(!dojo._hasResource["mm.HttpUrl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.HttpUrl"] = true;
dojo.provide("mm.HttpUrl");
/**
 * Url utilites
 */
dojo.declare("mm.HttpUrl", null, {
    constructor: function( /*String*/urlString){
        this.scheme = this._extractScheme(urlString);
        this.server = this._extractServer(urlString);
        this.port = this._extractPort(urlString);
        this.path = this._extractPath(urlString);
        this.query = this._extractQuery(urlString);
        this.anchor = this._extractAnchor(urlString);
    },
    
    addParameter: function( /*String*/name, /*String*/ value){
        this.query += ((this.query != null && this.query != "") ? "&" : "") + name + "=" + value;
    },
	
	getHttpUrl: function(){
		//using the covention to convert url from https to http
		var result =  new mm.HttpUrl(this.toString());
		if(result.scheme == "https:"){
			result.scheme = "http:";
			var port = parseInt(this.port) + 80 - 443;
			result.port = port + ""
		};
		return result;
	},
	
	getHttpsUrl: function(){
		//using the covention to convert url from http to https
		var result = new mm.HttpUrl(this.toString());
		if(result.scheme == "http:"){
			result.scheme = "https:";
			var port = parseInt(this.port) - 80 + 443;
			result.port = port + ""
		};
		return result;
	},
	
    toString: function(){
        var str = "";
        if (this.server != "") {
            str += this.scheme + "//" + this.server;
        }
        if (this.port != "") {
            str += ":" + this.port;
        }
        
        str += "/" + this.path;
        
        if (this.query != "") {
            str += "?" + this.query;
        }
        if (this.anchor != "") {
            str += "#" + this.anchor;
        }
        
        return str;
    },
    _extractScheme: function(/*String*/urlString){
        var indexOfScheme = urlString.indexOf("://");
        
        if (indexOfScheme == -1) 
            return window.location.protocol; // location.protocol returns the protocol like http: or https:
        return urlString.substring(0, indexOfScheme + 1);
    },
    _extractServer: function(/*String*/urlString){
        var indexOfScheme = urlString.indexOf(this.scheme);
        var retVal = "";
        if (indexOfScheme == 0) {
            var indexOfSlash = urlString.indexOf("/", indexOfScheme + this.scheme.length + 2);
            var serverAndPort = urlString.substring(indexOfScheme + this.scheme.length + 2, indexOfSlash);
            retVal = serverAndPort.split(":")[0];
        }
        else {
            retVal = window.location.hostname;
        }
        
        return retVal;
    },
    _extractPort: function(/*String*/urlString){
        var indexOfServer = urlString.indexOf(this.server);
        var retVal = "";
        if (indexOfServer >= 0) {
            var indexOfSlash = urlString.indexOf("/", indexOfServer);
            var serverAndPort = urlString.substring(indexOfServer, indexOfSlash);
            var serverAndPortParts = serverAndPort.split(":");
            if (serverAndPortParts.length > 1) {
                retVal = serverAndPortParts[1];
            }
        }
        
        if (retVal == "") {
            if (urlString.indexOf("/") == 0) 
                retVal = window.location.port;
            else 
                retVal = (this.scheme == "https:" ? "443" : "80");
        }
        
        return retVal;
    },
    _extractPath: function( /*String*/urlString){
        var indexOfScheme = urlString.indexOf(this.scheme);
        var startIndex = 0;
        
        if (indexOfScheme == 0) 
            startIndex = this.scheme.length + 2;
        
        var retVal = "";
        
        var indexOfSlash = urlString.indexOf("/", startIndex);
        var indexOfQuery = urlString.indexOf("?");
        var indexOfHash = urlString.lastIndexOf("#");
        if (indexOfQuery >= 0) {
            retVal = urlString.substring(indexOfSlash + 1, indexOfQuery);
        }
        else {
            if (indexOfHash >= 0) {
                retVal = urlString.substring(indexOfSlash + 1, indexOfHash);
            }
            else {
                retVal = urlString.substring(indexOfSlash + 1);
            }
        }
        
        return retVal;
    },
    _extractQuery: function( /*String*/urlString){
        var retVal = "";
        var urlParts = urlString.split("?");
        
        if (urlParts.length > 1) {
            retVal = urlParts[1].split("#")[0];
        }
        
        return retVal;
    },
    _extractAnchor: function( /*String*/urlString){
        var retVal = "";
        var urlParts = urlString.split("#");
        
        if (urlParts.length > 1) {
            retVal = urlParts[urlParts.length - 1];
        }
        
        return retVal;
    }
});

}

if(!dojo._hasResource["mm.xhr.CrossDomainXmlHttpRequest"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.xhr.CrossDomainXmlHttpRequest"] = true;
/*
 * summary:
 *   An Object that implement Cross Domain XmlHttpRequest 
 *   this object implements XmlHttpRequest API defined by W3C 
 *   please reference http://www.w3.org/TR/XMLHttpRequest/
 *   var xhr = new mm.xhr.CrossDomainXmlHttpRequest(serviceProxyUrl);
 * serviceProxyUrl:
 *   URL to Service Proxy of target domain, e.g. "http://target_domain/crossdomain/ServiceProxy.jsp"
 *   There must be a service proxy support at target domain, so that we can invoke serivce on target domain
 * example:
 *	var xhr = new mm.xhr.CrossDomainXmlHttpRequest("http://target_domain/crossdomain/ServiceProxy.jsp");
 *	xhr.onreadystatechange = function(){
 *		if(xhr.readyState == 4){
 *			alert(xhr.responseText);
 *		}
 *	}
 *	xhr.open("POST", "http://target_domain/crossdomain/sample/service.jsp");
 *	xhr.send("zhang ke");
 */
dojo.provide("mm.xhr.CrossDomainXmlHttpRequest");

mm.xhr.CALLBACK_PROXY_URL = [window.location.protocol,"//",window.location.host, window.location.pathname.substring(0, window.location.pathname.lastIndexOf("/")), "/", dojo.moduleUrl("mm.xhr", "CallbackProxy.jsp").uri].join("");
mm.xhr.REQUEST_PAGE_URL = [window.location.protocol,"//",window.location.host, window.location.pathname.substring(0, window.location.pathname.lastIndexOf("/")), "/", dojo.moduleUrl("mm.xhr", "Request.html").uri].join("");
mm.xhr.DEFAULT_TIMEOUT = 30000;

mm.xhr.CrossDomainXmlHttpRequest = function(/*String*/serviceProxyUrl){
	
	if(!serviceProxyUrl) throw new Error("serviceProxyUrl not set!");
	
	//construct the request object
	var request = {
		"callbackProxyUrl" : mm.xhr.CALLBACK_PROXY_URL,
		"serviceProxyUrl" : serviceProxyUrl,
		
		"method": "",
		"url": "",
		"async": true,
		"username": "",
		"password": "",
		
		"headers" : "",
		"body" : ""
	};
	
	//event handler
	this.onreadystatechange = function(){};
	
	//state constant
	this.UNSENT = 0;
  	this.OPENED = 1;
  	this.HEADERS_RECEIVED = 2;
  	this.LOADING = 3;
  	this.DONE = 4;
  	
	this.readyState = this.UNSENT;
	
	//request API
	this.open = function(/*in DOMString*/method, /*in DOMString*/url, /*in boolean?*/ async, /*in DOMString?*/username, /*in DOMString?*/ password){
		if((typeof method == "undefined") || (typeof url == "undefined")){
			throw new Error("method or url not set");
		}
		request.method = method;
		request.url = url;
		request.async = typeof async == "undefined" ? true : async;
		request.username = typeof username == "undefined" ? "" : username; 
		request.password = typeof password == "undefined" ? "" : password;
		//change to OPENED state
		this.readyState = this.OPENED;
		this.onreadystatechange();
	}
	
	this.setRequestHeader = function(/*in DOMString*/header, /*in DOMString*/value){
		//encode request header into string
		//please reference http://www.w3.org/TR/XMLHttpRequest/#getallresponseheaders
		request.headers = request.headers + header + ":" + value + "\r\n";
	}
	
  	this.send = function(/*in DOMString or Document?*/ data){
		//TODO: support Document
		request.body = data?data:"";
		//create an iframe to send request to ServiceProxy.jsp in target domain
		//1, the iframe load Request.html in this domain
		//  Request.html retrive request object from its frameElement._request,
		//  encode request object into a form, 
		//  submit the form to ServiceProxy.jsp in target domain
		//2, iframe load ServiceProxy.jsp in target domain
		//  extract request object,
		//  use XmlHttpRequest to invoke service
		//  encode response object into a form
		//  submit the form to CallbackProxy.jsp in this domain
		//3, iframe load CallbackProxy.jsp in this domain
		//  extract response object
		//  notify parent by invoke iframe._complete()
		var iframe = document.createElement("iframe");
		iframe.setAttribute("src", mm.xhr.REQUEST_PAGE_URL);
		iframe.setAttribute("style", "width:0px;height:0px;border:0px;");
		iframe._request = request;
		var _this = this;
		//call back function, will be invoked by CallBackProxy.jsp
		iframe._complete = function(){
			iframe._stopTimeout();
			//set response
			_this.responseText = iframe._response.responseText;
			_this.status = iframe._response.status;
			_this.statusText = iframe._response.statusText;
			responseHeaders = iframe._response.responseHeaders;
			//change to DONE state
			_this.readyState = _this.DONE;
			_this.onreadystatechange();
			//delete the iframe
			window.setTimeout(function(){document.documentElement.removeChild(iframe)},0);
		};
		
		//FIXME: temp solution for time out
		var timeoutId = null;
		iframe._startTimeout = function(){
			var timeout = mm.xhr.DEFAULT_TIMEOUT;
			timeoutId = window.setTimeout(function(){
				_this.responseText = "";
				_this.status = 408;
				_this.statusText = "Time out";
				_this.readyState = _this.DONE;
				_this.onreadystatechange();
				window.setTimeout(function(){document.documentElement.removeChild(iframe)},0);
			}, timeout);
		}
		iframe._stopTimeout = function(){
			window.clearTimeout(timeoutId);
		}
		
		//FIXME: temp solution for test service proxy
		/*var link = document.createElement("link");
		link.setAttribute("href", serviceProxyUrl);
		link.setAttribute("type", "text/css");
		link.setAttribute("rel", "stylesheet");
		console.debug(link.sheet);
		document.documentElement.appendChild(link);
		link.on
		var interval = window.setInterval(function(){
			alert(link.sheet);
			if(link.sheet)window.clearInterval(interval);
			//link.parentNode.removeChild(link);
		},1);*/
		
		
		
		document.documentElement.appendChild(iframe);
		//FIXME: how to get notified, if service proxy url is incoreect or unreachable
	};
	
	this.abort = function(){
		throw new Error("abourt() not supported!");
	}
	
	// response API
	var responseHeaders = "";
	
	this.responseText;
	this.responseXML = null;
	this.status;
	this.statusText;
	
	this.getAllResponseHeaders = function(){
		//return a string contains all response headers
		//please reference http://www.w3.org/TR/XMLHttpRequest/#getallresponseheaders
		return responseHeaders;
	};
	
	this.getResponseHeader = function(/*in DOMString*/header){
		//return value of specific header
		var headers = responseHeaders.split("\r\n");
		for(var i=0;i<headers.length;i++){
			if(headers[i].indexOf(header + ":") == 0){
				return headers[i].substring(header.length + 1);
			}
		}
	};
};

}

if(!dojo._hasResource["mm.MashupMaker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.MashupMaker"] = true;
dojo.provide("mm.MashupMaker");



/**
 * The Mashup Maker Server from Hub
 **/
dojo.declare("mm.MashupMaker", null, {
	//the url to Mashup Maker
	_serverUrl: "http://localhost:9080/mum",
	
	//the MashupMaker related URLs relative to MashupMaker context root
	contextRootServer: "/",
	contextRootBuilder: "/",
	contextRootEnabler: "/",
	
	contentHandler: "/mycontenthandler",
	serviceProxy: "/crossdomain/ServiceProxy.jsp",
	iconConfig: "/conf/icons.conf",
	feedViewerConfig: "/conf/feedViewers.conf",
	
	_username: "",
	
	_password: "",
	
	//if the Mashup Maker in same server as Hub
	_isLocal: false,
	
	constructor: function(/*string*/server, /*string?*/ username, /*string?*/ password){
        //remove last "/' in server
        if(server.lastIndexOf("/") == server.length -1){
            server = server.substring(0, server.length -1);
        }
		this._username = username ? username : "";
		this._password = password ? password : "";
		
		var originalUrl = new mm.HttpUrl(server);
		var mumUrl = originalUrl;
		var hubUrl = new mm.HttpUrl(window.location.href);
		
		//use same url scheme for Mashup Maker and Mashup Hub
		if (hubUrl.scheme == "http:") {
			mumUrl = mumUrl.getHttpUrl();
		}
		if (hubUrl.scheme == "https:") {
			mumUrl = mumUrl.getHttpsUrl();
		}
		
		//determine if Mashup Maker in same server as Hub
		if ((mumUrl.scheme == hubUrl.scheme) && (mumUrl.server == hubUrl.server) && (mumUrl.port == hubUrl.port)) {
			//Local Mashup Maker Server
			this._isLocal = true;
		} else {
			//Remote Mashup Maker Server
			this._isLocal = false;
		};
		
		//trim url, and extract contextRootXXX from url
		this._processMashupMakerUrl(originalUrl);
		this._serverUrl = mumUrl.toString();
		this.serverUrl = originalUrl.toString();
	},
	
	GET: function(/*string*/url, /*object*/ headers, /*function*/ callback){
		this._send({
			"method": "GET",
			"url": url,
			"headers": headers,
			"data": null,
			"callback": callback
		});
	},
	
	POST: function(/*string*/url, /*object*/ headers, /*string*/ data, /*function*/ callback){
		this._send({
			"method": "POST",
			"url": url,
			"headers": headers,
			"data": data,
			"callback": callback
		});
	},
	
	_send: function(kwArgs){
		/*
		 * kwArgs:
		 * 	method: string, http method
		 *  url: string, url
		 *  headers: object, http headers
		 *  data: string, http request body
		 *  callback: function, the callback function
		 */
		var xhr;
		if (this._isLocal && !this._username) {
			xhr = this._getXhr();
			//TODO: convert username and password to LTPA token?
			xhr.open(kwArgs.method, this._serverUrl + kwArgs.url, true, this._username, this._password);
			
		}else{
			xhr = this._getNativeXhr();
			var url = new mm.HttpUrl(this.serverUrl + kwArgs.url);
			var urlStr = [
				hub.config.base_url, 
				"mm", 
				url.scheme, 
				encodeURIComponent((this._username ? (encodeURIComponent(this._username) + ":" + encodeURIComponent(this._password) + "@") : "") + url.server + ":" + url.port),
				url.path + (url.query ? "?" : "") + url.query
			].join("/");
			xhr.open(kwArgs.method, urlStr);
		}
		if (kwArgs.headers) {
			for (var key in kwArgs.headers) {
				xhr.setRequestHeader(key, kwArgs.headers[key]);
			};
		};
		xhr.onreadystatechange = function(){
			if (xhr.readyState == 4) {
				kwArgs.callback(xhr);
			}
		};
		xhr.send(kwArgs.data);
	},
	
	_getXhr: function(){
		if (this._isLocal) {
			return this._getNativeXhr();
		} else {
			return new mm.xhr.CrossDomainXmlHttpRequest(this._serverUrl + this.serviceProxy);
		}
	},
	
	_getNativeXhr: function(){
		if (window.XMLHttpRequest) {
			return new XMLHttpRequest();
		} else if (window.ActiveXObject) {//IE
			try {
				return new ActiveXObject("Msxml2.XMLHTTP");
			} catch (e) {
				try {
					return new ActiveXObject("Microsoft.XMLHTTP");
				} catch (e) {
				}
			};
					};
		throw new Error("No XmlHttpRequest support.");
	},
	
	_processMashupMakerUrl: function(/*mm.HttpUrl*/url){
		/*
		 * 1, detect Application Server type of Mashup Maker
		 *    Tomcat, WAS or LWI
		 * 2, trim URL path to content root, for example
		 *    http://localhost:8080/mashupmaker/enabler#
		 *    -->
		 *    http://locahost:8080/mashupmaker
		 * 3, change values of contextRootServer, contextRootBuilder
		 *    and contextRootEnabler based on type of Application Server
		 *    because in differnt build they have different URL
		 */
		url.query = "";//empty query (?xxx=xxx)
		url.anchor = "";//empty anchor (#xxx)
		var contextRoots = {
			"mashupmaker": "tomcat",
			//"LWI": "??" //TODO:LWI contextRoot
			"mum": "WAS"
		};
        
        var ctxRoot = url.path.substring(url.path.lastIndexOf("/"));
		
		var serverType = contextRoots[ctxRoot] ? contextRoots[ctxRoot] : "WAS";

		if (serverType == "WAS") {
			this.contextRootServer = "";
			this.contextRootBuilder = "";
			this.contextRootEnabler = "";
			
			this.contentHandler = "/mycontenthandler";
			//TODO: bug 581
			this.serviceProxy = "/crossdomain/ServiceProxy.jsp2";
			this.iconConfig = "/conf/icons.conf";
			this.feedViewerConfig = "/conf/feedViewers.conf";
			
		} else if (serverType == "tomcat") {
			this.contextRootServer = "/../mm.webapp";
			this.contextRootBuilder = "/../mm";
			this.contextRootEnabler = "/";
			
			this.contentHandler = "/../mm.webapp/mycontenthandler";
			this.serviceProxy = "/../mm/crossdomain/ServiceProxy.jsp";
			this.iconConfig = "/../mm/conf/icons.conf";
			this.feedViewerConfig = "/../mm/conf/feedViewers.conf";
		} else if (serverType == "LWI") {
			//TODO: path conf for LWI
		}
	}
});

}

if(!dojo._hasResource["mm.xpath.safari"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.xpath.safari"] = true;
dojo.provide( "mm.xpath" );

// generic implementation
mm.xpath.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
	namespaces = namespaces ? namespaces : {};
    if (navigator.userAgent.toLowerCase().indexOf("safari") != -1) {
        return mm.xpath.safari.evaluateXPath(xpathExpr,doc,namespaces);
    }
    else if (typeof ActiveXObject != "undefined") {
        return mm.xpath.ie.evaluateXPath(xpathExpr, doc, namespaces);
    }
    else {
        return mm.xpath.gecko.evaluateXPath(xpathExpr, doc, namespaces);
    }
}

// Safari implementation
dojo.provide("mm.xpath.safari");
mm.xpath.safari.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {

    var result;
    if (typeof XPathResult != "undefined") {
        //use build in xpath support for Safari 3.0
        var results = document.evaluate(xpathExpr,doc, function(prefix) { return namespaces[prefix] || null;}, XPathResult.ANY_TYPE, null );
        var thisResult;
        result = new Array();
        var len = 0;
        do {
            thisResult = results.iterateNext();
            if (thisResult) {
                result[len] = thisResult;
                len++;
            }
        } while ( thisResult );
    }
    else {
        //use javeline Safari2 xpath support
        if (doc.selectNodes) {
            result = doc.selectNodes(xpathExpr);
        }
    }
    return result;
}



// IE Implementation

dojo.provide( "mm.xpath.ie" );

mm.xpath.ie.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
        if (namespaces) {        
            var ns = "";
            for (var prop in namespaces) {
                ns += "xmlns:"+prop+"='"+namespaces[prop]+"' ";
            }
            if (doc.ownerDocument) {
                doc.ownerDocument.setProperty("SelectionNamespaces", ns); 
                doc.ownerDocument.setProperty("SelectionLanguage", "XPath"); 
            } else {
                doc.setProperty("SelectionNamespaces", ns);
                doc.setProperty("SelectionLanguage", "XPath"); 
            }
        }
        var result = doc.selectNodes(xpathExpr);

        var thisResult;
        var resultSet = new Array();
        var len = 0;
        for (var i=0; i<result.length; i++) {
            thisResult = result[i];
            if (thisResult) {
                resultSet[len] = thisResult;
                len++;
            }
        }
        return resultSet;
}


// GECKO Implementation

dojo.provide( "mm.xpath.gecko" );

mm.xpath.gecko.evaluateXPath = function(/*String*/xpathExpr, /*DOMDocument*/ doc, /*Object{prefix:ns,prefix2:ns2,...}*/namespaces) {
		var xmlDoc = doc.ownerDocument ? doc.ownerDocument : doc;
		var result;
        try {
            result = xmlDoc.evaluate( xpathExpr, doc, function(prefix) { return namespaces[prefix] || null;}, XPathResult.ANY_TYPE, null );
        } 
        catch (exc)
        {
            //Callers should catch this and can substitute their own error message
            throw new Error("Error with xpath expression" + exc);
        }
    
        var thisResult;
        var resultSet = new Array();
        var len = 0;
        do {
            thisResult = result.iterateNext();
            if (thisResult) {
                resultSet[len] = thisResult;
                len++;
            }
        } while ( thisResult );
        return resultSet;
}


}

if(!dojo._hasResource["mm.model.NavigationNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.model.NavigationNode"] = true;
dojo.provide("mm.model.NavigationNode");



dojo.declare("mm.model.NavigationNode", null, {
	XML_NS: {
		atom: "http://www.w3.org/2005/Atom",
		nm: "http://www.ibm.com/xmlns/prod/lotus/remix/v1.0/navigation-model",
		xhtml: "http://www.w3.org/1999/xhtml",
		thr: "http://purl.org/syndication/thread/1.0",
		app: "http://www.w3.org/2007/app",
		model: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements",
		base: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"
	},
	id: "",
	title: "",
	type: "stub",	// stub: not loaded; node: loaded; root: root(for query)
	parentId: "",
	
	constructor: function(/*Element*/entry, /*string*/rootId) {
		var idNode = mm.xpath.evaluateXPath("atom:id", entry, this.XML_NS)[0];
		this.id = idNode ? idNode.textContent || idNode.text : "";
		this.id = this.parseId(this.id);
		
		var titles = mm.xpath.evaluateXPath("atom:content/model:navigation-node/model:title/base:nls-string", entry, this.XML_NS);
		var title = titles ? mm.utils.getLocaleElement(titles) : null;
		if (title === null) {
			title = mm.xpath.evaluateXPath("atom:title", entry, this.XML_NS)[0];
		}
		this.title = title ? title.textContent || title.text : "";
		
		var countNode = mm.xpath.evaluateXPath("atom:link/@thr:count", entry, this.XML_NS)[0];
		if (countNode) {
			var hasChild = countNode.nodeValue > 1;
			if (!hasChild) {
				this.type = "node";
			} else {
				this.children = [];
			}
		}
		
		var parentNode = mm.xpath.evaluateXPath("thr:in-reply-to/@ref", entry, this.XML_NS)[0];
		if (parentNode) {
			var parentStr = parentNode.nodeValue;
			if (parentStr) {
				this.parentId = this.parseId(parentStr);
			} else {
				this.type = "root";
				this.title = "";
			}
		} else {
			this.type = "root";
			this.title = "";
		}
	},
	
	addChild: function(navigationNode) {
		if (navigationNode && navigationNode.id && navigationNode.title) {
			this.children.push(navigationNode);
		}
	},
	
	parseId: function(/*string*/id) {
		// get rid of "nm:oid:"
		if (dojo.isString(id) && id.indexOf("nm:oid:") == 0) {
			return id.substring(7);
		}
        if (dojo.isString(id) && id.indexOf("nm:id:") == 0) {
			return id.substring(6);
		}
		return id;
	}
});

}

if(!dojo._hasResource["mm.NavigationModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.NavigationModel"] = true;
dojo.provide("mm.NavigationModel");



dojo.declare("mm.NavigationModel", null, {
	XML_NS: {
		atom: "http://www.w3.org/2005/Atom",
		nm: "http://www.ibm.com/xmlns/prod/lotus/remix/v1.0/navigation-model",
		xhtml: "http://www.w3.org/1999/xhtml",
		thr: "http://purl.org/syndication/thread/1.0",
		app: "http://www.w3.org/2007/app",
		model: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements",
		base: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"
	},
	
	constructor: function(/*mm.Server*/server){
		this.server = server;
	},
	
	getNavigationNodes: function(/*string*/nodeId, /*integer*/levels, /*function*/callback) {
		nodeId = (nodeId == null || nodeId == "") ? "default" : nodeId;
		levels = levels < 1 ? 1 : levels;
		
		var url = this.server.contentHandler + "?uri=nm:id:" + nodeId + "&levels=" + levels;
		var _this = this;
		this.server.GET(url, {}, function(xhr){
			if (xhr.status == 200) {
				var doc = mm.utils.createDocument(xhr.responseText);
				var rootIdNode = mm.xpath.evaluateXPath("atom:id", doc.documentElement, _this.XML_NS)[0];
				var rootId = rootIdNode ? rootIdNode.textContent || rootIdNode.text : nodeId;
				rootId = _this.parseId(rootId);
				
				var nodes = [];
				var rootNode = null;
				var childNodes = [];
				var entries = mm.xpath.evaluateXPath("atom:entry", doc.documentElement, _this.XML_NS);
				dojo.forEach(entries, function(entry){
					var nodeTypeNode = mm.xpath.evaluateXPath("atom:content/model:navigation-node/@type", entry, _this.XML_NS)[0];
					if (!nodeTypeNode || (nodeTypeNode && nodeTypeNode.value != "shared-root")) {
						var idNode = mm.xpath.evaluateXPath("atom:id", entry, _this.XML_NS)[0];
						var id = idNode ? idNode.textContent || idNode.text : "";
						id = _this.parseId(id);
						if (rootId == id) {
							// root node
							rootNode = new mm.model.NavigationNode(entry);
						} else {
							childNodes.push(new mm.model.NavigationNode(entry));
						}
					}
				});
				if (rootNode) {
					dojo.forEach(childNodes, function(node){
						rootNode.addChild(node);
					});
				}
				nodes.push(rootNode);
				callback(nodes);
			} else {
				callback(xhr.responseText, {
					"status": xhr.status,
					"statusText": xhr.statusText
				});
			}
		});
	},
	
	parseId: function(/*string*/id) {
		// get rid of "nm:oid:"
		if (dojo.isString(id) && id.indexOf("nm:oid:") == 0) {
			return id.substring(7);
		}
        // get rid of "nm:id:"
        if (dojo.isString(id) && id.indexOf("nm:id:") == 0) {
			return id.substring(6);
		}
		return id;
	}
});

}

if(!dojo._hasResource["mm.data.NavigationModelStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.data.NavigationModelStore"] = true;
dojo.provide("mm.data.NavigationModelStore");





dojo.declare("mm.data.NavigationModelStore", dojo.data.ItemFileReadStore, {
	/*
	 * summary:
	 * 	NavigationMdoelReadStore extends ItemFileReadStore to implement an 
	 * 	lazy-loading/faulting in Lotus Mashups navigation nodes on-demand.
	 * 
	 * example:
	 * 	var NavigationModelStore = new mm.data.NavigationModelStore({url:"https://localhost:8443/mum"});
	 */
	constructor: function(/* object */ keywordParameters){
		// summary: constructor
		// keywordParameters: {url: String, label: String}  Where label is optional and configures what should be used as the return from getLabel()
		
		this._arrayOfTopLevelItems = [];
		this._arrayOfAllItems = [];
		this._metadataNodes = null;
		this._loadFinished = false;
		this.url = keywordParameters.url;
		this.username = keywordParameters.username;
		this.password = keywordParameters.password;
		
		var mashupMaker = new mm.MashupMaker(this.url, this.username, this.password);
		this.navigationMode = new mm.NavigationModel(mashupMaker);
		
		this.callback = keywordParameters.callback;
		
		if(keywordParameters.label){
			this.label = keywordParameters.label;
		}
		this._loadInProgress = false;	//Got to track the initial load to prevent duelling loads of the dataset.
		this._queuedFetches = [];
		this._identityMap = {};
		this._identCount = 0;
		this._idProp = "_I";

		this._features = {'dojo.data.api.Read':true, 'dojo.data.api.Identity':'id'};
		this._labelAttr = "title";
	},
	
/***************************************
     dojo.data.api.Read API
***************************************/
	// @Override
	getAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getAttributes()
		this._assertIsItem(item);
		var attributes = ['id','title','type','parentId'];
		return attributes; //Array
	},
	
	// @Override
	isItem: function(/* anything */ something){
		//	summary: 
		//		See dojo.data.api.Read.isItem()
		//	description:
		//		Item must has id and title 
		return (something && 
				something.id != null && 
				something.title != null); //Boolean
	},
	
	// @Override
	isItemLoaded: function(/*object*/ something) {
		//	summary:
		//		Overload of the isItemLoaded function to look for items of type 'stub', which indicate
		//		the data hasn't been loaded in yet.
		//
		//	item:
		//		The item to examine.
		
		//For this store, if it has the value of stub for its type attribute, 
		//then the item hasn't been fully loaded yet.  It's just a placeholder.
		if(this.getValue(something, "type") === "stub") {
			return false;
		}
		return true;
	},
	
	// @Override
	loadItem: function(keywordArgs){
		//	summary:
		//		Lazy load loadItem function.
		//		It will also detect stub items in the newly loaded item and insert the stubs into the NavigationModelStore
		//		list so they can also be loaded in on-demand.
		//
		//	item:
		//		The item to examine.

		var item = keywordArgs.item;
		this._assertIsItem(item);
		
		//Need a reference to the store to call back to its structures.
		var self = this;

		//Fire the get and pass the proper callbacks to the deferred.
		function callback(data, error){
			if (!error) {
				// Callback for handling a successful load.
				
				//Clear off the stub indicators.
				item.type = (item.type == "stub") ? "node" : item.type;
				item.children = [];
				
				//Set up the loaded values in the format ItemFileReadStore uses for attributes.
				if (dojo.isArray(data)) {
					data = data[0];
				}
				
				function formatNode(node) {
					for (attr in node) {
						if (node[attr] && !dojo.isArray(node[attr])) {
							node[attr] = [node[attr]];
						}
					}
				}
				
				dojo.forEach(data.children, function(child){
					var node = formatNode(child);
					item.children.push(node);
				});
				
				for (i in data) {
					if (i == 'parentId') continue;	// do not reset parentId
					if (dojo.isArray(data[i])) {
						item[i] = data[i];
					}else{
						item[i] = [data[i]];
					}
				}
				//Reset the item in the reference.
				self._itemsByIdentity[item.id] = item;
				self._arrayOfAllItems[item[self._itemNumPropName]] = item;
				
				//Scan the new values in the item for extra stub items we need to 
				//add to the items array of the store so they can be lazy-loaded later...
				dojo.forEach(item.children, function(child){
					self._itemsByIdentity[child.id] = child;
					self._arrayOfAllItems.push(child);
				});
	
				//Done processing!  Call the onItem, if any.
				if(keywordArgs.onItem){
					var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					keywordArgs.onItem.call(scope, item);
				}
			} else {
				//Callback for any errors that occur during load.
				//Call the onComplete, if any
				if(keywordArgs.onError){
					var scope = keywordArgs.scope ? keywordArgs.scope : dojo.global;
					keywordArgs.onError.call(scope, error);
				}
			}
		};
		this.navigationMode.getNavigationNodes(self.getValues(item, "id"), 2, callback);
	},
	
	// @Override
	// The dojo.data.api.Read.fetch() function is implemented as
	// a mixin from dojo.data.util.simpleFetch.
	// That mixin requires us to define _fetchItems().
	_fetchItems: function(	/* Object */ keywordArgs, 
							/* Function */ findCallback, 
							/* Function */ errorCallback){
		//	summary: 
		//		See dojo.data.util.simpleFetch.fetch()
		var self = this;
		var filter = function(requestArgs, arrayOfItems){
			var items = [];
			if(requestArgs.query){
				var ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false; 

				//See if there are any string values that can be regexp parsed first to avoid multiple regexp gens on the
				//same value for each item examined.  Much more efficient.
				var regexpList = {};
				for(var key in requestArgs.query){
					var value = requestArgs.query[key];
					if(typeof value === "string"){
						regexpList[key] = dojo.data.util.filter.patternToRegExp(value, ignoreCase);
					}
				}

				for(var i = 0; i < arrayOfItems.length; ++i){
					var match = true;
					var candidateItem = arrayOfItems[i];
					if(candidateItem === null){
						match = false;
					}else{
						for(var key in requestArgs.query) {
							var value = requestArgs.query[key];
							if (!self._containsValue(candidateItem, key, value, regexpList[key])){
								match = false;
							}
						}
					}
					if(match){
						items.push(candidateItem);
					}
				}
				findCallback(items, requestArgs);
			}else{
				// We want a copy to pass back in case the parent wishes to sort the array. 
				// We shouldn't allow resort of the internal list, so that multiple callers 
				// can get lists and sort without affecting each other.  We also need to
				// filter out any null values that have been left as a result of deleteItem()
				// calls in ItemFileWriteStore.
				for(var i = 0; i < arrayOfItems.length; ++i){
					var item = arrayOfItems[i];
					if(item !== null){
						items.push(item);
					}
				}
				findCallback(items, requestArgs);
			}
		};

		if(this._loadFinished){
			filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
		}else{

			if(this._jsonFileUrl){
				//If fetches come in before the loading has finished, but while
				//a load is in progress, we have to defer the fetching to be 
				//invoked in the callback.
				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs, filter: filter});
				}else{
					this._loadInProgress = true;
					function callback(data, error){
						if (!error) {
							self._processNavigationNode(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							
							filter(keywordArgs, self._getItemsArray(keywordArgs.queryOptions));
							self._handleQueuedFetches();
							// invoke outside callback
							self.callback(self._arrayOfTopLevelItems[0]);
						} else {
							self._loadInProgress = false;
							errorCallback(error, keywordArgs);
							self.callback(keywordArgs, error);
						}
					};
					this.navigationMode.getNavigationNodes("", 2, callback);
				}
			}else if(this._jsonData){
				try{
					this._loadFinished = true;
					this._processNavigationNode(this._jsonData);
					this._jsonData = null;
					filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
				}catch(e){
					errorCallback(e, keywordArgs);
				}
			}else{
				errorCallback(new Error("dojo.data.ItemFileReadStore: No JSON source data was provided as either URL or a nested Javascript object."), keywordArgs);
			}
		}
	},
	
	// @Override
	_processNavigationNode: function(dataObject) {
		//	summary:
		//		Function to parse the loaded data into item format and build the internal items array.
		//	description:
		//		Function to parse the loaded data into item format and build the internal items array.
		//
		//	dataObject:
		//		The JS data object containing the raw data to convery into item format.
		//
		// 	returns: array
		//		Array of items in store item format.
		
		// First, we define a couple little utility functions...
		function valueIsAnItem(/* anything */ aValue){
			// summary:
			//		Given any sort of value that could be in the raw json data,
			//		return true if we should interpret the value as being an
			//		item itself, rather than a literal value or a reference.
			// example:
			// 	|	false == valueIsAnItem("Kermit");
			// 	|	false == valueIsAnItem(42);
			// 	|	false == valueIsAnItem(new Date());
			// 	|	false == valueIsAnItem({_type:'Date', _value:'May 14, 1802'});
			// 	|	false == valueIsAnItem({_reference:'Kermit'});
			// 	|	true == valueIsAnItem({name:'Kermit', color:'green'});
			// 	|	true == valueIsAnItem({iggy:'pop'});
			// 	|	true == valueIsAnItem({foo:42});
			var isItem = (
				(aValue != null) &&
				(typeof aValue == "object") &&
				(!dojo.isArray(aValue)) &&
				(!dojo.isFunction(aValue)) &&
				(typeof aValue._reference == "undefined") && 
				(typeof aValue._type == "undefined") && 
				(typeof aValue._value == "undefined") && 
				(aValue.id != "undefined") && 
				(aValue.type != "undefined")
			);
			return isItem;
		}
		
		var self = this;
		
		function addItemAndSubItemsToArrayOfAllItems(/* Item */ anItem){
			self._arrayOfAllItems.push(anItem);
			for(var attribute in anItem){
				if (attribute == "XML_NS") {
					continue;
				}
				var valueForAttribute = anItem[attribute];
				if(valueForAttribute){
					if(dojo.isArray(valueForAttribute)){
						var valueArray = valueForAttribute;
						for(var k = 0; k < valueArray.length; ++k){
							var singleValue = valueArray[k];
							if(valueIsAnItem(singleValue)){
								addItemAndSubItemsToArrayOfAllItems(singleValue);
							}
						}
					}else{
						if(valueIsAnItem(valueForAttribute)){
							addItemAndSubItemsToArrayOfAllItems(valueForAttribute);
						}
					}
				}
			}
		}

		// We need to do some transformations to convert the data structure
		// that we read from the file into a format that will be convenient
		// to work with in memory.

		// Step 1: Walk through the object hierarchy and build a list of all items
		var i;
		var item;
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = dataObject;
		
		for(i = 0; i < this._arrayOfTopLevelItems.length; ++i){
			item = this._arrayOfTopLevelItems[i];
			addItemAndSubItemsToArrayOfAllItems(item);
			item[this._rootItemPropName]=true;
		}

		// Step 2: Walk through all the attribute values of all the items, 
		// and replace single values with arrays.  For example, we change this:
		//		{ name:'Miss Piggy', pets:'Foo-Foo'}
		// into this:
		//		{ name:['Miss Piggy'], pets:['Foo-Foo']}
		// 
		// We also store the attribute names so we can validate our store  
		// reference and item id special properties for the O(1) isItem
		var allAttributeNames = {};
		var key;

		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			for(key in item){
				if (key !== this._rootItemPropName)
				{
					var value = item[key];
					if(value !== null){
						if(!dojo.isArray(value)){
							item[key] = [value];
						}
					}else{
						item[key] = [null];
					}
				}
				allAttributeNames[key]=key;
			}
		}

		// Step 3: Build unique property names to use for the _storeRefPropName and _itemNumPropName
		// This should go really fast, it will generally never even run the loop.
		while(allAttributeNames[this._storeRefPropName]){
			this._storeRefPropName += "_";
		}
		while(allAttributeNames[this._itemNumPropName]){
			this._itemNumPropName += "_";
		}

		// Step 4: Some data files specify an optional 'identifier', which is 
		// the name of an attribute that holds the identity of each item. 
		// If this data file specified an identifier attribute, then build a 
		// hash table of items keyed by the identity of the items.
		var arrayOfValues;

		var identifier = 'id';
		if(identifier){
			this._itemsByIdentity = {};
			this._features['dojo.data.api.Identity'] = identifier;
			for(i = 0; i < this._arrayOfAllItems.length; ++i){
				item = this._arrayOfAllItems[i];
				arrayOfValues = item[identifier];
				var identity = arrayOfValues[0];
				if(!this._itemsByIdentity[identity]){
					this._itemsByIdentity[identity] = item;
				}else{
					if(this._jsonFileUrl){
						throw new Error("dojo.data.ItemFileReadStore:  The json data as specified by: [" + this._jsonFileUrl + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}else if(this._jsonData){
						throw new Error("dojo.data.ItemFileReadStore:  The json data provided by the creation arguments is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}
				}
			}
		}else{
			this._features['dojo.data.api.Identity'] = Number;
		}

		// Step 5: Walk through all the items, and set each item's properties 
		// for _storeRefPropName and _itemNumPropName, so that store.isItem() will return true.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			item[this._storeRefPropName] = this;
			item[this._itemNumPropName] = i;
		}

		// Step 6: We walk through all the attribute values of all the items,
		// looking for type/value literals and item-references.
		//
		// We replace item-references with pointers to items.  For example, we change:
		//		{ name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
		// into this:
		//		{ name:['Kermit'], friends:[miss_piggy] } 
		// (where miss_piggy is the object representing the 'Miss Piggy' item).
		//
		// We replace type/value pairs with typed-literals.  For example, we change:
		//		{ name:['Nelson Mandela'], born:[{_type:'Date', _value:'July 18, 1918'}] }
		// into this:
		//		{ name:['Kermit'], born:(new Date('July 18, 1918')) } 
		//
		// We also generate the associate map for all items for the O(1) isItem function.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i]; // example: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
			for(key in item){
				arrayOfValues = item[key]; // example: [{_reference:{name:'Miss Piggy'}}]
				for(var j = 0; j < arrayOfValues.length; ++j) {
					value = arrayOfValues[j]; // example: {_reference:{name:'Miss Piggy'}}
					if(value !== null && typeof value == "object"){
						if(value._type && value._value){
							var type = value._type; // examples: 'Date', 'Color', or 'ComplexNumber'
							var mappingObj = this._datatypeMap[type]; // examples: Date, dojo.Color, foo.math.ComplexNumber, {type: dojo.Color, deserialize(value){ return new dojo.Color(value)}}
							if(!mappingObj){ 
								throw new Error("dojo.data.ItemFileReadStore: in the typeMap constructor arg, no object class was specified for the datatype '" + type + "'");
							}else if(dojo.isFunction(mappingObj)){
								arrayOfValues[j] = new mappingObj(value._value);
							}else if(dojo.isFunction(mappingObj.deserialize)){
								arrayOfValues[j] = mappingObj.deserialize(value._value);
							}else{
								throw new Error("dojo.data.ItemFileReadStore: Value provided in typeMap was neither a constructor, nor a an object with a deserialize function");
							}
						}
						if(value._reference){
							var referenceDescription = value._reference; // example: {name:'Miss Piggy'}
							if(dojo.isString(referenceDescription)){
								// example: 'Miss Piggy'
								// from an item like: { name:['Kermit'], friends:[{_reference:'Miss Piggy'}]}
								arrayOfValues[j] = this._itemsByIdentity[referenceDescription];
							}else{
								// example: {name:'Miss Piggy'}
								// from an item like: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
								for(var k = 0; k < this._arrayOfAllItems.length; ++k){
									var candidateItem = this._arrayOfAllItems[k];
									var found = true;
									for(var refKey in referenceDescription){
										if(candidateItem[refKey] != referenceDescription[refKey]){ 
											found = false; 
										}
									}
									if(found){ 
										arrayOfValues[j] = candidateItem; 
									}
								}
							}
						}
					}
				}
			}
		}
	}
});

}

if(!dojo._hasResource["mm.pages.AddMashupPageToMashups"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.pages.AddMashupPageToMashups"] = true;
dojo.provide("mm.pages.AddMashupPageToMashups");





/**
 * Add Mashup Page to Mashups
 */
dojo.declare("mm.pages.AddMashupPageToMashups", hub.pages._Page, {
    // template to replace node with
    templateString:"<div>\r\n\t{% if object %}\r\n\t<!-- Add To Mashups Form  -->\r\n\t<form class=\"TOM_panel TOM_admin_panel\" id=\"addToMashupsForm_{{ object.id }}\" action=\"javascript:void(0);\">\r\n\t\t<div>\r\n\t\t\t<label>\t<img src='{{ icon_page }}' /></label>\r\n\t\t\t<div class='rightCol'><h1>{{ labels.ADDTOMASHUPS_PAGE_TITLE_TEXT }}</h1></div>\r\n\t\t</div>\t\r\n\t\t<br/>\r\n\t\t<label for=\"title\"><span class='required'>*</span> {{ labels.ADDTOMASHUPS_TITLE }}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<input name=\"title\" type=\"text\" value=\"{{ object.title }}\" maxlength=\"200\"></input>\r\n\t\t</div>\r\n\t\t\r\n\t\t<label for=\"description\">{{ labels.ADDTOMASHUPS_DESCRIPTION }}:</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<textarea rows=\"4\" name=\"description\" maxlength=\"1000\">{{ object.description }}</textarea>\r\n\t\t</div>\r\n\t\t\r\n\t\t<!-- Location on the Mashups -->\r\n\t\t<label for=\"navigation\" style=\"height: 7em;\">{{ labels.ADDTOMASHUPS_NAVIGATION }}:</label>\r\n\t\t<!-- Position of the page -->\r\n\t\t<div class=\"rightCol\">\r\n\t\t\t<ul style=\"padding:0; list-style:none;\">\r\n\t\t\t\t<li style=\"display:none\">\r\n\t\t\t\t\t<input type=\"radio\" name=\"position\" value=\"above\" dojoType='dijit.form.RadioButton'>{{ labels.ADDTOMASHUPS_PAGE_ABOVE_SELECTED_PAGE }}</input>\r\n\t\t\t\t</li>\r\n\t\t\t\t<li style=\"display:none\">\r\n\t\t\t\t\t<input type=\"radio\" name=\"position\" value=\"below\" dojoType='dijit.form.RadioButton'>{{ labels.ADDTOMASHUPS_PAGE_BELOW_SELECTED_PAGE }}</input>\r\n\t\t\t\t</li>\r\n\t\t\t\t<li>\r\n\t\t\t\t\t<input type=\"radio\" name=\"position\" value=\"child\" checked='checked' dojoType='dijit.form.RadioButton'>{{ labels.ADDTOMASHUPS_PAGE_AS_CHILD_OF_SELECTED_PAGE }}</input>\r\n\t\t\t\t</li>\r\n\t\t\t</ul>\r\n\t\t</div>\r\n\t\t<!-- Mashups Navigation Tree Nodes Selector -->\r\n\t\t<div class=\"rightCol\">\r\n\t\t\t<div class=\"TOM_widget_CheckBoxTitlePane\" id=\"navigation_node_{{ object.id }}\" dojoType=\"dijit.TitlePane\" title=\"{{ message.STATUS_LOADING }}\" open=\"true\">\r\n\t\t\t\t<div id=\"navigation_treeDiv_{{object.id}}\"></div>\r\n\t\t    </div>\r\n\t\t</div>\r\n\t\t\r\n\t\t<input type=\"hidden\" name=\"object_id\" value=\"{{object.id}}\"></input>\r\n\t\t<input type=\"hidden\" name=\"server\" value='{{ server }}' size=\"50\" readonly></input>\r\n\t\t\r\n\t\t<!-- Status Text Div -->\r\n\t\t<div class='rightCol' id=\"add_to_mashups_message_{{ object.id }}\" style=\"color:red;\"></div>\r\n\t</form>\r\n\t<div style=\"padding: 50px 0px 10px 180px; text-align: left;\">\r\n\t\t{% if palette_url %}\r\n\t\t<button class=\"BlueButton BlueButtonDisabled\" id=\"changeServerButton_{{ object.id }}\" disabled=\"true\"></button>\r\n\t\t{% else %}\r\n\t\t<button class=\"BlueButton\" id=\"changeServerButton_{{ object.id }}\"></button>\r\n\t\t{% endif %}\r\n\t\t<button class=\"BlueButton\" id=\"add_to_mashups_cancel_{{ object.id }}\"></button>\r\n\t\t<button class=\"BlueButton\" id=\"addtomashups_button_{{ object.id }}\" name=\"submit\"></button>\r\n\t</div>\r\n\t{% endif %}\r\n</div>\r\n",
	
	type: 'mashuppage',	// Constant
	
	isPackage: false,
	
	// attach points
	form: null,
	button: null,
	serverbutton: null,
	message: null,
	titlePane: null,
	tree: null,
	
	// parameters form caller
    object_id: "",
	object_title: "",
	server: "",
	username: "",
	password: "",
	
	//parameters from MashupMaker
	navigationRoot : null,
	navigationStore : null,
	navigationTree: null,
	icons: "",
	selectedIdentity: "",
	
	details: null,
	navigationPath: [],
	
	constructor:function(){
	},
	
    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getDetailsPage(this.object_id);
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
		
		
		
		
		
		this.details = server_data;
		
		if (!server_data.pluginData.definitionUrl) {
			this.isPackage = true;
		}
		
        var obj = {
			// mm parameters
			server: this.server,
			username: this.username,
			password: this.password,
			//navigationRoot: this.navigationRoot[0],
			icons: this.icons,
			isPackage: this.isPackage,	// Tell if the mashup page is a definitionURL or a Package
			palette_url: hub.config.palette_url,
			//messages
			labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups'),
            messages: dojo.i18n.getLocalization('hub', 'messages'),
			//icons
			icon_page: hub.urls.getImage('pageLarge.gif')
        };
        return dojo.mixin(obj, server_data);
    },
	
	/**
	 * @Override
     * Uses the rendered deferred to indicate page rendering is complete.
     * Can be used as an extension point for dojo.connect too, but only if it is
     * known ahead of time that rendering is not already complete at time of
     * connection.
     */
    onRendered: function() {
		this.inherited(arguments);
		/*
    	var navigationStore = new dojo.data.ItemFileReadStore(data);
		this.navigationTree = new dijit.Tree({store: navigationStore, onClick: this.onTitleClick}, treeDiv);
		*/
		window.setTimeout(dojo.hitch(this, function(){
			this.form = dojo.byId("addToMashupsForm_" + this.object_id);
			this.message = dojo.byId("add_to_mashups_message_" + this.object_id);
			this.button = dojo.byId("addtomashups_button_" + this.object_id);
			this.serverbutton = dojo.byId("changeServerButton_" + this.object_id);
			this.titlePane = dijit.byId("navigation_node_" + this.object_id);
			
			var labels = dojo.i18n.getLocalization('hub', 'labels');
			this.button.innerHTML = labels.MASHUPS_FINISH;
			this.serverbutton.innerHTML = labels.MASHUPS_PREVIOUS;
			var cancelButton = dojo.byId("add_to_mashups_cancel_" + this.object_id);
			cancelButton.innerHTML = labels.MASHUPS_CANCEL;
			
			var self = this;
			var callback = function(root, error) {
				if (!error) {
					self.selectedIdentity = dojo.isArray(root.id) ? root.id[0] : root.id;
					var title = dojo.isArray(root.title) ? root.title[0] : root.title;
					self.titlePane.setTitle("<img src='../images/nav_home.png'/>");
					var rootNode = self.tree._navToRootOrFirstNode();
					rootNode.labelNode.innerHTML = "<img src='../images/nav_home.png'/>";
					if (self.tree.isExpandable) {
						self.tree._expandNode(rootNode);
					}
				} else {
					hub.showPage(self.object_title, "SelectMashupsServer", {
						object_id: self.object_id,
						object_title: self.object_title,
						server: self.server,
						username: self.username,
						password: self.password,
						type: self.type,
						error: dojo.i18n.getLocalization('hub', 'messages').ERROR_MASHUPS_GET_NAVIGATIONNODES
					});
				}
			}
			var _navigationStore = new mm.data.NavigationModelStore({url:this.server, username:this.username, password: this.password, callback: callback});
			
			this.tree = new dijit.Tree({
				id: "navigation_tree_" + this.object_id,
				store : _navigationStore,
				labelAttr : "title",
                typeAttr : "type",
				persist : false
			}, dojo.byId("navigation_treeDiv_" + this.object_id));
			
			dojo.connect(this.button, "onclick", this, "onSubmit");
			dojo.connect(this.serverbutton, "onclick", this, "onChangeServer");
			dojo.connect(this.tree, "onClick", this, "onTitleClick");
			dojo.connect(cancelButton, "onclick", this, "onCancel");
		}), 0);
    },
	
	onTitleClick: function(node) {
		// summary:
		// 		set titlpane's title on tree node click
		var store = this.tree.store;
		var path = [];
		path.push(node.title[0]);
		var parentId = dojo.isArray(node.parentId) ? node.parentId[0] : node.parentId;
		this.selectedIdentity = dojo.isArray(node.id) ? node.id[0] : node.id;
		while (parentId) {
			var item = store._getItemByIdentity(parentId);
			path.push(item.title[0]);
			parentId = dojo.isArray(item.parentId) ? item.parentId[0] : item.parentId;
		}
		if (path.length > 0) {
			this.navigationPath = path.reverse();
			if (this.navigationPath.length > 0 && dojo.string.trim(this.navigationPath[0]) == "") {
				this.titlePane.setTitle("<img src='../images/nav_home.png'/>" + this.navigationPath.join(" > "));
			} else {
				this.titlePane.setTitle(this.navigationPath.join(" > "));
			}
		}
	},
	
	onSubmit: function() {
		var form = dojo.formToObject(this.form);
		var messages = dojo.i18n.getLocalization('hub', 'messages');
		this.message.innerHTML = "";
		
		// Validating form
		if (!form.title) {
			this.message.innerHTML = messages.ERROR_MASHUPS_TITLE;
			this.button.disabled = false;
			return;
		}
		if (!this.selectedIdentity) {
			this.message.innerHTML = messages.ERROR_MASHUPS_NAVIGATION;
			this.button.disabled = false;
			return;
		}
		
		//add to mashup maker
		var mashupMaker = new mm.MashupMaker(this.server, this.username, this.password);
		
		if (this.isPackage) {
			var deployer = new mm.MashupPageDeployer(mashupMaker);
			deployer.packageUrl = this.details.object.url;
			deployer.target = this.selectedIdentity;
			deployer.title = form.title;
			deployer.position = form.position;
			deployer.description = form.description; 
		
			this.button.disabled = true;
			
			var _this = this;
			deployer.deploy(dojo.hitch(this, function(data, error){
					if (!error) {
						_this.navigationPath.push(_this.object_title);
						hub.showPage(_this.object_title, "FinishedAddToMashups", {
							object_id: _this.object_id,
							object_title: _this.object_title,
							type: _this.type,
							navigationPath: _this.navigationPath
						});
					} else {
						_this.message.innerHTML = messages.STATUS_MASHUPS_FAIL + " " + error.status + "," + error.statusText;
						_this.button.disabled = false;
					}
			}));
		}
	},
	
	/**
	 * click handler: cancel the "add to mashups" action
	 */
	onCancel: function() {
		hub.showPage(this.object_title, "Details", {"object_id": this.object_id});
	},
	
	onChangeServer: function(){
		hub.showPage(this.object_title, "SelectMashupsServer", {
			object_id: this.object_id,
			object_title: this.object_title,
			server: this.server,
			username: this.username,
			password: this.password,
			type: this.type
		});
	}
});

}

if(!dojo._hasResource["mm.pages.FinishedAddToMashups"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.pages.FinishedAddToMashups"] = true;
dojo.provide("mm.pages.FinishedAddToMashups");




dojo.declare("mm.pages.FinishedAddToMashups", hub.pages._Page, {
	// template to replace node with
	templateString:"<div class=\"TOM_panel\">\r\n<h1 dojoAttachPoint=\"heading\">{{ title }}</h1>\r\n\t{% if isPage %}\r\n\t<p>\r\n\t\t{{ subtitle }}<img src='../images/nav_home.png'/>{{navigation_path}}\r\n\t</p>\r\n\t{% endif %}\r\n\t<br/>\r\n\t<div class=\"taskbox\">\r\n\t\t<h2>{{ panels.ADD_TO_MASHUPS_LANDING_NEXT_TASKS }}</h2>\r\n\t\t<ul class=\"TOM_action_list\">\r\n\t\t\t<li><a href='javascript:hub.showPage(\"{{ object_title }}\",\"Details\",{\"object_id\": \"{{ object_id }}\"})'>{{ taskDetails }}</a></li>\r\n\t\t\t<li><a href='javascript:hub.closeCurrentPage();' >{{ panels.ADD_TO_MASHUPS_LANDING_TASK_CLOSE }}</a></li>\r\n\t\t</ul>\r\n\t</div>\r\n</div>\r\n",
	
	object_title: "",
	object_id: "",
	type: "",
	
	// Page
	navigationPath: [],
	
	
	/**
	 * Builds an object with properties matching fields in the template. Merges
	 * fetched server data with local data.
	 *
	 * @param server_data: JSON decoded object fetched from server URL
	 */
	getContext: function(server_data){
		var panels = dojo.i18n.getLocalization('hub', 'panels');
		var obj = {
			object_id: this.object_id,
			object_title: this.object_title,
			title: dojo.string.substitute(panels.ADD_TO_MASHUPS_LANDING_TITLE, [this.object_title]),
			subtitle: dojo.string.substitute(panels.ADD_TO_MASHUPS_LANDING_SUBTITLE, [this.object_title]),
			navigation_path: this.navigationPath.join(" > "),
			taskDetails: dojo.string.substitute(panels.LANDING_TASK_DETAILS, [this.type]),
			//page
			isFeed: this.type == "feed",
			isWidget: this.type == "iwidget",
			isPage: this.type == "mashuppage",
			isMashup: this.type == "mashup",
			// messages
			labels: dojo.i18n.getLocalization('hub', 'labels'),
			messages: dojo.i18n.getLocalization('hub', 'messages'),
			panels: panels,
			//icons
			icon_feed: hub.urls.getImage('feedLarge.gif'),
			icon_widget: hub.urls.getImage('widgetLarge.gif'),
			icon_page: hub.urls.getImage('pageLarge.gif'),
			icon_mashup: hub.urls.getImage('mashupLarge.gif')
		};
		return obj;
	}
});

}

if(!dojo._hasResource["hub.pages.main"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.pages.main"] = true;
dojo.provide("hub.pages.main");


/**
 * Login page. No server data to fetch.
 */
dojo.declare("hub.pages.Login", hub.pages._Page, {
    // template to replace node with
    templateString:"<div class=\"TOM_editor_pane\">\r\n  <form id=\"TOM_large_login_form\" onsubmit=\"return false;\">\r\n    <fieldset>\r\n      <legend>{{panels.LOGIN_LEGEND}}</legend>\r\n      <label for='login'>{{panels.LOGIN_LOGIN}}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"text\" name=\"login\" maxlength=\"60\" />\r\n      </div>\r\n      \r\n      <label for=\"password\">{{panels.LOGIN_PASSWORD}}</label>\r\n      <div class=\"rightCol\">\r\n\t<input name=\"password\" maxlength=\"60\" type=\"password\" />\r\n      </div>\r\n      <div class=\"rightCol\">\r\n        <button type=\"submit\" dojoType=\"dijit.form.Button\">{{labels.BUTTON_LOGIN}}\r\n          <script type=\"dojo/method\" event=\"onClick\">\r\n            var form = new hub.submit.Login(\"TOM_large_login_form\", \r\n                                            \"TOM_large_login_message\", \r\n                                            null, true);\r\n            hub.submit.ajaxSubmit(form);\r\n          </script>\r\n        </button>\r\n        <p id=\"TOM_large_login_message\"></p>\r\n      </div>\r\n    </fieldset>\r\n  </form>\r\n</div>\r\n",

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function() {
        
        
        return {panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels')};
    }
});

/**
 * Login page. Ajax request server for login url.
 */
dojo.declare("hub.pages.LoginPlease", hub.pages._Page, {
    // template to replace node with
    templateString:"<div class=\"TOM_editor_pane\">\r\n<p class=\"TOM_editor_error\">\r\n        <a href='{{ link }}'>{{panels.LOGIN_PLEASE}}</a>\r\n</p>\r\n</div>\r\n",

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function() {

		// get login URL
		var loginURL='#';

		dojo.xhrGet({
			url: hub.config.base_url + "/ui/loginURL",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				loginURL = response.loginURL;
			}
		});

        return {link: loginURL,
        		panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels')};
    }
});

/**
 * Account registration page. No server data to fetch.
 */
dojo.declare("hub.pages.Register", hub.pages._Page, {
    // template to replace node with
    templateString:"<div class=\"TOM_editor_pane\">\r\n  <form id=\"TOM_register_form\" onsubmit=\"return false;\">\r\n    <fieldset>\r\n      <legend>{{ panels.REGISTER_LEGEND }}</legend>\r\n\r\n      <label for=\"registerName\">{{ panels.REGISTER_FULL_NAME }}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"text\" name=\"registerName\" maxlength=\"100\" title=\"{{ popups.REGISTER_FULL_NAME }}\" />\r\n      </div>\r\n\r\n      <label for=\"registerEmail\">{{ panels.REGISTER_EMAIL }}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"text\" name=\"registerEmail\" maxlength=\"128\" title=\"{{ popups.REGISTER_EMAIL }}\" />\r\n      </div>\r\n\r\n      <label for=\"registerLogin\">{{ panels.REGISTER_LOGIN }}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"text\" name=\"registerLogin\" maxlength=\"60\" title=\"{{ popups.REGISTER_LOGIN }}\" />\r\n      </div>\r\n\r\n      <label for=\"registerPassword\">{{ panels.REGISTER_PASSWORD }}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"password\" name=\"registerPassword\" maxlength=\"60\" title=\"{{ popups.REGISTER_PASSWORD }}\" />\r\n      </div>\r\n\r\n      <label for=\"registerPassword2\">{{ panels.REGISTER_PASSWORD2 }}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"password\" name=\"registerPassword2\" maxlength=\"60\" title=\"{{ popups.REGISTER_PASSWORD2 }}\" />\r\n      </div>\r\n\r\n\r\n      <div class=\"rightCol\">\r\n        <button type=\"submit\" dojoType=\"dijit.form.Button\">{{ labels.BUTTON_REGISTER }}\r\n          <script type=\"dojo/method\" event=\"onClick\">\r\n            var element = dojo.byId(\"TOM_register_form\");\r\n            var form = new hub.submit.Register(element, \"TOM_register_message\");\r\n            hub.submit.ajaxSubmit(form);\r\n          </script>\r\n        </button>\r\n        <p id=\"TOM_register_message\"></p>\r\n      </div>\r\n    </fieldset>\r\n  </form>\r\n</div>\r\n",

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function() {
        
        
        return {panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels'),
                popups: dojo.i18n.getLocalization('hub', 'popups')};;
    }
});

/**
 * Tag and category clouds page. No need to fetch server data for the template
 * because the widgets will use the global tags and categories models.
 */
dojo.declare("hub.pages.Clouds", hub.pages._Page, {
    // template to replace node with
    templateString:"<div class=\"TOM_home_panel\">\r\n  <table class=\"layout\" style=\"width: 100%;\">\r\n    <tr valign=\"top\" style=\"padding-top: 30px;\">\r\n      <td width=\"50%\" style=\"padding-right: 75px;\">\r\n        <h1>{{ panels.CLOUDS_TAGS }}</h1>\r\n        <p style=\"text-align: center;\"><label >{{ panels.CLOUDS_DENSITY }}</label></p>\r\n        <div name=\"density\" dojoType=\"dijit.form.HorizontalSlider\"\r\n             value=\"10\" minimum=\"1\" maximum=\"10\" discreteValues=\"10\"\r\n             intermediateChanges=\"false\" showButtons=\"false\">\r\n          <div dojoType=\"dijit.form.HorizontalRule\" container=\"bottomDecoration\" count=\"10\" style=\"height:5px;\" title=\"foobar\"></div>\r\n          <ol dojoType=\"dijit.form.HorizontalRuleLabels\" container=\"bottomDecoration\">\r\n            <li>{{ panels.CLOUDS_LESS }}</li>\r\n            <li>{{ panels.CLOUDS_MORE }}</li>\r\n          </ol>\r\n          <script type=\"dojo/method\" event=\"onChange\">\r\n            var percent = this.getValue()*10;\r\n            var filter = dijit.byId('hub.clouds.density');\r\n            filter.setValue(percent);\r\n            var box = dijit.byId('hub.clouds.tags');\r\n            box.redraw();\r\n          </script>\r\n        </div>\r\n        <br />\r\n        <div dojoType=\"hub.widgets.social.view.SocialBox\" model=\"hub.tags\" class=\"TOM_widget_TagsBox\" style=\"font-size: 1em;\" id=\"hub.clouds.tags\">\r\n          <div dojoType=\"hub.widgets.social.view.DensityTagsFilter\" value=\"100\" id=\"hub.clouds.density\"></div>\r\n          <div dojoType=\"hub.widgets.social.view.TagsCloudView\"></div>\r\n        </div>\r\n      </td>\r\n      <td width=\"50%\">\r\n        <h1>{{ panels.CLOUDS_CATEGORIES }}</h1>\r\n        <div dojoType=\"hub.widgets.social.view.SocialBox\" model=\"hub.cats\" class=\"TOM_widget_CategoriesBox\" style=\"font-size: 1em;\">\r\n          <div dojoType=\"hub.widgets.social.view.CategoriesListView\"></div>\r\n        </div>\r\n      </td>\r\n    </tr>\r\n  </table>\r\n</div>\r\n",

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function() {
        
        
        return {panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels'),
                popups: dojo.i18n.getLocalization('hub', 'popups')};;
    }
});


/**
 * Catalog display page. Consists mostly of other widgets. No need to fetch
 * server data for the template because the widgets will fetch it themselves.
 */
dojo.declare("hub.pages.Catalog", hub.pages._Page, {
    // template to replace node with
    templateString:"<div class=\"TOM_catalog\">\r\n  <div dojoType=\"dijit.layout.LayoutContainer\" style=\"width: 100%; height:100%\">\r\n  <div class=\"TOM_catalog_Client\" dojoType='dijit.layout.ContentPane' layoutAlign='client' style=\"width: 100%; height:100%\">\r\n\t<div dojoType=\"hub.data.catalog.AtomModel\" id=\"hub.catalog.{{ id }}\" collection=\"{{ collection }}\" term=\"{{ term }}\" searchType=\"{{ searchType }}\" sortBy=\"{{ sortBy }}\" sortOrder=\"{{ sortOrder }}\"></div>\r\n\t<div class=\"TOM_widget_CatalogTable\" dojoType=\"hub.widgets.catalog.CatalogTable\" model=\"hub.catalog.{{ id }}\" forceRefresh=\"true\"></div>\r\n  </div>\r\n  <div class=\"TOM_catalog_Sidebar\" dojoType='dijit.layout.ContentPane' layoutAlign='right' style='overflow-x: none; overflow-y: auto;'> \r\n    <div class=\"panel\">\r\n      <div class=\"panelHeader\">\r\n      \t<h3>{{ panels.CATALOG_TOP_TAGS_TITLE }}</h3>\r\n      </div>\r\n      <div dojoType=\"hub.widgets.social.view.SocialBox\" model=\"hub.tags\" class=\"TOM_catalog_TagsBox\">\r\n        <div dojoType=\"hub.widgets.social.view.TopTagsFilter\" value=\"20\"></div>\r\n        <div dojoType=\"hub.widgets.social.view.TagsCloudView\"></div>\r\n      </div>\r\n      <div class=\"footer\">\r\n        <a href=\"javascript:hub.showPage('{{ labels.HOME_TAGS_CATS }}', 'Clouds');\" title=\"{{ panels.ACTION_TAGS_CATS_HELP }}\">{{ links.ACTION_ALL_TAGS_LINK }}</a>\r\n      </div>\r\n    </div>\r\n    \r\n    <div class=\"panel\">\r\n      <div class=\"panelHeader\">\r\n      \t<h3>{{ panels.CATALOG_CATEGORIES_TITLE }}</h3>\r\n      </div>\r\n      <div dojoType=\"hub.widgets.social.view.SocialBox\" model=\"hub.cats\" class=\"TOM_catalog_CategoriesBox\">\r\n        <div dojoType=\"hub.widgets.social.view.CategoriesListView\"></div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  </div>\r\n</div>\r\n",
    // default catalog query
    query: 'all',
    // default catalog query search term
    term: '*',
    collection: 'all',
    searchType: "",
    sortBy: "",
    sortOrder: "",

    /**
     * Imports required modules and build an object with properties matching
     * fields in the template.
     */
    getContext: function() {
        // modules required to render all subwidgets
        
        
        
        

        // return object with data to interpolate into context
        // @note: uses the page's dojo assigned id to help uniquely name
        //   elements in the template
        return {id: this.id,
                collection: this.collection,
                term: this.term,
                searchType: this.searchType,
                sortBy: this.sortBy,
                sortOrder: this.sortOrder,
                panels: dojo.i18n.getLocalization('hub', 'panels'),
                links: dojo.i18n.getLocalization('hub', 'links'),
                labels: dojo.i18n.getLocalization('hub', 'labels')};
    }
});

/**
 * Home page. Fetches some data from the server to complete links for launching
 * plug-in editors.
 */
dojo.declare("hub.pages.Home", hub.pages._Page, {
    // template to replace node with
    templateString:"<div dojoType=\"dijit.layout.ContentPane\" title=\"Home\" class=\"TOM_home\">\r\n  <table align=\"center\" cellpadding=\"0\" cellspacing=\"10\" border=\"0\" style=\"width: 10%;\">\r\n    \r\n    <tr>\r\n      <td>\r\n\t\t<div class=\"hub_home_box\">\r\n\t\t\t<div class=\"hub_home_box_header\">\r\n\t\t\t\t<div class=\"hub_home_box_label\">{{ panels.HOME_CATALOG_TASKS }}</div>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"hub_home_box_body\">\r\n\t\t\t\t<p class=\"hub_home_box_description\">{{ panels.HOME_CATALOG_TASKS_DESC }}</p>\r\n\t\t\t\t<table cellpadding=\"0\" cellspacing=\"2\" border=\"0\" >\r\n\t\t\t\t\t<tr valign=\"top\">\r\n\t\t\t\t\t\t<td class=\"hub_home_box_table_left\" >\r\n\t\t\t\t\t\t\t<img src=\"{{ img_catalog }}\" border=\"0\"/>\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t<td class=\"hub_home_box_table_rigth\" >\r\n\t\t\t\t\t\t\t<ul class=\"hub_home_box_list\">\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showPage('{{ labels.HOME_FEEDS }}', 'Catalog', {collection: 'ui.feeds', searchType: '', term: '*', sortBy: 'title', sortOrder: 'asc'});\" title=\"{{ popups.HOME_FEEDS }}\">{{ links.HOME_FEEDS }}</a></li>\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showPage('{{ labels.HOME_WIDGETS }}', 'Catalog', {collection: 'ui.widgets', searchType: '', term: '*', sortBy: 'title', sortOrder: 'asc'});\" title=\"{{ popups.HOME_WIDGETS }}\">{{ links.HOME_WIDGETS }}</a></li>\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showPage('{{ labels.HOME_WORKSPACES }}', 'Catalog', {collection: 'ui.pages', searchType: '', term: '*', sortBy: 'title', sortOrder: 'asc'});\" title=\"{{ popups.HOME_WORKSPACES }}\">{{ links.HOME_WORKSPACES }}</a></li>\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showPage('{{ labels.HOME_EVERYTHING }}', 'Catalog', {collection: 'ui.all', searchType: '', term: '*', sortBy: 'title', sortOrder: 'asc'});\" title=\"{{ popups.HOME_EVERYTHING }}\">{{ links.HOME_EVERYTHING }}</a></li>\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showPage('{{ labels.HOME_FAVORITES }}', 'Catalog', {collection: 'ui.all', searchType: 'favorites', term: '*', sortBy: 'title', sortOrder: 'asc'});\" title=\"{{ popups.HOME_FAVORITES }}\">{{ links.HOME_FAVORITES }}</a></li>\r\n\t\t\t\t\t\t\t</ul>\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t</tr>\r\n\t\t\t\t</table>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"hub_home_box_footer\"></div>\r\n\t\t</div>\r\n      </td>\r\n\r\n      <td>\r\n\t\t<div class=\"hub_home_box\">\r\n\t\t\t<div class=\"hub_home_box_header\">\r\n\t\t\t\t<div class=\"hub_home_box_label\">{{ panels.HOME_CREATION_TASKS }}</div>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"hub_home_box_body\">\r\n\t\t\t\t<p class=\"hub_home_box_description\">{{ panels.HOME_CREATION_TASKS_DESC }}</p>\r\n\t\t\t\t<table cellpadding=\"0\" cellspacing=\"2\" border=\"0\" >\r\n\t\t\t\t\t<tr valign=\"top\">\r\n\t\t\t\t\t\t<td class=\"hub_home_box_table_left\" >\r\n\t\t\t\t\t\t\t<img src=\"{{ img_creation }}\" border=\"0\" />\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t<td class=\"hub_home_box_table_rigth\" >\r\n\t\t\t\t\t\t\t<ul class=\"hub_home_box_list\">\r\n\t\t\t\t\t          <li><a class=\"hub_home_box_link\" accesskey=\"f\" href=\"javascript:hub.showPlugin('{{ labels.HOME_NEW_FEED }}', 0, 0, {type: 'Feeds'});\" title=\"{{ popups.HOME_NEW_FEED }}\">{{ links.HOME_NEW_FEED }}</a>\r\n\t\t\t\t\t          </li>\r\n\t\t\t\t\t          <li><a class=\"hub_home_box_link\" accesskey=\"m\" href=\"javascript:hub.showPlugin('{{ labels.HOME_NEW_MASHUP }}', {{ pluginIds.Mashup }}, 0, {type: 'Mashups', fullscreen: true});\" title=\"{{ popups.HOME_NEW_MASHUP }}\">{{ links.HOME_NEW_MASHUP }}</a>\r\n\t\t\t\t\t          </li>\r\n\t\t\t\t\t          <li><a class=\"hub_home_box_link\" accesskey=\"w\" href=\"javascript:hub.showPlugin('{{ labels.HOME_UPLOAD_WIDGET }}', 0, 0, {type: 'Widgets'});\" title=\"{{ popups.HOME_UPLOAD_WIDGET }}\">{{ links.HOME_UPLOAD_WIDGET }}</a>\r\n\t\t\t\t\t          </li>\r\n\t\t\t\t\t          <li><a class=\"hub_home_box_link\" accesskey=\"s\" href=\"javascript:hub.showPlugin('{{ labels.HOME_UPLOAD_WORKSPACE }}', {{ pluginIds.MashupPage }}, 0, {type: 'pages'} );\" title=\"{{ popups.HOME_UPLOAD_WORKSPACE }}\">{{ links.HOME_UPLOAD_WORKSPACE }}</a>\r\n\t\t\t\t\t          </li>\r\n\t\t\t\t\t\t\t</ul>\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t</tr>\r\n\t\t\t\t</table>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"hub_home_box_footer\"></div>\r\n\t\t</div>\r\n      </td>\r\n    </tr>\r\n    \r\n    <tr>\r\n      <td>\r\n\t\t<div class=\"hub_home_box\">\r\n\t\t\t<div class=\"hub_home_box_header\">\r\n\t\t\t\t<div class=\"hub_home_box_label\">{{ panels.HOME_COMMUNITY_TASKS }}</div>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"hub_home_box_body\">\r\n\t\t\t\t<p class=\"hub_home_box_description\">{{ panels.HOME_COMMUNITY_TASKS_DESC }}</p>\r\n\t\t\t\t<table cellpadding=\"0\" cellspacing=\"2\" border=\"0\" >\r\n\t\t\t\t\t<tr valign=\"top\">\r\n\t\t\t\t\t\t<td class=\"hub_home_box_table_left\">\r\n\t\t\t\t\t\t\t<img src=\"{{ img_community }}\" border=\"0\"/>\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t<td class=\"hub_home_box_table_rigth\">\r\n\t\t\t\t\t\t\t<ul class=\"hub_home_box_list\">\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showPage('{{ labels.HOME_RECENT }}', 'Catalog', {collection: 'ui.all', searchType: '', term: '*', sortBy: 'updated', sortOrder: 'desc'});\" title=\"{{ popups.HOME_RECENT }}\">{{ links.HOME_RECENT }}</a></li>\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showPage('{{ labels.HOME_RATINGS }}', 'Catalog', {collection: 'ui.all', searchType: '', term: '*', sortBy: 'rating', sortOrder: 'desc'});\" title=\"{{ popups.HOME_RATING }}\">{{ links.HOME_RATING }}</a></li>\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showPage('{{ labels.HOME_VIEWS }}', 'Catalog', {collection: 'ui.all', searchType: '', term: '*', sortBy: 'useCount', sortOrder: 'desc'});\" title=\"{{ popups.HOME_VIEWS }}\">{{ links.HOME_VIEWS }}</a></li>\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showPage('{{ labels.HOME_COMMENTS }}', 'Catalog', {collection: 'ui.all', searchType: '', term: '*', sortBy: 'numComments', sortOrder: 'desc'});\" title=\"{{ popups.HOME_COMMENTS }}\">{{ links.HOME_COMMENTS }}</a></li>\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showPage('{{ labels.HOME_TAGS_CATS }}', 'Clouds');\" title=\"{{ popups.HOME_TAGS_CATS }}\">{{ links.HOME_TAGS_CATS }}</a></li>\r\n\t\t\t\t\t\t\t</ul>\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t</tr>\r\n\t\t\t\t</table>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"hub_home_box_footer\"></div>\r\n\t\t</div>\r\n      </td>\r\n      <td>\r\n\t\t<div class=\"hub_home_box\">\r\n\t\t\t<div class=\"hub_home_box_header\">\r\n\t\t\t\t<div class=\"hub_home_box_label\">{{ panels.HOME_START_TASKS }}</div>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"hub_home_box_body\">\r\n\t\t\t\t<p class=\"hub_home_box_description\">{{ panels.HOME_START_TASKS_DESC }}</p>\r\n\t\t\t\t<table cellpadding=\"0\" cellspacing=\"2\" border=\"0\" >\r\n\t\t\t\t\t<tr valign=\"top\">\r\n\t\t\t\t\t\t<td class=\"hub_home_box_table_left\">\r\n\t\t\t\t\t\t\t<img src=\"{{ img_start }}\" border=\"0\"/>\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t<td class=\"hub_home_box_table_rigth\">\r\n\t\t\t\t\t\t\t<ul class=\"hub_home_box_list\">\r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showUrl('{{ labels.HOME_EXAMPLES }}', 'doc/userhelp/editorhelp/examplehelp.html');\" title=\"{{ popups.HOME_EXAMPLES }}\">{{ links.HOME_EXAMPLES }}</a></li> \r\n\t          <li><a class=\"hub_home_box_link\" href=\"javascript:hub.showUrl('{{ labels.HOME_TUTORIALS }}', 'doc/userhelp/tutorials/Tutorials.html');\" title=\"{{ popups.HOME_TUTORIALS }}\">{{ links.HOME_TUTORIALS }}</a></li>\r\n\t<!--            \r\n\t          <li><a class=\"hub_home_box_link\" href=\"#\" title=\"{{ popups.HOME_DEMO }}\">{{ links.HOME_DEMO }}</a></li>\r\n\t-->\r\n\t\t\t\t\t\t\t</ul>\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t</tr>\r\n\t\t\t\t</table>\r\n\t\t\t</div>\r\n\t\t\t<div class=\"hub_home_box_footer\"></div>\r\n\t\t</div>\r\n      </td>\r\n    </tr>\r\n  </table>\r\n</div>\r\n",
    pluginsIds: {},

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the homepage.
     */
    getUrl: function() {
        return hub.urls.getHomePage();
    },

    /**
     * Gets an object indicating this page should be permanent.
     */
    getProperties: function() {
        return {permanent: true};
    },

    getPluginIds: function(plugins){
    	for(plugin in plugins){
//    		alert(plugins[plugin].name + ' = ' + plugins[plugin].id);
    		this.pluginsIds[plugins[plugin].name]=plugins[plugin].id;
    	}

    	return this.pluginsIds;
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
//    	alert(dojo.toJson(server_data.plugins));
//    	alert(dojo.toJson(this.getPluginIds(server_data.plugins)));
        var obj = {logged_in: hub.config.username != '',
            labels: dojo.i18n.getLocalization('hub', 'labels'),
            panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups'),
            pluginIds: this.getPluginIds(server_data.plugins),
            img_catalog: hub.urls.getImage('catalogIcon2.gif'),
            img_creation: hub.urls.getImage('creationIcon2.gif'),
            img_start: hub.urls.getImage('gettingStartedIcon2.gif'),
            img_community: hub.urls.getImage('communityIcon2.gif')};

        // merge server data into context as-is (no name changes)
        // assumes JSONified object from server has properties matching those
        // needed by the template (else, we'd have to manually add them all to
        // obj under the proper names)
        return dojo.mixin(obj, server_data);
    },
    
    onRendered: function(){
    	this.inherited('onRendered', arguments);
    	this.tab.setStatic(true);
    }
});

/**
 * Details page. Fetches metadata about a catalog entry from the server given
 * an object ID to complete the rendering of the template.
 */
dojo.declare("hub.pages.Details", hub.pages._Page, {
    // template to replace node with
    templateString:"<div>\r\n{% if object %}\r\n<div class='TOM_widget_Sidebar'>\r\n  <div class='panel'>\r\n  \t<div class=\"panelHeader\">\r\n    \t<h3>{{ panels.DETAILS_ACTIONS_TEXT }}</h3>\r\n    </div>\r\n    \r\n    <ul class=\"TOM_admin_action_list\">\r\n{% if object.isSaved %}\r\n\t{% if object.isPage %}\r\n\t\t{% if addtomashupsURL %}\r\n\t<li>\r\n<a href='{{ secureURL }}' title=\"{{ popups.ACTION_DOWNLOAD_PAGE }}\" >{{ links.ACTION_DOWNLOAD_PAGE_LINK }}</a>\r\n\t</li>\r\n\t\t{% endif %}\r\n\t{% endif %}\r\n\t\r\n\t{% if object.isWidget %}\r\n\t\t{% if pluginData.definitionUrl %}\r\n\t\t{% else %}\r\n\t\t{% if object.hasPromptParams %}\r\n<li><a href='{{ secureURL }}' title=\"{{ popups.ACTION_VIEW_WIDGET }}\" >{{ links.ACTION_VIEW_WIDGET_LINK }}</a></li>\r\n\t\t{% else %}\r\n<li><a href=\"{{ secureURL }}\" target=\"_blank\" title=\"{{ popups.ACTION_VIEW_WIDGET }}\" >{{ links.ACTION_VIEW_WIDGET_LINK }}</a></li>\r\n\t\t\t{% endif %}\r\n\t\t{% endif %} \r\n\t{% else %}\r\n\t\t{% if object.isPage %}\r\n\t\t\t{% if addtomashupsURL %}\r\n\t\t\t\t{% if user %}\r\n\t\t\t\t\t{% if user.isBanned %}\r\n\t\t\t\t\t{% else %}\r\n<li><a href='{{ addtomashupsURL }}' title=\"{{ popups.ACTION_ADD_TO_MASHUP }}\">{{ links.ACTION_ADD_TO_MASHUPS }}</a></li>\r\n\t\t\t\t\t{% endif %}\r\n\t\t\t\t{% endif %}\r\n\t\t\t{% else %}\r\n<li><a href=\"{{ pluginData.definitionUrl }}\" target=\"_blank\" title=\"{{ popups.ACTION_VIEW_PAGE }}\" >{{ links.ACTION_VIEW_PAGE_LINK }}</a></li>\t\t\t\t\r\n\t\t\t{% endif %}\r\n\t\t{% else %}\r\n\t\t\t{% if object.hasPromptParams %}\r\n<li><a href='{{ secureURL }}' title=\"{{ popups.ACTION_VIEW_FEED }}\" >{{ links.ACTION_VIEW_FEED_LINK }}</a></li>\r\n\t\t\t{% else %}\r\n<li><a href=\"{{ secureURL }}\" target=\"_blank\" title=\"{{ popups.ACTION_VIEW_FEED }}\" >{{ links.ACTION_VIEW_FEED_LINK }}</a></li>\r\n\t\t\t{% endif %}\r\n\t\t{% endif %} \r\n\t{% endif %} \r\n\t\r\n{% endif %}\r\n\r\n{% if user %}\r\n\t{% if object.isSaved %}\r\n\t\t{% if user.isBanned %}\r\n\t\t{% else %}\r\n\t\t\t{% if object.isPage %}\r\n\t\t\t{% else %}\r\n\t\t\t\t{% if addtomashupsURL %}\r\n\t<li>\r\n\t\t<a href='{{ addtomashupsURL }}' title=\"{{ popups.ACTION_ADD_TO_MASHUP }}\">{{ links.ACTION_ADD_TO_MASHUPS }}</a>\r\n\t</li>\r\n\t\t\t\t{% endif %}\r\n\t\t\t{% endif %}\r\n\t\t{% endif %}\r\n\t{% endif %}\r\n{% endif %}\r\n\r\n{% if user %}\r\n\t{% if object.isSaved %}\r\n\t\t{% if user.isBanned %}\r\n\t\t\t{% else %}\r\n\t\t\t{% if user.isOwner %}\r\n\t\t\t\t{% else %}\r\n\t<li>\r\n\t{% if object.isInFavorites %}\r\n<a id=\"favoritesMessage_{{ object.id }}\" href='javascript:hub.submit.ajaxSubmit(new hub.submit.edit.FavoritesForm(null, \"favoritesMessage_{{ object.id }}\", hub.urls.postFavorites(\"{{ object.id }}\")), true, true);'>{{ links.ACTION_REMOVE_FAVORITE_LINK }}</a>\r\n\t{% else %}\r\n<a id=\"favoritesMessage_{{ object.id }}\" href='javascript:hub.submit.ajaxSubmit(new hub.submit.edit.FavoritesForm(null, \"favoritesMessage_{{ object.id }}\", hub.urls.postFavorites(\"{{ object.id }}\")), true, true);'>{{ links.ACTION_ADD_FAVORITE_LINK }}</a>\r\n\t{% endif %} \r\n\t</li>\r\n\t\t\t{% endif %}\r\n\t\t{% endif %}\r\n\t{% endif %}\r\n{% endif %}\r\n\r\n<!-- \r\n{% if user %}\r\n\t{% if object.isSaved %}\r\n      <li>\r\n      \t<a href='javascript:hub.showPage(\"{{ object.title }}\",\"ReportAbuse\",{\"entryId\": \"{{ object.id }}\", \"title\": \"{{ object.title }}\"})' title=\"{{ popups.ACTION_REPORT_ABUSE }}\">{{ links.ACTION_REPORT_ABUSE_LINK }}</a>\r\n      </li>\r\n\t{% endif %}\r\n{% endif %}\r\n-->\r\n\r\n<!-- \r\n{% if user %}\r\n\t{% if object.isSaved %}\r\n\t\t{% if object.isFeed %}\r\n      <li>\r\n      <a href='#' title=\"{{ popups.ACTION_USE_IN_MASHUP }}\">{{ links.ACTION_USE_IN_MASHUP_LINK }}</a>\r\n      </li>\r\n\t\t{% endif %}\r\n\t{% endif %}\r\n{% endif %}\r\n -->\r\n \r\n{% if user.canEdit %}\r\n\t{% if object.isTrackedPerm %}\r\n      <li><a href=\"{{ trackedDataURL }}\" target=\"_blank\" title=\"{{ links.ACTION_VIEW_TRACKED_FEED_HELP }}\">{{ links.ACTION_VIEW_TRACKED_FEED_LINK }}</a></li>\r\n\t{% endif %}\r\n      <li><a href='javascript:hub.showPlugin(\"{{ object.title }}\", {{ object.pluginid }}, {{ object.id }}, {\"current\": true});' title=\"{{ popups.ACTION_EDIT_INFO }}\">{{ links.ACTION_EDIT_INFO_LINK }}</a></li>\r\n\t{% if object.isFeed %}\r\n      <li><a href='javascript:hub.showPlugin(\"{{ object.title }}\", {{ object.pluginid }}, {{ object.id }}, {\"editSource\": true, \"current\": true});' title=\"{{ popups.ACTION_EDIT_FEED }}\">{{ links.ACTION_EDIT_FEED_LINK }}</a></li>\r\n\t{% endif %}      \r\n\t{% if object.isWidget %}\r\n      <li><a href='javascript:hub.showPlugin(\"{{ object.title }}\", {{ object.pluginid }}, {{ object.id }}, {\"editSource\": true, \"current\": true});' title=\"{{ popups.ACTION_EDIT_WIDGET }}\">{{ links.ACTION_EDIT_WIDGET_LINK }}</a></li>\r\n\t{% endif %}      \r\n\t{% if object.isMashup %}\r\n      <li><a href='javascript:hub.showPlugin(\"{{ object.title }}\", {{ object.pluginid }}, {{ object.id }}, {\"editSource\": true, \"current\": true, \"fullscreen\": true});' title=\"{{ popups.ACTION_EDIT_FLOW }}\">{{ links.ACTION_EDIT_FLOW_LINK }}</a></li>\r\n\t{% endif %}      \r\n\t{% if object.isPage %}\r\n      <li><a href='javascript:hub.showPlugin(\"{{ object.title }}\", {{ object.pluginid }}, {{ object.id }}, {\"editSource\": true, \"current\": true});' title=\"{{ popups.ACTION_EDIT_PAGE }}\">{{ links.ACTION_EDIT_PAGE_LINK }}</a></li>\r\n\t{% endif %}  \r\n{% endif %}\t\r\n\r\n{% if user.canRead %}\r\n    {% if object.isMashup %}\r\n       {% if object.isSaved %}\r\n\t      {% if user.isBanned %}\r\n\t\t     {% else %}\r\n      <li><a href='javascript:hub.showPlugin(\"{{ object.title }}\", {{ object.pluginid }}, {{ object.id }}, {\"editSource\": true, \"current\": true, \"fullscreen\": true, \"editAsNew\": true});' title=\"{{ popups.ACTION_EDIT_FLOW_AS_NEW }}\">{{ links.ACTION_EDIT_FLOW_AS_NEW_LINK }}</a></li>\r\n\t      {% endif %} \r\n\t   {% endif %}\r\n\t{% endif %} \r\n{% endif %}\t\r\n\t\r\n{% if user.canEdit %}\t    \r\n      <li><a id=\"hub.del.link{{ object.id }}\" href='javascript:hub.submit.ajaxSubmit(new hub.submit.edit.DeleteConfirm(null, \"hub.del.link{{ object.id }}\", hub.urls.postDelete(\"{{ object.id }}\", \"{{ object.pluginid }}\")), true, false);' title=\"{{ popups.ACTION_DELETE }}\">{{ links.ACTION_DELETE_LINK }}</a></li>\r\n{% endif %}\r\n\r\n\t</ul>\r\n  </div>\r\n</div>\r\n\r\n<div class='TOM_panel TOM_admin_panel' id=\"TOM_details_{{ object.id }}\">\r\n<div>\r\n{% if object.isFeed %}\r\n\t<h1 style='overflow:hidden;text-overflow:ellipsis;'><img src='{{ icon_feed }}' />&nbsp;{{ title }}</h1>\r\n{% endif %}      \r\n{% if object.isWidget %}\r\n\t<h1 style='overflow:hidden;text-overflow:ellipsis;'><img src='{{ icon_widget }}' />&nbsp;{{ title }}</h1>\r\n{% endif %}      \r\n{% if object.isMashup %}\r\n\t<h1 style='overflow:hidden;text-overflow:ellipsis;'><img src='{{ icon_flow }}' />&nbsp;{{ title }}</h1>\r\n{% endif %}   \r\n{% if object.isPage %}\r\n\t<h1 style='overflow:hidden;text-overflow:ellipsis;'><img src='{{ icon_page }}' />&nbsp;{{ title }}</h1>\r\n{% endif %}\r\n</div>\r\n   \r\n<div class='rightCol' style='margin-left:3.5em;overflow:hidden;text-overflow:ellipsis;'>   \r\n  <p class=\"description\">{{ object.description }}</p>\r\n  <p class=\"generate\">\r\n\t<a href=\"{{ object.secureURL }}\" target=\"_blank\">{{ object.externalURL }}</a>\r\n  </p>\r\n</div>\r\n  \r\n  <h2>{{ panels.DETAILS_DETAILS_TITLE }}</h2>\r\n  \r\n  <table cellspacing=\"10\" border=\"0\" class='TOM_details_table'>\r\n    <tr>\r\n    \t<td class='label' >{{ panels.DETAILS_SOURCE_TEXT }}</td>\r\n    \t<td>{{ object.source }}&nbsp;</td>\r\n  \t\t<td class='right-label' >{{ panels.DETAILS_CREATOR_TEXT }}</td>\r\n    \t<td ><a href='javascript:hub.showPage(\"{{ object.creator }}\",\"Catalog\",{\"collection\": \"all\", \"searchType\": \"user\", \"term\": \"{{ object.creator }}\"})'>{{ object.creator }}</a></td>\r\n    </tr>\r\n    <tr>\r\n    \t<td class='label'>{{ panels.DETAILS_VERSION_TEXT }}</td>\r\n    \t<td>{{ object.version }}</td>\r\n  \t\t<td class='right-label'>{{ panels.DETAILS_AVERAGE_RATING_TEXT }}</td>\r\n    \t<td>\r\n    \t\t<div objectid=\"{{ object.id }}\" dojoType=\"hub.data.social.AverageRatingModel\" id=\"hub.avg.{{ object.id }}\" forceRefresh=\"true\" ></div>\r\n\t        <div dojoType=\"hub.widgets.social.view.SocialBox\" model=\"hub.avg.{{ object.id }}\" >\r\n\t\t\t\t<div id=\"hub_details_total_rating_{{ object.id }}\" dojoType=\"hub.widgets.social.view.RatingStarView\" objectid=\"{{ object.id }}\" ></div>\r\n\t\t\t</div>\r\n\t\t</td>\r\n    </tr>\r\n    <tr>\r\n    \t<td class='label'>{{ panels.DETAILS_DOC_TEXT }}</td>\r\n    \t<td>{% if object.doc %}{{ object.doc }}{% else %}{{ panels.NONE }}{% endif %}</td>\r\n  \t\t<td class='right-label'>{{ panels.DETAILS_MY_RATING_TEXT }}</td>\r\n    \t<td>\r\n    \t\t{% if user %}\r\n\t\t\t\t<div id=\"hub_details_user_rating_{{ object.id }}\" dojoType=\"hub.widgets.social.edit.RatingChooser\" objectid=\"{{ object.id }}\" value=\"{{ rating.value }}\" formid=\"hub_det_rating_form_{{ object.id }}\"></div>\r\n\t\t\t\t<form id=\"hub_det_rating_form_{{ object.id }}\" name=\"hub_det_rating_form_{{ object.id }}\" style=\"display: none;\"><input type=\"hidden\" name=\"value\" value=\"{{ rating.value }}\"/></form>\r\n\t\t\t{% else %}\r\n\t\t\t   \t<a href=\"{{ loginURL }}\">{{ panels.DETAILS_LOGIN_TO_RATE_LINK }}</a>\r\n\t\t\t{% endif %}&nbsp;\r\n\t\t</td>\r\n    </tr>\r\n    <tr>\r\n    \t<td class='label'>{{ panels.DETAILS_TIMES_ACCESSED_TEXT }}</td>\r\n    \t<td>{{ object.trackedhits }}</td>\r\n  \t\t<td class='right-label'>{{ panels.DETAILS_CATEGORIES_TEXT }}</td>\r\n    \t<td>\r\n\t    \t{% if categories %}\r\n\t\t\t\t{% for c in categories %}\r\n\t\t\t          <a href=\"javascript:hub.showPage('{{ c.name }}', 'Catalog', {'collection':'all', searchType:'category', 'term':'{{ c.name }}'});\">{{ c.name }}</a>\r\n\t\t\t\t{% endfor %}\r\n\t\t\t{% else %}\r\n\t\t\t\t{{ panels.NONE }}\r\n\t\t\t{% endif %}\r\n    \t</td>\r\n    </tr>\r\n    <tr>\r\n    \t<td class='label'>{{ panels.DETAILS_LAST_UPDATED_TEXT }}</td>\r\n    \t<td>{{ object.date }}</td>\r\n  \t\t<td class='right-label'>{{ panels.DETAILS_RELATED_TEXT }}</td>\r\n    \t<td>\r\n\t\t\t{% if related %}\r\n\t\t\t\t{% for r in related %}\r\n\t\t\t          <a href=\"javascript:hub.showPage('{{ r.name }}', 'Details', {'object_id':'{{ r.id }}'});\">{{ r.name }}</a>\r\n\t\t\t\t{% endfor %}\r\n\t\t\t{% else %}\r\n\t\t\t\t{{ panels.NONE }}\r\n\t\t\t{% endif %}\r\n    \t</td>\r\n    </tr>\r\n    <tr>\r\n    \t<td class='label'>{{ panels.DETAILS_SAVED_TEXT }}</td>\r\n    \t<td>{% if object.isSaved %}{{ panels.YES }}{% else %}{{ panels.NO }}{% endif %}</td>\r\n  \t\t<td class='right-label'>{{ panels.DETAILS_TAGS_TEXT }}</td>\r\n    \t<td>\r\n    \t\t<div dojoType=\"hub.data.social.TagsModel\" id=\"hub.det.tags.{{ object.id }}\" forceRefresh=\"true\" objectid=\"{{ object.id }}\" ></div>\r\n\t\t\t<div dojoType=\"hub.widgets.social.view.SocialBox\" model=\"hub.det.tags.{{ object.id }}\" >\r\n\t\t\t\t<div id=\"hub_details_tags_{{ object.id }}\" class=\"TOM_widget_TagsBox\" dojoType=\"hub.widgets.social.view.TagsCountView\" ></div>\r\n\t\t\t</div>\r\n\t\t\t&nbsp;\r\n    \t</td>\r\n    </tr>\r\n    <tr>\r\n    \t<td class='label'>{{ panels.DETAILS_PERMISSIONS_TEXT }}</td>\r\n    \t<td>\r\n    \t{% if user %}\r\n    \t\t{% if user.canRead %}\r\n    \t\t\t{{ panels.PERMISSION_READ }}\r\n    \t\t{% endif %}\r\n    \t\t{% if user.canRead %}\r\n    \t\t\t{% if user.canEdit %}, \r\n    \t\t\t{% endif %}\r\n    \t\t{% endif %}\r\n    \t\t{% if user.canEdit %}\r\n    \t\t\t{{ panels.PERMISSION_EDIT }}\r\n    \t\t{% endif %}\r\n    \t{% else %}\t\r\n    \t\t{% if guestCanRead %}\r\n    \t\t\t{{ panels.PERMISSION_READ }}\r\n    \t\t{% endif %}\r\n    \t\t{% if guestCanRead %}\r\n    \t\t\t{% if guestCanEdit %}, \r\n    \t\t\t{% endif %}\r\n    \t\t{% endif %}\r\n    \t\t{% if guestCanEdit %}\r\n    \t\t\t{{ panels.PERMISSION_EDIT }}\r\n    \t\t{% endif %}\r\n   \t\t{% endif %}\r\n    \t&nbsp;\r\n    \t</td>\r\n  \t\t<td class='right-label'>{{ panels.DETAILS_MY_TAGS_TEXT }}</td>\r\n    \t<td>\r\n\t\t\t{% if user %}\r\n\t\t\t\t{% if user.isBanned %}\r\n\t\t\t\t{% else %}\r\n\t\t\t\t\t\t<div dojoType=\"hub.widgets.social.view.SocialBox\" model=\"hub.tags\" class=\"TOM_widget_TagsBox\" showRefresh=\"false\" forceRefresh=\"false\">\r\n\t\t\t\t\t\t\t<div id=\"hub_details_tags_suggest_{{ object.id }}\" dojoType=\"hub.widgets.social.edit.TagSuggest\" objectid=\"{{ object.id }}\" value=\"{{ mytags }}\"></div>\r\n\t\t\t\t\t\t</div>\r\n\t\t\t\t{% endif %}\r\n\t\t\t{% else %}\r\n\t\t\t        \t<a href=\"{{ loginURL }}\">{{panels.DETAILS_LOGIN_TO_TAG_LINK }}</a>\r\n\t\t\t{% endif %}\r\n    \t</td>\r\n    </tr>\r\n  </table>\r\n\r\n  <h2>{{ panels.DETAILS_COMMENTS_TITLE }}</h2>\r\n\r\n\t<div objectid=\"{{ object.id }}\" dojoType=\"hub.data.social.CommentsModel\" id=\"hub.cmtn.{{ object.id }}\"></div>\r\n\t<div dojoType=\"hub.widgets.social.view.SocialBox\" model=\"hub.cmtn.{{ object.id }}\" forceRefresh=\"true\" >\r\n\t\t<div id=\"TOM_details_comments_{{ object.id }}\" class=\"TOM_widget_CommentBox\" dojoType=\"hub.widgets.social.view.CommentsListView\"></div>\r\n\t</div>\r\n\t\r\n\t\r\n  <h2>{{ panels.DETAILS_ADD_COMMENT_TITLE }}</h2>\r\n\r\n{% if user %}\r\n\t{% if user.isBanned %}\r\n\t{% else %}\r\n\t<form id=\"hub_details_user_comment_{{ object.id }}\" onsubmit=\"return false;\">\r\n\t\r\n\t  <table cellspacing=\"5\" border=\"0\" class='TOM_details_table'>\r\n\t    <tr>\r\n\t    \t<td class='label' ><span class='required'>*</span> {{ panels.DETAILS_COMMENT_TITLE_TEXT }}</td>\r\n\t    \t<td ><input style='width:23.5em;' name='commentTitle' maxlength='200' type='text' title=\"{{ panels.DETAILS_COMMENT_TITLE_HELP }}\" /></td>\r\n\t    </tr><tr>\t\r\n\t  \t\t<td class='label' ><span class='required'>*</span> {{ panels.DETAILS_COMMENT_BODY_TEXT }}</td>\r\n\t    \t<td ><textarea name='commentText' rows='5' cols='47' maxlength='1000' type='textarea' title=\"{{ panels.DETAILS_COMMENT_BODY_HELP }}\"></textarea></td>\r\n\t    </tr><tr>\t\r\n\t  \t\t<td class='label' ></td>\r\n\t    \t<td >\r\n\t    \t\t<button type=\"submit\" dojoType=\"hub.widgets.Button\" id=\"postCommentButton_{{ object.id }}\">{{ panels.DETAILS_COMMENT_SUBMIT_LABEL }}</button>\r\n\t\t        <br />\r\n\t\t        <span id=\"saveCommentMessage_{{ object.id }}\"></span>\r\n\t    \t</td>\r\n\t    </tr>\r\n      </table> \r\n    </form>\r\n\t{% endif %}        \r\n{% else %}\r\n\t<a style=\"font-size:0.8em;\" href=\"{{ loginURL }}\">{{ panels.DETAILS_LOGIN_TO_COMMENT_LINK }}</a>\r\n{% endif %}   \r\n    \r\n</div>\r\n{% else %}\r\n\t\t\t<span class='error'>{{ result }}</span>\r\n{% endif %}\r\n\r\n</div>\r\n",
    // show details of this item
    object_id: null,

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getDetailsPage(this.object_id);
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
    	this.tab.entryId = this.object_id;
        
        
        
        
        
        
        

		var gurl = "#";
		var orig_title = ""; 
		
		if (server_data.object) {
			// hub.escapeQuotes
			if (server_data.object.title){
				orig_title = server_data.object.title;
				server_data.object.title = hub.escapeQuotes(server_data.object.title);
			}
			if (server_data.object.externalURL) {
				server_data.object.secureURL = hub.urls.secureUrl(server_data.object.externalURL);
			}
			if (server_data.object.hasPromptParams) {
				gurl = 'javascript:hub.showUrl("Parameters","' + hub.urls.getParametersPage(server_data.object.id, server_data.object.pluginid) + '", {entryId: '+server_data.object.id+'});';
			} else {
//				gurl = hub.urls.getPluginGenerate(server_data.object.id, server_data.object.pluginid);
				gurl = server_data.object.externalURL ;
			}
			if ( gurl == "#" ) {
				gurl = server_data.object.url;
			}
		}
        var obj = {
        	objectid: this.object_id,
        	baseurl: hub.config.base_url,
        	generateURL: gurl,
        	secureURL: hub.urls.secureUrl(gurl),
        	trackedDataURL: hub.config.base_url+"/tracking/generate/entryid/"+this.object_id,
			addtomashupsURL: server_data.object?mm.createAddToMashupsLink(server_data):"", //Add To Mashups link
			title: orig_title,
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups'),
            icon_feed: hub.urls.getImage('feedLarge.gif'),
            icon_widget: hub.urls.getImage('widgetLarge.gif'),
            icon_flow: hub.urls.getImage('mashupLarge.gif'),
            icon_page: hub.urls.getImage('pageLarge.gif')
            };

        return dojo.mixin(obj, server_data);
    },
    
    submitComment: function(){
		var form = dojo.byId("hub_details_user_comment_" + this.object_id);
		var msg = dojo.byId("saveCommentMessage_" + this.object_id);
		hub.submit.ajaxSubmit(new hub.submit.social.Comment(form, msg, this.object_id));
    },

    onRendered: function(){
    console.log('onRendered');
    	this.inherited('onRendered', arguments);
		dojo.connect(dijit.byId('postCommentButton_'+this.object_id), 'onClick', dojo.hitch(this, this.submitComment));
    	console.dir({button: dijit.byId('postCommentButton_'+this.object_id)});
    }
});

/**
 * Page with a form accepting parameters to generate a feed. Fetches parameter
 * names, types, default values, etc. from the server.
 */
dojo.declare("hub.pages.Parameters", hub.pages._Page, {
    // template to replace node with
    templateString:"",
    // parameters for this item
    object_id: null,

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getParametersPage(this.object_id);
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    }
});
dojo.declare('hub.pages.Save', hub.pages._Page, {
    // template to replace node with
    templateString:"<div class='TOM_panel'>\r\n  <form id='saveEntry_{{ entry_id }}' onsubmit=\"alert(this);\">\r\n      <h1>{{ panels.SAVE_TITLE }}</h1>\r\n\t       \r\n      <label for='title'><span class='required'>*</span> {{ panels.SAVE_TITLE_LABEL }}</label>\r\n      <div class=\"rightCol\">\r\n        <input name='title' style=\"width: 32em;\" maxlength='200' type='text' value=\"{{ title|default:'' }}\" title=\"{{ popups.SAVE_TITLE }}\"/><br />\r\n      </div>\r\n      \r\n      <label for='description'><span class='required'>*</span> {{ panels.SAVE_DESCRIPTION_LABEL }}</label>\r\n      <div class=\"rightCol\">\r\n        <textarea name='description' style=\"width: 32em;\" maxlength='1000' title=\"{{ popups.SAVE_DESCRIPTION }}\">{{ description }}</textarea>\r\n      </div>\r\n      \r\n      <label for='version'><span class='required'>*</span> {{ panels.SAVE_VERSION_LABEL }}</label>\r\n      <div class=\"rightCol\">\r\n        <input name='version' style=\"width: 32em;\" maxlength='20' type='text' value=\"{{ version|default:'1.0' }}\" title=\"{{ popups.SAVE_VERSION }}\" /><br />\r\n      </div>\r\n      \r\n      <label for='tags'>{{ panels.SAVE_TAGS_LABEL }}</label>\r\n      <div class=\"rightCol\">\r\n\t\t<div dojoType=\"hub.widgets.social.view.SocialBox\" model=\"hub.tags\" class=\"TOM_widget_TagsBox\" showRefresh=\"false\">\r\n\t\t\t<div inputName='tags' value=\"{{ tags }}\" dojoType=\"hub.widgets.social.edit.TagSuggestLite\" objectid=\"{{ entry_id }}\" ></div>\r\n\t\t</div>\r\n      </div>\r\n      \r\n      \r\n      <label for='tags'>{{ panels.SAVE_PERMISSIONS_LABEL }}</label>\r\n      <div class=\"rightCol\" dojoType=\"hub.widgets.edit.AclAssign\" entryid=\"{{ entry_id }}\"></div>\r\n\r\n      <label>{{ panels.SAVE_ADVANCED_LABEL }}</label>\r\n      <div class=\"rightCol\">\r\n\t    <table style=\"padding: 0px; border: 0px none; margin: 0px;\"><tr><td>\r\n\t      <div style=\"width: 32em;\" dojoType=\"dijit.TitlePane\" title=\"{{ panels.SAVE_CATEGORIES_LABEL }}\" open=\"false\">\r\n\t\t      <div dojoType=\"hub.widgets.adminobjectscatalog.Categories\" entryid=\"{{ entry_id }}\"></div>\r\n\t      </div>    \r\n\t\r\n\t      <div style=\"width: 32em;\" dojoType=\"dijit.TitlePane\" title=\"{{ panels.SAVE_RELATED_LABEL }}\" open=\"false\">\r\n\t\t      <div dojoType=\"hub.widgets.adminobjectscatalog.Related\" entryid=\"{{ entry_id }}\"></div>\r\n\t      </div>    \r\n\t\r\n\t      <div style=\"width: 32em;\" dojoType=\"dijit.TitlePane\" title=\"{{ panels.SAVE_DOCUMENTATION_LABEL }}\" open=\"false\">\r\n\t\t\t\t<label class=\"unsized\" for=\"docurl\">{{ panels.SAVE_DOCUMENTATION_URL }}</label>\r\n\t\t\t\t<input type=\"text\" title=\"{{ popups.SAVE_DOC_URL }}\" value=\"{{ doc }}\" maxlength=\"100\" name=\"docurl\" style=\"width: 25em;\"/>\r\n\t      </div>    \r\n\r\n\t      <div style=\"width: 32em;\" dojoType=\"dijit.TitlePane\" title=\"{{ panels.SAVE_CACHE_LABEL }}\" open=\"false\">\r\n{% if useCache %}\r\n\t\t\t\t<label class=\"unsized\" for=\"usecache\">{{ panels.SAVE_CACHE_USE_LABEL }}</label>\r\n\t\t\t\t<input type=\"checkbox\" title=\"{{ popups.SAVE_CACHE_USE }}\" name=\"usecache\" onChange=\"if(this.checked){this.form.lifetime.disabled=false;}else{this.form.lifetime.disabled=true;}\" checked=\"checked\" />\r\n\t\t\t\t<br />\r\n\t\t\t\t<br />\r\n\t\t\t\t<label class=\"unsized\" for=\"lifetime\">{{ panels.SAVE_CACHE_LIFETIME_LABEL }}</label>\r\n\t\t\t\t<br />\r\n\t\t\t\t<input type=\"text\" title=\"{{ popups.SAVE_CACHE_LIFETIME }}\" value=\"{{ lifeTime }}\" maxlength=\"20\" name=\"lifetime\" style=\"width: 9em;\"/>\r\n{% else %}\r\n\t\t\t\t<label class=\"unsized\" for=\"usecache\">{{ panels.SAVE_CACHE_USE_LABEL }}</label>\r\n\t\t\t\t<input type=\"checkbox\" title=\"{{ popups.SAVE_CACHE_USE }}\" name=\"usecache\" onChange=\"if(this.checked){this.form.lifetime.disabled=false;}else{this.form.lifetime.disabled=true;}\" />\r\n\t\t\t\t<br />\r\n\t\t\t\t<br />\r\n\t\t\t\t<label class=\"unsized\" for=\"lifetime\">{{ panels.SAVE_CACHE_LIFETIME_LABEL }}</label>\r\n\t\t\t\t<br />\r\n\t\t\t\t<input type=\"text\" title=\"{{ popups.SAVE_CACHE_LIFETIME }}\" value=\"\" maxlength=\"20\" name=\"lifetime\" style=\"width: 9em;\" disabled=\"disabled\" />\r\n{% endif %}\r\n\t      </div> \r\n\t      </td></tr></table>  \r\n\t         \r\n      </div>\r\n      <div class=\"rightCol\">\r\n        <p id=\"saveEntryMessage_{{ id }}\">{{ message|default\"\" }}</p>\r\n      </div>\r\n  </form>\r\n</div>\r\n",

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
        return {id: 'object_id', legend: 'legend', name: 'empty', description: '', version: '0.5', tags: 'tags',
		        panels: dojo.i18n.getLocalization('hub', 'panels'),
                labels: dojo.i18n.getLocalization('hub', 'labels')};
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
//        return hub.urls.getSavePage();
		return null;
    }
});

/**
 * Settings page.
 */
dojo.declare("hub.pages.AdminTables", hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n<div class='TOM_panel TOM_admin_panel'>\r\n\t<form id='admin_initForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.InitForm(\"admin_initForm\", \"errorMessageTables\", hub.urls.postAdminInit()), false, true);'>\r\n\t\t<div >\r\n\t\t\t<h1>{{panels.ADMIN_INIT_TITLE}}</h1>\r\n\t\t\t<h3>{{panels.ADMIN_INIT_HEAD}}</h3>\r\n\r\n\t\t\t<input id='admin_clearData' name='clearData' type='checkbox'/>\r\n\t\t\t<span style=\"font-size:0.8em;\">{{panels.ADMIN_INIT_CHECKBOX}}  <font color='red'>{{panels.ADMIN_INIT_WARNING}}</font></span><br>\r\n\t\t</div>\r\n\t\t\r\n\t\t<br /><br /><br />\r\n\t\t<div>\r\n\t\t\t<input id='admin_typeSubmit' class='submit' type='submit' value=\"{{panels.ADMIN_INIT_BUTTON_VALUE}}\"/>\r\n\t\t\t<span id='errorMessageTables' class='error'></span>\r\n\t\t</div>\r\n\t</form>\r\n</div>\r\n{% endblock %}\r\n",

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return null;
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        
        
        

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Page for reinitializing the database and sample data. No server data to 
 * fetch.
 */
dojo.declare('hub.pages.AdminMigrateDatabase', hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n<div class=\"TOM_panel TOM_admin_panel\">\r\n  <h1>{{panels.MIGRATE_TITLE}}</h1>\r\n  <h3>{{panels.MIGRATE_INSTRUCTION}}</h3>\r\n  <form id=\"admin_migrate_form\" method=\"POST\" onsubmit=\"return hub.submit.ajaxSubmit(new hub.submit.admin.MigrateForm('admin_migrate_form', 'migrateStatus', hub.urls.postAdminMigrateDB()), false, true);\">\r\n    \r\n    <div>\r\n      <label id=\"db_name_label\" for=\"db_driver\">{{labels.MIGRATE_DB_TYPE}}</label>\r\n      <select tabindex=\"1\" onchange=\"_selectDbDriver(this);\" name=\"db_driver\" id=\"db_driver\" dojoAttachPoint=\"db_driver\">\r\n        <option value=\"DB2\" selected=\"selected\">DB2</option>\r\n        <option value=\"IDS\">Informix</option>\r\n        <!-- <option value=\"DERBY\">Derby Network Server</option> -->\r\n        <!-- <option value=\"DERBY_EMBEDDED\">Embedded Derby</option> -->\r\n      </select>\r\n    </div>\r\n        \r\n    \r\n    <div id=\"db_host_div\" class=\"right-column\" title=\"{{popups.MIGRATE_HOSTNAME_TOOLTIP}}\">\r\n      <label id=\"db_name_label\" for=\"db_host\">{{labels.MIGRATE_HOSTNAME}}</label>\r\n      <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_host\" id=\"db_host\" value=\"localhost\"/>\r\n    </div>\r\n        \r\n    <div id=\"db_port_div\" class=\"right-column\" title=\"{{popups.MIGRATE_PORT_TOOLTIP}}\"> \r\n      <label id=\"db_port_label\" for=\"db_port\">{{labels.MIGRATE_PORT}}</label>\r\n      <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_port\" id=\"db_port\" value=\"\"/>\r\n    </div>\r\n        \r\n    \r\n    <div id=\"db_name_div\" class=\"right-column\" title=\"{{popups.MIGRATE_DB_NAME_TOOLTIP}}\">\r\n      <label id=\"db_name_label\" for=\"db_name\">{{labels.MIGRATE_DB_NAME}}</label>\r\n      <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_name\" id=\"db_name\" value=\"hubdb\" />\r\n    </div>\r\n                            \r\n    \r\n    <div id=\"db_schema_div\" class=\"right-column\" title=\"{{popups.MIGRATE_SCHEMA_TOOLTIP}}\">\r\n      <label id=\"db_schema_label\" for=\"db_schema\">{{labels.MIGRATE_SCHEMA}}</label>\r\n      <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_schema\" value=\"\" id=\"db_schema\"/>\r\n    </div>\r\n        \r\n    \r\n    <div class=\"right-column\" title=\"{{popups.MIGRATE_USERNAME_TOOLTIP}}\">\r\n      <label id=\"db_username_label\" for=\"db_username\">{{labels.MIGRATE_USERNAME}}</label>\r\n      <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_username\" id=\"db_username\" value=\"\" />\r\n    </div>\r\n        \r\n    <div class=\"right-column\" title=\"{{popups.MIGRATE_PASSWORD_TOOLTIP}}\">\r\n      <label id=\"db_password_label\" for=\"db_password\">{{labels.MIGRATE_PASSWORD}}</label>\r\n      <input tabindex=\"1\" type=\"password\" size=\"40\" name=\"db_password\" id=\"db_password\" value=\"\" />\r\n    </div>\r\n    \r\n    <div>\r\n      <input id='admin_typeSubmit' class='submit' type='submit' value=\"{{panels.MIGRATE_SUBMIT_BUTTON_LABEL}}\"/>\r\n    </div>\r\n  </form>\r\n  <p id=\"migrate_status\"></p>\r\n  <script type=\"text/javascript\">\r\n      function _selectDbDriver() {\r\n        var driver = dojo.byId('db_driver');\r\n        var hostname = dojo.byId('db_host_div');\r\n        var schema = dojo.byId('db_schema_div');\r\n        var port = dojo.byId('db_port_div');\r\n        \r\n        switch(driver.value){\r\n            case hub.config.derby_driver:\r\n                hostname.style.visibility='hidden';\r\n                schema.style.visibility='hidden';\r\n                port.style.visibility='hidden';\r\n                break;\r\n            case hub.config.embedded_derby_driver:\r\n                hostname.style.visibility='visible';\r\n                schema.style.visibility='hidden';\r\n                port.style.visibility='visible';\r\n                port.value = hub.config.derby_port;\r\n                break;\r\n            case hub.config.ids_driver:\r\n                hostname.style.visibility='visible';\r\n                schema.style.visibility='hidden';\r\n                port.style.visibility='visible';\r\n                port.value = hub.config.ids_port;\r\n                break;\r\n            case hub.config.db2_driver:\r\n            default:\r\n                hostname.style.visibility='visible';\r\n                schema.style.visibility='visible';\r\n                port.style.visibility='visible';\r\n                port.value = hub.config.db2_port;\r\n        }\r\n      }\r\n  </script>\r\n</div>\r\n{% endblock %}\r\n",

    /**
     * Imports required modules and build an object with properties matching 
     * fields in the template.
     */
    getContext: function(server_data) {
        
        
        
        
        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
            panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Page for deleting arbitrary items. Fetches name, id, etc. of all items.
 */
dojo.declare('hub.pages.AdminObjects', hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n    <div class='TOM_panel TOM_admin_panel'>\r\n      <form id='admin_deletionForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.ObjectsDeleteForm(\"admin_deletionForm\", \"admin_deletionErrorMessage\", hub.urls.postAdminDelete()), false, true);'>\r\n \t\t  <h1>{{panels.ADMIN_OBJECTS_TITLE}}</h1>\r\n\t\t  <h3>{{panels.ADMIN_OBJECTS_HEAD}}<br><font color='red'>{{panels.ADMIN_OBJECTS_WARNING}}</font></h3>\r\n\t\t  <div dojoType='hub.widgets.adminobjectscatalog.Catalog' class='adminobjectscatalog' title=\"{{popups.ADMIN_OBJECTS_FIELD_TOOLTIP}}\"></div>\r\n          <div>\r\n            <input id='admin_deletionSubmit' class='submit' type='submit' value=\"{{panels.ADMIN_OBJECTS_BUTTON_VALUE}}\"/>\r\n            <br />\r\n            <span id='admin_deletionErrorMessage' class='error'></span>\r\n          </div>\r\n  \t   </form>\r\n    </div>\r\n{% endblock %}\r\n    \r\n",

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        
        
        

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};
            
        return dojo.mixin(obj, server_data);
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminDeletePage();
    }
});

/**
 * Settings page.
 */
dojo.declare("hub.pages.AdminProxy", hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n<div class='TOM_panel TOM_admin_panel'>\r\n\t<form id='admin_proxyForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.ProxyForm(\"admin_proxyForm\", \"proxyMessage\", hub.urls.postAdminProxy()), false, true);'>\r\n\t\t<h1>{{panels.ADMIN_PROXY_TITLE}}</h1>\r\n\t\t<h3>{{panels.ADMIN_PROXY_HEAD}}</h3>\r\n\t\t\r\n\t\t<label for=\"proxySet\">{{panels.ADMIN_PROXY_SET_LABEL}}</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t{% if proxySet %} \r\n\t\t\t\t<input type='checkbox' name='proxySet' checked title=\"{{popups.ADMIN_PROXY_SET_CHECKBOX_TOOLTIP}}\" />\r\n\t\t\t{% else %}\r\n\t\t\t\t<input type='checkbox' name='proxySet' title=\"{{popups.ADMIN_PROXY_SET_CHECKBOX_TOOLTIP}}\" />\r\n\t\t\t{% endif %}\r\n\t\t</div><br>\r\n\t\t\r\n\t\t<label for=\"proxyHost\">{{panels.ADMIN_PROXY_HOST_LABEL}}</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<input type='text' name=\"proxyHost\" id=\"proxyHost\" value=\"{{proxyHost}}\" title=\"{{popups.ADMIN_PROXY_HOST_FIELD_TOOLTIP}}\" />\r\n\t\t</div><br>\r\n\t\t\r\n\t\t<label for=\"proxyPort\">{{panels.ADMIN_PROXY_PORT_LABEL}}</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<input type='text' name=\"proxyPort\" id=\"proxyPort\" value=\"{{proxyPort}}\" title=\"{{popups.ADMIN_PROXY_PORT_FIELD_TOOLTIP}}\" />\r\n\t\t</div><br>\r\n\t\t\r\n\t\t<label for=\"proxyUser\">{{panels.ADMIN_PROXY_USER_LABEL}}</label>\r\n\t\t<div class='rightCol'>\r\n\t\t\t<input type='text' name=\"proxyUser\" id=\"proxyUser\" value=\"{{proxyUser}}\" title=\"{{popups.ADMIN_PROXY_USER_FIELD_TOOLTIP}}\" />\r\n\t\t</div><br>\r\n\t\t\r\n\t\t<label for=\"proxyPwd\">{{panels.ADMIN_PROXY_PWD_LABEL}}</label>\r\n\t\t<div class='rightCol'>\r\n{% if proxyPwd %}\t\t\r\n\t\t\t<input type='password' name=\"proxyPwd\" id=\"proxyPwd\"  value=\"{{ proxyPwd }}\" title=\"{{popups.ADMIN_PROXY_PWD_FIELD_TOOLTIP}}\" />\r\n{% else %}\r\n\t\t\t<input type='password' name=\"proxyPwd\" id=\"proxyPwd\"  value=\"\" title=\"{{popups.ADMIN_PROXY_PWD_FIELD_TOOLTIP}}\" />\r\n{% endif %}\r\n\t\t</div>\r\n\t\t\r\n\t\t<br /><br /><br />\r\n\t\t<div class='rightCol'>\r\n\t\t\t<input id='admin_typeSubmit' class='submit' type='submit' value=\"{{panels.ADMIN_PROXY_BUTTON_VALUE}}\"/>\r\n\t\t\t<br />\r\n\t\t\t<span id='proxyMessage' class='error'></span>\r\n\t\t</div>\r\n\t</form>\r\n</div>\r\n{% endblock %}\r\n",

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminProxyPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        
        
        

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};
            
        return dojo.mixin(obj, server_data);
    }
});

/**
 * Permissions page.
 */
dojo.declare("hub.pages.AdminPermissions", hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n<div class='TOM_panel TOM_admin_panel'>\r\n\t<form id='admin_permissionsForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.PermissionsForm(\"admin_permissionsForm\", \"errorMessage\", hub.urls.postAdminPermissions()), false, true);'>\r\n\t<h1>{{ panels.ADMIN_PERMISSIONS_TITLE }}</h1>\r\n\t<h3>{{ panels.ADMIN_PERMISSIONS_HEAD }}</h3>\r\n\t\r\n\t<div dojoType='hub.widgets.edit.AdminAssign'></div>\r\n\t\r\n\t<div class=\"rightCol\">\r\n\t\t<input id='admin_permissionSubmit' class='submit' type='submit' value=\"{{ panels.ADMIN_PERMISSIONS_BUTTON_VALUE }}\"/>\r\n\t\t<br/>\r\n\t\t<span id='errorMessage' class='error'></span>\r\n\t</div>\r\n\t</form>\r\n</div>\r\n{% endblock %}\r\n",

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminPermissionsPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        
        
        

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Tracking page.
 */
dojo.declare("hub.pages.AdminTracking", hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n\t<div >\r\n\t\t<div class='TOM_panel TOM_admin_panel' id=\"admin_global_container\">\r\n\t      <form id='admin_globalstatusForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.GlobalStatusForm(\"admin_globalstatusForm\", \"globalstatusMessage\", hub.urls.postAdminTrackingStatus()), false, false);'>\r\n \t\t\t  <h1>{{ panels.ADMIN_TRACKING_SETTINGS_TITLE }}</h1>\r\n\t\t\t  <h3>{{ panels.ADMIN_TRACKING_SETTINGS_HEAD }}</h3>\r\n\r\n\t          <label for='status'>{{ panels.ADMIN_TRACKING_SETTINGS_LABEL }}</label>\r\n\t          <div class=\"rightCol\">\r\n\t            {% if isOn %}\r\n\t            <select name='status' id='status' title=\"{{popups.ADMIN_TRACKING_SETTINGS_DROPDOWN_TOOLTIP}}\" >\r\n\t            \t<option value=\"off\" >{{ panels.ADMIN_TRACKING_SETTINGS_DROPDOWN_OFF }}</option>\r\n\t            \t<option value=\"on\" selected=\"selected\" >{{ panels.ADMIN_TRACKING_SETTINGS_DROPDOWN_ON }}</option>\r\n\t            </select>\r\n\t            {% else %}\r\n\t            <select name='status' id='status' title=\"{{popups.ADMIN_TRACKING_SETTINGS_DROPDOWN_TOOLTIP}}\" >\r\n\t            \t<option value=\"on\" >{{ panels.ADMIN_TRACKING_SETTINGS_DROPDOWN_ON }}</option>\r\n\t            \t<option value=\"off\" selected=\"selected\" >{{ panels.ADMIN_TRACKING_SETTINGS_DROPDOWN_OFF }}</option>\r\n\t            </select>\r\n\t            {% endif %}\t\r\n\t          </div>\r\n\t          <br />\r\n\t          <div class=\"rightCol\">\r\n\t            <input id='admin_globalstatusSubmit' class='submit' type='submit' value=\"{{ panels.ADMIN_TRACKING_SETTINGS_BUTTON_VALUE }}\" />\r\n\t            <br />\r\n\t            <span id='globalstatusMessage' class='error'></span>\r\n\t          </div>\r\n\r\n\t      </form>\r\n\t\t</div>\r\n{% if isOn %}\r\n\t\t<div id=\"admin_options_container\">\r\n\t      \r\n        <div class='TOM_panel TOM_admin_panel'>\r\n          <h1>{{panels.ADMIN_TRACKING_SETTINGS_TITLE4}}</h1>\r\n          <h3>{{panels.ADMIN_TRACKING_SETTINGS_HEAD4}}</h3>\r\n          <form id='admin_clearForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.ClearTrackedDataForm(\"admin_clearForm\", \"typeMessage4\", hub.urls.postAdminTrackingClear()), false, false);'>\r\n          <div id=\"admin_clearData\">\r\n          \t<label for='trackingDate'><span class='required'>*</span>{{ panels.ADMIN_TRACKING_SETTINGS_LABEL4 }}</label>\r\n             <div class=\"rightCol\">\r\n               <input name='trackingDate' maxlength='100' type='text' value=\"\"/><br />\r\n               <span class='helpertext'>({{ popups.ADMIN_TRACKING_SETTINGS_LABEL4_TOOLTIP }})</span><br/>\r\n             </div>\r\n           </div>\r\n          <div class=\"rightCol\">\r\n            <input id='admin_clearSubmit' class='submit' type='submit' value=\"{{ panels.ADMIN_TRACKING_SETTINGS_BUTTON4_VALUE }}\"/>\r\n            <br />\r\n            <span id='typeMessage4' class='error'></span>\r\n          </div>\r\n          </form>\r\n         </div>\r\n\r\n        <div class='TOM_panel TOM_admin_panel'>\r\n          <form id='admin_objectForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.AdminTrackingPermissionsForm(\"admin_objectForm\", \"typeMessage3\", hub.urls.postAdminTrackingPermissions()), false, false);'>\r\n            <h1>{{panels.ADMIN_TRACKING_SETTINGS_TITLE3}}</h1>\r\n            <h3>{{panels.ADMIN_TRACKING_SETTINGS_HEAD3}}</h3>\r\n            <div dojoType='hub.widgets.adminobjectscatalog.ObjectTracking' class='adminobjectscatalog' title=\"{{popups.ADMIN_OBJECTS_FIELD_TOOLTIP}}\"></div>\r\n            <div>\r\n              <br /><span class='helpertext'>* {{ panels.ADMIN_TRACKING_SETTINGS_TABLE_INFO }}</span><br />\r\n              <input id='admin_objectSubmit' class='submit' type='submit' value=\"{{ panels.ADMIN_TRACKING_SETTINGS_BUTTON3_VALUE }}\" />\r\n              <br />\r\n              <span id='typeMessage3' class='error'></span>\r\n            </div>\r\n          </form>\r\n        </div>\r\n\t\t</div>\r\n{% endif %}\r\n    </div>\r\n{% endblock %}\r\n\t\r\n",
    widgetsInTemplate: true,

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminEditTrackingPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        
        
        

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Tracking data page.
 */
dojo.declare("hub.pages.AdminTrackingData", hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n\r\n<div class='TOM_panel TOM_admin_panel'>\r\n  <h1>{{panels.ADMIN_TRACKED_DATA_HEAD}}</h1>\r\n  <div dojoType='hub.widgets.adminobjectscatalog.ObjectTrackingData' class='adminobjectscatalog' title=\"{{popups.ADMIN_TRACKED_DATA_OBJECTLIST_TABLE_TOOLTIP}}\"></div>\r\n  <div>\r\n    <span class='helpertext'>&nbsp;{{ panels.ADMIN_TRACKED_DATA_INFO }}</span>\r\n  </div>\r\n</div>\r\n{% endblock %}\r\n",

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminTrackingData();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        
        
        

        var obj = {
        	baseurl: hub.config.base_url,
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups'),
            icon: hub.urls.getIcon('btn16_feed.gif'),
            base_img_url: hub.urls.getImage('')
            };

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Bans page.
 */
dojo.declare("hub.pages.AdminBannedUsers", hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n<div class='TOM_panel TOM_admin_panel'>\r\n\t<form id='admin_bannedForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.BannedForm(\"admin_bannedForm\", \"errorMessage\", hub.urls.postAdminBan()), false, true);'>\r\n\t<h1>{{ panels.ADMIN_BANNED_TITLE }}</h1>\r\n\t<h3>{{ panels.ADMIN_BANNED_HEAD }}</h3>\r\n\t\r\n\t<div dojoType='hub.widgets.edit.BannedAssign'></div>\r\n\t\r\n\t<div class=\"rightCol\">\r\n\t\t<input id='admin_bannedSubmit' class='submit' type='submit' value=\"{{ panels.ADMIN_BANNED_BUTTON_VALUE }}\"/>\r\n\t\t<br/>\r\n\t\t<span id='errorMessage' class='error'></span>\r\n\t</div>\r\n\t</form>\r\n</div>\r\n{% endblock %}\r\n",

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminBanPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        
        
        

        var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Categories page.
 */
dojo.declare("hub.pages.AdminCategories", hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n    <div class='TOM_panel TOM_admin_panel'>\r\n      <form id='admin_categoriesForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.CategoriesForm(\"admin_categoriesForm\", \"errorMessage\", hub.urls.postAdminCategories()), false, true);'>\r\n\t\t  <h1>{{ panels.ADMIN_CATEGORIES_TITLE }}</h1>\r\n\t\t  <h3>{{ panels.ADMIN_CATEGORIES_HEAD }}</h3>\r\n\r\n\t\t  <div >\r\n\t\t  \t<input style=\"width:30em;\" name='categoryName' maxlength='100' type='text' title=\"{{popups.ADMIN_CATEGORIES_ADD_CATEGORY_TOOLTIP}}\"/>\r\n\t\t  \t<br />\r\n\t\t  \t<span style=\"font-size:0.8em;\" class='helpertext'>({{ panels.ADMIN_CATEGORIES_HEAD_INFO }})</span>\r\n\t\t  </div>\r\n\t\t  <br />\r\n\t\t  \r\n\t\t  <h3>{{ panels.ADMIN_CATEGORIES_HEAD2 }}</h3>\r\n\t\t  <div >\r\n            <div name='editlist' class=\"TOM_widget_CatalogTable\" title=\"{{popups.ADMIN_CATEGORIES_EDIT_CATEGORY_TOOLTIP}}\">\r\n          \t\t<ul>\r\n{% for cat in categories %}\r\n\t\t\t\t\t\t<li>\r\n\t\t\t\t\t\t\t<input type=\"text\" maxlength='100' style=\"float: none;width:30em;\" name=\"edit{{ cat.id }}\" id=\"edit{{ cat.id }}\"  value=\"{{ cat.name }}\" /><br />\r\n\t\t\t\t\t\t</li>\r\n{% endfor %}\r\n\t\t  \t\t</ul>\r\n\t\t  \t</div>\r\n\t\t  </div>\r\n\t\t  <br />\r\n\r\n\t\t  <h3>{{ panels.ADMIN_CATEGORIES_HEAD3 }}<br><font color='red'>{{ panels.ADMIN_CATEGORIES_WARNING }}</font></h3>\t\t  \r\n\t\t  <div >\r\n            <p>\r\n            <div name='catlist' class=\"TOM_widget_CheckBoxList\" style=\"width:24em;\" title=\"{{popups.ADMIN_CATEGORIES_DELETE_CATEGORY_TOOLTIP}}\">\r\n          \t\t<ul>\r\n{% for cat in categories %}\r\n\t\t\t\t\t\t<li>\r\n\t\t\t\t\t\t<input type='checkbox' name=\"deletion{{ cat.id }}\" id=\"deletion{{ cat.id }}\" />\r\n\t\t\t\t\t\t<label for=\"deletion{{ cat.id }}\">{{ cat.name }}</label><br/>\r\n\t\t\t\t\t\t</li>\r\n{% endfor %}\r\n\t\t  \t\t</ul>\r\n\t\t  \t</div>\r\n\t\t  </div>\r\n\t\t  <br /><br />\r\n          <div >\r\n            <input id='admin_categoriesSubmit' class='submit' type='submit' value='{{ panels.ADMIN_CATEGORIES_BUTTON_VALUE }}'/>\r\n            <br />\r\n            <span id='errorMessage' class='error'></span>\r\n          </div>\r\n  \t   </form>\r\n    </div>\r\n{% endblock %}\r\n    \r\n",

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminCategoryPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        
        
        

       var obj = {
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};
            
        return dojo.mixin(obj, server_data);
    }
});

/**
 * Admin Report Emails page.
 */
dojo.declare("hub.pages.AdminReportEmails", hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n<div>      \r\n{% if showReportForm %}\r\n      <div class='TOM_panel TOM_admin_panel'>\r\n        <form method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.ReportForm(\"report\", \"reportTestMessage\", hub.urls.postReportEmail()), false, true);' name='report' id='report'>\r\n \t\t  <h1>{{ panels.ADMIN_EMAILS_TEST }}</h1>\r\n   \t\t  <h3>{{ panels.ADMIN_EMAILS_HEAD3 }}</h3>\r\n            \r\n            <label for='reportBody'>{{ panels.ADMIN_EMAILS_LABEL3 }}</label>\r\n            <div class=\"rightCol\">\r\n              <textarea name='reportBody' rows='4' cols='47' maxlength='500' title=\"500 characters max\"></textarea><br />\r\n              <span class=\"helpertext\">({{ panels.ADMIN_EMAILS_TEST }})</span>\r\n            </div>\r\n            <br />\r\n            <br />\r\n            <div class=\"rightCol\">\r\n            <input name='reportSubject' type='hidden' value=\"{{ panels.ADMIN_EMAILS_TEST }}:  {{ currentUser }}\" />\r\n              <input name='submit' value=\"{{ panels.ADMIN_EMAILS_TEST }}\" type='submit' />\r\n            <p id=\"reportTestMessage\"></p>\r\n            </div>\r\n          \r\n        </form>\r\n      </div>\r\n{% endif %}\r\n    <div class='TOM_panel TOM_admin_panel'>\r\n      <form id='admin_reportsForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.EmailListForm(\"admin_reportsForm\", \"errorMessage\", hub.urls.postAdminEmail()), false, true);'>\r\n\t\t  <h1>{{ panels.ADMIN_EMAILS_TITLE }}</h1>\r\n\t\t  <h3>{{ panels.ADMIN_EMAILS_HEAD }}</h3>\r\n\r\n          <label for='emaillist'>{{ panels.ADMIN_EMAILS_LABEL }}</label>\r\n          <div class=\"rightCol\" title=\"{{popups.ADMIN_EMAILS_LIST_FIELD_TOOLTIP}}\" >\r\n             <textarea cols=\"40\" rows=\"5\" name=\"myname\">{% for user in users %}{% if user.isInEmailList %}{% if user.email %} {{ user.email }};{% endif %}{% endif %}{% endfor %}</textarea>\r\n\t\t  </div>\r\n\t\t  <br /><br />\r\n\r\n          <h3>{{ panels.ADMIN_EMAILS_HEAD2 }}</h3>\r\n\r\n          <label for='userlist'>{{ panels.ADMIN_EMAILS_LABEL2 }}</label>\r\n          <div class=\"rightCol\">\r\n              <div name='userlist' class=\"TOM_widget_CheckBoxList\">\r\n          \t\t<ul>\r\n{% for user in users %}\r\n\t\t\t\t\t<li>\r\n\t{% if user.email %}\r\n\t\t{% if user.isInEmailList %}\r\n\t\t\t\t\t<input type='checkbox' name=\"report{{ user.id }}\" id=\"report{{ user.id }}\" value=\"{{ user.id }}\" checked />\r\n\t\t{% else %}\r\n\t\t\t\t\t<input type='checkbox' name=\"report{{ user.id }}\" id=\"report{{ user.id }}\" value=\"{{ user.id }}\" />\r\n\t\t{% endif %}\r\n\t{% else %}\t\t\r\n\t\t\t\t\t<input type='checkbox' name=\"report{{ user.id }}\" id=\"report{{ user.id }}\" value=\"{{ user.id }}\" disabled=\"disabled\"/>\r\n\t{% endif %}\r\n                                        <label for=\"{{ user.id }}\">{{ user.name }}</label> <br/>\r\n\t\t\t\t\t</li>\r\n{% endfor %}\r\n\t\t         </ul>\r\n\t\t  \t</div>\r\n\t\t  </div>\r\n          <div class=\"rightCol\">\r\n            <input id='admin_reportsSubmit' class='submit' type='submit' value='{{ panels.ADMIN_EMAILS_BUTTON_VALUE }}'/>\r\n            <br />\r\n            <span id='errorMessage' class='error'></span>\r\n          </div>\r\n  \t   </form>\r\n    </div>\r\n\t\t     \r\n{% endblock %}\r\n",

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminEmailPage();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        
        
        

        var obj = {
        	currentUser: hub.config.userfullname,
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * Report Abuse page.
 */
dojo.declare("hub.pages.ReportAbuse", hub.pages._Page, {
    // template to replace node with
    templateString:"<div>      \r\n      <div class='TOM_panel TOM_admin_panel'>\r\n        <form method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.ReportForm(\"report_{{ id }}\", \"reportAbuseMessage_{{ id }}\", hub.urls.postReportEmail()), false, true);' name=\"report_{{ id }}\" id=\"report_{{ id }}\">\r\n\t\t\t<h1>{{ panels.ABUSE_REPORT_ABUSE_LEGEND }}</h1>\r\n            \r\n            <label for='reportBody'>{{ panels.ABUSE_EXPLANATION_TEXT }}</label>\r\n            <div class=\"rightCol\">\r\n            <input name='reportSubject' type='hidden' value=\"{{ title }} - {{ currentUser }}\" />\r\n              <textarea name='reportBody' rows='4' cols='47' maxlength='500' title=\"{{ popups.SAVE_DESCRIPTION }}\"></textarea><br />\r\n              <span class=\"helpertext\">({{ panels.ABUSE_TITLE }})</span>\r\n            </div>\r\n            <br />\r\n            <br />\r\n            <div class=\"rightCol\">\r\n\t        <button dojoType=\"hub.widgets.Button\">{{ labels.BUTTON_CANCEL }}\r\n\t          <script type=\"dojo/method\" event=\"onClick\">\r\n\t\t\t\thub.showPage(\"{{ title }}\",\"Details\",{\"object_id\": \"{{ id }}\"});\r\n\t          </script>\r\n\t        </button>\r\n\t        <button type=\"submit\" dojoType=\"hub.widgets.Button\">{{ labels.BUTTON_SUBMIT }}</button>\r\n\t        <p id=\"reportAbuseMessage_{{ id }}\"></p>\r\n            </div>\r\n          \r\n        </form>\r\n      </div>\r\n</div>\r\n",
    entryId: null,
    title: null,

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return null;
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
    	this.tab.entryId = this.entryId;
        
        
        

        var obj = {
        	currentUser: hub.config.userfullname,
        	id: this.entryId,
        	title: this.title,
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
	        links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups')};

        return dojo.mixin(obj, server_data);
    }
});

/**
 * AdminTDI page.
 */
dojo.declare("hub.pages.AdminTDI", hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n<div class='TOM_panel TOM_admin_panel'>\r\n\t<h1>{{panels.ADMIN_TDI_TITLE}}</h1>\r\n\r\n<form id='admin_tdiForm1' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.TDIForm(\"admin_tdiForm1\", \"errorMessage\", hub.urls.postAdminTDI()), false, false);'>\r\n<input type='hidden' name='admin' value='admin_tdiRefresh'/>\r\n<label for='admin_tdiRefresh'>{{panels.ADMIN_TDI_REFRESH}}</label>\r\n<div class=\"rightCol\" >\r\n\t<input id='admin_tdiRefresh' class='submit' type='submit' value='{{panels.ADMIN_TDI_REFRESH_BUTTON}}' title=\"{{popups.ADMIN_TDI_REFRESH_BUTTON_TOOLTIP}}\" />\r\n</div>\r\n</form>\r\n<form id='admin_tdiForm2' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.TDIForm(\"admin_tdiForm2\", \"errorMessage\", hub.urls.postAdminTDI()), false, false);'>\r\n<input type='hidden' name='admin' value='admin_tdiStart'/>\r\n<label for='admin_tdiStart'>{{panels.ADMIN_TDI_START}}</label>\r\n<div class=\"rightCol\">\r\n\t<input id='admin_tdiStart' class='submit' type='submit' value='{{panels.ADMIN_TDI_START_BUTTON}}' title=\"{{popups.ADMIN_TDI_START_BUTTON_TOOLTIP}}\" />\r\n</div>\r\n</form>\r\n<form id='admin_tdiForm3' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.TDIForm(\"admin_tdiForm3\", \"errorMessage\", hub.urls.postAdminTDI()), false, false);'>\r\n<input type='hidden' name='admin' value='admin_tdiStop'/>\r\n<label for='admin_tdiStop'>{{panels.ADMIN_TDI_STOP}}</label>\r\n<div class=\"rightCol\">\r\n\t<input id='admin_tdiStop' class='submit' type='submit' value='{{panels.ADMIN_TDI_STOP_BUTTON}}' title=\"{{popups.ADMIN_TDI_STOP_BUTTON_TOOLTIP}}\" />\r\n</div>\r\n</form>\r\n<br>\r\n<span id='errorMessage' class='error'>{{initialStatus}}</span>\r\n</div>\r\n{% endblock %}\r\n",

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminTDI();
    },

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        
        
        

        var obj = {
        	baseurl: hub.config.base_url,
            labels: dojo.i18n.getLocalization('hub', 'labels'),
	        panels: dojo.i18n.getLocalization('hub', 'panels'),
            links: dojo.i18n.getLocalization('hub', 'links'),
            popups: dojo.i18n.getLocalization('hub', 'popups'),
            icon: hub.urls.getIcon('btn16_feed.gif'),
			icon_feed: hub.urls.getImage('feedLarge.gif')
            };

        return dojo.mixin(obj, server_data);
    }
});

/**
 * SelectMashupsServer page.
 */

dojo.declare("hub.pages.SelectMashupsServer", mm.pages.SelectMashupsServer, {
});

/**
 * AddFeedToMashups page. 
 */

dojo.declare("hub.pages.AddFeedToMashups", mm.pages.AddFeedToMashups, {
});

/**
 * AddIWidgetToMashups page. 
 */

dojo.declare("hub.pages.AddIWidgetToMashups", mm.pages.AddIWidgetToMashups, {
});

/**
 * AddMashupPageToMashups page.
 */

dojo.declare("hub.pages.AddMashupPageToMashups", mm.pages.AddMashupPageToMashups, {
});

/**
 * FinishedAddToMashups page.
 */

dojo.declare("hub.pages.FinishedAddToMashups", mm.pages.FinishedAddToMashups,{
});


}

if(!dojo._hasResource["hub.pages.admin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.pages.admin"] = true;
dojo.provide("hub.pages.admin");








/**
 * Page for adding, removing, or renaming categories. No server data to fetch.
 * Categories can be gotten from the global dijit named "hub.cats."
 */
dojo.declare('hub.pages.admin.Categories', hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Imports required modules and build an object with properties matching 
     * fields in the template.
     */
    getContext: function() {
        // @todo
    }
});

/**
 * Page for reinitializing the database and sample data. No server data to 
 * fetch.
 */
dojo.declare('hub.pages.admin.Initialize', hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Imports required modules and build an object with properties matching 
     * fields in the template.
     */
    getContext: function() {
        // @todo
    }
});

/**
 * Page for reinitializing the database and sample data. No server data to 
 * fetch.
 */
dojo.declare('hub.pages.admin.AdminMigrateDatabase', hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n<div class=\"TOM_panel TOM_admin_panel\">\r\n  <h1>{{panels.MIGRATE_TITLE}}</h1>\r\n  <h3>{{panels.MIGRATE_INSTRUCTION}}</h3>\r\n  <form id=\"admin_migrate_form\" method=\"POST\" onsubmit=\"return hub.submit.ajaxSubmit(new hub.submit.admin.MigrateForm('admin_migrate_form', 'migrateStatus', hub.urls.postAdminMigrateDB()), false, true);\">\r\n    \r\n    <div>\r\n      <label id=\"db_name_label\" for=\"db_driver\">{{labels.MIGRATE_DB_TYPE}}</label>\r\n      <select tabindex=\"1\" onchange=\"_selectDbDriver(this);\" name=\"db_driver\" id=\"db_driver\" dojoAttachPoint=\"db_driver\">\r\n        <option value=\"DB2\" selected=\"selected\">DB2</option>\r\n        <option value=\"IDS\">Informix</option>\r\n        <!-- <option value=\"DERBY\">Derby Network Server</option> -->\r\n        <!-- <option value=\"DERBY_EMBEDDED\">Embedded Derby</option> -->\r\n      </select>\r\n    </div>\r\n        \r\n    \r\n    <div id=\"db_host_div\" class=\"right-column\" title=\"{{popups.MIGRATE_HOSTNAME_TOOLTIP}}\">\r\n      <label id=\"db_name_label\" for=\"db_host\">{{labels.MIGRATE_HOSTNAME}}</label>\r\n      <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_host\" id=\"db_host\" value=\"localhost\"/>\r\n    </div>\r\n        \r\n    <div id=\"db_port_div\" class=\"right-column\" title=\"{{popups.MIGRATE_PORT_TOOLTIP}}\"> \r\n      <label id=\"db_port_label\" for=\"db_port\">{{labels.MIGRATE_PORT}}</label>\r\n      <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_port\" id=\"db_port\" value=\"\"/>\r\n    </div>\r\n        \r\n    \r\n    <div id=\"db_name_div\" class=\"right-column\" title=\"{{popups.MIGRATE_DB_NAME_TOOLTIP}}\">\r\n      <label id=\"db_name_label\" for=\"db_name\">{{labels.MIGRATE_DB_NAME}}</label>\r\n      <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_name\" id=\"db_name\" value=\"hubdb\" />\r\n    </div>\r\n                            \r\n    \r\n    <div id=\"db_schema_div\" class=\"right-column\" title=\"{{popups.MIGRATE_SCHEMA_TOOLTIP}}\">\r\n      <label id=\"db_schema_label\" for=\"db_schema\">{{labels.MIGRATE_SCHEMA}}</label>\r\n      <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_schema\" value=\"\" id=\"db_schema\"/>\r\n    </div>\r\n        \r\n    \r\n    <div class=\"right-column\" title=\"{{popups.MIGRATE_USERNAME_TOOLTIP}}\">\r\n      <label id=\"db_username_label\" for=\"db_username\">{{labels.MIGRATE_USERNAME}}</label>\r\n      <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_username\" id=\"db_username\" value=\"\" />\r\n    </div>\r\n        \r\n    <div class=\"right-column\" title=\"{{popups.MIGRATE_PASSWORD_TOOLTIP}}\">\r\n      <label id=\"db_password_label\" for=\"db_password\">{{labels.MIGRATE_PASSWORD}}</label>\r\n      <input tabindex=\"1\" type=\"password\" size=\"40\" name=\"db_password\" id=\"db_password\" value=\"\" />\r\n    </div>\r\n    \r\n    <div>\r\n      <input id='admin_typeSubmit' class='submit' type='submit' value=\"{{panels.MIGRATE_SUBMIT_BUTTON_LABEL}}\"/>\r\n    </div>\r\n  </form>\r\n  <p id=\"migrate_status\"></p>\r\n  <script type=\"text/javascript\">\r\n      function _selectDbDriver() {\r\n        var driver = dojo.byId('db_driver');\r\n        var hostname = dojo.byId('db_host_div');\r\n        var schema = dojo.byId('db_schema_div');\r\n        var port = dojo.byId('db_port_div');\r\n        \r\n        switch(driver.value){\r\n            case hub.config.derby_driver:\r\n                hostname.style.visibility='hidden';\r\n                schema.style.visibility='hidden';\r\n                port.style.visibility='hidden';\r\n                break;\r\n            case hub.config.embedded_derby_driver:\r\n                hostname.style.visibility='visible';\r\n                schema.style.visibility='hidden';\r\n                port.style.visibility='visible';\r\n                port.value = hub.config.derby_port;\r\n                break;\r\n            case hub.config.ids_driver:\r\n                hostname.style.visibility='visible';\r\n                schema.style.visibility='hidden';\r\n                port.style.visibility='visible';\r\n                port.value = hub.config.ids_port;\r\n                break;\r\n            case hub.config.db2_driver:\r\n            default:\r\n                hostname.style.visibility='visible';\r\n                schema.style.visibility='visible';\r\n                port.style.visibility='visible';\r\n                port.value = hub.config.db2_port;\r\n        }\r\n      }\r\n  </script>\r\n</div>\r\n{% endblock %}\r\n",

    /**
     * Imports required modules and build an object with properties matching 
     * fields in the template.
     */
    getContext: function() {
        // @todo
    }
});

/**
 * Page for assigning user permissions. Fetches existing permissions.
 */
dojo.declare('hub.pages.admin.Permissions', hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminPermissionsPage();
    }
});

/**
 * Page for deleting arbitrary items. Fetches name, id, etc. of all items.
 */
dojo.declare('hub.pages.admin.AdminObjects', hub.pages._Page, {
    // template to replace node with
    templateString:"{% extends \"../scripts/hub/pages/templates/AdminMenu.html\" %}\r\n{% block content %}\r\n    <div class='TOM_panel TOM_admin_panel'>\r\n      <form id='admin_deletionForm' method='POST' onsubmit='return hub.submit.ajaxSubmit(new hub.submit.admin.ObjectsDeleteForm(\"admin_deletionForm\", \"admin_deletionErrorMessage\", hub.urls.postAdminDelete()), false, true);'>\r\n \t\t  <h1>{{panels.ADMIN_OBJECTS_TITLE}}</h1>\r\n\t\t  <h3>{{panels.ADMIN_OBJECTS_HEAD}}<br><font color='red'>{{panels.ADMIN_OBJECTS_WARNING}}</font></h3>\r\n\t\t  <div dojoType='hub.widgets.adminobjectscatalog.Catalog' class='adminobjectscatalog' title=\"{{popups.ADMIN_OBJECTS_FIELD_TOOLTIP}}\"></div>\r\n          <div>\r\n            <input id='admin_deletionSubmit' class='submit' type='submit' value=\"{{panels.ADMIN_OBJECTS_BUTTON_VALUE}}\"/>\r\n            <br />\r\n            <span id='admin_deletionErrorMessage' class='error'></span>\r\n          </div>\r\n  \t   </form>\r\n    </div>\r\n{% endblock %}\r\n    \r\n",
    widgetsInTemplate: true,
    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        if (!this.atomcatalogid) {
        	this.atomcatalogid = "atomcatalogid" + 11111;
        }
        console.log("this.atomcatalogid = " + this.atomcatalogid);
        return {"atomcatalogid": this.atomcatalogid};
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminDeletePage();
    }
});

/**
 * Page for banning or unbanning users. Fetches metadata about banned users.
 */
dojo.declare('hub.pages.admin.Ban', hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminBanPage();
    }
});

/**
 * Page for configuring email notification. Fetches the current config.
 */
dojo.declare('hub.pages.admin.Email', hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminEmailPage();
    }
});

/**
 * Page for configuring tracking. Fetches the current config.
 */
dojo.declare('hub.pages.admin.EditTracking', hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminEditTrackingPage();
    }
});

/**
 * Page for view tracking data. Fetches tracked item info.
 */
dojo.declare('hub.pages.admin.ViewTracking', hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getAdminViewTrackingPage();
    }
});

}

if(!dojo._hasResource["hub.pages.social"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.pages.social"] = true;
dojo.provide("hub.pages.social");








/**
 * Report abuse page. No server data to fetch.
 */
dojo.declare("hub.pages.social.Abuse", hub.pages._Page, {
    // template to replace node with
    templateString:"",

    /**
     * Imports required modules and build an object with properties matching 
     * fields in the template.
     */
    getContext: function() {
        // @todo
    }
});


/**
 * Profile page letting the user change name, password, etc. Fetches existing
 * user information from the server.
 */
dojo.declare('hub.pages.social.Profile', hub.pages._Page, {
    // template to replace node with
    templateString:"<div class=\"TOM_editor_pane\">\r\n  <form id=\"TOM_register_form\" onsubmit=\"return false;\">\r\n    <fieldset>\r\n      <legend>{{ panels.REGISTER_LEGEND }}</legend>\r\n\r\n      <label for=\"registerName\">{{ panels.REGISTER_FULL_NAME }}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"text\" name=\"registerName\" maxlength=\"100\" title=\"{{ popups.REGISTER_FULL_NAME }}\" />\r\n      </div>\r\n\r\n      <label for=\"registerEmail\">{{ panels.REGISTER_EMAIL }}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"text\" name=\"registerEmail\" maxlength=\"128\" title=\"{{ popups.REGISTER_EMAIL }}\" />\r\n      </div>\r\n\r\n      <label for=\"registerLogin\">{{ panels.REGISTER_LOGIN }}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"text\" name=\"registerLogin\" maxlength=\"60\" title=\"{{ popups.REGISTER_LOGIN }}\" />\r\n      </div>\r\n\r\n      <label for=\"registerPassword\">{{ panels.REGISTER_PASSWORD }}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"password\" name=\"registerPassword\" maxlength=\"60\" title=\"{{ popups.REGISTER_PASSWORD }}\" />\r\n      </div>\r\n\r\n      <label for=\"registerPassword2\">{{ panels.REGISTER_PASSWORD2 }}</label>\r\n      <div class=\"rightCol\">\r\n        <input type=\"password\" name=\"registerPassword2\" maxlength=\"60\" title=\"{{ popups.REGISTER_PASSWORD2 }}\" />\r\n      </div>\r\n\r\n\r\n      <div class=\"rightCol\">\r\n        <button type=\"submit\" dojoType=\"dijit.form.Button\">{{ labels.BUTTON_REGISTER }}\r\n          <script type=\"dojo/method\" event=\"onClick\">\r\n            var element = dojo.byId(\"TOM_register_form\");\r\n            var form = new hub.submit.Register(element, \"TOM_register_message\");\r\n            hub.submit.ajaxSubmit(form);\r\n          </script>\r\n        </button>\r\n        <p id=\"TOM_register_message\"></p>\r\n      </div>\r\n    </fieldset>\r\n  </form>\r\n</div>\r\n\r\n<!-- +++++++++++++++++++++++++++++++++++++++++ -->\r\n\r\n    <div class='TOM_editor_pane'>\r\n      <form id='profileForm' method='POST' onsubmit='return TOM.editor.ajaxSubmit(new TOM.editor.ProfileForm(this, \"profileMessage\"));'>\r\n        <fieldset>\r\n          <legend>User profile for <?php echo $this->user->username; ?></b></legend>\r\n          <h3>Please provide information about yourself.</h3>\r\n          <label for='profileName'>Full Name:</label>\r\n          <div class=\"rightCol\">\r\n            <input id='profileName' name='profileName' maxlength='100' size='20' type='text' value=\"<?php echo $this->user->fullname; ?>\" /><br />\r\n            <span class='helpertext'>(Example: Dr. Marcus Reynolds, M.D.)</span><br/>\r\n          </div>\r\n          <label for='profileEmail'>Email Address:</label>\r\n          <div class=\"rightCol\">\r\n            <input id='profileEmail' name='profileEmail' maxlength='128' size='20' value=\"<?php echo $this->user->email; ?>\" type='text'/><br />\r\n            <span class='helpertext'>(Example: reynolds@doctors.com)</span><br/>\r\n          </div>\r\n<?php if ($this->passwordEditable) { ?>\r\n          <label for='profilePassword'>Password:</label>\r\n          <div class=\"rightCol\">\r\n            <input id='profilePassword' name='profilePassword' maxlength='60' size='20' type='password' /><br />\r\n            <span class='helpertext'>(Leave blank to keep your current password or enter a new one.)</span><br/>\r\n          </div>\r\n          <label for='profilePassword2'>Confirm password:</label>\r\n          <div class=\"rightCol\">\r\n            <input id='profilePassword2' name='profilePassword2' maxlength='60' size='20' type='password' /><br />\r\n            <span class='helpertext'>(Enter your new password again)</span><br/>\r\n          </div>\r\n<?php } else { ?>\r\n          <input id='profilePassword' name='profilePassword' maxlength='60' size='20' type='hidden' /><br />\r\n          <input id='profilePassword2' name='profilePassword2' maxlength='60' size='20' type='hidden' /><br />\r\n<?php } ?>\r\n          <br />\r\n          <div class=\"rightCol\">\r\n            <input id='profileSubmit' class='submit' type='submit' value='Save Profile'/>\r\n            <p id='profileMessage'></p>\r\n          </div>\r\n        </fieldset>\r\n      </form>\r\n    </div>\r\n",

    /**
     * Builds an object with properties matching fields in the template. Merges
     * fetched server data with local data.
     *
     * @param server_data: JSON decoded object fetched from server URL
     */
    getContext: function(server_data) {
        // @todo
    },

    /**
     * Gets the RESTful URL to query for information required to complete the
     * rendering of the page.
     */
    getUrl: function() {
        return hub.urls.getProfilePage();
    }
});

}

if(!dojo._hasResource['hub.widgets.chrome']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.widgets.chrome'] = true;

dojo.provide('hub.widgets.chrome');








 // for search form
 // for search form
 // radio button is in here





/**
 * Shows and hides the login box and welcome area panels. Observes the
 * date-profile, login, and logout signals.
 */
dojo.declare("hub.widgets.chrome.LoginBox", [dijit.layout.StackContainer, dijit._Templated], {
    // template to replace node with
    templateString:"<div class=\"TOM_widget_LoginBox\" \r\n\t><div dojoType=\"dijit.layout.ContentPane\" selected=\"${login_selected}\"\r\n\t><a href=\"${loginURL}\" >${labels.BUTTON_LOGIN}</a\r\n\t>&nbsp;|&nbsp;<a href=\"#\" dojoAttachEvent=\"onclick: _onShowHelp\">${links.LOGIN_HELP}</a></div\r\n\t><div selected=\"${showSettings}\" dojoType=\"dijit.layout.ContentPane\"\r\n\t><a href=\"#\" dojoAttachEvent=\"onclick: _onShowSettings\">${links.LOGIN_PROFILE}</a\r\n\t>&nbsp;|&nbsp;<a href=\"${logoutURL}\" >${links.LOGIN_LOGOUT}</a\r\n\t>&nbsp;|&nbsp;<a href=\"#\" dojoAttachEvent=\"onclick: _onShowHelp\">${links.LOGIN_HELP}</a\r\n\t></div><div selected=\"${welcome_selected}\" dojoType=\"dijit.layout.ContentPane\"\r\n\t><a href=\"${logoutURL}\" >${links.LOGIN_LOGOUT}</a\r\n\t>&nbsp;|&nbsp;<a href=\"#\" dojoAttachEvent=\"onclick: _onShowHelp\">${links.LOGIN_HELP}</a\r\n></div></div>\r\n",
    // dojo.form.Button and dojo.layout.ContentPane in the template
    widgetsInTemplate: true,
    // resize login box to content, not the other way around
    doLayout: true,
    
    loginURL: "#", //"/mashuphub/secured",
    logoutURL: "#", //"/mashuphub/client/ui/logout",

    postMixInProperties: function() {
        // fetch all localized text
        this.panels = dojo.i18n.getLocalization('hub', 'panels');
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.links = dojo.i18n.getLocalization('hub', 'links');

		// get login URL
		_this = this;
		dojo.xhrGet({
			//url: "https://localhost:8443/mashuphub/client/ui/searchtypes",
			//url: "../scripts/hub/widgets/json",
			url: hub.config.base_url + "/ui/loginURL",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				_this.loginURL = response.loginURL;
				_this.logoutURL = response.logoutURL;
			},
			error: function(response, args) {
				console.log('An error occurred when retrieving login info from the server.');
				
				var sUrl = hub.config.base_url;
				var pos=sUrl.indexOf("/client");
				if (pos > 0) {
				   sUrl = sUrl.substring(0, pos);
				   _this.loginURL = sUrl + '/secured';
				   _this.logoutURL = sUrl + '/secured/logout';
				}
			}
		});

        // decide what to show first ... TODO remake 
        var logged_in = (hub.config.username && hub.config.username != '');
        //alert('logged in:' + logged_in);
        if(logged_in) {
	        if ( hub.config.isAdmin ) {
	            this.showSettings = 'true';
	            this.login_selected = 'false';
    	        this.welcome_selected = 'false';
	        } else {
	        	this.welcome_selected = 'true';
	            this.login_selected = 'false';
				this.showSettings = 'false';
	        }
        } else {
            this.login_selected = 'true';
            this.welcome_selected = 'false';
			this.showSettings = 'false';
        }

        // interpolate values
        this.inherited(arguments);
    },

    postCreate: function() {
        // show initial welcome
//        this._doWelcome(hub.config.username);

        // listen to bus events
        dojo.subscribe('hub.login', this, '_onLogin');
        dojo.subscribe('hub.logout', this, '_onLogoutDone');
        dojo.subscribe('hub.update-profile', this, '_onProfile');
        
        this.inherited(arguments);
    },
/*
    _doWelcome: function(name) {
        // set username
        var msg = dojo.string.substitute(this.panels.LOGIN_WELCOME, [name]);
        this.welcome_message.innerHTML = msg;
    },
*/    
    _onLogin: function(name) {
        // show logged in user
//        this._doWelcome(name);
        // show welcome pane
//        this.forward();
        // clear entered username and pass
//        this.login_form.reset();        
    },
    
    _onLogoutDone: function() {
        // show the login box again
        this.back();
    },
    
    _onProfile: function(signal, name) {
        this.welcome.innerHTML = name;
    },

    _onSubmit: function(event) {
    alert(this.loginURL);
    	window.location = this.loginURL; 
//        var form = new hub.submit.Login(this.login_form, this.login_message);
        // safe submit, catches all exceptions
//        hub.submit.ajaxSubmit(form);
    },

    _onShowRegister: function(event) {
        hub.showPage(this.labels.LOGIN_REGISTER_TAB, 'Register');
    },

    _onShowSettings: function(event) {
        hub.showPage(this.labels.LOGIN_SETTINGS_TAB, 'AdminMigrateDatabase');
    },

    _onShowHelp: function(event) {
		hub.showUrl(this.labels.LOGIN_HELP_TAB, 'doc/userhelp/hubhelp/help_hub_overview.htm');
    },

    _onLogout: function(event) {
        hub.submit.ajaxSubmit(new hub.submit.Logout());
    }
});

/**
 * Initiates a search of the catalog on user request. Just a controller, doesn't
 * actually have anything to do with submitting the search terms.
 */
dojo.declare("hub.widgets.chrome.SearchBox", [dijit._Widget, dijit._Templated], {
    // template to replace node with
    templateString:"<div class=\"TOM_widget_SearchBox\">\r\n\t<form onsubmit=\"return false;\" dojoAttachPoint=\"form\" >\r\n\t\t<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\" align=\"right\" height=\"100%\">\r\n\t\t\t<tr valign=\"middle\" align=\"center\">\r\n\t\t\t\t<td width=\"404\" align=\"left\" >\r\n\t\t\t\t\t\t\t<div class=\"TOM_widget_SearchBox_cell\" id=\"mh_sb_term\" title=\"${popups.SEARCH_TERMS}\">\r\n\t\t\t\t\t\t\t\t<div class=\"TOM_widget_SearchBox_label\">${labels.SEARCH}</div>\r\n\t\t\t\t\t\t\t\t<input dojoType=\"dijit.form.TextBox\" name=\"searchTerm\" style=\" width:218px; padding: 0px; height: 18px;\" type=\"text\" value=\"\"/>\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t<div class=\"TOM_widget_SearchBox_cell\" id=\"mh_sb_scope\" title=\"${popups.SEARCH_SCOPE}\">\r\n\t\t\t\t\t\t\t\t<div class=\"TOM_widget_SearchBox_label\">${labels.SCOPE}</div>\r\n\t\t\t\t\t\t\t\t<div dojoType=\"hub.widgets.chrome.DropDownSelect\" id=\"hubSelectSearchType\" >&nbsp;</div>\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t<div id=\"mh_sb_submit\" class=\"TOM_widget_SearchBox_cell\" style=\"padding: 0px; vertical-align: middle; padding-top: 5px;\" title=\"${popups.SEARCH_SUBMIT}\">\r\n\t\t\t\t\t\t\t\t<button type=\"submit\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick: _onSubmit\"></button>\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td width=\"5\" >\r\n\t\t\t\t</td>\r\n\t\t\t\t<td align=\"center\" nowrap=\"nowrap\">\r\n\t\t\t\t\t<div dojoType=\"hub.widgets.chrome.LoginBox\" id=\"hub.login.box\"></div>\t\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t</table>\r\n\t</form>\r\n</div>\r\n",
    // dojo.form.button, etc. in the template
    widgetsInTemplate: true,

    postMixInProperties: function() {
        // fetch all localized text
        this.labels = dojo.i18n.getLocalization('hub', 'labels');
        this.popups = dojo.i18n.getLocalization('hub', 'popups');
        
        // interpolate values
        this.inherited(arguments);
    },

    _onSubmit: function(event) {
        var term = this.form.searchTerm.value;
        if(term == null || term == "") {
            // default to all text
            term = "*";
        }
        var collection = dijit.byId("hubSelectSearchType").getValue();
        if(collection) {
        	if (collection == "favorites") {
            	hub.showPage(term, 'Catalog', {'collection': "ui.all", searchType: "favorites", term: term});
            }
            else {
            	hub.showPage(term, 'Catalog', {'collection': collection, searchType: "", term: term});
            }
        } else {
            hub.showPage(term, 'Catalog', {'collection': "ui.all", searchType: "", term: term});
        }
        return;
    }
});

dojo.declare("hub.widgets.chrome.DropDownSelect", dijit.form.DropDownButton, {
	showLabel: true,
	store: null,
	dropDown: null,
	_isPopulated: false,
	_lastOption: "",
	onChange: function(){},
	items: null,

	postMixInProperties: function(){
		this.inherited(arguments);

		// try
		_this = this;
		dojo.xhrGet({
			//url: "https://localhost:8443/mashuphub/client/ui/searchtypes",
			//url: "../scripts/hub/widgets/json",
			url: hub.config.base_url + "/ui/searchtypes",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				_this.items = response.types;
			}
		});
		// 
		var span = dojo.doc.createElement("span"), selItem = null;
		if (!this.store)
		{
//			var items = dojo.query("> option", this.srcNodeRef).map(function(node){
//					node.style.display="none";
//					return { value: node.getAttribute("value"), name: String(node.innerHTML) };
//			});
//			var items = this.types;
			this.store = new dojo.data.ItemFileReadStore({data: {identifier:"value", items:this.items}});
		}
		var selItem;
		if(this.items && this.items.length && !this.value)
		{
			if ( this.srcNodeRef.selectedIndex && this.srcNodeRef.selectedIndex != -1 ) {
				selItem = this.items[this.srcNodeRef.selectedIndex];
			} else {
				var _selIndex = 0;
				for (var i = 0; i < this.items.length; i++) {
					if (this.items[i].value && this.items[i].value.indexOf("all") != -1) {
						_selIndex = i;
					}
				}
				selItem = this.items[_selIndex];
			}
		}
		this._initSelect(selItem, span);
		if (!this.dropDown)
		{
			this.dropDown = new dijit.Menu();
		}
		dojo.place(span, this.srcNodeRef, "first");
	},

	_initSelect: function(item, span){
		if (item)
		{
			this.value = item.value;
			span.innerHTML = item.name;
		}
		else if (this.store)
		{
			var _this = this;
			this.store.fetch({onComplete: function(i, r){ 
				if (i.length && i[0]) _this._initSelect(i[0], span);
			}});
		}
	},
	_menuItemClick: function(item){
//		alert(item.name);
		var val = item.value;
		if (this._lastOption != val)
		{
			this.setValue(val);
			this.setLabel(item.name); //-- to avoid jumping html elements, 
//			but can uncommented, if the widget will be enclosed into container with fixed
//			size		
			this._lastOption = val;
			this.onChange(val);
			dojo.forEach(this.dropDown.getChildren(), function(ch){
				if (ch.label == item.name){
					dojo.addClass(ch.iconNode, "selectedIcon");
				} else {
					dojo.removeClass(ch.iconNode, "selectedIcon");
				}
			});
		}
	},
	_loadCallback: function(items, request){
		var dropDown = this.dropDown, _this = this;
		if (!dropDown) { return; }
		dojo.forEach(_this.items, function(item){
			if ( item.value == _this.value ) {
			dropDown.addChild(new dijit.MenuItem({
					label: item.name, iconClass: "selectedIcon",
					onClick: function(){ _this._menuItemClick(item);}}));
			} else {
			dropDown.addChild(new dijit.MenuItem({
					label: item.name,
					onClick: function(){ _this._menuItemClick(item);}}));
			}
		});
		_this._isPopulated = true;
		hub.widgets.chrome.DropDownSelect.superclass._toggleDropDown.call(this);
	},
	_toggleDropDown: function(){
		var dropDown = this.dropDown;
		if (dropDown && !dropDown.isShowingNow && !this._isPopulated)
		{
			this.store.fetch({onComplete:dojo.hitch(this, "_loadCallback")});
			return;			
		}
		this.inherited(arguments);
	}
});

}

if(!dojo._hasResource["hub.install"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.install"] = true;
dojo.provide("hub.install");
dojo.provide("hub.install");



 
 





/**
 * Initializes the Mashup Hub installer. Sets up the global configuration and
 * URL manager.
 */
dojo.declare("hub.install.Main",        
	[dijit._Widget, dijit._Templated ],{
	templateString:"<div class=\"hubWidgetWizard\">\r\n\t\t<div id=\"sidebar\">\r\n\t\t    <div>\r\n\t\t      <img id=\"name\" src=\"../images/LogInProductname.gif\" alt=\"InfoSphere MashupHub\"/>\r\n\t\t\t  <img id=\"logo\" style=\"margin-left: 30px\" src=\"../images/LogInIllustration.gif\" alt=\"IBM InfoSphere MashupHub image\" />\r\n\t\t\t</div>\r\n\t\t\t<h1>Installation Steps</h1>\r\n\r\n\t\t\t<ol id=\"install_steps\" dojoAttachPoint=\"install_steps\">\r\n\t\t\t\t<li id=\"intro_step\">Introduction</li>\r\n\t\t\t\t<li id=\"license_step\">License</li>\r\n\t\t\t\t<li id=\"db_step\">Database</li>\r\n\t\t\t\t<li id=\"site_step\">Website</li>\r\n\t\t\t\t<li id=\"install_step\">Install</li>\r\n\t\t\t\t<li id=\"report_step\">Report</li>\r\n\t\t\t</ol>\r\n\t\t</div>\r\n\t\t<div class=\"wizard\" >\r\n\t\t\t<div id=\"mainTabContainer\" dojoType=\"dijit.layout.StackContainer\" dojoAttachPoint=\"tabContainer\"\r\n\t\t\t\t style=\"height:600px;\">\r\n\t\t\t\t <!-- style=\"width: 100%; border:1px solid #9b9b9b; margin: 0px 0px 20px 10px; padding: 0px;\"-->\r\n\r\n\t\t\t\t<!-- intro step -->\r\n\t\t\t\t\r\n\t\t\t\t<div id=\"intro\" dojoType=\"dijit.layout.ContentPane\">\r\n\t\t\t\t\t<div class=\"scrollable padded bordered panel\">\r\n\t\t\t\t\t\t<h2>Introduction</h2>\r\n\t\t\t\t\t\t<p>You are about to install IBM InfoSphere MashupHub version <span dojoAttachPoint=\"buildVersion\">@BUILD_VERSION.</span></p>\r\n\r\n\t\t\t\t\t\t<div dojoAttachPoint=\"preInstallCheck\" id=\"preInstallCheck\">\r\n\t\t\t\t\t\t\t<p>The installer is checking if all the requirements for installing Mashup Hub are satisfied. </p>\r\n\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t<div dojoAttachPoint=\"preInstallOk\" id=\"preInstallOk\" style=\"display: none;\">\r\n\t\t\t\t\t\t\t<p>The installer has detected that you have satisfied the basic prerequisites for installing Mashup Hub. The installer will check for additional requirements as you enter information.</p>\r\n\t\t\t\t\t\t\t<p>Click Next to begin.</p>\r\n\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t<div dojoAttachPoint=\"preInstallErrors\" id=\"preInstallErrors\" style=\"display: none;\">\r\n\t\t\t\t\t\t\t<p>The installer has detected that you have <b>not</b> satisifed the prerequisites for installing Mashup Hub. Please see the error messages below. Refer to the Getting Started guide for assistance in correcting these problems.</p>\r\n\t\t\t\t\t\t\t<ul dojoAttachPoint=\"preInstallErrorsUL\" id=\"preInstallErrorsUL\"></ul>\r\n\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t<p id=\"init_status\"></p>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\r\n\t\t\t\t<!-- license step -->\r\n\t\t\t\t\r\n\t\t\t\t<div id=\"license\" dojoType=\"dijit.layout.ContentPane\" href=\"license.html\" style=\"border: 1px solid black;\"></div>\r\n\r\n\t\t\t\t<!-- db step -->\r\n\t\t\t\t\r\n\t\t\t\t<div id=\"db\" dojoType=\"dijit.layout.ContentPane\" >\r\n\t\t\t\t\t<div class=\"scrollable padded bordered panel\">\r\n\t\t\t\t\t\t<h2>Database Configuration</h2>\r\n\r\n\t\t\t\t\t\t<p>Enter the following information about your database. The default values match those used in the Getting Started guide. All enabled fields require values.</p>\r\n\t\t\t\t\t\t<form id=\"wizard_form_db\" method=\"POST\" action=\"\">\r\n\t\t\t\t\t\t\t<label class=\"section\">Database</label>\r\n\t\t\r\n\t\t\t\t\t\t\t<label id=\"db_driver_label\" for=\"db_driver\">Driver:</label>\r\n\t\t\t\t\t\t\t<div class=\"right-column\">\r\n\t\t\t\t\t\t\t  <!-- select tabindex=\"1\" onchange=\"hub.install.selectDbDriver(this);\" name=\"db_driver\" id=\"db_driver\" -->\r\n\t\t\t\t\t\t\t  <select tabindex=\"1\" dojoAttachEvent=\"onchange:_selectDbDriver\" name=\"db_driver\" id=\"db_driver\" dojoAttachPoint=\"db_driver\">\r\n\t\t\t\t\t\t\t\t<option value='DB2' selected>DB2</option>\r\n\t\t\t\t\t\t\t\t<option value='IDS' selected>Informix</option>\r\n\t\t\t\t\t\t\t\t<option value='MYSQL'>MySQL</option>\r\n\t\t\t\t\t\t\t\t<option value='DERBY'>Derby</option>\r\n                                <option value='DERBY_EMBEDDED'>Embedded Derby</option>\r\n\t\t\t\t\t\t\t  </select>\r\n\t\t\t\t\t\t\t  <p class=\"helper\">Select the database server you have installed.</p>\r\n\t\t\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\r\n\t\t\t\t\t\t\t<label id=\"db_host_label\" for=\"db_host\">Hostname:</label>\r\n\t\t\t\t\t\t\t<div class=\"right-column\">\r\n\t\t\t\t\t\t\t  <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_host\" id=\"db_host\" dojoAttachPoint=\"db_host\" value=\"localhost\"/>\r\n\t\t\t\t\t\t\t  <p class=\"helper\">Enter the hostname of your database server.</p>\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\r\n\t\t\t\t\t\t\t<label id=\"db_port_label\" for=\"db_port\">Port:</label>\r\n\t\t\r\n\t\t\t\t\t\t\t<div class=\"right-column\">\r\n\t\t\t\t\t\t\t  <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_port\" id=\"db_port\" dojoAttachPoint=\"db_port\" value=\"\"/>\r\n\t\t\t\t\t\t\t  <p class=\"helper\">Enter the port of your database server.</p>\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\r\n\t\t\t\t\t\t\t<label id=\"db_name_label\" for=\"db_name\">Database Name:</label>\r\n\t\t\t\t\t\t\t<div class=\"right-column\">\r\n\t\t\t\t\t\t\t  <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_name\" id=\"db_name\" dojoAttachPoint=\"db_name\" value=\"hubdb\" />\r\n\t\t\t\t\t\t\t  <p class=\"helper\">Enter the name of the database.</p>\r\n\t\t\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t\t\r\n\t\t\t\t\t\t\t<label id=\"db_schema_label\" for=\"db_schema\">Schema:</label>\r\n\t\t\t\t\t\t\t<div class=\"right-column\">\r\n\t\t\t\t\t\t\t  <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_schema\" dojoAttachPoint=\"db_schema\" value=\"\" id=\"db_schema\"/>\r\n\t\t\t\t\t\t\t  <p class=\"helper\">Enter the schema used to access the database tables.</p>\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\r\n\t\t\t\t\t\t\t<label id=\"db_username_label\" for=\"db_username\">Username:</label>\r\n\t\t\r\n\t\t\t\t\t\t\t<div class=\"right-column\">\r\n\t\t\t\t\t\t\t  <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"db_username\" id=\"db_username\" dojoAttachPoint=\"db_username\" value=\"\" />\r\n\t\t\t\t\t\t\t  <p class=\"helper\">Enter the username you use to access to the database.</p>\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\r\n\t\t\t\t\t\t\t<label id=\"db_password_label\" for=\"db_password\">Password:</label>\r\n\t\t\t\t\t\t\t<div class=\"right-column\">\r\n\t\t\t\t\t\t\t  <input tabindex=\"1\" type=\"password\" size=\"40\" name=\"db_password\" id=\"db_password\" dojoAttachPoint=\"db_password\" value=\"\" />\r\n\t\t\t\t\t\t\t  <p class=\"helper\">Enter the password you use to connect to the database.</p>\r\n\t\t\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t</form>\t\r\n\t\t\t\t\t\t<p id=\"db_status\"></p>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\r\n\t\t\t\t<!-- site config step -->\r\n\t\t\t\t\r\n\t\t\t\t<div id=\"site\" dojoType=\"dijit.layout.ContentPane\">\r\n\t\t\t\t\t<div class=\"scrollable padded bordered panel\">\r\n\t\t\t\t\t\t<form id=\"wizard_form_site\" method=\"POST\" action=\"\">\r\n\t\t\t\t\t\t\t<h2>Website Configuration</h2>\r\n\t\t\r\n\t\t\t\t\t\t\t<p>Enter the following information about your website. The default values match those used in the Getting Started guide. All fields require values.</p>\r\n\t\t\r\n\t\t\t\t\t\t\t<label class=\"section\">Website</label>\r\n\t\t\t\t\t\t\t<label id=\"app_name_label\" for=\"app_name\">Website Name:</label>\r\n\t\t\t\t\t\t\t<div class=\"right-column\">\r\n\t\t\t\t\t\t\t  <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"app_name\" id=\"app_name\" dojoAttachPoint=\"app_name\" value=\"Mashup Hub\"/>\r\n\t\t\t\t\t\t\t  <p class=\"helper\">Enter a name for your site.</p>\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\r\n\t\t\t\t\t\t\t<label id=\"app_path_label\" for=\"app_path\">Application Path:</label>\r\n\t\t\r\n\t\t\t\t\t\t\t<div class=\"right-column\">\r\n\t\t\t\t\t\t\t  <input tabindex=\"1\" type=\"text\" size=\"40\" name=\"app_path\" id=\"app_path\" dojoAttachPoint=\"app_path\" value=\"\"/>\r\n\t\t\t\t\t\t\t  <p class=\"helper\">Enter the path where the Mashup Hub server code is located.</p>\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\r\n\t\t\t\t\t\t\t<label id=\"app_timezone_label\" for=\"app_timezone\">Timezone:</label>\r\n\t\t\t\t\t\t\t<div class=\"right-column\">\r\n\t\t\t\t\t\t\t  <select tabindex=\"1\" name=\"app_timezone\" id=\"app_timezone\" dojoAttachPoint=\"app_timezone\">\r\n\t\t\t\t\t\t\t\t<option value='Africa/Abidjan'>Africa/Abidjan</option><option value='Africa/Accra'>Africa/Accra</option><option value='Africa/Addis_Ababa'>Africa/Addis_Ababa</option><option value='Africa/Algiers'>Africa/Algiers</option><option value='Africa/Asmara'>Africa/Asmara</option><option value='Africa/Asmera'>Africa/Asmera</option><option value='Africa/Bamako'>Africa/Bamako</option><option value='Africa/Bangui'>Africa/Bangui</option><option value='Africa/Banjul'>Africa/Banjul</option><option value='Africa/Bissau'>Africa/Bissau</option><option value='Africa/Blantyre'>Africa/Blantyre</option><option value='Africa/Brazzaville'>Africa/Brazzaville</option><option value='Africa/Bujumbura'>Africa/Bujumbura</option><option value='Africa/Cairo'>Africa/Cairo</option><option value='Africa/Casablanca'>Africa/Casablanca</option><option value='Africa/Ceuta'>Africa/Ceuta</option><option value='Africa/Conakry'>Africa/Conakry</option><option value='Africa/Dakar'>Africa/Dakar</option><option value='Africa/Dar_es_Salaam'>Africa/Dar_es_Salaam</option><option value='Africa/Djibouti'>Africa/Djibouti</option><option value='Africa/Douala'>Africa/Douala</option><option value='Africa/El_Aaiun'>Africa/El_Aaiun</option><option value='Africa/Freetown'>Africa/Freetown</option><option value='Africa/Gaborone'>Africa/Gaborone</option><option value='Africa/Harare'>Africa/Harare</option><option value='Africa/Johannesburg'>Africa/Johannesburg</option><option value='Africa/Kampala'>Africa/Kampala</option><option value='Africa/Khartoum'>Africa/Khartoum</option><option value='Africa/Kigali'>Africa/Kigali</option><option value='Africa/Kinshasa'>Africa/Kinshasa</option><option value='Africa/Lagos'>Africa/Lagos</option><option value='Africa/Libreville'>Africa/Libreville</option><option value='Africa/Lome'>Africa/Lome</option><option value='Africa/Luanda'>Africa/Luanda</option><option value='Africa/Lubumbashi'>Africa/Lubumbashi</option><option value='Africa/Lusaka'>Africa/Lusaka</option><option value='Africa/Malabo'>Africa/Malabo</option><option value='Africa/Maputo'>Africa/Maputo</option><option value='Africa/Maseru'>Africa/Maseru</option><option value='Africa/Mbabane'>Africa/Mbabane</option><option value='Africa/Mogadishu'>Africa/Mogadishu</option><option value='Africa/Monrovia'>Africa/Monrovia</option><option value='Africa/Nairobi'>Africa/Nairobi</option><option value='Africa/Ndjamena'>Africa/Ndjamena</option><option value='Africa/Niamey'>Africa/Niamey</option><option value='Africa/Nouakchott'>Africa/Nouakchott</option><option value='Africa/Ouagadougou'>Africa/Ouagadougou</option><option value='Africa/Porto-Novo'>Africa/Porto-Novo</option><option value='Africa/Sao_Tome'>Africa/Sao_Tome</option><option value='Africa/Timbuktu'>Africa/Timbuktu</option><option value='Africa/Tripoli'>Africa/Tripoli</option><option value='Africa/Tunis'>Africa/Tunis</option><option value='Africa/Windhoek'>Africa/Windhoek</option><option value='America/Adak'>America/Adak</option><option value='America/Anchorage'>America/Anchorage</option><option value='America/Anguilla'>America/Anguilla</option><option value='America/Antigua'>America/Antigua</option><option value='America/Araguaina'>America/Araguaina</option><option value='America/Argentina/Buenos_Aires'>America/Argentina/Buenos_Aires</option><option value='America/Argentina/Catamarca'>America/Argentina/Catamarca</option><option value='America/Argentina/ComodRivadavia'>America/Argentina/ComodRivadavia</option><option value='America/Argentina/Cordoba'>America/Argentina/Cordoba</option><option value='America/Argentina/Jujuy'>America/Argentina/Jujuy</option><option value='America/Argentina/La_Rioja'>America/Argentina/La_Rioja</option><option value='America/Argentina/Mendoza'>America/Argentina/Mendoza</option><option value='America/Argentina/Rio_Gallegos'>America/Argentina/Rio_Gallegos</option><option value='America/Argentina/San_Juan'>America/Argentina/San_Juan</option><option value='America/Argentina/Tucuman'>America/Argentina/Tucuman</option><option value='America/Argentina/Ushuaia'>America/Argentina/Ushuaia</option><option value='America/Aruba'>America/Aruba</option><option value='America/Asuncion'>America/Asuncion</option><option value='America/Atikokan'>America/Atikokan</option><option value='America/Atka'>America/Atka</option><option value='America/Bahia'>America/Bahia</option><option value='America/Barbados'>America/Barbados</option><option value='America/Belem'>America/Belem</option><option value='America/Belize'>America/Belize</option><option value='America/Blanc-Sablon'>America/Blanc-Sablon</option><option value='America/Boa_Vista'>America/Boa_Vista</option><option value='America/Bogota'>America/Bogota</option><option value='America/Boise'>America/Boise</option><option value='America/Buenos_Aires'>America/Buenos_Aires</option><option value='America/Cambridge_Bay'>America/Cambridge_Bay</option><option value='America/Campo_Grande'>America/Campo_Grande</option><option value='America/Cancun'>America/Cancun</option><option value='America/Caracas'>America/Caracas</option><option value='America/Catamarca'>America/Catamarca</option><option value='America/Cayenne'>America/Cayenne</option><option value='America/Cayman'>America/Cayman</option><option value='America/Chicago'>America/Chicago</option><option value='America/Chihuahua'>America/Chihuahua</option><option value='America/Coral_Harbour'>America/Coral_Harbour</option><option value='America/Cordoba'>America/Cordoba</option><option value='America/Costa_Rica'>America/Costa_Rica</option><option value='America/Cuiaba'>America/Cuiaba</option><option value='America/Curacao'>America/Curacao</option><option value='America/Danmarkshavn'>America/Danmarkshavn</option><option value='America/Dawson'>America/Dawson</option><option value='America/Dawson_Creek'>America/Dawson_Creek</option><option value='America/Denver'>America/Denver</option><option value='America/Detroit'>America/Detroit</option><option value='America/Dominica'>America/Dominica</option><option value='America/Edmonton'>America/Edmonton</option><option value='America/Eirunepe'>America/Eirunepe</option><option value='America/El_Salvador'>America/El_Salvador</option><option value='America/Ensenada'>America/Ensenada</option><option value='America/Fort_Wayne'>America/Fort_Wayne</option><option value='America/Fortaleza'>America/Fortaleza</option><option value='America/Glace_Bay'>America/Glace_Bay</option><option value='America/Godthab'>America/Godthab</option><option value='America/Goose_Bay'>America/Goose_Bay</option><option value='America/Grand_Turk'>America/Grand_Turk</option><option value='America/Grenada'>America/Grenada</option><option value='America/Guadeloupe'>America/Guadeloupe</option><option value='America/Guatemala'>America/Guatemala</option><option value='America/Guayaquil'>America/Guayaquil</option><option value='America/Guyana'>America/Guyana</option><option value='America/Halifax'>America/Halifax</option><option value='America/Havana'>America/Havana</option><option value='America/Hermosillo'>America/Hermosillo</option><option value='America/Indiana/Indianapolis'>America/Indiana/Indianapolis</option><option value='America/Indiana/Knox'>America/Indiana/Knox</option><option value='America/Indiana/Marengo'>America/Indiana/Marengo</option><option value='America/Indiana/Petersburg'>America/Indiana/Petersburg</option><option value='America/Indiana/Vevay'>America/Indiana/Vevay</option><option value='America/Indiana/Vincennes'>America/Indiana/Vincennes</option><option value='America/Indiana/Winamac'>America/Indiana/Winamac</option><option value='America/Indianapolis'>America/Indianapolis</option><option value='America/Inuvik'>America/Inuvik</option><option value='America/Iqaluit'>America/Iqaluit</option><option value='America/Jamaica'>America/Jamaica</option><option value='America/Jujuy'>America/Jujuy</option><option value='America/Juneau'>America/Juneau</option><option value='America/Kentucky/Louisville'>America/Kentucky/Louisville</option><option value='America/Kentucky/Monticello'>America/Kentucky/Monticello</option><option value='America/Knox_IN'>America/Knox_IN</option><option value='America/La_Paz'>America/La_Paz</option><option value='America/Lima'>America/Lima</option><option value='America/Los_Angeles'>America/Los_Angeles</option><option value='America/Louisville'>America/Louisville</option><option value='America/Maceio'>America/Maceio</option><option value='America/Managua'>America/Managua</option><option value='America/Manaus'>America/Manaus</option><option value='America/Martinique'>America/Martinique</option><option value='America/Mazatlan'>America/Mazatlan</option><option value='America/Mendoza'>America/Mendoza</option><option value='America/Menominee'>America/Menominee</option><option value='America/Merida'>America/Merida</option><option value='America/Mexico_City'>America/Mexico_City</option><option value='America/Miquelon'>America/Miquelon</option><option value='America/Moncton'>America/Moncton</option><option value='America/Monterrey'>America/Monterrey</option><option value='America/Montevideo'>America/Montevideo</option><option value='America/Montreal'>America/Montreal</option><option value='America/Montserrat'>America/Montserrat</option><option value='America/Nassau'>America/Nassau</option><option value='America/New_York'>America/New_York</option><option value='America/Nipigon'>America/Nipigon</option><option value='America/Nome'>America/Nome</option><option value='America/Noronha'>America/Noronha</option><option value='America/North_Dakota/Center'>America/North_Dakota/Center</option><option value='America/North_Dakota/New_Salem'>America/North_Dakota/New_Salem</option><option value='America/Panama'>America/Panama</option><option value='America/Pangnirtung'>America/Pangnirtung</option><option value='America/Paramaribo'>America/Paramaribo</option><option value='America/Phoenix'>America/Phoenix</option><option value='America/Port-au-Prince'>America/Port-au-Prince</option><option value='America/Port_of_Spain'>America/Port_of_Spain</option><option value='America/Porto_Acre'>America/Porto_Acre</option><option value='America/Porto_Velho'>America/Porto_Velho</option><option value='America/Puerto_Rico'>America/Puerto_Rico</option><option value='America/Rainy_River'>America/Rainy_River</option><option value='America/Rankin_Inlet'>America/Rankin_Inlet</option><option value='America/Recife'>America/Recife</option><option value='America/Regina'>America/Regina</option><option value='America/Resolute'>America/Resolute</option><option value='America/Rio_Branco'>America/Rio_Branco</option><option value='America/Rosario'>America/Rosario</option><option value='America/Santiago'>America/Santiago</option><option value='America/Santo_Domingo'>America/Santo_Domingo</option><option value='America/Sao_Paulo'>America/Sao_Paulo</option><option value='America/Scoresbysund'>America/Scoresbysund</option><option value='America/Shiprock'>America/Shiprock</option><option value='America/St_Johns'>America/St_Johns</option><option value='America/St_Kitts'>America/St_Kitts</option><option value='America/St_Lucia'>America/St_Lucia</option><option value='America/St_Thomas'>America/St_Thomas</option><option value='America/St_Vincent'>America/St_Vincent</option><option value='America/Swift_Current'>America/Swift_Current</option><option value='America/Tegucigalpa'>America/Tegucigalpa</option><option value='America/Thule'>America/Thule</option><option value='America/Thunder_Bay'>America/Thunder_Bay</option><option value='America/Tijuana'>America/Tijuana</option><option value='America/Toronto'>America/Toronto</option><option value='America/Tortola'>America/Tortola</option><option value='America/Vancouver'>America/Vancouver</option><option value='America/Virgin'>America/Virgin</option><option value='America/Whitehorse'>America/Whitehorse</option><option value='America/Winnipeg'>America/Winnipeg</option><option value='America/Yakutat'>America/Yakutat</option><option value='America/Yellowknife'>America/Yellowknife</option><option value='Antarctica/Casey'>Antarctica/Casey</option><option value='Antarctica/Davis'>Antarctica/Davis</option><option value='Antarctica/DumontDUrville'>Antarctica/DumontDUrville</option><option value='Antarctica/Mawson'>Antarctica/Mawson</option><option value='Antarctica/McMurdo'>Antarctica/McMurdo</option><option value='Antarctica/Palmer'>Antarctica/Palmer</option><option value='Antarctica/Rothera'>Antarctica/Rothera</option><option value='Antarctica/South_Pole'>Antarctica/South_Pole</option><option value='Antarctica/Syowa'>Antarctica/Syowa</option><option value='Antarctica/Vostok'>Antarctica/Vostok</option><option value='Arctic/Longyearbyen'>Arctic/Longyearbyen</option><option value='Asia/Aden'>Asia/Aden</option><option value='Asia/Almaty'>Asia/Almaty</option><option value='Asia/Amman'>Asia/Amman</option><option value='Asia/Anadyr'>Asia/Anadyr</option><option value='Asia/Aqtau'>Asia/Aqtau</option><option value='Asia/Aqtobe'>Asia/Aqtobe</option><option value='Asia/Ashgabat'>Asia/Ashgabat</option><option value='Asia/Ashkhabad'>Asia/Ashkhabad</option><option value='Asia/Baghdad'>Asia/Baghdad</option><option value='Asia/Bahrain'>Asia/Bahrain</option><option value='Asia/Baku'>Asia/Baku</option><option value='Asia/Bangkok'>Asia/Bangkok</option><option value='Asia/Beirut'>Asia/Beirut</option><option value='Asia/Bishkek'>Asia/Bishkek</option><option value='Asia/Brunei'>Asia/Brunei</option><option value='Asia/Calcutta'>Asia/Calcutta</option><option value='Asia/Choibalsan'>Asia/Choibalsan</option><option value='Asia/Chongqing'>Asia/Chongqing</option><option value='Asia/Chungking'>Asia/Chungking</option><option value='Asia/Colombo'>Asia/Colombo</option><option value='Asia/Dacca'>Asia/Dacca</option><option value='Asia/Damascus'>Asia/Damascus</option><option value='Asia/Dhaka'>Asia/Dhaka</option><option value='Asia/Dili'>Asia/Dili</option><option value='Asia/Dubai'>Asia/Dubai</option><option value='Asia/Dushanbe'>Asia/Dushanbe</option><option value='Asia/Gaza'>Asia/Gaza</option><option value='Asia/Harbin'>Asia/Harbin</option><option value='Asia/Hong_Kong'>Asia/Hong_Kong</option><option value='Asia/Hovd'>Asia/Hovd</option><option value='Asia/Irkutsk'>Asia/Irkutsk</option><option value='Asia/Istanbul'>Asia/Istanbul</option><option value='Asia/Jakarta'>Asia/Jakarta</option><option value='Asia/Jayapura'>Asia/Jayapura</option><option value='Asia/Jerusalem'>Asia/Jerusalem</option><option value='Asia/Kabul'>Asia/Kabul</option><option value='Asia/Kamchatka'>Asia/Kamchatka</option><option value='Asia/Karachi'>Asia/Karachi</option><option value='Asia/Kashgar'>Asia/Kashgar</option><option value='Asia/Katmandu'>Asia/Katmandu</option><option value='Asia/Krasnoyarsk'>Asia/Krasnoyarsk</option><option value='Asia/Kuala_Lumpur'>Asia/Kuala_Lumpur</option><option value='Asia/Kuching'>Asia/Kuching</option><option value='Asia/Kuwait'>Asia/Kuwait</option><option value='Asia/Macao'>Asia/Macao</option><option value='Asia/Macau'>Asia/Macau</option><option value='Asia/Magadan'>Asia/Magadan</option><option value='Asia/Makassar'>Asia/Makassar</option><option value='Asia/Manila'>Asia/Manila</option><option value='Asia/Muscat'>Asia/Muscat</option><option value='Asia/Nicosia'>Asia/Nicosia</option><option value='Asia/Novosibirsk'>Asia/Novosibirsk</option><option value='Asia/Omsk'>Asia/Omsk</option><option value='Asia/Oral'>Asia/Oral</option><option value='Asia/Phnom_Penh'>Asia/Phnom_Penh</option><option value='Asia/Pontianak'>Asia/Pontianak</option><option value='Asia/Pyongyang'>Asia/Pyongyang</option><option value='Asia/Qatar'>Asia/Qatar</option><option value='Asia/Qyzylorda'>Asia/Qyzylorda</option><option value='Asia/Rangoon'>Asia/Rangoon</option><option value='Asia/Riyadh'>Asia/Riyadh</option><option value='Asia/Saigon'>Asia/Saigon</option><option value='Asia/Sakhalin'>Asia/Sakhalin</option><option value='Asia/Samarkand'>Asia/Samarkand</option><option value='Asia/Seoul'>Asia/Seoul</option><option value='Asia/Shanghai'>Asia/Shanghai</option><option value='Asia/Singapore'>Asia/Singapore</option><option value='Asia/Taipei'>Asia/Taipei</option><option value='Asia/Tashkent'>Asia/Tashkent</option><option value='Asia/Tbilisi'>Asia/Tbilisi</option><option value='Asia/Tehran'>Asia/Tehran</option><option value='Asia/Tel_Aviv'>Asia/Tel_Aviv</option><option value='Asia/Thimbu'>Asia/Thimbu</option><option value='Asia/Thimphu'>Asia/Thimphu</option><option value='Asia/Tokyo'>Asia/Tokyo</option><option value='Asia/Ujung_Pandang'>Asia/Ujung_Pandang</option><option value='Asia/Ulaanbaatar'>Asia/Ulaanbaatar</option><option value='Asia/Ulan_Bator'>Asia/Ulan_Bator</option><option value='Asia/Urumqi'>Asia/Urumqi</option><option value='Asia/Vientiane'>Asia/Vientiane</option><option value='Asia/Vladivostok'>Asia/Vladivostok</option><option value='Asia/Yakutsk'>Asia/Yakutsk</option><option value='Asia/Yekaterinburg'>Asia/Yekaterinburg</option><option value='Asia/Yerevan'>Asia/Yerevan</option><option value='Atlantic/Azores'>Atlantic/Azores</option><option value='Atlantic/Bermuda'>Atlantic/Bermuda</option><option value='Atlantic/Canary'>Atlantic/Canary</option><option value='Atlantic/Cape_Verde'>Atlantic/Cape_Verde</option><option value='Atlantic/Faeroe'>Atlantic/Faeroe</option><option value='Atlantic/Faroe'>Atlantic/Faroe</option><option value='Atlantic/Jan_Mayen'>Atlantic/Jan_Mayen</option><option value='Atlantic/Madeira'>Atlantic/Madeira</option><option value='Atlantic/Reykjavik'>Atlantic/Reykjavik</option><option value='Atlantic/South_Georgia'>Atlantic/South_Georgia</option><option value='Atlantic/St_Helena'>Atlantic/St_Helena</option><option value='Atlantic/Stanley'>Atlantic/Stanley</option><option value='Australia/ACT'>Australia/ACT</option><option value='Australia/Adelaide'>Australia/Adelaide</option><option value='Australia/Brisbane'>Australia/Brisbane</option><option value='Australia/Broken_Hill'>Australia/Broken_Hill</option><option value='Australia/Canberra'>Australia/Canberra</option><option value='Australia/Currie'>Australia/Currie</option><option value='Australia/Darwin'>Australia/Darwin</option><option value='Australia/Eucla'>Australia/Eucla</option><option value='Australia/Hobart'>Australia/Hobart</option><option value='Australia/LHI'>Australia/LHI</option><option value='Australia/Lindeman'>Australia/Lindeman</option><option value='Australia/Lord_Howe'>Australia/Lord_Howe</option><option value='Australia/Melbourne'>Australia/Melbourne</option><option value='Australia/North'>Australia/North</option><option value='Australia/NSW'>Australia/NSW</option><option value='Australia/Perth'>Australia/Perth</option><option value='Australia/Queensland'>Australia/Queensland</option><option value='Australia/South'>Australia/South</option><option value='Australia/Sydney'>Australia/Sydney</option><option value='Australia/Tasmania'>Australia/Tasmania</option><option value='Australia/Victoria'>Australia/Victoria</option><option value='Australia/West'>Australia/West</option><option value='Australia/Yancowinna'>Australia/Yancowinna</option><option value='Brazil/Acre'>Brazil/Acre</option><option value='Brazil/DeNoronha'>Brazil/DeNoronha</option><option value='Brazil/East'>Brazil/East</option><option value='Brazil/West'>Brazil/West</option><option value='Canada/Atlantic'>Canada/Atlantic</option><option value='Canada/Central'>Canada/Central</option><option value='Canada/East-Saskatchewan'>Canada/East-Saskatchewan</option><option value='Canada/Eastern'>Canada/Eastern</option><option value='Canada/Mountain'>Canada/Mountain</option><option value='Canada/Newfoundland'>Canada/Newfoundland</option><option value='Canada/Pacific'>Canada/Pacific</option><option value='Canada/Saskatchewan'>Canada/Saskatchewan</option><option value='Canada/Yukon'>Canada/Yukon</option><option value='CET'>CET</option><option value='Chile/Continental'>Chile/Continental</option><option value='Chile/EasterIsland'>Chile/EasterIsland</option><option value='CST6CDT'>CST6CDT</option><option value='Cuba'>Cuba</option><option value='EET'>EET</option><option value='Egypt'>Egypt</option><option value='Eire'>Eire</option><option value='EST'>EST</option><option value='EST5EDT'>EST5EDT</option><option value='Etc/GMT'>Etc/GMT</option><option value='Etc/GMT+0'>Etc/GMT+0</option><option value='Etc/GMT+1'>Etc/GMT+1</option><option value='Etc/GMT+10'>Etc/GMT+10</option><option value='Etc/GMT+11'>Etc/GMT+11</option><option value='Etc/GMT+12'>Etc/GMT+12</option><option value='Etc/GMT+2'>Etc/GMT+2</option><option value='Etc/GMT+3'>Etc/GMT+3</option><option value='Etc/GMT+4'>Etc/GMT+4</option><option value='Etc/GMT+5'>Etc/GMT+5</option><option value='Etc/GMT+6'>Etc/GMT+6</option><option value='Etc/GMT+7'>Etc/GMT+7</option><option value='Etc/GMT+8'>Etc/GMT+8</option><option value='Etc/GMT+9'>Etc/GMT+9</option><option value='Etc/GMT-0'>Etc/GMT-0</option><option value='Etc/GMT-1'>Etc/GMT-1</option><option value='Etc/GMT-10'>Etc/GMT-10</option><option value='Etc/GMT-11'>Etc/GMT-11</option><option value='Etc/GMT-12'>Etc/GMT-12</option><option value='Etc/GMT-13'>Etc/GMT-13</option><option value='Etc/GMT-14'>Etc/GMT-14</option><option value='Etc/GMT-2'>Etc/GMT-2</option><option value='Etc/GMT-3'>Etc/GMT-3</option><option value='Etc/GMT-4'>Etc/GMT-4</option><option value='Etc/GMT-5'>Etc/GMT-5</option><option value='Etc/GMT-6'>Etc/GMT-6</option><option value='Etc/GMT-7'>Etc/GMT-7</option><option value='Etc/GMT-8'>Etc/GMT-8</option><option value='Etc/GMT-9'>Etc/GMT-9</option><option value='Etc/GMT0'>Etc/GMT0</option><option value='Etc/Greenwich'>Etc/Greenwich</option><option value='Etc/UCT'>Etc/UCT</option><option value='Etc/Universal'>Etc/Universal</option><option value='Etc/UTC'>Etc/UTC</option><option value='Etc/Zulu'>Etc/Zulu</option><option value='Europe/Amsterdam'>Europe/Amsterdam</option><option value='Europe/Andorra'>Europe/Andorra</option><option value='Europe/Athens'>Europe/Athens</option><option value='Europe/Belfast'>Europe/Belfast</option><option value='Europe/Belgrade'>Europe/Belgrade</option><option value='Europe/Berlin'>Europe/Berlin</option><option value='Europe/Bratislava'>Europe/Bratislava</option><option value='Europe/Brussels'>Europe/Brussels</option><option value='Europe/Bucharest'>Europe/Bucharest</option><option value='Europe/Budapest'>Europe/Budapest</option><option value='Europe/Chisinau'>Europe/Chisinau</option><option value='Europe/Copenhagen'>Europe/Copenhagen</option><option value='Europe/Dublin'>Europe/Dublin</option><option value='Europe/Gibraltar'>Europe/Gibraltar</option><option value='Europe/Guernsey'>Europe/Guernsey</option><option value='Europe/Helsinki' selected>Europe/Helsinki</option><option value='Europe/Isle_of_Man'>Europe/Isle_of_Man</option><option value='Europe/Istanbul'>Europe/Istanbul</option><option value='Europe/Jersey'>Europe/Jersey</option><option value='Europe/Kaliningrad'>Europe/Kaliningrad</option><option value='Europe/Kiev'>Europe/Kiev</option><option value='Europe/Lisbon'>Europe/Lisbon</option><option value='Europe/Ljubljana'>Europe/Ljubljana</option><option value='Europe/London'>Europe/London</option><option value='Europe/Luxembourg'>Europe/Luxembourg</option><option value='Europe/Madrid'>Europe/Madrid</option><option value='Europe/Malta'>Europe/Malta</option><option value='Europe/Mariehamn'>Europe/Mariehamn</option><option value='Europe/Minsk'>Europe/Minsk</option><option value='Europe/Monaco'>Europe/Monaco</option><option value='Europe/Moscow'>Europe/Moscow</option><option value='Europe/Nicosia'>Europe/Nicosia</option><option value='Europe/Oslo'>Europe/Oslo</option><option value='Europe/Paris'>Europe/Paris</option><option value='Europe/Podgorica'>Europe/Podgorica</option><option value='Europe/Prague'>Europe/Prague</option><option value='Europe/Riga'>Europe/Riga</option><option value='Europe/Rome'>Europe/Rome</option><option value='Europe/Samara'>Europe/Samara</option><option value='Europe/San_Marino'>Europe/San_Marino</option><option value='Europe/Sarajevo'>Europe/Sarajevo</option><option value='Europe/Simferopol'>Europe/Simferopol</option><option value='Europe/Skopje'>Europe/Skopje</option><option value='Europe/Sofia'>Europe/Sofia</option><option value='Europe/Stockholm'>Europe/Stockholm</option><option value='Europe/Tallinn'>Europe/Tallinn</option><option value='Europe/Tirane'>Europe/Tirane</option><option value='Europe/Tiraspol'>Europe/Tiraspol</option><option value='Europe/Uzhgorod'>Europe/Uzhgorod</option><option value='Europe/Vaduz'>Europe/Vaduz</option><option value='Europe/Vatican'>Europe/Vatican</option><option value='Europe/Vienna'>Europe/Vienna</option><option value='Europe/Vilnius'>Europe/Vilnius</option><option value='Europe/Volgograd'>Europe/Volgograd</option><option value='Europe/Warsaw'>Europe/Warsaw</option><option value='Europe/Zagreb'>Europe/Zagreb</option><option value='Europe/Zaporozhye'>Europe/Zaporozhye</option><option value='Europe/Zurich'>Europe/Zurich</option><option value='Factory'>Factory</option><option value='GB'>GB</option><option value='GB-Eire'>GB-Eire</option><option value='GMT'>GMT</option><option value='GMT+0'>GMT+0</option><option value='GMT-0'>GMT-0</option><option value='GMT0'>GMT0</option><option value='Greenwich'>Greenwich</option><option value='Hongkong'>Hongkong</option><option value='HST'>HST</option><option value='Iceland'>Iceland</option><option value='Indian/Antananarivo'>Indian/Antananarivo</option><option value='Indian/Chagos'>Indian/Chagos</option><option value='Indian/Christmas'>Indian/Christmas</option><option value='Indian/Cocos'>Indian/Cocos</option><option value='Indian/Comoro'>Indian/Comoro</option><option value='Indian/Kerguelen'>Indian/Kerguelen</option><option value='Indian/Mahe'>Indian/Mahe</option><option value='Indian/Maldives'>Indian/Maldives</option><option value='Indian/Mauritius'>Indian/Mauritius</option><option value='Indian/Mayotte'>Indian/Mayotte</option><option value='Indian/Reunion'>Indian/Reunion</option><option value='Iran'>Iran</option><option value='Israel'>Israel</option><option value='Jamaica'>Jamaica</option><option value='Japan'>Japan</option><option value='Kwajalein'>Kwajalein</option><option value='Libya'>Libya</option><option value='MET'>MET</option><option value='Mexico/BajaNorte'>Mexico/BajaNorte</option><option value='Mexico/BajaSur'>Mexico/BajaSur</option><option value='Mexico/General'>Mexico/General</option><option value='MST'>MST</option><option value='MST7MDT'>MST7MDT</option><option value='Navajo'>Navajo</option><option value='NZ'>NZ</option><option value='NZ-CHAT'>NZ-CHAT</option><option value='Pacific/Apia'>Pacific/Apia</option><option value='Pacific/Auckland'>Pacific/Auckland</option><option value='Pacific/Chatham'>Pacific/Chatham</option><option value='Pacific/Easter'>Pacific/Easter</option><option value='Pacific/Efate'>Pacific/Efate</option><option value='Pacific/Enderbury'>Pacific/Enderbury</option><option value='Pacific/Fakaofo'>Pacific/Fakaofo</option><option value='Pacific/Fiji'>Pacific/Fiji</option><option value='Pacific/Funafuti'>Pacific/Funafuti</option><option value='Pacific/Galapagos'>Pacific/Galapagos</option><option value='Pacific/Gambier'>Pacific/Gambier</option><option value='Pacific/Guadalcanal'>Pacific/Guadalcanal</option><option value='Pacific/Guam'>Pacific/Guam</option><option value='Pacific/Honolulu'>Pacific/Honolulu</option><option value='Pacific/Johnston'>Pacific/Johnston</option><option value='Pacific/Kiritimati'>Pacific/Kiritimati</option><option value='Pacific/Kosrae'>Pacific/Kosrae</option><option value='Pacific/Kwajalein'>Pacific/Kwajalein</option><option value='Pacific/Majuro'>Pacific/Majuro</option><option value='Pacific/Marquesas'>Pacific/Marquesas</option><option value='Pacific/Midway'>Pacific/Midway</option><option value='Pacific/Nauru'>Pacific/Nauru</option><option value='Pacific/Niue'>Pacific/Niue</option><option value='Pacific/Norfolk'>Pacific/Norfolk</option><option value='Pacific/Noumea'>Pacific/Noumea</option><option value='Pacific/Pago_Pago'>Pacific/Pago_Pago</option><option value='Pacific/Palau'>Pacific/Palau</option><option value='Pacific/Pitcairn'>Pacific/Pitcairn</option><option value='Pacific/Ponape'>Pacific/Ponape</option><option value='Pacific/Port_Moresby'>Pacific/Port_Moresby</option><option value='Pacific/Rarotonga'>Pacific/Rarotonga</option><option value='Pacific/Saipan'>Pacific/Saipan</option><option value='Pacific/Samoa'>Pacific/Samoa</option><option value='Pacific/Tahiti'>Pacific/Tahiti</option><option value='Pacific/Tarawa'>Pacific/Tarawa</option><option value='Pacific/Tongatapu'>Pacific/Tongatapu</option><option value='Pacific/Truk'>Pacific/Truk</option><option value='Pacific/Wake'>Pacific/Wake</option><option value='Pacific/Wallis'>Pacific/Wallis</option><option value='Pacific/Yap'>Pacific/Yap</option><option value='Poland'>Poland</option><option value='Portugal'>Portugal</option><option value='PRC'>PRC</option><option value='PST8PDT'>PST8PDT</option><option value='ROC'>ROC</option><option value='ROK'>ROK</option><option value='Singapore'>Singapore</option><option value='Turkey'>Turkey</option><option value='UCT'>UCT</option><option value='Universal'>Universal</option><option value='US/Alaska'>US/Alaska</option><option value='US/Aleutian'>US/Aleutian</option><option value='US/Arizona'>US/Arizona</option><option value='US/Central'>US/Central</option><option value='US/East-Indiana'>US/East-Indiana</option><option value='US/Eastern'>US/Eastern</option><option value='US/Hawaii'>US/Hawaii</option><option value='US/Indiana-Starke'>US/Indiana-Starke</option><option value='US/Michigan'>US/Michigan</option><option value='US/Mountain'>US/Mountain</option><option value='US/Pacific'>US/Pacific</option><option value='US/Pacific-New'>US/Pacific-New</option><option value='US/Samoa'>US/Samoa</option><option value='UTC'>UTC</option><option value='W-SU'>W-SU</option><option value='WET'>WET</option><option value='Zulu'>Zulu</option>      </select>\r\n\t\t\r\n\t\t\t\t\t\t\t  <p class=\"helper\">Select the local timezone of your web server.</p>\r\n\t\t\r\n\t\t\t\t\t\t\t</div>\r\n\t\t\t\t\t\t</form>\r\n\t\t\t\t\t\t<p id=\"site_status\"></p>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t\t\r\n\t\t\t\t<!-- install step -->\r\n\t\t\t\t\r\n\t\t\t\t<div id=\"install\" dojoType=\"dijit.layout.ContentPane\">\r\n\t\t\t\t\t<div class=\"scrollable padded bordered panel\">\r\n\t\t\t\t\t\t<h2>Ready to Install</h2>\r\n\t\t\t\t\t\t <p>You have entered all of the information required to install Mashup Hub. Please review your input below.</p>\r\n\r\n\t\t\t\t\t\t <div id=\"install_review\"></div>\r\n\r\n\t\t\t\t\t\t <p>Click Previous if you need to go back and change your input. Otherwise, click Install to begin the installation process.</p>\r\n\r\n\t\t\t\t\t\t <p class=\"centered\" id=\"install_status\"></p>\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t\t<div id=\"report\" dojoType=\"dijit.layout.ContentPane\">\r\n\t\t\t\t\t<div class=\"scrollable padded bordered panel\">\r\n\t\t\t\t\t\t<h2>Success</h2>\r\n\t\t\t\t\t\t\r\n\t\t\t\t\t\t<p>Mashup Hub is now installed on your Web server.</p>\r\n\t\t\t\t\t\t<p>Click Finish to proceed to the Mashup Hub homepage.</p>\r\n\t\t\t\t\r\n\t\t\t\t\t</div>\r\n\t\t\t\t</div>\r\n\t\t\t</div>\r\n\t\t\t<div style=\"text-align: right; padding: 10px 0px 10px 0px;\">\r\n\t\t\t\t<button id=\"buttonDecline\" dojoAttachPoint=\"buttonDecline\" dojoAttachEvent=\"onclick:_declineLicense\" tabindex=\"3\" style=\"visibility: hidden;\">I Decline</button>\r\n\t\t\t\t<button id=\"buttonPrevious\" dojoAttachPoint=\"buttonPrevious\" dojoAttachEvent=\"onclick:_onPrevious\" tabindex=\"2\">Previous</button> \r\n\t\t\t\t<button id=\"buttonNext\" dojoAttachPoint=\"buttonNext\" dojoAttachEvent=\"onclick:_onNext\" tabindex=\"1\">Next</button>\r\n\t\t\t</div>\r\n\t\t</div>\r\n\t\t\r\n</div>\r\n",
	// Necessary to keep Dijit from using templateString
	templateString: "",
	// src: String
	// src url for header
	src: "",
	widgetsInTemplate: true,

    initStep: 'intro',
    
    localAuth: '',
    db2Driver: 'DB2',
    db2Port: 50000,
    mysqlDriver: 'MYSQL',
    mysqlPort: 3306,
    derbyDriver: 'DERBY',
    derbyPort: 1527,
    embeddedDerbyDriver: 'DERBY_EMBEDDED',
    idsDriver: 'IDS',
    idsPort: 9089,
   
    isInitialized: false,
    
    postMixInProperties: function() {
        // create base URL first
        var split = dojo.baseUrl.split('/');
        // based on dojo-1.0.1 folder location only
        split = split.slice(0, -4);

        // global configuration
        hub.config = {base_url: split.join('/'),
                      db2_port: this.db2Port,
                      db2_driver: this.db2Driver,
                      mysql_port: this.mysqlPort,
                      mysql_driver: this.mysqlDriver,
                      derby_port: this.derbyPort,
                      derby_driver: this.derbyDriver,
                      embedded_derby_driver: this.embeddedDerbyDriver,
                      ids_port: this.idsPort,
                      ids_driver: this.idsDriver,
                      max_label_length: 15};

        // create the URL manager
        
        hub.urls = new hub.managers.Urls(hub.config.base_url);

    },

    startup: function() {
        // initialize DB driver selection
        this._initialize();
        
        // show initial step
		var wiz = this.tabContainer;
        wiz.selectChild(this.initStep);
        this._updateControls(this.initStep);
    },

    /**
     * Flip the stack to the next page and highlight the associated step name.
     */
    _onNext: function() {
        // push stack container to next item
		// var wiz = this.tabContainer;
		var id = this.tabContainer.selectedChildWidget.id;
		switch(id){
			case 'db':
				this._checkDb();
				break;
			case 'site':
				this._checkSite();
	            this._onPreviewInstall();
				break;
			case 'install':
				this._doInstall();
				break;
			case 'report':
				window.location='/mashuphub/secured';
				break;
			default:	
		        this.tabContainer.forward();
		}
		id = this.tabContainer.selectedChildWidget.id;
        this._updateControls(id);
    },

    /**
     * Flip the stack to the prior  page and highlight the associated step name.
     */
    _onPrevious: function() {
        // push stack container to next item
		var wiz = this.tabContainer;
        wiz.back();
 		var id = wiz.selectedChildWidget.id;
        // get the list of steps
        this._updateControls(id);
    },

	_performStepAction: function(step){
	},

	/**
	 * Update Previous|Next Buttons
	 */
	_updateControls: function(step){
		var buttonDecline = this.buttonDecline;
		var buttonPrevious = this.buttonPrevious;
		var buttonNext = this.buttonNext;
		
		buttonDecline.style.visibility = "hidden";
		buttonPrevious.disabled = "";
		buttonNext.innerHTML = "Next";

		switch(step){
			case 'intro':
				buttonPrevious.disabled = "true";
				break;
			case 'license':
				buttonDecline.style.visibility = "visible";
				buttonNext.innerHTML = "I Accept";
				break;
			case 'db':
				break;
			case 'site':
				break;
			case 'install':
				buttonNext.innerHTML = "Install";			
				break;
			case 'report':
				buttonNext.innerHTML = "Finish";
				buttonPrevious.disabled = "true";
				break;
		}
        var steps = this.install_steps;
        dojo.query("li", steps).removeClass("selected");
        // highlight newly activated step
        dojo.query('li[id="' + step + '_step"]', steps).addClass("selected");
		
	},

    /**
     * Generate a report of all selected options before installing.
     */
    _onPreviewInstall: function() {
        var list = dojo.byId("install_review");
        var labels = dojo.query('label');
        // reset previous text
        list.innerHTML = "";

        for(var i=0; i < labels.length; i++) {
            var label = labels[i];
            if(label.className == 'section') {
                // create a heading
                var head = document.createElement("H3");
                head.innerHTML = label.innerHTML;
                // start a new list
                var ul = document.createElement("UL");
                list.appendChild(head);
                list.appendChild(ul);
            } 
            else {
                var li = document.createElement("LI");
                li.innerHTML = label.innerHTML;
                var controls = dojo.query("[name="+label.htmlFor+"]");
                // assume first match
                var c = controls[0];
                if(c.type != "password" && !c.disabled) {
                    li.innerHTML += " "+controls[0].value;
                    ul.appendChild(li);
                }
            }
        }
    },

    /**
     * Generate a report of all pre-install errors before installing.
     */
    _onPreInstallErrors: function(errors) {
		this.preInstallOk.style.display = "none";
		this.preInstallErrors.style.display = "block";
		dojo.forEach(errors, function(error){
            var li = document.createElement("li");
            li.innerHTML = error;
            dojo.byId("preInstallErrorsUL").appendChild(li);
		});
		this.buttonNext.disabled = "true";
		
    },

	_reqStatus: function(id, fields) {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    if(fields.length == 1) {
	        // just one missing field, state it
	        var ml = dojo.string.substitute(msgs.ERROR_INSTALL_MISSING_VALUE, 
	            [fields[0]]);
	    } else if(fields.length > 1) {
	        // multiple missing fields, format them
	        var ml = '';
	        for(var i=0; i<fields.length; i++) {
	            if (i < fields.length - 1) {
	                ml += fields[i] + ', ';
	            } else {
	                ml += fields[i];
	            }
	        }
	        ml = dojo.string.substitute(msgs.ERROR_INSTALL_MISSING_VALUES, [ml]);
	    }
	    this._helpStatus(id, ml);
	},
	
	/**
	 * Shows a help message after a failure to verify entered data.
	 *
	 * @param id: Where to show the message
	 * @param text: Message
	 */
	_helpStatus: function(id, text) {
	    var e = dojo.byId(id);
	    e.scrollIntoView();
	    var url = hub.urls.getImage('help-browser.png');
	    e.innerHTML = dojo.string.substitute('<img src="${0}" alt="Help icon" /> ${1}', [url, text]);
	},
	
	/**
	 * Clears a help message or busy status.
	 *
	 * @param id: Where to clear the message
	 */
	_clearStatus: function(id) {
	    var e = dojo.byId(id);
	    e.scrollIntoView();
	    e.innerHTML = '';
	},
	
	/**
	 * Shows a busy message while verifying data with a server.
	 *
	 * @param id: Where to show the message
	 * @param text: Message
	 */
	_busyStatus: function(id, text) {
	    var e = dojo.byId(id);
	    e.scrollIntoView();
	    var url = hub.urls.getImage('spinner.gif');
	    e.innerHTML = dojo.string.substitute('<img src="${0}" alt="Busy icon" /> ${1}', [url, text]);
	},
	
	/**
	 * Queries the server to verify entered data such as database server, 
	 * authentication server, etc. Publishes a request to continue to the next page
	 * on success.
	 *
	 * @param id: Where to post busy/help messages
	 * @param action: Action indicating what method should verify on the server
	 * @param form: Data to verify, possibly including more than what will be verified
	 *   (i.e., the entire form, not just the current page)
	 */
	_queryServer: function(id, action, form) {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    // go to server and check the connection
	    var kw = {
		url: hub.urls.postInstall(action),
		sync: true,
		form: form,
	        handleAs: 'json',
	        timeout: 120000, // 2 minutes in milliseconds
	        
	        error: dojo.hitch(this, function(response, args) {           
	            if(response.dojoType == "cancel") {
	                this._helpStatus(id, msgs.ERROR_REQUEST_CANCELLED);
	            } else if (response.dojoType == "timeout"){
	                this._helpStatus(id, msgs.ERROR_REQUEST_TIMEOUT);
	            } else {
	                this._helpStatus(id, response.message);
	            }
	            return response;
	        }),
	        
	        load: dojo.hitch(this, function(response, args) {
	            if(!response) {
	                this._helpStatus(id, msgs.ERROR_REQUEST_SERVER);
	            } 
	            else if(response.error) {
	                this._helpStatus(id, response.message);
	            } 
	            else {
	                this._clearStatus(id);
//	                this._next();
					this.tabContainer.forward();
	            }
	            return response;
	        })
	    };
	    // do the post
	    dojo.xhrPost(kw);
//		dojo._blockAsync = false;
	},
	
	/**
	 * Shows an alert indicating the license agreement must be accepted before
	 * continuing.
	 */
	_declineLicense: function() {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    alert(msgs.ERROR_INSTALL_LICENSE);
	},
	
	_setVars: function(vars){
		this.app_name.value = vars.app_name;
		this.app_path.value = vars.app_path;
		this.app_timezone.value = vars.app_timezone;
		this.db_driver.value = vars.db_driver;
		this.db_host.value = vars.db_host;
		this.db_name.value = vars.db_name;
		this.db_password.value = vars.db_password;
		this.db_port.value = vars.db_port;
		this.db_schema.value = vars.db_schema;	
		this.db_username.value = vars.db_username;
	},
	
	/**
	 * Updates default database field values based on the selected driver.
	 *
	 * @param widget: Driver selection box 
	 */
	_selectDbDriver: function() {
	    var driver = dojo.byId('db_driver');
	    var schema = dojo.byId('db_schema');
	    var port = dojo.byId('db_port');
	    
	    switch(driver.value){
	    	case hub.config.mysql_driver:
		        schema.disabled = true;
		        schema.className = 'disabled';
		        port.value = hub.config.mysql_port;
		        port.disabled = true;
		        port.className = 'disabled';
	    		break;
	    	case hub.config.derby_driver:
	    	case hub.config.embedded_derby_driver:
		        schema.disabled = true;
		        schema.className = 'disabled';
		        port.value = hub.config.derby_port;
		        port.disabled = true;
		        port.className = 'disabled';
	    		break;
	    	case hub.config.ids_driver:
	    		schema.disabled = true;
		        schema.className = 'disabled';
		        port.value = hub.config.ids_port;
		        port.disabled = false;
		        port.className = '';
	    		break;
	    	case hub.config.db2_driver:
	    	default:
		        schema.disabled = false;
		        schema.className = '';
		        port.value = hub.config.db2_port;
		        port.disabled = false;
		        port.className = '';
	    }
/*	    if(widget.value != hub.config.db2_driver) {
	        schema.disabled = true;
	        schema.className = 'disabled';
	        port.value = hub.config.mysql_port;
	        port.disabled = true;
	        port.className = 'disabled';
	    } else {
	        schema.disabled = false;
	        schema.className = '';
	        port.value = hub.config.db2_port;
	        port.disabled = false;
	        port.className = '';
	    }*/
	},
	
	
	/**
	 * Loads defaults from server
	 *
	 */
	_initialize: function(){
		if(this.isInitialized) return;
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
       	var id = 'init_status';           
	    this._busyStatus(id, "Initializing...");
	    var kw = {
			url: hub.urls.postInstall("init"),
		    handleAs: 'json',
	        timeout: 120000, // 2 minutes in milliseconds
	        
	        error: dojo.hitch(this, function(response, args) {
	            if(response.dojoType == "cancel") {
	                this._helpStatus(id, msgs.ERROR_REQUEST_CANCELLED);
	            } else if (response.dojoType == "timeout"){
	                this._helpStatus(id, msgs.ERROR_REQUEST_TIMEOUT);
	            } else {
	                this._helpStatus(id, response.message);
	            }
	            return response;
	        }),
	        
	        load: dojo.hitch(this, function(response, args) {
	            if(!response) {
	                this._helpStatus(id, msgs.ERROR_REQUEST_SERVER);
	            } 
	            else if(response.error) {
	                this._clearStatus(id);
					this.preInstallCheck.style.display = "none";
					this.preInstallOk.style.display = "none";
					this.preInstallErrors.style.display = "block";
					if(response.messages) {
						dojo.forEach(response.messages, function(error){
				            var li = document.createElement("li");
				            li.innerHTML = error;
				            dojo.byId("preInstallErrorsUL").appendChild(li);
						});
					}
					else {
		                this._helpStatus(id, response.message);
					}
					this.buttonNext.disabled = "true";
	            } 
	            else {
	                this._clearStatus(id);
					this.preInstallCheck.style.display = "none";
					this.preInstallOk.style.display = "block";
					this.preInstallErrors.style.display = "none";
					this._setVars(response);
					this.isInitialized = true;
	            }
	            return response;
	        })
	    };
	    // do the post
	    dojo.xhrGet(kw);
	},

	/**
	 * Checks if all required database fields exist. Sends a request to the server
	 * to attempt a database connection, login, and reset.
	 *
	 * @todo: we must warn the user about clearing the DB before doing it
	 *   on the server; right now, the clear is silent
	 */
	_checkDb: function() {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    this._busyStatus('db_status', msgs.STATUS_VERIFYING);
	
	    // check the values locally
	    var missing = new Array();
	    var form = dojo.byId('wizard_form_db');
	    if(!form.db_host.value)
	        missing.push(dojo.byId('db_host_label').innerHTML);
	    if(form.db_driver.value == hub.config.db2_driver && !form.db_port.value)
	        missing.push(dojo.byId('db_port_label').innerHTML);
	    if(!form.db_name.value)
	        missing.push(dojo.byId('db_name_label').innerHTML);
	    if(form.db_driver.value == hub.config.db2_driver && !form.db_schema.value)
	        missing.push(dojo.byId('db_schema_label').innerHTML);
	    if(!form.db_username.value)
	        missing.push(dojo.byId('db_username_label').innerHTML);
	    if(!form.db_password.value)
	        missing.push(dojo.byId('db_password_label').innerHTML);
	    if(missing.length) {
	        this._reqStatus('db_status', missing);
	        return;
	    }
	
	    // check server
	    this._queryServer('db_status', 'db', form);
	    return false;
	},
	
	/**
	 * Checks if all site config fields exist. Sends a request to the server to 
	 * test if the server code is where the path says it is.
	 */
	_checkSite: function() {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    this._busyStatus('site_status', msgs.STATUS_VERIFYING);
	
	    var missing = new Array();
	    var form = dojo.byId('wizard_form_site');
	    if(!form.app_name.value)
	        missing.push(dojo.byId('app_name_label').innerHTML);
	    if(!form.app_path.value)
	        missing.push(dojo.byId('app_path').innerHTML);
	        
	    this._queryServer('site_status', 'site', form);
	    return false;
	},
	
	/**
	 * Executes the install.
	 */
	_doInstall: function() {
	    var msgs = dojo.i18n.getLocalization('hub', 'messages');
	    this._busyStatus('install_status', msgs.STATUS_INSTALLING);
       	var id = 'install_status';           
	    var kw = {
			url: hub.urls.postInstall("install"),
			sync: true,
		    handleAs: 'json',
	        timeout: 120000, // 2 minutes in milliseconds
	        
	        content: {
				app_name: dojo.byId('app_name').value,
				app_path: dojo.byId('app_path').value,
				app_timezone: dojo.byId('app_timezone').value,
				db_driver: dojo.byId('db_driver').value,
				db_host: dojo.byId('db_host').value,
				db_name: dojo.byId('db_name').value,
				db_password: dojo.byId('db_password').value,
				db_port: dojo.byId('db_port').value,
				db_schema: dojo.byId('db_schema').value,	
				db_username: dojo.byId('db_username').value
	        },
	
	        error: dojo.hitch(this, function(response, args) {
	            if(response.dojoType == "cancel") {
	                this._helpStatus(id, msgs.ERROR_REQUEST_CANCELLED);
	            } else if (response.dojoType == "timeout"){
	                this._helpStatus(id, msgs.ERROR_REQUEST_TIMEOUT);
	            } else {
	                this._helpStatus(id, response.message);
	            }
	            return response;
	        }),
	        
	        load: dojo.hitch(this, function(response, args) {
	            if(!response) {
	                this._helpStatus(id, msgs.ERROR_REQUEST_SERVER);
	            } else if(response.error) {
	                this._helpStatus(id, response.message);
	            } else {
	                this._clearStatus(id);
//	                this._next();
					this.tabContainer.forward();
	            }
	            return response;
	        })
	    };
	    // do the post
	    dojo.xhrPost(kw);
	}
    
});

/**
 * Override stack behavior so form element spanning multiple panels is ignored
 * and any panels it contains are treated as part of the wizard flow.
 *
dojo.declare('hub.install.Wizard', dijit.layout.StackContainer, {
    getChildren: function() {
        // get all descendant widgets
        var all = this.getDescendants();
        // get all children of the container and the form
        var children = this.inherited(arguments);
        var form = dojo.query("form > [widgetId]", this.containerNode || this.domNode).map(dijit.byNode); // Array
        children = children.concat(form);
        // now filter out those not in the children; preserves order
        return dojo.filter(all, function(e) {
            var i = dojo.indexOf(children, e);
            return i != -1;
        });
    }
});

*/

}

if(!dojo._hasResource["mm.xhr"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.xhr"] = true;
dojo.provide("mm.xhr");


mm.xhr.getXhr = function(/*String*/serviceProxyUrl){
	function getNativeXhr(){
		if (window.XMLHttpRequest) {
			return new XMLHttpRequest();
		}else if (window.ActiveXObject) {//IE
				try {
					return new ActiveXObject("Msxml2.XMLHTTP");
				} catch (e) {
					try {
						return new ActiveXObject("Microsoft.XMLHTTP");
					}catch (e) {}
				};
		}
		throw new Error("No XmlHttpRequest support.");
	}
	if (serviceProxyUrl) {
		var this_host = window.location.protocol + "//" +  window.location.host;
		if(serviceProxyUrl.indexOf(this_host) == 0
		|| (serviceProxyUrl.indexOf("http://")!=0 &&  serviceProxyUrl.indexOf("https://")!=0)){
			return getNativeXhr();
		}else{
			return new mm.xhr.CrossDomainXmlHttpRequest(serviceProxyUrl);
		}
	}else{
		return getNativeXhr();
	}
}

}

if(!dojo._hasResource["mm.urls"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.urls"] = true;
dojo.provide("mm.urls");


mm.urls.getContextRoot = function(/*String*/server){
	if (/mashupmaker$/.test(server)) {
		//FIXME: tomcat
		var index = server.lastIndexOf('/');
		return mm.urls.http2https((index != -1 ? server.substring(0, index) : server) + "/mm.webapp");
	}else{
		//WAS
		return mm.urls.http2https(server);
	}
};

mm.urls.getContextRootBuilder = function(/*String*/server){
	if (/mashupmaker$/.test(server)) {
		//FIXME: tomcat
		var index = server.lastIndexOf('/');
		return mm.urls.http2https((index != -1 ? server.substring(0, index) : server) + "/mm");
	}else{
		//WAS
		return mm.urls.http2https(server);
	}
};

mm.urls.getContentHandlerUrl = function(/*String*/ server){
	return mm.urls.getContextRoot(server) + "/mycontenthandler";
};

mm.urls.getDefaultMMServerUrl = function(){
	var path = (window.location.port == "8443") ? "/mashupmaker" : "/mum"; 
	return window.location.protocol + "//" + window.location.host + path;
}

mm.urls.getServiceProxyUrl = function(/*String*/ server){
	if (/mashupmaker$/.test(server)) {
		//FIXME: tomcat
		return mm.urls.getContextRootBuilder(server) + "/crossdomain/ServiceProxy.jsp";
	}else{
		//WAS
		return mm.urls.getContextRootBuilder(server) + "/crossdomain/ServiceProxy.jsp2";
	}
}


mm.urls.http2https = function(/*String*/urlStr){
	//return url;
	var url = new mm.HttpUrl(urlStr);
	return url.getHttpsUrl().toString();
}

}

if(!dojo._hasResource["mm.model.CatalogCategory"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.model.CatalogCategory"] = true;
dojo.provide("mm.model.CatalogCategory");



dojo.declare("mm.model.CatalogCategory", null, {
	XML_NS: {
		atom: "http://www.w3.org/2005/Atom",
		model: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements",
		base: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"
	},
	
	id: "",
	name: "",
	title: "",
	description: "",
	
	constructor: function(/*Element*/entry){
		var id = mm.xpath.evaluateXPath("atom:id", entry, this.XML_NS)[0];
		//localization
		var titles = mm.xpath.evaluateXPath("atom:content/model:catalog-category/model:title/base:nls-string", entry, this.XML_NS);
		var title = titles ? mm.utils.getLocaleElement(titles) : null;
		
		var descriptions = mm.xpath.evaluateXPath("atom:content/model:catalog-category/model:description/base:nls-string", entry, this.XML_NS);
		var description = descriptions ? mm.utils.getLocaleElement(descriptions) : null;
		
		this.id = id ? id.textContent || id.text : "";
		this.title = title ? title.textContent || title.text : "";
		this.description = description ? description.textContent || description.text : "";
		
		if (this.id.indexOf("catalog:id:") == 0) {
			this.name = this.id.substring(11);
		}
	}
});

}

if(!dojo._hasResource["mm.model.CatalogEntry"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.model.CatalogEntry"] = true;
dojo.provide("mm.model.CatalogEntry");



dojo.declare("mm.model.CatalogEntry", null, {
	
	id: "catalog:cid:test",
	category: "",
	title: "",
	description: "",
	definition:"",
	icon:"",
	content:"http://www.ibm.com",
	preview:"http://www.ibm.com",
	metadata: {},
	
	constructor: function(){
		this.metadata = {
			widgetParam:[],
			onNewWire:"",
			systemWidget: false,
			visible: true
		};
	},
	
	toString: function(){
		var entry = [
	  	'<?xml version="1.0" encoding="UTF-8"?>',
		'<atom:feed xmlns:atom="http://www.w3.org/2005/Atom">',
		'<atom:title>IBM MashupMaker Catalog Feed</atom:title>',
		'<atom:id>' + mm.utils.escapeXml(this.id) + '</atom:id>',
		'<atom:updated>' + mm.utils.escapeXml(mm.utils.getTimeStamp()) + '</atom:updated>',
		'<atom:author>IBM Mashup Hub</atom:author>',
		'<atom:entry>',
		'  <atom:id>' + mm.utils.escapeXml(this.id) + '</atom:id>',
		'  <atom:category term="' + mm.utils.escapeXml(this.category) + '"/>',
		'  <atom:content xmlns:model="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements">',
		'    <model:catalog-entry>',
		'       <model:title>',
		'         <base:nls-string xmlns:base="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base" xml:lang="en">' + mm.utils.escapeXml(this.title) + '</base:nls-string>',	
		'       </model:title>',
		'       <model:description>',
		'         <base:nls-string xmlns:base="http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base" xml:lang="en">' + mm.utils.escapeXml(this.description) + '</base:nls-string>',	
		'       </model:description>',
		'       <atom:link href="' + mm.utils.escapeXml(this.definition) + '" rel="definition"/>',
		'       <atom:link href="' + mm.utils.escapeXml(this.icon) + '" rel="icon"/>',
		'       <atom:link href="' + mm.utils.escapeXml(this.content) + '" rel="content"/>',
		'       <atom:link href="' + mm.utils.escapeXml(this.preview) + '" rel="preview"/>', 
		'    </model:catalog-entry>',
		'    <model:metadata name="widgetParam">'+mm.utils.escapeXml(dojo.toJson(this.metadata.widgetParam))+'</model:metadata>',
		'    <model:metadata name="onNewWire">' + mm.utils.escapeXml(this.metadata.onNewWire) + '</model:metadata>',
		'    <model:metadata name="systemWidget">' + mm.utils.escapeXml(this.metadata.systemWidget) + '</model:metadata>',
		'    <model:metadata name="visible">' + mm.utils.escapeXml(this.metadata.visible) + '</model:metadata>',
		'  </atom:content>',
		'</atom:entry>',
		'</atom:feed>'
		].join("\n");
		return entry;
	}
});



}

if(!dojo._hasResource["mm.model.CatalogFeedEntry"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.model.CatalogFeedEntry"] = true;
dojo.provide("mm.model.CatalogFeedEntry");




dojo.declare("mm.model.CatalogFeedEntry", mm.model.CatalogEntry, {
	feedViewer: "",
	feedUrl: "",
	feedUrlItemName:"",
	feedParams:{},
	
	constructor: function(){
		this.feedPrarams = {};
	},
	
	toString: function(){
		this.definition = this.feedViewer;
		var params = "";
		if(this.feedParams){
			for(var key in this.feedParams){
				params += (params ? "&" : "?") + escape(key) + "=" + escape(this.feedParams[key]);
			}
		};
		if(dojo.isArray(this.metadata.widgetParam)){
			for(var i=0;i<this.metadata.widgetParam.length;i++){
				if(this.metadata.widgetParam[i].itemName == this.feedUrlItemName){
					this.metadata.widgetParam[i].itemValue = this.feedUrl + params;
				}
			}
		};
		this.content = this.feedUrl + params;
		
		return mm.model.CatalogFeedEntry.superclass.toString.call(this);
	}
});

}

if(!dojo._hasResource["mm.CatalogModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.CatalogModel"] = true;
dojo.provide("mm.CatalogModel");





dojo.declare("mm.CatalogModel", null, {
	
	XML_NS: {
		atom: "http://www.w3.org/2005/Atom",
		model: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/model-elements",
		base: "http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/base"
	},
	
	constructor: function(/*mm.Server*/server){
		this.server = server;
	},
	
	getCategories: function(/*function*/callback){
		var url = this.server.contentHandler + "?uri=catalog:collection&rep=full";
		var _this = this;
		this.server.GET(url, {}, function(xhr){
			if (xhr.status == 200) {
				var doc = mm.utils.createDocument(xhr.responseText);
				var categories = [];
				var entries = mm.xpath.evaluateXPath("atom:entry", doc.documentElement, _this.XML_NS);
				dojo.forEach(entries, function(entry){
					categories.push(new mm.model.CatalogCategory(entry));
				});
				callback(categories);
			} else {
				callback(xhr.responseText, {
					"status": xhr.status,
					"statusText": xhr.statusText
				});
			};
		});
	},
	
	addToCatalog: function(/*mm.model.CatalogEntry*/entry, callback){
		var url = this.server.contentHandler + "?uri=catalog:collection@" + mm.utils.encodeModelID4Uri("id:" + entry.category);
		this.server.POST(url, {"Content-Type" : "application/atom+xml"}, entry.toString(), function(xhr){
			if (xhr.status == 201) {
				callback(xhr.responseText);
			} else {
				callback(xhr.responseText, {
					"status": xhr.status,
					"statusText": xhr.statusText
				});
			}
		});
	}
});

}

if(!dojo._hasResource["mm.model.FeedViewer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.model.FeedViewer"] = true;
dojo.provide("mm.model.FeedViewer");



dojo.declare("mm.model.FeedViewer", null, {
	displayName: "",
	definition: "",
	icon: "",
	content: "",
	preview: "",
	feedUrlItemName: "",
	metadata: {},
	types: [],
	
	constructor: function(/*Element*/entry, /*string*/contextRoot){
		this.metadata = {};
		this.types = [];
		var displayName = mm.xpath.evaluateXPath("displayName", entry)[0];
		var definition = mm.xpath.evaluateXPath("definition", entry)[0];
		var icon = mm.xpath.evaluateXPath("icon", entry)[0];
		var content = mm.xpath.evaluateXPath("content", entry)[0];
		var preview = mm.xpath.evaluateXPath("preview", entry)[0];
		var feedUrlItemName = mm.xpath.evaluateXPath("feedUrlItemName", entry)[0];
		
        var labels = dojo.i18n.getLocalization('hub', 'labels');
		this.displayName = this._text(displayName);
        if(this.displayName == "Feed Reader"){
            this.displayName = labels.ADDTOMASHUPS_FEEDVIEWER_FEEDREADER;
        }
        if(this.displayName == "Data Viewer"){
            this.displayName = labels.ADDTOMASHUPS_FEEDVIEWER_DATAVIEWER;
        }
		this.definition = contextRoot + this._text(definition);
		this.definition = this._applyContextRoot(this.definition, contextRoot);
		this.icon = this._text(icon);
		this.icon = this._applyContextRoot(this.icon, contextRoot);
		this.content = this._text(content);
		this.content = this._applyContextRoot(this.content, contextRoot);
		this.preview = this._text(preview);
		this.preview = this._applyContextRoot(this.preview, contextRoot);
		this.feedUrlItemName = this._text(feedUrlItemName);
		
		var metadatas = mm.xpath.evaluateXPath("metadata", entry);
		for(var i=0;i<metadatas.length;i++){
			var metadata = metadatas[i];
			this.metadata[metadata.getAttribute("name")] = this._text(metadata);
		}
		
		if(this.metadata.widgetParam){
			this.metadata.widgetParam = dojo.fromJson(this.metadata.widgetParam);
		}
		
		var types = mm.xpath.evaluateXPath("type", entry);
		for(var i=0;i<types.length;i++){
			var type = types[i];
			this.types.push(this._text(type));
		}
	},
	
	_text: function(/*Element*/element){
		var text = element ? element.textContent || element.text : "";
		return text ? text : "";
	},
	
	_applyContextRoot: function(url, contextRoot){
		if((url.indexOf("http://") != 0) && (url.indexOf("https://") != 0) && contextRoot){
			return contextRoot + url; 
		}else{
			return url;
		}
	}
});

}

if(!dojo._hasResource["mm.FeedViewerConfig"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.FeedViewerConfig"] = true;
dojo.provide("mm.FeedViewerConfig");





dojo.declare("mm.FeedViewerConfig", null, {
	
	constructor: function(/*mm.MashupMaker*/server){
		this.server = server;
	},
	
	load: function(/*function*/callback){
		var url = this.server.contextRootBuilder + "/conf/feedViewers.conf";
		var _this = this;
		this.server.GET(url, {},function(xhr){
			if (xhr.status == 200) {
				var doc = mm.utils.createDocument(xhr.responseText);
				var feedviewers = [];
				var entries = mm.xpath.evaluateXPath("entry", doc.documentElement);
				dojo.forEach(entries, function(entry){
					feedviewers.push(new mm.model.FeedViewer(entry, _this.server.serverUrl + _this.server.contextRootBuilder));
				});
				callback(feedviewers);
			} else {
				callback(xhr.responseText, {
					"status": xhr.status,
					"statusText": xhr.statusText
				});
			};
		});
	}
});

}

if(!dojo._hasResource["mm.model.Icon"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.model.Icon"] = true;
dojo.provide("mm.model.Icon");




dojo.declare("mm.model.Icon", null, {
	url: "",
	
	constructor: function(/*Element*/entry, /*string*/contextRoot){
		var url = mm.xpath.evaluateXPath("url", entry)[0];
		this.url = url ? url.textContent || url.text : "";
		
		//append to context root
		if((this.url.indexOf("http://") != 0) && (this.url.indexOf("https://") != 0) && contextRoot){
			this.url = contextRoot + this.url; 
		}
	}
});

}

if(!dojo._hasResource["mm.IconConfig"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.IconConfig"] = true;
dojo.provide("mm.IconConfig");





dojo.declare("mm.IconConfig", null, {
	
	constructor: function(/*mm.MashupMaker*/server){
		this.server = server;
	},
	
	load:function(/*function*/callback){
		var url = this.server.contextRootBuilder + "/conf/icons.conf";
		var _this = this;
		this.server.GET(url, {},function(xhr){
			if (xhr.status == 200) {
				var doc = mm.utils.createDocument(xhr.responseText);
				var icons = [];
				var entries = mm.xpath.evaluateXPath("entry", doc.documentElement);
				dojo.forEach(entries, function(entry){
					icons.push(new mm.model.Icon(entry, _this.server.serverUrl + _this.server.contextRootBuilder));
				});
				callback(icons);
			} else {
				callback(xhr.responseText, {
					"status": xhr.status,
					"statusText": xhr.statusText
				});
			};
		});
	}
});

}

if(!dojo._hasResource["mm.IWidgetDeployer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.IWidgetDeployer"] = true;
dojo.provide("mm.IWidgetDeployer");




dojo.declare("mm.IWidgetDeployer", null, {
	
	packageUrl: "", //url to package
	category: "",
	title: "",
	description: "",
	
	constructor: function(/*mm.MashupMaker*/server){
		this.server = server;
	},
	
	deploy: function(/*function*/callback){
		//invoke generator plugin to deploy
		var url = this.packageUrl;
		url = url.substring(url.indexOf("://") + 3);
		url = url.substring(url.indexOf("/"));
		dojo.xhrPost({
			url: url,
			content: {
				deploy_service_url: this.server.serverUrl + this.server.contentHandler + "?uri=deploy:widget",
				username: this.server._username,
				password: this.server._password,
				title: this.title,
				description: this.description,
				category: this.category
			},
			error: dojo.hitch(this, function(response, args){
				callback(response, {
					"status": args.xhr.status,
					"statusText": args.xhr.statusText
				})
			}),
			load: dojo.hitch(this, function(response, args){
				response = dojo.fromJson(response);
				if (response.status == 201) {
					callback(response.responseText);
				}else{
					callback(response.responseText, response);
				}
			})
		});
	}
});

}

if(!dojo._hasResource['mm.MashupPageDeployer']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['mm.MashupPageDeployer'] = true;
dojo.provide('mm.MashupPageDeployer');

dojo.declare('mm.MashupPageDeployer', null, {
	packageUrl: "",	//url to package

	title: "",
	description: "",
	position: "",
	target: "",
	
	constructor: function(/*mm.MashupMaker*/server){
		this.server = server;
	},
	
	// deploy mashuppage to mashups
	deploy: function(/*function*/callback){
		//invoke generator plugin to deploy
		var url = this.packageUrl;
		url = url.substring(url.indexOf("://") + 3);
		url = url.substring(url.indexOf("/"));
		dojo.xhrPost({
			url: url,
			content: {
				deploy_service_url: this.server.serverUrl + this.server.contentHandler + "?type=page",	// url of parent resource
				username: this.server._username,
				password: this.server._password,
				
				title: this.title,
				description: this.description,
				position: this.position,
				target: this.target
			},
			error: dojo.hitch(this, function(response, args){
				callback(response, {
					"status": args.xhr.status,
					"statusText": args.xhr.statusText
				})
			}),
			load: dojo.hitch(this, function(response, args){
				response = dojo.fromJson(response);
				if (response.status == 201) {
					callback(response.responseText);
				}else{
					callback(response.responseText, response);
				}
			})
		});
	}
	
});

}

if(!dojo._hasResource["mm.main"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["mm.main"] = true;
dojo.provide("mm.main");













/**
 * create "Addtomashups" link in details page
 * @param {Object} details: details page's server data.
 */
mm.createAddToMashupsLink = function(/*object*/details){
	var type;
	//type of the object
	if (details.object.isFeed) {
		type = "feed";
	} else if (details.object.isMashup) {
		type = "mashup";
	} else if (details.object.objectType == "iwidget") {
		type = "iwidget";
	} else if (details.object.objectType == "mashuppage"){
		type = "mashuppage";
	}
	
	if (!type) return false;
	
	// if is package page
	if (type == "mashuppage" && details.pluginData.definitionUrl) {
		return false;
	}
	
	var serverUrl = hub.config.palette_url ? hub.config.palette_url : mm.getDefaultMMServerUrl();
	var templateName;
	
	//the template to show
	if (!hub.config.palette_url) {
		templateName = "SelectMashupsServer";
	} else if (details.object.isFeed || details.object.isMashup) {
		templateName = "AddFeedToMashups";
	} else if (details.object.objectType == 'iwidget') {
		templateName = "AddIWidgetToMashups";
	} else if (details.object.objectType == 'mashuppage'){
		templateName = "AddMashupPageToMashups";
	};
	
	var link = 'javascript:hub.showPage("' 
				+ details.object.title + '","' 
				+ templateName 
				+ '",{"object_id": "' + details.object.id 
				+ '", "object_title": "' + details.object.title 
				+ '", "server": "' + serverUrl 
				+ '", "type": "' + type + '"})';
	return link;
};

mm.getDefaultMMServerUrl = function(){
	var path = (window.location.port == "8443") ? "/mashupmaker" : "/mum";
	var url = new mm.HttpUrl(window.location.protocol + "//" + window.location.host + path);
	return url.getHttpUrl().toString();
};

}


dojo.i18n._preloadLocalizations("dojo.nls.hub", ["es-es", "es", "hu", "it-it", "de", "pt-br", "pl", "fr-fr", "zh-cn", "pt", "en-us", "zh", "ru", "xx", "fr", "zh-tw", "it", "cs", "en-gb", "de-de", "ja-jp", "ko-kr", "ko", "en", "ROOT", "ja"]);
