/*

	Copyright (c) 2007-2008, IBM Corp.
*/

/*
	This is a compiled version of Dojo, built for deployment and not for
	development. To get an editable version, please visit:

		http://dojotoolkit.org

	for documentation and information on getting the source.
*/

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");

// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become actived/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example:  Combobutton --> Menu -->
//		MenuItem.   The onBlur event for Combobutton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		Combobutton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit,
{
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary: tests whether the current selection is empty
		var _window = dojo.global;
		var _document = dojo.doc;
		if(_document.selection){ // IE
			return !_document.selection.createRange().text; // Boolean
		}else if(_window.getSelection){
			var selection = _window.getSelection();
			if(dojo.isString(selection)){ // Safari
				return !selection; // Boolean
			}else{ // Mozilla/W3
				return selection.isCollapsed || !selection.toString(); // Boolean
			}
		}
	},

	getBookmark: function(){
		// summary: Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bookmark, selection = dojo.doc.selection;
		if(selection){ // IE
			var range = selection.createRange();
			if(selection.type.toUpperCase()=='CONTROL'){
				bookmark = range.length ? dojo._toArray(range) : null;
			}else{
				bookmark = range.getBookmark();
			}
		}else{
			if(dojo.global.getSelection){
				selection = dojo.global.getSelection();
				if(selection){
					var range = selection.getRangeAt(0);
					bookmark = range.cloneRange();
				}
			}else{
				console.debug("No idea how to store the current selection for this browser!");
			}
		}
		return bookmark; // Array
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary: Moves current selection to a bookmark
		// bookmark: this should be a returned object from dojo.html.selection.getBookmark()
		var _document = dojo.doc;
		if(_document.selection){ // IE
			var range;
			if(dojo.isArray(bookmark)){
				range = _document.body.createControlRange();
				dojo.forEach(bookmark, range.addElement);
			}else{
				range = _document.selection.createRange();
				range.moveToBookmark(bookmark);
			}
			range.select();
		}else{ //Moz/W3C
			var selection = dojo.global.getSelection && dojo.global.getSelection();
			if(selection && selection.removeAllRanges){
				selection.removeAllRanges();
				selection.addRange(bookmark);
			}else{
				console.debug("No idea how to restore selection for this browser!");
			}
		}
	},

	getFocus: function(/*Widget*/menu, /*Window*/ openedForWindow){
		// summary:
		//	Returns the current focus and selection.
		//	Called when a popup appears (either a top level menu or a dialog),
		//	or when a toolbar/menubar receives focus
		//
		// menu:
		//	the menu that's being opened
		//
		// openedForWindow:
		//	iframe in which menu was opened
		//
		// returns:
		//	a handle to restore focus/selection

		return {
			// Node to return focus to
			node: menu && dojo.isDescendant(dijit._curFocus, menu.domNode) ? dijit._prevFocus : dijit._curFocus,

			// Previously selected text
			bookmark:
				!dojo.withGlobal(openedForWindow||dojo.global, dijit.isCollapsed) ?
				dojo.withGlobal(openedForWindow||dojo.global, dijit.getBookmark) :
				null,

			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase()=="iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}			
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item)
		if(bookmark && dojo.withGlobal(openedForWindow||dojo.global, dijit.isCollapsed)){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow||dojo.global, moveToBookmark, null, [bookmark]);
			}catch(e){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerWin: function(/*Window?*/targetWindow){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe) to detect when the user has clicked somewhere.
		//		Anyone that creates an iframe should call this function.

		if(!targetWindow){
			targetWindow = window;
		}

		dojo.connect(targetWindow.document, "onmousedown", null, function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			dijit._onTouchNode(evt.target||evt.srcElement);
		});
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's body
		var body = targetWindow.document.body || targetWindow.document.getElementsByTagName("body")[0];
		if(body){
			if(dojo.isIE){
				body.attachEvent('onactivate', function(evt){
					if(evt.srcElement.tagName.toLowerCase() != "body"){
						dijit._onFocusNode(evt.srcElement);
					}
				});
				body.attachEvent('ondeactivate', function(evt){ dijit._onBlurNode(evt.srcElement); });
			}else{
				body.addEventListener('focus', function(evt){ dijit._onFocusNode(evt.target); }, true);
				body.addEventListener('blur', function(evt){ dijit._onBlurNode(evt.target); }, true);
			}
		}
		body = null;	// prevent memory leak (apparent circular reference via closure)
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		var w = dijit.getEnclosingWidget(node);
		if (w && w._setStateClass){
			w._focused = false;
			w._setStateClass();
		}
		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer; dijit._setStack([]); }, 100);
	},

	_onTouchNode: function(/*DomNode*/ node){
		// summary
		//		Callback when node is focused or mouse-downed

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				if(node.dijitPopupParent){
					node=dijit.byId(node.dijitPopupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase()=="body"){
					// is this the root of the document or just the root of an iframe?
					if(node===dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead) and continue tracing up the document
					node=dojo.query("iframe").filter(function(iframe){ return iframe.contentDocument.body===node; })[0];
				}else{
					var id = node.getAttribute && node.getAttribute("widgetId");
					if(id){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary
		//		Callback when node is focused
		if(node && node.tagName && node.tagName.toLowerCase() == "body"){
			return;
		}
		dijit._onTouchNode(node);
		if(node==dijit._curFocus){ return; }
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);

		// handle focus/blur styling
		var w = dijit.getEnclosingWidget(node);
		if (w && w._setStateClass){
			w._focused = true;
			w._setStateClass();
		}
	},

	_setStack: function(newStack){
		// summary
		//	The stack of active widgets has changed.  Send out appropriate events and record new stack

		var oldStack = dijit._activeStack;		
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			var widget = dijit.byId(oldStack[i]);
			if(widget){
				dojo.publish("widgetBlur", [widget]);
				if(widget._onBlur){
					widget._onBlur();
				}
			}
		}

		// for all element that have come into focus, send focus event
		for(var i=nCommon; i<newStack.length; i++){
			var widget = dijit.byId(newStack[i]);
			if(widget){
				dojo.publish("widgetFocus", [widget]);
				if(widget._onFocus){
					widget._onFocus();
				}
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(dijit.registerWin);

}

if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");

dojo.declare("dijit.WidgetSet", null, {
	constructor: function(){
		// summary:
		//	A set of widgets indexed by id
		this._hash={};
	},

	add: function(/*Widget*/ widget){
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id]=widget;
	},

	remove: function(/*String*/ id){
		delete this._hash[id];
	},

	forEach: function(/*Function*/ func){
		for(var id in this._hash){
			func(this._hash[id]);
		}
	},

	filter: function(/*Function*/ filter){
		var res = new dijit.WidgetSet();
		this.forEach(function(widget){
			if(filter(widget)){ res.add(widget); }
		});
		return res;		// dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		return this._hash[id];
	},

	byClass: function(/*String*/ cls){
		return this.filter(function(widget){ return widget.declaredClass==cls; });	// dijit.WidgetSet
	}
	});

// registry: list of all widgets on page
dijit.registry = new dijit.WidgetSet();

dijit._widgetTypeCtr = {};

dijit.getUniqueId = function(/*String*/widgetType){
	// summary
	//	Generates a unique id for a given widgetType

	var id;
	do{
		id = widgetType + "_" +
			(dijit._widgetTypeCtr[widgetType] !== undefined ?
				++dijit._widgetTypeCtr[widgetType] : dijit._widgetTypeCtr[widgetType] = 0);
	}while(dijit.byId(id));
	return id; // String
};


if(dojo.isIE){
	// Only run this for IE because we think it's only necessary in that case,
	// and because it causes problems on FF.  See bug #3531 for details.
	dojo.addOnUnload(function(){
		dijit.registry.forEach(function(widget){ widget.destroy(); });
	});
}

dijit.byId = function(/*String|Widget*/id){
	// summary:
	//		Returns a widget by its id, or if passed a widget, no-op (like dojo.byId())
	return (dojo.isString(id)) ? dijit.registry.byId(id) : id; // Widget
};

dijit.byNode = function(/* DOMNode */ node){
	// summary:
	//		Returns the widget as referenced by node
	return dijit.registry.byId(node.getAttribute("widgetId")); // Widget
};

dijit.getEnclosingWidget = function(/* DOMNode */ node){
	// summary:
	//		Returns the widget whose dom tree contains node or null if
	//		the node is not contained within the dom tree of any widget
	while(node){
		if(node.getAttribute && node.getAttribute("widgetId")){
			return dijit.registry.byId(node.getAttribute("widgetId"));
		}
		node = node.parentNode;
	}
	return null;
};

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");

// ported from dojo.html.util

dijit.getViewport = function(){
	//	summary
	//	Returns the dimensions and scroll position of the viewable area of a browser window

	var _window = dojo.global;
	var _document = dojo.doc;

	// get viewport size
	var w = 0, h = 0;
	if(dojo.isMozilla){
		// mozilla
		// _window.innerHeight includes the height taken by the scroll bar
		// clientHeight is ideal but has DTD issues:
		// #4539: FF reverses the roles of body.clientHeight/Width and documentElement.clientHeight/Width based on the DTD!
		// check DTD to see whether body or documentElement returns the viewport dimensions using this algorithm:
		var minw, minh, maxw, maxh;
		if(_document.body.clientWidth>_document.documentElement.clientWidth){
			minw = _document.documentElement.clientWidth;
			maxw = _document.body.clientWidth;
		}else{
			maxw = _document.documentElement.clientWidth;
			minw = _document.body.clientWidth;
		}
		if(_document.body.clientHeight>_document.documentElement.clientHeight){
			minh = _document.documentElement.clientHeight;
			maxh = _document.body.clientHeight;
		}else{
			maxh = _document.documentElement.clientHeight;
			minh = _document.body.clientHeight;
		}
		w = (maxw > _window.innerWidth) ? minw : maxw;
		h = (maxh > _window.innerHeight) ? minh : maxh;
	}else if(!dojo.isOpera && _window.innerWidth){
		//in opera9, dojo.body().clientWidth should be used, instead
		//of window.innerWidth/document.documentElement.clientWidth
		//so we have to check whether it is opera
		w = _window.innerWidth;
		h = _window.innerHeight;
	}else if(dojo.isIE && _document.documentElement && _document.documentElement.clientHeight){
		w = _document.documentElement.clientWidth;
		h = _document.documentElement.clientHeight;
	}else if(dojo.body().clientWidth){
		// IE5, Opera
		w = dojo.body().clientWidth;
		h = dojo.body().clientHeight;
	}

	// get scroll position
	var scroll = dojo._docScroll();

	return { w: w, h: h, l: scroll.x, t: scroll.y };	//	object
};

dijit.placeOnScreen = function(
	/* DomNode */	node,
	/* Object */		pos,
	/* Object */		corners,
	/* boolean? */		tryOnly){
	//	summary:
	//		Keeps 'node' in the visible area of the screen while trying to
	//		place closest to pos.x, pos.y. The input coordinates are
	//		expected to be the desired document position.
	//
	//		Set which corner(s) you want to bind to, such as
	//		
	//			placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])
	//		
	//		The desired x/y will be treated as the topleft(TL)/topright(TR) or
	//		BottomLeft(BL)/BottomRight(BR) corner of the node. Each corner is tested
	//		and if a perfect match is found, it will be used. Otherwise, it goes through
	//		all of the specified corners, and choose the most appropriate one.
	//		
	//		NOTE: node is assumed to be absolutely or relatively positioned.

	var choices = dojo.map(corners, function(corner){ return { corner: corner, pos: pos }; });

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, /* Array */ choices, /* Function */ layoutNode){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	//	layoutNode: Function(node, orient)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dijit.getViewport();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to document.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best=null;
	for(var i=0; i<choices.length; i++){
		var corner = choices[i].corner;
		var pos = choices[i].pos;

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltips size changes based on position, due to triangle)
		if(layoutNode){
			layoutNode(corner);
		}

		// get node's size
		var oldDisplay = node.style.display;
		var oldVis = node.style.visibility;
		node.style.visibility = "hidden";
		node.style.display = "";
		var mb = dojo.marginBox(node);
		node.style.display = oldDisplay;
		node.style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = (corner.charAt(1)=='L' ? pos.x : Math.max(view.l, pos.x - mb.w)),
			startY = (corner.charAt(0)=='T' ? pos.y : Math.max(view.t, pos.y -  mb.h)),
			endX = (corner.charAt(1)=='L' ? Math.min(view.l+view.w, startX+mb.w) : pos.x),
			endY = (corner.charAt(0)=='T' ? Math.min(view.t+view.h, startY+mb.h) : pos.y),
			width = endX-startX,
			height = endY-startY,
			overflow = (mb.w-width) + (mb.h-height);

		if(best==null || overflow<best.overflow){
			best = {
				corner: corner,
				aroundCorner: choices[i].aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow
			};
		}
		if(overflow==0){
			break;
		}
	}

	node.style.left = best.x + "px";
	node.style.top = best.y + "px";
	return best;
}

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	//	summary
	//	Like placeOnScreen, except it accepts aroundNode instead of x,y
	//	and attempts to place node around it.  Uses margin box dimensions.
	//
	//	aroundCorners
	//		specify Which corner of aroundNode should be
	//		used to place the node => which corner(s) of node to use (see the
	//		corners parameter in dijit.placeOnScreen)
	//		e.g. {'TL': 'BL', 'BL': 'TL'}
	//
	//	layoutNode: Function(node, orient)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.


	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var oldDisplay = aroundNode.style.display;
	aroundNode.style.display="";
	// #3172: use the slightly tighter border box instead of marginBox
	var aroundNodeW = aroundNode.offsetWidth; //mb.w;
	var aroundNodeH = aroundNode.offsetHeight; //mb.h;
	var aroundNodePos = dojo.coords(aroundNode, true);
	aroundNode.style.display=oldDisplay;

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: aroundNodePos.x + (nodeCorner.charAt(1)=='L' ? 0 : aroundNodeW),
				y: aroundNodePos.y + (nodeCorner.charAt(0)=='T' ? 0 : aroundNodeH)
			}
		});
	}

	return dijit._place(node, choices, layoutNode);
}

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");

dijit.getDocumentWindow = function(doc){
	//	summary
	// 	Get window object associated with document doc

	// With Safari, there is not way to retrieve the window from the document, so we must fix it.
	if(dojo.isSafari && !doc._parentWindow){
		/*
			This is a Safari specific function that fix the reference to the parent
			window from the document object.
		*/
		var fix=function(win){
			win.document._parentWindow=win;
			for(var i=0; i<win.frames.length; i++){
				fix(win.frames[i]);
			}
		}
		fix(window.top);
	}

	//In some IE versions (at least 6.0), document.parentWindow does not return a
	//reference to the real window object (maybe a copy), so we must fix it as well
	//We use IE specific execScript to attach the real window reference to
	//document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow && !doc._parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc._parentWindow || doc.parentWindow || doc.defaultView;	//	Window
}

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");





dijit.popup = new function(){
	// summary:
	//		This class is used to show/hide widgets as popups.
	//

	var stack = [],
		beginZIndex=1000,
		idGen = 1;

	this.open = function(/*Object*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// args: Object
		//		popup: Widget
		//			widget to display,
		//		parent: Widget
		//			the button etc. that is displaying this popup
		//		around: DomNode
		//			DOM node (typically a button); place popup relative to this node
		//		orient: Object
		//			structure specifying possible positions of popup relative to "around" node
		//		onCancel: Function
		//			callback when user has canceled the popup by
		//				1. hitting ESC or
		//				2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
		//				   ie: whenever popupWidget.onCancel() is called, args.onCancel is called
		//		onClose: Function
		//			callback whenever this popup is closed
		//		onExecute: Function
		//			callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
		//
		// examples:
		//		1. opening at the mouse position
		//			dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//		2. opening the widget as a dropdown
		//			dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}  });
		//
		//	Note that whatever widget called dijit.popup.open() should also listen to it's own _onBlur callback
		//	(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var widget = args.popup,
			orient = args.orient || {'BL':'TL', 'TL':'BL'},
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+idGen++);

		// make wrapper div to hold widget and possibly hold iframe behind it.
		// we can't attach the iframe as a child of the widget.domNode because
		// widget.domNode might be a <table>, <ul>, etc.
		var wrapper = dojo.doc.createElement("div");
		wrapper.id = id;
		wrapper.className="dijitPopup";
		wrapper.style.zIndex = beginZIndex + stack.length;
		wrapper.style.visibility = "hidden";
		if(args.parent){
			wrapper.dijitPopupParent=args.parent.id;
		}
		dojo.body().appendChild(wrapper);

		widget.domNode.style.display="";
		wrapper.appendChild(widget.domNode);

		var iframe = new dijit.BackgroundIframe(wrapper);

		// position the wrapper node
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR']);

		wrapper.style.visibility = "visible";
		// TODO: use effects to fade in wrapper

		var handlers = [];

		// Compute the closest ancestor popup that's *not* a child of another popup.
		// Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		function getTopPopup(){
			for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--);
			return stack[pi];
		}

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.keyCode == dojo.keys.ESCAPE && args.onCancel){
				args.onCancel();
			}else if(evt.keyCode == dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", null, args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", null, function(){
			var topPopup = getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			wrapper: wrapper,
			iframe: iframe,
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			widget.onOpen(best);
		}

		return best;
	};

	this.close = function(/*Widget*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented
		while(dojo.some(stack, function(elem){return elem.widget == popup;})){
			var top = stack.pop(),
				wrapper = top.wrapper,
				iframe = top.iframe,
				widget = top.widget,
				onClose = top.onClose;
	
			if(widget.onClose){
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);
	
			// #2685: check if the widget still has a domNode so ContentPane can change its URL without getting an error
			if(!widget||!widget.domNode){ return; }
			dojo.style(widget.domNode, "display", "none");
			dojo.body().appendChild(widget.domNode);
			iframe.destroy();
			dojo._destroyElement(wrapper);
	
			if(onClose){
				onClose();
			}
		}
	};
}();

dijit._frames = new function(){
	// summary: cache of iframes
	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE){
				var html="<iframe src='javascript:\"\"'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	var iframe = dojo.doc.createElement("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didnt work.
			dojo.body().appendChild(iframe);
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="";
		if(dojo.isIE){
			iframe.style.removeExpression("width");
			iframe.style.removeExpression("height");
		}
		queue.push(iframe);
	}
}();

// fill the queue
if(dojo.isIE && dojo.isIE < 7){
	dojo.addOnLoad(function(){
		var f = dijit._frames;
		dojo.forEach([f.pop()], f.push);
	});
}


dijit.BackgroundIframe = function(/* DomNode */node){
	//	summary:
	//		For IE z-index schenanigans. id attribute is required.
	//
	//	description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if((dojo.isIE && dojo.isIE < 7) || (dojo.isFF && dojo.isFF < 3 && dojo.hasClass(dojo.body(), "dijit_a11y"))){
		var iframe = dijit._frames.pop();
		node.appendChild(iframe);
		if(dojo.isIE){
			iframe.style.setExpression("width", "document.getElementById('" + node.id + "').offsetWidth");
			iframe.style.setExpression("height", "document.getElementById('" + node.id + "').offsetHeight");
		}
		this.iframe = iframe;
	}
};

dojo.extend(dijit.BackgroundIframe, {
	destroy: function(){
		//	summary: destroy the iframe
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");

dijit.scrollIntoView = function(/* DomNode */node){
	//	summary
	//	Scroll the passed node into view, if it is not.

	// don't rely on that node.scrollIntoView works just because the function is there
	// it doesnt work in Konqueror or Opera even though the function is there and probably
	// not safari either
	// dont like browser sniffs implementations but sometimes you have to use it
	if(dojo.isIE){
		//only call scrollIntoView if there is a scrollbar for this menu,
		//otherwise, scrollIntoView will scroll the window scrollbar
		if(dojo.marginBox(node.parentNode).h <= node.parentNode.scrollHeight){ //PORT was getBorderBox
			node.scrollIntoView(false);
		}
	}else if(dojo.isMozilla){
		node.scrollIntoView(false);
	}else{
		var parent = node.parentNode;
		var parentBottom = parent.scrollTop + dojo.marginBox(parent).h; //PORT was getBorderBox
		var nodeBottom = node.offsetTop + dojo.marginBox(node).h;
		if(parentBottom < nodeBottom){
			parent.scrollTop += (nodeBottom - parentBottom);
		}else if(parent.scrollTop > node.offsetTop){
			parent.scrollTop -= (parent.scrollTop - node.offsetTop);
		}
	}
};

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
dojo.provide("dijit._base.sniff");

// ported from dojo.html.applyBrowserClass (style.js)

//	summary:
//		Applies pre-set class names based on browser & version to the
//		top-level HTML node.  Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.
(function(){
	var d = dojo;
	var ie = d.isIE;
	var opera = d.isOpera;
	var maj = Math.floor;
	var classes = {
		dj_ie: ie,
//		dj_ie55: ie == 5.5,
		dj_ie6: maj(ie) == 6,
		dj_ie7: maj(ie) == 7,
		dj_iequirks: ie && d.isQuirks,
// NOTE: Opera not supported by dijit
		dj_opera: opera,
		dj_opera8: maj(opera) == 8,
		dj_opera9: maj(opera) == 9,
		dj_khtml: d.isKhtml,
		dj_safari: d.isSafari,
		dj_gecko: d.isMozilla
	}; // no dojo unsupported browsers

	for(var p in classes){
		if(classes[p]){
			var html = dojo.doc.documentElement; //TODO browser-specific DOM magic needed?
			if(html.className){
				html.className += " " + p;
			}else{
				html.className = p;
			}
		}
	}
})();

}

if(!dojo._hasResource["dijit._base.bidi"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.bidi"] = true;
dojo.provide("dijit._base.bidi");

// summary: applies a class to the top of the document for right-to-left stylesheet rules

dojo.addOnLoad(function(){
	if(!dojo._isBodyLtr()){
		dojo.addClass(dojo.body(), "dijitRtl");
	}
});

}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");

dijit.typematic = {
	// summary:
	//	These functions are used to repetitively call a user specified callback
	//	method when a specific key or mouse click over a specific DOM node is
	//	held down for a specific amount of time.
	//	Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		this._currentTimeout = (this._currentTimeout < 0) ? this._initialDelay : ((this._subsequentDelay > 1) ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay));
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /* Object */ _this, /*DOMNode*/ node, /* Function */ callback, /* Object */ obj, /* Number */ subsequentDelay, /* Number */ initialDelay){
		// summary:
		//      Start a timed, repeating callback sequence.
		//      If already started, the function call is ignored.
		//      This method is not normally called by the user but can be
		//      when the normal listener code is insufficient.
		//	Parameters:
		//	evt: key or mouse event object to pass to the user callback
		//	_this: pointer to the user's widget space.
		//	node: the DOM node object to pass the the callback function
		//	callback: function to call until the sequence is stopped called with 3 parameters:
		//		count: integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		//		node: the DOM node object passed in
		//		evt: key or mouse event object
		//	obj: user space object used to uniquely identify each typematic sequence
		//	subsequentDelay: if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		//	initialDelay: the number of milliseconds until the 2nd event occurs, default=500ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
		}
	},

	stop: function(){
		// summary:
		//	  Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a specific typematic key.
		//	keyObject: an object defining the key to listen for.
		//		key: (mandatory) the keyCode (number) or character (string) to listen for.
		//		ctrlKey: desired ctrl key state to initiate the calback sequence:
		//			pressed (true)
		//			released (false)
		//			either (unspecified)
		//		altKey: same as ctrlKey but for the alt key
		//		shiftKey: same as ctrlKey but for the shift key
		//	See the trigger method for other parameters.
		//	Returns an array of dojo.connect handles
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.keyCode == keyObject.keyCode && (!keyObject.charCode || keyObject.charCode == evt.charCode) &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.ctrlKey) &&
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.ctrlKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(keyObject, _this, node, callback, keyObject, subsequentDelay, initialDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a typematic mouse click.
		//	See the trigger method for other parameters.
		//	Returns an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				dojo.stopEvent(evt);
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay);
					setTimeout(dijit.typematic.stop, 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a specific typematic key and mouseclick.
		//	This is a thin wrapper to addKeyListener and addMouseListener.
		//	mouseNode: the DOM node object to listen on for mouse events.
		//	keyNode: the DOM node object to listen on for key events.
		//	See the addMouseListener and addKeyListener methods for other parameters.
		//	Returns an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");

dijit.wai = {
	onload: function(){
		// summary:
		//		Function that detects if we are in high-contrast mode or not,
		//		and sets up a timer to periodically confirm the value.
		//		figure out the background-image style property
		//		and apply that to the image.src property.
		// description:
		//		This must be a named function and not an anonymous
		//		function, so that the widget parsing code can make sure it
		//		registers its onload function after this function.
		//		DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = document.createElement("div");
		div.id = "a11yTestNode";
		div.style.cssText = 'border: 1px solid;'
			+ 'border-color:red green;'
			+ 'position: absolute;'
			+ 'height: 5px;'
			+ 'top: -999px;'
			+ 'background-image: url("' + dojo.moduleUrl("dijit", "form/templates/blank.gif") + '");';
		dojo.body().appendChild(div);

		// test it
		function check(){
			var cs = dojo.getComputedStyle(div);
			if(cs){
				var bkImg = cs.backgroundImage;
				var needsA11y = (cs.borderTopColor==cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
				dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			}
		}
		check();
		if(dojo.isIE){
			setInterval(check, 4000);
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit,
{
	hasWaiRole: function(/*Element*/ elem){
		// Summary: Return true if elem has a role attribute and false if not.
		if(elem.hasAttribute){
			return elem.hasAttribute("role");
		}else{
			return elem.getAttribute("role") ? true : false;
		}
	},

	getWaiRole: function(/*Element*/ elem){
		// Summary: Return the role of elem or an empty string if
		//		elem does not have a role.
		var value = elem.getAttribute("role");
		if(value){
			var prefixEnd = value.indexOf(":");
			return prefixEnd == -1 ? value : value.substring(prefixEnd+1);
		}else{
			return "";
		}
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// Summary: Set the role on elem. On Firefox 2 and below, "wairole:" is
		//		prepended to the provided role value.
		if(dojo.isFF && dojo.isFF < 3){
			elem.setAttribute("role", "wairole:"+role);
		}else{
			elem.setAttribute("role", role);
		}
	},

	removeWaiRole: function(/*Element*/ elem){
		// Summary: Removes the role attribute from elem.
		elem.removeAttribute("role");
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Return true if elem has a value for the given state and
		//		false if it does not.
		//		On Firefox 2 and below, we check for an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we check for an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			return elem.hasAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			if(elem.hasAttribute){
				return elem.hasAttribute("aria-"+state);
			}else{
				return elem.getAttribute("aria-"+state) ? true : false;
			}
		}
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Return the value of the requested state on elem
		//		or an empty string if elem has no value for state.
		//		On Firefox 2 and below, we check for an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we check for an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			return elem.getAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			var value =  elem.getAttribute("aria-"+state);
			return value ? value : "";
		}
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// Summary: Set state on elem to value.
		//		On Firefox 2 and below, we set an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we set an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			elem.setAttributeNS("http://www.w3.org/2005/07/aaa",
				"aaa:"+state, value);
		}else{
			elem.setAttribute("aria-"+state, value);
		}
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Removes the given state from elem.
		//		On Firefox 2 and below, we remove the attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we remove the attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			elem.removeAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			elem.removeAttribute("aria-"+state);
		}
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");












}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");



dojo.declare("dijit._Widget", null, {
	// summary:
	//		The foundation of dijit widgets. 	
	//
	// id: String
	//		a unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: String
	//	Language to display this widget in (like en-us).
	//	Defaults to brower's specified preferred language (typically the language of the OS)
	lang: "",

	// dir: String
	//  Bi-directional support, as defined by the HTML DIR attribute. Either left-to-right "ltr" or right-to-left "rtl".
	dir: "",

	// class: String
	// HTML class attribute
	"class": "",

	// style: String
	// HTML style attribute
	style: "",

	// title: String
	// HTML title attribute
	title: "",

	// srcNodeRef: DomNode
	//		pointer to original dom node
	srcNodeRef: null,

	// domNode: DomNode
	//		this is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPonit syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// attributeMap: Object
	//		A map of attributes and attachpoints -- typically standard HTML attributes -- to set
	//		on the widget's dom, at the "domNode" attach point, by default.
	//		Other node references can be specified as properties of 'this'
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},  // TODO: add on* handlers?

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(params, srcNodeRef){
		this.create(params, srcNodeRef);
	},

	create: function(params, srcNodeRef){
		// summary:
		//		To understand the process by which widgets are instantiated, it
		//		is critical to understand what other methods create calls and
		//		which of them you'll want to override. Of course, adventurous
		//		developers could override create entirely, but this should
		//		only be done as a last resort.
		//
		//		Below is a list of the methods that are called, in the order
		//		they are fired, along with notes about what they do and if/when
		//		you should over-ride them in your widget:
		//			
		//			postMixInProperties:
		//				a stub function that you can over-ride to modify
		//				variables that may have been naively assigned by
		//				mixInProperties
		//			# widget is added to manager object here
		//			buildRendering
		//				Subclasses use this method to handle all UI initialization
		//				Sets this.domNode.  Templated widgets do this automatically
		//				and otherwise it just uses the source dom node.
		//			postCreate
		//				a stub function that you can over-ride to modify take
		//				actions once the widget has been placed in the UI

		// store pointer to original dom tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects=[];

		// _attaches: String[]
		// 		names of all our dojoAttachPoint variables
		this._attaches=[];

		//mixin our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			dojo.mixin(this,params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.
		if(!this.id){
			this.id=dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
		// The placement of these attributes is according to the property mapping in attributeMap.
		// Note special handling for 'style' and 'class' attributes which are lists and can
		// have elements from both old and new structures, and some attributes like "type"
		// cannot be processed this way as they are not mutable.
		if(this.domNode){
			for(var attr in this.attributeMap){
				var mapNode = this[this.attributeMap[attr] || "domNode"];
				var value = this[attr];
				if(typeof value != "object" && (value !== "" || (params && params[attr]))){
					switch(attr){
					case "class":
						dojo.addClass(mapNode, value);
						break;
					case "style":
						if(mapNode.style.cssText){
							mapNode.style.cssText += "; " + value;// FIXME: Opera
						}else{
							mapNode.style.cssText = value;
						}
						break;
					default:
						mapNode.setAttribute(attr, value);
					}
				}
			}
		}

		if(this.domNode){
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}	
	},

	postMixInProperties: function(){
		// summary
		//	Called after the parameters to the widget have been read-in,
		//	but before the widget template is instantiated.
		//	Especially useful to set properties that are referenced in the widget template.
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode.
		//		Most widgets will mixin TemplatedWidget, which overrides this method.
		this.domNode = this.srcNodeRef || dojo.doc.createElement('div');
	},

	postCreate: function(){
		// summary:
		//		Called after a widget's dom has been setup
	},

	startup: function(){
		// summary:
		//		Called after a widget's children, and other widgets on the page, have been created.
		//		Provides an opportunity to manipulate any children before they are displayed
		//		This is useful for composite widgets that need to control or layout sub-widgets
		//		Many layout widgets can use this as a wiring phase
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean*/ finalize){
		// summary:
		// 		Destroy this widget and it's descendants. This is the generic
		// 		"destructor" function that all widget users should call to
		// 		cleanly discard with a widget. Once a widget is destroyed, it's
		// 		removed from the manager object.
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?

		this.destroyDescendants();
		this.destroy();
	},

	destroy: function(/*Boolean*/ finalize){
		// summary:
		// 		Destroy this widget, but not its descendants
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?
		this.uninitialize();
		dojo.forEach(this._connects, function(array){
			dojo.forEach(array, dojo.disconnect);
		});
		this.destroyRendering(finalize);
		dijit.registry.remove(this.id);
	},

	destroyRendering: function(/*Boolean*/ finalize){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?

		if(this.bgIframe){
			this.bgIframe.destroy();
			delete this.bgIframe;
		}

		if(this.domNode){
			dojo._destroyElement(this.domNode);
			delete this.domNode;
		}

		if(this.srcNodeRef){
			dojo._destroyElement(this.srcNodeRef);
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.

		// TODO: should I destroy in the reverse order, to go bottom up?
		dojo.forEach(this.getDescendants(), function(widget){ widget.destroy(); });
	},

	uninitialize: function(){
		// summary:
		//		stub function. Over-ride to implement custom widget tear-down
		//		behavior.
		return false;
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	toString: function(){
		// summary:
		//		returns a string that represents the widget. When a widget is
		//		cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversable
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//	return all the descendant widgets
		var list = dojo.query('[widgetId]', this.domNode);
		return list.map(dijit.byNode);		// Array
	},

	nodesWithKeyClick : ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String*/ event,
			/*String|Function*/ method){

		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		//		Special event: "ondijitclick" triggers on a click or enter-down or space-up
		//		Similar to dojo.connect() but takes three arguments rather than four.
		var handles =[];
		if(event == "ondijitclick"){
			var w = this;
			// add key based click activation for unsupported nodes.
			if(!this.nodesWithKeyClick[obj.nodeName]){
				handles.push(dojo.connect(obj, "onkeydown", this,
					function(e){
						if(e.keyCode == dojo.keys.ENTER){
							return (dojo.isString(method))?
								w[method](e) : method.call(w, e);
						}else if(e.keyCode == dojo.keys.SPACE){
							// stop space down as it causes IE to scroll
							// the browser window
							dojo.stopEvent(e);
						}
			 		}));
				handles.push(dojo.connect(obj, "onkeyup", this,
					function(e){
						if(e.keyCode == dojo.keys.SPACE){
							return dojo.isString(method) ?
								w[method](e) : method.call(w, e);
						}
			 		}));
			}
			event = "onclick";
		}
		handles.push(dojo.connect(obj, event, this, method));

		// return handles for FormElement and ComboBox
		this._connects.push(handles);
		return handles;
	},

	disconnect: function(/*Object*/ handles){
		// summary:
		//		Disconnects handle created by this.connect.
		//		Also removes handle from this widget's list of connects
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i]==handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Checks the DOM to for the text direction for bi-directional support
		// description:
		//		This method cannot be used during widget construction because the widget
		//		must first be connected to the DOM tree.  Parent nodes are searched for the
		//		'dir' attribute until one is found, otherwise left to right mode is assumed.
		//		See HTML spec, DIR attribute for more information.

		if(typeof this._ltr == "undefined"){
			this._ltr = dojo.getComputedStyle(this.domNode).direction != "rtl";
		}
		return this._ltr; //Boolean
	},

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	}
});

}

if(!dojo._hasResource["dojo.string"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.string"] = true;
dojo.provide("dojo.string");

dojo.string.pad = function(/*String*/text, /*int*/size, /*String?*/ch, /*boolean?*/end){
	// summary:
	//		Pad a string to guarantee that it is at least 'size' length by
	//		filling with the character 'c' at either the start or end of the
	//		string. Pads at the start, by default.
	// text: the string to pad
	// size: length to provide padding
	// ch: character to pad, defaults to '0'
	// end: adds padding at the end if true, otherwise pads at start

	var out = String(text);
	if(!ch){
		ch = '0';
	}
	while(out.length < size){
		if(end){
			out += ch;
		}else{
			out = ch + out;
		}
	}
	return out;	// String
};

dojo.string.substitute = function(	/*String*/template, 
									/*Object or Array*/map, 
									/*Function?*/transform, 
									/*Object?*/thisObject){
	// summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	// description:
	//		For example,
	//		|	dojo.string.substitute("File '${0}' is not found in directory '${1}'.",["foo.html","/temp"]);
	//		|	dojo.string.substitute("File '${name}' is not found in directory '${info.dir}'.",{name: "foo.html", info: {dir: "/temp"}});
	//		both return
	//			"File 'foo.html' is not found in directory '/temp'."
	// template: 
	//		a string with expressions in the form ${key} to be replaced or
	//		${key:format} which specifies a format function.  NOTE syntax has
	//		changed from %{key}
	// map: where to look for substitutions
	// transform: 
	//		a function to process all parameters before substitution takes
	//		place, e.g. dojo.string.encodeXML
	// thisObject: 
	//		where to look for optional format function; default to the global
	//		namespace

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g, function(match, key, format){
		var value = dojo.getObject(key,false,map);
		if(format){ value = dojo.getObject(format,false,thisObject)(value);}
		if(transform){ value = transform(value, key); }
		return value.toString();
	}); // string
};

dojo.string.trim = function(/*String*/ str){
	// summary: trims whitespaces from both sides of the string
	// description:
	//	This version of trim() was taken from Steven Levithan's blog: 
	//	http://blog.stevenlevithan.com/archives/faster-trim-javascript.
	//	The short yet good-performing version of this function is 
	//	dojo.trim(), which is part of the base.
	str = str.replace(/^\s+/, '');
	for(var i = str.length - 1; i > 0; i--){
		if(/\S/.test(str.charAt(i))){
			str = str.substring(0, i + 1);
			break;
		}
	}
	return str;	// String
};

}

if(!dojo._hasResource["dijit._Templated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Templated"] = true;
dojo.provide("dijit._Templated");






dojo.declare("dijit._Templated",
	null,
	{
		// summary:
		//		mixin for widgets that are instantiated from a template

		// templateNode: DomNode
		//		a node that represents the widget template. Pre-empts both templateString and templatePath.
		templateNode: null,

		// templateString String:
		//		a string that represents the widget template. Pre-empts the
		//		templatePath. In builds that have their strings "interned", the
		//		templatePath is converted to an inline templateString, thereby
		//		preventing a synchronous network call.
		templateString: null,

		// templatePath: String
		//	Path to template (HTML file) for this widget
		templatePath: null,

		// widgetsInTemplate Boolean:
		//		should we parse the template to find widgets that might be
		//		declared in markup inside it? false by default.
		widgetsInTemplate: false,

		// containerNode DomNode:
		//		holds child elements. "containerNode" is generally set via a
		//		dojoAttachPoint assignment and it designates where children of
		//		the src dom node will be placed
		containerNode: null,

		// skipNodeCache Boolean:
		//		if using a cached widget template node poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// method over-ride
		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = dijit._Templated.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache);

			var node;
			if(dojo.isString(cached)){
				var className = this.declaredClass, _this = this;
				// Cache contains a string because we need to do property replacement
				// do the property replacement
				var tstr = dojo.string.substitute(cached, this, function(value, key){
					if(key.charAt(0) == '!'){ value = _this[key.substr(1)]; }
					if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
					if(!value){ return ""; }

					// Substitution keys beginning with ! will skip the transform step,
					// in case a user wishes to insert unescaped markup, e.g. ${!foo}
					return key.charAt(0) == "!" ? value :
						// Safer substitution, see heading "Attribute values" in
						// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
						value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
				}, this);

				node = dijit._Templated._createNodesFromText(tstr)[0];
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			// recurse through the node, looking for, and attaching to, our
			// attachment points which should be defined on the template node.
			this._attachTemplateNodes(node);

			var source = this.srcNodeRef;
			if(source && source.parentNode){
				source.parentNode.replaceChild(node, source);
			}

			this.domNode = node;
			if(this.widgetsInTemplate){
				var childWidgets = dojo.parser.parse(node);
				this._attachTemplateNodes(childWidgets, function(n,p){
					return n[p];
				});
			}

			this._fillContent(source);
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		relocate source contents to templated container node
			//		this.containerNode must be able to receive children, or exceptions will be thrown
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint
			//			* dojoAttachEvent	
			//			* waiRole
			//			* waiState
			// rootNode: DomNode|Array[Widgets]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: function?
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget

			getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };

			var nodes = dojo.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x=dojo.isArray(rootNode)?0:-1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && getAttrFunc(baseNode,'dojoType')){
					continue;
				}
				// Process dojoAttachPoint
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while(point=points.shift()){
						if(dojo.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
					}
				}

				// Process dojoAttachEvent
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent");
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = dojo.trim;
					while(event=events.shift()){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							this.connect(baseNode, event, thisFunc);
						}
					}
				}

				// waiRole, waiState
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					dijit.setWaiRole(baseNode, role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							dijit.setWaiState(baseNode, pair[0], pair[1]);
						}
					});
				}

			}
		}
	}
);

// key is either templatePath or templateString; object is either string or DOM tree
dijit._Templated._templateCache = {};

dijit._Templated.getCachedTemplate = function(templatePath, templateString, alwaysUseString){
	// summary:
	//		static method to get a template based on the templatePath or
	//		templateString key
	// templatePath: String
	//		the URL to get the template from. dojo.uri.Uri is often passed as well.
	// templateString: String?
	//		a string to use in lieu of fetching the template from a URL
	// Returns:
	//	Either string (if there are ${} variables that need to be replaced) or just
	//	a DOM tree (if the node can be cloned directly)

	// is it already cached?
	var tmplts = dijit._Templated._templateCache;
	var key = templateString || templatePath;
	var cached = tmplts[key];
	if(cached){
		return cached;
	}

	// If necessary, load template string from template path
	if(!templateString){
		templateString = dijit._Templated._sanitizeTemplateString(dojo._getText(templatePath));
	}

	templateString = dojo.string.trim(templateString);

	if(templateString.match(/\$\{([^\}]+)\}/g) || alwaysUseString){
		// there are variables in the template so all we can do is cache the string
		return (tmplts[key] = templateString); //String
	}else{
		// there are no variables in the template so we can cache the DOM tree
		return (tmplts[key] = dijit._Templated._createNodesFromText(templateString)[0]); //Node
	}
};

dijit._Templated._sanitizeTemplateString = function(/*String*/tString){
	// summary: 
	//		Strips <?xml ...?> declarations so that external SVG and XML
	// 		documents can be added to a document without worry. Also, if the string
	//		is an HTML document, only the part inside the body tag is returned.
	if(tString){
		tString = tString.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
		var matches = tString.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
		if(matches){
			tString = matches[1];
		}
	}else{
		tString = "";
	}
	return tString; //String
};


if(dojo.isIE){
	dojo.addOnUnload(function(){
		var cache = dijit._Templated._templateCache;
		for(var key in cache){
			var value = cache[key];
			if(!isNaN(value.nodeType)){ // isNode equivalent
				dojo._destroyElement(value);
			}
			delete cache[key];
		}
	});
}

(function(){
	var tagMap = {
		cell: {re: /^<t[dh][\s\r\n>]/i, pre: "<table><tbody><tr>", post: "</tr></tbody></table>"},
		row: {re: /^<tr[\s\r\n>]/i, pre: "<table><tbody>", post: "</tbody></table>"},
		section: {re: /^<(thead|tbody|tfoot)[\s\r\n>]/i, pre: "<table>", post: "</table>"}
	};

	// dummy container node used temporarily to hold nodes being created
	var tn;

	dijit._Templated._createNodesFromText = function(/*String*/text){
		// summary:
		//	Attempts to create a set of nodes based on the structure of the passed text.

		if(!tn){
			tn = dojo.doc.createElement("div");
			tn.style.display="none";
			dojo.body().appendChild(tn);
		}
		var tableType = "none";
		var rtext = text.replace(/^\s+/, "");
		for(var type in tagMap){
			var map = tagMap[type];
			if(map.re.test(rtext)){
				tableType = type;
				text = map.pre + text + map.post;
				break;
			}
		}

		tn.innerHTML = text;
		if(tn.normalize){
			tn.normalize();
		}

		var tag = { cell: "tr", row: "tbody", section: "table" }[tableType];
		var _parent = (typeof tag != "undefined") ?
						tn.getElementsByTagName(tag)[0] :
						tn;

		var nodes = [];
		while(_parent.firstChild){
			nodes.push(_parent.removeChild(_parent.firstChild));
		}
		tn.innerHTML="";
		return nodes;	//	Array
	}
})();

// These arguments can be specified for widgets which are used in templates.
// Since any widget can be specified as sub widgets in template, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget,{
	dojoAttachEvent: "",
	dojoAttachPoint: "",
	waiRole: "",
	waiState:""
})

}

if(!dojo._hasResource["dijit._Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Container"] = true;
dojo.provide("dijit._Container");

dojo.declare("dijit._Contained",
	null,
	{
		// summary
		//		Mixin for widgets that are children of a container widget

		getParent: function(){
			// summary:
			//		returns the parent widget of this widget, assuming the parent
			//		implements dijit._Container
			for(var p=this.domNode.parentNode; p; p=p.parentNode){
				var id = p.getAttribute && p.getAttribute("widgetId");
				if(id){
					var parent = dijit.byId(id);
					return parent.isContainer ? parent : null;
				}
			}
			return null;
		},

		_getSibling: function(which){
			var node = this.domNode;
			do{
				node = node[which+"Sibling"];
			}while(node && node.nodeType != 1);
			if(!node){ return null; } // null
			var id = node.getAttribute("widgetId");
			return dijit.byId(id);
		},

		getPreviousSibling: function(){
			// summary:
			//		returns null if this is the first child of the parent,
			//		otherwise returns the next element sibling to the "left".

			return this._getSibling("previous");
		},

		getNextSibling: function(){
			// summary:
			//		returns null if this is the last child of the parent,
			//		otherwise returns the next element sibling to the "right".

			return this._getSibling("next");
		}
	}
);

dojo.declare("dijit._Container",
	null,
	{
		// summary
		//		Mixin for widgets that contain a list of children like SplitContainer

		isContainer: true,

		addChild: function(/*Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Process the given child widget, inserting it's dom node as
			//		a child of our dom node

			if(insertIndex === undefined){
				insertIndex = "last";
			}
			var refNode = this.containerNode || this.domNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = dojo.query("> [widgetid]", refNode);
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1]; insertIndex = "after";
				}
			}
			dojo.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget*/ widget){
			// summary:
			//		removes the passed widget instance from this widget but does
			//		not destroy it
			var node = widget.domNode;
			node.parentNode.removeChild(node);	// detach but don't destroy
		},

		_nextElement: function(node){
			do{
				node = node.nextSibling;
			}while(node && node.nodeType != 1);
			return node;
		},

		_firstElement: function(node){
			node = node.firstChild;
			if(node && node.nodeType != 1){
				node = this._nextElement(node);
			}
			return node;
		},

		getChildren: function(){
			// summary:
			//		Returns array of children widgets
			return dojo.query("> [widgetId]", this.containerNode || this.domNode).map(dijit.byNode); // Array
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has children
			var cn = this.containerNode || this.domNode;
			return !!this._firstElement(cn); // Boolean
		},

		_getSiblingOfChild: function(/*Widget*/ child, /*int*/ dir){
			// summary:
			//		get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			var node = child.domNode;
			var which = (dir>0 ? "nextSibling" : "previousSibling");
			do{
				node = node[which];
			}while(node && (node.nodeType != 1 || !dijit.byNode(node)));
			return node ? dijit.byNode(node) : null;
		}
	}
);

dojo.declare("dijit._KeyNavContainer",
	[dijit._Container],
	{

		// summary:
		//		A _Container with keyboard navigation of its children.
		//		To use this mixin, call connectKeyNavHandlers() in
		//		postCreate() and call startupKeyNavChildren() in startup().

/*=====
		// focusedChild: Widget
		//		The currently focused child widget, or null if there isn't one
		focusedChild: null,
=====*/

		_keyNavCodes: {},

		connectKeyNavHandlers: function(/*Array*/ prevKeyCodes, /*Array*/ nextKeyCodes){
			// summary:
			//		Call in postCreate() to attach the keyboard handlers
			//		to the container.
			// preKeyCodes: Array
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: Array
			//		Key codes for navigating to the next child.

			var keyCodes = this._keyNavCodes = {};
			var prev = dojo.hitch(this, this.focusPrev);
			var next = dojo.hitch(this, this.focusNext);
			dojo.forEach(prevKeyCodes, function(code){ keyCodes[code] = prev });
			dojo.forEach(nextKeyCodes, function(code){ keyCodes[code] = next });
			this.connect(this.domNode, "onkeypress", "_onContainerKeypress");
			if(dojo.isIE){
				this.connect(this.domNode, "onactivate", "_onContainerFocus");
				this.connect(this.domNode, "ondeactivate", "_onContainerBlur");
			}else{
				this.connect(this.domNode, "onfocus", "_onContainerFocus");
				this.connect(this.domNode, "onblur", "_onContainerBlur");
			}
		},

		startupKeyNavChildren: function(){
			// summary:
			//		Call in startup() to set child tabindexes to -1
			dojo.forEach(this.getChildren(), dojo.hitch(this, "_setTabIndexMinusOne"));
		},

		addChild: function(/*Widget*/ widget, /*int?*/ insertIndex){
			// summary: Add a child to our _Container
			dijit._KeyNavContainer.superclass.addChild.apply(this, arguments);
			this._setTabIndexMinusOne(widget);
		},

		focus: function(){
			// summary: Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		focusFirstChild: function(){
			// summary: Focus the first focusable child in the container.
			this.focusChild(this._getFirstFocusableChild());
		},

		focusNext: function(){
			// summary: Focus the next widget or focal node (for widgets
			//		with multiple focal nodes) within this container.
			if(this.focusedChild && this.focusedChild.hasNextFocalNode
					&& this.focusedChild.hasNextFocalNode()){
				this.focusedChild.focusNext();
				return;
			}
			var child = this._getNextFocusableChild(this.focusedChild, 1);
			if(child.getFocalNodes){
				this.focusChild(child, child.getFocalNodes()[0]);
			}else{
				this.focusChild(child);
			}
		},

		focusPrev: function(){
			// summary: Focus the previous widget or focal node (for widgets
			//		with multiple focal nodes) within this container.
			if(this.focusedChild && this.focusedChild.hasPrevFocalNode
					&& this.focusedChild.hasPrevFocalNode()){
				this.focusedChild.focusPrev();
				return;
			}
			var child = this._getNextFocusableChild(this.focusedChild, -1);
			if(child.getFocalNodes){
				var nodes = child.getFocalNodes();
				this.focusChild(child, nodes[nodes.length-1]);
			}else{
				this.focusChild(child);
			}
		},

		focusChild: function(/*Widget*/ widget, /*Node?*/ node){
			// summary: Focus widget. Optionally focus 'node' within widget.
			if(widget){
				if(this.focusedChild && widget !== this.focusedChild){
					this._onChildBlur(this.focusedChild);
				}
				this.focusedChild = widget;
				if(node && widget.focusFocalNode){
					widget.focusFocalNode(node);
				}else{
					widget.focus();
				}
			}
		},

		_setTabIndexMinusOne: function(/*Widget*/ widget){
			if(widget.getFocalNodes){
				dojo.forEach(widget.getFocalNodes(), function(node){
					node.setAttribute("tabIndex", -1);
				});
			}else{
				(widget.focusNode || widget.domNode).setAttribute("tabIndex", -1);
			}
		},

		_onContainerFocus: function(evt){
			this.domNode.setAttribute("tabIndex", -1);
			if(evt.target === this.domNode){
				this.focusFirstChild();
			}else{
				var widget = dijit.getEnclosingWidget(evt.target);
				if(widget && widget.isFocusable()){
					this.focusedChild = widget;
				}
			}
		},

		_onContainerBlur: function(evt){
			if(this.tabIndex){
				this.domNode.setAttribute("tabIndex", this.tabIndex);
			}
		},

		_onContainerKeypress: function(evt){
			if(evt.ctrlKey || evt.altKey){ return; }
			var func = this._keyNavCodes[evt.keyCode];
			if(func){
				func();
				dojo.stopEvent(evt);
			}
		},

		_onChildBlur: function(/*Widget*/ widget){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
		},

		_getFirstFocusableChild: function(){
			return this._getNextFocusableChild(null, 1);
		},

		_getNextFocusableChild: function(child, dir){
			if(child){
				child = this._getSiblingOfChild(child, dir);
			}
			var children = this.getChildren();
			for(var i=0; i < children.length; i++){
				if(!child){
					child = children[(dir>0) ? 0 : (children.length-1)];
				}
				if(child.isFocusable()){
					return child;
				}
				child = this._getSiblingOfChild(child, dir);
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.layout._LayoutWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._LayoutWidget"] = true;
dojo.provide("dijit.layout._LayoutWidget");




dojo.declare("dijit.layout._LayoutWidget",
	[dijit._Widget, dijit._Container, dijit._Contained],
	{
		// summary
		//		Mixin for widgets that contain a list of children like SplitContainer.
		//		Widgets which mixin this code must define layout() to lay out the children

		isLayoutContainer: true,

		postCreate: function(){
			dojo.addClass(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under document.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }
			this._started=true;

			if(this.getChildren){
				dojo.forEach(this.getChildren(), function(child){ child.startup(); });
			}

			// If I am a top level widget
			if(!this.getParent || !this.getParent()){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// since my parent isn't a layout container, and my style is width=height=100% (or something similar),
				// then I need to watch when the window resizes, and size myself accordingly
				// (passing in no argument to resize means that it has to glean the size itself)
				this.connect(window, 'onresize', function(){this.resize();});
			}
		},

		resize: function(args){
			// summary:
			//		Explicitly set this widget's size (in pixels),
			//		and then call layout() to resize contents (and maybe adjust child widgets)
			//	
			// args: Object?
			//		{w: int, h: int, l: int, t: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(args){
				dojo.marginBox(node, args);

				// set offset of the node
				if(args.t){ node.style.top = args.t + "px"; }
				if(args.l){ node.style.left = args.l + "px"; }
			}
			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = dojo.mixin(dojo.marginBox(node), args||{});

			// Save the size of my content box.
			this._contentBox = dijit.layout.marginBox2contentBox(node, mb);

			// Callback for widget to adjust size of it's children
			this.layout();
		},

		layout: function(){
			//	summary
			//		Widgets override this method to size & position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
		}
	}
);

dijit.layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
	// summary:
	//		Given the margin-box size of a node, return it's content box size.
	//		Functions like dojo.contentBox() but is more reliable since it doesn't have
	//		to wait for the browser to compute sizes.
	var cs = dojo.getComputedStyle(node);
	var me=dojo._getMarginExtents(node, cs);
	var pb=dojo._getPadBorderExtents(node, cs);
	return {
		l: dojo._toPixelValue(node, cs.paddingLeft),
		t: dojo._toPixelValue(node, cs.paddingTop),
		w: mb.w - (me.w + pb.w),
		h: mb.h - (me.h + pb.h)
	};
};

(function(){
	var capitalize = function(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	};

	var size = function(widget, dim){
		// size the child
		widget.resize ? widget.resize(dim) : dojo.marginBox(widget.domNode, dim);

		// record child's size, but favor our own numbers when we have them.
		// the browser lies sometimes
		dojo.mixin(widget, dojo.marginBox(widget.domNode));
		dojo.mixin(widget, dim);
	};

	dijit.layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Object[]*/ children){
		/**
		 * summary
		 *		Layout a bunch of child dom nodes within a parent dom node
		 * container:
		 *		parent node
		 * dim:
		 *		{l, t, w, h} object specifying dimensions of container into which to place children
		 * children:
		 *		an array like [ {domNode: foo, layoutAlign: "bottom" }, {domNode: bar, layoutAlign: "client"} ]
		 */

		// copy dim because we are going to modify it
		dim = dojo.mixin({}, dim);

		dojo.addClass(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.
		children = dojo.filter(children, function(item){ return item.layoutAlign != "client"; })
			.concat(dojo.filter(children, function(item){ return item.layoutAlign == "client"; }));

		// set positions/sizes
		dojo.forEach(children, function(child){
			var elm = child.domNode,
				pos = child.layoutAlign;

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.bottom = elmStyle.right = "auto";

			dojo.addClass(elm, "dijitAlign" + capitalize(pos));

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect it's height.
			if(pos=="top" || pos=="bottom"){
				size(child, { w: dim.w });
				dim.h -= child.h;
				if(pos=="top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos=="left" || pos=="right"){
				size(child, { h: dim.h });
				dim.w -= child.w;
				if(pos=="left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos=="client"){
				size(child, dim);
			}
		});
	};

})();

}

if(!dojo._hasResource["dijit.form._FormWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormWidget"] = true;
dojo.provide("dijit.form._FormWidget");




dojo.declare("dijit.form._FormWidget", [dijit._Widget, dijit._Templated],
{
	/*
	Summary:
		FormElement widgets correspond to native HTML elements such as <input> or <button> or <select>.
		Each FormElement represents a single input value, and has a (possibly hidden) <input> element,
		to which it serializes its input value, so that form submission (either normal submission or via FormBind?)
		works as expected.

		All these widgets should have these attributes just like native HTML input elements.
		You can set them during widget construction, but after that they are read only.

		They also share some common methods.
	*/

	// baseClass: String
	//		Root CSS class of the widget (ex: dijitTextBox), used to add CSS classes of widget
	//		(ex: "dijitTextBox dijitTextBoxInvalid dijitTextBoxFocused dijitTextBoxInvalidFocused")
	//		See _setStateClass().
	baseClass: "",

	// value: String
	//		Corresponds to the native HTML <input> element's attribute.
	value: "",

	// name: String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// id: String
	//		Corresponds to the native HTML <input> element's attribute.
	//		Also becomes the id for the widget.
	id: "",

	// alt: String
	//		Corresponds to the native HTML <input> element's attribute.
	alt: "",

	// type: String
	//		Corresponds to the native HTML <input> element's attribute.
	type: "text",

	// tabIndex: Integer
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// These mixins assume that the focus node is an INPUT, as many but not all _FormWidgets are.
	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically 
	// require the 'name' attribute at element creation time.
	attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
		{id:"focusNode", tabIndex:"focusNode", alt:"focusNode"}),

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Set disabled state of widget.

		this.domNode.disabled = this.disabled = disabled;
		if(this.focusNode){
			this.focusNode.disabled = disabled;
		}
		if(disabled){
			//reset those, because after the domNode is disabled, we can no longer receive
			//mouse related events, see #4200
			this._hovering = false;
			this._active = false;
		}
		dijit.setWaiState(this.focusNode || this.domNode, "disabled", disabled);
		this._setStateClass();
	},


	_onMouse : function(/*Event*/ event){
		// summary:
		//	Sets _hovering, _active, and stateModifier properties depending on mouse state,
		//	then calls setStateClass() to set appropriate CSS classes for this.domNode.
		//
		//	To get a different CSS class for hover, send onmouseover and onmouseout events to this method.
		//	To get a different CSS class while mouse button is depressed, send onmousedown to this method.

		var mouseNode = event.target;
		if(mouseNode && mouseNode.getAttribute){
			this.stateModifier = mouseNode.getAttribute("stateModifier") || "";
		}

		if(!this.disabled){
			switch(event.type){
				case "mouseenter" :	
				case "mouseover" :
					this._hovering = true;
					break;

				case "mouseout" :	
				case "mouseleave" :	
					this._hovering = false;
					break;

				case "mousedown" :
					this._active = true;
					// set a global event to handle mouseup, so it fires properly
					//	even if the cursor leaves the button
					var self = this;
					// #2685: use this.connect and disconnect so destroy works properly
					var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
						self._active = false;
						self._setStateClass();
						self.disconnect(mouseUpConnector);
					});
					break;
			}
			this._setStateClass();
		}
	},

	isFocusable: function(){
		return !this.disabled && (dojo.style(this.domNode, "display") != "none");
	},

	focus: function(){
		dijit.focus(this.focusNode);
	},

	_setStateClass: function(){
		// summary
		//	Update the visual state of the widget by setting the css classes on this.domNode
		//  (or this.stateNode if defined) by combining this.baseClass with
		//	various suffixes that represent the current widget state(s).
		//
		//	In the case where a widget has multiple
		//	states, it sets the class based on all possible
		//  combinations.  For example, an invalid form widget that is being hovered
		//	will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//	For complex widgets with multiple regions, there can be various hover/active states,
		//	such as "Hover" or "CloseButtonHover" (for tab buttons).
		//	This is controlled by a stateModifier="CloseButton" attribute on the close button node.
		//
		//	The widget may have one or more of the following states, determined
		//	by this.state, this.checked, this.valid, and this.selected:
		//		Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//		Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//		Selected - ex: currently selected tab will have this.selected==true
		//
		//	In addition, it may have at most one of the following states,
		//	based on this.disabled and flags set in _onMouse (this._active, this._hovering, this._focused):
		//		Disabled	- if the widget is disabled
		//		Active		- if the mouse (or space/enter key?) is being pressed down
		//		Focused		- if the widget has focus
		//		Hover		- if the mouse is over the widget
		//
		//	(even if multiple af the above conditions are true we only pick the first matching one)


		// Get original (non state related, non baseClass related) class specified in template
		if(!("staticClass" in this)){
			this.staticClass = (this.stateNode||this.domNode).className;
		}

		// Compute new set of classes
		var classes = [ this.baseClass ];

		function multiply(modifier){
			classes=classes.concat(dojo.map(classes, function(c){ return c+modifier; }));
		}

		if(this.checked){
			multiply("Checked");
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}

		// Only one of these four can be applied.
		// Active trumps Focused, Focused trumps Hover, and Disabled trumps all.
		if(this.disabled){
			multiply("Disabled");
		}else if(this._active){
			multiply(this.stateModifier+"Active");
		}else{
			if(this._focused){
				multiply("Focused");
			}
			if((this.stateModifier || !this._focused) && this._hovering){
				multiply(this.stateModifier+"Hover");
			}
		}
		(this.stateNode || this.domNode).className = this.staticClass + " " + classes.join(" ");
	},

	onChange: function(newValue){
		// summary: callback when value is changed
	},

	postCreate: function(){
		this.setValue(this.value, null); // null reserved for initial value
		this.setDisabled(this.disabled);
		this._setStateClass();
	},

	setValue: function(/*anything*/ newValue, /*Boolean, optional*/ priorityChange){
		// summary: set the value of the widget.
		this._lastValue = newValue;
		dijit.setWaiState(this.focusNode || this.domNode, "valuenow", this.forWaiValuenow());
		if(priorityChange === undefined){ priorityChange = true; } // setValue with value only should fire onChange
		if(this._lastValueReported == undefined && priorityChange === null){ // don't report the initial value
			this._lastValueReported = newValue;
		}
		if((this.intermediateChanges || priorityChange) && 
			((newValue && newValue.toString)?newValue.toString():newValue) !== ((this._lastValueReported && this._lastValueReported.toString)?this._lastValueReported.toString():this._lastValueReported)){
			this._lastValueReported = newValue;
			this.onChange(newValue);
		}
	},

	getValue: function(){
		// summary: get the value of the widget.
		return this._lastValue;
	},

	undo: function(){
		// summary: restore the value to the last value passed to onChange
		this.setValue(this._lastValueReported, false);
	},

	_onKeyPress: function(e){
		if(e.keyCode == dojo.keys.ESCAPE && !e.shiftKey && !e.ctrlKey && !e.altKey){
			var v = this.getValue();
			var lv = this._lastValueReported;
			// Equality comparison of objects such as dates are done by reference so
			// two distinct objects are != even if they have the same data. So use
			// toStrings in case the values are objects.
			if((typeof lv != "undefined") && ((v!==null && v.toString)?v.toString():null) !== lv.toString()){	
				this.undo();
				dojo.stopEvent(e);
				return false;
			}
		}
		return true;
	},

	forWaiValuenow: function(){
		// summary: returns a value, reflecting the current state of the widget,
		//		to be used for the ARIA valuenow.
		// 		This method may be overridden by subclasses that want
		// 		to use something other than this.getValue() for valuenow
		return this.getValue();
	}
});

}

if(!dojo._hasResource["dijit.dijit"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.dijit"] = true;
dojo.provide("dijit.dijit");

// All the stuff in _base (these are the function that are guaranteed available without an explicit dojo.require)


// And some other stuff that we tend to pull in all the time anyway







}

if(!dojo._hasResource["dojo.i18n"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.i18n"] = true;
dojo.provide("dojo.i18n");

dojo.i18n.getLocalization = function(/*String*/packageName, /*String*/bundleName, /*String?*/locale){
	//	summary:
	//		Returns an Object containing the localization for a given resource
	//		bundle in a package, matching the specified locale.
	//	description:
	//		Returns a hash containing name/value pairs in its prototypesuch
	//		that values can be easily overridden.  Throws an exception if the
	//		bundle is not found.  Bundle must have already been loaded by
	//		dojo.requireLocalization() or by a build optimization step.  NOTE:
	//		try not to call this method as part of an object property
	//		definition (var foo = { bar: dojo.i18n.getLocalization() }).  In
	//		some loading situations, the bundle may not be available in time
	//		for the object definition.  Instead, call this method inside a
	//		function that is run after all modules load or the page loads (like
	//		in dojo.adOnLoad()), or in a widget lifecycle method.
	//	packageName:
	//		package which is associated with this resource
	//	bundleName:
	//		the base filename of the resource bundle (without the ".js" suffix)
	//	locale:
	//		the variant to load (optional).  By default, the locale defined by
	//		the host environment: dojo.locale

	locale = dojo.i18n.normalizeLocale(locale);

	// look for nearest locale match
	var elements = locale.split('-');
	var module = [packageName,"nls",bundleName].join('.');
	var bundle = dojo._loadedModules[module];
	if(bundle){
		var localization;
		for(var i = elements.length; i > 0; i--){
			var loc = elements.slice(0, i).join('_');
			if(bundle[loc]){
				localization = bundle[loc];
				break;
			}
		}
		if(!localization){
			localization = bundle.ROOT;
		}

		// make a singleton prototype so that the caller won't accidentally change the values globally
		if(localization){
			var clazz = function(){};
			clazz.prototype = localization;
			return new clazz(); // Object
		}
	}

	throw new Error("Bundle not found: " + bundleName + " in " + packageName+" , locale=" + locale);
};

dojo.i18n.normalizeLocale = function(/*String?*/locale){
	//	summary:
	//		Returns canonical form of locale, as used by Dojo.
	//
	//  description:
	//		All variants are case-insensitive and are separated by '-' as specified in RFC 3066.
	//		If no locale is specified, the dojo.locale is returned.  dojo.locale is defined by
	//		the user agent's locale unless overridden by djConfig.

	var result = locale ? locale.toLowerCase() : dojo.locale;
	if(result == "root"){
		result = "ROOT";
	}
	return result; // String
};

dojo.i18n._requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale, /*String?*/availableFlatLocales){
	//	summary:
	//		See dojo.requireLocalization()
	//	description:
	// 		Called by the bootstrap, but factored out so that it is only
	// 		included in the build when needed.

	var targetLocale = dojo.i18n.normalizeLocale(locale);
 	var bundlePackage = [moduleName, "nls", bundleName].join(".");
	// NOTE: 
	//		When loading these resources, the packaging does not match what is
	//		on disk.  This is an implementation detail, as this is just a
	//		private data structure to hold the loaded resources.  e.g.
	//		tests/hello/nls/en-us/salutations.js is loaded as the object
	//		tests.hello.nls.salutations.en_us={...} The structure on disk is
	//		intended to be most convenient for developers and translators, but
	//		in memory it is more logical and efficient to store in a different
	//		order.  Locales cannot use dashes, since the resulting path will
	//		not evaluate as valid JS, so we translate them to underscores.
	
	//Find the best-match locale to load if we have available flat locales.
	var bestLocale = "";
	if(availableFlatLocales){
		var flatLocales = availableFlatLocales.split(",");
		for(var i = 0; i < flatLocales.length; i++){
			//Locale must match from start of string.
			if(targetLocale.indexOf(flatLocales[i]) == 0){
				if(flatLocales[i].length > bestLocale.length){
					bestLocale = flatLocales[i];
				}
			}
		}
		if(!bestLocale){
			bestLocale = "ROOT";
		}		
	}

	//See if the desired locale is already loaded.
	var tempLocale = availableFlatLocales ? bestLocale : targetLocale;
	var bundle = dojo._loadedModules[bundlePackage];
	var localizedBundle = null;
	if(bundle){
		if(djConfig.localizationComplete && bundle._built){return;}
		var jsLoc = tempLocale.replace(/-/g, '_');
		var translationPackage = bundlePackage+"."+jsLoc;
		localizedBundle = dojo._loadedModules[translationPackage];
	}

	if(!localizedBundle){
		bundle = dojo["provide"](bundlePackage);
		var syms = dojo._getModuleSymbols(moduleName);
		var modpath = syms.concat("nls").join("/");
		var parent;

		dojo.i18n._searchLocalePath(tempLocale, availableFlatLocales, function(loc){
			var jsLoc = loc.replace(/-/g, '_');
			var translationPackage = bundlePackage + "." + jsLoc;
			var loaded = false;
			if(!dojo._loadedModules[translationPackage]){
				// Mark loaded whether it's found or not, so that further load attempts will not be made
				dojo["provide"](translationPackage);
				var module = [modpath];
				if(loc != "ROOT"){module.push(loc);}
				module.push(bundleName);
				var filespec = module.join("/") + '.js';
				loaded = dojo._loadPath(filespec, null, function(hash){
					// Use singleton with prototype to point to parent bundle, then mix-in result from loadPath
					var clazz = function(){};
					clazz.prototype = parent;
					bundle[jsLoc] = new clazz();
					for(var j in hash){ bundle[jsLoc][j] = hash[j]; }
				});
			}else{
				loaded = true;
			}
			if(loaded && bundle[jsLoc]){
				parent = bundle[jsLoc];
			}else{
				bundle[jsLoc] = parent;
			}
			
			if(availableFlatLocales){
				//Stop the locale path searching if we know the availableFlatLocales, since
				//the first call to this function will load the only bundle that is needed.
				return true;
			}
		});
	}

	//Save the best locale bundle as the target locale bundle when we know the
	//the available bundles.
	if(availableFlatLocales && targetLocale != bestLocale){
		bundle[targetLocale.replace(/-/g, '_')] = bundle[bestLocale.replace(/-/g, '_')];
	}
};

(function(){
	// If other locales are used, dojo.requireLocalization should load them as
	// well, by default. 
	// 
	// Override dojo.requireLocalization to do load the default bundle, then
	// iterate through the extraLocale list and load those translations as
	// well, unless a particular locale was requested.

	var extra = djConfig.extraLocale;
	if(extra){
		if(!extra instanceof Array){
			extra = [extra];
		}

		var req = dojo.i18n._requireLocalization;
		dojo.i18n._requireLocalization = function(m, b, locale, availableFlatLocales){
			req(m,b,locale, availableFlatLocales);
			if(locale){return;}
			for(var i=0; i<extra.length; i++){
				req(m,b,extra[i], availableFlatLocales);
			}
		};
	}
})();

dojo.i18n._searchLocalePath = function(/*String*/locale, /*Boolean*/down, /*Function*/searchFunc){
	//	summary:
	//		A helper method to assist in searching for locale-based resources.
	//		Will iterate through the variants of a particular locale, either up
	//		or down, executing a callback function.  For example, "en-us" and
	//		true will try "en-us" followed by "en" and finally "ROOT".

	locale = dojo.i18n.normalizeLocale(locale);

	var elements = locale.split('-');
	var searchlist = [];
	for(var i = elements.length; i > 0; i--){
		searchlist.push(elements.slice(0, i).join('-'));
	}
	searchlist.push(false);
	if(down){searchlist.reverse();}

	for(var j = searchlist.length - 1; j >= 0; j--){
		var loc = searchlist[j] || "ROOT";
		var stop = searchFunc(loc);
		if(stop){ break; }
	}
};

dojo.i18n._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated){
	//	summary:
	//		Load built, flattened resource bundles, if available for all
	//		locales used in the page. Only called by built layer files.

	function preload(locale){
		locale = dojo.i18n.normalizeLocale(locale);
		dojo.i18n._searchLocalePath(locale, true, function(loc){
			for(var i=0; i<localesGenerated.length;i++){
				if(localesGenerated[i] == loc){
					dojo["require"](bundlePrefix+"_"+loc);
					return true; // Boolean
				}
			}
			return false; // Boolean
		});
	}
	preload();
	var extra = djConfig.extraLocale||[];
	for(var i=0; i<extra.length; i++){
		preload(extra[i]);
	}
};

}

if(!dojo._hasResource["dijit.layout.ContentPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.ContentPane"] = true;
dojo.provide("dijit.layout.ContentPane");








dojo.declare(
	"dijit.layout.ContentPane",
	dijit._Widget,
{
	// summary:
	//		A widget that acts as a Container for other widgets, and includes a ajax interface
	// description:
	//		A widget that can be used as a standalone widget
	//		or as a baseclass for other widgets
	//		Handles replacement of document fragment using either external uri or javascript
	//		generated markup or DOM content, instantiating widgets within that content.
	//		Don't confuse it with an iframe, it only needs/wants document fragments.
	//		It's useful as a child of LayoutContainer, SplitContainer, or TabContainer.
	//		But note that those classes can contain any widget as a child.
	// example:
	//		Some quick samples:
	//		To change the innerHTML use .setContent('<b>new content</b>')
	//
	//		Or you can send it a NodeList, .setContent(dojo.query('div [class=selected]', userSelection))
	//		please note that the nodes in NodeList will copied, not moved
	//
	//		To do a ajax update use .setHref('url')
	//
	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; see setHref();
	href: "",

	// extractContent: Boolean
	//	Extract visible content from inside of <body> .... </body>
	extractContent: false,

	// parseOnLoad: Boolean
	//	parse content and create the widgets, if any
	parseOnLoad:	true,

	// preventCache: Boolean
	//		Cache content retreived externally
	preventCache:	false,

	// preload: Boolean
	//	Force load of data even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//	Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'>${loadingState}</span>", 

	// errorMessage: String
	//	Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'>${errorState}</span>", 

	// isLoaded: Boolean
	//	Tells loading status see onLoad|onUnload for event hooks
	isLoaded: false,

	// class: String
	//	Class name to apply to ContentPane dom nodes
	"class": "dijitContentPane",

	postCreate: function(){
		// remove the title attribute so it doesn't show up when i hover
		// over a node
		this.domNode.title = "";

		if(this.preload){
			this._loadCheck();
		}

		var messages = dojo.i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = dojo.string.substitute(this.loadingMessage, messages);
		this.errorMessage = dojo.string.substitute(this.errorMessage, messages);

		// for programatically created ContentPane (with <span> tag), need to muck w/CSS
		// or it's as though overflow:visible is set
		dojo.addClass(this.domNode, this["class"]);
	},

	startup: function(){
		if(this._started){ return; }
		this._checkIfSingleChild();
		if(this._singleChild){
			this._singleChild.startup();
		}
		this._loadCheck();
		this._started = true;
	},

	_checkIfSingleChild: function(){
		// summary:
		// 	Test if we have exactly one widget as a child, and if so assume that we are a container for that widget,
		//	and should propogate startup() and resize() calls to it.
		var childNodes = dojo.query(">", this.containerNode || this.domNode),
			childWidgets = childNodes.filter("[widgetId]");

		if(childNodes.length == 1 && childWidgets.length == 1){
			this.isContainer = true;
			this._singleChild = dijit.byNode(childWidgets[0]);
		}else{
			delete this.isContainer;
			delete this._singleChild;
		}
	},

	refresh: function(){
		// summary:
		//	Force a refresh (re-download) of content, be sure to turn off cache

		// we return result of _prepareLoad here to avoid code dup. in dojox.layout.ContentPane
		return this._prepareLoad(true);
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false
		//	href:
		//		url to the page you want to get, must be within the same domain as your mainpage
		this.href = href;

		// we return result of _prepareLoad here to avoid code dup. in dojox.layout.ContentPane
		return this._prepareLoad();
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Replaces old content with data content, include style classes from old content
		//	data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we cant run refresh and clear content
		// refresh should only work if we downloaded the content
		if(!this._isDownloaded){
			this.href = "";
			this._onUnloadHandler();
		}

		this._setContent(data || "");

		this._isDownloaded = false; // must be set after _setContent(..), pathadjust in dojox.layout.ContentPane

		if(this.parseOnLoad){
			this._createSubWidgets();
		}

		this._checkIfSingleChild();
		if(this._singleChild && this._singleChild.resize){
			this._singleChild.resize(this._contentBox);
		}

		this._onLoadHandler();
	},

	cancel: function(){
		// summary:
		//		Cancels a inflight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect
	},

	destroy: function(){
		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		// make sure we call onUnload
		this._onUnloadHandler();
		this._beingDestroyed = true;
		this.inherited("destroy",arguments);
	},

	resize: function(size){
		dojo.marginBox(this.domNode, size);

		// Compute content box size in case we [later] need to size child
		// If either height or width wasn't specified by the user, then query node for it.
		// But note that setting the margin box and then immediately querying dimensions may return
		// inaccurate results, so try not to depend on it.
		var node = this.containerNode || this.domNode,
			mb = dojo.mixin(dojo.marginBox(node), size||{});

		this._contentBox = dijit.layout.marginBox2contentBox(node, mb);

		// If we have a single widget child then size it to fit snugly within my borders
		if(this._singleChild && this._singleChild.resize){
			this._singleChild.resize(this._contentBox);
		}
	},

	_prepareLoad: function(forceLoad){
		// sets up for a xhrLoad, load is deferred until widget onShow
		// cancels a inflight download
		this.cancel();
		this.isLoaded = false;
		this._loadCheck(forceLoad);
	},

	_loadCheck: function(forceLoad){
		// call this when you change onShow (onSelected) status when selected in parent container
		// it's used as a trigger for href download when this.domNode.display != 'none'

		// sequence:
		// if no href -> bail
		// forceLoad -> always load
		// this.preload -> load when download not in progress, domNode display doesn't matter
		// this.refreshOnShow -> load when download in progress bails, domNode display !='none' AND
		//						this.open !== false (undefined is ok), isLoaded doesn't matter
		// else -> load when download not in progress, if this.open !== false (undefined is ok) AND
		//						domNode display != 'none', isLoaded must be false

		var displayState = ((this.open !== false) && (this.domNode.style.display != 'none'));

		if(this.href &&	
			(forceLoad ||
				(this.preload && !this._xhrDfd) ||
				(this.refreshOnShow && displayState && !this._xhrDfd) ||
				(!this.isLoaded && displayState && !this._xhrDfd)
			)
		){
			this._downloadExternalContent();
		}
	},

	_downloadExternalContent: function(){
		this._onUnloadHandler();

		// display loading message
		this._setContent(
			this.onDownloadStart.call(this)
		);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(dojo.isObject(this.ioArgs)){
			dojo.mixin(getArgs, this.ioArgs);
		}

		var hand = this._xhrDfd = (this.ioMethod || dojo.xhrGet)(getArgs);

		hand.addCallback(function(html){
			try{
				self.onDownloadEnd.call(self);
				self._isDownloaded = true;
				self.setContent.call(self, html); // onload event is called from here
			}catch(err){
				self._onError.call(self, 'Content', err); // onContentError
			}
			delete self._xhrDfd;
			return html;
		});

		hand.addErrback(function(err){
			if(!hand.cancelled){
				// show error message in the pane
				self._onError.call(self, 'Download', err); // onDownloadError
			}
			delete self._xhrDfd;
			return err;
		});
	},

	_onLoadHandler: function(){
		this.isLoaded = true;
		try{
			this.onLoad.call(this);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code');
		}
	},

	_onUnloadHandler: function(){
		this.isLoaded = false;
		this.cancel();
		try{
			this.onUnload.call(this);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code');
		}
	},

	_setContent: function(cont){
		this.destroyDescendants();

		try{
			var node = this.containerNode || this.domNode;
			while(node.firstChild){
				dojo._destroyElement(node.firstChild);
			}
			if(typeof cont == "string"){
				// dijit.ContentPane does only minimal fixes,
				// No pathAdjustments, script retrieval, style clean etc
				// some of these should be available in the dojox.layout.ContentPane
				if(this.extractContent){
					match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
					if(match){ cont = match[1]; }
				}
				node.innerHTML = cont;
			}else{
				// domNode or NodeList
				if(cont.nodeType){ // domNode (htmlNode 1 or textNode 3)
					node.appendChild(cont);
				}else{// nodelist or array such as dojo.Nodelist
					dojo.forEach(cont, function(n){
						node.appendChild(n.cloneNode(true));
					});
				}
			}
		}catch(e){
			// check if a domfault occurs when we are appending this.errorMessage
			// like for instance if domNode is a UL and we try append a DIV
			var errMess = this.onContentError(e);
			try{
				node.innerHTML = errMess;
			}catch(e){
				console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
			}
		}
	},

	_onError: function(type, err, consoleText){
		// shows user the string that is returned by on[type]Error
		// overide on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent.call(this, errText);
		}
	},

	_createSubWidgets: function(){
		// summary: scan my contents and create subwidgets
		var rootNode = this.containerNode || this.domNode;
		try{
			dojo.parser.parse(rootNode, true);
		}catch(e){
			this._onError('Content', e, "Couldn't create widgets in "+this.id
				+(this.href ? " from "+this.href : ""));
		}
	},

	// EVENT's, should be overide-able
	onLoad: function(e){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
	},

	onUnload: function(e){
		// summary:
		//		Event hook, is called before old content is cleared
	},

	onDownloadStart: function(){
		// summary:
		//		called before download starts
		//		the string returned by this function will be the html
		//		that tells the user we are loading something
		//		override with your own function if you want to change text
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ error){
		// summary:
		//		called on DOM faults, require fault etc in content
		//		default is to display errormessage inside pane
	},

	onDownloadError: function(/*Error*/ error){
		// summary:
		//		Called when download error occurs, default is to display
		//		errormessage inside pane. Overide function to change that.
		//		The string returned by this function will be the html
		//		that tells the user a error happend
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		called when download is finished
	}
});

}

if(!dojo._hasResource["dijit.form.Form"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Form"] = true;
dojo.provide("dijit.form.Form");




dojo.declare("dijit.form._FormMixin", null,
	{
		/*
		summary:
			Widget corresponding to <form> tag, for validation and serialization

		usage:
			<form dojoType="dijit.form.Form" id="myForm">
				Name: <input type="text" name="name" />
			</form>
			myObj={name: "John Doe"};
			dijit.byId('myForm').setValues(myObj);

			myObj=dijit.byId('myForm').getValues();
		TODO:
		* Repeater
		* better handling for arrays.  Often form elements have names with [] like
		* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])

		*/

		// HTML <FORM> attributes

		action: "",
		method: "",
		enctype: "",
		name: "",
		"accept-charset": "",
		accept: "",
		target: "",

		attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
			{action: "", method: "", enctype: "", "accept-charset": "", accept: "", target: ""}),

		// execute: Function
		//	User defined function to do stuff when the user hits the submit button
		execute: function(/*Object*/ formContents){},

		// onCancel: Function
		//	Callback when user has canceled dialog, to notify container
		//	(user shouldn't override)
		onCancel: function(){},

		// onExecute: Function
		//	Callback when user is about to execute dialog, to notify container
		//	(user shouldn't override)
		onExecute: function(){},

		templateString: "<form dojoAttachPoint='containerNode' dojoAttachEvent='onsubmit:_onSubmit' name='${name}' enctype='multipart/form-data'></form>",

		_onSubmit: function(/*event*/e) {
			// summary: callback when user hits submit button
			dojo.stopEvent(e);
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.getValues());
		},

		submit: function() {
			// summary: programatically submit form
			this.containerNode.submit();
		},

		setValues: function(/*object*/obj) {
			// summary: fill in form values from a JSON structure

			// generate map from name --> [list of widgets with that name]
			var map = {};
			dojo.forEach(this.getDescendants(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			// call setValue() or setChecked() for each widget, according to obj
			for(var name in map){
				var widgets = map[name],						// array of widgets w/this name
					values = dojo.getObject(name, false, obj);	// list of values for those widgets
				if(!dojo.isArray(values)){
					values = [ values ];
				}
				if(widgets[0].setChecked){
					// for checkbox/radio, values is a list of which widgets should be checked
					dojo.forEach(widgets, function(w, i){
						w.setChecked(dojo.indexOf(values, w.value) != -1);
					});
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					dojo.forEach(widgets, function(w, i){
						w.setValue(values[i]);
					});					
				}
			}

			/***
			 * 	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets

			dojo.forEach(this.containerNode.elements, function(element){
				if (element.name == ''){return};	// like "continue"	
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j) {
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if (nameA.length > 1) {
						if(typeof(myObj[nameA[0]]) == "undefined") {
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined") {
							myObj[nameA[0]][nameIndex]={};
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined") {
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if (typeof(myObj) == "undefined") {
					return;		// like "continue"
				}
				if (typeof(myObj[name]) == "undefined" && this.ignoreNullValues) {
					return;		// like "continue"
				}

				// TODO: widget values (just call setValue() on the widget)

				switch(element.type) {
					case "checkbox":
						element.checked = (name in myObj) &&
							dojo.some(myObj[name], function(val){ return val==element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name]==element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						dojo.forEach(element.options, function(option){
							option.selected = dojo.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						dojo.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
	  		});
	  		*/
		},

		getValues: function() {
			// summary: generate JSON structure from form values

			// get widget values
			var obj = {};
			dojo.forEach(this.getDescendants(), function(widget){
				var value = widget.getValue ? widget.getValue() : widget.value;
				var name = widget.name;
				if(!name){ return; }

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(widget.setChecked){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(widget.checked){
							dojo.setObject(name, value, obj);
						}
					}else{
						// checkbox/toggle button
						var ary=dojo.getObject(name, false, obj);
						if(!ary){
							ary=[];
							dojo.setObject(name, ary, obj);
						}
						if(widget.checked){
							ary.push(value);
						}
					}
				}else{
					// plain input
					dojo.setObject(name, value, obj);
				}
			});

			/***
			 * code for plain input boxes (see also dojo.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			dojo.forEach(this.containerNode.elements, function(elm){
				if (!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j) {
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if (nameA.length > 1) {
						if(typeof(myObj[nameA[0]]) == "undefined") {
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined") {
							myObj[nameA[0]][nameIndex]={};
						}
					} else if(typeof(myObj[nameA[0]]) == "undefined") {
						myObj[nameA[0]]={}
					} // if

					if (nameA.length == 1) {
						myObj=myObj[nameA[0]];
					} else {
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if ((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type=="radio" && elm.checked)) {
					if(name == name.split("[")[0]) {
						myObj[name]=elm.value;
					} else {
						// can not set value when there is no name
					}
				} else if (elm.type == "checkbox" && elm.checked) {
					if(typeof(myObj[name]) == 'undefined') {
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				} else if (elm.type == "select-multiple") {
					if(typeof(myObj[name]) == 'undefined') {
						myObj[name]=[ ];
					}
					for (var jdx=0,len3=elm.options.length; jdx<len3; ++jdx) {
						if (elm.options[jdx].selected) {
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

	 	isValid: function() {
	 		// TODO: ComboBox might need time to process a recently input value.  This should be async?
	 		// make sure that every widget that has a validator function returns true
	 		return dojo.every(this.getDescendants(), function(widget){
	 			return !widget.isValid || widget.isValid();
	 		});
		}
	});

dojo.declare(
	"dijit.form.Form",
	[dijit._Widget, dijit._Templated, dijit.form._FormMixin],
	null
);

}

if(!dojo._hasResource["dijit.Dialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Dialog"] = true;
dojo.provide("dijit.Dialog");









dojo.declare(
	"dijit.DialogUnderlay",
	[dijit._Widget, dijit._Templated],
	{
		// summary: the thing that grays out the screen behind the dialog

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class=dijitDialogUnderlayWrapper id='${id}_underlay'><div class=dijitDialogUnderlay dojoAttachPoint='node'></div></div>",

		postCreate: function(){
			dojo.body().appendChild(this.domNode);
			this.bgIframe = new dijit.BackgroundIframe(this.domNode);
		},

		layout: function(){
			// summary
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.

			var viewport = dijit.getViewport();
			var is = this.node.style,
				os = this.domNode.style;

			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";

			// process twice since the scroll bar may have been removed
			// by the previous resizing
			var viewport2 = dijit.getViewport();
			if(viewport.w != viewport2.w){ is.width = viewport2.w + "px"; }
			if(viewport.h != viewport2.h){ is.height = viewport2.h + "px"; }
		},

		show: function(){
			this.domNode.style.display = "block";
			this.layout();
			if(this.bgIframe.iframe){
				this.bgIframe.iframe.style.display = "block";
			}
			this._resizeHandler = this.connect(window, "onresize", "layout");
		},

		hide: function(){
			this.domNode.style.display = "none";
			if(this.bgIframe.iframe){
				this.bgIframe.iframe.style.display = "none";
			}
			this.disconnect(this._resizeHandler);
		},

		uninitialize: function(){
			if(this.bgIframe){
				this.bgIframe.destroy();
			}
		}
	}
);

dojo.declare(
	"dijit.Dialog",
	[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin],
	{
		// summary:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.)

		templateString: null,
		templateString:"<div class=\"dijitDialog\">\r\n\t<div dojoAttachPoint=\"titleBar\" class=\"dijitDialogTitleBar\" tabindex=\"0\" waiRole=\"dialog\">\r\n\t<span dojoAttachPoint=\"titleNode\" class=\"dijitDialogTitle\">${title}</span>\r\n\t<span dojoAttachPoint=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" dojoAttachEvent=\"onclick: hide\">\r\n\t\t<span dojoAttachPoint=\"closeText\" class=\"closeText\">x</span>\r\n\t</span>\r\n\t</div>\r\n\t\t<div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\r\n\t<span dojoAttachPoint=\"tabEnd\" dojoAttachEvent=\"onfocus:_cycleFocus\" tabindex=\"0\"></span>\r\n</div>\r\n",

		// open: Boolean
		//		is True or False depending on state of dialog
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: 400,

		_lastFocusItem:null,

		attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
			{title: "titleBar"}),

		postCreate: function(){
			dojo.body().appendChild(this.domNode);
			this.inherited("postCreate",arguments);
			this.domNode.style.display="none";
			this.connect(this, "onExecute", "hide");
			this.connect(this, "onCancel", "hide");
		},

		onLoad: function(){
			// summary: 
			//		when href is specified we need to reposition the dialog after the data is loaded
			this._position();
			this.inherited("onLoad",arguments);
		},

		_setup: function(){
			// summary:
			//		stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons)

			this._modalconnects = [];

			if(this.titleBar){
				this._moveable = new dojo.dnd.Moveable(this.domNode, { handle: this.titleBar });
			}

			this._underlay = new dijit.DialogUnderlay();

			var node = this.domNode;
			this._fadeIn = dojo.fx.combine(
				[dojo.fadeIn({
					node: node,
					duration: this.duration
				 }),
				 dojo.fadeIn({
					node: this._underlay.domNode,
					duration: this.duration,
					onBegin: dojo.hitch(this._underlay, "show")
				 })
				]
			);

			this._fadeOut = dojo.fx.combine(
				[dojo.fadeOut({
					node: node,
					duration: this.duration,
					onEnd: function(){
						node.style.display="none";
					}
				 }),
				 dojo.fadeOut({
					node: this._underlay.domNode,
					duration: this.duration,
					onEnd: dojo.hitch(this._underlay, "hide")
				 })
				]
			);
		},

		uninitialize: function(){
			if(this._underlay){
				this._underlay.destroy();
			}
		},

		_position: function(){
			// summary: position modal dialog in center of screen
			
			if(dojo.hasClass(dojo.body(),"dojoMove")){ return; }
			var viewport = dijit.getViewport();
			var mb = dojo.marginBox(this.domNode);

			var style = this.domNode.style;
			style.left = Math.floor((viewport.l + (viewport.w - mb.w)/2)) + "px";
			style.top = Math.floor((viewport.t + (viewport.h - mb.h)/2)) + "px";
		},

		_findLastFocus: function(/*Event*/ evt){
			// summary:  called from onblur of dialog container to determine the last focusable item
			this._lastFocused = evt.target;
		},

		_cycleFocus: function(/*Event*/ evt){
			// summary: when tabEnd receives focus, advance focus around to titleBar

			// on first focus to tabEnd, store the last focused item in dialog
			if(!this._lastFocusItem){
				this._lastFocusItem = this._lastFocused;
			}
			this.titleBar.focus();
		},

		_onKey: function(/*Event*/ evt){
			if(evt.keyCode){
				var node = evt.target;
				// see if we are shift-tabbing from titleBar
				if(node == this.titleBar && evt.shiftKey && evt.keyCode == dojo.keys.TAB){
					if(this._lastFocusItem){
						this._lastFocusItem.focus(); // send focus to last item in dialog if known
					}
					dojo.stopEvent(evt);
				}else{
					// see if the key is for the dialog
					while(node){
						if(node == this.domNode){
							if(evt.keyCode == dojo.keys.ESCAPE){
								this.hide(); 
							}else{
								return; // just let it go
							}
						}
						node = node.parentNode;
					}
					// this key is for the disabled document window
					if(evt.keyCode != dojo.keys.TAB){ // allow tabbing into the dialog for a11y
						dojo.stopEvent(evt);
					// opera won't tab to a div
					}else if (!dojo.isOpera){
						try{
							this.titleBar.focus();
						}catch(e){/*squelch*/}
					}
				}
			}
		},

		show: function(){
			// summary: display the dialog

			// first time we show the dialog, there's some initialization stuff to do			
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized=true;
			}

			if(this._fadeOut.status() == "playing"){
				this._fadeOut.stop();
			}

			this._modalconnects.push(dojo.connect(window, "onscroll", this, "layout"));
			this._modalconnects.push(dojo.connect(document.documentElement, "onkeypress", this, "_onKey"));

			// IE doesn't bubble onblur events - use ondeactivate instead
			var ev = typeof(document.ondeactivate) == "object" ? "ondeactivate" : "onblur";
			this._modalconnects.push(dojo.connect(this.containerNode, ev, this, "_findLastFocus"));

			dojo.style(this.domNode, "opacity", 0);
			this.domNode.style.display="block";
			this.open = true;
			this._loadCheck(); // lazy load trigger

			this._position();

			this._fadeIn.play();

			this._savedFocus = dijit.getFocus(this);

			// set timeout to allow the browser to render dialog
			setTimeout(dojo.hitch(this, function(){
				dijit.focus(this.titleBar);
			}), 50);
		},

		hide: function(){
			// summary
			//		Hide the dialog

			// if we haven't been initialized yet then we aren't showing and we can just return		
			if(!this._alreadyInitialized){
				return;
			}

			if(this._fadeIn.status() == "playing"){
				this._fadeIn.stop();
			}
			this._fadeOut.play();

			if (this._scrollConnected){
				this._scrollConnected = false;
			}
			dojo.forEach(this._modalconnects, dojo.disconnect);
			this._modalconnects = [];

			this.connect(this._fadeOut,"onEnd",dojo.hitch(this,function(){
				dijit.focus(this._savedFocus);
			}));
			this.open = false;
		},

		layout: function() {
			// summary: position the Dialog and the underlay
			if(this.domNode.style.display == "block"){
				this._underlay.layout();
				this._position();
			}
		}
	}
);

dojo.declare(
	"dijit.TooltipDialog",
	[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin],
	{
		// summary:
		//		Pops up a dialog that appears like a Tooltip
		// title: String
		// 		Description of tooltip dialog (required for a11Y)
		title: "",

		_lastFocusItem: null,

		templateString: null,
		templateString:"<div class=\"dijitTooltipDialog\" >\r\n\t<div class=\"dijitTooltipContainer\">\r\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" dojoAttachPoint=\"containerNode\" tabindex=\"0\" waiRole=\"dialog\"></div>\r\n\t</div>\r\n\t<span dojoAttachPoint=\"tabEnd\" tabindex=\"0\" dojoAttachEvent=\"focus:_cycleFocus\"></span>\r\n\t<div class=\"dijitTooltipConnector\" ></div>\r\n</div>\r\n",

		postCreate: function(){
			this.inherited("postCreate",arguments);
			this.connect(this.containerNode, "onkeypress", "_onKey");

			// IE doesn't bubble onblur events - use ondeactivate instead
			var ev = typeof(document.ondeactivate) == "object" ? "ondeactivate" : "onblur";
			this.connect(this.containerNode, ev, "_findLastFocus");
			this.containerNode.title=this.title;
		},

		orient: function(/*Object*/ corner){
			// summary: configure widget to be displayed in given position relative to the button
			this.domNode.className="dijitTooltipDialog " +" dijitTooltipAB"+(corner.charAt(1)=='L'?"Left":"Right")+" dijitTooltip"+(corner.charAt(0)=='T' ? "Below" : "Above");
		},

		onOpen: function(/*Object*/ pos){
			// summary: called when dialog is displayed
			this.orient(pos.corner);
			this._loadCheck(); // lazy load trigger
			this.containerNode.focus();
		},

		_onKey: function(/*Event*/ evt){
			// summary: keep keyboard focus in dialog; close dialog on escape key
			if(evt.keyCode == dojo.keys.ESCAPE){
				this.onCancel();
			}else if(evt.target == this.containerNode && evt.shiftKey && evt.keyCode == dojo.keys.TAB){
				if (this._lastFocusItem){
					this._lastFocusItem.focus();
				}
				dojo.stopEvent(evt);
			}else if(evt.keyCode == dojo.keys.TAB){
				// we want the browser's default tab handling to move focus
				// but we don't want the tab to propagate upwards
				evt.stopPropagation();
			}
		},

		_findLastFocus: function(/*Event*/ evt){
			// summary: called from onblur of dialog container to determine the last focusable item
			this._lastFocused = evt.target;
		},

		_cycleFocus: function(/*Event*/ evt){
			// summary: when tabEnd receives focus, advance focus around to containerNode

			// on first focus to tabEnd, store the last focused item in dialog
			if(!this._lastFocusItem){
				this._lastFocusItem = this._lastFocused;
			}
			this.containerNode.focus();
		}
	}	
);


}

if(!dojo._hasResource["dijit.layout.LayoutContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.LayoutContainer"] = true;
dojo.provide("dijit.layout.LayoutContainer");



dojo.declare(
	"dijit.layout.LayoutContainer",
	dijit.layout._LayoutWidget,
{
	// summary
	//	Provides Delphi-style panel layout semantics.
	//
	// details
	//	A LayoutContainer is a box with a specified size (like style="width: 500px; height: 500px;"),
	//	that contains children widgets marked with "layoutAlign" of "left", "right", "bottom", "top", and "client".
	//	It takes it's children marked as left/top/bottom/right, and lays them out along the edges of the box,
	//	and then it takes the child marked "client" and puts it into the remaining space in the middle.
	//
	//  Left/right positioning is similar to CSS's "float: left" and "float: right",
	//	and top/bottom positioning would be similar to "float: top" and "float: bottom", if there were such
	//	CSS.
	//
	//	Note that there can only be one client element, but there can be multiple left, right, top,
	//	or bottom elements.
	//
	// usage
	//	<style>
	//		html, body{ height: 100%; width: 100%; }
	//	</style>
	//	<div dojoType="dijit.layout.LayoutContainer" style="width: 100%; height: 100%">
	//		<div dojoType="dijit.layout.ContentPane" layoutAlign="top">header text</div>
	//		<div dojoType="dijit.layout.ContentPane" layoutAlign="left" style="width: 200px;">table of contents</div>
	//		<div dojoType="dijit.layout.ContentPane" layoutAlign="client">client area</div>
	//	</div>
	//
	//	Lays out each child in the natural order the children occur in.
	//	Basically each child is laid out into the "remaining space", where "remaining space" is initially
	//	the content area of this widget, but is reduced to a smaller rectangle each time a child is added.
	//	

	layout: function(){
		dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
	},

	addChild: function(/*Widget*/ child, /*Integer?*/ insertIndex){
		dijit._Container.prototype.addChild.apply(this, arguments);
		if(this._started){
			dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
		}
	},

	removeChild: function(/*Widget*/ widget){
		dijit._Container.prototype.removeChild.apply(this, arguments);
		if(this._started){
			dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
		}
	}
});

// This argument can be specified for the children of a LayoutContainer.
// Since any widget can be specified as a LayoutContainer child, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// layoutAlign: String
	//		"none", "left", "right", "bottom", "top", and "client".
	//		See the LayoutContainer description for details on this parameter.
	layoutAlign: 'none'
});

}

if(!dojo._hasResource["dojo.cookie"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cookie"] = true;
dojo.provide("dojo.cookie");

/*=====
dojo.__cookieProps = function(kwArgs){
	//	expires: Date|Number?
	//		If a number, seen as the number of days from today. If a date, the
	//		date past which the cookie is invalid. If expires is in the past,
	//		the cookie will be deleted If expires is left out or is 0, the
	//		cookie will expire when the browser closes.
	//	path: String?
	//		The path to use for the cookie.
	//	domain: String?
	//		The domain to use for the cookie.
	//	secure: Boolean?
	//		Whether to only send the cookie on secure connections
}
=====*/


dojo.cookie = function(/*String*/name, /*String?*/value, /*dojo.__cookieProps?*/props){
	//	summary: 
	//		Get or set a cookie.
	//	description:
	// 		If you pass in one argument, the the value of the cookie is returned
	//
	// 		If you pass in two arguments, the cookie value is set to the second
	// 		argument.
	//
	// 		If you pass in three arguments, the cookie value is set to the
	// 		second argument, and the options on the third argument are used for
	// 		extended properties on the cookie
	//	name:
	//		The name of the cookie
	//	value:
	//		Optional. The value for the cookie.
	//	props: 
	//		Optional additional properties for the cookie
	//	example:
	//		set a cookie with the JSON-serialized contents of an object which
	//		will expire 5 days from now:
	//	|	dojo.cookie("configObj", dojo.toJson(config), { expires: 5 });
	//	
	//	example:
	//		de-serialize a cookie back into a JavaScript object:
	//	|	var config = dojo.fromJson(dojo.cookie("configObj"));
	//	
	//	example:
	//		delete a cookie:
	//	|	dojo.cookie("configObj", null);
	var c = document.cookie;
	if(arguments.length == 1){
		var idx = c.lastIndexOf(name+'=');
		if(idx == -1){ return null; }
		var start = idx+name.length+1;
		var end = c.indexOf(';', idx+name.length+1);
		if(end == -1){ end = c.length; }
		return decodeURIComponent(c.substring(start, end)); 
	}else{
		props = props || {};
		value = encodeURIComponent(value);
		if(typeof(props.expires) == "number"){ 
			var d = new Date();
			d.setTime(d.getTime()+(props.expires*24*60*60*1000));
			props.expires = d;
		}
		document.cookie = name + "=" + value 
			+ (props.expires ? "; expires=" + props.expires.toUTCString() : "")
			+ (props.path ? "; path=" + props.path : "")
			+ (props.domain ? "; domain=" + props.domain : "")
			+ (props.secure ? "; secure" : "");
		return null;
	}
};

}

if(!dojo._hasResource["dijit.layout.SplitContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.SplitContainer"] = true;
dojo.provide("dijit.layout.SplitContainer");

//
// FIXME: make it prettier
// FIXME: active dragging upwards doesn't always shift other bars (direction calculation is wrong in this case)
//




dojo.declare("dijit.layout.SplitContainer",
	dijit.layout._LayoutWidget,
	{
	// summary: 
	//	A Container widget with sizing handles in-between each child
	// description:
	//		Contains multiple children widgets, all of which are displayed side by side
	//		(either horizontally or vertically); there's a bar between each of the children,
	//		and you can adjust the relative size of each child by dragging the bars.
	//
	//		You must specify a size (width and height) for the SplitContainer.
	//
	// activeSizing: Boolean
	//		If true, the children's size changes as you drag the bar;
	//		otherwise, the sizes don't change until you drop the bar (by mouse-up)
	activeSizing: false,

	// sizerWidth: Integer
	//		Size in pixels of the bar between each child
	sizerWidth: 7, // FIXME: this should be a CSS attribute (at 7 because css wants it to be 7 until we fix to css)

	// orientation: String
	//		either 'horizontal' or vertical; indicates whether the children are
	//		arranged side-by-side or up/down.
	orientation: 'horizontal',

	// persist: Boolean
	//		Save splitter positions in a cookie
	persist: true,

	postMixInProperties: function(){
		this.inherited("postMixInProperties",arguments);
		this.isHorizontal = (this.orientation == 'horizontal');
	},

	postCreate: function(){
		this.inherited("postCreate",arguments);
		this.sizers = [];
		dojo.addClass(this.domNode, "dijitSplitContainer");
		// overflow has to be explicitly hidden for splitContainers using gekko (trac #1435)
		// to keep other combined css classes from inadvertantly making the overflow visible
		if(dojo.isMozilla){
			this.domNode.style.overflow = '-moz-scrollbars-none'; // hidden doesn't work
		}

		// create the fake dragger
		if(typeof this.sizerWidth == "object"){
			try{ //FIXME: do this without a try/catch
				this.sizerWidth = parseInt(this.sizerWidth.toString());
			}catch(e){ this.sizerWidth = 7; }
		}
		var sizer = this.virtualSizer = document.createElement('div');
		sizer.style.position = 'relative';

		// #1681: work around the dreaded 'quirky percentages in IE' layout bug
		// If the splitcontainer's dimensions are specified in percentages, it
		// will be resized when the virtualsizer is displayed in _showSizingLine
		// (typically expanding its bounds unnecessarily). This happens because
		// we use position: relative for .dijitSplitContainer.
		// The workaround: instead of changing the display style attribute,
		// switch to changing the zIndex (bring to front/move to back)

		sizer.style.zIndex = 10;
		sizer.className = this.isHorizontal ? 'dijitSplitContainerVirtualSizerH' : 'dijitSplitContainerVirtualSizerV';
		this.domNode.appendChild(sizer);
		dojo.setSelectable(sizer, false);
	},

	startup: function(){
		if(this._started){ return; }
		dojo.forEach(this.getChildren(), function(child, i, children){
			// attach the children and create the draggers
			this._injectChild(child);

			if(i < children.length-1){
				this._addSizer();
			}
		}, this);

		if(this.persist){
			this._restoreState();
		}
		this.inherited("startup",arguments); 
		this._started = true;
	},

	_injectChild: function(child){
		child.domNode.style.position = "absolute";
		dojo.addClass(child.domNode, "dijitSplitPane");
	},

	_addSizer: function(){
		var i = this.sizers.length;

		// TODO: use a template for this!!!
		var sizer = this.sizers[i] = document.createElement('div');
		sizer.className = this.isHorizontal ? 'dijitSplitContainerSizerH' : 'dijitSplitContainerSizerV';

		// add the thumb div
		var thumb = document.createElement('div');
		thumb.className = 'thumb';
		sizer.appendChild(thumb);

		// FIXME: are you serious? why aren't we using mover start/stop combo?
		var self = this;
		var handler = (function(){ var sizer_i = i; return function(e){ self.beginSizing(e, sizer_i); } })();
		dojo.connect(sizer, "onmousedown", handler);

		this.domNode.appendChild(sizer);
		dojo.setSelectable(sizer, false);
	},

	removeChild: function(widget){
		// summary: Remove sizer, but only if widget is really our child and
		// we have at least one sizer to throw away
		if(this.sizers.length && dojo.indexOf(this.getChildren(), widget) != -1){
			var i = this.sizers.length - 1;
			dojo._destroyElement(this.sizers[i]);
			this.sizers.length--;
		}

		// Remove widget and repaint
		this.inherited("removeChild",arguments); 
		if(this._started){
			this.layout();
		}
	},

	addChild: function(/*Widget*/ child, /*Integer?*/ insertIndex){
		// summary: Add a child widget to the container
		// child: a widget to add
		// insertIndex: postion in the "stack" to add the child widget
		
		this.inherited("addChild",arguments); 

		if(this._started){
			// Do the stuff that startup() does for each widget
			this._injectChild(child);
			var children = this.getChildren();
			if(children.length > 1){
				this._addSizer();
			}

			// and then reposition (ie, shrink) every pane to make room for the new guy
			this.layout();
		}
	},

	layout: function(){
		// summary:
		//		Do layout of panels

		// base class defines this._contentBox on initial creation and also
		// on resize
		this.paneWidth = this._contentBox.w;
		this.paneHeight = this._contentBox.h;

		var children = this.getChildren();
		if(!children.length){ return; }

		//
		// calculate space
		//

		var space = this.isHorizontal ? this.paneWidth : this.paneHeight;
		if(children.length > 1){
			space -= this.sizerWidth * (children.length - 1);
		}

		//
		// calculate total of SizeShare values
		//
		var outOf = 0;
		dojo.forEach(children, function(child){
			outOf += child.sizeShare;
		});

		//
		// work out actual pixels per sizeshare unit
		//
		var pixPerUnit = space / outOf;

		//
		// set the SizeActual member of each pane
		//
		var totalSize = 0;
		dojo.forEach(children.slice(0, children.length - 1), function(child){
			var size = Math.round(pixPerUnit * child.sizeShare);
			child.sizeActual = size;
			totalSize += size;
		});

		children[children.length-1].sizeActual = space - totalSize;

		//
		// make sure the sizes are ok
		//
		this._checkSizes();

		//
		// now loop, positioning each pane and letting children resize themselves
		//

		var pos = 0;
		var size = children[0].sizeActual;
		this._movePanel(children[0], pos, size);
		children[0].position = pos;
		pos += size;

		// if we don't have any sizers, our layout method hasn't been called yet
		// so bail until we are called..TODO: REVISIT: need to change the startup
		// algorithm to guaranteed the ordering of calls to layout method
		if(!this.sizers){
			return;
		}

		dojo.some(children.slice(1), function(child, i){
			// error-checking
			if(!this.sizers[i]){
				return true;
			}
			// first we position the sizing handle before this pane
			this._moveSlider(this.sizers[i], pos, this.sizerWidth);
			this.sizers[i].position = pos;
			pos += this.sizerWidth;

			size = child.sizeActual;
			this._movePanel(child, pos, size);
			child.position = pos;
			pos += size;
		}, this);
	},

	_movePanel: function(panel, pos, size){
		if(this.isHorizontal){
			panel.domNode.style.left = pos + 'px';	// TODO: resize() takes l and t parameters too, don't need to set manually
			panel.domNode.style.top = 0;
			var box = {w: size, h: this.paneHeight};
			if(panel.resize){
				panel.resize(box);
			}else{
				dojo.marginBox(panel.domNode, box);
			}
		}else{
			panel.domNode.style.left = 0;	// TODO: resize() takes l and t parameters too, don't need to set manually
			panel.domNode.style.top = pos + 'px';
			var box = {w: this.paneWidth, h: size};
			if(panel.resize){
				panel.resize(box);
			}else{
				dojo.marginBox(panel.domNode, box);
			}
		}
	},

	_moveSlider: function(slider, pos, size){
		if(this.isHorizontal){
			slider.style.left = pos + 'px';
			slider.style.top = 0;
			dojo.marginBox(slider, { w: size, h: this.paneHeight });
		}else{
			slider.style.left = 0;
			slider.style.top = pos + 'px';
			dojo.marginBox(slider, { w: this.paneWidth, h: size });
		}
	},

	_growPane: function(growth, pane){
		if(growth > 0){
			if(pane.sizeActual > pane.sizeMin){
				if((pane.sizeActual - pane.sizeMin) > growth){

					// stick all the growth in this pane
					pane.sizeActual = pane.sizeActual - growth;
					growth = 0;
				}else{
					// put as much growth in here as we can
					growth -= pane.sizeActual - pane.sizeMin;
					pane.sizeActual = pane.sizeMin;
				}
			}
		}
		return growth;
	},

	_checkSizes: function(){

		var totalMinSize = 0;
		var totalSize = 0;
		var children = this.getChildren();

		dojo.forEach(children, function(child){
			totalSize += child.sizeActual;
			totalMinSize += child.sizeMin;
		});

		// only make adjustments if we have enough space for all the minimums

		if(totalMinSize <= totalSize){

			var growth = 0;

			dojo.forEach(children, function(child){
				if(child.sizeActual < child.sizeMin){
					growth += child.sizeMin - child.sizeActual;
					child.sizeActual = child.sizeMin;
				}
			});

			if(growth > 0){
				var list = this.isDraggingLeft ? children.reverse() : children;
				dojo.forEach(list, function(child){
					growth = this._growPane(growth, child);
				}, this);
			}
		}else{
			dojo.forEach(children, function(child){
				child.sizeActual = Math.round(totalSize * (child.sizeMin / totalMinSize));
			});
		}
	},

	beginSizing: function(e, i){
		var children = this.getChildren();
		this.paneBefore = children[i];
		this.paneAfter = children[i+1];

		this.isSizing = true;
		this.sizingSplitter = this.sizers[i];

		if(!this.cover){
			this.cover = dojo.doc.createElement('div');
			this.domNode.appendChild(this.cover);
			var s = this.cover.style;
			s.position = 'absolute';
			s.zIndex = 1;
			s.top = 0;
			s.left = 0;
			s.width = "100%";
			s.height = "100%";
		}else{
			this.cover.style.zIndex = 1;
		}
		this.sizingSplitter.style.zIndex = 2;

		// TODO: REVISIT - we want MARGIN_BOX and core hasn't exposed that yet (but can't we use it anyway if we pay attention? we do elsewhere.)
		this.originPos = dojo.coords(children[0].domNode, true);
		if(this.isHorizontal){
			var client = (e.layerX ? e.layerX : e.offsetX);
			var screen = e.pageX;
			this.originPos = this.originPos.x;
		}else{
			var client = (e.layerY ? e.layerY : e.offsetY);
			var screen = e.pageY;
			this.originPos = this.originPos.y;
		}
		this.startPoint = this.lastPoint = screen;
		this.screenToClientOffset = screen - client;
		this.dragOffset = this.lastPoint - this.paneBefore.sizeActual - this.originPos - this.paneBefore.position;

		if(!this.activeSizing){
			this._showSizingLine();
		}

		//					
		// attach mouse events
		//
		this._connects = [];
		this._connects.push(dojo.connect(document.documentElement, "onmousemove", this, "changeSizing"));
		this._connects.push(dojo.connect(document.documentElement, "onmouseup", this, "endSizing"));

		dojo.stopEvent(e);
	},

	changeSizing: function(e){
		if(!this.isSizing){ return; }
		this.lastPoint = this.isHorizontal ? e.pageX : e.pageY;
		this.movePoint();
		if(this.activeSizing){
			this._updateSize();
		}else{
			this._moveSizingLine();
		}
		dojo.stopEvent(e);
	},

	endSizing: function(e){
		if(!this.isSizing){ return; }
		if(this.cover){
			this.cover.style.zIndex = -1;
		}
		if(!this.activeSizing){
			this._hideSizingLine();
		}

		this._updateSize();

		this.isSizing = false;

		if(this.persist){
			this._saveState(this);
		}

		dojo.forEach(this._connects,dojo.disconnect); 
	},

	movePoint: function(){

		// make sure lastPoint is a legal point to drag to
		var p = this.lastPoint - this.screenToClientOffset;

		var a = p - this.dragOffset;
		a = this.legaliseSplitPoint(a);
		p = a + this.dragOffset;

		this.lastPoint = p + this.screenToClientOffset;
	},

	legaliseSplitPoint: function(a){

		a += this.sizingSplitter.position;

		this.isDraggingLeft = !!(a > 0);

		if(!this.activeSizing){
			var min = this.paneBefore.position + this.paneBefore.sizeMin;
			if(a < min){
				a = min;
			}

			var max = this.paneAfter.position + (this.paneAfter.sizeActual - (this.sizerWidth + this.paneAfter.sizeMin));
			if(a > max){
				a = max;
			}
		}

		a -= this.sizingSplitter.position;

		this._checkSizes();

		return a;
	},

	_updateSize: function(){
	//FIXME: sometimes this.lastPoint is NaN
		var pos = this.lastPoint - this.dragOffset - this.originPos;

		var start_region = this.paneBefore.position;
		var end_region   = this.paneAfter.position + this.paneAfter.sizeActual;

		this.paneBefore.sizeActual = pos - start_region;
		this.paneAfter.position	= pos + this.sizerWidth;
		this.paneAfter.sizeActual  = end_region - this.paneAfter.position;

		dojo.forEach(this.getChildren(), function(child){
			child.sizeShare = child.sizeActual;
		});

		if(this._started){
			this.layout();
		}
	},

	_showSizingLine: function(){

		this._moveSizingLine();

		dojo.marginBox(this.virtualSizer,
			this.isHorizontal ? { w: this.sizerWidth, h: this.paneHeight } : { w: this.paneWidth, h: this.sizerWidth });

		this.virtualSizer.style.display = 'block';
	},

	_hideSizingLine: function(){
		this.virtualSizer.style.display = 'none';
	},

	_moveSizingLine: function(){
		var pos = (this.lastPoint - this.startPoint) + this.sizingSplitter.position;
		dojo.style(this.virtualSizer,(this.isHorizontal ? "left" : "top"),pos+"px");
		// this.virtualSizer.style[ this.isHorizontal ? "left" : "top" ] = pos + 'px'; // FIXME: remove this line if the previous is better
	},

	_getCookieName: function(i){
		return this.id + "_" + i;
	},

	_restoreState: function(){
		dojo.forEach(this.getChildren(), function(child, i){
			var cookieName = this._getCookieName(i);
			var cookieValue = dojo.cookie(cookieName);
			if(cookieValue){
				var pos = parseInt(cookieValue);
				if(typeof pos == "number"){
					child.sizeShare = pos;
				}
			}
		}, this);
	},

	_saveState: function(){
		dojo.forEach(this.getChildren(), function(child, i){
			dojo.cookie(this._getCookieName(i), child.sizeShare);
		}, this);
	}
});

// These arguments can be specified for the children of a SplitContainer.
// Since any widget can be specified as a SplitContainer child, mix them
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// sizeMin: Integer
	//	Minimum size (width or height) of a child of a SplitContainer.
	//	The value is relative to other children's sizeShare properties.
	sizeMin: 10,

	// sizeShare: Integer
	//	Size (width or height) of a child of a SplitContainer.
	//	The value is relative to other children's sizeShare properties.
	//	For example, if there are two children and each has sizeShare=10, then
	//	each takes up 50% of the available space.
	sizeShare: 10
});

}

if(!dojo._hasResource["dijit.form.Button"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Button"] = true;
dojo.provide("dijit.form.Button");




dojo.declare("dijit.form.Button", dijit.form._FormWidget, {
/*
 * usage
 *	<button dojoType="button" onClick="...">Hello world</button>
 *
 *  var button1 = new dijit.form.Button({label: "hello world", onClick: foo});
 *	dojo.body().appendChild(button1.domNode);
 */
	// summary
	//	Basically the same thing as a normal HTML button, but with special styling.

	// label: String
	//	text to display in button
	label: "",

	// showLabel: Boolean
	//	whether or not to display the text label in button
	showLabel: true,

	// iconClass: String
	//	class to apply to div in button to make it display an icon
	iconClass: "",

	type: "button",
	baseClass: "dijitButton",
	templateString:"<div class=\"dijit dijitLeft dijitInline dijitButton\"\r\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\r\n\t><div class='dijitRight'\r\n\t\t><button class=\"dijitStretch dijitButtonNode dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\r\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\r\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \r\n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \r\n\t\t\t></span\r\n\t\t\t><span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span\r\n\t\t></button\r\n\t></div\r\n></div>\r\n",

	// TODO: set button's title to this.containerNode.innerText

	_onClick: function(/*Event*/ e){
		// summary: internal function to handle click actions
		if(this.disabled){ return false; }
		this._clicked(); // widget click actions
		return this.onClick(e); // user click actions
	},

	_onButtonClick: function(/*Event*/ e){
		// summary: callback when the user mouse clicks the button portion
		dojo.stopEvent(e);
		var okToSubmit = this._onClick(e) !== false; // returning nothing is same as true

		// for some reason type=submit buttons don't automatically submit the form; do it manually
		if(this.type=="submit" && okToSubmit){
			for(var node=this.domNode; node; node=node.parentNode){
				var widget=dijit.byNode(node);
				if(widget && widget._onSubmit){
					widget._onSubmit(e);
					break;
				}
				if(node.tagName.toLowerCase() == "form"){
					if(!node.onsubmit || node.onsubmit()){ node.submit(); }
					break;
				}
			}
		}
	},

	postCreate: function(){
		// summary:
		//	get label and set as title on button icon if necessary
		if (this.showLabel == false){
			var labelText = "";
			this.label = this.containerNode.innerHTML;
			labelText = dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
			// set title attrib on iconNode
			this.titleNode.title=labelText;
			dojo.addClass(this.containerNode,"dijitDisplayNone");
		}
		this.inherited(arguments);
	},

	onClick: function(/*Event*/ e){
		// summary: user callback for when button is clicked
		//      if type="submit", return value != false to perform submit
		return true;
	},

	_clicked: function(/*Event*/ e){
		// summary: internal replaceable function for when the button is clicked
	},

	setLabel: function(/*String*/ content){
		// summary: reset the label (text) of the button; takes an HTML string
		this.containerNode.innerHTML = this.label = content;
		if(dojo.isMozilla){ // Firefox has re-render issues with tables
			var oldDisplay = dojo.getComputedStyle(this.domNode).display;
			this.domNode.style.display="none";
			var _this = this;
			setTimeout(function(){_this.domNode.style.display=oldDisplay;},1);
		}
		if (this.showLabel == false){
				this.titleNode.title=dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
		}
	}		
});

/*
 * usage
 *	<button dojoType="DropDownButton" label="Hello world"><div dojotype=dijit.Menu>...</div></button>
 *
 *  var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
 *	dojo.body().appendChild(button1);
 */
dojo.declare("dijit.form.DropDownButton", [dijit.form.Button, dijit._Container], {
	// summary
	//		push the button and a menu shows up

	baseClass : "dijitDropDownButton",

	templateString:"<div class=\"dijit dijitLeft dijitInline\"\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse,onclick:_onDropDownClick,onkeydown:_onDropDownKeydown,onblur:_onDropDownBlur,onkeypress:_onKey\"\r\n\t><div class='dijitRight'>\r\n\t<button class=\"dijitStretch dijitButtonNode dijitButtonContents\" type=\"${type}\"\r\n\t\tdojoAttachPoint=\"focusNode,titleNode\" waiRole=\"button\" waiState=\"haspopup-true,labelledby-${id}_label\"\r\n\t\t><div class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\"></div\r\n\t\t><span class=\"dijitButtonText\" \tdojoAttachPoint=\"containerNode,popupStateNode\"\r\n\t\tid=\"${id}_label\">${label}</span\r\n\t\t><span class='dijitA11yDownArrow'>&#9660;</span>\r\n\t</button>\r\n</div></div>\r\n",

	_fillContent: function(){
		// my inner HTML contains both the button contents and a drop down widget, like
		// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
		// The first node is assumed to be the button content. The widget is the popup.
		if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
			//FIXME: figure out how to filter out the widget and use all remaining nodes as button
			//	content, not just nodes[0]
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.form.DropDownButton.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
		// make it invisible, and store a reference to pass to the popup code.
		if(!this.dropDown){
			var dropDownNode = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.dropDown = dijit.byNode(dropDownNode);
			delete this.dropDownContainer;
		}
		dojo.body().appendChild(this.dropDown.domNode);
		this.dropDown.domNode.style.display="none";
	},

	_onArrowClick: function(/*Event*/ e){
		// summary: callback when the user mouse clicks on menu popup node
		if(this.disabled){ return; }
		this._toggleDropDown();
	},

	_onDropDownClick: function(/*Event*/ e){
		// on Firefox 2 on the Mac it is possible to fire onclick
		// by pressing enter down on a second element and transferring
		// focus to the DropDownButton;
		// we want to prevent opening our menu in this situation
		// and only do so if we have seen a keydown on this button;
		// e.detail != 0 means that we were fired by mouse
		var isMacFFlessThan3 = dojo.isFF && dojo.isFF < 3
			&& navigator.appVersion.indexOf("Macintosh") != -1;
		if(!isMacFFlessThan3 || e.detail != 0 || this._seenKeydown){
			this._onArrowClick(e);
		}
		this._seenKeydown = false;
	},

	_onDropDownKeydown: function(/*Event*/ e){
		this._seenKeydown = true;
	},

	_onDropDownBlur: function(/*Event*/ e){
		this._seenKeydown = false;
	},

	_onKey: function(/*Event*/ e){
		// summary: callback when the user presses a key on menu popup node
		if(this.disabled){ return; }
		if(e.keyCode == dojo.keys.DOWN_ARROW){
			if(!this.dropDown || this.dropDown.domNode.style.display=="none"){
				dojo.stopEvent(e);
				return this._toggleDropDown();
			}
		}
	},

	_onBlur: function(){
		// summary: called magically when focus has shifted away from this widget and it's dropdown
		this._closeDropDown();
		// don't focus on button.  the user has explicitly focused on something else.
	},

	_toggleDropDown: function(){
		// summary: toggle the drop-down widget; if it is up, close it, if not, open it
		if(this.disabled){ return; }
		dijit.focus(this.popupStateNode);
		var dropDown = this.dropDown;
		if(!dropDown){ return false; }
		if(!dropDown.isShowingNow){
			// If there's an href, then load that first, so we don't get a flicker
			if(dropDown.href && !dropDown.isLoaded){
				var self = this;
				var handler = dojo.connect(dropDown, "onLoad", function(){
					dojo.disconnect(handler);
					self._openDropDown();
				});
				dropDown._loadCheck(true);
				return;
			}else{
				this._openDropDown();
			}
		}else{
			this._closeDropDown();
		}
	},

	_openDropDown: function(){
		var dropDown = this.dropDown;
		var oldWidth=dropDown.domNode.style.width;
		var self = this;

		dijit.popup.open({
			parent: this,
			popup: dropDown,
			around: this.domNode,
			orient: this.isLeftToRight() ? {'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'}
				: {'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'},
			onExecute: function(){
				self._closeDropDown(true);
			},
			onCancel: function(){
				self._closeDropDown(true);
			},
			onClose: function(){
				dropDown.domNode.style.width = oldWidth;
				self.popupStateNode.removeAttribute("popupActive");
				this._opened = false;
			}
		});
		if(this.domNode.offsetWidth > dropDown.domNode.offsetWidth){
			var adjustNode = null;
			if(!this.isLeftToRight()){
				adjustNode = dropDown.domNode.parentNode;
				var oldRight = adjustNode.offsetLeft + adjustNode.offsetWidth;
			}
			// make menu at least as wide as the button
			dojo.marginBox(dropDown.domNode, {w: this.domNode.offsetWidth});
			if(adjustNode){
				adjustNode.style.left = oldRight - this.domNode.offsetWidth + "px";
			}
		}
		this.popupStateNode.setAttribute("popupActive", "true");
		this._opened=true;
		if(dropDown.focus){
			dropDown.focus();
		}
		// TODO: set this.checked and call setStateClass(), to affect button look while drop down is shown
	},
	
	_closeDropDown: function(/*Boolean*/ focus){
		if(this._opened){
			dijit.popup.close(this.dropDown);
			if(focus){ this.focus(); }
			this._opened = false;			
		}
	}
});

/*
 * usage
 *	<button dojoType="ComboButton" onClick="..."><span>Hello world</span><div dojoType=dijit.Menu>...</div></button>
 *
 *  var button1 = new dijit.form.ComboButton({label: "hello world", onClick: foo, dropDown: "myMenu"});
 *	dojo.body().appendChild(button1.domNode);
 */
dojo.declare("dijit.form.ComboButton", dijit.form.DropDownButton, {
	// summary
	//		left side is normal button, right side displays menu
	templateString:"<table class='dijit dijitReset dijitInline dijitLeft'\r\n\tcellspacing='0' cellpadding='0'\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\">\r\n\t<tr>\r\n\t\t<td\tclass=\"dijitStretch dijitButtonContents dijitButtonNode\"\r\n\t\t\ttabIndex=\"${tabIndex}\"\r\n\t\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\"  dojoAttachPoint=\"titleNode\"\r\n\t\t\twaiRole=\"button\" waiState=\"labelledby-${id}_label\">\r\n\t\t\t<div class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\"></div>\r\n\t\t\t<span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span>\r\n\t\t</td>\r\n\t\t<td class='dijitReset dijitRight dijitButtonNode dijitDownArrowButton'\r\n\t\t\tdojoAttachPoint=\"popupStateNode,focusNode\"\r\n\t\t\tdojoAttachEvent=\"ondijitclick:_onArrowClick, onkeypress:_onKey\"\r\n\t\t\tstateModifier=\"DownArrow\"\r\n\t\t\ttitle=\"${optionsTitle}\" name=\"${name}\"\r\n\t\t\twaiRole=\"button\" waiState=\"haspopup-true\"\r\n\t\t><div waiRole=\"presentation\">&#9660;</div>\r\n\t</td></tr>\r\n</table>\r\n",

	attributeMap: dojo.mixin(dojo.clone(dijit.form._FormWidget.prototype.attributeMap),
		{id:"", name:""}),

	// optionsTitle: String
	//  text that describes the options menu (accessibility)
	optionsTitle: "",

	baseClass: "dijitComboButton",

	_focusedNode: null,

	postCreate: function(){
		this.inherited(arguments);
		this._focalNodes = [this.titleNode, this.popupStateNode];
		dojo.forEach(this._focalNodes, dojo.hitch(this, function(node){
			if(dojo.isIE){
				this.connect(node, "onactivate", this._onNodeFocus);
			}else{
				this.connect(node, "onfocus", this._onNodeFocus);
			}
		}));
	},

	focusFocalNode: function(node){
		// summary: Focus the focal node node.
		this._focusedNode = node;
		dijit.focus(node);
	},

	hasNextFocalNode: function(){
		// summary: Returns true if this widget has no node currently
		//		focused or if there is a node following the focused one.
		//		False is returned if the last node has focus.
		return this._focusedNode !== this.getFocalNodes()[1];
	},

	focusNext: function(){
		// summary: Focus the focal node following the current node with focus
		//		or the first one if no node currently has focus.
		this._focusedNode = this.getFocalNodes()[this._focusedNode ? 1 : 0];
		dijit.focus(this._focusedNode);
	},

	hasPrevFocalNode: function(){
		// summary: Returns true if this widget has no node currently
		//		focused or if there is a node before the focused one.
		//		False is returned if the first node has focus.
		return this._focusedNode !== this.getFocalNodes()[0];
	},

	focusPrev: function(){
		// summary: Focus the focal node before the current node with focus
		//		or the last one if no node currently has focus.
		this._focusedNode = this.getFocalNodes()[this._focusedNode ? 0 : 1];
		dijit.focus(this._focusedNode);
	},

	getFocalNodes: function(){
		// summary: Returns an array of focal nodes for this widget.
		return this._focalNodes;
	},

	_onNodeFocus: function(evt){
		this._focusedNode = evt.currentTarget;
	},

	_onBlur: function(evt){
		this.inherited(arguments);
		this._focusedNode = null;
	}
});

dojo.declare("dijit.form.ToggleButton", dijit.form.Button, {
	// summary
	//	A button that can be in two states (checked or not).
	//	Can be base class for things like tabs or checkbox or radio buttons

	baseClass: "dijitToggleButton",

	// checked: Boolean
	//		Corresponds to the native HTML <input> element's attribute.
	//		In markup, specified as "checked='checked'" or just "checked".
	//		True if the button is depressed, or the checkbox is checked,
	//		or the radio button is selected, etc.
	checked: false,

	_clicked: function(/*Event*/ evt){
		this.setChecked(!this.checked);
	},

	setChecked: function(/*Boolean*/ checked){
		// summary
		//	Programatically deselect the button
		this.checked = checked;
		dijit.setWaiState(this.focusNode || this.domNode, "pressed", this.checked);
		this._setStateClass();		
		this.onChange(checked);
	}
});

}

if(!dojo._hasResource["dijit.layout.StackContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.StackContainer"] = true;
dojo.provide("dijit.layout.StackContainer");





dojo.declare(
	"dijit.layout.StackContainer",
	dijit.layout._LayoutWidget,

	// summary
	//	A container that has multiple children, but shows only
	//	one child at a time (like looking at the pages in a book one by one).
	//
	//	Publishes topics <widgetId>-addChild, <widgetId>-removeChild, and <widgetId>-selectChild
	//
	//	Can be base class for container, Wizard, Show, etc.
{
	// doLayout: Boolean
	//  if true, change the size of my currently displayed child to match my size
	doLayout: true,

	_started: false,

	// selectedChildWidget: Widget
	//	References the currently selected child widget, if any

	postCreate: function(){
		dijit.setWaiRole((this.containerNode || this.domNode), "tabpanel");
		this.connect(this.domNode, "onkeypress", this._onKeyPress);
	},
	
	startup: function(){
		if(this._started){ return; }

		var children = this.getChildren();

		// Setup each page panel
		dojo.forEach(children, this._setupChild, this);

		// Figure out which child to initially display
		dojo.some(children, function(child){
			if(child.selected){
				this.selectedChildWidget = child;
			}
			return child.selected;
		}, this);

		var selected = this.selectedChildWidget;

		// Default to the first child
		if(!selected && children[0]){
			selected = this.selectedChildWidget = children[0];
			selected.selected = true;
		}
		if(selected){
			this._showChild(selected);
		}

		// Now publish information about myself so any StackControllers can initialize..
		dojo.publish(this.id+"-startup", [{children: children, selected: selected}]);
		this.inherited("startup",arguments);
		this._started = true;
	},

	_setupChild: function(/*Widget*/ page){
		// Summary: prepare the given child

		page.domNode.style.display = "none";

		// since we are setting the width/height of the child elements, they need
		// to be position:relative, or IE has problems (See bug #2033)
		page.domNode.style.position = "relative";

		return page; // dijit._Widget
	},

	addChild: function(/*Widget*/ child, /*Integer?*/ insertIndex){
		// summary: Adds a widget to the stack
		 
		dijit._Container.prototype.addChild.apply(this, arguments);
		child = this._setupChild(child);

		if(this._started){
			// in case the tab titles have overflowed from one line to two lines
			this.layout();

			dojo.publish(this.id+"-addChild", [child, insertIndex]);

			// if this is the first child, then select it
			if(!this.selectedChildWidget){
				this.selectChild(child);
			}
		}
	},

	removeChild: function(/*Widget*/ page){
		// summary: Removes the pane from the stack

		dijit._Container.prototype.removeChild.apply(this, arguments);

		// If we are being destroyed than don't run the code below (to select another page), because we are deleting
		// every page one by one
		if(this._beingDestroyed){ return; }

		if(this._started){
			// this will notify any tablists to remove a button; do this first because it may affect sizing
			dojo.publish(this.id+"-removeChild", [page]);

			// in case the tab titles now take up one line instead of two lines
			this.layout();
		}

		if(this.selectedChildWidget === page){
			this.selectedChildWidget = undefined;
			if(this._started){
				var children = this.getChildren();
				if(children.length){
					this.selectChild(children[0]);
				}
			}
		}
	},

	selectChild: function(/*Widget*/ page){
		// summary:
		//	Show the given widget (which must be one of my children)

		page = dijit.byId(page);

		if(this.selectedChildWidget != page){
			// Deselect old page and select new one
			this._transition(page, this.selectedChildWidget);
			this.selectedChildWidget = page;
			dojo.publish(this.id+"-selectChild", [page]);
		}
	},

	_transition: function(/*Widget*/newWidget, /*Widget*/oldWidget){
		if(oldWidget){
			this._hideChild(oldWidget);
		}
		this._showChild(newWidget);

		// Size the new widget, in case this is the first time it's being shown,
		// or I have been resized since the last time it was shown.
		// page must be visible for resizing to work
		if(this.doLayout && newWidget.resize){
			newWidget.resize(this._containerContentBox || this._contentBox);
		}
	},

	_adjacent: function(/*Boolean*/ forward){
		// summary: Gets the next/previous child widget in this container from the current selection
		var children = this.getChildren();
		var index = dojo.indexOf(children, this.selectedChildWidget);
		index += forward ? 1 : children.length - 1;
		return children[ index % children.length ]; // dijit._Widget
	},

	forward: function(){
		// Summary: advance to next page
		this.selectChild(this._adjacent(true));
	},

	back: function(){
		// Summary: go back to previous page
		this.selectChild(this._adjacent(false));
	},

	_onKeyPress: function(e){
		dojo.publish(this.id+"-containerKeyPress", [{ e: e, page: this}]);
	},

	layout: function(){
		if(this.doLayout && this.selectedChildWidget && this.selectedChildWidget.resize){
			this.selectedChildWidget.resize(this._contentBox);
		}
	},

	_showChild: function(/*Widget*/ page){
		var children = this.getChildren();
		page.isFirstChild = (page == children[0]);
		page.isLastChild = (page == children[children.length-1]);
		page.selected = true;

		page.domNode.style.display="";
		if(page._loadCheck){
			page._loadCheck(); // trigger load in ContentPane
		}
		if(page.onShow){
			page.onShow();
		}
	},

	_hideChild: function(/*Widget*/ page){
		page.selected=false;
		page.domNode.style.display="none";
		if(page.onHide){
			page.onHide();
		}
	},

	closeChild: function(/*Widget*/ page){
		// summary
		//	callback when user clicks the [X] to remove a page
		//	if onClose() returns true then remove and destroy the childd
		var remove = page.onClose(this, page);
		if(remove){
			this.removeChild(page);
			// makes sure we can clean up executeScripts in ContentPane onUnLoad
			page.destroy();
		}
	},

	destroy: function(){
		this._beingDestroyed = true;
		this.inherited("destroy",arguments);
	}
});

dojo.declare(
	"dijit.layout.StackController",
	[dijit._Widget, dijit._Templated, dijit._Container],
	{
		// summary:
		//	Set of buttons to select a page in a page list.
		//	Monitors the specified StackContainer, and whenever a page is
		//	added, deleted, or selected, updates itself accordingly.

		templateString: "<span wairole='tablist' dojoAttachEvent='onkeypress' class='dijitStackController'></span>",

		// containerId: String
		//	the id of the page container that I point to
		containerId: "",

		// buttonWidget: String
		//	the name of the button widget to create to correspond to each page
		buttonWidget: "dijit.layout._StackButton",

		postCreate: function(){
			dijit.setWaiRole(this.domNode, "tablist");

			this.pane2button = {};		// mapping from panes to buttons
			this._subscriptions=[
				dojo.subscribe(this.containerId+"-startup", this, "onStartup"),
				dojo.subscribe(this.containerId+"-addChild", this, "onAddChild"),
				dojo.subscribe(this.containerId+"-removeChild", this, "onRemoveChild"),
				dojo.subscribe(this.containerId+"-selectChild", this, "onSelectChild"),
				dojo.subscribe(this.containerId+"-containerKeyPress", this, "onContainerKeyPress")
			];
		},

		onStartup: function(/*Object*/ info){
			// summary: called after StackContainer has finished initializing
			dojo.forEach(info.children, this.onAddChild, this);
			this.onSelectChild(info.selected);
		},

		destroy: function(){
			dojo.forEach(this._subscriptions, dojo.unsubscribe);
			this.inherited("destroy",arguments);
		},

		onAddChild: function(/*Widget*/ page, /*Integer?*/ insertIndex){
			// summary:
			//   Called whenever a page is added to the container.
			//   Create button corresponding to the page.

			// add a node that will be promoted to the button widget
			var refNode = document.createElement("span");
			this.domNode.appendChild(refNode);
			// create an instance of the button widget
			var cls = dojo.getObject(this.buttonWidget);
			var button = new cls({label: page.title, closeButton: page.closable}, refNode);
			this.addChild(button, insertIndex);
			this.pane2button[page] = button;
			page.controlButton = button;	// this value might be overwritten if two tabs point to same container
			
			dojo.connect(button, "onClick", dojo.hitch(this,"onButtonClick",page));
			dojo.connect(button, "onClickCloseButton", dojo.hitch(this,"onCloseButtonClick",page));
			
			if(!this._currentChild){ // put the first child into the tab order
				button.focusNode.setAttribute("tabIndex","0");
				this._currentChild = page;
			}
		},

		onRemoveChild: function(/*Widget*/ page){
			// summary:
			//   Called whenever a page is removed from the container.
			//   Remove the button corresponding to the page.
			if(this._currentChild === page){ this._currentChild = null; }
			var button = this.pane2button[page];
			if(button){
				// TODO? if current child { reassign }
				button.destroy();
			}
			this.pane2button[page] = null;
		},

		onSelectChild: function(/*Widget*/ page){
			// summary:
			//	Called when a page has been selected in the StackContainer, either by me or by another StackController

			if(!page){ return; }

			if(this._currentChild){
				var oldButton=this.pane2button[this._currentChild];
				oldButton.setChecked(false);
				oldButton.focusNode.setAttribute("tabIndex", "-1");
			}

			var newButton=this.pane2button[page];
			newButton.setChecked(true);
			this._currentChild = page;
			newButton.focusNode.setAttribute("tabIndex", "0");
		},

		onButtonClick: function(/*Widget*/ page){
			// summary:
			//   Called whenever one of my child buttons is pressed in an attempt to select a page
			var container = dijit.byId(this.containerId);	// TODO: do this via topics?
			container.selectChild(page); 
		},

		onCloseButtonClick: function(/*Widget*/ page){
			// summary:
			//   Called whenever one of my child buttons [X] is pressed in an attempt to close a page
			var container = dijit.byId(this.containerId);
			container.closeChild(page);
			var b = this.pane2button[this._currentChild];
			if(b){
				dijit.focus(b.focusNode || b.domNode);
			}
		},
		
		// TODO: this is a bit redundant with forward, back api in StackContainer
		adjacent: function(/*Boolean*/ forward){
			// find currently focused button in children array
			var children = this.getChildren();
			var current = dojo.indexOf(children, this.pane2button[this._currentChild]);
			// pick next button to focus on
			var offset = forward ? 1 : children.length - 1;
			return children[ (current + offset) % children.length ]; // dijit._Widget
		},

		onkeypress: function(/*Event*/ e){
			// summary:
			//   Handle keystrokes on the page list, for advancing to next/previous button
			//   and closing the current page if the page is closable.

			if(this.disabled || e.altKey ){ return; }
			var forward = true;
			if(e.ctrlKey || !e._djpage){
				var k = dojo.keys;
				switch(e.keyCode){
					case k.LEFT_ARROW:
					case k.UP_ARROW:
					case k.PAGE_UP:
						forward = false;
						// fall through
					case k.RIGHT_ARROW:
					case k.DOWN_ARROW:
					case k.PAGE_DOWN:
						this.adjacent(forward).onClick();
						dojo.stopEvent(e);
						break;
					case k.DELETE:
						if(this._currentChild.closable){
							this.onCloseButtonClick(this._currentChild);
						}
						dojo.stopEvent(e);
						break;
					default:
						if(e.ctrlKey){
							if(e.keyCode == k.TAB){
								this.adjacent(!e.shiftKey).onClick();
								dojo.stopEvent(e);
							}else if(e.keyChar == "w"){
								if(this._currentChild.closable){
									this.onCloseButtonClick(this._currentChild);
								}
								dojo.stopEvent(e); // avoid browser tab closing.
							}
						}
				}
			}
		},

		onContainerKeyPress: function(/*Object*/ info){
			info.e._djpage = info.page;
			this.onkeypress(info.e);
		}
});

dojo.declare("dijit.layout._StackButton",
	dijit.form.ToggleButton,
	{
	// summary
	//	Internal widget used by StackContainer.
	//	The button-like or tab-like object you click to select or delete a page
	
	tabIndex: "-1", // StackContainer buttons are not in the tab order by default
	
	postCreate: function(/*Event*/ evt){
		dijit.setWaiRole((this.focusNode || this.domNode), "tab");
		this.inherited("postCreate", arguments);
	},
	
	onClick: function(/*Event*/ evt){
		// summary: This is for TabContainer where the tabs are <span> rather than button,
		// 	so need to set focus explicitly (on some browsers)
		dijit.focus(this.focusNode);

		// ... now let StackController catch the event and tell me what to do
	},

	onClickCloseButton: function(/*Event*/ evt){
		// summary
		//	StackContainer connects to this function; if your widget contains a close button
		//	then clicking it should call this function.
		evt.stopPropagation();
	}
});

// These arguments can be specified for the children of a StackContainer.
// Since any widget can be specified as a StackContainer child, mix them
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// title: String
	//		Title of this widget.  Used by TabContainer to the name the tab, etc.
	title: "",

	// selected: Boolean
	//		Is this child currently selected?
	selected: false,

	// closable: Boolean
	//		True if user can close (destroy) this child, such as (for example) clicking the X on the tab.
	closable: false,	// true if user can close this tab pane

	onClose: function(){
		// summary: Callback if someone tries to close the child, child will be closed if func returns true
		return true;
	}
});

}

if(!dojo._hasResource["dijit.form.TextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.TextBox"] = true;
dojo.provide("dijit.form.TextBox");



dojo.declare(
	"dijit.form.TextBox",
	dijit.form._FormWidget,
	{
		// summary:
		//		A generic textbox field.
		//		Serves as a base class to derive more specialized functionality in subclasses.

		//	trim: Boolean
		//		Removes leading and trailing whitespace if true.  Default is false.
		trim: false,

		//	uppercase: Boolean
		//		Converts all characters to uppercase if true.  Default is false.
		uppercase: false,

		//	lowercase: Boolean
		//		Converts all characters to lowercase if true.  Default is false.
		lowercase: false,

		//	propercase: Boolean
		//		Converts the first character of each word to uppercase if true.
		propercase: false,

		// maxLength: String
		//		HTML INPUT tag maxLength declaration.
		maxLength: "",

		templateString:"<input class=\"dojoTextBox\" dojoAttachPoint='textbox,focusNode' name=\"${name}\"\r\n\tdojoAttachEvent='onmouseenter:_onMouse,onmouseleave:_onMouse,onfocus:_onMouse,onblur:_onMouse,onkeyup,onkeypress:_onKeyPress'\r\n\tautocomplete=\"off\" type=\"${type}\"\r\n\t/>\r\n",
		baseClass: "dijitTextBox",

		attributeMap: dojo.mixin(dojo.clone(dijit.form._FormWidget.prototype.attributeMap),
			{maxLength:"focusNode"}),

		getDisplayedValue: function(){
			return this.filter(this.textbox.value);
		},

		getValue: function(){
			return this.parse(this.getDisplayedValue(), this.constraints);
		},

		setValue: function(value, /*Boolean, optional*/ priorityChange, /*String, optional*/ formattedValue){
			var filteredValue = this.filter(value);
			if((typeof filteredValue == typeof value) && (formattedValue == null || formattedValue == undefined)){
				formattedValue = this.format(filteredValue, this.constraints);
			}
			if(formattedValue != null && formattedValue != undefined){
				this.textbox.value = formattedValue;
			}
			dijit.form.TextBox.superclass.setValue.call(this, filteredValue, priorityChange);
		},

		setDisplayedValue: function(/*String*/value){
			this.textbox.value = value;
			this.setValue(this.getValue(), true);
		},

		forWaiValuenow: function(){
			return this.getDisplayedValue();
		},

		format: function(/* String */ value, /* Object */ constraints){
			// summary: Replacable function to convert a value to a properly formatted string
			return ((value == null || value == undefined) ? "" : (value.toString ? value.toString() : value));
		},

		parse: function(/* String */ value, /* Object */ constraints){
			// summary: Replacable function to convert a formatted string to a value
			return value;
		},

		postCreate: function(){
			// setting the value here is needed since value="" in the template causes "undefined"
			// and setting in the DOM (instead of the JS object) helps with form reset actions
			this.textbox.setAttribute("value", this.getDisplayedValue());
			this.inherited('postCreate', arguments);

			if(this.srcNodeRef){
				dojo.style(this.textbox, "cssText", this.style);
				this.textbox.className += " " + this["class"];
			}
			this._layoutHack();
		},

		_layoutHack: function(){
			// summary: work around table sizing bugs on FF2 by forcing redraw
			if(dojo.isFF == 2 && this.domNode.tagName=="TABLE"){
				var node=this.domNode;
				var old = node.style.opacity;
				node.style.opacity = "0.999";
				setTimeout(function(){
					node.style.opacity = old;
				}, 0);
			}			
		},

		filter: function(val){
			// summary: Apply various filters to textbox value
			if(val == undefined || val == null){ return ""; }
			else if(typeof val != "string"){ return val; }
			if(this.trim){
				val = dojo.trim(val);
			}
			if(this.uppercase){
				val = val.toUpperCase();
			}
			if(this.lowercase){
				val = val.toLowerCase();
			}
			if(this.propercase){
				val = val.replace(/[^\s]+/g, function(word){
					return word.substring(0,1).toUpperCase() + word.substring(1);
				});
			}
			return val;
		},

		// event handlers, you can over-ride these in your own subclasses
		_onBlur: function(){
			this.setValue(this.getValue(), (this.isValid ? this.isValid() : true));
		},

		onkeyup: function(){
			// TODO: it would be nice to massage the value (ie: automatic uppercase, etc) as the user types
			// but this messes up the cursor position if you are typing into the middle of a word, and
			// also trimming doesn't work correctly (it prevents spaces between words too!)
			// this.setValue(this.getValue());
		}
	}
);

}

if(!dojo._hasResource["dijit.Tooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Tooltip"] = true;
dojo.provide("dijit.Tooltip");




dojo.declare(
	"dijit._MasterTooltip",
	[dijit._Widget, dijit._Templated],
	{
		// summary
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: 200,

		templateString:"<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\">\r\n\t<div class=\"dijitTooltipContainer dijitTooltipContents\" dojoAttachPoint=\"containerNode\" waiRole='alert'></div>\r\n\t<div class=\"dijitTooltipConnector\"></div>\r\n</div>\r\n",

		postCreate: function(){
			dojo.body().appendChild(this.domNode);

			this.bgIframe = new dijit.BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = dojo.fadeIn({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onShow") });
			this.fadeOut = dojo.fadeOut({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onHide") });

		},

		show: function(/*String*/ innerHTML, /*DomNode*/ aroundNode){
			// summary:
			//	Display tooltip w/specified contents to right specified node
			//	(To left if there's no space on the right, or if LTR==right)

			if(this.aroundNode && this.aroundNode === aroundNode){
				return;
			}

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			// Firefox bug. when innerHTML changes to be shorter than previous
			// one, the node size will not be updated until it moves.
			this.domNode.style.top = (this.domNode.offsetTop + 1) + "px";

			// position the element and change CSS according to position	
			var align = this.isLeftToRight() ? {'BR': 'BL', 'BL': 'BR'} : {'BL': 'BR', 'BR': 'BL'};
			var pos = dijit.placeOnScreenAroundElement(this.domNode, aroundNode, align);
			this.domNode.className="dijitTooltip dijitTooltip" + (pos.corner=='BL' ? "Right" : "Left");//FIXME: might overwrite class

			// show it
			dojo.style(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		_onShow: function(){
			if(dojo.isIE){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary: hide the tooltip
			if(!this.aroundNode || this.aroundNode !== aroundNode){
				return;
			}
			if(this._onDeck){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
				return;
			}
			this.fadeIn.stop();
			this.isShowingNow = false;
			this.aroundNode = null;
			this.fadeOut.play();
		},

		_onHide: function(){
			this.domNode.style.cssText="";	// to position offscreen again
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		}

	}
);

dijit.showTooltip = function(/*String*/ innerHTML, /*DomNode*/ aroundNode){
	// summary:
	//	Display tooltip w/specified contents to right specified node
	//	(To left if there's no space on the right, or if LTR==right)
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.show(innerHTML, aroundNode);
};

dijit.hideTooltip = function(aroundNode){
	// summary: hide the tooltip
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.hide(aroundNode);
};

dojo.declare(
	"dijit.Tooltip",
	dijit._Widget,
	{
		// summary
		//		Pops up a tooltip (a help message) when you hover over a node.

		// label: String
		//		Text to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: String[]
		//		Id(s) of domNodes to attach the tooltip to.
		//		When user hovers over any of the specified dom nodes, the tooltip will appear.
		connectId: [],

		postCreate: function(){
			if(this.srcNodeRef){
				this.srcNodeRef.style.display = "none";
			}

			this._connectNodes = [];
			
			dojo.forEach(this.connectId, function(id) {
				var node = dojo.byId(id);
				if (node) {
					this._connectNodes.push(node);
					dojo.forEach(["onMouseOver", "onMouseOut", "onFocus", "onBlur", "onHover", "onUnHover"], function(event){
						this.connect(node, event.toLowerCase(), "_"+event);
					}, this);
					if(dojo.isIE){
						// BiDi workaround
						node.style.zoom = 1;
					}
				}
			}, this);
		},

		_onMouseOver: function(/*Event*/ e){
			this._onHover(e);
		},

		_onMouseOut: function(/*Event*/ e){
			if(dojo.isDescendant(e.relatedTarget, e.target)){
				// false event; just moved from target to target child; ignore.
				return;
			}
			this._onUnHover(e);
		},

		_onFocus: function(/*Event*/ e){
			this._focus = true;
			this._onHover(e);
		},
		
		_onBlur: function(/*Event*/ e){
			this._focus = false;
			this._onUnHover(e);
		},

		_onHover: function(/*Event*/ e){
			if(!this._showTimer){
				var target = e.target;
				this._showTimer = setTimeout(dojo.hitch(this, function(){this.open(target)}), this.showDelay);
			}
		},

		_onUnHover: function(/*Event*/ e){
			// keep a tooltip open if the associated element has focus
			if(this._focus){ return; }
			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
 			// summary: display the tooltip; usually not called directly.
			target = target || this._connectNodes[0];
			if(!target){ return; }

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			dijit.showTooltip(this.label || this.domNode.innerHTML, target);
			
			this._connectNode = target;
		},

		close: function(){
			// summary: hide the tooltip; usually not called directly.
			dijit.hideTooltip(this._connectNode);
			delete this._connectNode;
			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
		},

		uninitialize: function(){
			this.close();
		}
	}
);

}

if(!dojo._hasResource["dijit.form.ValidationTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ValidationTextBox"] = true;
dojo.provide("dijit.form.ValidationTextBox");








dojo.declare(
	"dijit.form.ValidationTextBox",
	dijit.form.TextBox,
	{
		// summary:
		//		A subclass of TextBox.
		//		Over-ride isValid in subclasses to perform specific kinds of validation.

		templateString:"<table style=\"display: -moz-inline-stack;\" class=\"dijit dijitReset dijitInlineTable\" cellspacing=\"0\" cellpadding=\"0\"\r\n\tid=\"widget_${id}\" name=\"${name}\"\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse\" waiRole=\"presentation\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset dijitInputField\" width=\"100%\"\r\n\t\t\t><input dojoAttachPoint='textbox,focusNode' dojoAttachEvent='onfocus,onblur:_onMouse,onkeyup,onkeypress:_onKeyPress' autocomplete=\"off\"\r\n\t\t\ttype='${type}' name='${name}'\r\n\t\t/></td\r\n\t\t><td class=\"dijitReset dijitValidationIconField\" width=\"0%\"\r\n\t\t\t><div dojoAttachPoint='iconNode' class='dijitValidationIcon'></div><div class='dijitValidationIconText'>&Chi;</div\r\n\t\t></td\r\n\t></tr\r\n></table>\r\n",
		baseClass: "dijitTextBox",

		// default values for new subclass properties
		// required: Boolean
		//		Can be true or false, default is false.
		required: false,
		// promptMessage: String
		//		Hint string
		promptMessage: "",
		// invalidMessage: String
		// 		The message to display if value is invalid.
		invalidMessage: "$_unset_$", // read from the message file if not overridden
		// constraints: Object
		//		user-defined object needed to pass parameters to the validator functions
		constraints: {},
		// regExp: String
		//		regular expression string used to validate the input
		//		Do not specify both regExp and regExpGen
		regExp: ".*",
		// regExpGen: Function
		//		user replaceable function used to generate regExp when dependent on constraints
		//		Do not specify both regExp and regExpGen
		regExpGen: function(constraints){ return this.regExp; },
		
		// state: String
		//		Shows current state (ie, validation result) of input (Normal, Warning, or Error)
		state: "",

		setValue: function(){
			this.inherited('setValue', arguments);
			this.validate(false);
		},

		validator: function(value,constraints){
			// summary: user replaceable function used to validate the text input against the regular expression.
			return (new RegExp("^(" + this.regExpGen(constraints) + ")"+(this.required?"":"?")+"$")).test(value) &&
				(!this.required || !this._isEmpty(value)) &&
				(this._isEmpty(value) || this.parse(value, constraints) !== null);
		},

		isValid: function(/* Boolean*/ isFocused){
			// summary: Need to over-ride with your own validation code in subclasses
			return this.validator(this.textbox.value, this.constraints);
		},

		_isEmpty: function(value){
			// summary: Checks for whitespace
			return /^\s*$/.test(value); // Boolean
		},

		getErrorMessage: function(/* Boolean*/ isFocused){
			// summary: return an error message to show if appropriate
			return this.invalidMessage;
		},

		getPromptMessage: function(/* Boolean*/ isFocused){
			// summary: return a hint to show if appropriate
			return this.promptMessage;
		},

		validate: function(/* Boolean*/ isFocused){
			// summary:
			//		Called by oninit, onblur, and onkeypress.
			// description:
			//		Show missing or invalid messages if appropriate, and highlight textbox field.
			var message = "";
			var isValid = this.isValid(isFocused);
			var isEmpty = this._isEmpty(this.textbox.value);
			this.state = (isValid || (!this._hasBeenBlurred && isEmpty)) ? "" : "Error";
			this._setStateClass();
			dijit.setWaiState(this.focusNode, "invalid", (isValid? "false" : "true"));
			if(isFocused){
				if(isEmpty){
					message = this.getPromptMessage(true);
				}
				if(!message && !isValid){
					message = this.getErrorMessage(true);
				}
			}
			this._displayMessage(message);
		},

		// currently displayed message
		_message: "",

		_displayMessage: function(/*String*/ message){
			if(this._message == message){ return; }
			this._message = message;
			this.displayMessage(message);
		},

		displayMessage: function(/*String*/ message){
			// summary:
			//		User overridable method to display validation errors/hints.
			//		By default uses a tooltip.
			if(message){
				dijit.showTooltip(message, this.domNode);
			}else{
				dijit.hideTooltip(this.domNode);
			}
		},

		_hasBeenBlurred: false,

		_onBlur: function(evt){
			this._hasBeenBlurred = true;
			this.validate(false);
			this.inherited('_onBlur', arguments);
		},

		onfocus: function(evt){
			// TODO: change to _onFocus?
			this.validate(true);
			this._onMouse(evt);	// update CSS classes
		},

		onkeyup: function(evt){
			this.onfocus(evt);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////
		constructor: function(){
			this.constraints = {};
		},

		postMixInProperties: function(){
			this.inherited('postMixInProperties', arguments);
			this.constraints.locale=this.lang;
			this.messages = dojo.i18n.getLocalization("dijit.form", "validate", this.lang);
			if(this.invalidMessage == "$_unset_$"){ this.invalidMessage = this.messages.invalidMessage; }
			var p = this.regExpGen(this.constraints);
			this.regExp = p;
			// make value a string for all types so that form reset works well
		}
	}
);

dojo.declare(
	"dijit.form.MappedTextBox",
	dijit.form.ValidationTextBox,
	{
		// summary:
		//		A subclass of ValidationTextBox.
		//		Provides a hidden input field and a serialize method to override

		serialize: function(val, /*Object?*/options){
			// summary: user replaceable function used to convert the getValue() result to a String
			return (val.toString ? val.toString() : "");
		},

		toString: function(){
			// summary: display the widget as a printable string using the widget's value
			var val = this.filter(this.getValue());
			return (val!=null) ? ((typeof val == "string") ? val : this.serialize(val, this.constraints)) : "";
		},

		validate: function(){
			this.valueNode.value = this.toString();
			this.inherited('validate', arguments);
		},

		postCreate: function(){
			var textbox = this.textbox;
			var valueNode = (this.valueNode = document.createElement("input"));
			valueNode.setAttribute("type", textbox.type);
			valueNode.setAttribute("value", this.toString());
			dojo.style(valueNode, "display", "none");
			valueNode.name = this.textbox.name;
			this.textbox.name = "_" + this.textbox.name + "_displayed_";
			this.textbox.removeAttribute("name");
			dojo.place(valueNode, textbox, "after");

			this.inherited('postCreate', arguments);
		}
	}
);

dojo.declare(
	"dijit.form.RangeBoundTextBox",
	dijit.form.MappedTextBox,
	{
		// summary:
		//		A subclass of MappedTextBox.
		//		Tests for a value out-of-range
		/*===== contraints object:
		// min: Number
		//		Minimum signed value.  Default is -Infinity
		min: undefined,
		// max: Number
		//		Maximum signed value.  Default is +Infinity
		max: undefined,
		=====*/

		// rangeMessage: String
		//		The message to display if value is out-of-range
		rangeMessage: "",

		compare: function(val1, val2){
			// summary: compare 2 values
			return val1 - val2;
		},

		rangeCheck: function(/* Number */ primitive, /* Object */ constraints){
			// summary: user replaceable function used to validate the range of the numeric input value
			var isMin = (typeof constraints.min != "undefined");
			var isMax = (typeof constraints.max != "undefined");
			if(isMin || isMax){
				return (!isMin || this.compare(primitive,constraints.min) >= 0) &&
					(!isMax || this.compare(primitive,constraints.max) <= 0);
			}else{ return true; }
		},

		isInRange: function(/* Boolean*/ isFocused){
			// summary: Need to over-ride with your own validation code in subclasses
			return this.rangeCheck(this.getValue(), this.constraints);
		},

		isValid: function(/* Boolean*/ isFocused){
			return this.inherited('isValid', arguments) &&
				((this._isEmpty(this.textbox.value) && !this.required) || this.isInRange(isFocused));
		},

		getErrorMessage: function(/* Boolean*/ isFocused){
			if(dijit.form.RangeBoundTextBox.superclass.isValid.call(this, false) && !this.isInRange(isFocused)){ return this.rangeMessage; }
			else{ return this.inherited('getErrorMessage', arguments); }
		},

		postMixInProperties: function(){
			this.inherited('postMixInProperties', arguments);
			if(!this.rangeMessage){
				this.messages = dojo.i18n.getLocalization("dijit.form", "validate", this.lang);
				this.rangeMessage = this.messages.rangeMessage;
			}
		},

		postCreate: function(){
			this.inherited('postCreate', arguments);
			if(typeof this.constraints.min != "undefined"){
				dijit.setWaiState(this.focusNode, "valuemin", this.constraints.min);
			}
			if(typeof this.constraints.max != "undefined"){
				dijit.setWaiState(this.focusNode, "valuemax", this.constraints.max);
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.form.ComboBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ComboBox"] = true;
dojo.provide("dijit.form.ComboBox");





dojo.declare(
	"dijit.form.ComboBoxMixin",
	null,
	{
		// summary:
		//		Auto-completing text box, and base class for FilteringSelect widget.
		//
		//		The drop down box's values are populated from an class called
		//		a data provider, which returns a list of values based on the characters
		//		that the user has typed into the input box.
		//
		//		Some of the options to the ComboBox are actually arguments to the data
		//		provider.
		//
		//		You can assume that all the form widgets (and thus anything that mixes
		//		in ComboBoxMixin) will inherit from _FormWidget and thus the "this"
		//		reference will also "be a" _FormWidget.

		// item: Object
		//		This is the item returned by the dojo.data.store implementation that
		//		provides the data for this cobobox, it's the currently selected item.
		item: null,

		// pageSize: Integer
		//		Argument to data provider.
		//		Specifies number of search results per page (before hitting "next" button)
		pageSize: Infinity,

		// store: Object
		//		Reference to data provider object used by this ComboBox
		store: null,

		// query: Object
		//		A query that can be passed to 'store' to initially filter the items,
		//		before doing further filtering based on searchAttr and the key.
		query: {},

		// autoComplete: Boolean
		//		If you type in a partial string, and then tab out of the <input> box,
		//		automatically copy the first entry displayed in the drop down list to
		//		the <input> field
		autoComplete: true,

		// searchDelay: Integer
		//		Delay in milliseconds between when user types something and we start
		//		searching based on that value
		searchDelay: 100,

		// searchAttr: String
		//		Searches pattern match against this field
		searchAttr: "name",

		// ignoreCase: Boolean
		//		Set true if the ComboBox should ignore case when matching possible items
		ignoreCase: true,

		// hasDownArrow: Boolean
		//		Set this textbox to have a down arrow button.
		//		Defaults to true.
		hasDownArrow:true,

		// _hasFocus: Boolean
		//		Represents focus state of the textbox
		// TODO: get rid of this; it's unnecessary (but currently referenced in FilteringSelect)
		_hasFocus:false,

		templateString:"<table class=\"dijit dijitReset dijitInlineTable dijitLeft\" cellspacing=\"0\" cellpadding=\"0\"\r\n\tid=\"widget_${id}\" name=\"${name}\" dojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse\" waiRole=\"presentation\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class='dijitReset dijitStretch dijitInputField' width=\"100%\"\r\n\t\t\t><input type=\"text\" autocomplete=\"off\" name=\"${name}\"\r\n\t\t\tdojoAttachEvent=\"onkeypress, onkeyup, onfocus, compositionend\"\r\n\t\t\tdojoAttachPoint=\"textbox,focusNode\" waiRole=\"combobox\"\r\n\t\t/></td\r\n\t\t><td class=\"dijitReset dijitValidationIconField\" width=\"0%\"\r\n\t\t\t><div dojoAttachPoint='iconNode' class='dijitValidationIcon'></div\r\n\t\t\t><div class='dijitValidationIconText'>&Chi;</div\r\n\t\t></td\r\n\t\t><td class='dijitReset dijitRight dijitButtonNode dijitDownArrowButton' width=\"0%\"\r\n\t\t\tdojoAttachPoint=\"downArrowNode\"\r\n\t\t\tdojoAttachEvent=\"onmousedown:_onArrowMouseDown,onmouseup:_onMouse,onmouseenter:_onMouse,onmouseleave:_onMouse\"\r\n\t\t\t><div class=\"dijitDownArrowButtonInner\" waiRole=\"presentation\"\r\n\t\t\t\t><div class=\"dijitDownArrowButtonChar\">&#9660;</div\r\n\t\t\t></div\r\n\t\t></td\t\r\n\t></tr\r\n></table>\r\n",

		baseClass:"dijitComboBox",

		_lastDisplayedValue: "",

		getValue:function(){
			// don't get the textbox value but rather the previously set hidden value
			return dijit.form.TextBox.superclass.getValue.apply(this, arguments);
		},

		setDisplayedValue:function(/*String*/ value){
			this._lastDisplayedValue = value;
			this.setValue(value, true);
		},

		_getCaretPos: function(/*DomNode*/ element){
			// khtml 3.5.2 has selection* methods as does webkit nightlies from 2005-06-22
			if(typeof(element.selectionStart)=="number"){
				// FIXME: this is totally borked on Moz < 1.3. Any recourse?
				return element.selectionStart;
			}else if(dojo.isIE){
				// in the case of a mouse click in a popup being handled,
				// then the document.selection is not the textarea, but the popup
				// var r = document.selection.createRange();
				// hack to get IE 6 to play nice. What a POS browser.
				var tr = document.selection.createRange().duplicate();
				var ntr = element.createTextRange();
				tr.move("character",0);
				ntr.move("character",0);
				try{
					// If control doesnt have focus, you get an exception.
					// Seems to happen on reverse-tab, but can also happen on tab (seems to be a race condition - only happens sometimes).
					// There appears to be no workaround for this - googled for quite a while.
					ntr.setEndPoint("EndToEnd", tr);
					return String(ntr.text).replace(/\r/g,"").length;
				}catch(e){
					return 0; // If focus has shifted, 0 is fine for caret pos.
				}
			}
		},

		_setCaretPos: function(/*DomNode*/ element, /*Number*/ location){
			location = parseInt(location);
			this._setSelectedRange(element, location, location);
		},

		_setSelectedRange: function(/*DomNode*/ element, /*Number*/ start, /*Number*/ end){
			if(!end){
				end = element.value.length;
			}  // NOTE: Strange - should be able to put caret at start of text?
			// Mozilla
			// parts borrowed from http://www.faqts.com/knowledge_base/view.phtml/aid/13562/fid/130
			if(element.setSelectionRange){
				dijit.focus(element);
				element.setSelectionRange(start, end);
			}else if(element.createTextRange){ // IE
				var range = element.createTextRange();
				with(range){
					collapse(true);
					moveEnd('character', end);
					moveStart('character', start);
					select();
				}
			}else{ //otherwise try the event-creation hack (our own invention)
				// do we need these?
				element.value = element.value;
				element.blur();
				dijit.focus(element);
				// figure out how far back to go
				var dist = parseInt(element.value.length)-end;
				var tchar = String.fromCharCode(37);
				var tcc = tchar.charCodeAt(0);
				for(var x = 0; x < dist; x++){
					var te = document.createEvent("KeyEvents");
					te.initKeyEvent("keypress", true, true, null, false, false, false, false, tcc, tcc);
					element.dispatchEvent(te);
				}
			}
		},

		onkeypress: function(/*Event*/ evt){
			// summary: handles keyboard events

			//except for pasting case - ctrl + v(118)
			if(evt.altKey || (evt.ctrlKey && evt.charCode != 118)){
				return;
			}
			var doSearch = false;
			this.item = null; // #4872
			if(this._isShowingNow){this._popupWidget.handleKey(evt);}
			switch(evt.keyCode){
				case dojo.keys.PAGE_DOWN:
				case dojo.keys.DOWN_ARROW:
					if(!this._isShowingNow||this._prev_key_esc){
						this._arrowPressed();
						doSearch=true;
					}else{
						this._announceOption(this._popupWidget.getHighlightedOption());
					}
					dojo.stopEvent(evt);
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					break;

				case dojo.keys.PAGE_UP:
				case dojo.keys.UP_ARROW:
					if(this._isShowingNow){
						this._announceOption(this._popupWidget.getHighlightedOption());
					}
					dojo.stopEvent(evt);
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					break;

				case dojo.keys.ENTER:
					// prevent submitting form if user presses enter
					// also prevent accepting the value if either Next or Previous are selected
					var highlighted;
					if(this._isShowingNow&&(highlighted=this._popupWidget.getHighlightedOption())){
						// only stop event on prev/next
						if(highlighted==this._popupWidget.nextButton){
							this._nextSearch(1);
							dojo.stopEvent(evt);
							break;
						}
						else if(highlighted==this._popupWidget.previousButton){
							this._nextSearch(-1);
							dojo.stopEvent(evt);
							break;
						}
					}else{
						this.setDisplayedValue(this.getDisplayedValue());
					}
					// default case:
					// prevent submit, but allow event to bubble
					evt.preventDefault();
					// fall through

				case dojo.keys.TAB:
					var newvalue=this.getDisplayedValue();
					// #4617: if the user had More Choices selected fall into the _onBlur handler
					if(this._popupWidget &&
						(newvalue == this._popupWidget._messages["previousMessage"] ||
							newvalue == this._popupWidget._messages["nextMessage"])){
						break;
					}
					if(this._isShowingNow){
						this._prev_key_backspace = false;
						this._prev_key_esc = false;
						if(this._popupWidget.getHighlightedOption()){
							this._popupWidget.setValue({target:this._popupWidget.getHighlightedOption()}, true);
						}
						this._hideResultList();
					}
					break;

				case dojo.keys.SPACE:
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					if(this._isShowingNow && this._popupWidget.getHighlightedOption()){
						dojo.stopEvent(evt);
						this._selectOption();
						this._hideResultList();
					}else{
						doSearch = true;
					}
					break;

				case dojo.keys.ESCAPE:
					this._prev_key_backspace = false;
					this._prev_key_esc = true;
					this._hideResultList();
					if(this._lastDisplayedValue != this.getDisplayedValue()){
						this.setDisplayedValue(this._lastDisplayedValue);
						dojo.stopEvent(evt);
					}else{
						this.setValue(this.getValue(), false);
					}
					break;

				case dojo.keys.DELETE:
				case dojo.keys.BACKSPACE:
					this._prev_key_esc = false;
					this._prev_key_backspace = true;
					doSearch = true;
					break;

				case dojo.keys.RIGHT_ARROW: // fall through

				case dojo.keys.LEFT_ARROW: // fall through
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					break;

				default:// non char keys (F1-F12 etc..)  shouldn't open list
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					if(dojo.isIE || evt.charCode != 0){
						doSearch=true;
					}
			}
			if(this.searchTimer){
				clearTimeout(this.searchTimer);
			}
			if(doSearch){
				// need to wait a tad before start search so that the event bubbles through DOM and we have value visible
				this.searchTimer = setTimeout(dojo.hitch(this, this._startSearchFromInput), this.searchDelay);
			}
		},

		_autoCompleteText: function(/*String*/ text){
			// summary:
			// Fill in the textbox with the first item from the drop down list, and
			// highlight the characters that were auto-completed.   For example, if user
			// typed "CA" and the drop down list appeared, the textbox would be changed to
			// "California" and "ifornia" would be highlighted.

			// IE7: clear selection so next highlight works all the time
			this._setSelectedRange(this.focusNode, this.focusNode.value.length, this.focusNode.value.length);
			// does text autoComplete the value in the textbox?
			// #3744: escape regexp so the user's input isn't treated as a regular expression.
			// Example: If the user typed "(" then the regexp would throw "unterminated parenthetical."
			// Also see #2558 for the autocompletion bug this regular expression fixes.
			if(new RegExp("^"+escape(this.focusNode.value), this.ignoreCase ? "i" : "").test(escape(text))){
				var cpos = this._getCaretPos(this.focusNode);
				// only try to extend if we added the last character at the end of the input
				if((cpos+1) > this.focusNode.value.length){
					// only add to input node as we would overwrite Capitalisation of chars
					// actually, that is ok
					this.focusNode.value = text;//.substr(cpos);
					// visually highlight the autocompleted characters
					this._setSelectedRange(this.focusNode, cpos, this.focusNode.value.length);
					dijit.setWaiState(this.focusNode, "valuenow", text);
				}
			}else{
				// text does not autoComplete; replace the whole value and highlight
				this.focusNode.value = text;
				this._setSelectedRange(this.focusNode, 0, this.focusNode.value.length);
				dijit.setWaiState(this.focusNode, "valuenow", text);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ dataObject){
			if(this.disabled || dataObject.query[this.searchAttr] != this._lastQuery){
				return;
			}
			this._popupWidget.clearResultList();
			if(!results.length){
				this._hideResultList();
				return;
			}

			// Fill in the textbox with the first item from the drop down list, and
			// highlight the characters that were auto-completed.   For example, if user
			// typed "CA" and the drop down list appeared, the textbox would be changed to
			// "California" and "ifornia" would be highlighted.

			var zerothvalue=new String(this.store.getValue(results[0], this.searchAttr));
			if(zerothvalue && this.autoComplete && !this._prev_key_backspace &&
			// when the user clicks the arrow button to show the full list,
			// startSearch looks for "*".
			// it does not make sense to autocomplete
			// if they are just previewing the options available.
				(dataObject.query[this.searchAttr] != "*")){
				this._autoCompleteText(zerothvalue);
				// announce the autocompleted value
				dijit.setWaiState(this.focusNode || this.domNode, "valuenow", zerothvalue);
			}
			this._popupWidget.createOptions(results, dataObject, dojo.hitch(this, this._getMenuLabelFromItem));

			// show our list (only if we have content, else nothing)
			this._showResultList();

			// #4091: tell the screen reader that the paging callback finished by shouting the next choice
			if(dataObject.direction){
				if(dataObject.direction==1){
					this._popupWidget.highlightFirstOption();
				}else if(dataObject.direction==-1){
					this._popupWidget.highlightLastOption();
				}
				this._announceOption(this._popupWidget.getHighlightedOption());
			}
		},

		_showResultList: function(){
			this._hideResultList();
			var items = this._popupWidget.getItems(),
				visibleCount = Math.min(items.length,this.maxListLength);
			this._arrowPressed();
			// hide the tooltip
			this._displayMessage("");
			
			// Position the list and if it's too big to fit on the screen then
			// size it to the maximum possible height
			// Our dear friend IE doesnt take max-height so we need to calculate that on our own every time
			// TODO: want to redo this, see http://trac.dojotoolkit.org/ticket/3272, http://trac.dojotoolkit.org/ticket/4108
			with(this._popupWidget.domNode.style){
				// natural size of the list has changed, so erase old width/height settings,
				// which were hardcoded in a previous call to this function (via dojo.marginBox() call) 
				width="";
				height="";
			}
			var best=this.open();
			// #3212: only set auto scroll bars if necessary
			// prevents issues with scroll bars appearing when they shouldn't when node is made wider (fractional pixels cause this)
			var popupbox=dojo.marginBox(this._popupWidget.domNode);
			this._popupWidget.domNode.style.overflow=((best.h==popupbox.h)&&(best.w==popupbox.w))?"hidden":"auto";
			// #4134: borrow TextArea scrollbar test so content isn't covered by scrollbar and horizontal scrollbar doesn't appear
			var newwidth=best.w;
			if(best.h<this._popupWidget.domNode.scrollHeight){newwidth+=16;}
			dojo.marginBox(this._popupWidget.domNode, {h:best.h,w:Math.max(newwidth,this.domNode.offsetWidth)});
		},

		_hideResultList: function(){
			if(this._isShowingNow){
				dijit.popup.close(this._popupWidget);
				this._arrowIdle();
				this._isShowingNow=false;
			}
		},

		_onBlur: function(){
			// summary: called magically when focus has shifted away from this widget and it's dropdown
			this._hasFocus=false;
			this._hasBeenBlurred = true;
			this._hideResultList();
			this._arrowIdle();
			// if the user clicks away from the textbox OR tabs away, set the value to the textbox value
			// #4617: if value is now more choices or previous choices, revert the value
			var newvalue=this.getDisplayedValue();
			if(this._popupWidget&&(newvalue==this._popupWidget._messages["previousMessage"]||newvalue==this._popupWidget._messages["nextMessage"])){
				this.setValue(this._lastValueReported, true);
			}else{
				this.setDisplayedValue(newvalue);
			}
		},

		onfocus:function(/*Event*/ evt){
			this._hasFocus=true;
			
			// update styling to reflect that we are focused
			this._onMouse(evt);
		},

		_announceOption: function(/*Node*/ node){
			// summary:
			//	a11y code that puts the highlighted option in the textbox
			//	This way screen readers will know what is happening in the menu

			if(node==null){return;}
			// pull the text value from the item attached to the DOM node
			var newValue;
			if(node==this._popupWidget.nextButton||node==this._popupWidget.previousButton){
				newValue=node.innerHTML;
			}else{
				newValue=this.store.getValue(node.item, this.searchAttr);
			}
			// get the text that the user manually entered (cut off autocompleted text)
			this.focusNode.value=this.focusNode.value.substring(0, this._getCaretPos(this.focusNode));
			// autocomplete the rest of the option to announce change
			this._autoCompleteText(newValue);
		},

		_selectOption: function(/*Event*/ evt){
			var tgt = null;
			if(!evt){
				evt ={ target: this._popupWidget.getHighlightedOption()};
			}
				// what if nothing is highlighted yet?
			if(!evt.target){
				// handle autocompletion where the the user has hit ENTER or TAB
				this.setDisplayedValue(this.getDisplayedValue());
				return;
			// otherwise the user has accepted the autocompleted value
			}else{
				tgt = evt.target;
			}
			if(!evt.noHide){
				this._hideResultList();
				this._setCaretPos(this.focusNode, this.store.getValue(tgt.item, this.searchAttr).length);
			}
			this._doSelect(tgt);
		},

		_doSelect: function(tgt){
			this.item = tgt.item;
			this.setValue(this.store.getValue(tgt.item, this.searchAttr), true);
		},

		_onArrowMouseDown: function(evt){
			// summary: callback when arrow is clicked
			if(this.disabled){
				return;
			}
			dojo.stopEvent(evt);
			this.focus();
			if(this._isShowingNow){
				this._hideResultList();
			}else{
				// forces full population of results, if they click
				// on the arrow it means they want to see more options
				this._startSearch("");
			}
		},

		_startSearchFromInput: function(){
			this._startSearch(this.focusNode.value);
		},

		_startSearch: function(/*String*/ key){
			if(!this._popupWidget){
				this._popupWidget = new dijit.form._ComboBoxMenu({
					onChange: dojo.hitch(this, this._selectOption)
				});
			}
			// create a new query to prevent accidentally querying for a hidden value from FilteringSelect's keyField
			var query=this.query;
			this._lastQuery=query[this.searchAttr]=key+"*";
			var dataObject=this.store.fetch({queryOptions:{ignoreCase:this.ignoreCase, deep:true}, query: query, onComplete:dojo.hitch(this, "_openResultList"), start:0, count:this.pageSize});
			function nextSearch(dataObject, direction){
				dataObject.start+=dataObject.count*direction;
				// #4091: tell callback the direction of the paging so the screen reader knows which menu option to shout
				dataObject.direction=direction;
				dataObject.store.fetch(dataObject);
			}
			this._nextSearch=this._popupWidget.onPage=dojo.hitch(this, nextSearch, dataObject);
		},

		_getValueField:function(){
			return this.searchAttr;
		},

		/////////////// Event handlers /////////////////////

		_arrowPressed: function(){
			if(!this.disabled&&this.hasDownArrow){
				dojo.addClass(this.downArrowNode, "dijitArrowButtonActive");
			}
		},

		_arrowIdle: function(){
			if(!this.disabled&&this.hasDownArrow){
				dojo.removeClass(this.downArrowNode, "dojoArrowButtonPushed");
			}
		},

		compositionend: function(/*Event*/ evt){
			// summary: When inputting characters using an input method, such as Asian
			// languages, it will generate this event instead of onKeyDown event
			// Note: this event is only triggered in FF (not in IE)
			this.onkeypress({charCode:-1});
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////
		constructor: function(){
			this.query={};
		},

		postMixInProperties: function(){
			if(!this.hasDownArrow){
				this.baseClass = "dijitTextBox";
			}
			if(!this.store){
				// if user didn't specify store, then assume there are option tags
				var items = this.srcNodeRef ? dojo.query("> option", this.srcNodeRef).map(function(node){
					node.style.display="none";
					return { value: node.getAttribute("value"), name: String(node.innerHTML) };
				}) : {};
				this.store = new dojo.data.ItemFileReadStore({data: {identifier:this._getValueField(), items:items}});

				// if there is no value set and there is an option list,
				// set the value to the first value to be consistent with native Select
				if(items && items.length && !this.value){
					// For <select>, IE does not let you set the value attribute of the srcNodeRef (and thus dojo.mixin does not copy it).
					// IE does understand selectedIndex though, which is automatically set by the selected attribute of an option tag
					this.value = items[this.srcNodeRef.selectedIndex != -1 ? this.srcNodeRef.selectedIndex : 0]
						[this._getValueField()];
				}
			}
		},

		uninitialize:function(){
			if(this._popupWidget){
				this._hideResultList();
				this._popupWidget.destroy()
			};
		},

		_getMenuLabelFromItem:function(/*Item*/ item){
			return {html:false, label:this.store.getValue(item, this.searchAttr)};
		},

		open:function(){
			this._isShowingNow=true;
			return dijit.popup.open({
				popup: this._popupWidget,
				around: this.domNode,
				parent: this
			});
		}
	}
);

dojo.declare(
	"dijit.form._ComboBoxMenu",
	[dijit._Widget, dijit._Templated],

	{
		// summary:
		//	Focus-less div based menu for internal use in ComboBox

		templateString:"<div class='dijitMenu' dojoAttachEvent='onmousedown,onmouseup,onmouseover,onmouseout' tabIndex='-1' style='overflow:\"auto\";'>"
				+"<div class='dijitMenuItem dijitMenuPreviousButton' dojoAttachPoint='previousButton'></div>"
				+"<div class='dijitMenuItem dijitMenuNextButton' dojoAttachPoint='nextButton'></div>"
			+"</div>",
		_messages:null,

		postMixInProperties:function(){
			this._messages = dojo.i18n.getLocalization("dijit.form", "ComboBox", this.lang);
			this.inherited("postMixInProperties", arguments);
		},

		setValue:function(/*Object*/ value){
			this.value=value;
			this.onChange(value);
		},

		onChange:function(/*Object*/ value){},
		onPage:function(/*Number*/ direction){},

		postCreate:function(){
			// fill in template with i18n messages
			this.previousButton.innerHTML=this._messages["previousMessage"];
			this.nextButton.innerHTML=this._messages["nextMessage"];
			this.inherited("postCreate", arguments);
		},

		onClose:function(){
			this._blurOptionNode();
		},

		_createOption:function(/*Object*/ item, labelFunc){
			// summary: creates an option to appear on the popup menu
			// subclassed by FilteringSelect

			var labelObject=labelFunc(item);
			var menuitem = document.createElement("div");
			if(labelObject.html){menuitem.innerHTML=labelObject.label;}
			else{menuitem.appendChild(document.createTextNode(labelObject.label));}
			// #3250: in blank options, assign a normal height
			if(menuitem.innerHTML==""){
				menuitem.innerHTML="&nbsp;"
			}
			menuitem.item=item;
			return menuitem;
		},

		createOptions:function(results, dataObject, labelFunc){
			//this._dataObject=dataObject;
			//this._dataObject.onComplete=dojo.hitch(comboBox, comboBox._openResultList);
			// display "Previous . . ." button
			this.previousButton.style.display=dataObject.start==0?"none":"";
			// create options using _createOption function defined by parent ComboBox (or FilteringSelect) class
			// #2309: iterate over cache nondestructively
			var _this=this;
			dojo.forEach(results, function(item){
				var menuitem=_this._createOption(item, labelFunc);
				menuitem.className = "dijitMenuItem";
				_this.domNode.insertBefore(menuitem, _this.nextButton);
			});
			// display "Next . . ." button
			this.nextButton.style.display=dataObject.count==results.length?"":"none";
		},

		clearResultList:function(){
			// keep the previous and next buttons of course
			while(this.domNode.childNodes.length>2){
				this.domNode.removeChild(this.domNode.childNodes[this.domNode.childNodes.length-2]);
			}
		},

		// these functions are called in showResultList
		getItems:function(){
			return this.domNode.childNodes;
		},

		getListLength:function(){
			return this.domNode.childNodes.length-2;
		},

		onmousedown:function(/*Event*/ evt){
			dojo.stopEvent(evt);
		},

		onmouseup:function(/*Event*/ evt){
			if(evt.target === this.domNode){
				return;
			}else if(evt.target==this.previousButton){
				this.onPage(-1);
			}else if(evt.target==this.nextButton){
				this.onPage(1);
			}else{
				var tgt=evt.target;
				// while the clicked node is inside the div
				while(!tgt.item){
					// recurse to the top
					tgt=tgt.parentNode;
				}
				this.setValue({target:tgt}, true);
			}
		},

		onmouseover:function(/*Event*/ evt){
			if(evt.target === this.domNode){ return; }
			var tgt=evt.target;
			if(!(tgt==this.previousButton||tgt==this.nextButton)){
				// while the clicked node is inside the div
				while(!tgt.item){
					// recurse to the top
					tgt=tgt.parentNode;
				}
			}
			this._focusOptionNode(tgt);
		},

		onmouseout:function(/*Event*/ evt){
			if(evt.target === this.domNode){ return; }
			this._blurOptionNode();
		},

		_focusOptionNode:function(/*DomNode*/ node){
			// summary:
			//	does the actual highlight
			if(this._highlighted_option != node){
				this._blurOptionNode();
				this._highlighted_option = node;
				dojo.addClass(this._highlighted_option, "dijitMenuItemHover");
			}
		},

		_blurOptionNode:function(){
			// summary:
			//	removes highlight on highlighted option
			if(this._highlighted_option){
				dojo.removeClass(this._highlighted_option, "dijitMenuItemHover");
				this._highlighted_option = null;
			}
		},

		_highlightNextOption:function(){
			// because each press of a button clears the menu,
			// the highlighted option sometimes becomes detached from the menu!
			// test to see if the option has a parent to see if this is the case.
			if(!this.getHighlightedOption()){
				this._focusOptionNode(this.domNode.firstChild.style.display=="none"?this.domNode.firstChild.nextSibling:this.domNode.firstChild);
			}else if(this._highlighted_option.nextSibling&&this._highlighted_option.nextSibling.style.display!="none"){
				this._focusOptionNode(this._highlighted_option.nextSibling);
			}
			// scrollIntoView is called outside of _focusOptionNode because in IE putting it inside causes the menu to scroll up on mouseover
			dijit.scrollIntoView(this._highlighted_option);
		},

		highlightFirstOption:function(){
			// highlight the non-Previous choices option
			this._focusOptionNode(this.domNode.firstChild.nextSibling);
			dijit.scrollIntoView(this._highlighted_option);
		},

		highlightLastOption:function(){
			// highlight the noon-More choices option
			this._focusOptionNode(this.domNode.lastChild.previousSibling);
			dijit.scrollIntoView(this._highlighted_option);
		},

		_highlightPrevOption:function(){
			// if nothing selected, highlight last option
			// makes sense if you select Previous and try to keep scrolling up the list
			if(!this.getHighlightedOption()){
				this._focusOptionNode(this.domNode.lastChild.style.display=="none"?this.domNode.lastChild.previousSibling:this.domNode.lastChild);
			}else if(this._highlighted_option.previousSibling&&this._highlighted_option.previousSibling.style.display!="none"){
				this._focusOptionNode(this._highlighted_option.previousSibling);
			}
			dijit.scrollIntoView(this._highlighted_option);
		},

		_page:function(/*Boolean*/ up){
			var scrollamount=0;
			var oldscroll=this.domNode.scrollTop;
			var height=parseInt(dojo.getComputedStyle(this.domNode).height);
			// if no item is highlighted, highlight the first option
			if(!this.getHighlightedOption()){this._highlightNextOption();}
			while(scrollamount<height){
				if(up){
					// stop at option 1
					if(!this.getHighlightedOption().previousSibling||this._highlighted_option.previousSibling.style.display=="none"){break;}
					this._highlightPrevOption();
				}else{
					// stop at last option
					if(!this.getHighlightedOption().nextSibling||this._highlighted_option.nextSibling.style.display=="none"){break;}
					this._highlightNextOption();
				}
				// going backwards
				var newscroll=this.domNode.scrollTop;
				scrollamount+=(newscroll-oldscroll)*(up ? -1:1);
				oldscroll=newscroll;
			}
		},

		pageUp:function(){
			this._page(true);
		},

		pageDown:function(){
			this._page(false);
		},

		getHighlightedOption:function(){
			// summary:
			//	Returns the highlighted option.
			return this._highlighted_option&&this._highlighted_option.parentNode ? this._highlighted_option : null;
		},

		handleKey:function(evt){
			switch(evt.keyCode){
				case dojo.keys.DOWN_ARROW:
					this._highlightNextOption();
					break;
				case dojo.keys.PAGE_DOWN:
					this.pageDown();
					break;	
				case dojo.keys.UP_ARROW:
					this._highlightPrevOption();
					break;
				case dojo.keys.PAGE_UP:
					this.pageUp();
					break;	
			}
		}
	}
);

dojo.declare(
	"dijit.form.ComboBox",
	[dijit.form.ValidationTextBox, dijit.form.ComboBoxMixin],
	{
		postMixInProperties: function(){
			dijit.form.ComboBoxMixin.prototype.postMixInProperties.apply(this, arguments);
			dijit.form.ValidationTextBox.prototype.postMixInProperties.apply(this, arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit.Menu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Menu"] = true;
dojo.provide("dijit.Menu");





dojo.declare(
	"dijit.Menu",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	constructor: function() {
		this._bindings = [];
	},

	templateString:
			'<table class="dijit dijitMenu dijitReset dijitMenuTable" waiRole="menu" dojoAttachEvent="onkeypress:_onKeyPress">' +
				'<tbody class="dijitReset" dojoAttachPoint="containerNode"></tbody>'+
			'</table>',

	// targetNodeIds: String[]
	//	Array of dom node ids of nodes to attach to.
	//	Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
	targetNodeIds: [],

	// contextMenuForWindow: Boolean
	//	if true, right clicking anywhere on the window will cause this context menu to open;
	//	if false, must specify targetNodeIds
	contextMenuForWindow: false,

	// parentMenu: Widget
	// pointer to menu that displayed me
	parentMenu: null,

	// popupDelay: Integer
	//	number of milliseconds before hovering (without clicking) causes the popup to automatically open
	popupDelay: 500,

	// _contextMenuWithMouse: Boolean
	//	used to record mouse and keyboard events to determine if a context
	//	menu is being opened with the keyboard or the mouse
	_contextMenuWithMouse: false,

	postCreate: function(){
		if(this.contextMenuForWindow){
			this.bindDomNode(dojo.body());
		}else{
			dojo.forEach(this.targetNodeIds, this.bindDomNode, this);
		}
		this.connectKeyNavHandlers([dojo.keys.UP_ARROW], [dojo.keys.DOWN_ARROW]);
	},

	startup: function(){
		dojo.forEach(this.getChildren(), function(child){ child.startup(); });
		this.startupKeyNavChildren();
	},

	onExecute: function(){
		// summary: attach point for notification about when a menu item has been executed
	},

	onCancel: function(/*Boolean*/ closeAll){
		// summary: attach point for notification about when the user cancels the current menu
	},

	_moveToPopup: function(/*Event*/ evt){
		if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
			this.focusedChild._onClick(evt);
		}
	},

	_onKeyPress: function(/*Event*/ evt){
		// summary
		//	Handle keyboard based menu navigation.
		if(evt.ctrlKey || evt.altKey){ return; }

		switch(evt.keyCode){
			case dojo.keys.RIGHT_ARROW:
				this._moveToPopup(evt);
				dojo.stopEvent(evt);
				break;
			case dojo.keys.LEFT_ARROW:
				if(this.parentMenu){
					this.onCancel(false);
				}else{
					dojo.stopEvent(evt);
				}
				break;
		}
	},

	onItemHover: function(/*MenuItem*/ item){
		this.focusChild(item);

		if(this.focusedChild.popup && !this.focusedChild.disabled && !this.hover_timer){
			this.hover_timer = setTimeout(dojo.hitch(this, "_openPopup"), this.popupDelay);
		}
	},

	_onChildBlur: function(item){
		// Close all popups that are open and descendants of this menu
		dijit.popup.close(item.popup);
		item._blur();
		this._stopPopupTimer();
	},

	onItemUnhover: function(/*MenuItem*/ item){
	},

	_stopPopupTimer: function(){
		if(this.hover_timer){
			clearTimeout(this.hover_timer);
			this.hover_timer = null;
		}
	},

	_getTopMenu: function(){
		for(var top=this; top.parentMenu; top=top.parentMenu);
		return top;
	},

	onItemClick: function(/*Widget*/ item){
		// summary: user defined function to handle clicks on an item
		// summary: internal function for clicks
		if(item.disabled){ return false; }

		if(item.popup){
			if(!this.is_open){
				this._openPopup();
			}
		}else{
			// before calling user defined handler, close hierarchy of menus
			// and restore focus to place it was when menu was opened
			this.onExecute();

			// user defined handler for click
			item.onClick();
		}
	},

	// thanks burstlib!
	_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el) {
		//	summary
		//	returns the window reference of the passed iframe
		var win = dijit.getDocumentWindow(dijit.Menu._iframeContentDocument(iframe_el)) ||
			// Moz. TODO: is this available when defaultView isn't?
			dijit.Menu._iframeContentDocument(iframe_el)['__parent__'] ||
			(iframe_el.name && document.frames[iframe_el.name]) || null;
		return win;	//	Window
	},

	_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
		//	summary
		//	returns a reference to the document object inside iframe_el
		var doc = iframe_el.contentDocument // W3
			|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
			|| (iframe_el.name && document.frames[iframe_el.name] && document.frames[iframe_el.name].document)
			|| null;
		return doc;	//	HTMLDocument
	},

	bindDomNode: function(/*String|DomNode*/ node){
		// summary: attach menu to given node
		node = dojo.byId(node);

		//TODO: this is to support context popups in Editor.  Maybe this shouldn't be in dijit.Menu
		var win = dijit.getDocumentWindow(node.ownerDocument);
		if(node.tagName.toLowerCase()=="iframe"){
			win = this._iframeContentWindow(node);
			node = dojo.withGlobal(win, dojo.body);
		}

		// to capture these events at the top level,
		// attach to document, not body
		var cn = (node == dojo.body() ? dojo.doc : node);

		node[this.id] = this._bindings.push([
			dojo.connect(cn, "oncontextmenu", this, "_openMyself"),
			dojo.connect(cn, "onkeydown", this, "_contextKey"),
			dojo.connect(cn, "onmousedown", this, "_contextMouse")
		]);
	},

	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary: detach menu from given node
		var node = dojo.byId(nodeName);
		var bid = node[this.id]-1, b = this._bindings[bid];
		dojo.forEach(b, dojo.disconnect);
		delete this._bindings[bid];
	},

	_contextKey: function(e){
		this._contextMenuWithMouse = false;
		if (e.keyCode == dojo.keys.F10) {
			dojo.stopEvent(e);
			if (e.shiftKey && e.type=="keydown") {
				// FF: copying the wrong property from e will cause the system
				// context menu to appear in spite of stopEvent. Don't know
				// exactly which properties cause this effect.
				var _e = { target: e.target, pageX: e.pageX, pageY: e.pageY };
				_e.preventDefault = _e.stopPropagation = function(){};
				// IE: without the delay, focus work in "open" causes the system
				// context menu to appear in spite of stopEvent.
				window.setTimeout(dojo.hitch(this, function(){ this._openMyself(_e); }), 1);
			}
		}
	},

	_contextMouse: function(e){
		this._contextMenuWithMouse = true;
	},

	_openMyself: function(/*Event*/ e){
		// summary:
		//		Internal function for opening myself when the user
		//		does a right-click or something similar

		dojo.stopEvent(e);

		// Get coordinates.
		// if we are opening the menu with the mouse or on safari open
		// the menu at the mouse cursor
		// (Safari does not have a keyboard command to open the context menu
		// and we don't currently have a reliable way to determine
		// _contextMenuWithMouse on Safari)
		var x,y;
		if(dojo.isSafari || this._contextMenuWithMouse){
			x=e.pageX;
			y=e.pageY;
		}else{
			// otherwise open near e.target
			var coords = dojo.coords(e.target, true);
			x = coords.x + 10;
			y = coords.y + 10;
		}

		var self=this;
		var savedFocus = dijit.getFocus(this);
		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			dijit.focus(savedFocus);
			dijit.popup.close(self);
		}
		dijit.popup.open({
			popup: this,
			x: x,
			y: y,
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			dijit.popup.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		}
	},

	onOpen: function(/*Event*/ e){
		// summary
		//		Open menu relative to the mouse
		this.isShowingNow = true;
	},

	onClose: function(){
		// summary: callback when this menu is closed
		this._stopPopupTimer();
		this.parentMenu = null;
		this.isShowingNow = false;
		this.currentPopup = null;
		if(this.focusedChild){
			this._onChildBlur(this.focusedChild);
			this.focusedChild = null;
		}
	},

	_openPopup: function(){
		// summary: open the popup to the side of the current menu item
		this._stopPopupTimer();
		var from_item = this.focusedChild;
		var popup = from_item.popup;

		if(popup.isShowingNow){ return; }
		popup.parentMenu = this;
		var self = this;
		dijit.popup.open({
			parent: this,
			popup: popup,
			around: from_item.arrowCell,
			orient: this.isLeftToRight() ? {'TR': 'TL', 'TL': 'TR'} : {'TL': 'TR', 'TR': 'TL'},
			onCancel: function(){
				// called when the child menu is canceled
				dijit.popup.close(popup);
				from_item.focus();	// put focus back on my node
				self.currentPopup = null;
			}
		});

		this.currentPopup = popup;

		if(popup.focus){
			popup.focus();
		}
	}
}
);

dojo.declare(
	"dijit.MenuItem",
	[dijit._Widget, dijit._Templated, dijit._Contained],
{
	// summary
	//	A line item in a Menu2

	// Make 3 columns
	//   icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
	templateString:
		 '<tr class="dijitReset dijitMenuItem"'
		+'dojoAttachEvent="onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick">'
		+'<td class="dijitReset"><div class="dijitMenuItemIcon ${iconClass}" dojoAttachPoint="iconNode" ></div></td>'
		+'<td tabIndex="-1" class="dijitReset dijitMenuItemLabel" dojoAttachPoint="containerNode" waiRole="menuitem"></td>'
		+'<td class="dijitReset" dojoAttachPoint="arrowCell">'
			+'<div class="dijitMenuExpand" dojoAttachPoint="expand" style="display:none">'
			+'<span class="dijitInline dijitArrowNode dijitMenuExpandInner">+</span>'
			+'</div>'
		+'</td>'
		+'</tr>',

	// label: String
	//	menu text
	label: '',

	// iconClass: String
	//	class to apply to div in button to make it display an icon
	iconClass: "",

	// disabled: Boolean
	//  if true, the menu item is disabled
	//  if false, the menu item is enabled
	disabled: false,

	postCreate: function(){
		dojo.setSelectable(this.domNode, false);
		this.setDisabled(this.disabled);
		if(this.label){
			this.containerNode.innerHTML=this.label;
		}
	},

	_onHover: function(){
		// summary: callback when mouse is moved onto menu item
		this.getParent().onItemHover(this);
	},

	_onUnhover: function(){
		// summary: callback when mouse is moved off of menu item
		// if we are unhovering the currently selected item
		// then unselect it
		this.getParent().onItemUnhover(this);
	},

	_onClick: function(evt){
		this.getParent().onItemClick(this);
		dojo.stopEvent(evt);
	},

	onClick: function() {
		// summary
		//	User defined function to handle clicks
	},

	focus: function(){
		dojo.addClass(this.domNode, 'dijitMenuItemHover');
		try{
			dijit.focus(this.containerNode);
		}catch(e){
			// this throws on IE (at least) in some scenarios
		}
	},

	_blur: function(){
		dojo.removeClass(this.domNode, 'dijitMenuItemHover');
	},

	setDisabled: function(/*Boolean*/ value){
		// summary: enable or disable this menu item
		this.disabled = value;
		dojo[value ? "addClass" : "removeClass"](this.domNode, 'dijitMenuItemDisabled');
		dijit.setWaiState(this.containerNode, 'disabled', value ? 'true' : 'false');
	}
});

dojo.declare(
	"dijit.PopupMenuItem",
	dijit.MenuItem,
{
	_fillContent: function(){
		// my inner HTML contains both the menu item text and a popup widget, like
		// <div dojoType="dijit.PopupMenuItem">
		//		<span>pick me</span>
		//		<popup> ... </popup>
		// </div>
		// the first part holds the menu item text and the second part is the popup
		if(this.srcNodeRef){
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.PopupMenuItem.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		// we didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
		// land now.  move it to document.body.
		if(!this.popup){
			var node = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.popup = dijit.byNode(node);
		}
		dojo.body().appendChild(this.popup.domNode);

		this.popup.domNode.style.display="none";
		dojo.addClass(this.expand, "dijitMenuExpandEnabled");
		dojo.style(this.expand, "display", "");
		dijit.setWaiState(this.containerNode, "haspopup", "true");
	}
});

dojo.declare(
	"dijit.MenuSeparator",
	[dijit._Widget, dijit._Templated, dijit._Contained],
{
	// summary
	//	A line between two menu items

	templateString: '<tr class="dijitMenuSeparator"><td colspan=3>'
			+'<div class="dijitMenuSeparatorTop"></div>'
			+'<div class="dijitMenuSeparatorBottom"></div>'
			+'</td></tr>',

	postCreate: function(){
		dojo.setSelectable(this.domNode, false);
	},
	
	isFocusable: function(){
		// summary:
		//		over ride to always return false
		return false;
	}
});

}

if(!dojo._hasResource["dijit.Tree"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Tree"] = true;
dojo.provide("dijit.Tree");








dojo.declare(
	"dijit._TreeNode",
	[dijit._Widget, dijit._Templated, dijit._Container, dijit._Contained],
{
	// summary
	//		Single node within a tree

	// item: dojo.data.Item
	//		the dojo.data entry this tree represents
	item: null,	

	isTreeNode: true,

	// label: String
	//		Text of this tree node
	label: "",
	
	isExpandable: null, // show expando node
	
	isExpanded: false,

	// state: String
	//		dynamic loading-related stuff.
	//		When an empty folder node appears, it is "UNCHECKED" first,
	//		then after dojo.data query it becomes "LOADING" and, finally "LOADED"	
	state: "UNCHECKED",
	
	templateString:"<div class=\"dijitTreeNode dijitTreeExpandLeaf dijitTreeChildrenNo\" waiRole=\"presentation\"\r\n\t><span dojoAttachPoint=\"expandoNode\" class=\"dijitTreeExpando\" waiRole=\"presentation\"\r\n\t></span\r\n\t><span dojoAttachPoint=\"expandoNodeText\" class=\"dijitExpandoText\" waiRole=\"presentation\"\r\n\t></span\r\n\t>\r\n\t<div dojoAttachPoint=\"contentNode\" class=\"dijitTreeContent\" waiRole=\"presentation\">\r\n\t\t<div dojoAttachPoint=\"iconNode\" class=\"dijitInline dijitTreeIcon\" waiRole=\"presentation\"></div>\r\n\t\t<span dojoAttachPoint=\"labelNode\" class=\"dijitTreeLabel\" wairole=\"treeitem\" tabindex=\"-1\"></span>\r\n\t</div>\r\n</div>\r\n",		

	postCreate: function(){
		// set label, escaping special characters
		this.setLabelNode(this.label);

		// set expand icon for leaf 	
		this._setExpando();

		// set icon and label class based on item
		this._updateItemClasses(this.item);

		if(this.isExpandable){
			dijit.setWaiState(this.labelNode, "expanded", this.isExpanded);
		}
	},

	markProcessing: function(){
		// summary: visually denote that tree is loading data, etc.
		this.state = "LOADING";
		this._setExpando(true);	
	},

	unmarkProcessing: function(){
		// summary: clear markup from markProcessing() call
		this._setExpando(false);	
	},

	_updateItemClasses: function(item){
		// summary: set appropriate CSS classes for item (used to allow for item updates to change respective CSS)
		this.iconNode.className = "dijitInline dijitTreeIcon " + this.tree.getIconClass(item);
		this.labelNode.className = "dijitTreeLabel " + this.tree.getLabelClass(item);
	},
	
	_updateLayout: function(){
		// summary: set appropriate CSS classes for this.domNode
		var parent = this.getParent();
		if(parent && parent.isTree && parent._hideRoot){
			/* if we are hiding the root node then make every first level child look like a root node */
			dojo.addClass(this.domNode, "dijitTreeIsRoot");
		}else{
			dojo.toggleClass(this.domNode, "dijitTreeIsLast", !this.getNextSibling());
		}
	},

	_setExpando: function(/*Boolean*/ processing){
		// summary: set the right image for the expando node

		// apply the appropriate class to the expando node
		var styles = ["dijitTreeExpandoLoading", "dijitTreeExpandoOpened",
			"dijitTreeExpandoClosed", "dijitTreeExpandoLeaf"];
		var idx = processing ? 0 : (this.isExpandable ?	(this.isExpanded ? 1 : 2) : 3);
		dojo.forEach(styles,
			function(s){
				dojo.removeClass(this.expandoNode, s);
			}, this
		);
		dojo.addClass(this.expandoNode, styles[idx]);

		// provide a non-image based indicator for images-off mode
		this.expandoNodeText.innerHTML =
			processing ? "*" :
				(this.isExpandable ?
					(this.isExpanded ? "-" : "+") : "*");
	},	

	expand: function(){
		// summary: show my children
		if(this.isExpanded){ return; }
		// cancel in progress collapse operation
		if(this._wipeOut.status() == "playing"){
			this._wipeOut.stop();
		}

		this.isExpanded = true;
		dijit.setWaiState(this.labelNode, "expanded", "true");
		dijit.setWaiRole(this.containerNode, "group");

		this._setExpando();

		this._wipeIn.play();
	},

	collapse: function(){					
		if(!this.isExpanded){ return; }

		// cancel in progress expand operation
		if(this._wipeIn.status() == "playing"){
			this._wipeIn.stop();
		}

		this.isExpanded = false;
		dijit.setWaiState(this.labelNode, "expanded", "false");
		this._setExpando();

		this._wipeOut.play();
	},

	setLabelNode: function(label){
		this.labelNode.innerHTML="";
		this.labelNode.appendChild(document.createTextNode(label));
	},

	_setChildren: function(/* Object[] */ childrenArray){
		// summary:
		//		Sets the children of this node.
		//		Sets this.isExpandable based on whether or not there are children
		// 		Takes array of objects like: {label: ...} (_TreeNode options basically)
		//		See parameters of _TreeNode for details.

		this.destroyDescendants();

		this.state = "LOADED";
		var nodeMap= {};
		if(childrenArray && childrenArray.length > 0){
			this.isExpandable = true;
			if(!this.containerNode){ // maybe this node was unfolderized and still has container
				this.containerNode = this.tree.containerNodeTemplate.cloneNode(true);
				this.domNode.appendChild(this.containerNode);
			}

			// Create _TreeNode widget for each specified tree node
			dojo.forEach(childrenArray, function(childParams){
				var child = new dijit._TreeNode(dojo.mixin({
					tree: this.tree,
					label: this.tree.getLabel(childParams.item)
				}, childParams));
				this.addChild(child);
				var identity = this.tree.store.getIdentity(childParams.item);
				nodeMap[identity] = child;
				if(this.tree.persist){
					if(this.tree._openedItemIds[identity]){
						this.tree._expandNode(child);
					}
				}
			}, this);

			// note that updateLayout() needs to be called on each child after
			// _all_ the children exist
			dojo.forEach(this.getChildren(), function(child, idx){
				child._updateLayout();
			});
		}else{
			this.isExpandable=false;
		}

		if(this._setExpando){
			// change expando to/form dot or + icon, as appropriate
			this._setExpando(false);
		}

		if(this.isTree && this._hideRoot){
			// put first child in tab index if one exists.
			var fc = this.getChildren()[0];
			var tabnode = fc ? fc.labelNode : this.domNode;
			tabnode.setAttribute("tabIndex", "0");
		}

		// create animations for showing/hiding the children (if children exist)
		if(this.containerNode && !this._wipeIn){
			this._wipeIn = dojo.fx.wipeIn({node: this.containerNode, duration: 150});
			this._wipeOut = dojo.fx.wipeOut({node: this.containerNode, duration: 150});
		}

		return nodeMap;
	},

	_addChildren: function(/* object[] */ childrenArray){
		// summary:
		//		adds the children to this node.
		// 		Takes array of objects like: {label: ...}  (_TreeNode options basically)

		//		See parameters of _TreeNode for details.
		var nodeMap = {};
		if(childrenArray && childrenArray.length > 0){
			dojo.forEach(childrenArray, function(childParams){
				var child = new dijit._TreeNode(
					dojo.mixin({
						tree: this.tree,
						label: this.tree.getLabel(childParams.item)
					}, childParams)
				);
				this.addChild(child);
				nodeMap[this.tree.store.getIdentity(childParams.item)] = child;
			}, this);

			dojo.forEach(this.getChildren(), function(child, idx){
				child._updateLayout();
			});
		}

		return nodeMap;
	},

	deleteNode: function(/* treeNode */ node){
		node.destroy();

		var children = this.getChildren();		
		if(children.length == 0){
			this.isExpandable = false;
			this.collapse();
		}

		dojo.forEach(children, function(child){
				child._updateLayout();
		});
	},

	makeExpandable: function(){
		//summary
		//		if this node wasn't already showing the expando node,
		//		turn it into one and call _setExpando()
		this.isExpandable = true;
		this._setExpando(false);
	}
});

dojo.declare(
	"dijit.Tree",
	dijit._TreeNode,
{
	// summary
	//	This widget displays hierarchical data from a store.  A query is specified
	//	to get the "top level children" from a data store, and then those items are
	//	queried for their children and so on (but lazily, as the user clicks the expand node).
	//
	//	Thus in the default mode of operation this widget is technically a forest, not a tree,
	//	in that there can be multiple "top level children".  However, if you specify label,
	//	then a special top level node (not corresponding to any item in the datastore) is
	//	created, to father all the top level children.

	// store: String||dojo.data.Store
	//	The store to get data to display in the tree
	store: null,

	// query: String
	//	query to get top level node(s) of tree (ex: {type:'continent'})
	query: null,

	// childrenAttr: String
	//		one ore more attributes that holds children of a tree node
	childrenAttr: ["children"],

	templateString:"<div class=\"dijitTreeContainer\" style=\"\" waiRole=\"tree\"\r\n\tdojoAttachEvent=\"onclick:_onClick,onkeypress:_onKeyPress\">\r\n\t<div class=\"dijitTreeNode  dijitTreeIsRoot dijitTreeExpandLeaf dijitTreeChildrenNo\" waiRole=\"presentation\"\r\n\t\tdojoAttachPoint=\"rowNode\"\r\n\t\t><span dojoAttachPoint=\"expandoNode\" class=\"dijitTreeExpando\" waiRole=\"presentation\"\r\n\t\t></span\r\n\t\t><span dojoAttachPoint=\"expandoNodeText\" class=\"dijitExpandoText\" waiRole=\"presentation\"\r\n\t\t></span\r\n\t\t>\r\n\t\t<div dojoAttachPoint=\"contentNode\" class=\"dijitTreeContent\" waiRole=\"presentation\">\r\n\t\t\t<div dojoAttachPoint=\"iconNode\" class=\"dijitInline dijitTreeIcon\" waiRole=\"presentation\"></div>\r\n\t\t\t<span dojoAttachPoint=\"labelNode\" class=\"dijitTreeLabel\" wairole=\"treeitem\" tabindex=\"0\"></span>\r\n\t\t</div>\r\n\t</div>\r\n</div>\r\n",		

	isExpandable: true,

	isTree: true,

	// persist: Boolean
	//	enables/disables use of cookies for state saving.
	persist: true,
	
	// dndController: String
	//	class name to use as as the dnd controller
	dndController: null,

	//parameters to pull off of the tree and pass on to the dndController as its params
	dndParams: ["onDndDrop","itemCreator","onDndCancel","checkAcceptance", "checkItemAcceptance"],

	//declare the above items so they can be pulled from the tree's markup
	onDndDrop:null,
	itemCreator:null,
	onDndCancel:null,
	checkAcceptance:null,	
	checkItemAcceptance:null,

	_publish: function(/*String*/ topicName, /*Object*/ message){
		// summary:
		//		Publish a message for this widget/topic
		dojo.publish(this.id, [dojo.mixin({tree: this, event: topicName}, message||{})]);
	},

	postMixInProperties: function(){
		this.tree = this;
		this.lastFocused = this.labelNode;

		this._itemNodeMap={};

		this._hideRoot = !this.label;

		if(!this.store.getFeatures()['dojo.data.api.Identity']){
			throw new Error("dijit.tree requires access to a store supporting the dojo.data Identity api");			
		}

		if(!this.cookieName){
			this.cookieName = this.id + "SaveStateCookie";
		}

		// if the store supports Notification, subscribe to the notification events
		if(this.store.getFeatures()['dojo.data.api.Notification']){
			this.connect(this.store, "onNew", "_onNewItem");
			this.connect(this.store, "onDelete", "_onDeleteItem");
			this.connect(this.store, "onSet", "_onSetItem");
		}
	},

	postCreate: function(){
		// load in which nodes should be opened automatically
		if(this.persist){
			var cookie = dojo.cookie(this.cookieName);
			this._openedItemIds = {};
			if(cookie){
				dojo.forEach(cookie.split(','), function(item){
					this._openedItemIds[item] = true;
				}, this);
			}
		}
		
		// make template for container node (we will clone this and insert it into
		// any nodes that have children)
		var div = document.createElement('div');
		div.style.display = 'none';
		div.className = "dijitTreeContainer";	
		dijit.setWaiRole(div, "presentation");
		this.containerNodeTemplate = div;

		if(this._hideRoot){
			this.rowNode.style.display="none";
		}

		this.inherited("postCreate", arguments);

		// load top level children
		this._expandNode(this);

		if(this.dndController){
			if(dojo.isString(this.dndController)){
				this.dndController= dojo.getObject(this.dndController);
			}	
			var params={};
			for (var i=0; i<this.dndParams.length;i++){
				if(this[this.dndParams[i]]){
					params[this.dndParams[i]]=this[this.dndParams[i]];
				}
			}
			this.dndController= new this.dndController(this, params);
		}

		if (dojo.isIE) {
			this.connect(this.domNode, "onactivate", "_onTreeFocus");
		} else if ((dojo.isFF > 2) || dojo.isSafari) {
			// On FF3beta3, onfocus appears to be broken currently.
			// When it is fixed, this case should be removed
			// Emulate onfocus event
			this.connect(this.domNode, "onclick", "_onTreeFocus");
			this.connect(this.domNode, "onkeyup", "_onTreeFocus");
		} else {
			this.connect(this.domNode, "onfocus", "_onTreeFocus");
		}

	},

	////////////// Data store related functions //////////////////////

	mayHaveChildren: function(/*dojo.data.Item*/ item){
		// summary
		//		User overridable function to tell if an item has or may have children.
		//		Controls whether or not +/- expando icon is shown.
		//		(For efficiency reasons we may not want to check if an element has
		//		children until user clicks the expando node)

		return dojo.some(this.childrenAttr, function(attr){
			return this.store.hasAttribute(item, attr);
		}, this);
	},

	getItemChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ onComplete){
		// summary
		// 		User overridable function that return array of child items of given parent item,
		//		or if parentItem==null then return top items in tree
		var store = this.store;
		if(parentItem == null){
			// get top level nodes
			store.fetch({ query: this.query, onComplete: onComplete});
		}else{
			// get children of specified node
			var childItems = [];
			for (var i=0; i<this.childrenAttr.length; i++){	
				childItems= childItems.concat(store.getValues(parentItem, this.childrenAttr[i]));
			}
			// count how many items need to be loaded
			var _waitCount = 0;
			dojo.forEach(childItems, function(item){ if(!store.isItemLoaded(item)){ _waitCount++; } });

			if(_waitCount == 0){
				// all items are already loaded.  proceed..
				onComplete(childItems);
			}else{
				// still waiting for some or all of the items to load
				function onItem(item){
					if(--_waitCount == 0){
						// all nodes have been loaded, send them to the tree
						onComplete(childItems);
					}
				}
				dojo.forEach(childItems, function(item){
					if(!store.isItemLoaded(item)){
						store.loadItem({item: item, onItem: onItem});
					}
				});
			}
		}
	},

	getItemParentIdentity: function(/*dojo.data.Item*/ item, /*Object*/ parentInfo){
		// summary
		//		User overridable function, to return id of parent (or null if top level).
		//		It's called with args from dojo.store.onNew
		return this.store.getIdentity(parentInfo.item);		// String
	},

	getLabel: function(/*dojo.data.Item*/ item){
		// summary: user overridable function to get the label for a tree node (given the item)
		return this.store.getLabel(item);	// String
	},

	getIconClass: function(/*dojo.data.Item*/ item){
		// summary: user overridable function to return CSS class name to display icon
	},

	getLabelClass: function(/*dojo.data.Item*/ item){
		// summary: user overridable function to return CSS class name to display label
	},

	_onLoadAllItems: function(/*_TreeNode*/ node, /*dojo.data.Item[]*/ items){
		// sumary: callback when all the children of a given node have been loaded
		var childParams=dojo.map(items, function(item){
			return {
				item: item,
				isExpandable: this.mayHaveChildren(item)
			};
		}, this);

		dojo.mixin(this._itemNodeMap,node._setChildren(childParams));

		this._expandNode(node);
	},

	/////////// Keyboard and Mouse handlers ////////////////////

	_onKeyPress: function(/*Event*/ e){
		// summary: translates keypress events into commands for the controller
		if(e.altKey){ return; }
		var treeNode = dijit.getEnclosingWidget(e.target);
		if(!treeNode){ return; }

		// Note: On IE e.keyCode is not 0 for printables so check e.charCode.
		// In dojo charCode is universally 0 for non-printables.
		if(e.charCode){  // handle printables (letter navigation)
			// Check for key navigation.
			var navKey = e.charCode;
			if(!e.altKey && !e.ctrlKey && !e.shiftKey && !e.metaKey){
				navKey = (String.fromCharCode(navKey)).toLowerCase();
				this._onLetterKeyNav( { node: treeNode, key: navKey } );
				dojo.stopEvent(e);
			}
		}else{  // handle non-printables (arrow keys)
			var map = this._keyHandlerMap;
			if(!map){
				// setup table mapping keys to events
				map = {};
				map[dojo.keys.ENTER]="_onEnterKey";
				map[dojo.keys.LEFT_ARROW]="_onLeftArrow";
				map[dojo.keys.RIGHT_ARROW]="_onRightArrow";
				map[dojo.keys.UP_ARROW]="_onUpArrow";
				map[dojo.keys.DOWN_ARROW]="_onDownArrow";
				map[dojo.keys.HOME]="_onHomeKey";
				map[dojo.keys.END]="_onEndKey";
				this._keyHandlerMap = map;
			}
			if(this._keyHandlerMap[e.keyCode]){
				this[this._keyHandlerMap[e.keyCode]]( { node: treeNode, item: treeNode.item } );	
				dojo.stopEvent(e);
			}
		}
	},

	_onEnterKey: function(/*Object*/ message){
		this._publish("execute", { item: message.item, node: message.node} );
		this.onClick(message.item, message.node);
	},

	_onDownArrow: function(/*Object*/ message){
		// summary: down arrow pressed; get next visible node, set focus there
		var returnNode = this._navToNextNode(message.node);
		if(returnNode && returnNode.isTreeNode){
			returnNode.tree.focusNode(returnNode);
			return returnNode;
		}	
	},

	_onUpArrow: function(/*Object*/ message){
		// summary: up arrow pressed; move to previous visible node

		var nodeWidget = message.node;
		var returnWidget = nodeWidget;

		// if younger siblings		
		var previousSibling = nodeWidget.getPreviousSibling();
		if(previousSibling){
			nodeWidget = previousSibling;
			// if the previous nodeWidget is expanded, dive in deep
			while(nodeWidget.isExpandable && nodeWidget.isExpanded && nodeWidget.hasChildren()){
				returnWidget = nodeWidget;
				// move to the last child
				var children = nodeWidget.getChildren();
				nodeWidget = children[children.length-1];
			}
		}else{
			// if this is the first child, return the parent
			// unless the parent is the root of a tree with a hidden root
			var parent = nodeWidget.getParent();
			if(!(this._hideRoot && parent === this)){
				nodeWidget = parent;
			}
		}

		if(nodeWidget && nodeWidget.isTreeNode){
			returnWidget = nodeWidget;
		}

		if(returnWidget && returnWidget.isTreeNode){
			returnWidget.tree.focusNode(returnWidget);
			return returnWidget;
		}
	},

	_onRightArrow: function(/*Object*/ message){
		// summary: right arrow pressed; go to child node
		var nodeWidget = message.node;
		var returnWidget = nodeWidget;

		// if not expanded, expand, else move to 1st child
		if(nodeWidget.isExpandable && !nodeWidget.isExpanded){
			this._expandNode(nodeWidget);
		}else if(nodeWidget.hasChildren()){
			nodeWidget = nodeWidget.getChildren()[0];
		}

		if(nodeWidget && nodeWidget.isTreeNode){
			returnWidget = nodeWidget;
		}

		if(returnWidget && returnWidget.isTreeNode){
			returnWidget.tree.focusNode(returnWidget);
			return returnWidget;
		}
	},

	_onLeftArrow: function(/*Object*/ message){
		// summary: left arrow pressed; go to parent

		var node = message.node;
		var returnWidget = node;

		// if not collapsed, collapse, else move to parent
		if(node.isExpandable && node.isExpanded){
			this._collapseNode(node);
		}else{
			node = node.getParent();
		}
		if(node && node.isTreeNode){
			returnWidget = node;
		}

		if(returnWidget && returnWidget.isTreeNode){
			returnWidget.tree.focusNode(returnWidget);
			return returnWidget;
		}
	},

	_onHomeKey: function(){
		// summary: home pressed; get first visible node, set focus there
		var returnNode = this._navToRootOrFirstNode();
		if(returnNode){
			returnNode.tree.focusNode(returnNode);
			return returnNode;
		}
	},

	_onEndKey: function(/*Object*/ message){
		// summary: end pressed; go to last visible node

		var returnWidget = message.node.tree;

		var lastChild = returnWidget;
		while(lastChild.isExpanded){
			var c = lastChild.getChildren();
			lastChild = c[c.length - 1];
			if(lastChild.isTreeNode){
				returnWidget = lastChild;
			}
		}

		if(returnWidget && returnWidget.isTreeNode){
			returnWidget.tree.focusNode(returnWidget);
			return returnWidget;
		}
	},

	_onLetterKeyNav: function(message){
		// summary: letter key pressed; search for node starting with first char = key
		var node = startNode = message.node;
		var key = message.key;
		do{
			node = this._navToNextNode(node);
			//check for last node, jump to first node if necessary
			if(!node){
				node = this._navToRootOrFirstNode();
			}
		}while(node !== startNode && (node.label.charAt(0).toLowerCase() != key));
		if(node && node.isTreeNode){
			// no need to set focus if back where we started
			if(node !== startNode){
				node.tree.focusNode(node);
			}
			return node;
		}
	},

	_onClick: function(/*Event*/ e){
		// summary: translates click events into commands for the controller to process
		var domElement = e.target;

		// find node
		var nodeWidget = dijit.getEnclosingWidget(domElement);	
		if(!nodeWidget || !nodeWidget.isTreeNode){
			return;
		}

		if(domElement == nodeWidget.expandoNode ||
			 domElement == nodeWidget.expandoNodeText){
			// expando node was clicked
			if(nodeWidget.isExpandable){
				this._onExpandoClick({node:nodeWidget});
			}
		}else{
			this._publish("execute", { item: nodeWidget.item, node: nodeWidget} );
			this.onClick(nodeWidget.item, nodeWidget);
			this.focusNode(nodeWidget);
		}
		dojo.stopEvent(e);
	},

	_onExpandoClick: function(/*Object*/ message){
		// summary: user clicked the +/- icon; expand or collapse my children.
		var node = message.node;
		if(node.isExpanded){
			this._collapseNode(node);
		}else{
			this._expandNode(node);
		}
	},

	onClick: function(/* dojo.data */ item, /*TreeNode*/ node){
		// summary: user overridable function for executing a tree item
	},

	_navToNextNode: function(node){
		// summary: get next visible node
		var returnNode;
		// if this is an expanded node, get the first child
		if(node.isExpandable && node.isExpanded && node.hasChildren()){
			returnNode = node.getChildren()[0];			
		}else{
			// find a parent node with a sibling
			while(node && node.isTreeNode){
				returnNode = node.getNextSibling();
				if(returnNode){
					break;
				}
				node = node.getParent();
			}	
		}
		return returnNode;
	},

	_navToRootOrFirstNode: function(){
		// summary: get first visible node
		if(!this._hideRoot){
			return this;
		}else{
			var returnNode = this.getChildren()[0];
			if(returnNode && returnNode.isTreeNode){
				return returnNode;
			}
		}
	},

	_collapseNode: function(/*_TreeNode*/ node){
		// summary: called when the user has requested to collapse the node

		if(node.isExpandable){
			if(node.state == "LOADING"){
				// ignore clicks while we are in the process of loading data
				return;
			}
			if(this.lastFocused){
				// are we collapsing a descendant with focus?
				if(dojo.isDescendant(this.lastFocused.domNode, node.domNode)){
					this.focusNode(node);
				}else{
					// clicking the expando node might have erased focus from
					// the current item; restore it
					this.focusNode(this.lastFocused);
				}
			}
			node.collapse();
			if(this.persist && node.item){
				delete this._openedItemIds[this.store.getIdentity(node.item)];
				this._saveState();
			}
		}
	},

	_expandNode: function(/*_TreeNode*/ node){
		// summary: called when the user has requested to expand the node

		// clicking the expando node might have erased focus from the current item; restore it
		var t = node.tree;
		if(t.lastFocused){ t.focusNode(t.lastFocused); }

		if(!node.isExpandable){
			return;
		}

		var store = this.store;
		var getValue = this.store.getValue;

		switch(node.state){
			case "LOADING":
				// ignore clicks while we are in the process of loading data
				return;

			case "UNCHECKED":
				// need to load all the children, and then expand
				node.markProcessing();
				var _this = this;
				var onComplete = function(childItems){
					node.unmarkProcessing();
					_this._onLoadAllItems(node, childItems);
				};
				this.getItemChildren(node.item, onComplete);
				break;

			default:
				// data is already loaded; just proceed
				if(node.expand){	// top level Tree doesn't have expand() method
					node.expand();
					if(this.persist && node.item){
						this._openedItemIds[this.store.getIdentity(node.item)] = true;
						this._saveState();
					}
				}
				break;
		}
	},

	////////////////// Miscellaneous functions ////////////////

	blurNode: function(){
		// summary
		//	Removes focus from the currently focused node (which must be visible).
		//	Usually not called directly (just call focusNode() on another node instead)
		var node = this.lastFocused;
		if(!node){ return; }
		var labelNode = node.labelNode;
		dojo.removeClass(labelNode, "dijitTreeLabelFocused");
		labelNode.setAttribute("tabIndex", "-1");
		this.lastFocused = null;
	},

	focusNode: function(/* _tree.Node */ node){
		// summary
		//	Focus on the specified node (which must be visible)

		// set focus so that the label will be voiced using screen readers
		node.labelNode.focus();
	},

	_onBlur: function(){
		// summary:
		// 		We've moved away from the whole tree.  The currently "focused" node
		//		(see focusNode above) should remain as the lastFocused node so we can
		//		tab back into the tree.  Just change CSS to get rid of the dotted border
		//		until that time
		if(this.lastFocused){
			var labelNode = this.lastFocused.labelNode;
			dojo.removeClass(labelNode, "dijitTreeLabelFocused");	
		}
	},

	_onTreeFocus: function(evt){
		var node = dijit.getEnclosingWidget(evt.target);
		if(node != this.lastFocused){
			this.blurNode();
		}
		var labelNode = node.labelNode;
		// set tabIndex so that the tab key can find this node
		labelNode.setAttribute("tabIndex", "0");
		dojo.addClass(labelNode, "dijitTreeLabelFocused");
		this.lastFocused = node;
	},

	//////////////// Events from data store //////////////////////////


	_onNewItem: function(/*Object*/ item, parentInfo){
		//summary: callback when new item has been added to the store.

		var loadNewItem;	// should new item be displayed in tree?

		if(parentInfo){
			var parent = this._itemNodeMap[this.getItemParentIdentity(item, parentInfo)];
			
			// If new item's parent item not in tree view yet, can safely ignore.
			// Also, if a query of specified parent wouldn't return this item, then ignore.
			if(!parent ||
				dojo.indexOf(this.childrenAttr, parentInfo.attribute) == -1){
				return;
			}
		}

		var childParams = {
			item: item,
			isExpandable: this.mayHaveChildren(item)
		};
		if(parent){
			if(!parent.isExpandable){
				parent.makeExpandable();
			}
			if(parent.state=="LOADED" || parent.isExpanded){
				var childrenMap=parent._addChildren([childParams]);
			}
		}else{
			// top level node
			var childrenMap=this._addChildren([childParams]);		
		}

		if(childrenMap){
			dojo.mixin(this._itemNodeMap, childrenMap);
			//this._itemNodeMap[this.store.getIdentity(item)]=child;
		}
	},
	
	_onDeleteItem: function(/*Object*/ item){
		//summary: delete event from the store
		//since the object has just been deleted, we need to
		//use the name directly
		var identity = this.store.getIdentity(item);
		var node = this._itemNodeMap[identity];

		if(node){
			var parent = node.getParent();
			parent.deleteNode(node);
			this._itemNodeMap[identity]=null;
		}
	},

	_onSetItem: function(/*Object*/ item){
		//summary: set data event  on an item in the store
		var identity = this.store.getIdentity(item);
		node = this._itemNodeMap[identity];

		if(node){
			node.setLabelNode(this.getLabel(item));
			node._updateItemClasses(item);
		}
	},
	
	_saveState: function(){
		//summary: create and save a cookie with the currently expanded nodes identifiers
		if(!this.persist){
			return;
		}
		var ary = [];
		for(var id in this._openedItemIds){
			ary.push(id);
		}
		dojo.cookie(this.cookieName, ary.join(","));
	}
});

}

if(!dojo._hasResource["dojox.data.dom"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.dom"] = true;
dojo.provide("dojox.data.dom");

//DOM type to int value for reference.
//Ints make for more compact code than full constant names.
//ELEMENT_NODE                  = 1;
//ATTRIBUTE_NODE                = 2;
//TEXT_NODE                     = 3;
//CDATA_SECTION_NODE            = 4;
//ENTITY_REFERENCE_NODE         = 5;
//ENTITY_NODE                   = 6;
//PROCESSING_INSTRUCTION_NODE   = 7;
//COMMENT_NODE                  = 8;
//DOCUMENT_NODE                 = 9;
//DOCUMENT_TYPE_NODE            = 10;
//DOCUMENT_FRAGMENT_NODE        = 11;
//NOTATION_NODE                 = 12;

//FIXME:  Remove this file when possible.
//This file contains internal/helper APIs as holders until the true DOM apis of Dojo 0.9 are finalized.
//Therefore, these should not be generally used, they are present only for the use by XmlStore and the
//wires project until proper dojo replacements are available.  When such exist, XmlStore and the like
//will be ported off these and this file will be deleted.
dojo.experimental("dojox.data.dom");

dojox.data.dom.createDocument = function(/*string?*/ str, /*string?*/ mimetype){
	//	summary:
	//		cross-browser implementation of creating an XML document object.
	//
	//	str:
	//		Optional text to create the document from.  If not provided, an empty XML document will be created.
	//	mimetype:
	//		Optional mimetype of the text.  Typically, this is text/xml.  Will be defaulted to text/xml if not provided.
	var _document = dojo.doc;

	if(!mimetype){ mimetype = "text/xml"; }
	if(str && (typeof dojo.global["DOMParser"]) !== "undefined"){
		var parser = new DOMParser();
		return parser.parseFromString(str, mimetype);	//	DOMDocument
	}else if((typeof dojo.global["ActiveXObject"]) !== "undefined"){
		var prefixes = [ "MSXML2", "Microsoft", "MSXML", "MSXML3" ];
		for(var i = 0; i<prefixes.length; i++){
			try{
				var doc = new ActiveXObject(prefixes[i]+".XMLDOM");
				if(str){
					if(doc){
						doc.async = false;
						doc.loadXML(str);
						return doc;	//	DOMDocument
					}else{
						console.log("loadXML didn't work?");
					}
				}else{
					if(doc){ 
						return doc; //DOMDocument
					}
				}
			}catch(e){ /* squelch */ };
		}
	}else if((_document.implementation)&&
		(_document.implementation.createDocument)){
		if(str){
			if(_document.createElement){
				// FIXME: this may change all tags to uppercase!
				var tmp = _document.createElement("xml");
				tmp.innerHTML = str;
				var xmlDoc = _document.implementation.createDocument("foo", "", null);
				for(var i = 0; i < tmp.childNodes.length; i++) {
					xmlDoc.importNode(tmp.childNodes.item(i), true);
				}
				return xmlDoc;	//	DOMDocument
			}
		}else{
			return _document.implementation.createDocument("", "", null); // DOMDocument
		}
	}
	return null;	//	DOMDocument
}

dojox.data.dom.textContent = function(/*Node*/node, /*string?*/text){
	//	summary:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//	description:
	//		Implementation of the DOM Level 3 attribute; scan node for text
	//		This function can also update the text of a node by replacing all child 
	//		content of the node.
	//	node:
	//		The node to get the text off of or set the text on.
	//	text:
	//		Optional argument of the text to apply to the node.
	if(arguments.length>1){
		var _document = node.ownerDocument || dojo.doc;  //Preference is to get the node owning doc first or it may fail
		dojox.data.dom.replaceChildren(node, _document.createTextNode(text));
		return text;	//	string
	} else {
		if(node.textContent !== undefined){ //FF 1.5
			return node.textContent;	//	string
		}
		var _result = "";
		if(node == null){
			return _result; //empty string.
		}
		for(var i = 0; i < node.childNodes.length; i++){
			switch(node.childNodes[i].nodeType){
				case 1: // ELEMENT_NODE
				case 5: // ENTITY_REFERENCE_NODE
					_result += dojox.data.dom.textContent(node.childNodes[i]);
					break;
				case 3: // TEXT_NODE
				case 2: // ATTRIBUTE_NODE
				case 4: // CDATA_SECTION_NODE
					_result += node.childNodes[i].nodeValue;
					break;
				default:
					break;
			}
		}
		return _result;	//	string
	}
}

dojox.data.dom.replaceChildren = function(/*Element*/node, /*Node || array*/ newChildren){
	//	summary:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	//	description:
	//		Removes all children of node and appends newChild. All the existing
	//		children will be destroyed.
	// 	node:
	//		The node to modify the children on
	//	newChildren:
	//		The children to add to the node.  It can either be a single Node or an
	//		array of Nodes.
	var nodes = [];
	
	if(dojo.isIE){
		for(var i=0;i<node.childNodes.length;i++){
			nodes.push(node.childNodes[i]);
		}
	}

	dojox.data.dom.removeChildren(node);
	for(var i=0;i<nodes.length;i++){
		dojo._destroyElement(nodes[i]);
	}

	if(!dojo.isArray(newChildren)){
		node.appendChild(newChildren);
	}else{
		for(var i=0;i<newChildren.length;i++){
			node.appendChild(newChildren[i]);
		}
	}
}

dojox.data.dom.removeChildren = function(/*Element*/node){
	//	summary:
	//		removes all children from node and returns the count of children removed.
	//		The children nodes are not destroyed. Be sure to call dojo._destroyElement on them
	//		after they are not used anymore.
	//	node:
	//		The node to remove all the children from.
	var count = node.childNodes.length;
	while(node.hasChildNodes()){
		node.removeChild(node.firstChild);
	}
	return count; // int
}


dojox.data.dom.innerXML = function(/*Node*/node){
	//	summary:
	//		Implementation of MS's innerXML function.
	//	node:
	//		The node from which to generate the XML text representation.
	if(node.innerXML){
		return node.innerXML;	//	string
	}else if (node.xml){
		return node.xml;		//	string
	}else if(typeof XMLSerializer != "undefined"){
		return (new XMLSerializer()).serializeToString(node);	//	string
	}
}


}

if(!dojo._hasResource["dojox.gfx.matrix"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.matrix"] = true;
dojo.provide("dojox.gfx.matrix");

(function(){
	var m = dojox.gfx.matrix;
	
	// candidates for dojox.math:
	m._degToRad = function(degree){ return Math.PI * degree / 180; };
	m._radToDeg = function(radian){ return radian / Math.PI * 180; };

	m.Matrix2D = function(arg){
		// summary: a 2D matrix object
		// description: Normalizes a 2D matrix-like object. If arrays is passed, 
		//		all objects of the array are normalized and multiplied sequentially.
		// arg: Object
		//		a 2D matrix-like object, a number, or an array of such objects
		if(arg){
			if(typeof arg == "number"){
				this.xx = this.yy = arg;
			}else if(arg instanceof Array){
				if(arg.length > 0){
					var matrix = m.normalize(arg[0]);
					// combine matrices
					for(var i = 1; i < arg.length; ++i){
						var l = matrix, r = dojox.gfx.matrix.normalize(arg[i]);
						matrix = new m.Matrix2D();
						matrix.xx = l.xx * r.xx + l.xy * r.yx;
						matrix.xy = l.xx * r.xy + l.xy * r.yy;
						matrix.yx = l.yx * r.xx + l.yy * r.yx;
						matrix.yy = l.yx * r.xy + l.yy * r.yy;
						matrix.dx = l.xx * r.dx + l.xy * r.dy + l.dx;
						matrix.dy = l.yx * r.dx + l.yy * r.dy + l.dy;
					}
					dojo.mixin(this, matrix);
				}
			}else{
				dojo.mixin(this, arg);
			}
		}
	};

	// the default (identity) matrix, which is used to fill in missing values
	dojo.extend(m.Matrix2D, {xx: 1, xy: 0, yx: 0, yy: 1, dx: 0, dy: 0});

	dojo.mixin(m, {
		// summary: class constants, and methods of dojox.gfx.matrix
		
		// matrix constants
		
		// identity: dojox.gfx.matrix.Matrix2D
		//		an identity matrix constant: identity * (x, y) == (x, y)
		identity: new m.Matrix2D(),
		
		// flipX: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at x = 0 line: flipX * (x, y) == (-x, y)
		flipX:    new m.Matrix2D({xx: -1}),
		
		// flipY: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at y = 0 line: flipY * (x, y) == (x, -y)
		flipY:    new m.Matrix2D({yy: -1}),
		
		// flipXY: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at the origin of coordinates: flipXY * (x, y) == (-x, -y)
		flipXY:   new m.Matrix2D({xx: -1, yy: -1}),
		
		// matrix creators
		
		translate: function(a, b){
			// summary: forms a translation matrix
			// description: The resulting matrix is used to translate (move) points by specified offsets.
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value
			if(arguments.length > 1){
				return new m.Matrix2D({dx: a, dy: b}); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: dojox.gfx.Point: a point-like object, which specifies offsets for both dimensions
			// b: null
			return new m.Matrix2D({dx: a.x, dy: a.y}); // dojox.gfx.matrix.Matrix2D
		},
		scale: function(a, b){
			// summary: forms a scaling matrix
			// description: The resulting matrix is used to scale (magnify) points by specified offsets.
			// a: Number: a scaling factor used for the x coordinate
			// b: Number: a scaling factor used for the y coordinate
			if(arguments.length > 1){
				return new m.Matrix2D({xx: a, yy: b}); // dojox.gfx.matrix.Matrix2D
			}
			if(typeof a == "number"){
				// branch
				// a: Number: a uniform scaling factor used for the both coordinates
				// b: null
				return new m.Matrix2D({xx: a, yy: a}); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: dojox.gfx.Point: a point-like object, which specifies scale factors for both dimensions
			// b: null
			return new m.Matrix2D({xx: a.x, yy: a.y}); // dojox.gfx.matrix.Matrix2D
		},
		rotate: function(angle){
			// summary: forms a rotating matrix
			// description: The resulting matrix is used to rotate points 
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an angle of rotation in radians (>0 for CW)
			var c = Math.cos(angle);
			var s = Math.sin(angle);
			return new m.Matrix2D({xx: c, xy: -s, yx: s, yy: c}); // dojox.gfx.matrix.Matrix2D
		},
		rotateg: function(degree){
			// summary: forms a rotating matrix
			// description: The resulting matrix is used to rotate points
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.rotate() for comparison.
			// degree: Number: an angle of rotation in degrees (>0 for CW)
			return m.rotate(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		skewX: function(angle) {
			// summary: forms an x skewing matrix
			// description: The resulting matrix is used to skew points in the x dimension
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an skewing angle in radians
			return new m.Matrix2D({xy: -Math.tan(angle)}); // dojox.gfx.matrix.Matrix2D
		},
		skewXg: function(degree){
			// summary: forms an x skewing matrix
			// description: The resulting matrix is used to skew points in the x dimension
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.skewX() for comparison.
			// degree: Number: an skewing angle in degrees
			return m.skewX(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		skewY: function(angle){
			// summary: forms a y skewing matrix
			// description: The resulting matrix is used to skew points in the y dimension
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an skewing angle in radians
			return new m.Matrix2D({yx: Math.tan(angle)}); // dojox.gfx.matrix.Matrix2D
		},
		skewYg: function(degree){
			// summary: forms a y skewing matrix
			// description: The resulting matrix is used to skew points in the y dimension
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.skewY() for comparison.
			// degree: Number: an skewing angle in degrees
			return m.skewY(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		reflect: function(a, b){
			// summary: forms a reflection matrix
			// description: The resulting matrix is used to reflect points around a vector, 
			//		which goes through the origin.
			// a: dojox.gfx.Point: a point-like object, which specifies a vector of reflection
			// b: null
			if(arguments.length == 1){
				b = a.y;
				a = a.x;
			}
			// branch
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value
			
			// make a unit vector
			var a2 = a * a, b2 = b * b, n2 = a2 + b2, xy = 2 * a * b / n2;
			return new m.Matrix2D({xx: 2 * a2 / n2 - 1, xy: xy, yx: xy, yy: 2 * b2 / n2 - 1}); // dojox.gfx.matrix.Matrix2D
		},
		project: function(a, b){
			// summary: forms an orthogonal projection matrix
			// description: The resulting matrix is used to project points orthogonally on a vector, 
			//		which goes through the origin.
			// a: dojox.gfx.Point: a point-like object, which specifies a vector of projection
			// b: null
			if(arguments.length == 1){
				b = a.y;
				a = a.x;
			}
			// branch
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value
			
			// make a unit vector
			var a2 = a * a, b2 = b * b, n2 = a2 + b2, xy = a * b / n2;
			return new m.Matrix2D({xx: a2 / n2, xy: xy, yx: xy, yy: b2 / n2}); // dojox.gfx.matrix.Matrix2D
		},
		
		// ensure matrix 2D conformance
		normalize: function(matrix){
			// summary: converts an object to a matrix, if necessary
			// description: Converts any 2D matrix-like object or an array of
			//		such objects to a valid dojox.gfx.matrix.Matrix2D object.
			// matrix: Object: an object, which is converted to a matrix, if necessary
			return (matrix instanceof m.Matrix2D) ? matrix : new m.Matrix2D(matrix); // dojox.gfx.matrix.Matrix2D
		},
		
		// common operations
		
		clone: function(matrix){
			// summary: creates a copy of a 2D matrix
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object to be cloned
			var obj = new m.Matrix2D();
			for(var i in matrix){
				if(typeof(matrix[i]) == "number" && typeof(obj[i]) == "number" && obj[i] != matrix[i]) obj[i] = matrix[i];
			}
			return obj; // dojox.gfx.matrix.Matrix2D
		},
		invert: function(matrix){
			// summary: inverts a 2D matrix
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object to be inverted
			var M = m.normalize(matrix),
				D = M.xx * M.yy - M.xy * M.yx,
				M = new m.Matrix2D({
					xx: M.yy/D, xy: -M.xy/D, 
					yx: -M.yx/D, yy: M.xx/D, 
					dx: (M.xy * M.dy - M.yy * M.dx) / D, 
					dy: (M.yx * M.dx - M.xx * M.dy) / D
				});
			return M; // dojox.gfx.matrix.Matrix2D
		},
		_multiplyPoint: function(matrix, x, y){
			// summary: applies a matrix to a point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// x: Number: an x coordinate of a point
			// y: Number: a y coordinate of a point
			return {x: matrix.xx * x + matrix.xy * y + matrix.dx, y: matrix.yx * x + matrix.yy * y + matrix.dy}; // dojox.gfx.Point
		},
		multiplyPoint: function(matrix, /* Number||Point */ a, /* Number, optional */ b){
			// summary: applies a matrix to a point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// a: Number: an x coordinate of a point
			// b: Number: a y coordinate of a point
			var M = m.normalize(matrix);
			if(typeof a == "number" && typeof b == "number"){
				return m._multiplyPoint(M, a, b); // dojox.gfx.Point
			}
			// branch
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// a: dojox.gfx.Point: a point
			// b: null
			return m._multiplyPoint(M, a.x, a.y); // dojox.gfx.Point
		},
		multiply: function(matrix){
			// summary: combines matrices by multiplying them sequentially in the given order
			// matrix: dojox.gfx.matrix.Matrix2D...: a 2D matrix-like object, 
			//		all subsequent arguments are matrix-like objects too
			var M = m.normalize(matrix);
			// combine matrices
			for(var i = 1; i < arguments.length; ++i){
				var l = M, r = m.normalize(arguments[i]);
				M = new m.Matrix2D();
				M.xx = l.xx * r.xx + l.xy * r.yx;
				M.xy = l.xx * r.xy + l.xy * r.yy;
				M.yx = l.yx * r.xx + l.yy * r.yx;
				M.yy = l.yx * r.xy + l.yy * r.yy;
				M.dx = l.xx * r.dx + l.xy * r.dy + l.dx;
				M.dy = l.yx * r.dx + l.yy * r.dy + l.dy;
			}
			return M; // dojox.gfx.matrix.Matrix2D
		},
		
		// high level operations
		
		_sandwich: function(matrix, x, y){
			// summary: applies a matrix at a centrtal point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object, which is applied at a central point
			// x: Number: an x component of the central point
			// y: Number: a y component of the central point
			return m.multiply(m.translate(x, y), matrix, m.translate(-x, -y)); // dojox.gfx.matrix.Matrix2D
		},
		scaleAt: function(a, b, c, d){
			// summary: scales a picture using a specified point as a center of scaling
			// description: Compare with dojox.gfx.matrix.scale().
			// a: Number: a scaling factor used for the x coordinate
			// b: Number: a scaling factor used for the y coordinate
			// c: Number: an x component of a central point
			// d: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) uniform scale factor, Point
			//	2) uniform scale factor, x, y
			//	3) x scale, y scale, Point
			//	4) x scale, y scale, x, y
			
			switch(arguments.length){
				case 4:
					// a and b are scale factor components, c and d are components of a point
					return m._sandwich(m.scale(a, b), c, d); // dojox.gfx.matrix.Matrix2D
				case 3:
					if(typeof c == "number"){
						// branch
						// a: Number: a uniform scaling factor used for both coordinates
						// b: Number: an x component of a central point
						// c: Number: a y component of a central point
						// d: null
						return m._sandwich(m.scale(a), b, c); // dojox.gfx.matrix.Matrix2D
					}
					// branch
					// a: Number: a scaling factor used for the x coordinate
					// b: Number: a scaling factor used for the y coordinate
					// c: dojox.gfx.Point: a central point
					// d: null
					return m._sandwich(m.scale(a, b), c.x, c.y); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: Number: a uniform scaling factor used for both coordinates
			// b: dojox.gfx.Point: a central point
			// c: null
			// d: null
			return m._sandwich(m.scale(a), b.x, b.y); // dojox.gfx.matrix.Matrix2D
		},
		rotateAt: function(angle, a, b){
			// summary: rotates a picture using a specified point as a center of rotation
			// description: Compare with dojox.gfx.matrix.rotate().
			// angle: Number: an angle of rotation in radians (>0 for CW)
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) rotation angle in radians, Point
			//	2) rotation angle in radians, x, y
			
			if(arguments.length > 2){
				return m._sandwich(m.rotate(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}
			
			// branch
			// angle: Number: an angle of rotation in radians (>0 for CCW)
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.rotate(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		rotategAt: function(degree, a, b){
			// summary: rotates a picture using a specified point as a center of rotation
			// description: Compare with dojox.gfx.matrix.rotateg().
			// degree: Number: an angle of rotation in degrees (>0 for CW)
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) rotation angle in degrees, Point
			//	2) rotation angle in degrees, x, y
			
			if(arguments.length > 2){
				return m._sandwich(m.rotateg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an angle of rotation in degrees (>0 for CCW)
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.rotateg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewXAt: function(angle, a, b){
			// summary: skews a picture along the x axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewX().
			// angle: Number: an skewing angle in radians
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) skew angle in radians, Point
			//	2) skew angle in radians, x, y
			
			if(arguments.length > 2){
				return m._sandwich(m.skewX(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// angle: Number: an skewing angle in radians
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewX(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewXgAt: function(degree, a, b){
			// summary: skews a picture along the x axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewXg().
			// degree: Number: an skewing angle in degrees
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) skew angle in degrees, Point
			//	2) skew angle in degrees, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewXg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an skewing angle in degrees
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewXg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewYAt: function(angle, a, b){
			// summary: skews a picture along the y axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewY().
			// angle: Number: an skewing angle in radians
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) skew angle in radians, Point
			//	2) skew angle in radians, x, y
			
			if(arguments.length > 2){
				return m._sandwich(m.skewY(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// angle: Number: an skewing angle in radians
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewY(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewYgAt: function(/* Number */ degree, /* Number||Point */ a, /* Number, optional */ b){
			// summary: skews a picture along the y axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewYg().
			// degree: Number: an skewing angle in degrees
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) skew angle in degrees, Point
			//	2) skew angle in degrees, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewYg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an skewing angle in degrees
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewYg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		}
		
		//TODO: rect-to-rect mapping, scale-to-fit (isotropic and anisotropic versions)
		
	});
})();

// propagate Matrix2D up
dojox.gfx.Matrix2D = dojox.gfx.matrix.Matrix2D;

}

if(!dojo._hasResource["dojox.gfx._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx._base"] = true;
dojo.provide("dojox.gfx._base");

// candidates for dojox.style (work on VML and SVG nodes)
dojox.gfx._hasClass = function(/*DomNode*/node, /*String*/classStr){
	//	summary:
	//		Returns whether or not the specified classes are a portion of the
	//		class list currently applied to the node. 
	// return (new RegExp('(^|\\s+)'+classStr+'(\\s+|$)')).test(node.className)	// Boolean
	return ((" "+node.getAttribute("className")+" ").indexOf(" "+classStr+" ") >= 0);  // Boolean
}

dojox.gfx._addClass = function(/*DomNode*/node, /*String*/classStr){
	//	summary:
	//		Adds the specified classes to the end of the class list on the
	//		passed node.
	var cls = node.getAttribute("className");
	if((" "+cls+" ").indexOf(" "+classStr+" ") < 0){
		node.setAttribute("className", cls + (cls ? ' ' : '') + classStr);
	}
}

dojox.gfx._removeClass = function(/*DomNode*/node, /*String*/classStr){
	//	summary: Removes classes from node.
	node.setAttribute("className", node.getAttribute("className").replace(new RegExp('(^|\\s+)'+classStr+'(\\s+|$)'), "$1$2"));
}


// candidate for dojox.html.metrics (dynamic font resize handler is not implemented here)

//	derived from Morris John's emResized measurer
dojox.gfx._base._getFontMeasurements = function(){
	//	summary
	//	Returns an object that has pixel equivilents of standard font size values.
	var heights = {
		'1em':0, '1ex':0, '100%':0, '12pt':0, '16px':0, 'xx-small':0, 'x-small':0,
		'small':0, 'medium':0, 'large':0, 'x-large':0, 'xx-large':0
	};

	if(dojo.isIE){
		//	we do a font-size fix if and only if one isn't applied already.
		//	NOTE: If someone set the fontSize on the HTML Element, this will kill it.
		dojo.doc.documentElement.style.fontSize="100%";
	}

	//	set up the measuring node.
	var div=dojo.doc.createElement("div");
	div.style.position="absolute";
	div.style.left="-100px";
	div.style.top="0";
	div.style.width="30px";
	div.style.height="1000em";
	div.style.border="0";
	div.style.margin="0";
	div.style.padding="0";
	div.style.outline="0";
	div.style.lineHeight="1";
	div.style.overflow="hidden";
	dojo.body().appendChild(div);

	//	do the measurements.
	for(var p in heights){
		div.style.fontSize = p;
		heights[p] = Math.round(div.offsetHeight * 12/16) * 16/12 / 1000;
	}
	
	dojo.body().removeChild(div);
	div = null;
	return heights; 	//	object
};

dojox.gfx._base._fontMeasurements = null;

dojox.gfx._base._getCachedFontMeasurements = function(recalculate){
	if(recalculate || !dojox.gfx._base._fontMeasurements){
		dojox.gfx._base._fontMeasurements = dojox.gfx._base._getFontMeasurements();
	}
	return dojox.gfx._base._fontMeasurements;
};

// candidate for dojo.dom

dojox.gfx._base._uniqueId = 0;
dojox.gfx._base._getUniqueId = function(){
	// summary: returns a unique string for use with any DOM element
	var id;
	do{
		id = "dojoUnique" + (++dojox.gfx._base._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.mixin(dojox.gfx, {
	// summary: defines constants, prototypes, and utility functions
	
	// default shapes, which are used to fill in missing parameters
	defaultPath:     {type: "path",     path: ""},
	defaultPolyline: {type: "polyline", points: []},
	defaultRect:     {type: "rect",     x: 0, y: 0, width: 100, height: 100, r: 0},
	defaultEllipse:  {type: "ellipse",  cx: 0, cy: 0, rx: 200, ry: 100},
	defaultCircle:   {type: "circle",   cx: 0, cy: 0, r: 100},
	defaultLine:     {type: "line",     x1: 0, y1: 0, x2: 100, y2: 100},
	defaultImage:    {type: "image",    x: 0, y: 0, width: 0, height: 0, src: ""},
	defaultText:     {type: "text",     x: 0, y: 0, text: "",
		align: "start", decoration: "none", rotated: false, kerning: true },
	defaultTextPath: {type: "textpath", text: "",
		align: "start", decoration: "none", rotated: false, kerning: true },

	// default geometric attributes
	defaultStroke: {type: "stroke", color: "black", style: "solid", width: 1, cap: "butt", join: 4},
	defaultLinearGradient: {type: "linear", x1: 0, y1: 0, x2: 100, y2: 100, 
		colors: [{offset: 0, color: "black"}, {offset: 1, color: "white"}]},
	defaultRadialGradient: {type: "radial", cx: 0, cy: 0, r: 100, 
		colors: [{offset: 0, color: "black"}, {offset: 1, color: "white"}]},
	defaultPattern: {type: "pattern", x: 0, y: 0, width: 0, height: 0, src: ""},
	defaultFont: {type: "font", style: "normal", variant: "normal", weight: "normal", 
		size: "10pt", family: "serif"},

	normalizeColor: function(/*Color*/ color){
		// summary: converts any legal color representation to normalized dojo.Color object
		return (color instanceof dojo.Color) ? color : new dojo.Color(color); // dojo.Color
	},
	normalizeParameters: function(existed, update){
		// summary: updates an existing object with properties from an "update" object
		// existed: Object: the "target" object to be updated
		// update:  Object: the "update" object, whose properties will be used to update the existed object
		if(update){
			var empty = {};
			for(var x in existed){
				if(x in update && !(x in empty)){
					existed[x] = update[x];
				}
			}
		}
		return existed;	// Object
	},
	makeParameters: function(defaults, update){
		// summary: copies the original object, and all copied properties from the "update" object
		// defaults: Object: the object to be cloned before updating
		// update:   Object: the object, which properties are to be cloned during updating
		if(!update) return dojo.clone(defaults);
		var result = {};
		for(var i in defaults){
			if(!(i in result)){
				result[i] = dojo.clone((i in update) ? update[i] : defaults[i]);
			}
		}
		return result; // Object
	},
	formatNumber: function(x, addSpace){
		// summary: converts a number to a string using a fixed notation
		// x:			Number:		number to be converted
		// addSpace:	Boolean?:	if it is true, add a space before a positive number
		var val = x.toString();
		if(val.indexOf("e") >= 0){
			val = x.toFixed(4);
		}else{
			var point = val.indexOf(".");
			if(point >= 0 && val.length - point > 5){
				val = x.toFixed(4);
			}
		}
		if(x < 0){
			return val; // String
		}
		return addSpace ? " " + val : val; // String
	},
	// font operations
	makeFontString: function(font){
		// summary: converts a font object to a CSS font string
		// font:	Object:	font object (see dojox.gfx.defaultFont)
		return font.style + " " + font.variant + " " + font.weight + " " + font.size + " " + font.family; // Object
	},
	splitFontString: function(str){
		// summary: converts a CSS font string to a font object
		// str:		String:	a CSS font string
		var font = dojo.clone(dojox.gfx.defaultFont);
		var t = str.split(/\s+/);
		do{
			if(t.length < 5){ break; }
			font.style  = t[0];
			font.varian = t[1];
			font.weight = t[2];
			var i = t[3].indexOf("/");
			font.size = i < 0 ? t[3] : t[3].substring(0, i);
			var j = 4;
			if(i < 0){
				if(t[4] == "/"){
					j = 6;
					break;
				}
				if(t[4].substr(0, 1) == "/"){
					j = 5;
					break;
				}
			}
			if(j + 3 > t.length){ break; }
			font.size = t[j];
			font.family = t[j + 1];
		}while(false);
		return font;	// Object
	},
	// length operations
	cm_in_pt: 72 / 2.54,	// Number: centimeters per inch
	mm_in_pt: 7.2 / 2.54,	// Number: millimeters per inch
	px_in_pt: function(){
		// summary: returns a number of pixels per point
		return dojox.gfx._base._getCachedFontMeasurements()["12pt"] / 12;	// Number
	},
	pt2px: function(len){
		// summary: converts points to pixels
		// len: Number: a value in points
		return len * dojox.gfx.px_in_pt();	// Number
	},
	px2pt: function(len){
		// summary: converts pixels to points
		// len: Number: a value in pixels
		return len / dojox.gfx.px_in_pt();	// Number
	},
	normalizedLength: function(len) {
		// summary: converts any length value to pixels
		// len: String: a length, e.g., "12pc"
		if(len.length == 0) return 0;
		if(len.length > 2){
			var px_in_pt = dojox.gfx.px_in_pt();
			var val = parseFloat(len);
			switch(len.slice(-2)){
				case "px": return val;
				case "pt": return val * px_in_pt;
				case "in": return val * 72 * px_in_pt;
				case "pc": return val * 12 * px_in_pt;
				case "mm": return val / dojox.gfx.mm_in_pt * px_in_pt;
				case "cm": return val / dojox.gfx.cm_in_pt * px_in_pt;
			}
		}
		return parseFloat(len);	// Number
	},
	
	// a constant used to split a SVG/VML path into primitive components
	pathVmlRegExp: /([A-Za-z]+)|(\d+(\.\d+)?)|(\.\d+)|(-\d+(\.\d+)?)|(-\.\d+)/g,
	pathSvgRegExp: /([A-Za-z])|(\d+(\.\d+)?)|(\.\d+)|(-\d+(\.\d+)?)|(-\.\d+)/g,
	
	equalSources: function(a, b){
		// summary: compares event sources, returns true if they are equal
		return a && b && a == b;
	}
});

}

if(!dojo._hasResource["dojox.gfx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx"] = true;
dojo.provide("dojox.gfx");




(function(){
	var renderers = (typeof djConfig["gfxRenderer"] == "string" ?
		djConfig["gfxRenderer"] : "svg,vml,silverlight,canvas").split(",");
	for(var i = 0; i < renderers.length; ++i){
		switch(renderers[i]){
			case "svg":
				//TODO: need more comprehensive test for SVG
				if(!dojo.isIE && (navigator.userAgent.indexOf("iPhone") < 0) && (navigator.userAgent.indexOf("iPod") < 0)){ dojox.gfx.renderer = "svg"; }
				break;
			case "vml":
				if(dojo.isIE != 0){ dojox.gfx.renderer = "vml"; }
				break;
			case "silverlight":
				//TODO: need more comprehensive test for Silverlight
				if(window.Silverlight){ dojox.gfx.renderer = "silverlight"; }
				break;
			case "canvas":
				//TODO: need more comprehensive test for Canvas
				if(dojo.isIE == 0){ dojox.gfx.renderer = "canvas"; }
				break;
		}
		if(dojox.gfx.renderer){ break; }
	}
})();

// include a renderer conditionally
dojo.requireIf(dojox.gfx.renderer == "svg", "dojox.gfx.svg");
dojo.requireIf(dojox.gfx.renderer == "vml", "dojox.gfx.vml");
dojo.requireIf(dojox.gfx.renderer == "silverlight", "dojox.gfx.silverlight");
dojo.requireIf(dojox.gfx.renderer == "canvas", "dojox.gfx.canvas");

}

if(!dojo._hasResource["dojox.gfx.shape"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.shape"] = true;
dojo.provide("dojox.gfx.shape");



dojo.declare("dojox.gfx.Shape", null, {
	// summary: a Shape object, which knows how to apply 
	// graphical attributes and transformations
	
	constructor: function(){
		// rawNode: Node: underlying node
		this.rawNode = null;
		
		// shape: Object: an abstract shape object
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)
		this.shape = null;
		
		// matrix: dojox.gfx.Matrix2D: a transformation matrix
		this.matrix = null;
		
		// fillStyle: Object: a fill object 
		//	(see dojox.gfx.defaultLinearGradient, 
		//	dojox.gfx.defaultRadialGradient, 
		//	dojox.gfx.defaultPattern, 
		//	or dojo.Color)
		this.fillStyle = null;
		
		// strokeStyle: Object: a stroke object 
		//	(see dojox.gfx.defaultStroke) 
		this.strokeStyle = null;
		
		// bbox: dojox.gfx.Rectangle: a bounding box of this shape
		//	(see dojox.gfx.defaultRect)
		this.bbox = null;
		
		// virtual group structure
		
		// parent: Object: a parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		this.parent = null;
		
		// parentMatrix: dojox.gfx.Matrix2D
		//	a transformation matrix inherited from the parent
		this.parentMatrix = null;
	},
	
	// trivial getters
	
	getNode: function(){
		// summary: returns the current DOM Node or null
		return this.rawNode; // Node
	},
	getShape: function(){
		// summary: returns the current shape object or null
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)
		return this.shape; // Object
	},
	getTransform: function(){
		// summary: returns the current transformation matrix or null
		return this.matrix;	// dojox.gfx.Matrix2D
	},
	getFill: function(){
		// summary: returns the current fill object or null
		//	(see dojox.gfx.defaultLinearGradient, 
		//	dojox.gfx.defaultRadialGradient, 
		//	dojox.gfx.defaultPattern, 
		//	or dojo.Color)
		return this.fillStyle;	// Object
	},
	getStroke: function(){
		// summary: returns the current stroke object or null
		//	(see dojox.gfx.defaultStroke) 
		return this.strokeStyle;	// Object
	},
	getParent: function(){
		// summary: returns the parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		return this.parent;	// Object
	},
	getBoundingBox: function(){
		// summary: returns the bounding box or null
		//	(see dojox.gfx.defaultRect)
		return this.bbox;	// dojox.gfx.Rectangle
	},
	getTransformedBoundingBox: function(){
		// summary: returns an array of four points or null
		//	four points represent four corners of the untransformed bounding box
		var b = this.getBoundingBox();
		if(!b){
			return null;	// null
		}
		var m = this._getRealMatrix();
		var r = [];
		var g = dojox.gfx.matrix;
		r.push(g.multiplyPoint(m, b.x, b.y));
		r.push(g.multiplyPoint(m, b.x + b.width, b.y));
		r.push(g.multiplyPoint(m, b.x + b.width, b.y + b.height));
		r.push(g.multiplyPoint(m, b.x, b.y + b.height));
		return r;	// Array
	},
	getEventSource: function(){
		// summary: returns a Node, which is used as 
		//	a source of events for this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return this.rawNode;	// Node
	},
	
	// empty settings
	
	setShape: function(shape){
		// summary: sets a shape object
		//	(the default implementation simply ignores it)
		// shape: Object: a shape object
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		this.shape = dojox.gfx.makeParameters(this.shape, shape);
		this.bbox = null;
		return this;	// self
	},
	setFill: function(fill){
		// summary: sets a fill object
		//	(the default implementation simply ignores it)
		// fill: Object: a fill object
		//	(see dojox.gfx.defaultLinearGradient, 
		//	dojox.gfx.defaultRadialGradient, 
		//	dojox.gfx.defaultPattern, 
		//	or dojo.Color)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		if(!fill){
			// don't fill
			this.fillStyle = null;
			return this;	// self
		}
		var f = null;
		if(typeof(fill) == "object" && "type" in fill){
			// gradient or pattern
			switch(fill.type){
				case "linear":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultLinearGradient, fill);
					break;
				case "radial":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultRadialGradient, fill);
					break;
				case "pattern":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultPattern, fill);
					break;
			}
		}else{
			// color object
			f = dojox.gfx.normalizeColor(fill);
		}
		this.fillStyle = f;
		return this;	// self
	},
	setStroke: function(stroke){
		// summary: sets a stroke object
		//	(the default implementation simply ignores it)
		// stroke: Object: a stroke object
		//	(see dojox.gfx.defaultStroke) 

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		if(!stroke){
			// don't stroke
			this.strokeStyle = null;
			return this;	// self
		}
		// normalize the stroke
		if(typeof stroke == "string"){
			stroke = {color: stroke};
		}
		var s = this.strokeStyle = dojox.gfx.makeParameters(dojox.gfx.defaultStroke, stroke);
		s.color = dojox.gfx.normalizeColor(s.color);
		return this;	// self
	},
	setTransform: function(matrix){
		// summary: sets a transformation matrix
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D 
		//	constructor for a list of acceptable arguments)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		this.matrix = dojox.gfx.matrix.clone(matrix ? dojox.gfx.matrix.normalize(matrix) : dojox.gfx.matrix.identity);
		return this._applyTransform();	// self
	},
	
	_applyTransform: function(){
		// summary: physically sets a matrix

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return this;	// self
	},
	
	// z-index
	
	moveToFront: function(){
		// summary: moves a shape to front of its parent's list of shapes
		var p = this.getParent();
		if(p){
			p._moveChildToFront(this);
			this._moveToFront();	// execute renderer-specific action
		}
		return this;	// self
	},
	moveToBack: function(){
		// summary: moves a shape to back of its parent's list of shapes
		var p = this.getParent();
		if(p){
			p._moveChildToBack(this);
			this._moveToBack();	// execute renderer-specific action
		}
		return this;
	},
	_moveToFront: function(){
		// summary: renderer-specific hook, see dojox.gfx.shape.Shape.moveToFront()

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!
	},
	_moveToBack: function(){
		// summary: renderer-specific hook, see dojox.gfx.shape.Shape.moveToFront()

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!
	},

	// apply left & right transformation
	
	applyRightTransform: function(matrix){
		// summary: multiplies the existing matrix with an argument on right side
		//	(this.matrix * matrix)
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D 
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([this.matrix, matrix]) : this;	// self
	},
	applyLeftTransform: function(matrix){
		// summary: multiplies the existing matrix with an argument on left side
		//	(matrix * this.matrix)
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D 
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([matrix, this.matrix]) : this;	// self
	},
	applyTransform: function(matrix){
		// summary: a shortcut for dojox.gfx.Shape.applyRightTransform
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D 
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([this.matrix, matrix]) : this;	// self
	},
	
	// virtual group methods
	
	removeShape: function(silently){
		// summary: removes the shape from its parent's list of shapes
		// silently: Boolean?: if true, do not redraw a picture yet
		if(this.parent){
			this.parent.remove(this, silently);
		}
		return this;	// self
	},
	_setParent: function(parent, matrix){
		// summary: sets a parent
		// parent: Object: a parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		// matrix: dojox.gfx.Matrix2D:
		//	a 2D matrix or a matrix-like object
		this.parent = parent;
		return this._updateParentMatrix(matrix);	// self
	},
	_updateParentMatrix: function(matrix){
		// summary: updates the parent matrix with new matrix
		// matrix: dojox.gfx.Matrix2D:
		//	a 2D matrix or a matrix-like object
		this.parentMatrix = matrix ? dojox.gfx.matrix.clone(matrix) : null;
		return this._applyTransform();	// self
	},
	_getRealMatrix: function(){
		// summary: returns the cumulative ("real") transformation matrix
		//	by combining the shape's matrix with its parent's matrix
		var m = this.matrix;
		var p = this.parent;
		while(p){
			if(p.matrix){
				m = dojox.gfx.matrix.multiply(p.matrix, m);
			}
			p = p.parent;
		}
		return m;	// dojox.gfx.Matrix2D
	}
});

dojox.gfx.shape._eventsProcessing = {
	connect: function(name, object, method){
		// summary: connects a handler to an event on this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return arguments.length > 2 ?	// Object
			dojo.connect(this.getEventSource(), name, object, method) :
			dojo.connect(this.getEventSource(), name, object);
	},
	disconnect: function(token){
		// summary: connects a handler by token from an event on this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		dojo.disconnect(token);
	}
};

dojo.extend(dojox.gfx.Shape, dojox.gfx.shape._eventsProcessing);

dojox.gfx.shape.Container = {
	// summary: a container of shapes, which can be used 
	//	as a foundation for renderer-specific groups, or as a way 
	//	to logically group shapes (e.g, to propagate matricies)
	
	_init: function() {
		// children: Array: a list of children
		this.children = [];
	},
	
	// group management
	
	add: function(shape){
		// summary: adds a shape to the list
		// shape: dojox.gfx.Shape: a shape
		var oldParent = shape.getParent();
		if(oldParent){
			oldParent.remove(shape, true);
		}
		this.children.push(shape);
		return shape._setParent(this, this._getRealMatrix());	// self
	},
	remove: function(shape, silently){
		// summary: removes a shape from the list
		// silently: Boolean?: if true, do not redraw a picture yet
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				if(silently){
					// skip for now
				}else{
					shape._setParent(null, null);
				}
				this.children.splice(i, 1);
				break;
			}
		}
		return this;	// self
	},
	clear: function(){
		// summary: removes all shapes from a group/surface
		this.children = [];
		return this;	// self
	},
	
	// moving child nodes
	
	_moveChildToFront: function(shape){
		// summary: moves a shape to front of the list of shapes
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				this.children.splice(i, 1);
				this.children.push(shape);
				break;
			}
		}
		return this;	// self
	},
	_moveChildToBack: function(shape){
		// summary: moves a shape to back of the list of shapes
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				this.children.splice(i, 1);
				this.children.unshift(shape);
				break;
			}
		}
		return this;	// self
	}
};

dojo.declare("dojox.gfx.shape.Surface", null, {
	// summary: a surface object to be used for drawings
	constructor: function(){
		// underlying node
		this.rawNode = null;
	},
	getEventSource: function(){
		// summary: returns a node, which can be used to attach event listeners
		return this.rawNode; // Node
	},
	_getRealMatrix: function(){
		// summary: always returns the identity matrix
		return null;	// dojox.gfx.Matrix2D
	}
});

dojo.extend(dojox.gfx.shape.Surface, dojox.gfx.shape._eventsProcessing);

dojo.declare("dojox.gfx.Point", null, {
	// summary: a hypothetical 2D point to be used for drawings - {x, y}
	// description: This object is defined for documentation purposes.
	//	You should use the naked object instead: {x: 1, y: 2}.
});

dojo.declare("dojox.gfx.Rectangle", null, {
	// summary: a hypothetical rectangle - {x, y, width, height}
	// description: This object is defined for documentation purposes.
	//	You should use the naked object instead: {x: 1, y: 2, width: 100, height: 200}.
});

dojo.declare("dojox.gfx.shape.Rect", dojox.gfx.Shape, {
	// summary: a generic rectangle
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultRect);
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box (its shape in this case)
		return this.shape;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Ellipse", dojox.gfx.Shape, {
	// summary: a generic ellipse
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultEllipse);
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {x: shape.cx - shape.rx, y: shape.cy - shape.ry, 
				width: 2 * shape.rx, height: 2 * shape.ry};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Circle", dojox.gfx.Shape, {
	// summary: a generic circle
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultCircle);
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {x: shape.cx - shape.r, y: shape.cy - shape.r, 
				width: 2 * shape.r, height: 2 * shape.r};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Line", dojox.gfx.Shape, {
	// summary: a generic line
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultLine);
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {
				x:		Math.min(shape.x1, shape.x2),
				y:		Math.min(shape.y1, shape.y2),
				width:	Math.abs(shape.x2 - shape.x1),
				height:	Math.abs(shape.y2 - shape.y1)
			};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Polyline", dojox.gfx.Shape, {
	// summary: a generic polyline/polygon
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultPolyline);
		this.rawNode = rawNode;
	},
	setShape: function(points, closed){
		// summary: sets a polyline/polygon shape object
		// points: Object: a polyline/polygon shape object
		// closed: Boolean: close the polyline to make a polygon
		if(points && points instanceof Array){
			// points: Array: an array of points
			dojox.gfx.Shape.prototype.setShape.call(this, {points: points});
			if(closed && this.shape.points.length){ 
				this.shape.points.push(this.shape.points[0]);
			}
		}else{
			dojox.gfx.Shape.prototype.setShape.call(this, points);
		}
		return this;	// self
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox && this.shape.points.length){
			var p = this.shape.points;
			var l = p.length;
			var t = p[0];
			var bbox = {l: t.x, t: t.y, r: t.x, b: t.y};
			for(var i = 1; i < l; ++i){
				t = p[i];
				if(bbox.l > t.x) bbox.l = t.x;
				if(bbox.r < t.x) bbox.r = t.x;
				if(bbox.t > t.y) bbox.t = t.y;
				if(bbox.b < t.y) bbox.b = t.y;
			}
			this.bbox = {
				x:		bbox.l, 
				y:		bbox.t, 
				width:	bbox.r - bbox.l, 
				height:	bbox.b - bbox.t
			};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Image", dojox.gfx.Shape, {
	// summary: a generic image
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultImage);
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box (its shape in this case)
		return this.shape;	// dojox.gfx.Rectangle
	},
	setStroke: function(){
		// summary: ignore setting a stroke style
		return this;	// self
	},
	setFill: function(){
		// summary: ignore setting a fill style
		return this;	// self
	}
});

dojo.declare("dojox.gfx.shape.Text", dojox.gfx.Shape, {
	// summary: a generic text
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.fontStyle = null;
		this.shape = dojo.clone(dojox.gfx.defaultText);
		this.rawNode = rawNode;
	},
	setFont: function(newFont){
		// summary: sets a font for text
		// newFont: Object: a font object (see dojox.gfx.defaultFont) or a font string
		this.fontStyle = typeof newFont == "string" ? dojox.gfx.splitFontString(newFont) :
			dojox.gfx.makeParameters(dojox.gfx.defaultFont, newFont);
		this._setFont();
		return this;	// self
	}
});

dojox.gfx.shape.Creator = {
	// summary: shape creators
	createShape: function(shape){
		// summary: creates a shape object based on its type; it is meant to be used
		//	by group-like objects
		// shape: Object: a shape descriptor object
		switch(shape.type){
			case dojox.gfx.defaultPath.type:		return this.createPath(shape);
			case dojox.gfx.defaultRect.type:		return this.createRect(shape);
			case dojox.gfx.defaultCircle.type:		return this.createCircle(shape);
			case dojox.gfx.defaultEllipse.type:		return this.createEllipse(shape);
			case dojox.gfx.defaultLine.type:		return this.createLine(shape);
			case dojox.gfx.defaultPolyline.type:	return this.createPolyline(shape);
			case dojox.gfx.defaultImage.type:		return this.createImage(shape);
			case dojox.gfx.defaultText.type:		return this.createText(shape);
			case dojox.gfx.defaultTextPath.type:	return this.createTextPath(shape);
		}
		return null;
	},
	createGroup: function(){
		// summary: creates an SVG group shape
		return this.createObject(dojox.gfx.Group);	// dojox.gfx.Group
	},
	createRect: function(rect){
		// summary: creates an SVG rectangle shape
		// rect: Object: a path object (see dojox.gfx.defaultRect)
		return this.createObject(dojox.gfx.Rect, rect);	// dojox.gfx.Rect
	},
	createEllipse: function(ellipse){
		// summary: creates an SVG ellipse shape
		// ellipse: Object: an ellipse object (see dojox.gfx.defaultEllipse)
		return this.createObject(dojox.gfx.Ellipse, ellipse);	// dojox.gfx.Ellipse
	},
	createCircle: function(circle){
		// summary: creates an SVG circle shape
		// circle: Object: a circle object (see dojox.gfx.defaultCircle)
		return this.createObject(dojox.gfx.Circle, circle);	// dojox.gfx.Circle
	},
	createLine: function(line){
		// summary: creates an SVG line shape
		// line: Object: a line object (see dojox.gfx.defaultLine)
		return this.createObject(dojox.gfx.Line, line);	// dojox.gfx.Line
	},
	createPolyline: function(points){
		// summary: creates an SVG polyline/polygon shape
		// points: Object: a points object (see dojox.gfx.defaultPolyline)
		//	or an Array of points
		return this.createObject(dojox.gfx.Polyline, points);	// dojox.gfx.Polyline
	},
	createImage: function(image){
		// summary: creates an SVG image shape
		// image: Object: an image object (see dojox.gfx.defaultImage)
		return this.createObject(dojox.gfx.Image, image);	// dojox.gfx.Image
	},
	createText: function(text){
		// summary: creates an SVG text shape
		// text: Object: a text object (see dojox.gfx.defaultText)
		return this.createObject(dojox.gfx.Text, text);	// dojox.gfx.Text
	},
	createPath: function(path){
		// summary: creates an SVG path shape
		// path: Object: a path object (see dojox.gfx.defaultPath)
		return this.createObject(dojox.gfx.Path, path);	// dojox.gfx.Path
	},
	createTextPath: function(text){
		// summary: creates an SVG text shape
		// text: Object: a textpath object (see dojox.gfx.defaultTextPath)
		return this.createObject(dojox.gfx.TextPath, {}).setText(text);	// dojox.gfx.TextPath
	},
	createObject: function(shapeType, rawShape){
		// summary: creates an instance of the passed shapeType class
		// shapeType: Function: a class constructor to create an instance of
		// rawShape: Object: properties to be passed in to the classes "setShape" method

		// SHOULD BE RE-IMPLEMENTED BY THE RENDERER!

		return null;	// dojox.gfx.Shape
	}
};

}

if(!dojo._hasResource["dojox.gfx.path"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.path"] = true;
dojo.provide("dojox.gfx.path");



dojo.declare("dojox.gfx.path.Path", dojox.gfx.Shape, {
	// summary: a generalized path shape
	
	constructor: function(rawNode){
		// summary: a path constructor
		// rawNode: Node: a DOM node to be used by this path object
		this.shape = dojo.clone(dojox.gfx.defaultPath);
		this.segments = [];
		this.absolute = true;
		this.last = {};
		this.rawNode = rawNode;
	},
	
	// mode manipulations
	setAbsoluteMode: function(mode){
		// summary: sets an absolute or relative mode for path points
		// mode: Boolean: true/false or "absolute"/"relative" to specify the mode
		this.absolute = typeof mode == "string" ? (mode == "absolute") : mode;
		return this; // self
	},
	getAbsoluteMode: function(){
		// summary: returns a current value of the absolute mode
		return this.absolute; // Boolean
	},
	
	getBoundingBox: function(){
		// summary: returns the bounding box {x, y, width, height} or null
		return (this.bbox && ("l" in this.bbox)) ? {x: this.bbox.l, y: this.bbox.t, width: this.bbox.r - this.bbox.l, height: this.bbox.b - this.bbox.t} : null; // dojox.gfx.Rectangle
	},
	
	getLastPosition: function(){
		// summary: returns the last point in the path, or null
		return "x" in this.last ? this.last : null; // Object
	},
	
	// segment interpretation
	_updateBBox: function(x, y){
		// summary: updates the bounding box of path with new point
		// x: Number: an x coordinate
		// y: Number: a y coordinate
		
		// we use {l, b, r, t} representation of a bbox
		if(this.bbox && ("l" in this.bbox)){
			if(this.bbox.l > x) this.bbox.l = x;
			if(this.bbox.r < x) this.bbox.r = x;
			if(this.bbox.t > y) this.bbox.t = y;
			if(this.bbox.b < y) this.bbox.b = y;
		}else{
			this.bbox = {l: x, b: y, r: x, t: y};
		}
	},
	_updateWithSegment: function(segment){
		// summary: updates the bounding box of path with new segment
		// segment: Object: a segment
		var n = segment.args, l = n.length;
		// update internal variables: bbox, absolute, last
		switch(segment.action){
			case "M":
			case "L":
			case "C":
			case "S":
			case "Q":
			case "T":
				for(var i = 0; i < l; i += 2){
					this._updateBBox(n[i], n[i + 1]);
				}
				this.last.x = n[l - 2];
				this.last.y = n[l - 1];
				this.absolute = true;
				break;
			case "H":
				for(var i = 0; i < l; ++i){
					this._updateBBox(n[i], this.last.y);
				}
				this.last.x = n[l - 1];
				this.absolute = true;
				break;
			case "V":
				for(var i = 0; i < l; ++i){
					this._updateBBox(this.last.x, n[i]);
				}
				this.last.y = n[l - 1];
				this.absolute = true;
				break;
			case "m":
				var start = 0;
				if(!("x" in this.last)){
					this._updateBBox(this.last.x = n[0], this.last.y = n[1]);
					start = 2;
				}
				for(var i = start; i < l; i += 2){
					this._updateBBox(this.last.x += n[i], this.last.y += n[i + 1]);
				}
				this.absolute = false;
				break;
			case "l":
			case "t":
				for(var i = 0; i < l; i += 2){
					this._updateBBox(this.last.x += n[i], this.last.y += n[i + 1]);
				}
				this.absolute = false;
				break;
			case "h":
				for(var i = 0; i < l; ++i){
					this._updateBBox(this.last.x += n[i], this.last.y);
				}
				this.absolute = false;
				break;
			case "v":
				for(var i = 0; i < l; ++i){
					this._updateBBox(this.last.x, this.last.y += n[i]);
				}
				this.absolute = false;
				break;
			case "c":
				for(var i = 0; i < l; i += 6){
					this._updateBBox(this.last.x + n[i], this.last.y + n[i + 1]);
					this._updateBBox(this.last.x + n[i + 2], this.last.y + n[i + 3]);
					this._updateBBox(this.last.x += n[i + 4], this.last.y += n[i + 5]);
				}
				this.absolute = false;
				break;
			case "s":
			case "q":
				for(var i = 0; i < l; i += 4){
					this._updateBBox(this.last.x + n[i], this.last.y + n[i + 1]);
					this._updateBBox(this.last.x += n[i + 2], this.last.y += n[i + 3]);
				}
				this.absolute = false;
				break;
			case "A":
				for(var i = 0; i < l; i += 7){
					this._updateBBox(n[i + 5], n[i + 6]);
				}
				this.last.x = n[l - 2];
				this.last.y = n[l - 1];
				this.absolute = true;
				break;
			case "a":
				for(var i = 0; i < l; i += 7){
					this._updateBBox(this.last.x += n[i + 5], this.last.y += n[i + 6]);
				}
				this.absolute = false;
				break;
		}
		// add an SVG path segment
		var path = [segment.action];
		for(var i = 0; i < l; ++i){
			path.push(dojox.gfx.formatNumber(n[i], true));
		}
		if(typeof this.shape.path == "string"){
			this.shape.path += path.join("");
		}else{
			var l = path.length, a = this.shape.path;
			for(var i = 0; i < l; ++i){
				a.push(path[i]);
			}
		}
	},
	
	// a dictionary, which maps segment type codes to a number of their argemnts
	_validSegments: {m: 2, l: 2, h: 1, v: 1, c: 6, s: 4, q: 4, t: 2, a: 7, z: 0},
	
	_pushSegment: function(action, args){
		// summary: adds a segment
		// action: String: valid SVG code for a segment's type
		// args: Array: a list of parameters for this segment
		var group = this._validSegments[action.toLowerCase()];
		if(typeof group == "number"){
			if(group){
				if(args.length >= group){
					var segment = {action: action, args: args.slice(0, args.length - args.length % group)};
					this.segments.push(segment);
					this._updateWithSegment(segment);
				}
			}else{
				var segment = {action: action, args: []};
				this.segments.push(segment);
				this._updateWithSegment(segment);
			}
		}
	},
	
	_collectArgs: function(array, args){
		// summary: converts an array of arguments to plain numeric values
		// array: Array: an output argument (array of numbers)
		// args: Array: an input argument (can be values of Boolean, Number, dojox.gfx.Point, or an embedded array of them)
		for(var i = 0; i < args.length; ++i){
			var t = args[i];
			if(typeof t == "boolean"){
				array.push(t ? 1 : 0);
			}else if(typeof t == "number"){
				array.push(t);
			}else if(t instanceof Array){
				this._collectArgs(array, t);
			}else if("x" in t && "y" in t){
				array.push(t.x, t.y);
			}
		}
	},

	// segments	
	moveTo: function(){
		// summary: formes a move segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "M" : "m", args);
		return this; // self
	},
	lineTo: function(){
		// summary: formes a line segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "L" : "l", args);
		return this; // self
	},
	hLineTo: function(){
		// summary: formes a horizontal line segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "H" : "h", args);
		return this; // self
	},
	vLineTo: function(){
		// summary: formes a vertical line segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "V" : "v", args);
		return this; // self
	},
	curveTo: function(){
		// summary: formes a curve segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "C" : "c", args);
		return this; // self
	},
	smoothCurveTo: function(){
		// summary: formes a smooth curve segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "S" : "s", args);
		return this; // self
	},
	qCurveTo: function(){
		// summary: formes a quadratic curve segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "Q" : "q", args);
		return this; // self
	},
	qSmoothCurveTo: function(){
		// summary: formes a quadratic smooth curve segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "T" : "t", args);
		return this; // self
	},
	arcTo: function(){
		// summary: formes an elliptic arc segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "A" : "a", args);
		return this; // self
	},
	closePath: function(){
		// summary: closes a path
		this._pushSegment("Z", []);
		return this; // self
	},
	
	// setShape
	_setPath: function(path){
		// summary: forms a path using an SVG path string
		// path: String: an SVG path string
		var p = dojo.isArray(path) ? path : path.match(dojox.gfx.pathSvgRegExp);
		this.segments = [];
		this.absolute = true;
		this.bbox = {};
		this.last = {};
		if(!p) return;
		// create segments
		var action = "",	// current action
			args = [],		// current arguments
			l = p.length;
		for(var i = 0; i < l; ++i){
			var t = p[i], x = parseFloat(t);
			if(isNaN(x)){
				if(action){
					this._pushSegment(action, args);
				}
				args = [];
				action = t;
			}else{
				args.push(x);
			}
		}
		this._pushSegment(action, args);
	},
	setShape: function(newShape){
		// summary: forms a path using a shape
		// newShape: Object: an SVG path string or a path object (see dojox.gfx.defaultPath)
		dojox.gfx.Shape.prototype.setShape.call(this, typeof newShape == "string" ? {path: newShape} : newShape);
		var path = this.shape.path;
		// switch to non-updating version of path building
		this.shape.path = [];
		this._setPath(path);
		// switch back to the string path
		this.shape.path = this.shape.path.join("");
		return this; // self
	},
	
	// useful constant for descendants
	_2PI: Math.PI * 2
});

dojo.declare("dojox.gfx.path.TextPath", dojox.gfx.path.Path, {
	// summary: a generalized TextPath shape

	constructor: function(rawNode){
		// summary: a TextPath shape constructor
		// rawNode: Node: a DOM node to be used by this TextPath object
		if(!("text" in this)){
			this.text = dojo.clone(dojox.gfx.defaultTextPath);
		}
		if(!("fontStyle" in this)){
			this.fontStyle = dojo.clone(dojox.gfx.defaultFont);
		}
	},
	setText: function(newText){
		// summary: sets a text to be drawn along the path
		this.text = dojox.gfx.makeParameters(this.text, 
			typeof newText == "string" ? {text: newText} : newText);
		this._setText();
		return this;	// self
	},
	setFont: function(newFont){
		// summary: sets a font for text
		this.fontStyle = typeof newFont == "string" ? 
			dojox.gfx.splitFontString(newFont) :
			dojox.gfx.makeParameters(dojox.gfx.defaultFont, newFont);
		this._setFont();
		return this;	// self
	}
});

}

if(!dojo._hasResource["dojox.layout.ResizeHandle"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.layout.ResizeHandle"] = true;
dojo.provide("dojox.layout.ResizeHandle");
dojo.experimental("dojox.layout.ResizeHandle"); 


 
 

dojo.declare("dojox.layout.ResizeHandle", [dijit._Widget, dijit._Templated], {
	// summary
	//	The handle on the bottom-right corner of FloatingPane or other widgets that allows
	//	the widget to be resized.
	//	Typically not used directly.

	// targetId: String
	//	id of the Widget OR DomNode that I will size
	targetId: '',

	// targetContainer: DomNode
	//	over-ride targetId and attch this handle directly to a reference of a DomNode
	targetContainer: null, 

	// resizeAxis: String
	//	one of: x|y|xy limit resizing to a single axis, default to xy ... 
	resizeAxis: "xy",

	// activeResize: Boolean
	// 	if true, node will size realtime with mouse movement, 
	//	if false, node will create virtual node, and only resize target on mouseUp
	activeResize: false,
	
	// activeResizeClass: String
	//	css class applied to virtual resize node. 
	activeResizeClass: 'dojoxResizeHandleClone',

	// animateSizing: Boolean
	//	only applicable if activeResize = false. onMouseup, animate the node to the
	//	new size
	animateSizing: true,
	
	// animateMethod: String
	// 	one of "chain" or "combine" ... visual effect only. combine will "scale" 
	// 	node to size, "chain" will alter width, then height
	animateMethod: 'chain',

	// animateDuration: Integer
	//	time in MS to run sizing animation. if animateMethod="chain", total animation 
	//	playtime is 2*animateDuration
	animateDuration: 225,

	// minHeight: Integer
	//	smallest height in px resized node can be
	minHeight: 100,

	// minWidth: Integer
	//	smallest width in px resize node can be
	minWidth: 100,

	// resize handle template, fairly easy to override: 
	templateString: '<div dojoAttachPoint="resizeHandle" class="dojoxResizeHandle"><div></div></div>',

	// private propteries and holders
	_isSizing: false,
	_connects: [],
	_activeResizeNode: null,	
	_activeResizeLastEvent: null,
	// defaults to match default resizeAxis. set resizeAxis variable to modify. 
	_resizeX: true,
	_resizeY: true,


	postCreate: function(){
		// summary: setup our one major listener upon creation
		dojo.connect(this.resizeHandle, "onmousedown", this, "_beginSizing");
		if(!this.activeResize){ 
			this._activeResizeNode = document.createElement('div');
			dojo.addClass(this._activeResizeNode,this.activeResizeClass); 
		}else{ this.animateSizing = false; } 	

		if (!this.minSize) { 
			this.minSize = { w: this.minWidth, h: this.minHeight };
		}
		// should we modify the css for the cursor hover to n-resize nw-resize and w-resize?
		this._resizeX = this._resizeY = false; 
		switch (this.resizeAxis.toLowerCase()) {
		case "xy" : 
			this._resizeX = this._resizeY = true; 
			// FIXME: need logic to determine NW or NE class to see
			// based on which [todo] corner is clicked
			dojo.addClass(this.resizeHandle,"dojoxResizeNW"); 
			break; 
		case "x" : 
			this._resizeX = true; 
			dojo.addClass(this.resizeHandle,"dojoxResizeW");
			break;
		case "y" : 
			this._resizeY = true; 
			dojo.addClass(this.resizeHandle,"dojoxResizeN");
			break;
		}
	},

	_beginSizing: function(/*Event*/ e){
		// summary: setup movement listeners and calculate initial size
		
		if (this._isSizing){ return false; }

		this.targetWidget = dijit.byId(this.targetId);

		// FIXME: resizing widgets does weird things, disable virtual resizing for now:
		if (this.targetWidget) { this.activeResize = true; } 

		this.targetDomNode = this.targetWidget ? this.targetWidget.domNode : dojo.byId(this.targetId);
		if (this.targetContainer) { this.targetDomNode = this.targetContainer; } 
		if (!this.targetDomNode){ return; }

		if (!this.activeResize) {
			this.targetDomNode.appendChild(this._activeResizeNode); 
			dojo.fadeIn({ node: this._activeResizeNode, duration:120, 
				beforeBegin: dojo.hitch(this,function(){
					this._activeResizeNode.style.display=''; 
				})
			}).play(); 
		}

		this._isSizing = true;
		this.startPoint  = {'x':e.clientX, 'y':e.clientY};

		// FIXME: this is funky: marginBox adds height, contentBox ignores padding (expected, but foo!)
		var mb = (this.targetWidget) ? dojo.marginBox(this.targetDomNode) : dojo.contentBox(this.targetDomNode);  
		this.startSize  = { 'w':mb.w, 'h':mb.h };

		this._connects = []; 
		this._connects.push(dojo.connect(document,"onmousemove",this,"_updateSizing")); 
		this._connects.push(dojo.connect(document,"onmouseup", this, "_endSizing"));

		e.preventDefault();
	},

	_updateSizing: function(/*Event*/ e){
		// summary: called when moving the ResizeHandle ... determines 
		//	new size based on settings/position and sets styles.

		if(this.activeResize){
			this._changeSizing(e);
		}else{
			var tmp = this._getNewCoords(e);	
			if(tmp === false){ return; }
			dojo.style(this._activeResizeNode,"width",tmp.width+"px");
			dojo.style(this._activeResizeNode,"height",tmp.height+"px"); 
			this._activeResizeNode.style.display=''; 
		}
	},

	_getNewCoords: function(/* Event */ e){
		
		// On IE, if you move the mouse above/to the left of the object being resized,
		// sometimes clientX/Y aren't set, apparently.  Just ignore the event.
		try{
			if(!e.clientX  || !e.clientY){ return false; }
		}catch(e){
			// sometimes you get an exception accessing above fields...
			return false;
		}
		this._activeResizeLastEvent = e; 

		var dx = this.startPoint.x - e.clientX;
		var dy = this.startPoint.y - e.clientY;
		
		var newW = (this._resizeX) ? this.startSize.w - dx : this.startSize.w;
		var newH = (this._resizeY) ? this.startSize.h - dy : this.startSize.h;

		// minimum size check
		if(this.minSize){
			//var mb = dojo.marginBox(this.targetDomNode);
			if(newW < this.minSize.w){
				newW = this.minSize.w;
			}
			if(newH < this.minSize.h){
				newH = this.minSize.h;
			}
		}
		return {width:newW, height:newH};  // Object
	},
	
	_changeSizing: function(/*Event*/ e){
		// summary: apply sizing information based on information in (e) to attached node
		var tmp = this._getNewCoords(e);
		if(tmp===false){ return; }

		if(this.targetWidget && typeof this.targetWidget.resize == "function"){ 
			this.targetWidget.resize({ w: tmp.width, h: tmp.height });
		}else{
			if(this.animateSizing){
				var anim = dojo.fx[this.animateMethod]([
					dojo.animateProperty({
						node: this.targetDomNode,
						properties: { 
							width: { start: this.startSize.w, end: tmp.width, unit:'px' } 
						},	
						duration: this.animateDuration
					}),
					dojo.animateProperty({
						node: this.targetDomNode,
						properties: { 
							height: { start: this.startSize.h, end: tmp.height, unit:'px' }
						},
						duration: this.animateDuration
					})
				]);
				anim.play();
			}else{
				dojo.style(this.targetDomNode,"width",tmp.width+"px"); 
				dojo.style(this.targetDomNode,"height",tmp.height+"px");
			}
		}	
		e.preventDefault();
	},

	_endSizing: function(/*Event*/ e){
		// summary: disconnect listenrs and cleanup sizing
		dojo.forEach(this._connects,function(c){
			dojo.disconnect(c); 
		});
		if(!this.activeResize){
			dojo.fadeOut({ node:this._activeResizeNode, duration:250,
				onEnd: dojo.hitch(this,function(){
					this._activeResizeNode.style.display="none";
				})
			}).play();
			this._changeSizing(e);
		}
		this._isSizing = false;
	}

});

}

if(!dojo._hasResource["dojo.colors"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.colors"] = true;
dojo.provide("dojo.colors");

(function(){
	// this is a standard convertion prescribed by the CSS3 Color Module
	var hue2rgb = function(m1, m2, h){
		if(h < 0){ ++h; }
		if(h > 1){ --h; }
		var h6 = 6 * h;
		if(h6 < 1){ return m1 + (m2 - m1) * h6; }
		if(2 * h < 1){ return m2; }
		if(3 * h < 2){ return m1 + (m2 - m1) * (2 / 3 - h) * 6; }
		return m1;
	};
	
	dojo.colorFromRgb = function(/*String*/ color, /*dojo.Color?*/ obj){
		// summary:
		//		get rgb(a) array from css-style color declarations
		// description:
		//		this function can handle all 4 CSS3 Color Module formats: rgb,
		//		rgba, hsl, hsla, including rgb(a) with percentage values.
		var m = color.toLowerCase().match(/^(rgba?|hsla?)\(([\s\.\-,%0-9]+)\)/);
		if(m){
			var c = m[2].split(/\s*,\s*/), l = c.length, t = m[1];
			if((t == "rgb" && l == 3) || (t == "rgba" && l == 4)){
				var r = c[0];
				if(r.charAt(r.length - 1) == "%"){
					// 3 rgb percentage values
					var a = dojo.map(c, function(x){
						return parseFloat(x) * 2.56;
					});
					if(l == 4){ a[3] = c[3]; }
					return dojo.colorFromArray(a, obj);	// dojo.Color
				}
				return dojo.colorFromArray(c, obj);	// dojo.Color
			}
			if((t == "hsl" && l == 3) || (t == "hsla" && l == 4)){
				// normalize hsl values
				var H = ((parseFloat(c[0]) % 360) + 360) % 360 / 360,
					S = parseFloat(c[1]) / 100,
					L = parseFloat(c[2]) / 100,
					// calculate rgb according to the algorithm 
					// recommended by the CSS3 Color Module 
					m2 = L <= 0.5 ? L * (S + 1) : L + S - L * S, 
					m1 = 2 * L - m2,
					a = [hue2rgb(m1, m2, H + 1 / 3) * 256,
						hue2rgb(m1, m2, H) * 256, hue2rgb(m1, m2, H - 1 / 3) * 256, 1];
				if(l == 4){ a[3] = c[3]; }
				return dojo.colorFromArray(a, obj);	// dojo.Color
			}
		}
		return null;	// dojo.Color
	};
	
	var confine = function(c, low, high){
		// summary:
		//		sanitize a color component by making sure it is a number,
		//		and clamping it to valid values
		c = Number(c);
		return isNaN(c) ? high : c < low ? low : c > high ? high : c;	// Number
	};
	
	dojo.Color.prototype.sanitize = function(){
		// summary: makes sure that the object has correct attributes
		var t = this;
		t.r = Math.round(confine(t.r, 0, 255));
		t.g = Math.round(confine(t.g, 0, 255));
		t.b = Math.round(confine(t.b, 0, 255));
		t.a = confine(t.a, 0, 1);
		return this;	// dojo.Color
	};
})();


dojo.colors.makeGrey = function(/*Number*/ g, /*Number?*/ a){
	// summary: creates a greyscale color with an optional alpha
	return dojo.colorFromArray([g, g, g, a]);
};


// mixin all CSS3 named colors not already in _base, along with SVG 1.0 variant spellings
dojo.Color.named = dojo.mixin({
	aliceblue:	[240,248,255],
	antiquewhite:	[250,235,215],
	aquamarine:	[127,255,212],
	azure:	[240,255,255],
	beige:	[245,245,220],
	bisque:	[255,228,196],
	blanchedalmond:	[255,235,205],
	blueviolet:	[138,43,226],
	brown:	[165,42,42],
	burlywood:	[222,184,135],
	cadetblue:	[95,158,160],
	chartreuse:	[127,255,0],
	chocolate:	[210,105,30],
	coral:	[255,127,80],
	cornflowerblue:	[100,149,237],
	cornsilk:	[255,248,220],
	crimson:	[220,20,60],
	cyan:	[0,255,255],
	darkblue:	[0,0,139],
	darkcyan:	[0,139,139],
	darkgoldenrod:	[184,134,11],
	darkgray:	[169,169,169],
	darkgreen:	[0,100,0],
	darkgrey:	[169,169,169],
	darkkhaki:	[189,183,107],
	darkmagenta:	[139,0,139],
	darkolivegreen:	[85,107,47],
	darkorange:	[255,140,0],
	darkorchid:	[153,50,204],
	darkred:	[139,0,0],
	darksalmon:	[233,150,122],
	darkseagreen:	[143,188,143],
	darkslateblue:	[72,61,139],
	darkslategray:	[47,79,79],
	darkslategrey:	[47,79,79],
	darkturquoise:	[0,206,209],
	darkviolet:	[148,0,211],
	deeppink:	[255,20,147],
	deepskyblue:	[0,191,255],
	dimgray:	[105,105,105],
	dimgrey:	[105,105,105],
	dodgerblue:	[30,144,255],
	firebrick:	[178,34,34],
	floralwhite:	[255,250,240],
	forestgreen:	[34,139,34],
	gainsboro:	[220,220,220],
	ghostwhite:	[248,248,255],
	gold:	[255,215,0],
	goldenrod:	[218,165,32],
	greenyellow:	[173,255,47],
	grey:	[128,128,128],
	honeydew:	[240,255,240],
	hotpink:	[255,105,180],
	indianred:	[205,92,92],
	indigo:	[75,0,130],
	ivory:	[255,255,240],
	khaki:	[240,230,140],
	lavender:	[230,230,250],
	lavenderblush:	[255,240,245],
	lawngreen:	[124,252,0],
	lemonchiffon:	[255,250,205],
	lightblue:	[173,216,230],
	lightcoral:	[240,128,128],
	lightcyan:	[224,255,255],
	lightgoldenrodyellow:	[250,250,210],
	lightgray:	[211,211,211],
	lightgreen:	[144,238,144],
	lightgrey:	[211,211,211],
	lightpink:	[255,182,193],
	lightsalmon:	[255,160,122],
	lightseagreen:	[32,178,170],
	lightskyblue:	[135,206,250],
	lightslategray:	[119,136,153],
	lightslategrey:	[119,136,153],
	lightsteelblue:	[176,196,222],
	lightyellow:	[255,255,224],
	limegreen:	[50,205,50],
	linen:	[250,240,230],
	magenta:	[255,0,255],
	mediumaquamarine:	[102,205,170],
	mediumblue:	[0,0,205],
	mediumorchid:	[186,85,211],
	mediumpurple:	[147,112,219],
	mediumseagreen:	[60,179,113],
	mediumslateblue:	[123,104,238],
	mediumspringgreen:	[0,250,154],
	mediumturquoise:	[72,209,204],
	mediumvioletred:	[199,21,133],
	midnightblue:	[25,25,112],
	mintcream:	[245,255,250],
	mistyrose:	[255,228,225],
	moccasin:	[255,228,181],
	navajowhite:	[255,222,173],
	oldlace:	[253,245,230],
	olivedrab:	[107,142,35],
	orange:	[255,165,0],
	orangered:	[255,69,0],
	orchid:	[218,112,214],
	palegoldenrod:	[238,232,170],
	palegreen:	[152,251,152],
	paleturquoise:	[175,238,238],
	palevioletred:	[219,112,147],
	papayawhip:	[255,239,213],
	peachpuff:	[255,218,185],
	peru:	[205,133,63],
	pink:	[255,192,203],
	plum:	[221,160,221],
	powderblue:	[176,224,230],
	rosybrown:	[188,143,143],
	royalblue:	[65,105,225],
	saddlebrown:	[139,69,19],
	salmon:	[250,128,114],
	sandybrown:	[244,164,96],
	seagreen:	[46,139,87],
	seashell:	[255,245,238],
	sienna:	[160,82,45],
	skyblue:	[135,206,235],
	slateblue:	[106,90,205],
	slategray:	[112,128,144],
	slategrey:	[112,128,144],
	snow:	[255,250,250],
	springgreen:	[0,255,127],
	steelblue:	[70,130,180],
	tan:	[210,180,140],
	thistle:	[216,191,216],
	tomato:	[255,99,71],
	transparent: [0, 0, 0, 0],
	turquoise:	[64,224,208],
	violet:	[238,130,238],
	wheat:	[245,222,179],
	whitesmoke:	[245,245,245],
	yellowgreen:	[154,205,50]
}, dojo.Color.named);

}

if(!dojo._hasResource["dojox.color._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.color._base"] = true;
dojo.provide("dojox.color._base");


//	alias all the dojo.Color mechanisms
dojox.color.Color=dojo.Color;
dojox.color.blend=dojo.blendColors;
dojox.color.fromRgb=dojo.colorFromRgb;
dojox.color.fromHex=dojo.colorFromHex;
dojox.color.fromArray=dojo.colorFromArray;
dojox.color.fromString=dojo.colorFromString;

//	alias the dojo.colors mechanisms
dojox.color.greyscale=dojo.colors.makeGrey;

//	static methods
dojo.mixin(dojox.color, {
	fromCmy: function(/* Object|Array|int */cyan, /*int*/magenta, /*int*/yellow){
		//	summary
		//	Create a dojox.color.Color from a CMY defined color.
		//	All colors should be expressed as 0-100 (percentage)

		if(dojo.isArray(cyan)){
			magenta=cyan[1], yellow=cyan[2], cyan=cyan[0];
		} else if(dojo.isObject(cyan)){
			magenta=cyan.m, yellow=cyan.y, cyan=cyan.c;
		}
		cyan/=100, magenta/=100, yellow/=100;

		var r=1-cyan, g=1-magenta, b=1-yellow;
		return new dojox.color.Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	},

	fromCmyk: function(/* Object|Array|int */cyan, /*int*/magenta, /*int*/yellow, /*int*/black){
		//	summary
		//	Create a dojox.color.Color from a CMYK defined color.
		//	All colors should be expressed as 0-100 (percentage)

		if(dojo.isArray(cyan)){
			magenta=cyan[1], yellow=cyan[2], black=cyan[3], cyan=cyan[0];
		} else if(dojo.isObject(cyan)){
			magenta=cyan.m, yellow=cyan.y, black=cyan.b, cyan=cyan.c;
		}
		cyan/=100, magenta/=100, yellow/=100, black/=100;
		var r,g,b;
		r = 1-Math.min(1, cyan*(1-black)+black);
		g = 1-Math.min(1, magenta*(1-black)+black);
		b = 1-Math.min(1, yellow*(1-black)+black);
		return new dojox.color.Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	},
	
	fromHsl: function(/* Object|Array|int */hue, /* int */saturation, /* int */luminosity){
		//	summary
		//	Create a dojox.color.Color from an HSL defined color.
		//	hue from 0-359 (degrees), saturation and luminosity 0-100.

		if(dojo.isArray(hue)){
			saturation=hue[1], luminosity=hue[2], hue=hue[0];
		} else if(dojo.isObject(hue)){
			saturation=hue.s, luminosity=hue.l, hue=hue.h;
		}
		saturation/=100;
		luminosity/=100;

		while(hue<0){ hue+=360; }
		while(hue>=360){ hue-=360; }
		
		var r, g, b;
		if(hue<120){
			r=(120-hue)/60, g=hue/60, b=0;
		} else if (hue<240){
			r=0, g=(240-hue)/60, b=(hue-120)/60;
		} else {
			r=(hue-240)/60, g=0, b=(360-hue)/60;
		}
		
		r=2*saturation*Math.min(r, 1)+(1-saturation);
		g=2*saturation*Math.min(g, 1)+(1-saturation);
		b=2*saturation*Math.min(b, 1)+(1-saturation);
		if(luminosity<0.5){
			r*=luminosity, g*=luminosity, b*=luminosity;
		}else{
			r=(1-luminosity)*r+2*luminosity-1;
			g=(1-luminosity)*g+2*luminosity-1;
			b=(1-luminosity)*b+2*luminosity-1;
		}
		return new dojox.color.Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	},
	
	fromHsv: function(/* Object|Array|int */hue, /* int */saturation, /* int */value){
		//	summary
		//	Create a dojox.color.Color from an HSV defined color.
		//	hue from 0-359 (degrees), saturation and value 0-100.

		if(dojo.isArray(hue)){
			saturation=hue[1], value=hue[2], hue=hue[0];
		} else if (dojo.isObject(hue)){
			saturation=hue.s, value=hue.v, hue=hue.h;
		}
		
		if(hue==360){ hue=0; }
		saturation/=100;
		value/=100;
		
		var r, g, b;
		if(saturation==0){
			r=value, b=value, g=value;
		}else{
			var hTemp=hue/60, i=Math.floor(hTemp), f=hTemp-i;
			var p=value*(1-saturation);
			var q=value*(1-(saturation*f));
			var t=value*(1-(saturation*(1-f)));
			switch(i){
				case 0:{ r=value, g=t, b=p; break; }
				case 1:{ r=q, g=value, b=p; break; }
				case 2:{ r=p, g=value, b=t; break; }
				case 3:{ r=p, g=q, b=value; break; }
				case 4:{ r=t, g=p, b=value; break; }
				case 5:{ r=value, g=p, b=q; break; }
			}
		}
		return new dojox.color.Color({ r:Math.round(r*255), g:Math.round(g*255), b:Math.round(b*255) });	//	dojox.color.Color
	}
});

//	Conversions directly on dojox.color.Color
dojo.extend(dojox.color.Color, {
	toCmy: function(){
		//	summary
		//	Convert this Color to a CMY definition.
		var cyan=1-(this.r/255), magenta=1-(this.g/255), yellow=1-(this.b/255);
		return { c:Math.round(cyan*100), m:Math.round(magenta*100), y:Math.round(yellow*100) };		//	Object
	},
	
	toCmyk: function(){
		//	summary
		//	Convert this Color to a CMYK definition.
		var cyan, magenta, yellow, black;
		var r=this.r/255, g=this.g/255, b=this.b/255;
		black = Math.min(1-r, 1-g, 1-b);
		cyan = (1-r-black)/(1-black);
		magenta = (1-g-black)/(1-black);
		yellow = (1-b-black)/(1-black);
		return { c:Math.round(cyan*100), m:Math.round(magenta*100), y:Math.round(yellow*100), b:Math.round(black*100) };	//	Object
	},
	
	toHsl: function(){
		//	summary
		//	Convert this Color to an HSL definition.
		var r=this.r/255, g=this.g/255, b=this.b/255;
		var min = Math.min(r, b, g), max = Math.max(r, g, b);
		var delta = max-min;
		var h=0, s=0, l=(min+max)/2;
		if(l>0 && l<1){
			s = delta/((l<0.5)?(2*l):(2-2*l));
		}
		if(delta>0){
			if(max==r && max!=g){
				h+=(g-b)/delta;
			}
			if(max==g && max!=b){
				h+=(2+(b-r)/delta);
			}
			if(max==b && max!=r){
				h+=(4+(r-g)/delta);
			}
			h*=60;
		}
		return { h:h, s:Math.round(s*100), l:Math.round(l*100) };	//	Object
	},

	toHsv: function(){
		//	summary
		//	Convert this Color to an HSV definition.
		var r=this.r/255, g=this.g/255, b=this.b/255;
		var min = Math.min(r, b, g), max = Math.max(r, g, b);
		var delta = max-min;
		var h = null, s = (max==0)?0:(delta/max);
		if(s==0){
			h = 0;
		}else{
			if(r==max){
				h = 60*(g-b)/delta;
			}else if(g==max){
				h = 120 + 60*(b-r)/delta;
			}else{
				h = 240 + 60*(r-g)/delta;
			}

			if(h<0){ h+=360; }
		}
		return { h:h, s:Math.round(s*100), v:Math.round(max*100) };	//	Object
	}
});

}

if(!dojo._hasResource["dojox.color"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.color"] = true;
dojo.provide("dojox.color");


}

if(!dojo._hasResource["flow.data.XMLStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.data.XMLStore"] = true;
dojo.provide("flow.data.XMLStore");



dojo.declare("flow.data.XMLStore", null, {
  path: null,
  removeDuplicates: false,

  constructor: function(params){
    this.rootElement = params.rootElement;
    this.removeDuplicates = params.removeDuplicates;

    this.setNamespace("http://ibm.com/mashuphub");

    this.nodeHash = [];
    this.typeHash = [];
  },

  nodeHash: null,

  operator : null,

  setNamespace: function(namespace) {
  	if (this.operator) {
  		return this.operator.setNamespace(namespace);
    }

    return "";
  },

  getNode: function(item) {
    return this.nodeHash[item.id];
  },

  getNamespace: function(item) {
    return this.getNode(item).namespaceURI;
  },

  getId: function(node) {
    //return node.xmlstoreid;

    // really really really slow
    for (var id in this.nodeHash) {
      if (this.nodeHash[id] == node) {
        return id;
      }
    }

    return null;
  },

  setId: function(node, id) {
    //node.xmlstoreid = id;
    this.nodeHash[id] = node;
  },

  _fetchItems: function(request, fetchHandler, errorHandler) {
    var query = request.query;

    var items = [];
    items.push(this.getItem(this.rootElement))

    fetchHandler(items, request);
  },

  hasAttribute: function(item, attribute) {
    return this.getValue(item, attribute);
  },

  getValue: function(item, attribute, defaultvalue) {
    var node = this.getNode(item);

    if (attribute == "children") {
      if (node.nodeType == 1) {
        return (node.childNodes.length > 0 || node.attributes.length > 0)
      }

      return false;
    } else if (attribute == "canHaveChildren") {
      return (node.nodeType == 1);
    } else if (attribute == "flowtype") {
      var xpath = this.buildXPath(node, true);
      return this.operator.model.getDataType(xpath, this.operator.id);
    }
  },

  getValues: function(item, attribute, defaultvalue) {
    var elm = this.getNode(item);

    var values = [];

    if (attribute == "children") {

      for (var i = 0; i < elm.attributes.length; i++){
        var item = this.getItem(elm.attributes[i]);
        if (item) {
          values.push(item);
        }
      }

      var dupeName;
      var dupeNS;

      for (var i = 0; i < elm.childNodes.length; i++) {
        var node = elm.childNodes[i];

        // if text node and is empty, don't load
        // TODO: we used to check for nextSibling to keep last text node around
        if ((node.nodeType == 3 || node.nodeType == 4) && !dojo.trim(node.nodeValue)) {
          continue;
        }

        if (this.removeDuplicates) {
          var name = flow.xml.getLocalName(node);
          var ns = node.namespaceURI;

          if (dupeName) {
            if (name == dupeName && ns == dupeNS) {
              continue;
            }
          }

          dupeName = name;
          dupeNS = ns;
        }

        var item = this.getItem(node);
        if (item) {
          values.push(item);
        }
      }
    }

    return values;
  },

  getAttributes: function(item) {
    var attributes = [];

    if (this.getNode(item).nodeType == 1) {
      // element
      attributes.push("children");
    }

    return attributes;
  },

  getLabel: function(item) {
    var node = this.getNode(item);

    if (node.nodeType == 1) {
      return flow.xml.getLocalName(node);
    } else if (node.nodeType == 2) {
      return node.nodeName + " = " + node.nodeValue;
    } else {
      var str = node.nodeValue;

      if (str.length > 30) {
        // check for image pattern
        var imgPattern = new RegExp("<img[^>]+>");

        if (!imgPattern.test(str)) {
          str = str.substr(0, 30) + "...";
        }
      }

      return str;
    }
  },

  getTitle: function(item) {
    var node = this.getNode(item);
    var title = null;

    if (node.nodeType == 3 || node.nodeType == 4 ) {
      var str = node.nodeValue;
      if (str.length > 30) {
        title = str;
      }
    } else {
      title = node.namespaceURI;
    }

    return title;
  },

  getIdentity: function(item) {
    // return unique id
    return item.id;
  },

  getFeatures: function() {
    var features = {
      "dojo.data.api.Read": true,
      "dojo.data.api.Identity": true,
      "dojo.data.api.Write": false,
      "dojo.data.api.Notification": true
    };

    return features;
  },

  getItem: function(node) {
    if (!node) {
      return;
    }

    // We have an issue here - each node needs a unique id, but the id cannot
    // change as the tree widget caches ids.  We could use xpath, except we have
    // editable trees, so the xpath could change but the node is the same (change
    // node name, add a same named element as a sibling, etc).  So we use random
    // ids and have a lookup hash of our own.

    var id = this.getId(node);

    if (!id) {
      id = Math.floor(Math.random()*1000000);
      this.setId(node, id);
    }

    return {id: id, store: this};
  },

  isItemLoaded: function(item) {
    return true;
  },

  buildXPath: function(node, usepath, ignoreduplicatenodes, skipParent, parentOverride) {
    var xpath = "";

    while (node) {
      if (node.nodeType == 2) {
        if (ignoreduplicatenodes) {
          xpath = "/@" + node.nodeName + xpath;
        } else {
          xpath = "/@" + node.nodeName + '=\"' + node.nodeValue + '"' + xpath;
        }

        node = node.ownerElement;

        // IE doesn't support owner element
        if (!node && parentOverride) {
          node = parentOverride;
        }
      } else if (node.nodeType == 1) {
        var ns = "";
        // ns is "" in IE
        if (node.namespaceURI != null && node.namespaceURI != "") {
        	ns = this.setNamespace(node.namespaceURI)+":";
        }

        var dupecount = 1;

        // check for any previous siblings with the same name
        var parent = node.parentNode;
        var localname = flow.xml.getLocalName(node);

        // skip if parent is dataNode
        if (parent && parent != this.operator.model.dataNode) {
          for (var i = 0; i < parent.childNodes.length; i++) {
            var child = parent.childNodes[i];
            if (child == node) {
              break;
            }

            if (flow.xml.getLocalName(child) == localname && child.namespaceURI == node.namespaceURI) {
              dupecount++;
            }
          }
        }

        var position = "";
        if (dupecount > 1 && !ignoreduplicatenodes) {
          position = "["+dupecount+"]"
        }

        xpath = "/" + ns + flow.xml.getLocalName(node) + position + xpath;
        node = node.parentNode;
      }	else if (node.nodeType == 3 || node.nodeType == 4 ) {
        var dupecount = 1;
        var parent = node.parentNode;
        for (var i = 0; i < parent.childNodes.length; i++) {
          var child = parent.childNodes[i];
          if (child == node) break;
          if (child.nodeType == 3 || child.nodeType == 4)  dupecount++;
        }

        xpath = "/text()["+dupecount+"]" +xpath;
        node = node.parentNode;
      } else {
        //xpath = "/" + node.nodeValue + xpath;
        node = node.parentNode;
      }

      if (skipParent) {
        break;
      }
    }

    if (!xpath) {
      xpath = "/";
    }

    if (usepath && this.path) {
      // TODO: this code looks really wrong.
      var startpos = xpath.indexOf(this.path) + this.path.length;
      if (startpos > 0) {
        xpath = xpath.substring(startpos);
      }
    }

    return xpath;
  },

  addNewNode: function(node, parentnode) {
    var importednode = flow.xml._importNode(this.rootElement.ownerDocument, node, true);

    if (!parentnode) {
      parentnode = this.rootElement;
    }

    var parentitem = this.getItem(parentnode);

    parentnode.appendChild(importednode);
    var item = this.getItem(importednode);

    this.onNew(item, {item: parentitem, attribute: this.getAttributes(parentitem)});

    return item;
  },

  addNewAttribute: function(parentnode) {
    return this.addAttribute(parentnode, null, null);
  },

  addAttribute: function(parentnode, attrname, attrvalue) {
    if (!parentnode) {
      parentnode = this.rootElement;
    }

    var parentitem = this.getItem(parentnode);

    attrname = (attrname != null) ? attrname : "newattr";
    attrvalue = (attrvalue != null) ? attrvalue: "";

    var count = 0;
    // IE has no hasAttribute
    while (typeof(this.getDOMAttribute(parentnode, attrname)) != "undefined") {
      count++;
      attrname = "newattr"+count;
    }

    parentnode.setAttribute(attrname, attrvalue);

    var newattr = this.getDOMAttribute(parentnode, attrname);

    var item = this.getItem(newattr);

    this.onNew(item, {item: parentitem, attribute: this.getAttributes(parentitem)});

    return item;
  },

  getDOMAttribute: function(node, name) {
    var attr;

    // IE doesn't allow foo.attributes["key"] - so walk!
    for (var i = 0; i < node.attributes.length; i++) {
      if (node.attributes[i].name == name) {
        attr = node.attributes[i];
        break;
      }
    }

    return attr;
  },

  addNewTextNode: function (parentnode) {
    if (!parentnode) {
      parentnode = this.rootElement;
    }

    var parentitem = this.getItem(parentnode);
    var textnode = this.rootElement.ownerDocument.createTextNode("some text");
    parentnode.appendChild(textnode);

    var item = this.getItem(textnode);

    this.onNew(item, {item: parentitem, attribute: this.getAttributes(parentitem)});

    return item;
  },

  deleteNode: function(item,parentnode) {
    var node = this.getNode(item);

    if (node.nodeType == 1 || node.nodeType == 3 || node.nodeType == 4 ) {
      // element or textnode
      if (!node.parentNode) {
        return;
      }

      node.parentNode.removeChild(node);
    } else if (node.nodeType == 2) {
      // attribute
      if (dojo.isIE && parentnode ) parentnode.removeAttribute(node.name);
      else node.ownerElement.removeAttribute(node.name);
    }

    if (this.typeHash[item.id]) {
      delete this.typeHash[item.id];
    }

    this.onDelete(item);
  },

  cloneNode: function(node) {
    var newnode;

    if (node.namespaceURI) {
      newnode = this.rootElement.ownerDocument.createElementNS(node.namespaceURI, node.nodeName);
    } else {
      newnode = this.rootElement.ownerDocument.createElement(node.nodeName);
    }

    this.setId(newnode, this.getId(node));

    // copy over attributes
    for (var i = 0; i < node.attributes.length; i++) {
      var attr = node.attributes[i];
      newnode.setAttribute(attr.name, attr.value);

      // update hash as node has changed
      this.setId(this.getDOMAttribute(newnode, attr.name), this.getId(attr));
    }

    // copy over children
    for (var i = 0; i < node.childNodes.length; i++) {
      newnode.appendChild(this.cloneNode(node.childNodes[i]));
    }

    return newnode;
  },

  setValue: function(item, attribute, value, parentItem) {
    try   {
    var doc = this.rootElement.ownerDocument;
    var node = this.getNode(item);
    var clearChildren = false;

    if (attribute == "value") {
      if (node.nodeType == 1) {
        clearChildren = true;
        // build a new node
        var newnode;

        if (node.namespaceURI) {
          newnode = doc.createElementNS(node.namespaceURI, value);
        } else {
          newnode = doc.createElement(value);
        }

        // copy over attributes
        for (var i = 0; i < node.attributes.length; i++) {
          var attr = node.attributes[i];
          newnode.setAttribute(attr.name, attr.value);

          // update hash as node has changed
          this.setId(this.getDOMAttribute(newnode, attr.name), this.getId(attr));
        }

        // copy over children
        for (var i = 0; i < node.childNodes.length; i++) {
          newnode.appendChild(this.cloneNode(node.childNodes[i]));
        }

        // update id hash
        this.setId(newnode, item.id);

        // finally replace the old with the new
        if (this.rootElement == node) {
          this.rootElement = newnode;
        } else {
          var parent = node.parentNode;
          parent.replaceChild(newnode, node);
        }

        /*this.onDelete(item);
        item.element = newnode;
        var parentitem = null;
        if (parent) {
          parentitem = this.getItem(newnode.parentNode);
        }

        this.onNew(item, parentitem ? {item: parentitem, attribute: this.getAttributes(parentitem)} : null);*/
      } else if (node.nodeType == 2) {
        var owner = node.ownerElement;
        if (!owner) {
          owner = this.getNode(parentItem);
        }

        owner.removeAttribute(node.name)
        owner.setAttribute(value.name, value.value);

        // update the hash
        this.setId(this.getDOMAttribute(owner, value.name), item.id);

        /*var parentitem = this.getItem(owner);
        this.onDelete(item);
        item.element = owner.attributes[value.name];
        this.onNew(item, {item: parentitem, attribute: this.getAttributes(parentitem)});*/
      } else if (node.nodeType == 3 || node.nodeType == 4 ) {
        node.nodeValue = value;
        //this.onSet(item);
      }
    }

    this.onSet(item);

    //console.log(flow.xml.innerXML(this.rootElement));
    }catch(ex){}
  },

  moveNode: function(item, direction) {
    var node = this.getNode(item);
    var parent = node.parentNode;

    var targetNode = (direction == "up") ? node.previousSibling : node.nextSibling;

    var clone = node.cloneNode(true);
    parent.removeChild(node);
    this.onDelete(item);

    if (direction == "up") {
      clone = parent.insertBefore(clone, targetNode);
    } else {
      if (targetNode.nextSibling) {
        // there is another node
        clone = parent.insertBefore(clone, targetNode.nextSibling);
      } else {
        // add to end
        clone = parent.appendChild(clone);
      }
    }

    // get position
    var pos = parent.attributes.length;

    for (var i = 0; i < parent.childNodes.length; i++) {
      if (parent.childNodes[i] == clone) {
        break;
      } else {
        pos++;
      }
    }

    var parentitem = this.getItem(parent);
    item = this.getItem(clone);

    this.onNew(item, {item: parentitem, attribute: this.getAttributes(parentitem)}, pos);
    //console.log(flow.xml.innerXML(this.rootElement));
  },

  // dojo.data.api.Write
  newItem: function(newitem, parentInfo){
    var item = newitem.item;
    var node = item.store.getNode(item);

    this.addNewNode(node, parentInfo ? this.getNode(parentInfo.parent) : null);
  },

  /* dojo.data.api.Notification */
  /* Dummy functions that the store user will use */
  onSet: function(item, attribute, oldValue, newValue){
	},

  // addition - pass through the index
	onNew: function(newItem, parentInfo, index){
	},

	onDelete: function(deletedItem){
	}
});

//Mix in the simple fetch implementation to this class.
dojo.extend(flow.data.XMLStore, dojo.data.util.simpleFetch);

}

if(!dojo._hasResource["btp.widget.tool.EditDomain"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.tool.EditDomain"] = true;
dojo.provide("btp.widget.tool.EditDomain");

dojo.declare("btp.widget.tool.EditDomain", null, {
	_commandStackName: null,
	_commandStack: null,
	_defaultToolName: null,
	_defaultTool: null,
	_activeTool: null,
	_paletteViewer: null,
	
	_events: [
		"MouseDown", "MouseUp", "MouseMove", "MouseOver", "MouseOut",
		"Click", "DblClick", "KeyDown", "KeyUp",
		"DragStart", "DragOver", "DragMove", "DragEnd", "DropStart",
		"Drop", "DropEnd", "DragOut"
	],

	constructor: function(defaultTool, defaultCommandStack){
		if(!defaultTool){
			defaultTool = "btp.widget.tool.DefaultSelectionTool";
		}
		this._defaultToolName = defaultTool;

		if(!defaultCommandStack){
			defaultCommandStack = "btp.widget.command.CommandProcessor";
		}
		this._commandStackName = defaultCommandStack;

		for(var i = 0, len = this._events.length; i < len; i++){
			var eventName = this._events[i];
			var lowerName = eventName.charAt(0).toLowerCase() + eventName.substring(1);
			this[lowerName] = this._createHandler(eventName);
		}
	},

	_createHandler: function(eventName){
		return function(e, editPartViewer){
			if(this.getActiveTool()["handle"+eventName]){
				this.getActiveTool()["handle"+eventName](e, editPartViewer);
			}
		};
	},

	setActiveTool: function(tool){
		if(this._activeTool){
			this._activeTool.deactivate();
		}
		this._activeTool = tool;
		this._activeTool.setEditDomain(this);
		this._activeTool.activate();
	},
	
	getActiveTool: function(){
		if(!this._activeTool){
			this.setActiveTool(this.getDefaultTool());
		}
		return this._activeTool;
	},
	
	getDefaultTool: function(){
		if(!this._defaultTool){
			dojo["require"](this._defaultToolName);
			var cls = dojo.getObject(this._defaultToolName);
			this._defaultTool = new cls();
		}
		return this._defaultTool;
	},

	loadDefaultTool: function(){
		this.setActiveTool(this.getDefaultTool());
	},
	
	setPaletteViewer: function(paletteViewer){
		this._paletteViewer = paletteViewer;
		dojo.connect(paletteViewer, "activeToolChanged", this, "activeToolChanged");
	},

	getCommandStack: function(){
		if(!this._commandStack){
			dojo["require"](this._commandStackName);
			var cls = dojo.getObject(this._commandStackName);
			this._commandStack = new cls();
		}
		return this._commandStack;
	},
	
	setCommandStack: function(stack){
		this._commandStack = stack;
	},
	
	executeCommand: function(command){
		this.getCommandStack().push(command);
		this.getCommandStack().execute_it();
	},
	
	undo: function(){
		this.getCommandStack().undo_it();
	},
	
	redo: function(){
		this.getCommandStack().execute_it();
	},
	
	clear: function(){
		this.getCommandStack().clear();
	},
	
	activeToolChanged: function(msg){
		if(msg.tool){
			this.setActiveTool(msg.tool);
		}else{
			this.loadDefaultTool();
		}
	}
});

}

if(!dojo._hasResource["btp._base.impl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp._base.impl"] = true;
dojo.provide("btp._base.impl");

// summary:
//		Load the implementation and call its method.

dojo.mixin(btp,
{
	loadImpl: function(instance, className, method, args, ret){
		if(!className){ return null; }
		var name = className.replace(/.*\./, ""); // Ex. btp.sub.focus -> focus
		var flagName = "use"+name.charAt(0).toUpperCase() + name.substring(1); // Ex. useFocus
		if(btpConfig[flagName] == false){ return ret; }
		dojo["require"](className);
		var props = dojo.getObject(className);
		dojo.mixin(instance, props);
		if(props[method]){ // method was replaced successfully
			return instance[method].apply(instance, args);
		}
		return ret;
	},

	callImpl: function _1_btp__base_impl_callImpl(instance, className, method, args, ret){
		if(!className){ return null; }
		if(!btp._cacheFlags){ btp._cacheFlags = []; }
		var flag = btp._cacheFlags[className];
		if(flag == undefined){
			var name = className.replace(/.*\./, ""); // Ex. btp.sub.focus -> focus
			var flagName = "use"+name.charAt(0).toUpperCase() + name.substring(1); // Ex. useFocus
			flag = btp._cacheFlags[className] = btpConfig[flagName];
		}
		if(flag == false){ return ret; }
		if(!dojo._loadedModules[className]){
			dojo["require"](className);
		}

		if(!btp._cacheLoadedModules){ btp._cacheLoadedModules = []; }
		var module = btp._cacheLoadedModules[className];
		if(module == undefined){
			module = btp._cacheLoadedModules[className] = dojo.getObject(className);
		}
		return module[method].apply(instance, args);
	},

	mixinExtension: function(owner, moduleName){
		var pos = moduleName.lastIndexOf(".");
		var name = (pos == -1) ? moduleName : moduleName.substring(pos + 1);
		if(owner[name]){ return; } // already loaded
		dojo["require"](moduleName);
		var props = dojo.getObject(moduleName);
		dojo.mixin(owner, props);
		if(owner[name]){
			owner[name]();
		}
	},

	loadExtension: function(owner, moduleName){
		if(!owner.extensions){
			owner.extensions = {};
		}
		if(owner.extensions[moduleName]){ return; } // already loaded
		dojo["require"](moduleName);
		var impl = dojo.getObject(moduleName);
		return owner.extensions[moduleName] = new impl(owner);
	}
});

}

if(!dojo._hasResource["btp.widget.data.dataStub"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.data.dataStub"] = true;
dojo.provide("btp.widget.data.dataStub");


// summary:
//		This is a stub for Loader/Generator.

btp.widget.data.dataStub = {
	generate: function(widget){
		if(!this.generatorClass){ return null; }
		dojo["require"](this.generatorClass);
		var props = dojo.getObject(this.generatorClass);
		dojo.mixin(this, props);
		return this.generate(widget);
	},

    load: function(ml){
		if(!this.loaderClass){ return null; }
		dojo["require"](this.loaderClass);
		var props = dojo.getObject(this.loaderClass);
		dojo.mixin(this, props);
		return this.load(ml);
	}
};

}

if(!dojo._hasResource["btp.widget._editArea.componentInfo"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._editArea.componentInfo"] = true;
dojo.provide("btp.widget._editArea.componentInfo");

// summary:
//		Handles component definition files.
//		'this' points to the EditArea.

btp.widget._editArea.componentInfo = {
	loadComponentInfo: function(/*String*/componentInfoId){
		if(!this._componentInfo){
			this._componentInfo = {};
		}
		var arr = componentInfoId.split(",");
		for(var i = 0, len = arr.length; i < len; i++){
			var model = dijit.byId(arr[i]);
			if(!model){
				alert("Model '"+arr[i]+"' not found.");
				continue;
			}
			var selectedNode = model.selectSingleNodeFromJson("."); // <components>
			this._loadComponentInfo_1.call(this, selectedNode);
		}
	},

	_loadComponentInfo_1: function(/*Object*/selectedNode){
		var categoryNodes = btp.getChildNodes(selectedNode, "category");
		for(var i = 0, len = categoryNodes.length; i < len; i++){
			var categoryNode = categoryNodes[i]; // <category>
			var componentNodes = btp.getChildNodes(categoryNode, "component");
			for(var j = 0, len2 = componentNodes.length; j < len2; j++){
				var componentNode = componentNodes[j]; // </component>
				var name = componentNode["name"];
				if(this._componentInfo[name]){
					var k;
					for(k = 2; this._componentInfo[name+":"+k]; k++){
						;
					}
					name = name+":"+k;
				}
				if(!componentNode.attribute){
					componentNode.attribute = [];
				}else if(!dojo.isArray(componentNode.attribute)){
					componentNode.attribute = [componentNode.attribute]; // make it an array
				}
				if(componentNode.helper && !dojo.isArray(componentNode.helper)){
					componentNode.helper = [componentNode.helper]; // make it an array
				}
				this._componentInfo[name] = componentNode;
			}
		}
	},

	getComponentInfo: function(widget){
		if(!this._componentInfo){
			this.loadComponentInfo(this.componentInfoId);
		}
		var name;
		if(widget instanceof dijit._Widget){
			if(widget.__component_info){
				return widget.__component_info;
			}
			name = widget.declaredClass;
		}else if(typeof(widget) == "string"){
			name = widget;
		}
		var info = this._componentInfo[name] ? this._componentInfo[name] : {};
		if(widget instanceof dijit._Widget){
			widget.__component_info = info;
		}
		return info;
	},

	getAttributeInfo: function(widget, attrName){
		var info = widget.__component_info;
		if(!info){ return null; }
		var attrs = info.attribute;
		for(var i = 0, len = attrs.length; i < len; i++){
			if(attrs[i].name == attrName){
				return attrs[i];
			}
		}
		return null;
	},

	getAttributes: function(widget){
		var info = widget.__component_info;
		if(!info){ return null; }
		return info.attribute;
	}
};

}

if(!dojo._hasResource["btp.widget._editArea.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._editArea.focus"] = true;
dojo.provide("btp.widget._editArea.focus");

// summary:
//		Handles focus for components on EditArea.
//		'this' points to the EditArea.

btp.widget._editArea.focus = {
	setFocus: function(widget, notFocusOnEditArea){
		if(!this.focusWidgets){
			this.focusWidgets = [];
		}
		if(this.focusWidgets[0] == widget){ return; }
		if(widget == this){
			widget = null;
		}
		this.activate(true);
		if(!widget){
			this.removeFocus(null, notFocusOnEditArea);
			return;
		}
		if(this.focusWidgets[0]){
			// Hopefully, this removes focus from the currently focused widget without firing
			// the selectionChanged event even if the clicked widget has no focusable elements.
			this.removeFocus(this.focusWidgets);
		}

		if(dojo.isMozilla){
			// Put focus on the EditArea.
			// FF needs this to respond to keys such as DEL, Ctrl+Z, etc.
			// However, IE scrolls up the canvas if you do this. IE doesn't need this to respond to keys.
			this.domNode.focus();
		}

		this.setFocusFrame(widget);

		if(widget.focus){
			widget.focus();
		}else{
			try {
//				widget.domNode.focus();
			} catch(e){}
		}

		this.focusWidgets = [widget];
		this.fireSelectionChanged(widget);
	},

	setFocusFrame: function(widget){
		if(widget.hlp_setFocusFrame){
			widget.hlp_setFocusFrame();
		}
	},

	// If from is given, remove focus from it.
	// Otherwise, remove focus from the this.focusWidgets and fire the event.
	removeFocus: function(from, notFocusOnEditArea){
		if((!this.focusWidgets || this.focusWidgets.length == 0) && !from){ return; }
		var widget = from ? from[0] : this.focusWidgets[0];

		this.removeFocusFrame(widget);

		if(widget.blur){
			widget.blur();
		}else{
			if(!notFocusOnEditArea){
				/* hack: When navigating widgets with shift+tab (reverse direction),
				 * domNode.blur() causes unnecessary focus on EditArea */
				widget.domNode.blur();
			}
		}
		if(!from){
			this.focusWidgets = [];
			this.fireSelectionChanged(null);
		}
	},

	removeFocusFrame: function(widget){
		if(widget.hlp_removeFocusFrame){
			widget.hlp_removeFocusFrame();
		}
	},

	refreshFocusFrame: function(widget){
		this.removeFocusFrame(widget);
		this.setFocusFrame(widget);
	},

	deleteFocusWidget: function(){
		if(!this.focusWidgets || this.focusWidgets.length == 0){ return; }

		var focusWidgets = this.focusWidgets;
		this.removeFocus();

		// TODO: What if two widgets have parent-child relationship?
		var compoundCommand = new btp.widget.command.CompoundCommand();
		for(var i = 0, len = focusWidgets.length; i < len; i++){
			var focusWidget = focusWidgets[i];
			var parent = focusWidget.getParent();
			if(parent.update){
				compoundCommand.append(new btp.widget.command.EditCommand.UpdateCommand(parent));
				compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(focusWidget));
				compoundCommand.append(new btp.widget.command.EditCommand.UpdateCommand(parent));
			}else{
				compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(focusWidget));
			}
		}
		this.getEditDomain().executeCommand(compoundCommand);
	}
};

}

if(!dojo._hasResource["btp.widget._editArea.canvas"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._editArea.canvas"] = true;
dojo.provide("btp.widget._editArea.canvas");

// summary:
//		Canvas size adjustment.
//		'this' points to the EditArea.

btp.widget._editArea.canvas = {
	// stub: subclass must override
	adjustCanvasWidth: function(widget){
	},
	
	// stub: subclass must override
	adjustCanvasHeight: function(widget){
	},

	adjustCanvasSize: function(widget){
		var _this = this;
		if(!widget){
			widget = this;
		}
		var children = widget.getChildren ? widget.getChildren() : null;
		if(children){
			for(var i = 0, len = children.length; i < len; i++){
				var child = children[i];
				this.adjustCanvasWidth(child);
				this.adjustCanvasHeight(child);
				this.adjustCanvasSize(child);
			}
		}
	}
};

}

if(!dojo._hasResource["btp.widget._editArea.event"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._editArea.event"] = true;
dojo.provide("btp.widget._editArea.event");

// summary:
//		Publishers and Subscribers.
//		'this' points to the EditArea.

btp.widget._editArea.event = {
	// ======== Topic Publishers ====================================
	firePropertyChanged: function(propName, oldValue, newValue, /*Object?*/hint){
		var message = {
			type: "PropertyChangedNotification",
			src: this,
			propName: propName,
			oldValue: oldValue,
			newValue: newValue,
			hint: hint
		};
		dojo.publish("btp:PropertyChanged", [message]);
	},

	fireSelectionChanged: function(selection){ // cf. StructuredViewer
		var message = {
			type: "SelectionChangedNotification",
			src: this,
			selection: selection
		};
		dojo.publish("btp:SelectionChanged", [message]);
	},

	fireMessagePosted: function(text){
		var message = {
			type: "MessagePostedNotification",
			src: this,
			text: text
		};
		dojo.publish("btp:MessagePosted", [message]);
	},

	fireInputChanged: function(){
		var message = {
			type: "InputChangedNotification",
			src: this
		};
		dojo.publish("btp:InputChanged", [message]);
	},

	// ======== Topic Subscribers ====================================
	paletteSelectionChanged: function(message){
		if(!btp.matchFilter(message, this.paletteSelectionChangedFilter)){
			return; // filter out the message
		}

		if(!btp.isShowing(this)){ return; }
		this.selectedPaletteItem = message.selection;
		this.activeToolChanged(message);
	},

	activeToolChanged: function(message){
		// [Temporary Implementation]
		// EditDomain connects to this.
	},

	preferenceChanged: function(evt){
		var workbench = evt.src;
		var showGrid = workbench.getCookieValue("General/Appearance:showGrid");
		var gridSize = workbench.getCookieValue("General/Appearance:gridSize");
		var snapTo = workbench.getCookieValue("General/Appearance:snapTo");
		if(gridSize){
			this.gridWidth = gridSize;
		}
		this.snapObjectsToGrid(snapTo)
		this.displayGrid(showGrid)
	},

	propertyChanged: function(message){
		if(!btp.matchFilter(message, this.propertyChangedFilter)){
			return; // filter out the message
		}
		if(!btp.isShowing(this)){
			return; // EditArea is inactive
		}

		if(this.focusWidgets && this.focusWidgets[0]){
			this.refreshFocusFrame(this.focusWidgets[0]);
		}
	}
};

}

if(!dojo._hasResource["btp.widget._editArea.snap"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._editArea.snap"] = true;
dojo.provide("btp.widget._editArea.snap");

// summary:
//		Snap and Grid.
//		'this' points to the EditArea.

btp.widget._editArea.snap = {
	gridWidth: 10,
	displayGridFlag: false,
	snapObjectsToGridFlag: false,

	displayGrid: function(flag){
		if(flag){
			this.domNode.style.backgroundImage = "url("+dojo.moduleUrl("btp", "templates/EditArea/grid-multi.gif")+")";
		}else{
			this.domNode.style.backgroundImage = "";
		}
		this.displayGridFlag = flag;
	},

	snapObjectsToGrid: function(flag){
		this.snapObjectsToGridFlag = flag;
	},

	snap: function(pos, force){
		if(this.snapObjectsToGridFlag || force){
			var w = this.gridWidth;
			var t = pos + w/2;
	 		return t - t % w;
	 	}
	 	return pos;
	}
};

}

if(!dojo._hasResource["btp.widget._editArea.outline"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._editArea.outline"] = true;
dojo.provide("btp.widget._editArea.outline");

// summary:
//		Get or create a ContentOutlinePage
//		'this' points to the EditArea.

btp.widget._editArea.outline = {
	contentOutlinePageClass: null, // class name
	_contentOutlinePages: null, // array of ContentOutlinePage instance
	_outlineMode: "default",

	// subclass may want to override this
	getContentOutlinePageClass: function(){
		return this.contentOutlinePageClass;
	},

	getContentOutlinePage: function(){
		return this._contentOutlinePages[this._outlineMode];
	},

	createContentOutlinePage: function(){
		var mode = this._outlineMode;
		if(this._contentOutlinePages[mode]){
			return this._contentOutlinePages[mode];
		}
		var widgetName = this.getContentOutlinePageClass();
		if(!widgetName){ return null; } // no ContentOutlinePage support
		if(!dojo._loadedModules[widgetName]){
			dojo["require"](widgetName);
		}
		impl = dojo.getObject(widgetName);
		this._contentOutlinePages[mode] = new impl(this);
		return this._contentOutlinePages[mode];
	},

	getOutlineToolbarConf: function(){
		return null;
	},

	getOutlineMode: function(){
		return this._outlineMode;
	},

	setOutlineMode: function(mode){
		this._outlineMode = mode;
	}
};

}

if(!dojo._hasResource["btp.widget._editArea.provider"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._editArea.provider"] = true;
dojo.provide("btp.widget._editArea.provider");

// summary:
//		Get or create a ContentProvider
//		'this' points to the EditArea.

btp.widget._editArea.provider = {
	contentProviderClass: null, // class name
	_contentProviders: null, // array of ContentProvider instance

	// subclass may want to override this
	getContentProviderClass: function(/*String?*/mode){
		return this.contentProviderClass;
	},

	getContentProvider: function(/*String?*/mode){
		if(!mode){ mode = "default"; }
		return this._contentProviders[mode];
	},

	createContentProvider: function(/*String?*/mode){
		if(!mode){ mode = "default"; }
		if(this._contentProviders[mode]){
			return this._contentProviders[mode];
		}
		var widgetName = this.getContentProviderClass(mode);
		if(!widgetName){ return null; } // no ContentProvider support
		if(!dojo._loadedModules[widgetName]){
			dojo["require"](widgetName);
		}
		impl = dojo.getObject(widgetName);
		this._contentProviders[mode] = new impl(this);
		return this._contentProviders[mode];
	}
};

}

if(!dojo._hasResource["btp.widget.EditArea"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.EditArea"] = true;
dojo.provide("btp.widget.EditArea");















dojo.declare(
	"btp.widget.EditArea",
	[dijit._Widget, dijit._Container, dijit._Contained],
{
	isEditor: true, // indicates this is an editor

	src: '',
	label: '',
	componentInfoId: '',
	commands: '',
	target: '',
	actionInvokedFilter: '',
	paletteSelectionChangedFilter: '*',
	propertyChangedFilter: '*',

	freeLayoutMode: true,

	// subclass may want to override these to configure the EditArea
	defaultHelperClasses: null, // array of class names
	actionClasses: null, // array of class names
	ioClass: null, // class name
	loaderClass: null, // class name
	generatorClass: null, // class name
	resizeTrackerClass: "btp.widget.tool.ResizeTracker",
	dragMoveTrackerClass: "btp.widget.tool.DragGhostMoveTracker",

    __editArea: null,
	_editDomain: null,
	_moveTarget: null,
	_connectors: null,
	_events: [
		"MouseDown", "MouseUp", "MouseMove", "MouseOver", "MouseOut",
		"Click", "DblClick", "KeyDown", "KeyUp",
		"DragStart", "DragOver", "DragMove", "DragEnd", "DropStart",
		"Drop", "DropEnd", "DragOut"
	],
	
	_designTimeMethods: null,

	constructor: function(){
		this.defaultHelperClasses = [];
		this.actionClasses = [];
		this._contentOutlinePages = {};
		this._connectors = [];
	},

	destroy: function(){
		//TODO
		this.inherited("destroy", arguments);
	},

	startup: function(){
		dojo.forEach(this.getChildren(), function(child){if(child.startup){child.startup();}});
		this._started = true; // By doing this, startup() of dynamically added child will be called.
	},

	postCreate: function(){
		this.containerNode = this.domNode;
		this.domNode.className = "btpEditArea";
		this.domNode.setAttribute("tabIndex", "0");
		this.__editArea = this;
		this.hlp_dropTargets = ["component"];
//		this._editDomain = new btp.widget.tool.EditDomain("btp.widget.tool.PortalLayoutTool");
//		this._editDomain = new btp.widget.tool.EditDomain("btp.widget.tool.DefaultSelectionTool");
		if(this.commands) //TODO: Do not omit curly braces, '{' and '}'.
			this.actionInvokedFilter = this.commands;

		this.initCanvas();
		for(var i = 0, len = this._events.length; i < len; i++){
			var eventName = this._events[i];
			var handlerName = "onEa"+eventName+"Handler";
			var lowerName = eventName.charAt(0).toLowerCase() + eventName.substring(1);
			this[handlerName] = this._createHandler(lowerName);
			if(eventName.charAt(0) != "D"){
				this._connectors.push(dojo.connect(this.domNode, "on"+eventName.toLowerCase(), this, handlerName));
			}
		}

		dojo.subscribe("btp:ActionInvoked", this, "dispatchCommand");
		dojo.subscribe("btp:PaletteSelectionChanged", this, "paletteSelectionChanged");
		dojo.subscribe("btp:PreferenceChanged", this, "preferenceChanged");
		dojo.subscribe("btp:PropertyChanged", this, "propertyChanged");

		// load contents
		this.loadContents();
	},

	_createHandler: function(lowerName){
		return function(e){
			this.getEditDomain()[lowerName](e, this);
		};
	},

	initCanvas: function(){
		if(btpConfig.useDropTarget){
			var _this = this;
			dojo.addOnLoad(function(){ // lazy initializing to improve startup performance.
				dojo["require"]("btp.widget.helper.dropHelper");
				btp.weakMixin(_this, btp.widget.helper.dropHelper);
				_this.hlp_setupDropTarget();
			});
		}
	},

	getEditDomain: function(){
		return this._editDomain;
	},

	setEditDomain: function(domain){
		this._editDomain = domain;
	},

	dispatchCommand: function(message){
		if(!btp.matchFilter(message, this.actionInvokedFilter)){
			return; // filter out the message
		}
		if(!btp.isShowing(this)){ return; }

		for(var i = 0; i < this.actionClasses.length; i++){
			btp.mixinExtension(this, this.actionClasses[i]);
		}
		this.actionClasses = [];

		if(this.ioClass){
			btp.mixinExtension(this, this.ioClass);
			this.ioClass = null;
		}

		if(!this[message.command]){ return; }
		this[message.command](message);
	},

	// subclass may want to override this
	loadContents: function(){
		var url = this.src;
    	if(!url){ return false; }
		if(url.indexOf("://") == -1){
    		url = dojo.moduleUrl("btp", url).toString();
    	}
		var contents = dojo._getText(url, true);
		if(this.load(contents)){
			this.setTabTitle(url);
		}
	},

	getDragTracker: function(){
		return this.dragMoveTrackerClass;
	},

	setTabTitle: function(url){
		if(!url){ return; }
		var parent = this.getParent();
		if(!parent)
			return; // this EditArea was instantiated dynamically
		var tabPanel;
		var pane;
		if(parent.declaredClass == "btp.widget.ToolTabPanel"){
			tabPanel = parent;
			pane = this;
		}else if(parent.declaredClass == "btp.widget.EditorFrame" && parent.getParent().declaredClass == "btp.widget.ToolTabPanel"){
			tabPanel = parent.getParent();
			pane = parent;
		}else
			return;
		pane.label = btp.createLabelStringFromUrl(url);
		pane.tooltip = url;
		if(pane.tab)
			tabPanel.setTitle(pane.tab, pane.label, url);
	},

	activate: function(force){
/*
		if(!force && btp.isShowing(this)) //TODO: Do not omit curly braces, '{' and '}'.
			return;
		var page = this;
		var tabPanel = this.parent;
		if(tabPanel.widgetType == "BottomTabPanel"){
			tabPanel.selectTab(page.tab);
			page = tabPanel;
			tabPanel = tabPanel.parent;
		}
		if(tabPanel.widgetType == "TabPanel") //TODO: Do not omit curly braces, '{' and '}'.
			tabPanel.selectTab(page.tab);
//		this.domNode.focus(); // can't do this. this removes focus from a clicked component.
*/
	},

	loadComponent: function(widgetName){
		var info = this.getComponentInfo(widgetName); // Ex. widgetName: "btp.widget.Button"
		dojo["require"](widgetName);
		var impl = dojo.getObject(widgetName);
		if(!impl.__loadComponentOk){
			impl.__loadComponentOk = true;
			if(info.helper){ // add helpers to the widget prototype
				for(var i = 0, len = info.helper.length; i < len; i++){
					dojo["require"](info.helper[i]);
					var obj = dojo.getObject(info.helper[i]);
					dojo.extend(impl, obj);
				}
			}
		}
		return impl;
	},

	loadComponents: function(rootNode){
		var list = dojo.query('[dojoType]', rootNode);
		for(var i = 0, len = list.length; i < len; i++){
			this.loadComponent(list[i].getAttribute("dojoType"));
		}
	},

	createComponent: function(/*String*/name, /*Object?*/props){
		var info = this.getComponentInfo(name); // Ex. name: "btp.widget.Button"

		if(!this.hlp_getEditArea){
			dojo["require"]("btp.widget.helper.baseHelper");
			btp.weakMixin(this, btp.widget.helper.baseHelper);
		}

		// remove component index. (btp.widget.Button:1 -> btp.widget.Button)
		var widgetName = name.replace(/:.*/, "");

		var impl = this.loadComponent(widgetName);

		// set default attribute values if any
		if(!props){
			props = {};
		}
		var attrs = info.attribute;
		for(var i = 0, len = attrs ? attrs.length : 0; i < len; i++){
			var attr = attrs[i];
			if(props[attr.name]){
				continue; // respect the value of props
			}
			if(typeof(attr.value) != "undefined"){
				var type = typeof(impl.prototype[attr.name]);
				if(type == "number"){
					props[attr.name] = attr.value - 0;
				}else if(type == "boolean"){
					props[attr.name] = (attr.value == "true");
				}else if(!attr.value){
					props[attr.name] = "";
				}else{
					// get localized string
					dojo["require"]("btp.i18n");
					var s = btp.i18n.gets(attr.value, "component_"+name.replace(/\./g, "_")+"_"); // Ex. "component_com_ibm_button_Push Button"
					if(s == attr.value){
						s = btp.i18n.gets(attr.value, "component_"); // Ex. "component_Push Button"
					}
					props[attr.name] = s;
				}
			}
		}

		var widget = new impl(props);
		widget.__component_info = info;
		widget.btpComponentName = name; // name with component index.

		this.setupWidgetForEdit(widget);
		if(props.__style){
			for(var property in props.__style){
				btp.setStyleValue(widget, property, props.__style[property]);
			}
		}

		return widget;
	},

	setupWidgetsForEdit: function(widget){
		this.setupWidgetForEdit(widget);
		var children = widget.getChildren();
		for(var i = 0, len = children.length; i < len; i++){
			this.setupWidgetForEdit(children[i]);
		}
	},

	setupWidgetForEdit: function(widget){
		btp.weakMixin(widget, dijit._Contained.prototype);
		this.mixinDesignTimeObjs(widget);
		this.getComponentInfo(widget); // to assign info to each widget
		widget.hlp_setEditArea(this);
		widget.hlp_initLiveWidget(this);
	},

	getDesignTimeObjs: function(widget){
		if(!this._designTimeMethods){
			this._designTimeMethods = {};
			for(var i = 0; i < this.defaultHelperClasses.length; i++){
				var helper = this.defaultHelperClasses[i];
				dojo["require"](helper);
				var props = dojo.getObject(helper);
				dojo.mixin(this._designTimeMethods, props);
			}
		}
		return this._designTimeMethods;
	},

	mixinDesignTimeObjs: function(widget){
		var cls = dojo.getObject(widget.declaredClass);
		var mp = cls.prototype; // my prototype
		var sp = cls.superclass; // superclass's prototype
		var objs = this.getDesignTimeObjs(widget);
		for(var x in objs){
			if(x == "destroy"){
				// if the helper object has the destroy method,
				// it should be called before the widget.destroy() is called.
				if(!widget.__destructors){
					widget.__destructors = [];
				}
				widget.__destructors.push(objs.destroy);
			}else if(!widget[x] || (mp[x] && mp[x] == sp[x])){
				// x doesn't exist or x is an inherited property.
				// If the widget has x, do not mix-in the LiveWidget's x.
				// But if x is an inherited property such as addChild, override it with LiveWidget's addChild.
				// Of course, if the widget has its own addChild, it should not be overridden.
				widget[x] = objs[x];
			}
		}
		if(widget.__destructors){
			if(widget.destroy){
				widget.__destructors.push(widget.destroy);
			}
			widget.destroy = function widget_destroy(){
				alert("destroy");
				for(var i = 0, len = widget.__destructors.length; i < len; i++){
					widget.__destructors[i]();
				}
			}
		}
	},

	getFreeLayoutMode: function(){
		return this.freeLayoutMode;
	},

	getMoveTarget: function(){
		if(!this._moveTarget){
			var moveTarget = this._moveTarget = dojo.doc.createElement("DIV");
			// On IE, rubber band doesn't work smoothly if you set width and height to 0px
			// or set style.display to "none".
			moveTarget.className = "moveTarget";
			moveTarget.style.fontSize = "1px";
			moveTarget.style.width = "1px";
			moveTarget.style.height = "1px";
			moveTarget.style.position = "absolute";
		}
		return this._moveTarget;
	},

	createKnob: function(type){
		if(typeof(type) == "object" && type.tagName){
			return type;
		}else if(type == "blackSquare"){
			var div = dojo.doc.createElement("DIV");
			div.className = "resizeKnob";
			div.style.width = "4px";
			div.style.height = "4px";
			div.style.fontSize = "1px";
			div.style.backgroundColor = "black";
			div.style.position = "absolute";
			div.style.border = "1px solid white";
			div.style.zIndex = "1100";
			return div;
		}else if(type == "whiteCircle"){
			var img = dojo.doc.createElement("IMG");
			img.className = "resizeKnob";
			img.alt = "";
			img.src = dojo.moduleUrl("btp", "widget/templates/EditArea/white-circle-knob.gif")+""
			img.style.zIndex = "1100";
			img.style.position = "absolute";
			return img;
		}else if(type.match(/\.(gif|jpg|jpeg|png)$/i)){
			var img = dojo.doc.createElement("IMG");
			img.className = "resizeKnob";
			img.alt = "";
			img.src = dojo.moduleUrl("btp", type)+""
			img.style.zIndex = "1100";
			img.style.position = "absolute";
			return img;
		}
		return null; // never reached
	}
});
dojo.extend(btp.widget.EditArea, btp.widget.data.dataStub);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.componentInfo);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.focus);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.canvas);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.event);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.snap);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.outline);
dojo.extend(btp.widget.EditArea, btp.widget._editArea.provider);

}

if(!dojo._hasResource["gdl._base.utils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl._base.utils"] = true;
dojo.provide("gdl._base.utils");

// summary:
//		Utility functions.

dojo.mixin(gdl,
{
	// Return nearest parent widget that has both widget1 and widget2 as ancestors
	getOffsetParent: function(widget1, widget2){
		// first pass through the widget tree
		var temp1 = widget1;
		var temp2 = widget2;
		while(temp1){
			temp2 = widget2;
			while(temp2){
				if(temp1 == temp2){
					return temp1;
				}
				temp2 = temp2.getParent();
			}
			temp1 = temp1.getParent();
		}
		
		// second pass through the widget tree
		temp1 = widget1;
		temp2 = widget2;
		while(temp2){
			temp1 = widget1;
			while(temp1){
				if(temp1 == temp2){
					return temp1;
				}
				temp1 = temp1.getParent();
			}
			temp2 = temp2.getParent();
		}
		
		return null;
	}
});

}

if(!dojo._hasResource["gdl._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl._base"] = true;
dojo.provide("gdl._base");


}

if(!dojo._hasResource["gdl.widget._editArea.canvas"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget._editArea.canvas"] = true;
dojo.provide("gdl.widget._editArea.canvas");

// summary:
//		Canvas size adjustment.
//		'this' points to the EditArea.

gdl.widget._editArea.canvas = {

	_getCanvasMarginH: function() {
		//to remove unnecessary scrollbar
		return dojo.isIE ? 2:0;
	},
	
	_getCanvasMarginV: function() {
		//to remove unnecessary scrollbar
		return dojo.isIE ? 2:7;
	},
	
	_getMostDistantPositionFromOrigin: function(/*"vertical"|"horizontal"*/direction) {
		var isHorizontal = true;
		if(direction == "vertical") isHorizontal = false;
		
		var max = 0;
		var children = this.getChildren();
		for(var i=0; i<children.length; i++){
			var w = children[i];
			var pos = isHorizontal ? w.domNode.offsetLeft : w.domNode.offsetTop;
			var most = pos + (isHorizontal ? w.domNode.offsetWidth : w.domNode.offsetHeight);
			max = Math.max(most,max);
		}
		return max;
	},
	
	// override hale.widget.EditArea
	// so that it resize surface when canvas is extended
	// TODO : this resizes only when node is dropped to different position
	//		should also resize while resizing connection
	adjustCanvasWidth: function(widget){
		var diff = this._getCanvasMarginH();		
		var rightMost = this._getMostDistantPositionFromOrigin("horizontal");
		var editAreaRight = Math.max(rightMost, this.domNode.clientWidth);
		var dim = this.surface.getDimensions();
		if(editAreaRight != parseInt(dim.width-diff)){
			this.surface.setDimensions(editAreaRight-diff, dim.height);
		}
	},

	// override hale.widget.EditArea
	// so that it resize surface when canvas is extended
	// TODO : this resizes only when node is dropped to different position
	//		should also resize while resizing connection
	adjustCanvasHeight: function(widget){
		var diff = this._getCanvasMarginV();
		var bottomMost = this._getMostDistantPositionFromOrigin("vertical");
		
		var editAreaBottom = Math.max(bottomMost, this.domNode.clientHeight);
		var dim = this.surface.getDimensions();
		if(editAreaBottom != parseInt(dim.height-diff)){
			this.surface.setDimensions(dim.width, editAreaBottom-diff);
		}
	}
};

}

if(!dojo._hasResource["gdl.widget._editArea.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget._editArea.focus"] = true;
dojo.provide("gdl.widget._editArea.focus");

// summary:
//		Handles focus for components on EditArea.
//		'this' points to the EditArea.

gdl.widget._editArea.focus = {
	// override btp.widget._editArea.focus.deleteFocusWidget
	deleteFocusWidget: function(){
		if(!this.focusWidgets || this.focusWidgets.length == 0){ return; }

		var focusWidgets = this.focusWidgets;
		this.removeFocus();

		// TODO: What if two widgets have parent-child relationship?
		var compoundCommand = new btp.widget.command.CompoundCommand();
		for(var i = 0, len = focusWidgets.length; i < len; i++){
			var focusWidget = focusWidgets[i];
			var parent = focusWidget.getParent();
			if(parent.update){
				compoundCommand.append(new btp.widget.command.EditCommand.UpdateCommand(parent));
				compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(focusWidget));
				compoundCommand.append(new btp.widget.command.EditCommand.UpdateCommand(parent));
			}else{
				compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(focusWidget));
			}

			// remove connections
			for(var j = 0; j < 2; j++){
				var anchors = (j == 0) ? focusWidget.sourceConnectionAnchors : focusWidget.targetConnectionAnchors;
				if(!anchors){ return; }
				for(var anchorName in anchors){
					var listeners = anchors[anchorName].listeners;
					if(!listeners){ continue; }
					for(var k = 0, len2 = listeners.length; k < len2; k++){
						var connectionWidget = listeners[k];
						compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(connectionWidget));
					}
				}
			}
		}
		this.getEditDomain().executeCommand(compoundCommand);
	}
};

}

if(!dojo._hasResource["gdc.widget.GfxWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.GfxWidget"] = true;
dojo.provide("gdc.widget.GfxWidget");




// for widgets that is added to common gfx surface

// Structure of container-type nodes:
// +-----------------+
// | groupBox        |
// |    +-------+    |
// |    | shape |    |
// |    +-------+    |
// |    (child1)     |
// |    (child2)     |
// |    ...          |
// +-----------------+

// Structure of non-container-type nodes:
// +-------+
// | shape |
// +-------+

dojo.declare(
	"gdc.widget.GfxWidget",
	[dijit._Widget, dijit._Contained],
{
	shape: null, // gfx Shape object
	groupBox: null, // gfx Group object that contains 'shape' and children
	parentGroupBox: null, // dojox.gfx.Surface or dojox.gfx.Group to add this shape

	createShapeCallbacks: null, // array of functions to be called at the end of createShape()
	restoreShapeCallbacks: null, // array of functions to be called at the end of restoreShape()
	updateCallbacks: null, // array of functions to be called at the end of update()
	removeShapeCallbacks: null, // array of functions to be called at the end of removeShape()

	postCreate: function(){
		for(var n = this.domNode; n; n = n.parentNode){
			if(n.tagName == "BODY"){
				// BODY found means this widget is online, i.e. this widget
				// was instantiated with markup, not by the new operator.
				this.createShape(this.getParent().getShape());
				this.update();
				break;
			}
		}
	},

	startup: function(){
		var children = this.getChildren ? this.getChildren() : null;
		if(children){
			for(var i = 0, len = children.length; i < len; i++){
				children[i].startup();
			}
		}
	},

	// subclass may override
	createShape: function(parentGroupBox){
		this.parentGroupBox = parentGroupBox;
		if(!this.shape){ // create new
			if(this.isContainer){
				this.groupBox = parentGroupBox.createGroup();
				this.shape = this.createTargetShape(this.groupBox);
			}else{
				this.shape = this.createTargetShape(parentGroupBox);
			}
			if(this.hlp_setupDropTarget){
				this.hlp_setupDropTarget();
			}

			// assign this widget to the top level dom node
			// so that the widget can be found from dom node. (See btp._base.utils.getWidgetByElement)
			this.shape.rawNode.widget = this; // [custom property]

			// There may be pending shapes, which are created offline.
			var children = this.getChildren ? this.getChildren() : null;
			if(!children){
				this._callCallbacks(this.createShapeCallbacks);
				return;
			}
			for(var i = 0, len = children.length; i < len; i++){
				var child = children[i];
				if(child instanceof gdc.widget.GfxWidget){
					child.createShape(this.getShape());
				}
			}
		}else if(!this.getShape().getParent()){
			this.restoreShape();
		}/*else{
			// shape exists and it is online
		}*/
		this._callCallbacks(this.createShapeCallbacks);
	},

	// restore the removed shape
	restoreShape: function(){
		this.parentGroupBox.add(this.getShape());
		this.recoverLostInfo(this.shape);
		this._callCallbacks(this.restoreShapeCallbacks);
	},

	// subclass should implement
	update: function(props){
		if(props){
			dojo.mixin(this, props);
			var s = this.domNode.style;
			if(props.top){ s.top = props.top + "px"; }
			if(props.left){ s.left = props.left + "px"; }
			if(props.width){ s.width = props.width + "px"; }
			if(props.height){ s.height = props.height + "px"; }
		}
		this._callCallbacks(this.updateCallbacks);
	},

	updateRecursively: function(){
		this.update();
		var children = this.getChildren ? this.getChildren() : null;
		if(!children){ return; }
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			if(child instanceof gdc.widget.GfxWidget){
				child.update();
				child.updateRecursively();
			}
		}
	},

	removeShape: function(){
		var shape = this.getShape();
		var ret = shape ? shape.removeShape() : null;
		this._callCallbacks(this.removeShapeCallbacks);
		return ret;
	},

	// this retrieves the containing gfx shape that can be used for remove/move etc.
	getShape: function(){
		if(this.isContainer){
			return this.groupBox;
		}else{
			return this.shape;
		}
	},

	// this retrieves the target gfx shape that can be used for hit test
	getTargetShape: function(){
		return this.shape;
	},

	// subclass must implement
	createTargetShape: function(parent){
		console.debug("createTargetShape not implemented!!");
	},

	recoverLostInfo: function(shape){
		if(dojo.isIE){
			// to recover the lost information. VML's bug.
			shape.setTransform(shape.matrix).setFill(shape.fillStyle).setStroke(shape.strokeStyle);
		}
	},

	// event must be either "createShape", "restoreShape", "update", or "removeShape"
	registerCallback: function(/*String*/event, /*Function*/method){
		if(!this[event+"Callbacks"]){
			this[event+"Callbacks"] = [];
		}
		if(dojo.indexOf(this[event+"Callbacks"], method) != -1){ return; } // already registered
		this[event+"Callbacks"].push(method);
	},

	_callCallbacks: function(callbacks){
		if(!callbacks){ return; }
		for(var i = 0, len = callbacks.length; i < len; i++){
			callbacks[i].call(this);
		}
	}
});


}

if(!dojo._hasResource["gdl.widget.EditArea"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.EditArea"] = true;
dojo.provide("gdl.widget.EditArea");








dojo.declare(
	"gdl.widget.EditArea",
	btp.widget.EditArea,
{
	surface: null, // surface to add gfx shapes
	rootGroup: null, // Group object for zoom-in/out

	reconnectTrackerClass: "gdl.widget.tool.ReconnectTracker",

	// override
	freeLayoutMode: true,

	// override
	initCanvas: function(){
		this.inherited("initCanvas", arguments);
		this.initSurface();
	},

	initSurface: function(){
		this.surface = dojox.gfx.createSurface(this.domNode, "1px", "1px");
		this.rootGroup = this.groupBox = this.surface.createGroup();
	},

	getShape: function(){
		return this.groupBox;
	},

	// override
	addChild: function(/*Widget*/widget, /*int?*/insertIndex){
		this.inherited("addChild", arguments);
		if(window.gdc && widget instanceof gdc.widget.GfxWidget && this.groupBox){
			widget.createShape(this.groupBox); // subclass must prepare "this.groupBox"
			widget.updateRecursively();
		}
	},

	// override
	removeChild: function(/*Widget*/widget){
		this.inherited("removeChild", arguments);
		if(widget instanceof gdc.widget.GfxWidget){
			widget.removeShape();
		}
	}
});
dojo.extend(gdl.widget.EditArea, gdl.widget._editArea.canvas);
dojo.extend(gdl.widget.EditArea, gdl.widget._editArea.focus);

}

if(!dojo._hasResource["btp.widget.Model"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.Model"] = true;
dojo.provide("btp.widget.Model");



dojo.declare(
	"btp.widget.Model",
	dijit._Widget,
{
	src: '',
	format: '', // "json" or "xml"(default)

	ml: '',
	dom: null,
	doc: null,
	json: null,
	parsedSrc: null,

	loadXMLString: function(value){
		var xml = null;
		if((typeof dojo.global["DOMParser"]) !== "undefined"){
			var parser = new DOMParser();
			xml = parser.parseFromString(value, "text/xml");
		}else if((typeof dojo.global["ActiveXObject"]) !== "undefined"){
			xml = new ActiveXObject("Microsoft.XMLDOM");
			xml.async = false;
			xml.validateOnParse = false;
			xml.resolveExternals = false;
			if(!xml.loadXML(value)){
				var args = new Array;
				args[0] = xml.parseError.line;
				args[1] = xml.parseError.linepos;
				args[2] = xml.parseError.srcText;
				args[3] = xml.parseError.reason;
				if(args[0] == 0 && args[1] == 0){
//					return null; // maybe the file is empty
				}
				var result = "Xml error at Line: " + args[0] + 
					" Pos: " + args[1] + 
					" \nSource: " + args[2] +
					" \nReason: " + args[3];
				alert(result);
				throw new Error(result);
			}else{
				xml.setProperty("SelectionLanguage", "XPath");
			}
		}
		return xml;
	},

	loadXML: function(url){
		if(!url){ return null; }
		var xml = null;
		if((typeof dojo.global["DOMParser"]) !== "undefined"){
			var req = new XMLHttpRequest();
			req.open("GET",url,false);
			req.overrideMimeType("text/xml");
			req.send(null);
			xml = req.responseXML;
		}else if((typeof dojo.global["ActiveXObject"]) !== "undefined"){
			xml = new ActiveXObject("Microsoft.XMLDOM");
			xml.async = false;
			xml.validateOnParse = false;
			xml.resolveExternals = false;
			if(!xml.load(url)){
				var args = new Array;
				args[0] = xml.parseError.line;
				args[1] = xml.parseError.linepos;
				args[2] = xml.parseError.srcText;
				args[3] = xml.parseError.reason;
				if(args[0] == 0 && args[1] == 0){
//					return null; // maybe the file is empty
				}
				var result = "Xml error at Line: " + args[0] + 
					" Pos: " + args[1] + 
					" \nSource: " + args[2] +
					" \nReason: " + args[3] +
					" \nUrl: " + url;
				alert(result);
				throw new Error(result);
			}else{
				xml.setProperty("SelectionLanguage", "XPath");
			}	
		}
		return xml;
	},

	getInstanceDocument: function(){
		if(!this.src){ return null; }
		var prev = this.parsedSrc;
		this.parsedSrc = this.parseSrc();
		if(this.parsedSrc != prev){
			this.dom = null;
			this.json = null;
		}
		if(this.json){
			return this.json;
		}else if(this.format == "json" || this.parsedSrc.match(/\.json$/)){
			this.doc = this.json = this._getJsonObject();
			return null;
		}

		if(this.dom){
			return this.dom;
		}else{
			this.dom = this.loadXML(dojo.moduleUrl("btp", this.parsedSrc).uri);
			if(this.dom != null){
				this.doc = this.dom.documentElement;
			}
			if(navigator.appName == "Netscape"){
				this._removeWhiteSpaceNodes(this.dom);
			}
			return this.dom;
		}
	},

	_removeWhiteSpaceNodes: function(xml){
		for(var i = 0; i < xml.childNodes.length; i++){
			var child = xml.childNodes[i];
			if(child.nodeType == 1){
				this._removeWhiteSpaceNodes(child);
			}
			if(child.nodeType == 3 && child.nodeValue.match(/^\s*$/)){
				xml.removeChild(child);
				i--;
			}
		}
	},

	getDocumentElement: function(){
		if(!this.src){ return null; }
		this.getInstanceDocument();
		return this.doc;
	},

	selectSingleNode: function(ref){
		var root = this.getDocumentElement();
		if(root == null){ return null; }
		return this.evaluateXPath(root, ref)[0];
	},

	selectSingleNodeFromJson: function(ref){
		var cache = window.haleModelCache ? window.haleModelCache[this.src] : null;
		if(cache){
			this.json = cache;
		}else {
			var root = this.getDocumentElement();
			if(root == null){ return null; }
			if(!this.json){
				this._convertToJson();
			}
		}
		if(!ref || ref == "/"){
			return this.json;
		}else if(ref == "."){
			for(var i in this.json){
				return this.json[i];
			}
			return null;
		}else if(ref.charAt(0) == "/"){ // ex. "/root/users"
			var arr = ref.split("/");
			var node = this.json;
			for(var i = 1, len = arr.length; i < len; i++){
				node = node[arr[i]];
				if(!node){ return null; }
			}
			return node;
		}
		return null;
	},

	selectNodes: function(nodeset){
		var root = this.getDocumentElement();
		if(root == null){ return []; }
		return this.evaluateXPath(root, nodeset);
	},

	selectNodesFromJson: function(nodeset){
		var cache = window.haleModelCache ? window.haleModelCache[this.src] : null;
		if(cache){
			this.json = cache;
		}else {
			var root = this.getDocumentElement();
			if(root == null){ return null; }
			if(!this.json){
				this._convertToJson();
			}
		}
		if(!nodeset || nodeset == "/"){
			return [this.json];
		}else if(nodeset == "."){
			for(var i in this.json){
				return [this.json[i]];
			}
			return null;
		}else if(nodeset.charAt(0) == "/"){ // ex. "/root/users"
			var arr = nodeset.split("/");
			var node = this.json;
			for(var i = 1, len = arr.length; i < len; i++){
				node = node[arr[i]];
				if(!node){ return null; }
			}
			return dojo.isArray(node) ? node : [node];
		}
		return null;
	},

	evaluateXPath: function(node, expression){
		var arr = new Array();	
		if(navigator.appName == "Netscape"){
			var xpath = new XPathEvaluator();
			var nsnode = node.ownerDocument.documentElement;
			var resolver = xpath.createNSResolver(nsnode);
			var result = xpath.evaluate(expression, node, resolver, 0, null);
			while (res = result.iterateNext()){
				arr.push(res);
			}
		}else{
			var result = node.selectNodes(expression); 
			for(var i=0; i < result.length; ++i){
				arr.push(result[i]); 
			}	
		}
		return arr;
	},

	_getJsonObject: function(){
		var text = dojo._getText(dojo.moduleUrl("btp", this.parsedSrc).uri);
		var obj = eval('(' + text + ')');
		return obj;
	},

	_convertToJson: function(){
		if(this.json || this.format == "json" || this.parsedSrc.match(/\.json$/)){
			return;
		}
		var root = this.getDocumentElement();
		if(root == null){ return; }
		var json = {};
		json[root.nodeName] = this._convertToJson_1(root);
		this.json = json;
	},

	_convertToJson_1: function(node){
		var json = null;

		for(var i = 0, len = node.attributes.length; i < len; i++){
			if(!json) {
				json = {};
			}
			var attr = node.attributes[i];
			json["@"+attr.nodeName] = attr.nodeValue;
		}

		var len = node.childNodes.length;
		if(len == 1 && (node.firstChild.nodeType == 3 || node.firstChild.nodeType == 4)){
			// eg. <name>foo</name>
			if(!json){
				json = node.firstChild.nodeValue;
			}else{
				json["#text"] = node.firstChild.nodeValue;
			}
		}
		else{
			for(var i = 0, len; i < len; i++){
				var child = node.childNodes[i];
				if(child.nodeType != 1){
					continue;
				}
				if(!json){
					json = {};
				}
				var ret = this._convertToJson_1(child);
				if(!json[child.nodeName]){
					json[child.nodeName] = ret;
				}else{
					var tmp = json[child.nodeName];
					if(typeof(tmp) != "string" && typeof(tmp.length) != "undefined"){ // tmp is an array
						tmp.push(ret);
					}else{
						json[child.nodeName] = [tmp,ret];
					}
				}
			}
		}
		return json;
	},

	clear: function(){
		this.src = null;
		this.parsedSrc = null;
		this.dom = null;
		this.json = null;
	},

	setSrc: function(src){
		this.clear();
		this.src = src;
	},

	parseSrc: function(){
		if(!this.src || this.src.indexOf('$') == -1){
			return this.src;
		}
		var src = this.src;
		var val = "";
		while(true){
			var idx = src.indexOf("${");
			if(idx == -1){
				val += src;
				break;
			}
			var head = src.substring(0, idx);
			var tail = src.substring(idx + 2);
			val += head;
			idx = tail.indexOf("}");
			if(idx == -1){ break; }
			var ref = tail.substring(0, idx);
			var tail = tail.substring(idx + 1);
			var widget = dijit.byId(ref);
			if(widget){
				val += widget.value;
			}
			src = tail;
		}
		return val;
	},

	fireModelChanged: function(triggerWidget){
		var message = {
			type: "ModelChangedNotification",
			src: this,
			triggerWidget: triggerWidget
		};
		dojo.event.topic.publish("btp:ModelChanged", message);
	}
});

}

if(!dojo._hasResource["gdl.widget.command.ConnectionCommand"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.command.ConnectionCommand"] = true;
dojo.provide("gdl.widget.command.ConnectionCommand");

gdl.widget.command.ConnectionCommand = {
	AddConnectionCommand: function(parent, connection) {
		this._parent = parent;
		this._connection = connection;
	
		this.execute = function(){
			this._parent.hlp_addChild(this._connection);
			this._connection.getSource().onSourceConnectionAdded(this._connection);
			this._connection.getTarget().onTargetConnectionAdded(this._connection);
		}
	
		this.undo = function(){
			this._connection.getSource().onRemovingSourceConnection(this._connection);
			this._connection.getTarget().onRemovingTargetConnection(this._connection);
			this._parent.hlp_removeChild(this._connection);
		}
	},		
	
	RemoveConnectionCommand: function(connection) {
		this._connection = connection;
	
		this.execute = function(){
			this._connection.getSource().onRemovingSourceConnection(this._connection);
			this._connection.getTarget().onRemovingTargetConnection(this._connection);
			this._parent = this._connection.getParent();
			this._refChild = this._connection.getNextSibling();
			this._parent.hlp_removeChild(this._connection);
		}
	
		this.undo = function(){
			if(this._refChild){
				this._parent.hlp_addChild(this._connection, this._refChild);
			}else{
				this._parent.hlp_addChild(this._connection);
			}
			this._connection.getSource().onSourceConnectionAdded(this._connection);
			this._connection.getTarget().onTargetConnectionAdded(this._connection);
		}
	}
	
}

}

if(!dojo._hasResource["btp.i18n"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.i18n"] = true;
dojo.provide("btp.i18n");


dojo.mixin(btp.i18n,
{
	loadBundle: function(bundleName){
		if(!window.btpConfig || !btpConfig.i18nClassName){ return; }
		return btp.loadImpl(btp.i18n, btpConfig.i18nClassName, "loadBundle", arguments);
	},

	gets: function(s, prefix){
		if(!window.btpConfig || !btpConfig.i18nClassName){ return s; }
		return btp.loadImpl(btp.i18n, btpConfig.i18nClassName, "gets", arguments, s);
	}
});

}

if(!dojo._hasResource["gdl.widget.ext.disconnectPopup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.ext.disconnectPopup"] = true;
dojo.provide("gdl.widget.ext.disconnectPopup");





dojo.declare(
"gdl.widget.disconnectPopup",
null,
{
	domNode: null,	//div
	
	_iconImgSrc: "widget/templates/Toolbar/delete.gif",
	
	iconWidth: 18,
	iconHeight: 18,
	
	popupArea: null, // div(EditArea)
	
	connection: null,
			
	constructor: function(div) {
		var bundle = btp.i18n.loadBundle("gdl.widget.common");
		this.domNode = div;
		this.domNode.title = btp.i18n.gets("deleteIconTooltip");
		dojo.addClass(this.domNode, "gdlDisconnectPopup");
		this.domNode.style.cssText = "display: none;position: absolute;width: 22px;height: 22px;top: 0px;left: 0px;border-width: 1px;border-color: black;border-style: solid;background-color: white;z-index: 10000;";
		var imgsrc =  dojo.moduleUrl("btp", this._iconImgSrc)+"";
		this.domNode.style.backgroundRepeat = "no-repeat";
		this.domNode.style.backgroundImage = "url(" + imgsrc +  ")";
		dojo.connect(this.domNode, "onmouseout", this, "_onMouseOut");
		dojo.connect(this.domNode, "onclick", this, "_onClick");
	},
	
	_onMouseOut: function(e){
		if(this.connection && this.connection.origStroke){
			this.connection.shape.setStroke(this.connection.origStroke);
		}
		this.domNode.style.display = "none";
		this.connection = null;
	},
	
	_onClick: function(e){
		var command = new gdl.widget.command.ConnectionCommand.RemoveConnectionCommand(this.connection);
		var editArea = btp.getEditors()[0];
		if(editArea){
			editArea.getEditDomain().executeCommand(command);
		}
		this.domNode.style.display = "none";
	},
	
	_changeConnectionColor: function(){
		var orig = this.connection.shape.getStroke();
		var newStroke = dojo.clone(orig);
		newStroke.color = this.connection.mouseOverColor;
		this.connection.shape.setStroke(newStroke);
		return orig;
	},
		
	showPopup: function(e, connection){
		this.connection = connection;
		var points = connection.points;
		var x = (points[0].x + points[points.length-1].x) / 2;
		var y = (points[0].y + points[points.length-1].y) / 2;		
		var editArea = this.domNode.parentNode;
		var coords = dojo.coords(editArea);		
//		x+= editArea.scrollLeft - coords.x;
//		y+= editArea.scrollTop - coords.y;
		this.domNode.style.left = x -  this.iconWidth/2 + "px";
		this.domNode.style.top = y - this.iconHeight/2 + "px";
		this.domNode.style.display = "";
		
		if(connection.mouseOverColor && connection.mouseOverColor != ""){
			var origStroke = this._changeConnectionColor();
			if(!this.connection.origStroke){
				this.connection.origStroke = origStroke;
			}			
		}
	},
	
	hide: function(){
		this.domNode.style.display = "none";
		if(this.connection && this.connection.origStroke){
			this.connection.shape.setStroke(this.connection.origStroke);
		}
		this.connection = null;
	}
	
});

}

if(!dojo._hasResource["btp.widget.action.Actions"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.action.Actions"] = true;
dojo.provide("btp.widget.action.Actions");

// summary:
//		EditArea actions that are usually invoked from toolbar, menubar, etc.
//		'this' points to the EditArea.

btp.widget.action.Actions = {
	clear: function(){
		function deleteRecursively(w){
			dojo.forEach(w.getChildren(),
				function(widget){
					if(widget.getChildren){
						deleteRecursively(widget);
					}
					dijit._Contained.prototype.getParent.call(widget).removeChild(widget);
					widget.destroy();
				});
		}
		this.removeFocus();
		deleteRecursively(this);
		this.firePropertyChanged("@RemoveChild");
		this.getEditDomain().clear();
	},

	urlToBeOpenedAfterSave: function(){
		return this.getViewUrl();
	},
	save: function(){
		alert("save");
		var ml = this.generate(this);
	alert(ml);
	return;


		var url = this.getResourceUrl(null);
		if(!url){
			var props = {
				title: "Save As",
				labels: ["Name"],
				onOk: "_onSaveOk"
			};
			this.dialog = this.createInputDialog(props);
			return;
		}
		var ml = this.generate(this);
		if(!this.sendSaveRequest(ml)){ return; }
		//this.commandProcessor.clear();
		this.getEditDomain().clear();
		this.undirtied();
		if(typeof(btpConfig) != "undefined" && btpConfig.closeOnSave){
			setTimeout("window.location.href = '"+this.urlToBeOpenedAfterSave()+"';", 0);
		}else{
			this.fireMessagePosted(url+" saved.");
		}
	},

	_onSaveOk: function(){
		var resourceId = this.dialog.inputFields[0].value;
		if(!resourceId){ return; }
		this.closeDialog(this.dialog);
		var frag = btp.splitUrl(window.location.href);
		var url = frag["list"] + "/" + resourceId;
		var model = dijit.byId(this.modelId);
		model.src = url;
		this.save();
		this.setTabTitle();
	},

	urlToBeOpenedOnCancel: function(){
		var url = this.getViewUrl();
		if(!url){
			var frag = btp.splitUrl(window.location.href);
			url = frag["base"] + "/" + frag["contextRoot"] + "/";
		}
		return url;
	},
	cancel: function(){
		// Your application should implement this.
		setTimeout("window.location.href = '"+this.urlToBeOpenedOnCancel()+"';", 0);
	},

	del: function(){
		this.deleteFocusWidget();
	},

	undo: function(){
		if(this.moveImg){
			this.moveImg.style.display = "none";
		}
		//this.commandProcessor.undo_it();
		this.getEditDomain().undo();

		this.removeFocus();
	},

	redo: function(){
		if(this.moveImg){
			this.moveImg.style.display = "none";
		}
		//this.commandProcessor.execute_it();
		this.getEditDomain().redo();

		this.removeFocus();
	},

	freeLayout: function(message){
		this.freeLayoutMode = message.item.toggleOn;
	},
	
	showPreference: function(){
		var widget = dojo.widget.byType("btp:preferences")[0];
		widget.showPreferences();
	},
	
	bringFront: function(){
		if(!this.focusWidgets || !this.focusWidgets[0]){ return; }
		var parent = this.focusWidgets[0].getParent();
		var children = parent.getChildren();
		var cur;
		var len = children.length;
		if(len < 2){ return; }
		for(var i=0; i<len; i++){
			cur = children[i];
			if(cur == this.focusWidgets[0]){
				if(i == len-1){ return; }
				var ref = len-2 == i ? null : children[i+2];
				var compoundCommand = new btp.widget.command.CompoundCommand();
				compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(cur));
				if(ref){
					compoundCommand.append(new btp.widget.command.EditCommand.InsertBeforeCommand(parent, cur, ref));
				}else{
					compoundCommand.append(new btp.widget.command.EditCommand.AppendChildCommand(parent, cur));
				}
				this.getEditDomain().executeCommand(compoundCommand);
				return;
			}
		}
	},
	
	bringBack: function(){
		if(!this.focusWidgets || !this.focusWidgets[0]){ return; }
		var parent = this.focusWidgets[0].getParent();
		var children = parent.getChildren();
		var cur;
		var len = children.length;
		if(len < 2){ return; }
		for(var i=0; i<len; i++){
			cur = children[i];
			if(cur == this.focusWidget){
				if(i==0){ return; }
				var ref = children[i-1];
				var compoundCommand = new btp.widget.command.CompoundCommand();
				compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(cur));
				compoundCommand.append(new btp.widget.command.EditCommand.InsertBeforeCommand(parent, cur, ref));
				this.getEditDomain().executeCommand(compoundCommand);
				return;
			}
		}
	}
};

}

if(!dojo._hasResource["flow.hale.EditArea"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.hale.EditArea"] = true;
dojo.provide("flow.hale.EditArea");





dojo.declare(
	"flow.hale.EditArea",
	gdl.widget.EditArea,
{
  canvasmodel: null,

  popup: null,

	constructor: function() {
		this.defaultHelperClasses = [
			"btp.widget.helper.baseHelper",
			"btp.widget.helper.focusHelper",
			"btp.widget.helper.styleHelper",
			"btp.widget.helper.dropHelper",
			"btp.widget.propsrc.componentPropertySource"
		];
		this.actionClasses = [
			"btp.widget.action.Actions"
		];
		this.loaderClass = "btp.widget.data.htmlLoader";
		this.generatorClass = "btp.widget.data.htmlGenerator";
		this.contentProviderClass = "btp.widget.provider.GfxNodeTreeContentProvider";

		// contentOutlinePageClass: Specifies a content outline page class which creates an
		// outline of the EditArea to be displayed on the Outline view.
		// TreeContentOutlinePage creates a tree view by traversing a widget tree on the EditArea.
		// CloneContentOutlinePage creates a pane that displays a copy of the EditArea.
		// CloneContentOutlinePage is just an experiment, not for practical use.
		//
		//this.contentOutlinePageClass = "btp.widget.outline.CloneContentOutlinePage";
		this.contentOutlinePageClass = "ged.widget.outline.CloneContentOutlinePage";

		// dragMoveTrackerClass: Specifies a DragTracker class that moves a component on the EditArea.
		// DragMoveTracker moves an actual component while dragging.
		// DragGhostMoveTracker moves a dashed bounding box of the component just like eclipse GEF.
		//
		//this.dragMoveTrackerClass = "btp.widget.tool.DragGhostMoveTracker";
		this.dragMoveTrackerClass = "btp.widget.tool.DragMoveTracker";
	},

  startup: function () {
    
    this.setEditDomain(new btp.widget.tool.EditDomain());
    this.getEditDomain().setPaletteViewer(this);

    this.inherited( "startup" , arguments );

		
		var div = dojo.doc.createElement("div");
		this.popup = new gdl.widget.disconnectPopup(div);

    
    this.popup.domNode.title = dojo.i18n.getLocalization("flow", "labels").CONNECTION_LINE_DELETE;

		this.domNode.appendChild(this.popup.domNode);
  },

	postCreate: function(){
		this.inherited("postCreate", arguments);
		this.resizeConn = dojo.connect(this, "onEaMouseOverHandler", this, "loadResizeExt");
		this.setOutlineMode("outlineMode");
	},

	loadResizeExt: function(){
		this.disconnect(this.resizeConn);

		// selectionHandles displays selection handles for a resizable component when it is focused.
		// Events for the handles are handled by a DragTracker (ResizeTracker).
		btp.loadExtension(this, "btp.widget.ext.selectionHandles");

		// selectionKnobType: Specifies a selection knob type from one of the followings.
		// "blackSquare" - A small black square, which is made of a <div> element. This is the default.
		// "whiteCircle" - A small circle just like a PowerPoint's selection handle. Made of an <img> element.
		// path - A path of an image file relative to the btp folder.
		// element - A DOM element.
		//
		// Some examples:
		//this.extensions["btp.widget.ext.selectionHandles"].selectionKnobType = "blackSquare";
		//this.extensions["btp.widget.ext.selectionHandles"].selectionKnobType = "whiteCircle";
		//this.extensions["btp.widget.ext.selectionHandles"].selectionKnobType = "widget/templates/EditArea/handle.gif";
		//var img = dojo.doc.createElement("IMG");
		//img.src = dojo.moduleUrl("btp", "widget/templates/EditArea/handle.gif")+""
		//img.style.zIndex = "1100";
		//img.style.position = "absolute";
		//this.extensions["btp.widget.ext.selectionHandles"].selectionKnobType = img;


		// connectionEndpoints places handles at the two ends of a gfx line widget.
		// Events for the handles are handled by a DragTracker (ReconnectTracker).
		btp.loadExtension(this, "gdl.widget.ext.connectionEndpoints");

		// endpointKnobType: Specifies an endpoint knob type.
		// Possible types are the same as those of selectionKnobType.
		//this.extensions["gdl.widget.ext.connectionEndpoints"].endpointKnobType = "blackSquare";
		var div = dojo.doc.createElement("DIV");
		div.className = "endpointKnob";
		div.style.fontSize = "1px";
		div.style.position = "absolute";
		div.style.zIndex = "1100";
		if(dojo.isIE) {
			div.style.backgroundColor = "#FFF";
			div.style.filter = "Alpha(opacity=0)";
		}
		this.extensions["gdl.widget.ext.connectionEndpoints"].endpointKnobType = div;
	},

	// override
	getContentOutlinePageClass: function(){
		this.contentOutlinePageClass = "gdl.widget.outline.CloneContentOutlinePage";
		this.contentProviderClass = "btp.widget.provider.GfxNodeTreeContentProvider";
		return this.contentOutlinePageClass;
	},

	getOutlineToolbarConf: function(){
		var json = {
		    toolBar: {
		      item: [{
		        "@command": "changeOutlineMode",
		        "@arg": "outlineMode",
		        "@tooltip": "Outline Mode",
		        "@radio": "true",
		        "@name": "outline1",
		        "@selected": "true",
		        "@iconPath": "widget/templates/OutlineView/outline.gif"
		      },
		      {
		        "@command": "changeOutlineMode",
		        "@arg": "overviewMode",
		        "@tooltip": "Overview Mode",
		        "@radio": "true",
		        "@name": "outline1",
		        "@iconPath": "widget/templates/OutlineView/overview.gif"
		      }]
		    }
		};
		return json;
	},

  propertyChanged: function(message) {
    if (message.propName == "@AddChild") {
      var widget = message.hint.widget;

      // new element added
      if (widget.declaredClass == "flow.dijit.CanvasOperator") {
        this.addOperatorToContextMenu(widget);
      } else if (widget.declaredClass == "flow.hale.CurvylineConnection") {
        // new connection
      }

    }

    this.inherited("propertyChanged", arguments);
  },

  clear: function() {
    
    btp.widget.action.Actions.clear.apply(this);
  },

  getParent: function() {
    // TODO: Hale dies if getParent doesn't exist
    return null;
  }
});

}

if(!dojo._hasResource["flow.operator.Data"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.operator.Data"] = true;
dojo.provide("flow.operator.Data");

/*
  Data class passed around between operators and other UI components
*/

dojo.declare("flow.operator.BaseParameterData", null, {
  // required
  id: null,
  type: null,

  // optional
  position: null,
  comments: null,
  caption: null,
  inputid: null
});

dojo.declare("flow.operator.ParameterData", flow.operator.BaseParameterData, {
  type: "text",
  value: null,

  constructor: function(type, id) {
    this.id = id;

    if (type) {
      this.type = type;
    }
  }
});

dojo.declare("flow.operator.DynamicParameterData", flow.operator.BaseParameterData, {
  type: "dynamic",
  value: null,
  name: null,

  constructor: function(id) {
    this.id = id;
  }
});

dojo.declare("flow.operator.FunctionParameterData", flow.operator.BaseParameterData, {
  type: "function",

  name: null,
  parameters: null,

  constructor: function(id) {
    this.id = id;
  }
});

flow.operator.Data = {
  create: function(type, id) {
    if (type == "function") {
      return new flow.operator.FunctionParameterData(id);
    } else if (type == "dynamic") {
      return new flow.operator.DynamicParameterData(id);
    } else {
      return new flow.operator.ParameterData(type, id);
    }
  }
}

}

if(!dojo._hasResource["flow.dijit.XMLViewTree"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.XMLViewTree"] = true;
dojo.provide("flow.dijit.XMLViewTree");





dojo.declare(
  "flow.dijit.XMLViewTree",
  dijit.Tree,
  {
    xmlRootNode: null,
    store: null,
    path: null,
    operator: null,
    removeDuplicates: false,
    selectedXPath: null,

    selectedItem: null,
    selectedTreeNode: null,

    imagePath: dojo.moduleUrl("flow.dijit", "templates/images"),

    postMixInProperties: function() {
      // setup the store
      if (!this.xmlRootNode) {
        var x = document.createElement("root");
        x.setAttribute("id", " ");

        this.store = new flow.data.XMLStore({rootElement: x});
      } else {
        this.store = new flow.data.XMLStore({rootElement: this.xmlRootNode, removeDuplicates: this.removeDuplicates});
      }

      if (this.operator) {
        this.store.operator = this.operator;
      }

      if (this.path) {
        this.store.path = this.path;
      }

      flow.dijit.XMLViewTree.superclass.postMixInProperties.call(this);
    },

    postCreate: function() {
      this.inherited("postCreate", arguments);

      // make sure root is expanded
      var children = this.getChildren();
      if (!children[0].isExpanded && children[0].isExpandable) {
        this._expandNode(children[0])
      }

      dojo.addClass(this.domNode, "flowXMLTree");

      // if selected xpath exists, open tree
      if (this.selectedXPath) {
        var expandedXPath = this.selectedXPath.split("/");
        var count = 1;
        var node = children[0];

        while (count < expandedXPath.length) {
          if (!node.isExpandable) {
            // should never happen
            break;
          }

          var children = node.getChildren();
          for (var i = 0; i < children.length; i++) {
            var xpath = this.store.buildXPath(this.store.getNode(children[i].item), false, null, true);

            if (xpath == ("/" + expandedXPath[count])) {
              node = children[i];

              if (count < expandedXPath.length-1) {
                this._expandNode(node)
              }
              break;
            }
          }

          count++;
        }

      }
    },

    onClick: function(item, node) {
      if (this.selectedTreeNode) {
        dojo.removeClass(this.selectedTreeNode.labelNode, "selected");
      }

      this.selectedItem = item;
      this.selectedTreeNode = node;

      dojo.addClass(this.selectedTreeNode.labelNode, "selected");
    },

    getIconClass: function(item) {
      if (!item) {
        return;
      }

      var type = "flowtype ";

      // called to get the icon for a tree
      var node = this.store.getNode(item);
      if (node.nodeType == 1) {
        type += "flowtypeElement";
      } else if (node.nodeType == 2) {
        type += "flowtypeAttribute";
      } else if (node.nodeType == 3 || node.nodeType == 4) {
        type += "flowtypeText";
      }

      return type;
    },

    getTreeNodeByItem: function(item) {
      return this._itemNodeMap[this.store.getIdentity(item)];
    },

    getXPathForTreeItem: function(item) {
      var node = this.store.getNode(item);

      var parentnode;

      if (dojo.isIE && node.nodeType == 2) {
        // IE doesn't support getting parent of an attribute node
        parentnode = this.store.getNode(this.getTreeNodeByItem(item).getParent().item);
      }

      return this.store.buildXPath(node, true, true, null, parentnode);
    }
  }
);

// update tree node for our needs (type stuff)
var treenodepostcreate = dijit._TreeNode.prototype.postCreate;
dojo.extend(dijit._TreeNode, {
  postCreate: function() {
    treenodepostcreate.apply(this, arguments);
    if (this.tree.isTypeTree && !this.isTree) {
      var datatype = this.tree.store.getValue(this.item, "flowtype");
      this.tree.setType(datatype, this);
    }

    // image check
    var content = dojox.data.dom.textContent(this.labelNode);
    var imgPattern = new RegExp("<img[^>]+>");
    if (imgPattern.test(content)) {
      this.labelNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(content);
    }

    if (this.item) {
      var title = this.item.store.getTitle(this.item);
      if (title) {
        this.labelNode.setAttribute("title", title);
      }
    }
  }
});



}

if(!dojo._hasResource["dijit._tree.dndContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._tree.dndContainer"] = true;
dojo.provide("dijit._tree.dndContainer");



dojo.declare("dijit._tree.dndContainer",
	null, 
	{
		constructor: function(tree, params){
			// summary: a constructor of the Container
			// tree: Node: node or node's id to build the container on
			// params: Object: a dict of parameters, which gets mixed into the object
			this.tree = tree;
			this.node = tree.domNode;
			dojo.mixin(this, params);
	
			// class-specific variables
			this.map = {};
			this.current = null;
	
			// states
			this.ContainerState = "";
			dojo.addClass(this.node, "dojoDndContainer");
			
			// mark up children
			if(!(params && params._skipStartup)){
				this.startup();
			}

			// set up events
			this.events = [
				dojo.connect(this.node, "onmouseover", this, "onMouseOver"),
				dojo.connect(this.node, "onmouseout",  this, "onMouseOut"),

				// cancel text selection and text dragging
				dojo.connect(this.node, "ondragstart",   dojo, "stopEvent"),
				dojo.connect(this.node, "onselectstart", dojo, "stopEvent")
			];
		},


		// abstract access to the map
		getItem: function(/*String*/ key){
			// summary: returns a data item by its key (id)
			//console.log("Container getItem()", arguments,this.map, this.map[key], this.selection[key]);
			return this.selection[key];
			//return this.map[key];	// Object
		},

		// mouse events
		onMouseOver: function(e){
			// summary: event processor for onmouseover
			// e: Event: mouse event
			var n = e.relatedTarget;
			while(n){
				if(n == this.node){ break; }
				try{
					n = n.parentNode;
				}catch(x){
					n = null;
				}
			}
			if(!n){
				this._changeState("Container", "Over");
				this.onOverEvent();
			}
			n = this._getChildByEvent(e);
			if(this.current == n){ return; }
			if(this.current){ this._removeItemClass(this.current, "Over"); }
			if(n){ this._addItemClass(n, "Over"); }
			this.current = n;
		},

		onMouseOut: function(e){
			// summary: event processor for onmouseout
			// e: Event: mouse event
			for(var n = e.relatedTarget; n;){
				if(n == this.node){ return; }
				try{
					n = n.parentNode;
				}catch(x){
					n = null;
				}
			}
			if(this.current){
				this._removeItemClass(this.current, "Over");
				this.current = null;
			}
			this._changeState("Container", "");
			this.onOutEvent();
		},

		_changeState: function(type, newState){
			// summary: changes a named state to new state value
			// type: String: a name of the state to change
			// newState: String: new state
			var prefix = "dojoDnd" + type;
			var state  = type.toLowerCase() + "State";
			//dojo.replaceClass(this.node, prefix + newState, prefix + this[state]);
			dojo.removeClass(this.node, prefix + this[state]);
			dojo.addClass(this.node, prefix + newState);
			this[state] = newState;
		},

                _getChildByEvent: function(e){
                        // summary: gets a child, which is under the mouse at the moment, or null
                        // e: Event: a mouse event
                        var node = e.target;

                        if (node && dojo.hasClass(node,"dijitTreeLabel")){
                                return node;
                        }
                        return null;
                },

		markupFactory: function(tree, params){
			params._skipStartup = true;
			return new dijit._tree.dndContainer(tree, params);
		},

		_addItemClass: function(node, type){
			// summary: adds a class with prefix "dojoDndItem"
			// node: Node: a node
			// type: String: a variable suffix for a class name
			dojo.addClass(node, "dojoDndItem" + type);
		},

		_removeItemClass: function(node, type){
			// summary: removes a class with prefix "dojoDndItem"
			// node: Node: a node
			// type: String: a variable suffix for a class name
			dojo.removeClass(node, "dojoDndItem" + type);
		},

		onOverEvent: function(){
			// summary: this function is called once, when mouse is over our container
			console.log("onOverEvent parent");
		},

		onOutEvent: function(){
			// summary: this function is called once, when mouse is out of our container
		}
});

}

if(!dojo._hasResource["dijit._tree.dndSelector"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._tree.dndSelector"] = true;
dojo.provide("dijit._tree.dndSelector");



dojo.declare("dijit._tree.dndSelector",
	dijit._tree.dndContainer,
	{
		constructor: function(tree, params){
			this.selection={};
			this.anchor = null;
			this.simpleSelection=false;
		
			this.events.push(
				dojo.connect(this.tree.domNode, "onmousedown", this,"onMouseDown"),
				dojo.connect(this.tree.domNode, "onmouseup", this,"onMouseUp")
			);
		},
	
		// object attributes (for markup)
		singular: false,	// is singular property
	
		// methods
		getSelectedItems: function(){
			var selectedItems = []
			for (var i in this.selection){
				selectedItems.push(dijit.getEnclosingWidget(this.selection[i]).item);
			}
			return selectedItems;
		},

		getSelectedNodes: function(){
			return this.selection;
		},

		selectNone: function(){
			// summary: unselects all items
			return this._removeSelection()._removeAnchor();	// self
		},

		insertItems: function(item, parent){
			// summary: inserts new data items (see Container's insertNodes method for details)
			
			//we actually need to add things to the store here instead of adding noes to the tree directly		
		},

		destroy: function(){
			// summary: prepares the object to be garbage-collected
			dojo.dnd.Selector.superclass.destroy.call(this);
			this.selection = this.anchor = null;
		},

		// mouse events
		onMouseDown: function(e){
			// summary: event processor for onmousedown
			// e: Event: mouse event
			if(!this.current){ return; }

			var item = dijit.getEnclosingWidget(this.current).item
			var id = this.tree.store.getIdentity(item);

			if (!this.current.id) {
				this.current.id=id;
			}

			if (!this.current.type) {
				this.current.type="data";
			}

			if(!this.singular && !dojo.dnd.getCopyKeyState(e) && !e.shiftKey && (this.current.id in this.selection)){
				this.simpleSelection = true;
				dojo.stopEvent(e);
				return;
			}

			if(this.singular){
				if(this.anchor == this.current){
					if(dojo.dnd.getCopyKeyState(e)){
						this.selectNone();
					}
				}else{
					this.selectNone();
					this.anchor = this.current;
					this._addItemClass(this.anchor, "Anchor");

					this.selection[this.current.id] = this.current;
				}
			}else{
				if(!this.singular && e.shiftKey){	
					if (dojo.dnd.getCopyKeyState(e)){
						//TODO add range to selection
					}else{
						//TODO select new range from anchor 
					}
				}else{
					if(dojo.dnd.getCopyKeyState(e)){
						if(this.anchor == this.current){
							delete this.selection[this.anchor.id];
							this._removeAnchor();
						}else{
							if(this.current.id in this.selection){
								this._removeItemClass(this.current, "Selected");
								delete this.selection[this.current.id];
							}else{
								if(this.anchor){
									this._removeItemClass(this.anchor, "Anchor");
									this._addItemClass(this.anchor, "Selected");
								}
								this.anchor = this.current;
								this._addItemClass(this.current, "Anchor");
								this.selection[this.current.id] = this.current;
							}
						}
					}else{
						var item = dijit.getEnclosingWidget(this.current).item
						var id = this.tree.store.getIdentity(item);
						if(!(id in this.selection)){
							this.selectNone();
							this.anchor = this.current;
							this._addItemClass(this.current, "Anchor");
							this.selection[id] = this.current;
						}
					}
				}
			}

			dojo.stopEvent(e);
		},

		onMouseMove: function() {

		},

		onOverEvent: function() {
			this.onmousemoveEvent = dojo.connect(this.node, "onmousemove", this, "onMouseMove");
		},

		onMouseUp: function(e){
			// summary: event processor for onmouseup
			// e: Event: mouse event
			if(!this.simpleSelection){ return; }
			this.simpleSelection = false;
			this.selectNone();
			if(this.current){
				this.anchor = this.current;
				this._addItemClass(this.anchor, "Anchor");
				this.selection[this.current.id] = this.current;
			}
		},
		_removeSelection: function(){
			// summary: unselects all items
			var e = dojo.dnd._empty;
			for(var i in this.selection){
				if(i in e){ continue; }
				var node = dojo.byId(i);
				if(node){ this._removeItemClass(node, "Selected"); }
			}
			this.selection = {};
			return this;	// self
		},
		_removeAnchor: function(){
			if(this.anchor){
				this._removeItemClass(this.anchor, "Anchor");
				this.anchor = null;
			}
			return this;	// self
		}
});

}

if(!dojo._hasResource["dijit._tree.dndSource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._tree.dndSource"] = true;
dojo.provide("dijit._tree.dndSource");




dojo.declare("dijit._tree.dndSource", dijit._tree.dndSelector, {
	// summary: a Source object, which can be used as a DnD source, or a DnD target
	
	// object attributes (for markup)
	isSource: true,
	copyOnly: false,
	skipForm: false,
	accept: ["text"],
	
	constructor: function(node, params){
		// summary: a constructor of the Source
		// node: Node: node or node's id to build the source on
		// params: Object: a dict of parameters, recognized parameters are:
		//	isSource: Boolean: can be used as a DnD source, if true; assumed to be "true" if omitted
		//	accept: Array: list of accepted types (text strings) for a target; assumed to be ["text"] if omitted
		//	horizontal: Boolean: a horizontal container, if true, vertical otherwise or when omitted
		//	copyOnly: Boolean: always copy items, if true, use a state of Ctrl key otherwise
		//	skipForm: Boolean: don't start the drag operation, if clicked on form elements
		//	the rest of parameters are passed to the selector
		if(!params){ params = {}; }
		dojo.mixin(this, params);
		this.isSource = typeof params.isSource == "undefined" ? true : params.isSource;
		var type = params.accept instanceof Array ? params.accept : ["text"];
		this.accept = null;
		if(type.length){
			this.accept = {};
			for(var i = 0; i < type.length; ++i){
				this.accept[type[i]] = 1;
			}
		}

		// class-specific variables
		this.isDragging = false;
		this.mouseDown = false;
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;

		// states
		this.sourceState  = "";
		if(this.isSource){
			dojo.addClass(this.node, "dojoDndSource");
		}
		this.targetState  = "";
		if(this.accept){
			dojo.addClass(this.node, "dojoDndTarget");
		}
		if(this.horizontal){
			dojo.addClass(this.node, "dojoDndHorizontal");
		}
		// set up events
		this.topics = [
			dojo.subscribe("/dnd/source/over", this, "onDndSourceOver"),
			dojo.subscribe("/dnd/start",  this, "onDndStart"),
			dojo.subscribe("/dnd/drop",   this, "onDndDrop"),
			dojo.subscribe("/dnd/cancel", this, "onDndCancel")
		];
	},

	startup: function(){
	},
	
	// methods
	checkAcceptance: function(source, nodes){
		// summary: checks, if the target can accept nodes from this source
		// source: Object: the source which provides items
		// nodes: Array: the list of transferred items
		return true;	// Boolean
	},
	copyState: function(keyPressed){
		// summary: Returns true, if we need to copy items, false to move.
		//		It is separated to be overwritten dynamically, if needed.
		// keyPressed: Boolean: the "copy" was pressed
		return this.copyOnly || keyPressed;	// Boolean
	},
	destroy: function(){
		// summary: prepares the object to be garbage-collected
		this.inherited("destroy",arguments);
		dojo.forEach(this.topics, dojo.unsubscribe);
		this.targetAnchor = null;
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dijit._tree.dndSource(node, params);
	},

	// mouse event processors
	onMouseMove: function(e){
		// summary: event processor for onmousemove
		// e: Event: mouse event
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		this.inherited("onMouseMove", arguments);
		var m = dojo.dnd.manager();
		if(this.isDragging){
			// calculate before/after

			if (this.allowBetween){ // not implemented yet for tree since it has no concept of order
				var before = false;
				if(this.current){
					if(!this.targetBox || this.targetAnchor != this.current){
						this.targetBox = {
							xy: dojo.coords(this.current, true),
							w: this.current.offsetWidth,
							h: this.current.offsetHeight
						};
					}
					if(this.horizontal){
						before = (e.pageX - this.targetBox.xy.x) < (this.targetBox.w / 2);
					}else{
						before = (e.pageY - this.targetBox.xy.y) < (this.targetBox.h / 2);
					}
				}
				if(this.current != this.targetAnchor || before != this.before){
					this._markTargetAnchor(before);
					m.canDrop(!this.current || m.source != this || !(this.current.id in this.selection));
				}
			}
		}else{
			if(this.mouseDown && this.isSource){
				var n = this.getSelectedNodes();
				var nodes=[];
				for (var i in n){
					nodes.push(n[i]);
				}
				if(nodes.length){
					m.startDrag(this, nodes, this.copyState(dojo.dnd.getCopyKeyState(e)));
				}
			}
		}
	},

	onMouseDown: function(e){
		// summary: event processor for onmousedown
		// e: Event: mouse event
		this.mouseDown = true;
		this.mouseButton = e.button;
		this.inherited("onMouseDown",arguments);
	},

	onMouseUp: function(e){
		// summary: event processor for onmouseup
		// e: Event: mouse event
		if(this.mouseDown){
			this.mouseDown = false;
			this.inherited("onMouseUp",arguments);
		}
	},

	onMouseOver: function(e){
		// summary: event processor for onmouseover
		// e: Event: mouse event
		var n = e.relatedTarget;
		while(n){
			if(n == this.node){ break; }
			try{
				n = n.parentNode;
			}catch(x){
				n = null;
			}
		}
		if(!n){
			this._changeState("Container", "Over");
			this.onOverEvent();
		}
		n = this._getChildByEvent(e);
		if(this.current == n){ return; }
		if(this.current){ this._removeItemClass(this.current, "Over"); }
		if(n){ 
			this._addItemClass(n, "Over"); 
			if(this.isDragging){
				var m = dojo.dnd.manager();
				if(this.checkItemAcceptance(n,m.source)){
					m.canDrop(this.targetState != "Disabled" && (!this.current || m.source != this || !(this.current.id in this.selection)));
				}else{
					m.canDrop(false);
				}
			}
		}else{
			if(this.isDragging){
				var m = dojo.dnd.manager();
				if (m.source && this.checkAcceptance(m.source,m.source.getSelectedNodes())){
					m.canDrop(this.targetState != "Disabled" && (!this.current || m.source != this || !(this.current.id in this.selection)));
				}else{
					m.canDrop(false);
				}
			}
		}
		this.current = n;
	},

	checkItemAcceptance: function(node, source){
		// summary: stub funciton to be overridden if one wants to check for the ability to drop at the node/item level 
			return true;	
	},
	
	// topic event processors
	onDndSourceOver: function(source){
		// summary: topic event processor for /dnd/source/over, called when detected a current source
		// source: Object: the source which has the mouse over it
		if(this != source){
			this.mouseDown = false;
			if(this.targetAnchor){
				this._unmarkTargetAnchor();
			}
		}else if(this.isDragging){
			var m = dojo.dnd.manager();
			m.canDrop(this.targetState != "Disabled" && (!this.current || m.source != this || !(this.current.id in this.selection)));
		}
	},
	onDndStart: function(source, nodes, copy){
		// summary: topic event processor for /dnd/start, called to initiate the DnD operation
		// source: Object: the source which provides items
		// nodes: Array: the list of transferred items
		// copy: Boolean: copy items, if true, move items otherwise

		if(this.isSource){
			this._changeState("Source", this == source ? (copy ? "Copied" : "Moved") : "");
		}
		var accepted = this.checkAcceptance(source, nodes);

		this._changeState("Target", accepted ? "" : "Disabled");

		if(accepted){
			dojo.dnd.manager().overSource(this);
		}

		this.isDragging = true;
	},

	itemCreator: function(nodes){
		var items = []

		for(var i=0;i<nodes.length;i++){
			items.push({
				"name":nodes[i].textContent,
				"id": nodes[i].id
			});
		}

		return items;
	},

	onDndDrop: function(source, nodes, copy){
		// summary: topic event processor for /dnd/drop, called to finish the DnD operation
		// source: Object: the source which provides items
		// nodes: Array: the list of transferred items
		// copy: Boolean: copy items, if true, move items otherwise

		if(this.containerState == "Over"){
			this.isDragging = false;
			var target = this.current;
			var items = this.itemCreator(nodes, target);
			if(!target || target == this.tree.domNode){
				for(var i = 0; i < items.length; i++){
					this.tree.store.newItem(items[i], null);
				}
			}else{
				for(var i = 0; i < items.length; i++){
					pInfo = {parent: dijit.getEnclosingWidget(target).item, attribute: "children"};
					var newItem = this.tree.store.newItem(items[i], pInfo);
					console.log("newItem:", newItem);
				}
			}
		}
		this.onDndCancel();
	},
	onDndCancel: function(){
		// summary: topic event processor for /dnd/cancel, called to cancel the DnD operation
		if(this.targetAnchor){
			this._unmarkTargetAnchor();
			this.targetAnchor = null;
		}
		this.before = true;
		this.isDragging = false;
		this.mouseDown = false;
		delete this.mouseButton;
		this._changeState("Source", "");
		this._changeState("Target", "");
	},
	
	// utilities

	onOverEvent: function(){
		// summary: this function is called once, when mouse is over our container
		this.inherited("onOverEvent",arguments);
		dojo.dnd.manager().overSource(this);
	},
	onOutEvent: function(){
		// summary: this function is called once, when mouse is out of our container
		this.inherited("onOutEvent",arguments);	
		dojo.dnd.manager().outSource(this);
	},
	_markTargetAnchor: function(before){
		// summary: assigns a class to the current target anchor based on "before" status
		// before: Boolean: insert before, if true, after otherwise
		if(this.current == this.targetAnchor && this.before == before){ return; }
		if(this.targetAnchor){
			this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
		this.targetAnchor = this.current;
		this.targetBox = null;
		this.before = before;
		if(this.targetAnchor){
			this._addItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
	},
	_unmarkTargetAnchor: function(){
		// summary: removes a class of the current target anchor based on "before" status
		if(!this.targetAnchor){ return; }
		this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		this.targetAnchor = null;
		this.targetBox = null;
		this.before = true;
	},
	_markDndStatus: function(copy){
		// summary: changes source's state based on "copy" status
		this._changeState("Source", copy ? "Copied" : "Moved");
	}
});

dojo.declare("dijit._tree.dndTarget", dijit._tree.dndSource, {
	// summary: a Target object, which can be used as a DnD target
	
	constructor: function(node, params){
		// summary: a constructor of the Target --- see the Source constructor for details
		this.isSource = false;
		dojo.removeClass(this.node, "dojoDndSource");
	},

	// markup methods
	markupFactory: function(params, node){
		params._skipStartup = true;
		return new dijit._tree.dndTarget(node, params);
	}
});

}

if(!dojo._hasResource["flow.dijit.XMLEditTree"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.XMLEditTree"] = true;
dojo.provide("flow.dijit.XMLEditTree");




dojo.declare(
  "flow.dijit.XMLEditTree",
  flow.dijit.XMLViewTree,
  {
    menus: null,
    store: null,
    path: null,
    canDrag: false,
    canDrop: false,
    operator: null,

    menuinfo: null,
    contextmenu: null,

    hoverTargetTreeNode: null,

    postCreate: function() {
      // drag and drop
      if (this.canDrag || this.canDrop) {
        
        this.dndController = this.canDrag ? "dijit._tree.dndSource" : "dijit._tree.dndTarget";
      }

      this.inherited("postCreate", arguments);
      this.store.operator = this.operator;

      if (this.menuinfo) {
        this.contextmenu = new dijit.Menu({targetNodeIds: [this.domNode.getAttribute("id")]});

        if (this.menuinfo.items) {
          this.buildMenuItems(this.menuinfo.items);
        }

        dojo.connect(this.contextmenu, "onItemClick", this, this.onItemClick);
        dojo.connect(this.contextmenu, "_openMyself", this, this.openContextMenu);
        dojo.connect(this.contextmenu, "onClose", this, this.closeContextMenu);
      }
    },

    buildMenuItems: function(items) {
      var contextmenu = this.contextmenu;
      dojo.forEach(this.contextmenu.getChildren(), function(child){ contextmenu.removeChild(child); });

      for (var i = 0; i < items.length; i++) {
        var item = items[i];

        if (item.hidden) {
          continue;
        }

        if (item.label) {
          var menuitem = new dijit.MenuItem({label: item.label, action: item.id, disabled: item.disabled});
          this.contextmenu.addChild(menuitem);
        } else {
          this.contextmenu.addChild(new dijit.MenuSeparator());
        }
      }
    },

    openContextMenu: function(event) {
      if (this.selectedTreeNode) {
        dojo.removeClass(this.selectedTreeNode.labelNode, "selected");
      }

      this.selectedTreeNode = dijit.getEnclosingWidget(event.target);

      if (this.selectedTreeNode.isTree) {
        // is tree
        this.selectedTreeNode = this.selectedTreeNode.getChildren()[0];
      }

      dojo.addClass(this.selectedTreeNode.labelNode, "selected");

      if (this.menuinfo.getMenuItems) {
        var items = this.menuinfo.getMenuItems(this.selectedTreeNode);
        this.buildMenuItems(items);
      }
    },

    closeContextMenu: function(event) {
    },

    onItemClick: function(menuitem) {
      if (menuitem.disabled) {
        return false;
      }

      this.menuinfo.callback(menuitem, this.selectedTreeNode);
    },

    makeNodeEditable: function(treenode, item, editValue) {
      var myThis = this;
      var node = this.store.getNode(item);

      function setNodeValue() {
        var legalNamePattern;

        if (node.nodeType == 2 && editValue) {
          // attribute values can have numbers anywhere
          legalNamePattern = /^[a-zA-Z0-9\u0100-\uFFFF_\-\.]*$/;
        } else {
          legalNamePattern = /^[a-zA-Z\u0100-\uFFFF_][a-zA-Z0-9\u0100-\uFFFF_\-\.]*$/;
        }

        if (node.nodeType == 3 || (input && input.value.match(legalNamePattern))) {
          var newvalue;
          var parentitem;

          if (node.nodeType == 1 || node.nodeType == 3) {
            newvalue = input.value;
          } else if (node.nodeType == 2) {
	          var name = node.nodeName, value = node.nodeValue;
	          if (editValue) {
	            value = input.value;
	          } else {
	            name = input.value;
	          }

            // attributes are bad in IE, so pass parent along
            parentitem = treenode.getParent()

	          newvalue = {name: name, value: value};
	        }
          myThis.setStoreValue(treenode, newvalue, parentitem);
          resetNodeValue();
	        //treenode.labelNode.focus();
	        //treenode.labelNode.setAttribute("tabIndex", "0");
	        //dojo.addClass(treenode.labelNode, "dijitTreeLabelFocused");
	        //myThis.lastFocused = treenode;
	        var menuitem = {};
	        menuitem.action = "valuechanged";
	        myThis.menuinfo.callback(menuitem, treenode);
        } else {
          resetNodeValue();
        }
      }

      function keyPressed(e) {
        try {
        	var evtobj = window.event? event : e;
          var key = evtobj.charCode? evtobj.charCode : evtobj.keyCode;
        	//var key = window.event ? e.keyCode : e.which ? e.which : e.charCode;;

          if (key == dojo.keys.ESCAPE) {
            resetNodeValue();
          } else if (key == dojo.keys.ENTER) {
            setNodeValue();
	        } else if (node.nodeType != 3) {
            // 37 - left arrow, 39 - right arrow
            if (!evtobj.charCode && (key == 37 || key == 39)) {
              return true;
            } else if (node.nodeType == 2 && editValue && (key >= 48 && key <= 57)) {
              // editing an attribute and editing value - allow numbers anywhere
              return true;
            } else if (key < 8 || (key >= 9 && key < 27) || (key >= 30 && key <= 44) || key == 47 || (key >= 58 && key <= 64)
		        		       || (key >= 91 && key <= 94) || key == 96 || (key >= 123 && key <= 126)
                       || (((key >= 48 && key <= 57) || key == 45 || key == 46 )&& (input && input.value == ""))
		        		      ) {
		        	return false;
            } else {
		        	return true;
		        }
          }
        } catch(ex){}
      }

      function onBlur(e) {
        setNodeValue();
      }

      function resetNodeValue() {
        var value = myThis.store.getLabel(item);
        treenode.labelNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(value);
        treenode.labelNode.focus();
        treenode.labelNode.setAttribute("tabIndex", "0");
        dojo.addClass(treenode.labelNode, "dijitTreeLabelFocused");
        myThis.lastFocused = treenode;
      }

      var value = treenode.labelNode.firstChild.nodeValue;
      var span = document.createElement("span");

      var input;

      if (node.nodeType == 1 || node.nodeType == 3) {
        // element
        input = document.createElement("input");
        input.setAttribute("value", value);
        span.appendChild(input);
      } else if (node.nodeType == 2) {
        // attribute - either edit name or value
        input = document.createElement("input");

        if (editValue) {
          span.appendChild(document.createTextNode(node.nodeName));
          span.appendChild(document.createTextNode(" = "));

          input.setAttribute("value", node.nodeValue);
          span.appendChild(input);
        } else {
          input.setAttribute("value", node.nodeName);
          span.appendChild(input);
          span.appendChild(document.createTextNode(" = "));
          span.appendChild(document.createTextNode(node.nodeValue));
        }
      }

      input.onkeypress = keyPressed;
      //dojo.connect(input, 'onkeypress', function(e) {keyPressed(e);});
      dojo.connect(input, 'onblur', function(e) {onBlur(e);});

      treenode.labelNode.replaceChild(span, treenode.labelNode.firstChild);

      // stop the treenode to steal focus from our controls
      dojo.connect(span, "onkeypress", function(e) {
                   e.stopPropagation();
      });

      dojo.connect(span, "onfocus", function(e) {
                   e.stopPropagation();
      });

      dojo.connect(span, "onclick", function(e) {
                   e.stopPropagation();
      });

      // unfocus existing and focus us
      this.blurNode();
      //span.setAttribute("tabIndex", "0");
      input.focus();
    },

    setStoreValue: function(treenode, value, parenttreenode) {
      this.store.setValue(treenode.item, "value", value, parenttreenode ? parenttreenode.item : null);
    },

    importTreeNode: function(treenode, parentitem) {
      var domnode = treenode.item.store.getNode(treenode.item);

      if (!domnode) {
        return;
      }

      var parentnode = null

      if (parentitem) {
        parentnode = this.store.getNode(parentitem.item);
      }

      this.store.addNewNode(domnode, parentnode);
    },

    addNewElement: function(parentitem) {
      var parentnode = null

      if (parentitem) {
        parentnode = this.store.getNode(parentitem.item);
      }

      var element = this.xmlRootNode.ownerDocument.createElement("new");
      var newitem = this.store.addNewNode(element, parentnode);

      // expand parent item - else no treenode will be created
      if (parentitem) {
        this._expandNode(parentitem);
      } else {
        this._expandNode(this._itemNodeMap[this.store.getIdentity(this.store.getItem(this.store.rootElement))]);
      }

      // find new node
      var newtreenode = this._itemNodeMap[this.store.getIdentity(newitem)];
      // 150 is the wipe-in timeout - so double it
      setTimeout(dojo.hitch(this, function(){this.makeNodeEditable(newtreenode, newitem)}), 300)
    },

    addElement: function(parentitem, name) {
      if (parentitem) {
        var parentnode = this.store.getNode(parentitem.item);
        var element = this.xmlRootNode.ownerDocument.createElement(name);
        var newitem = this.store.addNewNode(element, parentnode);
      }

      if (parentitem) {
        this._expandNode(parentitem);
      } else {
        this._expandNode(this._itemNodeMap[this.store.getIdentity(this.store.getItem(this.store.rootElement))]);
      }

      return newitem;
    },

    addAttribute: function(parentitem, name, value) {
      if (parentitem) {
        parentnode = this.store.getNode(parentitem.item);
        var newitem = this.store.addAttribute(parentnode, name, value);
      }
    },

    addNewAttribute: function(parentitem) {
      var parentnode = null

      if (parentitem) {
        parentnode = this.store.getNode(parentitem.item);
      }

      var newitem = this.store.addNewAttribute(parentnode);

      // expand parent item - else no treenode will be created
      this._expandNode(parentitem);

      // find new node
      var newtreenode = this._itemNodeMap[this.store.getIdentity(newitem)];
      // 150 is the wipe-in timeout
      setTimeout(dojo.hitch(this, function(){this.makeNodeEditable(newtreenode, newitem)}), 250)
    },

    _onNewItem: function(item, parentInfo, index) {
      var node = this.store.getNode(item);

      if (typeof(index) != "number" && node.nodeType != 2) {
        this.inherited("_onNewItem", arguments);
        return;
      }

      // attributes go at the beginning
      var parent = this._itemNodeMap[this.getItemParentIdentity(item, parentInfo)];

			if (!parent.isExpandable) {
				parent.makeExpandable();
			}

      var childParams = {
        item: item,
        isExpandable: this.mayHaveChildren(item)
      };

      var child = new dijit._TreeNode(
        dojo.mixin({
					tree: this.tree,
				  label: this.tree.getLabel(childParams.item)
			  }, childParams)
      );

      // calculate index - length of attributes
      if (typeof(index) == "undefined") {
        index = this.store.getNode(parent.item).attributes.length - 1;
      }

      parent.addChild(child, index);
      child._updateLayout();

      var childrenMap = [];
      childrenMap[this.tree.store.getIdentity(childParams.item)] = child;

			dojo.mixin(this._itemNodeMap, childrenMap);
    },

    addNewTextNode: function(parentitem) {
      var parentnode = null;

      if (parentitem) {
        parentnode = this.store.getNode(parentitem.item);
      }

      var newitem = this.store.addNewTextNode(parentnode);

      // expand parent item - else no treenode will be created
      this._expandNode(parentitem);

      // find new node
      var newtreenode = this._itemNodeMap[this.store.getIdentity(newitem)];
      // 150 is the wipe-in timeout
      setTimeout(dojo.hitch(this, function(){this.makeNodeEditable(newtreenode, newitem)}), 250)
    },

    editNode: function(targetitem, editValue) {
      var treenode = this._itemNodeMap[this.store.getIdentity(targetitem.item)];
      this.makeNodeEditable(treenode, targetitem.item, editValue);
    },

    deleteNode: function(targetitem) {
		var node = this.store.getNode(targetitem.item);
		var parentnode=null;
    	if (dojo.isIE && node.nodeType == 2) {
        	parentnode =this.store.getNode(this.selectedTreeNode.getParent().item);
		}
      	this.store.deleteNode(targetitem.item,parentnode);

      // clear out selected node
      this.selectedTreeNode = null;
    },

    moveNode: function(targetitem, direction) {
      this.store.moveNode(targetitem.item, direction);

      // clear out selected node
      this.selectedTreeNode = null;
    },

    // drag and drop support
    checkAcceptance: function(source, nodes) {
      return this.tree.canDrop;
    },

    checkItemAcceptance: function(node, source) {
      // who do we want to allow to drop on:
      //   - elements
      //   - attribute if the source is a attribute or text node
      //   - text node if the source is a attribute or text node
      var widget = dijit.getEnclosingWidget(node);

      if (this.hoverTargetTreeNode) {
        dojo.removeClass(this.hoverTargetTreeNode.labelNode, "flowTreeNodeHover");
      }

      dojo.addClass(widget.labelNode, "flowTreeNodeHover");
      this.hoverTargetTreeNode = widget;

      var targetItem = widget.item;

      var allow = false;

      if (this.tree.store.hasAttribute(targetItem, "canHaveChildren")) {
        allow = true
      } else {
        // only one allowed
        var sourceitem = source.getSelectedItems()[0];

        var targetNode = this.tree.store.getNode(targetItem);
        var sourceNode = sourceitem.store.getNode(sourceitem);

        if ((targetNode.nodeType == 2 || targetNode.nodeType == 3) &&
            (sourceNode.nodeType == 2 || sourceNode.nodeType == 3)) {
          allow = true;
        }
      }

      return allow;
    },

    itemCreator: function(nodes) {
      var items = []

      for (var i=0; i < nodes.length; i++) {
        var widget = dijit.getEnclosingWidget(nodes[i]);
        if (!widget) {
          continue;
        }

        var item = widget.item;
        items.push({
          id: nodes[i].id,
          item: item
        });
      }
      return items;
    },

    onDndDrop: function(source, nodes, copy) {
      if (this.containerState == "Over") {
        var target = this.current;
        var items = this.itemCreator(nodes, target);

        var targetitem;
        if (!target || target == this.tree.domNode) {
          var tree = this.tree;
          var store = tree.store;
          targetitem = tree._itemNodeMap[store.getIdentity(store.getItem(store.rootElement))];
        } else {
          targetitem = dijit.getEnclosingWidget(target);
        }

        for (var i = 0; i < items.length; i++) {
          this.tree.operator.dragDrop(items[i], targetitem);
        }

        // if we drop on an expandable but closed tree item, expand it
        if (targetitem.isExpandable && !targetitem.isExpanded) {
          this.tree._expandNode(targetitem);
        }

      }

      this.onDndCancel();
    },

    onDndCancel: function() {
      this.inherited("onDndCancel", arguments);

      if (this.hoverTargetTreeNode) {
        dojo.removeClass(this.hoverTargetTreeNode.labelNode, "flowTreeNodeHover");
        this.hoverTargetTreeNode = null;
      }
    },

    getSelectedNode: function() {
      return this.selectedTreeNode ?  this.selectedTreeNode : this.getChildren()[0];
    }
  }
);

}

if(!dojo._hasResource["flow.dijit.Transform"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.Transform"] = true;
dojo.provide("flow.dijit.Transform");














dojo.declare(
  "flow.dijit.Transform",
  [dijit._Widget, dijit._Container, dijit._Templated],
  {
    "class": "flowResizable",

    // args
    operator: null,

    // attach points
    layoutNode: null,
    inputNode: null,
    outputNode: null,

    widgetsInTemplate: true,
    templateString:"<div>\r\n  <div dojoAttachPoint=\"layoutNode\" dojoType=\"dijit.layout.LayoutContainer\" style=\"width: 99%; height: 100%;\">\r\n    <div dojoType=\"dijit.layout.ContentPane\" layoutAlign=\"top\">\r\n      <div style=\"padding: 3px;\">${i18n.TRANSFORM_TREE_LABEL}</div>\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" layoutAlign=\"client\">\r\n      <div dojoType=\"dijit.layout.SplitContainer\" style=\"border: 1px solid black; width: 99%; height: 100%\">\r\n        <div dojoType=\"dijit.layout.ContentPane\" layoutAlign=\"left\" tabindex=\"0\">\r\n          <div dojoType=\"dijit.layout.LayoutContainer\" style=\"width: 99%; height: 100%;\">\r\n            <div dojoType=\"dijit.layout.ContentPane\" layoutAlign=\"top\">\r\n              <div class=\"flowgridheadercell\" style=\"text-align: center; border-bottom: 1px solid black;\">${i18n.TRANSFORM_HEADER_INPUT}</div>\r\n            </div>\r\n            <div dojoType=\"dijit.layout.ContentPane\" layoutAlign=\"client\" style=\"padding: 3px;\">\r\n              <div dojoAttachPoint=\"inputNode\"></div>\r\n            </div>\r\n          </div>\r\n        </div>\r\n\r\n        <div dojoType=\"dijit.layout.ContentPane\" layoutAlign=\"right\" tabindex=\"0\">\r\n          <div dojoType=\"dijit.layout.LayoutContainer\" style=\"width: 99%; height: 100%;\">\r\n            <div dojoType=\"dijit.layout.ContentPane\" layoutAlign=\"top\">\r\n              <div class=\"flowgridheadercell\" style=\"text-align: center; border-bottom: 1px solid black;\">${i18n.TRANSFORM_HEADER_OUTPUT}</div>\r\n            </div>\r\n            <div dojoType=\"dijit.layout.ContentPane\" layoutAlign=\"client\" style=\"padding: 3px; padding-bottom: 10px;\">\r\n              <div dojoAttachPoint=\"outputNode\"></div>\r\n            </div>\r\n          </div>\r\n        </div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    buildRendering: function() {
      this.inherited("buildRendering", arguments);

      var xmlDoc = this.operator.getTree(0);
      if (xmlDoc == null) {
        return;
      }

      if (xmlDoc) {
        this.operator.model.setValue("flow:input[@position='0']/@timestamp", this.operator.node, xmlDoc.timestamp);
      }
      var idArray = new Array('copy_to_output');
      var captionArr = new Array(this.i18n.TRANSFORM_MENU_DRAGTOOUTPUT);

      var menuinfo = {callback: dojo.hitch(this, this.menuCallback),
                      items: [{id: "copy_to_output", label: this.i18n.TRANSFORM_MENU_DRAGTOOUTPUT}]};

      this.sourceTree = new flow.dijit.XMLEditTree({canDrag: true, menuinfo: menuinfo, xmlRootNode: xmlDoc.root, path: xmlDoc.path, operatorId: this.operator.id, operator: this.operator}, this.inputNode);

      var templ = this.operator.model.getNode("flow:template/*", this.operator.node);

      menuinfo = {getMenuItems: dojo.hitch(this, this.getMenuItems),
                  callback: dojo.hitch(this, this.menuCallback)};

      var myThis = this;

      function _buildType(type) {
        return "flowtype" + type.substr(0,1).toUpperCase() + type.substr(1);
      }

      function _getIconClass(item) {
        if (!item) {
          return;
        }

        var type = "flowtype ";

        var node = this.store.getNode(item);

        var typeitem;
        if (node.nodeType == 1 && node.childNodes.length == 0) {
          var xpath = item.store.buildXPath(item.store.getNode(item), true);
          var param = myThis.operator.model.getNode("flow:param[@operatorspecific='"+xpath+"']", this.operator.node);
          if (param) {
            typeitem = myThis.operator.model.getValue(param, "@type");
          }
        }

        if (typeitem) {
          type += _buildType(typeitem);
        } else {
          type = this.inherited("getIconClass", arguments);
        }

        return type;
      }

      this.targetTree = new flow.dijit.XMLEditTree({canDrop: true, menuinfo: menuinfo, xmlRootNode: templ, operatorId: this.operator.id, path: this.operator.TEMPLATEPATH, operator: this.operator, getIconClass: _getIconClass}, this.outputNode);

      this.resize();
      dojo.connect(dijit.byId("flowcanvas").dialogs["property"].widget, "resize", this, "resize");

      // make the tree cover the whole pane
      this.targetTree.domNode.style.height = "100%";
    },

    checkAncestorNodesForParam: function(treenode, direction) {
      var item = this.getParamForTreeNode(treenode);
      if (!item.param) {
        var parameters = this.operator.model.runXPath("flow:param", this.operator.node);
        for (var i = 0; i < parameters.length; i++) {
          var parameter = parameters[i];
          var operatorspecific = this.operator.model.getValue(parameter, "@operatorspecific");
          if (direction) {
            if ((item.operatorspecific.indexOf(operatorspecific) != -1) && operatorspecific.indexOf("@")==-1) {
              item.param = parameter;
              item.type = this.operator.model.getValue(item.param, "@type");
              break;
            }
          } else {
            if ((operatorspecific.indexOf(item.operatorspecific) != -1)  && item.operatorspecific.indexOf("@")==-1) {
              item.param = parameter;
              item.type = this.operator.model.getValue(item.param, "@type");
              break;
            }
          }
        }
      } else {
        item.exactmatch = true;
      }
      return item;
    },

    getMenuItems: function(treenode) {
      var node = treenode.tree.store.getNode(treenode.item);

      var isAttribute = (node.nodeType == 2) ? true : false;
      var canMoveDown = ((node.nodeType == 1 || node.nodeType == 3 || node.nodeType == 4) && node.nextSibling) ? true : false;
      var canMoveUp = ((node.nodeType == 1 || node.nodeType == 3 || node.nodeType == 4) && node.previousSibling) ? true : false;
      var canEdit=  true;
      var canHaveChild = node.nodeType == 1;
      var isRoot = treenode.getParent().isTree;
      var item = this.checkAncestorNodesForParam(treenode, true);

      if (item.param) {
        if (item.type == "xpath") {
          canEdit = false;
          canHaveChild = false;
          if(!item.exactmatch) {
            canMoveDown=false;
            canMoveUp=false;
            isRoot=true;
          }
        }

        if (item.type == "function") {
          canHaveChild = false;
        }
      }

      var item1 = this.checkAncestorNodesForParam(treenode, false);

      if (item1.param) {
        if ((item1.type == "xpath" || item1.type == "function") && !item1.exactmatch) {
          canEdit = false;
          isRoot = true;
        }
      }

      return [{id: "new_element", label: this.i18n.TRANSFORM_MENU_NEWELEMENT, disabled: !canHaveChild},
              {id: "new_attribute", label: this.i18n.TRANSFORM_MENU_NEWATTRIBUTE, disabled: !canHaveChild},
              {id: "new_text", label: this.i18n.TRANSFORM_MENU_NEWTEXT, disabled: !canHaveChild},
              {id: ""},
              {id: "functions", label: this.i18n.DROPDOWNCOMBO_LABEL_FUNCTION, disabled: !canHaveChild},
              {id: ""},
              {id: "edit", label: this.i18n.TRANSFORM_MENU_EDIT, disabled: !canEdit, hidden: isAttribute},
              {id: "edit_name", label: this.i18n.TRANSFORM_MENU_EDIT_ATTRIBUTE_NAME, disabled: !canEdit, hidden: !isAttribute},
              {id: "edit_value", label: this.i18n.TRANSFORM_MENU_EDIT_ATTRIBUTE_VALUE, disabled: !canEdit, hidden: !isAttribute},
              {id: "delete", label: this.i18n.TRANSFORM_MENU_DELETE, disabled: isRoot},
              {id: "move_up", label: this.i18n.TRANSFORM_MENU_MOVEUP, disabled: !canMoveUp},
              {id: "move_down", label: this.i18n.TRANSFORM_MENU_MOVEDOWN, disabled: !canMoveDown}
             ];
    },

    getParamForTreeNode: function(treenode) {
      if (!treenode) {
        return {};
      }

      var item = {};
      try {
        if (treenode.nodeType) {
          // treenode can be the domnode for iconclass stuff
          item.node = treenode;
        } else {
          item.node = treenode.tree.store.getNode(treenode.item);
        }
        
		var parentnode =  (item.node.nodeType==2)?this.targetTree.store.getNode(this.targetTree.getTreeNodeByItem(treenode.item).getParent().item):null;
        
        item.operatorspecific = this.targetTree.store.buildXPath(item.node,true,true,null,parentnode);

        item.param = this.operator.model.getNode("flow:param[@operatorspecific='"+item.operatorspecific+"']", this.operator.node);
        if(item.param) item.type= this.operator.model.getValue(item.param, "@type");
      } catch(ex){}

      return item;
    },

    menuCallback: function(menuitem, treenode) {
      var action = menuitem.action;
      this.menuNode = treenode;
  
      if (action == "copy_to_output")  {
        this.dragDrop(treenode, this.targetTree.getSelectedNode())
      } else if (action == "new_element") {
        this.targetTree.addNewElement(treenode);
      } else if (action == "new_attribute") {
        this.targetTree.addNewAttribute(treenode);
      } else if (action == "new_text") {
        this.targetTree.addNewTextNode(treenode);
      } else if (action == "edit") {
        var item = this.getParamForTreeNode(this.menuNode);
        if (item.param) {
          if (item.type == "function") {
            this.showFunctionDialog(this.operator.generateDataFromParameter(item.param));
          } else if (item.type == "xpath") {
          } else {
            this.targetTree.editNode(treenode);
          }
        } else {
          this.targetTree.editNode(treenode);
        }
      } else if (action == "edit_name" || action == "edit_value") {
        // for editing attributes
        var editValue = (action == "edit_value") ? true : false;
        this.targetTree.editNode(treenode, editValue);
      } else if (action == "delete") {
        var item= this.getParamForTreeNode(this.menuNode);
        if (item.param) {
          item.param.parentNode.removeChild(item.param);
          this.operator.model.setValue("@dirty", this.operator.node, "1");
        }
        this.targetTree.deleteNode(treenode);
      } else if (action == "move_up") {
        this.targetTree.moveNode(treenode, "up");
      } else if (action == "move_down") {
        this.targetTree.moveNode(treenode, "down");
      } else if (action == "valuechanged") {
      } else if (action == "functions")  {
        this.showFunctionDialog({});
      }
    },

    dragDrop: function(srctreenode, targettreenode) {
      if (targettreenode.item.store == this.sourceTree.store) {
        // dojo bug, target is in source tree
        return false;
      }

      if (this.getParamForTreeNode(targettreenode).param) {
        alert(this.i18n.TRANSFORM_ERRORMSG_CANNOTCOPYONEXISTINGSOURCE);
        return false;
      }

      var titem = this.checkAncestorNodesForParam(targettreenode, true);

      if (titem.param) {
        if (titem.type == "xpath") {
          alert(this.i18n.TRANSFORM_ERRORMSG_CANNOTCOPYONEXISTINGSOURCE);
          return false;
        }
      }

      var srcnode = this.sourceTree.store.getNode(srctreenode.item);

      // uh, IE doesn't allow you to go from attribute to parent
      var parentnode =  (srcnode.nodeType==2)?this.sourceTree.store.getNode(this.sourceTree.getTreeNodeByItem(srctreenode.item).getParent().item):null;

      var srcpath = this.sourceTree.store.buildXPath(srcnode, true, true, null, parentnode);

      //var srcpath = this.getOperatorSpecificPathForNode(this.FEEDPATH, srcnode);
      var item = this.getParamForTreeNode(targettreenode);
      var targetnode = item.node;

      if (srcnode.nodeType == 1 && (item.node.nodeType==3 || item.node.nodeType==2 || item.node.nodeType == 4)) {
        alert(this.i18n.TRANSFORM_ERRORMSG_CANNOTCOPYELEMENTONATTRIBUTE);
        return false;
      }

      // new item is an element
      if (item.node.nodeType == 1) {
        if (srcnode.nodeType == 3 || srcnode.nodeType == 4) {
          var dupecount = 1;
          for (var i = 0; i < item.node.childNodes.length; i++) {
            if (item.node.childNodes[i].nodeType == 3 || item.node.childNodes[i].nodeType == 4)  dupecount++;
          }

          item.operatorspecific+="/text()["+dupecount+"]";
        } else if (srcnode.nodeType == 2) {
          item.operatorspecific+="/@"+srcnode.name;
        	if(this.operator.model.getNode("flow:param[@operatorspecific='"+item.operatorspecific+"']", this.operator.node)!=null)
        	{
        		alert(dojo.string.substitute(this.i18n.TRANSFORM_ERRORMSG_ATTRIBUTE_EXISTS, [srcnode.name]));
        		return;
        	}
        } else if (srcnode.nodeType == 1) {
          var preview = this.operator.setNamespace(srcnode.namespaceURI);
          preview= (preview != "") ? preview + ":" : "";
          item.operatorspecific += "/" + preview + flow.xml.getLocalName(srcnode);
        }
      }

      var value = {type: "xpath", caption: "", inputid: ""};
      value.operatorspecific = item.operatorspecific;

      if (item.param) {
        value.id = this.operator.model.getValue(item.param, "@id");
        value.data = srcpath;
        this.operator.modifyChild(value.id, "value", value);
      } else {
        value.id = this.operator.model.generateId("param");
        value.value = srcpath;
        this.operator.createParameter(value, this.operator.node);
      }

      var targetparentnode =  (item.node.nodeType==2)?this.targetTree.getTreeNodeByItem(targettreenode.item).getParent():null;
      if ((srcnode.nodeType == 2 || srcnode.nodeType == 3|| srcnode.nodeType == 4) && (item.node.nodeType == 2 || item.node.nodeType == 3 || item.node.nodeType == 4)) {
        // dnd a text/attr to a text/attr
        var newvalue;

        if (item.node.nodeType == 3 ||item.node.nodeType == 4) {
          newvalue = srcnode.nodeValue;
        } else if (item.node.nodeType == 2) {
          newvalue = {name: item.node.name, value: srcnode.nodeValue};
        }

        this.targetTree.setStoreValue(targettreenode, newvalue,targetparentnode);
      } else if (item.param || item.node.nodeType == 2) {
        var newvalue={};

        if (item.node.nodeType == 1) {
          newvalue = srcnode.value;
        } else if (item.node.nodeType == 3||item.node.nodeType == 4) {
          newvalue = srcnode.nodeValue;
        } else if (item.node.nodeType == 2) {
          newvalue = {name: item.node.name, value: (srcnode.value!=null?srcnode.value:srcnode.nodeValue)};
        }

        this.targetTree.setStoreValue(targettreenode, newvalue,targetparentnode);
      } else {
        if (srcnode.nodeType == 2) {
          this.targetTree.addAttribute(targettreenode, srcnode.name, srcnode.value);
        } else {
          this.targetTree.importTreeNode(srctreenode, targettreenode);
        }
      }

      return true;
    },

    showFunctionDialog: function(value) {
      

      var div = document.createElement("div");
      dojo.body().appendChild(div);

      if (this.dialog) {
        this.dialog.hide();
        this.dialog.destroyRecursive();
        this.dialog = null;
      }

      var labels = dojo.i18n.getLocalization("flow", "labels");

      this.dialog = new flow.dijit.Dialog({title: this.i18n.USE_FUNCTION_TITLE, chain: 0, showUnderlay: true, buttons:[{label: labels.BUTTON_OK, onclick: dojo.hitch(this, this._acceptDialog)}, {label:labels.BUTTON_CANCEL, onclick: dojo.hitch(this, this._cancelDialog)}]}, div);
      this.dialog.resize({w:400, h: 400});
      this.dialog.show();

      div = document.createElement("div");
      this.dialog.containerNode.appendChild(div);
      
      this.selectWidget = new flow.dijit.FunctionInputSelection({data: value, chain: 0, chainParent: this.operator, operator: this.operator}, div);
    },

    _cancelDialog: function(event) {
      this.selectWidget.destroy();
      this.selectWidget = null;

      this.dialog.hide();
      this.dialog.destroyRecursive();
      this.dialog = null;
    },

    _acceptDialog: function(event) {
      var value = this.selectWidget.getData();

      if (value) {
        var item = this.getParamForTreeNode(this.menuNode);
        value.operatorspecific = item.operatorspecific;
        var count = this.operator.model.runXPath(".//flow:param[@type='function']/flow:function[@name='"+value.name+"']",this.operator.node).length;

        if (item.param) {
          // this is an edit
          var orgname=item.node.nodeName;
		  var expectedname=this.operator.model.getValue(item.param,"flow:function/@name")          
		  if(orgname!=expectedname)	value.caption = value.name+(count>0?"-"+count:"");

          value.type = "function";
          value.id = this.operator.model.getValue(item.param, "@id");
          value.operatorspecific=value.operatorspecific.substring(0,value.operatorspecific.lastIndexOf("/"))
          value.operatorspecific += "/" + value.caption;
          this.operator.model.setValue(".//flow:param[@id='"+value.id+"']/@operatorspecific", this.operator.node, value.operatorspecific);
          this.operator.model.setValue(".//flow:param[@id='"+value.id+"']/@caption", this.operator.node, value.caption);
          this.operator.modifyChild(value.id, "value", value);
		  if(orgname!=expectedname)	this.targetTree.setStoreValue(this.menuNode, value.caption);
        } else {
		  value.caption = value.name+(count>0?"-"+count:"");
          value.operatorspecific += "/" + value.caption;
          value.id = this.operator.model.generateId("param");
          this.operator.createParameter(value, this.operator.node);
          var newitem = this.targetTree.addElement(this.menuNode, value.caption);
        }
      }

      this._cancelDialog();
    },

    getOperatorSpecificPathForNode: function(basepath, node) {
    //not used should be removed
      var tree= basepath==this.TEMPLATEPATH?this.targetTree:this.sourceTree;
      var path=tree.store.buildXPath(node,true,true);
      /*
      if(node.nodeType == 2) {
        return path.substring(basepath.length,path.lastIndexOf("/"))+"/@"+node.name;
      } else {
        return  path.substring(basepath.length);
      }
      */
    },

    resize: function(args) {
      if (!this.propertyEditor) {
        this.propertyEditor = dijit.byNode(this.domNode.parentNode.parentNode.parentNode.parentNode.parentNode);
      }

      var height = this.propertyEditor.tabcontainerNode.containerNode.offsetHeight;
      height -= this.propertyEditor.captioninput.parentNode.offsetHeight + 22;

      this.domNode.style.height = height + "px";

      this.layoutNode.resize();
    }
  }
);


}

if(!dojo._hasResource["flow.dijit.Properties"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.Properties"] = true;
dojo.provide("flow.dijit.Properties");








dojo.declare(
  "flow.dijit.Properties",
  [dijit._Widget, dijit._Templated, dijit._Container],
  {
    selectNode: null,
    helpNode: null,

    rssDetailsNode: null,
    atomDetailsNode: null,
    xmlDetailsNode: null,

    rssDetailsForm: null,
    atomDetailsForm: null,
    xmlDetailsForm: null,

    // impl
    mode: null,
    model: null,

    templateString:"<div>\r\n  ${i18n.labels.PUBLISH_FEED_TYPE}\r\n\r\n  <select dojoAttachPoint=\"selectNode\" dojoAttachEvent=\"onchange:onChange\">\r\n \t \t<option value=\"rss\">${i18n.labels.PUBLISH_RSSXML_TYPE}\r\n    \t<option value=\"atom\">${i18n.labels.PUBLISH_ATOMXML_TYPE}\r\n    \t<option value=\"xml\">${i18n.labels.PUBLISH_XML_TYPE}\r\n  </select>\r\n\r\n  <p></p>\r\n\r\n  <div style=\"display: none\" dojoAttachPoint=\"rssDetailsNode\">\r\n    <div>${i18n.labels.PUBLISH_RSS_DESCRIPTION}</div>\r\n    <form dojoAttachPoint=\"rssDetailsForm\" onsubmit=\"return false\">\r\n      <table width=\"100%\">\r\n        <tr>\r\n          <td NOWRAP>${i18n.labels.PUBLISH_TITLE}*</td>\r\n          <td width=\"100%\"><input name=\"title\" title=\"${i18n.labels.PUBLISH_TITLE_TOOLTIP}\" style=\"width: 95%\"></td>\r\n        </tr>\r\n        <tr>\r\n          <td NOWRAP>${i18n.labels.PUBLISH_LINK}*</td>\r\n          <td width=\"100%\"><input name=\"link\" title=\"${i18n.labels.PUBLISH_LINK_TOOLTIP}\" style=\"width: 95%\"></td>\r\n        </tr>\r\n        <tr>\r\n          <td NOWRAP>${i18n.labels.PUBLISH_DESCRIPTION}*</td>\r\n          <td width=\"100%\"><input name=\"desc\" title=\"${i18n.labels.PUBLISH_DESCRIPTION_TOOLTIP}\" style=\"width: 95%\"></td>\r\n        </tr>\r\n      </table>\r\n    </form>\r\n  </div>\r\n\r\n  <div style=\"display: none\" dojoAttachPoint=\"atomDetailsNode\">\r\n    <div>${i18n.labels.PUBLISH_ATOM_DESCRIPTION}</div>\r\n    <form dojoAttachPoint=\"atomDetailsForm\" onsubmit=\"return false\">\r\n    <table width=\"100%\">\r\n      <tr>\r\n        <td NOWRAP>${i18n.labels.PUBLISH_TITLE}*</td>\r\n        <td width=\"100%\"><input name=\"title\" title=\"${i18n.labels.PUBLISH_TITLE_TOOLTIP}\" style=\"width: 95%\"></td>\r\n      </tr>\r\n      <tr>\r\n        <td NOWRAP>${i18n.labels.PUBLISH_ID}*</td>\r\n        <td width=\"100%\"><input name=\"id\" title=\"${i18n.labels.PUBLISH_ID_TOOLTIP}\" style=\"width: 95%\"></td>\r\n      </tr>\r\n      <tr>\r\n        <td NOWRAP>${i18n.labels.PUBLISH_LINK}*</td>\r\n        <td width=\"100%\"><input name=\"link\" title=\"${i18n.labels.PUBLISH_LINK_TOOLTIP}\" style=\"width: 95%\"></td>\r\n      </tr>\r\n      <tr>\r\n        <td NOWRAP>${i18n.labels.PUBLISH_AUTHOR}*</td>\r\n        <td width=\"100%\"><input name=\"author\" title=\"${i18n.labels.PUBLISH_AUTHOR_TOOLTIP}\" style=\"width: 95%\"></td>\r\n      </tr>\r\n      <tr>\r\n        <td NOWRAP>${i18n.labels.PUBLISH_RIGHTS}</td>\r\n        <td width=\"100%\"><input name=\"rights\" title=\"${i18n.labels.PUBLISH_RIGHTS_TOOLTIP}\" style=\"width: 95%\"></td>\r\n      </tr>\r\n    </table>\r\n    </form>\r\n  </div>\r\n\r\n  <div style=\"display: none\" dojoAttachPoint=\"xmlDetailsNode\">\r\n    <div>${i18n.labels.PUBLISH_XML_DESCRIPTION}</div>\r\n    <form dojoAttachPoint=\"xmlDetailsForm\" onsubmit=\"return false\">\r\n    <table width=\"100%\">\r\n      <tr>\r\n        <td NOWRAP>${i18n.labels.PUBLISH_XML_REPEATING_TAG}*</td>\r\n        <td width=\"100%\"><input name=\"repeating\" title=\"${i18n.labels.PUBLISH_XML_REPEATING_TAG_TOOLTIP}\" style=\"width: 95%;\"></td>\r\n      </tr>\r\n      <tr>\r\n        <td NOWRAP valign=\"top\">${i18n.labels.PUBLISH_XML_REPEATING_HEADER}*</td>\r\n        <td width=\"100%\"><textarea name=\"header\" title=\"${i18n.labels.PUBLISH_XML_REPEATING_HEADER_TOOLTIP}\" maxlength=\"500\" style=\"width: 95%\"></textarea></td>\r\n      </tr>\r\n    </table>\r\n    </form>\r\n  </div>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");
    },

    postCreate: function() {
      //var img = document.createElement("img");
      //img.setAttribute("src", dojo.moduleUrl("flow.dijit", "templates/images/dialoghelp.png"));
      //img.setAttribute("title", this.i18n.labels.TOOLTIP_HELP);
      //this.helpNode.appendChild(img);

      // load in model detaults
      var model = this.model;

      var type = model.getPublishType();
      this.selectNode.value = type.toLowerCase();
      this.updateMode();

      var form = this[this.mode+"DetailsForm"];
      for (var i = 0; i < form.elements.length; i++) {
        var name = form.elements[i].name;
        var val = model.getPublishParam(name);
        form.elements[i].value = val ? val : "";
      }

      this.validateInputs();

      dojo.connect(this.domNode, "onkeyup", this, "onKeyUp");
    },

    getMode: function() {
      return this.selectNode.value;
    },

    updateMode: function() {
      var mode = this.getMode();

      if (mode == this.mode) {
        return;
      }

      if (this.mode) {
        this[this.mode+"DetailsNode"].style.display = "none";
      }

      this[mode+"DetailsNode"].style.display = "block";
      this.mode = mode;

      this.validateInputs();
    },

    onChange: function(event) {
      this.updateMode();
      this.updateModel();
    },

    validateInputs: function() {
      var isValid = true;
      var mode = this.getMode().toUpperCase();

      var requirements = this.model.getRequiredPublishParams(mode);

      var form = this[this.mode+"DetailsForm"];
      for (var i = 0; i < requirements.length; i++) {
        var formelement;
        for (var j = 0; j < form.elements.length; j++) {
          if (form.elements[j].name == requirements[i]) {
            formelement = form.elements[j];
            break;
          }
        }

        if (!dojo.trim(formelement.value)) {
        	if((mode=="ATOM" ||mode=="RSS") && (requirements[i]=="id" || requirements[i]=="link"))
        	{
		        formelement.value =this.model.getValue(null,"/flow:mashup/@url"); 
				formelement.parentNode.parentNode.cells[0].style.color = "";
        	}
        	else
        	{
		          // mark as invalid
				formelement.parentNode.parentNode.cells[0].style.color = "red";
				isValid = false;
          	}
        } else {
          formelement.parentNode.parentNode.cells[0].style.color = "";
        }
      }

      if (this.onValidate) {
        this.onValidate(isValid);
      }
    },

    onKeyUp: function(e) {
      var target = e.target;
      if (target.tagName == "INPUT" || target.tagName == "TEXTAREA") {
        var mode = this.getMode().toUpperCase();
        var requirements = this.model.getRequiredPublishParams(mode);

        if (dojo.indexOf(requirements, target.name) >= 0) {
          if (!dojo.trim(target.value)) {
            // mark as invalid
            target.parentNode.parentNode.cells[0].style.color = "red";
          } else {
            target.parentNode.parentNode.cells[0].style.color = "";
          }
        }

        if (this.onValidate) {
          this.validateInputs()
        }

        // update the model
        var node = this.model.getNode(".//flow:param[@caption = '"+target.name+"']", this.model.publishNode);
        node.firstChild.nodeValue =  target.value;
      }
    },

    updateModel: function() {
      var mode = this.getMode();

      this.model.setPublishInput(mode.toUpperCase());
      var publishnode = this.model.getNode("flow:publish", this.model.flowNode);
      this.model.removeNodes(".//flow:param[@position !='0']", publishnode);

      //RSS- Title, Link, Description
      //Atom Title, ID, Link, Author, Rights, Link
      var form = this[mode+"DetailsForm"];
      // copy over elements from form into our data for the server to handle
      // as it sees fit
      for (var i = 0; i < form.elements.length; i++) {
        //data[form.elements[i].name] = form.elements[i].value;
        var value = form.elements[i].value;

        var template='<param xmlns="'+this.model.flowNS+'" position="'+(i+1)+'"  caption="'+form.elements[i].name+'" operatorspecific="" inputid="" id="'+form.elements[i].name+'" type="text" comments=""><![CDATA['+value+']]></param>';
        publishnode.appendChild(this.model.parseXML(template));
      }
    }
  }
);

}

if(!dojo._hasResource["flow.operator.BaseOperator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.operator.BaseOperator"] = true;
dojo.provide("flow.operator.BaseOperator");

/*
  Every Operator has two representations - the visual part (CanvasOperator) and
  a functional part, which derives from BaseOperator.  The functional code will
  be called to render the property editor (setupPropertyEditor) and can intercept
  calls for adding/changing/removing values from the property editor if needed.
*/







dojo.declare("flow.operator.BaseOperator", null,
{
  type: null,
  model: null,
  id: null,
  node: null,

  flowNS: "http://ibm.com/mashuphub",
  mhubNS: "http://w3.ibm.com/mhub",

  MASHUPPATH: "/flow:mashup/",
  FEEDPATH: "/flow:mashup/flow:data/flow:feed/",
  SOURCEXMLPATH: "/flow:mashup/flow:data/flow:sourcexml",
  OPERATORPATH: "/flow:mashup/flow:flow/flow:operator/",
  SOURCEPATH: "/flow:mashup/flow:flow/flow:source/",
  TEMPLATEPATH: "/flow:mashup/flow:flow/flow:operator/flow:template",

  hasOutput: true,
  canRename: true,
  minimumInputs: 1,

  // -1 means unlimited
  maximumInputs: 1,

  conditions: [],

  init: function(model, id){
    this.model = model;

    if (id) {
      this.id = id;

      if(this.type=="source") {
        var url= this.model.getValue(this.model.flowNode, "flow:source[@id='"+this.id+"']/flow:url/text()");
        if (url!=null && url!="") {
          this.reloadPrimitiveData();
        }
      }

      // TODO: load the data from model?
    }

    
    this.i18n = dojo.i18n.getLocalization("flow", "propertyeditor");
  },

  // generated a random ID for our type
  generateId: function() {
    return this.model.generateId(this.type);
  },

  getLabel: function() {
    var labels = dojo.i18n.getLocalization("flow", "labels");
    return labels["OPERATOR_" + this.type.toUpperCase()];
  },

  getName: function() {
    var name = this.model.getValue(this.node, "@caption");

    if (!name) {
      name = this.id;
    }

    return name;
  },

  getDisplayName: function() {
    var name = this.getName();
    if (name == this.id) {
      name = this.getLabel();
    }

    return name;
  },

  getCanvas: function() {
    return this.model.canvas;
  },

  // returns the XML Document for the Model
  getModel: function() {
    return this.model.xmlModel;
  },

  // adds a top-level Model child
  addToModel: function(elm) {
    return this.model.flowNode.appendChild(elm);
  },

  // adds to data
  addToData: function(elm) {
    return this.model.dataNode.appendChild(elm);
  },

  // XPath helper method
  runXPath: function(xpath, getFirst) {
    var type = this.model.xpathTypes.anyType;

    if (typeof(getFirst) != "undefined" && getFirst) {
      type = this.model.xpathTypes.firstType;
    }

    return this.getModel().evaluate(xpath, this.node, null, type, null);
  },

  // verifies that all requirements are met
  meetsRequirements: function() {
    var result = {pass: true, error: null};

    // check if enough inputs are available
    var count = this.model.getInputCount(this.id);

    if (count < this.minimumInputs) {
      if (this.minimumInputs == 1) {
        result.error = this.i18n.ERROR_REQUIRES_ONE_INPUT;
      } else if (this.maximumInputs == -1) {
        result.error = dojo.string.substitute(this.i18n.ERROR_REQUIRES_MINIMUM_INPUTS, [this.minimumInputs]);
      } else {
        result.error = dojo.string.substitute(this.i18n.ERROR_REQUIRES_MULTIPLE_INPUTS, [this.minimumInputs]);
      }

      result.pass = false;
    }

    // check if inputs generate data
    if (result.pass) {
      var inputsSatisfied = this.model.areAllInputsSatisfied(this.id);
      if (!inputsSatisfied.satisfied) {
        // result.cause is the operator object
        result.error = dojo.string.substitute(this.i18n.LABEL_INPUT_NOT_VAlID, [inputsSatisfied.cause.getName()]);
        result.pass = false;
      }
    }

    // check if enough params/conditions exist
    if (result.pass) {
      var pass = this.hasEnoughParameters()
      // either true or a string with the error msg
      if (pass !== true) {
        result.pass = false;
        result.error = pass;
      }
    }

    return result;
  },

  // returns true or an error message
  hasEnoughParameters: function() {
    return true;
  },

  // returns all child parameter nodes
  getParams: function(options, parent) {
    var xpath = "flow:param";

    if (options) {
      for (var option in options) {
        xpath += "[@"+option+"='"+options[option]+"']"
      }
    }

    return this.model.runXPath(xpath, parent ? parent : this.node);
  },

  // returns a js representation of all child parameters
  getParameters: function(options, parent) {
    var params = this.getParams(options, parent);
    var result = [];

    for (var i = 0; i < params.length; i++) {
      var item = this.generateDataFromParameter(params[i]);

      // use the position attribute to place the item in the right position in
      // the data array
      var pos = parseInt(this.model.getValue(params[i], "@position"), 10);
      if (!pos) {
        pos = 0;
      }

      result[pos] = item;
    }

    return result;
  },

  // generates a data object for a parameter
  generateDataFromParameter: function(parameter) {
    //console.log("generateDataFromParameter called with id: "+parameter.getAttribute("id"))

    /*var data = {id: this.model.getValue(parameter, "@id"),
                type: this.model.getValue(parameter, "@type"),
                position: this.model.getValue(parameter, "@position"),
                comments: this.model.getValue(parameter, "@comments"),
                caption: this.model.getValue(parameter, "@caption")
    };*/

    var id = this.model.getValue(parameter, "@id");
    var type = this.model.getValue(parameter, "@type");

    var data = flow.operator.Data.create(type, id);
    data.position = this.model.getValue(parameter, "@position");
    data.inputid = this.model.getValue(parameter, "@inputid");
    data.comments = this.model.getValue(parameter, "@comments");
    data.caption = this.model.getValue(parameter, "@caption");

    if (type == "function") {
      var funcnode = this.model.getNode("flow:function", parameter);

      data.name = this.model.getValue(funcnode, "@name");

      // build the function params
      data.parameters = this.getParameters(null, funcnode);
    } else if (data.type == "dynamic") {
	    data.value = this.model.getValue(parameter, "text()");
      data.name = this.model.getValue(parameter, "@caption");
    } else {
      data.value = this.model.getValue(parameter, "text()");
    }

    return data;
  },

  getNode: function(name, parent) {
    // adds flow: to name
    return this.model.getNode("flow:"+name, parent ? parent : this.node);
  },

  // creates an operator in the model
  create: function() {
    this.id = this.generateId();

    var template = '<operator xmlns="'+this.model.flowNS+'" dirty="1" caption="'+this.id+'" type="'+this.type+'" id="'+this.id+'" x="" y="" />';
    this.node = this.addToModel(this.model.parseXML(template));
  },

  generateData: function() {
    return {layout: []};
  },

  generateGridData: function() {
    var data = [];

    var conditions = this.model.runXPath("flow:condition", this.node);

    for (var i = 0; i < conditions.length; i++) {
      var item = {};
      var condition = conditions[i];

      item.id = this.model.getValue(condition, "@id");
      item.operation = this.model.getValue(condition, "@operation");
      item.conjunction= this.model.getValue(condition, "@conjunction");

      var pos = this.model.getValue(condition, "@position");
      pos = (!pos) ? 0 : pos;

      item.input = this.generateDataFromParameter(this.model.getNode("flow:param[@position='0']", condition));
      item.value = this.generateDataFromParameter(this.model.getNode("flow:param[@position='1']", condition));

      data[pos] = item;

      var conditem = {id: item.id};
      conditem.inputParamId = this.model.getValue(condition, "flow:param[@position='0']/@id");
      conditem.valueParamId = this.model.getValue(condition, "flow:param[@position='1']/@id");
      this.conditions[item.id] = conditem;
    }

    return data;
  },

  getVariableItems: function(varparams, ignoreexistingparams) {
    var itemlist = [];
    for (var i = 0; i < varparams.length; i++) {
      var param = varparams[i];

      var parentpath = flow.xml.getLocalName(this.model.getNode("..", param)) == "condition"? "../..":".." ;
      var parentid = this.model.getValue(param, parentpath+"/@caption");
      var id = this.model.getValue(param, "@id");
      var caption = this.model.getValue(param, "@caption");

      if (ignoreexistingparams) {
        if (this.model.getValue(this.node,".//flow:condition/flow:param[@position='0' and text()='"+id+"']/@id") == null) {
          itemlist.push({label: this.createVariableLabel(parentid, caption), id: id});
        }
      } else {
        itemlist.push({label: this.createVariableLabel(parentid, caption), id: id});
      }
    }

    return itemlist;
  },

  getDirtyInputs: function(nodeID, includeSelf) {
    var startnode = this.model.getNode("./flow:*[@id='"+nodeID+"']",this.model.flowNode);

    var params = [];

    var inputs = this.model.runXPath("flow:input",startnode);

    for (var i = 0; i < inputs.length; i++) {
      var srcid = this.model.getValue(inputs[i], "@id");

      var retparams = this.getDirtyInputs(srcid, true);
      for (var j=0; j < retparams.length; j++) {
        params.push(retparams[j]);
      }
    }

    // when calling dirty inputs, skip the caller operator
    if (includeSelf) {
      if (this.model.getValue(startnode, "@dirty") == 1 || params.length > 0) {
        params.push(startnode);
      }
    }

    return params;
  },

  isDirty: function() {
    return (this.model.getValue(this.node, "@dirty") == 1);
  },

  getInscopeVaraibles: function(nodeID, all) {
    var startnode = this.model.getNode("./flow:*[@id='"+nodeID+"']",this.model.flowNode);
    var searchstr=".//flow:param";
    searchstr += (all) ? "[@type='dynamic' or (name(..)='source') or (../../@type='filter' and @type='variable')  ]" : "[@type='dynamic']";

    var tparams = this.model.runXPath(searchstr,startnode);
    var params =[];

    for (var j=0; j < tparams.length; j++) {
      if (tparams[j].getAttribute("type") == "variable") {
        params[j] = this.model.getNode("./flow:variables/flow:param[@id='"+tparams[j].firstChild.nodeValue+"']",this.model.flowNode)	      
      } else {
        params[j] = tparams[j];
      }
    }

    var startctr = params.length;
    var inputs = this.model.runXPath("flow:input",startnode);
    for(var i = 0; i < inputs.length; i++) {
      var srcid = this.model.getValue(inputs[i], "@id");
      var retparams = this.getInscopeVaraibles(srcid, all);

      for (var j = 0; j < retparams.length; j++) {
      	var notfound=true;
        for (var k = 0; k < params.length; k++) {
        	if (params[k] == retparams[j]) {
            notfound=false;
            break;
        	}
        }

        if (notfound) {
          params[startctr++]=retparams[j];
        }
      }
    }

    return params;
  },

  getVariableCount: function() {
    var items = this.model.runXPath("//flow:param[@type='dynamic']", this.model.flowNode);
    return items ? items.length : 0;
  },

  createVariableLabel: function(parentid, caption) {
    return dojo.string.substitute(this.i18n.VARIABLE_DISPLAY_LABEL, [parentid, caption]);
  },

  generateVariableListBox: function(selectObj, searchstr, defaultValue) {
    var items = this.model.runXPath(searchstr, this.model.flowNode);

    for(var i = 0; i < items.length; i++) {
      var item = items[i];
      var parentid = this.model.getValue(item, "../@caption");
      if(parentid==null) parentid=this.i18n.VARIABLE_GLOBAL_CAPTION;
      var id = this.model.getValue(item, "@id");
      var caption = this.model.getValue(item, "@caption");

      var option = document.createElement("option");
      option.setAttribute("value", id);
      option.appendChild(document.createTextNode(this.createVariableLabel(parentid, caption)));
      if (defaultValue && id == defaultValue) {
        option.setAttribute("selected", true);
      }

      selectObj.appendChild(option);
    }
  },

  generateConfig: function() {
    return {canAddRows: true, canMoveRows: true, allowGridEditing: true};
  },

  isBuiltInType: function(type) {
    return (type == "text" || type == "xpath" || type == "variable");
  },

  createParameter: function(data, parentnode) {
    if (!parentnode) {
      parentnode = this.node;
    }

    this.model.setValue("@dirty", this.node, "1");

    var template='<param xmlns="'+this.model.flowNS +
                      '" caption="'+((typeof(data.caption) != "undefined" && data.caption) ? data.caption : "") +
                      '" operatorspecific="'+(typeof(data.operatorspecific) !="undefined" ? data.operatorspecific:"") +
                      '" inputid="" id="'+data.id +
                      '" position="'+this.model.runXPath("flow:param", parentnode).length +
                      '" type="'+(typeof(data.type) != "undefined" ? data.type:"") +
                      '" comments=""></param>';

    parentnode.appendChild(this.model.parseXML(template));

    if (this.isBuiltInType(data.type)) {
      var inputid = data.inputid ? data.inputid : "";
      var value = (typeof(data.value) != "undefined" && data.value!=null ?data.value: data);
      if (data.type == "text") {
        if (isNaN(value)) {
          value = dojo.trim(value);
          if (value.charAt(0)=='"') {
            value=value.substring(1);
          }

          if (value.charAt(value.length-1) == '"'){
            value = value.substring(0, value.length-1);
          }
          value = '"' + value + '"';
        }
      }

      this.model.setValue("flow:param[@id='"+data.id+"']/text()", parentnode, value);
      this.model.setValue("flow:param[@id='"+data.id+"']/@inputid", parentnode, inputid);

      if (typeof(data.comments) != "undefined" && data.comments!=null) {
        this.model.setValue("flow:param[@id='"+data.id+"']/@comments", parentnode, data.comments);
      }
    } else if (data.type == "function") {
      if (typeof(data.comments) != "undefined"  && data.comments!=null) {
        this.model.setValue("flow:param[@id='"+data.id+"']/@comments", parentnode, data.comments);
      }

      var pnode = this.model.getNode("flow:param[@id='"+data.id+"']", parentnode);
      this.createFunction(data, pnode);
    }
  },

  createCondition: function(data) {
    var template = '<condition xmlns="'+this.model.flowNS+'" id="'+data.id+'" conjunction="'+data.conjunction+
                    '" operation="" position="'+this.model.runXPath("flow:condition", this.node).length+'"></condition >';

    this.node.appendChild(this.model.parseXML(template));

    var cond = this.model.getNode("flow:condition[@id='"+data.id+"']", this.node);
    this.model.setValue("@operation", cond, data.operation);

    for (var i = 0; i < data.parameters.length; i++) {
      this.createParameter(data.parameters[i], cond);
    }
  },

  createFunction: function(data, parentnode) {
    var fdata=(data.parameters==null && data.value!=null && data.value.type=="function")
    				 ?data.value:data; // the function information is defined in the value
    var template = '<function xmlns="'+this.model.flowNS+'" name="'+fdata.name+'"></function>';

    var funcnode = null;
    if (parentnode) {
      funcnode = parentnode.appendChild(this.model.parseXML(template));
    } else {
      funcnode = this.node.appendChild(this.model.parseXML(template));
    }

    // add any parameters
    for (var i = 0; i < fdata.parameters.length; i++) {
      this.createParameter(fdata.parameters[i], funcnode);
    }
  },

  addChild: function(gridid, data, position) {},

  modifyChild: function(id, key, data) {
    this.model.setValue("@dirty", this.node, "1");

    if (key == "value") {
      var type = data.type ? data.type : "text";

      this.model.setValue(".//flow:param[@id='"+id+"']/@type", this.node, type);
      this.model.removeAllChildren(".//flow:param[@id='"+id+"']", this.node)

      if (type == "function") {
        this.model.setValue(".//flow:param[@id='"+id+"']/@type", this.node, type);

        if (typeof(data.comments) != "undefined") {
          this.model.setValue(".//flow:param[@id='"+id+"']/@comments", this.node, data.comments ? data.comments : "");
        }

        var param = this.model.getNode(".//flow:param[@id='"+id+"']", this.node);
        this.createFunction(data, param);
      } else if (type == "dynamic") {
        var value = typeof(data.value) != "undefined" ? data.value : data;
        /*
        if(isNaN(value)) {
	          value = dojo.trim(value);
              if(value[0]=='"') value=value.substring(1);
              if(value[value.length-1]=='"') value=value.substring(0,value.length-1);
              value = '"' + value + '"';
        }
        */
        this.model.setValue(".//flow:param[@id='"+id+"']/text()", this.node, value);
        this.model.setValue(".//flow:param[@id='"+id+"']/@caption", this.node, data.name);
      } else if (type == "variable") {
        var val= typeof(data.value)=="object" && data.value.value!=null ? data.value.value: data.value;
        this.model.setValue(".//flow:param[@id='"+id+"']/text()", this.node, val);
      } else {
        var value = typeof(data.value) != "undefined" ? data.value : data;
        if (type == "text") {
          if (isNaN(value)) {
            value = dojo.trim(value);

            if (value.charAt(0) == '"') {
              value = value.substring(1);
            }

            if (value.charAt(value.length-1) == '"') {
              value = value.substring(0,value.length-1);
            }

            value = '"' + value + '"';
          }
        }

        this.model.setValue(".//flow:param[@id='"+id+"']/text()", this.node, value);
        if (data.inputid) {
          this.model.setValue(".//flow:param[@id='"+id+"']/@inputid", this.node, data.inputid);
        }
      }
    } else if (key == "position") {
      this.model.setValue(".//flow:*[@id='"+id+"']/@position", this.node, data);
    } else {
      alert("modifyChild"+id +"  "+key+"  "+data.type + " " +data);
    }
  },

  // resets a node to a default value
  // TODO: remove
  resetToValue: function(node) {
    var valuenode = this.model.createElementNS(this.model.flowNS, "Value");
    node.parentNode.replaceChild(valuenode, node);
  },

  handleChainedValue: function(obj) {
    var model = this.getModel();
    var newNode = model.createElementNS(this.model.flowNS, obj.type);
    newNode.setAttribute("name", obj.name)

    if (obj.type) {
      newNode.setAttribute("type", obj.type)
    }

    for (var key in obj.parameters) {
      var param = obj.parameters[key];
      this.createParameter(param, newNode);
    }

    return newNode;
  },

  modifyCondition: function(id, key, data) {
    //console.log("modifyCondition: "+id+"  "+key+"  "+value);

    if (key == "position") {
      this.model.setValue("flow:condition[@id='"+id+"']/@position", this.node, data);
    } else {
      this.model.setValue("flow:condition[@id='"+id+"']/@" +key, this.node, data.value);
    }
  },

  removeChild: function(id) {
    var param = this.model.getById(id);

    if (!param) {
      return;
    }

    param.parentNode.removeChild(param);
  },

  inputRemoved: function(srcOperator) {
    // by default, all params/conditions will be removed.  If an operator
    // wants to take care of that, overwrite and return true
    return false;
  },

  getTree: function(pos, rawxml) {
    // default to 0
    if (!pos) {
      pos = 0;
    }

    var inputid = this.model.getValue(this.model.flowNode, "flow:operator[@id='"+this.id+"']/flow:input[@position='"+pos+"']/@id");

    if (!inputid) {
      // perhaps source - the id is the inputid
      if (this.type == "source") {
        inputid = this.id;
      } else {
        // TODO: error!
        return null;
      }
    }

    var type = this.model.getValue(this.model.dataNode, "flow:feed[@id='"+inputid+"']/@type");
    var timestamp = this.model.getValue(this.model.dataNode, "flow:feed[@id='"+inputid+"']/@timestamp");

    var startpath = this.FEEDPATH +"result/";

    if (rawxml) {
      var rootnode = this.model.getNode(this.SOURCEXMLPATH+ "[@id='"+inputid+"']/*");

      if (!rootnode) {
        return null;
      }

      var preview = this.setNamespace(rootnode.namespaceURI);
      preview = (preview != "") ? (preview + ":") : "";
      startpath = this.SOURCEXMLPATH +"/"+preview + flow.xml.getLocalName(rootnode) + "/";
    }
    else if (type == "RSS") {
      var rootnode = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+inputid+"']/*/rss/channel/item");
      startpath += "rss/channel/item/";
    } else if (type == "XML") {
      var rootnode = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+inputid+"']/*/*");
    } else if (type == "ATOM") {
      var rootnode = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+inputid+"']/*/*");
      startpath += "mhub:entry/";
    } else {
      var rootnode = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+inputid+"']/*/*");

      if (!rootnode) {
        return null;
      }

      var preview = this.setNamespace(rootnode.namespaceURI);
	    preview = (preview != "") ? (preview + ":") : "";

      startpath += preview + flow.xml.getLocalName(rootnode) + "/";
    }

    //console.log("getTree: "+inputid+","+type+","+startpath+","+rootnode);
    return {id: inputid, root: rootnode, path: startpath, timestamp: timestamp};
  },

  setChildValue: function(id, key, value) {
    if (key == "operation" || key == "position") {
      this.modifyCondition(id, key, value);
    } else {
      var condition = this.conditions[id];
      if (!condition) {
        throw "Flow Error: Could not find condition!";
      }

      var paramid = (key == "input") ? condition.inputParamId : condition.valueParamId;
      this.modifyChild(paramid, "value", value);
    }
  },

  loadXMLUrl: function(url, callback) {
    dojo.xhrGet({url: url,
                handleAs: "xml",
                timeout: 5000,
                load: function(response, ioArgs) {
                  callback(response);
                  return response;
                },

                error: function(response, ioArgs) {
                  callback(null);
                  return response;
                }
               });
  },

  callServer: function(actionUrl, postContent, callback) {
     dojo.xhrPost( {
         url: actionUrl,
         handleAs: dojo.isIE ? "text" : "xml",
         content: postContent,
         contentType: "application/x-www-form-urlencoded; charset=utf-8", 
         load: function(response, ioArgs) {
           if (dojo.isIE) {
             var doc = flow.xml.loadDocument(response);
             callback(doc);
             return doc;
           } else {
             callback(response);
             return response;
           }
         },
         error: function(response, ioArgs) {
            alert(response.message +" "+ioArgs.xhr.responseText);
            callback(null);
            return response;
         }
     });
  },

  // Called when operator is connected as input to another operator
  connectedToTarget: function(action) {
    if (action) {
      //If operator does not have any data then do a preview on it.
      if (!this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+this.id+"']/*/*")) {
        this.doPreview();
      }
    }
  },

  reloadPrimitiveData: function() {
  	this.model.flowNode.setAttribute("operatorid",this.id);
  	this.model.flowNode.setAttribute("primitive","true");
    var xmlString = '<?xml version="1.0" encoding="utf-8"?>'+flow.xml.innerXML(this.model.flowNode);
    var url = dijit.byId("flowid_flowwidget").getEditorActionUrl("preview"); 
    var content = {xml: xmlString, op: '$' + this.id, seq: '$' + this.id, topk: 50, format: "xml"};
    this.callServer(url, content, dojo.hitch(this, this.reloadPrimitiveDataCallback));
  },

  reloadPrimitiveDataCallback:function(result) {
    if (result) {
      var startnode = flow.xml.findNodes(result.documentElement, "/*")[0]
      if (flow.xml.getLocalName(startnode) == "ERRORS") return;
      var dt = new Date();
      this.model.removeNodes("flow:sourcexml[@id='"+this.id+"']", this.model.dataNode);
      this.addToData(this.model.parseXML("<sourcexml xmlns='"+this.model.flowNS+"' timestamp='"+dt.getTime()+"' id='"+this.id+"'/>"));
      var node=flow.xml.findNodes(result.documentElement, "/*/*")[0];
      this.model.getNode("flow:sourcexml[@id='"+this.id+"']", this.model.dataNode).appendChild(flow.xml._importNode(this.model.xmlModel, node, true));
      this.removeDuplicateNodes(node);
    }
  },

  removeDuplicateNodes: function(node)
  {

    var children = this.model.runXPath("*", node);
    for (var i = 50; i < children.length; i++) {
            children[i].parentNode.removeChild(children[i]);
    
    }
    this.model.removeDuplicateNodes(node);
  
  },
  doPreview: function(showError, callback) {
  
	if(this.type=="source") {
	  var turl=this.model.getValue(this.node,"./flow:url/text()")
//	  if(turl==null || dojo.trim(turl)=="") return;
	}
	if(this.type=="publish" && this.model.getNode("./flow:input",this.node)==null) return

  	
  	this.model.flowNode.setAttribute("operatorid",this.id);
  	this.model.flowNode.setAttribute("primitive","");

    var xmlString = '<?xml version="1.0" encoding="utf-8"?>'+flow.xml.innerXML(this.model.flowNode);
    var url = dijit.byId("flowid_flowwidget").getEditorActionUrl("preview"); 
    var content = {xml: xmlString, op: '$' + this.id, seq: '$' + this.id, topk: 50, format: "xml"};

    this.loadingPreviewResult = true;

    if (showError) {
      this.previewshowError = showError;
    }

    this.callServer(url, content, dojo.hitch(this, callback ? callback : this.previewCallback));
  },

  previewLoadComplete: function() {
    this.loadingPreviewResult = false;
  },

  previewCallback: function(result) {
    var showError = this.previewshowError;

    if (result) {
      //alert(flow.xml.innerXML (result.documentElement));
      var startnode = flow.xml.findNodes(result.documentElement, "/*")[0];

      if (flow.xml.getLocalName(startnode) == "ERRORS") {
      	var errmsg="<b>"+this.i18n.SERVER_PREVIEW_ERROR+"</b>";
      	var errornodes = flow.xml.findNodes(result.documentElement, "/*/*");

      	for (var er = 0; er < errornodes.length; er++) {
      		if (errornodes[er].lastChild.nodeName=="MSGTEXT") {
            errmsg+="<br>"+ errornodes[er].lastChild.firstChild.nodeValue;
          }
        }

        if (showError) {
          showError.style.display = "block";
          showError.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(errmsg);
        }

        dojo.publish("flow:"+this.id+":loadURL", [{error: true, message: errmsg}]);

        this.previewLoadComplete();
        return;
      }

      this.model.removeNodes("flow:feed[@id='"+this.id+"']", this.model.dataNode);
      var dt = new Date();

      this.addToData(this.model.parseXML("<flow:feed xmlns='' xmlns:flow='"+this.model.flowNS+"' timestamp='"+dt.getTime()+"' id='"+this.id+"'/>"));

      this.model.getNode("flow:feed[@id='"+this.id+"']", this.model.dataNode).appendChild(flow.xml._importNode(this.model.xmlModel, startnode, true));
      var node = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+this.id+"']/*/*");

      // TODO: better error handling
      if (!node) {
        var errmsg=dojo.string.substitute(this.i18n.SERVER_PREVIEW_MISSING_ITEM, [this.model.getValue(this.node,"@caption")]);
        if (showError) {
          showError.style.display = "block";
         	showError.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(errmsg);
        }

        dojo.publish("flow:"+this.id+":loadURL", [{error: true, message: errmsg}]);

        this.previewLoadComplete();
        return;
      }

      var localName = flow.xml.getLocalName(node).toLowerCase();
      this.model.setValue("/flow:mashup/flow:data/flow:feed[@id='"+this.id+"']/@type", null, localName);
      var dtype = "XML";
      
	  var nsuri=node.namespaceURI;
      if (localName == "rss") {
      	var rssnode = this.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+this.id+"']/*/rss/channel/item");
        if(rssnode !=null) dtype = "RSS";
      } else if (localName == "feed" && nsuri=="http://www.w3.org/2005/Atom") {
        dtype = "ATOM";
      }
      localName = flow.xml.getLocalName(node);

      var stype = "";
      if (flow.xml.getLocalName(this.node) == "source") {
        stype=this.model.getValue(this.model.flowNode,"//flow:source[@id='"+this.id+"']/@type");
        if (stype == null || stype == "") {
          this.model.removeNode(".//flow:sourcexml[@id='"+this.id+"']", this.model.dataNode,true);
          this.addToData(this.model.parseXML("<sourcexml xmlns='"+this.model.flowNS+"' timestamp='"+dt.getTime()+"' id='"+this.id+"'/>"));
          this.model.getNode("flow:sourcexml[@id='"+this.id+"']", this.model.dataNode).appendChild(node);
          this.removeDuplicateNodes(node);

          this.model.setValue("/flow:mashup/flow:flow/flow:source[@id='"+this.id+"']/@type", null, dtype);
          this.model.setValue("/flow:mashup/flow:flow/flow:source[@id='"+this.id+"']/@originaltype", null, dtype);
          var advancednode=this.model.getNode(".//flow:advanced", this.node);
          this.model.removeNodes(".//flow:param", advancednode);
          if (dtype == "RSS") {
           this.model.setValue("@repeatelement", advancednode, "/rss/channel/item");
            this.model.setValue("@originalrepeatelement", advancednode, "/rss/channel/item");
            this.doPreview();
            return;
          } else if (dtype == "ATOM") {
            var preview = this.setNamespace(node.namespaceURI);
            preview = preview!=""?preview+":":"";
            this.model.setValue("@repeatelement", advancednode, "/"+preview+"feed/"+preview+"entry");
            this.model.setValue("@originalrepeatelement", advancednode, "/"+preview+"feed/"+preview+"entry");

            var fid = this.model.getValue(node,"./*[name()='id']/text()");
            this.addAdvancedParamToSource(advancednode,fid,"ID");
            var ftitle = this.model.getValue(node,"./*[name()='title']/text()");
            this.addAdvancedParamToSource(advancednode,ftitle,"Title");
            var flink = this.model.getValue(node,"./*[name()='link']/text()");
            this.addAdvancedParamToSource(advancednode,flink,"Link");
            var authornode = this.model.getNode("./*[name()='author']/text()",node);
            var fname = this.model.getValue(authornode,"./*[name()='link']/text()");

            this.addAdvancedParamToSource(advancednode,fname,"Name");
            this.doPreview();
            return;
          } else {
            var preview = this.setNamespace(node.namespaceURI);
            preview = preview!=""?preview+":":"";
            this.model.setValue("@repeatelement", advancednode, "/"+preview+localName);
            this.model.setValue("@originalrepeatelement", advancednode, "/"+preview+localName);
            this.doPreview();
            return;
          }
          this.refreshPreviewTree();
        } else {
          this.refreshPreviewTree();
	      }
      } else {
        this.refreshPreviewTree();
      }

      dojo.publish("flow:"+this.id+":loadURL", [{success: true}]);
    }
    return;
  },

  addAdvancedParamToSource: function(advancednode, value, caption) {
  	var template='<param xmlns="'+this.model.flowNS+'" position="0"  caption="'+caption+'" operatorspecific="" '+ 
  				'inputid="" id="'+this.id+"_advanced"+caption+'" type="text" comments=""><![CDATA['+value+']]></param>';
    var ps = this.model.parseXML(template);
    advancednode.appendChild(ps);
  },

  refreshPreviewTree: function() {
  	this.model.setValue("@dirty", this.node, "0");

    this.previewLoadComplete();

    var propeditor = this.getCanvas().getPropertyEditor();
    if (propeditor) {
      propeditor.refreshPreviewTree();
    }
  },

  setNamespace: function(namespace) {
    if (namespace == null || namespace == "") {
      return "";
    }

    var prefix = this.model.getValue(this.flowNode, ".//flow:namespaces/flow:namespace[text()='"+namespace+"']/@prefix");
    if (prefix == null || prefix == "") {
      var ns= this.model.runXPath(".//flow:namespaces/flow:namespace",this.flowNode);
      var prefix=(namespace==this.model.flowNS)?"flow":(
        (namespace==this.model.mhubNS)?"mhub":"ns"+ns.length);

      var template='<namespace xmlns="'+this.model.flowNS +
                    '" prefix="'+prefix + '"><![CDATA['+namespace+']]></namespace>';
      var nsnode=this.model.getNode(".//flow:namespaces", this.flowNode);
      nsnode.appendChild(this.model.parseXML(template));
    }
    return prefix;
  },

  saveComment: function(text) {
    // attribute might not be set
    this.node.setAttribute("comment", text);
  },

  getComment: function(text) {
    return this.node.getAttribute("comment");
  },

  createErrorMarkup: function(error) {
    return "<div class='errorMsg'><table width='100%'><tr><td width='27px' class='errorMsgIcon'></td><td valign='center'>"+error+"</td></tr></table></div>"
  },

  edit: function() {},
  setupPropertyEditor: function() {},

  hasAdvancedOptions: false,
  renderAdvancedOptions: function(containerNode){},

  destroy: function() {
    // clear dom reference
    this.node = null;
  }
});

dojo.declare("flow.operator.SourceOperator", flow.operator.BaseOperator,
{
  type: "source",

  maximumInputs: 0,
  minimumInputs: 0,

  hasAdvancedOptions: true,
  isAdvancedOptionsVisible: false,

  create: function() {
    this.id = this.generateId();

    var template = '<source xmlns="'+this.model.flowNS+'" originaltype="" type="" dirty="1" caption="'+this.id+'" id="'+this.id+'" x="" y=""><advanced originalrepeatelement="" repeatelement="/*" interval="3600"></advanced><url xmlns="'+this.model.flowNS+'" catalogid="" caption="" type="url"></url></source>';
    this.node = this.addToModel(this.model.parseXML(template));
  },

  setupPropertyEditor: function() {
    var url = this.model.getValue(this.node, "flow:url/text()");
    if (url == null){ 
      url = "";
    }

    var type = this.model.getValue(this.node, "flow:url/@type");
    var catalogid = this.model.getValue(this.node, "flow:url/@catalogid");
    var caption = this.model.getValue(this.node, "flow:url/@caption");

    var data = this.generateData();
    data.layout.push({type: "url", data: {value: url, type: type, catalogid: catalogid, name: caption}, parent: this, callback: dojo.hitch(this, this.getUrl)});
    var grid = {id: "sourceparams", type: "grid", layout: null, data: [], config: this.generateConfig(), hidden: true, collapsable: true, collapseLabel: this.i18n.SOURCE_PARAM_COLLAPSE, expandLabel: this.i18n.SOURCE_PARAM_EXPAND};
    grid.layout = [
                   {name: "caption", label: this.i18n.SOURCE_HEADER_URL_PARAMETERS, editable: false, width: "5%"},
                   {name: "value", label: this.i18n.SOURCE_HEADER_VALUE_PARAMETERS, options: ["text",  "variable"], type: "param", width: "95%"}
                  ];

    grid.config.canAddRows = false;
    grid.config.canMoveRows = false;
    grid.config.allowGridEditing = false;

    this.showURLParams(grid);

    data.layout.push(grid);

    return data;
  },

  renderAdvancedOptions: function(containerNode) {
    containerNode.innerHTML = "";

    // source has advanced options - repeating item
    var div = document.createElement("div");
    containerNode.appendChild(div);

    // check if we are satisfied
    var url = this.getUrlValue();
    var tree = this.getTree(0,true);

    if (url && tree && tree.root && !this.loadingPreviewResult) {
      
      var x = new flow.dijit.SourceAdvanced({operator: this}, div);
    } else {
      if (!url) {
        div.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(this.createErrorMarkup(this.i18n.SOURCE_ADVANCED_ERROR_NO_URL));
      } else if (this.loadingPreviewResult) {
        div.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(this.createErrorMarkup(this.i18n.SOURCE_ADVANCED_ERROR_WAITING));
      } else {
        div.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(this.createErrorMarkup(this.i18n.SOURCE_ADVANCED_ERROR));
      }

      this.advancedContainerNode = div;
    }
  },

  showURLParams: function(grid) {
    // reset the grid
    if (this.grid) {
      this.grid.clearGrid(false);

      // grid with no params is hidden
      this.grid.domNode.style.display = "block";
    }

    var parameters = this.model.runXPath("flow:param", this.node);

    for (var i = 0; i < parameters.length; i++) {
      var parameter = parameters[i];
      var id = this.model.getValue(parameter, "@id");
      var caption = this.model.getValue(parameter, "@caption");
      var type = this.model.getValue(parameter, "@type");

      var item = {id: id, operator: this, caption: caption,  type: type};
      item.value = this.generateDataFromParameter(parameter);

      if (grid) {
        grid.data.push(item);
        // unhide grid if items exist
        grid.hidden = false;
      } else if (this.grid) {
        this.grid.addRow(id, item);
      }
    }
  },

  setChildValue: function(id, key, value) {
    this.modifyChild(id, key, value);
  },

  getParametersFromURL: function(url, grid) {
    url=url.replace(/\s*((\S+\s*)*)/, "$1"); 
    var spliturl=url.split("?");
    var addr=spliturl[0].split("/");
    var paramid =[];
    var paramvalue =[];
    var addrvalue =[];
    var j=0;

    var parentid=this.model.getValue(this.node, "@id");
    this.model.removeNodes("flow:param", this.node);
    for (var i = 0; i < addr.length;i ++) {
      if(addr[i] != "") {
        addrvalue[j++] = addr[i];
      }
    }

    var pos=0;
    for (var i = 0; i< addrvalue.length; i++) {
      var value = addrvalue[i];
      var id = parentid+"_path"+pos;
      var caption = "path"+pos;
      var type = "text";
      var def = "/";
      if (i == 0) {
        caption="domain";
        def="http";
        if(value=="http:" || value=="https:")
        {
          def=(value=="http:")?"http":"https";
          value=addrvalue[++i];
        }
      }
      var template='<param xmlns="'+this.model.flowNS+'" caption="'+caption+'" operatorspecific="'+def+'" id="'+id+
            '" position="'+(pos++)+'" type="'+type+'">'+value+'</param>';

      this.node.appendChild(this.model.parseXML(template));
    }

    if (spliturl.length>1) {
      var params = spliturl[1].split("&");
      for (var i=0; i<params.length; i++) {
        var oneparam=params[i].split("=");

        var value = oneparam[0];
        var id = parentid+"_param"+pos;
        var caption="param"+pos;
        var def="";
        if(i==0)
        {
        	def=url.indexOf("/?")>0?"/?":"?";
        }
        if (oneparam.length>1) {
          caption = oneparam[0];
          value = params[i].substring(oneparam[0].length+1);
          def += "=";
        }

	    //value=unescape(value);
        var template='<param xmlns="'+this.model.flowNS+'" caption="'+caption+'" operatorspecific="'+def+'" id="'+id+
            '" position="'+(pos++)+'" type="text"><![CDATA['+value+']]></param>';
        this.node.appendChild(this.model.parseXML(template));
        //this.model.setValue("flow:param[@id='"+id+"']/@operatorspecific",this.node,value);
      }
    }
    this.showURLParams();
  },

  getUrl: function(url, catalogid, name, errorNode) {
    if (url == null) {
      // checkbox changed, no value set
      this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@type", this.model.flowNode, catalogid ? "catalog" : "external");
      this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@catalogid", this.model.flowNode, "");
      this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@caption", this.model.flowNode, "");
      this.model.setValue("flow:source[@id='"+this.id+"']/@type", this.model.flowNode, "");
      this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/text()", this.model.flowNode, "");

      if (this.grid) {
        this.grid.clearGrid(false);
      }

      if (errorNode) {
        errorNode.innerHTML = "";
      }

      return;
    }

    this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@type", this.model.flowNode, catalogid ? "catalog" : "external");
    this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@catalogid", this.model.flowNode, catalogid ? catalogid : "");
    this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/@caption", this.model.flowNode, name ? name : "");

    this.model.setValue("flow:source[@id='"+this.id+"']/@type", this.model.flowNode, "");
    this.model.setValue("flow:source[@id='"+this.id+"']/flow:url/text()", this.model.flowNode, url);

    this.getParametersFromURL(url, null);

/*    if (catalogid) {
      // from catalog
      if (this.grid) {
        this.grid.clearGrid(false);
      }
    } else {
      this.getParametersFromURL(url, null);
    }*/

    this.model.setValue("@type", this.node, "");

    this.doPreview(errorNode);
  },

  getUrlValue: function() {
    return this.model.getValue(this.model.flowNode, "flow:source[@id='"+this.id+"']/flow:url/text()");
  },

  getUrlType: function() {
    return this.model.getValue(this.model.flowNode, "flow:source[@id='"+this.id+"']/flow:url/@type");
  },

  getType: function() {
    return this.model.getValue(this.model.flowNode, "flow:source[@id='"+this.id+"']/@type");
  },

  loadUrl: function() {
    // called during startup
    var url = this.getUrlValue();
    if (url) {
      this.doPreview();
    }
  },

  previewLoadComplete: function() {
    this.inherited("previewLoadComplete", arguments);

    // check if advanced options are visible
    if (this.isAdvancedOptionsVisible && this.advancedContainerNode) {
      this.renderAdvancedOptions(this.advancedContainerNode);
    }
  }
});

dojo.declare("flow.operator.CombineOperator", flow.operator.BaseOperator,
{
  type: "combine",

  minimumInputs: 2,
  maximumInputs: -1
});

dojo.declare("flow.operator.SortOperator", flow.operator.BaseOperator,
{
  type: "sort",

  setupPropertyEditor: function() {
    var data = this.generateData();

    var grid = {id: "sortparams", type: "grid", layout: null, data: this.generateGridData(), config: this.generateConfig()};
    grid.layout = [
                   {name: "input", label: this.i18n.SORT_HEADER_INPUT, type: "tree", pos: 0, width: "95%"},
                   {name: "value", label: this.i18n.SORT_HEADER_SORT_ORDER, width: "5%",
                    values: [{label: this.i18n.SORT_ASCENDING, value: 0},
                             {label: this.i18n.SORT_DESCENDING, value: 1}]
                   }
                  ];

    grid.defaultchild = {value: {data: "", type: "xpath"}, type: "asc"};

    data.layout.push(grid);

    return data;
  },

  addChild: function(gridid) {
    var id = this.model.generateId("cond");

    var conddata = {id: id, conjunction: "", operation: ""};
    var conditem = {id: id};

    var paramdata = [];
    var paramid = this.model.generateId("param");

    var dataitem = flow.operator.Data.create("xpath", paramid);
    dataitem.value = "";
    paramdata.push(dataitem);

    conditem.inputParamId = paramid;

    paramid = this.model.generateId("param");
    dataitem = flow.operator.Data.create("text", paramid);
    dataitem.value = 0;
    paramdata.push(dataitem);

    conditem.valueParamId = paramid;

    conddata.parameters = paramdata;
    this.createCondition(conddata);

    this.conditions[id] = conditem;

    return {id: id, input: "",  value: ""};
  }
});

dojo.declare("flow.operator.GroupbyOperator", flow.operator.BaseOperator,
{
  type: "groupby",

  setupPropertyEditor: function() {
    var data = this.generateData();

    var grid = {id: "groupbyparams", type: "grid", label: this.i18n.GROUPBY_GROUPED, labelPrefix: "GROUPBY_GROUPED", layout: null, data: [], config: this.generateConfig()}
    grid.layout = [{name: "value", pos: 0, label: this.i18n.GROUPBY_HEADER_INPUT, type: "tree", passAllData: true}];
    grid.data = this.getParameters({operatorspecific: "grouped"}, this.node);
    data.layout.push(grid);

    grid = {id: "nestedbyparams", type: "grid", label: this.i18n.GROUPBY_NESTED, labelPrefix: "GROUPBY_NESTED", layout: null, data: [], config: this.generateConfig()}
    grid.layout = [{name: "value", pos: 0, label: this.i18n.GROUPBY_HEADER_INPUT_NEST, type: "tree", passAllData: true}];
    grid.data = this.getParameters({operatorspecific: "nested"}, this.node);
    data.layout.push(grid);

    return data;
  },

  addChild: function(gridid) {
    var id = this.model.generateId("param");
    var type = (gridid == "groupbyparams") ? "grouped" : "nested";

    var paramdata = flow.operator.Data.create("xpath", id);
    paramdata.operatorspecific = type
    paramdata.value = "";

    this.createParameter(paramdata);

    return {id: id, value: ""};
  },

  setChildValue: function(id, key, value) {
    this.modifyChild(id, key, value);
  }
});

dojo.declare("flow.operator.FilterOperator", flow.operator.BaseOperator,
{
  type: "filter",

  conjunction: "and",

  setupPropertyEditor: function() {
    var data = this.generateData();

    data.layout.push({id: "satisfy", type: "custom"});

    var grid = {id: "filterparams", type: "grid", layout: null, data: this.generateGridData(), config: this.generateConfig()}
    // can't move rows
    grid.config.canMoveRows = false;

    grid.layout = [
                   {name: "input", pos: 0, label: this.i18n.FILTER_HEADER_INPUT,
                    type: "tree", width: "46%"},
                   {name: "operation", label: this.i18n.FILTER_HEADER_OPERATOR,
                    values: [{value: "<", label: this.i18n.OPERATION_LESSER_THAN},
                             {value: ">", label: this.i18n.OPERATION_GREATER_THAN},
                             {value: "=", label: this.i18n.OPERATION_EQUALS},
                             {value: "!=", label: this.i18n.OPERATION_NOT_EQUALS},
                             {value: "<=", label: this.i18n.OPERATION_LESSER_OR_EQUALS},
                             {value: ">=", label: this.i18n.OPERATION_GREATER_OR_EQUALS},
                             {value: "contains", label: this.i18n.OPERATION_CONTAINS},
                             {value: "notcontains", label: this.i18n.OPERATION_NOTCONTAINS},
                             {value: "starts-with", label: this.i18n.OPERATION_STARTS_WITH},
                             {value: "ends-with", label: this.i18n.OPERATION_ENDS_WITH}
                            ],
                    width: "8%"},
                   {name: "value", label: this.i18n.FILTER_HEADER_VALUE, options: ["text",  "function", "variable", "xpath"], type: "param", width: "46%"}
                  ];
    data.layout.push(grid);

    return data;
  },

  buildCustomValue: function(parentnode, dataitem) {
    // can only be satisfy
    var div = document.createElement("div");

    this.conjunction = this.model.getValue(this.node, "flow:condition/@conjunction");
    this.conjunction = this.conjunction ? this.conjunction : "and";

    var select = document.createElement("select");
    var option = document.createElement("option");
    option.setAttribute("value", "and");
    option.appendChild(document.createTextNode(this.i18n.FILTER_ALL));
    if (this.conjunction == "and") {
      option.setAttribute("selected", "true");
    }
    select.appendChild(option);

    option = document.createElement("option");
    option.setAttribute("value", "or");
    option.appendChild(document.createTextNode(this.i18n.FILTER_ANY));
    if (this.conjunction == "or") {
      option.setAttribute("selected", "true");
    }

    select.appendChild(option);

    div.appendChild(select);
    dojo.connect(select, "onchange", this, "onSelectChange");

    div.appendChild(document.createTextNode(" " + this.i18n.FILTER_CONDITIONS));
    div.appendChild(document.createElement("br"));

    parentnode.appendChild(div);
  },

  onSelectChange: function(event) {
    var select = event.target;
    this.conjunction = select.value;

    var conditions = this.model.runXPath("flow:condition", this.node);

    for (var i = 0; i < conditions.length; i++) {
      this.model.setValue("@conjunction", conditions[i], this.conjunction)
    }
  },

  addChild: function(gridid) {
    var id = this.model.generateId("cond");

    var conddata = {id: id, conjunction: this.conjunction, operation: "<"};

    // we store per condition the two param ids we generate for later use
    var conditem = {id: id};

    var paramid = this.model.generateId("param");

    var paramdata = [];

    var data = flow.operator.Data.create("xpath", paramid);
    data.value = "";
    paramdata.push(data);

    conditem.inputParamId = paramid;

    paramid = this.model.generateId("param");
    var data2 = flow.operator.Data.create("text", paramid);
    data2.value = "";
    paramdata.push(data2);

    conditem.valueParamId = paramid;

    conddata.parameters = paramdata;
    this.createCondition(conddata);

    this.conditions[id] = conditem;

    return {id: id, input: data, value: data2};
  }
});

dojo.declare("flow.operator.MergeOperator", flow.operator.BaseOperator,
{
  type: "merge",

  minimumInputs: 2,
  maximumInputs: 2,
  conjunction: "and",
  
  setupPropertyEditor: function() {
    var data = this.generateData();

    var src1id = this.model.getValue(this.node, "flow:input[@position='0']/@id");
    var src2id = this.model.getValue(this.node, "flow:input[@position='1']/@id");

    var src1cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src1id+"']/@caption");
    var src2cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src2id+"']/@caption");

    // first row
    data.layout.push({type: "inputselect", captions: [src1cap, src2cap], 
                      data: [src1id, src2id],
                      labels: [this.i18n.MERGE_FIRST_INPUT+" ", this.i18n.MERGE_SECOND_INPUT+" "]});

    data.layout.push({id: "satisfy", type: "custom"});

    // the grid
    var grid = {id: "mergeparams", type: "grid", layout: null, data: this.generateGridData(), config: this.generateConfig()}
    grid.layout = [
                   {name: "input", pos: 0, label: this.i18n.MERGE_HEADER_INPUT, type: "tree", width: "46%"},
                   {name: "operation", label: this.i18n.MERGE_HEADER_OPERATOR,
                    values: [{value: "<", label: this.i18n.OPERATION_LESSER_THAN},
                             {value: ">", label: this.i18n.OPERATION_GREATER_THAN},
                             {value: "=", label: this.i18n.OPERATION_EQUALS},
                             {value: "!=", label: this.i18n.OPERATION_NOT_EQUALS},
                             {value: "<=", label: this.i18n.OPERATION_LESSER_OR_EQUALS},
                             {value: ">=", label: this.i18n.OPERATION_GREATER_OR_EQUALS},
                             {value: "contains", label: this.i18n.OPERATION_CONTAINS},
                             {value: "notcontains", label: this.i18n.OPERATION_NOTCONTAINS},
                             {value: "starts-with", label: this.i18n.OPERATION_STARTS_WITH},
                             {value: "ends-with", label: this.i18n.OPERATION_ENDS_WITH}
                            ],
                    width: "8%"},
                   {name: "value", pos: 1, label: this.i18n.MERGE_HEADER_VALUE, type: "tree", width: "46%"}
                  ];
    data.layout.push(grid);

    return data;
  },

  buildCustomValue: function(parentnode, dataitem) {
    // can only be satisfy
    var div = document.createElement("div");

    this.conjunction = this.model.getValue(this.node, "flow:condition/@conjunction");
    this.conjunction = this.conjunction ? this.conjunction : "and";

    var select = document.createElement("select");
    var option = document.createElement("option");
    option.setAttribute("value", "and");
    option.appendChild(document.createTextNode(this.i18n.FILTER_ALL));
    if (this.conjunction == "and") {
      option.setAttribute("selected", "true");
    }
    select.appendChild(option);

    option = document.createElement("option");
    option.setAttribute("value", "or");
    option.appendChild(document.createTextNode(this.i18n.FILTER_ANY));
    if (this.conjunction == "or") {
      option.setAttribute("selected", "true");
    }

    select.appendChild(option);

    div.appendChild(select);
    dojo.connect(select, "onchange", this, "onSelectChange");

    div.appendChild(document.createTextNode(" " + this.i18n.FILTER_CONDITIONS));
    div.appendChild(document.createElement("br"));

    parentnode.appendChild(div);
  },

  onSelectChange: function(event) {
    var select = event.target;
    this.conjunction = select.value;

    var conditions = this.model.runXPath("flow:condition", this.node);

    for (var i = 0; i < conditions.length; i++) {
      this.model.setValue("@conjunction", conditions[i], this.conjunction)
    }
  },

  setChildValue: function(id, key, value) {
    var condition = this.conditions[id];
    if (key == "operation") {
      this.modifyCondition(id, key, value);
    } else if(key == "input") {
      value.inputid = this.model.getValue(this.node, "flow:input[@position='0']/@id");
      this.modifyChild(condition.inputParamId, "value", value);
    } else if (key == "value") {
      value.inputid = this.model.getValue(this.node, "flow:input[@position='1']/@id");
      this.modifyChild(condition.valueParamId, "value", value);
    }
  },

  addChild: function(gridid) {
    var id = this.model.generateId("cond");

    var conddata = {id: id, conjunction: this.conjunction , operation: "="};

    // we store per condition the two param ids we generate for later use
    var conditem = {id: id};

    var paramdata = [];
    var paramid = this.model.generateId("param");
    paramdata.push({id: paramid, value: {type: "xpath", data: ""}});
    conditem.inputParamId = paramid;

    paramid = this.model.generateId("param");
    paramdata.push({id: paramid, value: {type: "xpath", data: ""}});
    conditem.valueParamId = paramid;

    conddata.parameters = paramdata;
    this.createCondition(conddata);

    this.conditions[id] = conditem;

    return {id: id, input: "", value: ""};
  }
});

dojo.declare("flow.operator.ForeachOperator", flow.operator.BaseOperator,
{
  type: "foreach",

  minimumInputs: 2,
  maximumInputs: 2,

  selectNodes: null,
  variableList: null,
  conjunction: "and",

  setupPropertyEditor: function() {
    // variable init
    var src2id = this.model.getValue(this.node, "flow:input[@position='1']/@id");
    var params = this.getInscopeVaraibles(src2id, true);
    this.variableList = this.getVariableItems(params, false);

    this.selectNodes = [];

    // now build the data
    var data = this.generateData();

    var src1id = this.model.getValue(this.node, "flow:input[@position='0']/@id");
    var src2id = this.model.getValue(this.node, "flow:input[@position='1']/@id");

    var src1cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src1id+"']/@caption");
    var src2cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src2id+"']/@caption");

    // first row - input selection
    data.layout.push({type: "inputselect", captions: [src1cap, src2cap],
                      data: [src1id, src2id],
                      labels: [this.i18n.FOREACH_ITEM+" ", this.i18n.FOREACH_EXECUTE+" "]});

    // grid - pick up the tree from input 0
    var inputlabel, valuelabel;
    if (src2cap) {
      inputlabel = dojo.string.substitute(this.i18n.FOREACH_HEADER_INPUT, [src2cap]);
    }

    if (src1cap) {
      valuelabel = dojo.string.substitute(this.i18n.FOREACH_HEADER_VALUE, [src1cap]);
    }

    var grid = {id: "foreachparams", type: "grid", layout: null,
                data: this.generateGridData(), config: this.generateConfig()}
    grid.layout = [{name: "input", label: inputlabel, type: "custom", callback: dojo.hitch(this, this.buildSelect)},
                   {name: "value", pos: 0, label: valuelabel, type: "tree"}
                  ];

    grid.config.canAddRows = true;
    grid.config.canMoveRows = false;
    grid.config.maxRows = this.variableList.length;
    data.layout.push(grid);

    return data;
  },

  setChildValue: function(id, key, value) {
    var condition = this.conditions[id];

    if (key == "input") {
      this.modifyChild(condition.inputParamId , "value", {type:"variable",value:value});
    } else {
      value.inputid = this.model.getValue(this.node, "flow:input[@position='0']/@id");
      this.modifyChild(condition.valueParamId, "value", value);
    }
  },

  buildSelect: function(parentnode, dataitem) {
    var div = document.createElement("div");

    var select = document.createElement("select");
    select.setAttribute("rowid", dataitem.id);
    this.updateSelect(select, null, dataitem.input.value);

    div.appendChild(select);
    dojo.connect(select, "onchange", this, "onSelectChange");

    this.selectNodes.push(select);

    parentnode.appendChild(div);
  },

  onSelectChange: function(event) {
    var select = event.target;
    var value = select.value;

    // notify model
    this.setChildValue(select.getAttribute("rowid"), "input", value);

    // need to rebuild the other selects
    var selectedValues = this.getSelectedValues();

    for (var i = 0; i < this.selectNodes.length; i++) {
      if (this.selectNodes[i] != select) {
        this.updateSelect(this.selectNodes[i], selectedValues);
      }
    }
  },

  getSelectedValues: function() {
    var selectedValues = [];

    var params = this.model.runXPath(".//flow:condition/flow:param[@position='0']", this.node);
    for (var i = 0; i < params.length; i++) {
      var value = this.model.getValue(params[i], "text()");
      if (value) {
        selectedValues[value] = true;
      }
    }

    return selectedValues;
  },

  updateSelect: function(select, selectedValues, defaultValue) {
    if (!selectedValues) {
      selectedValues = this.getSelectedValues();
    }

    var selectedValue = select.value;
    if (defaultValue) {
      selectedValue = defaultValue;
    }

    // clear select
    for (var i = select.options.length-1; i >= 0; i--) {
      select.removeChild(select.options[i]);
    }

    var option = document.createElement("option");
    select.appendChild(option);

    for (var i = 0; i < this.variableList.length; i++) {
      var item = this.variableList[i];

      // skip if already selected, but not if it is our selected value
      if (selectedValues[item.id] && selectedValue != item.id) {
        continue;
      }

      var option = document.createElement("option");
      option.setAttribute("value", item.id);
      option.appendChild(document.createTextNode(item.label));

      if (item.id == selectedValue) {
        option.setAttribute("selected", true);
      }

      select.appendChild(option);
    }
  },

  createVariableDropDown: function() {
    var src2id = this.model.getValue(this.node, "flow:input[@position='1']/@id");
    var params=this.getInscopeVaraibles(src2id,true);
    var itemlist=this.getVariableItems(params, true);
    this.variableList = itemlist;
    var values = new Array();
    var j = 0;
    for (var i = 0; i < itemlist.length; i++) {
      var item = itemlist[i];
      values[j++]={label: item.label, value: item.id};
    }
    return values;
  },

  firstInputSelected: function() {
    try {
      if (this.grid) {
        this.grid.grid.layout[0].values = this.createVariableDropDown();
        this.grid.clearGrid();

        // clear all data
        this.model.clearOperatorData(this.id);

        // add dummy first row
        this.grid.addNewRow();

        var src1id = this.model.getValue(this.node, "flow:input[@position='0']/@id");
        var src2id = this.model.getValue(this.node, "flow:input[@position='1']/@id");
        var src1cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src1id+"']/@caption");
        var src2cap = this.model.getValue(this.model.flowNode, "./flow:*[@id='"+src2id+"']/@caption");

        var inputlabel, valuelabel;
        if (src2cap) {
          this.grid.grid.layout[0].label = dojo.string.substitute(this.i18n.FOREACH_HEADER_INPUT, [src2cap]);
        }

        if (src1cap) {
          this.grid.grid.layout[1].label = dojo.string.substitute(this.i18n.FOREACH_HEADER_VALUE, [src1cap]);
        }

        this.grid.buildHeaders();
      }
    } catch(ex){}

    this.selectNodes = new Array();
  },

  addChild: function(gridid) {
    var id = this.model.generateId("cond");

    var conddata = {id: id, conjunction: this.conjunction, operation: ""};

    // we store per condition the two param ids we generate for later use
    var conditem = {id: id};

    var paramdata = [];
    var paramid = this.model.generateId("param");
    paramdata.push({id: paramid, value: {type: "text", data: ""}});
    conditem.inputParamId = paramid;

    paramid = this.model.generateId("param");
    paramdata.push({id: paramid, value: {type: "xpath", data: ""}});
    conditem.valueParamId = paramid;

    conddata.parameters = paramdata;
    this.createCondition(conddata);

    this.conditions[id] = conditem;

    return {id: id, input: "", value: ""};
  }
});

dojo.declare("flow.operator.TransformOperator", flow.operator.BaseOperator,
{
  type: "transform",

  minimumInputs: 1,
  maximumInputs: 1,

  sourceTree: null,
  targetTree: null,
  menuNode: null,

  setupPropertyEditor: function() {
    if (this.model.getNode("flow:template", this.node) == null) {
      var template = '<template xmlns="'+this.model.flowNS+'"><mhub:entry xmlns=""  xmlns:mhub="'+this.model.mhubNS+'"/></template>';
      this.node.appendChild(this.model.parseXML(template));
    }

    var data = this.generateData();
    data.layout.push({id:"transform", type: "custom"});
    return data;
  },

  buildCustomValue: function(parentnode, dataitem) {
    var div = document.createElement("div");
    parentnode.appendChild(div);

    
    this.transformWidget = new flow.dijit.Transform({operator: this}, div);
  },

  dragDrop: function(srctreenode, targettreenode) {
    this.transformWidget.dragDrop(srctreenode, targettreenode);
  }
});


dojo.declare("flow.operator.PublishOperator", flow.operator.BaseOperator,
{
  type: "publish",
  hasOutput: false,
  canRename: false,

  propertiesWidget: null,

  setupPropertyEditor: function() {
    var data = this.generateData();
    data.layout.push({id:"publish", type: "custom"});
    return data;
  },

  buildCustomValue: function(parentnode, dataitem) {
    var div = document.createElement("div");
    parentnode.appendChild(div);

    
    this.propertiesWidget = new flow.dijit.Properties({model: this.model}, div);
  },

  inputRemoved: function(srcOperator) {
    // publish is special - only remove the input, not all params
    this.model.removeInput(this.node, srcOperator.id);

    return true;
  }
});


}

if(!dojo._hasResource["flow.Model"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.Model"] = true;
dojo.provide("flow.Model");

dojo.declare("flow.Model", null, {
  flowNS: "http://ibm.com/mashuphub",
  mhubNS: "http://w3.ibm.com/mhub",
  atomNS: "http://www.w3.org/2005/Atom",

  xpathTypes: {firstType: 1,
               stringType: 2,
               anyType: 3},

  // the xml document
  xmlModel: null,

  // model shortcuts
  flowNode: null,
  dataNode: null,
  prologNode: null,
  publishNode: null,

  // indicator for when we load in mashups
  isLoading: false,

  // array of loaded BaseOperators
  operators: [],

  // reference to the canvas
  canvas: null,

  constructor: function(canvas) {
    this.xmlModel = flow.xml.loadDocument("<mashup xmlns='"+this.flowNS+"' xmlns:flow='"+this.flowNS+"' version='1.0' id='' url=''><flow action='' operatorid=''><prolog/><namespaces><namespace prefix='flow'><![CDATA["+this.flowNS+"]]></namespace><namespace prefix='mhub'><![CDATA["+this.mhubNS+"]]></namespace><namespace prefix='atom'><![CDATA["+this.atomNS+"]]></namespace></namespaces><variables caption='Global Variables'/><publish type='publish' caption='' id='publish1' x='-1' y='-1'><param caption='' operatorspecific='' inputid='' id='' position='0' type='' comments=''>XML</param><param position='1' caption='repeating' operatorspecific='' inputid='' id='repeating' type='text' comments=''>item</param><param caption='header' operatorspecific='' inputid='' id='header' position='2' type='text' comments=''>root</param></publish></flow><data/></mashup>");
    this.flowNode = this.getNode("/flow:mashup/flow:flow");
    this.dataNode = this.getNode("/flow:mashup/flow:data");
    this.prologNode = this.getNode("/flow:mashup/flow:flow/flow:prolog");
    this.publishNode = this.getNode("//flow:publish", this.flowNode);

    this.loadFunctions();
    this.canvas = canvas;
  },

  /* 
   * Loads in a mashup by passing in the XML root node or the serialized string
   *
   */
  load: function(flownode) {
    this.clear();

    if (typeof(flownode) == "string") {
      flownode = this.parseXML(flownode);
    } else {
      flownode = flow.xml._importNode(this.xmlModel, flownode, true);
    }

    this.publishNode = null;
    this.flowNode.parentNode.replaceChild(flownode, this.flowNode);
    this.flowNode = flownode;
    // since we replaced the flow node, reget the publish node
    this.publishNode = this.getNode("//flow:publish", this.flowNode);

    // now parse out all operators - runXPath can return a nodelist for IE
    var operators = [];

    var sources = this.runXPath("flow:source", this.flowNode);
    for (var i = 0; i < sources.length; i++) {
      operators.push(sources[i]);
    }

    var flowoperators = this.runXPath("flow:operator", this.flowNode);
    for (var i = 0; i < flowoperators.length; i++) {
      operators.push(flowoperators[i]);
    }

    for (var i = 0; i < operators.length; i++) {
      var op = operators[i];

      var type;
      if (op.nodeName == "source") {
        type = "source";
      } else {
        type = this.getValue(op, "@type");
      }

      this.addItem(type, op);
      this.canvas.createOperator(type, op.getAttribute("id"), op.getAttribute("x"), op.getAttribute("y"));
    }

    // handle publish operator
    if (operators.length > 0 && !this.canvas.hasEndpointOperator) {
      this.canvas.hasEndpointOperator = true;
      this.canvas.createPublishOperator();
    }

    operators.push(this.publishNode);

    // draw connections - gotta make sure all operators are loaded
    for (var i = 0; i < operators.length; i++) {
      var op = operators[i];
      var opid = this.getValue(op, "@id");

      var inputs = this.getInputs(opid);

      for (var j = 0; j < inputs.length; j++) {
        var input = inputs[j];
        var sourceid = this.getValue(input, "@id");
        this.canvas.createConnection(sourceid, opid);
      }
    }

    this.setProtected(false);
  },

  getOperatorCount: function() {
    var count = 0;
    for (var x in this.operators) {
      count++;
    }

    return count;
  },

  clear: function() {
    for (var id in this.operators) {
      if (this.operators[id].type != "publish") {
        this.removeItem(id);
      }
    }

    this.operators = [];
    this.removeNodes("flow:*[name()='source' or name()='operator']", this.flowNode)
    this.removeNodes("./flow:feed", this.dataNode)
    this.removeNodes("./flow:variables/flow:param", this.flowNode);

    this.setProtected(true);
  },

  createItem: function(type) {
    var count = 0;
    for (var id in this.operators) {
      count++;
    }

    var offset = count * 10;

    var id = this.addItem(type);
    this.canvas.createOperator(type, id, offset, offset, true);
    this.canvas.showPropertyEditorForNewOperator(id);

    this.setProtected(true);
  },

  addItem: function(type, node) {
    

    // Upper case first letter
    type = type.substr(0, 1).toUpperCase() + type.substr(1);

    var op = new flow.operator[type+"Operator"]();

    if (node) {
      op.init(this, node.getAttribute("id"));
      op.node = node;
    } else {
      op.init(this);
      op.create();
    }

    this.operators[op.id] = op;

    this.setProtected(true);

    // if source and publish operator, add the endpoint publish operator
    if (op.type == "source" && !this.canvas.hasEndpointOperator) {
      this.canvas.createPublishOperator();
      this.canvas.hasEndpointOperator = true;
    }

    return op.id;
  },

  createPublishOperator: function() {
    

    var node = this.getPublishNode();

    var op = new flow.operator.PublishOperator();
    op.init(this, node.getAttribute("id"));
    op.node = node;

    this.operators[op.id] = op;
  },

  removeItem: function(id) {
    var elm = this.getById(id);

    this.operators[id].destroy();
    delete this.operators[id];

    // return a list of all inputs this operator had
    var ids = [];
    var inputs = this.runXPath("flow:input", elm);
    for (var i = 0; i < inputs.length; i++) {
      ids.push(inputs[i].getAttribute("id"));
    }

    if (elm) {
      elm.parentNode.removeChild(elm);
    }

    this.setProtected(true);

    return ids;
  },

  connectOperators: function(srcId, targetId) {
    var pos = this.runXPath("flow:operator[@id='"+targetId+"']/flow:input", this.flowNode).length;
    var template = '<input xmlns="'+this.flowNS+'" timestamp="" caption="" id="'+srcId+'" position="'+pos+'"/>';

    var node;
    if (targetId == "publish1") {
      // special case publish
      node = this.getPublishNode();

      // mark as dirty
      this.setValue("@dirty", node, "1");
    } else {
      node = this.getNode("flow:operator[@id='"+targetId+"']", this.flowNode);
    }

    node.appendChild(this.parseXML(template));

    this.setProtected(true);
  },

  disconnectOperator: function(id) {
    // removes any inputs that use the operator being removed.  Returns an
    // array of ids of the operators who had an input
    var ids = [];

    var nodes = this.runXPath("//flow:input[@id='"+id+"']", this.flowNode);
    for (var i = 0; i < nodes.length; i++) {
      ids.push(nodes[i].getAttribute("id"));
      nodes[i].parentNode.removeChild(nodes[i]);
    }

    this.setProtected(true);

    return ids;
  },

  // removes the input and any associated params
  removeInput: function(node, srcId) {
    this.removeNode("flow:input[@id='"+srcId+"']", node);
    this.removeNode("//flow:param[@inputid='"+srcId+"']", node);
  },

  disconnectOperators: function(srcId, targetId) {
    var sourceOperator = this.operators[srcId];
    var targetOperator = this.operators[targetId];

    var handled = targetOperator.inputRemoved(sourceOperator);

    if (!handled) {
      var inputposition = this.getNode("flow:input[@id='"+srcId+"']", targetOperator.node).getAttribute("position");
      this.removeInput(targetOperator.node, srcId);

      if (targetOperator.maximumInputs == 1) {
        // only one input, so remove all conditions/params
        this.removeNodes("flow:param", targetOperator.node);
        this.removeNodes("flow:condition", targetOperator.node);
      }

      if (targetOperator.maximumInputs > 1 && this.getInputCount(targetId) == 0) {
        // all inputs gone, remove all children
        this.removeNodes("flow:param", targetOperator.node);
        this.removeNodes("flow:condition", targetOperator.node);
      }

      // reset positions of other inputs
      var inputs = this.runXPath("flow:input", targetOperator.node);
      for (var i = 0; i < inputs.length; i++) {
        var input = inputs[i];
        var position = this.getValue(input, "@position");

        if (position > inputposition) {
          this.setValue("@position", input, position-1);
        }
      }
    }

    this.setProtected(true);
  },

  clearOperatorData: function(operatorId) {
    var op = this.getOperatorById(operatorId);

    this.removeNodes("flow:param", op.node);
    this.removeNodes("flow:condition", op.node);
  },

  getOperatorById: function(id) {
    return this.operators[id];
  },

  getById: function(id) {
    return this.runXPath("//*[@id='"+id+"']", null, this.xpathTypes.firstType);
  },

  generateId: function(type) {
    return type + Math.ceil(Math.random()*10000)
  },

  runXPath: function(xpath, parent, type) {
    if (!parent) {
      parent = this.xmlModel.documentElement;
    }

    if (typeof(type) == "undefined" || !type) {
      type = this.xpathTypes.anyType;
    }

    var result;
    var nsProps = {flow:this.flowNS,mhub:this.mhubNS};

    if (type == this.xpathTypes.anyType) {
      result = flow.xml.findNodes(parent, xpath, nsProps);
    } else if (type == this.xpathTypes.firstType) {
      result = flow.xml.findNode(parent, xpath, nsProps);
    } else if (type == this.xpathTypes.stringType) {
      var res = flow.xml.findNode(parent, xpath, nsProps);
      //console.log(xpath)
      if (res.firstChild) {
        result = res.firstChild.nodeValue;
      } else {
        result = "";
      }
    }

    return result;
  },

  getCount: function(node, tagName) {
    return this.runXPath(tagName, node).length;
  },

  getInputCount: function(operatorId) {
    return this.getInputs(operatorId).length;
  },

  getInputs: function(operatorId) {
    var op = this.getOperatorById(operatorId);
    return this.runXPath("flow:input", op.node);
  },

  areAllInputsSatisfied: function(operatorId) {
    // make sure all inputs produce data

    var inputs = this.getInputs(operatorId);

    var res = {satisfied: true};

    for (var i = 0; i < inputs.length; i++) {
      var id = this.getValue(inputs[0], "@id");
      var op = this.getOperatorById(id);

      if (op.getTree() == null) {
        // can't find tree
        res.satisfied = false;
        res.cause = op;
        break;
      }
    }

    return res;
  },

  createElementNS: function(namespace, tagname) {
    return flow.xml._createElementNS(this.model, namespace, tagname);
  },

  getNode: function(xpath, startnode) {
    return this.runXPath(xpath, startnode, this.xpathTypes.firstType);
  },

  getValue: function(node, xpath) {
    var value = null;

    try {
      var node = this.getNode(xpath, node);

      if (node == null && xpath != null && xpath.indexOf("text()")!=-1) {
        if (xpath.substring(xpath.length-6) == "text()") {
          var newxpath = xpath.substring(0, xpath.length-6);
          if (newxpath.charAt(newxpath.length-1) == '/'){
            newxpath = newxpath.substring(0, newxpath.length-1);
          }

          node = this.getNode(newxpath, node);
        }
      }

      if (node) {
        var type = node.nodeType;

        if (type == Node.ATTRIBUTE_NODE) {
          value = node.value;
        } else if (type == Node.TEXT_NODE) {
          value = node.nodeValue;
        } else if (type == Node.CDATA_SECTION_NODE) {
          value = node.nodeValue;
        }
      }
    } catch(e) {
    }

    return value;
  },

  setValue: function(xpath, startnode, value) {
    var node = this.getNode(xpath, startnode);

    if (!node) {
      if (xpath != null && xpath.indexOf("text()") != -1) {
        var trailing = xpath.substring(xpath.length-6);
        if (trailing == "text()") {
          var newxpath= xpath.substring(0, xpath.length-6);
          if (newxpath.charAt(newxpath.length-1)=='/') {
            newxpath = newxpath.substring(0, newxpath.length-1);
          }

          if (newxpath == "" && startnode != null) {
            node = startnode;
          } else {
            node = this.getNode(newxpath, startnode);
          }

          if (node) {
            node.appendChild(this.xmlModel.createTextNode(value));
          }
        }
      } else {
        var lastindex = xpath.lastIndexOf("/");
        if (xpath.substr(lastindex+1, 1) == "@") {
          var newxpath = xpath.substr(0, lastindex);
          var attributename = xpath.substring(lastindex+2);

          if (!newxpath) {
            node = startnode;
          } else {
            node = this.getNode(newxpath, startnode);
          }

          if (node) {
            node.setAttribute(attributename, value);
          }
        }
      }
    } else {
      var type = node.nodeType;

      if (type == Node.ATTRIBUTE_NODE) {
        node.value = value;
      } else if (node.nodeType == Node.TEXT_NODE) {
        node.nodeValue = value;
      } else if (node.nodeType == Node.CDATA_SECTION_NODE) {
        node.nodeValue = value;
      }
    }

    this.setProtected(true);
  },

  removeNode: function(string, startnode, removeall) {
    if (removeall) {
      var nodes = this.runXPath(string, startnode);

      for (var i = nodes.length-1; i >= 0; i--) {
        nodes[i].parentNode.removeChild(nodes[i]);
      }
    } else {
      var node = this.getNode(string, startnode);

      if (node && node.parentNode) {
        node.parentNode.removeChild(node);
      }
    }

    this.setProtected(true);
  },

  removeNodes: function(string, startnode) {
    this.removeNode(string, startnode, true);
  },

  removeAllChildren: function(string, startnode) {
    var node = this.getNode(string,startnode);
    for (var i = node.childNodes.length-1; i >= 0; i--) {
      node.removeChild(node.childNodes[i]);
    }

    this.setProtected(true);
  },

  parseXML: function(string) {
    var node = null;

    try {
      var doc = flow.xml.loadDocument(string);
      node = flow.xml._importNode(this.xmlModel, doc.documentElement, true);
    } catch(e) {
    }

    return node;
  },

  removeDuplicateNodes: function(node) {
    try {
      node = (node==null) ? this.xmlRoot : node;

      var ns = node.namespaceURI;
      var nn = node.nodeName;
      var parent = node.parentNode;

      //remove duplicate siblings but copy their attributes  && children first
      for (var i=parent.childNodes.length-1; parent != null && i >= 0; i--)  {
        var tNode=parent.childNodes[i];
        if (tNode!=node && tNode.nodeName ==nn && tNode.namespaceURI==ns) {
          flow.xml.copyAttributes(node, tNode);
          for (var j=0;j<tNode.childNodes.length; j++) {
            if (tNode.childNodes[j].nodeType !=window.Node.TEXT_NODE) {
              node.appendChild(tNode.childNodes[j]);
            }
          }
          parent.removeChild(tNode);
        }
      }
      for (var i = 0; i < node.childNodes.length; i++) {
        this.removeDuplicateNodes(node.childNodes[i]);
      }

      this.setProtected(true);
    } catch(ex) {
      alert("removeDuplicateNodes:"+ex.name+"   "+ex.message);
    }
  },

  loadFunctions: function(node) {
    var myThis = this;
    dojo.xhrGet(
      {url: dojo._getModulePrefix("flow")+"/data/functions.xml",
       handleAs: "xml",
       timeout: 5000,
       load: function(response, ioArgs) {
         myThis.functionXML = response;
         return response;
       },

       error: function(response, ioArgs) {
         return response;
       }
      });
  },

  getDataType: function(xpath, operatorid) {
    var opnode = this.getOperatorById(operatorid).node;
    var datatype = this.getNode("flow:datatype[text()='./"+xpath+"']", opnode);

    var type = null;
    if (datatype) {
      type = this.getValue(datatype, "@type");
    }

    return type;
  },

  updateCoordinates: function(id, left, top) {
    var op = this.getOperatorById(id);
    this.setValue("@x", op.node, left);
    this.setValue("@y", op.node, top);

    this.setProtected(true);
  },

  getPublishNode: function() {
    return this.getNode("//flow:publish", this.flowNode);
  },

  getPublishType: function() {
    var param = this.getNode("//flow:publish/flow:param[@position=0]", this.flowNode);
    return this.getValue(param, "text()");
  },

  getPublishParam: function(type) {
    var param = this.getNode("//flow:publish/flow:param[@caption='"+type+"']", this.flowNode);
    return this.getValue(param, "text()");
  },

  getRequiredPublishParams: function(type) {
    if (type == "RSS") {
      return ["title", "link", "desc"];
    } else if (type == "ATOM") {
      return ["title", "id", "link", "author"];
    } else if (type == "XML") {
      return ["repeating", "header"];
    }
  },

  getPublishInputs: function() {
    return this.getInputs(this.getValue(this.publishNode, "@id"));
  },

  isPublishValid: function() {
    var isValid = true;
    var type = this.getPublishType();

    var requirements = this.getRequiredPublishParams(type);
    for (var i = 0; i < requirements.length; i++) {
      if (!this.getPublishParam(requirements[i])) {
        isValid = false;
        break;
      }
    }

    return isValid;
  },

  setProtected: function(p) {
    if (FLOW && FLOW.plugin) {
      FLOW.plugin.tab.setProtected(p);
    }
  },

  setPublishInput: function(ptype) {
    try {
      if(ptype!=null) this.setValue("flow:publish/flow:param/text()", this.flowNode, ptype);
      var operators = this.runXPath("flow:operator/@id", this.flowNode);
      if (operators.length > 0) {
        for (var i = 0; i < operators.length; i++) {
          var operatorid = operators[i].value;
          if (this.getValue(this.flowNode, "flow:operator/flow:input[@id='"+operatorid+"']/@id") == null) {
            this.setValue("flow:publish/flow:input/@id", this.flowNode, operatorid);
            break;
          }
        }
      } else {
        var operatorid=this.getValue(this.flowNode, "flow:source/@id");
        this.setValue("flow:publish/flow:input/@id", this.flowNode, operatorid);
      }
    }catch(ex){}
  },

  stripSctipt:function(str)
  {
  	return str.replace(/<script/g,"");
  },
  destroy: function(id) {
    for (var index in this.operators) {
      delete this.operators[index];
    }

    this.setProtected(true);
  }
});

}

if(!dojo._hasResource["btp.widget.command.CompoundCommand"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.command.CompoundCommand"] = true;
dojo.provide("btp.widget.command.CompoundCommand");

// ----------------------------------------------------------------------------
// Compose multiple commands into a single command
// ----------------------------------------------------------------------------

btp.widget.command.CompoundCommand = function(){
	this._commands = [];

	this.execute = function(){
		for(var i = 0; i < this._commands.length; i++){
			var cmd = this._commands[i];
			cmd.execute();
		}
	}

	this.undo = function(){
		for(var i = this._commands.length - 1; i >= 0; i--){
			var cmd = this._commands[i];
			cmd.undo();
		}
	}

	this.append = function(cmd){
		this._commands[this._commands.length] = cmd;
	}
}

}

if(!dojo._hasResource["btp.widget.command.EditCommand"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.command.EditCommand"] = true;
dojo.provide("btp.widget.command.EditCommand");

// ----------------------------------------------------------------------------
// Command classes which encapsulate a request as an object
// ----------------------------------------------------------------------------

btp.widget.command.EditCommand = {

	/*
	 * appendChild Command
	 */
	AppendChildCommand: function(parent, child){
		this._parent = parent;
		this._child = child;
	
		this.execute = function(){
			this._parent.hlp_addChild(this._child);
		}
	
		this.undo = function(){
			this._parent.hlp_removeChild(this._child);
		}
	},

	/*
	 * insertBefore Command
	 */
	InsertBeforeCommand: function(parent, newChild, refChild){
		this._parent = parent;
		this._newChild = newChild;
		this._refChild = refChild;

		this.execute = function(){
			this._parent.hlp_addChild(this._newChild, this._refChild);
		}
	
		this.undo = function(){
			this._parent.hlp_removeChild(this._newChild);
		}
	},

	/*
	 * removeChild Command
	 */
	RemoveChildCommand: function(oldChild){
		this._oldChild = oldChild;
	
		this.execute = function(){
			this._parent = this._oldChild.getParent();
			this._refChild = this._oldChild.getNextSibling();
			this._parent.hlp_removeChild(this._oldChild);
		}
	
		this.undo = function(){
			if(this._refChild){
				this._parent.hlp_addChild(this._oldChild, this._refChild);
			}else{
				this._parent.hlp_addChild(this._oldChild);
			}
		}
	},

	/*
	 * replaceChild Command
	 */
	ReplaceChildCommand: function(parent, newChild, refChild){
		this._parent = parent;
		this._newChild = newChild;
		this._refChild = refChild;
	
		this.execute = function(){
			this._parent.hlp_addChild(this._newChild, this._refChild);
			this._parent.hlp_removeChild(this._refChild);
		}
	
		this.undo = function(){
			this._parent.hlp_addChild(this._refChild, this._newChild);
			this._parent.hlp_removeChild(this._newChild);
		}
	},

	/*
	 * changeName Command (for change operator)
	 */
	ChangeNameCommand: function(target, newName){
		this._target = target;
		this._newName = newName;
	
		this.execute = function(){
			this._oldName = this._target.nodeName;
			this._target.nodeName = this._newName;
		}
	
		this.undo = function(){
			this._target.nodeName = this._oldName;
		}
	},

	/*
	 * move Command
	 */
	MoveCommand: function(widget, /*Object*/toDim, /*Object?*/fromDim){
		// Type of toDim/fromDim: {top:"#px", left:"#px"}
		this._widget = widget;
		this._toDim = toDim;
		this._fromDim = fromDim;
	
		this.execute = function(){
			if(!this._fromDim){
				this._fromDim = {
					top: btp.getStyleValue(this._widget, "top") + "px",
					left: btp.getStyleValue(this._widget, "left") + "px"
				};
			}
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._toDim.top;
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._toDim.left;
			}
			this._widget.hlp_setStyles(props);
		}
	
		this.undo = function(){
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._fromDim.top;
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._fromDim.left;
			}
			this._widget.hlp_setStyles(props);
		}
	},

	/*
	 * resize Command
	 */
	ResizeCommand: function(widget, toDim){
		this._widget = widget;
		this._toDim = toDim;
	
		this.execute = function(){
			this._fromDim = {
				top: btp.getStyleValue(this._widget, "top") + "px",
				left: btp.getStyleValue(this._widget, "left") + "px",
				width: btp.getStyleValue(this._widget, "width") + "px",
				height: btp.getStyleValue(this._widget, "height") + "px"
			};
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._toDim.top;
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._toDim.left;
			}
			if(this._fromDim.width != this._toDim.width){
				props["width"] = this._toDim.width;
			}
			if(this._fromDim.height != this._toDim.height){
				props["height"] = this._toDim.height;
			}
			this._widget.hlp_setStyles(props);
		}
	
		this.undo = function(){
			var props = {};
			if(this._fromDim.top != this._toDim.top){
				props["top"] = this._fromDim.top;
			}
			if(this._fromDim.left != this._toDim.left){
				props["left"] = this._fromDim.left;
			}
			if(this._fromDim.width != this._toDim.width){
				props["width"] = this._fromDim.width;
			}
			if(this._fromDim.height != this._toDim.height){
				props["height"] = this._fromDim.height;
			}
			this._widget.hlp_setStyles(props);
		}
	},

	/*
	 * setPropertyValue Command
	 */
	SetPropertyValueCommand: function(widget, /*String|Object*/attribute, value){
		// Note: ATTRIBUTE should be String, but accepts Object for backward compatibility.
		this._widget = widget;
		this._attribute = attribute;
		this._value = value;
	
		this.execute = function(){
			var attrName = typeof(this._attribute) == "string" ? this._attribute : this._attribute.name;
			this._oldValue = this._widget.ps_getPropertyValue(attrName);
			this._widget.ps_setPropertyValue(attrName, this._value);
		}
	
		this.undo = function(){
			var attrName = typeof(this._attribute) == "string" ? this._attribute : this._attribute.name;
			this._widget.ps_setPropertyValue(attrName, this._oldValue);
		}
	},

	/*
	 * setStyle Command
	 */
	SetStyleCommand: function(widget, styleName, value){
		this._widget = widget;
		this._styleName = styleName;
		this._value = value;
	
		this.execute = function(){
			if(this._styleName == "top" || this._styleName == "left"
				|| this._styleName == "width" || this._styleName == "height"){
				this._oldValue = btp.getStyleValue(this._widget, this._styleName) + "px";
			}else{
				this._oldValue = dojo.style(this._widget.domNode, this._styleName);
			}
			this._widget.hlp_setStyle(this._styleName, this._value);
		}
	
		this.undo = function(){
			this._widget.hlp_setStyle(this._styleName, this._oldValue);
		}
	},

	/*
	 * update Command
	 */
	UpdateCommand: function(widget){
		this._widget = widget;
	
		this.execute = function(){
			this._widget.update();
		}
	
		this.undo = this.execute;
	}
};

}

if(!dojo._hasResource["flow.dijit.Dialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.Dialog"] = true;
dojo.provide("flow.dijit.Dialog");





dojo.declare(
	"flow.dijit.Dialog",
	[dijit.Dialog, dijit._Container],
  {
    templateString: null,
    templateString:"<div class=\"dijitDialog\">\r\n  <div dojoAttachPoint=\"titleBar\" class=\"dijitDialogTitleBar\" tabindex=\"0\" waiRole=\"dialog\">\r\n    <span dojoAttachPoint=\"titleNode\" class=\"dijitDialogTitle\">${title}</span>\r\n    <span dojoAttachPoint=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" dojoAttachEvent=\"onclick: hide\">\r\n      <span dojoAttachPoint=\"closeText\" class=\"closeText\">x</span>\r\n    </span>\r\n  </div>\r\n  <div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\" style=\"overflow:auto;\"></div>\r\n  <div dojoAttachPoint=\"actionNode\" class=\"dijitDialogPaneContent flowDialogActionBar\" style=\"padding:2px;\">\r\n    <div dojoAttachPoint=\"buttonNode\"></div>\r\n    <span dojoAttachPoint=\"resizeHandleNode\" class=\"dojoxFloatingResizeHandle\"></span>\r\n  </div>\r\n  <span dojoAttachPoint=\"tabEnd\" dojoAttachEvent=\"onfocus:_cycleFocus\" tabindex=\"0\"></span>\r\n</div>\r\n",
    actionNode: null,
    buttonNode: null,
    resizeHandleNode: null,
    operatorId: null,

    showUnderlay: false,
    hideTitleBar: false,
    hideActionBar: false,
    buttons: null,

    hasBeenResized: false,

    postCreate: function() {
      if (this.hideTitleBar) {
        this.titleBar.style.display = "none";
      }

      if (this.hideActionBar) {
        this.actionNode.style.display = "none";
      }

      //this.inherited("postCreate",arguments);
			this.bgIframe = new dijit.BackgroundIframe(this.domNode);

      if (this.buttons) {
        for (var i = 0; i < this.buttons.length; i++) {
          var button = document.createElement("button")
          button.appendChild(document.createTextNode(this.buttons[i].label));

          if (this.buttons[i].onclick) {
            button.onclick = this.buttons[i].onclick;
          }

          this.buttons[i].node = this.buttonNode.appendChild(button);
        }
      } else {
        // since IE doesn't do min-height
        this.actionNode.style.height = "13px";
      }
    },

    _position: function() {
      if (this.domNode.parentNode == dojo.body()) {
        // default behavior
        if (!dojo.getComputedStyle(this.domNode)) {
          // means the document isn't visible, and dojo will barf
          return;
        }

        this.inherited("_position", arguments);
      } else {
        // Our dialogs sometimes need to live within tabs - so handle specially
        // - viewport is the FlowContainer
        var viewport = dojo.marginBox(this.domNode.parentNode.parentNode);
        var mb = dojo.marginBox(this.domNode);

        var style = this.domNode.style;
        style.left = Math.floor((viewport.l + (viewport.w - mb.w)/2)) + "px";
        style.top = Math.floor((viewport.t + (viewport.h - mb.h)/2)) + "px";
      }
		},

    show: function() {
      var doLayout = false;

      if (!this._alreadyInitialized) {
        this._setup();
        this._alreadyInitialized = true;
        doLayout = true;
      }

      this.domNode.style.display = "block";

      dojo.addClass(this.domNode, "flowdijitDialog");

      if (this.showUnderlay) {
        this._underlay.show();
      } else {
      }

      if (doLayout) {
        this.layout();
      }

      this.open = true;
      this._savedFocus = dijit.getFocus(this);
      dijit.focus(this.titleBar);

      // TODO: doron - hack to make tabs work in dialogs for us
      this.resizeChildren();
    },

    hide: function() {
      if (!this._alreadyInitialized) {
        return;
      }

      this.domNode.style.display = "none";

      if (this.showUnderlay) {
        this._underlay.hide();
      }

      dijit.focus(this._savedFocus);
      this.open = false;
    },

    isVisible: function() {
      return this.domNode.style.display == "block";
    },

    layout: function() {
      if (this.domNode.style.display == "block") {
        this._position();

        if (this._underlay) {
          this._underlay.layout();
          if (this.chain) {
            var zindex = parseInt(dojo.style(this._underlay.domNode, "zIndex"), 10)+this.chain;
            this._underlay.domNode.style.zIndex = zindex;
            this.domNode.style.zIndex = zindex+1;
          }
        } else if (this.chain) {
          this.updateZIndex();
        }
      }
    },

    resize: function(data) {
      var w = data.w;
      var h = data.h;

      // check if width/height puts us outside of window area
      if (this.hasBeenResized) {
        var coords = dojo.coords(this.domNode);
        var windowcoords = dojo.coords(dojo.body());

        //console.log("1: "+coords.x + " "+w+" > "+windowcoords.w);
        //console.log("2: "+coords.y + " "+h+" > "+windowcoords.h);

        if (coords.x + w >= windowcoords.w || coords.y + h >= windowcoords.h) {
          return;
        }
      } else {
        this.hasBeenResized = true;
      }

      dojo.style(this.domNode, "width", w+"px");
      dojo.style(this.domNode, "height", h+"px");

      // now resize the dialog contents
      var titlebarBox = dojo.marginBox(this.titleBar);

      // TODO: 4 is fixed padding (both sides, so 8), needs to be calculated
      // IE works differently, so no padding is needed (else we get a visual drawing bug)
      h -= titlebarBox.h + dojo.marginBox(this.actionNode).h + (dojo.isIE ? 1 : 9);
      dojo.style(this.containerNode, "height", h+"px");

      // TODO: doron - hack to make tabs work in dialogs for us
      this.resizeChildren();
    },

    resizeChildren: function(data) {
      var children = this.getChildren();
      for (var i = 0; i < children.length; i++) {
        if (children[i].resize) {
          children[i].resize();
        }
      }

      dojo.query(".flowResizable", this.domNode).forEach(function(node){
         dijit.byNode(node).resize();
      });
    },

    setTitle: function(title) {
      this.titleNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(title);
    },

    _setup: function() {
			if (this.titleBar) {
				this._moveable = new dojo.dnd.move.parentConstrainedMoveable(this.domNode, {handle: this.titleBar, area: "margin", within: true});
        dojo.connect(this._moveable, "onFirstMove", function(){this.handle.focus()});
			}

      if (this.showUnderlay) {
        this._underlay = new dijit.DialogUnderlay();
      }

      

      var tmp = new dojox.layout.ResizeHandle({ 
        targetId: this.id,
        targetWidget: this,
        resizeAxis: "xy"
      }, this.resizeHandleNode);
    },

    updateZIndex: function() {
      var zindex = parseInt(dojo.style(this.domNode, "zIndex"), 10) + this.chain;
      dojo.style(this.domNode, "zIndex", zindex);
    }
  }
);

}

if(!dojo._hasResource["dijit.layout.TabContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.TabContainer"] = true;
dojo.provide("dijit.layout.TabContainer");




dojo.declare("dijit.layout.TabContainer",
	[dijit.layout.StackContainer, dijit._Templated],
	{	
	// summary: 
	//	A Container with Title Tabs, each one pointing at a pane in the container.
	// description:
	//	A TabContainer is a container that has multiple panes, but shows only
	//	one pane at a time.  There are a set of tabs corresponding to each pane,
	//	where each tab has the title (aka title) of the pane, and optionally a close button.
	//
	//	Publishes topics <widgetId>-addChild, <widgetId>-removeChild, and <widgetId>-selectChild
	//	(where <widgetId> is the id of the TabContainer itself.
	//
	// tabPosition: String
	//   Defines where tabs go relative to tab content.
	//   "top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	templateString: null,	// override setting in StackContainer
	templateString:"<div class=\"dijitTabContainer\">\r\n\t<div dojoAttachPoint=\"tablistNode\"></div>\r\n\t<div class=\"dijitTabPaneWrapper\" dojoAttachPoint=\"containerNode\"></div>\r\n</div>\r\n",

	postCreate: function(){	
		dijit.layout.TabContainer.superclass.postCreate.apply(this, arguments);
		// create the tab list that will have a tab (a.k.a. tab button) for each tab panel
		this.tablist = new dijit.layout.TabController(
			{
				id: this.id + "_tablist",
				tabPosition: this.tabPosition,
				doLayout: this.doLayout,
				containerId: this.id
			}, this.tablistNode);		
	},

	_setupChild: function(/* Widget */tab){
		dojo.addClass(tab.domNode, "dijitTabPane");
		this.inherited("_setupChild",arguments);
		return tab; // Widget
	},

	startup: function(){
		if(this._started){ return; }

		// wire up the tablist and its tabs
		this.tablist.startup();
		this.inherited("startup",arguments);

		if(dojo.isSafari){
			// sometimes safari 3.0.3 miscalculates the height of the tab labels, see #4058
			setTimeout(dojo.hitch(this, "layout"), 0);
		}
	},

	layout: function(){
		// Summary: Configure the content pane to take up all the space except for where the tabs are
		if(!this.doLayout){ return; }

		// position and size the titles and the container node
		var titleAlign=this.tabPosition.replace(/-h/,"");
		var children = [
			{domNode: this.tablist.domNode, layoutAlign: titleAlign},
			{domNode: this.containerNode, layoutAlign: "client"}
		];
		dijit.layout.layoutChildren(this.domNode, this._contentBox, children);

		// Compute size to make each of my children.
		// children[1] is the margin-box size of this.containerNode, set by layoutChildren() call above
		this._containerContentBox = dijit.layout.marginBox2contentBox(this.containerNode, children[1]);

		if(this.selectedChildWidget){
			this._showChild(this.selectedChildWidget);
			if(this.doLayout && this.selectedChildWidget.resize){
				this.selectedChildWidget.resize(this._containerContentBox);
			}
		}
	},

	destroy: function(){
		this.tablist.destroy();
		this.inherited("destroy",arguments);
	}
});

//TODO: make private?
dojo.declare("dijit.layout.TabController",
	dijit.layout.StackController,
	{
	// summary:
	// 	Set of tabs (the things with titles and a close button, that you click to show a tab panel).
	// description:
	//	Lets the user select the currently shown pane in a TabContainer or StackContainer.
	//	TabController also monitors the TabContainer, and whenever a pane is
	//	added or deleted updates itself accordingly.

	templateString: "<div wairole='tablist' dojoAttachEvent='onkeypress:onkeypress'></div>",

	// tabPosition: String
	//   Defines where tabs go relative to the content.
	//   "top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	// doLayout: Boolean
	// 	TODOC: deprecate doLayout? not sure.
	doLayout: true,

	// buttonWidget: String
	//	the name of the tab widget to create to correspond to each page
	buttonWidget: "dijit.layout._TabButton",

	postMixInProperties: function(){
		this["class"] = "dijitTabLabels-" + this.tabPosition + (this.doLayout ? "" : " dijitTabNoLayout");
		this.inherited("postMixInProperties",arguments);
	}
});

dojo.declare("dijit.layout._TabButton",
	dijit.layout._StackButton,
	{
	// summary:
	//	A tab (the thing you click to select a pane).
	// description:
	//	Contains the title of the pane, and optionally a close-button to destroy the pane.
	//	This is an internal widget and should not be instantiated directly.

	baseClass: "dijitTab",

	templateString:"<div dojoAttachEvent='onclick:onClick,onmouseenter:_onMouse,onmouseleave:_onMouse'>\r\n    <div class='dijitTabInnerDiv' dojoAttachPoint='innerDiv'>\r\n        <span dojoAttachPoint='containerNode,focusNode'>${!label}</span>\r\n        <span dojoAttachPoint='closeButtonNode' class='closeImage' dojoAttachEvent='onmouseenter:_onMouse, onmouseleave:_onMouse, onclick:onClickCloseButton' stateModifier='CloseButton'>\r\n            <span dojoAttachPoint='closeText' class='closeText'>x</span>\r\n        </span>\r\n    </div>\r\n</div>\r\n",

	postCreate: function(){
		if(this.closeButton){
			dojo.addClass(this.innerDiv, "dijitClosable");
		} else {
			this.closeButtonNode.style.display="none";
		}
		this.inherited("postCreate",arguments); 
		dojo.setSelectable(this.containerNode, false);
	}
});

}

if(!dojo._hasResource["flow.dijit.XMLTypeTree"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.XMLTypeTree"] = true;
dojo.provide("flow.dijit.XMLTypeTree");







dojo.declare(
  "flow.dijit.XMLTypeTree",
  flow.dijit.XMLViewTree,
  {
    chain: null,
    contextmenu: null,
    isTypeTree: true,

    postCreate: function() {
      this.inherited("postCreate", arguments);

      if (dojo.byId(this.domNode.getAttribute("id"))) {
        // make sure the domNode is in the DOM for real - XMLViewTextBox is the
        // culprit.
        this.initContextMenu();
      }
    },

    initContextMenu: function() {
      if (this.contextmenu) {
        return;
      }

      this.contextmenu = new flow.dijit.Menu({hparent: this, targetNodeIds: [this.domNode.getAttribute("id")]});

      dojo.connect(this.contextmenu, "onItemClick", this, this.onItemClick);
      dojo.connect(this.contextmenu, "_openMyself", this, this.openContextMenu);
      dojo.connect(this.contextmenu, "onClose", this, this.closeContextMenu);
    },

    buildMenuItems: function() {
      var contextmenu = this.contextmenu;
      dojo.forEach(this.contextmenu.getChildren(), function(child){ contextmenu.removeChild(child); });

      var labels = dojo.i18n.getLocalization("flow", "labels");
      var items = [
                   {id: "date", label: labels.TYPE_DATE},
                   {id: "decimal", label: labels.TYPE_DECIMAL},
                   {id: "integer", label: labels.TYPE_INTEGER},
                   {id: "string", label: labels.TYPE_STRING},
                   {},
                   {id: "clear", label: labels.TYPE_CLEAR, disabled: this.selectedTreeNode.flowtype ? false : true}
                   ];

      for (var i = 0; i < items.length; i++) {
        var item = items[i];

        if (typeof(item.label) == "string") {
          var type = item.id.substr(0, 1).toUpperCase() + item.id.substr(1)
          var menuitem = new dijit.MenuItem({label: item.label, action: item.id, disabled: item.disabled, iconClass: "flowtype flowtype"+type});
          this.contextmenu.addChild(menuitem);
        } else {
          this.contextmenu.addChild(new dijit.MenuSeparator());
        }
      }
    },

    openContextMenu: function(event) {
      if (this.selectedTreeNode) {
        dojo.removeClass(this.selectedTreeNode.domNode, "selected");
      }

      this.selectedTreeNode = dijit.getEnclosingWidget(event.target);
      dojo.addClass(this.selectedTreeNode.domNode, "selected");

      this.buildMenuItems();

      if (this.chain) {
        // reset zIndex of popup
        var zindex = parseInt(dojo.style(this.contextmenu.domNode.parentNode, "zIndex"), 10)+this.chain;
        dojo.style(this.contextmenu.domNode.parentNode, "zIndex", zindex);
      }
      dojo.stopEvent(event);
    },

    closeContextMenu: function(event) {
    },

    onItemClick: function(menuitem) {
      if (menuitem.disabled) {
        return false;
      }

      var action = menuitem.action;

      if (action == "clear") {
        this.clearType();
      } else {
        this.setType(action);
      }

      //dijit.popup.close(this.contextmenu);
      //dijit.focus(this.contextmenu.savedFocus);
      // can't close the context menu via popup as it will hide the dropdown
      dijit.focus(this.contextmenu.savedFocus);
      dojo.style(this.contextmenu.domNode, "display", "none");
    },

    setType: function(type, treenode) {
      var node = treenode ? treenode : this.selectedTreeNode;

      if (node.flowtype == type) {
        return;
      }

      // check if type exists.  If so, remove
      this.clearType(node);

      // remember the type
      node.flowtype = type;

	  var parentnode =  this.store.getNode(this.getTreeNodeByItem(node.item).getParent().item);
      var path = "./"+this.store.buildXPath(this.store.getNode(node.item),true,true,null,parentnode);
      var template = '<datatype xmlns="'+this.operator.flowNS+'" type="'+type+'" ><![CDATA['+path+']]></datatype>';
      this.operator.node.appendChild(this.operator.model.parseXML(template));

      node.iconNode.style.width = node.iconNode.offsetWidth + 16 + "px";
      var newnode = document.createElement("div");
      newnode.className = "dijitInline flowtype "+this.getActionClassName(type);
      //newnode.style.marginLeft = "16px";
      node.iconNode.appendChild(newnode);
    },

    clearType: function(treenode) {
      var node = treenode ? treenode : this.selectedTreeNode;

	  var parentnode =  this.store.getNode(this.getTreeNodeByItem(node.item).getParent().item);
      var path = "./"+this.store.buildXPath(this.store.getNode(node.item),true,true,null,parentnode);
      this.operator.model.removeNodes(".//flow:datatype[text()='"+path+"']", this.operator.node);

      if (!node.flowtype) {
        return;
      }

      var nodes = dojo.query("div."+this.getActionClassName(node.flowtype), node.iconNode);

      if (nodes[0]) {
        nodes[0].parentNode.removeChild(nodes[0]);
        node.iconNode.style.width = "";
        node.flowtype = null;
      }
    },

    getActionClassName: function(action) {
      var classname = "flowtype"+action.charAt(0).toUpperCase() + action.substr(1);
      return classname;
    },

    getIconClass: function(item) {
      if (item) {

      }

      this.inherited("getIconClass", arguments);
    }
  }
);

// menu subclass to work around a dijit.popup bug.  We need to have the popup the
// menu creates be parented to our dropdown, or else showing the context menu
// will close the dropdown.

dojo.declare(
	"flow.dijit.Menu",
	dijit.Menu,
{
  _openMyself: function(/*Event*/ e){
		// summary:
		//		Internal function for opening myself when the user
		//		does a right-click or something similar

		dojo.stopEvent(e);

		// Get coordinates.
		// if we are opening the menu with the mouse or on safari open
		// the menu at the mouse cursor
		// (Safari does not have a keyboard command to open the context menu
		// and we don't currently have a reliable way to determine
		// _contextMenuWithMouse on Safari)
		var x,y;
		if(dojo.isSafari || this._contextMenuWithMouse){
			x=e.pageX;
			y=e.pageY;
		}else{
			// otherwise open near e.target
			var coords = dojo.coords(e.target, true);
			x = coords.x + 10;
			y = coords.y + 10;
		}

		var self=this;
		var savedFocus = dijit.getFocus(this);
    this.savedFocus = savedFocus;
		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			dijit.focus(savedFocus);
			dijit.popup.close(self);
		}
		dijit.popup.open({
			popup: this,
			x: x,
			y: y,
      parent: this.hparent, // needed to make context menu work on a tree in a dropdown
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			dijit.popup.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		}
	}
});

}

if(!dojo._hasResource["flow.dijit.XMLViewTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.XMLViewTextBox"] = true;
dojo.provide("flow.dijit.XMLViewTextBox");






dojo.declare(
  "flow.dijit.XMLViewTextBox",
  [dijit.form.TextBox, dijit.form.ComboBoxMixin],
  {
    // callback to call with set value
    callback: null,
    data: null,
    operator: null,
    removeDuplicates: false,

    selectedTreeItem: null,
    xmlRootNode: null,
    inputid: null,
    path: null,

    isLoaded: false,

    baseClass: "flowdijitXMLViewTextBox dijitComboBox",

    postMixInProperties: function(){
      dijit.form.ComboBoxMixin.prototype.postMixInProperties.apply(this, arguments);
      dijit.form.TextBox.prototype.postMixInProperties.apply(this, arguments);
    },

    postCreate: function() {
      dijit.form.TextBox.prototype.postCreate.apply(this, arguments);

      if (!this.operator) {
        this.setDisabled(true);
      }

      // don't disable the textbox for now
      //this.textbox.setAttribute("disabled", "true");

      if (this.data && this.data.value) {
        this.selectedTreeItem = this.data.value;
        this.setValue(this.data.value);
      } else {
        this.setValue("");
      }

      dojo.addClass(this.iconNode, "flowtypeCombobox")
      dojo.addClass(this.iconNode, "flowtypeComboboxTree")
      this.iconNode.style.visibility = "visible";

      this.isLoaded = true;
    },

    _onArrowMouseDown: function(e) {
      dojo.stopEvent(e);
      if (this._isShowingNow) {
        this.closeDropdown();
        this._isShowingNow = false;
      } else {
        this._picker = new flow.dijit.XMLTypeTree({xmlRootNode: this.xmlRootNode,
                                                   operator: this.operator,
                                                   path: this.path,
                                                   selectedXPath: this.selectedTreeItem,
                                                   onClick: dojo.hitch(this, this.onTreeItemClick),
                                                   getLabelClass: dojo.hitch(this, this.onTreeGetLabelClass),
                                                   removeDuplicates: this.removeDuplicates
                                                  });
        this._picker.domNode.style.height = "200px";

        this.showDropdown();
        this._isShowingNow = true;
      }
    },

		_hideResultList: function(){
			if(this._isShowingNow){
				dijit.popup.close(this._picker);
				this._arrowIdle();
				this._isShowingNow=false;
			}
		},

    showDropdown: function() {
      dijit.popup.open({
          parent: this,
          popup: this._picker,
          around: this.domNode,
          onCancel: dojo.hitch(this, this.closeDropdown),
          onClose: dojo.hitch(this, this.closeDropdown)
      });

      dojo.marginBox(this._picker.domNode,{ w:this.domNode.offsetWidth, h: 200});
      this._picker.domNode.style.backgroundColor = "white";
      this._picker.domNode.style.overflow = "auto";
      this._picker.domNode.style.border = "1px solid black";

      // focus the node so that blur happens
      this._picker.domNode.focus();
      this._picker.initContextMenu();
    },

    closeDropdown: function() {
      this._isShowingNow = false;
      dijit.popup.close(this._picker);
    },

    onTreeItemClick: function(item, node) {
      var value = "./"+this._picker.getXPathForTreeItem(item);

      this.selectedTreeItem = value;

      this.setValue(value);

      this.closeDropdown();
    },

    setValue: function(value, priorityChange) {
      this.inherited("setValue", arguments);

      this.callCallback(value);
    },

    callCallback: function(value) {
      // don't callback if not loaded yet
      if (this.callback && this.isLoaded) {
        var data = flow.operator.Data.create("xpath");
        data.value = value;
        data.inputid = this.inputid;
        this.callback(data);
      }
    },

    onTreeGetLabelClass: function(item) {
      if (item && this.selectedTreeItem && "./"+item.store.buildXPath(item.store.getNode(item), true) == this.selectedTreeItem) {
        return "xmlviewtreeselected";
      }

      return "";
    }
  }
);

}

if(!dojo._hasResource["flow.dijit.XMLInputSelection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.XMLInputSelection"] = true;
dojo.provide("flow.dijit.XMLInputSelection");









dojo.declare(
	"flow.dijit.XMLInputSelection",
	[dijit._Widget, dijit._Templated, dijit._Container],
  {
    // attach points
    labelNode: null,
    xmlTreeNode: null,

    // arguments
    data: null,
    operator: null,
    chain: null,

    // impl
    xmltree: null,
    treeid: null,

    templateString:"<div>\r\n  <div dojoAttachPoint=\"labelNode\">${i18n.propertyeditor.GET_TREE_DESCRIPTION}</div>\r\n  <div dojoAttachPoint=\"xmlTreeNode\" style=\"margin-top: 10px\"></div>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    postCreate: function() {
      var div = document.createElement("div");
      this.xmlTreeNode.appendChild(div);

      // setup the tree
      var treedata = this.operator.getTree(0);
      if (treedata) {
        this.operator.model.setValue("flow:input[@position='0']/@timestamp",this.operator.node,treedata.timestamp);
      }
      this.treeid = treedata.id;
      this.xmltree = new flow.dijit.XMLTypeTree({xmlRootNode: treedata.root, path: treedata.path, getLabelClass: dojo.hitch(this, this.onTreeGetLabelClass), operator: this.operator, chain: this.chain}, div);
    },

    onTreeGetLabelClass: function(item) {
      var value = this.data ? this.data.value : null;

      if (item && value && item.store.buildXPath(item.store.getNode(item), true) == value) {
        return "xmlviewtreeselected";
      }

      return "";
    },

    getData: function() {
      var data = flow.operator.Data.create("xpath");

      if (this.xmltree.selectedItem) {
        var store = this.xmltree.store;
        data.value = "./"+this.xmltree.getXPathForTreeItem(this.xmltree.selectedItem);
        data.inputid = this.treeid;
      }

      return data;
    }
  }
);

}

if(!dojo._hasResource["flow.dijit.RuntimeParameterInputSelection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.RuntimeParameterInputSelection"] = true;
dojo.provide("flow.dijit.RuntimeParameterInputSelection");







dojo.declare(
	"flow.dijit.RuntimeParameterInputSelection",
	[dijit._Widget, dijit._Templated],
  {
    // attach points
    labelNodeName: null,
    nameNode: null,
    labelNodeDefault: null,
    defaultValueNode: null,

    // arguments
    operator: null,
    data: null,

    templateString:"<div>\r\n  <div dojoAttachPoint=\"labelNodeName\">${i18n.propertyeditor.DECLARE_VARIABLE_NAME}</div>\r\n  <input type=\"text\" dojoAttachPoint=\"nameNode\" style=\"margin-top: 10px\" />\r\n  <div dojoAttachPoint=\"labelNodeDefault\">${i18n.propertyeditor.DECLARE_VARIABLE_DEFAULT_VALUE}</div>\r\n  <input type=\"text\" dojoAttachPoint=\"defaultValueNode\" style=\"margin-top: 10px\" />\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    postCreate: function() {
      // TODO: If type of data is text, use caption.  This is when we convert text
      // to dynamic.  Probably needs to be fixed at some point
      if (this.data.type == "text") {
        this.nameNode.value = (this.data && this.data.caption) ? this.data.caption : "";
      } else {
        this.nameNode.value = (this.data && this.data.name) ? this.data.name : "";
      }
      this.defaultValueNode.value = (this.data && this.data.value) ? this.data.value : "";
    },

    getData: function() {
      if (this.nameNode.value.replace(/^\s+|\s+$/g,"")=="") {
        alert(this.operator.i18n.RUNTIMEPARAMSELCTIONDIALOG_NAME_MISSING);
        return null;
      }

      var data = flow.operator.Data.create("dynamic");

      if (this.nameNode.value) {
        data.name = this.nameNode.value;
      }

      if (this.defaultValueNode.value) {
        data.value = this.defaultValueNode.value;
      }

      return data;
    }
  }
);

}

if(!dojo._hasResource["flow.dijit.FunctionInputSelection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.FunctionInputSelection"] = true;
dojo.provide("flow.dijit.FunctionInputSelection");





dojo.declare(
	"flow.dijit.FunctionInputSelection",
	[dijit._Widget, dijit._Templated],
  {
    // attach points
    labelNode: null,
    selectNode: null,
    gridNode: null,
    commentLableNode: null,
    commentInputNode:null,

    // arguments
    chain: null,
    data: null,
    operator: null,
    chainParent: null,

    // impl
    grid: null,

    // means it can open new dialogs
    isChainable: true,

    // holds the generated tmp model data
    data: null,

    imgPath: dojo.moduleUrl("flow.dijit", "templates/images"),
    templateString:"<div>\r\n  <table width=\"100%\">\r\n    <tr>\r\n      <td width=\"100%\">\r\n        <div dojoAttachPoint=\"labelNode\"><select dojoAttachPoint=\"selectNode\" dojoAttachEvent=\"onchange:onFunctionSelect\"><option value=\"-1\">${i18n.propertyeditor.USE_FUNCTION_LABEL}</option></select></div>\r\n      </td>\r\n      <td><img dojoAttachEvent=\"onclick:showHelp\" src=\"${imgPath}/dialoghelp.png\" title=\"${i18n.propertyeditor.USE_FUNCTIONS_HELP_TOOLTIP}\"></td>\r\n    </tr>\r\n  </table>\r\n\r\n  <div dojoAttachPoint=\"gridNode\" style=\"margin-top: 10px\"></div>\r\n  <div dojoAttachPoint=\"commentLableNode\" style=\"margin-top: 10px\">${i18n.propertyeditor.USE_FUNCTION_COMMENTS} <input dojoAttachPoint=\"commentInputNode\" dojoAttachEvent=\"onchange:onCommentsChanged\"/></div>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
      this.i18n.functions = dojo.i18n.getLocalization("flow", "functions");
    },

    postCreate: function() {
      var defaultValue = null;

      if (this.data && this.data.name) {
        defaultValue = this.data.name;
      }

      var selectedFunction = null;
      var model = this.operator.model;

      var groupings = model.runXPath("//grouping", model.functionXML);
      for (var i = 0; i < groupings.length; i++) {
        var grouping = groupings[i];
        var optgroup = document.createElement("optgroup");

        var id = model.getValue(grouping, "@id");
        var label = this.getFunctionsString(id.toUpperCase()+"_GROUP_LABEL");

        optgroup.setAttribute("label", label);

        var functions = model.runXPath("function", grouping);

        for (var j = 0; j < functions.length; j++) {
          var func = functions[j];
          var id = model.getValue(func, "@id");
          var caption = this.getFunctionsString(id.toUpperCase()+"_LABEL");

          var option = document.createElement("option");
          option.setAttribute("value", id);
          option.setAttribute("title", this.getFunctionsString(id.toUpperCase()+"_TOOLTIP"));
          option.appendChild(document.createTextNode(caption));

          if (defaultValue && id == defaultValue) {
            option.setAttribute("selected", "true");
            selectedFunction = functions[j];
          }

          optgroup.appendChild(option);
        }

        this.selectNode.appendChild(optgroup);
      }

      if (this.data && selectedFunction) {
        // we have data, populate!
        this.commentInputNode.value = this.data.comments ? this.data.comments : "";
        var allowdynamic = (model.getValue(selectedFunction, "@fixedargs") != "true") ? true : false;

        // if we have data, we need to annote fixed from the available inputs
        if (selectedFunction && selectedFunction.inputs) {
          var args = model.runXPath("arg", selectedFunction);
          for (var i = 0; i < args.length; i++) {
            var arg = args[i];
            var id = model.getValue(arg, "@id");

            for (var j = 0; j < this.data.parameters.length; j++) {
              if (this.data.parameters[i].caption = id) {
                this.data.parameters[i].fixed = true;
                break;
              }
            }

          }
        }

        this.buildGrid(allowdynamic);
      } else {
        this.selectNode.selectedIndex = 0;
        this.data = flow.operator.Data.create("function");
      }
    },

    prepareLocalizationString: function(str) {
      // can't contain "-"
      str = str.replace(/-/g, "_");
      return str.toUpperCase();
    },

    getFunctionsString: function(name) {
      return this.i18n.functions[this.prepareLocalizationString(name)];
    },

    onCommentsChanged: function(e){
      this.data.comments = this.commentInputNode.value;
    },

    onFunctionSelect: function(e) {
      var value = this.selectNode.value;

      if (this.grid) {
        this.grid.destroy();
        this.grid = null;
      }

      // clear out data
      this.data.parameters = [];

      if (value == "-1") {
        // no function selected
        this.data.name = "";
        return;
      }

      var model = this.operator.model;
      this.data.name = value;

      var dynamicInputs = (model.getValue(model.functionXML, "//function[@id='"+value+"']/@fixedargs") != "true") ? true : false;

      var args = model.runXPath("//function[@id='"+value+"']/arg", model.functionXML);
      for (var i = 0; i < args.length; i++) {
        var arg = args[i];

        var argid = model.getValue(arg, "@id");

        var type = model.getValue(arg, "@type");
        // TODO: hack here!
        type = "text";

        var defaultvalue = model.getValue(arg, "@default");
        if (defaultvalue == null) {
          defaultvalue = "";
        }

        var param = flow.operator.Data.create(type, this.operator.model.generateId("param"));
        param.caption = argid;
        param.value = defaultvalue;
        param.fixed = true;

        // check if has values
        var options = model.runXPath(".//option", arg);
        if (options && options.length > 0) {
          param.values = [];
          for (var j = 0; j < options.length; j++) {
            var value = model.getValue(options[j], "@value");

            // if no default value, the value becomes the first option
            if (j == 0 && !defaultvalue) {
              param.value = value;
            }

            param.values.push({value: value, label: this.getFunctionsString(this.data.name.toUpperCase()+ "_ARGUMENTVALUE_" + value.toUpperCase())});
          }
        }

        this.data.parameters.push(param);
      }

      // build the grid - only if it has args
      if (args.length > 0) {
        this.buildGrid(dynamicInputs);
      }
    },

    buildGrid: function(allowDynamicRows) {
      

      var grid = {id: this.id, layout: [{name: "name", label: this.i18n.propertyeditor.USE_FUNCTIONS_GRID_ARGUMENT_NAME, type: "text", width: "33%"},
                                        {name: "value", label: this.i18n.propertyeditor.USE_FUNCTIONS_GRID_ARGUMENT_VALUE, options: ["text", "xpath", "function", "variable"], type: "param",  width: "67%"}],
                  data: [], config: {}};
      grid.config.canAddRows = allowDynamicRows ? true : false;
      grid.config.canMoveRows = false;
      grid.config.allowGridEditing = true;

      if (this.data) {
        // load in parameters - and translate

        for (var i = 0; i < this.data.parameters.length; i++) {
          var param = this.data.parameters[i];
          var caption = this.getFunctionsString(this.data.name.toUpperCase()+ "_ARGUMENT_" + param.caption.toUpperCase());
          var item = {id: param.id, name: caption, fixed: param.fixed, value: {}};

          // TODO: value needs to exist to tell it to keep working.  ugly!
          item.value = param;

          grid.data.push(item);
        }
      }

      var div = document.createElement("div");
      this.gridNode.appendChild(div);
      this.grid = new flow.dijit.Grid({grid: grid, operator: this, chain: this.chain, chainParent: this}, div);
    },

    addChild: function() {
      var param = flow.operator.Data.create("text", this.operator.model.generateId("param"));
      param.name = "Custom";
      param.caption = "Custom";
      param.value = "";

      this.data.parameters.push(param);
      return param;
    },

    getChildIndexById: function(id) {
      var index = null;
      for (var i = 0; i < this.data.parameters.length; i++) {
        if (this.data.parameters[i].id == id) {
          index = i;
          break;
        }
      }

      return i;
    },

    getChildById: function(id) {
      return this.data.parameters[this.getChildIndexById(id)];
    },

    setChildValue: function(id, key, value) {
      var param = this.getChildById(id);

      if (value == null) {
        // clearing value
        param.value = "";
        param.type = "text";
      } else {
        // can either contain value/type or data/dtype
        if (value.type == "function") {
          param.value = value
        } else {
          if (value.type == "xpath" && value.inputid!=null) {
            param.inputid = value.inputid;
          }

          if (value.value != null) {
            param.value = value.value;
          } else {
            param.value = value.data;
          }
        }

        param.type = value.type;
      }
    },

    modifyChild: function(id, key, value) {
      // empty, as data will be updated during dialog acceptance - not dynamic!
    },

    removeChild: function(id) {
      // TODO: use splice?
      var index = this.getChildIndexById(id);

      this.data.parameters[index] = null;
      delete this.data.parameters[index];
    },

    getData: function() {
      return this.data;
    },

    showHelp: function() {
    	var title = dojo.string.substitute(this.i18n.propertyeditor.HELP_TITLE, ["Transform Functions"]);
    	var selectedFun = this.selectNode.value;
    	var url;
    	//console.debug(this.selectNode.value);

    	switch (selectedFun) {
        case "concat":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_concatenate.htm";
          break;
        case "contains":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_contains.htm";
          break;
        case "regexp":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_regexp.htm";
          break;
        case "trim":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_trim.htm";
          break;
        case "ltrim":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_lefttrim.htm";
          break;
        case "rtrim":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_righttrim.htm";
          break;
        case "starts-with":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_startswith.htm";
          break;
        case "ends-with":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_endswith.htm";
          break;
        case "substring":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_substring.htm";
          break;
        case "not":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_not.htm";
          break;
        case "numeric-unary-plus":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_unaryplus.htm";
          break;
        case "numeric-unary-minus":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_unaryminus.htm";
          break;
        case "numeric-add":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_add.htm";
          break; 
        case "numeric-subtract":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_subtract.htm";
          break;
        case "numeric-multiply":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_multiply.htm";
          break;
        case "numeric-divide":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_divide.htm";
          break;
        case "max":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_maximum.htm";
          break;
        case "min":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_minimum.htm";
          break; 
        case "avg":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_average.htm";
          break;
        case "sum":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_sum.htm";
          break;
        case "count":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_count.htm";
          break;
        case "variance":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_variance.htm";
          break;
        case "current-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_currentdatetime.htm";
          break;
        case "format-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_formatdatetime.htm";
          break;
        case "day-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_dayfromdatetime.htm";
          break;
        case "month-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_monthfromdatetime.htm";
          break;
       	case "year-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_yearfromdatetime.htm";
          break;
        case "hours-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_hoursfromdatetime.htm";
          break;
        case "minutes-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_minutesfromdatetime.htm";
          break;
        case "seconds-from-datetime":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_secondsfromdatetime.htm";
          break;
        case "coalesce":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_coalesce.htm";
          break;
        case "urlencode":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_urlencode.htm";
          break;
        case "uuid":
              url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_uuid.htm";
          break; 
        default:
          url = FLOW.baseUrl + "/doc/userhelp/functionshelp/help_functions_overview.htm";
      }

    	this.operator.getCanvas().showHelp(url, title, this.chain);
    }
  }
);

}

if(!dojo._hasResource["flow.dijit.Variables"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.Variables"] = true;
dojo.provide("flow.dijit.Variables");










dojo.declare(
  "flow.dijit.Variables",
  [dijit._Widget, dijit._Templated, dijit._Container],
  {
    // attach points
    gridNode: null,
    grid: null,

    // impl
    model: null,
    canvas: null,
    chain: null,

    imgPath: dojo.moduleUrl("flow.dijit", "templates/images"),

    templateString:"<div>\r\n <table width=\"100%\"><tr><td width=\"100%\">${i18n.labels.VARIABLES_DESCRIPTION}</td><td><img dojoAttachEvent=\"onclick:showHelp\" src=\"${imgPath}/dialoghelp.png\" title=\"${i18n.labels.TOOLTIP_HELP}\"></td></tr></table>\r\n <div dojoAttachPoint=\"gridNode\"></div>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");
    },

    postCreate: function() {
      this.variables = [];

      // load variables
      var variables = this.model.runXPath("flow:variables/flow:param", this.model.flowNode);
      var data = [];

      for (var i = 0; i < variables.length; i++) {
        this.variables[variables[i].getAttribute("id")] = {id: variables[i].getAttribute("id"),
                                                           name: variables[i].getAttribute("caption"),
                                                           defaultvalue: this.model.getValue(variables[i], "text()"),
                                                           description: variables[i].getAttribute("comments")
                                                          };

        data.push(this.variables[variables[i].getAttribute("id")]);
      }

      var griddata = {layout: null, data: data, config: {allowGridEditing: true, canMoveRows: false, canAddRows: true}}
      griddata.layout = [
                         {name: "name", label: this.i18n.labels.VARIABLES_GRID_TITLE_NAME, tooltip: this.i18n.labels.VARIABLES_GRID_NAME_TOOLTIP, editable: true, width: "30%"},
                         {name: "defaultvalue", label: this.i18n.labels.VARIABLES_GRID_TITLE_DEFAULT_VALUE, tooltip: this.i18n.labels.VARIABLES_GRID_DEFAULT_VALUE_TOOLTIP, editable: true, width: "30%"},
                         {name: "description", label: this.i18n.labels.VARIABLES_GRID_TITLE_DESCRIPTION, tooltip: this.i18n.labels.VARIABLES_GRID_DESCRIPTION_TOOLTIP, editable: true, width: "40%"}
                        ];

      this.grid = new flow.dijit.Grid({grid: griddata, operator: this}, this.gridNode);
    },

    addChild: function() {
      var id = this.model.generateId("variable");

      this.variables[id] = {id: id, name: "", defaultvalue: "", description: ""};
      return this.variables[id];
    },

    setChildValue: function(id, key, value) {
      this.modifyChild(id, key, value);
    },

    modifyChild: function(id, key, value) {
      if (key == "name" || key == "defaultvalue" || key == "description") {
        this.variables[id][key] = value;
      }
    },

    removeChild: function(id) {
      delete this.variables[id];
    },

    updateModel: function() {
      // update/delete/create variables
      var variablesNode = this.model.getNode("flow:variables", this.model.flowNode)

      // make list of existing variables
      var variables = this.model.runXPath("flow:param", variablesNode);
      var variablelist = {};
      for (var i = 0; i < variables.length; i++) {
        variablelist[variables[i].getAttribute("id")] = true;
      }

      // update existing variables
      for (var id in this.variables) {
        if (!this.variables[id].name) {
          // no name, delete it
          delete variablelist[id];
          continue;
        }

        if (variablelist[id]) {
          // exists already
          var variable = this.model.getNode("flow:param[@id='"+id+"']", variablesNode);

          // remove from the list, already handled
          delete variablelist[id];
        } else {
          // create

          var template='<param xmlns="'+this.model.flowNS +
                      '" caption="" operatorspecific="" inputid="" id="'+id +
                      '" position="" type="dynamic" comments=""></param>';

//          var template = '<variable xmlns="'+this.model.flowNS+'" name="" default="" caption="" id="'+id+'"/>';

          var variable = this.model.parseXML(template);
          variable = variablesNode.appendChild(variable);
        }

        this.model.setValue("@caption", variable, this.variables[id].name);
        this.model.setValue("text()", variable, this.variables[id].defaultvalue);
        this.model.setValue("@comments", variable, this.variables[id].description);
      }

      // remove any variables still in the list
      for (var id in variablelist) {
        this.model.removeNode("flow:param[@id='"+id+"']", variablesNode);
      }
    },

    getData: function() {
      this.updateModel();	
      if (this.grid.selectedRow) {
        var data = {};
        data.type = "variable";
        var id = this.grid.selectedRow.getAttribute("rowid");
        var name = this.variables[id].name;
        data.value = id;
        return data;
      }
      return null;
    },

    showHelp: function() {
      this.canvas.showHelp(FLOW.baseUrl + "/doc/userhelp/editorhelp/help_builder_variables.htm", this.i18n.labels.VARIABLES_HELP_TITLE, this.chain);
    }
  }
);

}

if(!dojo._hasResource["flow.dijit.FlowComboBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.FlowComboBox"] = true;
dojo.provide("flow.dijit.FlowComboBox");







dojo.declare(
  "flow.dijit.FlowComboBox",
  dijit.form.ComboBox,
  {
    // don't autocomplete
    autoComplete: false,
    dialog: null,
    callback: null,
    options: null,

    // parent chain count
    chain: null,
    chainParent: null,
    operator: null,
    selectType: "text",
    data: null,
    previousData: null,

    // for editing the value
    editButton: null,
    clearButton: null,
    handles: [],

    // icon stuff
    currentValueType: null,

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");

      // build our data store for the comobox dropdown
      this.operator = this;
      while (this.operator.chainParent) {
        this.operator = this.operator.chainParent;
      }

      var items = [];

      for (var i = 0; i < this.options.length; i++) {
        var caption = eval("this.operator.i18n.DROPDOWNCOMBO_LABEL_"+this.options[i].toUpperCase());

        // for variables check if any exist
        /*
        //Since this is checked when the property view is created. If the user creates a variable after the
        //property window is opened the change will not be reflected in the grid
        //moved the check to before opening the variable dialog
        if (this.options[i] == "variable") {
          if (!this.operator.getVariableCount()) {
            continue;
          }
        }
		*/
        items.push({name: caption, label: caption, type: this.options[i]});
      }

      this.store = new dojo.data.ItemFileReadStore({data: {identifier: "name", items: items}});

      // lets load in the text box code
      dijit.form.ValidationTextBox.prototype.postMixInProperties.call(this);
    },

    postCreate: function() {
      this.inherited("postCreate", arguments);

      // set text value
      if (this.data) {
        var type = this.data.type;
        this.showValue(type, this.data);

        if (this.data.tooltip) {
          this.textbox.setAttribute("title", this.data.tooltip);
        }

        if (type && type != "text") {
          this.createButtons();

          if (type != "xpath") {
            this.textbox.setAttribute("disabled", "true");
          }
          //this.hasDownArrow=false;
        }
      }
    },

    _startSearchFromInput: function() {
      // do nothing
    },

    getChainParent: function() {
      var parent = this;

      while (parent.chainParent) {
        parent = parent.chainParent;
        if (parent.model) {
          break;
        }
      }

      return parent;
    },

    showValue: function(type, data) {
      var newvalue;

      var icon = "";
      this.iconNode.style.visibility = "hidden";

      if (this.currentIconClass) {
        dojo.removeClass(this.iconNode, "flowtypeCombobox"+this.currentIconClass)
      }

      if (type == "variable") {
        var chainParent = this.getChainParent();
        var parentcaption;

        if (chainParent) {
          parentcaption = chainParent.model.getValue(chainParent.model.xmlModel, ".//flow:param[@id='"+data.value+"']/../@caption");
        }

        if (parentcaption == null) {
          parentcaption = chainParent.i18n.VARIABLE_GLOBAL_CAPTION;
        }

        newvalue = chainParent.createVariableLabel(parentcaption, chainParent.model.getValue(chainParent.model.xmlModel, ".//flow:param[@id='"+data.value+"']/@caption"));
        icon = "GetFromVariable";
      } else if (type == "dynamic") {
        newvalue = data.value ? data.value : "";
        icon = "DefineAsVariable";
      } else if (type == "function") {
        newvalue = data.name + "()";
        icon = "Function";
      } else if (type == "xpath") {
        newvalue = data.value;
        icon = "Tree";
      } else if (type == "text") {
        newvalue = data.value ? data.value : "";
        icon = "Text";
      }

      if (icon) {
        this.currentIconClass = icon
        dojo.addClass(this.iconNode, "flowtypeCombobox")
        dojo.addClass(this.iconNode, "flowtypeCombobox"+icon)
        this.iconNode.style.visibility = "visible";
      } else {
        this.currentIconClass = "";
      }

      this.setValue(newvalue);
    },

    _showResultList: function() {
      this.inherited("_showResultList", arguments);

      // TODO: big hack
      var zindex = parseInt(dojo.style(this._popupWidget.domNode.parentNode, "zIndex"), 10)+this.chain;
      this._popupWidget.domNode.parentNode.style.zIndex = zindex;
    },

    _doSelect: function(tgt) {
      var type = this.store.getValue(tgt.item, "type");

      if (type == "text" && this.item == tgt.item) {
        // if text and already text, do nothing
        return;
      }

      this.item = tgt.item;

      var type = this.store.getValue(tgt.item, "type");

      this.showInputSelection(type, this.data);
    },

    showInputSelection: function(type, data) {
      this.selectType = type;

      if (type == "text" || type == "xpath") {
        this.textbox.removeAttribute("disabled");
      } else {
        this.textbox.setAttribute("disabled", "true");
      }

      if (type == "text") {
        this.textbox.focus();
        this.showValue(type, "");
      } else {
				var div = document.createElement("div");
        dojo.body().appendChild(div);

        if (this.dialog) {
          this.dialog.hide();
          this.dialog.destroyRecursive();
          this.dialog = null;
        }

        
        this.dialog = new flow.dijit.Dialog({chain: this.chain, showUnderlay: true,
                                             buttons:[{label: this.i18n.labels.BUTTON_OK, onclick: dojo.hitch(this, this._acceptDialog)},
                                                      {label: this.i18n.labels.BUTTON_CANCEL, onclick: dojo.hitch(this, this._cancelDialog)}]}, div);

        div = document.createElement("div");
        this.dialog.containerNode.appendChild(div);

        var width = 450, height = 350;

        if (type == "xpath") {
          
          this.selectWidget = new flow.dijit.XMLInputSelection({data: data, operator: this.operator, chain: this.chain}, div);
          this.dialog.setTitle(this.i18n.propertyeditor.GET_TREE_TITLE);
        } else if (type == "dynamic") {
          
          this.selectWidget = new flow.dijit.RuntimeParameterInputSelection({data: data, operator: this.operator}, div);
          width = 200;
          height = 200;
          this.dialog.setTitle(this.i18n.propertyeditor.DECLARE_VARIABLE_TITLE);
        } else if (type == "function") {
          
          this.selectWidget = new flow.dijit.FunctionInputSelection({data: data, chain: this.chain, chainParent: this.chainParent, operator: this.operator}, div);
          width = 450;
          height = 300;
          this.dialog.setTitle(this.i18n.propertyeditor.USE_FUNCTION_TITLE);
        } else if (type == "variable") {
          
          this.selectWidget = new flow.dijit.Variables({model: this.getChainParent().model, chain: this.chain, canvas: this.operator.getCanvas()}, div);
          width = 600; height = 250;
          this.dialog.setTitle(this.i18n.labels.VARIABLES_DIALOG_TITLE);

        }

        this.dialog.resize({w: width, h: height});
        this.dialog.show();
      }
    },

    _onBlur: function() {
      this.inherited("_onBlur", arguments);

      // if type is text
      if (this.selectType == "text" || this.selectType == "xpath") {
        var data = flow.operator.Data.create(this.selectType);
        data.value = this.getDisplayedValue();
        this.callback(data);
      }
    },

    _acceptDialog: function(event) {
    	try {
	      var data = this.selectWidget.getData();

	      if (data) {
	        this.showValue(data.type, data);

	        this.callback(data);

          if (!this.previousData) {
            this.previousData = this.data;
          }

	        this.data = data;
	      }

	      if (this.selectType != "text") {
	        this.createButtons();
	      } else {
	        this.clearButtons();
	      }

	      this._cancelDialog();
      } catch(ex) {
        //debugger keyword chokes Dojo custom build
        //debugger;
        alert(ex)
      }
    },

    getDataType: function() {
      var type = (this.data) ? this.data.type : null;
      return type;
    },

    editButtonClick: function() {
      // edit the value
      this.showInputSelection(this.getDataType(), this.data);
    },

    clearButtonClick: function() {
      // create empty data
      var data;
      if (this.previousData) {
        data = this.previousData;
      } else {
        data = flow.operator.Data.create("text");
      }

      this.showValue("text", data);
      this.callback(data);

      this.selectType = null;
      // make sure it is editable
      this.textbox.removeAttribute("disabled");
      this.clearButtons();
    },

    createButtons: function() {
      if (!this.editButton) {
        // show buttons only if they don't exist
        var button = document.createElement("button");
        button.appendChild(document.createTextNode(this.i18n.propertyeditor.DROPDOWNCOMBO_BUTTON_EDIT));
        this.editButton = this.domNode.parentNode.appendChild(button);
        dojo.connect(this.editButton, "onclick", this, "editButtonClick");
      }

      if (!this.clearButton) {
        var button = document.createElement("button");
        button.appendChild(document.createTextNode(this.i18n.propertyeditor.DROPDOWNCOMBO_BUTTON_CLEAR));
        this.clearButton = this.domNode.parentNode.appendChild(button);
        dojo.connect(this.clearButton, "onclick", this, "clearButtonClick");
      }
    },

    clearButtons: function() {
      dojo.forEach(this.handles, dojo.disconnect);

      if (this.editButton) {
        this.editButton.parentNode.removeChild(this.editButton);
        this.editButton = null;
      }

      if (this.clearButton) {
        this.clearButton.parentNode.removeChild(this.clearButton);
        this.clearButton = null;
      }
    },

    _cancelDialog: function(event) {
      try {
        this.selectWidget.destroy();
        this.selectWidget = null;
      } catch(ex){}

      this.dialog.hide();
      this.dialog.destroyRecursive();
      this.dialog = null;
    },

    onkeypress: function(event) {
      // TODO: hack to allow ( and " - remove for Dojo 1.2
      if (dojo.isIE && (event.shiftKey && (event.keyCode == 40 || event.keyCode == 34))) {
        // do nothing
      } else {
        return this.inherited("onkeypress", arguments);
      }
    }
  }
);

}

if(!dojo._hasResource["flow.dijit.Grid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.Grid"] = true;
dojo.provide("flow.dijit.Grid");









dojo.declare(
	"flow.dijit.Grid",
	[dijit._Widget, dijit._Templated, dijit._Container],
  {
    labelNode: null,
    theadNode: null,
    tbodyNode: null,
    footerNode: null,
    addButtonNode: null,
    upButtonNode: null,
    downButtonNode: null,
    removeButtonNode: null,
    contentNode: null,
    actionsNode: null,
    collapseNode: null,
    collapseNodeButton: null,

    grid: null,
    operatorId: null,
    operator: null,
               
    collapsable: false,
    collapseLabel: null,
    expandLabel: null,

    imagePath: dojo.moduleUrl("flow.dijit", "templates/images"),

    // parent chain
    chain: null,

    isEmpty: true,

    selectedRow: null,

    widgetsInTemplate: true,
    templateString:"<div style=\"margin-top: 10px;\">\r\n  <div style=\"display: none\" dojoAttachPoint=\"collapseNode\"><button dojoAttachPoint=\"collapseNodeButton\" dojoAttachEvent=\"onclick:toggleCollapse\"></button></div>\r\n  <div style=\"display: none\" dojoAttachPoint=\"contentNode\">\r\n    <div dojoAttachPoint=\"labelNode\"></div>\r\n    <table class=\"flowpropeditorgrid\" cellspacing=\"0\" cellpadding=\"5\" width=\"100%\">\r\n      <thead dojoAttachPoint=\"theadNode\"></thead>\r\n      <tbody dojoAttachPoint=\"tbodyNode\" dojoAttachEvent=\"onmouseup:selectRow\"></tbody>\r\n    </table>\r\n    <div dojoAttachPoint=\"footerNode\" style=\"clear: both;\">\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:addRowClick\" class=\"flowIconButton\" dojoAttachPoint=\"addButtonNode\" iconClass=\"flowAddIcon\" showLabel=\"false\">${i18n.propertyeditor.LABEL_ADD_ROW}</button>\r\n      <span dojoAttachPoint=\"actionsNode\">\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"upButtonNode\" class=\"flowIconButton\" disabled=\"true\" dojoAttachEvent=\"onClick:moveRowUpClick\" iconClass=\"flowMoveUpIcon\" showLabel=\"false\">${i18n.propertyeditor.LABEL_MOVE_ROW_UP}</button>\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"downButtonNode\" class=\"flowIconButton\" disabled=\"true\" dojoAttachEvent=\"onClick:moveRowDownClick\" iconClass=\"flowMoveDownIcon\" showLabel=\"false\">${i18n.propertyeditor.LABEL_MOVE_ROW_DOWN}</button>\r\n        <button dojoType=\"dijit.form.Button\" class=\"flowIconButton\" dojoAttachPoint=\"removeButtonNode\" disabled=\"true\" dojoAttachEvent=\"onClick:removeRowClick\" iconClass=\"flowDeleteIcon\" showLabel=\"false\">${i18n.propertyeditor.LABEL_REMOVE_ROW}</button>\r\n      </span>\r\n    </div>\r\n  </div>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    postCreate: function() {
      // handle collapsing
      if (this.collapsable) {
        // default to collapsed
        this.isCollapsed = true;
        this.collapseNodeButton.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(this.expandLabel);
        dojo.style(this.collapseNode, "display", "block");
      } else {
        dojo.style(this.contentNode, "display", "block");
      }

      // let's build the table - first the label
      if (this.grid.label) {
        this.labelNode.appendChild(document.createTextNode(this.grid.label));
      }

      // set button tooltips dynamically
      if (this.operator.type) {
        var type = this.operator.type.toUpperCase();

        if (this.grid.labelPrefix) {
          type = this.grid.labelPrefix;
        }

        // set add/remove row labels
        if (this.grid.config.canAddRows) {
          this.addButtonNode.setLabel(this.i18n.propertyeditor[type+"_ADD_ROW"]);
          this.removeButtonNode.setLabel(this.i18n.propertyeditor[type+"_REMOVE_ROW"]);
        }
      }

      // headers
      this.buildHeaders();

      // config the action buttons
      if (!this.grid.config.canAddRows) {
        dojo.addClass(this.addButtonNode.domNode, "dijitDisplayNone");
      }

      // config the action buttons
      if (!this.grid.config.canMoveRows) {
        this.upButtonNode.domNode.style.display = "none";
        this.downButtonNode.domNode.style.display = "none";
      }

      if (!this.grid.config.allowGridEditing) {
        this.actionsNode.style.display = "none";
      }

      // now add data rows
      if (this.grid.data) {
        if (this.grid.data.length > 0) {
          this.isEmpty = false;
        }

        for (var i = 0; i < this.grid.data.length; i++) {
          if (this.grid.data[i]) {
            var itemid = this.grid.data[i].id;
            var item = this.grid.data[i];

            this.addRow(itemid, item, i);
          }
        }
      }

      // if no rows, add some helpfull text
      if (this.isEmpty) {
        if (this.grid.config.canAddRows) {
          // if can add rows, add one for the user
          this.addNewRow();
        } else {
          this.addEmptyRow();
        }
      }
    },

    buildHeaders: function() {
      while (this.theadNode.hasChildNodes()) {
        this.theadNode.removeChild(this.theadNode.firstChild);
      }

      var tr = document.createElement("tr");

      for (var i = 0; i < this.grid.layout.length; i++) {
        var layout = this.grid.layout[i];

        if (layout.hidden) {
          continue;
        }

        var th = document.createElement("th");
        dojo.addClass(th, "flowgridcell");
        dojo.addClass(th, "flowgridheadercell");
        th.setAttribute("noWrap", "true");

        if (layout.width) {
          th.setAttribute("width", layout.width);
        }

        if (this.operator.type && this.operator.type.toUpperCase() == "FOREACH") {
          var src1id = this.operator.model.getValue(this.operator.node, "flow:input[@position='0']/@id");
          var src2id = this.operator.model.getValue(this.operator.node, "flow:input[@position='1']/@id");

          var src1cap = this.operator.model.getValue(this.operator.model.flowNode, "./flow:*[@id='"+src1id+"']/@caption");
          var src2cap = this.operator.model.getValue(this.operator.model.flowNode, "./flow:*[@id='"+src2id+"']/@caption");

          var i1 = layout.label.indexOf(src1cap) + layout.label.indexOf(src2cap)+ 1;
          var i2 = layout.label.indexOf(" ", i1);

          var font = document.createElement("font");
          font.style.color = "blue";

          th.appendChild(document.createTextNode(layout.label.substring(0,i1)));

          var forEachText;
          if (i2 == -1) {
            forEachText = document.createTextNode(layout.label.substring(i1));
            font.appendChild(forEachText);
            th.appendChild(font);
          } else {
            forEachText = document.createTextNode(layout.label.substring(i1,i2));
            font.appendChild(forEachText);
            th.appendChild(font);
            th.appendChild(document.createTextNode(layout.label.substring(i2)));
          }
        } else {
          th.appendChild(document.createTextNode(layout.label));
        }

        tr.appendChild(th);
      }

      this.theadNode.appendChild(tr);
    },

    addEmptyRow: function() {
      var tr = document.createElement("tr");
      var td = document.createElement("td");
      dojo.addClass(td, "flowgridcell");
      td.setAttribute("align", "center");
      td.appendChild(document.createTextNode(this.i18n.propertyeditor.GRID_NO_DATA_ENTERED));

      // only add link if one can add rows
      if (this.grid.config.canAddRows) {
        td.appendChild(document.createTextNode(" "));
        var a = document.createElement("a");
        a.setAttribute("href", "javascript:void()");
        var myThis = this;
        a.onclick = function() {
          myThis.addRowClick();
          return false;
        }
        a.appendChild(document.createTextNode(this.i18n.propertyeditor.GRID_CLICK_ADD_ROW));
        td.appendChild(a);
      }
      tr.appendChild(td);
      this.tbodyNode.appendChild(tr);

      td.setAttribute("colSpan", this.grid.layout.length);
    },

    addRow: function(itemid, item) {
      var tr = document.createElement("tr");

      tr.setAttribute("rowid", itemid);
      if (item.fixed) {
        tr.setAttribute("fixed", "true");
      }

      var grid = this.grid;
      var gridlayout = grid.layout;

      for (var j = 0; j < gridlayout.length; j++) {
        var info = gridlayout[j];
        var key = info.name;

        var td = document.createElement("td");
        td.setAttribute("valign", "top");
        dojo.addClass(td, "flowgridcell");

        var itemvalue;

        if (info.passAllData) {
          itemvalue = item;
        } else {
          itemvalue = item[key];
        }

        var itemdom;

        if (info.hidden) {
          continue;
        }

        var myThis = this;
        function generateCallback() {
          var storedkey = key;

          return function(value) {
            myThis.operator.setChildValue(itemid, storedkey, value);
          }
        }

        if (info.editable || (info.type && info.type == "textarea")) {
          if (info.editable) {
            itemdom = document.createElement("input");
            itemdom.setAttribute("value", itemvalue);
            itemdom.style.width = "90%";
            td.setAttribute("align", "center");
          } else {
            itemdom = document.createElement("textarea");
            itemdom.appendChild(document.createTextNode(itemvalue));
          }

          if (info.tooltip) {
            itemdom.setAttribute("title", info.tooltip);
          }

          // update the model
          itemdom.onblur = function() {
            var callback = generateCallback();
            var input = itemdom;
            return function() {
              callback(input.value);
            };
          }();
        } else if (info.values || itemvalue.values) {
          var values = info.values ? info.values : itemvalue.values;

          // we need a select
          itemvalue = typeof(itemvalue)=="object" ? itemvalue.value : itemvalue;
          itemdom = document.createElement("select");
          for (var k = 0; k < values.length; k++) {
            var label, value;

            if (typeof(values[k]) == "object") {
              label = values[k].label;
              value = values[k].value;
            } else {
              label = value = values[k];
            }

            var option = document.createElement("option");
            option.setAttribute("value", value);
            option.appendChild(document.createTextNode(label));

            // select the right option
            if (value == itemvalue) {
              option.setAttribute("selected", "true")
            }
            itemdom.appendChild(option);
          }

          // update the model
          itemdom.onchange = function() {
            var callback = generateCallback();
            var select = itemdom;
            return function() {
              callback({value: select.value, type: "text"});
            };
          }();
        } else if (info.type && info.type == "text") {
          itemdom = document.createElement("div");
          itemdom.appendChild(document.createTextNode(itemvalue));
        } else if (info.type && info.type == "tree") {
          // we need a tree dropdown
          itemdom = document.createElement("div");

          

          var treediv = document.createElement("div");
          itemdom.appendChild(treediv);

          var templ = myThis.operator.getTree(info.pos);

          if (templ == null) {
            var src1id = myThis.operator.model.getValue(myThis.operator.node, "flow:input[@position='"+info.pos+"']/@id");
            var src1cap = myThis.operator.model.getValue(myThis.operator.model.flowNode, ".//flow:*[@id='"+src1id+"']/@caption");
            alert(dojo.string.substitute(myThis.operator.i18n.LABEL_INPUT_NOT_VAlID, [src1cap]));
            return;
          }

          myThis.operator.model.setValue("flow:input[@position='"+info.pos+"']/@timestamp",myThis.operator.node,templ.timestamp);
          var treewidget = new flow.dijit.XMLViewTextBox({xmlRootNode: templ.root, operator: myThis.operator, inputid: templ.id, path: templ.path, data: itemvalue, callback: generateCallback()}, treediv);
        } else if (info.type && info.type == "param") {
          

          itemdom = document.createElement("div");
          var combodiv = document.createElement("select");
          itemdom.appendChild(combodiv);
          var widget = new flow.dijit.FlowComboBox({data: itemvalue, operator: myThis.operator, options: info.options, callback: generateCallback(), chainParent: this.operator, chain: this.chain+1}, combodiv);
        } else if (info.type && info.type == "custom") {
          itemdom = document.createElement("div");
          info.callback(itemdom, item);
        } else {
          itemdom = document.createTextNode(itemvalue);
        }

        td.appendChild(itemdom);
        tr.appendChild(td);
      }

      this.tbodyNode.appendChild(tr);

      if (this.grid.config.maxRows && this.tbodyNode.rows.length == this.grid.config.maxRows) {
        this.addButtonNode.setDisabled(true);
      }
    },

    selectRow: function(event) {
      if (this.isEmpty || !this.grid.config.allowGridEditing) {
        return;
      }

      var target = event.target;

      // make sure we only select tr's with a rowid
      while (target.parentNode && (target.tagName.toLowerCase() != "tr" || typeof(target.getAttribute("rowid")) != "string")) {
        target = target.parentNode;
      }

      // not in a row?
      if (target.tagName.toLowerCase() != "tr") {
        return;
      }

      if (this.selectedRow == target) {
        // already selected
        return;
      }

      if (this.selectedRow) {
        this.selectedRow.className = "";
      }

      target.className = "selectedrow";
      this.selectedRow = target;

      this.updateActions();
    },

    updateActions: function() {
      // update the actions based on the selection
      if (!this.grid.config.allowGridEditing) {
        return;
      }

      if (!this.selectedRow) {
        // nothing selected
        this.removeButtonNode.setDisabled(true);
        this.upButtonNode.setDisabled(true);
        this.downButtonNode.setDisabled(true);
      } else {
        var isFixed = false;
        if (this.selectedRow.getAttribute("fixed") == "true") {
          isFixed = true;
        }

        // we have something selected, so we can remove it
        if (!isFixed) {
          this.removeButtonNode.setDisabled(false);
        } else {
          this.removeButtonNode.setDisabled(true);
        }

        var length = this.tbodyNode.rows.length;

        // handle up button
        if (this.tbodyNode.rows[0] == this.selectedRow) {
          // if the currently selected row is the first row, disable
          this.upButtonNode.setDisabled(true);
        } else if (length == 1 || isFixed) {
          // only one row, can't move up
          this.upButtonNode.setDisabled(true);
        } else {
          this.upButtonNode.setDisabled(false);
        }

        // handle down button
        if (this.tbodyNode.rows[this.tbodyNode.rows.length-1] == this.selectedRow) {
          // if the currently selected row is the last row, disable
          this.downButtonNode.setDisabled(true);
        } else if (length == 1 || isFixed) {
          // only one row, can't move down
          this.downButtonNode.setDisabled(true);
        } else {
          this.downButtonNode.setDisabled(false);
        }
      }
    },

    addRowClick: function(event) {
      this.addNewRow();
    },

    addNewRow: function() {
      if (this.isEmpty) {
        this.isEmpty = false;

        if (this.tbodyNode.firstChild) {
          this.tbodyNode.removeChild(this.tbodyNode.firstChild);
        }
      }

      // tell model we added a row
      var newitem = this.operator.addChild(this.grid.id);
      this.grid.data.push(newitem);
      this.addRow(newitem.id, newitem);

      // finally, update the actions
      this.updateActions();
    },

    moveRowUpClick: function(event) {
      // probably useless check to make sure we are not the first row
      if (this.tbodyNode.rows[0] == this.selectedRow) {
        return;
      }

      // tell model we are changing the position!
      var pos = this.getPositionForRow(this.selectedRow);

      // lets swap out the rows - move them in the DOM
      var target = this.selectedRow.previousSibling;
      var source = this.tbodyNode.removeChild(this.selectedRow);

      var sourceid = source.getAttribute("rowid");
      var targetid = target.getAttribute("rowid");
      this.operator.modifyChild(sourceid, "position", pos-1);
      this.operator.modifyChild(targetid, "position", pos);

     this.selectedRow = this.tbodyNode.insertBefore(source, target);

      // finally, update the actions
      this.updateActions();
    },

    moveRowDownClick: function(event) {
      // probably useless check to make sure we are not the last row
      if (this.tbodyNode.rows[this.tbodyNode.rows.length-1] == this.selectedRow) {
        return;
      }

      // tell model we are changing the position!
      var pos = this.getPositionForRow(this.selectedRow);

      // lets swap out the rows - move them in the DOM
      var target = this.selectedRow.nextSibling;
      var source = this.tbodyNode.removeChild(this.selectedRow);

      var sourceid = source.getAttribute("rowid");
      var targetid = target.getAttribute("rowid");
      this.operator.modifyChild(sourceid, "position", pos+1);
      this.operator.modifyChild(targetid, "position", pos);

      if (target.nextSibling) {
        this.selectedRow = this.tbodyNode.insertBefore(source, target.nextSibling);
      } else {
        this.selectedRow = this.tbodyNode.appendChild(source);
      }

      // finally, update the actions
      this.updateActions();
    },

    clearGrid: function(onlygrid) {
      for (var i = this.tbodyNode.rows.length-1; i >= 0; i--) {
        this.tbodyNode.removeChild(this.tbodyNode.rows[i]);
      }
    },

    removeRowClick: function(event) {
      var id = this.selectedRow.getAttribute("rowid");
      var pos = this.getPositionForRow(this.selectedRow);

      this.operator.removeChild(id);

      // update all next sibling's position
      var elm = this.selectedRow;
      var count = 0;
      while (elm.nextSibling) {
        elm = elm.nextSibling;
        this.operator.modifyChild(elm.getAttribute("rowid"), "position", pos+count);
        count++;
      }

      this.tbodyNode.removeChild(this.selectedRow);
      this.selectedRow = null;

      // finally, update the actions
      this.updateActions();

      // if no rows, add some helpfull text
      if (this.tbodyNode.childNodes.length == 0) {
        this.isEmpty = true;
        this.addEmptyRow();
      }

      if (this.grid.config.maxRows && this.tbodyNode.rows.length < this.grid.config.maxRows) {
        this.addButtonNode.setDisabled(false);
      }
    },

    getPositionForRow: function(row) {
      // tell model we are changing the position!
      var pos = 0;
      for (var i = 0; i < this.tbodyNode.rows.length; i++) {
        if (this.tbodyNode.rows[i] != row) {
          pos++;
        } else {
          break;
        }
      }

      return pos;
    },

    toggleCollapse: function() {
      if (this.isCollapsed) {
        this.collapseNodeButton.innerHTML = this.collapseLabel;
        dojo.style(this.contentNode, "display", "block");
      } else {
        this.collapseNodeButton.innerHTML = this.expandLabel;
        dojo.style(this.contentNode, "display", "none");
      }

      this.isCollapsed = !this.isCollapsed;
    }
  }
);

}

if(!dojo._hasResource['hub.misc.popup']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.misc.popup'] = true;
dojo.provide('hub.misc.popup');
//
// JavaScript popup layer support
// Version 1.11
//
// Author: Scott Morgan (morgans@us.ibm.com)
//
// Change history:
//    13 Oct 2006 - 1.00 Initial release.
//    25 Oct 2006 - 1.01 Add SurfAid support.
//    07 Nov 2006 - 1.02 Change visuals to support latest UI design.
//    11 Nov 2006 - 1.03 Modify zIndex on show and hide.
//    29 Nov 2006 - 1.04 Add support for lining up the layer to the left/right of the target.
//    17 Jan 2007 - 1.05 Add support for layers to be displayed in "bubbles" with new positioning styles.
//    22 Jan 2007 - 1.06 Put popup styles into a flag and add showPopupLayer2 function that uses them.
//    22 Jan 2007 - 1.07 Modal dialog layers now reposition like other layers on window resize, and
//                       moveable layers now reposition on window resize until the user has manually moved them.
//                       Previously neither repostioned on window resize.
//    29 Jan 2007 - 1.08 Allow the bubble width or height dimension to be smaller than the fin base if the fin
//                       is not to be displayed on that side.
//    08 Feb 2007 - 1.09 Allow forms displayed in bubble to retain field values on subsequent appearances.
//    09 Feb 2007 - 1.10 Add a flag that indicates the layer is a form, and stop using the modal dialog flag
//                       to indicate this as well, as not all forms need be modal.
//    18 Feb 2007 - 1.11 Prepare file for sharing with other development organizations.
//

//
// Global constants
//

//
// Path of the common image repository where the images will be found.
// The popup_layer.css file also references an image whose path will need to be changed to correspond
// to the path defined here.
//
pl_IMAGE_PATH = "images/popup/";

set_pl_IMAGE_PATH = function (path)
{
	pl_IMAGE_PATH = path;
	pl_BUBBLE_FIN_FULLPATH_NL = pl_IMAGE_PATH + "fin_nl.gif";
	pl_BUBBLE_FIN_FULLPATH_NR = pl_IMAGE_PATH + "fin_nr.gif";
	pl_BUBBLE_FIN_FULLPATH_SL = pl_IMAGE_PATH + "fin_sl.gif";
	pl_BUBBLE_FIN_FULLPATH_SR = pl_IMAGE_PATH + "fin_sr.gif";
	pl_BUBBLE_FIN_FULLPATH_EU = pl_IMAGE_PATH + "fin_eu.gif";
	pl_BUBBLE_FIN_FULLPATH_ED = pl_IMAGE_PATH + "fin_ed.gif";
	pl_BUBBLE_FIN_FULLPATH_WU = pl_IMAGE_PATH + "fin_wu.gif";
	pl_BUBBLE_FIN_FULLPATH_WD = pl_IMAGE_PATH + "fin_wd.gif";
	pl_BUBBLE_CORNER_FULLPATH_NW = pl_IMAGE_PATH + "corner_nw.gif";
	pl_BUBBLE_CORNER_FULLPATH_NE = pl_IMAGE_PATH + "corner_ne.gif";
	pl_BUBBLE_CORNER_FULLPATH_SW = pl_IMAGE_PATH + "corner_sw.gif";
	pl_BUBBLE_CORNER_FULLPATH_SE = pl_IMAGE_PATH + "corner_se.gif";
	pl_BUBBLE_SIDE_FULLPATH_N = pl_IMAGE_PATH + "side_n.gif";
	pl_BUBBLE_SIDE_FULLPATH_S = pl_IMAGE_PATH + "side_s.gif";
	pl_BUBBLE_SIDE_FULLPATH_E = pl_IMAGE_PATH + "side_e.gif";
	pl_BUBBLE_SIDE_FULLPATH_W = pl_IMAGE_PATH + "side_w.gif";
	// Images for the banner - may also be referenced in popup_layers.css.
	pl_POPUPBANNER_FULLPATH = pl_IMAGE_PATH + "popupbanner.gif";
	pl_POPUPBANNERCLOSE_FULLPATH = pl_IMAGE_PATH + "popupclosex.gif";
	pl_CLEARPIX_FULLPATH = pl_IMAGE_PATH + "c.gif";
	pl_BUBBLE_CONTENT_MINWIDTH_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='1' alt='' />";
	pl_BUBBLE_CONTENT_MINHEIGHT_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='1' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	// HTML that is swapped to replace the fine being used with one pointing in a different direction.
	pl_BUBBLE_FIN_NL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_NR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_SL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_SR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
	pl_BUBBLE_FIN_EU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_EU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	pl_BUBBLE_FIN_ED = "<img src='" + pl_BUBBLE_FIN_FULLPATH_ED + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	pl_BUBBLE_FIN_WU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
	pl_BUBBLE_FIN_WD = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WD + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
}

//
// Externalized variables and constants
//
BANNER_ID_SUFFIX = "popupbanner";  // Banner id is the layer id + this suffix
login_iframe_active = false;


//
// Image constants
//
// Images for the bubble
pl_BUBBLE_FIN_FULLPATH_NL = pl_IMAGE_PATH + "fin_nl.gif";
pl_BUBBLE_FIN_FULLPATH_NR = pl_IMAGE_PATH + "fin_nr.gif";
pl_BUBBLE_FIN_FULLPATH_SL = pl_IMAGE_PATH + "fin_sl.gif";
pl_BUBBLE_FIN_FULLPATH_SR = pl_IMAGE_PATH + "fin_sr.gif";
pl_BUBBLE_FIN_FULLPATH_EU = pl_IMAGE_PATH + "fin_eu.gif";
pl_BUBBLE_FIN_FULLPATH_ED = pl_IMAGE_PATH + "fin_ed.gif";
pl_BUBBLE_FIN_FULLPATH_WU = pl_IMAGE_PATH + "fin_wu.gif";
pl_BUBBLE_FIN_FULLPATH_WD = pl_IMAGE_PATH + "fin_wd.gif";
pl_BUBBLE_CORNER_FULLPATH_NW = pl_IMAGE_PATH + "corner_nw.gif";
pl_BUBBLE_CORNER_FULLPATH_NE = pl_IMAGE_PATH + "corner_ne.gif";
pl_BUBBLE_CORNER_FULLPATH_SW = pl_IMAGE_PATH + "corner_sw.gif";
pl_BUBBLE_CORNER_FULLPATH_SE = pl_IMAGE_PATH + "corner_se.gif";
pl_BUBBLE_SIDE_FULLPATH_N = pl_IMAGE_PATH + "side_n.gif";
pl_BUBBLE_SIDE_FULLPATH_S = pl_IMAGE_PATH + "side_s.gif";
pl_BUBBLE_SIDE_FULLPATH_E = pl_IMAGE_PATH + "side_e.gif";
pl_BUBBLE_SIDE_FULLPATH_W = pl_IMAGE_PATH + "side_w.gif";
// Images for the banner - may also be referenced in popup_layers.css.
pl_POPUPBANNER_FULLPATH = pl_IMAGE_PATH + "popupbanner.gif";
pl_POPUPBANNERCLOSE_FULLPATH = pl_IMAGE_PATH + "popupclosex.gif";
pl_POPUPBANNERCLOSE_WIDTH = "6";
pl_POPUPBANNERCLOSE_HEIGHT = "7";
// 1 pixel transparent image
pl_CLEARPIX_FULLPATH = pl_IMAGE_PATH + "c.gif";
// Bubble image dimensions
pl_BUBBLE_FIN_REACH = 42;//33;  // From the tip of the fin to the base.
pl_BUBBLE_FIN_BASE = 61;//56;   // Length of the base of the fin.
pl_BUBBLE_FIN_INSET = 12;//11;  // Amount the fin image overlays the bubble.
pl_BUBBLE_FIN_INSET_Y = 15; //@GINO: necessary for new GIF images
pl_BUBBLE_CORNER_WIDTH = 15;
pl_BUBBLE_CORNER_HEIGHT = 15;
pl_BUBBLE_SIDE_DEPTH = 15;
pl_BUBBLE_SIDE_LENGTH = 5;
// Banner image dimensions
pl_BANNER_BG_WIDTH = 84;
pl_BANNER_BG_HEIGHT = 28;
pl_BANNER_CLOSE_X_WIDTH = 6;
pl_BANNER_CLOSE_X_HEIGHT = 7;

// Bubble-specific table cell IDs
pl_BUBBLE_CONTENT_MINHEIGHT_ID = "popupLayerBubbleHeight";
pl_BUBBLE_CONTENT_MINWIDTH_ID = "popupLayerBubbleWidth";
// HTML that is swapped in and out of the table cell IDs that regulate a minimum dimension to ensure the fin fits on that side.
pl_BUBBLE_CONTENT_MINWIDTH_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='1' alt='' />";
pl_BUBBLE_CONTENT_MINHEIGHT_HTML = "<img src='" + pl_CLEARPIX_FULLPATH + "' border='0' width='1' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
// HTML that is swapped to replace the fine being used with one pointing in a different direction.
pl_BUBBLE_FIN_NL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_NR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_NR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_SL = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SL + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_SR = "<img src='" + pl_BUBBLE_FIN_FULLPATH_SR + "' border='0' width='" + pl_BUBBLE_FIN_BASE + "' height='" + pl_BUBBLE_FIN_REACH + "' alt='' />";
pl_BUBBLE_FIN_EU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_EU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
pl_BUBBLE_FIN_ED = "<img src='" + pl_BUBBLE_FIN_FULLPATH_ED + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
pl_BUBBLE_FIN_WU = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WU + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";
pl_BUBBLE_FIN_WD = "<img src='" + pl_BUBBLE_FIN_FULLPATH_WD + "' border='0' width='" + pl_BUBBLE_FIN_REACH + "' height='" + pl_BUBBLE_FIN_BASE + "' alt='' />";

pl_LOGIN_IFRAME_ID_SUFFIX = "w3signonIFrame";  // IFrame id is the layer id + this suffix
pl_DRAG_START_DIFFERENTIAL = 0;  // Pixels the mouse must be moved more than while help down to start the drag operation
pl_MOUSEOVER_SHOW_DELAY = 250;  // Milliseconds
pl_MOUSEOUT_HIDE_DELAY = 250;  // Milliseconds
pl_DRAG_VISIBILITY_BUFFER = 20;  // Pixels of drag object to always appears in view when moved top and left

pl_BUBBLE_ID = "popupLayerBubble";  // ID of the bubble
pl_BUBBLE_CONTENT_ID = "popupLayerBubbleContent";  // ID of the bubble cell that contains the layer content
pl_BUBBLE_FIN_ID = "popupLayerBubbleFin";  // ID of the object that contains the bubble fin
pl_BUBBLECLONE_ID_ADDON = "_bubbleclone";  // ID of the layer within a bubble id layer id + this suffix
pl_CONTENT_MAIN_ID = "content-main";  // ID of section of web page where cloned layer within bubble is placed

// Sizing and positioning constants
pl_IDEAL_NORTH_SOUTH_POSITION = .4;
pl_IDEAL_EAST_WEST_FIN_POSITION = .25;
pl_BIG_OFFSCREEN = 10000;
pl_FINPOS_INDENT_DEFAULT = 8;

//
// External flag bits for the showPopupLayer call
//
PL_ADD_CLOSE_LINK =       0x0001;
PL_ADD_BANNER =           0x0002;
PL_DIALOG_BEHAVIOR =      0x0004;
PL_LOGIN_REQUIRED =       0x0008;
PL_MOUSE_OVER_SUPPORT =   0x0010;
PL_DISPLAY_IN_BUBBLE =    0x0020;
PL_SUGGESTED_POS_BELOW =  0x0040;
PL_SUGGESTED_POS_ABOVE =  0x0080;
PL_SUGGESTED_POS_LEFT =   0x0100;
PL_SUGGESTED_POS_RIGHT =  0x0200;
PL_FORCE_VISIBLE_TARGET = 0x0400;
PL_FORCE_POSITION =       0x0800;
PL_FORM =                 0x1000;


//
// Global variables
//
pl_active_layer_id = "";
pl_active_layer_obj = null;
pl_active_clonedlayer_obj = null;
pl_active_contentclone_obj = null;
pl_active_trigger_id = "";
pl_active_trigger_obj = null;
pl_active_target_obj = null;
pl_active_drag_obj = null;
pl_active_mybubblefin = null;
pl_active_layer_html = "";
pl_active_IFrame_id = "";
pl_active_surfaid_base_string = "";
pl_active_style_flags = 0x0;
pl_active_has_dragged = false;
pl_active_bubble_minwidth_cell = null;
pl_active_bubble_minheight_cell = null;
pl_clicked_layer = false;
pl_onclick_doc_prevEH =  "";
pl_onkeypress_doc_prevEH =  "";
pl_onresize_win_prevEH =  "";
pl_onunload_win_prevEH = "";
pl_onmousemove_body_prevEH = "";
pl_onmouseup_body_prevEH = "";
pl_onclick_trigger_prevEH =  "";
pl_onmouseover_trigger_prevEH = "";
pl_onfocus_trigger_prevEH = "";
pl_active_drag_obj_prevcursor = "";
pl_queued_show_layer_request = false;
pl_queued_hide_layer_request = false;
pl_queued_show_layer_timer_id = -1;
pl_queued_hide_layer_timer_id = -1;
pl_move_in_process = false;
pl_drag_mouse_down = false;
pl_move_init_x = 0;
pl_move_init_y = 0;
pl_move_min_x = 0;
pl_move_min_y = 0;
pl_move_offset_x = 0;
pl_move_offset_y = 0;
pl_bubble_images_preloaded = false;
pl_bubble_orig_height = 0;
pl_bubble_orig_width = 0;

//
// Preloaded images
//
pl_img_clearpix = null;
pl_img_banner_bg = null;
pl_img_close_x = null;
pl_img_bubble_fin_nl = null;
pl_img_bubble_fin_nr = null;
pl_img_bubble_fin_sl = null;
pl_img_bubble_fin_sr = null;
pl_img_bubble_fin_eu = null;
pl_img_bubble_fin_ed = null;
pl_img_bubble_fin_wu = null;
pl_img_bubble_fin_wd = null;
pl_img_bubble_corner_nw = null;
pl_img_bubble_corner_ne = null;
pl_img_bubble_corner_sw = null;
pl_img_bubble_corner_se = null;
pl_img_bubble_side_w = null;
pl_img_bubble_side_e = null;
pl_img_bubble_side_n = null;
pl_img_bubble_side_s = null;




//
// Call to preload the images when this script file loads, so they will already be loaded the first time requested.
//
//pl_preloadPopupLayerImages();



// -----------------------------------------------------------------------------
//
// ENTRY POINT functions called externally (no pl_ prefix)
//
// showPopupLayer2 - position and show the popup layer.
// hidePopupLayer - hide the popup layer if one is showing.
// getPopupLayerElementById - return the object in the popup layer with the specified ID.
//
// showPopupLayer - DEPRECATED, included for backward compatibility only.
//
// -----------------------------------------------------------------------------

//
// ENTRY POINT from web pages: position and show the popup layer.
// Calling convention:
//     href="javascript:showPopupLayer2(....)" or href="#" onclick="showPopupLayer(....)" for anchor tag click support
//     onclick="showPopupLayer2(....)" for spans, divs, etc. click support
//     onmouseover=showPopupLayer2(....) if mouse over and mouse out support desired
//     onfocus=showPopupLayer2(....) for keyboard support when mouse over and mouse out support desired
// Required parms:
// Parm 1 (id of the popup layer)
// Parm 2 (id of the target object to which it should align itself)
// Parm 3 (id of the trigger object which when clicked on caused this call)
// Optional parms:
// Parm 4 (style flags, bitwise OR'ed or added together, default = no styles set)
// Parm 5 (id of the object that will be used for dragging the layer, default = "")
// Parm 6 (SurfAid base string used for logging that layer was dismissed, default = "")
//
// Style flags passed in parm 4:
// - PL_ADD_CLOSE_LINK = add close link at lower left corner of popup.
// - PL_ADD_BANNER = add the onePlace banner with close icon to top of popup.
// - PL_FORM = the layer contains a form that may be submitted by the user.
// - PL_DIALOG_BEHAVIOR = popup layer acts like a modal dialog.
// - PL_LOGIN_REQUIRED = secure login is required.
// - PL_MOUSE_OVER_SUPPORT = mouse over and out support is desired to show and hide this object.
// - PL_DISPLAY_IN_BUBBLE = display the popup as a bubble enclosing the user-defined popup layer.
// - PL_SUGGESTED_POS_BELOW = Default if no position suggested or multiple positions suggested: position the popup below the target if it fits, else try above, left, then right.
// - PL_SUGGESTED_POS_ABOVE = position the popup above the target if it fits, else try below, left, then right.
// - PL_SUGGESTED_POS_LEFT = position the popup to the left of the target if it fits, else try right, below, then above.
// - PL_SUGGESTED_POS_RIGHT = position the popup to the right of the target if it fits, else try left, below, then above.
// - PL_FORCE_VISIBLE_TARGET = Always kep the target visible, even if this results in the popup layer not being fully visible.
// - PL_FORCE_POSITION = always use the suggested position and keep the target visible, even if this results in the popup layer not being fully visible.
//
showPopupLayer2 = function (idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string) {
   // If acts like a dialog, clicking or mouse over outside the layer does not get processed,
   // and we do not display any other layers while this one is up, simulating the layer being modal.
   if (pl_active_style_flags & PL_DIALOG_BEHAVIOR) {
      return;
   }

   // First we see if the optional parms are passed in - if not, we set any not passed in to their default values.
   // If the three required parms are not passed in, nothing we can do here so just return.
   var numArgs = showPopupLayer2.arguments.length;
   if (numArgs < 6) {
      surfaid_base_string = "";
      if (numArgs < 5) {
         idDragger = "";
         if (numArgs < 4) {
            style_flags = 0x0;
            if (numArgs < 3) {
               return;
            }
         }
      }
   }
   // Now support passing in null for any optional parm to take its default value.
   if (style_flags == null) {
      style_flags = 0x0;
   }
   if (idDragger == null) {
      idDragger = "";
   }
   if (surfaid_base_string == null) {
      surfaid_base_string = "";
   }

   // If no suggested position was passed in, or more than one suggested position was passed in, set to default position of below pos.
   var sug_pos = style_flags & (PL_SUGGESTED_POS_BELOW | PL_SUGGESTED_POS_ABOVE | PL_SUGGESTED_POS_LEFT | PL_SUGGESTED_POS_RIGHT);
   if (sug_pos == 0x0) {
      style_flags |= PL_SUGGESTED_POS_BELOW;
   } else if ((sug_pos != PL_SUGGESTED_POS_BELOW) && (sug_pos != PL_SUGGESTED_POS_ABOVE) && (sug_pos != PL_SUGGESTED_POS_LEFT) && (sug_pos != PL_SUGGESTED_POS_RIGHT)) {
      if (style_flags & PL_SUGGESTED_POS_ABOVE) {
         style_flags = ~((~style_flags) ^ PL_SUGGESTED_POS_ABOVE);
      }
      if (style_flags & PL_SUGGESTED_POS_LEFT) {
         style_flags = ~((~style_flags) ^ PL_SUGGESTED_POS_LEFT);
      }
      if (style_flags & PL_SUGGESTED_POS_RIGHT) {
         style_flags = ~((~style_flags) ^ PL_SUGGESTED_POS_RIGHT);
      }
      if (!(style_flags & PL_SUGGESTED_POS_BELOW)) {
         style_flags |= PL_SUGGESTED_POS_BELOW;
      }
   }

   var trigger = document.getElementById(idTrigger);
   var layer = document.getElementById(idLayer);
   var target = document.getElementById(idTarget);
   // If any of the three objects is undefined, do not attempt to process.
   if ((layer == null) || (target == null) || (trigger == null)) {
      return;
   }

   var mybubble = null;
   var mybubblefin = null;

   // Set the mouse out and blur event handlers since they are cleared when called if mouse support requested.
   if (style_flags & PL_MOUSE_OVER_SUPPORT) {
      trigger.onmouseout = pl_processTriggerMouseOutAndBlur;
      trigger.onblur = pl_processTriggerMouseOutAndBlur;
   }

   // If this is a mouse over action, and we do not have a previous request in the queue to show a layer.
   if ((style_flags & PL_MOUSE_OVER_SUPPORT) && (!pl_queued_show_layer_request)) {
      // If there is a queued hide request, kill it as any layer will be hidden when this queued show request runs.
      if (pl_queued_hide_layer_request) {
         pl_queued_hide_layer_request = false;
         pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 1");
      }
      pl_queued_show_layer_request = true;
      // Set a timer to show the layer.
      pl_queued_show_layer_timer_id = window.setTimeout("pl_showDelayedPopupLayer('" + idLayer + "', '" + idTarget + "', '" + idTrigger + "', " + style_flags + ", '" + idDragger + "', '" + surfaid_base_string + "')", pl_MOUSEOVER_SHOW_DELAY);
      // Exit and wait for the timer.
      return;
   }

   // Hide current layer if one is already showing.
   if ((pl_active_layer_obj != null)) {
      hidePopupLayer();
   }

   // Clear any possible queued requests
   if (pl_queued_show_layer_request) {
      pl_queued_show_layer_request = false;
      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 2");
   }
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 3");
   }

   // If we are displaying the layer in a bubble, create the bubble now.
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {

      mybubblefin = document.getElementById(pl_BUBBLE_FIN_ID);
      if (mybubblefin == null) {
         // Create the new bubble fin
         mybubblefin = document.createElement("div");
         mybubblefin.id = pl_BUBBLE_FIN_ID;
         mybubblefin.style.position = "absolute";
         mybubblefin.style.top = "10px";
         mybubblefin.style.left = "10px";
         mybubblefin.style.zIndex = -1;
         mybubblefin.style.visibility = "hidden";
         mybubblefin.innerHTML = "";
         // Add it to the document
         document.body.appendChild(mybubblefin);
      }
      pl_active_mybubblefin = mybubblefin;

      // Get the bubble object, creating it the first time needed.
      mybubble = document.getElementById(pl_BUBBLE_ID);
      if (mybubble == null) {
         // Create the table layout for the bubble.
         var bubble_table = "";
         bubble_table += "<table class='w3pluto1' cellpadding='0' cellspacing='0' border='0' summary=''>";
         bubble_table += "<tr>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_NW + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "<td id='" + pl_BUBBLE_CONTENT_MINWIDTH_ID + "' style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_N + "); background-repeat:repeat-x; background-position:top left; font-size: 1px;'></td>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_NE + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "</tr>";
         bubble_table += "<tr>";
         bubble_table += "<td id='" + pl_BUBBLE_CONTENT_MINHEIGHT_ID + "' style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_W + "); background-repeat:repeat-y; background-position:top left; font-size: 1px;'></td>";
         bubble_table += "<td class='pl_bubble_content_area' id='" + pl_BUBBLE_CONTENT_ID + "'></td>";
         bubble_table += "<td style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_E + "); background-repeat:repeat-y; background-position:top right; font-size: 1px;'></td>";
         bubble_table += "</tr>";
         bubble_table += "<tr>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_SW + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "<td style='background-image:url(" + pl_BUBBLE_SIDE_FULLPATH_S + "); background-repeat:repeat-x; background-position:bottom left; font-size: 1px;'></td>";
         bubble_table += "<td style='font-size: 1px'><img src='" + pl_BUBBLE_CORNER_FULLPATH_SE + "' border='0' width='" + pl_BUBBLE_CORNER_WIDTH + "' height='" + pl_BUBBLE_CORNER_HEIGHT + "' alt='' /></td>";
         bubble_table += "</tr>";
         bubble_table += "</table>";

         // Create the new bubble object and assign the bubble table to it.
         mybubble = document.createElement("div");
         mybubble.id = pl_BUBBLE_ID;
         mybubble.style.position = "absolute";
         mybubble.style.top = "10px";
         mybubble.style.left = "10px";
         mybubble.style.zIndex = -1;
         mybubble.style.visibility = "hidden";
         mybubble.innerHTML = bubble_table;
         // Make it a child of the w3 content area if one exists so it picks up styles like the rest of the content.
         var content_main = document.getElementById(pl_CONTENT_MAIN_ID);
         if (content_main != null) {
            content_main.appendChild(mybubble);
         } else {
            document.body.appendChild(mybubble);
         }
      }
      // Store the bubble cells that dictate minumum width and height depending in which side the fin will appear.
      if (pl_active_bubble_minwidth_cell == null) {
         pl_active_bubble_minwidth_cell = document.getElementById(pl_BUBBLE_CONTENT_MINWIDTH_ID);
      }
      if (pl_active_bubble_minheight_cell == null) {
         pl_active_bubble_minheight_cell = document.getElementById(pl_BUBBLE_CONTENT_MINHEIGHT_ID);
      }
      // Start with no min width or height until the positioning calc is run.
      pl_active_bubble_minwidth_cell.innerHTML = "";
      pl_active_bubble_minheight_cell.innerHTML = "";
      // Reset global vars that indicate the original layer width and height have not yet been saved for the positioning calc.
      pl_bubble_orig_width = 0;
      pl_bubble_orig_height = 0;

      // Put the layout table in the bubble.
      var mybubblecontent = document.getElementById(pl_BUBBLE_CONTENT_ID);
      // Clear out any html from a previous bubble.
      mybubblecontent.innerHTML = "";
      // Clone the display layer so we get the entire DIV and all its styles, etc.
      var mycontentclone = layer.cloneNode(true);
      mycontentclone.style.position = "static";
      mycontentclone.style.visibility = "inherit";
      // Give the clone layer a unique ID so there is no confusion on the layer level caused by duplicate IDs.
      mycontentclone.id = mycontentclone.id + pl_BUBBLECLONE_ID_ADDON;
      // Add the cloned display layer into the bubble.
      mybubblecontent.appendChild(mycontentclone);

      // Now set the layer variable to point to the cloned layer for the following code, as we no longer need
      // to reference the original layer.
      pl_active_clonedlayer_obj = layer;
      pl_active_contentclone_obj = mycontentclone;
      layer = mycontentclone;
   }

   // If we are displaying the layer in a bubble, it is the bubble that will be the active layer.
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      pl_active_layer_id = pl_BUBBLE_ID;
      pl_active_layer_obj = mybubble;
   } else {
      pl_active_layer_id = idLayer;
      pl_active_layer_obj = layer;
   }
   // Set globals for this showing of the layer
   pl_active_trigger_id = idTrigger;
   pl_active_trigger_obj = trigger;
   pl_active_target_obj = target;
   pl_active_style_flags = style_flags;
   pl_active_surfaid_base_string = surfaid_base_string;

   // Store the original html for the popup layer
   pl_active_layer_html = layer.innerHTML;
   // Add the Close link unless the optional parm said to not show it, and as long as it has not already been added.
   if (style_flags & PL_ADD_CLOSE_LINK) {
      if (layer.innerHTML.indexOf("pl_bottom_close_link") < 0) {
         layer.innerHTML = layer.innerHTML + "<table width='100%' cellpadding='0' cellspacing='0' border='0' summary=''><tr><td align='right' class='pl_bottom_close_link'><a class='pl_bottom_close_link_anchor' href='javascript:hidePopupLayer();'>Close</a></td></tr></table>"
      }
   }

   // Add the w3 banner to the beginning if optional parm said to do so.
   var banner = null;
   if (style_flags & PL_ADD_BANNER) {
      var banner_id = idLayer + BANNER_ID_SUFFIX;
     if (layer.innerHTML.indexOf(banner_id) < 0) {
        var banner_html = "<div id='" + banner_id + "' class='pl_banner'><a href='javascript:hidePopupLayer();'><img src='" + pl_POPUPBANNERCLOSE_FULLPATH + "' class='pl_banner_close' width='" + pl_POPUPBANNERCLOSE_WIDTH + "' height='" + pl_POPUPBANNERCLOSE_HEIGHT + "' alt='Close' /></a></div>"
        layer.innerHTML = banner_html + layer.innerHTML;
     }
     banner = document.getElementById(banner_id);
   }

   // Secure login support (supported for the Pluto-driven onePlace site only)
   var IFrame_id = "";
   if (style_flags & PL_LOGIN_REQUIRED) {
      // Set the flag that tells the login failure page code we are playing in an IFrame
      login_iframe_active = true;
      IFrame_id = idLayer + pl_LOGIN_IFRAME_ID_SUFFIX;
      pl_active_IFrame_id = IFrame_id;
      // Add the hidden layer that will contain the IFrame if not already added.
      if (layer.innerHTML.indexOf(IFrame_id) < 0) {
         layer.innerHTML = layer.innerHTML + "<div id='" + IFrame_id + "' name='" + IFrame_id + "' style='width:1px; height:1px; border-style:none; display:none; position:absolute; top:1px; left:1px;'></div>"
      }
      // Load the blank secure page into the IFrame
      document.getElementById(IFrame_id).innerHTML = "<iframe src='../spage/13' style='width:1px; height:1px; border-style:none; display:none; position:absolute; top:1px; left:1px;'></iframe>";
   }

   // Set the zIndex if not already set so the layer will appear above the w3 banner, which is zIndex=1
   // If display in bubble, set the bubble's z-order
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      mybubble.style.zIndex = 2;
      mybubblefin.style.zIndex = 3;
   }
   if (layer.style.zIndex < 2) {
      layer.style.zIndex = 2;
   }

   // Get the object that will be used for dragging the layer, if any.
   if (idDragger != "") {
      // If display in bubble, we need to get the object from the cloned layer, not the display layer.
      if (style_flags & PL_DISPLAY_IN_BUBBLE) {
         // if the request was to make the entire layer draggable, and the layer is in a bubble, make the bubble draggable instead..
         if (idDragger == idLayer) {
            pl_active_drag_obj = document.getElementById(pl_BUBBLE_ID);
         } else {
            pl_active_drag_obj = getPopupLayerElementById(idDragger);
         }
      } else {
         pl_active_drag_obj = document.getElementById(idDragger);
      }
   }

   // Position the layer relative to the target
   var show_fin;
   show_fin = pl_positionPopupLayer();

   // Now make the popup layer visible
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      mybubble.style.visibility = "visible";
      // Is fin showing or not based on position?
      if (show_fin) {
         mybubblefin.style.visibility = "visible";
      } else {
         mybubblefin.style.zIndex = -1;
      }
   } else {
      layer.style.visibility = "visible";
   }

   // And we reassign the layer var to be the bubble itself for event handling assignment below.
   if (style_flags & PL_DISPLAY_IN_BUBBLE) {
      layer = mybubble;
   }

   // Capture relevant events while the popup layer is up.
   // First we need to save any event handlers already assigned for system objects before we override them.
   // Generic handlers for all types of layers.
   pl_onkeypress_doc_prevEH = document.onkeypress;
   document.onkeypress = pl_processBodyKey;
   pl_onresize_win_prevEH = window.onresize;
   window.onresize = pl_processResize;
   pl_onunload_win_prevEH = window.onunload;
   window.onunload = pl_processWindowUnload;
   // Handlers for layers that support mouse over and out for display
   if (style_flags & PL_MOUSE_OVER_SUPPORT) {
      pl_onmouseover_trigger_prevEH = trigger.onmouseover;
      trigger.onmouseover = pl_processTriggerMouseOverAndFocus;
      pl_onfocus_trigger_prevEH =  trigger.onfocus;
      trigger.onfocus = pl_processTriggerMouseOverAndFocus;
      layer.onmouseover = pl_processLayerMouseOver;
      layer.onmouseout = pl_processLayerMouseOut;
      if ((style_flags & PL_DISPLAY_IN_BUBBLE) && (mybubblefin != null)) {
         mybubblefin.onmouseover = pl_processLayerMouseOver;
         mybubblefin.onmouseout = pl_processLayerMouseOut;
      }

   } else {
      // Handlers for layers that support clicking for display
      pl_onclick_doc_prevEH = document.onclick;
      document.onclick = pl_processBodyClick;
      pl_onclick_trigger_prevEH = trigger.onclick;
      trigger.onclick = pl_processTriggerClick;
      layer.onclick = pl_processLayerClick;
      if ((style_flags & PL_DISPLAY_IN_BUBBLE) && (mybubblefin != null)) {
         mybubblefin.onclick = pl_processLayerClick;
      }
   }
   // Handlers for layers that support drag/drop.
   if (pl_active_drag_obj != null) {
      pl_onmousemove_body_prevEH = document.body.onmousemove;
      document.body.onmousemove = pl_processDragMouseMove;
      pl_onmouseup_body_prevEH = document.body.onmouseup;
      document.body.onmouseup = pl_processDragMouseUp;
      pl_active_drag_obj.onmousedown = pl_processDragMouseDown;
      pl_active_drag_obj.onlosecapture = pl_processDragLoseCapture;
      // Set cursor to the move pointer when the mouse is over the drag area.
      pl_active_drag_obj_prevcursor = pl_active_drag_obj.style.cursor;
      pl_active_drag_obj.style.cursor = "move";
   }
}


//
// DEPRECTATED - OLD ENTRY POINT from web pages, supported for backward compatibility: position and show the popup layer.
// Calling convention:
//     href="javascript:showPopupLayer(....)" or href="#" onclick="showPopupLayer(....)" for anchor tag click support
//     onclick="showPopupLayer(....)" for spans, divs, etc. click support
//     onmouseover=showPopupLayer(....) if mouse over and mouse out support desired
//     onfocus=showPopupLayer(....) for keyboard support when mouse over and mouse out support desired
// Required parms:
// Parm 1 (id of the popup layer)
// Parm 2 (id of the target object to which it should align itself)
// Parm 3 (id of the trigger object which when clicked on caused this call)
// Optional parms:
// Parm 4 (add the close link, default = true)
// Parm 5 (add the onePlace banner with close icon to top, default = false)
// Parm 6 (popup layer acts like a modal dialog, default = false)
// Parm 7 (secure login is required, default = false)
// Parm 8 (id of the object that will be used for dragging the layer, default = "")
// Parm 9 (mouse over and out support is desired to show and hide this object, default = false)
// Parm 10 (SurfAid base string used for logging that layer was dismissed, default = "")
// Parm 11 (align popup layer to the left or right of the target if possible, default = false)
//
showPopupLayer = function (idLayer, idTarget, idTrigger, add_close, add_w3_banner, dialog_behavior, login_required, idDragger, mouse_over_out_support, surfaid_base_string, align_left_right) {

   // First we see if the optional parms are passed in - if not, we set any not passed in to their default values.
   // If the three required parms are not passed in, nothing we can do here so just return.
   var numArgs = showPopupLayer.arguments.length;
   if (numArgs < 11) {
      align_left_right = false;
      if (numArgs < 10) {
         surfaid_base_string = "";
         if (numArgs < 9) {
            mouse_over_out_support = false;
            if (numArgs < 8) {
               idDragger = "";
               if (numArgs < 7) {
                  login_required = false;
                  if (numArgs < 6) {
                     dialog_behavior = false;
                     if (numArgs < 5) {
                        add_w3_banner = false;
                        if (numArgs < 4) {
                           add_close = true;
                           if (numArgs < 3) {
                              return;
                           }
                        }
                     }
                  }
               }
            }
         }
      }
   }

   // Now set the new style flag to pass to the new function.
   var style_flags = 0x0;
   if (add_close) {
      style_flags |= PL_ADD_CLOSE_LINK;
   }
   if (add_w3_banner) {
      style_flags |= PL_ADD_BANNER;
   }
   if (dialog_behavior) {
      style_flags |= PL_DIALOG_BEHAVIOR;
      style_flags |= PL_FORM;
   }
   if (login_required) {
      style_flags |= PL_LOGIN_REQUIRED;
   }
   if (mouse_over_out_support) {
      style_flags |= PL_MOUSE_OVER_SUPPORT;
   }
   if (align_left_right) {
      style_flags |= PL_SUGGESTED_POS_LEFT;
   }

   // Now pass the modified parms to the new function call.
   showPopupLayer2(idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string);
}


//
// ENTRY POINT from web pages: hide the popup layer if one is showing.
//
// Optional parm:
// Parm 1 (do not notify SurfAid of this layer closing, default = false)
//
hidePopupLayer = function (ignore_surfaid) {
   if (pl_active_layer_obj != null) {
      var numArgs = hidePopupLayer.arguments.length;
      // Default SurfAid action is to log that the layer was canceled
      if (numArgs < 1) {
         ignore_surfaid = false;
      }
      // End any dragging that may be going on.
      pl_endLayerDrag();
      // Hid the bubble fin if visible
      if (pl_active_mybubblefin != null) {
         pl_active_mybubblefin.style.visibility = "hidden";
         pl_active_mybubblefin.style.zIndex = -1;
         pl_active_mybubblefin.style.left = 0;
         pl_active_mybubblefin.style.top = 0;
         pl_active_mybubblefin = null;
      }
      // Hide the popup layer
      pl_active_layer_obj.style.visibility = "hidden";
      pl_active_layer_obj.style.left = 0;
      pl_active_layer_obj.style.top = 0;

      // 1.03 Set the zIndex so the layer is below the content.
      pl_active_layer_obj.style.zIndex = -1;

      // Restore the original html to the popup layer, unless this is a layer
      // which will be submitting a form and therefore can not have its html reset,
      // or the form submission stops.
      if (!(pl_active_style_flags & PL_FORM)) {
         if (pl_active_layer_html != "") {
            // No need to restore the HTML if in a bubble since we cloned the original layer and used the clone.
            if (!(pl_active_style_flags & PL_DISPLAY_IN_BUBBLE)) {
               pl_active_layer_obj.innerHTML = pl_active_layer_html;
            }
            pl_active_layer_html = "";
         }
      } else {
         // So this is a form.
         // Clear secure hidden IFrame if set. There was no need to clear them if we restored the inner html above,
         // which would have cleared the hidden IFrame as well.
         if (login_iframe_active) {
            if (pl_active_IFrame_id != "") {
               var secure_iframe_obj = document.getElementById(pl_active_IFrame_id);
               if (secure_iframe_obj != null) {
                  secure_iframe_obj.innerHTML = "";
               }
            }
         }
         // If the form is displayed in a bubble, we want to copy the cloned layer contents to the
         // original layer so that any changes to form controls will be present the next time the form is displayed.
         if (pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) {
            if ((pl_active_clonedlayer_obj != null) && (pl_active_contentclone_obj != null)) {
               pl_active_clonedlayer_obj.innerHTML = pl_active_contentclone_obj.innerHTML;
            }
         }
      }

      login_iframe_active = false;
      pl_active_layer_id = "";
      pl_active_trigger_id = "";
      pl_active_IFrame_id = "";
      pl_clicked_layer = false;
      pl_active_has_dragged = false;

      // Reset or clear event handlers.
      // Generic handlers for all types of layers.
      document.onkeypress = pl_onkeypress_doc_prevEH;
      pl_onkeypress_doc_prevEH = "";
      window.onresize = pl_onresize_win_prevEH;
      pl_onresize_win_prevEH = "";
      window.onunload = pl_onunload_win_prevEH;
      pl_onunload_win_prevEH = "";
      // Handlers for layers that support mouse over and out for display
      if (pl_active_style_flags & PL_MOUSE_OVER_SUPPORT) {
         pl_active_trigger_obj.onmouseout = "";
         pl_active_trigger_obj.onblur = "";
         pl_active_trigger_obj.onmouseover = pl_onmouseover_trigger_prevEH;
         pl_onmouseover_trigger_prevEH = "";
         pl_active_trigger_obj.onfocus = pl_onfocus_trigger_prevEH;
         pl_onfocus_trigger_prevEH = "";
         pl_active_layer_obj.onmouseout = "";
         pl_active_layer_obj.onmouseover = "";
         if ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) && (pl_active_mybubblefin != null)) {
            pl_active_mybubblefin.onmouseover = "";
            pl_active_mybubblefin.onmouseout = "";
         }
      } else {
         // Handlers for layers that support clicking for display
         document.onclick = pl_onclick_doc_prevEH;
         pl_onclick_doc_prevEH = "";
         pl_active_trigger_obj.onclick = pl_onclick_trigger_prevEH;
         pl_onclick_trigger_prevEH = "";
         pl_active_layer_obj.onclick = "";
         if ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) && (pl_active_mybubblefin != null)) {
            pl_active_mybubblefin.onclick = "";
         }
      }
      // Handlers for layers that support drag/drop.
      if (pl_active_drag_obj != null) {
         document.body.onmousemove = pl_onmousemove_body_prevEH;
         pl_onmousemove_body_prevEH = "";
         document.body.onmouseup = pl_onmouseup_body_prevEH;
         pl_onmouseup_body_prevEH = "";
         pl_active_drag_obj.onmousedown = "";
         pl_active_drag_obj.onlosecapture = "";
         pl_active_drag_obj.style.cursor = pl_active_drag_obj_prevcursor;
         pl_active_drag_obj_prevcursor = "default";
         pl_active_drag_obj = null;
      }

// Do not reset queued items here as there may be valid queued requests when the current popup layer is hidden.
//      pl_queued_show_layer_request = false;
//      pl_queued_hide_layer_request = false;
//      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 8");
//      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 9");

      // Clear out the global objects.
      pl_active_target_obj = null;
      pl_active_trigger_obj = null;
      pl_active_layer_obj = null;
      pl_active_clonedlayer_obj = null;
      pl_active_contentclone_obj = null;
      pl_active_style_flags = 0x0;

      // Finally, notify SurfAid if necessary
      if ((pl_active_surfaid_base_string != "") && (!ignore_surfaid)) {
         sa_onclick(pl_active_surfaid_base_string);
      }
      pl_active_surfaid_base_string = "";
   }
}


//
// ENTRY POINT from web pages: return the object in the popup layer with the specified ID.
// Special function needed to return the field of the passed ID when included in a bubble layer.
// Also works when not in a bubble, but the field can be called directly using document.getElementById in that case.
//
getPopupLayerElementById = function (id) {
   var obj;
   var mybubblecontent = document.getElementById(pl_BUBBLE_CONTENT_ID);
   obj = pl_findChildNode(mybubblecontent, id);
   if (obj == null) {
      obj = document.getElementById(id);
   }
   return obj;
}


// -----------------------------------------------------------------------------
//
// Local functions not called externally (pl_ prefix)
//
// -----------------------------------------------------------------------------

//
// Position the layer.
//
pl_positionPopupLayer = function () {

   var left_set = false;
   var top_set = false;
   var index = -1;
   var pos_left = false;
   var pos_right = false;
   var pos_below= false;
   var pos_above = false;
   var bubble_fin_space = 0;
   var left_layer_left;
   var left_offscreen;
   var right_layer_left;
   var right_offscreen;
   var below_layer_top;
   var below_offscreen;
   var above_layer_top;
   var above_offscreen;
   var offscreen = 0;
   var aOffScreen;
   var offset_for_bubble = 0;
   // Amount to adjust the fin from the corner of the target
   var finpos_indent = 0;

   // Return value of whether or not to show the fin
   var show_fin = false;

   // Global positioning attributes - assign to local variables here for convenience.
   var layer = pl_active_layer_obj;
   var target = pl_active_target_obj;
   var suggested_pos = (pl_active_style_flags & (PL_SUGGESTED_POS_BELOW | PL_SUGGESTED_POS_ABOVE | PL_SUGGESTED_POS_LEFT | PL_SUGGESTED_POS_RIGHT));
   var force_pos = ((pl_active_style_flags & PL_FORCE_POSITION) != 0x0);
   var force_visible_target = ((pl_active_style_flags & PL_FORCE_VISIBLE_TARGET) != 0x0);
   var display_in_bubble = ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) != 0x0);

   // Default position is below the target.
   if ((suggested_pos != PL_SUGGESTED_POS_LEFT) && (suggested_pos != PL_SUGGESTED_POS_RIGHT) && (suggested_pos != PL_SUGGESTED_POS_ABOVE) && (suggested_pos != PL_SUGGESTED_POS_BELOW)) {
      suggested_pos = PL_SUGGESTED_POS_BELOW;
   }

   var target_parent_offsetTop = 0;
   var target_parent_offsetLeft = 0;
   var target_parent = target.offsetParent;
   // Work up the object offset hierarchy, adding up the offsets up to the body tag
   while (target_parent != null) {
// 1.11 Keep going past the BODY tag in case the body itself has a margin, even though w3 and onePlace pages do not.
//      if (target_parent.tagName == "BODY") {
//         break;
//      }
      target_parent_offsetTop += target_parent.offsetTop;
      target_parent_offsetLeft += target_parent.offsetLeft;
      target_parent_offsetTop -= target_parent.scrollTop;   // MH-75
      target_parent_offsetLeft -= target_parent.scrollLeft; // MH-75
      target_parent = target_parent.offsetParent;
   }
   var client_width = document.documentElement.clientWidth;
   var client_height = document.documentElement.clientHeight;
   var client_v_scrolled = document.documentElement.scrollTop;
   var client_h_scrolled = document.documentElement.scrollLeft;
   var layer_left = -1;
   var layer_top = -1;
   var amt_hidden_below = 0;
   var amt_hidden_above = 0;
   var layer_height;
   var layer_width;
   // If we are in a bubble, then we need to run the positioning calcs based on the original size of the
   // bubble layer without considering any increases in width or height we may make later to accomodate the fin
   // if the original layer is not larget enough for the fin to fit on the desired side.  This comes into play
   // on resizes of the window which may change which side the fin appears on.
   if ((display_in_bubble) && (pl_bubble_orig_width > 0)) {
      layer_width = pl_bubble_orig_width;
   } else {
      layer_width = layer.offsetWidth;
      pl_bubble_orig_width = layer_width;
   }
   if ((display_in_bubble) && (pl_bubble_orig_height > 0)) {
      layer_height = pl_bubble_orig_height;
   } else {
      layer_height = layer.offsetHeight;
      pl_bubble_orig_height = layer_height;
   }

   // First we calculate all 4 positions (for left and right, if doesn't fit going down try again going up
   // and choose the best). We store how much it is off the screen if any for that axis.

   // Leave extra space for the fin if we are displaying the layer in a bubble.
   if (display_in_bubble) {
      bubble_fin_space = pl_BUBBLE_FIN_REACH - pl_BUBBLE_FIN_INSET;
   }

   // Left
   left_layer_left = target.offsetLeft + target_parent_offsetLeft - layer_width - bubble_fin_space;
   left_offscreen = Math.max(0, client_h_scrolled - left_layer_left);
   // Not allowed to be all the way off the left if we can't move it back over the target
   if (force_visible_target && (left_offscreen > client_h_scrolled)) {
      left_offscreen = pl_BIG_OFFSCREEN;
   }

   // Right
   right_layer_left = target.offsetLeft + target_parent_offsetLeft + target.offsetWidth + bubble_fin_space;
   right_offscreen = Math.max(0, (right_layer_left + layer_width) - (client_width + client_h_scrolled));

   // Below
   below_layer_top = target.offsetTop + target_parent_offsetTop + target.offsetHeight + bubble_fin_space;
   below_offscreen = Math.max(0, (below_layer_top + layer_height) - (client_height + client_v_scrolled));

   // Above
   above_layer_top = target.offsetTop + target_parent_offsetTop - layer_height - bubble_fin_space;
   above_offscreen = Math.max(0, client_v_scrolled - above_layer_top);
   // Not allowed to be all the way off the top if we can't move it back over the target
   if (force_visible_target && (above_offscreen > client_v_scrolled)) {
      above_offscreen = pl_BIG_OFFSCREEN;
   }

   // If we are forcing the suggested position to be used, use it even if if doesn't fit well.
   if (force_pos) {

      switch(suggested_pos) {
      case PL_SUGGESTED_POS_LEFT:
         layer_left = left_layer_left;
         left_set = true;
         pos_left = true;
         break;
      case PL_SUGGESTED_POS_RIGHT:
         layer_left = right_layer_left;
         left_set = true;
         pos_right = true;
         break;
      case PL_SUGGESTED_POS_BELOW:
         layer_top = below_layer_top;
         top_set = true;
         pos_below = true;
         break;
      case PL_SUGGESTED_POS_ABOVE:
         layer_top = above_layer_top;
         top_set = true;
         pos_above = true;
         break;
      }
      // Since forced to be where it wants to be, always show the fin.
      if (display_in_bubble) {
         show_fin = true;
      }

   } else {
      // Else we are sugesting a desired placement, but choose the first that fits.  The order we check for a fit depends on
      // which was the suggested pos.
      switch(suggested_pos) {
      case PL_SUGGESTED_POS_LEFT:
         aOffScreen = [left_offscreen, right_offscreen, below_offscreen, above_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 0:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 1:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 2:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 3:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;

      case PL_SUGGESTED_POS_RIGHT:
         aOffScreen = [right_offscreen, left_offscreen, below_offscreen, above_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 1:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 0:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 2:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 3:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;

      case PL_SUGGESTED_POS_BELOW:
         aOffScreen = [below_offscreen, above_offscreen, left_offscreen, right_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 2:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 3:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 0:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 1:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;

      case PL_SUGGESTED_POS_ABOVE:
         aOffScreen = [above_offscreen, below_offscreen, left_offscreen, right_offscreen];
         index = pl_getFirstPosition(aOffScreen);
         switch(index) {
         case 2:
            layer_left = left_layer_left;
            offscreen = left_offscreen;
            left_set = true;
            pos_left = true;
            break;
         case 3:
            layer_left = right_layer_left;
            offscreen = right_offscreen;
            left_set = true;
            pos_right = true;
            break;
         case 1:
            layer_top = below_layer_top;
            offscreen = below_offscreen;
            top_set = true;
            pos_below = true;
            break;
         case 0:
            layer_top = above_layer_top;
            offscreen = above_offscreen;
            top_set = true;
            pos_above = true;
            break;
         }
         break;
      }
      // Now if we wanted to force that the target be visible, we are done now, even if the layer is off the screen a bit.
      // Otherwise we need to move the layer back onto the screen if it is off screen.
      if (force_visible_target) {
         if (display_in_bubble) {
            show_fin = true;
         }
      } else {
         // Are we off the screen? Then bring it back on.
         // If we wanted a bubble but have to adjust the layer because it was off the screen, and we therefore
         // are moving closer to the target than optimal, we don't get to display the bubble fin.
         if (offscreen > 0) {
            if (pos_left) {
               layer_left += offscreen;
            } else if (pos_right) {
               layer_left -= offscreen;
            } else if (pos_below) {
               layer_top -= offscreen;
            } else if (pos_above) {
               layer_top += offscreen;
            }
         } else {
            // If we were not off the screen, and we want a bubble, then we can display the fin.
            if (display_in_bubble) {
               show_fin = true;
            }
         }
      }
      // Now one last check to ensure that the layer is not off the screen in the opposite direction of what was
      // calculated when there is room to move it fully on. But such a move removes the space for the fin, if any.
      var offscreen_opposite = 0;
      if (pos_left) {
         offscreen_opposite = layer_left + layer_width - (client_width + client_h_scrolled);
         if ((layer_left > client_h_scrolled) && (offscreen_opposite > 0)) {
            layer_left -= Math.min(layer_left - client_h_scrolled, offscreen_opposite);
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      } else if (pos_right) {
         offscreen_opposite = client_h_scrolled - layer_left;
         if (offscreen_opposite > 0) {
            layer_left += offscreen_opposite;
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      } else if (pos_below) {
         offscreen_opposite = client_v_scrolled - layer_top;
         if (offscreen_opposite > 0) {
            layer_top += offscreen_opposite;
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      } else if (pos_above) {
         offscreen_opposite = layer_top + layer_height - (client_height + client_v_scrolled);
         if ((layer_top > client_v_scrolled) && (offscreen_opposite > 0)) {
            layer_top -= Math.min(layer_top - client_v_scrolled, offscreen_opposite);
            if (display_in_bubble) {
               show_fin = false;
            }
         }
      }
   }

   // Now we need to ensure the bubble is large enough for the fin to fit on the decided side by forcing the
   // necessary dimension to be able to accomodate the fin.  We then need to manually adjust the layer height
   // and width variables because IE does not reflect the changes immediately if we query them after we
   // make these changes.
   if (display_in_bubble) {
      if (pos_left || pos_right) {
         pl_active_bubble_minheight_cell.innerHTML = pl_BUBBLE_CONTENT_MINHEIGHT_HTML;
         pl_active_bubble_minwidth_cell.innerHTML = "";
         layer_height = Math.max(layer_height, pl_BUBBLE_FIN_BASE + (2 * pl_BUBBLE_CORNER_HEIGHT));
      } else if (pos_below || pos_above) {
         pl_active_bubble_minwidth_cell.innerHTML = pl_BUBBLE_CONTENT_MINWIDTH_HTML;
         pl_active_bubble_minheight_cell.innerHTML = "";
         layer_width = Math.max(layer_width, pl_BUBBLE_FIN_BASE + (2 * pl_BUBBLE_CORNER_WIDTH));
      }
   }

   // So we now have the key positioning, left/right/below/above, and we need to slide/position it along the
   // opposite axis now.

   // If we positioned it to the left or the right, we need to position it vertically to line up with
   // the target as best we can.  It may extend down from the top of the target, or extend up from the bottom
   // of the target.
   if (pos_left || pos_right) {
      if (display_in_bubble) {
         finpos_indent = Math.min(pl_FINPOS_INDENT_DEFAULT, Math.floor((target.offsetHeight / 2) + .25));
         var layer_top2 = 0;
         var bubblefin_top = 0;
         offset_for_bubble = Math.min(layer_height * pl_IDEAL_EAST_WEST_FIN_POSITION, layer_height - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_HEIGHT)
         layer_top = Math.max(target.offsetTop + target_parent_offsetTop - offset_for_bubble + finpos_indent, client_v_scrolled);

         // If layer off the top, move it on
         if (layer_top < client_v_scrolled) {
            layer_top = client_v_scrolled;
         // Else if layer off the bottom, move it on but keep it on the top as well
         } else if ((layer_top + layer_height) > (client_height + client_v_scrolled)) {
            layer_top = Math.max(client_v_scrolled, client_height + client_v_scrolled - layer_height);
         }

         // Now handle the case where the layer is being pushed up by the edge of the window, and there is not enough height under the
         // target to fit the fin. In this case we want to move the layer above the target so the fin can point downward, as long
         // as the layer is not off the glass. If neither fits then we do not want to display the fin.
         if (target.offsetTop + target_parent_offsetTop + finpos_indent + pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_HEIGHT > layer_top + layer_height) {
            layer_top2 = Math.min(layer_top, target.offsetTop + target_parent_offsetTop + finpos_indent - (pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_HEIGHT));
            if (layer_top2 != layer_top) {
               if ((layer_top2 + layer_height) > (client_height + client_v_scrolled)) {
                  layer_top = Math.max(client_v_scrolled, client_height + client_v_scrolled - layer_height);
               } else if (layer_top2 >= client_v_scrolled) {
                  layer_top = layer_top2;
               } else {
                  show_fin = false;
               }
            }
         }

         // Now where do we position the bubble fin, if any?
         if (show_fin) {
            // If the bubble is not aligned with the target, no fin to show.
            if ((layer_top + pl_BUBBLE_CORNER_HEIGHT > target.offsetTop + target_parent_offsetTop + target.offsetHeight - finpos_indent) || (layer_top + layer_height - pl_BUBBLE_CORNER_HEIGHT < target.offsetTop + target_parent_offsetTop + finpos_indent)) {
               show_fin = false;
            } else {
               if (pos_left) {
                  if (layer_top + (layer_height / 2) < target.offsetTop + target_parent_offsetTop) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_ED;
                     bubblefin_top = target.offsetTop + target_parent_offsetTop - pl_BUBBLE_FIN_BASE + finpos_indent;
                     if (bubblefin_top >= layer_top + pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_EU;
                     bubblefin_top = Math.max(target.offsetTop + target_parent_offsetTop + finpos_indent, layer_top + pl_BUBBLE_CORNER_HEIGHT);
                     if (bubblefin_top <= layer_top + layer_height - pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  }
                  pl_active_mybubblefin.style.left = layer_left + layer_width - pl_BUBBLE_FIN_INSET + "px";
               } else {
                  if (layer_top + (layer_height / 2) < target.offsetTop + target_parent_offsetTop) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_WD;
                     bubblefin_top = target.offsetTop + target_parent_offsetTop - pl_BUBBLE_FIN_BASE + finpos_indent;
                     if (bubblefin_top >= layer_top + pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_WU;
                     bubblefin_top = Math.max(target.offsetTop + target_parent_offsetTop + finpos_indent, layer_top + pl_BUBBLE_CORNER_HEIGHT);
                     if (bubblefin_top <= layer_top + layer_height - pl_BUBBLE_CORNER_HEIGHT) {
                        pl_active_mybubblefin.style.top = bubblefin_top + "px";
                     } else {
                        show_fin = false;
                     }
                  }
                  pl_active_mybubblefin.style.left = layer_left - (pl_BUBBLE_FIN_REACH - pl_BUBBLE_FIN_INSET) + "px";
               }
            }
         }
      // Else not displayed in bubble.
      } else {
         // Default top position is aligned with the top of the object but can't be off the top of the screen.
         layer_top = Math.max(target.offsetTop + target_parent_offsetTop, client_v_scrolled);
         amt_hidden_below = Math.max(0, (layer_top + layer_height) - (client_height + client_v_scrolled));
         if (amt_hidden_below > 0) {
            layer_top = Math.min(target.offsetTop + target_parent_offsetTop + target.offsetHeight - layer_height, client_height + client_v_scrolled - layer_height);
            amt_hidden_above = Math.max(0, (client_v_scrolled - layer_top));
            if (amt_hidden_above > 0) {
               // If the layer is bigger than the client area, top-justify it.
               if (client_height <= layer_height) {
                  layer_top = client_v_scrolled;
               } else {
                  // Else popup layer will fit fully in client, so move it up until it all shows.
                  // Move it the smallest amount from the two default positions.
                  if (amt_hidden_below > amt_hidden_above) {
                     // If more is hidden below, the top-justify the layer
                     layer_top = client_v_scrolled;
                  } else {
                     // Else more is hidden above, so slide the layer up so its bottom is on the screen.
                     layer_top = target.offsetTop + target_parent_offsetTop - amt_hidden_below;
                  }
               }
            }
         }
      }

   // If we positioned it below or above the target, we need to position it horizontally. We try to line the left side
   // of the layer with the left side of the target as best we can.
   } else if (pos_below || pos_above) {
      var layer_left2 = 0;
      var bubblefin_left = 0;
      if (display_in_bubble) {
         offset_for_bubble = Math.min(layer_width * pl_IDEAL_NORTH_SOUTH_POSITION, layer_width - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_WIDTH)
         finpos_indent = Math.min(pl_FINPOS_INDENT_DEFAULT, Math.floor((target.offsetWidth / 2) + .25));
      }
      // Default left position is left-aligned with the object
      layer_left = target.offsetLeft + target_parent_offsetLeft - offset_for_bubble + finpos_indent;
      // But if it extends off the screen to the right, right-justify it
      if (layer_left + layer_width > client_width + client_h_scrolled) {
         layer_left = client_width + client_h_scrolled - layer_width ;
      }
      // But if it now extends off the screen to the left, left-justify it so you can read the beginning of the text
      layer_left = Math.max(client_h_scrolled, layer_left);

      // Now if it is in a bubble, we slide it to further to the left if we can, then attach the fin unless not needed.
      // If we are displaying in a bubble, reposition the layer if possible so it is offset so the fin (if needed) will
      // not be on the end of the layer unless that moves it off the glass.
      if (display_in_bubble) {
         // Now handle the case where the layer is being pushed to the left by the edge of the window, and there is not enough width under the
         // target to fit the fin. In this case we want to move the layer to the left of the target so the fin can point to the right, as long
         // as the layer is not off the glass. If neither fits then we do not want to display the fin.
         if (target.offsetLeft + target_parent_offsetLeft + finpos_indent + pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_WIDTH > layer_left + layer_width) {
            layer_left2 = Math.min(layer_left, target.offsetLeft + target_parent_offsetLeft + finpos_indent - (pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_HEIGHT));
            if (layer_left2 != layer_left) {
               if ((layer_left2 + layer_width) > (client_width + client_h_scrolled)) {
                  layer_left = Math.max(client_h_scrolled, client_width + client_h_scrolled - layer_width);
               } else if (layer_left2 >= client_h_scrolled) {
                  layer_left = layer_left2;
               } else {
                  show_fin = false;
               }
            }
         }
         if (show_fin) {
            // Can only show fin if layer is under or over the target to a degree.
            if ((target.offsetLeft + target_parent_offsetLeft + target.offsetWidth - finpos_indent < layer_left + pl_BUBBLE_CORNER_WIDTH) || (target.offsetLeft + target_parent_offsetLeft + finpos_indent > layer_left + layer_width - pl_BUBBLE_CORNER_WIDTH)) {
               show_fin = false;
            // Now see where the layer ended out in relation to the target.
            // Is layer to the left of target by at least the width of the fin plus the table corner, but not all
            // the way past the target?  Then we will use the arrow pointing to the right.
            } else if ((target.offsetLeft + target_parent_offsetLeft + finpos_indent >= layer_left + pl_BUBBLE_FIN_BASE + pl_BUBBLE_CORNER_WIDTH) && (layer_left + layer_width - pl_BUBBLE_CORNER_WIDTH >= target.offsetLeft + target_parent_offsetLeft + finpos_indent)) {
               bubblefin_left = target.offsetLeft + target_parent_offsetLeft + finpos_indent - pl_BUBBLE_FIN_BASE;
               // Ensure fin fits at final layer position.
               if ((bubblefin_left >= layer_left + pl_BUBBLE_CORNER_WIDTH) && (bubblefin_left <= layer_left + layer_width - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_WIDTH)) {
                  pl_active_mybubblefin.style.left = bubblefin_left + "px"
                  if (pos_below) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_NR;
                     pl_active_mybubblefin.style.top = layer_top - pl_BUBBLE_FIN_REACH + pl_BUBBLE_FIN_INSET_Y + "px";
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_SR;
                     pl_active_mybubblefin.style.top = layer_top + layer_height - pl_BUBBLE_FIN_INSET_Y + "px";
                  }
               } else {
                  show_fin = false;
               }
            // Else we use the arrow pointing to the left, as long as it fits and will still point to the target
            } else {
               bubblefin_left = Math.max(target.offsetLeft + target_parent_offsetLeft + finpos_indent, layer_left + pl_BUBBLE_CORNER_WIDTH);
               // Ensure fin fits at final layer position.
               if ((bubblefin_left >= layer_left + pl_BUBBLE_CORNER_WIDTH) && (bubblefin_left <= layer_left + layer_width - pl_BUBBLE_FIN_BASE - pl_BUBBLE_CORNER_WIDTH)) {
                  pl_active_mybubblefin.style.left = bubblefin_left + "px";
                  if (pos_below) {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_NL;
                     pl_active_mybubblefin.style.top = layer_top - pl_BUBBLE_FIN_REACH + pl_BUBBLE_FIN_INSET_Y + "px";
                  } else {
                     pl_active_mybubblefin.innerHTML = pl_BUBBLE_FIN_SL;
                     pl_active_mybubblefin.style.top = layer_top + layer_height - pl_BUBBLE_FIN_INSET_Y + "px";
                  }
               } else {
                  show_fin = false;
               }
            }
         }
      }
   }

   // Set the position of the popup layer
   layer.style.top = layer_top + "px";
   layer.style.left = layer_left + "px";

   // return whether the fin should be displayed if appropriate.
   return show_fin;
}


//
// Called by the layer positioning routine, it is passed in an array of values indicatng how much the layer is off the screen
// in the default position in the four directions. It returns the index of the first that is not off the screen, and if all
// are off the screen it returns the index of which is off the screen the least.
//
pl_getFirstPosition = function (aOffScreen) {
   var index = -1;
   var minOffScreen = -1;

   if (aOffScreen[0] == 0) {
      index = 0;
   } else if (aOffScreen[1] == 0) {
      index = 1;
   } else if (aOffScreen[2] == 0) {
      index = 2;
   } else if (aOffScreen[3] == 0) {
      index = 3;
   } else {
      // Else all are off the screen.  Find which is off the screen the least.
      if (aOffScreen[0] <= aOffScreen[1]) {
         minOffScreen = aOffScreen[0];
         index = 0;
      } else {
         minOffScreen = aOffScreen[1];
         index = 1;
      }
      if (aOffScreen[2] < minOffScreen) {
         minOffScreen = aOffScreen[2];
         index = 2;
      }
      if (aOffScreen[3] < minOffScreen) {
         minOffScreen = aOffScreen[3];
         index = 3;
      }
   }
   return index;
}


//
// Recurse through children nodes, looking for node with the passed in id.
//
pl_findChildNode = function (root, idDragger) {
   var x;
   if (root != null) {
      if (root.id == idDragger) {
         return root;
      }
      if (root.childNodes.length > 0) {
         var childnode;
         for (x = 0; x < root.childNodes.length; x++) {
            childnode = pl_findChildNode(root.childNodes[x], idDragger);
            if (childnode != null) {
               return childnode;
            }
         }
      }
   }
   return null;
}


//
// Preloads the images used by the layer so they wil display quickly when first needed.
//
pl_preloadPopupLayerImages = function () {
   if (!pl_bubble_images_preloaded) {
      pl_img_clearpix = new Image(1, 1);
      pl_img_clearpix.src = pl_CLEARPIX_FULLPATH;
      pl_img_banner_bg = new Image(pl_BANNER_BG_WIDTH, pl_BANNER_BG_HEIGHT);
      pl_img_banner_bg.src = pl_POPUPBANNER_FULLPATH;
      pl_img_close_x = new Image(pl_BANNER_CLOSE_X_WIDTH, pl_BANNER_CLOSE_X_HEIGHT);
      pl_img_close_x.src = pl_POPUPBANNERCLOSE_FULLPATH;

      pl_img_bubble_corner_nw = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_nw.src = pl_BUBBLE_CORNER_FULLPATH_NW;
      pl_img_bubble_corner_ne = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_ne.src = pl_BUBBLE_CORNER_FULLPATH_NE;
      pl_img_bubble_corner_sw = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_sw.src = pl_BUBBLE_CORNER_FULLPATH_SW;
      pl_img_bubble_corner_se = new Image(pl_BUBBLE_CORNER_WIDTH, pl_BUBBLE_CORNER_HEIGHT);
      pl_img_bubble_corner_se.src = pl_BUBBLE_CORNER_FULLPATH_SE;

      pl_img_bubble_side_w = new Image(pl_BUBBLE_SIDE_DEPTH, pl_BUBBLE_SIDE_LENGTH);
      pl_img_bubble_side_w.src = pl_BUBBLE_SIDE_FULLPATH_W;
      pl_img_bubble_side_e = new Image(pl_BUBBLE_SIDE_DEPTH, pl_BUBBLE_SIDE_LENGTH);
      pl_img_bubble_side_e.src = pl_BUBBLE_SIDE_FULLPATH_E;
      pl_img_bubble_side_n = new Image(pl_BUBBLE_SIDE_LENGTH, pl_BUBBLE_SIDE_DEPTH);
      pl_img_bubble_side_n.src = pl_BUBBLE_SIDE_FULLPATH_N;
      pl_img_bubble_side_s = new Image(pl_BUBBLE_SIDE_LENGTH, pl_BUBBLE_SIDE_DEPTH);
      pl_img_bubble_side_s.src = pl_BUBBLE_SIDE_FULLPATH_S;

      pl_img_bubble_fin_nl = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_nl.src = pl_BUBBLE_FIN_FULLPATH_NL;
      pl_img_bubble_fin_nr = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_nr.src = pl_BUBBLE_FIN_FULLPATH_NR;
      pl_img_bubble_fin_sl = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_sl.src = pl_BUBBLE_FIN_FULLPATH_SL;
      pl_img_bubble_fin_sr = new Image(pl_BUBBLE_FIN_BASE, pl_BUBBLE_FIN_REACH);
      pl_img_bubble_fin_sr.src = pl_BUBBLE_FIN_FULLPATH_SR;
      pl_img_bubble_fin_eu = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_eu.src = pl_BUBBLE_FIN_FULLPATH_EU;
      pl_img_bubble_fin_ed = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_ed.src = pl_BUBBLE_FIN_FULLPATH_ED;
      pl_img_bubble_fin_wu = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_wu.src = pl_BUBBLE_FIN_FULLPATH_WU;
      pl_img_bubble_fin_wd = new Image(pl_BUBBLE_FIN_REACH, pl_BUBBLE_FIN_BASE);
      pl_img_bubble_fin_wd.src = pl_BUBBLE_FIN_FULLPATH_WD;

      pl_bubble_images_preloaded = true;
   }
}


//
// If there is a queued request to show the layer, do it now.
//
pl_showDelayedPopupLayer = function (idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string) {
   if (pl_queued_show_layer_request) {
      showPopupLayer2(idLayer, idTarget, idTrigger, style_flags, idDragger, surfaid_base_string);
   }
}


//
// If there is a queued request to hide the layer, do it now.
//
pl_hideDelayedPopupLayer = function () {
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      hidePopupLayer()
   }
}


//
// If a timer is set, clear it.
//
pl_cancelTimeout = function (timer_id, str_debug) {
   if (timer_id != -1) {
      clearTimeout(timer_id);
   }
   return -1;
}


//
// Event handler for cancelling a queued hide request, set for the layer object if displayed on mouse over.
//
pl_processLayerMouseOver = function (event) {
   // If there is a queued hide request, cancel it because we are back over again.
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 4");

   }
   return true;
}


//
// Event handler for cancelling a queued hide request, set for the trigger object if displayed on mouse over.
//
pl_processTriggerMouseOverAndFocus = function (event) {
   // Reset the mouse out and blur event handlers for the trigger now that we are over it again.
   if (event == null) {
      event = window.event
   }
   if (event.target) {
      event.target.onblur = pl_processTriggerMouseOutAndBlur;
      event.target.onmouseout = pl_processTriggerMouseOutAndBlur;
   } else if (event.srcElement) {
      event.srcElement.onblur = pl_processTriggerMouseOutAndBlur;
      event.srcElement.onmouseout = pl_processTriggerMouseOutAndBlur;
   }
   // If there is a queued hide request, cancel it because we are back over again.
   if (pl_queued_hide_layer_request) {
      pl_queued_hide_layer_request = false;
      pl_queued_hide_layer_timer_id = pl_cancelTimeout(pl_queued_hide_layer_timer_id, "hide 4");
   }
   return true;
}


//
// Event handler for queueing to dismiss the layer, set for the layer object if displayed on mouse over.
//
pl_processLayerMouseOut = function (event) {
   // If there is a queued show event, cancel it because we have moved back out again.
   if (pl_queued_show_layer_request) {
      pl_queued_show_layer_request = false;
      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 6");
   }
   // Modal dialogs do not go away on mouse out. Otherwise queue a hide event as long as the active layer
   // supports mouse outs.
   if ((!(pl_active_style_flags & PL_DIALOG_BEHAVIOR)) && (pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
      pl_queued_hide_layer_request = true;
      pl_queued_hide_layer_timer_id = window.setTimeout("pl_hideDelayedPopupLayer()", pl_MOUSEOUT_HIDE_DELAY);
   }
   return true;
}


//
// Event handler for queueing to dismiss the layer, set for the trigger object if displayed on mouse over or focus.
//
pl_processTriggerMouseOutAndBlur = function (event) {
   // When we get a blur or mouseout, we need to clear the objects event handler that caused it. We can't do
   // this on the close of the layer because blur and mouseover are set when a show is requested, and may
   // need to be cleared before the show takes place.
   if (event == null) {
      event = window.event
   }
   if (event.target) {
      event.target.onblur = "";
      event.target.onmouseout = "";
   } else if (event.srcElement) {
      event.srcElement.onblur = "";
      event.srcElement.onmouseout = "";
   }

   // If there is a queued show event, cancel it because we have moved back out again.
   if (pl_queued_show_layer_request) {
      pl_queued_show_layer_request = false;
      pl_queued_show_layer_timer_id = pl_cancelTimeout(pl_queued_show_layer_timer_id, "show 7");
   }
   // Modal dialogs do not go away on mouse out. Otherwise queue a hide event as long as the active layer
   // supports mouse outs.
   if ((!(pl_active_style_flags & PL_DIALOG_BEHAVIOR)) && (pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
      pl_queued_hide_layer_request = true;
      pl_queued_hide_layer_timer_id = window.setTimeout("pl_hideDelayedPopupLayer()", pl_MOUSEOUT_HIDE_DELAY);
   }
   return true;
}


//
// Event handler for drag support, set for the drag object.
//
pl_processDragMouseDown = function (event) {
   if (event == null) {
      event = window.event
   }
   if (pl_isButton1(event)) {
      if ((event != null) && (pl_active_layer_obj != null) && (pl_active_drag_obj != null)) {
         // Store the initial pos of the mouse so we can know when it is moved enough to start the drag.
         pl_drag_mouse_down = true;
         pl_move_init_x = event.clientX;
         pl_move_init_y = event.clientY;
         // Return false if user clicks on the frag area so default drag action on an object in Firefox does not occur.
         return pl_setReturnValue(false, event);
      }
   }
   return pl_setReturnValue(true, event);
}


//
// Event handler for drag support, set for the document body.
//
pl_processDragMouseUp = function (event) {
   if (pl_drag_mouse_down) {
      pl_endLayerDrag();
   }
   return pl_setReturnValue(true, event);
}


//
// Event handler for drag support, set for the document body.
//
pl_processDragMouseMove = function (event) {
   if (event == null) {
      event = window.event
   }
   // If we are not dragging yet, and the mouse is down, have we moved enough to start the drag?
   if ((!pl_move_in_process) && (pl_drag_mouse_down) && (event != null) && (pl_active_layer_obj != null) && (pl_active_drag_obj != null)) {
      if ((Math.abs(pl_move_init_x - event.clientX) >= pl_DRAG_START_DIFFERENTIAL) || (Math.abs(pl_move_init_y - event.clientY) >= pl_DRAG_START_DIFFERENTIAL)) {
         // Record we are dragging now.
         pl_move_in_process = true;
         pl_active_has_dragged = true;
         // Store the initial offset of the mouse from the layer top-left, so we can move the layer with the mouse.
         pl_move_offset_x = pl_active_layer_obj.offsetLeft  - document.documentElement.scrollLeft - event.clientX;
         pl_move_offset_y = pl_active_layer_obj.offsetTop - document.documentElement.scrollTop - event.clientY;
         // Store the min position the layer can move up and left. so part of the drag area is always visible.
         var drag_x = pl_active_drag_obj.offsetLeft;
         var drag_y = pl_active_drag_obj.offsetTop;
         var drag_width = pl_active_drag_obj.clientWidth;
         var drag_height = pl_active_drag_obj.clientHeight;
         // Account for the drag area being the layer itself, or nested within the layer. We want the offset from the top left of the layer.
         if (pl_active_drag_obj.id == pl_active_layer_id) {
            drag_x = 0;
            drag_y = 0;
         } else {
            var parent = pl_active_drag_obj.offsetParent;
            while (parent != null) {
               if (parent.id == pl_active_layer_id) {
                  break;
               }
               drag_x += parent.offsetLeft;
               drag_y += parent.offsetTop;
               parent = parent.offsetParent;
            }
         }
         // Now we get the offset of the lower right corner of the drag area from the upper left of the layer, adjusted by how much we want to stay visible.
         var drag_x_right = Math.max(drag_x, drag_x + drag_width - pl_DRAG_VISIBILITY_BUFFER);
         var drag_y_bottom = Math.max(drag_y, drag_y + drag_height - pl_DRAG_VISIBILITY_BUFFER);
         // And now we calculate the min position of the layer so that the visibility buffer of the drag area is always visible upper left.
         pl_move_min_x = -drag_x_right;
         pl_move_min_y = -drag_y_bottom;
         // Capture the mouse to catch mouse ups when the mouse is outside the window.
         if (pl_active_drag_obj.setCapture) {
            pl_active_drag_obj.setCapture(true);
         }
      }
   }

   // If we are dragging, adjust the layer position
   if (pl_move_in_process && pl_isButton1(event) && (event != null) && (pl_active_layer_obj != null)) {
      pl_active_layer_obj.style.left = Math.max(pl_move_min_x, event.clientX + pl_move_offset_x + document.documentElement.scrollLeft) + "px";
      pl_active_layer_obj.style.top = Math.max(pl_move_min_y, event.clientY + pl_move_offset_y + document.documentElement.scrollTop) + "px";
      // Remove the bubble fin on a move if visible
      if (pl_active_mybubblefin != null) {
         pl_active_mybubblefin.style.visibility = "hidden";
         pl_active_mybubblefin.style.left = 0;
         pl_active_mybubblefin.style.top = 0;
         pl_active_mybubblefin = null;
      }
      return pl_setReturnValue(false, event);
   }
   return pl_setReturnValue(true, event);
}


//
// Event handler for drag support, set for the drag object, called when mouse capture is terminated.
//
pl_processDragLoseCapture = function (event) {
   pl_endLayerDrag();
   return pl_setReturnValue(true, event);
}


//
// Called when drag ends.
//
pl_endLayerDrag = function () {
   pl_move_in_process = false;
   pl_drag_mouse_down = false;
   pl_move_init_x = 0;
   pl_move_init_y = 0;
   pl_move_min_x = 0;
   pl_move_min_y = 0;
   pl_move_offset_x = 0;
   pl_move_offset_y = 0;
   if (document.releaseCapture) {
      document.releaseCapture();
   }
}


//
// Called to set return values during drag operations.
//
pl_setReturnValue = function (retval, event) {
   if (event == null) {
      event = window.event
   }
   if (event) {
      event.returnValue = retval;
//      if (event.cancelBubble) {
//         event.cancelBubble = retval;
//      }
//      if (!retval) {
//         if (event.stopPropagation) {
//            event.stopPropagation();
//         }
//         if (event.preventDefault) {
//            event.preventDefault();
//         }
//      }
   }
   return retval;
}


//
// Called to see if button 1 is down during drag operations.
//
pl_isButton1 = function (event) {
   if (event == null) {
      event = window.event
   }
   if (event) {
      if (event.button) {
         if (event.button != 1) {
            return false;
         }
      } else {
         if (event.which) {
            if (event.which != 1) {
               return false;
            }
         }
      }
   }
   return true;
}


//
// Event handler: Set a flag that the popup layer was just clicked and send the click on to the general click handler
//
pl_processLayerClick = function () {
   pl_clicked_layer = true;
   return true;
}


//
// Event handler: Set a flag that the trigger was just clicked and send the click on to the general click handler
//
pl_processTriggerClick = function () {
   // If acts like a dialog, clicking outside the layer does not get processed, like it was modal.
   if (pl_active_style_flags & PL_DIALOG_BEHAVIOR) {
      return false;
   }
   // Clicking on trigger when layer is showing gets rid if it, and does not pass click on to the body.
   if (pl_active_layer_obj != null) {
      // Unless the layer was shown using mouse over and out support, in which case click does not get rid of layer.
      if (!(pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
         hidePopupLayer();
      }
      return false;
   }
   return true;
}


//
// Event handler: Cancel popup layer on a mouse click anywhere except on the popup layer itself
//
pl_processBodyClick = function (event) {
   if (!pl_isButton1(event)) {
      return true;
   }
   if (pl_active_layer_obj != null) {
      if (event == null) {
         event = window.event
      }
      // Do not dismiss the popup layer if it was the popup layer that was clicked
      if (pl_clicked_layer) {
         pl_clicked_layer = false;
         return true;
      }
      // If acts like a dialog, clicking outside the layer does not get processed, like it was modal.
      if (pl_active_style_flags & PL_DIALOG_BEHAVIOR) {
         return false;
      }
      // If the mouse was released outside of the window, ignore this click event generated during a drag. Only IE will
      // handle this since Firefox does not capture mouse events outside the window.
      if ((event.clientX < 0) || (event.clientY < 0)) {
         return false;
      }
      if (document.documentElement) {
         if (document.documentElement.clientWidth && document.documentElement.clientHeight) {
            if ((event.clientX > document.documentElement.clientWidth) || (event.clientY > document.documentElement.clientHeight)) {
               return false;
            }
         }
      }
      // If the click came from the trigger, ignore it.
      if (event.target) {
         if (event.target.id == pl_active_trigger_id) {
            return false;
         }
      } else if (event.srcElement) {
         if (event.srcElement.id == pl_active_trigger_id) {
            return false;
         }
      }
      // Otherwise a click hides the layer unless the layer was shown using mouse over and out support.
      if (!(pl_active_style_flags & PL_MOUSE_OVER_SUPPORT)) {
         hidePopupLayer();
      }
   }
   pl_clicked_layer = false;
   return true;
}


//
// Event handler: Cancel popup layer on ESC key
//
pl_processBodyKey = function (event) {
   if (event == null) {
      event = window.event
   }
   if (event != null) {
      // Modal dialogs do not get dismissed with the ESC key.
      if ((event.keyCode == 27) && (!(pl_active_style_flags & PL_DIALOG_BEHAVIOR))) {
         hidePopupLayer();
      }
   }
   return true;
}


//
// Event handler: Reposition the popup layer on resize to stay aligned with target if it reflowed
//
pl_processResize = function () {
//   // Movable objects and modal objects do not get repositioned on window resize.
//   if ((pl_active_drag_obj != null) || (pl_active_style_flags & PL_DIALOG_BEHAVIOR)) {
   // Movable objects that have been moved by the user do not get repositioned on window resize.
   if ((pl_active_drag_obj != null) && pl_active_has_dragged) {
      return true;
   }
   // Reposition the layer relative to the target
   var show_fin;
   show_fin = pl_positionPopupLayer();
   if ((pl_active_style_flags & PL_DISPLAY_IN_BUBBLE) && (pl_active_mybubblefin != null)) {
      if (show_fin) {
         pl_active_mybubblefin.style.visibility = "visible";
         pl_active_mybubblefin.style.zIndex = 3;
      } else {
         pl_active_mybubblefin.style.visibility = "hidden";
         pl_active_mybubblefin.style.zIndex = -1;
      }
   }
   return true;
}


//
// Event handler: Cancel popup layer if another page loads in the same window
//
pl_processWindowUnload = function () {
   hidePopupLayer();
   return true;
}

}

if(!dojo._hasResource["hub.widgets.Dialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.widgets.Dialog"] = true;
dojo.provide("hub.widgets.Dialog");





dojo.declare(
	"hub.widgets.Dialog",
	[dijit.Dialog, dijit._Container],
  {
    templateString: null,
    templateString:"<div class=\"dijitDialog\">\r\n  <div dojoAttachPoint=\"titleBar\" class=\"dijitDialogTitleBar\" tabindex=\"0\" waiRole=\"dialog\">\r\n    <span dojoAttachPoint=\"titleNode\" class=\"dijitDialogTitle\">${title}</span>\r\n    <span dojoAttachPoint=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" dojoAttachEvent=\"onclick: hide\">\r\n      <span dojoAttachPoint=\"closeText\" class=\"closeText\">x</span>\r\n    </span>\r\n  </div>\r\n  <div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\" style=\"overflow:auto;\"></div>\r\n  <div dojoAttachPoint=\"actionNode\" class=\"dijitDialogPaneContent flowDialogActionBar\" style=\"padding:2px;\">\r\n    <div dojoAttachPoint=\"buttonNode\"></div>\r\n    <span dojoAttachPoint=\"resizeHandleNode\" class=\"dojoxFloatingResizeHandle\"></span>\r\n  </div>\r\n  <span dojoAttachPoint=\"tabEnd\" dojoAttachEvent=\"onfocus:_cycleFocus\" tabindex=\"0\"></span>\r\n</div>\r\n",
    actionNode: null,
    buttonNode: null,
    resizeHandleNode: null,
    operatorId: null,

    showUnderlay: false,
    hideTitleBar: false,
    hideActionBar: false,
    buttons: null,

    postCreate: function() {
      if (this.hideTitleBar) {
        this.titleBar.style.display = "none";
      }

      if (this.hideActionBar) {
        this.actionNode.style.display = "none";
      }

      //this.inherited("postCreate",arguments);
	  this.bgIframe = new dijit.BackgroundIframe(this.domNode);

      if (this.buttons) {
        for (var i = 0; i < this.buttons.length; i++) {
          var button = document.createElement("button")
          button.appendChild(document.createTextNode(this.buttons[i].label));

          if (this.buttons[i].onclick) {
            button.onclick = this.buttons[i].onclick;
          }

          this.buttons[i].node = this.buttonNode.appendChild(button);
        }
      } 
      else {
        // since IE doesn't do min-height
        this.actionNode.style.height = "13px";  // resizehandle image height size
      }
    },
    
    _setup: function() {
		if (this.titleBar) {
			//this._moveable = new dojo.dnd.move.parentConstrainedMoveable(this.domNode, {handle: this.titleBar, area: "margin", within: true});
			new dojo.dnd.Moveable(this.domNode, { handle: this.titleBar });
            dojo.connect(this._moveable, "onFirstMove", function(){this.handle.focus()});
        }
        
        if (this.showUnderlay) {
           this._underlay = new dijit.DialogUnderlay();
        }
        
        //

		var tmp = new dojox.layout.ResizeHandle({ 
				targetId: this.id,
				resizeAxis: "xy"}, this.resizeHandleNode);
    },
    
    layout: function() {
			// summary: position the Dialog and the underlay
			if(this.domNode.style.display == "block"){
			    if (this.showUnderlay) {
				   this._underlay.layout();
				}
				this._position();
			}
	},
		
    show: function() {
      var doLayout = false;

      if (!this._alreadyInitialized) {
        this._setup();
        this._alreadyInitialized = true;
        doLayout = true;
      }

      this.domNode.style.display = "block";

      dojo.addClass(this.domNode, "hubDialog");

      if (this.showUnderlay) {
        this._underlay.show();
      } 
      else {
      }

      if (doLayout) {
        this.layout();
      }

      this.open = true;
      this._savedFocus = dijit.getFocus(this);
      dijit.focus(this.titleBar);

      // TODO: doron - hack to make tabs work in dialogs for us
      this.resizeChildren();
    },

    hide: function() {
      if (!this._alreadyInitialized) {
        return;
      }

      this.domNode.style.display = "none";

      if (this.showUnderlay) {
        this._underlay.hide();
      }

      dijit.focus(this._savedFocus);
      this.open = false;
    },
    
    resize: function(data) {
      var w = data.w;
      var h = data.h;

      dojo.style(this.domNode, "width", w+"px"); 
      dojo.style(this.domNode, "height", h+"px");

      // now resize the dialog contents
      var titlebarBox = dojo.marginBox(this.titleBar);

      // TODO: 4 is fixed padding (both sides, so 8), needs to be calculated
      // IE works differently, so no padding is needed (else we get a visual drawing bug)
      h -= titlebarBox.h + dojo.marginBox(this.actionNode).h + (dojo.isIE ? 1 : 9);
      
      dojo.style(this.containerNode, "height", h+"px");
   
      // TODO: doron - hack to make tabs work in dialogs for us
      this.resizeChildren();
    },

    resizeChildren: function(data) {
      var children = this.getChildren();
      for (var i = 0; i < children.length; i++) {
        if (children[i].resize) {
          children[i].resize();
        }
      }
/*
      dojo.query(".hubResizable", this.domNode).forEach(function(node){
         dijit.byNode(node).resize();
      });
*/      
    }
    
    
  }
);

}

if(!dojo._hasResource["hub.main"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.main"] = true;
dojo.provide("hub.main");






/**
 * Initializes the Mashup Hub UI and client-side configuration variables. 
 * It's a class so we can use the hub package elsewhere without all this
 *  initialization code running on page load.
 */
dojo.declare("hub.Main", dijit._Widget, {
    // holds the initial full name of the logged in user, or blank if anonymous
    username: '',
    userfullname: '',
    isAdmin: '',
    isBanned: '',
    isSecure: false,
    securePort: '',
    baseurl: '',
    pingSessionHandle: null,
    
    postMixInProperties: function() {
        // create base URL first
        var split = dojo.baseUrl.split('/');
        // based on dojo-1.0.1 folder location only
        split = split.slice(0, -4);
		this.baseurl = split.join('/');		
        var durl = new dojo._Url(window.location, this.baseurl);
        this.baseurl = durl.uri;
        this.isSecure = (durl.scheme == 'https');
        if (this.isSecure) {
        	console.debug("isSecure == " + this.isSecure);
        	if (durl.port) {
        		this.securePort = ":" + durl.port; 
        	} 
        }
        if(this.baseurl.charAt(this.baseurl.length-1)=="/"){
        	this.baseurl = this.baseurl.substring(0, this.baseurl.length-1);
        }
		 
        // get username  TODO !!!
		_this = this;
		
		dojo.xhrGet({
			url: this.baseurl + "/ui/currentuser",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				_this.username = response.username;
				_this.userfullname = response.userfullname;
				_this.isAdmin = ("true" == response.isAdmin);
				_this.isBanned= ("true" == response.isBanned);
				var configured = ("true" == response.configured);
				if (!configured) {
					// come away from this layout.html to application root
					console.info("Application is not configured!");
					var p = window.location.pathname;
					var cut = p.indexOf("client/layout/layout.");
					if (cut >= 0) {
						p = p.substring(0, cut);
						console.info("Go to application root: " + p);
						if (dojo.isIE) {
							window.setTimeout('window.location.href = "' + p + '";', 50);
						}
						else {
							window.location.href = p;
						}
						return;
					}
				}
			}
		});
		
        // global configuration
        hub.config = {base_url: this.baseurl,
                      username: this.username,
                      userfullname: this.userfullname,
                      isAdmin: this.isAdmin,
                      isBanned: this.isBanned,
                      isSecure: this.isSecure,
                      securePort: this.securePort,
                      max_label_length: 15};
		
		// load paltette url
		dojo.xhrGet({
			url: this.baseurl + "/ui/paletteurl",
			handleAs: 'json',
			sync: true,
			load: function(response, args) {
				hub.config.palette_url = response.palette_url;
			}
		});
                      
        hub.pingSession = function() {
        /*
			dojo.xhrGet({
				url: hub.config.base_url + "/ui/pingSession?dt=" + new Date().getTime(),
				handleAs: 'json',
				sync: false,
				load: function(response, args) {
					// we just ignore it
					console.debug("session pinged");
				}
			});
		*/	
        };

        // start listening for events that affect configuration
        dojo.subscribe('hub.login', this, '_onLogin');
        dojo.subscribe('hub.logout', this, '_onLogout');

        // initialize popup library
        
        set_pl_IMAGE_PATH(hub.config.base_url+'/images/popup/');
        pl_preloadPopupLayerImages();

        // create important managers
        
        hub.urls = new hub.managers.Urls(hub.config.base_url);
    },

    startup: function() {
        // show the home page
        var labels = dojo.i18n.getLocalization('hub', 'labels');
        hub.showPage(labels.HOME_TAB, 'Home');
        
        this.pingSessionHandle = null;
        /*
        if (this.pingSessionHandle != null) {
            // startup() normally should never called twice,
            // but to have it 100% safe
        	window.clearInterval(pingSessionHandle);
        	this.pingSessionHandle = null;
        }
        this.pingSessionHandle = window.setInterval(hub.pingSession, 15 * 60 * 1000); // 15 minutes are adequate
        */
    },

    _onLogin: function(username) {
        hub.config.username = username;
    },

    _onLogout: function() {
        hub.config.username = '';
    }
});

/**
 * General exception class for hub. Contains a message.
 */
dojo.declare("hub.Exception", null, {
    message: '',
    constructor: function(msg) {
        this.message = msg ? msg : "";
    }
});


/**
 * Notifies the user of a message, either by writing it into the given element.
 * or by popping up a JS alert box.
 *
 * @param text: Text to display
 * @param element: Element in which to insert the text, or null to use a popup
 * @param cls: CSS classname to use
 */
hub.notify = function(text, element, class_name) {
    if(element) {
        if(!class_name) {
            class_name = 'TOM_editor_error';
        }
        element.className = class_name;
        element.innerHTML = text;
    } else {
        alert(text);
    }
}

/**
 * Ellipsizes a label to the given maximum number of characters, including the
 * ellipsis.
 *
 * @param label
 * @param max
 */
hub.shortenLabel = function(label) {
    // truncate name chars first, so comparison by label works
    if(label.length > hub.config.max_label_length) {
    	label = hub.srahclaicepslmth(label);
        return hub.htmlspecialchars(label.substr(0 , hub.config.max_label_length-3)) + '...';
    } else {
        return label;
    }
}

/**
 * Mimick's PHP's htmlspecialchars for &, <, and >. Use to encode text before
 * output from JS to avoid security issues.
 *
 * @param text String to encode
 * @return Encoded text
 */
hub.htmlspecialchars = function(text) {
    text = text.replace(/&/g,"&amp;");
    text = text.replace(/</g,"&lt;");
    text = text.replace(/>/g,"&gt;");
    text = text.replace(/\'/g,"&#39;");
    text = text.replace(/\"/g,'&quot;');
    return text;
}

hub.srahclaicepslmth = function(text) {
    text = text.replace(/&amp;/g,"&");
    text = text.replace(/&lt;/g,"<");
    text = text.replace(/&gt;/g,">");
    text = text.replace(/&#039;/g,"'");
    text = text.replace(/&quot;/g,'"');
    return text;
}

/**
 * Replace ' -> \' and " -> \". Use to encode text before
 * output as JS strings.
 *
 * @param text String to encode
 * @return Encoded text
 */
hub.escapeQuotes = function(text) {
    text = text.replace(/\'/g,"\\\'");
    text = text.replace(/\"/g,'\\\"');
    return text;
}
hub.escapeQuotes3 = function(text) {
    text = text.replace(/\'/g,"&#92;&#39;");
    text = text.replace(/\"/g,'&#92;&#34;');
    return text;
}
hub.escapeQuotes2 = function(text) {
    text = text.replace(/&#39;/g,"&#92;&#39;");
    text = text.replace(/&quot;/g,'&#92;&#34;');
    return text;
}

/**
 * Evaluates JavaScript and Dojo content in the given DOM node.
 * 
 * @param node The DOM node to evaluate
 * @param js_eval True to eval all script tags
 * @param dojo_prase True to parse with Dojo
 */
hub.evaluate = function(node, js_eval, dojo_parse) {
    var html = node.innerHTML;
    
    if(js_eval) {
        // execute any scripts
        var expr = new RegExp('(?:<!--(?:\n|\r|\s|.)*?-->)|(?:<SCRIPT.*?>)((\n|\r|\s|.)*?)(?:<\/SCRIPT>)', 'ig');
        for (var results = expr.exec(html); results != null; results = expr.exec(html)) {
            if (!results[1]) {
                continue;
            }
            // IE doesn't allow script elements to have childnodes,
            // so another way is to use the non-standard .text for
            // IE:  script.text = results[1]
            if (dojo.isIE > 0) {
                window.execScript(results[1]);
            } else {
                eval(results[1]);
            }
        }
    }
    
    if(dojo_parse) {
        // tell dojo to parse
        dojo.parser.parse(node); 
    }
}

/**
 * Renders a standard refreshing message either inline or as a block element.
 * 
 * @param node: Node to render into
 * @param block: True to render as a <p>, false as a <span>
 * @param className: CSS class name to attach to the <p> or <span>
 */
hub.busyMessage = function(node, block, class_name) {
    var display;
    // decide how to render
    if(block) {
        display = "block";
    } else {
        display = "inline";
    }
    if(!class_name) {
        // make sure we have an empty string
        class_name = '';
    }
    var msgs = dojo.i18n.getLocalization('hub', 'messages');
    var ml = "<span class='${0}' style='display: ${1}'><img src='${2}' /> ${3}</span>";
    var img = hub.urls.getImage('spinner.gif');
    var str = dojo.string.substitute(ml, [class_name, display, img, 
                                                     msgs.STATUS_LOADING]);
    if(node) {
        node.innerHTML = str;
    } else {
        return str;
    }
}

/**
 * Builds a link representation of a call to hub.showPage.
 *
 * First three params match those of hub.showPage.
 * 
 * @param shorten: Truncate the displayed text?
 * @param text: Text to display, defaults to title
 * @param popup: Text to popup, defaults to title
 */
hub.buildPageLink = function(title, template, args, shorten, text, popup) {
    if(!text) {
        // use title if no other text specified
        text = title;
    }
    if(shorten) {
        // truncate to configured length with ellipsis
        text = hub.shortenLabel(text);
    }
     if(popup == null) {
        // use the full title as the popup
        popup = title;
    } else if(shorten) {
        // use the full title plus the provided popup
        popup = title + " (" + popup + ")";
    }
    // always escape strings, even to prevent ' and " problems
//    title = hub.escapeQuotes2(title);
//    text = hub.escapeQuotes2(text);
//    popup = hub.escapeQuotes2(popup);
    result = "<a href='javascript:hub.showPage(\"";
    result += hub.escapeQuotes2(title);
    result += "\",\"";
    result += template;
    result += "\",";
    result += dojo.toJson(args);
    result += ")\' title='";
    result += hub.escapeQuotes2(popup);
    result += "'>";
    result += text;
    result += "</a>";
    return result;
//     var slots = "<a href='javascript:hub.showPage(\"${title}\",\"${template}\",${args})\' title='${popup}'>${text}</a>";
//    return dojo.string.substitute(slots, {title: title,
//                                          text: text,
//                                          popup: popup,
//                                          template: template,
//                                          args: dojo.toJson(args)});  
}

/**
 * Posts a request to show a client side templated page. The page will render
 * if an observer is available that can render it, else the request is safely 
 * ignored.
 * 
 * @param title: Title of the page
 * @param template: Name of the template to render, rooted at hub.pages
 * @param args: Object of args to pass to the listener
 */
hub.showPage = function(title, template, args) {
    dojo.publish('hub.show-template-page', [title, template, args]);
}

/**
 * Posts a request to show the result of visiting a URL. The URL will render
 * if an observer is available that can render it, else the request is safely 
 * ignored.
 * 
 * @param title: Title of the page
 * @param url: Location of the page to render
 * @param args: Object of options to pass to the listener
 */
hub.showUrl = function(title, url, args) {
    dojo.publish('hub.show-url-page', [title, url, args]);
}

/**
 * Posts a request to show the first screen of a plug-in editor UI. The plug-in
 * will render if an observer is available that can render it, else the request
 * is safely ignored.
 * 
 * @param title: Title of the page
 * @param plugin_id: Plug-in editor to show
 * @param object_id: Object to edit in the plug-in. If null, creates a new 
 * object.
 * @param args: Object of options to pass to the listener
 */
hub.showPlugin = function(title, plugin_id, object_id, args) {
    dojo.publish('hub.show-plugin-page', [title, plugin_id, object_id, args]);
}

/**
 * Posts a request to rename an existing page. The rename will happen if the 
 * given node exists in a page and an observer is available that can render it,
 * else the request is safely ignored.
 * 
 * @param node: Any node in the page
 * @param title: New title of the page
 * @param args: Object of options to pass to the listener
 */
hub.renamePage = function(node, title,  args) {
    dojo.publish('hub.rename-page', [node, title, args]);
}

/**
 * Posts a request to mark or unmark a page as busy and/or to lock it from all
 * interaction with the user. The page will indicate busy or lock if an 
 * observer is available that can handle the request, else the request is 
 * safely ignored.
 * 
 * @param node: Any node in the page
 * @param busy: True to mark busy, false to unmark
 * @param locked: True to lock, false to unlock
 */
hub.busyPage = function(node, busy, locked) {
    dojo.publish('hub.busy-page', [node, busy, locked]);
}

/**
 * Posts a request to close a page. The page will close if there an 
 * observer is available that can handle the request, else the request is 
 * safely ignored. The observer will indicate whether the close event was
 * successful or if something cancelled it.
 * 
 * @param node: Any node in the page
 * @return dojo.Deferred: Result is true if closed, false if aborted
 */
hub.closePage = function(node) {
    var d = new dojo.Deferred();
    if(node){
    	dojo.publish('hub.close-page', [node, d]);
    }
    // answer will come back later
    return d;
}

hub.closeAllPages = function() {
    var d = new dojo.Deferred();
    dojo.publish('hub.close-page', [null, d]);
    // answer will come back later
    return d;
}

hub.closeCurrentPage = function() {
    dojo.publish('hub.close-current-page', []);
}

hub.getCurrentTab = function(){
	var tab_container = dijit.byId('hub.tabs');
	return tab_container.selectedChildWidget;
}

/**
 * Posts a request to check if a page is active. The request will be answered
 * if an observer is available to handle it, else the request is 
 * safely ignored. The observer will indicate the result.
 * 
 * @param node: Any node in the page
 * @return dojo.Deferred: Result is true if active, false if not
 */
hub.isPageActive = function(node) {
    var d = new dojo.Deferred();
    dojo.publish('hub.is-page-active', [node, d]);
    // answer will come back later
    return d;
}

/**
 * Finds all child layout containers and resizes them (also resizes passed dijit).
 * @param d: dijit to find layout containers in and resize them
 */
hub.resizeLayoutContainers = function(d) {
	d.resize();
	var layoutContainers = dojo.query(".dijitLayoutContainer", d.containerNode || d.domNode).map(dijit.byNode);
	dojo.forEach(layoutContainers, function(child){hub.resizeLayoutContainers(child); });
}

hub.showHelp = function( title, url ) {
    
    
    if (url.indexOf("http") == 0) {
    } 
    else {
        url = hub.config.base_url + '/' + url;
    }
    
    //<%= me %>.workflow.test(); 
    //var node = dojo.doc.createElement('div');
    var node = dojo.body().appendChild(document.createElement("div"));
    var dialog = new hub.widgets.Dialog({title: title, showUnderlay: false, resizable: true, closeable: true}, node);
    
    //dialog.setContent("<img src='https://localhost:8443/mashuphub/client/plugins/com.ibm.mashuphub.plugin.feed/icons/btn32_atom1.gif' />");
    dialog.resize({w: 950, h: 450});
    dialog.show();
    
    // var test_url = 'https://localhost:8443/mashuphub/client/doc/userhelp/feedhelp/help_feeds_register.htm';
    
    dojo.style(dialog.containerNode, "overflow", "hidden");
    var iframe = document.createElement("iframe");
    iframe.style.width = "100%";
    iframe.style.height = "100%";
    iframe.style.borderWidth = "0";
    iframe.setAttribute("src", url);
    dialog.containerNode.appendChild(iframe);
}

}

if(!dojo._hasResource['hub.managers.Urls']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['hub.managers.Urls'] = true;
dojo.provide('hub.managers.Urls');


/**
 * Manages the cache of all RESTful server URLs required by the core UI.
 * Currently, all paths are hardcoded off of a base URL calcuated when the
 * initial page renders. Later, this manager could grow to receive the entire
 * REST API in some format from the server (e.g., from the APP service
 * document).
 */
dojo.declare('hub.managers.Urls', null, {
    constructor: function(base_url) {
        this.base_url = base_url;
        this.server_url = base_url.substring(0, base_url.lastIndexOf('/client'));
    },

    /////////////////////////////////////
    // URLs for template page data
    ////////////////////////////////////

    /**
     * URL to object containing all data required for rendering the home page.
     */
    getHomePage: function() {
        return this.base_url + '/ui/home';
    },

    /**
     * URL to object containing search type info required for rendering the search box.
     */
    getSearchTypes: function() {
        return this.base_url + '/ui/searchtypes';
    },

    /**
     * URL to object containing all entry details.
     */
    getDetailsPage: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: getDetailsPage requires object_id');
        }
        return this.base_url + '/ui/details/entryid/'+object_id;
    },

    /**
     * URL to object containing information about the logged in user.
     */
    getProfilePage: function() {
        return base_url + '/ui/profile';
    },

    /**
     * URL to object containing all details about feed parameters.
     */
    getParametersPage: function(object_id, plugin_id) {
        if(object_id == null) {
            throw new Error('Urls: getParametersPage requires object_id');
        }
        return this.base_url + '/plugin/params/entryid/'+object_id+'/pluginid/'+plugin_id;
    },

    /**
     * URL to object containing user permissions.
     */
    getUpdateAdminTables: function() {
        return this.base_url + '/ui/update/name/admintables';
    },

    /**
     * URL to object containing user permissions.
     */
    getUpdateAdminProxy: function() {
        return this.base_url + '/ui/update/name/proxy';
    },

    /**
     * URL to object containing user permissions.
     */
    getAdminPermissionsPage: function() {
        return this.base_url + '/ui/editor/name/adminpermissions';
    },

    /**
     * URL to object containing list of items that can be deleted.
     */
    getAdminDeletePage: function() {
        return this.base_url + '/ui/editor/name/adminobjects';
    },

    /**
     * URL to object containing banned users.
     */
    getAdminBanPage: function() {
        return this.base_url + '/ui/editor/name/adminusers';
    },

    /**
     * URL to object containing email configuration.
     */
    getAdminEmailPage: function() {
        return this.base_url + '/ui/editor/name/adminreports';
    },

    /**
     * URL to object containing email configuration.
     */
    getAdminCategoryPage: function() {
        return this.base_url + '/ui/editor/name/admincategory';
    },

    /**
     * URL to object containing tracking configuration.
     */
    getAdminEditTrackingPage: function() {
        return this.base_url + '/ui/editor/name/admintracking';
    },

    /**
     * URL to object containing tracked feeds.
     */
    getAdminTrackingData: function() {
        return this.base_url + '/ui/editor/name/admintrackeddata';
    },

    /**
     * URL to object containing tracked feeds.
     */
    getAdminProxyPage: function() {
        return this.base_url + '/ui/editor/name/adminproxy';
    },

    /**
     * URL to object containing admintdi.
     */
    getAdminTDI: function() {
        return this.base_url + '/ui/editor/name/admintdi';
    },

    /**
     * URL to object containing tracked feeds.
     */
    getSettingsPage: function() {
        // @todo
    },

    /////////////////////////////////////
    // URLs for widget data / actions
    ////////////////////////////////////

    /**
     * URL to object or XML feed listing catalog entries for the given search
     * query and term.
     */
    getCatalog: function(collection, term, searchType, sortBy, sortOrder) {
		var url = "";
		var map = {
		};
		var c = collection;
		if (c.indexOf("ui.") != 0) {
			c = "ui." + c; 
		}
		map["collection"] = c;
		if (searchType != null) {
			map["searchType"] = searchType;
		} else if (term != null) {
			map["searchType"] = ""; // Just ensure it is set
		}
		if (term != null) {
			map["searchTerms"] = term;
		}
		else if (searchType != null){
			map["searchTerms"] = ""; // Just ensure it is set
		}
		if (sortBy != null) {
			map["sortBy"] = sortBy;
		}
		if (sortOrder != null) {
			map["sortOrder"] = sortOrder;
		}
		url = this.base_url + "/.." + "/atom"; // TODO: make it more robust
        url = url + '?' + dojo.objectToQuery(map);
        return url;
    },

    /**
     * URL to array of all category names if object_id is null or all names
     * categories assigned to the object_id if not null.
     */
    getCategories: function(object_id) {
        if(object_id == null) {
            return this.base_url + '/ui/categories';
        } else {
            return this.base_url + '/ui/categories/entryid/'+object_id;
        }
    },

    /**
     * URL to array of all category names if object_id is null or all names
     * categories assigned to the object_id if not null.
     */
    getCategoriesList: function(object_id) {
        if(object_id == null) {
            return this.base_url + '/ui/categorieslist';
        } else {
            return this.base_url + '/ui/categorieslist/entryid/'+object_id;
        }
    },

    /**
     * URL to array of all association names if object_id is null or all names
     * associations assigned to the object_id if not null.
     */
    getAssociationsList: function(object_id) {
        if(object_id == null) {
            return this.base_url + '/ui/associationslist';
        } else {
            return this.base_url + '/ui/associationslist/entryid/'+object_id;
        }
    },

    /**
     * URL to array of ids of all association assigned to the object_id.
     */
    getAssociationsIdsList: function(object_id) {
        return this.base_url + '/ui/associationsidslist/entryid/'+object_id;
    },

    /**
     * URL to array of all comments for the object_id. Array items are objects
     * having 'commenttitle', 'commenttext', 'creatorid', and 'timestamp'.
     */
    getComments: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: getComments requires object_id');
        }
        return this.base_url + '/ui/comments/entryid/'+object_id;
    },

    /**
     * URL to object containing average rating 'value' and voter 'count' for the
     * object_id.
     */
    getRatings: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: getRatings requires object_id');
        }
        return this.base_url + '/ui/ratings/entryid/'+object_id;
    },

    /**
     * URL to object containing average rating 'value' and voter 'count' for the
     * object_id.
     */
    getTags: function(objectid) {
        if(objectid == null) {
            return this.base_url + '/ui/tags';
        } else {
            return this.base_url + '/ui/tags/entryid/'+objectid;
        }
    },

    /**
     * URL to array of objects with username as key and fullname as value.
     */
    getUserSearch: function(user) {
	    user = user.replace(/\%/g,'%25');
        return this.base_url + '/ui/searchusers/user/' + user;
    },

    /**
     * URL to array of objects with groupname as both key and value.
     */
    getGroupSearch: function(group) {
	    group = group.replace(/\%/g,'%25');
        return this.base_url + '/ui/searchgroups/group/' + group;
    },

    /////////////////////////////////////
    // URLs for submitting data back to server
    ////////////////////////////////////

    /**
     * URL to post a report about abuse on an object_id.
     */
    postAbuse: function(object_id) {
        // @todo
    },

    /**
     * URL to post a deletion request for an object_id.
     */
    postDelete: function(object_id, plugin_id) {
        if(object_id == null) {
            throw new Error('Urls: postDelete requires object_id');
        }
        return this.base_url+'/plugin/delete/entryid/'+object_id+'/pluginid/'+plugin_id;
    },

    /**
     * URL to post a cancel request for an object_id.
     */
    postCancel: function(object_id, plugin_id) {
        if(object_id == null) {
            throw new Error('Urls: postCancel requires object_id');
        }
        return this.base_url+'/plugin/cancel/entryid/'+object_id+'/pluginid/'+plugin_id;
    },

    /**
     * URL to post tags for an object_id with 'tags' as key and a comma
     * separated string of tag names as value.
     */
    postTags: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postTags requires object_id');
        }
        return this.base_url+'/ui/update/name/tags/entryid/'+object_id;
    },

    /**
     * URL to post a comment for an object_id.
     */
    postComment: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postComment requires object_id');
        }
        return this.base_url+'/ui/update/name/comment/entryid/'+object_id;
    },

    /**
     * URL to post a rating for an object_id.
     */
    postRating: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postRating requires object_id');
        }
        return this.base_url+'/ui/update/name/rating/entryid/'+object_id;
    },

    /**
     * URL to post a bunch of metadata for an object_id.
     */
    postMetadata: function(object_id) {
        if(object_id == null) {
            throw new Error('Urls: postMetadata requires object_id');
        }
        // @todo: implement
    },

    /**
     * URL to post profile updates for an object_id.
     */
    postProfile: function() {
        return this.base_url + '/ui/update/name/profile';
    },

    /**
     * URL to post a reset of the entire Mashup Hub database.
     */
    postAdminInit: function() {
        return this.base_url + '/ui/update/name/admintables';
    },

    /**
     * URL to post multiple object deletions.
     */
    postAdminDelete: function() {
        return this.base_url + '/ui/update/name/adminobjects';
    },

    /**
     * URL to post changes to categories.
     */
    postAdminCategories: function() {
        return this.base_url + '/ui/update/name/admincategory';
    },

    /**
     * URL to post changes to user permissions.
     */
    postAdminPermissions: function() {
        return this.base_url + '/ui/update/name/adminpermissions';
    },

    /**
     * URL to post changes to the banned users.
     */
    postAdminBan: function() {
        return this.base_url + '/ui/update/name/adminusers';
    },

    /**
     * URL to post changes to the email notification config
     */
    postAdminEmail: function() {
        return this.base_url + '/ui/update/name/adminreports';
    },

    /**
     * URL to post changes to the email notification config
     */
    postReportEmail: function() {
        return this.base_url + '/ui/update/name/report';
    },

    /**
     * URL to post tracking status configuration.
     */
    postAdminTrackingStatus: function() {
        return this.base_url + '/ui/update/name/admintrackingglobal';
    },

    /**
     * URL to post tracking status configuration.
     */
    postAdminTrackingSettings: function() {
        return this.base_url + '/ui/update/name/trackingedittype';
    },
    
    /**
     * URL to clear tracking data.
     */
    postAdminTrackingClear: function() {
        return this.base_url + '/ui/update/name/trackingcleardata';
    },

    /**
     * URL to post tracking status configuration.
     */
    postAdminTrackingPermissions: function() {
        return this.base_url + '/ui/update/name/admintrackingobject';
    },

    /**
     * URL to post tdi configuration.
     */
    postAdminTDI: function() {
        return this.base_url + '/ui/update/name/admintdi';
    },
    
    /**
     * URL to post connection info for database migration.
     */
    postAdminMigrateDB: function() {
        return this.base_url + '/ui/update/name/adminmigratedb';
    },

    /**
     * URL to post multiple object deletions.
     */
    postAdminProxy: function() {
        return this.base_url + '/ui/update/name/adminproxy';
    },


    /**
     * URL to post a login request.
     */
    postLogin: function() {
        return this.base_url + '/ui/login';
    },

    /**
     * URL to post a logout request.
     */
    postLogout: function() {
        return this.base_url + '/ui/logout';
    },

    /**
     * URL to post an account registration request.

     */
    postRegister: function() {
        return this.base_url + '/ui/register';
    },

    /**
     * URL to post data for verification before continuing in the installer
     * wizard.
     *
     * @param action: Action name corresponding with the page to verify
     */
    postInstall: function(action) {
        return this.base_url + '/installer/' + action;
    },

    /////////////////////////////////////
    // URLs for static resources
    ////////////////////////////////////

    /**
     * URL to the base images directory.
     */
    getImage: function(filename) {
        return this.base_url + '/images/' + filename;
    },

    /**
     * URL to the icons directory under the images folder.
     *
     */
    getIcon: function(filename) {
        return this.base_url + '/images/icons/' + filename;
    },

    //////////////////////////////////////
    // FROM DETAILS PAGE
    //////////////////////////////////////

    postFavorites: function(objectid) {
        if(objectid == null) {
            throw new Error('Urls: postFavorites requires objectid');
        }
        return this.base_url + '/ui/update/name/stuff/entryid/'+objectid;
    },
    
    getPluginGenerate: function(objectid, pluginid) {
        if(objectid == null) {
        	return "#";
//            throw new Error('Urls: getPluginGenerate requires objectid');
        }       
        if(pluginid == null) {
        	return "#";
//            throw new Error('Urls: getPluginGenerate requires pluginid');
        }       
        return this.base_url + '/plugin/generate/entryid/'+objectid+'/pluginid/'+pluginid;
    },
    
    getPluginParams: function(objectid, pluginid) {
        if(objectid == null) {
        	return "#";
//            throw new Error('Urls: getPluginParams requires objectid');
        }       
        if(pluginid == null) {
        	return "#";
//            throw new Error('Urls: getPluginParams requires pluginid');
        }       
        return this.base_url + '/plugin/params/entryid/'+objectid+'/pluginid/'+pluginid;
    },

    getAjaxUrl: function(plugin_id,entry_id,method){
    	return this.base_url + '/plugin/ajax/entryid/'+entry_id+'/pluginid/'+plugin_id+ '/method/'+method+'/';
    },
    
    secureUrl: function(ourl) {
    	if (hub.config.isSecure) {
	        var purl = new dojo._Url(ourl);
	        if ((purl.scheme == 'http') && purl.authority) {
	        	// we need to securize it:
	        	var result = "https://" + purl.host + hub.config.securePort + 
	        	(purl.path ? purl.path : "")  + 
	        	(purl.query ? ("?" + purl.query) : "") +
	        	(purl.fragment ? ("#" + purl.fragment) : "");
	        	return result;   
	        }
	    }
    	return ourl;
    }
});

}

if(!dojo._hasResource["dojox.data.XmlStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.XmlStore"] = true;
dojo.provide("dojox.data.XmlStore");
dojo.provide("dojox.data.XmlItem");





dojo.declare("dojox.data.XmlStore", null, {
	//	summary:
	//		A data store for XML based services or documents
	//	description:
	//		A data store for XML based services or documents
	
	constructor: function(/* object */ args) {
		//	summary:
		//		Constructor for the XML store.  
		//	args:
		//		An anonymous object to initialize properties.  It expects the following values:
		//		url:		The url to a service or an XML document that represents the store
		//		rootItem:	A tag name for root items
		//		keyAttribute:	An attribute name for a key or an indentify
		// 		attributeMap:   An anonymous object contains properties for attribute mapping,
		//						{"tag_name.item_attribute_name": "@xml_attribute_name", ...}
		//		sendQuery:		A boolean indicate to add a query string to the service URL 
		console.log("XmlStore()");
		if(args){
			this.url = args.url;
			this.rootItem = (args.rootItem || args.rootitem || this.rootItem);
			this.keyAttribute = (args.keyAttribute || args.keyattribute || this.keyAttribute);
			this._attributeMap = (args.attributeMap || args.attributemap);
			this.label = args.label || this.label;
			this.sendQuery = (args.sendQuery || args.sendquery || this.sendQuery);
		}
		this._newItems = [];
		this._deletedItems = [];
		this._modifiedItems = [];
	},

	//Values that may be set by the parser.  
	//Ergo, have to be instantiated to something
	//So the parser knows how to set them.
	url: "",

	rootItem: "",

	keyAttribute: "",

	label: "",

	sendQuery: false,

/* dojo.data.api.Read */

	getValue: function(/* item */ item, /* attribute || attribute-name-string */ attribute, /* value? */ defaultValue){
		//	summary:
		//		Return an attribute value
		//	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		If 'attribute' specifies "tagName", the tag name of the element is
		//		returned.
		//		If 'attribute' specifies "childNodes", the first element child is
		//		returned.
		//		If 'attribute' specifies "text()", the value of the first text
		//		child is returned.
		//		For generic attributes, if '_attributeMap' is specified,
		//		an actual attribute name is looked up with the tag name of
		//		the element and 'attribute' (concatenated with '.').
		//		Then, if 'attribute' starts with "@", the value of the XML
		//		attribute is returned.
		//		Otherwise, the first child element of the tag name specified with
		//		'attribute' is returned.
		//	item:
		//		An XML element that holds the attribute
		//	attribute:
		//		A tag name of a child element, An XML attribute name or one of
		// 		special names
		//	defaultValue:
		//		A default value
		//	returns:
		//		An attribute value found, otherwise 'defaultValue'
		var element = item.element;
		if(attribute === "tagName"){
			return element.nodeName;
		}else if (attribute === "childNodes"){
			for (var i = 0; i < element.childNodes.length; i++) {
				var node = element.childNodes[i];
				if (node.nodeType === 1 /*ELEMENT_NODE*/) {
					return this._getItem(node); //object
				}
			}
			return defaultValue;
		}else if(attribute === "text()"){
			for(var i = 0; i < element.childNodes.length; i++){
				var node = element.childNodes[i];
				if(node.nodeType === 3 /*TEXT_NODE*/ ||
					node.nodeType === 4 /*CDATA_SECTION_NODE*/){
					return node.nodeValue; //string
				}
			}
			return defaultValue;
		}else{
			attribute = this._getAttribute(element.nodeName, attribute);
			if(attribute.charAt(0) === '@'){
				var name = attribute.substring(1);
				var value = element.getAttribute(name);
				return (value !== undefined) ? value : defaultValue; //object
			}else{
				for(var i = 0; i < element.childNodes.length; i++){
					var node = element.childNodes[i];
					if(	node.nodeType === 1 /*ELEMENT_NODE*/ &&
						node.nodeName === attribute){
						return this._getItem(node); //object
					}
				}
				return defaultValue; //object
			}
		}
	},

	getValues: function(/* item */ item, /* attribute || attribute-name-string */ attribute){
		//	summary:
		//		Return an array of attribute values
		//	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		If 'attribute' specifies "tagName", the tag name of the element is
		//		returned.
		//		If 'attribute' specifies "childNodes", child elements are returned.
		//		If 'attribute' specifies "text()", the values of child text nodes
		//		are returned.
		//		For generic attributes, if '_attributeMap' is specified,
		//		an actual attribute name is looked up with the tag name of
		//		the element and 'attribute' (concatenated with '.').
		//		Then, if 'attribute' starts with "@", the value of the XML
		//		attribute is returned.
		//		Otherwise, child elements of the tag name specified with
		//		'attribute' are returned.
		//	item:
		//		An XML element that holds the attribute
		//	attribute:
		//		A tag name of child elements, An XML attribute name or one of
		//		special names
		//	returns:
		//		An array of attribute values found, otherwise an empty array
		var element = item.element;
		if(attribute === "tagName"){
			return [element.nodeName];
		}else if(attribute === "childNodes"){
			var values = [];
			for(var i = 0; i < element.childNodes.length; i++){
				var node = element.childNodes[i];
				if(node.nodeType === 1 /*ELEMENT_NODE*/){
					values.push(this._getItem(node));
				}
			}
			return values; //array
		}else if(attribute === "text()"){
			var values = [];
			for(var i = 0; i < element.childNodes.length; i++){
				var node = childNodes[i];
				if(node.nodeType === 3){
					values.push(node.nodeValue);
				}
			}
			return values; //array
		}else{
			attribute = this._getAttribute(element.nodeName, attribute);
			if(attribute.charAt(0) === '@'){
				var name = attribute.substring(1);
				var value = element.getAttribute(name);
				return (value !== undefined) ? [value] : []; //array
			}else{
				var values = [];
				for(var i = 0; i < element.childNodes.length; i++){
					var node = element.childNodes[i];
					if(	node.nodeType === 1 /*ELEMENT_NODE*/ &&
						node.nodeName === attribute){
						values.push(this._getItem(node));
					}
				}
				return values; //array
			}
		}
	},

	getAttributes: function(/* item */ item) {
		//	summary:
		//		Return an array of attribute names
		// 	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		tag names of child elements and XML attribute names of attributes
		//		specified to the element are returned along with special attribute
		//		names applicable to the element including "tagName", "childNodes"
		//		if the element has child elements, "text()" if the element has
		//		child text nodes, and attribute names in '_attributeMap' that match
		//		the tag name of the element.
		//	item:
		//		An XML element
		//	returns:
		//		An array of attributes found
		var element = item.element;
		var attributes = [];
		attributes.push("tagName");
		if(element.childNodes.length > 0){
			var names = {};
			var childNodes = true;
			var text = false;
			for(var i = 0; i < element.childNodes.length; i++){
				var node = element.childNodes[i];
				if (node.nodeType === 1 /*ELEMENT_NODE*/) {
					var name = node.nodeName;
					if(!names[name]){
						attributes.push(name);
						names[name] = name;
					}
					childNodes = true;
				}else if(node.nodeType === 3){
					text = true;
				}
			}
			if(childNodes){
				attributes.push("childNodes");
			}
			if(text){
				attributes.push("text()");
			}
		}
		for(var i = 0; i < element.attributes.length; i++){
			attributes.push("@" + element.attributes[i].nodeName);
		}
		if(this._attributeMap){
			for (var key in this._attributeMap){
				var i = key.indexOf('.');
				if(i > 0){
					var tagName = key.substring(0, i);
					if (tagName === element.nodeName){
						attributes.push(key.substring(i + 1));
					}
				}else{ // global attribute
					attributes.push(key);
				}
			}
		}
		return attributes; //array
	},

	hasAttribute: function(/* item */ item, /* attribute || attribute-name-string */ attribute){
		//	summary:
		//		Check whether an element has the attribute
		//	item:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//	attribute:
		//		A tag name of a child element, An XML attribute name or one of
		//		special names
		//	returns:
		//		True if the element has the attribute, otherwise false
		return (this.getValue(item, attribute) !== undefined); //boolean
	},

	containsValue: function(/* item */ item, /* attribute || attribute-name-string */ attribute, /* anything */ value){
		//	summary:
		//		Check whether the attribute values contain the value
		//	item:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//	attribute:
		//		A tag name of a child element, An XML attribute name or one of
		//		special names
		//	returns:
		//		True if the attribute values contain the value, otherwise false
		var values = this.getValues(item, attribute);
		for(var i = 0; i < values.length; i++){
			if((typeof value === "string")){
				if(values[i].toString && values[i].toString() === value){
					return true;
				}
			}else if (values[i] === value){
				return true; //boolean
			}
		}
		return false;//boolean
	},

	isItem: function(/* anything */ something){
		//	summary:
		//		Check whether the object is an item (XML element)
		//	item:
		//		An object to check
		// 	returns:
		//		True if the object is an XML element, otherwise false
		if(something && something.element && something.store && something.store === this){
			return true; //boolean
		}
		return false; //boolran
	},

	isItemLoaded: function(/* anything */ something){
		//	summary:
		//		Check whether the object is an item (XML element) and loaded
		//	item:
		//		An object to check
		//	returns:
		//		True if the object is an XML element, otherwise false
		return this.isItem(something); //boolean
	},

	loadItem: function(/* object */ keywordArgs){
		//	summary:
		//		Load an item (XML element)
		//	keywordArgs:
		//		object containing the args for loadItem.  See dojo.data.api.Read.loadItem()
	},

	getFeatures: function() {
		//	summary:
		//		Return supported data APIs
		//	returns:
		//		"dojo.data.api.Read" and "dojo.data.api.Write"
		var features = {
			"dojo.data.api.Read": true,
			"dojo.data.api.Write": true
		};
		return features; //array
	},

	getLabel: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabel()
		if((this.label !== "") && this.isItem(item)){
			var label = this.getValue(item,this.label);
			if(label){
				return label.toString();
			}
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabelAttributes()
		if(this.label !== ""){
			return [this.label]; //array
		}
		return null; //null
	},

	_fetchItems: function(request, fetchHandler, errorHandler) {
		//	summary:
		//		Fetch items (XML elements) that match to a query
		//	description:
		//		If 'sendQuery' is true, an XML document is loaded from
		//		'url' with a query string.
		//		Otherwise, an XML document is loaded and list XML elements that
		//		match to a query (set of element names and their text attribute
		//		values that the items to contain).
		//		A wildcard, "*" can be used to query values to match all
		//		occurrences.
		//		If 'rootItem' is specified, it is used to fetch items.
		//	request:
		//		A request object
		//	fetchHandler:
		//		A function to call for fetched items
		//	errorHandler:
		//		A function to call on error
		var url = this._getFetchUrl(request);
		console.log("XmlStore._fetchItems(): url=" + url);
		if(!url){
			errorHandler(new Error("No URL specified."));
			return;
		}
		var localRequest = (!this.sendQuery ? request : null); // use request for _getItems()

		var self = this;
		var getArgs = {
				url: url,
                                // BEGIN FIX FOR: http://trac.dojotoolkit.org/ticket/5388
                                handleAs: (dojo.isIE ? "text" : "xml"),
                                // END FIX FOR: http://trac.dojotoolkit.org/ticket/5388
				preventCache: true
			};
		var getHandler = dojo.xhrGet(getArgs);
		getHandler.addCallback(function(data){
                        // BEGIN FIX FOR: http://trac.dojotoolkit.org/ticket/5388
 	  	        var document = data;
 	  	        if (dojo.isIE) {
 	  	            document = dojox.data.dom.createDocument(document);
 	  	        }
 	  	        var items = self._getItems(document, localRequest);
 	  	        // END FIX FOR: http://trac.dojotoolkit.org/ticket/5388
			console.log("XmlStore._fetchItems(): length=" + (items ? items.length : 0));
			if (items && items.length > 0) {
				fetchHandler(items, request);
			}
			else {
				fetchHandler([], request);
			}
		});
		getHandler.addErrback(function(data){
			errorHandler(data, request);
		});
	},

	_getFetchUrl: function(request){
		//	summary:
		//		Generate a URL for fetch
		//	description:
		//		This default implementation generates a query string in the form of
		//		"?name1=value1&name2=value2..." off properties of 'query' object
		//		specified in 'request' and appends it to 'url', if 'sendQuery'
		//		is set to false.
		//		Otherwise, 'url' is returned as is.
		//		Sub-classes may override this method for the custom URL generation.
		//	request:
		//		A request object
		//	returns:
		//		A fetch URL
		if(!this.sendQuery){
			return this.url;
		}
		var query = request.query;
		if(!query){
			return this.url;
		}
		if(dojo.isString(query)){
			return this.url + query;
		}
		var queryString = "";
		for(var name in query){
			var value = query[name];
			if(value){
				if(queryString){
					queryString += "&";
				}
				queryString += (name + "=" + value);
			}
		}
		if(!queryString){
			return this.url;
		}
		//Check to see if the URL already has query params or not.
		var fullUrl = this.url;
		if(fullUrl.indexOf("?") < 0){
			fullUrl += "?";
		}else{
			fullUrl += "&";
		}
		return fullUrl + queryString;
	},

	_getItems: function(document, request) {
		//	summary:
		//		Fetch items (XML elements) in an XML document based on a request
		//	description:
		//		This default implementation walks through child elements of
		//		the document element to see if all properties of 'query' object
		//		match corresponding attributes of the element (item).
		//		If 'request' is not specified, all child elements are returned.
		//		Sub-classes may override this method for the custom search in
		//		an XML document.
		//	document:
		//		An XML document
		//	request:
		//		A request object
		//	returns:
		//		An array of items
		var query = null;
		if(request){
			query = request.query;
		}
		var items = [];
		var nodes = null;

		console.log("Looking up root item: " + this.rootItem);
		if(this.rootItem !== ""){
			
			nodes = document.getElementsByTagName(this.rootItem);
		}
		else{
			nodes = document.documentElement.childNodes;
		}
		for(var i = 0; i < nodes.length; i++){
			var node = nodes[i];
			if(node.nodeType != 1 /*ELEMENT_NODE*/){
				continue;
			}
            var item = this._getItem(node);
			if(query){
				var found = true;
				var ignoreCase = request.queryOptions ? request.queryOptions.ignoreCase : false; 

				//See if there are any string values that can be regexp parsed first to avoid multiple regexp gens on the
				//same value for each item examined.  Much more efficient.
				var regexpList = {};
				for(var key in query){
					var value = query[key];
					if(typeof value === "string"){
						regexpList[key] = dojo.data.util.filter.patternToRegExp(value, ignoreCase);
					}
				}

				for(var attribute in query){
					var value = this.getValue(item, attribute);
					if(value){
						var queryValue = query[attribute];
						if ((typeof value) === "string" && 
							(regexpList[attribute])){
							if((value.match(regexpList[attribute])) !== null){
								continue;
							}
						}else if((typeof value) === "object"){
							if(	value.toString && 
								(regexpList[attribute])){
								var stringValue = value.toString();
								if((stringValue.match(regexpList[attribute])) !== null){
									continue;
								}
							}else{
								if(queryValue === "*" || queryValue === value){
									continue;
								}
							}
						}
					}
					found = false;
					break;
				}
				if(!found){
					continue;
				}
			}
			items.push(item);
		}
		dojo.forEach(items,function(item){ 
			item.element.parentNode.removeChild(item.element); // make it root
		},this); 
		return items;
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		 //	summary: 
		 //		See dojo.data.api.Read.close()
	},

/* dojo.data.api.Write */

	newItem: function(/* object? */ keywordArgs){
		//	summary:
		//		Return a new dojox.data.XmlItem
		//	description:
		//		At least, 'keywordArgs' must contain "tagName" to be used for
		//		the new	element.
		//		Other attributes in 'keywordArgs' are set to the new element,
		//		including "text()", but excluding "childNodes".
		// 	keywordArgs:
		//		An object containing initial attributes
		//	returns:
		//		An XML element
		console.log("XmlStore.newItem()");
		keywordArgs = (keywordArgs || {});
		var tagName = keywordArgs.tagName;
		if(!tagName){
			tagName = this.rootItem;
			if(tagName === ""){
				return null;
			}
		}

		var document = this._getDocument();
		var element = document.createElement(tagName);
		for(var attribute in keywordArgs){
			if(attribute === "tagName"){
				continue;
			}else if(attribute === "text()"){
				var text = document.createTextNode(keywordArgs[attribute]);
				element.appendChild(text);
			}else{
				attribute = this._getAttribute(tagName, attribute);
				if(attribute.charAt(0) === '@'){
					var name = attribute.substring(1);
					element.setAttribute(name, keywordArgs[attribute]);
				}else{
					var child = document.createElement(attribute);
					var text = document.createTextNode(keywordArgs[attribute]);
					child.appendChild(text);
					element.appendChild(child);
				}
			}
		}

		var item = this._getItem(element);
		this._newItems.push(item);
		return item; //object
	},
	
	deleteItem: function(/* item */ item){
		//	summary:
		//		Delete an dojox.data.XmlItem (wrapper to a XML element).
		//	item:
		//		An XML element to delete
		//	returns:
		//		True
		console.log("XmlStore.deleteItem()");
		var element = item.element;
		if(element.parentNode){
			this._backupItem(item);
			element.parentNode.removeChild(element);
			return true;
		}
		this._forgetItem(item);
		this._deletedItems.push(item);
		return true; //boolean
	},
	
	setValue: function(/* item */ item, /* attribute || string */ attribute, /* almost anything */ value){
		//	summary:
		//		Set an attribute value
		//	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		If 'attribute' specifies "tagName", nothing is set and false is
		//		returned.
		//		If 'attribute' specifies "childNodes", the value (XML element) is
		//		added to the element.
		//		If 'attribute' specifies "text()", a text node is created with
		//		the value and set it to the element as a child.
		//		For generic attributes, if '_attributeMap' is specified,
		//		an actual attribute name is looked up with the tag name of
		//		the element and 'attribute' (concatenated with '.').
		//		Then, if 'attribute' starts with "@", the value is set to the XML
		//		attribute.
		//		Otherwise, a text node is created with the value and set it to
		//		the first child element of the tag name specified with 'attribute'.
		//		If the child element does not exist, it is created.
		//	item:
		//		An XML element that holds the attribute
		//	attribute:
		//		A tag name of a child element, An XML attribute name or one of
		//		special names
		//	value:
		//		A attribute value to set
		//	returns:
		//		False for "tagName", otherwise true
		if(attribute === "tagName"){
			return false; //boolean
		}

		this._backupItem(item);

		var element = item.element;
		if(attribute === "childNodes"){
			var child = value.element;
			element.appendChild(child);
		}else if(attribute === "text()"){
			while (element.firstChild){
				element.removeChild(element.firstChild);
			}
			var text = this._getDocument(element).createTextNode(value);
			element.appendChild(text);
		}else{
			attribute = this._getAttribute(element.nodeName, attribute);
			if(attribute.charAt(0) === '@'){
				var name = attribute.substring(1);
				element.setAttribute(name, value);
			}else{
				var child = null;
				for(var i = 0; i < element.childNodes.length; i++){
					var node = element.childNodes[i];
					if(	node.nodeType === 1 /*ELEMENT_NODE*/&&
						node.nodeName === attribute){
						child = node;
						break;
					}
				}
				var document = this._getDocument(element);
				if(child){
					while(child.firstChild){
						child.removeChild(child.firstChild);
					}
				}else{
					child = document.createElement(attribute);
					element.appendChild(child);
				}
				var text = document.createTextNode(value);
				child.appendChild(text);
			}
		}
		return true; //boolean
	},
	
	setValues: function(/* item */ item, /* attribute || string */ attribute, /* array */ values){
		//	summary:
		//		Set attribute values
		//	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		If 'attribute' specifies "tagName", nothing is set and false is
		//		returned.
		//		If 'attribute' specifies "childNodes", the value (array of XML
		//		elements) is set to the element's childNodes.
		//		If 'attribute' specifies "text()", a text node is created with
		//		the values and set it to the element as a child.
		//		For generic attributes, if '_attributeMap' is specified,
		//		an actual attribute name is looked up with the tag name of
		//		the element and 'attribute' (concatenated with '.').
		//		Then, if 'attribute' starts with "@", the first value is set to
		//		the XML attribute.
		//		Otherwise, child elements of the tag name specified with
		//		'attribute' are replaced with new child elements and their
		//		child text nodes of values.
		//	item:
		//		An XML element that holds the attribute
		//	attribute:
		//		A tag name of child elements, an XML attribute name or one of
		//		special names
		//	value:
		//		A attribute value to set
		//	returns:
		//		False for "tagName", otherwise true
		if(attribute === "tagName"){
			return false; //boolean
		}

		this._backupItem(item);

		var element = item.element;
		if(attribute === "childNodes"){
			while(element.firstChild){
				element.removeChild(element.firstChild);
			}
			for(var i = 0; i < values.length; i++){
				var child = values[i].element;
				element.appendChild(child);
			}
		}else if(attribute === "text()"){
			while (element.firstChild){
				element.removeChild(element.firstChild);
			}
			var value = "";
			for(var i = 0; i < values.length; i++){
				value += values[i];
			}
			var text = this._getDocument(element).createTextNode(value);
			element.appendChild(text);
		}else{
			attribute = this._getAttribute(element.nodeName, attribute);
			if(attribute.charAt(0) === '@'){
				var name = attribute.substring(1);
				element.setAttribute(name, values[0]);
			}else{
				for(var i = element.childNodes.length - 1; i >= 0; i--){
					var node = element.childNodes[i];
					if(	node.nodeType === 1 /*ELEMENT_NODE*/ &&
						node.nodeName === attribute){
						element.removeChild(node);
					}
				}
				var document = this._getDocument(element);
				for(var i = 0; i < values.length; i++){
					var child = document.createElement(attribute);
					var text = document.createTextNode(values[i]);
					child.appendChild(text);
					element.appendChild(child);
				}
			}
		}
		return true; //boolean
	},
	
	unsetAttribute: function(/* item */ item, /* attribute || string */ attribute){
		//	summary:
		//		Remove an attribute
		//	description:
		//		'item' must be an instance of a dojox.data.XmlItem from the store instance.
		//		'attribute' can be an XML attribute name of the element or one of
		//		special names described below.
		//		If 'attribute' specifies "tagName", nothing is removed and false is
		//		returned.
		//		If 'attribute' specifies "childNodes" or "text()", all child nodes
		//		are removed.
		//		For generic attributes, if '_attributeMap' is specified,
		//		an actual attribute name is looked up with the tag name of
		//		the element and 'attribute' (concatenated with '.').
		//		Then, if 'attribute' starts with "@", the XML attribute is removed.
		//		Otherwise, child elements of the tag name specified with
		//		'attribute' are removed.
		//	item:
		//		An XML element that holds the attribute
		//	attribute:
		//		A tag name of child elements, an XML attribute name or one of
		//		special names
		//	returns:
		//		False for "tagName", otherwise true
		if(attribute === "tagName"){
			return false; //boolean
		}

		this._backupItem(item);

		var element = item.element;
		if(attribute === "childNodes" || attribute === "text()"){
			while(element.firstChild){
				element.removeChild(element.firstChild);
			}
		}else{
			attribute = this._getAttribute(element.nodeName, attribute);
			if(attribute.charAt(0) === '@'){
				var name = attribute.substring(1);
				element.removeAttribute(name);
			}else{
				for(var i = element.childNodes.length - 1; i >= 0; i--){
					var node = element.childNodes[i];
					if(	node.nodeType === 1 /*ELEMENT_NODE*/ &&
						node.nodeName === attribute){
						element.removeChild(node);
					}
				}
			}
		}
		return true; //boolean
	},
	
	save: function(/* object */ keywordArgs){
		//	summary:
		//		Save new and/or modified items (XML elements)
		// 	description:
		//		'url' is used to save XML documents for new, modified and/or
		//		deleted XML elements.
		// 	keywordArgs:
		//		An object for callbacks
		if(!keywordArgs){
			keywordArgs = {};
		}
		for(var i = 0; i < this._modifiedItems.length; i++){
			this._saveItem(this._modifiedItems[i], keywordArgs, "PUT");
		}
		for(var i = 0; i < this._newItems.length; i++){
			var item = this._newItems[i];
			if(item.element.parentNode){ // reparented
				this._newItems.splice(i, 1);
				i--;
				continue;
			}
			this._saveItem(this._newItems[i], keywordArgs, "POST");
		}
		for(var i = 0; i < this._deletedItems.length; i++){
			this._saveItem(this._deletedItems[i], keywordArgs, "DELETE");
		}
	},

	revert: function(){
		// summary:
		//	Invalidate changes (new and/or modified elements)
		// returns:
		//	True
		console.log("XmlStore.revert() _newItems=" + this._newItems.length);
		console.log("XmlStore.revert() _deletedItems=" + this._deletedItems.length);
		console.log("XmlStore.revert() _modifiedItems=" + this._modifiedItems.length);
		this._newItems = [];
		this._restoreItems(this._deletedItems);
		this._deletedItems = [];
		this._restoreItems(this._modifiedItems);
		this._modifiedItems = [];
		return true; //boolean
	},
	
	isDirty: function(/* item? */ item){
		//	summary:
		//		Check whether an item is new, modified or deleted
		//	description:
		//		If 'item' is specified, true is returned if the item is new,
		//		modified or deleted.
		//		Otherwise, true is returned if there are any new, modified
		//		or deleted items.
		//	item:
		//		An item (XML element) to check
		//	returns:
		//		True if an item or items are new, modified or deleted, otherwise
		//		false
		if (item) {
			var element = this._getRootElement(item.element);
			return (this._getItemIndex(this._newItems, element) >= 0 ||
				this._getItemIndex(this._deletedItems, element) >= 0 ||
				this._getItemIndex(this._modifiedItems, element) >= 0); //boolean
		}
		else {
			return (this._newItems.length > 0 ||
				this._deletedItems.length > 0 ||
				this._modifiedItems.length > 0); //boolean
		}
	},

	_saveItem: function(item, keywordArgs, method){
		if(method === "PUT"){
			url = this._getPutUrl(item);
		}else if(method === "DELETE"){
			url = this._getDeleteUrl(item);
		}else{ // POST
			url = this._getPostUrl(item);
		}
		if(!url){
			if(keywordArgs.onError){
				keywordArgs.onError.call(scope, new Error("No URL for saving content: " + postContent));
			}
			return;
		}

		var saveArgs = {
			url: url,
			method: (method || "POST"),
			contentType: "text/xml",
			handleAs: "xml"
		};
		var saveHander;
		if(method === "PUT"){
			saveArgs.putData = this._getPutContent(item);
			saveHandler = dojo.rawXhrPut(saveArgs);
		}else if(method === "DELETE"){
			saveHandler = dojo.xhrDelete(saveArgs);
		}else{ // POST
			saveArgs.postData = this._getPostContent(item);
			saveHandler = dojo.rawXhrPost(saveArgs);
		}
		var scope = (keywordArgs.scope || dojo.global);
		var self = this;
		saveHandler.addCallback(function(data){
			self._forgetItem(item);
			if(keywordArgs.onComplete){
				keywordArgs.onComplete.call(scope);
			}
		});
		saveHandler.addErrback(function(error){
			if(keywordArgs.onError){
				keywordArgs.onError.call(scope, error);
			}
		});
	},

	_getPostUrl: function(item){
		//	summary:
		//		Generate a URL for post
		//	description:
		//		This default implementation just returns 'url'.
		//		Sub-classes may override this method for the custom URL.
		//	item:
		//		An item to save
		//	returns:
		//		A post URL
		return this.url; //string
	},

	_getPutUrl: function(item){
		//	summary:
		//		Generate a URL for put
		//	description:
		//		This default implementation just returns 'url'.
		//		Sub-classes may override this method for the custom URL.
		//	item:
		//		An item to save
		//	returns:
		//		A put URL
		return this.url; //string
	},

	_getDeleteUrl: function(item){
		//	summary:
		//		Generate a URL for delete
		// 	description:
		//		This default implementation returns 'url' with 'keyAttribute'
		//		as a query string.
		//		Sub-classes may override this method for the custom URL based on
		//		changes (new, deleted, or modified).
		// 	item:
		//		An item to delete
		// 	returns:
		//		A delete URL
		if (!this.url !== "") {
			return this.url; //string
		}
		var url = this.url;
		if (item && this.keyAttribute !== "") {
			var value = this.getValue(item, this.keyAttribute);
			if (value) {
				url = url + '?' + this.keyAttribute + '=' + value;
			}
		}
		return url;	//string
	},

	_getPostContent: function(item){
		//	summary:
		//		Generate a content to post
		// 	description:
		//		This default implementation generates an XML document for one
		//		(the first only) new or modified element.
		//		Sub-classes may override this method for the custom post content
		//		generation.
		//	item:
		//		An item to save
		//	returns:
		//		A post content
		var element = item.element;
		var declaration = "<?xml version=\"1.0\"?>"; // FIXME: encoding?
		return declaration + dojox.data.dom.innerXML(element); //XML string
	},

	_getPutContent: function(item){
		//	summary:
		//		Generate a content to put
		// 	description:
		//		This default implementation generates an XML document for one
		//		(the first only) new or modified element.
		//		Sub-classes may override this method for the custom put content
		//		generation.
		//	item:
		//		An item to save
		//	returns:
		//		A post content
		var element = item.element;
		var declaration = "<?xml version=\"1.0\"?>"; // FIXME: encoding?
		return declaration + dojox.data.dom.innerXML(element); //XML string
	},

/* internal API */

	_getAttribute: function(tagName, attribute){
		if(this._attributeMap){
			var key = tagName + "." + attribute;
			var value = this._attributeMap[key];
			if(value){
				attribute = value;
			}else{ // look for global attribute
				value = this._attributeMap[attribute];
				if(value){
					attribute = value;
				}
			}
		}
		return attribute; //object
	},

	_getItem: function(element){
		return new dojox.data.XmlItem(element, this); //object
	},

	_getItemIndex: function(items, element){
		for(var i = 0; i < items.length; i++){
			if(items[i].element === element){
				return i; //int
			}
		}
		return -1; //int
	},

	_backupItem: function(item){
		var element = this._getRootElement(item.element);
		if(	this._getItemIndex(this._newItems, element) >= 0 ||
			this._getItemIndex(this._modifiedItems, element) >= 0){
			return; // new or already modified
		}
		if(element != item.element){
			item = this._getItem(element);
		}
		item._backup = element.cloneNode(true);
		this._modifiedItems.push(item);
	},

	_restoreItems: function(items){

		dojo.forEach(items,function(item){ 
			if(item._backup){
				item.element = item._backup;
				item._backup = null;
			}
		},this); 
	},

	_forgetItem: function(item){
		var element = item.element;
		var index = this._getItemIndex(this._newItems, element);
		if(index >= 0){
			this._newItems.splice(index, 1);
		}
		index = this._getItemIndex(this._deletedItems, element);
		if(index >= 0){
			this._deletedItems.splice(index, 1);
		}
		index = this._getItemIndex(this._modifiedItems, element);
		if(index >= 0){
			this._modifiedItems.splice(index, 1);
		}
	},

	_getDocument: function(element){
		if(element){
			return element.ownerDocument;  //DOMDocument
		}else if(!this._document){
			return dojox.data.dom.createDocument(); // DOMDocument
		}
	},

	_getRootElement: function(element){
		while(element.parentNode){
			element = element.parentNode;
		}
		return element; //DOMElement
	}

});

//FIXME: Is a full class here really needed for containment of the item or would
//an anon object work fine?
dojo.declare("dojox.data.XmlItem", null, {
	constructor: function(element, store) {
		//	summary:
		//		Initialize with an XML element
		//	element:
		//		An XML element
		//	store:
		//		The containing store, if any.
		this.element = element;
		this.store = store;
	}, 
	//	summary:
	//		A data item of 'XmlStore'
	//	description:
	//		This class represents an item of 'XmlStore' holding an XML element.
	//		'element'
	//	element:
	//		An XML element

	toString: function() {
		//	summary:
		//		Return a value of the first text child of the element
		// 	returns:
		//		a value of the first text child of the element
		var str = "";
		if (this.element) {
			for (var i = 0; i < this.element.childNodes.length; i++) {
				var node = this.element.childNodes[i];
				if (node.nodeType === 3) {
					str = node.nodeValue;
					break;
				}
			}
		}
		return str;	//String
	}

});
dojo.extend(dojox.data.XmlStore,dojo.data.util.simpleFetch);

}

if(!dojo._hasResource["hub.data.catalog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["hub.data.catalog"] = true;
dojo.provide("hub.data.catalog");




/**
 * Abstract class for accessing the Mashup Hub catalog. Subclasses implement
 * fetch logic for specific catalog formats. Derives from dijit._Widget to 
 * support proper cleanup.
 */
dojo.declare('hub.data.catalog._Model', dijit._Widget, {
    // default query is entire catalog
    query: 'all',
    // default term is everything
    term: '*',
    collection: "all",
    searchType: "",
    sortBy: null,
    sortOrder: null,
    count: null,
    constructor: function(args, node) {
        // source for the server data
        this.source = null;
        // url to use to fetch the data
        this.url = null;
        // url to get the next batch of data
        this.next_url = null;
        // url to get the previous batch of data
        this.prev_url = null;
        // 3-tuple summary of the fetched data (first, last, total)
        this.summary = null;
        // outstanding request
        this.request = null;
        // subscriptions to dojo.publish'ed events so we know when to refresh
        this.conns = new Array();
    },

    /**
     * Starts watching for bus events.
     */
    postCreate: function() {
        // watch for bus events about entries
        this.conns.push(dojo.subscribe('hub.tag-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.save-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.delete-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.comment-entry', this, 'onStale'));
        this.conns.push(dojo.subscribe('hub.reset', this, 'onStale'));
        if(this.query == 'favorites') {
            // watch for changing of favorites too
            this.conns.push(dojo.subscribe('update-favorites', this, 
                                           'onStale'));
        }
    },
       
    /**
     * Stops watching for bus events.
     */
    destroy: function() {
        // cleanup all subscriptions
        dojo.forEach(this.conns, dojo.unsubscribe);
        // call base class method
        this.inherited(arguments);
    },
    
    /**
     * Clears the model
     */
    clear: function() {
      this.next_url = null;
      this.prev_url = null;
      this.source = null;
      this.url = null;
      this.summary = null;
      this.request = null;
      this.stale = null;
    },
    
    /**
     * Gets the URL last loaded by the catalog.
     */
    getUrl: function() {
        return this.url;
    },
    
    /**
     * Gets a tuple of start index, max per page, and total items. If any
     * of these items is unknown 
     */
    getSummary: function() {
        return this.summary;
    },
    
    /**
     * Get if the catalog has a next set of results.
     */
    hasNext: function() {
        return (this.next_url != null);
    },
    
    /**
     * Get if the catalog has a previous set of results.
     */
    hasPrevious: function() {
        return (this.prev_url != null);
    },
    
    /**
     * Fetch items.
     *
     * @param start: Starting index
     * @param max: Maximum number of items to fetch
     * @param sortby: Name of the field to sort by
     * @param asc: Order of the sort, true = ascending, false = descending
     */
    fetch: function(start, max, sortby, asc) {
        var url = this._createQueryUrl();
        console.debug("URL = " + url);
        var query = "";
        var questionPos = url.indexOf('?');
        if (questionPos >= 0) {
        	query = url.substring(questionPos + 1);
        	url = url.substring(0, questionPos);
        	console.debug("Parsed query = " + query + ", URL = " + url);
        }
        var map = dojo.queryToObject(query);
        map['nologin'] = 'true';
        if (sortby != null) {
        	map["sortBy"] = sortby;
        }
        else if (this.sortBy != null) {
        	map["sortBy"] = this.sortBy;
        }
        if (asc != null) {
        	map["sortOrder"] = asc ? "asc" : "desc";
        }
        else if (this.sortOrder != null) {
        	map["sortOrder"] = this.sortOrder;
        }
        if (start != null) {
        	map["startIndex"] = "" + start;
        }
        if (max != null) {
        	map["count"] = "" + max;
        } else if (this.count) {
            map["count"] = "" + this.count;
        }

        query = dojo.objectToQuery(map);
        if (query.length > 0) {
        	url = url + '?' + query;
        }
        console.debug("Result URL = " + url);
        this.url = url;
        
        // do the fetch
        this._doFetch();
    },
    
    /**
     * Fetch items again.
     */
    refresh: function() {
        if(this.url != null) {
            this._doFetch();
        } else {
            this.fetch();
        }
    },
    
    /**
     * Fetch the next batch of items, if it exists.
     */
    fetchNext: function() {
        if(this.next_url) {
            this.url = this.next_url;
            this._doFetch();
        } else {
            throw new Error("catalog._Model: No next URL");
        }
    },
    
    /**
     * Fetch the previous batch of items, if it exists.
     */
    fetchPrevious: function() {
        if(this.prev_url) {
            this.url = this.prev_url;
            this._doFetch();
        } else {
            throw new Error("catalog._Model: No previous URL");
        }
    },
    
    
    /**
     * Signals 'complete' after the last item signal.
     */
    _onComplete: function(empty, request) {
        this.onComplete();
        this.request = null;
        this.stale = false;
    },
    
    /**
     * Abstract method. Should return URL to query.
     */
    _createQueryUrl : function() {
        throw new Error("Abstract method _createQueryUrl()");
    },
    
    /**
     * Does some basic housekeeping before fetching. Must be overriden to
     * do an actual fetch.
     */
    _doFetch: function() {
        this.next_url = null;
        this.prev_url = null;
        this.summary = new Array(3);
        
        if(this.request) {
            // @todo: really we'd like to cancel the earlier request instead
            // of stopping the new request, but dojo isn't cooperating
            // supposedly there should be an abort() method on request, but
            // not for XML; so how do we reliably stop it?
            throw new Error('catalog._Model: request in progress');
        }
        
        // signal that we're about to fetch
        this.onFetch();          
    },

    /**
     * Signals when catalog data is likely stale because of local changes.
     */
    onStale: function() {
        // extension point
    },

    /**
     * Signals at start of fetch, before network request.
     */
    onFetch: function() {
        // extension point
    },

    /**
     * Signals after data received, before first item.
     */
    onBegin: function(total) {
        // extension point
    },

    /**
     * Signals after data received, before first item.
     */
    onItem: function(item) {
        // extension point
    },
    
    /**
     * Signals after an error.
     */
    onError: function(error) {
        // extension point
    },

    /**
     * Signals after completion.
     */
    onComplete: function() {
        // extension point
    }
});

/**
 * Fetches an Atom feed listing a portion of the Mashup Hub catalog.
 * Pulls out all available data into JS objects for use in rendering.
 **/
dojo.declare('hub.data.catalog.AtomModel', hub.data.catalog._Model, {
    _createQueryUrl: function() {
        return hub.urls.getCatalog(this.collection, this.term, this.searchType, this.sortBy, this.sortOrder);
    },

    /**
     * Performs the remote query. Signals 'fetch' first.
     */
    _doFetch: function() {
        this.inherited(arguments);
        // send the query to the remote data source
        this.source = new dojox.data.XmlStore({'url' : this.url});
        this.request = this.source.fetch(
            {'onBegin' : dojo.hitch(this, this.onBegin),
             'onItem' : dojo.hitch(this, this._onItem),
             'onComplete' :dojo.hitch(this, this._onComplete),
             'onError' : dojo.hitch(this, this.onError)});
    },
    
    /**
     * Parses a single, top level XML element for data. Looks for relative
     * links and entries. Signals onItem with entry data in JS format.
     */
    _onItem: function(item, request) {
        var dict = {};
        var value;
        var tag = this.source.getValue(item, 'tagName')
        if(tag == 'os:startIndex') {
            this.summary[0] = parseInt(this.source.getValue(item, 'text()'));
        } else if(tag == 'os:itemsPerPage') {
            this.summary[1] = parseInt(this.source.getValue(item, 'text()'));
        } else if(tag == 'os:totalResults') {
            this.summary[2] = parseInt(this.source.getValue(item, 'text()'));
        } else if(tag == 'link') {
            var rel = this.source.getValue(item, '@rel');
            var href = this.source.getValue(item, '@href');
            if(rel == 'next') {
                this.next_url = href;
            } else if (rel == 'previous') {
                this.prev_url = href;
            }
        } else if(tag == 'entry') {
            var entryCategoryElement = this.source.getValue(item, "category");
            var category = this.source.getValue(entryCategoryElement, "@term");
            dict["entryCategory"] = category;
            var content = this.source.getValue(item, "content");
            
            var cat = this.source.getValue(content, "childNodes");
            
            var atts = this.source.getValues(cat, "childNodes");
            
            for(var i=0; i < atts.length; i++) {
                var node = atts[i];
                if(this.source.isItem(node)) {
                    if(this.source.hasAttribute(node, "catalog:tag")) {
                        value = this.source.getValues(node, "catalog:tag");
                        value = this._parseArray(value);
                    } else if (this.source.hasAttribute(node, "catalog:category")) {
                        value = this.source.getValues(node, "catalog:category");
                        value = this._parseArray(value);
                    }
                    else {
                        value = this.source.getValue(node, "text()");
                    }
                }
                
                var key = this.source.getValue(atts[i], 'tagName');
                if (key.lastIndexOf(":") > 0)
                	key = key.substring(key.lastIndexOf(":") + 1);
                	
				dict[key] = value;                
            }
            
            var idfield = this.source.getValue(this.source.getValue(item, "id"), "text()");
            var unprefixedId = idfield;
            if (idfield.indexOf("urn:uuid:") == 0) {
            	unprefixedId = idfield.substring("urn:uuid:".length);
            }
            if (idfield.indexOf("urn:id:") == 0) {
            	unprefixedId = idfield.substring("urn:id:".length);
			}
            var pureId = unprefixedId;
            if (unprefixedId.lastIndexOf('-') >=0) {
            	pureId = unprefixedId.substring(unprefixedId.lastIndexOf('-') + 1);
            }
            dict['objectid'] = pureId;

            // signal with parsed data
            this.onItem(dict);
        }
    },
    
    /**
     * Pulls an array of strings out of dojo data XML values.
     *
     * @param values Dojo data items having the values we want
     */
    _parseArray: function(values) {
        var arr = new Array();
        for(i in values) {
            arr.push(this.source.getValue(values[i], "text()"));
        }
        return arr;
    }
});

/**
 * Fetches a JSON list of a portion of the Mashup Hub catalog.
 * Pulls out all available data into JS objects for use in rendering.
 */
dojo.declare('hub.data.catalog.JsonModel', hub.data.catalog._Model, {
    _createQueryUrl: function() {
        return hub.urls.getCatalog(this.query, this.term, 'json');
    },

    /**
     * Performs the remote query. Signals 'fetch' first.
     */
    _doFetch: function() {
        this.inherited(arguments);       
        // signal that we're about to fetch
        this.onFetch();
        // send the query to the remote data source
        this.source = {url : this.url,
                       handleAs: 'json',
                       load : dojo.hitch(this, this._onLoad),
                       error: dojo.hitch(this, this.onError)
                      }
        this.request = dojo.xhrGet(this.source);
    },
    
    _onLoad: function(response, args) {
        // grab relative urls
        this.next_url = response.next;
        this.prev_url = response.previous;
        
        try {
            // signal begin
            this.onBegin(response.content.length);
            
            // iterate over content
            for(i in response.content) {
                this.onItem(response.content[i]);
            }
            
            // signal complete
            this._onComplete(response, args);
        } catch (e) {
            // all errors go to the registered error handler
            this.onError(e, args);
        }
    }
});

}

if(!dojo._hasResource["flow.dijit.Catalog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.Catalog"] = true;
dojo.provide("flow.dijit.Catalog");









hub.urls = new hub.managers.Urls(self.parent.hub.config.base_url);



dojo.declare(
  "flow.dijit.Catalog",
  [dijit._Widget, dijit._Templated, dijit._Container],
  {
    // attach points
    searchNode: null,
    tableBodyNode: null,
    summaryNode: null,
    previousButton: null,
    refreshButton: null,
    nextButton: null,

    templateString:"<div class=\"flowCatalogNavigation\">\r\n  <form onsubmit=\"return false;\" >\r\n  <div class=\"flowCatalogTable\">\r\n    <table width=\"100%\">\r\n      <tr>\r\n        <td width=\"100%\"><input dojoAttachPoint=\"searchNode\" title=\"TOOLTIP_CATALOG_SEARCH_FIELD\" style=\"width:95%\"></td>\r\n        <td><button dojoAttachEvent=\"onclick:_onSearch\" >${i18n.propertyeditor.CATALOG_SEARCH_BUTTON}</button></td>\r\n      </tr>\r\n    </table>\r\n    <table class=\"flowpropeditorgrid\" cellspacing=\"0\" cellpadding=\"5\" width=\"100%\">\r\n      <thead>\r\n        <tr>\r\n          <th class=\"flowgridcell\">${i18n.propertyeditor.CATALOG_GRID_TITLE}</th>\r\n          <th class=\"flowgridcell\">${i18n.propertyeditor.CATALOG_GRID_AUTHOR}</th>\r\n        </tr>\r\n      </thead>\r\n      <tbody dojoAttachPoint=\"tableBodyNode\" dojoAttachEvent=\"onclick:_onRowClick\"></tbody>\r\n    </table>\r\n\r\n    <table width=\"100%\">\r\n      <tr>\r\n        <td align=\"left\" dojoAttachPoint=\"summaryNode\"></td>\r\n        <td align=\"right\">\r\n          <span class='prevButton'><button dojoAttachPoint=\"previousButton\" type=\"submit\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onPrevious\"></button></span>\r\n          <span class='refreshButton'><button dojoAttachPoint=\"refreshButton\" type=\"submit\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onRefresh\"></button></span>\r\n          <span class='nextButton'><button dojoAttachPoint=\"nextButton\" type=\"submit\" dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:_onNext\"></button></span>\r\n        </td>\r\n      </tr>\r\n    </table>\r\n  </div>\r\n  </form>\r\n</div>\r\n",
    widgetsInTemplate: true,

    // localization
    i18n: null,

    model: null,
    rowCount: 0,
    selectedRow: null,

    constructor: function(args, node) {
      this.stale = false;
      this.conns = new Array();
    },

    postMixInProperties: function() {
      this.model = new hub.data.catalog.AtomModel({collection: "feeds", term: "*", count: 10, sortBy: "title", sortOrder: "asc"});

      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    postCreate: function() {
      // register as an observer
      this.connect(this.model, 'onFetch', '_onBeforeFetch');
      this.connect(this.model, 'onBegin', '_onBeforeLoad');
      this.connect(this.model, 'onItem', '_onItemLoad');
      this.connect(this.model, 'onComplete', '_onAfterLoad');
      this.connect(this.model, 'onError', '_onLoadError');
      this.connect(this.model, 'onStale', '_onStale');

      // initialize navigation
      this._updateNavBar();

      // interpolate values
      this.inherited(arguments);
      this.startup();
    },

    startup: function() {
      if (this.model) {
        this.model.refresh();
      }
    },

    destroy: function() {
      // cleanup all subscriptions
      dojo.forEach(this.conns, dojo.unsubscribe);
      // call base class method
      this.inherited(arguments);
    },

    clearRows: function() {
      var rows = this.tableBodyNode.rows;

      for (var i = 0; i < rows.length; i++) {
        var row = this.tableBodyNode.rows[i];
        row.cells[0].innerHTML = "&nbsp;";
        row.cells[1].innerHTML = "&nbsp;";
        row.removeAttribute("objectid");
        row.removeAttribute("selflink");
        row.removeAttribute("objectname");
      }
    },

    initialize : function() {
      this.clearRows();
    },

    _onSearch: function() {
      var value = dojo.trim(this.searchNode.value);
      if (!value) {
        value = "*";
      }

      this.model.clear();
      this.model.term = value;
      this.model.refresh();
    },

    _onStale: function() {
      // refresh immediately
      this.stale = false;
      this.model.refresh();
    },

    _onBeforeFetch: function() {
      this.clearRows();
      this.summaryNode.innerHTML = this.i18n.propertyeditor.CATALOG_LOADING;
    },

    _onBeforeLoad: function(total) {
      // reset row count
      this.rowCount = 0;

      if (this.selectedRow) {
        dojo.removeClass(this.selectedRow, "selectedrow");
        this.selectedRow = null;
      }
    },

    _onLoadError: function(error) {
      alert(error);
    },

    _onItemLoad : function(dict) {
      // We reuse rows, lets check if we need to build it or not

      if (this.tableBodyNode.rows.length <= this.rowCount) {
        // build
        var tr = document.createElement("tr");

        var td = document.createElement("td");
        dojo.addClass(td, "flowgridcell");
        tr.appendChild(td)

        td = document.createElement("td");
        dojo.addClass(td, "flowgridcell");
        tr.appendChild(td);

        if (this.rowCount % 2) {
          dojo.addClass(tr, "alt");
        }

        this.tableBodyNode.appendChild(tr);
      }

      var id = dict.objectid;

      var row = this.tableBodyNode.rows[this.rowCount];
      row.cells[0].innerHTML = dijit.byId("flowcanvas").model.stripSctipt(dict.name);
      row.cells[1].innerHTML = dijit.byId("flowcanvas").model.stripSctipt(dict.author);
      row.setAttribute("objectid", id);
      row.setAttribute("selflink", dict.downloadURL);
      row.setAttribute("objectname", dict.name);

      this.rowCount++;
    },

    _onAfterLoad: function() {
      if (!this.rowCount) {
        this.summaryNode.innerHTML = this.i18n.propertyeditor.CATALOG_SUMMARY_EMPTY;
      } else {
        var label = dojo.string.substitute(this.i18n.propertyeditor.CATALOG_SUMMARY, [parseInt(this.model.summary[0], 10) + 1, parseInt(this.model.summary[0], 10) + this.rowCount, this.model.summary[2]]);
        this.summaryNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(label);
      }

      this._updateNavBar();
    },

    _updateNavBar : function() {
      if (this.model.hasPrevious()) {
        this.previousButton.setDisabled(false);
      } else {
        this.previousButton.setDisabled(true);
      }

      if (this.model.hasNext()) {
        this.nextButton.setDisabled(false);
      } else {
        this.nextButton.setDisabled(true);
      }
    },

    _onPrevious: function(event) {
      try {
        this.model.fetchPrevious();
      } catch(e) {}
    },

    _onNext: function(event) {
      try {
        this.model.fetchNext();
      } catch(e) {}
    },

    _onRefresh: function(event) {
      this.model.refresh();
    },

    _onRowClick: function(event) {
      var target = event.target;

      while (target.tagName != "TR" && target.parentNode) {
        target = target.parentNode;
      }

      if (target.tagName != "TR") {
        // should really never happen
        return;
      }

      dojo.stopEvent(event);

      if (this.selectedRow == target) {
        // already selected
        return;
      }

      if (this.selectedRow) {
        dojo.removeClass(this.selectedRow, "selectedrow");
      }

      this.selectedRow = target;
      dojo.addClass(this.selectedRow, "selectedrow");
    },

    getSelectedId: function() {
      var id = null;

      if (this.selectedRow) {
        id = this.selectedRow.getAttribute("objectid");
      }

      return id;
    },

    getSelectedName: function() {
      var name = null;

      if (this.selectedRow) {
        name = this.selectedRow.getAttribute("objectname");
      }

      return name;
    },

    getSelectedUrl: function() {
      var id = null;

      if (this.selectedRow) {
        id = this.selectedRow.getAttribute("selflink");
      }

      return id;
    }
});

}

if(!dojo._hasResource["flow.dijit.PropertyEditor"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.PropertyEditor"] = true;
dojo.provide("flow.dijit.PropertyEditor");












dojo.declare(
  "flow.dijit.PropertyEditor",
  [dijit._Widget, dijit._Container, dijit._Templated],
  {
    // arguments
    operator: null,
    canvas: null,

    // attach points
    containerNode: null,
    tabcontainerNode: null,
    previewContainerNode: null,
    advancedTabNode: null,
    commentsNode: null,
    previewTabNode: null,
    captionNode: null,

    // localization
    i18n: null,

    widgetsInTemplate: true,

    // impl
    operatorType: null,
    operatorId: null,
    operatorData: null,
    topSectionNode: null,
    refreshErrorMessageNode: null,
    messageNode: null,
    captioninput: null,
    previewTreeWidget: null,

    hasError: false,

    templateString:"<div style=\"width: 100%; height:100%\">\r\n  <div dojoType=\"dijit.layout.TabContainer\" dojoAttachPoint=\"tabcontainerNode\" style=\"width:100%; height:100%;\">\r\n\t\t<div dojoType=\"dijit.layout.ContentPane\" title=\"${i18n.propertyeditor.LABEL_TAB_EDIT}\">\r\n      <div style=\"clear: both\" dojoAttachPoint=\"topSectionNode\">\r\n        <div dojoAttachPoint=\"captionNode\">${i18n.propertyeditor.LABEL_NAME} <input dojoAttachPoint=\"captioninput\" title=\"${i18n.propertyeditor.TOOLTIP_TEXTBOX_NAME}\" dojoAttachEvent=\"onkeyup:onCaptionChanged\" style=\"width:80%\"></input></div>\r\n\r\n        <div style=\"display:none; margin-top: 5px;\" dojoAttachPoint=\"messageNode\"></div>\r\n      </div>\r\n      <div style=\"clear: both; padding-top: 10px;\" dojoAttachPoint=\"containerNode\">\r\n      </div>\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" dojoAttachPoint=\"previewTabNode\" title=\"${i18n.propertyeditor.LABEL_TAB_PREVIEW}\">\r\n      <div style=\"clear: both\">\r\n        <div style=\"float: left;\">\r\n          <div class=\"flowCatalogNavigation\">\r\n            <div class='refreshButton'>\r\n\t            <div style=\"display:none;\" class=\"warningMsg\" dojoAttachPoint=\"refreshErrorMessageNode\"></div>\r\n            \t<button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:doPreview\" title=\"${i18n.propertyeditor.PREVIEW_BUTTON_TOOLTIP}\"></button>\r\n            </div>\r\n          </div>\r\n        </div>\r\n        <div style=\"clear: both; padding-top: 10px;\" dojoAttachPoint=\"previewContainerNode\"></div>\r\n      </div>\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" title=\"${i18n.propertyeditor.LABEL_TAB_COMMENTS}\">\r\n      <textarea dojoAttachPoint=\"commentsNode\" style=\"width:98%; height:98%\" dojoAttachEvent=\"onblur:saveComments\"></textarea>\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" title=\"${i18n.propertyeditor.LABEL_TAB_ADVANCED}\" dojoAttachPoint=\"advancedTabNode\"></div>\r\n  </div>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");
    },

    postCreate: function() {
      var tab = this.tabcontainerNode;
      tab.startup();
      tab.resize();

      // add help icon
      var tablist = this.tabcontainerNode.tablist;
      var div = document.createElement("div");
      div.className = "flowDialogHelp"
      var img = document.createElement("img");
      img.setAttribute("src", dojo.moduleUrl("flow.dijit", "templates/images/dialoghelp.png"));
      img.setAttribute("title", this.i18n.propertyeditor.TOOLTIP_HELP);

      div.appendChild(img);
      dojo.connect(img, "onclick", this, "showHelp");

      tablist.domNode.appendChild(div);

      this.operatorType = this.operator.type;
      this.operatorId = this.operator.id;

      if (this.operator.canRename) {
        this.captioninput.value = this.operator.getName();
      } else {
        this.captionNode.style.display = "none";
      }

      var foo = function(widget){
        if (widget == this.previewTabNode) {this.refreshPreviewTree()};
      }
      dojo.connect(tablist, "onSelectChild", this, foo);

      // comments
      this.setComments(this.operator.getComment());

      // advanced tab
      if (!this.operator.hasAdvancedOptions) {
        this.tabcontainerNode.closeChild(this.advancedTabNode);
      }

      this.resizeComments();

      // operator data
      this.operatorData = this.operator.setupPropertyEditor();

      // check if requirements are met
      var requirements = this.operator.meetsRequirements();
      if (!requirements.pass) {
        this.containerNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(this.operator.createErrorMarkup(requirements.error));
        this.hasError = true;
        return;
      }

      var result = this.operator.getDirtyInputs(this.operator.id);
      if (result.length > 0) {
        this.messageNode.innerHTML = this.i18n.propertyeditor.LABEL_INPUT_DATA_DIRTY;
        var button = document.createElement("button");
        button.appendChild(document.createTextNode(this.i18n.propertyeditor.LABEL_INPUT_DATA_REFRESH));
        this.messageNode.appendChild(button);
        dojo.connect(button, "onclick", this, "refreshInputs");
        this.messageNode.style.display = "block";
        dojo.addClass(this.messageNode, "warningMsg");
      }

      this.hasError = false;

      if (!this.operatorData) {
        // no UI
        return;
      }

      var layout = this.operatorData.layout;

      for (var i = 0; i < layout.length; i++) {
        var item = layout[i];
        if (item.type == "grid") {
          this.generateGrid(i, item);
        } else if (item.type == "url") {
          var div = document.createElement("div");
          this.containerNode.appendChild(div);
          var x = new flow.dijit.PropertyEditor.URL({item: item, operatorId: this.operator.id, propEditor: this}, div);
        } else if (item.type == "inputselect") {
          var select = document.createElement("select");
          select.setAttribute("id", this.operatorId + "+_inputselect");

          for (var j = 0; j < item.data.length; j++) {
            var option = document.createElement("option");
            option.setAttribute("value", item.data[j]);
            option.appendChild(document.createTextNode(item.captions[j]));
            select.appendChild(option);
          }

          var myThis = this;
          select.onchange = function() {
            var index = (select.selectedIndex == 0) ? 1 : 0;
            if (myThis.operator.model.runXPath("flow:condition//flow:param[text()!='']", myThis.operator.node).length>0) {
              if (myThis.operator.model.getValue(myThis.operator.node, "flow:input[ @id='"+select.options[select.selectedIndex].value+"']/@position")!="0") {
                alert(myThis.operator.i18n.PROPERTY_TAB_INPUTCHANGE_ALERT);
                select.selectedIndex = index;
                return false;
              }
            }

            document.getElementById(myThis.operatorId + "+_inputselect_secondinput").setAttribute("value", select.options[index].text);

            myThis.operator.model.setValue("flow:input[ @id='"+select.options[select.selectedIndex].value+"']/@position", myThis.operator.node, 0);
            myThis.operator.model.setValue("flow:input[ @id='"+select.options[index].value+"']/@position", myThis.operator.node, 1);

            try {
              if (myThis.operator.firstInputSelected) {
                myThis.operator.firstInputSelected();
              }
            } catch(ex) {}
          }

          this.containerNode.appendChild(document.createTextNode(item.labels[0]));
          this.containerNode.appendChild(select);
          this.containerNode.appendChild(document.createTextNode(" "+item.labels[1]));

          // 2nd input
          var input = document.createElement("input");
          input.setAttribute("id", this.operatorId + "+_inputselect_secondinput");
          input.setAttribute("disabled", "true");
          input.setAttribute("value", item.captions[1]);
          this.containerNode.appendChild(input);

          // add new line
          this.containerNode.appendChild(document.createElement("br"));
        } else if (item.type == "custom") {
          this.operator.buildCustomValue(this.containerNode, item.id);
        }
      }

      // other tabs
      dojo.connect(this.advancedTabNode, "onShow", this, "loadAdvancedTab");
      dojo.connect(this.advancedTabNode, "onHide", this, "hideAdvancedTab");

    },

    loadAdvancedTab: function() {
      // always render when we show it - make sure to clear out first
      this.advancedTabNode.destroyDescendants();
      this.operator.renderAdvancedOptions(this.advancedTabNode.domNode)
      this.operator.isAdvancedOptionsVisible = true;
    },

    hideAdvancedTab: function() {
      this.operator.isAdvancedOptionsVisible = false;
    },

    onCaptionChanged: function() {
	  var val=dijit.byId("flowcanvas").model.stripSctipt(this.captioninput.value);
      this.operator.model.setValue("@caption", this.operator.node, val);
      this.operator.model.canvas.onOperatorCaptionChanged(this.operator, val);
    },

    doPreview: function() {
      this.operator.previewLoadComplete();
      this.operator.doPreview(this.refreshErrorMessageNode);
    },

    refreshPreviewTree: function() {
      this.refreshErrorMessageNode.innerHTML = "";
      this.refreshErrorMessageNode.style.display = "none";

      // remove the tree
      if (this.previewTreeWidget) {
        this.previewTreeWidget.destroyRecursive();
        this.previewTreeWidget = null;
      }

      if (this.operator.model.getValue(this.operator.node, "@dirty") == 1 &&
      	this.operator.loadingPreviewResult != true ) {

        // show loading message
        this.previewContainerNode.innerHTML = "<img align='top' src='"+FLOW.baseUrl+"/images/progress17x17.gif'> "+this.i18n.propertyeditor.PREVIEW_LOADING;

        this.operator.doPreview(this.refreshErrorMessageNode);
      }

      if (this.operator.loadingPreviewResult) {
        this.previewContainerNode.innerHTML = "";
        this.refreshErrorMessageNode.style.display = "block";
        this.refreshErrorMessageNode.innerHTML = this.operator.i18n.SOURCE_ADVANCED_ERROR_WAITING;
      }

      var node = this.operator.model.getNode("/flow:mashup/flow:data/flow:feed[@id='"+this.operator.id+"']/*");
      if (node) {
        
        this.previewContainerNode.innerHTML = "";

        var div = document.createElement("div");
        this.previewContainerNode.appendChild(div);
        this.previewTreeWidget = new flow.dijit.XMLViewTree({xmlRootNode: node, operatorId: this.operator.id, operator: this.operator}, div);
      }
    },

    generateGrid: function(index, layout) {
      var gridContainer = document.createElement("div");
      this.containerNode.appendChild(gridContainer);

      var grid = this.operatorData.layout[index];

      

      var widget = new flow.dijit.Grid({grid: grid, operatorId: this.operatorId,
                                        operator: this.operator, chain: 0,
                                        collapsable: layout.collapsable,
                                        collapseLabel: layout.collapseLabel,
                                        expandLabel: layout.expandLabel
                                       }, gridContainer);
      this.operator.grid = widget;

      if (layout.hidden) {
        widget.domNode.style.display = "none";
      }

    },

    checkForError: function() {
      var count = this.operator.model.getInputCount(this.operatorId);

      return (count < this.operator.minimumInputs);
    },

    saveComments: function() {
      var value = this.commentsNode.value;
      this.operator.saveComment(encodeURI(value));
    },

    setComments: function(value) {
      if (value) {
        this.commentsNode.value = decodeURI(value);
      }
    },

    showHelp: function() {
      this.canvas.showOperatorHelp(this.operator);
    },

    refreshInputs: function() {
      function callback() {
        this.messageNode.style.display = "none";
      }

      this.canvas.refreshDirtyInputs(this.operator.id, dojo.hitch(this, callback));
    },

    resize: function(args) {
      // resize tab container
      this.tabcontainerNode.resize(args);

      this.resizeComments();
    },

    resizeComments: function() {
      // IE doesn't support 100% height on textarea
      if (dojo.isIE) {
        var box = dojo.contentBox(this.tabcontainerNode.containerNode);
        this.commentsNode.style.height = (box.h - 20) + "px";
      }
    }
  }
);

dojo.declare(
  "flow.dijit.PropertyEditor.URL",
  [dijit._Widget, dijit._Templated],
  {
    // attach points
    urlRadioNode: null,
    urlNode: null,
    catalogRadioNode: null,
    catalogNode: null,
    urlLoadButtonNode: null,
    catalogBrowseButtonNode: null,
    messageNode: null,
    tableNode: null,

    item: null,
    operatorId: null,
    propEditor: null,

    templateString:"<div>\r\n  <fieldset>\r\n    <legend>${i18n.propertyeditor.SOURCE_HEADER_SOURCE_SELECTION}</legend>\r\n    <table width=\"100%\" dojoAttachPoint=\"tableNode\">\r\n      <tr>\r\n        <td><input type=\"radio\" checked=\"true\" name=\"${id}radiogroup\" dojoAttachPoint=\"urlRadioNode\" dojoAttachEvent=\"onclick:onRadioChange\"></td>\r\n        <td width=\"100%\"><table width=\"100%\"><tr><td NOWRAP>${i18n.propertyeditor.LABEL_URL}</td><td width=\"100%\"><input style=\"width:100%\" dojoAttachPoint=\"urlNode\" dojoAttachEvent=\"onkeypress:onKeyPress,onblur:onBlur\" title=\"${i18n.propertyeditor.TOOLTIP_URL}\"></td></tr></table></td>\r\n        <td><button dojoAttachPoint=\"urlLoadButtonNode\" dojoAttachEvent=\"onclick:loadButtonClick\">${i18n.propertyeditor.LABEL_BUTTON_LOAD}</button></td>\r\n      </tr>\r\n      <tr>\r\n        <td><input type=\"radio\" name=\"${id}radiogroup\" dojoAttachPoint=\"catalogRadioNode\" dojoAttachEvent=\"onclick:onRadioChange\"></td>\r\n        <td width=\"100%\"><table width=\"100%\"><tr><td NOWRAP>${i18n.propertyeditor.LABEL_CATALOG}</td><td width=\"100%\"><input style=\"width:100%\" disabled=\"true\" dojoAttachPoint=\"catalogNode\" title=\"${i18n.propertyeditor.TOOLTIP_TEXTBOX_CATALOG}\"></td></tr></table></td>\r\n        <td align=\"right\"><button dojoAttachPoint=\"catalogBrowseButtonNode\" dojoAttachEvent=\"onclick:loadCatalogClick\" title=\"${i18n.propertyeditor.TOOLTIP_BUTTON_BROWSE}\">${i18n.propertyeditor.LABEL_BUTTON_BROWSE}</button></td>\r\n      </tr>\r\n    </table>\r\n  </fieldset>\r\n\r\n  <div dojoAttachPoint=\"messageNode\" style=\"text-align: center; padding: 5px; font-size: 1.1em;\"></div>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");
    },

    postCreate: function() {
      if (this.item.data) {
        if (this.item.data.value) {
          // show loaded msg when data is present to make clear the data is there
          this.messageNode.innerHTML = this.i18n.propertyeditor.SOURCE_LOAD_FINISH;
        }

        if (this.item.data.type == "catalog") {
          var name = this.item.data.name;
          if (!name) {
            name = this.item.data.value
          }
          this.catalogNode.setAttribute("value", name);

          this.catalogRadioNode.setAttribute("checked", "true");
        } else {
          this.resize();
          if (this.item.data.value) {
            this.urlNode.value = this.item.data.value;
          } else {
            // default url
            var myThis = this;
            //window.setTimeout(function(){myThis.urlNode.setAttribute("value", "http://www-01.ibm.com/software/swnews/swnews.nsf/swnewsrss?openview&RestrictToCategory=data");}, 300);
            this.urlNode.value = "http://www-01.ibm.com/software/swnews/swnews.nsf/swnewsrss?openview&RestrictToCategory=data";
          }
        }
      }

      this.onRadioChange(null, true);
      dojo.addClass(this.domNode, "flowResizable");
    },

    onRadioChange: function(e, skipUpdate) {
      var isUrlMode = this.urlRadioNode.checked;

      function _setState(elm, enabled) {
        if (enabled) {
          elm.removeAttribute("disabled");
        } else {
          elm.setAttribute("disabled", true);
        }
      }

      _setState(this.urlNode, isUrlMode ? true : false);
      _setState(this.urlLoadButtonNode, isUrlMode ? true : false);
      _setState(this.catalogBrowseButtonNode, isUrlMode ? false : true);

      if (!skipUpdate) {
        // tell callback the radio changed
        if (isUrlMode) {
          this.item.callback(null);
        } else {
          this.item.callback(null, true);
        }
      }
    },

    onKeyPress: function(e) {
      if (!this.item.callback) {
        return;
      }

      if (e.target == this.urlNode) {
        if (e.which == 13) {
          this.item.callback(this.urlNode.value);
        }
      }
    },

    onBlur: function(e) {
      this.loadButtonClick();
    },

    loadButtonClick: function(e) {
      if (!this.item.callback) {
        return;
      }

      if (this.urlRadioNode.checked) {
        this.callCallback(this.urlNode.value);
      }
    },

    callCallback: function(url, catalogid, name) {
      this.messageNode.innerHTML = "<img align='top' src='"+FLOW.baseUrl+"/images/progress17x17.gif'> " + this.i18n.propertyeditor.SOURCE_LOADING;

      this.topics = {};
      this.topics["load"] = dojo.subscribe("flow:"+this.operatorId+":loadURL", this, this.processLoadTopic);

      this.item.callback(url, catalogid, name);
    },

    processLoadTopic: function(data) {
      if (data.success) {
         this.messageNode.innerHTML = this.i18n.propertyeditor.SOURCE_LOAD_FINISH;
      } else if (data.error) {
        if (data.message) {
          this.messageNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt("<div style='color: red'>"+data.message+"</div>");
        } else {
          this.messageNode.innerHTML = "<div style='color: red'>"+this.i18n.propertyeditor.SOURCE_LOAD_ERROR+"</div>";
        }
      }
    },

    loadCatalogClick: function(e) {
      var div = document.createElement("div");
      dojo.body().appendChild(div);

      

      this.dialog = new flow.dijit.Dialog({title: this.i18n.propertyeditor.CATALOG_TITLE, chain: 2, showUnderlay: true, buttons:[{label: this.i18n.labels.BUTTON_OK, onclick: dojo.hitch(this, this._acceptDialog)}, {label: this.i18n.labels.BUTTON_CANCEL, onclick: dojo.hitch(this, this._cancelDialog)}]}, div);

      // show and resize before we add contents
      this.dialog.resize({w: 400, h: 430});
      this.dialog.show();

      div = document.createElement("div");
      this.dialog.containerNode.appendChild(div);

      
      this.catalog = new flow.dijit.Catalog({}, div);
      this.dialog.show();
    },

    resize: function() {
      // IE table fix - the url input field will grow to the size of its value
      // due to some bizarre IE bug.  This "fixes" it by setting width on the
      // input field.
      if (!dojo.isIE) {
        return;
      }

      var width = this.propEditor.captionNode.offsetWidth;
      width -= this.tableNode.rows[1].cells[0].offsetWidth;
      width -= this.tableNode.rows[1].cells[2].offsetWidth;
      dojo.style(this.urlNode, "width", width-58);
    },

    _acceptDialog: function() {
      var id = this.catalog.getSelectedId();
      var name = this.catalog.getSelectedName();
      var link = this.catalog.getSelectedUrl();

      if (id && link) {
        this.callCallback(link, id, name);
        this.catalogNode.value = name;
      }

      this._cancelDialog();
    },

    _cancelDialog: function() {
      this.catalog.destroyRecursive();
      this.catalog = null;

      this.dialog.hide();
      this.dialog.destroyRecursive();
      this.dialog = null;
    }
  }
);


}

if(!dojo._hasResource["flow.dijit.Canvas"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.Canvas"] = true;
dojo.provide("flow.dijit.Canvas");







dojo.declare(
  "flow.dijit.Canvas",
  flow.hale.EditArea,
  {
    // data
    model: null,
    contextMenu: null,
    dialogContainer: null,
    operators: null,

    // internal constants	-- for autoArrange()
    LAYOUTBEINGCOMPUTED:  -1,
    LAYOUTTOBECOMPUTED:   -2,
    LAYOUTSKIPLOOP:       -3,

    operatorDropped: false,
    hasEndpointOperator: false,

    dialogs: [],

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");

      this.inherited("postMixInProperties", arguments);
    },

    postCreate: function() {
      // init the model
      this.model = new flow.Model(this);
      this.operators = [];
      this.inherited("postCreate", arguments);

      // add class ourselves
      dojo.addClass(this.domNode, "flowcanvas");

      // listen for events
      dojo.connect(this.domNode, "onkeydown", this, "onKeyDown");
    },

    loadMashup: function(mashup) {
      this.model.isLoading = true;
      this.model.load(mashup);
      this.model.isLoading = false;
    },

    createOperator: function(type, id, x, y, setFocus) {
      if (!x) {
        x = 0;
      }

      if (!y) {
        y = 0;
      }

      // called when we load a model in
      var props = {__style:{left: parseInt(x, 10)+"px", top: parseInt(y, 10)+"px", position: "absolute"}};

      var widget = this.createComponent("flow.dijit.CanvasOperator", props);
      widget.operatorType = type;
      widget.operatorId = id;

      this.operators[id] = widget;

      // associative array -- has to explicitly increment length
      // used in autoArrange()
      this.operators.length++;

      
      

      var command = new btp.widget.command.CompoundCommand();
      command.append(new btp.widget.command.EditCommand.AppendChildCommand(this, widget));
      this.getEditDomain().executeCommand(command);

      if (setFocus) {
        var editor = btp.getEditors()[0];
        editor.setFocus(widget);
      }

      // init preview
      // TODO: may cause errors, should we try/catch?
      this.model.getOperatorById(id).doPreview();
    },

    createConnection: function(srcId, targetId) {
      var srcWidget = this.operators[srcId];
      var targetWidget = this.operators[targetId];

      var props = {
        sourceAnchor: srcWidget.id+":input",
        targetAnchor: targetWidget.id+":output"
      };

      var connection = this.createComponent("gdc.widget.CurvylineConnection", props);
      var offsetParent = gdl.getOffsetParent(srcWidget, targetWidget);

      if (!offsetParent) {
        offsetParent = this;
      }

      
      var command = new gdl.widget.command.ConnectionCommand.AddConnectionCommand(offsetParent, connection);

      this.getEditDomain().executeCommand(command);
      connection.setMouseOverHandler();
    },

    addOperator: function(widget) {
      // called when a user drag and drops an operator
      var id = this.model.addItem(widget.operatorType);
      this.model.updateCoordinates(id, widget.__style.left, widget.__style.top);
      widget.operatorId = id;
      this.operators[id] = widget;

      // associative array -- has to explicitly increment length
      // used in autoArrange()
      this.operators.length++;

      // show the property editor
      this.showPropertyEditorForNewOperator(id)
 
      return id;
    },

    createPublishOperator: function() {
      var node = this.model.getPublishNode();

      // create model operator
      this.model.createPublishOperator();

      var x = parseInt(node.getAttribute("x"), 10);
      var y = parseInt(node.getAttribute("y"), 10);

      if (!!x && !!y) {
        // calculate position - width/height of op is 140/51
        var mb = dojo.marginBox(this.domNode);
        x = mb.w - 140 - 20; // 20 to make it not be at the complete right hand side
        y = (mb.h / 2) - (51/2);
      }

      // create visual operator
      this.createOperator("publish", node.getAttribute("id"), x, y);
    },

    showPropertyEditorForNewOperator: function(id) {
      if (!this.operatorDropped) {
        this.showPropertyEditor(id);
        this.operatorDropped = true;
      } else {
        if (this.dialogs["property"]) {
          var widget = this.dialogs["property"].widget;
          // update visible property editor
          if (widget.isVisible()) {
            this.showPropertyEditor(id, true);
          }
        }
      }
    },

    connectWidgets: function(sourceWidget, targetWidget) {
      // TODO: checks

      this.model.connectOperators(sourceWidget.operatorId, targetWidget.operatorId);
      this.updateDialogs(targetWidget.operatorId);

      this.model.getOperatorById(sourceWidget.operatorId).connectedToTarget(true);
    },

    disconnectWidgets: function(sourceWidget, targetWidget) {
      this.disconnectWidgetsById(sourceWidget.operatorId, targetWidget.operatorId);
    },

    disconnectWidgetsById: function(srcid,targetid) {
      // TODO: checks
      // TODO: ask user if he is sure, given there are params/conditions

      // call model
      this.model.disconnectOperators(srcid, targetid);
      this.updateDialogs(targetid);
    },

    showPropertyEditor: function(operatorId, forceReload) {
      var operator = this.model.getOperatorById(operatorId);
      if (!operator) {
        // TODO: throw?
        return;
      }

      
      var locale = dojo.i18n.getLocalization("flow", "propertyeditor");

      var caption = operator.getName();
      var title = dojo.string.substitute(locale.PROPERTYEDITOR_TITLE, [operator.getLabel(), caption]);

      if (this.dialogs["property"]) {
        // check if same operator! and the property dialog is open
        if (!forceReload && this.dialogs["property"].propertyEditor && this.dialogs["property"].propertyEditor.operatorId == operator.id && this.dialogs["property"].widget.isVisible()) {
          return;
        }

        if (this.dialogs["property"].propertyEditor) {
          this.dialogs["property"].propertyEditor.destroyRendering();
          this.dialogs["property"].propertyEditor = null;
        }

        if (this.dialogs["property"].widget) {
          // move domNode
          var node = dojo.body().removeChild(this.dialogs["property"].widget.domNode);
          dojo.body().appendChild(node);
          this.dialogs["property"].widget.domNode = node;
        }

        this.dialogs["property"].widget.setTitle(title);
        this.dialogs["property"].widget.show();
      } else {
        
        

        var div = document.createElement("div");
        this.dialogContainer.appendChild(div);

        var widget = new flow.dijit.Dialog({title: title, resizable: true, closeable: true}, div);
        this.dialogs["property"] = {widget: widget};

        // size the dialog - 65% of screen, or 500px at most.
        var box = dojo.contentBox(dojo.body());
        var height = Math.floor(box.h * .65);

        if (height > 500) {
          height = 500;
        }

        widget.resize({w: 550, h: height});
        widget.show();
      }

      
      var div = document.createElement("div");
      this.dialogs["property"].widget.containerNode.appendChild(div);

      this.dialogs["property"].propertyEditor = new flow.dijit.PropertyEditor({operator: this.model.operators[operator.id], canvas: this}, div);
    },

    closePropertyEditor: function(operatorId) {
      if (this.dialogs["property"]) {
        if (this.dialogs["property"].propertyEditor) {
          this.dialogs["property"].propertyEditor.destroyRendering();
          this.dialogs["property"].propertyEditor = null;
        }

        if (this.dialogs["property"].widget) {
          this.dialogs["property"].widget.hide();
        }
      }
    },

    getPropertyEditor: function(operator) {
      return this.dialogs["property"] ? this.dialogs["property"].propertyEditor : null;
    },

    showOperatorHelp: function(operator) {
      var locale = dojo.i18n.getLocalization("flow", "propertyeditor");
      var title = dojo.string.substitute(locale.HELP_TITLE, [operator.getLabel()]);
      var url = FLOW.baseUrl + "/doc/userhelp/editorhelp/help_builder_"+operator.type+".htm";
      this.showHelp(url, title);
    },

    showHelp: function(url, title, chain) {
      if (!this.dialogs["help"]) {
        
        

        var div = document.createElement("div");
        this.dialogContainer.appendChild(div);
        var dialog = new flow.dijit.Dialog({title: title, resizable: true, closeable: true, chain: chain ? chain : null}, div);

        this.dialogs["help"] = {widget: dialog};

        dialog.resize({w: 550, h: 300});
        dialog.show();
      } else {
        if (this.dialogs["help"].iframe) {
          this.dialogs["help"].iframe.parentNode.removeChild(this.dialogs["help"].iframe);
          this.dialogs["help"].iframe = null;
        }

        if (chain) {
          this.dialogs["help"].widget.chain = chain;
          this.dialogs["help"].widget.updateZIndex();
        }

        // move domNode
        var node = dojo.body().removeChild(this.dialogs["help"].widget.domNode);
        dojo.body().appendChild(node);
        this.dialogs["help"].widget.domNode = node;

        this.dialogs["help"].widget.setTitle(title);
        this.dialogs["help"].widget.show();
      }

      if (this.dialogs["property"]) {
        // move help dialog if it has the same x coordinate as the property editor
        var helpx = dojo.style(this.dialogs["help"].widget.domNode, "left");

        var propx = dojo.style(this.dialogs["property"].widget.domNode, "left");

        if (propx == helpx) {
          var helpy = dojo.style(this.dialogs["help"].widget.domNode, "top");
          dojo.style(this.dialogs["help"].widget.domNode, "left", helpx + 55 + "px");
          dojo.style(this.dialogs["help"].widget.domNode, "top", helpy + 35 + "px");
        }
      }

      dojo.style(this.dialogs["help"].widget.containerNode, "overflow", "hidden");
      var iframe = document.createElement("iframe");
      iframe.style.width = "100%";
      iframe.style.height = "100%";
      iframe.style.borderWidth = "0";
      iframe.setAttribute("src", url);
      this.dialogs["help"].iframe = this.dialogs["help"].widget.containerNode.appendChild(iframe);
    },

    closeHelp: function() {
      if (this.dialogs["help"]) {
        this.dialogs["help"].iframe = null;
        this.dialogs["help"].widget.destroyRendering();
        this.dialogs["help"].widget = null;
        this.dialogs["help"] = null;
      }
    },

    updateDialogs: function(operatorId) {
      // don't update if no dialog or it is hidden
      if (!this.dialogs["property"] || (this.dialogs["property"].widget && !this.dialogs["property"].widget.isVisible())) {
        return;
      }

      // called when an operator is changed in a way that might require an
      // update of the visible dialogs
      var prop = this.dialogs["property"].propertyEditor;

      if (prop.operatorId == operatorId) {
        // the operator has a property editor

        // lets check if the property editor is in an error state
        if (prop.hasError) {
          // lets check if the error has been fixed
          if (!prop.checkForError()) {
            // error is gone, re-render the property editor
            this.showPropertyEditor(operatorId, true);
          }
        } else {
          // check if an error has occured, and if so, re-render the property editor
          if (prop.checkForError()) {
            this.showPropertyEditor(operatorId, true);
          }
        }
      }
    },

    destroyDialogsForOperator: function(operatorId) {
      if (this.dialogs["property"]) {
        if (this.dialogs["property"].propertyEditor && this.dialogs["property"].propertyEditor.operatorId == operatorId) {
        try
        {
          this.dialogs["property"].propertyEditor.destroyRecursive();
          this.dialogs["property"].widget.hide();
          this.dialogs["property"].widget.destroyRecursive();
          delete this.dialogs["property"];
         }
         catch(ex){}
        }
      }
    },

    removeOperator: function(operator) {
      if (operator.operatorType == "publish") {
        return;
      }

      if (!confirm(this.i18n.labels.OPERATOR_DELETE_CONFIRM)) {
        return;
      }

      // update hale
      this.deleteFocusWidget();

      // update model
      this.model.disconnectOperator(operator.operatorId)

      var ids = this.model.removeItem(operator.operatorId);
      for (var i = 0; i < ids.length; i++) {
        // show the out connector, as the target was removed
        this.operators[ids[i]].showOutConnector(true);
      }

      delete this.operators[operator.operatorId];

      // associative array -- has to explicitly decrement length
      // used in autoArrange()
      this.operators.length--;

      // destroy any dialogs associated with operator
      this.destroyDialogsForOperator(operator.operatorId);

      // unhook from context menu
      this.removeOperatorFromContextMenu(operator);

      // finally destroy the visual component
      operator.destroyRendering();
    },

    deleteAllOperators: function() {
      this.clear();
      this.closeHelp();
      this.model.clear();

      // associative array -- has to set length to 0
      // used in autoArrange()
      for (var operatorId in this.operators) {
        this.destroyDialogsForOperator(operatorId);
        delete this.operators[operatorId];
      }

      this.operators.length = 0;
      this.hasEndpointOperator = false;
    },

    addOperatorToContextMenu: function(operator) {
      if (!this.contextMenu) {
        this.contextMenu = new dijit.Menu();

        dojo.connect(this.contextMenu, "_openMyself", this, this.buildMenuItems);
      }

      this.contextMenu.bindDomNode(operator.shape.rawNode);
    },

    buildMenuItems: function(event) {
      // make sure we focus the target
      if (event.target.widget) {
        this.setFocus(event.target.widget);
      }

      if (!this.focusWidgets[0]) {
        return;
      }

      var contextmenu = this.contextMenu;
      dojo.forEach(contextmenu.getChildren(), function(child){ contextmenu.removeChild(child); });

      var type = this.model.getOperatorById(this.focusWidgets[0].operatorId).type;

      this.contextMenu.addChild(new dijit.MenuItem({label: this.i18n.labels.CONTEXTMENU_PROPERTIES,
                                                    onClick: dojo.hitch(this, "contextMenuItemClick", "propertyEditor")}));
      this.contextMenu.addChild(new dijit.MenuItem({label: this.i18n.labels.HELP, 
                                                    onClick: dojo.hitch(this, "contextMenuItemClick", "help")}));
      this.contextMenu.addChild(new dijit.MenuSeparator());
      this.contextMenu.addChild(new dijit.MenuItem({label: this.i18n.labels.CONTEXTMENU_DELETE,
                                                    disabled: (type == "publish"),
                                                    iconClass:"dijitEditorIcon dijitEditorIconDelete",
                                                    onClick: dojo.hitch(this, "contextMenuItemClick", "delete")}));

      var thisOperator = this.model.getOperatorById(this.focusWidgets[0].operatorId);
      var type = thisOperator.type;

  	  if (type != "publish") {
        this.contextMenu.addChild(new dijit.MenuSeparator());
        var xmlmodel = thisOperator.model;

        if (xmlmodel.runXPath("//flow:input[@id='"+thisOperator.id+"']").length == 0) {
          // create the submenu
          var pSubMenu = new dijit.Menu({parentMenu: this.contextMenu});

          var outputs = xmlmodel.runXPath("/flow:mashup/flow:flow/flow:*[(name()='operator' or name()='publish') and @id!='"+thisOperator.id+"']");
          for (var i = 0; i < outputs.length; i++) {
            var id = xmlmodel.getValue(outputs[i], "@id");
            var op = this.model.getOperatorById(id);

            var caption = op.getName();

            var maxinputs = op.maximumInputs;
            var actualinputs = xmlmodel.runXPath("flow:input", outputs[i]).length;

            var cyclicConnection = xmlmodel.runXPath("flow:input[@id='"+id+"']", thisOperator.node).length;

            if ((actualinputs < maxinputs || maxinputs == -1) && cyclicConnection == 0) {
              pSubMenu.addChild(new dijit.MenuItem({label: caption, onClick: dojo.hitch(this, "connectionMenuItemClick", id)}));
            }
          }

          // attach submenu to the main context menu
          this.contextMenu.addChild(new dijit.PopupMenuItem({label: this.i18n.labels.OPERATOR_MENU_CONNECTTO, popup: pSubMenu}));
        } else {
          var targetid = xmlmodel.getValue(xmlmodel.flowNode,"//flow:input[@id='"+this.focusWidgets[0].operatorId+"']/../@id");
          this.contextMenu.addChild(new dijit.MenuItem({label: this.i18n.labels.OPERATOR_MENU_DISCONNECT,
                                    onClick: dojo.hitch(this, "disConnectionMenuItemClick", targetid)}));
        }
      }

      this.contextMenu.startup();
    },

    removeOperatorFromContextMenu: function(operator) {
      if (this.contextMenu) {
        this.contextMenu.unBindDomNode(operator.shape.rawNode);
      }
    },

	disConnectionMenuItemClick: function(action){
		this.disconnectWidgetsById(this.focusWidgets[0].operatorId,action);
		this.operators[this.focusWidgets[0].operatorId].showOutConnector(true);
		this.model.getOperatorById(this.focusWidgets[0].operatorId).connectedToTarget(false);

		// remove connections
		var compoundCommand = new btp.widget.command.CompoundCommand();
		var focusWidgets = this.focusWidgets;
		for(var i = 0, len = focusWidgets.length; i < len; i++){
			var focusWidget = focusWidgets[i];
			for(var i = 0; i < 2; i++){
				var anchors = (i == 0) ? focusWidget.sourceConnectionAnchors : focusWidget.targetConnectionAnchors;
				if(i==1) continue; //we dont want to disconnect the inputs
				if(!anchors){ return; }
				for(var anchorName in anchors){
					var listeners = anchors[anchorName].listeners;
					if(!listeners){ continue; }
					for(var j = 0, len = listeners.length; j < len; j++){
						var connectionWidget = listeners[j];
						compoundCommand.append(new btp.widget.command.EditCommand.RemoveChildCommand(connectionWidget));
					}
				}
			}
		}      
		this.getEditDomain().executeCommand(compoundCommand);
	},
	connectionMenuItemClick: function(action){
      this.createConnection(this.focusWidgets[0].operatorId, action);
	
	},
    contextMenuItemClick: function(action) {
      var selected = this.focusWidgets[0];
      if (action == "propertyEditor") {
        this.showPropertyEditor(selected.operatorId);
      } else if (action == "delete") {
        this.removeOperator(selected);
      } else if (action == "help") {
        // need the BaseOperator class
        this.showOperatorHelp(this.model.getOperatorById(selected.operatorId));
      }
    },

    onOperatorCaptionChanged: function(operator, caption) {
      var locale = dojo.i18n.getLocalization("flow", "propertyeditor");
      var title = dojo.string.substitute(locale.PROPERTYEDITOR_TITLE, [operator.getLabel(), caption]);

      this.dialogs["property"].widget.setTitle(title);
      this.operators[operator.id].setLabel(operator.getDisplayName());
    },

    onKeyDown: function(event) {
      if (event && event.keyCode == dojo.keys.DELETE && this && this.focusWidgets[0]) {
        this.removeOperator(this.focusWidgets[0]);
      }
    },

    refreshDirtyInputs: function(operatorId, callback) {
      var count = 0;
      var inputs = this.model.getInputs(operatorId);
      var operator;

      var myThis = this;

      function previewCallback(result) {
        operator.previewCallback(result);
        count++;

        if (count < inputs.length) {
          doPreview();
        } else {
          callback();
        }
      }

      function doPreview() {
        var id = inputs[count].getAttribute("id");
        operator = myThis.model.getOperatorById(id);

        myThis.model.setValue("@dirty", operator.node, "0");

        operator.doPreview(false, previewCallback);
      }

      doPreview();
    },

    autoArrange: function() {

    	if ( this.operators.length == 0 ) return;   // nothing to do

      // step 0 - cleanup or dynamically introduce working data members (properties) into each widget
      for (var operatorId in this.operators) {
		    var operator = this.operators[operatorId];
		    operator.dpCDPxlevel = this.LAYOUTTOBECOMPUTED;
		    operator.dpCDPylevel = this.LAYOUTTOBECOMPUTED;
      }

      // step 1. determine horizontal and vertical level of each widget
      //         Leaf operators (imports) are considered to be of horizontal level 0
      var  maxXLevel = 0;
      for (var operatorId in this.operators) {
		    var operator = this.operators[operatorId];
		    this.computeXLevel( operator );
		    if ( operator.dpCDPxlevel > maxXLevel ) maxXLevel = operator.dpCDPxlevel;
      }

    	// step 2. create an array to keep track of the next Y position for a given column i.e. X Level
      var  nextYLevel = new Array();
      for ( var i = 0 ;  i <= maxXLevel  ;  i++ )  nextYLevel[i] = 0;

      // setp 3. find a root (because of single out, we always have a tree) i.e. widget
      // with maximum X level, among widgets with not yet computed Y level
      while ( true ) {
		    var rootOperator = this.findXRoot( );
		    if ( rootOperator != null ) {
				rootOperator.dpCDPylevel = nextYLevel[ rootOperator.dpCDPxlevel ]++;
			    this.computeYLevel( nextYLevel, rootOperator );
        } else {
          break;
        }
      }

      // step 4. calculate and update top and left coordinates of each widget
      this.actualLayout( maxXLevel );
      this.adjustCanvasSize();

      // step 5. refresh bird view
      //this.firePropertyChanged("@ForceRefresh");
      var outlineView = this.getContentOutlinePage();
      if (outlineView != null) {
        outlineView.refreshOutline();
      }
 
    },

    computeXLevel: function( operator ) {
		if ( operator.dpCDPxlevel == this.LAYOUTTOBECOMPUTED ) {

		    operator.dpCDPxlevel = this.LAYOUTBEINGCOMPUTED;  // guard to stop "loops"

		    // recursively computeXLevel of upstream operators and take the max
		    var  maxChildLevel = -1;   // we always add 1 ====> xlevel >= 0

		    var  inputsXML   = this.model.getInputs(operator.operatorId);
		    var  upstreamOps = [];
		   	for (var i = 0  ;  i < inputsXML.length  ;  i++ )
		   	{
		   		upstreamOps[i] = this.operators[this.model.getValue(inputsXML[i], "@id")];
		   	}
		   	
		    for ( var i = 0  ;  i < upstreamOps.length  ;  i++ ) {
		        var  n = this.computeXLevel(  upstreamOps[i] );
		        if ( n == this.LAYOUTSKIPLOOP ) continue;  // skip and break the cycle
		        if ( n > maxChildLevel ) maxChildLevel = n;
		    }
		    operator.dpCDPxlevel = maxChildLevel + 1;
		    return operator.dpCDPxlevel; 

		} else if ( operator.dpCDPxlevel == this.LAYOUTBEINGCOMPUTED ) {
		    return this.LAYOUTSKIPLOOP;  // throw exception instead ????
		} else
		    return operator.dpCDPxlevel;
	},
	
	findXRoot: function(  ) {
		var maxLevel   = -1;   // level starts from 0, initialize to value less than 0
		var rootOperator = null;
		for ( var operatorId in this.operators ) {
			var	operator = this.operators[operatorId];

			if ( operator.dpCDPylevel != this.LAYOUTTOBECOMPUTED )  continue;  // skip computed entries
			
			if ( operator.dpCDPxlevel > maxLevel ) {
				maxLevel = operator.dpCDPxlevel;
			    rootOperator = operator;
			}
		}

		return rootOperator;
	},
	
	// set upstream widget's YLevel - for XLevel we set out own level based on upstream ops XLevel
	computeYLevel: function( nextYPosition, operator ) {

		var  inputsXML   = this.model.getInputs(operator.operatorId);
		var  upstreamOps = [];
		for (var i = 0  ;  i < inputsXML.length  ;  i++ )
	   	{
	   		upstreamOps[i] = this.operators[this.model.getValue(inputsXML[i], "@id")];
		}

		for ( var i = 0  ;  i < upstreamOps.length  ;  i++ ) {
		    if ( upstreamOps[i].dpCDPylevel != this.LAYOUTTOBECOMPUTED ) continue; // skip already assigned opers.  Happens because ofloop
		    // fix-up x level making all upstream operators 1 level away
		    upstreamOps[i].dpCDPxlevel = operator.dpCDPxlevel - 1;
		    upstreamOps[i].dpCDPylevel = nextYPosition[ operator.dpCDPxlevel - 1 ]++;
		    this.computeYLevel(  nextYPosition, upstreamOps[i] );
		}
	},

    actualLayout: function( maxXLevel ) {
      // layout one "column" (same x level) at a time from left to right
      var xMargin   = 135;
      var yMargin   = 51;

      var xPosition = 10;
      for ( var n = 0  ;  n <= maxXLevel   ;  n++ ) {

          for (var operatorId in this.operators) {
              var   operator = this.operators[operatorId];
              if ( operator.dpCDPxlevel != n || (operator.operatorType == "publish" && this.model.getPublishInputs().length == 0)) continue; // skip

              var yPosition = (2 * operator.dpCDPylevel + 1)  * yMargin;

              var props = {};
              props.left = xPosition;
              props.top = yPosition;
              this.model.updateCoordinates(operatorId, xPosition, yPosition);    		
            operator.update(props);
          }
          xPosition += xMargin;
      }
    },

    uninitialize: function() {
      if (this.contextMenu) {
        this.contextMenu.destroyRendering();
        this.contextMenu = null;
      }
    }
  }
);

}

if(!dojo._hasResource["gdc.widget.AbstractNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.AbstractNode"] = true;
dojo.provide("gdc.widget.AbstractNode");



dojo.declare(
	"gdc.widget.AbstractNode",
	gdc.widget.GfxWidget,
{
	left: 0, // Integer
	top: 0, // Integer
	width: 200, // Integer
	height: 150, // Integer
	sourceAnchor: "", // anchor class name (ex. "gdc.widget.anchor.ChopboxAnchor")
	targetAnchor: "", // anchor class name (ex. "gdc.widget.anchor.ChopboxAnchor")
	sourceAnchorName: "base", // anchor name
	targetAnchorName: "base", // anchor name

	sourceAnchorObj: null, // anchor object
	targetAnchorObj: null, // anchor object

	postMixInProperties: function(){
		if(this.sourceAnchor || this.targetAnchor){
			// mixin connectable.js to make this widget a connectable node
			dojo["require"]("gdc.widget.support.connectable");
			var props = dojo.getObject("gdc.widget.support.connectable");
			dojo.mixin(this, props);
		}
	},

	postCreate: function(){
		this.domNode.style.position = "absolute";
		this.domNode.style.zIndex = "-1"; // dummy domNode must be lower than shape nodes to receive events from shape nodes.
		this.domNode.style.top = this.top + "px";
		this.domNode.style.left = this.left + "px";
		this.domNode.style.width = this.width + "px";
		this.domNode.style.height = this.height + "px";
		this.inherited("postCreate", arguments);
	},
	
	createShape: function() {
		this.inherited("createShape", arguments);
		
		//@@must improve later@@ (should be implemented in gdl)
		if(this.domNode.dropTarget){
			this.domNode.dropTarget.isAcceptable = function(ds, dt) {
				var dsWidget = ds.data;
				var dtWidget = dt.data;
				
				if(ds.type != "connection") {
					if(!dsWidget && !dtWidget){ return true; }
					for(var w = dtWidget; w; w = w.parent){
						if(w == dsWidget){
							return false;
						}
					}
					return true;
				} else {
					if(dtWidget.isTargetConnectionAcceptable(dsWidget)) return true;
					else return false;
				}
			}
		}
	},	
	
	// subclass must implement
	getShapeInfo: function(){
		return null;
	},

	// subclass's createTargetShape() should call this after the target shape is created
	createConnectionAnchors: function(){
		if(this.sourceAnchor){
			dojo["require"](this.sourceAnchor);
			var anchorImpl = dojo.getObject(this.sourceAnchor);
			this.sourceAnchorObj = new anchorImpl(this);
			this.registerSourceConnectionAnchor(this.getShape().rawNode, this.sourceAnchorName, this.sourceAnchorObj);
		}
		if(this.targetAnchor){
			dojo["require"](this.targetAnchor);
			var anchorImpl = dojo.getObject(this.targetAnchor);
			this.targetAnchorObj = new anchorImpl(this);
			this.registerTargetConnectionAnchor(this.getShape().rawNode, this.targetAnchorName, this.targetAnchorObj);
		}
	},

	// subclass may override
	update: function(props){
		this.inherited("update", arguments);
		if(!this.shape){ return; }
// TODO: Is this really needed?
//		if(dojo.isMozilla){
//			this.shape.rawNode.offsetWidth = this.width;
//			this.shape.rawNode.offsetHeight = this.height;
//		}
		this.setPosition();
		this.setShape();
	},

	setPosition: function(){
		// Respect the DOM information. If this node is dropped on the canvas
		// from the palette, only the DOM information is updated.
		this.top = this.domNode.offsetTop;
		this.left = this.domNode.offsetLeft;
		this.width = this.domNode.offsetWidth;
		this.height = this.domNode.offsetHeight;

		var shape = this.getShape();
		if(!shape){ return; }
		var dx = this.left;
		var dy = this.top;
		shape.setTransform({ dx:dx, dy:dy });
		if(this.nodeMoved){
			this.nodeMoved(this);
		}
	},

	setShape: function(){
		var shapeInfo = this.getShapeInfo();
		if(!shapeInfo){ return; }
		var current = this.shape.getShape();
		if(current == null) return null;	// Group
		
		for(var prop in shapeInfo){
			if(shapeInfo[prop] != current[prop]){
				// something has been changed
				return this.shape.setShape(shapeInfo);
			}
		}
		return this.shape;
	},

	nodeMoved: function(/*Widget*/widget){
	},

// ======================= Setters/Getters =================================
	getTop: function(){
		return this.top;
	},

	setTop: function(/*Integer*/top){
		this.top = top;
		this.domNode.style.top = top + "px";
		this.setPosition();
	},


	getLeft: function(){
		return this.left;
	},

	setLeft: function(/*Integer*/left){
		this.left = left;
		this.domNode.style.left = left + "px";
		this.setPosition();
	},


	getWidth: function(){
		return this.width;
	},
	
	setWidth: function(/*Integer*/width){
		this.width = width;
		this.domNode.style.width = width + "px";
		this.update();
	},


	getHeight: function(){
		return this.height;
	},
	
	setHeight: function(/*Integer*/height){
		this.height = height;
		this.domNode.style.height = height + "px";
		this.update();
	},
	
	// subclass may override
	// Returns true if this node accepts connections of which this is the source.
	isTargetConnectionAcceptable: function(sourceNode/*Widget*/) {
		if(this.maxTargetConnections > -1) {
			if(this.getTargetConnections().length < this.maxTargetConnections) return true;
			else return false;
		} else return true;
	},
	
	// subclass rarely override
	// Returns true if user can start dragging a sourceAnchor to create connections
	canCreateSourceConnection: function() {
		if(this.maxSourceConnections > -1) {
			if(this.getSourceConnections().length < this.maxSourceConnections) return true;
			else return false;
		} else return true;
	},
	
	// === Events ===
	onSourceConnectionAdded: function(connection) {
	},
	onTargetConnectionAdded: function(connection) {
	},
	onRemovingSourceConnection: function(connection) {
	},
	onRemovingTargetConnection: function(connection) {
	}
});

}

if(!dojo._hasResource["gdc.widget.support.connectable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.support.connectable"] = true;
dojo.provide("gdc.widget.support.connectable");

// summary:
//		Make a gfx node widget connectable.
//		This class is intended to be mixed into a gfx node widget such as Rect.
//		'this' points to a gfx node widget.

gdc.widget.support.connectable = {
	sourceConnectionAnchors: null, // hash
	targetConnectionAnchors: null, // hash
	
	maxSourceConnections: -1,
	maxTargetConnections: -1,

	registerSourceConnectionAnchor: function(/*Element*/elem, /*String*/anchorName, /*Anchor*/anchor){
		if(!this.sourceConnectionAnchors){ this.sourceConnectionAnchors = {}; }
		this.sourceConnectionAnchors[anchorName] = anchor;
		anchor.anchorName = anchorName;
		if(!elem.anchorNames){ elem.anchorNames = []; } // [custom property]
		elem.anchorNames.push(anchorName); // [custom property]
	},

	getSourceConnectionAnchor: function(/*Element*/clickedElement){
		var rootNode = this.getShape().rawNode;
		for(var elem = clickedElement; elem; elem = elem.parentNode){
			if(elem.anchorNames){
				for(var i = 0, len = elem.anchorNames.length; i < len; i++){
					if(this.sourceConnectionAnchors[elem.anchorNames[i]]){
						return this.sourceConnectionAnchors[elem.anchorNames[i]];
					}
				}
			}
			if(elem == rootNode){ return null; }
		}
		return null;
	},

	getSourceConnectionAnchorByName: function(/*String*/anchorName){
		return this.sourceConnectionAnchors[anchorName];
	},

	getSourceConnectionAnchors: function(){
		return this.sourceConnectionAnchors;
	},


	registerTargetConnectionAnchor: function(/*Element*/elem, /*String*/anchorName, /*Anchor*/anchor){
		if(!this.targetConnectionAnchors){ this.targetConnectionAnchors = {}; }
		this.targetConnectionAnchors[anchorName] = anchor;
		anchor.anchorName = anchorName;
		if(!elem.anchorNames){ elem.anchorNames = []; } // [custom property]
		elem.anchorNames.push(anchorName); // [custom property]
	},

	getTargetConnectionAnchor: function(/*Element*/clickedElement){
		var rootNode = this.getShape().rawNode;
		for(var elem = clickedElement; elem; elem = elem.parentNode){
			if(elem.anchorNames){
				for(var i = 0, len = elem.anchorNames.length; i < len; i++){
					if(this.targetConnectionAnchors[elem.anchorNames[i]]){
						return this.targetConnectionAnchors[elem.anchorNames[i]];
					}
				}
			}
			if(elem == rootNode){ return null; }
		}
		return null;
	},

	getTargetConnectionAnchorByName: function(/*String*/anchorName){
		return this.targetConnectionAnchors[anchorName];
	},

	getTargetConnectionAnchors: function(){
		return this.targetConnectionAnchors;
	},
	
	getSourceConnections: function() {
		var connections = [];
		for(var anchorName in this.sourceConnectionAnchors) {
			var anchor = this.sourceConnectionAnchors[anchorName];
			if(anchor.listeners) {
				for(var j=0; j<anchor.listeners.length; j++) {
					var con = anchor.listeners[j];
					if(!con.isGhost) { connections.push(con); }
				}
			}
		}
		return connections;
	},
	
	getTargetConnections: function() {
		var connections = [];
		for(var anchorName in this.targetConnectionAnchors) {
			var anchor = this.targetConnectionAnchors[anchorName];
			if(anchor.listeners) {
				for(var j=0; j<anchor.listeners.length; j++) {
					var con = anchor.listeners[j];
					if(!con.isGhost) connections.push(con);
				}
			}
		}
		return connections;
	}
};

}

if(!dojo._hasResource["gdc._base.shape"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc._base.shape"] = true;
dojo.provide("gdc._base.shape");




gdc._base.shape.getClipPathId = function(width, height, dx, dy){
	var id = "w"+width+"h"+height;
	if(dx){
		id += "x"+dx+"y"+dy;
	}
	return id;
}

gdc._base.shape.getClipPath = function(width, height, dx, dy){
	var id = gdc._base.shape.getClipPathId(width, height, dx, dy);
	var editor = btp.getEditors()[0];	
	var defNode = editor.surface.defNode;
	var clipPath = document.getElementById(id);
	if(!clipPath){
		clipPath = dojo.doc.createElementNS(dojox.gfx.svg.xmlns.svg, "clipPath");
		clipPath.setAttributeNS(null, "id", id);
		defNode.appendChild(clipPath);
		var rect = dojo.doc.createElementNS(dojox.gfx.svg.xmlns.svg, "rect");
		height += 20;	// temporary hack
		rect.setAttributeNS(null, "width", width);
		rect.setAttributeNS(null, "height", height);
		if(dx){
			rect.setAttributeNS(null, "transform", "translate("+dx+","+(dy-20)+")");
		}
		clipPath.appendChild(rect);
	}
	return clipPath;
}

gdc._base.shape.Creator = {
	createGdcImage: function(image){ //for using transparent pngs in IE6 
		var ret = this.createImage(image);
		if(dojox.gfx.renderer == "vml"){
			ret.rawNode.style.filter = "";
			var img = ret.rawNode.firstChild;
			var imgSrc = img.src;
			if(imgSrc.match(/png$/i)) {
				var doDomReplace = function() {
					var imgStyle = "display: inline-block;" + img.style.cssText;
					var span = "<span style=\"" 
					 + "width:" + img.width + "px; height:" + img.height + "px;" 
					 + imgStyle + ";"
			         + "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader"
			         + "(src=\'" + img.src + "\', sizingMethod='scale');\"></span>"
					img.outerHTML = span;
				};
				setTimeout(doDomReplace, 0);
			}
		}
		return ret;
	},
	
	createGdcText: function(text){ //for clipping with defined size
		var ret;
		if(dojox.gfx.renderer == "vml"){
			ret = this.createText(text);
			var clipText = "rect(0px "+text.width+"px "+text.height+"px 0px)";
			ret.rawNode.style.clip = clipText;
			return ret;
		} else {
			ret = this.createGroup();
			var clipPathId = gdc._base.shape.getClipPathId(text.width, text.height);
			var clipPath = gdc._base.shape.getClipPath(text.width, text.height);
			ret.rawNode.setAttributeNS(null, "clip-path", "url(#"+ clipPathId +")");
			var textElem = ret.createText(text);
			
			textElem.setTransform = function(transform){
				dojox.gfx.Shape.prototype.setTransform.apply(this, arguments);
				var newClipPathId = gdc._base.shape.getClipPathId(text.width, text.height, transform.dx, transform.dy);
				gdc._base.shape.getClipPath(text.width, text.height, transform.dx, transform.dy);
				ret.rawNode.setAttributeNS(null, "clip-path", "url(#"+ newClipPathId +")");
			}
			return textElem;
		}
		
	}
}
dojo.extend(dojox.gfx.Group, gdc._base.shape.Creator);
dojo.extend(dojox.gfx.Surface, gdc._base.shape.Creator);

// hack for IE performance problem
if(dojox.gfx.renderer == "vml"){
	dojox.gfx.Image.prototype.setShape = function(newShape){
		var shape = this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		var firstChild = this.rawNode.firstChild;
		if(firstChild.tagName == "SPAN"){
			firstChild.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+shape.src+"\', sizingMethod='scale')";
		} else {
			firstChild.src = shape.src;
		}
        if(shape.width || shape.height){
			this.rawNode.style.width = firstChild.style.width  = shape.width;
			this.rawNode.style.height = firstChild.style.height = shape.height;
        }
		return this.setTransform(this.matrix);
	}
	
	dojox.gfx.Image.prototype._applyTransform = function(image){
		var matrix = this._getRealMatrix();
		if(!matrix) return this;
		matrix = dojox.gfx.matrix.multiply(matrix, {dx: this.shape.x, dy: this.shape.y});
		var imgNode = this.rawNode.firstChild;
		var shape = this.shape;
		if(shape.width || shape.height){
			imgNode.style.width = shape.width * matrix.xx;
			imgNode.style.height = shape.height * matrix.yy;
		}
		this.rawNode.style.left = matrix.dx;
		this.rawNode.style.top = matrix.dy;
		return this;			
	}		
}

}

if(!dojo._hasResource["flow.dijit.CanvasOperator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.CanvasOperator"] = true;
dojo.provide("flow.dijit.CanvasOperator");






dojo.declare(
	"flow.dijit.CanvasOperator",
	gdc.widget.AbstractNode,
  {
    // hale stuff
    width: 140,
    height: 51,
    selected: false,
    anchor: "gdc.widget.anchor.FixedAnchor",

    // attach points
    bodyNode: null,
    outConnectorNode: null,
    outConnectorParentNode: null,
    outConnectorDeleteNode: null,
    inConnectorNode: null,

    // data
    type: null,
    operatorType: "",
    operatorId: null,

    // impl

    images: {
             left: {url: "op-left-open.png", selectedUrl: "op-left-open-selected.png", w: 8, h: 40},
             leftNoInput: {url: "op-left-closed.png", selectedUrl: "op-left-closed-selected.png", w: 8, h: 40},
             body: {url: "op-body.png", selectedUrl: "op-body-selected.png", w: 97, h: 40},
             outputLine: {url: "op-output.gif", w: 8, h: 2},
             right: {url: "op-plug.png", w: 20, h: 23},
             reflection: {url: "op-reflection.png", selectedUrl: "op-reflection-selected.png", w: 105, h: 13}
            },

    // can have inputs
    canHaveInputs: true,

    constructor: function(){
      dojo.extend(flow.dijit.CanvasOperator, gdc.widget.support.connectable);
		},

    postCreate: function() {
      this.inherited("postCreate", arguments);
      dojo.subscribe("btp:SelectionChanged", this, "selectionChanged");
    },

    getCanvas: function() {
      return this.getParent();
    },

    getOperator: function() {
      return this.getCanvas().model.operators[this.operatorId];
    },

    buildImageUrl: function(image, selected) {
      var url = "";

      if (selected && this.images[image].selectedUrl) {
        url = this.images[image].selectedUrl;
      } else {
        url = this.images[image].url;
      }

      return this.getImagePath() + "/" + url;
    },

    getImageData: function(name, selected) {
      var image = this.images[name];
      var data = {width: image.w, height: image.h, src: this.buildImageUrl(name, selected)};
      return data;
    },

    createTargetShape: function(parent) {
      var operator = dijit.byId("flowcanvas").model.getOperatorById(this.operatorId);
      this.canHaveInputs = (operator.minimumInputs > 0);
      // how many inputs
      this.maxTargetConnections = operator.maximumInputs;
      this.maxSourceConnections = 1;

      var shape = this.shape = parent.createGroup();

      var leftImage = this.inConnectorNode = shape.createGdcImage(this.getImageData(this.canHaveInputs ? "left" : "leftNoInput"));
      leftImage.setTransform({dx: 0});
      leftImage.rawNode.widget = this;

      var mainImage = this.bodyNode = shape.createGdcImage(this.getImageData("body"));
      mainImage.setTransform({dx:8});
      dojo.connect(this.bodyNode.rawNode, "onclick", this, "titleBarClick");
      this.bodyNode.rawNode.style.cursor = "move";

      // output line
      if (operator.hasOutput) {
        var output = shape.createGdcImage(this.getImageData("outputLine"));
        output.setTransform({dx:105, dy: 20});
      }

      this.reflectionNode = shape.createGdcImage(this.getImageData("reflection"));
      this.reflectionNode.setTransform({dx:0, dy: 40});

      if (operator.hasOutput) {
        var plugImage = this.outConnectorNode = shape.createGdcImage(this.getImageData("right"));
        plugImage.setTransform({dx:112, dy: 9});
        plugImage.rawNode.widget = this;

        plugImage.rawNode.style.cursor = "url('"+FLOW.baseResUrl+"/js/flow/dijit/templates/images/rightHandGrab.cur'), move";

        plugImage.rawNode.getDragTracker = function() {
          return "flow.hale.ConnectionDragCreationTool";
        }

        plugImage.rawNode.getConnectionClass = function(){
          return "gdc.widget.CurvylineConnection";
        };

        // set tooltip
        var labels = dojo.i18n.getLocalization("flow", "labels");
        plugImage.rawNode.setAttribute("title", labels.OPERATOR_PLUG_TOOLTIP);

        dojo.connect(this.outConnectorNode.rawNode, "onmouseover", this, "onConnectorNodeMouseOver");
        dojo.connect(this.outConnectorNode.rawNode, "onmouseout", this, "onConnectorNodeMouseOut");
      }

      
      var labelText = this.labelText = shape.createGdcText({width: 59, height:17, text: operator.getDisplayName()});
      labelText.setFill(new dojo.Color("#000000"));
      labelText.setTransform({dx: 38, dy: 26});
 
      var fontName = "sans-serif"; // fr, de, it, es, pt, ca, da, nl, fi, el, no, sv, tr
      var locale = btpConfig.locale || dojo.locale;
      if(dojo.isFF && (dojo.isFF <= 2)) {
	      if(locale.indexOf("zh") != -1) {
	      	fontName = "SimHei";
	      } else if (locale.indexOf("jp") != -1) {
	      	fontName = "MS UI Gothic";
	      } else if (locale.indexOf("ko") != -1) {
	      	fontName = "Batang";
	      } else if (locale.indexOf("th") != -1) {
	      	fontName = "Tahoma";
	      } else if (locale.indexOf("ar") != -1) {
	      	fontName = "Arial";
	      } else if (locale.indexOf("he") != -1 || locale.indexOf("iw") != -1 ) {
	      	fontName = "Arial";
	      }
      }
      labelText.setFont({size: "10pt", family: fontName});

      labelText.rawNode.style.cursor = "move";
      dojo.connect(this.labelText.rawNode, "onclick", this, "titleBarClick");

      this.icon = shape.createGdcImage({width: 18, height: 19, src: this.getImagePath()+"/op-icon-"+this.operatorType+".png"});
      this.icon.setTransform({dx:13, dy: 11});
      dojo.connect(this.icon.rawNode, "onclick", this, "titleBarClick");

      this.createConnectionAnchors();

      // set label
      shape.rawNode.setAttribute("title", operator.getDisplayName());

      return shape;
    },

    onConnectorNodeMouseOver: function(e) {
      this.outConnectorNode.setShape({src: this.getImagePath() + "/op-plug-hover.png"});
    },

    onConnectorNodeMouseOut: function(e) {
      this.outConnectorNode.setShape({src: this.getImageData("right").src});
    },

    getShapeInfo: function(){
      return {
        type: 'group',
        width: this.getWidth(),
        height: this.getHeight()
      };
    },

    createConnectionAnchors: function() {
      dojo.require(this.anchor);
      var anchorImpl = dojo.getObject(this.anchor);

      // input anchor
      var anchor1 = new anchorImpl(this);
      anchor1.endPointShape = this.bodyNode;
      anchor1.centerOrigin = false;
      anchor1.offsetH = dojo.isIE ? 103 : 104;
      anchor1.offsetV = dojo.isIE ? 19 : 21;
      this.registerSourceConnectionAnchor(this.getShape().rawNode, "input", anchor1);

      // output anchor
      var anchor2 = new anchorImpl(this);
      anchor2.endPointShape = this.inConnectorNode;
      anchor2.centerOrigin = false;
      anchor2.offsetH = 10;
      anchor2.offsetV = 20;
      this.registerTargetConnectionAnchor(this.getShape().rawNode, "output", anchor2);
    },

    // for now, no focus frame
    hlp_setFocusFrame: function(){
    },

    hlp_removeFocusFrame: function(){
    },

    hlp_refreshFocusFrame: function(){
    },

    selectionChanged: function(message) {
      if (message.selection == this) {
        this.selected = true;
        this.inConnectorNode.setShape({src: this.buildImageUrl(this.canHaveInputs ? "left" : "leftNoInput", true)});
        this.bodyNode.setShape({src: this.buildImageUrl("body", true)});
        this.reflectionNode.setShape({src: this.buildImageUrl("reflection", true)});
      } else {
        if(this.selected) {
          this.selected = false;
          this.inConnectorNode.setShape({src: this.buildImageUrl(this.canHaveInputs ? "left" : "leftNoInput", false)});
          this.bodyNode.setShape({src: this.buildImageUrl("body", false)});
          this.reflectionNode.setShape({src: this.buildImageUrl("reflection", false)});
        }
      }
    },

    onSourceConnectionAdded: function(connection) {
      this.showOutConnector(false);

      if (this.getCanvas().model.isLoading) {
        // we are loading, so skip model notification
        return;
      }

      // notify model
      var targetWidget = connection.getTarget();

      this.getCanvas().connectWidgets(this, targetWidget);

      this.getOperator().connectedToTarget(true);
    },

    onRemovingSourceConnection: function(connection) {
      // notify model
      var targetWidget = connection.getTarget();
      this.getCanvas().disconnectWidgets(this, targetWidget);
      this.showOutConnector(true);
      this.getOperator().connectedToTarget(false);
    },

    titleBarMouseDown: function(event) {
      // focus node
      this.domNode.setAttribute("tabIndex", "-1");
      this.domNode.focus();
    },

    titleBarClick: function(event) {
      // if we were not moving the operator, show the property editor
      if (!this.moving) {
        this.getCanvas().showPropertyEditor(this.operatorId);
      } else {
        // since we were moving and now are after the move (click), clear it out
        this.moving = false;
      }
    },

    showOutConnector: function(flag) {
      if (flag) {
        this.outConnectorNode.rawNode.style.display = "";
      } else {
        this.outConnectorNode.rawNode.style.display = "none";
      }
    },

    setLabel: function(label) {
      this.labelText.setShape({text: label});

      // set label
      this.shape.rawNode.setAttribute("title", label);
    },

    hlp_onWidgetDragMove: function(event) {
      this.moving = true;
    },

    hlp_onWidgetDragEnd: function(event) {
      // finished moving - update the model with new coordinates
      this.getCanvas().model.updateCoordinates(this.operatorId, btp.getStyleValue(this, "left"), btp.getStyleValue(this, "top"));
    },

    // called when enter is pressed
    openContextMenu: function() {
      this.titleBarClick();
    },

    isTargetConnectionAcceptable: function(sourceNode) {
      // make sure you can't connect an operator to itself
      if (sourceNode == this){
        return false;
      } else {
        return this.inherited("isTargetConnectionAcceptable", arguments);
      }
    },

    getImagePath: function() {
      return dojo.moduleUrl("flow.dijit", "templates/images")
    },

    uninitialize: function() {
    }
  }
);

}

if(!dojo._hasResource["dijit.layout.AccordionContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.AccordionContainer"] = true;
dojo.provide("dijit.layout.AccordionContainer");








dojo.declare(
	"dijit.layout.AccordionContainer",
	dijit.layout.StackContainer,
	{
		// summary:
		//		Holds a set of panes where every pane's title is visible, but only one pane's content is visible at a time,
		//		and switching between panes is visualized by sliding the other panes up/down.
		// usage:
		// 	<div dojoType="dijit.layout.AccordionContainer">
		// 		<div dojoType="dijit.layout.AccordionPane" title="pane 1">
		// 			<div dojoType="dijit.layout.ContentPane">...</div>
		//  	</div>
		// 		<div dojoType="dijit.layout.AccordionPane" title="pane 2">
		//			<p>This is some text</p>
		// 		...
		// 	</div>

		// duration: Integer
		//		Amount of time (in ms) it takes to slide panes
		duration: 250,

		_verticalSpace: 0,

		postCreate: function(){
			this.domNode.style.overflow="hidden";
			this.inherited("postCreate",arguments); 
			dijit.setWaiRole(this.domNode, "tablist");
			dojo.addClass(this.domNode,"dijitAccordionContainer");
		},

		startup: function(){
			if(this._started){ return; }
			this.inherited("startup",arguments);	
			if(this.selectedChildWidget){
				var style = this.selectedChildWidget.containerNode.style;
				style.display = "";
				style.overflow = "auto";
				this.selectedChildWidget._setSelectedState(true);
			}
		},

		layout: function(){
			// summary
			//		Set the height of the open pane based on what room remains
			// get cumulative height of all the title bars, and figure out which pane is open
			var totalCollapsedHeight = 0;
			var openPane = this.selectedChildWidget;
			dojo.forEach(this.getChildren(), function(child){
				totalCollapsedHeight += child.getTitleHeight();
			});
			var mySize = this._contentBox;
			this._verticalSpace = (mySize.h - totalCollapsedHeight);
			if(openPane){
				openPane.containerNode.style.height = this._verticalSpace + "px";
/***
TODO: this is wrong.  probably you wanted to call resize on the SplitContainer
inside the AccordionPane??
				if(openPane.resize){
					openPane.resize({h: this._verticalSpace});
				}
***/
			}
		},

		_setupChild: function(/*Widget*/ page){
			// Summary: prepare the given child
			return page;
		},

		_transition: function(/*Widget?*/newWidget, /*Widget?*/oldWidget){
//TODO: should be able to replace this with calls to slideIn/slideOut
			if(this._inTransition){ return; }
			this._inTransition = true;
			var animations = [];
			var paneHeight = this._verticalSpace;
			if(newWidget){
				newWidget.setSelected(true);
				var newContents = newWidget.containerNode;
				newContents.style.display = "";

				animations.push(dojo.animateProperty({
					node: newContents,
					duration: this.duration,
					properties: {
						height: { start: "1", end: paneHeight }
					},
					onEnd: function(){
						newContents.style.overflow = "auto";
					}
				}));
			}
			if(oldWidget){
				oldWidget.setSelected(false);
				var oldContents = oldWidget.containerNode;
				oldContents.style.overflow = "hidden";
				animations.push(dojo.animateProperty({
					node: oldContents,
					duration: this.duration,
					properties: {
						height: { start: paneHeight, end: "1" }
					},
					onEnd: function(){
						oldContents.style.display = "none";
					}
				}));
			}

			this._inTransition = false;

			dojo.fx.combine(animations).play();
		},

		// note: we are treating the container as controller here
		_onKeyPress: function(/*Event*/ e){
			if(this.disabled || e.altKey ){ return; }
			var k = dojo.keys;
			switch(e.keyCode){
				case k.LEFT_ARROW:
				case k.UP_ARROW:
				case k.PAGE_UP:
					this._adjacent(false)._onTitleClick();
					dojo.stopEvent(e);
					break;
				case k.RIGHT_ARROW:
				case k.DOWN_ARROW:
				case k.PAGE_DOWN:
					this._adjacent(true)._onTitleClick();
					dojo.stopEvent(e);
					break;
				default:
					if(e.ctrlKey && e.keyCode == k.TAB){
						this._adjacent(e._dijitWidget, !e.shiftKey)._onTitleClick();
						dojo.stopEvent(e);
					}
				
			}
		}
	}
);

dojo.declare(
	"dijit.layout.AccordionPane",
	[dijit.layout.ContentPane, dijit._Templated, dijit._Contained],
{
	// summary
	//		AccordionPane is a ContentPane with a title that may contain another widget.
	//		Nested layout widgets, such as SplitContainer, are not supported at this time.

	templateString:"<div class='dijitAccordionPane'\r\n\t><div dojoAttachPoint='titleNode,focusNode' dojoAttachEvent='ondijitclick:_onTitleClick,onkeypress:_onTitleKeyPress,onfocus:_handleFocus,onblur:_handleFocus'\r\n\t\tclass='dijitAccordionTitle' wairole=\"tab\"\r\n\t\t><div class='dijitAccordionArrow'></div\r\n\t\t><div class='arrowTextUp' waiRole=\"presentation\">&#9650;</div\r\n\t\t><div class='arrowTextDown' waiRole=\"presentation\">&#9660;</div\r\n\t\t><div dojoAttachPoint='titleTextNode' class='dijitAccordionText'>${title}</div></div\r\n\t><div><div dojoAttachPoint='containerNode' style='overflow: hidden; height: 1px; display: none'\r\n\t\tclass='dijitAccordionBody' wairole=\"tabpanel\"\r\n\t></div></div>\r\n</div>\r\n",

	postCreate: function(){
		this.inherited("postCreate",arguments)
		dojo.setSelectable(this.titleNode, false);
		this.setSelected(this.selected);
	},

	getTitleHeight: function(){
		// summary: returns the height of the title dom node
		return dojo.marginBox(this.titleNode).h;	// Integer
	},

	_onTitleClick: function(){
		// summary: callback when someone clicks my title
		var parent = this.getParent();
		if(!parent._inTransition){
			parent.selectChild(this);
			dijit.focus(this.focusNode);
		}
	},

	_onTitleKeyPress: function(/*Event*/ evt){
		evt._dijitWidget = this;
		return this.getParent()._onKeyPress(evt);
	},

	_setSelectedState: function(/*Boolean*/ isSelected){
		this.selected = isSelected;
		dojo[(isSelected ? "addClass" : "removeClass")](this.domNode,"dijitAccordionPane-selected");
		this.focusNode.setAttribute("tabIndex", isSelected ? "0" : "-1");
	},
	
	_handleFocus: function(/*Event*/e){
		// summary: handle the blur and focus state of this widget
		dojo[(e.type=="focus" ? "addClass" : "removeClass")](this.focusNode,"dijitAccordionPaneFocused");		
	},

	setSelected: function(/*Boolean*/ isSelected){
		// summary: change the selected state on this pane
		this._setSelectedState(isSelected);
		if(isSelected){ this.onSelected(); }
	},

	onSelected: function(){
		// summary: called when this pane is selected
	}
});

}

if(!dojo._hasResource["btp.dnd.DragSource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.dnd.DragSource"] = true;
dojo.provide("btp.dnd.DragSource");


dojo.declare("btp.dnd.DragSource", null, {
	disabled: false, // disable this drag source
	dragObject: null,
	dragClone: null,
	offsetParent: null,
	offsetParentCoords: null,
	parentCoords: null,
	refNode: null, // get information (position,top,left) from this node instead of domNode
	positioning: null, // CSS position property of this drag source. "absolute" or "static"
	useCurrentPositioning: true,
	targetShouldShowCaret: false,
	draggingMousePosition: "TopLeft", // "TopLeft" or "ClickedPosition"
	data: null, // any user data associated with this ds. (widget, etc.)

	constructor: function(node, type, data){
		this.data = data;
		var node = dojo.byId(node);
		node.dragSource = this; // [custom property]
		this.domNode = node;
		this.dragObject = node;
		this.refNode = node;
		this.type = type;
		this.offsetParent = dojo.doc.body;
	},

	onDragDown: function(e){
		if(this.useCurrentPositioning){
			this.positioning = dojo.style(this.refNode, "position");
		}
		this.storeMouseDownInfo(e);
		this.initDrag(e);
	},

	onDragStart: function(e){
		this.createDragClone(e);
		return this.dragClone;
	},

	onDragMove: function(e){
		if(this.draggingMousePosition == "TopLeft"){
			this.dragClone.style.top = e.pageY + 4 + "px";
			this.dragClone.style.left = e.pageX + 4 + "px";
		}else if(this.draggingMousePosition == "ClickedPosition"){
			this.dragClone.style.top = e.pageY - this.dragOffset.y + "px";
			this.dragClone.style.left = e.pageX - this.dragOffset.x + "px";
		}
		if(this.dragClone.style.display == "none"){
			this.dragClone.style.display = "";
		}
	},

	onDragEnd: function(e){
		if(e.dragStatus == "dropSuccess"){
			this.removeClone();
		}else if(e.dragStatus == "dropFailure"){
			this.returnClone();
		}else{
			alert("DragSource#onDragEnd: internal error: e.dragStatus is not set");
		}
		this.cleanupDrag();
	},

	unregister: function(){
		this.domNode.dragSource = null; // [custom property]
		this.cleanupDrag();
	},

	reregister: function(){
		this.domNode.dragSource = this; // [custom property]
		this.initDrag();
	},


	storeMouseDownInfo: function(e){
		if(dojo.isIE){
			// take border width into account
			this.layerX = e.layerX + dojo._getBorderExtents(this.domNode).l; //TODO: is this really correct?
			this.layerY = e.layerY + dojo._getBorderExtents(this.domNode).t; //TODO: is this really correct?
		}else{
			this.layerX = e.layerX - 1;
			this.layerY = e.layerY - 1;
		}
		this.pageX = e.pageX;
		this.pageY = e.pageY;
		this.scrollOffset = dojo._docScroll();
		this.dragStartPosition = dojo._abs(this.refNode, true); // {x:#, y:#}
		this.dragStartCssPosition = {
			y: this.refNode.offsetTop,
			x: this.refNode.offsetLeft
		};
		this.dragOffset = {
			y: e.pageY - this.dragStartPosition.y,
			x: e.pageX - this.dragStartPosition.x
		};
		this.offsetParentCoords = dojo._abs(this.offsetParent, true); // {x:#, y:#}

		var parentPosition = dojo._abs(this.domNode.parentNode, true);
		this.parentCoords = {
			y: parentPosition.y - this.offsetParentCoords.y,
			x: parentPosition.x - this.offsetParentCoords.x
		}

		// Users of the DragSource may want to fix the mouse down info
		if(this.fixMouseDownInfo){
			this.fixMouseDownInfo(this);
		}
	},

	initDrag: function(){
		if(dojo.isIE){
			dojo._event_listener.add(this.domNode, "ondragstart", this.cancelEvent);
		}
	},

	cleanupDrag: function(){
		if(dojo.isIE){
			dojo._event_listener.remove(this.domNode, "ondragstart", this.cancelEvent);			
		}
	},

	createDragClone: function(e){
		var dragClone = this.dragClone = this.dragObject.cloneNode(true);
		dojo._setOpacity(dragClone, 0.5);
		dragClone.style.zIndex = 1000;

		var box = dojo.contentBox(this.dragObject);
	    dragClone.style.width = box.w + "px";
	    dragClone.style.height = box.h + "px";
	    dragClone.style.top = this.dragObject.offsetTop + 4 + "px";
	    dragClone.style.left = this.dragObject.offsetLeft + 4 + "px";
	    dragClone.style.position = "absolute";
	    dragClone.style.clear = "both";
		dragClone.style.display = "none";
		this.offsetParent.appendChild(dragClone);
	},

	setDragTarget: function(node){
		this.dragObject = node;
	},

	cancelEvent: function(e){
		e.stopPropagation();
		e.preventDefault();
	},

	removeClone: function(){
		// Use setTime to ensure that any methods that are connected to onDragEnd
		// can safely access the dragClone before the actual removal is performed.
		setTimeout(dojo.hitch(this, "removeClone_1"), 0);
	},

	removeClone_1: function(){
		this.dragClone.parentNode.removeChild(this.dragClone);
		this.dragClone = null;
	},

	returnClone: function(){
		var startCoords = dojo._abs(this.dragClone, true); // {x, y}
		var endCoords = this.dragStartPosition; // {x, y}
//		var endCoords = {
//			left: this.dragStartPosition.x,
//			top: this.dragStartPosition.y
//		};
		var overrunCoords = {
			x: startCoords.x > endCoords.x ? endCoords.x - 10 : endCoords.x + 10,
			y: startCoords.y > endCoords.y ? endCoords.y - 10 : endCoords.y + 10
		}

		var _this = this;
		dojo.fx.slideTo({ node: this.dragClone, top: overrunCoords.y, left: overrunCoords.x, duration: 400,
			onEnd: function(){
				dojo.fx.slideTo({ node: _this.dragClone, top: endCoords.y, left: endCoords.x, duration: 100,
					onEnd: function(){
						_this.dragClone.parentNode.removeChild(_this.dragClone);
						_this.dragClone = null;
						_this.onCloneReturned();
					}
				}).play();
			}
		}).play();
	},

	onCloneReturned: function(){
	},

	globalChangeCursor: function(cursor){
		var sheet = dojo.doc.styleSheets[0];
		if(!sheet){ return };
		if(cursor){
			if(this._showingCursor){
				if(this._showingCursor == cursor){ return; } // already showing
				this.globalChangeCursor(null);
			}
			this._showingCursor = cursor;
			if(dojo.isIE){
				this._ruleIndex = sheet.rules.length;
				sheet.addRule("*", "{cursor:"+cursor+" ! important;}", sheet.rules.length);
				this._prevCursor = dojo.doc.body.style.cursor;
				dojo.doc.body.style.cursor = null; // workaround to update cursor immediately
			}else {
				this._ruleIndex = sheet.cssRules.length;
				sheet.insertRule("*" + "{cursor:"+cursor+" ! important;}", sheet.cssRules.length);
			}
		}else{
			if(!this._showingCursor){ return };
			this._showingCursor = null;
			if(dojo.isIE){
				sheet.removeRule(this._ruleIndex);
				dojo.doc.body.style.cursol = this._prevCursor != null ? this._prevCursor : 'auto';
			}else {
				sheet.deleteRule(this._ruleIndex);
			}
		}
	}
});

}

if(!dojo._hasResource["btp.dnd.DragManager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.dnd.DragManager"] = true;
dojo.provide("btp.dnd.DragManager");

dojo.declare("btp.dnd.DragManager", null, {
	disabled: false, // disable all D&D operations
	currentDragSource: null,
	currentDropTarget: null,
	dropAcceptable: false,
	dropTargets: null,
	dragTriggered: false,
	currentDomElement: null,
	_connectors: null,
	_moveConnector: null,

	_ABSOLUTE_THRESHOLD: 3,
	_STATIC_THRESHOLD: 3,

	constructor: function(){
		this.dropTargets = [];
		this._connectors = [];
		this._connectors.push(dojo.connect(dojo.doc, "onkeydown", this, "onKeyDown"));
		this._connectors.push(dojo.connect(dojo.doc, "onmouseover", this, "onMouseOver"));
		this._connectors.push(dojo.connect(dojo.doc, "onmouseout", this, "onMouseOut"));
		this._connectors.push(dojo.connect(dojo.doc, "onmousedown", this, "onMouseDown"));
		this._connectors.push(dojo.connect(dojo.doc, "onmouseup", this, "onMouseUp"));
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
	},

	onKeyDown: function(e){
		if(e.keyCode == 27){ // ESC
			if(this.currentDragSource){
				this.dropAcceptable = false;
				this.onMouseUp(e);
			}
		}
	},

	onMouseDown: function(e, dragSource){
		if(this.disabled){ return; }
		if(this.dragTriggered){ return; }
		if(dojo.isIE){
			e.pageX -= 2;
			e.pageY -= 2;
		}
		if(this.dragTriggered){ return; } // still dragging
		var ds = this.currentDragSource = dragSource ? dragSource : this.findDragSource(e);
		if(!ds){ return; }
		ds.onDragDown(e);
		for(var i = 0, len = this.dropTargets.length; i < len; i++){
			var dt = this.dropTargets[i];
			if(!dt.disabled){
				dt.onDragDown(e);
			}
		}

		e.preventDefault();
		this._moveConnector = dojo.connect(dojo.doc, "onmousemove", this, "onMouseMove");
	},

	onMouseMove: function(e){
		e.preventDefault(); // this disables selection
		if(dojo.isIE){
			e.pageX -= 2;
			e.pageY -= 2;
		}
		var ds = e.dragSource = this.currentDragSource;
		if(!this.dragTriggered){
			var dx = e.pageX - ds.pageX;
			var dy = e.pageY - ds.pageY;
			var d = Math.sqrt(dx*dx + dy*dy);
			if(ds.positioning == "absolute"){
				if(d < this._ABSOLUTE_THRESHOLD){ return; }
			}else{
				if(d < this._STATIC_THRESHOLD){ return; }
			}

			ds.onDragStart(e);
			for(var i = 0, len = this.dropTargets.length; i < len; i++){
				var dt = this.dropTargets[i];
				if(!dt.disabled){
					e.dropTarget = dt;
					dt.onDragStart(e);
				}
			}

			this.dragTriggered = true;

			// we need a mouseover event here to setup drop target
			this.onMouseOver(e);
		}


		ds.onDragMove(e);

		var dt = this.currentDropTarget;
		if(dt){
			if(this.dropAcceptable){
				e.dropTarget = dt;
				dt.onDragMove(e, this.dragObjects);
			}
		}
		else{
			this.dropAcceptable = false;
		}
	},

	onMouseOver: function(e){
		this.currentDomElement = e.target;

		if(!this.dragTriggered){ return; }

		// find acceptable drop target
		var ds = this.currentDragSource;
		if(!ds){ return; }
		e.dragSource = ds;
		var dt = this.findDropTarget(this.currentDomElement, ds);
		if(dt && ds.domNode == dt.domNode){ return; } // source and target are the same
		if(dt != this.currentDropTarget){
			if(this.currentDropTarget){
				e.dropTarget = this.currentDropTarget;
				this.currentDropTarget.onDragOut(e);
			}
			if(dt){
				e.dropTarget = dt;
				this.dropAcceptable = dt.onDragOver(e);
			}
		}
		this.currentDropTarget = dt;

		/*
		// switch the positioning mode
		if(dt && ds && dt.nodeUnderMouse == ds.domNode){
			if(ds.positioning == "absolute"){
				ds.positioning = "static";
			}else{
				ds.positioning = "absolute";
			}
		}
		*/
	},

	onMouseOut: function(e){
	},

	onMouseUp: function(e){
		if(this.ignoreMouseUp){ return; }
		dojo.disconnect(this._moveConnector);
		if(!this.dragTriggered){ // we are not dragging
			this._resetStatus();
			return;
		}

		for(var i = 0, len = this.dropTargets.length; i < len; i++){
			var _dt = this.dropTargets[i];
			if(!_dt.disabled){
				e.dropTarget = _dt;
				_dt.onDragEnd(e);
			}
		}

		var ds = this.currentDragSource;
		var dt = this.currentDropTarget;
		e.dragSource = ds;
		e.dropTarget = dt;

		if(!ds){
			this._resetStatus();
			return;
		}

		if(dt){
			dt.onDropStart(e);
			if(this.dropAcceptable){
				var ret = dt.onDrop(e);
				dt.onDragOut(e);
				if(ret != undefined && !e.dragStatus){
					e.dragStatus = ret ? "dropSuccess" : "dropFailure";
				}
			}else{
				dt.onDragOut(e);
				e.dragStatus = "dropFailure";
			}
		}
		else{
			e.dragStatus = "dropFailure";
		}

		ds.onDragEnd(e);
		if(dt){
			dt.onDropEnd(e);
		}
		this._resetStatus();
	},

	_resetStatus: function(){
		this.currentDropTarget = null;
		this.currentDragSource = null;
		this.dropAcceptable = false;
		this.dragTriggered = false;
		this.currentDomElement = null;
	},

	findDragSource: function(e){
		for(var node = e.target; node && node != dojo.doc.body; node = node.parentNode){
			if(node.dragSource && !node.dragSource.disabled){
				return node.dragSource;
			}
		}
		return null;
	},

	findDropTarget: function(domElement, ds){
		var dt = null;
		for(var elem = domElement, prev = null; elem != null && elem != dojo.doc.body; elem = elem.parentNode){
			var _dt = elem.dropTarget;
			if(_dt && !_dt.disabled && _dt.accepts(ds)){
				dt = _dt;
				dt.nodeUnderMouse = prev; // direct child of the drop target
				break;
			}
			if(elem.dragSource && !elem.dragSource.disabled){
				prev = elem;
			}
		}
		return dt;
	},

	registerDropTarget: function(dt){
		this.dropTargets.push(dt);
	},

	unregisterDropTarget: function(dt){
		for(var i = 0, len = this.dropTargets.length; i < len; i++){
			if(this.dropTargets[i] == dt){
				this.dropTargets.splice(i, 1);
				break;
			}
		}
	}
});

btp.dnd.dragManager = new btp.dnd.DragManager();

}

if(!dojo._hasResource["btp.widget._PaletteBase"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._PaletteBase"] = true;
dojo.provide("btp.widget._PaletteBase");
dojo.provide("btp.widget._PaletteFolderBase");
dojo.provide("btp.widget._PaletteItemBase");







dojo.declare(
	"btp.widget._PaletteBase",
	[dijit._Widget, dijit._Container],
{
	modelId: '',
	ref: '.',
	label: '',
	lazy: '',

	defaultClickHandler: "btp.widget.tool.DefaultCreationTool",
	defaultDropHandler: "btp.widget.tool.DefaultCreationTool",

	// The default class names for PaletteFolder and PaletteItem.
	// Subclass may want to override these.
	paletteFolderClass: "btp.widget.PaletteFolder",
	paletteItemClass: "btp.widget.PaletteItem",

	i18nPrefix: "Palette_",
	folderTemplate: null,
	itemTemplate: null,
	dragSources: null,
	selectedItem: null,
	toolMap: {},
	_connectors: null,

	constructor: function(){
		this._connectors = [];
		this.dragSources = [];
	},

	postCreate: function(){
		this.domNode.className = "btpPalette";
		this.domNode.style.overflow = "auto"; // to suppress the unnecessary horizontal scrollbar
		if(this.lazy != "true"){
			this.postProcess();
		}
	},

	postProcess: function(){
		if(this._initialized){ return; }
//@TODO		if(!hale.widget.util.common.isShowing(this)){
//@TODO			this._pending_initialization = true;
//@TODO			return;
//@TODO		}
		this._initialized = true;

		this.makeFolderTemplate();
		this.makeItemTemplate();

		if(this.modelId){
			var arr = this.modelId.split(",");
			for(var i = 0, len = arr.length; i < len; i++){
				this.createPalette(arr[i]);
			}
		}
	},

	// override
	show: function(){
		if(!this._initialized && this._pending_initialization)
			this.postProcess(); // lazy initialization
	},

	makeFolderTemplate: function(){
		// subclass must implement
	},

	makeItemTemplate: function(){
		// subclass must implement
	},

	// subclass may want to override this to handle your own XML
	createPalette: function(modelId){
		var model = dijit.byId(modelId);
		var selectedNode = model.selectSingleNodeFromJson("."); // <components>
		var names = {};

		var categoryNodes = btp.getChildNodes(selectedNode, "category");
		for(var i = 0, len = categoryNodes.length; i < len; i++){
			var folderNode = categoryNodes[i]; // <category>
			if(folderNode["display"] == "false"){
				continue;
			}
			var folderIcon = folderNode["icon"];
			var folderDesc = folderNode["description"];
			var folderInitialState = folderNode["initialState"];
			var folder = this.createFolder(folderIcon, folderDesc);

			var componentNodes = btp.getChildNodes(folderNode, "component");
			for(var j = 0, len2 = componentNodes.length; j < len2; j++){
				var componentNode = componentNodes[j]; // </component>
				if(componentNode["display"] == "false"){
					continue;
				}
				var name = componentNode["name"];
				var description = componentNode["description"];
				var icon = componentNode["icon"];
				var clickHandler = componentNode["clickHandler"];
				var dropHandler = componentNode["dropHandler"];

				if(!icon){
					icon = "../hale/src/hale/widget/templates/Palette/noop.gif";
				}
				if(!clickHandler){
					clickHandler = this.defaultClickHandler;
				}
				if(!dropHandler){
					dropHandler = this.defaultDropHandler;
				}

				if(names[name]){
					var k;
					for(k = 2; names[name+":"+k]; k++)
						;
					name = name+":"+k;
				}
				names[name] = true;

				var opt = {
					icon: icon,
					description: description,
					name: name,
					paletteId: this.id,
					clickHandler: clickHandler,
					dropHandler: dropHandler
				};

				var item = this.createItem(opt);
			}
			if(folderInitialState == "closed"){
				folder.folderClickHandler();
			}
		}
	},

	createFolder: function(imagePath, text){
		var opt = {
			icon: imagePath,
			description: text,
			paletteId: this.id
		};

		var impl = dojo.getObject(this.paletteFolderClass);
		var node = new impl(opt);
		this.addChild(node);
		return node;
	},

	createItem: function(opt){
		var impl = dojo.getObject(this.paletteItemClass);
		var node = new impl(opt);
		this.addChild(node);

		var ds = new btp.dnd.DragSource(node.domNode, "component", node);
		ds.targetShouldShowCaret = true;
		dojo.connect(ds, "onDragStart", this, "onDragStart"); // move start
		dojo.connect(ds, "onDragEnd", this, "onDragEnd"); // move end
		ds.offsetParent = dojo.doc.body;
		this.dragSources.push(ds);
		return node;
	},
	
	onDragStart: function(e){
		this._connector = dojo.connect(btp.dnd.dragManager, "onMouseMove", this, "onDragMove"); // moving
		var item = dijit.byNode(e.dragSource.domNode);
		if(item.dropHandler)
			item.changeTool(item.dropHandler);
		else
			item.changeTool(this.defaultDropHandler);
					
	},

	onDragEnd: function(e){
		dojo.disconnect(this._connector); // moving
		this.pushedItem = null;
	},
	
	onDragMove: function(e){
		// someone may want to connect to this...
	},

	nop: function(){
		return false;
	},

	clearPaletteItems: function(){
		for(var i = 0, len = this.dragSources.length; i < len; i++){
			var ds = this.dragSources[i];
			ds.unregister();
			this.dragSources[i] = null;
		}
		this.dragSources = [];
		this.destroyDescendants();
	},
	
	deselect: function(){
		if(this.selectedItem){
			this.selectedItem.flat(this.selectedItem.domNode);
			this.selectedItem = null;
		}
	},


	itemSelected: function(itemWidget, tool){
		var message = {
			type: "PaletteSelectionChangedNotification",
			src: this,
			selection: itemWidget,
			tool: tool
		};
		dojo.publish("btp:PaletteSelectionChanged", [message]);
	},

	itemDeselected: function(itemWidget){
		var message = {
			type: "PaletteSelectionChangedNotification",
			src: this,
			selection: null
		};
		dojo.publish("btp:PaletteSelectionChanged", [message]);
	}
});


dojo.declare(
	"btp.widget._PaletteFolderBase",
	dijit._Widget,
{
	icon: "",
	description: "",
	paletteId: "",

	palette: null,

	buildRendering: function(){
		this.palette = dijit.byId(this.paletteId);
		var div = this.domNode = this.palette.folderTemplate.cloneNode(true);
		var a = div.firstChild;
		a.onclick = this.palette.nop; // to avoid firing the onbeforeunload event (dojo.event.connect doesn't work for this purpose)
		var img = a.firstChild;

		img.src = dojo.moduleUrl("btp", this.icon);
		var label = btp.i18n.gets(this.description, this.palette.i18nPrefix);
		a.appendChild(dojo.doc.createTextNode(label));
	},

	postCreate: function(){
		this._connects = [];
		this._connects.push(dojo.connect(this.domNode, "onmouseover", this, "folderMouseOverHandler"));
		this._connects.push(dojo.connect(this.domNode, "onmouseout", this, "folderMouseOutHandler"));
		this._connects.push(dojo.connect(this.domNode, "onclick", this, "folderClickHandler"));
	},

	destroy: function(){
		dojo.forEach(this._connects, dojo.disconnect);
		this.inherited("destroy", arguments);
	},

	folderClickHandler: function(evt){
		var children = this.palette.getChildren();
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			if(child != this){
				continue;
			}
			for(var j = i + 1; j < len; j++){
				child = children[j];
				if(!(child instanceof btp.widget._PaletteItemBase)){
					break;
				}
				if(child.domNode.style.display == "none"){
					child.domNode.style.display = "";
				}else{
					child.domNode.style.display = "none";
				}
			}
			break;
		}
		return false;
	},

	folderMouseOverHandler: function(evt){
		dojo.removeClass(this.domNode, "btpPaletteFolderLow");
		dojo.addClass(this.domNode, "btpPaletteFolderHi");
	},

	folderMouseOutHandler: function(evt){
		dojo.removeClass(this.domNode, "btpPaletteFolderHi");
		dojo.addClass(this.domNode, "btpPaletteFolderLow");
	}
});



dojo.declare(
	"btp.widget._PaletteItemBase",
	dijit._Widget,
{
	icon: "",
	description: "",
	name: "",
	paletteId: "",
	clickHandler: null,
	dropHandler: null,

	palette: null,

	buildRendering: function(){
		this.palette = dijit.byId(this.paletteId);
		var div = this.domNode = this.palette.itemTemplate.cloneNode(true);
		var a = div.firstChild;
		a.onclick = this.palette.nop; // to avoid firing the onbeforeunload event (dojo.event.connect doesn't work for this purpose)
		var img = a.firstChild;

		var offsetX = 0;
		var offsetY = 0;
		if(this.icon.indexOf(",") != -1){
			var arr = this.icon.split(",");
			this.icon = arr[0];
			offsetX = arr[1] - 0;
			offsetY = arr[2] - 0;
		}

		img.src = dojo.moduleUrl("btp", this.icon);
		var label = btp.i18n.gets(this.description, this.palette.i18nPrefix);
		a.appendChild(dojo.doc.createTextNode(label));

		this.domNode.description = this.description;
		this.domNode.componentClassName = this.name; // ex. "btp.widget.Hello"
	},

	postCreate: function(){
		this.inherited("postCreate", arguments);
		this._connects = [];
		this._connects.push(dojo.connect(this.domNode, "onmouseover", this, "itemMouseOverHandler"));
		this._connects.push(dojo.connect(this.domNode, "onmouseout", this, "itemMouseOutHandler"));
		this._connects.push(dojo.connect(this.domNode, "onmousedown", this, "itemMouseDownHander"));
		this._connects.push(dojo.connect(this.domNode, "onmouseup", this, "itemMouseUpHander"));
	},

	destroy: function(){
		dojo.forEach(this._connects, dojo.disconnect);
		this.inherited("destroy", arguments);
	},

	deselect: function(){
		this.flat(this.domNode);
		this.palette.selectedItem = null;
	},

	itemMouseOverHandler: function(e){
		var div = this.domNode;
		if(this.palette.selectedItem == this){
			;
		}else{
			this.raised(div);
		}
	},

	itemMouseOutHandler: function(e){
		var div = this.domNode;
		if(this.palette.selectedItem == this){
			this.sunken(div);
		}else{
			this.flat(div);
		}
	},

	itemMouseDownHander: function(e){
		var div = this.domNode;
		this.sunken(div);
		if(this.palette.selectedItem && this.palette.selectedItem != this){
			this.flat(this.palette.selectedItem.domNode);
			this.palette.selectedItem = null;
		}
		this.palette.pushedItem = this;
	},

	itemMouseUpHander: function(e){
		if(this.palette.pushedItem != this){
			this.palette.pushedItem = null;
			var div = this.domNode;
			this.raised(div);
			return;
		}
		if(this.palette.selectedItem == this){
			var div = this.domNode;
			this.raised(div);
			this.palette.selectedItem = null;
			this.palette.itemDeselected(this);
			return;
		}
		this.palette.selectedItem = this;
		this.palette.pushedItem = null;
		this.changeTool();
	},

	changeTool: function(tool){
		if(!tool){
			tool = this.clickHandler;
		}
		// Change the current tool to DefaultCreationTool or
		// a tool specified in the component definition file.
		var myTool = this.palette.toolMap[tool];
		if(!myTool) {
			dojo["require"](tool);
			var obj = dojo.getObject(tool);
			myTool = this.palette.toolMap[tool] = new obj();
		}
		this.palette.itemSelected(this, myTool);
	},

	flat: function(div){
		div.style.borderColor = "#ECE9D8";
		div.style.backgroundColor = "#ECE9D8";
		div.style.margin = "0px 1px 1px 0px";
	},

	raised: function(div){
		div.style.borderColor = "white #ACA899 #ACA899 white";
		div.style.backgroundColor = "#ECE9D8";
		div.style.margin = "0px 1px 1px 0px";
	},

	sunken: function(div){
		div.style.borderColor = "#ACA899 white white #ACA899";
		div.style.backgroundColor = "white";
		div.style.margin = "1px 0px 0px 1px";
	}
});

}

if(!dojo._hasResource["flow.dijit.PaletteItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.PaletteItem"] = true;
dojo.provide("flow.dijit.PaletteItem");












dojo.declare(
  "flow.dijit.PaletteItem",
  [btp.widget._PaletteItemBase, dijit._Templated],
  {
    // attach points
    node: null,

    // passed in data
    label: "",
    operatortype: "",

    dragSource: null,

    operatorTitle: null,
    templateString: "<div class='flowPaletteItem' dndType='operator' dojoAttachPoint='node' title='${operatorTitle}'><div class='flowPaletteItemIcon'></div><div class='flowPaletteItemText'><a href='javascript:void(0)'>${label}</a></div></div>",

    buildRendering: function() {
      var i18n = dojo.i18n.getLocalization("flow", "labels");
      this.label = i18n["OPERATOR_"+this.operatortype.toUpperCase()];
      this.operatorTitle = i18n["OPERATOR_"+this.operatortype.toUpperCase()+"_TOOLTIP"];

      dijit._Templated.prototype.buildRendering.apply(this);

      this.domNode.componentClassName = this.name;

      var imgurl = dojo.moduleUrl("flow.dijit", "templates/images") + "/op-icon-"+this.operatortype+".png";
      var div = document.createElement("div");
      div.className = "flowPaletteItemTypeIcon";

      if (dojo.isIE == 6) {
        div.runtimeStyle.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(enabled=true, sizingMethod=fixed src='"+imgurl+"');"
        div.style.backgroundImage = "none";
        // position the image
        div.style.marginLeft = "5px";
        div.style.marginTop = "4px";
      } else {
        div.style.backgroundImage = "url('"+imgurl+"')";
      }

      this.node.firstChild.appendChild(div);
    },

    postCreate: function() {
      this.inherited("postCreate", arguments);
      this._connects.push(dojo.connect(this.domNode, "ondblclick", this, "itemDoubleClickHander"));
    },

    // for double click and create
    itemDoubleClickHander: function(e){
      var editor = btp.getEditors()[0];
      if (this.palette.pushedItem == this) {
        var model = dijit.byId("flowcanvas").model;
        model.createItem(this.operatortype);
      }
      this.palette.pushedItem = null;
      editor.getEditDomain().loadDefaultTool();
    },

    itemMouseUpHander: function(e){
    },

    flat: function(div) {
      dojo.removeClass(this.domNode, "flowPaletteItemHover")
    },

    raised: function(div) {
      // mouse over
      dojo.addClass(this.domNode, "flowPaletteItemHover")
    },

    sunken: function(div) {
    }
  }
);

}

if(!dojo._hasResource["flow.dijit.Palette"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.Palette"] = true;
dojo.provide("flow.dijit.Palette");









dojo.declare(
  "flow.dijit.Palette",
  [btp.widget._PaletteBase],
  {
    accordion: null,
    "class": "flowPalette",

    //defaultClickHandler: "flow.hale.CreationTool",
    defaultDropHandler: "flow.hale.CreationTool",

    postCreate: function() {
      this.postProcess();
      dojo.setSelectable(this.domNode, false);
    },

    resize: function(args) {
      // pass resize to accordion
      this.accordion.resize(args);
    },

    createPalette: function(modelId) {
      var div = document.createElement("div");
      div.style.height = "400px";
      div.style.width = "100%";
      this.domNode.appendChild(div);
			this.accordion = new dijit.layout.AccordionContainer({}, div);

      var model = dijit.byId(modelId);
		  var selectedNode = model.selectSingleNodeFromJson(".");
      var categoryNodes = btp.getChildNodes(selectedNode, "category");

      for (var i = 0; i < categoryNodes.length; i++) {
        var folderNode = categoryNodes[i];
        //var desc = folderNode["description"];
        var i18n = dojo.i18n.getLocalization("flow", "labels");
        var desc = i18n.CATEGORY_DEFAULT;

        var div = document.createElement("div");
        div.style.height = "100%";
        div.style.width = "100%";
        this.accordion.domNode.appendChild(div)
        var container = new dijit.layout.AccordionPane({title: desc, selected: i == 1}, div);

        var componentNodes = btp.getChildNodes(folderNode, "component");
        for (var j = 0; j < componentNodes.length; j++) {
          var component = componentNodes[j];

          if (!component["dropTargets"] || component["operatortype"] == "publish") {
            continue;
          }

          var div = document.createElement("div");
          container.containerNode.appendChild(div);

          var clickHandler = component["clickHandler"];
          var dropHandler = component["dropHandler"];
          if (!clickHandler){
            clickHandler = this.defaultClickHandler;
          }
          if (!dropHandler){
            dropHandler = this.defaultDropHandler;
          }

          var item = new flow.dijit.PaletteItem({palette: this, name: component["name"], operatortype: component["operatortype"], clickHandler: clickHandler, dropHandler: dropHandler}, div);
          var ds = new btp.dnd.DragSource(item.domNode, "component", item);
          ds.targetShouldShowCaret = true;
          dojo.connect(ds, "onDragStart", this, "onDragStart");
          dojo.connect(ds, "onDragEnd", this, "onDragEnd");
          ds.offsetParent = dojo.doc.body;
          this.dragSources.push(ds);
        }

        this.accordion.addChild(container);
      }

      this.accordion.resize();
      this.accordion.startup();
    },

    createOperator: function(type) {
      var model = dijit.byId("flowcanvas").model;
      model.createItem(type);
    }
  }
);

}

if(!dojo._hasResource["btp._base.config"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp._base.config"] = true;
dojo.provide("btp._base.config");

(function(){
	var config = {
		useFocus: true,
		useComponentInfo: true,
		useDropTarget: true,
		i18nClassName: "btp.i18nImpl"
	};

	if(!this["btpConfig"]){
		btpConfig = {};
	}

	for (var key in config){
		if(btpConfig[key] == undefined){
			btpConfig[key] = config[key];
		}
	}
})();

}

if(!dojo._hasResource["btp._base.dom"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp._base.dom"] = true;
dojo.provide("btp._base.dom");

// summary:
//		Utility functions for html/dom manipulation.

dojo.mixin(btp,
{
	getEventCoords: function(e){
		var y = 0, x = 0;
		if(e.pageX){
			y = e.pageY;
			x = e.pageX;
		}else{
			x = e.clientX + dojo.doc.body.scrollLeft - dojo.doc.body.clientLeft;
			y = e.clientY + dojo.doc.body.scrollTop - dojo.doc.body.clientTop;
			if(dojo.doc.body.parentElement && dojo.doc.body.parentElement.clientLeft){
				var parent = dojo.doc.body.parentElement;
				x += parent.scrollLeft - parent.clientLeft;
				y += parent.scrollTop - parent.clientTop;
			}
		}
		return {x:x, y:y};
	},

	getDimensions: function(elem, parent){
		if(!elem.tagName){ alert("btp.getDimensions: internal error"); }
		var top = 0, left = 0;
		var width = elem.offsetWidth; 
		var height = elem.offsetHeight;
		for(; elem != null; elem = elem.offsetParent){
			if(typeof(parent) == "object" && elem == parent
				|| typeof(parent) == "string" && elem.className == parent)
				break;
			top += elem.offsetTop;
			left += elem.offsetLeft;
		}
		return {top:top, left:left, width:width, height:height};
	},

	getChildNodes: function(node, name){
		if(node.nodeName){ // XML
			return node.childNodes;
		}
		var val = node[name];
		if(!val){
			return [];
		}
		if(val.length != undefined){ // array
			return val;
		}else{
			return [val];
		}
	}
});

}

if(!dojo._hasResource["btp._base.style"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp._base.style"] = true;
dojo.provide("btp._base.style");

// summary:
//		Utility functions for style manipulatoin.

dojo.mixin(btp,
{
	hasAccessor: function(widget, property){
		var info = widget.hlp_getComponentInfo();
		if(!info){ return false; }
		var cap = property.charAt(0).toUpperCase() + property.substring(1);
		if(info.accessors && (info.accessors["int"+cap+"Accessor"] || info.accessors[cap+"Accessor"])){
			return true;
		}else{
			return false;
		}
	},

	isResizableHorizontally: function(widget){
		var info = widget.hlp_getComponentInfo();
		if(!info || !info.resizable){
			return false;
		}
		if(!info.resizable.directions){
			return false;
		}
		return info.resizable.directions.indexOf("EAST") != -1
			|| info.resizable.directions.indexOf("WEST") != -1;
	},

	isResizableVertically: function(widget){
		var info = widget.hlp_getComponentInfo();
		if(!info || !info.resizable){
			return false;
		}
		if(!info.resizable.directions){
			return false;
		}
		return info.resizable.directions.indexOf("NORTH") != -1
			|| info.resizable.directions.indexOf("SOUTH") != -1;
	},

	// summary: get style value from an accessor if available or from domNode.
	// widget: target widget
	// property: "top", "left", "width", or "height"
	// return: integer value
	getStyleValue: function(widget, property){
		var info = widget.hlp_getComponentInfo();
		if(info && info.accessors){
			var cap = property.charAt(0).toUpperCase() + property.substring(1);
			if(info.accessors["int"+cap+"Accessor"]){
				var accessor = info.accessors["int"+cap+"Accessor"];
				var cap = accessor.charAt(0).toUpperCase() + accessor.substring(1);
				if(widget["get"+cap]){
					return widget["get"+cap]();
				}else{
					return widget[accessor];
				}
			}else if(info.accessors[property+"Accessor"]){
				var accessor = info.accessors[property+"Accessor"];
				var cap = accessor.charAt(0).toUpperCase() + accessor.substring(1);
				if(widget["get"+cap]){
					return parseInt(widget["get"+cap]());
				}else{
					return parseInt(widget[accessor]);
				}
			}else if(info.accessors[property]){
				return parseInt(info.accessors[property]);
			}
			// through
		}
		// To get the value of top and left, you need to use dojo.style.
		// On FF, widget.domNode.offsetTop/Left always returns zero when the EditArea is hidden.
		if(property == "top"){
			return parseInt(dojo.style(widget.domNode, "top"));
		}else if(property == "left"){
			return parseInt(dojo.style(widget.domNode, "left"));
		}else if(property == "width"){
			return dojo.marginBox(widget.domNode).w;
		}else if(property == "height"){
			return dojo.marginBox(widget.domNode).h;
		}
		return null;
	},

	// summary: set style value through an accessor if available or directly to domNode.
	// widget: target widget
	// property: "top", "left", etc.
	// value: string value with "px". (Ex. "100px")
	setStyleValue: function(widget, property, value){
		var info = widget.hlp_getComponentInfo();
		if(info && info.accessors){
			var cap = property.charAt(0).toUpperCase() + property.substring(1);
			if(info.accessors["int"+cap+"Accessor"]){
				var accessor = info.accessors["int"+cap+"Accessor"];
				cap = accessor.charAt(0).toUpperCase() + accessor.substring(1);
				if(widget["set"+cap]){
					widget["set"+cap](parseInt(value));
				}else{
					widget[accessor] = parseInt(value);
				}
				return;
			}else if(info.accessors[property+"Accessor"]){
				var accessor = info.accessors[property+"Accessor"];
				var cap = accessor.charAt(0).toUpperCase() + accessor.substring(1);
				if(widget["set"+cap]){
					widget["set"+cap](value);
				}else{
					widget[accessor] = value;
				}
				return;
			}
			// through
		}
		widget.domNode.style[property] = value;
	},

/*
	XXgetStyleDimensions: function(widget, parent){
		var widgetDim = btp.getDimensions(widget.domNode);
		var parentDim = btp.getDimensions(parent.domNode);
		var obj = dojo._getBorderExtents(widget.domNode);
		var dim = {
			top: widgetDim.top - parentDim.top, // dy
			left: widgetDim.left - parentDim.left, // dx
			width: btp.getStyleValue(widget, "width")
				 + obj.w,
			height: btp.getStyleValue(widget, "height")
				 + obj.h
		};
		return dim;
	}
*/

	getStyleDimensions: function(widget, parent){
		var dim = btp.getDimensions(widget.domNode, parent.domNode);
		dim.width = btp.getStyleValue(widget, "width");
		dim.height = btp.getStyleValue(widget, "height");
		return dim;
	}
});

}

if(!dojo._hasResource["btp._base.url"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp._base.url"] = true;
dojo.provide("btp._base.url");

// summary:
//		Utility functions for url manipulation.

dojo.mixin(btp,
{
	// See also RepositoryService.java
	splitUrl: function(url){
		// url: http://localhost:8080/btp/repos/kami/led/clock;edit
		var map = {};
		if(!url) //TODO: Do not omit curly braces, '{' and '}'.
			return map;
		if(url.indexOf(';') != -1){
			var frag = url.split(";");
			url = frag[0];
			map["action"] = frag[1];
		}
		else
			map["action"] = "";
		map["url"] = url;
		var pos = url.indexOf("://");
		if(pos == -1) //TODO: Do not omit curly braces, '{' and '}'.
			return map;
		var pos = url.indexOf("/", pos + 3);
		if(pos == -1) //TODO: Do not omit curly braces, '{' and '}'.
			return map;
		map["base"] = url.substring(0, pos + 1); // http://localhost:8080/
		var path = map["path"] = url.substring(pos + 1); // btp/repos/kami/led/clock
		var arr = path.split("/", 5); // Ex. "btp", "repos", "kami", "led", "clock"
		map["contextRoot"] = arr[0]; // btp
		map["pattern"] = arr[1]; // repos
		map["spaceId"] = arr[2]; // kami
		map["applicationId"] = arr[3]; // led
		map["resourceId"] = arr[4]; // clock
		map["key"] = arr[2]+"/"+arr[3]+"/"+arr[4];
		if(map["resourceId"]){
			var suffix = "";
			var idx = map["resourceId"].lastIndexOf('.');
			if(idx != -1){
				suffix = map["resourceId"].substring(idx + 1);
			}
			map["suffix"] = suffix;
		}
		map["list"] = map["base"]+arr[0]+"/"+arr[1]+"/"+arr[2]+"/"+arr[3];
		return map;
	},

	createLabelStringFromUrl: function(url){
		var idx = url.lastIndexOf("/");
		var basename = idx == -1 ? url : url.substring(idx + 1);
		idx = basename.lastIndexOf("=");
		basename = idx == -1 ? basename : basename.substring(idx + 1);
		if(basename.length > 18)
			basename.substring(0, 15)+"...";
		return basename;
	}
});

}

if(!dojo._hasResource["btp._base.utils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp._base.utils"] = true;
dojo.provide("btp._base.utils");

// summary:
//		Utility functions.

dojo.mixin(btp,
{
	weakMixin: function(/*Object*/obj, /*Object*/props){
		// summary:
		//		Copies non-existing properties and methods of props to obj.
		var o = {};
		for(var x in props){
			if((o[x] == undefined || o[x] != props[x])
				&& obj[x] == undefined){
				obj[x] = props[x];
			}
		}
		return obj;
	},

	matchFilter: function(message, filter){
		if(!filter){ return false; }
		if(filter == "*"){ return true; }
		var arr = filter.split(",");
		var id = message.src.domNode.getAttribute("widgetId")
		var type = message.src.declaredClass;
		for(var i = 0, len = arr.length; i < len; i++){
			if(arr[i] == id || arr[i] == type){
				return true;
			}
		}
		return false;
	},

	getWidgetByElement: function(elem){
		while(elem){
			if(elem.widget){ // gfx raw node has the widget property
				return elem.widget;
			}
			var widget = dijit.byNode(elem);
			if(widget){
				return widget;
			}
			elem = elem.parentNode;
			if(!elem || elem.tagName == "BODY"){
				return null;
			}
		}
		return null;
	},

	isOrphan: function(elem){
		for(var n = elem; n; n = n.parentNode){
			if(n.tagName == "BODY")
				return false;
		}
		return true;
	},

	getEditors: function(/*Widget?*/top){
		var editors = [];
		if(top){
			dojo.query("[widgetId]", top.domNode).forEach(function(elem){
				var widget = dijit.byNode(elem);
				if(widget.isEditor){ editors.push(widget); }
			});
		}else{
			dijit.registry.forEach(function(widget){
				if(widget.isEditor){ editors.push(widget); }
			});
		}
		return editors;
	},

	isShowing: function(widget){
		for(var w = widget; w; w = w.getParent()){
			if(dojo.style(w.domNode, "display") == "none"){
				return false;
			}
		}
		return true;
	},

	isGfxWidget: function(widget){
		return (window.gdc && gdc.widget && gdc.widget.GfxWidget
				&& widget instanceof gdc.widget.GfxWidget);
	},

	isGfxLineWidget: function(widget){
		return (window.gdc && gdc.widget && gdc.widget.AbstractLine
				&& widget instanceof gdc.widget.AbstractLine);
	},

	isGfxNodeWidget: function(widget){
		return (window.gdc && gdc.widget && gdc.widget.AbstractNode
				&& widget instanceof gdc.widget.AbstractNode);
	},
	
	getLayoutParent: function(domNode) {
		var node = domNode;
		var layoutParent = null;
		while(node != dojo.body()){
			node = node.parentNode;
			var widget = dijit.byNode(node);
			if(widget && widget.layout) return widget;
		}
		return null;
	}
});

}

if(!dojo._hasResource["btp._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp._base"] = true;
dojo._bodyLtr = true; // To avoid script error in dojo._isBodyLtr. The error sometimes occurs on IE. Maybe some timing issue.
dojo.provide("btp._base");






}

if(!dojo._hasResource["btp.widget.Palette"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.Palette"] = true;
dojo.provide("btp.widget.Palette");
dojo.provide("btp.widget.PaletteFolder");
dojo.provide("btp.widget.PaletteItem");



dojo.declare(
	"btp.widget.Palette",
	btp.widget._PaletteBase,
{
	postCreate: function(){
		this.inherited("postCreate", arguments);
		dojo.setSelectable(this.domNode, false);
	},
	
	makeFolderTemplate: function(){
		// <DIV class="btpPaletteFolder">
		//     <A href="javascript:void(0)"><IMG src="a.gif">label</A>
		// </DIV>
		var div = this.folderTemplate = dojo.doc.createElement("DIV");
		div.className = "btpPaletteCommon btpPaletteFolder btpPaletteFolderLow";
		div.style.whiteSpace = "nowrap";
		var a = dojo.doc.createElement("A");
		a.href = "javascript:void(0)";
		var img = dojo.doc.createElement("IMG");
		img.border = "0";
		a.appendChild(img);
		div.appendChild(a);
	},

	makeItemTemplate: function(){
		var div = this.itemTemplate = dojo.doc.createElement("DIV");
		div.className = "btpPaletteCommon btpPaletteItem";
		div.style.whiteSpace = "nowrap";
		var a = dojo.doc.createElement("A");
		a.href = "javascript:void(0)";
		var img = dojo.doc.createElement("IMG");
		img.border = "0";
		a.appendChild(img);
		div.appendChild(a);
	}
});

dojo.declare(
	"btp.widget.PaletteFolder",
	btp.widget._PaletteFolderBase,
{
});

dojo.declare(
	"btp.widget.PaletteItem",
	btp.widget._PaletteItemBase,
{
	// for click and create
	itemMouseUpHander: function(e){
//		if(this.name looks like a line widget){ return; }
		var editor = btp.getEditors()[0];
		if(this.palette.pushedItem == this){
			var widget = editor.createComponent(this.name);
			dojo["require"]("btp.widget.command.EditCommand");
			var command = new btp.widget.command.EditCommand.AppendChildCommand(editor, widget);
			editor.getEditDomain().executeCommand(command);
			editor.setFocus(widget);
		}
		this.raised(this.domNode);
		this.palette.pushedItem = null;
		editor.getEditDomain().loadDefaultTool();
	}	
});

}

if(!dojo._hasResource["btp.widget.OutlineView"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.OutlineView"] = true;
dojo.provide("btp.widget.OutlineView");



dojo.declare(
	"btp.widget.OutlineView",
	[dijit._Widget, dijit._Contained, dijit._Container],
{
	target: '',
	propertyChangedFilter: '',
	selectionChangedFilter: '',

	editArea: null,
	
	postCreate: function(){
		this.domNode.className = "btpOutlineView btpSimplePane";
		if(this.target){
			this.propertyChangedFilter = this.target;
			this.selectionChangedFilter = this.target;
		}

		this._topics = [
			dojo.subscribe("btp:PropertyChanged", this, "propertyChanged"),
			dojo.subscribe("btp:SelectionChanged", this, "selectionChanged"),
			dojo.subscribe("btp:ActiveEditorChanged", this, "editorActivated"),
			dojo.subscribe("btp:InputChanged", this, "editorActivated")
		];
	},

	destroy: function(){
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	loadContentOutlinePage: function(){
		var contentOutlinePage = this.editArea.getContentOutlinePage()
			|| this.editArea.createContentOutlinePage();
		if(contentOutlinePage){
			contentOutlinePage.setOutlineView(this);
			contentOutlinePage.createOutlinePage();
			contentOutlinePage.refreshOutline();
		}
	},

	// ======== Topic Subscribers ====================================
	editorActivated: function(message){
		var oldPage = this.editArea && this.editArea.getContentOutlinePage();
		if(message.type == "ActiveEditorChangedNotification"){
			if(this.editArea == message.editor){ return; } // already activated
			this.editArea = message.editor; // tab has been changed
			if(oldPage){ oldPage.detachOutline(); }
			var newPage = this.editArea.getContentOutlinePage();
			if(newPage){
				if(newPage.attachOutline()){
					return; // cache exists
				}
			}
		}else if(message.type == "InputChangedNotification"){
			this.editArea = message.src; // document has been loaded
			if(oldPage){ oldPage.detachOutline(); }
		}

		if(this.editArea.getChildren().length == 0){
			// If the editor starts up from scratch, we do not load ContentOutlinePage
			// until propertyChanged or selectionChanged is called.
		}else{
			this.loadContentOutlinePage();
		}
	},

	propertyChanged: function(message){
		if(!btp.matchFilter(message, this.propertyChangedFilter)){
			return; // filter out the message
		}

		var contentOutlinePage = this.editArea.getContentOutlinePage();
		if(!contentOutlinePage){
			this.loadContentOutlinePage();
		}
		contentOutlinePage = this.editArea.getContentOutlinePage();
		if(contentOutlinePage){
			contentOutlinePage.updateOutline(message);
		}
	},

	selectionChanged: function(message){
		if(!btp.matchFilter(message, this.selectionChangedFilter)){
			return; // filter out the message
		}

		var contentOutlinePage = this.editArea.getContentOutlinePage();
		if(!contentOutlinePage && message.selection){
			this.loadContentOutlinePage();
		}
		contentOutlinePage = this.editArea.getContentOutlinePage();
		if(contentOutlinePage){
			contentOutlinePage.selectOutline(message.selection);
		}
	}
});

}

if(!dojo._hasResource["btp.dnd.DragMoveSource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.dnd.DragMoveSource"] = true;
dojo.provide("btp.dnd.DragMoveSource");


dojo.declare("btp.dnd.DragMoveSource", btp.dnd.DragSource, {
	constrained: false,
	constraintBox: null,
	constraintMarginTop: 0,
	constraintMarginLeft: 0,
	
	// override
	onDragMove: function(e){
		var dx = e.pageX - this.pageX;
		var dy = e.pageY - this.pageY;
		var left = this.dragStartCssPosition.x + dx;
		var top = this.dragStartCssPosition.y + dy;

		if(this.constrained) {
			var box = this.constraintBox;
			if(top < box.offsetTop - this.constraintMarginTop){
				top = box.offsetTop - this.constraintMarginTop;
			}
			if(left < box.offsetLeft - this.constraintMarginLeft){
				left = box.offsetLeft - this.constraintMarginLeft;
			}
			var diffTop = box.offsetTop + box.offsetHeight - this.domNode.clientHeight;
			if(top > diffTop - this.constraintMarginTop) {
				top = diffTop - this.constraintMarginTop;
			}
			var diffLeft = box.offsetLeft + box.offsetWidth - this.domNode.clientWidth;
			if(left > diffLeft - this.constraintMarginLeft) {
				left = diffLeft - this.constraintMarginLeft;
			}
		} else {
			//@knitta@ temporary implementation...must fix later
			if(top < 0) {
				top = 0;
			}
			if(left < 0) {
				left = 0;
			}			
		}
		this.dragClone.style.left = left + "px";
		this.dragClone.style.top = top + "px";

		// The following code is equivalent to the above.
		// this.dragClone.style.top = e.pageY - this.offsetParentCoords.y - this.layerY + "px";
		// this.dragClone.style.left = e.pageX - this.offsetParentCoords.x - this.layerX + "px";
	},

	// override
	onDragEnd: function(e){
	},

	// override
	createDragClone: function(e){
		this.dragClone = this.dragObject;
	},
	
	constrainTo: function(div) {
		this.constrained = true;
		if(div){
			this.constraintBox = div;
		} else {
			this.constraintBox = this.domNode.parentNode;
		}
		
	},
	
	setOffsetX: function(diff){
		this.constraintMarginLeft = diff;
	},
	
	setOffsetY: function(diff){
		this.constraintMarginTop = diff;
	}
});

}

if(!dojo._hasResource["btp.widget.ScrollWindow"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.ScrollWindow"] = true;
dojo.provide("btp.widget.ScrollWindow");





dojo.declare(
	"btp.widget.ScrollWindow",
	dijit._Widget,
{
	target: "",
	
	dragSource: null,
	
	parentDiv: null,
	
	_conn: null,
	
	_dragging: false,
	
	postCreate: function(){
		this.domNode.className = "btpScrollWindow";
		this.dragSource = new btp.dnd.DragMoveSource(this.domNode);
		this.dragSource.constrainTo();
		dojo.connect(this.dragSource, "onDragMove", this, "_onWindowScroll");
		dojo.connect(this.dragSource, "onDragEnd", this, "_onWindowScrollEnd");
	},
	
	startup: function() {
		var targetWidget = dijit.byId(this.target);
		if(!targetWidget) return;
		var layoutParent = btp.getLayoutParent(this.domNode);
		if(layoutParent) {
			dojo.connect(layoutParent, "layout", this, "_resizeWindow");
		}
		if(targetWidget.adjustCanvasSize){
			dojo.connect(targetWidget, "adjustCanvasWidth", this, "_resizeWindow");
			dojo.connect(targetWidget, "adjustCanvasHeight", this, "_resizeWindow");
		}
		this._resizeWindow();
		
		dojo.connect(targetWidget.domNode, "onscroll", this, "_onTargetScroll");
		
		var borderLeftWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderLeftWidth);
		var borderTopWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderTopWidth);
		this.dragSource.setOffsetX(borderLeftWidth)
		this.dragSource.setOffsetY(borderTopWidth);
	},
	
	constrainTo: function(div){
		this.parentDiv = div;
		this.dragSource.constrainTo(div);
	},
	
	_resizeWindow: function() {
		var targetWidget = dijit.byId(this.target);
		if(!targetWidget) return;		
		
		var div = this.parentDiv ? this.parentDiv : this.domNode.parentNode;
		var scrollArea = targetWidget.domNode;
		
		var propX = 1.0 * scrollArea.clientWidth / scrollArea.scrollWidth;
		var propY = 1.0 * scrollArea.clientHeight / scrollArea.scrollHeight;
		
		var width = Math.floor(div.clientWidth * propX);
		var height = Math.floor(div.clientHeight * propY);
		
		this.domNode.style.width = width + "px";
		this.domNode.style.height = height + "px";
		
		this._adjustWindowPos();
	},
	
	_onWindowScroll: function(e) {
		this._dragging = true;
		var targetWidget = dijit.byId(this.target);
		if(!targetWidget){ return; }
		
		var div = e.target;
		if(div != this.domNode){ return;}
		var parentDiv = this.parentDiv ? this.parentDiv: this.domNode.parentNode;

		var borderLeftWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderLeftWidth);
		var borderTopWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderTopWidth);
		
		var x = div.offsetLeft - parentDiv.offsetLeft + borderLeftWidth;
		var y = div.offsetTop - parentDiv.offsetTop + borderTopWidth;
		
		var scrollArea = targetWidget.domNode;
		var propX = 1.0 * scrollArea.scrollWidth / parentDiv.offsetWidth;
		var propY = 1.0 * scrollArea.scrollHeight / parentDiv.offsetHeight;

		scrollArea.scrollLeft = Math.floor( x * propX );		
		scrollArea.scrollTop = Math.floor( y * propY );
	},
	
	_onWindowScrollEnd: function(e) {
		this._dragging = false;
	},
	
	_adjustWindowPos: function(){
		var targetWidget = dijit.byId(this.target);
		if(!targetWidget){ return; }
		
		var parentDiv = this.parentDiv ? this.parentDiv : this.domNode.parentNode;
		var target = targetWidget.domNode;
		var x = target.scrollLeft;
		var y = target.scrollTop;
		x *= parentDiv.offsetWidth / target.scrollWidth;
		y *= parentDiv.offsetHeight / target.scrollHeight;
		var borderLeftWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderLeftWidth);
		var borderTopWidth = dojox.gfx.normalizedLength(dojo.getComputedStyle(this.domNode).borderTopWidth);
		this.domNode.style.left = Math.floor(x) + parentDiv.offsetLeft - borderLeftWidth + "px";
		this.domNode.style.top = Math.floor(y) + parentDiv.offsetTop - borderTopWidth + "px";
	},
	
	_onTargetScroll: function(e) {
		if(this._dragging) return;
		this._adjustWindowPos();
	}
});

}

if(!dojo._hasResource["gdl.widget.OutlineView"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.OutlineView"] = true;
dojo.provide("gdl.widget.OutlineView");




dojo.declare(
	"gdl.widget.OutlineView",
	btp.widget.OutlineView,
{
	scrollWindow: null,
	
	_initialize: function() {
		var w = this.scrollWindow = new btp.widget.ScrollWindow({target: this.editArea});
		w.domNode.style.width = "1px";
		w.domNode.style.height = "1px";
		this.domNode.appendChild(w.domNode);
		dojo.setSelectable(this.domNode, false);
	},
	
	loadContentOutlinePage: function(){
		var contentOutlinePage = this.editArea.getContentOutlinePage()
			|| this.editArea.createContentOutlinePage();
		if(contentOutlinePage){
			contentOutlinePage.setOutlineView(this);
			contentOutlinePage.createOutlinePage();
			
			this._initialize();
			this.scrollWindow.constrainTo(contentOutlinePage.canvas.domNode);
			this.scrollWindow.startup();
			contentOutlinePage.refreshOutline();
		}
	}	

});

}

if(!dojo._hasResource["flow.dijit.ShowModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.ShowModel"] = true;
dojo.provide("flow.dijit.ShowModel");













dojo.declare(
  "flow.dijit.ShowModel",
  [dijit._Widget, dijit._Container, dijit._Templated],
  {
    // arguments
    model: null,

    // attach points
    tabcontainerNode: null,
    treeContainerNode: null,
    sourceTextAreaNode: null,

    // localization
    i18n: null,

    widgetsInTemplate: true,

    // impl
    templateString:"<div style=\"width: 100%; height:100%\">\r\n  <div dojoType=\"dijit.layout.TabContainer\" class=\"flowResizable\" dojoAttachPoint=\"tabcontainerNode\" style=\"width:100%; height:100%;\">\r\n\t\t<div dojoType=\"dijit.layout.ContentPane\" title=\"${i18n.labels.MODEL_TAB_TREE}\">\r\n      <div dojoAttachPoint=\"treeContainerNode\"></div>\r\n    </div>\r\n\r\n    <div dojoType=\"dijit.layout.ContentPane\" title=\"${i18n.labels.MODEL_TAB_TEXT}\">\r\n      <textarea dojoAttachPoint=\"sourceTextAreaNode\" dojoAttachEvent=\"onchange:onModelChanged\" style=\"width: 95%; height: 95%;\"></textarea>\r\n    </div>\r\n  </div>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");
    },

    postCreate: function() {
      var tab = this.tabcontainerNode;
      tab.startup();
      tab.resize();

      // setup textarea
      this.sourceTextAreaNode.value = dojox.data.dom.innerXML(this.model.xmlModel);

      new flow.dijit.XMLViewTree({xmlRootNode: this.model.xmlModel.documentElement}, this.treeContainerNode);
    },
    onModelChanged: function()
    {
		this.model.xmlModel= flow.xml.loadDocument(this.sourceTextAreaNode.value);
		var masupnode = flow.xml.loadDocument(this.sourceTextAreaNode.value);
    	var flownode=masupnode.firstChild.childNodes[0];
    	var data=masupnode.firstChild.childNodes[1];
		this.model.load(flownode);
		this.model.removeNodes(".//flow:input[@timestamp='']", this.model.flowNode);
    }
  }
);


}

if(!dojo._hasResource["flow.dijit.FlowContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.FlowContainer"] = true;
dojo.provide("flow.dijit.FlowContainer");




















//






dojo.declare(
  "flow.dijit.FlowContainer",
  [dijit._Widget, dijit._Templated, dijit._Container],
  {
    "class": "flowContainer",

    // the id for the plugin
    flowid: null,

    // base ajax url
    ajaxUrl: null,

    // editor actions
    actions: null,

    hasBeenSaved: false,

    // attach points
    canvas: null,
    layoutcontainer: null,
    dialogContainer: null,
    outlineView: null,
    saveButtonNode: null,
    runAsMenuNode: null,

    i18n: null,

    // entry meta information
    title: "",
    creatorId: "",
    description: "",
    version: "",
    tags: "",
    isNew: false,

    widgetsInTemplate: true,
    templateString:"<div style=\"width: 100%; height: 100%;\">\r\n  <div dojoAttachPoint=\"layoutcontainer\" dojoType=\"dijit.layout.LayoutContainer\" style=\"width: 100%; height: 100%;\">\r\n    <div dojoType=\"dijit.layout.ContentPane\" class=\"flowActionBar\" layoutAlign=\"top\" tabindex=\"0\">\r\n      <div style=\"float:right\">\r\n        <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:onShowHelp\" title=\"${i18n.labels.HELP_BUTTON_TOOLTIP}\">${i18n.labels.HELP}</button>\r\n      </div>\r\n\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"saveButtonNode\" dojoAttachEvent=\"onClick:onSave\" title=\"${i18n.labels.SAVE_BUTTON_TOOLTIP}\">${i18n.labels.SAVE}</button>\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:onRun\" title=\"${i18n.labels.RUN_BUTTON_TOOLTIP}\">${i18n.labels.RUN}</button>\r\n\r\n      <img class=\"flowActionbarSeparator\" src=\"${imagePath}/divider.gif\" align=\"top\" width=\"6\" height=\"22\">\r\n\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:onArrange\" title=\"${i18n.labels.ARRANGE_BUTTON_TOOLTIP}\">${i18n.labels.ARRANGE}</button>\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachEvent=\"onClick:onClear\" title=\"${i18n.labels.CLEAR_BUTTON_TOOLTIP}\">${i18n.labels.CLEAR}</button>\r\n\r\n\t<!-- \r\n      <img class=\"flowActionbarSeparator\" src=\"${imagePath}/divider.gif\" align=\"top\" width=\"6\" height=\"22\">\r\n      <div dojoType=\"dijit.form.ComboButton\" dojoAttachEvent=\"onClick:onRun\">\r\n        <span>${i18n.labels.RUN}</span>\r\n        <div dojoType=\"dijit.Menu\" style=\"display: none;\" dojoAttachPoint=\"runAsMenuNode\">\r\n        </div>\r\n      </div>\r\n      -->\r\n    </div>\r\n    <div dojoType=\"dijit.layout.ContentPane\" layoutAlign=\"client\" style=\"\" tabindex=\"0\">\r\n      <div dojoType=\"dijit.layout.SplitContainer\" orientation=\"horizontal\" sizerWidth=\"7\" activeSizing=\"true\" style=\"border: 1px solid #bfbfbf; width: 100%; height: 100%;\">\r\n        <div dojoType=\"dijit.layout.ContentPane\" sizeMin=\"20\" sizeShare=\"10\">\r\n          <div dojoType=\"dijit.layout.LayoutContainer\" class=\"flowSidebar\" style=\"width: 100%; height: 100%;\">\r\n            <div dojoType=\"flow.dijit.Palette\" modelId=\"${canvasmodel.id}\" layoutAlign=\"client\" sizeShare=\"50\" style=\"width: 100%;\"></div>\r\n            <div dojoType=\"dijit.layout.ContentPane\" layoutAlign=\"bottom\" sizeShare=\"10\" style=\"width: 100%; border-top: 1px solid black;\">\r\n              <div class=\"flowBirdEyeViewTitle\">${i18n.labels.BIRD_VIEW_TITLE}</div>\r\n              <div style=\"height: 120px;\" class=\"flowBirdEyeView\" dojoAttachpoint=\"outlineView\"></div>\r\n            </div>\r\n          </div>\r\n        </div>\r\n        <div dojoType=\"flow.dijit.Canvas\" id=\"flowcanvas\" dojoAttachPoint=\"canvas\" componentInfoId=\"${canvasmodel.id}\" commands=\"*\" sizeMin=\"50\" sizeShare=\"50\"></div>\r\n      </div>\r\n    </div>\r\n  </div>\r\n  <div dojoAttachPoint=\"dialogContainer\"></div>\r\n  <div style=\"display: none; visibility: hidden\">\r\n      <form name=\"flowContainer_runForm1\" \r\n            method=\"post\" \r\n            action=\"\" \r\n            target=\"flowResult1\"> \r\n            <input type=\"hidden\" name=\"xml\" value=\"\" />\r\n      </form>\r\n  </div>\r\n</div>\r\n\r\n",
    imagePath: dojo.moduleUrl("flow.dijit", "templates/images"),

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.labels = dojo.i18n.getLocalization("flow", "labels");

      this.canvasmodel = new btp.widget.Model({src: dojo.moduleUrl("flow", "data/HaleOperators.xml").path});

      /*var doc = this.canvasmodel.getInstanceDocument();
      var elm = doc.createElement("component");
      var opts = [["name", "flow.dijit.CanvasOperator"],["description", "Combine"],["type", "combine"],["clickHandler", "btp.widget.tool.ResizeCreationTool"],["dropTargets", "connection"]];
      for (var i = 0; i < opts.length; i++) {
        var e = doc.createElement(opts[i][0]);
        e.appendChild(doc.createTextNode(opts[i][1]));
        elm.appendChild(e);
      }
      this.canvasmodel.getInstanceDocument().documentElement.firstChild.appendChild(elm);*/

      this.actions = {};
      this.updateActions( this.ajaxUrl );
    },

    postCreate: function() {
      dojo.connect(this.layoutcontainer, "layout", this, "layout");
      //this.canvas.reLayout();

      // we are in an iframe now
      this.canvas.dialogContainer = dojo.body(); //this.dialogContainer;

      // create menu items
//      var menu = this.runAsMenuNode
//      menu.addChild(new dijit.MenuItem({label: this.i18n.labels.RUN_RSS, onClick: dojo.hitch(this, this.onRunAsRSS)}));
//      menu.addChild(new dijit.MenuItem({label: this.i18n.labels.RUN_ATOM, onClick: dojo.hitch(this, this.onRunAsAtom)}));

      // setup outline
      var div = this.outlineView.appendChild(document.createElement("div"));
      //div.style.height = "100%";
      //div.style.width = "100%";
      var outlineview = new gdl.widget.OutlineView({target: "flow.dijit.Canvas", editArea: this.canvas}, div);

      // event catching for key bindings
      dojo.connect(dojo.body(), "onkeypress", this, "onKeyPress");
    },

    onKeyPress: function(event) {
      if (event.ctrlKey) {
        if (event.charCode == 109) {
          // ctrl-m - show model
          this.showModel();
        }
      }
    },

    layout: function() {
      // make sure dialogs are in view
      if (this.canvas.dialogs["property"]) {
        this.fitDialogIntoView(this.canvas.dialogs["property"].widget);
      }

      if (this.canvas.dialogs["help"]) {
        this.fitDialogIntoView(this.canvas.dialogs["help"].widget);
      }

      if (this.helpDialog) {
        this.fitDialogIntoView(this.helpDialog);
      }
    },

    fitDialogIntoView: function(dialog) {
      var top = parseInt(dojo.style(dialog.domNode, "top"), 10);
      if (top < 0) {
        dojo.style(dialog.domNode, "top", "0px");
      }

      // make sure the height is not taller than visible area
      var dialogbox = dojo.marginBox(dialog.domNode);
      var box = dojo.contentBox(dojo.body());

      // TODO: width resizing?
      var height = dialogbox.h
      var width = dialogbox.w
      var needToResize = false;

      if ((dialogbox.t + dialogbox.h) > box.h) {
        height = box.h - dialogbox.t - 10;
        needToResize = true;
      }

      if ((dialogbox.l + dialogbox.w) > box.w) {
        width = box.w - dialogbox.l - 10;
        needToResize = true;
      }

      if (needToResize) {
        dialog.resize({h: height, w: width});
      }
    },

    modelDialog: null,

    // updates the base ajax url and its editor actions
    updateActions: function ( baseAjaxUrl ) {
      if (baseAjaxUrl) {
        var actionList = [ "render", "store", "load", "preview", "execute", "cloneObject", "getObjectList", "close" ];
        this.ajaxUrl = baseAjaxUrl;

        for (var i = 0; i < actionList.length; i++) {
          this.actions[actionList[i]] = baseAjaxUrl + "/method/" + actionList[i];
        }
      }
    },

    getEditorActionUrl: function ( actionName ) {
      return this.actions[actionName];
    },

    onArrange: function() {
      var div = document.createElement("div");
      dojo.body().appendChild(div);
      

      var splashScreen = new flow.dijit.Dialog({title: "Arranging", hideTitleBar: true, hideActionBar: true, resizable: false}, div);
      var ml = "<center><span class='${0}' style='display: ${1}'><img src='${2}' /> ${3}</span></center>";
      var img = FLOW.baseUrl + "/images/spinner.gif";
      var str = dojo.string.substitute(ml, ["", "block", img, this.i18n.labels.ARRANGE_SPLASH_MESSAGE]);
      splashScreen.containerNode.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(str);
      splashScreen.resize({w: 300, h: 30});
      splashScreen.show();

      this.canvas.autoArrange();

      splashScreen.hide();
      splashScreen.destroyRecursive();
      splashScreen = null;
    },

    onClear: function() {
      if (confirm(this.i18n.labels.CLEAR_CONFIRM)) {
        // lets remove all operators
        this.canvas.deleteAllOperators();
      }
    },

    onLoad: function(id) {
      var actionUrl = this.getEditorActionUrl("load");
      var content = { id: id };
      this.callServer(actionUrl, content, dojo.hitch(this, this.loadCallback));
    },

    onSave: function() {
      // check if any operators exist.
      if (this.canvas.model.getOperatorCount() == 0) {
        return;
      }

      // check if publish is connected
      var publishinputs = this.canvas.model.getPublishInputs();
      if (!publishinputs || publishinputs.length == 0) {
        //alert(this.i18n.labels.ERROR_SAVE_PUBLISH_NO_INPUT);
        //return;
      }

      // check if the properties are set
      var isValid = this.canvas.model.isPublishValid();
      if (!isValid) {
        alert(this.i18n.labels.ERROR_SAVE_PUBLISH_DATA_INVALID);
      } else {
        this.doSave();
      }
    },

    doSave: function() {
      // save our plugin specific data
      if (this._hasBeenSaved()) {
      	this.showSaveProgress();
      }

      //this.setPublishInput(null);
      var actionUrl = this.getEditorActionUrl("store");
      var content = {storemode: 'normal', xml: flow.xml.innerXML (this.canvas.model.flowNode)};
      this.callServer( actionUrl, content, dojo.hitch(this, this.saveCallback) );
    },

    showDialog: function(title, buttons, width, height, hideUnderlay) {
      

      var div = dojo.body().appendChild(document.createElement("div"));
      var dialog = new flow.dijit.Dialog({title: title, buttons: buttons, showUnderlay: (hideUnderlay ? false : true), resizable: true, closeable: true}, div);

      if (!width) {
        width = 530;
      }

      if (!height) {
        height = 400;
      }

      dialog.resize({w: width, h: height});
      dialog.show();

      return dialog;
    },

    onShowHelp: function() {
      if (this.helpDialog) {
        this.helpDialog.hide();
        this.helpDialog.destroyRendering();
        this.helpDialog = null;
      }

      this.helpDialog = this.showDialog(this.i18n.labels.BUILDER_HELP_TITLE, null, null, null, true);

      
      var url = FLOW.baseUrl + "/doc/userhelp/editorhelp/help_builder_overview.htm";

      dojo.style(this.helpDialog.containerNode, "overflow", "hidden");
      var iframe = document.createElement("iframe");
      iframe.style.width = "100%";
      iframe.style.height = "100%";
      iframe.style.borderWidth = "0";
      iframe.setAttribute("src", url);
      this.helpDialog.containerNode.appendChild(iframe);
    },

    showSaveProgress: function() {
      
      var div = document.createElement("div");
      dojo.body().appendChild(div);
      this.saveProgressDialog = new flow.dijit.Dialog({resizable: false, closeable: false, hideTitleBar: true, hideActionBar: true}, div);
      this.saveProgressDialog.containerNode.innerHTML = "<center><img src='"+FLOW.baseUrl+"/images/spinner.gif'> "+this.i18n.labels.SAVE_PROGRESS_MSG+"</center>";
      this.saveProgressDialog.domNode.style.width = "300px";
      this.saveProgressDialog.domNode.style.zIndex = "1001";
      this.saveProgressDialog.show();
    },

    finishSave: function() {
    	if (this.saveProgressDialog) {
        this.saveProgressDialog.hide();
        this.saveProgressDialog.destroyRecursive();
        this.saveProgressDialog = null;
    	}
    },

    showSaveStatus: function(status) {
      var button = [{label:this.i18n.labels.BUTTON_OK, onclick: dojo.hitch(this, _acceptDialog)}];
      var dialog = this.showDialog(this.i18n.labels.SAVE, button, 252, 100);

      function _acceptDialog() {
        dialog.hide();
        dialog.destroyRecursive();
        properties = null;
        dialog = null;
      }

      //
      var div = document.createElement("div");
      div.innerHTML = dijit.byId("flowcanvas").model.stripSctipt(status);
      dialog.containerNode.appendChild(div);
    },

    showModel: function() {
      if (this.modelDialog) {
        this.modelDialog.hide();
        this.modelDialog.destroyRecursive();
        this.modelDialog = null;
      }

      this.onShowModel();
    },

    onShowModel: function() {
      this.modelDialog = this.showDialog(this.i18n.labels.MODEL_DIALOG_TITLE);

      

      var div = document.createElement("div");
      this.modelDialog.containerNode.appendChild(div);
      
      new flow.dijit.ShowModel({model: this.canvas.model}, div);
    },

    setPublishInput: function(ptype) {
      this.canvas.model.setPublishInput(ptype);
    },

    onRun: function(type) {
      var funcnode = this.canvas.model.getNode("flow:*[name()='operator' or name()='source']", this.canvas.model.flowNode);

      if (!funcnode) {
        var property = dojo.i18n.getLocalization("flow", "propertyeditor");
	  		alert(property.SERVER_EMPTY_MASHUP);
	  		return;
      }

      this.setPublishInput(null);
      document.flowContainer_runForm1.action = this.getEditorActionUrl("execute");
      document.flowContainer_runForm1.xml.value = flow.xml.innerXML(this.canvas.model.flowNode);

      // NOTE:  second arg cannot have space in between.  Will not work on IE.
      window.open("", "flowResult1", "");

      window.setTimeout("document.flowContainer_runForm1.submit();", 500); 
    },

    onRunAsRSS: function() {
      this.onRun("RSS");
    },

    onRunAsAtom: function() {
      this.onRun("ATOM");
    },

    getObjectMockupXml: function(mashupXml) {
       var xml = "";

       xml += '<OBJECTS><OBJECT>';
       xml += '<OBJECTID>35</OBJECTID>';
       xml += '<TITLE><![CDATA[CD Warehouse Mashup]]></TITLE>'
       xml += '<CREATORID>kfbombar</CREATORID>';
       xml += '<VERSION>1.0</VERSION>';
       xml += '<TIMESTAMP>1197935041</TIMESTAMP>';
       xml += '<DESCRIPTION><![CDATA[Example mashup for kfbombar]]></DESCRIPTION>';
       xml += '<STATUS>NEW</STATUS>';
       xml += '<URL><![CDATA[]]></URL>';
       xml += '<AJAXURL><![CDATA[http://localhost:8080/mashuphub/client/plugin/ajax/pluginid/6/entryid/35]]></AJAXURL>';
       xml += '<VISIBILITY>1</VISIBILITY>';
       xml += '<PUB>1</PUB>';
       xml += '<PARAMETERS/>';
       xml += '<TAGS/>';
       xml += '<FLOW>';
       xml += '<OBJECTID>35</OBJECTID>';
       xml += '<GRAPH><![CDATA[Data for the graph needed]]></GRAPH>';
       xml += '<RUNTIMEMODEL><MASHUP>' + mashupXml +'</MASHUP></RUNTIMEMODEL>';
       xml += '</FLOW>';
       xml += '</OBJECT></OBJECTS>';

       return xml;
    },

    _set_tab_title: function(title) {
       if (FLOW.plugin) {
          FLOW.plugin.tab.setTitle( title );
       }
    },

    _hasBeenSaved: function() {
       return ( FLOW.plugin.workflow.isReedit() || FLOW.plugin.workflow.isSaved() );
    },

    _isNewEntry: function(objectXml) {
       var status = objectXml.childNodes[6].firstChild.nodeValue;

       return (status == "NEW")?true:false;
    },

    loadCallback: function(result) {
      // TODO: use xpath
      var obj = result.documentElement.childNodes[0];
      if (obj.nodeName != "OBJECT") {
        // error ?
        this.finishLoad();
        return;
      }

      // Note:  new entry may contain content (e.g.: clone entry)
      if (this._isNewEntry(obj)) {
        this.isNew = true;
      } else {
        // notify mhub workflow that this is a saved entry.  Do not perform
        // a delete when the tab is closed.
        var title = obj.childNodes[1].firstChild.nodeValue;
        this._set_tab_title(title);
        this.isNew = false;
      }

      // get the flow model
      var graphNode = obj.getElementsByTagName('GRAPH')[0];
      var flowModelNode = "";

      if (graphNode) {
        flowModelNode = graphNode.firstChild;
      }

      // get the generator url
      var mashupurlNode = obj.childNodes[7].firstChild;
      var mashupurl = (mashupurlNode != null)?mashupurlNode.nodeValue:"";

      // get the ajax url
      var ajaxurl = obj.childNodes[8].firstChild.nodeValue;

      this.updateActions(ajaxurl);

      if (flowModelNode) {
         this.canvas.loadMashup(flowModelNode);
      }

	  if(this.isNew)
	  {
			var mashupid= obj.childNodes[0].firstChild.nodeValue;
			this.canvas.model.setValue("/flow:mashup/@id", null, mashupid);
			this.canvas.model.setValue("/flow:mashup/@url", null, mashupurl);
	  }

      this.finishLoad();
    },

    finishLoad: function() {
      // get rid of the splash screen
      if (gSplashDialog) {
      	//console.debug(gSplashDialog);
        gSplashDialog.hide();
        gSplashDialog.destroyRecursive();
        gSplashDialog = null;
      }
    },

    saveCallback: function(result) {
      var obj = result.documentElement.childNodes[0];

      // save entry metadata
      if (FLOW.plugin) {
        // clear protected since our plugin specific data has been saved.
        FLOW.plugin.tab.setProtected(false); 

        if (!this._hasBeenSaved()) {
          FLOW.plugin.done( { action: "save", text: "" } );
        } else {
          // silent save
          this.finishSave();
          //this.showSaveStatus(this.i18n.labels.SAVE_SUCC_MSG);
        }
      }

      this.hasBeenSaved = true;

      //alert( flow.xml.innerXML(result) );
    },

    saveAsCallback: function(result) {
      var obj = result.documentElement.childNodes[0];

      this.hasBeenSaved = true;

      this._set_tab_title( this.title );    // update tab title

      // show save button
      this.saveButtonNode.style.display = "inline";

      // clear protected
      if (FLOW.plugin) {
         FLOW.plugin.tab.setProtected(false); 
      }
    },

    runCallback: function(result) {
      alert( flow.xml.innerXML(result) );
    },

    callServer: function(actionUrl, postContent, callback) {
       dojo.xhrPost( {
           url: actionUrl,
           handleAs: "xml",
           content: postContent,
           contentType: "application/x-www-form-urlencoded; charset=utf-8",            
           load: function(response, ioArgs) {
              callback(response);
              return response;
           },
           error: function(response, ioArgs) {
              callback(null);
              return response;
           }
       });
    }
  }
);

}

if(!dojo._hasResource["flow.dijit.SourceAdvanced"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.SourceAdvanced"] = true;
dojo.provide("flow.dijit.SourceAdvanced");









dojo.declare(
  "flow.dijit.SourceAdvanced",
  [dijit._Widget, dijit._Container, dijit._Templated],
  {
    operator: null,

    // attach points
    typeNode: null,
    inputNode: null,
    durationNode: null,

    widgetsInTemplate: true,
    templateString:"<div>\r\n  <table width=\"100%\">\r\n    <tr>\r\n      <td NOWRAP>${i18n.propertyeditor.SOURCE_FEED_TYPE}</td>\r\n      <td width=\"100%\">\r\n        <select dojoAttachPoint=\"typeNode\" disabled=\"true\" title=\"${i18n.propertyeditor.SOURCE_FEED_TYPE_TOOLTIP}\" dojoAttachEvent=\"onchange:onModeChanged\"></select>\r\n      </td>\r\n    </tr>\r\n    <tr>\r\n      <td NOWRAP>${i18n.propertyeditor.SOURCE_REPEATING_ELEMENT}</td>\r\n      <td width=\"100%\"><input dojoAttachPoint=\"inputNode\" dojoType=\"flow.dijit.XMLViewTextBox\" removeDuplicates=\"true\" disabled=\"true\" title=\"${i18n.propertyeditor.SOURCE_REPEATING_ELEMENT_TOOLTIP}\"/></td>\r\n    </tr>\r\n    <tr>\r\n      <td NOWRAP>${i18n.propertyeditor.SOURCE_REFRESH_INTERVAL}</td>\r\n      <td width=\"100%\">\r\n        <select dojoAttachPoint='durationNode' style=\"width: 100%;\" dojoAttachEvent=\"onchange:onIntervalChange\" title=\"${i18n.propertyeditor.SOURCE_REFRESH_INTERVAL_TOOLTIP}\">\r\n          <option value=\"0\">${i18n.propertyeditor.SOURCE_INTERVAL_ALWAYS}</option>\r\n          <option value=\"600\">${i18n.propertyeditor.SOURCE_INTERVAL_10_MIN}</option>\r\n          <option value=\"1800\">${i18n.propertyeditor.SOURCE_INTERVAL_30_MIN}</option>\r\n          <option value=\"3600\" selected=\"yes\">${i18n.propertyeditor.SOURCE_INTERVAL_1_HOUR}</option>\r\n          <option value=\"7200\">${i18n.propertyeditor.SOURCE_INTERVAL_2_HOUR}</option>\r\n          <option value=\"10800\">${i18n.propertyeditor.SOURCE_INTERVAL_3_HOUR}</option>\r\n          <option value=\"14400\">${i18n.propertyeditor.SOURCE_INTERVAL_4_HOUR}</option>\r\n          <option value=\"36000\">${i18n.propertyeditor.SOURCE_INTERVAL_10_HOUR}</option>\r\n          <option value=\"86400\">${i18n.propertyeditor.SOURCE_INTERVAL_1_DAY}</option>\r\n        </select>\r\n      </td>\r\n    </tr>\r\n  </table>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    buildRendering: function() {
      this.buildingRendering = true;

      this.inherited("buildRendering", arguments);
      var url = this.operator.getUrlValue();

      // check if operator has a url
      if (url) {
        var settype = this.operator.model.getValue(this.operator.node,"@type");
        var type = this.operator.model.getValue(this.operator.node,"@originaltype");
        var option = document.createElement("option");

        if (type == "RSS" || type == "ATOM") {
          option.setAttribute("value", (type == "RSS") ? "RSS" : "ATOM");
          option.appendChild(document.createTextNode((type == "RSS") ? "RSS" : "ATOM"));
          this.typeNode.appendChild(option);
        }

        option = document.createElement("option");
        option.setAttribute("value", "XML");
        option.appendChild(document.createTextNode("XML"));

        if (settype == "XML") {
          option.setAttribute("selected","true");
        }

        this.typeNode.appendChild(option);
        this.typeNode.removeAttribute("disabled");
        this.inputNode.setDisabled(false);
      } else {
        this.typeNode.setAttribute("disabled", true);
        this.inputNode.setDisabled(true);
      }

      this.durationNode.value = this.operator.model.getValue(this.operator.node,".//flow:advanced/@interval");
      this.inputNode.callback = dojo.hitch(this, this.onTreeSelect);

      var tree = this.operator.getTree(0,true);

      if (tree && tree.root) {
        this.inputNode.textbox.value = this.operator.model.getValue(this.operator.node, ".//flow:advanced/@originalrepeatelement");
	  	  this.inputNode.xmlRootNode = tree.root;
        this.inputNode.path = tree.path;
        this.inputNode.operator = this.operator;
        if (settype == "XML") this.inputNode.setValue(this.operator.model.getValue(this.operator.node, ".//flow:advanced/@repeatelement"));
      } else {
        // no tree data found
        this.inputNode.setDisabled(true);
      }

      this.inputNode.callback = dojo.hitch(this, this.onTreeSelect);
      this.inputNode.removeDuplicates = true;

      if (url && tree && tree.root) {
        this.onModeChanged();
      }

      this.durationNode.value = this.operator.model.getValue(this.operator.node, ".//flow:advanced/@interval");
/*
      if (this.operator.getUrlType() == "external") {
        // load interval from model
        this.durationNode.value = this.operator.model.getValue(this.operator.node, ".//flow:advanced/@interval");
      } else {
        // catalog feeds have their own duration setting
        this.durationNode.setAttribute("disabled", true);
      }
*/

      this.buildingRendering = false;
    },

    onModeChanged: function() {
      var mode = this.typeNode.value;

      var settype = this.operator.model.getValue(this.operator.node,"@type");

      this.operator.model.setValue("@type", this.operator.node, mode);

      if (!this.buildingRendering) {
        this.operator.model.setValue("@dirty", this.operator.node, "1");
      }

      if (mode == "XML") {
        dojo.removeClass(this.inputNode.downArrowNode, "dijitDisplayNone");

        if (settype != "XML") {
          value =this.operator.model.getValue(this.operator.node, ".//flow:advanced/@originalrepeatelement");
          this.operator.model.setValue(".//flow:advanced/@repeatelement", this.operator.node, value);
          this.inputNode.textbox.value=value;
        }

        var tree = this.operator.getTree();
        if (tree && tree.root) {
          this.inputNode.setDisabled(false);
        } else {
          this.inputNode.setDisabled(true);
        }
      } else if (mode == "ATOM") {
        dojo.addClass(this.inputNode.downArrowNode, "dijitDisplayNone");
        var value=this.operator.model.getValue(this.operator.node, ".//flow:advanced/@originalrepeatelement");
        this.operator.model.setValue(".//flow:advanced/@repeatelement", this.operator.node, value);
        //this.inputNode.setValue("//atom:entry");
        this.inputNode.textbox.value = value;
        this.inputNode.setDisabled(true);
      } else {
        dojo.addClass(this.inputNode.downArrowNode, "dijitDisplayNone");
        var value=this.operator.model.getValue(this.operator.node, ".//flow:advanced/@originalrepeatelement");
        this.operator.model.setValue(".//flow:advanced/@repeatelement", this.operator.node, value);
        this.inputNode.textbox.value = value;
        this.inputNode.setDisabled(true);
      }
    },

    onIntervalChange: function(e) {
      var value = e.target.value;

      if (!this.buildingRendering) {
        this.operator.model.setValue("@dirty", this.operator.node, "1");
      }

      this.operator.model.setValue(".//flow:advanced/@interval", this.operator.node, value);
    },

    onTreeSelect: function(data) {
      var value = data.value;

      if (value.substring(0,2)=="./") {
        value = value.substring(2);
      }

      var org = this.operator.model.getValue(this.operator.node, ".//flow:advanced/@originalrepeatelement")
      org = (org.indexOf("/",1)>0) ? org.substring(0,org.indexOf("/",1)):org;

      if (value == "" || value == "/*") {
      	value = org;
      } else {
      	if(value.substring(0,1) != "/") {
          value = org + "/"+value;
        }
      }

      this.operator.model.setValue(".//flow:advanced/@repeatelement", this.operator.node, value)
      if (!this.buildingRendering) {
        this.operator.model.setValue("@dirty", this.operator.node, "1");
      }

      this.inputNode.textbox.value = value;
    }
  }
);


}

if(!dojo._hasResource["flow.dijit.VariableInputSelection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.dijit.VariableInputSelection"] = true;
dojo.provide("flow.dijit.VariableInputSelection");







dojo.declare(
	"flow.dijit.VariableInputSelection",
	[dijit._Widget, dijit._Templated],
  {
    // attach points
    labelNode: null,
    selectNode: null,

    // arguments
    operator: null,
    defaultValue: null,

    // impl
    templateString:"<div>\r\n  <div dojoAttachPoint=\"labelNode\">${i18n.propertyeditor.USE_VARIABLE_DESCRIPTION}</div>\r\n  <select multiple=\"true\" dojoAttachPoint=\"selectNode\" style=\"margin-top: 10px\"></select>\r\n</div>\r\n",

    postMixInProperties: function() {
      this.i18n = {};
      this.i18n.propertyeditor = dojo.i18n.getLocalization("flow", "propertyeditor");
    },

    postCreate: function() {
      this.operator.generateVariableListBox(this.selectNode, "//flow:param[@type='dynamic']");
    },

    getData: function() {
      var data = flow.operator.Data.create("variable");

      if (this.selectNode.value) {
        data.value = this.selectNode.value;
      }

      return data;
    }
  }
);

}

if(!dojo._hasResource["btp.widget.tool.AbstractTool"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.tool.AbstractTool"] = true;
dojo.provide("btp.widget.tool.AbstractTool");




dojo.declare("btp.widget.tool.AbstractTool", null, {
	_editDomain: null,
	_editPartViewer: null,
	
	activate: function(){
	},
	
	deactivate: function(){
	},
	
	getEditDomain: function(){
		return this._editDomain;
	},

	setEditDomain: function(domain){
		this._editDomain = domain;
	},
	
	setEditPartViewer: function(editPartViewer){
		this._editPartViewer = editPartViewer;
	},


	createInsertComponentCommand: function(e, refWidget, targetWidget, widget){
		var command = new btp.widget.command.CompoundCommand();
		if(!refWidget || refWidget == targetWidget){
			command.append(new btp.widget.command.EditCommand.AppendChildCommand(targetWidget, widget));
		}else{
			command.append(new btp.widget.command.EditCommand.InsertBeforeCommand(targetWidget, widget, refWidget));
		}
		return command;
	},

	insertComponent: function(e, refWidget){
		var targetWidget = this.getDropTargetWidget(e);
		var widget = this.createDroppedComponent(e);
		var command = this.createInsertComponentCommand(e, refWidget, targetWidget, widget);
		this.getEditDomain().executeCommand(command);
		this._editPartViewer.setFocus(widget);
		return true;
	},

	createReparentComponentCommand: function(e, refWidget, targetWidget, widget){
		var ds = e.dragSource;
		var dx = e.pageX - ds.pageX;
		var dy = e.pageY - ds.pageY;
		var fromParent = dojo._abs(widget.getParent().domNode, false);
		var toParent = dojo._abs(targetWidget.domNode, false);
		var pdx = fromParent.x - toParent.x;
		var pdy = fromParent.y - toParent.y;
		var toDim = {
			top: ds.dragStartCssPosition.y + dy + pdy + "px",
			left: ds.dragStartCssPosition.x + dx + pdx + "px"
		};

		var command = new btp.widget.command.CompoundCommand();
		if(dojo.style(widget.domNode, "position") != ds.positioning){
			command.append(new btp.widget.command.EditCommand.SetStyleCommand(widget, "position", ds.positioning));
		}
		command.append(new btp.widget.command.EditCommand.RemoveChildCommand(widget));
		if(!refWidget || refWidget == targetWidget){
			command.append(new btp.widget.command.EditCommand.AppendChildCommand(targetWidget, widget));
		}else{
			command.append(new btp.widget.command.EditCommand.InsertBeforeCommand(targetWidget, widget, refWidget));
		}
		command.append(new btp.widget.command.EditCommand.MoveCommand(widget, toDim));
		return command;
	},

	reparentComponent: function(e, refWidget){
		var targetWidget = this.getDropTargetWidget(e);
		var widget = this.getDragSourceWidget(e);
		if(refWidget == widget){ return; }
		var command = this.createReparentComponentCommand(e, refWidget, targetWidget, widget);
		this._editDomain.executeCommand(command);
		this._editPartViewer.setFocusFrame(widget);
		return true;
	},


	createDroppedComponent: function(e){
		var targetWidget = this.getDropTargetWidget(e);
		var targetDomNode = targetWidget.domNode; // domNode of EditArea or parent container
		var srcElem = this.getDragSourceDomNode(e);
		var dim = btp.getDimensions(targetDomNode);
		var props;
		if(srcElem.props){
			props = srcElem.props;
			srcElem.props = null; // clear
		}else{
			props = {};
		}
		var x, y;
		if(e.dragObject && e.dragObject.dragClone){
			var dragClone = e.dragObject.dragClone;
			y = dragClone.offsetTop - dim.top + targetDomNode.parentNode.scrollTop;
			x = dragClone.offsetLeft - dim.left + targetDomNode.parentNode.scrollLeft;
		}else{ // click and release
			var coords;
			if(props.coords){ // ex. ResizeCreationTool stores the drag start position in props.coords
				coords = props.coords;
			}else{
				coords = btp.getEventCoords(e);
			}
			y = coords.y - dim.top + targetDomNode.scrollTop;// - 4; // for some reason, both IE and FF need -4
			x = coords.x - dim.left + targetDomNode.scrollLeft;// - 4;
		}

		if(this._editPartViewer.snapObjectsToGridFlag){
			y = this._editPartViewer.snap(y);
			x = this._editPartViewer.snap(x);
		}
		if(x < 0) x=0;
		if(y < 0) y=0;
		if(this._editPartViewer.fireMessagePosted){
			this._editPartViewer.fireMessagePosted("("+x+","+y+")");
		}
		var name = srcElem.componentClassName;
		if(this._editPartViewer.freeLayoutMode == true){//TODO: should allow customization with config and widget property
			if(!props.__style){
				props.__style = {};
			}
			if(!props.__style.top){
				props.__style.top = y + "px";
			}
			if(!props.__style.left){
				props.__style.left = x + "px";
			}
			if(!props.__style.position){
				props.__style.position = "absolute";
			}
		}
		var widget = this._editPartViewer.createComponent(name, props);
		var info = this._editPartViewer.getComponentInfo(name);
		this.addChildWidgets(widget, info);
		
		return widget;
	},

	addChildWidgets: function(widget, info){
		if(!info || !info.children){ return; }
		var childNodes = btp.getChildNodes(info.children, "child");

		for(var i = 0, len = childNodes.length; i < len; i++){
			var childNode = childNodes[i]; // <child>
			if(childNode.attribute && typeof(childNode.attribute.length) == "undefined"){
				childNode.attribute = [childNode.attribute]; // make it an array
			}

			// set default attribute values if any
			var opt = {};
			for(var j = 0, len2 = childNode.attribute.length; j < len2; j++){
				var attr = childNode.attribute[j];
				if(attr.value){
					opt[attr.name] = attr.value;
				}
			}
			var childName = childNode.name;
			var childWidget = this._editPartViewer.createComponent(childName, opt);

			// repeat for 1-level deeper depth
			this.addChildWidgets(childWidget, childNode);

			widget.addChild(childWidget);
		}
	},

	findRefWidget: function(e){
		var targetWidget = this.getDropTargetWidget(e);
		var refWidget = null;
		if(e.type == "mouseup"){ // Dropped with D&D
			//TODO: what happens when the target is absolute positioning?
			var node = this.getNodeUnderMouse(e);
			var ref = null;
			if(node){
				refWidget = btp.getWidgetByElement(node);
				var isVertical = this.getOrientation(e);
				if(isVertical){
					if(dojo.html.gravity(node, e) & dojo.html.gravity.EAST){
						refWidget = refWidget.getNextSibling();
					}
				}else{
					if(dojo.html.gravity(node, e) & dojo.html.gravity.SOUTH){
						refWidget = refWidget.getNextSibling();
					}
				}
			}
		}
		else /*if(e.type == "click")*/ { // Dropped with click and release
			// nop
		}
		return refWidget;
	},

	getDragSourceDomNode: function(e){
		var node;
		if(e.type == "mouseup"){ // Dropped with D&D
			var ds = e.dragSource;
			srcElem = ds.dragObject; // dropped DOM node
		}
		else /*if(e.type == "click")*/ { // Dropped with click and release
			srcElem = this._editPartViewer.selectedPaletteItem.domNode;
		}
		return srcElem;
	},

	getDragSourceWidget: function(e){
		var sourceWidget;
		var node;
		if(e.type == "mouseup"){ // Dropped with D&D
			sourceWidget = e.dragSource.data;
		}
		else /*if(e.type == "click")*/ { // Dropped with click and release
			sourceWidget = this._editPartViewer.selectedPaletteItem;
		}
		return sourceWidget;
	},

	getDropTargetWidget: function(e){
		var targetWidget; // EditArea or parent container
		if(e.type == "mouseup"){ // Dropped with D&D
			targetWidget = e.dropTarget && e.dropTarget.data;
		}else /*if(e.type == "click")*/ { // Dropped with click and release
			targetWidget = btp.getWidgetByElement(e.target);
		}
		return targetWidget;
	},

	getNodeUnderMouse: function(e){
		var dt = e.dropTarget;
		return dt.getNodeUnderMouse();
	},

	// true: orientation is vertical      (caret shape is like |)
	// false: orientation is horizontal   (caret shape is like -)
	getOrientation: function(e){
		var dt = e.dropTarget;
		return dt.vertical;
	},


	// -------------------------------------------------------------------
	// Default Event Handlers
	// -------------------------------------------------------------------

	// Called when the mouse button has been pressed.
	handleMouseDown: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse button has been released.
	handleMouseUp: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse is moved.
	handleMouseMove: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse pointer is moved onto an element.
	handleMouseOver: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse pointer is moved out of an element.
	handleMouseOut: function(e, editPartViewer){
		return false;
	},

	// Called when a mouse click occurs.
	handleClick: function(e, editPartViewer){
		return false;
	},

	// Called when a mouse double-click occurs.
	handleDblClick: function(e, editPartViewer){
		return false;
	},

	// Called when the key is pressed.
	handleKeyDown: function(e, editPartViewer){
		return false;
	},

	// Called when the key is released.
	handleKeyUp: function(e, editPartViewer){
		return false;
	},

	// Called only once during a drag when the drag threshold has been exceeded.
	handleDragStart: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse hovers over an element during dragging.
	handleDragOver: function(e, editPartViewer){
		return false;
	},

	// Called whenever the mouse is being dragged.
	handleDragMove: function(e, editPartViewer){
		return false;
	},

	// Called when the drag is finished. handleDropXX will be called soon.
	handleDragEnd: function(e, editPartViewer){
		return false;
	},

	// Called before checking whether the drop is allowed.
	handleDropStart: function(e, editPartViewer){
		return false;
	},

	// Called when a drop operation is performed.
	handleDrop: function(e, editPartViewer){
		return false;
	},

	// Called when the drop operation has been finished. e.dragStatus has the result.
	handleDropEnd: function(e, editPartViewer){
		return false;
	},

	// Called when the mouse pointer moves out of an element during dragging.
	handleDragOut: function(e, editPartViewer){
		return false;
	}
});

}

if(!dojo._hasResource["gdl.widget.tool.AbstractConnectionCreationTool"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.tool.AbstractConnectionCreationTool"] = true;
dojo.provide("gdl.widget.tool.AbstractConnectionCreationTool");




dojo.declare(
	"gdl.widget.tool.AbstractConnectionCreationTool",
	btp.widget.tool.AbstractTool,
{
	sourceConnectionAnchor: null,
	targetConnectionAnchor: null,
	sourceWidget: null,
	targetWidget: null,
	ghostLine: null,
	ghostLineCache: null,
	connectionWidgetClassName: null, // class name with suffix

	_operationStarted: false,

	// When the button is first pressed, the source node is determined and locked in.
	// After that time, the tool will be looking for the target node to complete the connection.
	handleMouseDown: function(e, editPartViewer){
		var widget = this.sourceWidget = btp.getWidgetByElement(e.target);
		if(!widget || !widget.getSourceConnectionAnchor){ return false; }
		if(!widget.canCreateSourceConnection()){
			this.sourceWidget = null;
			return false;
		}
		this.sourceConnectionAnchor = widget.getSourceConnectionAnchor(e.target);

		// get connection widget class name (ex. "gdc.widget.PolylineConnection:1")
		var elem = e.target;
		while(elem){
			this.connectionWidgetClassName = elem.getConnectionClass && elem.getConnectionClass(); // from the clicked DOM node
			if(this.connectionWidgetClassName){ break; }
			var widget = elem.widget || dijit.byNode(elem); // gfx raw node may have the widget property
			if(widget){
				this.connectionWidgetClassName = widget.getConnectionClass && widget.getConnectionClass(); // from the node widget
				break;
			}
			elem = elem.parentNode;
			if(!elem || elem.tagName == "BODY"){ break; }
		}
		if(!this.connectionWidgetClassName){
			this.connectionWidgetClassName = editPartViewer.selectedPaletteItem.name; // from the selected palette item
		}
		return true;
	},

	createGhostLine: function(e, editPartViewer){
		var className = this.connectionWidgetClassName.split(":")[0];
		if(!this.ghostLineCache){ this.ghostLineCache = {}; }
		if(this.ghostLineCache[className]){
			this.ghostLine = this.ghostLineCache[className];
		}else{
			this.ghostLine = this.ghostLineCache[className] = editPartViewer.createComponent(className, {lineColor:"#DCDCDC"});
			this.ghostLine.isGhost = true;
			if(this.ghostLine.sourceDecorationObj){
				this.ghostLine.sourceDecorationObj.lineColor = "#DCDCDC";
				this.ghostLine.sourceDecorationObj.fillColor = "#DCDCDC";
			}
			if(this.ghostLine.targetDecorationObj){
				this.ghostLine.targetDecorationObj.lineColor = "#DCDCDC";
				this.ghostLine.targetDecorationObj.fillColor = "#DCDCDC";
			}
		}
		editPartViewer.addChild(this.ghostLine);
		this.ghostLine.setSourceAnchor(this.sourceConnectionAnchor);

		// btp.widget._editArea.componentInfo#getAttributeInfo may be modified
		// so that it works even an argument is component name string (not an widget)
		var info = editPartViewer.getComponentInfo(this.connectionWidgetClassName);
		if(info){
			var attrs = info.attribute;
			for(var i = 0, len = attrs.length; i < len; i++){
				if(attrs[i].name == "router"){
					this.ghostLine.setRouter(attrs[i].value);
					break;
				}
			}
		}
	},

	removeGhostLine: function(e, editPartViewer){
		if(!this.ghostLine){ return; }
		editPartViewer.extensions["gdl.widget.ext.connectionEndpoints"].removeConnectionEndpoints(this.ghostLine);
		this.ghostLine.setEnd(this.ghostLine.getStart());
		this.ghostLine.setSourceAnchor(null); // remove the anchorListener
		this.ghostLine.setTargetAnchor(null); // remove the anchorListener if any
		editPartViewer.removeChild(this.ghostLine);
		this.ghostLine = null;
	},

	// Execute a command to create a connection
	handleCreateConnection: function(e, editPartViewer){
		var srcAnchor = this.sourceConnectionAnchor;
		var tgtAnchor = this.targetConnectionAnchor;
		if(srcAnchor && tgtAnchor){
			var props = {
				sourceAnchor: srcAnchor.owner.id+":"+srcAnchor.anchorName,
				targetAnchor: tgtAnchor.owner.id+":"+tgtAnchor.anchorName
			};
			var connection = editPartViewer.createComponent(this.connectionWidgetClassName, props);
			var offsetParent = gdl.getOffsetParent(this.sourceWidget, this.targetWidget);
			if(!offsetParent){ offsetParent = editPartViewer; }
			var command = new gdl.widget.command.ConnectionCommand.AddConnectionCommand(offsetParent, connection);
			this.getEditDomain().executeCommand(command);
			connection.setMouseOverHandler();
		}
		return true;
	},

	finishCreateConnection: function(e, editPartViewer){
		if(!this.sourceWidget){ return; } // already finished
		this.removeGhostLine(e, editPartViewer);
		if(editPartViewer.selectedPaletteItem){
			if(editPartViewer.selectedPaletteItem.deselect){
				editPartViewer.selectedPaletteItem.deselect();
			}
		}
		this._editDomain.loadDefaultTool();
		this.sourceWidget = this.targetWidget = null;
		this._operationStarted = false;
	}
});

}

if(!dojo._hasResource["gdl.widget.tool.ConnectionDragCreationTool"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.tool.ConnectionDragCreationTool"] = true;
dojo.provide("gdl.widget.tool.ConnectionDragCreationTool");





dojo.declare(
	"gdl.widget.tool.ConnectionDragCreationTool",
	gdl.widget.tool.AbstractConnectionCreationTool,
{
	_ds: null,
	_type: "connection",

	handleMouseDown: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
		if (!this.inherited("handleMouseDown", arguments)){ return false; }
		var widget = this.sourceWidget = btp.getWidgetByElement(e.target);
		if(!widget){ return false; }
		var moveTarget = editPartViewer.getMoveTarget();
		editPartViewer.domNode.appendChild(moveTarget);
		this._ds = new btp.dnd.DragMoveSource(e.target, "connection", widget); // @TODO should not hard-code "connection"
		this._ds.setDragTarget(moveTarget);
		this._ds.fixMouseDownInfo = function(ds){
			// See also btp.dnd.DragMoveSource#onDragMove
			ds.dragStartCssPosition.x += ds.dragOffset.x;
			ds.dragStartCssPosition.y += ds.dragOffset.y;
		}
		this._operationStarted = true;
		this.createGhostLine(e, editPartViewer);
		this.handleMouseMove(e, editPartViewer);
		
		this._conn = dojo.connect(dojo.doc, "onmouseup", dojo.hitch(this, "handleMouseUp", e, editPartViewer));
		return true;
	},

	// Updates ghost line
	handleMouseMove: function(e, editPartViewer){
		e.preventDefault(); // this disables selection
		if (!this._operationStarted){ return false; }
		var dt = btp.dnd.dragManager.currentDropTarget;
		var widget = this.targetWidget = dt && dt.data;
		if(widget && widget.getTargetConnectionAnchor){
			var anchor = widget.getTargetConnectionAnchor(e.target);
			this.targetConnectionAnchor = anchor;
			this.ghostLine.setTargetAnchor(anchor);
		}else{
			var pos = dojo._abs(editPartViewer.domNode, false);
			var x = e.pageX - pos.x + editPartViewer.domNode.scrollLeft - 2;
			var y = e.pageY - pos.y + editPartViewer.domNode.scrollTop;
			this.targetConnectionAnchor = null;
			this.ghostLine.setTargetAnchor(null);
			this.ghostLine.setEnd({x:x, y:y, dir:"left"});	//hardcoded temporalily
		}
		this.sourceConnectionAnchor.anchorMoved();
		return true;
	},

	handleMouseUp: function(e, editPartViewer){
		dojo.disconnect(this._conn);
		// Use setTime so that handleDrop is processed before finishCreateConnection.
		setTimeout(dojo.hitch(this, "finishCreateConnection", e, editPartViewer), 0);
	},

	handleDrop: function(e, editPartViewer){
		if(this._operationStarted) {
			this.finishCreateConnection(e, editPartViewer);
			var ret = this.handleCreateConnection(e, editPartViewer);
			this._operationStarted = false;
			return ret;			
		}
	}
});

}

if(!dojo._hasResource["flow.hale.ConnectionDragCreationTool"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.hale.ConnectionDragCreationTool"] = true;
dojo.provide("flow.hale.ConnectionDragCreationTool");



dojo.declare(
	"flow.hale.ConnectionDragCreationTool",
	gdl.widget.tool.ConnectionDragCreationTool,
  {
    handleMouseDown: function(e, editPartViewer) {
      var rv = this.inherited("handleMouseDown", arguments);
      if (rv == true) {
        this.sourceWidget.showOutConnector(false);
      }
      return rv;
    },

    handleMouseUp: function(e, editPartViewer) {
      if (this.sourceWidget) {
        this.sourceWidget.showOutConnector(true);
      }
      return this.inherited("handleMouseUp", arguments);
    }
  }
);

}

if(!dojo._hasResource["btp.widget.tool.DefaultCreationTool"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.tool.DefaultCreationTool"] = true;
dojo.provide("btp.widget.tool.DefaultCreationTool");


dojo.declare(
	"btp.widget.tool.DefaultCreationTool",
	btp.widget.tool.AbstractTool,
{
	handleClick: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
		if(editPartViewer.selectedPaletteItem){
			if(editPartViewer.selectedPaletteItem.deselect){
				editPartViewer.selectedPaletteItem.deselect();
			}
		}
		var refWidget = this.findRefWidget(e);
		this.insertComponent(e, refWidget);
		this._editDomain.loadDefaultTool();
	},

	handleDrop: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
		var targetWidget = this.getDropTargetWidget(e);
		var ret;
		if(targetWidget.componentDropHandler){
			ret = targetWidget.componentDropHandler(e, editPartViewer);
		}else{
			ret = this.componentDropHandler(e, editPartViewer);
		}
		this._editDomain.loadDefaultTool();
		e.dragStatus = ret ? "dropSuccess" : "dropFailure"; // See DragManager.onMouseUp()
		editPartViewer.domNode.style.cursor = "auto";
	},

	handleMouseOver: function(e, editPartViewer){
		editPartViewer.domNode.style.cursor = "pointer";
	},


	componentDropHandler: function(e, editPartViewer){
		// this: drop target widget
		var srcElem = this.getDragSourceDomNode(e);
		var ret;
		if(srcElem.componentClassName){
			var refWidget = this.findRefWidget(e);
			ret = this.insertComponent(e, refWidget);
		}
		return ret;
	}
});

}

if(!dojo._hasResource["flow.hale.CreationTool"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.hale.CreationTool"] = true;
dojo.provide("flow.hale.CreationTool");


dojo.declare(
	"flow.hale.CreationTool",
	btp.widget.tool.DefaultCreationTool,
  {
    createDroppedComponent: function(e, editPartViewer){
      var widget = this.inherited("createDroppedComponent", arguments);
      var targetWidget = this.getDropTargetWidget(e);
      var sourceWidget = this.getDragSourceWidget(e);

      var type = sourceWidget.operatortype;
      widget.operatorType = type;
      targetWidget.addOperator(widget);
      return widget
    }
  }
);

}

if(!dojo._hasResource["gdc.widget.decoration.AbstractDecoration"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.decoration.AbstractDecoration"] = true;
dojo.provide("gdc.widget.decoration.AbstractDecoration");

dojo.declare(
	"gdc.widget.decoration.AbstractDecoration",
	null,
{
	owner: null,

	constructor: function(owner, props){
		this.setProps(props);
		this.owner = owner;
	},

	setProps: function(props){
		if(props && typeof(props) == "string"){
			props = eval('('+props+')');
		}
		dojo.mixin(this, props);
	},

	createTargetShape: function(parent){
		// subclass must implement
	},

	update: function(props){
		// subclass must implement
	},

	removeShape: function(){
		// subclass must implement
	},

	setLocation: function(/*{x:#,y:#}*/pt){
		// subclass must implement
	},

	setRotation: function(/*float*/angle){
		// subclass must implement
	}
});

}

if(!dojo._hasResource["flow.hale.DraggableImageDecoration"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["flow.hale.DraggableImageDecoration"] = true;
dojo.provide("flow.hale.DraggableImageDecoration");

// Summary:
//		A decorative Figure intended to be placed on a gfx line.
//		Note that this is not a widget.



dojo.declare(
  "flow.hale.DraggableImageDecoration",
  gdc.widget.decoration.AbstractDecoration,
{
  src: dojo.moduleUrl("flow.dijit", "templates/images/op-plug.png"),
  width: 20,
  height: 23,
  offsetX: -21,
  offsetY: -12,

  constructor: function(owner, props){
  },

  createTargetShape: function(parent){
    var shape = this.shape = parent.createGdcImage({width:this.width, height:this.height, src: this.src+""});

    var _this = this;
    shape.rawNode.getDragTracker = function(){
      _this.owner.__editArea.setFocus(_this.owner);
      return "gdl.widget.tool.ReconnectTracker";
    };

    shape.rawNode.style.cursor = "url("+FLOW.baseResUrl+"/js/flow/dijit/templates/images/rightHandGrab.cur), default";

    // assign the owner of this decoration to the top level dom node
    // so that the owner widget can be found from dom node. (See btp._base.utils.getWidgetByElement)
    shape.rawNode.widget = this.owner; // [custom property]
    return shape;
  },

  update: function(props){
    var shape = this.shape;
    if(!shape){ return; }
    shape.setShape({width:this.width, height:this.height, src:this.src+""});
  },

  removeShape: function(){
    this.owner.__editArea.extensions["gdl.widget.ext.connectionEndpoints"].removeConnectionEndpoints(this.owner);
  },

  // Returns dimensions of this decoration.
  // cf. gdl.widget.ext.connectionEndpoints#addConnectionEndpoints
  getDimensions: function(){
    if(!this.shape){ return null; }
    var transform = this.shape.getTransform();
    return {
      width: this.width,
      height: this.height,
      top: transform.dy,
      left: transform.dx
    };
  },

  setLocation: function(/*{x:#,y:#}*/pt){
    this.shape.setTransform({dx:pt.x + this.offsetX, dy:pt.y + this.offsetY});
    if (this.owner.__editArea.extensions && this.owner.__editArea.extensions["gdl.widget.ext.connectionEndpoints"]) {
      this.owner.__editArea.extensions["gdl.widget.ext.connectionEndpoints"].addConnectionEndpoints(this.owner);
    }
  },

  setRotation: function(/*float*/angle){
    //var rotate = dojox.gfx.matrix.rotategAt(angle, 0, 0);
    //this.shape.applyTransform(rotate);
  }
});

}

if(!dojo._hasResource["btp.dnd.DragGhostMoveSource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.dnd.DragGhostMoveSource"] = true;
dojo.provide("btp.dnd.DragGhostMoveSource");


dojo.declare("btp.dnd.DragGhostMoveSource", btp.dnd.DragSource, {
	ghost: null,

	// override
	onDragMove: function(e){
		var dx = e.pageX - this.pageX + this.offsetParent.scrollLeft;
		var dy = e.pageY - this.pageY + this.offsetParent.scrollTop;
		this.dragClone.style.top = this.dragStartPosition.y - this.offsetParentCoords.y + dy + "px";
		this.dragClone.style.left = this.dragStartPosition.x - this.offsetParentCoords.x + dx + "px";
		if(this.positioning == "absolute"){
			this.dragClone.style.display = "";
			this.globalChangeCursor(null);
		}else{
			this.dragClone.style.display = "none";
			this.globalChangeCursor("move");
		}
	},
	
	// override
	onDragEnd: function(e){
		if(this.positioning == "absolute"){
			this.removeClone();
		}else{
			this.globalChangeCursor(null);
		}
	},

	// override
	createDragClone: function(e){
		this.createGhost(e);
		this.dragClone = this.ghost; // this.ghost is cached, while this.dragClone is cleared on drag end.
		this.dragClone.style.display = "none";
		this.offsetParent.appendChild(this.dragClone);
	},

	// Create a dashed rectangle (ghost)
	createGhost: function(e){
		if(!this.ghost){
			var line = this.ghost = dojo.doc.createElement("DIV");
			line.className = "btpGhostFrame";
			line.style.zIndex = 1000;
			line.style.position = "absolute";
			line.style.fontSize = "1px";
			line.style.borderWidth = "1px";
			line.style.borderStyle = "dashed";
			var line2 = line.cloneNode(false);
			var line3 = line.cloneNode(false);
			var line4 = line.cloneNode(false);
			line.appendChild(line2);
			line.appendChild(line3);
			line.appendChild(line4);
		}
		var topLine = this.ghost;
		var bottomLine = topLine.childNodes[0];
		var leftLine = topLine.childNodes[1];
		var rightLine = topLine.childNodes[2];

		// Top Line
		topLine.style.width = this.refNode.offsetWidth + "px";
		topLine.style.height = "1px";
		topLine.style.borderWidth = "0px 0px 1px 0px";

		// Bottom Line
		bottomLine.style.top = this.refNode.offsetHeight + "px";
		bottomLine.style.left = "0px";
		bottomLine.style.width = this.refNode.offsetWidth + "px";
		bottomLine.style.height = "1px";
		bottomLine.style.borderWidth = "1px 0px 0px 0px";

		// Left Line
		leftLine.style.top = "1px";
		leftLine.style.left = "-1px";
		leftLine.style.width = "1px";
		leftLine.style.height = this.refNode.offsetHeight + "px";
		leftLine.style.borderWidth = "0px 1px 0px 0px";

		// Right Line
		rightLine.style.top = "1px";
		rightLine.style.left = this.refNode.offsetWidth - 1 + "px";
		rightLine.style.width = "1px";
		rightLine.style.height = this.refNode.offsetHeight + "px";
		rightLine.style.borderWidth = "0px 0px 0px 1px";
	}
});

}

if(!dojo._hasResource["btp.dnd.DropTarget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.dnd.DropTarget"] = true;
dojo.provide("btp.dnd.DropTarget");


dojo.declare("btp.dnd.DropTarget", null, {
	disabled: false, // disable this drop target
	offsetParent: null,
	showingFeedback: false,
	acceptedTypes: null,
	nodeUnderMouse: null,
	caret: null,
	vertical: false,
	data: null, // any user data associated with this dt. (widget, etc.)

	constructor: function(node, types, data){
		this.data = data;
		var node = this.domNode = dojo.byId(node);
		node.dropTarget = this; // [custom property]
		if(!types){
			this.acceptedTypes = [];
		}else if(typeof(types) == "string"){
			this.acceptedTypes = [types];
		}else{
			this.acceptedTypes = types;
		}
		btp.dnd.dragManager.registerDropTarget(this);
	},

	// The onmousedown event has fired, but it doesn't mean D&D has started.
	onDragDown: function(e){
	},

	// D&D operation has been started
	onDragStart: function(e){
	},

	onDragOver: function(e){
		if(!this.accepts(e.dragSource)){ return false; }
		this.showFeedback(this);
		this.showingFeedback = true;
		return true;
	},

	onDragMove: function(e){
		if(this.shouldShowCaret(e)){
			this.showCaret(e);
		}
	},

	// D&D operation has been ended
	onDragEnd: function(e){
	},

	onDropStart: function(e){
	},

	onDrop: function(e){
	},

	onDropEnd: function(){
	},

	onDragOut: function(e){
		if(this.showingFeedback){
			this.hideFeedback(this);
			this.showingFeedback = false;
		}
		this.hideCaret();
	},

	unregister: function(){
		btp.dnd.dragManager.unregisterDropTarget(this);
		this.domNode.dropTarget = null; // [custom property]
	},

	reregister: function(){
		btp.dnd.dragManager.registerDropTarget(this);
		this.domNode.dropTarget = this; // [custom property]
	},

	// You may want to override this
	isAcceptable: function(ds, dt){
		return true;
	},

	accepts: function(ds){
		if(!this.isAcceptable(ds, this)){ return false; }
		for(var i = 0, len = this.acceptedTypes.length; i < len; i++){
			if(this.acceptedTypes[i] == ds.type){ return true; }
		}
		return false;
	},

	showFeedback: function(dt){
	},

	hideFeedback: function(dt){
	},

	getNodeUnderMouse: function(){
		return this.nodeUnderMouse;
	},

	shouldShowCaret: function(e){
		// check the node under the mouse cursor
		var node = this.nodeUnderMouse;
		if(!node){ return false; }
		if(dojo.style(node, "position") != "static"){ return false; }

		// check the drag source
		var ds = e.dragSource;
		if(!ds.targetShouldShowCaret || ds.positioning != "static"){
			return false;
		}

		return true;
	},

	showCaret: function(e){
	return;
		if(!this.nodeUnderMouse){ return; }
		this.createCaret();
		var caret = this.caret;
		var node = this.nodeUnderMouse;
		var gravity = dojo.html.gravity(node, e);
		var display = dojo.style(node, "display");
		if(display == "block"){
			caret.style.width = node.offsetWidth + 3 + "px";
			caret.style.height = "1px";
			caret.style.borderWidth = "0px 0px 2px 0px";
			if(gravity & dojo.html.gravity.NORTH){
				caret.style.top = node.offsetTop + "px";
			}else{
				caret.style.top = node.offsetTop + node.offsetHeight + "px";
			}
			caret.style.left = node.offsetLeft + "px";
			this.vertical = false;
		}else{
			caret.style.width = "1px";
			caret.style.height = node.offsetHeight + 3 + "px";
			caret.style.borderWidth = "0px 2px 0px 0px";
			caret.style.top = node.offsetTop + "px";
			if(gravity & dojo.html.gravity.WEST){
				caret.style.left = node.offsetLeft + "px";
			}else{
				caret.style.left = node.offsetLeft + node.offsetWidth + "px";
			}
			this.vertical = true;
		}
		caret.style.display = "";
	},

	hideCaret: function(){
		if(this.caret){
			this.caret.style.display = "none";
		}
	},

	createCaret: function(){
	alert("createCaret");
		if(!this.caret){
			var caret = this.caret = dojo.doc.createElement("DIV");
			caret.className = "btpCaret";
			caret.style.zIndex = 1000;
			caret.style.position = "absolute";
			caret.style.fontSize = "1px";
			caret.style.borderStyle = "solid";
			caret.style.borderColor = "black";
			caret.style.display = "none";
			this.offsetParent.appendChild(this.caret);
		}
	}
});

}

if(!dojo._hasResource["btp.i18nImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.i18nImpl"] = true;
dojo.provide("btp.i18nImpl");



btp.i18nImpl = {
	loadBundle: function(bundleName){
		// ex. "led.widget.LiveEditor" //TODO:
		if (!bundleName){ return null; }
		var pos = bundleName.lastIndexOf(".");
		var packageName = bundleName.substring(0, pos); // "led.widget"
		var name = bundleName.substring(pos + 1); // "LiveEditor"
		dojo["requireLocalization"](packageName, name);
		var bundle = dojo.i18n.getLocalization(packageName, name);
		if(!btp.i18n.bundle){
			btp.i18n.bundle = bundle;
		}else{
			dojo.mixin(btp.i18n.bundle, bundle);
		}
		return btp.i18n.bundle;
	},

	gets: function(s, prefix){
		if(!s){ return s; }
		var bundle = btp.i18n.bundle;
		if(bundle){
			if(s.charAt(0) == '$' && s.charAt(1) == '{' && s.charAt(s.length - 1) == '}'){
				var key = s.substring(2, s.length - 1);
				if (bundle[key]){
					return bundle[key];
				}
				return s;
			}
			if(bundle[prefix+s]){
				return bundle[prefix+s];
			}
			if(bundle[s]){
				return bundle[s];
			}
		}
		return s;
	}
};

}

if(!dojo._hasResource["btp.widget._TabPanelBase"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._TabPanelBase"] = true;
dojo.provide("btp.widget._TabPanelBase");






dojo.declare(
	"btp.widget._TabPanelBase",
//	[dijit.layout.LayoutContainer, dijit._Templated],
	[dijit.layout._LayoutWidget, dijit._Templated],
//	[dijit._Widget, dijit._Templated, dijit._Container, dijit._Contained],
{
	closable: false,
	isContainer: true,
	isActive: false,
	showInactiveIcons: false,
	type: "simple", // "eclipse"(default) or "simple"

	templateString: '<div class="tabPanel" dojoAttachPoint="tabPanelNode">'+
						'<div class="tabPanelHeader" dojoAttachPoint="tabHeaderNode"></div>'+
						'<div class="tabPanelPane" dojoAttachPoint="containerNode"></div>'+
					'</div>',
	i18nPrefix: "TabPanel_",

	deleteButton: null,
	selectedPage: null,
	tabs: null,
	crossImgPath: null,
	redCrossImgPath: null,

	_connectors: null,
	_topics: null,

	constructor: function(){
		this._connectors = [];
		this._topics = [];
	},

	postCreate: function(){
		btp.widget._TabPanelBase.superclass.postCreate.apply(this, arguments);
		this._connectors.push(dojo.connect(this.containerNode, "onmousedown", this, "onTabClick"));
		this._topics.push(dojo.subscribe("btp:"+this.topicNamePrefix+"Changed", this, "onTabChanged"));
	},

	startup: function(){
		this.setupChildren();
		btp.widget._TabPanelBase.superclass.startup.apply(this, arguments);
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	setupChildren: function(){
		var children = this.getChildren();
		dojo.forEach(children, this.setupChild, this);
		if(this.tabs.length > 0){
			if(this.deleteButton){
				this.deleteButton.style.display = "";
			}
			this.selectTab(this.tabs[0], true);
			var node = this.tabs[0].page.domNode;
			if(this.tabs[0].page.parseOnLoad == false){
				setTimeout(function(){ dojo.parser.parse(node); }, 0);
			}
		}
	},

	setupChild: function(/*ContentPane*/ page){
		alert("subclass must implement this");
	},

	selectTab: function(tab, init, focus){
		if(typeof(tab) == "number"){
			tab = this.tabs[tab];
		}
		var page = tab.page;

		if(this.selectedPage == tab.page){ // already showing
			if(this.isActive){
				return; // already active
			}
		}else{
			if(this.selectedPage){
				var selectedTab = this.selectedPage.tab;
				this.grayoutTab(selectedTab);
				this.selectedPage.domNode.style.display = "none";
				if(this.selectedPage.hide){
					this.selectedPage.hide();
				}
			}
			page.domNode.style.display = "";
			if(page.show){
				page.show();
			}
			if(page.parseOnLoad == false){
				dojo.parser.parse(page.domNode);
			}
			if(!init && page.resize){
				page.resize();
			}
		}
		this.selectedPage = page;

		if(init){
			this.unhilightTab(tab);
			if(!this.showInactiveIcons && tab.iconNode){
				tab.iconNode.style.display = "";
			}
		}else{
			this.hilightTab(tab, focus);
			this.fireTabChanged(page);
		}
	},

	// override dijit._Container.addChild
	addChild: function(page){
		btp.widget._TabPanelBase.superclass.addChild.apply(this, arguments);
		if(this.tabs.length == 0 && this.deleteButton){
			this.deleteButton.style.display = "";
		}
		this.setupChild(page);
		this.selectTab(page.tab);
//		this.onResized();
	},

	hilightTab: function(tab, focus){
		// subclass may want to override this
	},

	unhilightTab: function(tab){
		// subclass may want to override this
	},

	grayoutTab: function(tab){
		// subclass may want to override this
	},

	reposition: function(tab){
		// subclass may want to override this
	},

	setDirty: function(page, flag){
		var tab = page.tab;
		var titleNode = tab.titleNode;
		var title = titleNode.innerHTML;
		if(flag && title.charAt(0) != "*"){
			title = "*" + title;
			titleNode.innerHTML = title;
			this.reposition(tab);
		}else if(!flag && title.charAt(0) == "*"){
			title = title.substring(1);
			titleNode.innerHTML = title;
			this.reposition(tab);
		}
	},

	setTitle: function(tab, title, tooltip){
		if(typeof(tab) == "number"){
			tab = this.tabs[tab];
		}
		var page = tab.page;
		var titleNode = tab.titleNode;
		titleNode.innerHTML = title;
		if(tooltip){
			tab.title = tooltip;
			page.tooltip = tooltip;
		}
		page.label = title;
		this.onResized();
	},

	// ======== Topic Publisher ====================================
	fireTabChanged: function(page){
		var message = {
			type: this.topicNamePrefix+"ChangedNotification",
			src: this,
			tabPage: page
		};
		dojo.publish("btp:"+this.topicNamePrefix+"Changed", [message]);
	},

	fireTabDoubleClicked: function(page){
		var message = {
			type: this.topicNamePrefix+"DoubleClickedNotification",
			src: this,
			tabPage: page
		};
		dojo.publish("btp:"+this.topicNamePrefix+"DoubleClicked", [message]);
	},


	layout: function(){
		dijit.layout.layoutChildren(this.domNode, this._contentBox, this.childWidgets);

		if(!this.selectedPage){ return; }
		if(this.selectedPage.resize){
			var content = dojo.contentBox(this.containerNode);
			this.selectedPage.resize(content.width, content.height);
		}

		if(!this.selectedPage){ return; }
		var tab = this.selectedPage.tab;
		this.reposition(tab);
	},

	onTabClick: function(e){
		if(this.tabs.length == 0){
			return;
		}
		var target = e.currentTarget;
		var tab;
		if(target.className == "tabPanelTab"){
			tab = target;
			this.selectTab(tab);
		}else if(target.className.indexOf("tabPanelPane") != -1){
			tab = this.selectedPage.tab;
			this.selectTab(tab);
		}
	},

	onTabDblClick: function(e){
		var tab = e.currentTarget;
		this.fireTabDoubleClicked(tab.page);
	},

	onTabKeyDown: function(e){
		var tab = e.currentTarget;
		var key = e.keyCode;
		if(key == 37 || key == 38){ // 37: left arrow, 38: up arrow
			var prev = tab.previousSibling;
			if(!prev || !prev.page){
				return;
			}
			this.selectTab(prev, false, true);
		}else if(key == 39 || key == 40){ // 39: right arrow, 40: down arrow
			var next = tab.nextSibling;
			if(!next || !next.page){
				return;
			}
			this.selectTab(next, false, true);
		}
	},

	// ======== Topic Subscriber ====================================
	onTabChanged: function(message){
		if(message.src == this){
			return; // I am interested in the other tabPanel only
		}
		if(!message.tabPage){
			return; // all tabs have been deleted
		}
		var tab = message.tabPage.tab;
		if(tab && this.selectedPage){
			this.unhilightTab(this.selectedPage.tab);
		}
	},


	deleteButtonMouseOverHandler: function(e){
		var img = e.currentTarget;
		img.src = this.redCrossImgPath;
	},

	deleteButtonMouseOutHandler: function(e){
		var img = e.currentTarget;
		img.src = this.crossImgPath;
	},

	deleteButtonClickHandler: function(e){
		this.removeChild(this.selectedPage);
	}
});

dojo.extend(dijit._Widget, {
	label: "",
	icon: "",
	selected: false
});

}

if(!dojo._hasResource["btp.widget.BottomTabPanel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.BottomTabPanel"] = true;
dojo.provide("btp.widget.BottomTabPanel");



dojo.declare(
	"btp.widget.BottomTabPanel",
	btp.widget._TabPanelBase,
{
	lowerLineNode: null,

	lightGray: "#DEDFDE",
	darkGray: "#ACA899",

	topicNamePrefix: "BottomTab",

	templateString: '<div class="tabPanel" dojoAttachPoint="tabPanelNode">'+
						'<div class="tabPanelPane" dojoAttachPoint="containerNode"></div>'+
						'<div class="tabPanelHeader" dojoAttachPoint="tabHeaderNode"></div>'+
					'</div>',

	postCreate: function(){
		this.domNode.className = "tabPanel bottomTabPanel";
		var header = this.tabHeaderNode;
		this.tabs = [];

		this.childWidgets = [
			{domNode: this.tabHeaderNode, layoutAlign: "bottom"},
			{domNode: this.containerNode, layoutAlign: "client"}
		];

		// Lower Line
		var line = this.lowerLineNode = dojo.doc.createElement("DIV");
		line.className = "tabPanelLine tabPanelLineLower";
		header.appendChild(line);

		// to keep height even when there are no tabs
		var dummy = dojo.doc.createElement("DIV");
		dummy.className = "tabPanelDummyTab";
		header.appendChild(dummy);

		btp.widget.BottomTabPanel.superclass.postCreate.apply(this, arguments);
	},

	setupChild: function(/*ContentPane*/ page){
		var header = this.tabHeaderNode;
		var tab = dojo.doc.createElement("DIV");
		tab.className = "tabPanelTab";
		tab.style.height = "21px";
		tab.page = page; // [custom property]
		page.tab = tab; // [custom property]
		this.tabs.push(tab);

		page.domNode.style.position = "relative";
		page.domNode.style.display = "block";

		// Title Node
		var titleNode = dojo.doc.createElement("DIV");
		titleNode.className = "tabPanelTabTitle";
		titleNode.tabIndex = 0;
		var label = page.label; //TODO: hale.i18n.gets(page.label, this.i18nPrefix);
		titleNode.innerHTML = label;
		if(page.tooltip){
			tab.title = page.tooltip;
		}
		tab.appendChild(titleNode);
		tab.titleNode = titleNode; // [custom property]

		header.appendChild(tab);

		tab._connectors = [];
		tab._connectors.push(dojo.connect(tab, "onclick", this, "onTabClick"));
		tab._connectors.push(dojo.connect(tab, "ondblclick", this, "onTabDblClick"));
		tab._connectors.push(dojo.connect(tab, "onkeydown", this, "onTabKeyDown"));

		page.domNode.style.display = "none";
	},

	// override dijit._Container.removeChild
	removeChild: function(/*Widget*/widget){
		var page = widget;
		var tab = widget.tab;
		for(var i = 0, len = this.tabs.length; i < len; i++){
			if(this.tabs[i] == tab){
				this.tabs.splice(i, 1); // remove tab from tabs
				break;
			}
		}
		if(this.tabs.length == 0){
			this.unhilightTab(tab);
		}
		this.tabHeaderNode.removeChild(tab);
		if(this.selectedPage == page){
			this.selectedPage = null;
		}
		dojo.forEach(tab._connectors, dojo.disconnect);
		page.destroy();
		if(this.tabs.length == 0 && this.deleteButton){
			this.deleteButton.style.display = "none";
			this.edgeImages[0].style.display = "none";
			this.edgeImages[1].style.display = "none";
			this.containerNode.style.backgroundColor = "#EFEBDE";
			this.fireTabChanged(null);
		}else{
			this.selectTab(this.tabs[0]);
			this.reposition(this.tabs[0]);
		}
	},

	hilightTab: function(tab, focus){
//		tab.style.backgroundImage = "url("+dojo.moduleUrl("btp", "widget/themes/images/simple-tab-bg.gif")+")";
		tab.style.borderTopColor = this.darkGray;
		tab.style.borderTopColor = "white";
		tab.style.borderLeftColor = this.darkGray;
		tab.style.borderRightColor = this.darkGray;
		tab.style.backgroundColor = "white";
		this.isActive = true;

		// focus (necessary for keyboard navigation)
		if(focus){
			tab.titleNode.focus();
		}

		// set tabIndex only for the selected tab
		for(var i = 0, len = this.tabs.length; i < len; i++){
			this.tabs[i].titleNode.tabIndex = (this.tabs[i] == tab) ? 0 : -1;
		}
	},

	unhilightTab: function(tab){
		this.hilightTab(tab);
	},

	grayoutTab: function(tab){
		tab.style.backgroundColor = "#EFEBDE";
		tab.style.borderColor = this.darkGray;
	},

	reposition: function(tab){
		// Line
		var lastTab = this.tabs[this.tabs.length - 1];
		var dim = btp.getDimensions(lastTab, this.tabHeaderNode);
		this.lowerLineNode.style.width = tab.offsetWidth - 1 + "px";
		this.lowerLineNode.style.top = "0px";
		this.lowerLineNode.style.left = dim.left + dim.width + "px";
		this.lowerLineNode.style.width = this.tabHeaderNode.offsetWidth + "px";
	}
});

}

if(!dojo._hasResource["btp.widget.command.CommandProcessor"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.command.CommandProcessor"] = true;
dojo.provide("btp.widget.command.CommandProcessor");

// ----------------------------------------------------------------------------
// Manage Command instances
// ----------------------------------------------------------------------------

//  0 1 2           0 1 2
// --------        --------
// | | | |         |x| | |
// --------        --------
//  ^_top=0           ^_top=1
// (empty stack)   (one command)

btp.widget.command.CommandProcessor = function(){
	this._stack = new Array();
	this._top = 0;     // stack top position
	this._current = 0; // current position

	this.push = function(cmd){
		this._stack[this._current] = cmd;
		this._top = this._current + 1;
		for(var i = this._top; i < this._stack.length; i++){
			this._stack[i] = null;
		}
	}

	this.pop = function(cmd){
		if(this._top == 0){
			return;
		}
		this._stack[--this._top] = null;
		this._current = this._top;
	}

	this.clear = function(cmd){
		this._top = 0;
		this._current = 0;
		for(var i = 0; i < this._stack.length; i++){
			this._stack[i] = null;
		}
	}

	this.execute_it = function(){
		if(!this._stack[this._current]){ return; }
		var cmd = this._stack[this._current++];
		cmd.execute();
		if(this._current == 1){
			this.dirtied();
		}
	}

	this.undo_it = function(){
		if(this._current == 0){
			window.status = "No further undo information";
			window.setTimeout("window.status='';", 5000);
			return;
		}
		var cmd = this._stack[--this._current];
		cmd.undo();
		if(this._current == 0){
			this.undirtied();
		}
	}

	this.dirtied = function(){
	}

	this.undirtied = function(){
	}
};

}

if(!dojo._hasResource["btp.widget.ext.localToolbar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.ext.localToolbar"] = true;
dojo.provide("btp.widget.ext.localToolbar");

dojo.declare(
	"btp.widget.ext.localToolbar",
	null,
{
	owner: null,
	currentToolbarWidget: null,

	// initializer
	constructor: function(owner){
		this.owner = owner;
		this._connectors = [];
		if(this.show){
			this._connectors.push(dojo.connect(owner, "show", this, "_show"));
		}else{
			this.show = this._show;
		}

		if(this.hide){
			this._connectors.push(dojo.connect(owner, "hide", this, "_hide"));
		}else{
			this.hide = this._hide;
		}
		dojo.subscribe("btp:ActionInvoked", this, "dispatchCommand");
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
	},

	createLocalToolbar: function(json){
		var widget = new btp.widget.Toolbar();
		widget.domNode.style.padding = "2px 0px 0px 0px";
		widget.domNode.style.styleFloat = "right"; // IE
		widget.domNode.style.cssFloat = "right";   // Mozilla
		widget.updateToolbar(json);
		return widget;
	},

	attachLocalToolbar: function(toolbarWidget){
		if(this.currentToolbarWidget == toolbarWidget){ return; }
		this._hide(); // hide the previous toolbar
		var parentWidget = this.owner.getParent();
		if(!toolbarWidget.domNode.parentNode || toolbarWidget.domNode.parentNode != parentWidget.localToolbarArea){
			if(!parentWidget.localToolbarArea){ return; }
			parentWidget.localToolbarArea.appendChild(toolbarWidget.domNode);
		}
		this.currentToolbarWidget = toolbarWidget;
		if(btp.isShowing(this.owner)){
			this._show(); // show the new toolbar
		}else{
			this._hide(); // hide the new toolbar
		}
	},

	dispatchCommand: function(message){
		if(!this.currentToolbarWidget
			|| !btp.matchFilter(message, this.currentToolbarWidget.domNode.getAttribute("widgetId"))){
			return; // filter out the message
		}
		this.owner[message.command](message);
	},
	
	_show: function(){
		if(this.currentToolbarWidget){
			this.currentToolbarWidget.domNode.style.display = "";
		}
	},
	
	_hide: function(){
		if(this.currentToolbarWidget){
			this.currentToolbarWidget.domNode.style.display = "none";
		}
	}
});

}

if(!dojo._hasResource["btp.widget.Console"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.Console"] = true;
dojo.provide("btp.widget.Console");

var btpConsoleMsg = "";
if(!djConfig["isDebug"]){
	var _btpConsoleConnector = dojo.connect(console, "debug", function(msg){
		if(btpConsoleMsg){ 
			btpConsoleMsg += "\n";
		}
		btpConsoleMsg += msg;
	});
}





dojo.declare(
	"btp.widget.Console",
	[dijit._Widget, dijit._Contained],
{
	_localToolbar: null,

	postCreate: function(){
		this.domNode.className = "btpConsole btpSimplePane";
		if(djConfig["isDebug"]){ return; }
		dojo.disconnect(_btpConsoleConnector);
		var lines = btpConsoleMsg.split("\n");
		for(var i = 0, len = lines.length; i < len; i++){
			this.debug(lines[i]);
		}
		btpConsoleMsg = null;
		this.connect(console, "debug", "debug");
	},

	show: function(){
		if(!this._localToolbar){
			var ext = btp.loadExtension(this, "btp.widget.ext.localToolbar");
			var json = {
			    toolBar: {
			      item: [{
			        "@command": "clearConsole",
			        "@tooltip": "Clear Console",
			        "@iconPath": "widget/templates/Console/clear_co.gif"
			      }]
			    }
			};
			this._localToolbar = ext.createLocalToolbar(json);
			ext.attachLocalToolbar(this._localToolbar);
		}
	},

	debug: function(message){
		var node = dojo.doc.createTextNode(message);
		this.domNode.appendChild(node);
		this.domNode.appendChild(dojo.doc.createElement("BR"));
		this.domNode.scrollTop = this.domNode.scrollHeight;
	},

	clearConsole: function(){
		this.domNode.innerHTML = "";
	}
});

}

if(!dojo._hasResource["btp.widget.data.htmlGenerator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.data.htmlGenerator"] = true;
dojo.provide("btp.widget.data.htmlGenerator");

btp.widget.data.htmlGenerator = {
	generate: function(widget){
		var ml =
			 '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"\n'
			+'  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">\n'
			+'<html xmlns="http://www.w3.org/1999/xhtml">\n'
			+'<head>\n</head>\n<body>\n';
		var children = widget.getChildren ? widget.getChildren() : null;
		if(children){
			for(var i = 0, len = children.length; i < len; i++){
				ml += this.generate_1(children[i], 0);
			}
		}
		ml += '</body>\n</html>\n';
		return ml;
	},

	generate_1: function(widget, indent){
		var ml = "";
		for(var i = 0; i < indent; i++){
			ml += "  ";
		}
		var dojoType = widget.declaredClass;

		// begin tag
		if(widget.encodeBegin){
			ml += widget.encodeBegin();
		}else{
			ml += "<div dojoType=\""+dojoType+"\"";
			var attrs = this.getAttributes(widget);
			if(attrs){
				for(var j = 0, len2 = attrs.length; j < len2; j++){
					var attr = attrs[j];
					var name = attr.name;
					var value = widget.ps_getPropertyValue(name);
					if(value != null && value != "" && typeof(value) != "undefined"){ // if(!value) doesn't work. value can be a number.
						if(typeof(value) == "string"){
							value = value.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
						}
						ml += " "+name+"=\""+value+"\"";
					}
				}
			}
			var s = widget.hlp_generateStyleValues();
			if(s){
				ml += " style=\""+s+"\"";
			}
			ml += ">";
		}

		// children
		if(widget.encodeChildren){
			ml += widget.encodeChildren();
		}else{
			var ml2 = "";
			var children = widget.getChildren ? widget.getChildren() : null;
			if(children){
				for(var i = 0, len = children.length; i < len; i++){
					ml2 += this.generate_1(children[i], indent + 1);
				}
			}
			if(ml2){
				ml += "\n" + ml2;
				for(var i = 0; i < indent; i++){
					ml += "  ";
				}
			}
		}

		// end tag
		if(widget.encodeEnd){
			var s = widget.encodeEnd();
			if(s){
				ml += s + "\n";
			}
		}else{
			ml += "</div>\n";
		}
		return ml;
	}
};

}

if(!dojo._hasResource["btp.widget.data.htmlLoader"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.data.htmlLoader"] = true;
dojo.provide("btp.widget.data.htmlLoader");

btp.widget.data.htmlLoader = {
    load: function(contents){
		if(!contents){ return false; }
		this.domNode.innerHTML = contents;
		this.loadComponents(this.domNode);
		dojo.parser.parse(this.domNode);

		var children = this.getChildren();
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			this.setupWidgetsForEdit(child);
		}

		var _this = this;
		setTimeout(function(){
			_this.adjustCanvasSize();
		}, 0);

		this.fireInputChanged();
		return true;
	}
};

}

if(!dojo._hasResource["btp.widget.SimpleTree"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.SimpleTree"] = true;
dojo.provide("btp.widget.SimpleTree");




dojo.declare(
	"btp.widget.SimpleTree",
	[dijit._Widget, dijit._Container, dijit._Contained],
{
	selectedNode: null,

	postCreate: function(){
		this.domNode.className = "btpSimpleTree";
		this.connect(this.domNode, "onkeydown", "onKeyDown");
		this.connect(this.domNode, "onmousedown", "onMouseDown");
	},

	addChild: function(/*Widget*/widget, /*int?*/insertIndex){
		widget.tree = this;
		btp.widget.SimpleTree.superclass.addChild.apply(this, arguments);
		this.fireTreeNodeAdded(widget);
	},

	onKeyDown: function(e){
		var elem = e.target; // clicked element
		if(elem.tagName != "SPAN"){ return; }
		var node = dijit.byNode(elem.parentNode);
		if(e.keyCode == dojo.keys.UP_ARROW){
			var prev;
			if(node.getPreviousSibling()){
				var prev = node.getPreviousSibling();
				if(prev.containerNode.style.display != "none" && prev.hasChildren()){
					var children = prev.getChildren();
					prev = children[children.length - 1];
				}
			}else if(node.getParent()){
				prev = node.getParent();
			}
			if(prev && prev.declaredClass == "btp.widget.SimpleTreeNode"){
				this.select(prev);
				prev.setFocus();
			}
		}else if(e.keyCode == dojo.keys.DOWN_ARROW){
			var next;
			if(node.containerNode.style.display != "none" && node.hasChildren()){
				next = node.getChildren()[0];
			}else if(node.getNextSibling()){
				next = node.getNextSibling();
			}else if(node.getParent()){
				next = node.getParent().getNextSibling();
			}
			if(next){
				this.select(next);
				next.setFocus();
			}
		}else if(e.keyCode == dojo.keys.LEFT_ARROW){
			node.collapse();
		}else if(e.keyCode == dojo.keys.RIGHT_ARROW){
			node.expand();
		}
	},

	onMouseDown: function(e){
		var elem = e.target; // clicked element
		var node;
		if(elem.className == "btpTreeLabel" || elem.className == "btpTreeFolderIcon"){
			node = dijit.byNode(elem.parentNode.parentNode);
		}
		if(!node){ return; }
		this.select(node);
		node.setFocus();
	},

	select: function(/*SimpleTreeNode*/node){
		if(node == this.selectedNode){ return; }
		if(this.selectedNode){
			this.selectedNode.deselect();
		}
		if(!node){ return; }
		for(var n = node.getParent(); n.declaredClass == "btp.widget.SimpleTreeNode"; n = n.getParent()){
			n.expand();
		}
		node.select();
		this.selectedNode = node;
		this.fireTreeNodeSelected(node);
	},

	deselectAll: function(){
		if(this.getSelectedNode()){
			this.getSelectedNode().deselect();
		}
	},

	getSelectedNode: function(){
		return this.selectedNode;
	},

	fireTreeNodeSelected: function(node){
		var message = {
			type: "TreeNodeSelectedNotification",
			src: this,
			selection: node
		};
		dojo.publish("btp:TreeNodeSelected", [message]);
	},

	fireTreeNodeAdded: function(node){
		var message = {
			type: "TreeNodeAddedNotification",
			src: this,
			node: node
		};
		dojo.publish("btp:TreeNodeAdded", [message]);
	},

	onBeforeExpand: function(/*SimpleTreeNode*/node){
		// Called before a node is expanded. You may want to override this or connect to this.
	}
});

dojo.declare(
	"btp.widget.SimpleTreeNode",
	[dijit._Widget, dijit._Container, dijit._Contained],
{
	label: "",
	iconImage: "",
	isExpandable: "",
	tooltip: "",

	tree: null,

	imgDir: dojo.moduleUrl("btp", "widget/templates/SimpleTree/"),
	plusIcons: ["plus0.gif", "plus2.gif", "plus12.gif", "plus23.gif", "plus123.gif"],
	minusIcons: ["minus0.gif", "minus2.gif", "minus12.gif", "minus23.gif", "minus123.gif"],
	emptyIcons: ["empty0.gif", "empty2.gif", "empty12.gif", "empty23.gif", "empty123.gif", "empty13.gif"],
	folderIcon: "folderXP.gif",
	fileIcon: "file.gif",

	constructor: function(params){
		dojo.mixin(this, params);
	},

	postCreate: function(){
		var node = this.domNode;
		node.className = "btpSimpleTreeNode";

		var sectionImg = this.sectionImg = dojo.doc.createElement("IMG");
		sectionImg.className = "btpTreeSectionIcon";
		if(this.isExpandable){
			sectionImg.src = this.imgDir + this.plusIcons[0];
		}else{
			sectionImg.src = this.imgDir + this.emptyIcons[0];
		}
		sectionImg.align = "middle";
		node.appendChild(sectionImg);
		this.connect(sectionImg, "onclick", "changeTreeState");
	
		var contentNode = this.contentNode = dojo.doc.createElement("SPAN");
		contentNode.className = "btpTreeContentNode";
		node.appendChild(contentNode);

		var folderImg = this.folderImg = dojo.doc.createElement("IMG");
		folderImg.className = "btpTreeFolderIcon";
		if(this.iconImage){
			folderImg.src = (this.iconImage.indexOf("/") != -1) ? this.iconImage : (this.imgDir + this.iconImage);
		}else{
			folderImg.src = this.imgDir + this.folderIcon;
		}
		folderImg.align = "middle";
		contentNode.appendChild(folderImg);
	
		var labelNode = this.labelNode = dojo.doc.createElement("SPAN");
		labelNode.className = "btpTreeLabel";
		labelNode.tabIndex = "0";
		labelNode.title = this.tooltip;
		labelNode.innerHTML = this.label;
		contentNode.appendChild(labelNode);

		var containerNode = this.containerNode = dojo.doc.createElement("DIV");
		containerNode.style.display = "none";
		node.appendChild(containerNode);
	},

	addChild: function(/*Widget*/widget, /*int?*/insertIndex){
		if(!this.isExpandable){
			this.isExpandable = true;
			this.sectionImg.src = this.imgDir + this.plusIcons[0];
		}
		widget.tree = this.tree;
		btp.widget.SimpleTreeNode.superclass.addChild.apply(this, arguments);
		this.tree.fireTreeNodeAdded(widget);
	},

	changeTreeState: function(e){
		if(this.containerNode.style.display == "none"){
			this.expand();
		}else{
			this.collapse();
		}
	},

	expand: function(){
		if(this.containerNode.style.display != "none"){ return; }
		if(this.tree){ this.tree.onBeforeExpand(this); }
		if(!this.hasChildren()){
			this.sectionImg.src = this.imgDir + this.emptyIcons[0];
			return;
		}
		this.sectionImg.src = this.imgDir + this.minusIcons[0];
		this.containerNode.style.display = "";
	},

	collapse: function(){
		if(this.containerNode.style.display == "none" || !this.hasChildren()){ return; }
		this.sectionImg.src = this.imgDir + this.plusIcons[0];
		this.containerNode.style.display = "none";
	},

	setFocus: function(){
		this.labelNode.focus();
	},

	select: function(){
		dojo.addClass(this.labelNode, "btpSimpleTreeNodeSelected");
	},

	deselect: function(){
		dojo.removeClass(this.labelNode, "btpSimpleTreeNodeSelected");
	}
});

}

if(!dojo._hasResource["btp.widget.XmlTree"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.XmlTree"] = true;
dojo.provide("btp.widget.XmlTree");




dojo.declare(
	"btp.widget.XmlTree",
	btp.widget.SimpleTree,
{
	modelId: '',
	nodeset: '',

	selectedNodes: null,

	postCreate: function(){
		btp.widget.XmlTree.superclass.postCreate.apply(this, arguments);
		this.updateTree();
	},

	updateTree: function(json){
		this.selectedNode = null;
		if(json)
			this.selectedNode = json;
		else if(this.modelId){
			var model = dijit.byId(this.modelId);
			if(!model)
				return;
			this.selectedNodes = model.selectNodesFromJson(this.nodeset);
			if(!this.selectedNodes){
				alert("XmlTree: "+model.src+" not found.");
				return;
			}
		}
		else
			return;
		for(var i = 0, len = this.selectedNodes.length; i < len; i++){
			var node = this.createNode(this.selectedNodes[i]);
			this.addChild(node);
		}
	},

	createNode: function(obj){
		var label = obj["@label"];
		var tooltip = obj["@tooltip"];
		var iconPath = obj["@iconPath"];
		var isFolder = obj["@isFolder"] == "true";
		var iconImage = iconPath ? dojo.moduleUrl("btp", iconPath)+"" : null;
		if(!isFolder){
			for(var key in obj){
				if(key.charAt(0) != "@"){
					isFolder = true;
					break;
				}
			}
		}

		var props = {
			label: label,
			tooltip: tooltip,
			iconImage: iconImage,
			isExpandable: isFolder
		};
		var node = new btp.widget.SimpleTreeNode(props);
		node.jsonObj = obj;
		return node;
	},

	// override
	onBeforeExpand: function(/*SimpleTreeNode*/node){
		if(!node.isExpandable || node.getChildren().length > 0 || !node.jsonObj){ return; }
		for(var key in node.jsonObj){
			if(key.charAt(0) == "@"){ continue; }
			var arr = node.jsonObj[key];
			if(!dojo.isArray(arr)){
				arr = [arr];
			}
			for(var i = 0, len = arr.length; i < len; i++){
				var child = this.createNode(arr[i]);
				node.addChild(child);
			}
		}
	}
});

}

if(!dojo._hasResource["btp.widget.DataSourceView"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.DataSourceView"] = true;
dojo.provide("btp.widget.DataSourceView");





dojo.declare(
	"btp.widget.DataSourceView",
	[dijit._Widget, dijit._Contained, dijit._Container],
{
	modelId: '',
	nodeset: '.',
	label: '',

	tree: null,
	tmpModel: null,

	
	postCreate: function(){
		this.domNode.className = "btpDataSourceView btpSimplePane";

		this._topics = [
			dojo.subscribe("btp:TreeNodeAdded", this, "onAfterAddChild"),
//			dojo.subscribe("btp:ActiveViewChanged", this, "activeViewChanged"),
//			dojo.subscribe("btp:TreeNodeSelected", this, "treeNodeSelected"),
//			dojo.subscribe("btp:TreeNodeDblSelected", this, "treeNodeDblSelected")
		];

		// tree		
		this.tree = new btp.widget.XmlTree({modelId:this.modelId, nodeset:this.nodeset});
		this.addChild(this.tree);

		this.connect(this.tree, "onBeforeExpand", "onBeforeExpand");
	},

	destroy: function(){
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},


	onBeforeExpand: function(/*SimpleTreeNode*/node){
		if(!node.isExpandable || node.getChildren().length > 0){ return; }
		if(node.jsonObj && node.jsonObj["@data"]){
			if(!this.tmpModel){
				this.tmpModel = new btp.widget.Model();
			}
			var model = this.tmpModel;
			model.setSrc(node.jsonObj["@data"]);
			var root = model.selectSingleNode(".");
			var child = this.createNodeFromXmlNode(root);
			node.addChild(child);
		}else if(node.xmlNode){
			var elem = node.xmlNode;
			for(var i = 0, len = elem.childNodes.length; i < len; i++){
				var childElem = elem.childNodes[i];
				if(childElem.nodeType != 1){
					continue;
				}
				var child = this.createNodeFromXmlNode(childElem);
				node.addChild(child);
			}
		}
	},

	createNodeFromXmlNode: function(elem){
		var label = elem.nodeName;
		var tooltip = ""; // @TODO
		var isFolder = elem.childNodes.length > 0;
		var iconImage = dojo.moduleUrl("btp", "widget/templates/SimpleTree/xml_image.gif")+"";

		var props = {
			label: label,
			tooltip: tooltip,
			iconImage: iconImage,
			isExpandable: isFolder
		};
		var node = new btp.widget.SimpleTreeNode(props);
		node.xmlNode = elem;
		return node;
	},

	onAfterAddChild: function(message){
		var node = message.node;
		var ds = new btp.dnd.DragSource(node.contentNode, this.id);
		ds.treeNode = node;
	}
});

}

if(!dojo._hasResource["btp.widget.EditorFrame"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.EditorFrame"] = true;
dojo.provide("btp.widget.EditorFrame");




dojo.declare(
	"btp.widget.EditorFrame",
	btp.widget.BottomTabPanel,
{
	_editDomain: null,
	defaultTool: "",
	
	postCreate: function(){
		btp.widget.EditorFrame.superclass.postCreate.apply(this, arguments);
	},
	
	startup: function(){
		btp.widget.EditorFrame.superclass.startup.apply(this, arguments);
		this._editDomain = new btp.widget.tool.EditDomain(this.defaultTool);
		for(var i = 0, len = this.tabs.length; i < len; i++){
			var page = this.tabs[i].page;
			if(page.setEditDomain){
				page.setEditDomain(this.getEditDomain());
			}
			if(page.activeToolChanged){
				this.getEditDomain().setPaletteViewer(page);
			}
		}
	},

	getEditDomain: function(){
		return this._editDomain;
	}
});

}

if(!dojo._hasResource["btp.widget.ext.selectionHandles"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.ext.selectionHandles"] = true;
dojo.provide("btp.widget.ext.selectionHandles");

// summary:
//		Extension for creating selection handles for a component on the EditArea.
//		Note that this is not a widget.
//		this.owner points to the EditArea.




dojo.declare(
	"btp.widget.ext.selectionHandles",
	null,
{
	owner: null,
	selectionKnobType: "blackSquare",

	// constants
	_NORTH: 1,
	_SOUTH: 2,
	_WEST: 4,
	_EAST: 8,

	// initializer
	constructor: function(owner){
		this.owner = owner;
		this._connectors = [];
		this._connectors.push(dojo.connect(owner, "setFocusFrame", this, "addSelectionHandles"));
		this._connectors.push(dojo.connect(owner, "removeFocusFrame", this, "removeSelectionHandles"));
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
	},

	/*              NORTH
	 *       0 ------ 1 ------ 2
	 *       |                 |
	 *  WEST 7                 3 EAST
	 *       |                 |
	 *       6 ------ 5 ------ 4
	 *              SOUTH
	 */
	addSelectionHandles: function(widget){
		if(!widget){
			widget = this.owner.focusWidgets[0];
		}
		if(!widget
			|| widget.domNode.offsetWidth == 0 || widget.domNode.offsetHeight == 0){ // Maybe it is a gfx line widget
			return;
		}
		var info = widget.hlp_getComponentInfo();
		if(!info.resizable){ return; }
		if(widget.__showingSelectionHandles){
			; // already showing
		}else if(!widget.__selectionHandles){
			this.createSelectionHandles(widget);
		}else{
			for(var i = 0, len = widget.__selectionHandles.length; i < len; i++){
				this.owner.domNode.appendChild(widget.__selectionHandles[i]);
			}
		}
		widget.__showingSelectionHandles = true;

		var dim = btp.getStyleDimensions(widget, this.owner);
		var north = dim.top - 4 + "px";
		var south = dim.top + dim.height - 2 + "px";
		var west = dim.left - 4 + "px";
		var east = dim.left + dim.width - 2 + "px";
		var center = dim.left + Math.round(dim.width/2) - 3 + "px";
		var middle = dim.top + Math.round(dim.height/2) - 3 + "px";

		var knob;
		var i = 0;

		// NORTH-WEST
		knob = widget.__selectionHandles[i++];
		knob.style.top = north;
		knob.style.left = west;

		// NORTH
		knob = widget.__selectionHandles[i++];
		knob.style.top = north;
		knob.style.left = center;

		// NORTH-EAST
		knob = widget.__selectionHandles[i++];
		knob.style.top = north;
		knob.style.left = east;

		// EAST
		knob = widget.__selectionHandles[i++];
		knob.style.top = middle;
		knob.style.left = east;

		// SOUTH-EAST
		knob = widget.__selectionHandles[i++];
		knob.style.top = south;
		knob.style.left = east;

		// SOUTH
		knob = widget.__selectionHandles[i++];
		knob.style.top = south;
		knob.style.left = center;

		// SOUTH-WEST
		knob = widget.__selectionHandles[i++];
		knob.style.top = south;
		knob.style.left = west;

		// WEST
		knob = widget.__selectionHandles[i++];
		knob.style.top = middle;
		knob.style.left = west;
	},

	removeSelectionHandles: function(widget){
		if(!widget){
			widget = this.owner.focusWidgets[0];
		}
		if(!widget){
			return; // removeFocus() was called when nothing is focused.
		}
		if(!widget.__showingSelectionHandles){
			return;
		}
		for(var i = 0, len = widget.__selectionHandles.length; i < len; i++){
			this.owner.domNode.removeChild(widget.__selectionHandles[i]);
		}
		widget.__showingSelectionHandles = false;
	},

	createSelectionHandles: function(widget){
		var moveTarget = this.owner.getMoveTarget();
		var knobElem = this.owner.createKnob(this.selectionKnobType);

		var info = widget.hlp_getComponentInfo();
		var dirs = 0;
		if(info.resizable.directions){
			var arr = info.resizable.directions.split("|");
			for(var i = 0; i < arr.length; i++){
				if(arr[i] == "NORTH"){
					dirs |= this._NORTH;
				}else if(arr[i] == "SOUTH"){
					dirs |= this._SOUTH;
				}else if(arr[i] == "WEST"){
					dirs |= this._WEST;
				}else if(arr[i] == "EAST"){
					dirs |= this._EAST;
				}else{
					alert("unrecognized direction: "+arr[i]);
				}
			}
		}
		widget.__resizeDirections = dirs; // bitwise

		widget.__selectionHandles = [];
		var knob;
		var _this = this;
		for(var i = 0; i < 8; i++){
			knob = knobElem.cloneNode(true);
			knob.knobPosition = i; // [custom property]
			knob.getDragTracker = function(){ return _this.owner.resizeTrackerClass; };
			widget.__selectionHandles.push(knob);
			this.owner.domNode.appendChild(knob);

			var ds = new btp.dnd.DragMoveSource(knob);
			ds.setDragTarget(moveTarget);
			
			switch (i){
				case 0:
					knob.style.cursor = "nw-resize"; break;
				case 1:
					knob.style.cursor = "n-resize";
					if(!(dirs & this._NORTH)){
						knob.style.display = "none";
					}
					break;
				case 2:
					knob.style.cursor = "ne-resize"; break;
				case 3:
					knob.style.cursor = "e-resize";
					if(!(dirs & this._EAST)){
						knob.style.display = "none";
					}
					break;
				case 4:
					knob.style.cursor = "se-resize"; break;
				case 5:
					knob.style.cursor = "s-resize";
					if(!(dirs & this._SOUTH)){
						knob.style.display = "none";
					}
					break;
				case 6:
					knob.style.cursor = "sw-resize"; break;
				case 7:
					knob.style.cursor = "w-resize";
					if(!(dirs & this._WEST)){
						knob.style.display = "none";
					}
					break;
			}
		}
	}
});

}

if(!dojo._hasResource["btp.widget.helper.baseHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.helper.baseHelper"] = true;
dojo.provide("btp.widget.helper.baseHelper");

btp.widget.helper.baseHelper = {
	hlp_getEditArea: function(){
		return this.__editArea;
	},

	hlp_setEditArea: function(editArea){
		this.__editArea = editArea;
	},

	hlp_getComponentInfo: function(){
		return this.__component_info;
	},

	hlp_initLiveWidget: function(editArea){
		var info = this.hlp_getComponentInfo();
		if(info && info.dropTargets){
			if(this.hlp_dropTargets){
				this.hlp_dropTargets = this.hlp_dropTargets.concat(info.dropTargets.split(","));
			}else{
				this.hlp_dropTargets = info.dropTargets.split(",");
			}
		}
		if(this.hlp_setupDropTarget){
			this.hlp_setupDropTarget();
		}
	},

	hlp_beforeAddChild: function(parent, widget){
		// Your widget may want to override this.
	},

	hlp_afterAddChild: function(parent, widget){
		// Your widget may want to override this.
	},

	hlp_beforeRemoveChild: function(parent, widget){
		// Your widget may want to override this.
	},

	hlp_afterRemoveChild: function(parent, widget){
		// Your widget may want to override this.
	},

	hlp_addChild: function(/*Widget*/widget, /*int|Widget?*/insertIndex){
		if(insertIndex && typeof(insertIndex) != "number"){ // convert refWidget to insertIndex 
			insertIndex = dojo.indexOf(this.getChildren(), insertIndex);
			if(insertIndex == -1){ alert("btp.widget.helper.baseHelper#hlp_addChild: internal error: refWidget not found"); } // TODO: remove this code later
		}
		widget.hlp_beforeAddChild(this, widget);
		this.addChild(widget, insertIndex);
		if(!btp.isOrphan(widget.domNode)){
			var hint = {
				parent: this,
				widget: widget,
				insertIndex: insertIndex
			};
			this.hlp_getEditArea().firePropertyChanged("@AddChild", null, null, hint);
			this.hlp_getEditArea().adjustCanvasWidth(widget);
			this.hlp_getEditArea().adjustCanvasHeight(widget);
		}
		widget.hlp_afterAddChild(this, widget);
	},

	hlp_removeChild: function(/*Widget*/widget){
		widget.hlp_beforeRemoveChild(this, widget);
		var isOrphan = btp.isOrphan(widget.domNode);
		this.removeChild(widget);
		if(!isOrphan){
			var hint = {
				parent: this,
				widget: widget
			};
			this.hlp_getEditArea().firePropertyChanged("@RemoveChild", null, null, hint);
		}
		widget.hlp_afterRemoveChild(this, widget);
	}
};

}

if(!dojo._hasResource["btp.widget.helper.dropHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.helper.dropHelper"] = true;
dojo.provide("btp.widget.helper.dropHelper");

// summary:
//		Make a widget "droppable".
//		If used for EditArea, methods here are called from btp.sub.dropTargetStub.
//		If used for a component, methods here are mixed into it.
//		'this' points to the EditArea or a container type widget.

btp.widget.helper.dropHelper = {
	hlp_dropTargets: null, // each component's helper may want to override this

	hlp_setupDropTarget: function(){
		if(!this.hlp_dropTargets || this.hlp_dropTargets.length == 0){ return; }
		dojo["require"]("btp.dnd.DropTarget");
		var dt;
		if(btp.isGfxWidget(this)){
			var shape = this.getTargetShape();
			if(!shape){ return; }
			dt = this.__dt = new btp.dnd.DropTarget(shape.rawNode, this.hlp_dropTargets, this);
			this.domNode.dropTarget = dt; // [custom property]

			// It looks shape's rowNode does not have parent-child relationship even when
			// two shapes has such relationship. DragManager#findDropTarget fails to find
			// a drop target because of this. Fortunately, parent of the shape's rawNode,
			// which is always the <group> element, has parent-child relationship.
			this.getShape().rawNode.dropTarget = dt;
//			shape.rawNode.parentNode.dropTarget = dt;
		}else{
			dt = this.__dt = new btp.dnd.DropTarget(this.domNode, this.hlp_dropTargets, this);
		}
		dt.offsetParent = this.__editArea.domNode;
		dt.isAcceptable = function dt_isAcceptable(ds, dt){
			// If the drag source is an ancestor of the drop target, do not accept.
			var dsWidget = ds.data;
			var dtWidget = dt.data;
			if(!dsWidget && !dtWidget){ return true; }
			for(var w = dtWidget; w; w = w.parent){
				if(w == dsWidget){
					return false;
				}
			}
			return true;
		};
		this.__dtConns = [];
		this.__dtConns.push(dojo.connect(dt, "onDragStart", this.__editArea, "onEaDragStartHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDragOver", this.__editArea, "onEaDragOverHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDragMove", this.__editArea, "onEaDragMoveHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDragEnd", this.__editArea, "onEaDragEndHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDropStart", this.__editArea, "onEaDropStartHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDrop", this.__editArea, "onEaDropHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDropEnd", this.__editArea, "onEaDropEndHandler"));
		this.__dtConns.push(dojo.connect(dt, "onDragOut", this.__editArea, "onEaDragOutHandler"));
		if(this.hlp_showAllowDropFeedback){
			this.__dtConns.push(dojo.connect(dt, "showFeedback", this, "hlp_showAllowDropFeedback"));
		}
		if(this.hlp_hideAllowDropFeedback){
			this.__dtConns.push(dojo.connect(dt, "hideFeedback", this, "hlp_hideAllowDropFeedback"));
		}
		if(this.hlp_dropTargetDragStart){
			this.__dtConns.push(dojo.connect(dt, "onDragStart", this, "hlp_dropTargetDragStart"));
		}
	},

	hlp_cleanupDropTarget: function(){
		var dt = this.__dt;
		if(dt){
			dojo.forEach(this.__dtConns, dojo.disconnect);
			dt.unregister();
			this.__dt = null;
		}
	},

	hlp_getDroppedDomNode: function(e){
		var node;
		if(e.type == "mouseup"){ // Dropped with D&D
			var ds = e.dragSource;
			srcElem = ds.dragObject; // dropped DOM node
		}
		else /*if(e.type == "click")*/ { // Dropped with click and release
			srcElem = this.__editArea.selectedPaletteItem.domNode;
		}
		return srcElem;
	}
};

}

if(!dojo._hasResource["btp.widget.helper.focusHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.helper.focusHelper"] = true;
dojo.provide("btp.widget.helper.focusHelper");

btp.widget.helper.focusHelper = {
	// ------------- Focus Frame ----------------------------
	hlp_setFocusFrame: function(){
		if(this.__component_info.visible == "false"){ return; }
		this.hlp_showComponentFrame("focus");
	},

	hlp_removeFocusFrame: function(){
		if(this.__component_info.visible == "false"){ return; }
		this.hlp_hideComponentFrame("focus");
	},

	hlp_refreshFocusFrame: function(){
		this.hlp_removeFocusFrame();
		this.hlp_setFocusFrame();
	},

	// ------------- Feedback Frame ----------------------------
	hlp_showAllowDropFeedback: function(){
		this.hlp_showComponentFrame("feedback");
	},

	hlp_hideAllowDropFeedback: function(){
		this.hlp_hideComponentFrame("feedback");
	},

	// -------- Show/Hide a frame around the component ----------
	hlp_showComponentFrame: function(type){
		// 'type' must be "focus" or "feedback"

		function __createFrame(color, zIndex){
			var line = dojo.doc.createElement("DIV");
			line.style.zIndex = zIndex;
			line.style.position = "absolute";
			line.style.fontSize = "1px";
			line.style.borderStyle = "solid";
			line.style.borderColor = color;
			var line2 = line.cloneNode(false);
			var line3 = line.cloneNode(false);
			var line4 = line.cloneNode(false);
			line.appendChild(line2);
			line.appendChild(line3);
			line.appendChild(line4);
			return line;
		}

		if(this == this.__editArea){ return; }
		var topLine;
		if(type == "focus"){
			if(!this.__focusFrame){
				this.__focusFrame = __createFrame("black", 1000);
			}
			topLine = this.__focusFrame;
		}else if(type == "feedback"){
			if(!this.__feedbackFrame){
				this.__feedbackFrame = __createFrame("yellow", 1001);
			}
			topLine = this.__feedbackFrame;
		}else{
			return; // never reached
		}
		var bottomLine = topLine.childNodes[0];
		var leftLine = topLine.childNodes[1];
		var rightLine = topLine.childNodes[2];

		var dim = btp.getStyleDimensions(this, this.__editArea);

		// Top Line
		topLine.style.top = dim.top - 2 + "px";
		topLine.style.left = dim.left - 1 + "px";
		topLine.style.width = dim.width + 2 + "px";
		topLine.style.height = "1px";
		topLine.style.borderWidth = "0px 0px 1px 0px";

		// Bottom Line
		bottomLine.style.top = dim.height + 2 + "px";
		bottomLine.style.left = "0px";
		bottomLine.style.width = dim.width + 2 + "px";
		bottomLine.style.height = "1px";
		bottomLine.style.borderWidth = "1px 0px 0px 0px";

		// Left Line
		leftLine.style.top = "1px";
		leftLine.style.left = "-1px";
		leftLine.style.width = "1px";
		leftLine.style.height = dim.height + 2 + "px";
		leftLine.style.borderWidth = "0px 1px 0px 0px";

		// Right Line
		rightLine.style.top = "1px";
		rightLine.style.left = dim.width + 1 + "px";
		rightLine.style.width = "1px";
		rightLine.style.height = dim.height + 2 + "px";
		rightLine.style.borderWidth = "0px 0px 0px 1px";

		this.__editArea.domNode.appendChild(topLine);
	},

	hlp_hideComponentFrame: function(type){
		if(this == this.__editArea){ return; }
		if(type == "focus"){
			this.__focusFrame.parentNode.removeChild(this.__focusFrame);
		}else if(type == "feedback"){
			this.__feedbackFrame.parentNode.removeChild(this.__feedbackFrame);
		}
	}
};

}

if(!dojo._hasResource["btp.widget.helper.styleHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.helper.styleHelper"] = true;
dojo.provide("btp.widget.helper.styleHelper");

btp.widget.helper.styleHelper = {
	// property: "top", "left", "width", "height", or "borderWidth"
	// return: integer value
	hlp_getStyle: function(property){
		return btp.getStyleValue(this, property);
	},

	// property: "top", "left", etc.
	// value: string value with "px". (Ex. "100px")
	hlp_setStyle: function(property, value){
		var props = {};
		props[property] = value;
		this.hlp_setStyles(props);
	},

	hlp_setStyles: function(props){
		var property, value;
		for(property in props){
			value = props[property];
			btp.setStyleValue(this, property, value);
		}
		if(value == undefined){ return; }
		var info = this.__component_info;
		if(info.resizable && info.resizable.refreshMethod){
			this[info.resizable.refreshMethod]();
		}
		if(this.__editArea.adjustCanvasWidth){
			this.__editArea.adjustCanvasWidth(this);
		}
		if(this.__editArea.adjustCanvasHeight){
			this.__editArea.adjustCanvasHeight(this);
		}
		if(btp.isGfxWidget(this)){
			this.update();
		}		
		if(this.__editArea.firePropertyChanged){
			this.__editArea.firePropertyChanged("@SetStyles", null, null, {widget:this,props:props});
		}

	},

	// summary: generates value for the style attribute.
	//   Ex. "position:absolute;top:25px;left:19px;width:429px;height:289px;"
	hlp_generateStyleValues: function(){
		var hasAccessor = btp.hasAccessor;
		var isResizableHorizontally = btp.isResizableHorizontally;
		var isResizableVertically = btp.isResizableVertically;

		var s = "";
		var info = this.hlp_getComponentInfo();
		var position = dojo.style(this.domNode, "position");
		if(position && (position == "absolute" || position == "relative")){
			s += "position:"+position+";";

			// Top
			if(!hasAccessor(this, "top")){
				var val = btp.getStyleValue(this, "top");
				if(val && val != "auto") //TODO: Do not omit curly braces, '{' and '}'.
					s += "top:" + val + "px;";
			}

			// Left
			if(!hasAccessor(this, "left")){
				var val = btp.getStyleValue(this, "left");
				if(val && val != "auto") //TODO: Do not omit curly braces, '{' and '}'.
					s += "left:" + val + "px;";
			}
		}
		
		if(info.resizable){
			// Width
			if(!hasAccessor(this, "width") && isResizableHorizontally(this)){
				var val = this.domNode.style.width;
				if(val && val != "auto") //TODO: Do not omit curly braces, '{' and '}'.
					s += "width:" + val + ";";
			}

			// Height
			if(!hasAccessor(this, "height") && isResizableVertically(this)){
				var val = this.domNode.style.height;
				if(val && val != "auto") //TODO: Do not omit curly braces, '{' and '}'.
					s += "height:" + val + ";";
			}
		}
		return s;
	}
};

}

if(!dojo._hasResource["btp.widget.io.FileIO"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.io.FileIO"] = true;
dojo.provide("btp.widget.io.FileIO");

btp.widget.io.FileIO = {
	dosometest: null
/*
	currentAppId: null,

	initcurrentAppId: function(){
		if(window.location.href.indexOf("/XmlEditor/") != -1){
			this.currentAppId = "xed";
		}else if(window.location.href.indexOf("/GraphicalEditor/") != -1){
			this.currentAppId = "ged";
		}else if(window.location.href.indexOf("/CustomizableRssReader/") != -1){
			this.currentAppId = "red";
		}else{
			this.currentAppId = "led";
		}
	},
	
	getResourceUrl: function(path){
		// summary:
		//		Get a url for the raw resource.
		//		Retuns a full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1)
		// path:
		//		can be one of these:
		//		- resourceID (Ex. cat1/resource1)
		//		- full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1;edit)
		//		- (null) (Assumes the current location is the parameter)
		if(!this.currentAppId) //TODO: Do not omit curly braces, '{' and '}'.
			this.initcurrentAppId();
		if(path){
			var fullpath = window.location.href;
			var pos = fullpath.lastIndexOf("/");
			fullpath = fullpath.substring(0, pos);
			fullpath += "/../hale/test/repository/"+this.currentAppId+"/"+path;
			return fullpath;
		}

		var model = dijit.byId(this.modelId);
		if(model){
			return model.src;
		}
		return null;
	},

	getViewUrl: function(path){
		return null;
	},

	getEditUrl: function(path){
		return null;
	},

	getListUrl: function(){
		if(!this.currentAppId){
			this.initcurrentAppId();
		}
	    if(window.location.href.match(/([^\/]+)\/[^\/]+$/)){
			appl = RegExp.$1;
		}
		return dojo.moduleUrl("hale", "../../test/data/resourceList-"+this.currentAppId+".xml")+"";
	},

	sendOpenFileRequest: function(path, opts){
		alert("sendOpenFileRequest");
	},
	
	sendNewFileRequest: function(path, opts){
		alert("sendNewFileRequest: not implemented yet.");
	},
	
	sendNewFolderRequest: function(path, opts){
		alert("sendNewFolderRequest: not implemented yet.");
	},
	
	sendRenameRequest: function(oldPath, newPath, opts){
		alert("sendRenameRequest: not implemented yet.");
	},
	
	sendDeleteFileRequest: function(path, opts){
		alert("sendDeleteFileRequest: not implemented yet.");
	},

	sendSaveRequest: function(content, path, opts){
		// path can be null or undefined
		var url = this.getResourceUrl(path);
		alert("sendSaveRequest: not implemented yet.\nThe content should be saved to "+url);
	}
*/
};

}

if(!dojo._hasResource["btp.widget.io.RestIO"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.io.RestIO"] = true;
dojo.provide("btp.widget.io.RestIO");

btp.widget.io.RestIO = {
	getResourceUrl: function(path){
		// summary:
		//		Get a url for the raw resource.
		//		Retuns a full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1)
		// path:
		//		can be one of these:
		//		- resourceID (Ex. cat1/resource1)
		//		- full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1;edit)
		//		- (null) (Assumes the current location is the parameter)
		var url = path;
		if(!url){
			var model = dijit.byId(this.modelId);
			url = model.src;
		}
		if(!url){
			url = window.location.href;
		}
		if(url.indexOf("://") != -1){ // full URL
			var frag = btp.splitUrl(url);
			if(!frag["resourceId"]){
				return null;
			}
			url = frag["url"];
		}else{
			var frag = btp.splitUrl(window.location.href);
			url = frag["list"]+"/"+path;
		}
		return url;
	},

	getViewUrl: function(path){
		// summary:
		//		Get a url for an html representation of resource.
		//		Retuns a full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1.html)
		// path:
		//		can be one of these:
		//		- resourceID (Ex. cat1/resource1)
		//		- full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1;edit)
		//		- (null) (Assumes the current location is the parameter)
		var url = this.getResourceUrl(path);
		if(!url){ return null; }
		return url+".html";
	},

	getEditUrl: function(path){
		// summary:
		//		Get a url for an editing form of resource.
		//		Retuns a full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1;edit)
		// path:
		//		can be one of these:
		//		- resourceID (Ex. cat1/resource1)
		//		- full URL (Ex. http://localhost/btp/repos/space1/cat1/resource1;edit)
		//		- (null) (Assumes the current location is the parameter)
		return this.getResourceUrl(path)+";edit";
	},
	
	getListUrl: function(){
		// summary:
		//		Get a url for a listing of resources.
		//		Retuns a full URL (Ex. http://localhost/btp/repos/space1)
		var url = window.location.href;
		var frag = btp.splitUrl(url);
		if(frag["spaceId"]){
			return frag["list"];
		}
		return null;
	},

	_sendRequest: function(props){
		var url = props.url;
		var method = props.method;
		var content = props.content;
		var postContent = props.postContent;

		var data = null;
		if(content){
			data = dojo.io.argsFromMap(content, "utf8");
		}else{
			data = postContent;
		}

		dojo.debug("sendRequest: "+method+" "+url);
		var http = dojo.hostenv.getXmlhttpObject();
		http.open(method, url, false);
		if(props.contentType){
			http.setRequestHeader("Content-Type", props.contentType);
		}else{
			http.setRequestHeader("Content-Type", "text/html; charset=UTF-8");
		}

		try {
			http.send(data);
		}
		catch (e){
			alert("HTTP "+method+" request Failed: "+url); // TODO: i18n
			return null;
		}

		if(method != "GET" && http.responseText.indexOf("Success") == -1){
			return false;
		}
		return true;
	},

	sendOpenFileRequest: function(path, opts){
		var url = this.getResourceUrl(path);
		var req = {
			method: "GET",
			url: url
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	},
	
	sendNewFileRequest: function(path, opts){
		var url = this.getResourceUrl(path);
		var req = {
			method: "POST",
			url: url
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	},
	
	sendNewFolderRequest: function(path, opts){
		var url = this.getResourceUrl(path)+"/";
		var req = {
			method: "PUT",
			url: url
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	},
	
	sendRenameRequest: function(oldPath, newPath, opts){
		var url = this.getResourceUrl(oldPath);
		var req = {
			method: "POST",
			url: url,
			content: {newResourceId:newPath},
			contentType: "application/x-www-form-urlencoded"
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	},
	
	sendDeleteFileRequest: function(path, opts){
		var url = this.getResourceUrl(path);
		var req = {
			method: "DELETE",
			url: url
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	},

	sendSaveRequest: function(content, path, opts){
		// path can be null or undefined
		var url = this.getResourceUrl(path);
		var req = {
			method: "PUT",
			url: url,
			postContent: content
		};
		if(opts){
			for(var key in opts){
				req[key] = opts[key];
			}
		}
		return this._sendRequest(req);
	}
};

}

if(!dojo._hasResource["btp.widget.outline.CloneContentOutlinePage"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.outline.CloneContentOutlinePage"] = true;
dojo.provide("btp.widget.outline.CloneContentOutlinePage");

// summary:
//		The default implementation of OutlinePage, which displays a structure
//		of some model provided with a ContentProvider in the OutlineView.
//		Your application may want to replace this with yours.
//
//	interface:
//		setContentProvider(/*Widget*/provider)
//		setOutlineView(/*Widget*/view)
//		createOutlinePage()
//		attachOutline()
//		detachOutline()
//		refreshOutline()
//		updateOutline(/*Object*/message)
//		selectOutline(/*Widget*/selectedWidget)
//		selectOutlineTarget(/*Widget*/outlineNode)
//		deleteOutlineTarget()

dojo.declare("btp.widget.outline.CloneContentOutlinePage", null,
{
	editArea: null,
	outlineView: null,
	contentProvider: null,
	outlineViewMenu: null,
	canvas: null,
	_connectors: null,
	_topics: null,

	imgDir: dojo.moduleUrl("btp", "widget/templates/OutlineView/"),

	constructor: function(editArea){
		this.editArea = editArea;
		this._connectors = [];
		this._topics = [
		];
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	setContentProvider: function(/*Widget*/provider){
		this.contentProvider = provider;
	},

	setOutlineView: function(/*Widget*/view){
		this.outlineView = view;
	},

	createOutlinePage: function(){
		if(!this.canvas){
			if(window.gdl){
				dojo["require"]("gdc.widget.Surface");
				this.canvas = new gdc.widget.Surface({width:1000,height:1000}); //@knitta@ temporary fix
				this.canvas.domNode.style.backgroundColor = 'pink'; //@kamki@ delete this later
			}else{
				dojo["require"]("btp.widget.SimplePane");
				this.canvas = new btp.widget.SimplePane();
			}
			this.outlineView.addChild(this.canvas);
		}else{
			this.detachOutline();
		}

		// if you want to have your own content provider, you should call setContentProvider() first.
		if(!this.contentProvider){
			var providerClassName = this.editArea.getContentProviderClass();
			if(providerClassName){
				dojo["require"](providerClassName);
				impl = dojo.getObject(providerClassName);
				this.setContentProvider(new impl());
			}
		}
	},

	attachOutline: function(){
		if(this.outlineView && this.canvas){
			this.outlineView.addChild(this.canvas);
			return true;
		}
		return false;
	},

	detachOutline: function(){
		if(this.outlineView && this.canvas){
			this.outlineView.removeChild(this.canvas);
		}
	},

	refreshOutline: function(){
		this.canvas.destroyDescendants();
		var root = this.contentProvider.getElements(this.editArea);
		for(var i = 0, len = root.length; i < len; i++){
			this._refreshCloneView(null, root[i]);
		}
	},

	_refreshCloneView: function(parentObj, obj){
		var node = obj.outlineNode = this._cloneWidget(obj.targetNode); // Widget
		node.outlineObj = obj;
		var parentNode = (parentObj && parentObj.outlineNode) ? parentObj.outlineNode : this.canvas;
		parentNode.addChild(node);
		if(btp.isGfxWidget(node)){
			node.createShape();
			node.update();
		}

		var children = this.contentProvider.getChildren(obj);
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			this._refreshCloneView(obj, child);
		}
	},

	updateOutline: function(message){
		if(message.propName == "@AddChild" || message.propName == "@RemoveChild"){
			var root = this.contentProvider.getElements(this.editArea);
			this._updateCloneView(root, this.canvas);
		}else if(message.propName == "@SetStyles"){
			var widget = message.hint.widget;
			var outlineNode = widget.outlineObj.outlineNode;
			for(var key in message.hint.props){
				outlineNode.domNode.style[key] = widget.domNode.style[key];
			}
		}else{
			var widget = message.hint.widget;
			var outlineNode = widget.outlineObj.outlineNode;
			var name = message.propName;
			var f = "set" + name.charAt(0).toUpperCase() + name.substring(1);
			if(outlineNode[f]){
				outlineNode[f](message.newValue);
			}else{
				outlineNode[name] = message.newValue;
			}
		}
	},

	_updateCloneView: function(objs, /*Widget*/parentNode){
		var nObjs = objs.length;
		var nNodes = parentNode.getChildren ? parentNode.getChildren().length : 0;
		if(nObjs == nNodes){
			// no changes
			for(var i = 0, len = objs.length; i < len; i++){
				var chlidObjs = this.contentProvider.getChildren(objs[i]);
				this._updateCloneView(chlidObjs, parentNode.getChildren()[i]);
			}
		}else if(nObjs > nNodes){
			// a new node has been added
			for(var i = 0, len = nObjs; i < len; i++){
				var obj = objs[i];
				var node = parentNode.getChildren()[i];
				if(!node || node.outlineObj != obj){ // newly added node found
					var parentObj = this.contentProvider.getParent(obj);
					this._refreshCloneView(parentObj, obj);
					break;
				}
			}
		}else{
			// a node has been removed
			for(var i = 0, len = nNodes; i < len; i++){
				var obj = objs[i];
				var node = parentNode.getChildren()[i];
				if(!obj || obj.outlineNode != node){ // removed node found
					node.destroy();
					break;
				}
			}
		}
	},

	_cloneWidget: function(/*Widget*/widget){
		var impl = dojo.getObject(widget.declaredClass);
		var clone = new impl();
		var styles = ["position", "top", "left"];
		for(var i = 0, len = styles.length; i < len; i++){
			clone.domNode.style[styles[i]] = widget.domNode.style[styles[i]];
		}
		return clone;
	},

	// select a node in the OutlineView
	selectOutline: function(/*Widget*/selectedWidget){
	},

	// select a widget in the EditArea
	selectOutlineTarget: function(/*Widget*/outlineNode){
	},

	deleteOutlineTarget: function(){
	}
});

}

if(!dojo._hasResource["btp.widget.outline.TreeContentOutlinePage"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.outline.TreeContentOutlinePage"] = true;
dojo.provide("btp.widget.outline.TreeContentOutlinePage");


// summary:
//		The default implementation of OutlinePage, which displays a structure
//		of some model provided with a ContentProvider in the OutlineView.
//		Your application may want to replace this with yours.
//
//	interface:
//		setContentProvider(/*Widget*/provider)
//		setOutlineView(/*Widget*/view)
//		createOutlinePage()
//		attachOutline()
//		detachOutline()
//		refreshOutline()
//		updateOutline(/*Object*/message)
//		selectOutline(/*Widget*/selectedWidget)
//		selectOutlineTarget(/*Widget*/outlineNode)
//		deleteOutlineTarget()

dojo.declare("btp.widget.outline.TreeContentOutlinePage", null,
{
	editArea: null,
	outlineView: null,
	contentProvider: null,
	outlineViewMenu: null,
	tree: null,
	_connectors: null,
	_topics: null,

	imgDir: dojo.moduleUrl("btp", "widget/templates/OutlineView/"),

	constructor: function(editArea){
		this.editArea = editArea;
		this._connectors = [];
		this._topics = [
			dojo.subscribe("btp:TreeNodeSelected", this, "_treeNodeSelected")
		];
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	setContentProvider: function(/*Widget*/provider){
		this.contentProvider = provider;
	},

	setOutlineView: function(/*Widget*/view){
		this.outlineView = view;
		if(!this._oncontextmenu_connected){
			this._connectors.push(dojo.connect(view.domNode, "oncontextmenu", this, "_onOutlineViewContextMenu"));
			this._oncontextmenu_connected = true;
		}
	},

	createOutlinePage: function(){
		if(!this.tree){
			this.tree = new btp.widget.SimpleTree();
			this.outlineView.addChild(this.tree);
		}else{
			this.detachOutline();
		}

		// if you want to have your own content provider, you should call setContentProvider() first.
		if(!this.contentProvider){
			var providerClassName = this.editArea.getContentProviderClass();
			if(providerClassName){
				dojo["require"](providerClassName);
				impl = dojo.getObject(providerClassName);
				this.setContentProvider(new impl());
			}
		}
	},

	attachOutline: function(){
		if(this.outlineView && this.tree){
			this.outlineView.addChild(this.tree);
			return true;
		}
		return false;
	},

	detachOutline: function(){
		if(this.outlineView && this.tree){
			this.outlineView.removeChild(this.tree);
		}
	},

	refreshOutline: function(){
		this.tree.destroyDescendants();
		var root = this.contentProvider.getElements(this.editArea);
		for(var i = 0, len = root.length; i < len; i++){
			this._refreshTreeView(null, root[i]);
		}
	},

	_createIconNode: function(opt){
		var props = {
			label: opt.label,
			tooltip: opt.tooltip,
			iconImage: this.imgDir + "element.gif"
		};
		var node = new btp.widget.SimpleTreeNode(props);
		return node;
	},

	_refreshTreeView: function(parentObj, obj){
		var node = obj.outlineNode = this._createIconNode(obj);
		node.outlineObj = obj;
		var parentNode = (parentObj && parentObj.outlineNode) ? parentObj.outlineNode : this.tree;
		parentNode.addChild(node);

		var children = this.contentProvider.getChildren(obj);
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			this._refreshTreeView(obj, child);
		}
	},

	updateOutline: function(message){
		var root = this.contentProvider.getElements(this.editArea);
		this._updateTreeView(root, this.tree);
	},

	_updateTreeView: function(objs, /*SimpleTreeNode*/parentNode){
		var nObjs = objs.length;
		var nNodes = parentNode.getChildren().length;
		if(nObjs == nNodes){
			// no changes
			for(var i = 0, len = objs.length; i < len; i++){
				var chlidObjs = this.contentProvider.getChildren(objs[i]);
				this._updateTreeView(chlidObjs, parentNode.getChildren()[i]);
			}
		}else if(nObjs > nNodes){
			// a new node has been added
			for(var i = 0, len = nObjs; i < len; i++){
				var obj = objs[i];
				var node = parentNode.getChildren()[i];
				if(!node || node.outlineObj != obj){ // newly added node found
					var parentObj = this.contentProvider.getParent(obj);
					this._refreshTreeView(parentObj, obj);
					break;
				}
			}
		}else{
			// a node has been removed
			for(var i = 0, len = nNodes; i < len; i++){
				var obj = objs[i];
				var node = parentNode.getChildren()[i];
				if(!obj || obj.outlineNode != node){ // removed node found
					node.destroy();
					break;
				}
			}
		}
	},

	// select a node in the OutlineView
	selectOutline: function(/*Widget*/selectedWidget){
		var node;
		if(selectedWidget){
			node = selectedWidget.outlineObj.outlineNode;
			this.tree.select(node);
		}else{ // BODY was selected on the canvas
			this.tree.deselectAll();
		}
	},


	// select a widget in the EditArea
	selectOutlineTarget: function(/*Widget*/outlineNode){
		var targetNode = outlineNode.outlineObj.targetNode;
		if(targetNode == this.editArea){
			this.editArea.setFocus(null); // BODY was clicked
		}else if(targetNode instanceof dijit._Widget){
			this.editArea.setFocus(targetNode);
		}
	},

	deleteOutlineTarget: function(){
		var treeNodeWidget = this.tree.getSelectedNode();
		var targetNode = treeNodeWidget.outlineObj.targetNode;
		if(targetNode == this.editArea){
			this.editArea.setFocus(null); // BODY was clicked
		}else if(targetNode instanceof dijit._Widget){
			this.editArea.setFocus(targetNode);
			this.editArea.deleteFocusWidget();
		}
	},


	_onOutlineViewContextMenu: function(e){
		if(!this.tree.getParent()){ return; }
		var elem = e.target; // clicked element
		if(elem.tagName != "SPAN"){
			dojo.stopEvent(e);
			return;
		}
		if(!this.outlineViewMenu){
			dojo["require"]("dijit.Menu");
			var outlineViewMenu = this.outlineViewMenu = new dijit.Menu();
			var mItem = new dijit.MenuItem({label:"Delete"});
			mItem.iconNode.style.backgroundImage = "url("+dojo.moduleUrl("btp", "widget/templates/Toolbar/delete.gif")+")";
			this._connectors.push(dojo.connect(mItem, "onClick", this, "deleteOutlineTarget"));
			outlineViewMenu.addChild(mItem);
			outlineViewMenu.startup();
		}
		this.outlineViewMenu._openMyself(e);
	},

	_treeNodeSelected: function(message){
		if(message.src != this.tree){ return; }
		var treeNodeWidget = message.selection;
		this.selectOutlineTarget(treeNodeWidget);
	}
});

}

if(!dojo._hasResource["btp.widget.OutlineViewEx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.OutlineViewEx"] = true;
dojo.provide("btp.widget.OutlineViewEx");




dojo.declare(
	"btp.widget.OutlineViewEx",
	btp.widget.OutlineView,
{
	_toolbars: null,
	_activeToolbar: null,

	postCreate: function(){
		btp.widget.OutlineViewEx.superclass.postCreate.apply(this, arguments);
		btp.loadExtension(this, "btp.widget.ext.localToolbar");
		this._toolbars = {};
	},

	changeOutlineMode: function(message){
		var oldPage = this.editArea.getContentOutlinePage();
		if(oldPage){ oldPage.detachOutline(); }

		this.editArea.setOutlineMode(message.arg);

		var newPage = this.editArea.getContentOutlinePage();
		if(newPage){
			newPage.attachOutline();
			newPage.refreshOutline();
		}else{
			this.loadContentOutlinePage();
		}
	},

	// override
	editorActivated: function(message){
		btp.widget.OutlineViewEx.superclass.editorActivated.apply(this, arguments);
		if(message.type == "ActiveEditorChangedNotification"){
			var ext = this.extensions["btp.widget.ext.localToolbar"];
			var toolbar = this._toolbars[this.editArea.declaredClass];
			if(!toolbar){
				var json = this.editArea.getOutlineToolbarConf();
				if(!json){ return; }
				toolbar = this._toolbars[this.editArea.declaredClass] = ext.createLocalToolbar(json);
			}
			ext.attachLocalToolbar(toolbar);
			var mode = this.editArea.getOutlineMode();
			var item;
			for(var i = 0, len = toolbar.items.length; i < len; i++){
				item = toolbar.items[i];
				if(item.arg == mode){
					break;
				}
			}
			toolbar.selectRadioButton(item);
		}
	}
});

}

if(!dojo._hasResource["btp.widget.PropertiesView"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.PropertiesView"] = true;
dojo.provide("btp.widget.PropertiesView");



dojo.declare(
	"btp.widget.PropertiesView",
	[dijit._Widget, dijit._Contained, dijit._Container],
{
	label: '',
	target: '',
	propertyChangedFilter: '',
	selectionChangedFilter: '',

	i18nPrefix: "PropertiesView_",

	postCreate: function(){
		this.domNode.className = "btpPropertiesView btpSimplePane";

		this.propPageMap = {};
		if(this.target){
			this.propertyChangedFilter = this.target;
			this.selectionChangedFilter = this.target;
		}

		this._topics = [
			dojo.subscribe("btp:PropertyChanged", this, "propertyChangedStub"),
			dojo.subscribe("btp:SelectionChanged", this, "selectionChangedStub")
		];
	},

	destroy: function(){
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	_loadImplementation: function(){
		var subModules = ["update", "event"];
		for(var i = 0, len = subModules.length; i < len; i++){
			var subModuleName = "btp.widget._propertiesView." + subModules[i];
			dojo["require"](subModuleName);
			var props = dojo.getObject(subModuleName);
			dojo.mixin(this, props);
		}
	},

	_changeSubscribers: function(){
		var _this = this;
		setTimeout(function(){
			dojo.forEach(_this._topics, dojo.unsubscribe);
			_this._topics = [
				dojo.subscribe("btp:PropertyChanged", _this, "propertyChanged"),
				dojo.subscribe("btp:SelectionChanged", _this, "selectionChanged")
			];
		});
	},

	propertyChangedStub: function(message){
		this._loadImplementation();
		this.propertyChanged(message);
		this._changeSubscribers();
	},

	selectionChangedStub: function(message){
		this._loadImplementation();
		this.selectionChanged(message);
		this._changeSubscribers();
	}
});

}

if(!dojo._hasResource["btp.widget.propsrc.componentPropertySource"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.propsrc.componentPropertySource"] = true;
dojo.provide("btp.widget.propsrc.componentPropertySource");

btp.widget.propsrc.componentPropertySource = {
	ps_getPropertyDescriptors: function(){
		var attributes = this.hlp_getEditArea().getAttributes(this);
		return attributes ? attributes : [];
	},

	ps_getPropertyValue: function(name){
		var f = "get" + name.charAt(0).toUpperCase() + name.substring(1);
		if(this[f]){
			return this[f]();
		}
		return this[name];
	},

	ps_setPropertyValue: function(name, value){
		var oldValue = this.ps_getPropertyValue(name);
		var f = "set" + name.charAt(0).toUpperCase() + name.substring(1);
		if(this[f]){
			this[f](value);
		}else{
			this[name] = value;
		}
		this.hlp_getEditArea().firePropertyChanged(name, oldValue, value, {widget:this});
	}
};

}

if(!dojo._hasResource["btp.widget.provider.DomTreeContentProvider"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.provider.DomTreeContentProvider"] = true;
dojo.provide("btp.widget.provider.DomTreeContentProvider");

dojo.declare("btp.widget.provider.DomTreeContentProvider", null,
{
	input: null,
	viewer: null,
	view: null,

	// public Object[] getElements(Object inputElement);
	getElements: function(/*EditArea*/inputElement){
		var input = this.input = inputElement; // input should be EditArea
		return this.getChildren({targetNode:input.domNode});
	},

	// public void inputChanged(Viewer viewer, Object oldInput, Object newInput);
	inputChanged: function(viewer, oldInput, newInput){
		this.viewer = viewer;
		this.input = newInput;
	},

	// public Object[] getChildren(Object parentElement);
	getChildren: function(/*Object*/parentElement){
		var parentNode = parentElement.targetNode;
		var arr = [];
		var len = parentNode.childNodes.length;
		for(var i = 0; i < len; i++){
			var child = parentNode.childNodes[i];
			if(parentNode == this.input.domNode && !dijit.byNode(child)){ continue; }
			if(child.nodeType != 1){ continue; }
			var obj = child.outlineObj;
			if(!obj){
				var title = child.tagName;
				obj = {
					// required fields
					label: title,
					icon: null,
					tooltip: child.outerHTML,
	
					// additional fields
					targetNode: child
				};
				child.outlineObj = obj;
				var widget = dijit.byNode(child);
				if(widget){
					widget.outlineObj = obj;
				}
			}
			obj.parentElement = parentElement;
			arr.push(obj);
		}
		return arr;
	},

	// public Object getParent(Object element);
	getParent: function(element){
		return element.parentElement;
	},

	// public boolean hasChildren(Object element);
	hasChildren: function(element){
	},

	getInput: function(){
		return this.input;
	}
});

}

if(!dojo._hasResource["btp.widget.provider.GfxNodeTreeContentProvider"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.provider.GfxNodeTreeContentProvider"] = true;
dojo.provide("btp.widget.provider.GfxNodeTreeContentProvider");

dojo.declare("btp.widget.provider.GfxNodeTreeContentProvider", null,
{
	input: null,
	viewer: null,
	view: null,

	// public Object[] getElements(Object inputElement);
	getElements: function(/*EditArea*/inputElement){
		var input = this.input = inputElement; // input should be EditArea
		return this.getChildren({targetNode:input});
	},

	// public void inputChanged(Viewer viewer, Object oldInput, Object newInput);
	inputChanged: function(viewer, oldInput, newInput){
		this.viewer = viewer;
		this.input = newInput;
	},

	// public Object[] getChildren(Object parentElement);
	getChildren: function(/*Object*/parentElement){
		var parentWidget = parentElement.targetNode;
		var arr = [];
		var surface;
		if(parentWidget.surface) surface = parentWidget.surface;
		else surface = parentWidget;
		var len = 0;
		if(surface && surface.children) len = surface.children.length;
		for(var i = 0; i < len; i++){
			var child = surface.children[i];
			var obj = child.outlineObj;
			if(!obj){
				var title = child.elementName ? child.elementName : child.declaredClass.replace(/.*\./, "");
				obj = {
					// required fields
					label: title,
					icon: null,

					// additional fields
					targetNode: child
				};
				child.outlineObj = obj;
			}
			obj.parentElement = parentElement;
			arr.push(obj);
		}
		return arr;
	},

	// public Object getParent(Object element);
	getParent: function(element){
		return element.parentElement;
	},

	// public boolean hasChildren(Object element);
	hasChildren: function(element){
	},

	getInput: function(){
		return this.input;
	}
});

}

if(!dojo._hasResource["btp.widget.provider.WidgetTreeContentProvider"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.provider.WidgetTreeContentProvider"] = true;
dojo.provide("btp.widget.provider.WidgetTreeContentProvider");

dojo.declare("btp.widget.provider.WidgetTreeContentProvider", null,
{
	input: null,
	viewer: null,
	view: null,

	// public Object[] getElements(Object inputElement);
	getElements: function(/*EditArea*/inputElement){
		var input = this.input = inputElement; // input should be EditArea
		return this.getChildren({targetNode:input});
	},

	// public void inputChanged(Viewer viewer, Object oldInput, Object newInput);
	inputChanged: function(viewer, oldInput, newInput){
		this.viewer = viewer;
		this.input = newInput;
	},

	// public Object[] getChildren(Object parentElement);
	getChildren: function(/*Object*/parentElement){
		var parentWidget = parentElement.targetNode;
		var arr = [];
		var len = parentWidget.getChildren ? parentWidget.getChildren().length : 0;
		for(var i = 0; i < len; i++){
			var child = parentWidget.getChildren()[i];
			var obj = child.outlineObj;
			if(!obj){
				var title = child.elementName ? child.elementName : child.declaredClass.replace(/.*\./, "");
				obj = {
					// required fields
					label: title,
					icon: null,

					// additional fields
					targetNode: child
				};
				child.outlineObj = obj;
			}
			obj.parentElement = parentElement;
			arr.push(obj);
		}
		return arr;
	},

	// public Object getParent(Object element);
	getParent: function(element){
		return element.parentElement;
	},

	// public boolean hasChildren(Object element);
	hasChildren: function(element){
	},

	getInput: function(){
		return this.input;
	}
});

}

if(!dojo._hasResource["btp.widget.SimplePane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.SimplePane"] = true;
dojo.provide("btp.widget.SimplePane");




dojo.declare(
	"btp.widget.SimplePane",
	[dijit._Widget, dijit._Container, dijit._Contained],
{
	postCreate: function(){
		dojo.addClass(this.domNode, "btpSimplePane");
	}
});

}

if(!dojo._hasResource["btp.widget.SimpleTabPanel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.SimpleTabPanel"] = true;
dojo.provide("btp.widget.SimpleTabPanel");



dojo.declare(
	"btp.widget.SimpleTabPanel",
	btp.widget._TabPanelBase,
{
	tailTab: null,

	lightGray: "#DEDFDE",
	darkGray: "#ACA899",

	topicNamePrefix: "SimpleTab",

	postCreate: function(){
		this.domNode.className = "tabPanel simpleTabPanel";
		var header = this.tabHeaderNode;
		this.tabs = [];

		this.childWidgets = [
			{domNode: this.tabHeaderNode, layoutAlign: "top"},
			{domNode: this.containerNode, layoutAlign: "client"}
		];

		// to keep height even when there are no tabs
		var dummy = dojo.doc.createElement("DIV");
		dummy.className = "tabPanelDummyTab";
		header.appendChild(dummy);

		// to draw an lower line on the right side
		var tab = this.tailTab = dojo.doc.createElement("DIV");
		tab.className = "tabPanelTab";
		tab.style.width = "20px";
		tab.style.borderTopColor = "white";
		tab.style.borderLeftColor = "white";
		tab.style.borderRightColor = "white";
		header.appendChild(tab);

		btp.widget.SimpleTabPanel.superclass.postCreate.apply(this, arguments);
	},

	setupChild: function(/*ContentPane*/ page){
		var header = this.tabHeaderNode;
		var tab = dojo.doc.createElement("DIV");
		tab.className = "tabPanelTab";
		tab.page = page; // [custom property]
		page.tab = tab; // [custom property]
		this.tabs.push(tab);

		page.domNode.style.position = "relative";
		page.domNode.style.display = "block";

		// Title Node
		var titleNode = dojo.doc.createElement("DIV");
		titleNode.className = "tabPanelTabTitle";
		titleNode.tabIndex = 0;
		var label = page.label; //TODO: hale.i18n.gets(page.label, this.i18nPrefix);
		titleNode.innerHTML = label;
		if(page.tooltip){
			tab.title = page.tooltip;
		}
		tab.appendChild(titleNode);
		tab.titleNode = titleNode; // [custom property]

		header.insertBefore(tab, this.tailTab);

		tab._connectors = [];
		tab._connectors.push(dojo.connect(tab, "onclick", this, "onTabClick"));
		tab._connectors.push(dojo.connect(tab, "ondblclick", this, "onTabDblClick"));
		tab._connectors.push(dojo.connect(tab, "onkeydown", this, "onTabKeyDown"));

		page.domNode.style.display = "none";
	},

	// override dijit._Container.removeChild
	removeChild: function(/*Widget*/widget){
		var page = widget;
		var tab = widget.tab;
		for(var i = 0, len = this.tabs.length; i < len; i++){
			if(this.tabs[i] == tab){
				this.tabs.splice(i, 1); // remove tab from tabs
				break;
			}
		}
		if(this.tabs.length == 0){
			this.unhilightTab(tab);
		}
		this.tabHeaderNode.removeChild(tab);
		if(this.selectedPage == page){
			this.selectedPage = null;
		}
		dojo.forEach(tab._connectors, dojo.disconnect);
		page.destroy();
		if(this.tabs.length == 0 && this.deleteButton){
			this.deleteButton.style.display = "none";
			this.edgeImages[0].style.display = "none";
			this.edgeImages[1].style.display = "none";
			this.containerNode.style.backgroundColor = "#EFEBDE";
			this.fireTabChanged(null);
		}else{
			this.selectTab(this.tabs[0]);
		}
	},

	hilightTab: function(tab, focus){
		tab.style.backgroundImage = "url("+dojo.moduleUrl("btp", "widget/themes/images/simple-tab-bg.gif")+")";
		tab.style.borderTopColor = this.darkGray;
		tab.style.borderBottomColor = "white";
		tab.style.borderLeftColor = this.darkGray;
		tab.style.borderRightColor = this.darkGray;
		this.isActive = true;

		// focus (necessary for keyboard navigation)
		if(focus){
			tab.titleNode.focus();
		}

		// set tabIndex only for the selected tab
		for(var i = 0, len = this.tabs.length; i < len; i++){
			this.tabs[i].titleNode.tabIndex = (this.tabs[i] == tab) ? 0 : -1;
		}
	},

	unhilightTab: function(tab){
		this.hilightTab(tab);
	},

	grayoutTab: function(tab){
		tab.style.backgroundImage = "";
		tab.style.borderColor = this.lightGray;
	}
});

}

if(!dojo._hasResource["btp.widget.SourceView"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.SourceView"] = true;
dojo.provide("btp.widget.SourceView");



dojo.declare(
	"btp.widget.SourceView",
	[dijit._Widget, dijit._Contained],
{
	modelId: '',
	ref: '',
	label: '',
	target: '',

	editAreaWidget: null,
	_topics: null,

	constructor: function(){
		this._topics = [];
	},

	postCreate: function(){
		this.domNode.className = "btpSourceView btpSimplePane";
		this._topics.push(dojo.subscribe("btp:InputChanged", this, "loadCompleted"));
	},

	destroy: function(){
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	show: function(){
		if(!this.editAreaWidget){
			this.editAreaWidget = dijit.byId(this.target);
		}
		this.updateView();
	},

	updateView: function(){
		if(!this.editAreaWidget){ return; }
		var ml = this.editAreaWidget.generate(this.editAreaWidget);
		if(!ml){ return; }
		ml = ml.replace(/(<\/?\w+[^>]+>)/g, "\f$1\b");
		ml = ml.replace(/&/g, '&amp;');
		ml = ml.replace(/>/g, '&gt;');
		ml = ml.replace(/</g, '&lt;');
		ml = ml.replace(/ /g, '&nbsp;');
		ml = ml.replace(/\t/g, '&nbsp;&nbsp;');
		ml = ml.replace(/\n/g, '<BR>');
		while(true){
			var idx = ml.indexOf("\f");
			if(idx == -1){
				break;
			}
			var idx2 = ml.indexOf("\b");
			var s = ml.substring(idx + 1, idx2);
			var spc = s.indexOf("&nbsp;");
			if(spc != -1){
				s = s.replace(/(\w+)=(\"[^\"]*\")/g, "<font color=\"#7B007B\">$1</font>=<font color=\"#2900FF\">$2</font>");
			}
			ml = ml.substring(0, idx)
				+ '<font color="#397D7B">' + s + '</font>'
				+ ml.substring(idx2 + 1);
		}
		this.domNode.innerHTML = ml;
	},

	loadCompleted: function(){
		if(btp.isShowing(this)){
			this.updateView();
		}
	}
});

}

if(!dojo._hasResource["btp.widget.StatusBar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.StatusBar"] = true;
dojo.provide("btp.widget.StatusBar");





dojo.declare(
	"btp.widget.StatusBar",
	[dijit.layout.ContentPane, dijit._Templated],
{
	timeout: '7000',
	target: '',
	messagePostedFilter: '',

	postCreate: function(){
		btp.widget.StatusBar.superclass.postCreate.apply(this, arguments);
		this.domNode.className = "statusBar";
		if(this.target){
			this.messagePostedFilter = this.target;
		}
		dojo.subscribe("btp:MessagePosted", this, "showMessage");
	},

	destroy: function(){
		dojo.unsubscribe("btp:MessagePosted", this, "showMessage");
		this.inherited("destroy", arguments);
	},

	showMessage: function(message){
		if(!btp.matchFilter(message, this.messagePostedFilter)){
			return; // filter out the message
		}
		
		this.setContent(message.text);
		var _this = this;
		window.setTimeout(function(){
			_this.setContent("");
		}, this.timeout - 0);
	}
});

}

if(!dojo._hasResource["dojox.fx._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.fx._base"] = true;
dojo.provide("dojox.fx._base");
// summary: add-on Animations to dojo.fx

 

// convenience functions: 
dojox.fx.chain = dojo.fx.chain;
dojox.fx.combine = dojo.fx.combine;
dojox.fx.wipeIn = dojo.fx.wipeIn;
dojox.fx.wipeOut = dojo.fx.wipeOut;
dojox.fx.slideTo = dojo.fx.slideTo;

dojox.fx.sizeTo = function(/* Object */args){
	// summary: Create an animation that will size a node
	// description:
	//	Returns an animation that will size "node" 
	//	defined in args Object about it's center to
	//	a width and height defined by (args.width, args.height), 
	//	supporting an optional method: chain||combine mixin
	//	(defaults to chain).	
	//
	//	- works best on absolutely or relatively positioned elements? 
	//	
	// example:
	// |	// size #myNode to 400px x 200px over 1 second
	// |	dojo.fx.sizeTo({ node:'myNode',
	// |		duration: 1000,
	// |		width: 400,
	// |		height: 200,
	// |		method: "chain"
	// |	}).play();
	//
	var node = (args.node = dojo.byId(args.node));
	var compute = dojo.getComputedStyle;

	var method = args.method || "chain"; 
	if (method=="chain"){ args.duration = Math.floor(args.duration/2); } 
	
	var top, newTop, left, newLeft, width, height = null;

	var init = (function(){
		var innerNode = node;
		return function(){
			var pos = compute(innerNode).position;
			top = (pos == 'absolute' ? node.offsetTop : parseInt(compute(node).top) || 0);
			left = (pos == 'absolute' ? node.offsetLeft : parseInt(compute(node).left) || 0);
			width = parseInt(dojo.style(node,'width'));
			height = parseInt(dojo.style(node,'height'));

			newLeft = left - Math.floor((args.width - width)/2); 
			newTop = top - Math.floor((args.height - height)/2); 

			if(pos != 'absolute' && pos != 'relative'){
				var ret = dojo.coords(innerNode, true);
				top = ret.y;
				left = ret.x;
				innerNode.style.position="absolute";
				innerNode.style.top=top+"px";
				innerNode.style.left=left+"px";
			}
		}
	})();
	init(); // hmmm, do we need to init() or just the once beforeBegin?

	var anim1 = dojo.animateProperty(dojo.mixin({
		properties: {
			height: { start: height, end: args.height || 0, unit:"px" },
			top: { start: top, end: newTop }
		}
	}, args));
	var anim2 = dojo.animateProperty(dojo.mixin({
		properties: {
			width: { start: width, end: args.width || 0, unit:"px" },
			left: { start: left, end: newLeft }
		}
	}, args));

	var anim = dojo.fx[((args.method == "combine") ? "combine" : "chain")]([anim1,anim2]);
	dojo.connect(anim, "beforeBegin", anim, init);
	return anim; // dojo._Animation
};

dojox.fx.slideBy = function(/* Object */args){
	// summary: Returns an animation to slide a node by a defined offset.
	//
	// description:
	//	Returns an animation that will slide a node (args.node) from it's
	//	current position to it's current posision plus the numbers defined
	//	in args.top and args.left. standard dojo.fx mixin's apply. 
	//	
	// example:
	// |	// slide domNode 50px down, and 22px left
	// |	dojox.fx.slideBy({ 
	// |		node: domNode, duration:400, 
	// |		top: 50, left: -22 
	// |	}).play();

	var node = (args.node = dojo.byId(args.node));	
	var compute = dojo.getComputedStyle;
	var top = null; var left = null;
	var init = (function(){
		var innerNode = node;
		return function(){
			var pos = compute(innerNode,'position');
			top = (pos == 'absolute' ? node.offsetTop : parseInt(compute(node, 'top')) || 0);
			left = (pos == 'absolute' ? node.offsetLeft : parseInt(compute(node, 'left')) || 0);
			if(pos != 'absolute' && pos != 'relative'){
				var ret = dojo.coords(innerNode, true);
				top = ret.y;
				left = ret.x;
				innerNode.style.position="absolute";
				innerNode.style.top=top+"px";
				innerNode.style.left=left+"px";
			}
		}
	})();
	init();
	var _anim = dojo.animateProperty(dojo.mixin({
		properties: {
			// FIXME: is there a way to update the _Line after creation?
			// null start values allow chaining to work, animateProperty will
			// determine them for us (except in ie6? -- ugh)
			top: {  /* start: top, */end: top+(args.top||0) },
			left: { /* start: left, */end: left+(args.left||0) }
		}
	}, args));
	dojo.connect(_anim,"beforeBegin",_anim,init);
	return _anim; // dojo._Animation
};

dojox.fx.crossFade = function(/* Object */args){
	// summary: Returns an animation cross fading two element simultaneously
	// 
	// args:
	//	args.nodes: Array - two element array of domNodes, or id's
	//
	// all other standard animation args mixins apply. args.node ignored.
	//
	if(dojo.isArray(args.nodes)){
		// simple check for which node is visible, maybe too simple?
		var node1 = args.nodes[0] = dojo.byId(args.nodes[0]);
		var op1 = dojo.style(node1,"opacity");
		var node2 = args.nodes[1] = dojo.byId(args.nodes[1]);
		var op2 = dojo.style(node2, "opacity");

		var _anim = dojo.fx.combine([
			dojo[((op1==0)?"fadeIn":"fadeOut")](dojo.mixin({
				node: node1
			},args)),
			dojo[((op1==0)?"fadeOut":"fadeIn")](dojo.mixin({
				node: node2
			},args))
		]);
		return _anim; // dojo._Animation
	}else{
		// improper syntax in args, needs Array
		return false; // Boolean
	}
};

dojox.fx.highlight = function(/*Object*/ args){
	// summary: Highlight a node
	// description:
	//	Returns an animation that sets the node background to args.color
	//	then gradually fades back the original node background color
	//	
	// example:
	//	dojox.fx.highlight({ node:"foo" }).play(); 

	var node = (args.node = dojo.byId(args.node));

	args.duration = args.duration || 400;
	// Assign default color light yellow
	var startColor = args.color || '#ffff99';
	var endColor = dojo.style(node, "backgroundColor");
	var wasTransparent = (endColor == "transparent" || endColor == "rgba(0, 0, 0, 0)");

	var anim = dojo.animateProperty(dojo.mixin({
		properties: {
			backgroundColor: { start: startColor, end: endColor }
		}
	}, args));

	dojo.connect(anim, "onEnd", anim, function(){
		if(wasTransparent){
			node.style.backgroundColor = "transparent";
		}
	});

	return anim; // dojo._Animation
};

}

if(!dojo._hasResource["dojox.fx.style"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.fx.style"] = true;
dojo.provide("dojox.fx.style");
dojo.experimental("dojox.fx.style"); 
//
// summary: dojox.fx CSS Class _Animations: 
//
// description: a set of functions to animate properties based on
// 	normalized CSS class definitions.
//
//	provides: addClass, removeClass, and toggleClass
//	
 

// FIXME: should the call signatures match dojo.addClass/removeClass/toggleClass and extend
// 	by having a third (or fourth) param to mix in additional _Animation args for advanced
//	usage (delay: curve: repeat: easing: etc ... )

dojox.fx.addClass = function(/*dojox.fx._arg.StyleArgs*/ args){
	// summary: Animate the effects of adding a class to a node
	// description:
	//	Creates an animation that will animate
	//	the properties of a node to the properties
	//	defined in a standard CSS .class definition.
	//	(calculating the differences itself)
	//
	// example:
	// 
	// 	.bar { line-height: 12px; }
	// 	.foo { line-height: 40px; }
	// 	<div class="bar" id="test">
	//	Multi<br>line<br>text
	//	</div> 
	// 
	// 	// animate to line-height:40px
	// 	dojo.fx.addClass({ node:"test", cssClass:"foo" }).play();
	// 
	var node = (args.node = dojo.byId(args.node)); 

	var pushClass = (function(){
		// summary: onEnd we want to add the class to the node 
		//	(as dojo.addClass naturally would) in case our 
		//	class parsing misses anything the browser would 
		// 	otherwise interpret. this may cause some flicker,
		//	and will only apply the class so children can inherit 
		//	after the animation is done (potentially more flicker)
		var innerNode = node; // FIXME: why do we do this like this?
		return function(){
			dojo.addClass(innerNode, args.cssClass); 
			innerNode.style.cssText = _beforeStyle; 
		}
	})();

	// _getCalculatedStleChanges is the core of our style/class animations
	var mixedProperties = dojox.fx._getCalculatedStyleChanges(args,true);
	var _beforeStyle = node.style.cssText; 
	var _anim = dojo.animateProperty(dojo.mixin({
		properties: mixedProperties
	},args));
	dojo.connect(_anim,"onEnd",_anim,pushClass); 
	return _anim; // dojo._Animation
};

dojox.fx.removeClass = function(/*dojox.fx._arg.StyleArgs*/ args){
	// summary: Animate the effects of removing a class from a node
	// description:
	//	Creates an animation that will animate the properties of a 
	// 	node (args.node) to the properties calculated after removing 
	//	a standard CSS className from a that node.
	//	
	//	calls dojo.removeClass(args.cssClass) onEnd of animation		
	//
	//	standard dojo._Animation object rules apply. 
	//
	// example:
	// |	// animate the removal of "foo" from a node with id="bar"
	// |	dojox.fx.removeClass({
	// |		node: "bar",
	// |		cssClass: "foo"
	// |	}).play();

	var node = (args.node = dojo.byId(args.node)); 

	var pullClass = (function(){
		// summary: onEnd we want to remove the class from the node 
		//	(as dojo.removeClass naturally would) in case our class
		//	parsing misses anything the browser would otherwise 
		//	interpret. this may cause some flicker, and will only 
		//	apply the class so children can inherit after the
		//	animation is done (potentially more flicker)
		//
		var innerNode = node;
		return function(){
			dojo.removeClass(innerNode, args.cssClass); 
			innerNode.style.cssText = _beforeStyle; 
		}
	})();

	var mixedProperties = dojox.fx._getCalculatedStyleChanges(args,false);
	var _beforeStyle = node.style.cssText; 
	var _anim = dojo.animateProperty(dojo.mixin({
		properties: mixedProperties
	},args));
	dojo.connect(_anim,"onEnd",_anim,pullClass); 
	return _anim; // dojo._Animation
};

dojox.fx.toggleClass = function(/*DomNode|String*/node, /*String*/cssClass, /*Boolean?*/condition){
        // summary:
	//	Animate the effects of Toggling a class on a Node
	//
	// description:
	//	creates an animation that will animate the effect of 
	//	toggling a class on or off of a node.
        //	Adds a class to node if not present, or removes if present.
        //	Pass a boolean condition if you want to explicitly add or remove.
	// node:
	//	The domNode (or string of the id) to toggle
	// cssClass:
	//	String of the classname to add to the node
        // condition:
        //	If passed, true means to add the class, false means to remove.
	//
	// example:
	// |	// add the class "sampleClass" to a node id="theNode"
	// |	dojox.fx.toggleClass("theNode","sampleClass",true).play();
	// example:
	// |	// toggle the class "sampleClass" on the node id="theNode"
	// |	dojox.fx.toggleClass("theNode","sampleClass").play();
	
        if(typeof condition == "undefined"){
                condition = !dojo.hasClass(node, cssClass);
        }
        return dojox.fx[(condition ? "addClass" : "removeClass")]({ node: node, cssClass:cssClass }); // dojo._Animation
	// TODO: support 4th param animMixin to allow passing of easing and duration and other _Animtion options
};

dojox.fx._allowedProperties = [
	// summary:
	//	this is our pseudo map of properties we will check for.
	//	it should be much more intuitive. a way to normalize and
	//	"predict" intent, or even something more clever ... 
	//	open to suggestions.

	// no-brainers:
	"width",
	"height",
	// only if position = absolute || relative?
	"left", "top", "right", "bottom", 
	// these need to be filtered through dojo.colors?
	// "background", // normalize to:
	/* "backgroundImage", */
	// "backgroundPosition", // FIXME: to be effective, this needs "#px #px"?
	"backgroundColor",

	"color",

	// "border", 
	"borderBottomColor", "borderBottomWidth",
	"borderTopColor","borderTopWidth",
	"borderLeftColor","borderLeftWidth",
	"borderRightColor","borderRightWidth",

	// "padding", // normalize to: 
	"paddingLeft", "paddingRight", "paddingTop", "paddingBottom",
	// "margin", // normalize to:
	"marginLeft", "marginTop", "marginRight", "marginBottom",

	// unit import/delicate?:
	"lineHeight",
	"letterSpacing",
	"fontSize"
];

dojox.fx._getStyleSnapshot = function(/* Object */cache){
	// summary: 
	//	uses a dojo.getComputedStyle(node) cache reference and
	// 	iterates through the 'documented/supported animate-able'
	// 	properties. 
	//
	// returns:  Array
	//	an array of raw, calculcated values (no keys), to be normalized/compared
	//	elsewhere	
	return dojo.map(dojox.fx._allowedProperties,function(style){
		return cache[style]; // String
	}); // Array
};

dojox.fx._getCalculatedStyleChanges = function(/*dojox.fx._arg.StyleArgs*/ args, /*Boolean*/addClass){
	// summary: calclate the difference in style properties between two states
	// description:
	//	calculate and normalize(?) the differences between two states
	//	of a node (args.node) by quickly adding or removing a class, and
	//	iterateing over the results of dojox.fx._getStyleSnapshot()
	//
	// addClass: 
	// 	true to calculate what adding a class would do, 
	// 	false to calculate what removing the class would do

	var node = (args.node = dojo.byId(args.node)); 
	var compute = dojo.getComputedStyle(node);

	// take our snapShots
	var _before = dojox.fx._getStyleSnapshot(compute);
	dojo[(addClass ? "addClass" : "removeClass")](node,args.cssClass); 
	var _after = dojox.fx._getStyleSnapshot(compute);
	dojo[(addClass ? "removeClass" : "addClass")](node,args.cssClass); 

	var calculated = {};
	var i = 0;
	dojo.forEach(dojox.fx._allowedProperties,function(prop){
		if(_before[i] != _after[i]){
			// FIXME: the static unit: px is not good, either. need to parse unit from computed style?
			calculated[prop] = { end: parseInt(_after[i]) /* start: parseInt(_before[i]), unit: 'px' */ }; 
		} 
		i++;
	});
	return calculated; 
};

}

if(!dojo._hasResource["btp.widget.StickyToaster"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.StickyToaster"] = true;
dojo.provide("btp.widget.StickyToaster");








dojo.declare(
	"btp.widget.StickyToaster",
	[dijit._Widget, dijit._Container, dijit._Contained, dijit._Templated],
{
	target: "",	// id which this widget sticks to
	
	templateString: '<div dojoAttachPoint="containerNode" class="btpStickyToaster btpSimplePane"><img class="btpStickyToasterHandleImg" dojoAttachPoint="handleImg" dojoAttachEvent="onclick:_onHandleImgClick" src="${handleImgSrc}" width="16" height="16" /></div>',
	
	handleImgSrc: dojo.moduleUrl("btp", "widget/templates/EditArea/handle.gif"),
	
	postCreate: function(){
	},
	
	startup: function() {
		var targetWidget = dijit.byId(this.target);
		if(!targetWidget){ return;}
		if(targetWidget.getParent) {
			var parent = targetWidget.getParent();
			if(parent && parent.layout) {
				dojo.connect(parent, "layout", this, "_placeClip");
			}
		}
	},
	
	_placeClip: function() {
		var targetWidget = dijit.byId(this.target);
		if(!targetWidget){ return; }
		var targetNode = targetWidget.domNode;
		
		var dim = btp.getDimensions(targetNode, this.domNode.offsetParent);
		var left =  dim.left + 2;
		var bottom = 18;
		this.domNode.style.left = left + "px";
		this.domNode.style.top = "auto";
		this.domNode.style.bottom = bottom + "px";
	},
	
	_onHandleImgClick: function(e) {
		var animation = dojox.fx.toggleClass(this.domNode, "btpStickyToasterMinimize");
		dojo.addClass(this.domNode, "btpStickyToasterResizing");
		dojo.connect(animation, "onEnd", dojo.hitch(this, function() {
			dojo.removeClass(this.domNode, "btpStickyToasterResizing");
		}));
		animation.play();
	}
});

}

if(!dojo._hasResource["btp.widget.tool.AbstractDragTool"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.tool.AbstractDragTool"] = true;
dojo.provide("btp.widget.tool.AbstractDragTool");



dojo.declare(
	"btp.widget.tool.AbstractDragTool",
	btp.widget.tool.AbstractTool,
{
	_dropTargetWidget: null,
	_nodeUnderMouse: null,
	_conn: null,
	_type: "component",

	handleMouseDown: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
		this.storeMouseDownInfo(e);
		this.findDropTarget(e);

		// In order to catch all the move events of the entire page, we do not use handleMouseMove.
		this._conn = dojo.connect(dojo.doc, "onmousemove", this, "onMouseMove");
	},
	
	onMouseMove: function(e){
		e.preventDefault(); // this disables selection
	},

	handleMouseUp: function(e, editPartViewer){
		// disconnect the mouseMove handler
		dojo.disconnect(this._conn);

		// deselect the palette
		if(editPartViewer.selectedPaletteItem){
			if(editPartViewer.selectedPaletteItem.deselect){
				editPartViewer.selectedPaletteItem.deselect();
			}
		}

		// reset the tool
		this._editDomain.loadDefaultTool();
	},

	storeMouseDownInfo: function(e){
		this.layerX = e.layerX;
		this.layerY = e.layerY;
		this.pageX = e.pageX;
		this.pageY = e.pageY;
		this.coords = btp.getEventCoords(e);
		var pos = dojo._abs(this._editPartViewer.domNode, false);
		this.offsetX = pos.x;
		this.offsetY = pos.y;
		this.startX = this.pageX - pos.x;
		this.startY = this.pageY - pos.y;
	},

	findDropTarget: function(e){
		var dt = btp.dnd.dragManager.findDropTarget(e.target, {type:this._type});
		if(dt){
			this._nodeUnderMouse = dt.nodeUnderMouse;
			this._dropTargetWidget = btp.getWidgetByElement(dt.domNode);
		}else{
			this._nodeUnderMouse = null;
			this._dropTargetWidget = null;
		}
		return dt;
	},

	// override (AbstractTool.js)
	findRefWidget: function(e){
		return this._nodeUnderMouse;
	},

	// override (AbstractTool.js)
	getDragSourceDomNode: function(e){
		return this._editPartViewer.selectedPaletteItem.domNode;
	},

	// override (AbstractTool.js)
	getDropTargetWidget: function(e){
		return this._dropTargetWidget;
	},

	// override (AbstractTool.js)
	getNodeUnderMouse: function(e){
		return this._dropTargetWidget.domNode;
	},

	// override (AbstractTool.js)
	getOrientation: function(e){
		var node = this._dropTargetWidget.domNode;
		var display = dojo.style(node, "display");
		if(display == "block"){
			return false;
		}else{
			return true;
		}
	}
});

}

if(!dojo._hasResource["btp.widget.tool.DefaultSelectionTool"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.tool.DefaultSelectionTool"] = true;
dojo.provide("btp.widget.tool.DefaultSelectionTool");





dojo.declare(
	"btp.widget.tool.DefaultSelectionTool",
	btp.widget.tool.AbstractTool,
{
	_draggingWidget: null,

	// -------------------------------------------------------------------
	// Event Handlers
	// -------------------------------------------------------------------
	handleMouseDown: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
		var dragTracker;

		// Look for a DragTracker from the clicked DOM node
		if(e.target.getDragTracker){ // probably resize knob
			dragTracker = e.target.getDragTracker();
		}

		// Look for a DragTracker from the clicked widget
		if(!dragTracker){
			var elem = e.target;
			while(elem){
				dragTracker = elem.getDragTracker && elem.getDragTracker();
				if(dragTracker){ break; }
				var widget = elem.widget || dijit.byNode(elem); // gfx raw node may have the widget property
				if(widget){
					editPartViewer.setFocus((editPartViewer != widget) ? widget : null);
					dragTracker = widget.getDragTracker && widget.getDragTracker();
					break;
				}
				elem = elem.parentNode;
				if(!elem || elem.tagName == "BODY"){ break; }
			}
		}

		// Look for a DragTracker from the EditArea
		if(!dragTracker){
			dragTracker = editPartViewer.getDragTracker && editPartViewer.getDragTracker();
		}

		// Use the default DragTracker
		if(!dragTracker){
			dragTracker = "btp.widget.tool.DragGhostMoveTracker";
		}

		this.setDragTracker(dragTracker);
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleMouseDown(e, editPartViewer);
		}
	},

	handleMouseUp: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleMouseUp(e, editPartViewer);
		}
	},
	
	handleMouseMove: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleMouseMove(e, editPartViewer);
		}
	},
	
	handleMouseOver: function(e, editPartViewer){
		editPartViewer.domNode.style.cursor = "auto";
	},
	
	handleClick: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleKeyDown(e, editPartViewer);
		}
		var widget = btp.getWidgetByElement(e.target);
		if(!widget){
			return;
		}else if(editPartViewer != widget){
			editPartViewer.setFocus(widget); // Google Maps needs this
		}
	},

	handleKeyDown: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			if(this.getDragTracker().handleKeyDown(e, editPartViewer) == true) return true; 
		}
		if(e.keyCode == 46){ // Del key
			this.del(editPartViewer);
		}
		else if(e.ctrlKey && e.keyCode == 123){ // Ctrl + F12
//			console.open();
		}
		else if(e.ctrlKey && e.keyCode == 90){ // Ctrl + Z
			this.undo();
		}
		else if(e.ctrlKey && e.keyCode == 89){ // Ctrl + Y
			this.redo();
		}
		else if(e.keyCode == 27){ // ESC
			this.setDragTracker(null);
		}
		if(window["btpConfig"] && btpConfig.keyboardNav){
			var diff = btpConfig.moveAmount ? btpConfig.moveAmount : 5;
			if(e.keyCode == dojo.keys.LEFT_ARROW) {
				var command = this.createMoveComponentCommand(e, editPartViewer, {x: -diff, y:0});
				if(command) {
					this.getEditDomain().executeCommand(command);
					dojo.stopEvent(e);
				}
			}
			else if(e.keyCode == dojo.keys.RIGHT_ARROW) {
				var command = this.createMoveComponentCommand(e, editPartViewer, {x: diff, y:0});
				if(command){
					this.getEditDomain().executeCommand(command);
					dojo.stopEvent(e);
				}			
			}
			else if(e.keyCode == dojo.keys.UP_ARROW) {
				var command = this.createMoveComponentCommand(e, editPartViewer, {x: 0, y:-diff});
				if(command){
					this.getEditDomain().executeCommand(command);
					dojo.stopEvent(e);
				}			
			}
			else if(e.keyCode == dojo.keys.DOWN_ARROW) {
				var command = this.createMoveComponentCommand(e, editPartViewer, {x: 0, y:diff});
				if(command){
					this.getEditDomain().executeCommand(command);
					dojo.stopEvent(e);
				}			
			}
			else if(e.keyCode == dojo.keys.TAB){
				if(!e.shiftKey){
					var ret = this._selectNextWidget(e, editPartViewer); 
					if(ret){
						dojo.stopEvent(e);
					}
				}else{
					var ret = this._selectPreviousWidget(e, editPartViewer);
					if(ret){
						dojo.stopEvent(e);
					}
				}
			}
			else if(e.keyCode == dojo.keys.ENTER){
				if(editPartViewer.focusWidgets && editPartViewer.focusWidgets[0]){
					var focused = editPartViewer.focusWidgets[0];
					if(focused && focused.openContextMenu){
						focused.openContextMenu();
					}
				}
			}	
		}
	},
	
	// utility function
	_nextNonLineWidget: function(widgets, index){
		var i = index + 1;
		for(; i<widgets.length; i++){
			if(widgets[i] instanceof gdc.widget.AbstractNode){ break;	};
		}
		return i;
	},
	
	_previousNonLineWidget: function(widgets, index){
		var i = index - 1;
		for(; i>= 0; i--){
			if(widgets[i] instanceof gdc.widget.AbstractNode){ break;	};
		}
		return i;
	},
	
	_selectNextWidget: function(e, editPartViewer){
		var widgets = editPartViewer.getChildren();
		var efs = editPartViewer.focusWidgets;
		var cur = -1;
		if(efs && efs[0]){
			for(; cur<widgets.length; cur++){
				if(widgets[cur] == efs[0]){ break; }
			}
		}
		var nextIndex = this._nextNonLineWidget(widgets, cur);
		if(nextIndex >= widgets.length){
			editPartViewer.setFocus(null);			
			return false;
		}
		else {
			editPartViewer.setFocus(widgets[nextIndex]);		
			return true;
		}				
	},
	
	_selectPreviousWidget: function(e, editPartViewer){
		var widgets = editPartViewer.getChildren();
		var efs = editPartViewer.focusWidgets;
		var cur = widgets.length;
		if(efs && efs[0]){
			for(; cur>= 0; cur--){
				if(widgets[cur] == efs[0]){ break; }
			}
		}
		var prevIndex = this._previousNonLineWidget(widgets, cur);
		if(prevIndex < 0 ){
			editPartViewer.setFocus(null, true);
			return false;
		} else {
			editPartViewer.setFocus(widgets[prevIndex]);		
			return true;
		}
	},
	
	// -------------------------------------------------------------------
	// Drop Target Handlers
	// -------------------------------------------------------------------
	handleDragStart: function(e, editPartViewer){
		if(e.dropTarget.data != editPartViewer){ return; } // dicard events for components on the editor
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleDragStart(e, editPartViewer);
		}
	},

	handleDropStart: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleDropStart(e, editPartViewer);
		}
	},

	handleDrop: function(e, editPartViewer){
		if (this.getDragTracker() != null){
			return this.getDragTracker().handleDrop(e, editPartViewer);
		}
		this.setEditPartViewer(editPartViewer);
		var targetWidget = this.getDropTargetWidget(e);
		var type = e.dragSource.type;
		var ret;
		if(targetWidget[type+"DropHandler"]){
			ret = targetWidget[type+"DropHandler"](e, editPartViewer);
		}else{
			ret = this[type+"DropHandler"](e, editPartViewer);
		}
		this.getEditDomain().loadDefaultTool();
		e.returnValue = ret; // [custom property]
	},

	handleDropEnd: function(e, editPartViewer){
		btp.dnd.dragManager.currentDropTarget = null;
		this.setDragTracker(null);
	},


	// -------------------------------------------------------------------
	// Internal methods
	// -------------------------------------------------------------------
	del: function(editPartViewer){
		// TODO: Can/Should we call an action here? We need to discuss it after the summer vacation...
		if(editPartViewer.del){
			editPartViewer.del();  // action
		}
/*
		if(editPartViewer.focusWidgets && editPartViewer.focusWidgets[0]){
			var command = new btp.widget.command.EditCommand.RemoveChildCommand(editPartViewer.focusWidgets[0]);
			this.getEditDomain().executeCommand(command);
		}
*/
	},
	
	undo: function(){
		this.getEditDomain().getCommandStack().undo_it();
	},
	
	redo: function(){
		this.getEditDomain().getCommandStack().execute_it();
	},


	createBindingDropCommand: function(e, editPartViewer){
		var ds = e.dragSource;
		var treeNodeWidget = ds.treeNode; // TreeNode widget
		var targetWidget = this.getDropTargetWidget(e);

		var model = null;
		var models = dojo.widget.manager.getWidgetsByType("Model"); // Hmm.. prefix doesn't seem to be allowed
		var ids = {};
		for(var i = 0, len = models.length; i < len; i++){
			var m = models[i];
			if(m.src == treeNodeWidget.data){
				model = m;
				break;
			}
			ids[m.id] = true;
		}

		var ref = treeNodeWidget.ref ? treeNodeWidget.ref : ".";
		if(model && targetWidget.modelId == model.id && targetWidget.ref == ref){
			return null; // nothing has been changed
		}

		var compoundCommand = new btp.widget.command.CompoundCommand();

		compoundCommand.append(new btp.widget.command.EditCommand.UpdateCommand(targetWidget));
		if(targetWidget.setTitle){
			var titleInfo = editPartViewer.getAttributeInfo(targetWidget, "title");
			compoundCommand.append(new btp.widget.command.EditCommand.SetPropertyValueCommand(targetWidget, titleInfo, treeNodeWidget.title));
		}

		var modelIdInfo = editPartViewer.getAttributeInfo(targetWidget, "modelId");
		var refInfo = editPartViewer.getAttributeInfo(targetWidget, "ref");
		if(model){
			compoundCommand.append(new btp.widget.command.EditCommand.SetPropertyValueCommand(targetWidget, modelIdInfo, model.id));
			compoundCommand.append(new btp.widget.command.EditCommand.SetPropertyValueCommand(targetWidget, refInfo, ref));
		}else{
			// <btp:model id="model1" src="some.xml"></btp:model>
			var modelId;
			for(var i = 1; ; i++){
				if(!ids["model"+i]){
					modelId = "model"+i;
					break;
				}
			}
			var opt = {
				id: modelId,
				src: treeNodeWidget.data,
				format: treeNodeWidget.format
			};
			var widget = editPartViewer.createLiveWidget("btp:model", opt);

			compoundCommand.append(new btp.widget.command.EditCommand.SetPropertyValueCommand(targetWidget, modelIdInfo, modelId));
			compoundCommand.append(new btp.widget.command.EditCommand.SetPropertyValueCommand(targetWidget, refInfo, ref));
			compoundCommand.append(new btp.widget.command.EditCommand.InsertBeforeCommand(editPartViewer, widget, editPartViewer.getChildren()[0]));

		}
		compoundCommand.append(new btp.widget.command.EditCommand.UpdateCommand(targetWidget));
		return compoundCommand;
	},

	bindingDropHandler: function(e, editPartViewer){
		var command = this.createBindingDropCommand(e, editPartViewer);
		if(command){
			this.getEditDomain().executeCommand(command);
		}
		return true;
	},

	getDragTracker: function(){
		return this.dragTracker;
	},

	setDragTracker: function(/*String|DragTracker*/newDragTracker){
		if(typeof(newDragTracker) == "string"){
			if(!this._dragTrackerCache){ this._dragTrackerCache = []; }
			if(this._dragTrackerCache[newDragTracker]){
				newDragTracker = this._dragTrackerCache[newDragTracker];
			}else{
				dojo["require"](newDragTracker);
				var impl = dojo.getObject(newDragTracker);
				newDragTracker = this._dragTrackerCache[newDragTracker] = new impl();
				newDragTracker.setEditDomain(this.getEditDomain());
			}
		}
		this.dragTracker = newDragTracker;
	},
	
	createMoveComponentCommand: function(e, editPartViewer, diff) {
		if(editPartViewer.focusWidgets && editPartViewer.focusWidgets.length > 0) {
			var widget = editPartViewer.focusWidgets[0];
			var x = widget.domNode.offsetLeft + diff.x;
			var y = widget.domNode.offsetTop + diff.y;
			if(this.snapObjectsToGridFlag){
				x = this.snap(x);
				y = this.snap(y);
			}
			if(x < 0){ x=0; }
			if(y < 0){ y=0; }
			var toDim = {
				top: y + "px",
				left: x + "px"
			};
			var fromDim = {
				top: widget.domNode.offsetTop + "px",
				left: widget.domNode.offsetLeft + "px"
			};
			var command = new btp.widget.command.EditCommand.MoveCommand(widget, toDim, fromDim);
			return command;			
		} else {
			return null;
		}
	}	
});

}

if(!dojo._hasResource["btp.widget.tool.DragMoveTracker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.tool.DragMoveTracker"] = true;
dojo.provide("btp.widget.tool.DragMoveTracker");




// summary:
//		A DragTracker that moves components on the editor with drag and drop operation.

dojo.declare(
	"btp.widget.tool.DragMoveTracker",
	btp.widget.tool.AbstractTool,
{
	_dragGhost: false, // true: drag a dashed frame, false: drag an actual component
	_moveConn: null,

	handleMouseDown: function(e, editPartViewer){
		// handle left buttun down only
		var leftButton = dojo.isIE ? 1 : 0;
		if(e.button != leftButton) return;
		
		this.setEditPartViewer(editPartViewer);
		var widget = btp.getWidgetByElement(e.target);
		if(!widget || editPartViewer == widget){ return; }
		this._draggingWidget = widget;
		this.enableDragMove(widget);
	},

	enableDragMove: function(widget){
		var ds = widget.__ds;
		if(!ds){
			var node = btp.isGfxWidget(widget) ? widget.getShape().rawNode : widget.domNode;
			if(this._dragGhost){
				ds = widget.__ds = new btp.dnd.DragGhostMoveSource(node, "component", widget);
			}else{
				ds = widget.__ds = new btp.dnd.DragMoveSource(node, "component", widget);
			}
			ds.setDragTarget(widget.domNode);
			ds.offsetParent = this._editPartViewer.domNode;
			if(btp.isGfxWidget(widget)){
				ds.positioning = "absolute";
				ds.useCurrentPositioning = false; // always absolute
				ds.refNode = widget.domNode; // FF cannot get dimension info from an SVG node
			}else{
				ds.targetShouldShowCaret = true;
			}
			dojo.connect(ds, "onDragStart", this, "onDragStart");
			dojo.connect(ds, "onDragEnd", this, "onDragEnd");
			ds.registered = true; // [custom property]
			return;
		}
		if(ds.registered){ return; }
		ds.reregister();
		ds.registered = true; // [custom property]
	},

	disableDragMove: function(widget){
		var ds = widget.__ds;
		if(!ds || !ds.registered){ return; }
		ds.unregister();
		ds.registered = false; // [custom property]
	},

	onDragStart: function(e){
		if(!this._dragGhost){
			this._editPartViewer.removeFocus();
		}
		this._moveConn = dojo.connect(btp.dnd.dragManager, "onMouseMove", this, "onDragMove");
		var widget = this._draggingWidget;
		if(widget && widget.hlp_onWidgetDragStart){
			widget.hlp_onWidgetDragStart(e);
		}
	},

	onDragMove: function(e){
		var ds = btp.dnd.dragManager.currentDragSource;
		var y = ds.dragClone.offsetTop;
		var x = ds.dragClone.offsetLeft;
		if(this.snapObjectsToGridFlag){
			x = this.snap(x);
			y = this.snap(y);
			ds.dragClone.style.top = y + "px";
			ds.dragClone.style.left = x + "px";
		}
		
		// EXPERIMENTAL CODE START
		// (must improve later)
		if(btpConfig && btpConfig.autoScroll){
			
			var threshold = 10;
			var scrollAmount = 5;
			
			var editArea = this._editPartViewer.domNode;
			
			var editorWidth = editArea.clientWidth;
			var editorHeight = editArea. clientHeight;
			var editorOffset = dojo.coords(editArea);
			
			var ds = btp.dnd.dragManager.currentDragSource;
			var dsLeft = ds.dragClone.offsetLeft; 
			var dsTop = ds.dragClone.offsetTop; 
			
			var pos = {x: e.clientX - editorOffset.x, y: e.clientY - editorOffset.y};
			var currentLeft = editArea.scrollLeft;
			var currentTop = editArea.scrollTop;
			if(pos.x < threshold){
				editArea.scrollLeft -= scrollAmount;
				x -= scrollAmount;
				ds.dragClone.style.left = x + "px";
				ds.dragStartCssPosition.x += editArea.scrollLeft - currentLeft;
			} else if ( pos.x > editorWidth - threshold){
				editArea.scrollLeft += scrollAmount;
				x += scrollAmount;
				ds.dragClone.style.left = x + "px";
				ds.dragStartCssPosition.x += editArea.scrollLeft - currentLeft;
			}
			if(pos.y < threshold){
				editArea.scrollTop -= scrollAmount;
				y -= scrollAmount;
				ds.dragClone.style.top = y + "px";
				ds.dragStartCssPosition.y += editArea.scrollTop - currentTop;
			} else if( pos.y > editorHeight - threshold){
				editArea.scrollTop += scrollAmount;
				y += scrollAmount;
				ds.dragClone.style.top = y + "px";
				ds.dragStartCssPosition.y += editArea.scrollTop - currentTop;
			}
		}
		// EXPERIMENTAL CODE END
				
		if(this._editPartViewer.fireMessagePosted){
			this._editPartViewer.fireMessagePosted("("+x+","+y+")");
		}
		var widget = this._draggingWidget;
		if(!this._dragGhost && btp.isGfxWidget(widget)){
			widget.update();
		}
		if(widget && widget.hlp_onWidgetDragMove){
			widget.hlp_onWidgetDragMove(e);
		}
	},

	onDragEnd: function(e){
		dojo.disconnect(this._moveConn);
		var widget = this._draggingWidget;
		if(!widget){ return; }
		var targetWidget = this.getDropTargetWidget(e);
		if(targetWidget && targetWidget.componentMoveHandler){
			targetWidget.componentMoveHandler(e, this._editPartViewer);
		}else{
			this.componentMoveHandler(e, this._editPartViewer);
		}
		this.disableDragMove(this._draggingWidget);
		this._draggingWidget = null;

		if(widget.hlp_onWidgetDragEnd){
			widget.hlp_onWidgetDragEnd(e);
		}
	},

	componentMoveHandler: function(e, editPartViewer){
		var sourceWidget = this._draggingWidget;
		var targetWidget = this.getDropTargetWidget(e) || this._editPartViewer;
		if(sourceWidget.getParent() != targetWidget // moved to the different level
			&& sourceWidget != targetWidget){ // sourceWidget was dropped on other than itself
			var refWidget = this.findRefWidget(e);
			return this.reparentComponent(e, refWidget);
		}else{
			return this.moveComponent(e);
		}
	},

	moveComponent: function(e){
		var widget = this._draggingWidget;
		var command = this.createMoveComponentCommand(e, widget);
		this._editDomain.executeCommand(command);
		this._editPartViewer.setFocusFrame(widget);
		return true;
	},

	createMoveComponentCommand: function(e, widget){
		var ds = e.dragSource;
		var dx = e.pageX - ds.pageX;
		var dy = e.pageY - ds.pageY;
		var x = ds.dragStartCssPosition.x + dx;
		var y = ds.dragStartCssPosition.y + dy;
		if(this._editPartViewer.snapObjectsToGridFlag){
			x = this._editPartViewer.snap(x);
			y = this._editPartViewer.snap(y);
		}
		if(x < 0) x = 0;
		if(y < 0) y = 0;
		var toDim = {
			top: y + "px",
			left: x + "px"
		};
		var fromDim;
		if(!this._dragGhost){
			fromDim = {
				top: ds.dragStartCssPosition.y + "px",
				left: ds.dragStartCssPosition.x + "px"
			};
		}
		var command = new btp.widget.command.CompoundCommand();
		if(ds.positioning && dojo.style(widget.domNode, "position") != ds.positioning){
			command.append(new btp.widget.command.EditCommand.SetStyleCommand(widget, "position", ds.positioning));
		}
		command.append(new btp.widget.command.EditCommand.MoveCommand(widget, toDim, fromDim));
		return command;
	}
});

}

if(!dojo._hasResource["btp.widget.tool.DragGhostMoveTracker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.tool.DragGhostMoveTracker"] = true;
dojo.provide("btp.widget.tool.DragGhostMoveTracker");



// summary:
//		A DragTracker that moves components on the editor with drag and drop operation.

dojo.declare(
	"btp.widget.tool.DragGhostMoveTracker",
	btp.widget.tool.DragMoveTracker,
{
	_dragGhost: true // true: drag a dashed frame, false: drag an actual component
});

}

if(!dojo._hasResource["btp.widget.tool.ResizeCreationTool"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.tool.ResizeCreationTool"] = true;
dojo.provide("btp.widget.tool.ResizeCreationTool");



dojo.declare(
	"btp.widget.tool.ResizeCreationTool",
	btp.widget.tool.AbstractDragTool,
{
	_rubberBand: null,

	handleMouseDown: function(e, editPartViewer){
		this.inherited("handleMouseDown", arguments);
		var rubberBand = this._rubberBand;
		if(!rubberBand){
			rubberBand = this._rubberBand = dojo.doc.createElement("DIV");
			rubberBand.style.fontSize = "1px";
			rubberBand.style.position = "absolute";
			rubberBand.style.border = "1px dashed black";
		}
		editPartViewer.domNode.appendChild(rubberBand);
		rubberBand.style.top = e.layerY + "px";
		rubberBand.style.left = e.layerX + "px";
		rubberBand.style.width = "0px";
		rubberBand.style.height = "0px";
	},
	
	onMouseMove: function(e){
		this.inherited("onMouseMove", arguments);
		var dy = e.pageY - this.pageY;
		var dx = e.pageX - this.pageX;
		var rubberBand = this._rubberBand;
		var top, left, width, height; // string
		if(dx > 0 && dy > 0){
			width = e.pageX - this.pageX + "px";
			height = e.pageY - this.pageY + "px";
		}

		if(top){ rubberBand.style.top = top; }
		if(left){ rubberBand.style.left = left; }
		if(width){ rubberBand.style.width = width; }
		if(height){ rubberBand.style.height = height; }
	},

	handleMouseUp: function(e, editPartViewer){
		this.inherited("handleMouseUp", arguments);
		if(dojo.isMozilla){
			// For some reason, FF starts auto scrolling after the resize creation.
			// I don't know why, but anyway this solves the problem. Very weird.
			var _this = this;
			setTimeout(function(){
				editPartViewer.domNode.removeChild(_this._rubberBand);
			}, 0);
		}else{
			editPartViewer.domNode.removeChild(this._rubberBand);
		}
		var srcElem = this.getDragSourceDomNode(e);
		var width = Math.abs(e.pageX - this.pageX);
		var height = Math.abs(e.pageY - this.pageY);
		srcElem.props = {};
		srcElem.props.coords = this.coords;
		srcElem.props.__style = {};
		if(width > 0){ srcElem.props.__style["width"] = width + "px"; }
		if(height > 0){ srcElem.props.__style["height"] = height + "px"; }
		this.insertComponent(e, editPartViewer);
	}
});

}

if(!dojo._hasResource["btp.widget.tool.ResizeTracker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.tool.ResizeTracker"] = true;
dojo.provide("btp.widget.tool.ResizeTracker");





// summary:
//		A DragTracker for dragging a resize handle.

dojo.declare(
	"btp.widget.tool.ResizeTracker",
	btp.widget.tool.AbstractTool,
{
	_rubberBand: null,

	// constants
	_NORTH: 1,
	_SOUTH: 2,
	_WEST: 4,
	_EAST: 8,

	stopResizing: function(){
		var rubberBand = this._editPartViewer._rubberBand;
		if(rubberBand.parentNode != this._editPartViewer.domNode){ return; } // already canceled

		this._editPartViewer.domNode.removeChild(rubberBand);
		this._editPartViewer.domNode.removeChild(this._editPartViewer.getMoveTarget());
		this._editPartViewer.domNode.style.cursor = "";
	},

	// ======== Event Handlers ====================================
	handleMouseDown: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
	},

	handleDragStart: function(e, editPartViewer){
		var movingKnob = btp.dnd.dragManager.currentDragSource.domNode;
		var widget = this._editPartViewer.focusWidgets[0];

		var rubberBand = this._editPartViewer._rubberBand;
		if(!rubberBand){
			rubberBand = this._editPartViewer._rubberBand = dojo.doc.createElement("DIV");
			rubberBand.style.fontSize = "1px";
			rubberBand.style.position = "absolute";
			rubberBand.style.border = "1px dashed black";
		}
		this._editPartViewer.domNode.appendChild(rubberBand);

		var dim = widget.__dim = btp.getStyleDimensions(widget, this._editPartViewer);
		rubberBand.style.top = dim.top + "px";
		rubberBand.style.left = dim.left + "px";
		rubberBand.style.width = dim.width + "px";
		rubberBand.style.height = dim.height + "px";

		var moveTarget = this._editPartViewer.getMoveTarget();
		moveTarget.style.top = movingKnob.offsetTop + "px";
		moveTarget.style.left = movingKnob.offsetLeft + "px";
		this._editPartViewer.domNode.appendChild(moveTarget);
		this._editPartViewer.domNode.style.cursor = "crosshair";
	},

	// Cannot use handleDragMove because it is called only when the drag target accepts the drag.
	handleMouseMove: function(e, editPartViewer){
		var rubberBand = this._editPartViewer._rubberBand;
		if(!rubberBand || rubberBand.parentNode != this._editPartViewer.domNode){
			return; // drag is not started yet or perhaps resizing was canceled with ESC
		}
		var movingKnob = btp.dnd.dragManager.currentDragSource.domNode;
		var widget = this._editPartViewer.focusWidgets[0];
		var dim = widget.__dim;
		var rubberBand = this._editPartViewer._rubberBand;
		var knobPosition = movingKnob.knobPosition; // [custom property]
		var info = widget.hlp_getComponentInfo();

		var moveTarget = this._editPartViewer.getMoveTarget();
		var mouseX = moveTarget.offsetLeft;
		var mouseY = moveTarget.offsetTop;
		if(dojo.isMozilla){
			mouseX--;
			mouseY--;
		}

		var dirs = widget.__resizeDirections;
		if(!e.shiftKey && !e.ctrlKey){ // resize freely
			if(dirs & this._NORTH
				&& (knobPosition == 0 || knobPosition == 1 || knobPosition == 2)){ // NORTH
				var h = dim.top - mouseY + dim.height;
				if(h > 0){
					rubberBand.style.top = this._editPartViewer.snap(mouseY) + "px";
					rubberBand.style.height = this._editPartViewer.snap(h) + "px";
				}
			}else if(dirs & this._SOUTH
				&& (knobPosition == 4 || knobPosition == 5 || knobPosition == 6)){ // SOUTH
				var h = mouseY - dim.top;
				if(h > 0){
					rubberBand.style.height = this._editPartViewer.snap(h) + "px";
				}
			}
	
			if(dirs & this._EAST
				&& (knobPosition == 2 || knobPosition == 3 || knobPosition == 4)){ // EAST
				var w = mouseX - dim.left;
				if(w > 0){
					rubberBand.style.width = this._editPartViewer.snap(w) + "px";
				}
			}else if(dirs & this._WEST
				&& (knobPosition == 0 || knobPosition == 6 || knobPosition == 7)){ // WEST
				var w = dim.left - mouseX + dim.width;
				if(w > 0){
					rubberBand.style.left = this._editPartViewer.snap(mouseX) + "px";
					rubberBand.style.width = this._editPartViewer.snap(w) + "px";
				}
			}
		}else if(e.shiftKey && !e.ctrlKey){ // resize keeping the aspect ratio
			if((dirs & this._NORTH) && (dirs & this._EAST) && knobPosition == 2){ // top-right
				var w = mouseX - dim.left;
				var h = dim.top + dim.height - mouseY;
				if(w > 0 && h > 0){
					var ratioX = w / dim.width;
					var ratioY = h / dim.height;
				}
			}
		}else if(!e.shiftKey && e.ctrlKey){ // resize keeping the center
		}else if(e.shiftKey && e.ctrlKey){ // resize keeping the aspect ratio and the center
		}
	},

	handleMouseUp: function(e, editPartViewer){
		var rubberBand = this._editPartViewer._rubberBand;
		if(!rubberBand || rubberBand.parentNode != this._editPartViewer.domNode){
			return; // drag is not started yet or perhaps resizing was canceled with ESC
		}
		var widget = this._editPartViewer.focusWidgets[0];
		var parentDim = btp.getDimensions(widget.getParent().domNode);
		var editAreaDim = btp.getDimensions(this._editPartViewer.domNode);
		var toBox = dojo.contentBox(rubberBand);
		var toDim = {
			top: (rubberBand.offsetTop - parentDim.top + editAreaDim.top) + "px",
			left: (rubberBand.offsetLeft - parentDim.left + editAreaDim.left) + "px",
			width: toBox.w + "px",
			height: toBox.h + "px"
		};

		var command = new btp.widget.command.EditCommand.ResizeCommand(widget, toDim);
		this._editPartViewer.getEditDomain().executeCommand(command);

		this.stopResizing();
		this._editPartViewer.fireSelectionChanged(widget);
	},

	handleKeyDown: function(e, editPartViewer){
		if(e.keyCode == 27){ // ESC
			this.stopResizing();
		}
	}
});

}

if(!dojo._hasResource["btp.widget.Toolbar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.Toolbar"] = true;
dojo.provide("btp.widget.Toolbar");






dojo.declare(
	"btp.widget.Toolbar",
	[dijit._Widget, dijit._Templated],
{
	modelId: '',
	ref: '.',
	target: '',

	conf: null,
	items: null,
	iconPalette: null,
	iconBase: null,

	templateString: '<div class="btpToolbar" dojoAttachPoint="toolbarNode"><table cellspacing="0" cellpadding="0"><tbody><tr></tr></tbody></table></div>',
//	templateCssString:"/*\r\n * ============================================================================\r\n * Licensed Materials - Property of IBM\r\n * \r\n * (C) Copyright IBM Corp. 2007  All Rights Reserved.\r\n * US Government Users Restricted Rights - Use, duplication or disclosure \r\n * restricted by GSA ADP Schedule Contract with IBM Corp.\r\n * ============================================================================\r\n */\r\n\r\n/* Toolbar */\r\nDIV.btpToolbar {\r\n    background-color: #ECE9D8;\r\n    padding: 2px 0px 2px 8px;\r\n}\r\n\r\nDIV.btpToolbarIcon {\r\n    border-style: solid;\r\n    border-width: 1px;\r\n    border-color: #ECE9D8;\r\n    white-space: nowrap;\r\n    margin-right: 1px;\r\n}\r\n\r\nDIV.btpToolbarIcon A {\r\n    text-decoration: none;\r\n    color: black;\r\n    font-family: tahoma, helvetica, arial, sans-serif;\r\n    font-size: 9pt;\r\n}\r\n\r\nDIV.btpToolbarIcon IMG {\r\n    vertical-align: middle;\r\n    padding: 1px 2px 1px 1px;\r\n}\r\n\r\nSELECT.btpToolbarDropdown {\r\n    margin: 0px 2px 0px 1px;\r\n}\r\n\r\nIMG.btpToolbarSeparator {\r\n    margin: 0px 2px 0px 2px;\r\n}\r\n\r\n/* for accessibility */\r\nDIV.btpToolbar label.hidden-label\r\n{\r\n    position:absolute;\r\n    left:0px\r\n    top:-100px;\r\n    width:0px; \r\n    height:0px; \r\n    overflow:hidden;\r\n}\r\n",templateCssPath: dojo.moduleUrl("btp", "widget/templates/Toolbar/Toolbar.css"),
	i18nPrefix: "Toolbar_",

	postCreate: function(){
		if(this.target){
			this.editorActivatedFilter = this.target;
			this._topics = [
				dojo.subscribe("btp:ActiveEditorChanged", this, "editorActivated")
			];
		}
		this.updateToolbar();
	},

	destroy: function(){
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	updateToolbar: function(json){
		var selectedNode;
		if(json){
			selectedNode = json;
		}else if(this.modelId){
			var model = dijit.byId(this.modelId);
			selectedNode = model.selectSingleNodeFromJson(this.ref); // <application>
		}else{
			return;
		}
		this.conf = selectedNode;
		this.iconBase = selectedNode.toolBar["@iconBase"];
		var items = btp.getChildNodes(selectedNode.toolBar, "item");
		this.items = [];
		for(var i = 0, len = items.length; i < len; i++){
			var item = items[i]; // <item>
			var elem;
			if(item["@separator"] == "true"){
				elem = this.createToolbarSeparator(this, item);
			}else if(item["@type"] == "select"){
				elem = this.createToolbarDropdown(this, item);
			}else{
				elem = this.createToolbarIcon(this, item);
			}
			this.items.push(elem);
			this.items[elem.command] = elem;
		}
	},

	createToolbarSeparator: function(parent, item){
		var img = dojo.doc.createElement("IMG");
		img.className = "btpToolbarSeparator";
		img.src = dojo.moduleUrl("btp", "widget/templates/Toolbar/icon-separator.gif");

		var tbl = this.toolbarNode.firstChild;
		var cell = tbl.rows[0].insertCell(-1);
		cell.appendChild(img);

		return img;
	},

	/*
	    <div class="btpToolbarIcon">
	      <a href="javascript:void(0)">
	        <img src="open.gif">
	        label text
	      </a>
	    </div>
	*/
	createToolbarIcon: function(parent, item){
		var command = item["@command"];
		var tooltip = item["@tooltip"];
		var arg = item["@arg"];
		var iconPath = item["@iconPath"];
		var iconFile = item["@iconFile"];
		var toggle = !!item["@toggle"];
		var radio = !!item["@radio"];
		var name = !!item["@name"];
		var selected = !!item["@selected"];
		var text = item["@text"];
		if(!tooltip){
			tooltip = command;
		}
		if(this.iconBase && !iconPath && iconFile){
			iconPath = this.iconBase + iconFile;
		}

		text = btp.i18n.gets(text, this.i18nPrefix);
		tooltip = btp.i18n.gets(tooltip, this.i18nPrefix);

		var div = dojo.doc.createElement("DIV");
		div.className = "btpToolbarIcon";

		div.title = tooltip;
		div.command = command; // [custom property]
		div.arg = arg; // [custom property]
		div.toggle = toggle; // [custom property]
		div.radio = radio; // [custom property]
		div._name = name; // [custom property]
		div._selected = selected; // [custom property]

		var a = dojo.doc.createElement("A");
		a.href = "javascript:void(0)";
		a.onmouseover = "window.status=''";
		a.onclick = this.nop; // to avoid firing the onbeforeunload event (dojo.connect doesn't work for this purpose)
		div.appendChild(a);

		var img = dojo.doc.createElement("IMG");
		img.alt = tooltip;
		img.src = dojo.moduleUrl("btp", iconPath);
		img.border = "0";
		div.iconImg = img; // [custom property]
		div.enabledSrc = img.src; // [custom property]
		div.disabledSrc = img.src.replace(/\.([^\.]+)$/, "_d.$1"); // [custom property]
		a.appendChild(img);
		
		if(text){
			a.appendChild(dojo.doc.createTextNode(text));
			div.style.paddingRight = "6px";
		}

		if(toggle || radio){
			if(selected == "true"){
				div.toggleOn = true;
				this.sunken(div);
			}
			else
				div.toggleOn = false;
		}

		var tbl = this.toolbarNode.firstChild;
		var cell = tbl.rows[0].insertCell(-1);
		cell.appendChild(div);

		dojo.connect(div, "onclick", this, "commandHandler");
		dojo.connect(div, "onmouseover", this, "onMouseOverHandler");
		dojo.connect(div, "onmouseout", this, "onMouseOutHandler");
		dojo.connect(div, "onmousedown", this, "onMouseDownHandler");
		dojo.connect(div, "onkeydown", this, "onKeyDownHandler");
		
		return div;
	},

	createToolbarDropdown: function(parent, item){
		var command = item["@command"];
		var tooltip = item["@tooltip"];
		var arg = item["@arg"];
		var sel = dojo.doc.createElement("SELECT");
		sel.className = "btpToolbarDropdown";
		sel.command = command; // [custom property]
		sel.arg = arg; // [custom property]
		sel.title = tooltip;

		var tbl = this.toolbarNode.firstChild;
		var cell = tbl.rows[0].insertCell(-1);
		cell.noWrap = true;
		cell.appendChild(sel);

		// for accessibility
		var label = dojo.doc.createElement("LABEL");
		label.innerHTML = tooltip;
		label.className = "hidden-label";
		cell.appendChild(label);

		dojo.connect(sel, "onchange", this, "commandHandler");
		
		return sel;
	},

	getItemElement: function(command){
		return this.items[command];
	},

	changeToggleButtonState: function(item, /*Boolean or undifined*/flag){
		if(!item){ return; }
		var elem;
		if(typeof(item) == "number"){ // index
			elem = this.items[i];
		}else if(typeof(item) == "string"){ // command name
			elem = this.getItemElement(item);
		}else{
			elem = item;
		}
		if(!elem.toggle){ return; }
		if(flag == true){
			if(elem.toggleOn != true){
				elem.toggleOn = true; // [custom property]
				this.sunken(elem);
			}
		}else if(flag == false){
			if(elem.toggleOn != false){
				elem.toggleOn = false; // [custom property]
				this.flat(elem);
			}
		}else{
			elem.toggleOn = !elem.toggleOn; // [custom property]
			if(elem.toggleOn){
				this.sunken(elem);
			}
			else{
				this.flat(elem);
			}
		}
	},

	selectRadioButton: function(item){
		if(!item){ return; }
		var elem;
		if(typeof(item) == "number"){ // index
			elem = this.items[i];
		}else if(typeof(item) == "string"){ // command name
			elem = this.getItemElement(item);
		}else{
			elem = item;
		}
		if(!elem.radio || elem.toggleOn){ return; }
		for(var i = 0, len = this.items.length; i < len; i++){
			var item = this.items[i];
			if(item == elem){
				elem.toggleOn = true; // [custom property]
				this.sunken(elem);
			}else if(item.radio && item._name == elem._name){
				item.toggleOn = false;
				this.flat(item);
			}
		}
	},

	refreshState: function(editor){
		var items = btp.getChildNodes(this.conf.toolBar, "item");
		for(var i = 0, len = items.length; i < len; i++){
			var item = items[i]; // <item>
			var prop = item["@prop"];
			if(prop){
				var div = this.items[i];
				if(div.toggle || div.radio){
					var cap = prop.charAt(0).toUpperCase() + prop.substring(1);
					var flag;
					if(editor["get"+cap]){
						flag = editor["get"+cap]();
					}else if (editor[prop]){
						flag = editor[prop];
					}else{
						break;
					}
					div.toggleOn = flag; // [custom property]
					if(flag){
						this.sunken(div);
					}else{
						this.flat(div);
					}
				}
			}
		}
	},

// ======================= Event handlers =================================
	_isMatch: function(editor, filter){ // same logic as btp.matchFilter()
		if(!filter){ return false; }
		if(filter == "*"){ return true; }
		var arr = filter.split(",");
		var id = editor.domNode.getAttribute("widgetId");
		var type = editor.declaredClass;
		for(var i = 0, len = arr.length; i < len; i++){
			if(arr[i] == id || arr[i] == type){
				return true;
			}
		}
		return false;
	},

	editorActivated: function(message){
		if(!this._isMatch(message.editor, this.target)){ return; }
		this.refreshState(message.editor);
	},

	commandHandler: function(e){
		var elem = e.currentTarget; // <div> or <select>
		if(!this.isEnabled(elem)){ return; }
		if(elem.toggle || elem.radio){
			elem.toggleOn = !elem.toggleOn; // [custom property]
			if(elem.toggleOn){
				this.sunken(elem);
			}
			else{
				this.flat(elem);
			}
			if(elem.radio){
				for(var i = 0, len = this.items.length; i < len; i++){
					var item = this.items[i];
					if(item != elem && item.radio && item._name == elem._name){
						item.toggleOn = false;
						this.flat(item);
					}
				}
			}
		}
		else if(elem.tagName == "DIV"){
			var _this = this;
			setTimeout(function(){_this.flat(elem);}, 100);
		}

		var message = {
			type: "ActionInvokedNotification",
			src: this,
			command: e.currentTarget.command,
			arg: e.currentTarget.arg,
			item: e.currentTarget
		};
		dojo.publish("btp:ActionInvoked", [message]);
	},

	onMouseOverHandler: function(e){
		var div = e.currentTarget;
		if(!this.isEnabled(div)){ return; }
		if(div.toggleOn){
			return;
		}else{
			this.raised(div);
		}
	},

	onMouseOutHandler: function(e){
		var div = e.currentTarget;
		if(!this.isEnabled(div)){ return; }
		if(div.toggleOn){
			this.sunken(div);
		}else{
			this.flat(div);
		}
	},

	onMouseDownHandler: function(e){
		var div = e.currentTarget;
		if(!this.isEnabled(div)){ return; }
		this.sunken(div);
	},

	onKeyDownHandler: function(e){
		if(e.keyCode == 13){ // Enter
			this.onMouseDownHandler(e);
		}
	},

	nop: function(){
		return false;
	},


	enable: function(div){
		div.iconImg.src = div.enabledSrc;
	},

	disable: function(div){
		div.iconImg.src = div.disabledSrc;
	},

	isEnabled: function(elem){
		if(elem.tagName == "DIV"){
			return !elem.iconImg.src.match(/_d\.([^\.]+)$/);
		}else if(elem.tagName == "SELECT"){
			return true;
		}
	},


	flat: function(div){
		div.style.borderColor = "#ECE9D8";
		div.iconImg.style.padding = "1px 2px 1px 1px";
	},

	raised: function(div){
		div.style.borderColor = "white #ACA899 #ACA899 white";
		div.iconImg.style.padding = "1px 2px 1px 1px";
	},

	sunken: function(div){
		div.style.borderColor = "#ACA899 white white #ACA899";
		div.iconImg.style.padding = "2px 1px 0px 2px";
	}

});

}

if(!dojo._hasResource["btp.widget.ToolTabPanel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.ToolTabPanel"] = true;
dojo.provide("btp.widget.ToolTabPanel");



dojo.declare(
	"btp.widget.ToolTabPanel",
	btp.widget._TabPanelBase,
{
	unhilightBorderColor: "#ECE9D8",

	edgeImages: null,
	upperLineNode: null,
	lowerLineNode: null,
	lowerRightLineNode: null,
	localToolbarArea: null,

	gray: "#ECE9D8",
	darkGray: "#ACA899",
	lightBlue: "#99BAF3",

	topicNamePrefix: "ToolTab",

	postCreate: function(){
		this.domNode.className = "tabPanel toolTabPanel";
		var header = this.tabHeaderNode;
		this.tabs = [];

		this.childWidgets = [
			{domNode: this.tabHeaderNode, layoutAlign: "top"},
			{domNode: this.containerNode, layoutAlign: "client"}
		];

		// Edge
		this.edgeImages = [];
		var img;
		img = this.edgeImages[0] = dojo.doc.createElement("IMG");
		img.alt = "";
		img.src = dojo.moduleUrl("btp", "widget/themes/images/edge-hi.gif");
		img.style.position = "absolute";
		img.style.display = "none";
		header.appendChild(img);

		img = this.edgeImages[1] = dojo.doc.createElement("IMG");
		img.alt = "";
		img.src = dojo.moduleUrl("btp", "widget/themes/images/edge-lo.gif");
		img.style.position = "absolute";
		img.style.display = "none";
		header.appendChild(img);

		// Delete Button
		if(this.closable){
			this.crossImgPath = dojo.moduleUrl("btp", "widget/themes/images/cross.gif");
			this.redCrossImgPath = dojo.moduleUrl("btp", "widget/themes/images/cross-red.gif");
			img = this.deleteButton = dojo.doc.createElement("IMG");
			img.alt = "";
			img.src = this.crossImgPath;
			img.style.position = "absolute";
			img.style.display = "none";
			this._connectors.push(dojo.connect(img, "onmouseover", this, "deleteButtonMouseOverHandler"));
			this._connectors.push(dojo.connect(img, "onmouseout", this, "deleteButtonMouseOutHandler"));
			this._connectors.push(dojo.connect(img, "onclick", this, "deleteButtonClickHandler"));
			header.appendChild(img);
		}

		// Upper Line
		var line = this.upperLineNode = dojo.doc.createElement("DIV");
		line.className = "tabPanelLine tabPanelLineUpper";
		line.style.top = "0px";
		line.style.left = "0px";
		header.appendChild(line);

		// Lower Line
		line = this.lowerLineNode = dojo.doc.createElement("DIV");
		line.className = "tabPanelLine tabPanelLineLower";
		line.style.left = "0px";
		header.appendChild(line);

		// Lower Right Line
		line = this.lowerRightLineNode = dojo.doc.createElement("DIV");
		line.className = "tabPanelLine tabPanelLineLower";
		header.appendChild(line);

		// to keep height even when there are no tabs
		var dummy = dojo.doc.createElement("DIV");
		dummy.className = "tabPanelDummyTab";
		header.appendChild(dummy);

		var area = this.localToolbarArea = dojo.doc.createElement("DIV");
		area.className = "localToolbarArea";
		header.appendChild(area);

		btp.widget.ToolTabPanel.superclass.postCreate.apply(this, arguments);
	},

	setupChild: function(/*ContentPane*/ page){
		var header = this.tabHeaderNode;
		var tab = dojo.doc.createElement("DIV");
		tab.className = "tabPanelTab";
		tab.page = page; // [custom property]
		page.tab = tab; // [custom property]
		this.tabs.push(tab);

		page.domNode.style.position = "relative";
		page.domNode.style.display = "block";

		if(page.icon){
			var icon = dojo.doc.createElement("IMG");
			icon.alt = "";
			icon.width = "16";
			icon.height = "16";
			icon.className = "tabPanelTabIcon";
			icon.src = dojo.moduleUrl("btp", page.icon);
			tab.appendChild(icon);
			tab.iconNode = icon; // [custom property]
			if(!this.showInactiveIcons){
				icon.style.display = "none";
			}
		}

		// Title Node
		var titleNode = dojo.doc.createElement("DIV");
		titleNode.className = "tabPanelTabTitle";
		titleNode.tabIndex = 0;
		var label = page.label; //TODO: hale.i18n.gets(page.label, this.i18nPrefix);
		titleNode.innerHTML = label;
		if(page.tooltip){
			tab.title = page.tooltip;
		}
		tab.appendChild(titleNode);
		tab.titleNode = titleNode; // [custom property]

		// Tail Image (hilight)
		var img = dojo.doc.createElement("IMG");
		img.alt = "";
		img.src = dojo.moduleUrl("btp", "widget/themes/images/tail-hi.gif");
		img.className = "tabPanelTailImg";
		img.style.width = "0px";
		header.appendChild(img);
		tab.appendChild(img);
		tab.hilightNode = img; // [custom property]

		// Tail Image (unhilight)
		img = dojo.doc.createElement("IMG");
		img.alt = "";
		img.src = dojo.moduleUrl("btp", "widget/themes/images/tail-lo.gif");
		img.className = "tabPanelTailImg";
		img.style.width = "0px";
		tab.appendChild(img);
		tab.unhilightNode = img; // [custom property]

		header.insertBefore(tab, this.localToolbarArea);

		// local toolbar
/*
		if(page.localToolbarWidget){
			page.localToolbarWidget.domNode.style.styleFloat = "right"; // IE
			page.localToolbarWidget.domNode.style.cssFloat = "right";   // Mozilla
			this.localToolbarArea.appendChild(page.localToolbarWidget.domNode);
		}
*/

		tab._connectors = [];
		tab._connectors.push(dojo.connect(tab, "onclick", this, "onTabClick"));
		tab._connectors.push(dojo.connect(tab, "ondblclick", this, "onTabDblClick"));
		tab._connectors.push(dojo.connect(tab, "onkeydown", this, "onTabKeyDown"));

		page.domNode.style.display = "none";
	},

	// override dijit._Container.removeChild
	removeChild: function(/*Widget*/widget){
		var page = widget;
		var tab = widget.tab;
		for(var i = 0, len = this.tabs.length; i < len; i++){
			if(this.tabs[i] == tab){
				this.tabs.splice(i, 1); // remove tab from tabs
				break;
			}
		}
		if(this.tabs.length == 0){
			this.unhilightTab(tab);
		}
		this.tabHeaderNode.removeChild(tab);
		dojo.forEach(tab._connectors, dojo.disconnect);
		page.destroy();
		if(this.tabs.length == 0 && this.deleteButton){
			this.deleteButton.style.display = "none";
			this.edgeImages[0].style.display = "none";
			this.edgeImages[1].style.display = "none";
			this.containerNode.style.backgroundColor = "#EFEBDE";
			this.fireTabChanged(null);
		}else{
			this.selectTab(this.tabs[0]);
			this.reposition(this.tabs[0]);
		}
	},

	hilightTab: function(tab, focus){
		tab.style.backgroundImage = "url("+dojo.moduleUrl("btp", "widget/themes/images/hilight-bg.gif")+")";
		tab.style.borderRightStyle = "none";

		this.isActive = true;

		// icon
		if(!this.showInactiveIcons && tab.iconNode){
			tab.iconNode.style.display = "";
		}

		// tail
		tab.hilightNode.style.width = "";
		tab.unhilightNode.style.width = "0px";

		// edge
		this.edgeImages[0].style.display = "";
		this.edgeImages[1].style.display = "none";
		this.reposition(tab);

		// border of the content
		this.containerNode.style.border = "solid 2px "+this.lightBlue;

		// focus (necessary for keyboard navigation)
		if(focus){
			tab.titleNode.focus();
		}

		// set tabIndex only for the selected tab
		for(var i = 0, len = this.tabs.length; i < len; i++){
			this.tabs[i].titleNode.tabIndex = (this.tabs[i] == tab) ? 0 : -1;
		}
	},

	unhilightTab: function(tab){
		tab.style.backgroundImage = "url("+dojo.moduleUrl("btp", "widget/themes/images/unhilight-bg.gif")+")";
		tab.style.borderRightStyle = "none";

		this.isActive = false;

		// tail
		tab.hilightNode.style.width = "0px";
		tab.unhilightNode.style.width = "";

		// edge
		this.edgeImages[0].style.display = "none";
		this.edgeImages[1].style.display = "";
		this.reposition(tab);

		// border of the content
		this.containerNode.style.border = "solid 2px "+this.unhilightBorderColor;
	},

	grayoutTab: function(tab){
		tab.style.backgroundImage = "";
		tab.style.borderRightStyle = "solid";
		tab.style.borderBottom = "solid 1px "+this.darkGray;

		// icon
		if(!this.showInactiveIcons && tab.iconNode){
			tab.iconNode.style.display = "none";
		}

		// tail
		tab.hilightNode.style.width = "0px";
		tab.unhilightNode.style.width = "0px";
	},

	reposition: function(tab){
		// Edge
		var img, idx;
		if(this.isActive){
			img = tab.hilightNode;
			idx = 0;
		}else{
			img = tab.unhilightNode;
			idx = 1;
		}
		var dim = btp.getDimensions(img, this.tabHeaderNode);
		this.edgeImages[idx].style.top = dim.top + "px";
		this.edgeImages[idx].style.left = dim.left + img.offsetWidth + "px";

		// Delete Button
		if(this.closable){
			this.deleteButton.style.top = dim.top + "px";
			this.deleteButton.style.left = dim.left - 5 + "px";
		}

		// Lower Line
		this.lowerLineNode.style.top = this.lowerRightLineNode.style.top = dim.top + dim.height + "px";
		if(this.isActive){
			this.lowerLineNode.style.backgroundColor = this.lightBlue;
		}else{
			this.lowerLineNode.style.backgroundColor = this.darkGray;
		}

		// Lower Right Line
		if(this.selectedPage){
			var lastTabDim = btp.getDimensions(this.selectedPage.tab, this.tabHeaderNode);
			var pos = lastTabDim.left + lastTabDim.width + 15;
			this.lowerRightLineNode.style.left = pos + "px";
		}
	}
});

}

if(!dojo._hasResource["btp.widget.Workbench"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget.Workbench"] = true;
dojo.provide("btp.widget.Workbench");

dojo.declare(
	"btp.widget.Workbench",
	dijit._Widget,
{
	commands: '',
	actionInvokedFilter: '',
	tabChangedFilter: '*',
	tabDoubleClickedFilter: '*',

	activeView: null,
	activeEditor: null,

	postCreate: function(){
		this._topics = [
			dojo.subscribe("btp:ToolTabChanged", this, "onTabChanged"),
			dojo.subscribe("btp:ToolTabDoubleClicked", this, "onTabDblClick"),
			dojo.subscribe("btp:ActionInvoked", this, "dispatchCommand")
		];

		var _this = this;
		dojo.addOnLoad(function(){
			var editors = btp.getEditors();
			var editArea = editors.length > 0 ? editors[0] : null;
			if(editArea){
				_this.fireActiveEditorChanged(editArea);
			}
		});
	},

	destroy: function(){
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	dispatchCommand: function(message){
	},

	onTabChanged: function(message){
		if(!btp.matchFilter(message, this.tabChangedFilter)){
			return; // filter out the message
		}

		this.activeView = message.tabPage;
		var message = {
			type: "ActiveViewChangedNotification",
			src: this,
			view: this.activeView
		};
		dojo.publish("btp:ActiveViewChanged", [message]);

		if(!this.activeView){
			var editors = btp.getEditors();
			if(editors.length == 0){
				this.activeEditor = null;
			}
			return;
		}

		var editors = btp.getEditors(this.activeView);
		var editArea = editors.length > 0 ? editors[0] : null;
		if(editArea){
			if(editArea == this.activeEditor){ return; }
			this.fireActiveEditorChanged(editArea);
			editArea.fireSelectionChanged(editArea.focusWidgets && editArea.focusWidgets[0]); // to update PropertiesView, StyleView and OutlineView
		}
	},

	onTabDblClick: function(message){
	},

	fireActiveEditorChanged: function(/*Widget?*/editArea){
		var message = {
			type: "ActiveEditorChangedNotification",
			src: this,
			editor: editArea
		};
		dojo.publish("btp:ActiveEditorChanged", [message]);
		this.activeEditor = editArea;
	}
});

}

if(!dojo._hasResource["btp.widget._propertiesView.event"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._propertiesView.event"] = true;
dojo.provide("btp.widget._propertiesView.event");

// summary:
//		Handles events on the PropertiesView.
//		'this' points to the PropertiesView.

btp.widget._propertiesView.event = {
	selection: null,
	editArea: null,

	// ======== Topic Subscribers ====================================
	propertyChanged: function(message){
		if(!btp.matchFilter(message, this.propertyChangedFilter)){
			return; // filter out the message
		}
		
		if(message.propName == "_RemoveChild_"){
			if(message.target == this.selection){
				return; // focused widget was removed. selectionChanged(null) will be fired soon.
			}
			// ex. DataGridColumn was removed, but the focus is DataGrid. selectionChanged won't be fired.
			if(!this.selection){
				return; // maybe an undo operation removed something while no focus exists.
			}
		}else if(message.propName == "_AddChild_"){
			if(!this.selection){
				return; // selectionChanged will be fired soon.
			}
		}

		// Call the PropPage's updateView if it exists
		if(this.currentPageObj instanceof dijit._Widget && this.currentPageObj.updateView){
			this.currentPageObj.updateView(this.selection);
		}else{
			this.updateView(this.selection);
		}
	},
	
	selectionChanged: function(message){
		if(!btp.matchFilter(message, this.selectionChangedFilter)){
			return; // filter out the message
		}

		var sel = this.selection = message.selection;
		var src = message.src;
		if(sel && !sel.ps_getPropertyDescriptors){
			return; // doesn't look PropertySource is implemented
		}
		this.editArea = (message.src instanceof btp.widget.EditArea) ? message.src : null;

		// Hide the previous PropPage
		if(this.currentPageNode){
			this.currentPageNode.style.display = "none";
		}
		if(!sel){ return; }

		var name = sel.declaredClass;
		if(sel.widgetTypeSuffix){
			name += ":" + sel.widgetTypeSuffix;
		}
		var obj = this.propPageMap[name];
		if(!obj && this.editArea){
			var info = sel.hlp_getComponentInfo();
			if(info){
				obj = info.proppage;
			}
		}

		// Create a PropPage instance unless cache exists
		if(obj && typeof(obj) == "string"){
			if(this.isWidgetName(obj)){ // PropPage Widget
				if(!this.createPropPageFromWidget){
					dojo["require"]("btp.widget._propertiesView.fromWidget");
					dojo.mixin(this, btp.widget._propertiesView.fromWidget);
				}
				obj = this.createPropPageFromWidget(obj, name, sel);
			}else{ // HTML Fragment
				if(!this.createPropPageFromHtml){
					dojo["require"]("btp.widget._propertiesView.fromHtml");
					dojo.mixin(this, btp.widget._propertiesView.fromHtml);
				}
				obj = this.createPropPageFromHtml(obj, name, sel);
			}
			this.propPageMap[name] = obj; // cache
		}else if(!obj){ // Default PropPage
			if(!this.createPropPageFromXml){
				dojo["require"]("btp.widget._propertiesView.fromXml");
				dojo.mixin(this, btp.widget._propertiesView.fromXml);
			}
			obj = this.createPropPageFromXml(name, sel);
			this.propPageMap[name] = obj; // cache
		}

		// Save the current status
		if(obj instanceof dijit._Widget){ // PropPage Widget
			this.currentPageObj = obj;
			this.currentPageNode = obj.domNode;
		}else /* if(obj.tagName && obj.name == name) */{ // HTML Fragment or Default PropPage
			this.currentPageObj = obj;
			this.currentPageNode = obj;
		}

		// Initialize the PropPage
		this.currentPageNode.style.display = "";

		// Call the PropPage's updateView if it exists
		if(this.currentPageObj instanceof dijit._Widget && this.currentPageObj.updateView){
			this.currentPageObj.updateView(sel);
		}else{
			this.updateView(sel);
		}
	},

	setOnChangeHandler: function(root){
		var fields = root.getElementsByTagName("input");
		var field;
		for(var i=0, len=fields.length; i<len; i++){
			field = fields[i];
			dojo.connect(field, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
			if(dojo.isIE){
				dojo.connect(field, "onkeypress", this, "onKeyPressHandler"); //TODO: you may want to put connect() into this._connects.push()
			}
		}
		fields = null;
		fields = root.getElementsByTagName("select");
		for(var j=0, len2=fields.length; j<len2; j++){
			field = fields[j];
			dojo.connect(field, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
		}
	},


	// ======== Handle User Input ================================
	onChangeHandler: function(e){
		// Call the PropPage's onChangeHandler if it exists
		if(this.currentPageObj instanceof dijit._Widget && this.currentPageObj.onChangeHandler){
			this.currentPageObj.onChangeHandler(e);
			return;
		}
		
		var field = e.target;
		var attrName = field.name;
		var val = field.value;
		if(!attrName){ return; }
		if(field.type != "checkbox"){
			this.setAttribute(this.selection, attrName, val);
		}else{
			this.setAttribute(this.selection, attrName, field.checked);
		}
	},
	
	onKeyPressHandler: function(e){
		if(e.keyCode == 13){ // Enter key
			this.onChangeHandler(e);
		}
	}
};

}

if(!dojo._hasResource["btp.widget._propertiesView.fromHtml"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._propertiesView.fromHtml"] = true;
dojo.provide("btp.widget._propertiesView.fromHtml");

// summary:
//		Creates a property page from HTML
//		'this' points to the PropertiesView.

btp.widget._propertiesView.fromHtml = {
	createPropPageFromHtmlFragment: function(url, name, sel){
		var _this = this;
		var root;
		function loadHtmlFragment (type, data, evt){
			var div = dojo.doc.createElement("DIV");
			div.innerHTML = data;
			root = div.firstChild;
			_this.domNode.appendChild(div.firstChild);
			div = null;
		}
		dojo.io.bind({url: url, load: loadHtmlFragment, sync: true, mimetype: "text/plain"});

		this.setOnChangeHandler(root);
		this.createElementsMap(root); // elements map
		root.name = name; // key to cache

		return root;
	}
};

}

if(!dojo._hasResource["btp.widget._propertiesView.fromWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._propertiesView.fromWidget"] = true;
dojo.provide("btp.widget._propertiesView.fromWidget");

// summary:
//		Creates a property page from Widget
//		'this' points to the PropertiesView.

btp.widget._propertiesView.fromWidget = {
	createPropPageFromWidget: function(widgetName, name, sel){
		// KLUGE: First, create a placeholder div and use it as a ref node for createWidget.
		// Then remove it from the DOM tree. Subsequent addChild append it to the DOM again.
		// This is because DataGridPropPage has subwidgets and the rendering must be
		// performed online. Otherwise, we see strange rendering on both IE and FF.
		// In addition to that, the view must be visible when the rendering is performed.
		// So, we need to temporarily clear the display property if it is set to "none".

		var disp = this.domNode.style.display;
		if(disp == "none"){
			this.domNode.style.display = "";
		}

		var div = dojo.doc.createElement("DIV");
		this.domNode.appendChild(div);
		var widget = dojo.widget.createWidget(widgetName, {propViewId: this.id}, div); //TODO: createWidget -> new
		this.domNode.removeChild(widget.domNode);
		this.addChild(widget);
		this.onResized();

		if(widget.useDefaultHandler){
			this.setOnChangeHandler(widget.domNode);
		}
		this.createElementsMap(widget.domNode); // elements map
		widget.domNode.name = name; // key to cache

		if(disp == "none"){
			this.domNode.style.display = disp;
		}

		return widget;
	}
};

}

if(!dojo._hasResource["btp.widget._propertiesView.fromXml"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._propertiesView.fromXml"] = true;
dojo.provide("btp.widget._propertiesView.fromXml");

// summary:
//		Creates a property page from XML
//		'this' points to the PropertiesView.

btp.widget._propertiesView.fromXml = {
	createPropPageFromXml: function(name, sel){
		var root;
		var table = root = dojo.doc.createElement("TABLE");
		table.className = "btpPropertyViewTable";
		table.cellSpacing = "2px";
		table.cellPadding = "0px";
		table.style.width = "90%";
		table.style.marginTop = "3px";
		table.style.marginLeft = "3px";
		var attributes = sel.ps_getPropertyDescriptors();
		for(var i = 0, len = attributes.length; i < len; i++){
			var attr = attributes[i];
			var attrName = attr.name;
			var type = attr.type;
			var fieldType = attr.fieldType;

			if(attr.display == "false"){
				continue;
			}

			var row = table.insertRow(-1);

			if(fieldType == "select"){
				this.createDefaultCombobox(name, row, attr);
			}else if(fieldType == "checkbox"){
				this.createDefaultCheckbox(name, row, attr);
			}else if(fieldType == "radio"){
				this.createDefaultRadioButtons(name, row, attr);
			}else /* if(fieldType == "input") */ {
				this.createDefaultTextField(name, row, attr);
				attr.fieldType = "text";
			}
		}
		this.domNode.appendChild(root);

		this.createElementsMap(root); // elements map
		root.name = name; // key to cache

		return root;
	},
	
	createDefaultTextField: function(widgetName, row, attr){
		var attrName = attr.name;
		var id = this.id + "_" + widgetName + "_" + attrName;
		
		// get localized string
		var label = btp.i18n.gets(attrName, "component_"+name+"_"); // Ex. component_live:pushbutton_value
		if(label == attrName){
			label = btp.i18n.gets(attrName, "component_"); // Ex. component_value
		}
		
		var cell = row.insertCell(-1);
		cell.className = "btpPorpInputLabel";
		cell.noWrap = true;
		cell.innerHTML = "<LABEL for='"+id+"'>"+label+"</LABEL>";
		
		cell = row.insertCell(-1);
		cell.className = "btpPorpInputField";
		cell.width = "100%";
		cell.attribute = attr;
		cell.innerHTML = "<INPUT id='"+id+"' name='"+attrName+"' class='btpPropInput' size='50'>";
		var input = cell.firstChild;
		if(attr.readonly == "true"){
			input.readOnly = true;
			input.style.color = "#ACA899";
		}

		dojo.connect(input, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
		if(dojo.isIE){
			dojo.connect(input, "onkeypress", this, "onKeyPressHandler"); //TODO: you may want to put connect() into this._connects.push()
		}
	},
	
	createDefaultCheckbox: function(widgetName, row, attr){
		var attrName = attr.name;
		var id = this.id + "_" + widgetName + "_" + attrName;
		
		// get localized string
		var label = btp.i18n.gets(attrName, "component_"+name+"_");
		if(label == attrName){
			label = btp.i18n.gets(attrName, "component_");
		}
		
		var cell = row.insertCell(-1);
		cell.className = "btpPropInputLabel";
		cell.noWrap = true;
		cell.colSpan = "2";
		cell.innerHTML = "<INPUT id='"+id+"' name='"+attrName+"' class='btpPropCheckBox' type='checkbox'>"
						+"<LABEL for='"+id+"'>"+label+"</LABEL>";
		var checkbox = cell.firstChild;
		if(attr.value && attr.value == "true"){
			checkbox.checked = "checked";
		}
		dojo.connect(checkbox, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
	},
	
	createDefaultRadioButtons: function(widgetName, row, attr){
		var attrName = attr.name;
		var id = this.id + "_" + widgetName + "_" + attrName;
		
		// get localized string
		var label = btp.i18n.gets(attrName, "component_"+name+"_");
		if(label == attrName){
			label = btp.i18n.gets(attrName, "component_");
		}
		
		var cell = row.insertCell(-1);
		cell.className = "btpPropInputLabel";
		cell.noWrap = true;
		cell.innerHTML = "<LABEL for='"+id+"'>"+label+"</LABEL>";
		
		cell = row.insertCell(-1);
		cell.className = "btpPropInputField";
		cell.width = "100%";
		cell.attribute = attr;
		var option;
		if(attr.option){
			option = attr.option;
		}else{
			option = ["true", "false"];
		}

		for(var i=0, len=option.length; i<len; i++){
			cell.innerHTML = "<INPUT id='"+id+"_"+option[i]+"' name='"+attrName+"' class='btpPropRadio' type='radio'>"
							+"<LABEL for='"+id+"_"+option[i]+"'>"+option[i]+"</LABEL>";
			var radioButton = cell.firstChild;
			radioButton.value = option[i];
			if(i == 0 || attr.value && attr.value == option[i]){
				radioButton.checked = "checked";
			}
			dojo.connect(radioButton, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
		}
	},
	
	createDefaultCombobox: function(widgetName, row, attr){
		var attrName = attr.name;
		var id = this.id + "_" + widgetName + "_" + attrName;
		
		// get localized string
		var label = btp.i18n.gets(attrName, "component_"+name+"_");
		if(label == attrName){
			label = btp.i18n.gets(attrName, "component_");
		}
		
		var cell = row.insertCell(-1);
		cell.className = "porpInputLabel";
		cell.noWrap = true;
		cell.innerHTML = "<LABEL for='"+id+"'>"+label+"</LABEL>";
		
		cell = row.insertCell(-1);
		cell.className = "propInputField";
		cell.width = "100%";
		cell.attribute = attr;
		cell.innerHTML = "<SELECT id='"+id+"' name='"+attrName+"' class='btpPropComboBox'>";
		var comboBox = cell.firstChild;
		var option;
		if(attr.option){
			option = attr.option;
		}else{
			option = ["true", "false"];
		}
		
		var selectedIndex = 0;
		for(var i = 0, len = option.length; i < len; i++){
			var opt = new Option(option[i], option[i]);
			comboBox.options[i] = opt;
			if(attr.value && attr.value == option[i]){
				selectedIndex = i;
			}
		}
		comboBox.options[selectedIndex].selected = true;
		comboBox.options[selectedIndex].defaultSelected = true;
		dojo.connect(comboBox, "onchange", this, "onChangeHandler"); //TODO: you may want to put connect() into this._connects.push()
	}
};

}

if(!dojo._hasResource["btp.widget._propertiesView.update"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["btp.widget._propertiesView.update"] = true;
dojo.provide("btp.widget._propertiesView.update");

// summary:
//		Updates the PropertiesView.
//		'this' points to the PropertiesView.

btp.widget._propertiesView.update = {
	currentPageObj: null,  // DOM node or widget
	currentPageNode: null, // DOM node

	// ======== Fill-in the view with the attributes of the widget =======
	// Your PropPage Widget may want to override this
	updateView: function(widget){
		if(!widget || !widget.ps_getPropertyDescriptors){
			if(this.currentPageNode){
				this.currentPageNode.style.display = "none";
			}
			return;
		}

		var attributes = widget.ps_getPropertyDescriptors();
		var root = this.currentPageNode;
		
		for(var i = 0, len = attributes.length; i < len; i++){
			var attr = attributes[i];
			var attrName = attr.name;
			var type = attr.type;
			var fieldType = attr.fieldType;
			var input = root.elements[widget.widgetType+":"+attrName] ? root.elements[widget.declaredClass+":"+attrName] : root.elements[attrName];
			if(!input){
				continue;
			}
			
			var val = widget.ps_getPropertyValue(attrName);
			switch(fieldType){
				case "radio":
					if(val && val == input.value){
						input.checked = "checked";
					}
					break;
				case "select":
					var option = input.options;
					for(var j=0, len2=option.length; j<len2; j++){
						if(option[j].value == val){
							input.selectedIndex = j;
						}
					}
					break;
				case "checkbox":
					if(val){
						input.checked = "checked";
					}else{
						input.checked = "";
					}
					break;
				case "text":
				default:
					input.value = val? val : "";
					break;
			}
		}
	},

	
	setAttribute: function(widget, attribute, val){
		if(typeof(attribute) == "string"){
			var idx = attribute.indexOf(":");
			if(idx != -1){ // remove widgetType prefix (ex. DataGrid:ref -> ref) //TODO: Do not omit curly braces, '{' and '}'. //TODO: widgetType -> declaredClass
				attribute = attribute.substring(idx + 1);
			}
			attribute = this.editArea.getAttributeInfo(widget, attribute);
		}
		if(attribute.type == "integer"){
			val = val - 0;
		}
		var command = new btp.widget.command.EditCommand.SetPropertyValueCommand(widget, attribute, val);
		this.executeCommand(command);
	},

	
	// ======== Utilities ========================================
	executeCommand: function(command){
		this.editArea.getEditDomain().executeCommand(command);
	},
	
	setFocus: function(widget){
		this.editArea.setFocus(widget);
	},
	
	isWidgetName: function(str){
		var exp = /^[A-z]+:+[A-z0-9]+$/;
		var result = str.match(exp);
		return !!result;
	},

	createElementsMap: function(node){
		node.elements = []; // [custom property]
		var elems = node.getElementsByTagName("INPUT");
		for(var i = 0, len = elems.length; i < len; i++){
			var input = elems[i];
			if(input.name){
				node.elements.push(input); // array
				node.elements[input.name] = input; // map
			}
		}
		elems = node.getElementsByTagName("SELECT");
		for(var i = 0, len = elems.length; i < len; i++){
			var input = elems[i];
			if(input.name){
				node.elements.push(input); // array
				node.elements[input.name] = input; // map
			}
		}
	}
};

}

if(!dojo._hasResource["gdc.widget.GfxContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.GfxContainer"] = true;
dojo.provide("gdc.widget.GfxContainer");


// Summary:
//		This class is intended to be used by container-type gfx widgets.
//		(GfxWidget cannot have addChild/removeChild since it does not
//		 inherit from dijit._Container. That's why this class exists.)

dojo.declare(
	"gdc.widget.GfxContainer",
	dijit._Container,
{
	// override
	addChild: function(/*Widget*/widget, /*int?*/insertIndex){
		this.inherited("addChild", arguments);
		if(widget instanceof gdc.widget.GfxWidget && this.groupBox){
			widget.createShape(this.groupBox); // subclass must prepare "this.groupBox"
			widget.updateRecursively();
		}
	},

	// override
	removeChild: function(/*Widget*/widget){
		this.inherited("removeChild", arguments);
		if(widget instanceof gdc.widget.GfxWidget){
			widget.removeShape();
		}
	}
});

}

if(!dojo._hasResource["gdc.widget.AbstractLine"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.AbstractLine"] = true;
dojo.provide("gdc.widget.AbstractLine");




dojo.declare(
	"gdc.widget.AbstractLine",
	[gdc.widget.GfxWidget, gdc.widget.GfxContainer],
{
	points: "", // ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"

	// default line styles (see also dojox.gfx.defaultStroke)
	lineColor: "black",
	lineStyle: "solid",
	lineWidth: 2, // Integer
	lineCap: "butt",
	lineJoin: 4, // Integer

	sourceDecoration: "", // decoration class name
	sourceDecorationProps: null, // properties for the source decoration
	targetDecoration: "", // decoration class name
	targetDecorationProps: null, // properties for the target decoration

	sourceDecorationObj: null, // decoration object
	targetDecorationObj: null, // decoration object

	postMixInProperties: function(){
		if (!this.points){
			this.points = [];
		}else if (typeof(this.points) == "string"){
			// ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"
			this.points = eval('('+this.points+')');
		}
		if(this.sourceDecoration){
			var dec = this.sourceDecoration;
			this.sourceDecoration = null;
			this.setSourceDecoration(dec);
		}
		if(this.targetDecoration){
			var dec = this.targetDecoration;
			this.targetDecoration = null;
			this.setTargetDecoration(dec);
		}
	},

	postCreate: function(){
		this.domNode.style.display = "none";
		this.inherited("postCreate", arguments);
	},

	createTargetShape: function(parent){
		if(this.sourceDecorationObj){
			this.sourceDecorationObj.createTargetShape(parent);
		}
		if(this.targetDecorationObj){
			this.targetDecorationObj.createTargetShape(parent);
		}
	},

	restoreShape: function(){
		this.inherited("restoreShape", arguments);
		if(this.sourceDecorationObj){
			this.recoverLostInfo(this.sourceDecorationObj.shape);
		}
		if(this.targetDecorationObj){
			this.recoverLostInfo(this.targetDecorationObj.shape);
		}
	},

	update: function(props){
		this.inherited("update", arguments);
		var shape = this.shape;
		if(!shape){ return; }

		if((props && props.forceUpdate) || !this._cur
			|| this._cur.color != this.lineColor
			|| this._cur.style != this.lineStyle
			|| this._cur.width != this.lineWidth
			|| this._cur.cap != this.lineCap
			|| this._cur.join != this.lineJoin){
			shape.setStroke({
				color: dojox.gfx.normalizeColor(this.lineColor),
				style: this.lineStyle,
				width: this.lineWidth,
				cap: this.lineCap,
				join: this.lineJoin
			});
		}

		// save the current status
		this._cur = {
			color: this.lineColor,
			style: this.lineStyle,
			width: this.lineWidth,
			cap: this.lineCap,
			join: this.lineJoin,
			fillColor: this.fillColor
		};

		if(this.sourceDecorationObj){
			this.sourceDecorationObj.update();

			var pt = this.points[0];
			this.sourceDecorationObj.setLocation(pt);

			var angle = this.calculateAngle(this.points[1], pt);
			this.sourceDecorationObj.setRotation(angle);
		}
		if(this.targetDecorationObj){
			this.targetDecorationObj.update();

			var pt = this.points[this.points.length - 1];
			this.targetDecorationObj.setLocation(pt);

			var angle = this.calculateAngle(this.points[this.points.length - 2], pt);
			this.targetDecorationObj.setRotation(angle);
		}
	},
	
	_comparePoints: function(points0, points1){
		if(!points0 || !points1 || points0.length != points1.length){ return -1; }
		for(var i = 0, len = points0.length; i < len; i++){
			if(points0[i].x != points1[i].x || points0[i].y != points1[i].y){
				return -1;
			}
		}
		return 0;
	},

	removeShape: function(){
		if(this.sourceDecorationObj){
			this.sourceDecorationObj.removeShape();
		}
		if(this.targetDecorationObj){
			this.targetDecorationObj.removeShape();
		}
		this.inherited("removeShape", arguments);
	},

	calculateAngle: function(pt0, pt1){
		return 0;
	},

	// Called when points are added or changed. You may want to connect to this.
	onPointsChanged: function(){
	},

	_setDecoration: function(/*String|Decoration*/dec, which){
		if(!dec){
			this[which+"Decoration"] = "";
			this[which+"DecorationObj"] = null;
			return;
		}
		if(this[which+"Decoration"] == dec || this[which+"DecorationObj"] == dec){ return; }
		var newInstance;
		if(typeof(dec) == "string"){
			this[which+"Decoration"] = dec;
			dojo["require"](dec);
			var decorationImpl = dojo.getObject(dec);
			var props = this[which+"DecorationProps"];
			if(props && typeof(props) == "string"){
				props = eval('('+props+')');
			}
			newInstance = new decorationImpl(this, props);
		}else{
			this[which+"Decoration"] = dec.declaredClass;
			newInstance = dec;
		}
		if(this[which+"DecorationObj"]){
			this[which+"DecorationObj"].destroy(); // destroy the old instance
			newInstance.createTargetShape(this.groupBox);
		}
		this[which+"DecorationObj"] = newInstance;
		this.update();
	},

	_setDecorationProps: function(/*String|Object*/props, which){
		if(props && typeof(props) == "string"){
			props = eval('('+props+')');
		}
		this[which+"DecorationProps"] = props;
		if(this[which+"DecorationObj"]){
			this[which+"DecorationObj"].setProps(props);
		}
		this.update();
	},

	setSourceDecoration: function(/*String|Decoration*/dec){
		this._setDecoration(dec, "source");
	},

	setSourceDecorationProps: function(/*String|Object*/props){
		this._setDecorationProps(props, "source");
	},

	setTargetDecoration: function(dec){
		this._setDecoration(dec, "target");
	},

	setTargetDecorationProps: function(/*String|Object*/props){
		this._setDecorationProps(props, "target");
	},

	// Adds the passed point to the Polyline.
	addPoint: function(/*{x:#,y:#}*/pt){
		this.points.push(pt);
	},

	// Inserts a given point at a specified index in the Polyline.
	insertPoint: function(/*{x:#,y:#}*/pt, /*Integer*/index){
		this.points.splice(index, 0, pt);
	},

	// Erases the Polyline and removes all of its points.
	removeAllPoints: function(){
		this.points = [];
	},

	// Removes a point from the Polyline.
	removePoint: function(index){
		this.points.splice(index, 1);
	},

	// Sets the points at both extremes of the Polyline.
	setEndpoints: function(/*{x:#,y:#}*/start, /*{x:#,y:#}*/end){
		this.setStart(start);
		this.setEnd(end);
	},

	// Sets the point at index to the point pt.
	setPoint: function(/*{x:#,y:#}*/pt, /*Integer*/index){
		this.points[index] = pt;
	},


// ======================= Setters/Getters =================================

	// Returns the first point in the Polyline.
	getStart: function(){
		if(this.points.length == 0){ return null; }
		return this.points[0];
	},

	// Sets the start point of the Polyline.
	setStart: function(/*{x:#,y:#}*/start){
		if (this.points.length == 0){
			this.addPoint(start);
		}else{
			this.setPoint(start, 0);
		}
	},


	// Returns the last point in the Polyline.
	getEnd: function(){
		if(this.points.length == 0){ return null; }
		return this.points[this.points.length - 1];
	},

	// Sets the end point of the Polyline.
	setEnd: function(/*{x:#,y:#}*/end){
		if (this.points.length < 2){
			this.addPoint(end);
		}else{
			this.setPoint(end, this.points.length - 1);
		}
	},


	// Returns the points in this Polyline.
	getPoints: function(){
		return this.points;
	},

	// Sets the list of points to be used by this polyline.
	// Removes any previously existing points.
	setPoints: function(/*Array|String*/points){
		if (!points){
			this.points = [];
		}else if (typeof(points) == "string"){
			// ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"
			this.points = eval('('+points+')');
		}else{
			this.points = points;
		}
	},
	
	
	getLineColor: function(){
		return this.lineColor;
	},

	setLineColor: function(color){
		this.lineColor = dojox.gfx.normalizeColor(color);
		if(!this.shape){ return; }
		var strokeStyle = this.shape.getStroke();
		strokeStyle.color = color;
		this.shape.setStroke(strokeStyle);
	},


	getLineStyle: function(){
		return this.lineStyle;
	},

	setLineStyle: function(style){
		this.lineStyle = style;
		if(!this.shape){ return; }
		var strokeStyle = this.shape.getStroke();
		strokeStyle.style = style;
		this.shape.setStroke(strokeStyle);
	},


	getLineWidth: function(){
		return this.lineWidth;
	},

	setLineWidth: function(width){
		this.lineWidth = width - 0;
		if(!this.shape){ return; }
		var strokeStyle = this.shape.getStroke();
		strokeStyle.width = width - 0;
		this.shape.setStroke(strokeStyle);
	},


	getLineCap: function(){
		return this.lineCap;
	},

	setLineCap: function(cap){
		this.lineCap = cap;
		if(!this.shape){ return; }
		var strokeStyle = this.shape.getStroke();
		strokeStyle.cap = cap;
		this.shape.setStroke(strokeStyle);
	},


	getLineJoin: function(){
		return this.lineJoin;
	},

	setLineJoin: function(join){
		this.lineJoin = join - 0;
		if(!this.shape){ return; }
		var strokeStyle = this.shape.getStroke();
		strokeStyle.join = join - 0;
		this.shape.setStroke(strokeStyle);
	}
});

}

if(!dojo._hasResource["gdc.widget.anchor.AbstractConnectionAnchor"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.anchor.AbstractConnectionAnchor"] = true;
dojo.provide("gdc.widget.anchor.AbstractConnectionAnchor");

dojo.declare(
	"gdc.widget.anchor.AbstractConnectionAnchor",
	null,
{
	owner: null,
	listeners: null, // Array

	constructor: function(owner){
		this.owner = owner;
		this._connector = dojo.connect(owner, "nodeMoved", this, "anchorMoved");
	},

	destroy: function(){
		dojo.disconnect(this._connector);
	},

	addAnchorListener: function(/*ConnectionWidget*/listener){
		if(!this.listeners){ this.listeners = []; }
		if(dojo.indexOf(this.listeners, listener) != -1){//TODO: remove this code later
			alert("gdc.widget.anchor.AbstractConnectionAnchor#addAnchorListener: internal error: already registered");
		}
		this.listeners.push(listener);
	},

	removeAnchorListener: function(/*ConnectionWidget*/listener){
		var idx = dojo.indexOf(this.listeners, listener);
		if(idx != -1){
			this.listeners.splice(idx, 1);
		}
	},

	anchorMoved: function(/*Widget*/widget){
		if(!this.listeners){ return; }
		for(var i = 0, len = this.listeners.length; i < len; i++){
			this.listeners[i].anchorMoved(widget);
		}
	},

	getLocation: function(/*{x:#,y:#}*/ref){
		// subclass must implement
	},

	// Returns direction of connection
	// pt is coordinate calculated by getLocation
	// subclass may override
	getDirection: function(/*{x:#,y:#}*/pt){
		if(!pt){ return; }
		var owner = this.owner;
		
		// lm1 and rm corrsponds to left-/right-hand member of line1, respectively :
		// 		y = (owner.height/owner.width)*(x - owner.left) + owner.top
		var lm1 = owner.width*(pt.y - owner.top);
		var rm = owner.height*(pt.x - owner.left);
		
		// lm2 corrsponds to left-hand member of line2 :
		// y = -(owner.height/owner.width)*(x - owner.left) + (owner.top + owner.height)
		var lm2 = owner.width*(pt.y - owner.top - owner.height);
		
		if((lm1 < rm) && (lm2 >= -rm)){
			return "right";
		}else if((lm1 >= rm) && (lm2 >= -rm)){
			return "bottom";
		}else if((lm1 < rm) && (lm2 < -rm)){
			return "top";
		}else{
			return "left";
		}
	}, 

	getReferencePoint: function(){
		// subclass must implement
	}
});

}

if(!dojo._hasResource["gdc.widget.anchor.ChopboxAnchor"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.anchor.ChopboxAnchor"] = true;
dojo.provide("gdc.widget.anchor.ChopboxAnchor");



// Summary:
//		The ChopboxAnchor's location is found by calculating the intersection of a line drawn
//		from the center point of its owner's box to a reference point on that box.

dojo.declare(
	"gdc.widget.anchor.ChopboxAnchor",
	gdc.widget.anchor.AbstractConnectionAnchor,
{
	getLocation: function(/*{x:#,y:#}*/ref){
		var owner = this.owner;
		var cx = owner.left + owner.width/2;
		var cy = owner.top + owner.height/2;
		var dx = (ref && ref.x) - cx;
		var dy = (ref && ref.y) - cy;
		var ratioX = (owner.width/2) / Math.abs(dx);
		var ratioY = (owner.height/2) / Math.abs(dy);
		var ratio = Math.min(ratioX, ratioY);
		var x = Math.round(cx + ratio * dx);
		var y = Math.round(cy + ratio * dy);
		return {x:x, y:y};
	},

	getReferencePoint: function(){
		var owner = this.owner;
		var x = owner.left + owner.width/2;
		var y = owner.top + owner.height/2;
		return {x:x, y:y}; // center of the bounding box
	}
});

}

if(!dojo._hasResource["gdc.widget.anchor.EllipseAnchor"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.anchor.EllipseAnchor"] = true;
dojo.provide("gdc.widget.anchor.EllipseAnchor");



// Summary:
//		The EllipseAnchor's location is found by calculating the intersection of a line drawn
//		from the center point of ellipse to a reference point on that box.

dojo.declare(
	"gdc.widget.anchor.EllipseAnchor",
	gdc.widget.anchor.AbstractConnectionAnchor,
{
	getLocation: function(/*{x:#,y:#}*/ref){
		var owner = this.owner;
		var cx = owner.left + owner.width/2;
		var cy = owner.top + owner.height/2;
		var dx = owner.height*((ref && ref.x) - cx);
		var dy = owner.width*((ref && ref.y) - cy);
		var r = Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2));
		return {
			x: Math.round(cx + owner.width/2*dx/r),
			y: Math.round(cy + owner.height/2*dy/r)
		};
	},

	getReferencePoint: function(){
		var owner = this.owner;
		var x = owner.left + owner.width/2;
		var y = owner.top + owner.height/2;
		return {x:x, y:y}; // center of the bounding box
	}
});

}

if(!dojo._hasResource["gdc.widget.anchor.FixedAnchor"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.anchor.FixedAnchor"] = true;
dojo.provide("gdc.widget.anchor.FixedAnchor");



// Summary:
//		The FixedAnchor's location is the center of the owner's box.

dojo.declare(
	"gdc.widget.anchor.FixedAnchor",
	gdc.widget.anchor.AbstractConnectionAnchor,
{
	endPointDomNode: null,
	endPointShape: null,
	
	offsetH: 0,
	offsetV: 0,
	centerOrigin: true,

	getLocation: function(/*{x:#,y:#}*/ref){
		var owner = this.owner;
		var matrix = this.endPointShape.getTransform();
		var bb = this.endPointShape.getBoundingBox();
		var x = owner.left + matrix.dx + bb.x + (this.centerOrigin ? bb.width/2 : 0) + this.offsetH;
		var y = owner.top + matrix.dy + bb.y + (this.centerOrigin ? bb.height/2 : 0) + this.offsetV;
		return {x:x, y:y}; // center of the bounding box
	},

	getReferencePoint: function(){
		var owner = this.owner;
		var x = owner.left + owner.width/2;
		var y = owner.top + owner.height/2;
		return {x:x, y:y}; // center of the bounding box
	}
});

}

if(!dojo._hasResource["gdc.widget.Curvyline"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.Curvyline"] = true;
dojo.provide("gdc.widget.Curvyline");



dojo.declare(
	"gdc.widget.Curvyline",
	gdc.widget.AbstractLine,
{
	createTargetShape: function(parent){
		var shape = parent.createPath();
		this.onPointsChanged();
		var pathStr = this._points2PathStr();
		if(pathStr){
			shape.setShape(pathStr);
		}
		this.inherited("createTargetShape", arguments);
		return shape;
	},

	update: function(props){
		if(!this.shape || this.points.length < 2){ return; }
		this.inherited("update", arguments);

		if(this._comparePoints(this.points, this.shape.shape.points) != 0){
			this.onPointsChanged();
			var pathStr = this._points2PathStr();
			if(pathStr){
				this.shape.setShape(pathStr);
			}
		}
	},
	
	_points2PathStr: function(){
		var pathStr = null;
		var str_M = "M";
		var str_Q = " Q";
		var str_T = " T";
		var strDelimiter = " ";
		var len = this.points.length;
		for(var i=0; i<len; i++){
			if(i==0){
				pathStr = str_M;
			}else if(i == len-1){
				pathStr += str_T;
			}else{
				pathStr += str_Q;
			}
			pathStr += this.points[i].x + strDelimiter + this.points[i].y;
			if(i > 0 && i < len-1){
			 	pathStr += strDelimiter + (this.points[i].x+this.points[i+1].x)/2 + strDelimiter
				+ (this.points[i].y+this.points[i+1].y)/2 + strDelimiter;
			}
		}
		return pathStr;
	}
});

}

if(!dojo._hasResource["gdc.widget.support.connection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.support.connection"] = true;
dojo.provide("gdc.widget.support.connection");

// summary:
//		Make a gfx line widget connection.
//		This class is intended to be mixed into a gfx line widget such as Polyline.
//		'this' points to a gfx line widget.

gdc.widget.support.connection = {
	sourceAnchor: "", // "nodeId" or "nodeId:anchorName"
	targetAnchor: "", // "nodeId" or "nodeId:anchorName"
	router: "",

	sourceAnchorObj: null, // anchor object
	targetAnchorObj: null, // anchor object
	routerObj: null, // router object
	
	mouseOverColor: "",

	processParams: function(){
		if(this.sourceAnchor && !this.sourceAnchorObj){
			var anchor = this.sourceAnchor;
			this.sourceAnchor = null;
			this.setSourceAnchor(anchor);
		}
		if(this.targetAnchor && !this.targetAnchorObj){
			var anchor = this.targetAnchor;
			this.targetAnchor = null;
			this.setTargetAnchor(anchor);
		}
		this.registerCallback("removeShape", this.disconnect);
		this.registerCallback("restoreShape", this.reconnect);
		this.setRouter(this.router);
		this.anchorMoved();
	},

	cleanup: function(){
		this.disconnect();
	},

	disconnect: function(){
		if(this.sourceAnchorObj){
			this.sourceAnchorObj.removeAnchorListener(this);
		}
		if(this.targetAnchorObj){
			this.targetAnchorObj.removeAnchorListener(this);
		}
		var editors = btp.getEditors();
		dojo.forEach(editors, function(editor){
			if(editor.popup){
				var popup = editor.popup;
				if(popup && popup.hide){
					popup.hide();
				}
			}
		});
		
	},

	reconnect: function(){
		if(this.sourceAnchorObj){
			this.sourceAnchorObj.addAnchorListener(this);
		}
		if(this.targetAnchorObj){
			this.targetAnchorObj.addAnchorListener(this);
		}
	},


	_setAnchor: function(/*String|Anchor*/anchor, which){
		if(anchor && (this[which+"Anchor"] == anchor || this[which+"AnchorObj"] == anchor)){ return; }
		if(!anchor && (!this[which+"Anchor"] && !this[which+"AnchorObj"])){ return; }
		if(this[which+"AnchorObj"]){
			this[which+"AnchorObj"].removeAnchorListener(this); // remove the old anchor
		}
		if(!anchor){
			this[which+"Anchor"] = "";
			this[which+"AnchorObj"] = null;
			return;
		}
		var anchorObj;
		if(typeof(anchor) == "string"){
			var arr = anchor.split(":"); // "nodeId:anchorName"
			var id =  arr[0];
			var anchorName = arr[1] || "base"; // default anchor name is "base"
			var nodeWidget = dijit.byId(id)
			var cap = which.charAt(0).toUpperCase() + which.substring(1);
			anchorObj = nodeWidget && nodeWidget["get"+cap+"ConnectionAnchorByName"](anchorName);
		}else{
			this[which+"Anchor"] = anchor.owner && anchor.owner.id;
			anchorObj = anchor;
		}
		anchorObj.addAnchorListener(this); // add the new anchor
		this[which+"AnchorObj"] = anchorObj;
		this.anchorMoved();
	},

	setSourceAnchor: function(/*String|Anchor*/anchor){
		this._setAnchor(anchor, "source");
	},

	setTargetAnchor: function(/*String|Anchor*/anchor){
		this._setAnchor(anchor, "target");
	},
	
	getSource: function() {
		if(this.sourceAnchorObj) return this.sourceAnchorObj.owner;
		else return null;
	},
	
	getTarget: function() {
		if(this.targetAnchorObj) return this.targetAnchorObj.owner;
		else return null;
	},	

	setRouter: function(/*String|Router*/router){
		if(!router){
			if(this.router && this.router.destroy){
				this.router.destroy();
			}
			this.router = this.routerObj = null;
			return;
		}
		if(typeof(router) == "string"){
			try{
				dojo["require"](router);
				var routerImpl = dojo.getObject(router);
				this.routerObj = new routerImpl(this);
			}catch(e){
				console.debug("Specified router was not found: '"+router+"'");
			}
		}else if(typeof(router) == "object"){
			this.routerObj = router;
		}
		this.anchorMoved();
	},
	
	setMouseOverHandler: function(){
		dojo.connect(this.shape.rawNode, "onmouseover", this, "_onMouseOver");
	},
	
	_onMouseOver: function(e){
		var editArea = this.__editArea;
		var popup = editArea.popup;
		var oldConnection = popup.connection;
		if(oldConnection && oldConnection.origStroke){
				oldConnection.shape.setStroke(oldConnection.origStroke);
		}

		popup.showPopup(e, this);
	},

	anchorMoved: function(/*Widget*/widget){
		if(this.sourceAnchorObj){
			var ref;
			if(this.targetAnchorObj){
				ref = this.targetAnchorObj.getReferencePoint();
			}else{
				ref = this.getEnd();
			}
			var pt = this.sourceAnchorObj.getLocation(ref);
			var widget = this.sourceAnchorObj.owner.getParent();
			var parent = this.getParent();
			if(!widget || !parent){ return; }
			var dim = this._getOffsetPos(widget.domNode, parent.domNode);
			pt.x += dim.left;
			pt.y += dim.top;
/**** Consider this later *****/
			var dir = this.sourceAnchorObj.getDirection(pt);
			if(dir){
				pt.dir = dir;
			}
/*******************************/
			this.setStart(pt);
		}

		if(this.targetAnchorObj){
			var ref;
			if(this.sourceAnchorObj){
				ref = this.sourceAnchorObj.getReferencePoint();
			}else{
				ref = this.getStart();
			}
			var pt = this.targetAnchorObj.getLocation(ref);
			var widget = this.targetAnchorObj.owner.getParent();
			var parent = this.getParent();
			if(!widget || !parent){ return; }
			var dim = this._getOffsetPos(widget.domNode, parent.domNode);
			pt.x += dim.left;
			pt.y += dim.top;
/**** Consider this later *****/
			var dir = this.targetAnchorObj.getDirection(pt);
			if(dir){
				pt.dir = dir;
			}
/*******************************/
			this.setEnd(pt);
		}
		/* hide disconnect popup */
		var editors = btp.getEditors();
		dojo.forEach(editors, function(editor){
			if(editor.popup){
				var popup = editor.popup;
				if(popup && popup.hide){
					popup.hide();
				}
			}
		});		
		this.update();
	},

	_getOffsetPos: function(elem, parent){
		var top = 0, left = 0;
		for(; elem != null; elem = elem.offsetParent){
			if(elem == parent){ break; }
			top += elem.offsetTop;
			left += elem.offsetLeft;
		}
		return {top:top, left:left};
	}
};

}

if(!dojo._hasResource["gdc.widget.CurvylineConnection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.CurvylineConnection"] = true;
dojo.provide("gdc.widget.CurvylineConnection");




dojo.declare(
	"gdc.widget.CurvylineConnection",
	gdc.widget.Curvyline,
{
	startup: function(){
		this.processParams();
	},

	update: function(){
		if(!this.shape || this.points.length < 2){ return; }
		this.inherited("update", arguments);
		if(this.targetDecoration && typeof(this.targetDecoration) != "string"){
			var pt = this.getEnd();
			this.targetDecoration.setLocation(pt);

			var pt0 = this.points[this.points.length - 2];
			var dx = Math.abs(pt.x - pt0.x);
			var dy = pt.y - pt0.y;
			var angle = Math.atan(dy/dx)/Math.PI*180;
			if(pt.x - pt0.x < 0){
				angle = 180 - angle;
			}
			this.targetDecoration.setRotation(angle);
		}
	}
});
dojo.extend(gdc.widget.CurvylineConnection, gdc.widget.support.connection);

}

if(!dojo._hasResource["gdc.widget.decoration.ImageDecoration"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.decoration.ImageDecoration"] = true;
dojo.provide("gdc.widget.decoration.ImageDecoration");

// Summary:
//		A decorative Figure intended to be placed on a Polyline.
//		Note that this is not a widget.



dojo.declare(
	"gdc.widget.decoration.ImageDecoration",
	gdc.widget.decoration.AbstractDecoration,
{
	src: "",
	width: 100,
	height: 100,

	constructor: function(owner, props){
	},

	createTargetShape: function(parent){
		var shape = this.shape = parent.createImage({width:this.width, height:this.height, src:dojo.moduleUrl("hale", "../"+this.src)+""});
		return shape;
	},

	update: function(props){
		var shape = this.shape;
		if(!shape){ return; }
		shape.setShape({width:this.width, height:this.height, src:dojo.moduleUrl("gdc", "../"+this.src)+""});
	},

	setLocation: function(/*{x:#,y:#}*/pt){
		this.shape.setTransform({dx:pt.x-16, dy:pt.y-19});
	},

	setRotation: function(/*float*/angle){
//		var rotate = dojox.gfx.matrix.rotategAt(angle, 0, 0);
//		this.shape.applyTransform(rotate);
	}
});

}

if(!dojo._hasResource["gdc.widget.decoration.PolylineDecoration"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.decoration.PolylineDecoration"] = true;
dojo.provide("gdc.widget.decoration.PolylineDecoration");

// Summary:
//		A decorative Figure intended to be placed on a Polyline.
//		Note that this is not a widget.



dojo.declare(
	"gdc.widget.decoration.PolylineDecoration",
	gdc.widget.decoration.AbstractDecoration,
{
	points: "", // ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"

	// default line styles (see also dojox.gfx.defaultStroke)
	lineColor: "black",
	lineStyle: "solid",
	lineWidth: 1, // Integer
	lineCap: "butt",
	lineJoin: 4, // Integer

	fillColor: "",
	scale: 1, // Integer

	shape: null, // gfx Shape object
	groupBox: null, // gfx Group object that contains 'shape'.
	parentGroupBox: null, // dojox.gfx.Group to add this shape

	constructor: function(owner, props){
		if (!this.points){
			this.points = [];
		}else if (typeof(this.points) == "string"){
			// ex. "[{x: 250, y: 250}, {x: 300, y: 300}]"
			this.points = eval('('+this.points+')');
		}
		this.processPreset();
	},

	destroy: function(){
		this.shape.removeShape();
	},

	// TODO: those pre-defined samples should be outside of this module
	processPreset: function(){
		if(!this.preset){ return; }
		if(this.preset == "white_arrow"){
			this.points = [{x: 0, y: 0}, {x: -10, y: -5}, {x: -10, y: 5}, {x: 0, y: 0}];
			this.fillColor = "white";
		}else if(this.preset == "black_arrow"){
			this.points = [{x: 0, y: 0}, {x: -10, y: -5}, {x: -10, y: 5}, {x: 0, y: 0}];
			this.fillColor = "black";
		}else if(this.preset == "arrow"){
			this.points = [{x: -10, y: -5}, {x: 0, y: 0}, {x: -10, y: 5}];
			this.fillColor = ""; // no fill
		}else if(this.preset == "white_diamond"){
			this.points = [{x: 0, y: 0}, {x: -8, y: 5}, {x: -16, y: 0}, {x: -8, y: -5}, {x: 0, y: 0}];
			this.fillColor = "white";
		}else if(this.preset == "black_diamond"){
			this.points = [{x: 0, y: 0}, {x: -8, y: 5}, {x: -16, y: 0}, {x: -8, y: -5}, {x: 0, y: 0}];
			this.fillColor = "black";
		}
	},

	setProps: function(props){
		var prev = this.preset;
		this.inherited("setProps", arguments);
		if(this.preset != prev){
			this.processPreset();
		}
	},

	createTargetShape: function(parent){
		this.parentGroupBox = parent;
		this.groupBox = parent.createGroup();
		var shape = this.shape = this.groupBox.createPolyline(this.points);

		// assign the owner of this decoration to the top level dom node
		// so that the owner widget can be found from dom node. (See btp._base.utils.getWidgetByElement)
		shape.rawNode.widget = this.owner; // [custom property]
		return shape;
	},

	update: function(props){
		var shape = this.shape;
		if(!shape){ return; }

		// setShape is not low cost especially on IE, so compare the points first to see if something has really changed.
		if(this._comparePoints(this.points, this.shape.shape.points) != 0){
			this.shape.setShape(this.points);
		}

		if((props && props.forceUpdate) || !this._cur
			|| this._cur.color != this.lineColor
			|| this._cur.style != this.lineStyle
			|| this._cur.width != this.lineWidth
			|| this._cur.cap != this.lineCap
			|| this._cur.join != this.lineJoin){
			shape.setStroke({
				color: this.lineColor,
				style: this.lineStyle,
				width: this.lineWidth,
				cap: this.lineCap,
				join: this.lineJoin
			});
		}

		if((props && props.forceUpdate) || !this._cur || this._cur.fillColor != this.fillColor){
			if(this.fillColor){
				shape.setFill(new dojo.Color(this.fillColor));
			}else{
				shape.setFill(null);
			}
		}

		// save the current status
		this._cur = {
			color: this.lineColor,
			style: this.lineStyle,
			width: this.lineWidth,
			cap: this.lineCap,
			join: this.lineJoin,
			fillColor: this.fillColor
		};
	},

	setLocation: function(/*{x:#,y:#}*/pt){
		this.shape.setTransform({dx:pt.x, dy:pt.y});
		if(!this._cur && this.scale != 1 || this._cur.scale != this.scale){
			var matrix = [dojox.gfx.matrix.scaleAt(this.scale, this.scale, pt.x, pt.y)];
			this.groupBox.setTransform(matrix);
		}
		this._cur.scale = this.scale;
	},

	setRotation: function(/*float*/angle){
		var rotate = dojox.gfx.matrix.rotategAt(angle, 0, 0);
		this.shape.applyTransform(rotate);
	},

	_comparePoints: function(points0, points1){
		if(!points0 || !points1 || points0.length != points1.length){ return -1; }
		for(var i = 0, len = points0.length; i < len; i++){
			if(points0[i].x != points1[i].x || points0[i].y != points1[i].y){
				return -1;
			}
		}
		return 0;
	}
});

}

if(!dojo._hasResource["gdc.widget.Ellipse"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.Ellipse"] = true;
dojo.provide("gdc.widget.Ellipse");




dojo.declare(
	"gdc.widget.Ellipse",
	[gdc.widget.AbstractNode, gdc.widget.GfxContainer],
{
	backgroundColor: 'white', // default value
	borderColor: 'black', // default value
	
	createTargetShape: function(parent){
		var shape = parent.createEllipse({
			cx:this.getWidth() / 2,
			cy:this.getHeight() / 2,
			rx:this.getWidth() / 2,
			ry:this.getHeight() / 2				
		});
		this.createConnectionAnchors();
		return shape;
	},

	// Override gdc.widget.AbstractNode
	getShapeInfo: function(){
		return { type:'ellipse',
			cx:this.getWidth()/2, cy:this.getHeight()/2,
			rx:this.getWidth()/2, ry:this.getHeight()/2
		};
	},
	
	update: function(){
		this.inherited("update", arguments);		
		this.setBgColor(this.backgroundColor);
		this.setBorderColor(this.borderColor);
	},

	getBgColor: function(){
		return this.backgroundColor;
	},
	
	setBgColor: function(color){
		this.backgroundColor = color;
		this.shape.setFill(new dojo.Color(this.backgroundColor)); // incorrect
	},
	
	getBorderColor: function(){
		return this.borderColor;
	},
	
	setBorderColor: function(color){
		this.borderColor = color;
		this.shape.setStroke(this.borderColor);
	}
});

}

if(!dojo._hasResource["gdc.widget.Image"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.Image"] = true;
dojo.provide("gdc.widget.Image");



dojo.declare(
	"gdc.widget.Image",
	[gdc.widget.AbstractNode, gdc.widget.GfxContainer],
{
	src: '',
	
	postCreate: function(){
		this.inherited("postCreate", arguments);
		if(dojo.isIE) {
			this.domNode.style.backgroundColor = "#FFF";
			this.domNode.style.filter = "Alpha(opacity=0)";
		}
	},
	
	createTargetShape: function(parent){
		var shape = parent.createImage({width:this.getWidth(), height:this.getHeight(), src:dojo.moduleUrl("hale", "../"+this.src)+""});
		return shape;
	},
	
	// Override gdc.widget.AbstractNode
	getShapeInfo: function(){
		return { type:"image",
			width:this.getWidth(), height:this.getHeight(),
			src:dojo.moduleUrl("hale", "../"+this.src)+""
		}
	},
	
	getSrc: function(){
		return this.src;
	},

	setSrc: function(src){
		this.src = src;
		this.update();
	},
	
	// TODO : need to transform
	setWidth: function(width){
		var temp = parseInt(width);
		if(isNaN(temp)){ return; }
		this.width = temp;
		this.update();
	},
	
	// TODO : need to transform
	setHeight: function(height){
		var temp = parseInt(height);
		if(isNaN(temp)){ return; }
		this.height = temp;
		this.update();
	}	
});

}

if(!dojo._hasResource["gdc.widget.Polyline"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.Polyline"] = true;
dojo.provide("gdc.widget.Polyline");



dojo.declare(
	"gdc.widget.Polyline",
	gdc.widget.AbstractLine,
{
	createTargetShape: function(parent){
		this.onPointsChanged();
		var shape = parent.createPolyline(this.points);
		this.inherited("createTargetShape", arguments);
		return shape;
	},

	update: function(props){
		if(!this.shape || this.points.length < 2){ return; }
		this.inherited("update", arguments);

		// setShape is not low cost especially on IE, so compare the points first to see if something has really changed.
		if(this._comparePoints(this.points, this.shape.shape.points) != 0){
			this.onPointsChanged();
			this.shape.setShape(this.points);
		}
	},

	calculateAngle: function(pt0, pt1){
		var dx = Math.abs(pt1.x - pt0.x);
		var dy = pt1.y - pt0.y;
		if(dx == 0 && dy == 0){ return 0; }
		var angle = Math.atan(dy/dx)/Math.PI*180;
		if(pt1.x - pt0.x < 0){
			angle = 180 - angle;
		}
		return angle;
	}
});

}

if(!dojo._hasResource["gdc.widget.PolylineConnection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.PolylineConnection"] = true;
dojo.provide("gdc.widget.PolylineConnection");




dojo.declare(
	"gdc.widget.PolylineConnection",
	gdc.widget.Polyline,
{
	startup: function(){
		this.processParams();
	}
});
dojo.extend(gdc.widget.PolylineConnection, gdc.widget.support.connection);

}

if(!dojo._hasResource["gdc.widget.Rect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.Rect"] = true;
dojo.provide("gdc.widget.Rect");




dojo.declare(
	"gdc.widget.Rect",
	[gdc.widget.AbstractNode, gdc.widget.GfxContainer],
{
	backgroundColor: 'white', // default value
	borderColor: 'black', // default value

	createTargetShape: function(parent){
		var shape = parent.createRect({width:this.getWidth(), height:this.getHeight()});
		this.createConnectionAnchors();
		return shape;
	},

	// Override gdc.widget.AbstractNode
	getShapeInfo: function(){
		return { type:'rect',
			width:this.getWidth(), height:this.getHeight()
		};
	},

	// Override gdc.widget.AbstractNode
	update: function(){
		this.inherited("update", arguments);
		this.setBackgroundColor(this.backgroundColor);
		this.setBorderColor(this.borderColor);
	},

// ======================= Setters/Getters =================================
	getBackgroundColor: function(){
		return this.backgroundColor;
	},
	
	setBackgroundColor: function(color){
		if(this._backgroundColorDone && this.backgroundColor == color){ return; }
		this.backgroundColor = color;
		if(!this.shape){ return; }
		this.shape.setFill(new dojo.Color(this.backgroundColor));
		this._backgroundColorDone = true;
	},
	
	getBorderColor: function(){
		return this.borderColor;
	},
	
	setBorderColor: function(color){
		if(this._borderColorDone && this.borderColor == color){ return; }
		this.borderColor = color;
		if(!this.shape){ return; }
		this.shape.setStroke(this.borderColor);
		this._borderColorDone = true;
	}
});

}

if(!dojo._hasResource["gdc.widget.RoundRect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.RoundRect"] = true;
dojo.provide("gdc.widget.RoundRect");



dojo.declare(
	"gdc.widget.RoundRect",
	gdc.widget.Rect,
{
	r: 20,

	// Override gdc.widget.AbstractNode
	getShapeInfo: function(){
		return { type:'rect',
			width:this.getWidth(), height:this.getHeight(),
			r:this.getR()
		};
	},
	
	getR: function(){
		return this.r;
	},
	
	setR: function(r){
		this.r = r;
		this.update();
	}
});

}

if(!dojo._hasResource["gdc.widget.router.AbstractConnectionRouter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.router.AbstractConnectionRouter"] = true;
dojo.provide("gdc.widget.router.AbstractConnectionRouter");

dojo.declare(
	"gdc.widget.router.AbstractConnectionRouter",
	null,
{
	constructor: function(/*ConnectionWidget*/ owner){
		this.owner = owner;
		this._connects = [
			dojo.connect(owner, "onPointsChanged", this, "update")
		];
	},

	destroy: function(){
		dojo.forEach(this._connects, dojo.disconnect);
	},
	
	update: function(){
		// subclass may implement
	}
});

}

if(!dojo._hasResource["gdc.widget.router.ManhattanConnectionRouter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.router.ManhattanConnectionRouter"] = true;
dojo.provide("gdc.widget.router.ManhattanConnectionRouter");



// summary:
//		Update points for a router.
//		'this' points to a gfx line widget.

dojo.declare(
	"gdc.widget.router.ManhattanConnectionRouter",
	gdc.widget.router.AbstractConnectionRouter,
{
	delta: 20,
		
	_getPoints: function(){
		var pt0 = this.owner.points[0];
		var pt = this.owner.points[this.owner.points.length-1];
		var x1 = pt0.x;
		var y1 = pt0.y;
		var startdir = pt0.dir;
		var x2 = pt.x;
		var y2 = pt.y;
		var enddir = pt.dir;		
		var delta = this.delta;

		// top-bottom combination
		var points = [];
		if((startdir=="top" && enddir=="bottom" && y1>=y2) ||
			(startdir=="bottom" && enddir=="top" && y1<y2)){
			points = [{x:x1,y:(y1+y2)/2},{x:x2,y:(y1+y2)/2}];
			
		}else if((startdir=="top" && enddir=="bottom" && y1<y2)){
			points = [{x:x1,y:(y1-delta)},{x:(x1+x2)/2,y:(y1-delta)},{x:(x1+x2)/2,y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else if((startdir=="bottom" && enddir=="top" && y1>=y2)) {
			points = [{x:x1,y:(y1+delta)},{x:(x1+x2)/2,y:(y1+delta)},{x:(x1+x2)/2,y:(y2-delta)},{x:x2,y:(y2-delta)}];

		// left-right combination
		}else if((startdir=="left" && enddir=="right" && x1<x2)){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y1+y2)/2},{x:(x2+delta),y:(y1+y2)/2},{x:(x2+delta),y:y2}];
		}else if((startdir=="right" && enddir=="left" && x1>=x2)){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y1+y2)/2},{x:(x2-delta),y:(y1+y2)/2},{x:(x2-delta),y:y2}];

		// left-top|bottom, right-top|bottom combination
		}else if(
			(startdir=="left" && enddir=="top" && x1>=x2 && y1<y2) ||
			(startdir=="left" && enddir=="bottom" && x1>=x2 && y1>=y2) ||
			(startdir=="right" && enddir=="top" && x1<x2 && y1<y2) ||
			(startdir=="right" && enddir=="bottom" && x1<x2 && y1>=y2)
			){
			points = [{x:x2,y:y1}];
		}else if(
			(startdir=="left" && enddir=="top" && x1<x2 && y1<y2) ||
			(startdir=="left" && enddir=="bottom" && x1<x2 && y1>=y2)
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y1+y2)/2},{x:x2,y:(y1+y2)/2}];
		}else if(
			(startdir=="right" && enddir=="top" && x1>=x2 && y1<y2) ||
			(startdir=="right" && enddir=="bottom" && x1>=x2 && y1>=y2)
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y1+y2)/2},{x:x2,y:(y1+y2)/2}];
		}else if(
			(startdir=="left" && enddir=="top" && x1>=x2 && y1>=y2) ||
			(startdir=="right" && enddir=="top" && x1<x2 && y1>=y2)
			){
			points = [{x:(x1+x2)/2,y:y1},{x:(x1+x2)/2,y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="left" && enddir=="bottom" && x1>=x2 && y1<y2) ||
			(startdir=="right" && enddir=="bottom" && x1<x2 && y1<y2)
			){
			points = [{x:(x1+x2)/2,y:y1},{x:(x1+x2)/2,y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else if(
			(startdir=="left" && enddir=="top" && x1<x2 && y1>=y2)
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="left" && enddir=="bottom" && x1<x2 && y1<y2)
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else if(
			(startdir=="right" && enddir=="top" && x1>=x2 && y1>=y2)
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="right" && enddir=="bottom" && x1>=x2 && y1<y2)
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:(y2+delta)},{x:x2,y:(y2+delta)}];

		// top-left|right, bottom-left|right combination
		}else if(
			(startdir=="top" && enddir=="left" && x1<x2 && y1>=y2) ||
			(startdir=="top" && enddir=="right" && x1>=x2 && y1>=y2) ||
			(startdir=="bottom" && enddir=="left" && x1<x2 && y1<y2) ||
			(startdir=="bottom" && enddir=="right" && x1>=x2 && y1<y2)
			){
			points = [{x:x1,y:y2}];
		}else if(
			(startdir=="top" && enddir=="left" && x1<x2 && y1<y2) ||
			(startdir=="top" && enddir=="right" && x1>=x2 && y1<y2)
			){
			points = [{x:x1,y:(y1-delta)},{x:(x1+x2)/2,y:(y1-delta)},{x:(x1+x2)/2,y:y2}];
		}else if(
			(startdir=="bottom" && enddir=="left" && x1<x2 && y1>=y2) ||
			(startdir=="bottom" && enddir=="right" && x1>=x2 && y1>=y2)
			){
			points = [{x:x1,y:(y1+delta)},{x:(x1+x2)/2,y:(y1+delta)},{x:(x1+x2)/2,y:y2}];
		}else if(
			(startdir=="top" && enddir=="left" && x1>=x2 && y1>=y2) ||
			(startdir=="bottom" && enddir=="left" && x1>=x2 && y1<y2)
			){
			points = [{x:x1,y:(y1+y2)/2},{x:(x2-delta),y:(y1+y2)/2},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="right" && x1<x2 && y1>=y2) ||
			(startdir=="bottom" && enddir=="right" && x1<x2 && y1<y2)
			){
			points = [{x:x1,y:(y1+y2)/2},{x:(x2+delta),y:(y1+y2)/2},{x:(x2+delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="left" && x1>=x2 && y1<y2)
			){
			points = [{x:x1,y:(y1-delta)},{x:(x2-delta),y:(y1-delta)},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="right" && x1<x2 && y1<y2)
			){
			points = [{x:x1,y:(y1-delta)},{x:(x2+delta),y:(y1-delta)},{x:(x2+delta),y:y2}];
		}else if(
			(startdir=="bottom" && enddir=="left" && x1>=x2 && y1>=y2)
			){
			points = [{x:x1,y:(y1+delta)},{x:(x2-delta),y:(y1+delta)},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="bottom" && enddir=="right" && x1<x2 && y1>=y2)
			){
			points = [{x:x1,y:(y1+delta)},{x:(x2+delta),y:(y1+delta)},{x:(x2+delta),y:y2}];

		// same direction
		}else if(
			(startdir=="left" && enddir=="left" && x1>=x2)
			){
			points = [{x:(x2-delta),y:y1},{x:(x2-delta),y:y2}];
		}else if(
			(startdir=="left" && enddir=="left" && x1<x2)
			){
			points = [{x:(x1-delta),y:y1},{x:(x1-delta),y:y2}];
		}else if(
			(startdir=="right" && enddir=="right" && x1>=x2)
			){
			points = [{x:(x1+delta),y:y1},{x:(x1+delta),y:y2}];
		}else if(
			(startdir=="right" && enddir=="right" && x1<x2)
			){
			points = [{x:(x2+delta),y:y1},{x:(x2+delta),y:y2}];
		}else if(
			(startdir=="top" && enddir=="top" && y1>=y2)
			){
			points = [{x:x1,y:(y2-delta)},{x:x2,y:(y2-delta)}];
		}else if(
			(startdir=="top" && enddir=="top" && y1<y2)
			){
			points = [{x:x1,y:(y1-delta)},{x:x2,y:(y1-delta)}];
		}else if(
			(startdir=="bottom" && enddir=="bottom" && y1>=y2)
			){
			points = [{x:x1,y:(y1+delta)},{x:x2,y:(y1+delta)}];
		}else if(
			(startdir=="bottom" && enddir=="bottom" && y1<y2)
			){
			points = [{x:x1,y:(y2+delta)},{x:x2,y:(y2+delta)}];
		}else{ // includes left-right, right-left
			points = [{x:(x1+x2)/2,y:y1},{x:(x1+x2)/2,y:y2}];
		}		
		return points;
	},
	
	update: function(){
		var len = this.owner.points.length;
		if(len<2){ return; }
		this.owner.points.splice(1, len-2);
		var points = this._getPoints();
		var i = 1;
		dojo.forEach(points, dojo.hitch(this, function(pt){
			this.owner.points.splice(i, 0, pt);
			i++;
		}));
	}
});

}

if(!dojo._hasResource["gdc.widget.samples.ConnectableRect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.samples.ConnectableRect"] = true;
dojo.provide("gdc.widget.samples.ConnectableRect");

// summary:
//		A sample gfx node widget that has a ChopboxAnchor.




dojo.declare(
	"gdc.widget.samples.ConnectableRect",
	gdc.widget.Rect,
{
	anchor: "gdc.widget.anchor.ChopboxAnchor",

	createTargetShape: function(parent){
		var shape = this.inherited("createTargetShape", arguments);
		this.createConnectionAnchors();
		return shape;
	},

	createConnectionAnchors: function(){
		dojo["require"](this.anchor);
		var anchorImpl = dojo.getObject(this.anchor);
		var anchorName = "base";
		this.registerSourceConnectionAnchor(this.getShape().rawNode, anchorName, new anchorImpl(this));
		this.registerTargetConnectionAnchor(this.getShape().rawNode, anchorName, new anchorImpl(this));
	}
});
dojo.extend(gdc.widget.samples.ConnectableRect, gdc.widget.support.connectable);

}

if(!dojo._hasResource["gdc.widget.samples.CustomNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.samples.CustomNode"] = true;
dojo.provide("gdc.widget.samples.CustomNode");

// summary:
//		A sample gfx node widget that has an input anchor and an output anchor.





dojo.declare(
	"gdc.widget.samples.CustomNode",
	[gdc.widget.AbstractNode, gdc.widget.GfxContainer],
{
	backgroundColor: 'white', // default value
	borderColor: 'black', // default value
	anchor: "gdc.widget.anchor.FixedAnchor",

	inputAnchorShape: null,
	outputAnchorShape: null,

	createTargetShape: function(parent){
		var shape = parent.createRect({width:this.getWidth(), height:this.getHeight()});

		// input anchor shape
		this.inputAnchorShape = this.groupBox.createRect({width:10, height:10});
		this.inputAnchorShape.setTransform({ dx:5, dy:18 });
		this.inputAnchorShape.rawNode.widget = this; // [custom property]

		// output anchor shape
		this.outputAnchorShape = this.groupBox.createRect({width:10, height:10});
		this.outputAnchorShape.setTransform({ dx:85, dy:18 });
		this.outputAnchorShape.rawNode.widget = this; // [custom property]

		this.createConnectionAnchors();
		return shape;
	},

	createConnectionAnchors: function(){
		dojo["require"](this.anchor);
		var anchorImpl = dojo.getObject(this.anchor);

		// input anchor
		var anchor1 = new anchorImpl(this);
		anchor1.endPointShape = this.inputAnchorShape;
		this.registerTargetConnectionAnchor(this.getShape().rawNode, "input", anchor1);

		// output anchor
		var anchor2 = new anchorImpl(this);
		anchor2.endPointShape = this.outputAnchorShape;
		this.registerSourceConnectionAnchor(this.getShape().rawNode, "output", anchor2);
	},

	// Override gdc.widget.AbstractNode
	getShapeInfo: function(){
		return { type:'rect',
			width:this.getWidth(), height:this.getHeight()
		};
	},

	// Override gdc.widget.AbstractNode
	update: function(){
		this.inherited("update", arguments);
		this.setBackgroundColor(this.backgroundColor);
		this.setBorderColor(this.borderColor);
	},

// ======================= Setters/Getters =================================
	getBackgroundColor: function(){
		return this.backgroundColor;
	},
	
	setBackgroundColor: function(color){
		this.backgroundColor = color;
		if(!this.shape){ return; }
		this.shape.setFill(new dojo.Color(this.backgroundColor));
		this.inputAnchorShape.setFill(new dojo.Color("blue"));
		this.outputAnchorShape.setFill(new dojo.Color("red"));
	},
	
	getBorderColor: function(){
		return this.borderColor;
	},
	
	setBorderColor: function(color){
		this.borderColor = color;
		if(!this.shape){ return; }
		this.shape.setStroke(this.borderColor);
	}
});
dojo.extend(gdc.widget.samples.CustomNode, gdc.widget.support.connectable);

}

if(!dojo._hasResource["gdc.widget.samples.PlugNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.samples.PlugNode"] = true;
dojo.provide("gdc.widget.samples.PlugNode");





dojo.declare(
"gdc.widget.samples.PlugNode",
gdc.widget.AbstractNode,
{
	backgroundColor: 'white', // default value
	borderColor: 'black', // default value
	anchor: "gdc.widget.anchor.FixedAnchor",
	
	inputAnchorShape: null,
	outputAnchorShape: null,
	
	leftImage: null,
	mainImage: null,
	plugImage: null,
	labelText: null,
	
	leftImageSrc: "../gdc/widget/templates/PlugNode/outlet-empty-blue.png",
	leftImageAltSrc: "../gdc/widget/templates/PlugNode/outlet-empty-green.png",
	leftImageW: 31,
	leftImageH: 38,
	
	mainImageSrc: "../gdc/widget/templates/PlugNode/body-blue.png",
	mainImageAltSrc: "../gdc/widget/templates/PlugNode/body-green.png",
	mainImageW: 86,
	mainImageH: 38,
	
	plugImageSrc: "../gdc/widget/templates/PlugNode/plug.png",
	plugImageW: 27,
	plugImageH: 38,
	
	backgroundColor: "yellow",
	anchor: "gdc.widget.anchor.FixedAnchor",
	
	width: 144,
	height: 38,
	
	selected: false,
	
	postCreate: function(){
		this.inherited("postCreate", arguments);
		
		dojo.subscribe("btp:SelectionChanged", this, "selectionChanged");
	},
	
	createTargetShape: function(parent){
		var shape = this.shape = parent.createGroup();
		
		var leftImage = this.leftImage = shape.createGdcImage({width:this.leftImageW, height:this.leftImageH, src:dojo.moduleUrl("gdc", this.leftImageAltSrc)+""});
		leftImage.setTransform({dx: 0});
		leftImage.rawNode.widget = this;
		
		var mainImage = this.mainImage = shape.createGdcImage({width:this.mainImageW, height:this.mainImageH, src:dojo.moduleUrl("gdc", this.mainImageAltSrc)+""});
		mainImage.setTransform({dx:31});
		
		var plugImage = this.plugImage = shape.createGdcImage({width:this.plugImageW, height:this.plugImageH, src:dojo.moduleUrl("gdc", this.plugImageSrc)+""});
		plugImage.setTransform({dx:117});
		plugImage.rawNode.widget = this;
		plugImage.rawNode.getDragTracker = function() {
			return "gdl.widget.tool.ConnectionDragCreationTool";
		}
		plugImage.rawNode.getConnectionClass = function(){
			return "gdc.widget.CurvylineConnection";
		};		
		
		var labelText = this.labelText = shape.createGdcText({width: 64, height:17, text: "Sample long text"});
		labelText.setFill(new dojo.Color("#FFFFFF"));
		labelText.setTransform({dx: 46, dy: 26});
		labelText.setFont({size: "10pt", family: "Arial,Verdana,san-serif"});
		
		this.createConnectionAnchors();
		return shape;
	},
	
	getShapeInfo: function(){
		return { type:'group',
			width:this.getWidth(), height:this.getHeight()
		};
	},
	
	update: function(){
		this.inherited("update", arguments);
		this.setBackgroundColor(this.backgroundColor);
		this.setBorderColor(this.borderColor);
	},
	
	setBackgroundColor: function(color){
		this.backgroundColor = color;
		if(!this.shape){ return; }
		this.shape.setFill(new dojo.Color(this.backgroundColor));
//		this.inputAnchorShape.setFill(new dojo.Color("red"));
//		this.outputAnchorShape.setFill(new dojo.Color("blue"));
	},
	
	setBorderColor: function(color){
		if(this._borderColorDone && this.borderColor == color){ return; }
		this._borderColorDone = true;
		this.borderColor = color;
		if(!this.shape){ return; }
		this.shape.setStroke(this.borderColor);
	},
	
	_onMouseOver: function(){
//		this.leftImage.
	},
	
	createConnectionAnchors: function(){
		dojo["require"](this.anchor);
		var anchorImpl = dojo.getObject(this.anchor);

		// input anchor
		var anchor1 = new anchorImpl(this);
		anchor1.endPointShape = this.mainImage;
		anchor1.centerOrigin = false;
		anchor1.offsetH = 86;
		anchor1.offsetV = 19;
		this.registerSourceConnectionAnchor(this.getShape().rawNode, "input", anchor1);

		// output anchor
		var anchor2 = new anchorImpl(this);
		anchor2.endPointShape = this.leftImage;
		anchor2.centerOrigin = false;
		anchor2.offsetH = 20;
		anchor2.offsetV = 21;
		this.registerTargetConnectionAnchor(this.getShape().rawNode, "output", anchor2);
	},
	
	selectionChanged: function(message){
		if(message.selection == this) {
			this.selected = true;
			this.leftImage.setShape({src: dojo.moduleUrl("gdc", this.leftImageAltSrc)+""});
			this.mainImage.setShape({src: dojo.moduleUrl("gdc", this.mainImageAltSrc)+""});
		} else {
			if(this.selected) {
				this.selected = false;
				this.leftImage.setShape({src: dojo.moduleUrl("gdc", this.leftImageSrc)+""});
				this.mainImage.setShape({src: dojo.moduleUrl("gdc", this.mainImageSrc)+""});
			}
		}
	},
	
	// Example:
	//  accept connections if the amount of input connections <= 2 (allows 3 connections) 
//	isTargetConnectionAcceptable: function(sourceNode, sourceAnchor) {
//		if(sourceNode.declaredClass=="gdc.widget.samples.PlugNode") return true;
//		else return false;
//	},
	
	onTargetConnectionAdded: function(connection) {
		console.debug(this.id +": connected from "+ connection.getSource().id);
	},
	
	onRemovingSourceConnection: function(connection) {
		console.debug(this.id+": disconnected from "+ connection.getTarget().id);
	},
	
	openContextMenu: function(){
		alert("dummy context menu: "+ this.id);	
	}
	
});
dojo.extend(gdc.widget.samples.PlugNode, gdc.widget.support.connectable);

}

if(!dojo._hasResource["gdc.widget.samples.SampleNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.samples.SampleNode"] = true;
dojo.provide("gdc.widget.samples.SampleNode");

// summary:
//		A sample gfx node widget that has an output anchor and an input anchor.





dojo.declare(
	"gdc.widget.samples.SampleNode",
	[gdc.widget.AbstractNode, gdc.widget.GfxContainer],
{
	backgroundColor: 'white', // default value
	borderColor: 'black', // default value
	anchor: "gdc.widget.anchor.FixedAnchor",

	inputAnchorShape: null,
	outputAnchorShape: null,

	createTargetShape: function(parent){
		var shape = parent.createRect({width:this.getWidth(), height:this.getHeight()});

		// input anchor shape
		this.inputAnchorShape = this.groupBox.createRect({width:10, height:10});
		this.inputAnchorShape.setTransform({ dx:5, dy:18 });
		this.inputAnchorShape.rawNode.widget = this; // [custom property]

		// output anchor shape
		this.outputAnchorShape = this.groupBox.createRect({width:10, height:10});
		this.outputAnchorShape.setTransform({ dx:100, dy:18 });
		this.outputAnchorShape.rawNode.widget = this; // [custom property]
		this.outputAnchorShape.rawNode.getDragTracker = function(){
			return "gdl.widget.tool.ConnectionDragCreationTool";
		};
		this.outputAnchorShape.rawNode.getConnectionClass = function(){
			return "gdc.widget.CurvylineConnection";
		};

		this.createConnectionAnchors();
		return shape;
	},

	createConnectionAnchors: function(){
		dojo["require"](this.anchor);
		var anchorImpl = dojo.getObject(this.anchor);

		// input anchor
		var anchor1 = new anchorImpl(this);
		anchor1.endPointShape = this.inputAnchorShape;
		this.registerTargetConnectionAnchor(this.getShape().rawNode, "input", anchor1);

		// output anchor
		var anchor2 = new anchorImpl(this);
		anchor2.endPointShape = this.outputAnchorShape;
		this.registerSourceConnectionAnchor(this.getShape().rawNode, "output", anchor2);
	},

	// Override gdc.widget.AbstractNode
	getShapeInfo: function(){
		return { type:'rect',
			width:this.getWidth(), height:this.getHeight()
		};
	},

	// Override gdc.widget.AbstractNode
	update: function(){
		this.inherited("update", arguments);
		this.setBackgroundColor(this.backgroundColor);
		this.setBorderColor(this.borderColor);
	},

// ======================= Setters/Getters =================================
	getBackgroundColor: function(){
		return this.backgroundColor;
	},
	
	setBackgroundColor: function(color){
		this.backgroundColor = color;
		if(!this.shape){ return; }
		this.shape.setFill(new dojo.Color(this.backgroundColor));
		this.inputAnchorShape.setFill(new dojo.Color("blue"));
		this.outputAnchorShape.setFill(new dojo.Color("red"));
	},
	
	getBorderColor: function(){
		return this.borderColor;
	},
	
	setBorderColor: function(color){
		this.borderColor = color;
		if(!this.shape){ return; }
		this.shape.setStroke(this.borderColor);
	},
	
	isTargetConnectionAcceptable: function(sourceNode, sourceAnchor) {
		if(sourceNode.declaredClass=="gdc.widget.samples.PlugNode") return true;
		else return false;
	}
	
	// respond attach/detach example
//	onTargetConnectionAdded: function(connection) {
//		alert(this.id +": connected from "+ connection.getSource().id);
//	},
//	onRemovingSourceConnection: function(connection) {
//		alert(this.id+": disconnected from "+ connection.getTarget().id);
//	}
});
dojo.extend(gdc.widget.samples.SampleNode, gdc.widget.support.connectable);

}

if(!dojo._hasResource["gdc.widget._GfxTemplated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget._GfxTemplated"] = true;
dojo.provide("gdc.widget._GfxTemplated");




/*
 * Creates a gfx shape from a template SVG file
 * Connection anchors can be added by adding attribute named "gdcAnchor" in an element in SVG
 * if gdcAnchor="source", the corresponding shape becomes source anchor for connection
 * if gdcAnchor="target", the corresponding shape becomes target anchor for connection
 * if gdcAnchor="both", the corresponding shape becomes source and target anchor for connection
 */
dojo.declare(
	"gdc.widget._GfxTemplated",
	null,
{
	// templatePath: String
	//	Path to template (SVG file) for this widget
	templatePath: null,
	anchor: "gdc.widget.anchor.FixedAnchor",
	
	_tagName2Type: {
		"rect": "rect",
		"ellipse": "ellipse",
		"path": "path",
		"polyline": "polyline",
		"text": "text"
	},
	
	_sourceAnchors: [],
	_targetAnchors: [],
	
	createTargetShape: function gdc_widget__GfxTemplated_createTargetShape(parent){
		var txt = dojo._getText(this.templatePath);
		var doc = dojox.data.dom.createDocument(txt, "text/xml");
		var shape = this._createChildShapes(doc, parent);

		// if anchor is defined, create connection anchors
		if(this._sourceAnchors.length > 0 || this._targetAnchors.length > 0){
			this.createConnectionAnchors();
		}
		return shape;
	},
	
	_createChildShapes: function(node, parent){
		var shape = this.createShapeFromNode(node, parent);
		if(node.nodeType === 9 /* Document */){
			shape = parent;
		}
		var self = this;
		dojo.forEach(node.childNodes, function(child){
			self._createChildShapes(child, shape);
		});
		return shape;
	},
	
	createShapeFromNode: function(node, parent){
		var shape = null;
		if(parent && node.nodeType === 1 /*ELEMENT_NODE*/){
			// create properties
			var props = { type: this._tagName2Type[node.nodeName] };
			var attrs = node.attributes;
			var fill = {};
			var stroke = {};
			var matrix = null;
			var attachPointName = null;
			var isSourceAnchor = false;
			var isTargetAnchor = false;
			dojo.forEach(attrs, dojo.hitch(this, function(attr){
				if(attr.nodeType == 2 /*ATTRIBUTE_NODE*/ && (attr.nodeName.indexOf("dojo") != 0)){
					switch(attr.nodeName){
						case "fill":
							fill = attr.nodeValue;
							break;
						case "stroke":
							stroke = attr.nodeValue;
							break;
						case "transform":
							matrix = this._generateMatrix(attr.nodeValue);
							break;
						case "gdcAttachPoint": // custom attribute
							attachPointName = attr.nodeValue;
							break;
						case "gdcAnchor": // custom attribute
							isSourceAnchor = (attr.nodeValue == "source" || attr.nodeValue == "both");
							isTargetAnchor = (attr.nodeValue == "target" || attr.nodeValue == "both");
							break;
						case "x":
						case "y":
						case "width":
						case "height":
						case "cx":
						case "cy":
						case "rx":
						case "ry":
							props[attr.nodeName] = parseInt(attr.nodeValue);
							break;
						default:
							props[attr.nodeName] = attr.nodeValue;
					}
				}
			}));
			if(node.nodeName == "g"){
				shape = parent.createGroup().setTransform(matrix);
			}else{
				shape = parent.createShape(props).setFill(fill).setStroke(stroke).setTransform(matrix);
			}
			if(attachPointName){
				this[attachPointName] = shape;
			}
			if(isSourceAnchor){
				shape.rawNode.widget = this; // [custom property]
				this._sourceAnchors.push(shape);
			}
			if(isTargetAnchor){
				shape.rawNode.widget = this; // [custom property]
				this._targetAnchors.push(shape);
			}
		}
		return shape;
	},
	
	_generateMatrix: function(str){
		if(str){
			var trim = dojo.string.trim;
			str = trim(str);
			var prefix = "matrix(";
			var index1 = str.indexOf(prefix);
			var index2 = str.indexOf(")");
			var len = str.length;
			if(index1 == 0 && index2 == len-1){
				var len2 = prefix.length;
				var str2 = str.substring(len2, len-len2+6);
				var arr = str2.split(",");
				return {
					xx: parseInt(trim(arr[0])),
					xy: parseInt(trim(arr[1])),
					yx: parseInt(trim(arr[2])),
					yy: parseInt(trim(arr[3])),
					dx: parseInt(trim(arr[4])),
					dy: parseInt(trim(arr[5]))
				}
			}
		}
		return {xx: 1, xy: 0, yx: 0, yy: 1, dx: 0, dy: 0};
	},
	
	createConnectionAnchors: function(){
		dojo["require"](this.anchor);
		var anchorImpl = dojo.getObject(this.anchor);

		// input anchors
		dojo.forEach(this._targetAnchors, dojo.hitch(this, function(anchorShape){
			var anchor = new anchorImpl(this);
			anchor.endPointShape = anchorShape;
			this.registerTargetConnectionAnchor(this.getShape().rawNode, "input", anchor);
		}));

		// input anchors
		dojo.forEach(this._sourceAnchors, dojo.hitch(this, function(anchorShape){
			var anchor = new anchorImpl(this);
			anchor.endPointShape = anchorShape;
			this.registerSourceConnectionAnchor(this.getShape().rawNode, "output", anchor);
		}));
	}
});

}

if(!dojo._hasResource["gdc.widget.samples.SampleTemplatedNode"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.samples.SampleTemplatedNode"] = true;
dojo.provide("gdc.widget.samples.SampleTemplatedNode");






dojo.declare(
	"gdc.widget.samples.SampleTemplatedNode",
	[gdc.widget.AbstractNode, gdc.widget.GfxContainer, gdc.widget._GfxTemplated],
{
	templateString:"<g>\r\n\t<rect fill=\"rgb(128, 128, 128)\" fill-opacity=\"1\" fill-rule=\"evenodd\"\r\n\t\tstroke=\"rgb(0, 0, 255)\" stroke-opacity=\"1\" stroke-width=\"1\" stroke-linecap=\"butt\"\r\n\t\t stroke-linejoin=\"miter\" stroke-miterlimit=\"4\" stroke-dasharray=\"none\"\r\n\t\tx=\"0\" y=\"0\" width=\"100\" height=\"50\" ry=\"20\" rx=\"20\" gdcAttachPoint=\"targetRect\" />\r\n\t<ellipse fill=\"rgb(255, 255, 255)\" fill-opacity=\"1\" fill-rule=\"evenodd\"\r\n\t\tstroke=\"rgb(0, 128, 0)\" stroke-opacity=\"1\" stroke-width=\"1\" stroke-linecap=\"butt\"\r\n\t\tstroke-linejoin=\"miter\" stroke-miterlimit=\"4\" stroke-dasharray=\"none\"\r\n\t\tcx=\"6\" cy=\"6\" rx=\"6\" ry=\"6\" transform=\"matrix(1, 0, 0, 1, 45, -5)\"\r\n\t\tgdcAttachPoint=\"connectionPt1\" gdcAnchor=\"target\" />\r\n\t<ellipse fill=\"rgb(255, 255, 255)\" fill-opacity=\"1\" fill-rule=\"evenodd\"\r\n\t\tstroke=\"rgb(0, 128, 0)\" stroke-opacity=\"1\" stroke-width=\"1\" stroke-linecap=\"butt\"\r\n\t\tstroke-linejoin=\"miter\" stroke-miterlimit=\"4\" stroke-dasharray=\"none\"\r\n\t\tcx=\"6\" cy=\"6\" rx=\"6\" ry=\"6\" transform=\"matrix(1, 0, 0, 1, 45, 45)\"\r\n\t\tgdcAttachPoint=\"connectionPt2\" gdcAnchor=\"source\" />\r\n</g>\r\n",
	
	radius: 6,
	
	setShape: function(){
		var w = this.getWidth();
		var h = this.getHeight();
		this.targetRect.setShape({
			type:'rect',
			width: w,
			height: h
		});
		this.connectionPt1.setTransform({ dx: w/2-this.radius, dy: -this.radius });
		this.connectionPt2.setTransform({ dx: w/2-this.radius, dy: h-this.radius });
		return this.shape;
	}
	
});
dojo.extend(gdc.widget.samples.SampleTemplatedNode, gdc.widget.support.connectable);

}

if(!dojo._hasResource["gdc.widget.Surface"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.Surface"] = true;
dojo.provide("gdc.widget.Surface");






dojo.declare(
	"gdc.widget.Surface",
	[dijit._Widget, dijit._Contained, gdc.widget.GfxContainer],
{
	width: 800, // Integer
	height: 600, // Integer
	surface: null,
	groupBox: null, // GfxContainer uses this
	_started: true, // By doing this, startup() of dynamically added child will be called.

	postCreate: function(){
		this.domNode.className = "gdlSurface";
		this.domNode.style.position = "relative";
		this.domNode.style.width = this.width + "px";
		this.domNode.style.height = this.height + "px";
		this.domNode.style.backgroundColor = "Window";
		this.surface = dojox.gfx.createSurface(this.domNode, this.width, this.height);
		this.rootGroup = this.groupBox = this.surface.createGroup();
	},

	startup: function(){
		dojo.forEach(this.getChildren(), function(child){if(child.startup){child.startup();}});
	},

	getShape: function(){
		return this.groupBox;
	}
});

}

if(!dojo._hasResource["gdc.widget.Text"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.Text"] = true;
dojo.provide("gdc.widget.Text");



dojo.declare(
	"gdc.widget.Text",
	gdc.widget.AbstractNode,
{
	text: 'text', // default value
	color: 'black', // default value
	size: '18pt', // default value
	width: 100, // default value
	height: 20, // default value

	createTargetShape: function(parent){
		var shape = parent.createText({width:this.getWidth(), height:this.getHeight(), text: this.getText()});
		return shape;
	},

	// Override gdc.widget.AbstractNode
	getShapeInfo: function(){
		return { type:'text',
			y:this.getHeight(),
			text:this.getText()
		};
	},

	// Override gdc.widget.AbstractNode
	update: function(){
		this.inherited("update", arguments);
		this.setColor(this.color);
		this.setSize(this.size);
	},

	getText: function(){
		return this.text;
	},
	
	setText: function(text){
		this.text = text;
		this.update();
	},
	
	getColor: function(){
		return this.color;
	},
	
	setColor: function(color){
		this.color = color;
		this.shape.setFill(new dojo.Color(this.color));
	},
	
	getSize: function(){
		return this.size;
	},
	
	setSize: function(size){
		this.size = size;
		this.shape.setFont({ size:this.size });
	}
});

}

if(!dojo._hasResource["gdc.widget.util.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdc.widget.util.common"] = true;
/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * 
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */

dojo.provide("gdc.widget.util.common");

//dojo.require("ibm_btp.widget.util.style");

gdc.widget.util.common = {
	// see dojo.overElement
	isOverElement: function(posX, posY, targetNode){
		var dh=dojo.html;
		var targetPos=dh.getAbsolutePosition(targetNode,true,dh.boxSizing.BORDER_BOX);
		var left=targetPos.x;
		var top=targetPos.y;
		var w=0;
		var h=0;
		var group=targetNode.getElementsByTagName('group')[0];
		if(group){
			w=parseInt(group.style.width);
			h=parseInt(group.style.height);
			if(isNaN(w)){ w=0; }
			if(isNaN(h)){ h=0; }
		} else {
			var targetBox=dh.getBorderBox(targetNode);
			w=targetBox.width;
			h=targetBox.height;
		}
		var delta = 5;
		return (left-delta<=posX && posX<=left+w+delta && top-delta<=posY && posY<=top+h+delta);	//	boolean
	},
	
	// instead of dojo.hasClass, make it work so that it works on both HTML node & gfx rawNode
	hasClass: function(node, className){
		var cls = (node.className) ? node.className : node.getAttribute('class');
		if(cls && typeof(cls) == "string"){
			var arr = cls.split(' ');
			for(var i=arr.length-1; i>=0; i--){
				if(arr[i] == className){
					return true;
				}
			}
		}
		return false;
	},

	getNodeCenterPos: function(node){
		var pos = dojo._abs(node, true);
		var box = dojo.contentBox(node);
		return { x:(pos.x+box.w/2), y:(pos.y+box.h/2) };
	},
	
	getBorderPos: function(posX, posY, node){
 		if(node){
			var widget = dijit.byNode(node);
			var pos = dojo._abs(node, true);
			var l = pos.x;
			var t = pos.y;
	 		var box = dojo.contentBox(node);
			var w = box.w;				
			var h = box.h;				
			var delta1=-h/w*(posX-l)+(posY-t);
			var delta2=h/w*(posX-l)+(posY-t-h);
			if(delta1>=0 && delta2>=0){
				return { x:posX, y:(t+h), dir:"bottom" };
			}else if(delta1>=0 && delta2<0){
				return { x:l, y:posY, dir:"left" };
			}else if(delta1<0 && delta2>=0){
				return { x:(l+w), y:posY, dir:"right" };
			}else if(delta1<0 && delta2<0){
				return { x:posX, y:t, dir:"top" };
			}
		}
 		return { x:posX, y:posY, dir:"left" };
	},

	getBorderMiddlePos: function(posX, posY, node){
 		if(node){
			var widget = dijit.byNode(node);
	 		var dh=dojo.html;
	 		var getPos = dh.getAbsolutePosition;
	 		var BB = dh.boxSizing.BORDER_BOX;
	 		var getBox = dh.getBorderBox;
 			var pos=getPos(node,true,BB);
 			var l=pos.x, t=pos.y;
			var w = null;
			var h = null;
			if(widget){
				w = ibm_btp.widget.getStyleValue(widget, "width");
				h = ibm_btp.widget.getStyleValue(widget, "height");
			}else{
		 		var box = dh.getBorderBox(node);
				w = box.width;				
				h = box.height;				
			}
			var delta1=-h/w*(posX-l)+(posY-t);
			var delta2=h/w*(posX-l)+(posY-t-h);
			if(delta1>=0 && delta2>=0){
				return { x:(l+w/2), y:(t+h), dir:"bottom" };
			}else if(delta1>=0 && delta2<0){
				return { x:l, y:(t+h/2), dir:"left" };
			}else if(delta1<0 && delta2>=0){
				return { x:(l+w), y:(t+h/2), dir:"right" };
			}else if(delta1<0 && delta2<0){
				return { x:(l+w/2), y:t, dir:"top" };
			}
		}
 		return { x:posX, y:posY, dir:"left" };
	},	
	
	getSurfaceRelativeBoundingBox: function(shape){
		var bb = shape.getBoundingBox();
		var left = bb.x;
		var top = bb.y;
		var matrix = null;
		while(!(shape instanceof dojox.gfx.Surface)){
			if(shape instanceof dojox.gfx.Group){
				matrix = shape.getTransform();
				if(matrix){
					left += matrix.dx;
					top += matrix.dy;
				}
			}
			shape = shape.getParent();
		}
		return { x:left, y:top, width:bb.width, height:bb.height };
	},

	// Return nearest parent widget that has both widget1 and widget2 as ancestors
	// There must be a better way...
	getOffsetParent: function(widget1, widget2){
		// first pass through the widget tree
		var temp1 = widget1;
		var temp2 = widget2;
		while(temp1){
			temp2 = widget2;
			while(temp2){
				if(temp1 == temp2){
					return temp1;
				}
				temp2 = temp2.parent;  //TODO: parent -> getParent()?
			}
			temp1 = temp1.parent;  //TODO: parent -> getParent()?
		}
		
		// second pass through the widget tree
		temp1 = widget1;
		temp2 = widget2;
		while(temp2){
			temp1 = widget1;
			while(temp1){
				if(temp1 == temp2){
					return temp1;
				}
				temp1 = temp1.parent;  //TODO: parent -> getParent()?
			}
			temp2 = temp2.parent;  //TODO: parent -> getParent()?
		}
		
		return null;
	},
	
	// Returns connect position within the target widget. Priority is the followings:
	//	1 : getConnectPosition is implementation in a widget
	//	2 : nearest border (default)
	getConnectPosition: function(e, widget){
		if(widget && !widget.isEditor){
			if(widget.getConnectPosition){ // if getConnectPosition is implemented in the widget
				return widget.getConnectPosition(e);
			}else{ // default
				return gdc.widget.util.common.getBorderPos(e.pageX, e.pageY, widget.domNode);
			}
		}else{
			return { x:e.pageX, y:e.pageY, dir:'left' };
		}
	},
	
	// returns parentNode of the surface
	// event handlers like onmousemove, onmouseop is attached to this node
	// since they cannot be attached to surface.rawNode
	getSurfaceParentNode: function(surface){
		return surface.rawNode.parentNode.parentNode;
	}
}

}

if(!dojo._hasResource["gdl.widget.action.Actions"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.action.Actions"] = true;
dojo.provide("gdl.widget.action.Actions");

// summary:
//		EditArea actions that are usually invoked from toolbar, menubar, etc.
//		'this' points to the EditArea.

gdl.widget.action.Actions = {
	scale: function(percent){
		var matrix = {};
		if (percent < 99 || 101 < percent){
			matrix = [dojox.gfx.matrix.scale(percent/100)];
		}
        this.rootGroup.setTransform(matrix);
	}
};

}

if(!dojo._hasResource["gdl.widget.ext.connectionEndpoints"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.ext.connectionEndpoints"] = true;
dojo.provide("gdl.widget.ext.connectionEndpoints");

// summary:
//		Extension for creating draggable end points for a gfx line widget.
//		Note that this is not a widget.
//		this.owner points to the EditArea.
//		See also btp.widget.ext.selectionHandles




dojo.declare(
	"gdl.widget.ext.connectionEndpoints",
	null,
{
	owner: null,
	endpointKnobType: "blackSquare",

	// constants
	_SOURCE: 1,
	_TARGET: 2,

	// initializer
	constructor: function(owner){
		this.owner = owner;
		this._connectors = [];
		this._connectors.push(dojo.connect(owner, "setFocusFrame", this, "addConnectionEndpoints"));
		this._connectors.push(dojo.connect(owner, "removeFocusFrame", this, "removeConnectionEndpoints"));
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
	},

	/*    SOURCE           TARGET
	 *       0 ------------> 1
	 */
	addConnectionEndpoints: function(widget){
		if(!widget){
			widget = this.owner.focusWidgets[0];
		}
		if(!btp.isGfxLineWidget(widget)) { return; }
		var info = widget.hlp_getComponentInfo();
		if(!info.resizable){ return; }
		if(widget.__showingConnectionEndpoints){
			; // already showing
		}else if(!widget.__connectionEndpoints){
			this.createConnectionEndpoints(widget);
		}else{
			for(var i = 0, len = widget.__connectionEndpoints.length; i < len; i++){
				this.owner.domNode.appendChild(widget.__connectionEndpoints[i]);
			}
		}
		widget.__showingConnectionEndpoints = true;

		// SOURCE
		knob = widget.__connectionEndpoints[0];
		if(widget.sourceDecorationObj && widget.sourceDecorationObj.getDimensions){
			var dim = widget.sourceDecorationObj.getDimensions();
			knob.style.width = dim.width + "px";
			knob.style.height = dim.height + "px";
			knob.style.top = dim.top + "px";
			knob.style.left = dim.left + "px";
		}else{
			var pt0 = widget.points[0];
			knob.style.top = pt0.y + "px";
			knob.style.left = pt0.x + "px";
		}

		// TARGET
		knob = widget.__connectionEndpoints[1];
		var target = widget.getTarget();
		if(target){
			knob.dropTarget = target.domNode.dropTarget;
			if(target.targetConnectionAnchors){
				knob.anchorNames = [];
				for(var n in target.targetConnectionAnchors){
					knob.anchorNames.push(n);
				}
			}
		}else{
			if(knob.dropTarget){
				knob.dropTarget = undefined;
				knob.anchorNames = undefined;
			}
		}
		if(widget.targetDecorationObj && widget.targetDecorationObj.getDimensions){
			var dim = widget.targetDecorationObj.getDimensions();
			knob.style.width = dim.width + "px";
			knob.style.height = dim.height + "px";
			knob.style.top = dim.top + "px";
			knob.style.left = dim.left + "px";
		}else{
			var pt1 = widget.points[widget.points.length - 1];
			knob.style.top = pt1.y + "px";
			knob.style.left = pt1.x + "px";
		}
	},

	removeConnectionEndpoints: function(widget){
		if(!widget){
			widget = this.owner.focusWidgets[0];
		}
		if(!widget.__showingConnectionEndpoints){ return; }
		for(var i = 0, len = widget.__connectionEndpoints.length; i < len; i++){
			var e = widget.__connectionEndpoints[i];
			this.owner.domNode.removeChild(e);
			if(e.dropTarget){
				e.dropTarget = undefined;
				e.anchorNames = undefined;
			}
		}
		widget.__showingConnectionEndpoints = false;
	},

	createConnectionEndpoints: function(widget){
		var knobElem = this.owner.createKnob(this.endpointKnobType);

		var info = widget.hlp_getComponentInfo();
		var dirs = 0;
		if(info.resizable.directions){
			var arr = info.resizable.directions.split("|");
			for(var i = 0; i < arr.length; i++){
				if(arr[i] == "SOURCE"){
					dirs |= this._SOURCE;
				}else if(arr[i] == "TARGET"){
					dirs |= this._TARGET;
				}else{
					alert("unrecognized direction: "+arr[i]);
				}
			}
		}
		widget.__resizeDirections = dirs; // bitwise

		widget.__connectionEndpoints = [];
		var knob;
		var _this = this;
		for(var i = 0; i < 2; i++){
			knob = knobElem.cloneNode(true);
			knob.knobPosition = i; // [custom property]
			knob.getDragTracker = function(){
				_this.owner.setFocus(widget);
				return _this.owner.reconnectTrackerClass;
			};
//			if(!knob.style.cursor){
//				knob.style.cursor = "crosshair";
//			}
			widget.__connectionEndpoints.push(knob);
			this.owner.domNode.appendChild(knob);

			var ds = new btp.dnd.DragMoveSource(knob, "connection"); // @TODO should not hard-code "connection"
			ds.setDragTarget(this.owner.getMoveTarget());
			ds.onDragMove = function btp_dnd_DragMoveSource_onDragMove(e){
				var dx = e.pageX - ds.pageX;
				var dy = e.pageY - ds.pageY;
				ds.dragClone.style.top = ds.dragStartCssPosition.y + dy + ds.dragOffset.y + "px";
				ds.dragClone.style.left = ds.dragStartCssPosition.x + dx + ds.dragOffset.x + "px";
			};
//			ds.onDragMove = function btp_dnd_DragMoveSource_onDragMove(e){
//				var dx = e.pageX - ds.pageX;
//				var dy = e.pageY - ds.pageY;
//				ds.dragClone.style.top = ds.dragStartCssPosition.y + dy + ds.layerY - ds.domNode.offsetHeight + "px";
//				ds.dragClone.style.left = ds.dragStartCssPosition.x + dx + ds.layerX - ds.domNode.offsetWidth + "px";
//			};

			if(i == 0 && !(dirs & this._SOURCE)){
				knob.style.display = "none";
			}else if(i == 1 && !(dirs & this._TARGET)){
				knob.style.display = "none";
			}
		}
	}
});

}

if(!dojo._hasResource["gdl.widget.helper.lineFocusHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.helper.lineFocusHelper"] = true;
dojo.provide("gdl.widget.helper.lineFocusHelper");

// Summary:
//		This class is mixed into gfx line widgets such as Polyline
//		and overrides btp.widget.helper.focusHelper.
//		'this' points to a gfx line widget. EditArea can be accessed via this.__editArea.

gdl.widget.helper.lineFocusHelper = {
	// ------------- Focus Frame ----------------------------
	hlp_setFocusFrame: function(){
		this.shape.setStroke({color: "yellow"}); // TODO: should not hard-code the color
		if(this.sourceDecorationObj){
			var dec = this.sourceDecorationObj;
			dec.shape.setStroke({color: "yellow"}); // TODO: should not hard-code the color
			if(dec.fillColor){
				dec.shape.setFill("yellow"); // TODO: should not hard-code the color
			}
		}
		if(this.targetDecorationObj){
			var dec = this.targetDecorationObj;
			dec.shape.setStroke({color: "yellow"}); // TODO: should not hard-code the color
			if(dec.fillColor){
				dec.shape.setFill("yellow"); // TODO: should not hard-code the color
			}
		}
	},

	hlp_removeFocusFrame: function(){
		this.update({forceUpdate:true});
		if(this.sourceDecorationObj){
			this.sourceDecorationObj.update({forceUpdate:true});
		}
		if(this.targetDecorationObj){
			this.targetDecorationObj.update({forceUpdate:true});
		}
	},

	hlp_refreshFocusFrame: function(){
		alert("hlp_refreshFocusFrame");
	},

	// ------------- Feedback Frame ----------------------------
	hlp_showAllowDropFeedback: function(){
		alert("hlp_showAllowDropFeedback");
	},

	hlp_hideAllowDropFeedback: function(){
		alert("hlp_hideAllowDropFeedback");
	},

	// -------- Show/Hide a frame around the component ----------
	hlp_showComponentFrame: function(type){
		alert("hlp_showComponentFrame");
	},

	hlp_hideComponentFrame: function(type){
		alert("hlp_hideComponentFrame");
	}
};

}

if(!dojo._hasResource["gdl.widget.ml.XmlGenerator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.ml.XmlGenerator"] = true;
// Default Source Generator for Graphical Editor
dojo.provide("gdl.widget.ml.XmlGenerator");

gdl.widget.ml.XmlGenerator = {
	generate: function(widget){
	}
};

}

if(!dojo._hasResource["gdl.widget.ml.XmlLoader"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.ml.XmlLoader"] = true;
dojo.provide("gdl.widget.ml.XmlLoader");

gdl.widget.ml.XmlLoader = {
    load: function(xml){
	}
};

}

if(!dojo._hasResource["gdl.widget.outline.CloneContentOutlinePage"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.outline.CloneContentOutlinePage"] = true;
dojo.provide("gdl.widget.outline.CloneContentOutlinePage");



// summary:
//		The default implementation of OutlinePage, which displays a structure
//		of some model provided with a ContentProvider in the OutlineView.
//		Your application may want to replace this with yours.
//
//	interface:
//		setContentProvider(/*Widget*/provider)
//		setOutlineView(/*Widget*/view)
//		createOutlinePage()
//		attachOutline()
//		detachOutline()
//		refreshOutline()
//		updateOutline(/*Object*/message)
//		selectOutline(/*Widget*/selectedWidget)
//		selectOutlineTarget(/*Widget*/outlineNode)
//		deleteOutlineTarget()

dojo.declare("gdl.widget.outline.CloneContentOutlinePage", null,
{
	editArea: null,
	outlineView: null,
	contentProvider: null,
	outlineViewMenu: null,
	canvas: null,
	rootGroup: null,
	_connectors: null,
	_topics: null,
	
	scale: 0.2,

	imgDir: dojo.moduleUrl("btp", "widget/templates/OutlineView/"),

	constructor: function(editArea){
		this.editArea = editArea;
		this._connectors = [];
		this._topics = [
		];
	},

	destroy: function(){
		dojo.forEach(this._connectors, dojo.disconnect);
		dojo.forEach(this._topics, dojo.unsubscribe);
		this.inherited("destroy", arguments);
	},

	setContentProvider: function(/*Widget*/provider){
		this.contentProvider = provider;
	},

	setOutlineView: function(/*Widget*/view){
		this.outlineView = view;
	},

	createOutlinePage: function(){
		if(!this.canvas){
			if(window.gdl){
				dojo["require"]("gdc.widget.Surface");
				var node = this.outlineView.domNode;
				this.canvas = new gdc.widget.Surface({width:1,height:1});
				node.style.backgroundColor = "InactiveBorder";
				node.style.overflow = "hidden";
				this.outlineView.addChild(this.canvas);
				this._resizeCanvas();
				
				var layoutParent = btp.getLayoutParent(this.outlineView.domNode);
				if(layoutParent) {
					dojo.connect(layoutParent, "layout", this, "_resizeCanvas");
				}
				
				var targetWidget = this.editArea;
				if(targetWidget.adjustCanvasSize){
					dojo.connect(targetWidget, "adjustCanvasWidth", this, "_resizeCanvas");
					dojo.connect(targetWidget, "adjustCanvasHeight", this, "_resizeCanvas");
				}				
				
			}
		}else{
			this.detachOutline();
		}

		// if you want to have your own content provider, you should call setContentProvider() first.
		if(!this.contentProvider){
			var providerClassName = this.editArea.getContentProviderClass();
			if(providerClassName){
				dojo["require"](providerClassName);
				impl = dojo.getObject(providerClassName);
				this.setContentProvider(new impl());
			}
		}
	},

	attachOutline: function(){
		if(this.outlineView && this.canvas){
			this.outlineView.addChild(this.canvas);
			return true;
		}
		return false;
	},

	detachOutline: function(){
		if(this.outlineView && this.canvas){
			this.outlineView.removeChild(this.canvas);
		}
	},
	
	_resizeCanvas: function() {
		var target = this.editArea.domNode;
		var targetWidth = target.scrollWidth;
		var targetHeight = target.scrollHeight;
		var outlineBox = dojo.contentBox(this.outlineView.domNode);
		var outlineWidth = outlineBox.w;
		var outlineHeight = outlineBox.h;
		
		var propX = outlineWidth / targetWidth;
		var propY = outlineHeight / targetHeight; 
		this.scale =  Math.min(propX, propY);
		
		var width = Math.floor(this.scale * targetWidth) + "px";
		var height = Math.floor(this.scale * targetHeight) + "px";
		this.canvas.domNode.style.width = width;
		this.canvas.domNode.style.height = height;
		this.canvas.surface.setDimensions(width, height);
		if(this.rootGroup) {
			this.rootGroup.setTransform(dojox.gfx.matrix.scale(this.scale, this.scale));
		}
	},

	refreshOutline: function(){
		this.canvas.surface.clear();
		var g = this.rootGroup = this.canvas.surface.createGroup();
		g.setTransform(dojox.gfx.matrix.scale(this.scale, this.scale));		

		var root = this.contentProvider.getElements(this.editArea.rootGroup);
		for(var i = 0, len = root.length; i < len; i++){
			this._refreshCloneView(null, root[i], true);
		}
	},

	_refreshCloneView: function(parentObj, obj, forceRefresh){
		var node;
		var parentNode = (parentObj && parentObj.outlineNode) ? parentObj.outlineNode : this.rootGroup; 
		
		var shape = obj.targetNode.shape;
		
		var fill = obj.targetNode.getFill();
		var stroke = obj.targetNode.getStroke();
		var transform = obj.targetNode.getTransform();

		if(!obj.outlineNode || forceRefresh) {
			if(shape) node = obj.outlineNode = parentNode.createShape(shape);
			else node = obj.outlineNode = parentNode.createGroup();	
		} else {
			if(shape) node = obj.outlineNode.setShape(shape);
			else node = obj.outlineNode;
		}
		node.outlineObj = obj;
		
		if(fill) node.setFill(fill);
		if(stroke) node.setStroke(stroke);
		if(transform) node.setTransform(transform);
		
		var children = this.contentProvider.getChildren(obj);
		for(var i = 0, len = children.length; i < len; i++){
			var child = children[i];
			this._refreshCloneView(obj, child, forceRefresh);
		}
	},

	updateOutline: function(message){
		if(message.propName == "@AddChild"
		|| message.propName == "@RemoveChild"
		|| message.propName == "@ForceRefresh"){
//			var root = this.contentProvider.getElements(this.editArea.rootGroup);
//			this._updateCloneView(root, this.rootGroup);
			this.refreshOutline();
		}else if(message.propName == "@SetStyles"){
//			var widget = message.hint.widget;
//			var gfxObject = widget.groupBox ? widget.groupBox : widget.shape;
//			var parentObj = this.contentProvider.getParent(gfxObject.outlineObj);
//			this._refreshCloneView(parentObj, gfxObject.outlineObj);
			var root = this.contentProvider.getElements(this.editArea.rootGroup);
			for(var i = 0, len = root.length; i < len; i++){
				this._refreshCloneView(null, root[i], false);
			}			
		}else{
			var widget = message.hint.widget;
			if(!widget || !widget.outlineObj) {
				this.refreshOutline();
				return;
			}
			var outlineNode = widget.outlineObj.outlineNode;
			var name = message.propName;
			var f = "set" + name.charAt(0).toUpperCase() + name.substring(1);
			if(outlineNode[f]){
				outlineNode[f](message.newValue);
			}else{
				outlineNode[name] = message.newValue;
			}
		}
	},

	_updateCloneView: function(objs, /*Widget*/parentNode){
		var nObjs = objs.length;
		var nNodes = parentNode.children ? parentNode.children.length : 0;
		if(nObjs == nNodes){
			// no changes
			for(var i = 0, len = objs.length; i < len; i++){
				var chlidObjs = this.contentProvider.getChildren(objs[i]);
				this._updateCloneView(chlidObjs, parentNode.children[i]);
			}
		}else if(nObjs > nNodes){
			// a new node has been added
			for(var i = 0, len = nObjs; i < len; i++){
				var obj = objs[i];
				var node = parentNode.children[i];
				if(!node || node.outlineObj != obj){ // newly added node found
					var parentObj = this.contentProvider.getParent(obj);
					this._refreshCloneView(parentObj, obj);
					break;
				}
			}
		}else{
			// a node has been removed
			for(var i = 0, len = nNodes; i < len; i++){
				var obj = objs[i];
				var node = parentNode.children[i];
				if(!obj || obj.outlineNode != node){ // removed node found
					node.removeShape();
					delete node.outlineObj.targetNode.outlineNode;
					delete node.outlineObj;
					break;
				}
			}
		}
	},

	// select a node in the OutlineView
	selectOutline: function(/*Widget*/selectedWidget){
	},

	// select a widget in the EditArea
	selectOutlineTarget: function(/*Widget*/outlineNode){
	},

	deleteOutlineTarget: function(){
	}
});

}

if(!dojo._hasResource["gdl.widget.tool.ConnectionCreationTool"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.tool.ConnectionCreationTool"] = true;
dojo.provide("gdl.widget.tool.ConnectionCreationTool");



dojo.declare(
	"gdl.widget.tool.ConnectionCreationTool",
	gdl.widget.tool.AbstractConnectionCreationTool,
{
	_ds: null,
	_type: "connection",

	handleMouseDown: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
		if (this._operationStarted){
			this._operationStarted = false;
			var ret = this.handleCreateConnection(e, editPartViewer);
			this.targetWidget.__dt.hideFeedback();
			this.finishCreateConnection(e, editPartViewer);
			return ret;
		}
		if (!this.inherited("handleMouseDown", arguments)){ return false; }
		this._operationStarted = true;
		this.createGhostLine(e, editPartViewer);
		return true;
	},

	// Updates ghost line
	handleMouseMove: function(e, editPartViewer){
		if (!this._operationStarted){ return false; }
		var widget = this.targetWidget;
		if(widget && widget.getTargetConnectionAnchor){
			var anchor = widget.getTargetConnectionAnchor(e.target);
			this.targetConnectionAnchor = anchor;
			this.ghostLine.setTargetAnchor(anchor);
		}else{
			var pos = dojo._abs(editPartViewer.domNode, false);
			var x = e.pageX - pos.x;
			var y = e.pageY - pos.y;
			this.targetConnectionAnchor = null;
			this.ghostLine.setTargetAnchor(null);
			this.ghostLine.setEnd({x:x, y:y});
		}
		this.sourceConnectionAnchor.anchorMoved();
		return true;
	},

	handleMouseOver: function(e, editPartViewer){
		if(this.sourceWidget == this.targetWidget){ return; }
		var dropTargetWidget = btp.getWidgetByElement(e.target);
		if(this.sourceWidget == dropTargetWidget){ return; }
		var dt = dropTargetWidget && dropTargetWidget.__dt;
		if(!dt){ return false; }
		var dropAcceptable = dt.accepts({type:"connection"}); // @TODO should not hard-code "connection"
		if(dropAcceptable){
			this.targetWidget = dropTargetWidget;
			dt.showFeedback();
		}else{
			this.targetWidget = null;
		}
		return true;
	},

	handleMouseOut: function(e, editPartViewer){
		if(!this.targetWidget || this.sourceWidget == this.targetWidget){ return; }
		this.targetWidget = null;
		var dropTargetWidget = btp.getWidgetByElement(e.target);
		var dt = dropTargetWidget && dropTargetWidget.__dt;
		if(!dt){ return false; }
		dt.hideFeedback();
		return true;
	}
});

}

if(!dojo._hasResource["gdl.widget.tool.ReconnectTracker"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["gdl.widget.tool.ReconnectTracker"] = true;
dojo.provide("gdl.widget.tool.ReconnectTracker");







// summary:
//		A DragTracker for dragging a connection endpoint to reconnect.

dojo.declare(
	"gdl.widget.tool.ReconnectTracker",
	btp.widget.tool.AbstractTool,
{
	sourceConnectionAnchor: null,
	targetConnectionAnchor: null,
	_operationStarted: false,

	// constants
	_SOURCE: 1,
	_TARGET: 2,

	_lineWidget: null,
	_draggingKnob: null,
	_oppositeKnob: null,
	_moveTarget: null, // dummy drag target (DIV element) for the dragging knob
	_knobPosition: null, // 0: source, 1: target
	_idx0: -1, // index of the dragging end point
	_idx1: -1, // index of the opposite end point
	_draggingSideAnchor: null,
	_oppositeSideAnchor: null,
	_draggingSideWidget: null,
	_oppositeSideWidget: null,

	stopReconnecting: function(){
		if(!this._operationStarted){ return; } // already canceled
		this._operationStarted = false;
		this._editPartViewer.domNode.removeChild(this._editPartViewer.getMoveTarget());
		this._editPartViewer.domNode.style.cursor = "";
		this._oppositeSideWidget.__dt.disabled = false;
		this._draggingKnob.style.display = "";

		if(!this.sourceConnectionAnchor && !this.targetConnectionAnchor){
			this.onDropAborted();
		}else{
			this._lineWidget.anchorMoved();
		}
	},

	onDropAborted: function(){
		if(true){ // disconnect from the nodes
			// recover the dragging end point
			if(this._knobPosition == 0){	// dragging the source-side knob
				this._lineWidget.setSourceAnchor(this._draggingSideAnchor);
			}else{							// dragging the target-side knob
				this._lineWidget.setTargetAnchor(this._draggingSideAnchor);
			}
			// remove the connection
			this._editPartViewer.removeFocus();
			var command = new gdl.widget.command.ConnectionCommand.RemoveConnectionCommand(this._lineWidget);
			this.getEditDomain().executeCommand(command);
		}else{ // restore the connection
			// recover the dragging end point
			if(this._knobPosition == 0){	// dragging the source-side knob
				this._lineWidget.setSourceAnchor(this._draggingSideAnchor);
			}else{							// dragging the target-side knob
				this._lineWidget.setTargetAnchor(this._draggingSideAnchor);
			}
			this._editPartViewer.setFocusFrame(this._lineWidget); // to reset the endpoints position
		}
	},

	// ======== Event Handlers ====================================
	handleMouseDown: function(e, editPartViewer){
		this.setEditPartViewer(editPartViewer);
	},

	handleDragStart: function(e, editPartViewer){
		this._operationStarted = true;
		this._lineWidget = this._editPartViewer.focusWidgets[0]; // gfx line widget
		if(!btp.isGfxLineWidget(this._lineWidget)){ alert("ReconnectTracker#handleDragStart: gfx line is expected, but found "+this._lineWidget.declaredClass); } // TODO remove this code later
		this._draggingKnob = btp.dnd.dragManager.currentDragSource.domNode;
		this._oppositeKnob = (this._draggingKnob == this._lineWidget.__connectionEndpoints[0])
			? this._lineWidget.__connectionEndpoints[1] : this._lineWidget.__connectionEndpoints[0];
		this._moveTarget = this._editPartViewer.getMoveTarget();
		this._knobPosition = this._draggingKnob.knobPosition; // [custom property]
		this._idx0 = (this._knobPosition == 0) ? 0 : this._lineWidget.points.length - 1;
		this._idx1 = (this._knobPosition == 0) ? this._lineWidget.points.length - 1 : 0;
		this._draggingSideAnchor = (this._knobPosition == 0) ?
			 this._lineWidget.sourceAnchorObj : this._lineWidget.targetAnchorObj;
		this._oppositeSideAnchor = (this._knobPosition == 0) ?
			 this._lineWidget.targetAnchorObj : this._lineWidget.sourceAnchorObj;
		this._draggingSideWidget = this._draggingSideAnchor.owner;
		this._oppositeSideWidget = this._oppositeSideAnchor.owner;

		this.sourceConnectionAnchor = (this._knobPosition == 0) ? this._lineWidget.sourceAnchorObj : null;
		this.targetConnectionAnchor = (this._knobPosition == 0) ? null : this._lineWidget.targetAnchorObj;
		this._moveTarget.style.top = this._draggingKnob.offsetTop + "px";
		this._moveTarget.style.left = this._draggingKnob.offsetLeft + "px";
		this._editPartViewer.domNode.appendChild(this._moveTarget);
//		this._editPartViewer.domNode.style.cursor = "crosshair";
		this._oppositeSideWidget.__dt.disabled = true;
		this._draggingKnob.style.display = "none"; // hide the dragging knob because it may intercept mouse events for a target
	},

	 _findAnchor: function(targetWidget, clickedElement){
		if(!targetWidget){ return; }
		if(this._knobPosition == 0 && targetWidget.getSourceConnectionAnchor){
			return targetWidget.getSourceConnectionAnchor(clickedElement);
		}else if(this._knobPosition == 1 && targetWidget.getTargetConnectionAnchor){
			return targetWidget.getTargetConnectionAnchor(clickedElement);
		}
		return null;
	},

	// Cannot use handleDragMove because it is called only when the drag target accepts the drag.
	handleMouseMove: function(e, editPartViewer){
		if(!this._operationStarted){ return; } // drag is not started yet or perhaps was canceled with ESC
		if(e.target == this._draggingKnob && this._lineWidget.sourceAnchorObj && this._lineWidget.targetAnchorObj){
			// Dragging on the endpoint knob while both ends are anchored.
			this._skipping = true;
			return;
		}
		if(this._skipping){
			// Moved out of the endpoint knob, but skip the first event, because
			// mouseover event has not come yet and currentDropTarget is not available.
			this._skipping = false;
			return;
		}
		var dt = btp.dnd.dragManager.currentDropTarget;
		var targetWidget = this.targetWidget = dt && dt.data; // drop target widget
		var anchor = this._findAnchor(targetWidget, e.target);
		if(anchor && (this.sourceConnectionAnchor == anchor
					|| this.targetConnectionAnchor == anchor)){ return; } // already anchored
		if(this.sourceConnectionAnchor){ // release the old source anchor
			this.sourceConnectionAnchor = null;
			this._lineWidget.setSourceAnchor(null);
		}
		if(this.targetConnectionAnchor){ // release the old target anchor
			this.targetConnectionAnchor = null;
			this._lineWidget.setTargetAnchor(null);
		}
		if(anchor){
			if(this._knobPosition == 0){	// dragging the source-side knob
				this.sourceConnectionAnchor = anchor;
				this._lineWidget.setSourceAnchor(anchor);
			}else{							// dragging the target-side knob
				this.targetConnectionAnchor = anchor;
				this._lineWidget.setTargetAnchor(anchor);
			}
			var pt0 = this._lineWidget.points[this._idx0];
			this._draggingKnob.style.top = pt0.y + "px";
			this._draggingKnob.style.left = pt0.x + "px";
			var pt1 = this._lineWidget.points[this._idx1];
			this._oppositeKnob.style.top = pt1.y + "px";
			this._oppositeKnob.style.left = pt1.x + "px";
		}else{
			var moveTarget = this._moveTarget;
			var top = this._moveTarget.offsetTop;
			var left = this._moveTarget.offsetLeft;
			this._draggingKnob.style.top = top + "px";
			this._draggingKnob.style.left = left + "px";
			this._lineWidget.setPoint({x: left, y: top}, this._idx0);
			this._lineWidget.update();
		}
	},
	
	_recreateConnection: function(lineWidget, editPartViewer) {
		var sourceWidget = lineWidget.getSource();
		var sourceAnchor = lineWidget.sourceAnchorObj;
		var targetWidget = lineWidget.getTarget();
		var targetAnchor = lineWidget.targetAnchorObj;
		var connectionWidgetClassName = lineWidget.declaredClass;
		
		if(sourceAnchor && targetAnchor) {
			this.onDropAborted();	// disconnect first
			var props = {
				sourceAnchor: sourceAnchor.owner.id+":"+sourceAnchor.anchorName,
				targetAnchor: targetAnchor.owner.id+":"+targetAnchor.anchorName
			}
			var connection = editPartViewer.createComponent(connectionWidgetClassName, props);
			var offsetParent = gdl.getOffsetParent(this.sourceWidget, this.targetWidget);
			if(!offsetParent){ offsetParent = editPartViewer; }
			var command = new gdl.widget.command.ConnectionCommand.AddConnectionCommand(offsetParent, connection);
			this.getEditDomain().executeCommand(command);
			connection.setMouseOverHandler();
		}		
	},

	handleMouseUp: function(e, editPartViewer){
		if(!this._operationStarted){ return; } // drag is not started yet or perhaps was canceled with ESC
		if(this.sourceConnectionAnchor && this.sourceConnectionAnchor != this._draggingSideAnchor){
			this._recreateConnection(this._lineWidget, editPartViewer);
		}else if(this.targetConnectionAnchor && this.targetConnectionAnchor != this._draggingSideAnchor){
			this._recreateConnection(this._lineWidget, editPartViewer);
		}else{
		}
		this._lineWidget.getShape().moveToFront();
		this.stopReconnecting();
	},

	handleKeyDown: function(e, editPartViewer){
		if(e.keyCode == 27){ // ESC
			this.stopReconnecting();
		}
	}
});

}

if(!dojo._hasResource["htc.widget.Hello"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["htc.widget.Hello"] = true;
dojo.provide("htc.widget.Hello");






dojo.declare(
	"htc.widget.Hello",
//	[dijit._Widget, dijit._Templated],
	[dijit.layout.ContentPane, dijit._Templated, dijit._Container],
{
	text: "",
	templateString:"<div class=\"hello\">Hello, all</div>\r\n",

	postCreate: function(){
		if(this.text){
			this.domNode.innerHTML = this.text;
		}
	},
	
	setText: function(text){
		this.text = text;
		this.domNode.innerHTML = this.text;
	}
});

}

if(!dojo._hasResource["htc.widget.helper.RoundCornerPaneHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["htc.widget.helper.RoundCornerPaneHelper"] = true;
dojo.provide("htc.widget.helper.RoundCornerPaneHelper");

htc.widget.helper.RoundCornerPaneHelper = {
	hlp_dropTargets: ["component"],

	// called from DefaultCreationTool#handleDrop
	componentDropHandler: function(e, editPartViewer){
		var srcElem = this.hlp_getDroppedDomNode(e);
		if(srcElem.componentClassName && srcElem.componentClassName == "htc.widget.RoundCornerPane"){
			alert("You cannot create RoundCornerPane on another RoundCornerPane");
			return false;
		}
		editPartViewer.getEditDomain().getActiveTool().componentDropHandler(e, editPartViewer);
		return true;
	}
};

}

if(!dojo._hasResource["htc.widget.RoundCornerPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["htc.widget.RoundCornerPane"] = true;
dojo.provide("htc.widget.RoundCornerPane");




dojo.declare(
	"htc.widget.RoundCornerPane",
	[dijit.layout.ContentPane, dijit._Templated, dijit._Container],
{
	title : "",
	top: '',
	left: '',
	borderStyle: "rich",
	bgColor: "white",

	templateString:"<table cellpadding=\"0\" cellspacing=\"0\" dojoAttachPoint=\"tableNode\">\r\n  <tr class=\"roundCornerPane_tr roundCornerPane_headerTr\">\r\n    <td class=\"roundCornerPane_topLeft\"></td>\r\n    <td class=\"roundCornerPane_topCenter\" dojoAttachPoint=\"topCenterNode\"><div dojoAttachPoint=\"titleNode\"></div></td>\r\n    <td class=\"roundCornerPane_topRight\"></td>\r\n  </tr>\r\n  <tr>\r\n    <td class=\"roundCornerPane_middleLeft\"><hr class=\"roundCornerPane_hr\"/></td>\r\n    <td class=\"roundCornerPane_middleCenter\" width=\"100%\"><div dojoAttachPoint=\"containerNode\"></div></td>\r\n    <td class=\"roundCornerPane_middleRight\"><hr class=\"roundCornerPane_hr\"/></td>\r\n  </tr>\r\n  <tr class=\"roundCornerPane_tr roundCornerPane_footerTr\">\r\n    <td class=\"roundCornerPane_bottomLeft\"></td>\r\n    <td class=\"roundCornerPane_bottomCenter\"></td>\r\n    <td class=\"roundCornerPane_bottomRight\"></td>\r\n  </tr>\r\n</table>\r\n",
//	templateCssPath: dojo.moduleUri("htc.widget", "templates/RoundCornerPane/RoundCornerPane.css"),

	postCreate: function(){
		htc.widget.RoundCornerPane.superclass.postCreate.apply(this, arguments);
		this.update();
	},
	
	update: function(){
		this.tableNode.className = this.borderStyle;
		this.tableNode.style.backgroundColor = this.bgColor;
		this.setTitle(this.title);
	},
	
// ======================= Setters/Getters =================================
	getTitle: function(){
		return this.title;
	},

	setTitle: function(title){
		this.title = title;
		this.titleNode.innerHTML = "";
		if(this.title){
			this.titleNode.appendChild(dojo.doc.createTextNode(this.title));
		}
	},

	getBorderStyle: function(){
		return this.borderStyle;
	},

	setBorderStyle: function(val){
		this.borderStyle = val;
		this.update();
	},

	getBgColor: function(){
		return this.bgColor;
	},

	setBgColor: function(val){
		this.bgColor = val;
		this.update();
	},

	getHeight: function(){
		return dojo.contentBox(this.domNode).h;
	},

	setHeight: function(/*int*/height){
		var tableH = dojo.contentBox(this.domNode).h;
		var containerH = dojo.contentBox(this.containerNode).h;
		this.containerNode.style.height = height - (tableH - containerH) + "px";
	}
});

}


dojo.i18n._preloadLocalizations("dojo.nls.flow", ["es-es", "es", "hu", "it-it", "de", "pt-br", "pl", "fr-fr", "zh-cn", "pt", "en-us", "zh", "ru", "xx", "fr", "zh-tw", "it", "cs", "en-gb", "de-de", "ja-jp", "ko-kr", "ko", "en", "ROOT", "ja"]);
